Package: sirt
Type: Package
Title: Supplementary Item Response Theory Models
Version: 4.2-133
Date: 2025-09-27 12:57:51
Authors@R: c( person(given = "Alexander",
                    family = "Robitzsch",
                    role = c("aut", "cre"),
                    email = "robitzsch@ipn.uni-kiel.de",
                    comment = c(ORCID = "0000-0002-8226-3132")) )
Maintainer: Alexander Robitzsch <robitzsch@ipn.uni-kiel.de>
Description: 
    Supplementary functions for item response models aiming
    to complement existing R packages. The functionality includes among others
    multidimensional compensatory and noncompensatory IRT models
    (Reckase, 2009, <doi:10.1007/978-0-387-89976-3>), 
    MCMC for hierarchical IRT models and testlet models
    (Fox, 2010, <doi:10.1007/978-1-4419-0742-4>), 
    NOHARM (McDonald, 1982, <doi:10.1177/014662168200600402>), 
    Rasch copula model (Braeken, 2011, <doi:10.1007/s11336-010-9190-4>;
    Schroeders, Robitzsch & Schipolowski, 2014, <doi:10.1111/jedm.12054>),
    faceted and hierarchical rater models (DeCarlo, Kim & Johnson, 2011,
    <doi:10.1111/j.1745-3984.2011.00143.x>),
    ordinal IRT model (ISOP; Scheiblechner, 1995, <doi:10.1007/BF02301417>), 
    DETECT statistic (Stout, Habing, Douglas & Kim, 1996, 
    <doi:10.1177/014662169602000403>), local structural equation modeling 
    (LSEM; Hildebrandt, Luedtke, Robitzsch, Sommer & Wilhelm, 2016,
    <doi:10.1080/00273171.2016.1142856>).
Depends: R (>= 3.5)
Imports: CDM, graphics, methods, parallel, pbapply, Rcpp, stats, TAM,
        utils
Suggests: coda, igraph, lavaan, MASS, Matrix, miceadds, minqa, mirt,
        mvtnorm, nloptr, optimx, pbivnorm, pbv, psych, sfsmisc, sm,
        survey
LinkingTo: pbv, Rcpp, RcppArmadillo
URL: https://github.com/alexanderrobitzsch/sirt,
        https://sites.google.com/view/alexander-robitzsch/software
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2025-09-27 11:06:19 UTC; sunpn563
Author: Alexander Robitzsch [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8226-3132>)
Repository: CRAN
Date/Publication: 2025-09-27 23:30:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-28 00:31:51 UTC; unix
Archs: sirt.so.dSYM

Package: sirt
Type: Package
Title: Supplementary Item Response Theory Models
Version: 4.2-133
Date: 2025-09-27 12:57:51
Authors@R: c( person(given = "Alexander",
                    family = "Robitzsch",
                    role = c("aut", "cre"),
                    email = "robitzsch@ipn.uni-kiel.de",
                    comment = c(ORCID = "0000-0002-8226-3132")) )
Maintainer: Alexander Robitzsch <robitzsch@ipn.uni-kiel.de>
Description: 
    Supplementary functions for item response models aiming
    to complement existing R packages. The functionality includes among others
    multidimensional compensatory and noncompensatory IRT models
    (Reckase, 2009, <doi:10.1007/978-0-387-89976-3>), 
    MCMC for hierarchical IRT models and testlet models
    (Fox, 2010, <doi:10.1007/978-1-4419-0742-4>), 
    NOHARM (McDonald, 1982, <doi:10.1177/014662168200600402>), 
    Rasch copula model (Braeken, 2011, <doi:10.1007/s11336-010-9190-4>;
    Schroeders, Robitzsch & Schipolowski, 2014, <doi:10.1111/jedm.12054>),
    faceted and hierarchical rater models (DeCarlo, Kim & Johnson, 2011,
    <doi:10.1111/j.1745-3984.2011.00143.x>),
    ordinal IRT model (ISOP; Scheiblechner, 1995, <doi:10.1007/BF02301417>), 
    DETECT statistic (Stout, Habing, Douglas & Kim, 1996, 
    <doi:10.1177/014662169602000403>), local structural equation modeling 
    (LSEM; Hildebrandt, Luedtke, Robitzsch, Sommer & Wilhelm, 2016,
    <doi:10.1080/00273171.2016.1142856>).
Depends: R (>= 3.5)
Imports: CDM, graphics, methods, parallel, pbapply, Rcpp, stats, TAM,
        utils
Suggests: coda, igraph, lavaan, MASS, Matrix, miceadds, minqa, mirt,
        mvtnorm, nloptr, optimx, pbivnorm, pbv, psych, sfsmisc, sm,
        survey
LinkingTo: pbv, Rcpp, RcppArmadillo
URL: https://github.com/alexanderrobitzsch/sirt,
        https://sites.google.com/view/alexander-robitzsch/software
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2025-09-27 11:06:19 UTC; sunpn563
Author: Alexander Robitzsch [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8226-3132>)
Repository: CRAN
Date/Publication: 2025-09-27 23:30:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 06:23:00 UTC; unix
Archs: sirt.so.dSYM

useDynLib(sirt)



####################################################
################## IMPORTS #########################
####################################################

#***************************************************
#*************** R Basics / Linking To *************
#***************************************************

importFrom(graphics, abline)
importFrom(graphics, dotchart)
importFrom(graphics, lines)
importFrom(graphics, matplot)
importFrom(graphics, par)
importFrom(graphics, plot)
importFrom(graphics, points)
importFrom(graphics, polygon)
importFrom(graphics, rug)
importFrom(graphics, segments)
importFrom(graphics, text)
importFrom(methods, is)
importFrom(Rcpp, evalCpp)
importFrom(stats, acf)
importFrom(stats, aggregate)
importFrom(stats, anova)
importFrom(stats, as.dist)
importFrom(stats, as.formula)
importFrom(stats, coef)
importFrom(stats, confint)
importFrom(stats, cor)
importFrom(stats, cov)
importFrom(stats, cov2cor)
importFrom(stats, cov.wt)
importFrom(stats, cutree)
importFrom(stats, dbeta)
importFrom(stats, dnorm)
importFrom(stats, fitted)
importFrom(stats, glm)
importFrom(stats, hclust)
importFrom(stats, ksmooth)
importFrom(stats, lm)
importFrom(stats, lm.wfit)
importFrom(stats, logLik)
importFrom(stats, mad)
importFrom(stats, median)
importFrom(stats, medpolish)
importFrom(stats, model.matrix)
importFrom(stats, na.omit)
importFrom(stats, optim)
importFrom(stats, optimize)
importFrom(stats, p.adjust)
importFrom(stats, pchisq)
importFrom(stats, plogis)
importFrom(stats, pnorm)
importFrom(stats, predict)
importFrom(stats, promax)
importFrom(stats, qlogis)
importFrom(stats, qnorm)
importFrom(stats, qt)
importFrom(stats, quantile)
importFrom(stats, rbeta)
importFrom(stats, rect.hclust)
importFrom(stats, resid)
importFrom(stats, residuals)
importFrom(stats, rgamma)
importFrom(stats, rnorm)
importFrom(stats, runif)
importFrom(stats, sd)
importFrom(stats, spline)
importFrom(stats, var)
importFrom(stats, vcov)
importFrom(stats, weighted.mean)
importFrom(utils, combn)
importFrom(utils, flush.console)
importFrom(utils, packageDescription)
importFrom(utils, read.table)
importFrom(utils, write.table)



#***************************************************
#********** Imports (own packages) *****************
#***************************************************

importFrom(CDM, CDM_rmvnorm)
importFrom(CDM, csink)
importFrom(CDM, IRT.anova)
importFrom(CDM, IRT.compareModels)
importFrom(CDM, IRT.data)
importFrom(CDM, IRT.derivedParameters)
importFrom(CDM, IRT.expectedCounts)
importFrom(CDM, IRT.factor.scores)
importFrom(CDM, IRT.IC)
importFrom(CDM, IRT.irfprob)
importFrom(CDM, IRT.irfprobPlot)
importFrom(CDM, IRT.itemfit)
importFrom(CDM, IRT.jackknife)
importFrom(CDM, IRT.likelihood)
importFrom(CDM, IRT.modelfit)
importFrom(CDM, IRT.parameterTable)
importFrom(CDM, IRT.posterior)
importFrom(CDM, IRT.predict)
importFrom(CDM, IRT.repDesign)
importFrom(CDM, IRT.se)
importFrom(CDM, IRT_RMSD_calc_rmsd)
importFrom(CDM, modelfit.cor2)
importFrom(CDM, numerical_gradient)
importFrom(CDM, numerical_Hessian)
importFrom(CDM, osink)
importFrom(TAM, IRT.drawPV)
importFrom(TAM, IRT.informationCurves)
importFrom(TAM, IRT.linearCFA)
importFrom(TAM, IRT.residuals)
importFrom(TAM, IRT.simulate)
importFrom(TAM, IRT.threshold)
importFrom(TAM, IRT.truescore)
importFrom(TAM, IRT.WrightMap)
importFrom(TAM, lavaanify.IRT)
importFrom(TAM, require_namespace_msg)
importFrom(TAM, tam.modelfit)
importFrom(TAM, tam.ctt3)
importFrom(TAM, tam_matrix2)
importFrom(TAM, tam_outer)
importFrom(TAM, tam_packageinfo)
importFrom(TAM, tam_print_call)
importFrom(TAM, tam_rsessinfo)
importFrom(TAM, weighted_mean)
importFrom(TAM, weighted_sd)



#***************************************************
#************** Imports ****************************
#***************************************************

importFrom(parallel, clusterEvalQ)
importFrom(parallel, clusterExport)
importFrom(parallel, detectCores)
importFrom(parallel, makeCluster)
importFrom(parallel, parSapply)
importFrom(parallel, stopCluster)
importFrom(pbapply, pblapply)



#***************************************************
#************** Suggests ***************************
#***************************************************
# requireNamespace("another_package")
# TAM::require_namespace_msg("another_package")

# importFrom(coda, as.mcmc.list)
# importFrom(coda, effectiveSize)
# importFrom(coda, mcmc)
# importFrom(igraph, graph.edgelist)
# importFrom(lavaan, cfa)
# importFrom(lavaan, fitMeasures)
# importFrom(lavaan, lavaan)
# importFrom(lavaan, lavaanify)
# importFrom(lavaan, parameterEstimates)
# importFrom(lavaan, parameterTable)
# importFrom(lavaan, standardizedSolution)
# importFrom(lavaan, sem)
# importFrom(lavaan.survey, lavaan.survey)
# importFrom(MASS, ginv)
# importFrom(Matrix, bdiag)
# importFrom(mirt, extract.item)
# importFrom(mirt, mirt)
# importFrom(mirt, itemplot)
# importFrom(mirt, mirt.model)
# importFrom(mirt, probtrace)
# importFrom(mvtnorm, dmvnorm)
# importFrom(mvtnorm, pmvnorm)
# importFrom(mvtnorm, rmvnorm) # replaced by CDM::CDM_rmvnorm
# importFrom(pbivnorm, pbivnorm)
# importFrom(psych, cor.smooth)
# importFrom(psych, fa)
# importFrom(psych, omega)
# importFrom(sfsmisc, QUnif)
# importFrom(sm, h.select)
# importFrom(sm, sm.binomial)
# importFrom(survey, svydesign)


####################################################
################## EXPORTS #########################
####################################################

export(automatic.recode)
export(bounds_parameters)
export(brm.irf)
export(brm.sim)
export(btm)
export(btm_sim)
export(categorize)
export(ccov.np)
export(cfa_meas_inv)
export(class.accuracy.rasch)
export(colCumsums.sirt)
export(conf.detect)
export(data.wide2long)
export(decategorize)
export(detect.index)
export(dexppow)
export(dif.logistic.regression)
export(dif.strata.variance)
export(dif.variance)
export(dimproper)
export(dinvgamma2)
export(dirichlet.mle)
export(dirichlet.simul)
export(dmlavaan)
export(eigenvalues.manymatrices)
export(equating.rasch)
export(equating.rasch.jackknife)
export(expl.detect)
export(f1d.irt)
export(fit.adisop)
export(fit.isop)
export(fuzcluster)
export(fuzdiscr)
export(genlogis.moments)
export(ginverse_sym)
export(gom.em)
export(gom.jml)
export(greenyang.reliability)
export(hard_thresholding)
export(invariance_alignment_cfa_config)
export(invariance_alignment_constraints)
export(invariance_alignment_simulate)
export(invariance.alignment)
export(IRT.mle)
export(isop.dich)
export(isop.poly)
export(isop.scoring)
export(isop.test)
export(L0_polish)
export(latent.regression.em.normal)
export(latent.regression.em.raschtype)
export(lavaan2mirt)
export(lc.2raters)
export(likelihood.adjustment)
export(linking_haberman_itempars_convert)
export(linking_haberman_itempars_prepare)
export(linking.haberman)
export(linking.haberman.lq)
export(linking.haebara)
export(linking.robust)
export(linking_2groups)
export(locpolycor)
export(lq_fit)
export(lq_fit_estimate_power)
export(lsdm)
export(lsem.bootstrap)
export(lsem.estimate)
export(lsem.MGM.stepfunctions)
export(lsem.permutationTest)
export(lsem.test)
export(lsem_local_weights)
export(marginal.truescore.reliability)
export(mcmc.2pno)
export(mcmc.2pno.ml)
export(mcmc.2pnoh)
export(mcmc.3pno.testlet)
export(mcmc.list.descriptives)
export(mcmc_coef)
export(mcmc_confint)
export(mcmc_derivedPars)
export(mcmc_plot)
export(mcmc_Rhat)
export(mcmc_summary)
export(mcmc_vcov)
export(mcmc_WaldTest)
export(mcmclist2coda)
export(md.pattern.sirt)
export(mirt_summary)
export(mirt.specify.partable)
export(mirt.wrapper.coef)
export(mirt.wrapper.fscores)
export(mirt.wrapper.itemplot)
export(mirt.wrapper.posterior)
export(mle.pcm.group)
export(modelfit.cor.poly)
export(modelfit.sirt)
export(monoreg.colwise)
export(monoreg.rowwise)
export(move_variables_df)
export(nedelsky.irf)
export(nedelsky.latresp)
export(nedelsky.sim)
export(noharm.sirt)
export(np.dich)
export(parmsummary_extend)
export(pbivnorm2)
export(pcm.conversion)
export(pcm.fit)
export(person.parameter.rasch.copula)
export(personfit.stat)
export(pgenlogis)
export(plausible.value.imputation.raschtype)
export(polychoric2)
export(pow)
export(predict_scale_group_means)
export(print_digits)
export(prior_model_parse)
export(prmse.subscores.scales)
export(prob.guttman)
export(Q3)
export(Q3.testlet)
export(qmc.nodes)
export(R2conquest)
export(R2noharm)
export(R2noharm.EAP)
export(R2noharm.jackknife)
export(rasch.conquest)		# defunct
export(rasch.copula2)
export(rasch.copula3)
export(rasch.evm.pcm)
export(rasch.jml)
export(rasch.jml.biascorr)
export(rasch.jml.jackknife1)
export(rasch.mirtlc)
export(rasch.mml2)
export(rasch.pairwise)
export(rasch.pairwise.itemcluster)
export(rasch.pml2)		# defunct
export(rasch.pml3)
export(rasch.prox)
export(rasch.va)
export(read.multidimpv)
export(read.pimap)
export(read.pv)
export(read.show)
export(read.show.regression)
export(read.show.term)
export(reliability.nonlinearSEM)
export(resp_groupwise)
export(rexppow)
export(rinvgamma2)
export(rm.facets)
export(rm.sdt)
export(rm_proc_data)
export(rmvn)
export(rowCumsums.sirt)
export(rowIntervalIndex.sirt)
export(rowKSmallest.sirt)
export(rowKSmallest2.sirt)
export(rowMaxs.sirt)
export(rowMins.sirt)
export(ruvn)
export(scale_group_means)
export(soft_thresholding)
export(sia.sirt)
export(sim.qm.ramsay)
export(sim.rasch.dep)
export(sim.raschtype)
export(sirt_abs_smooth)
export(sirt_antifisherz)
export(sirt_attach_list_elements)
export(sirt_colMaxs)
export(sirt_colMeans)
export(sirt_colMedians)
export(sirt_colMins)
export(sirt_colSDs)
export(sirt_dnorm_discrete)
export(sirt_eigenvalues)
export(sirt_fisherz)
export(sirt_matrix2)
export(sirt_optimizer)
export(sirt_permutations)
export(sirt_rbind_fill)
export(sirt_rcpp_discrete_inverse)
export(sirt_rcpp_polychoric2)
export(sirt_sum_norm)
export(sirt_summary_print_call)
export(sirt_summary_print_objects)
export(sirt_summary_print_package_rsession)
export(sirt_summary_print_package)
export(sirt_summary_print_rsession)
export(smirt)
export(stratified.cronbach.alpha)
export(summary.R2noharm)
export(tam2mirt)
export(testlet.marginalized)
export(testlet.yen.q3)		# defunct
export(tetrachoric2)
export(tracemat)
export(truescore.irt)
export(unidim.test.csn)
export(wle.rasch)
export(wle.rasch.jackknife)
export(xxirt)
export(xxirt_createDiscItem)
export(xxirt_createParTable)
export(xxirt_createThetaDistribution)
export(xxirt_hessian)
export(xxirt_modifyParTable)
export(xxirt_sandwich_pml)
export(yen.q3)		# defunct
    





################################################
################ S3 methods ####################
################################################

S3method(anova, gom)
S3method(anova, prob.guttman)
S3method(anova, rasch.copula2)
S3method(anova, rasch.copula3)
S3method(anova, rasch.mirtlc)
S3method(anova, rasch.mml)
S3method(anova, rm.facets)
S3method(anova, rm.sdt)
S3method(anova, smirt)
S3method(anova, xxirt)
S3method(coef, rasch.evm.pcm)
S3method(coef, xxirt)
S3method(confint, xxirt)
S3method(IRT.expectedCounts, MultipleGroupClass)
S3method(IRT.expectedCounts, rasch.mml)
S3method(IRT.expectedCounts, SingleGroupClass)
S3method(IRT.expectedCounts, xxirt)
S3method(IRT.factor.scores, rm.facets)
S3method(IRT.factor.scores, rm.sdt)
S3method(IRT.factor.scores, xxirt)
S3method(IRT.irfprob, gom)
S3method(IRT.irfprob, prob.guttman)
S3method(IRT.irfprob, rasch.mirtlc)
S3method(IRT.irfprob, rasch.mml)
S3method(IRT.irfprob, rm.facets)
S3method(IRT.irfprob, rm.sdt)
S3method(IRT.irfprob, MultipleGroupClass)
S3method(IRT.irfprob, SingleGroupClass)
S3method(IRT.irfprob, smirt)
S3method(IRT.irfprob, xxirt)
S3method(IRT.likelihood, gom)
S3method(IRT.likelihood, rasch.copula2)
S3method(IRT.likelihood, rasch.copula3)
S3method(IRT.likelihood, rasch.mirtlc)
S3method(IRT.likelihood, rasch.mml)
S3method(IRT.likelihood, rm.facets)
S3method(IRT.likelihood, MultipleGroupClass)
S3method(IRT.likelihood, SingleGroupClass)
S3method(IRT.likelihood, smirt)
S3method(IRT.likelihood, xxirt)
S3method(IRT.modelfit, gom)
S3method(IRT.modelfit, rasch.mirtlc)
S3method(IRT.modelfit, rasch.mml)
S3method(IRT.modelfit, rm.facets)
S3method(IRT.modelfit, rm.sdt)
S3method(IRT.modelfit, smirt)
S3method(IRT.modelfit, sirt)
S3method(IRT.modelfit, xxirt)
S3method(IRT.posterior, gom)
S3method(IRT.posterior, prob.guttman)
S3method(IRT.posterior, rasch.copula2)
S3method(IRT.posterior, rasch.copula3)
S3method(IRT.posterior, rasch.mirtlc)
S3method(IRT.posterior, rasch.mml)
S3method(IRT.posterior, rm.facets)
S3method(IRT.posterior, rm.sdt)
S3method(IRT.posterior, MultipleGroupClass)
S3method(IRT.posterior, SingleGroupClass)
S3method(IRT.posterior, smirt)
S3method(IRT.posterior, xxirt)
S3method(IRT.se, xxirt)
S3method(logLik, gom)
S3method(logLik, prob.guttman)
S3method(logLik, rasch.copula2)
S3method(logLik, rasch.copula3)
S3method(logLik, rasch.mirtlc)
S3method(logLik, rasch.mml)
S3method(logLik, rm.facets)
S3method(logLik, rm.sdt)
S3method(logLik, smirt)
S3method(logLik, xxirt)
S3method(plot, isop)
S3method(plot, linking.robust)
S3method(plot, lsem)
S3method(plot, lsem.permutationTest)
S3method(plot, lsdm)
S3method(plot, mcmc.sirt)
S3method(plot, np.dich)
S3method(plot, rasch.mml)
S3method(plot, rm.sdt)
S3method(predict, btm)
S3method(summary, btm)
S3method(summary, conf.detect)
S3method(summary, fuzcluster)
S3method(summary, gom)
S3method(summary, gom.jml)
S3method(summary, invariance_alignment_constraints)
S3method(summary, invariance.alignment)
S3method(summary, IRT.modelfit.gom)
S3method(summary, IRT.modelfit.rasch.mirtlc)
S3method(summary, IRT.modelfit.rasch.mml)
S3method(summary, IRT.modelfit.rm.facets)
S3method(summary, IRT.modelfit.rm.sdt)
S3method(summary, IRT.modelfit.smirt)
S3method(summary, isop)
S3method(summary, isop.test)
S3method(summary, latent.regression)
S3method(summary, lc.2raters)
S3method(summary, linking.haberman)
S3method(summary, linking.haberman.lq)
S3method(summary, linking.haebara)
S3method(summary, linking.robust)
S3method(summary, lsdm)
S3method(summary, lsem)
S3method(summary, lsem.permutationTest)
S3method(summary, mcmc.sirt)
S3method(summary, mcmc_WaldTest)
S3method(summary, modelfit.sirt)
S3method(summary, noharm.sirt)
S3method(summary, prob.guttman)
S3method(summary, R2conquest)
S3method(summary, R2noharm)
S3method(summary, R2noharm.jackknife)
S3method(summary, rasch.copula2)
S3method(summary, rasch.copula3)
S3method(summary, rasch.evm.pcm)
S3method(summary, rasch.jml)
S3method(summary, rasch.mirtlc)
S3method(summary, rasch.mml)
S3method(summary, rasch.pairwise)
S3method(summary, rasch.pml)
S3method(summary, rm.facets)
S3method(summary, rm.sdt)
S3method(summary, smirt)
S3method(summary, xxirt)
S3method(vcov, rasch.evm.pcm)
S3method(vcov, xxirt)

# sirt
#### Supplementary Item Response Theory Models


If you use `sirt` and have suggestions for improvement or have found bugs, please email me at robitzsch@leibniz-ipn.de.
Please always provide a minimal dataset, necessary to demonstrate the problem, 
a minimal runnable code necessary to reproduce the issue, which can be run on the given dataset, and
all necessary information on the used librarys, the R version, and the OS it is run on, perhaps a sessionInfo().


#### CRAN version

<!---
[![CRAN_Status_Badge](http://www.r-pkg.org/badges/version-last-release/sirt)](https://cran.r-project.org/package=sirt)
&#160;&#160;
-->

The official version of `sirt` is hosted on CRAN and may be found [here](https://cran.r-project.org/package=sirt). 
The CRAN version can be installed from within R using:

```r
utils::install.packages("sirt")
```

#### GitHub version

The version hosted [here](https://github.com/alexanderrobitzsch/sirt) is the development version of `sirt`. 
The GitHub version can be installed using `devtools` as:

```r
devtools::install_github("alexanderrobitzsch/sirt")
```

## File Name: mgsem_loglike_suffstat_derivative_parameter.R
## File Version: 0.218

mgsem_loglike_suffstat_derivative_parameter <- function(est, dermoments, suffstat,
        type, i1, i2, h, is_B=FALSE, eps=1e-12, num_approx=FALSE)
{

    #  num_approx <- TRUE

    #** derivative of moments with respect to parameter
    res <- mgsem_moments_derivative_parameter( est=est, type=type, i1=i1, i2=i2,
                h=h, is_B=is_B, eps=eps, num_approx=num_approx )
    Mu_der <- res$Mu_der
    Sigma_der <- res$Sigma_der
    Sigma_der_logical <- res$Sigma_der_logical

    #** derivatives
    if (res$calc_Mu){
        der1 <- sum( dermoments$dermean * Mu_der )
    } else {
        der1 <- 0
    }
    N <- suffstat$N
    S1 <- dermoments$S1
    y <- dermoments$y
    S2 <- dermoments$S2
    S3 <- dermoments$S3

    if (res$calc_Sigma){


        # t2 <- -( t(y) %*% Sigma_der %*% y )[1,1]
        # t2 <- -sirt_rcpp_mgsem_quadform_logical(y=y, A=Sigma_der,
        #                A_logical=Sigma_der_logical)
        # t3 <- - sum(diag( S3 %*% Sigma_der ))
        # t3 <- -sirt_rcpp_mgsem_trace_product_logical(A=S3, B=Sigma_der,
        #                B_logical=Sigma_der_logical)
        # t4 <- sum(diag(S1 %*% Sigma_der))
        # t4 <- sirt_rcpp_mgsem_trace_product_logical(A=S1, B=Sigma_der,
        #                B_logical=Sigma_der_logical)

        t4a <- sirt_rcpp_mgsem_loglike_derivative_parameters( S1=S1, S3=S3,
                        y=y, Sigma_der=Sigma_der, Sigma_der_logical=Sigma_der_logical)

#        der2 <- sum( -N/2*(t2+t3+t4) )
        der2 <- sum( -N/2*t4a )
    } else {
        der2 <- 0
    }

    # output
    res <- der1+der2
    return(res)
}

## File Name: mgsem_numerical_gradient.R
## File Version: 0.05


mgsem_numerical_gradient <- function(par, FUN, h, symmetrize=FALSE, ...)
{
    res <- CDM::numerical_gradient(par=par, FUN=FUN, h=h, ...)
    if (symmetrize){
        res <- sirt_symmetrize(x=res)
        res <- sirt_add_names(x=res, names=names(par))
    }
    return(res)
}

## File Name: mcmc_rename_parameter_names.R
## File Version: 0.121

mcmc_rename_parameter_names <- function( vec, orig, trans)
{
    NO <- length(orig)
    for (oo in 1L:NO){
        trans_oo <- mcmc_rename_helper( string=trans[oo] )
        trans_oo <- gsub( ' ', '', trans_oo)
        vec <- gsub( orig[oo], trans_oo, vec, fixed=TRUE)
    }
    return(vec)
}

## File Name: noharm_sirt_optim_function_R.R
## File Version: 0.05


noharm_sirt_optim_function_R <- function(gamma_val, delta, I, wgtm, pm,
    b0.jk, b1.jk, b2.jk, b3.jk)
{
    val <- 0
    for (ii in 1L:(I-1)){
        for (jj in (ii+1):I){
            if (wgtm[ii,jj]>0 ){
                x_ij <- gamma_val[ii,jj] / sqrt( delta[ii] * delta[jj] )
                pm_exp <- b0.jk[ii,jj] + b1.jk[ii,jj]*x_ij + b2.jk[ii,jj]*x_ij^2 +
                                                b3.jk[ii,jj]*x_ij^3
                val <- val + wgtm[ii,jj]*(pm[ii,jj] - pm_exp)^2
            }
        }
    }
    #-- output
    return(val)
}

## File Name: IRT.factor.scores.sirt.R
## File Version: 0.173


#--- rm.facets
IRT.factor.scores.rm.facets <- function( object, type="EAP", ... )
{
    # admissible factor score types
    x1 <- c('EAP','MLE','WLE')
    if ( ! ( type %in% x1 ) ){
        stop('Requested type is not supported!\n')
    }
    #**** EAP
    if ( type=='EAP'){
        res <- object$person
        res <- res[, c('pid', 'EAP', 'SE.EAP') ]
        attr(res,'type') <- type
        attr(res,'reliability') <- object$EAP.rel
    }
    #**** MLE or WLE
    if ( type %in% c('MLE','WLE') ){
        data <- object$procdata$dat2.NA
        a <- object$ipars.dat2$a
        b <- object$ipars.dat2$b
        theta0 <- object$person$EAP
        WLE <- if( type=='WLE'){ TRUE } else { FALSE }
        res <- rm_facets_pp_mle( data=data, a=a, b=b, theta=theta0,  WLE=WLE,
                        maxiter=30, maxincr=3, h=1e-3, convP=1e-3, maxval=9.99,
                        progress=TRUE )
        res <- data.frame(pid=object$person$pid, res )
        attr(res,'type') <- type
        attr(res,'reliability') <- mle.reliability( meas=res$est, se.meas=res$se )
    }
    return(res)
}

#--- rm.sdt
IRT.factor.scores.rm.sdt <- IRT.factor.scores.rm.facets

## File Name: gom_em_prepare_lambda_index.R
## File Version: 0.133

gom_em_prepare_lambda_index <- function(lambda.index, I, K, items)
{
    if (is.null(lambda.index)){
        lambda.index <- matrix(1:(I*K), nrow=I, ncol=K)
    }
    li <- as.vector(lambda.index)
    lambda.index <- matrix( match(li, unique(as.vector(li))), nrow=I, ncol=K)
    lambda_partable <- data.frame(par_index=as.vector(lambda.index),
                        item=rep(1:I, K), class=rep(1:K, each=I) )
    lambda_partable$free <- 1
    lambda_partable[ duplicated(lambda_partable$par_index), 'free'] <- 0
    lambda_partable$block <- 0

    #- check whether there are constraints among item parameters
    item_con <- any( unlist( apply( lambda.index, 2,
                                FUN=function(ll){ length(unique(ll))<I } )))
    lambda_partable$item_con <- 1*item_con

    lambda_index_blocks    <- list()
    bb <- 0
    for (kk in 1:K){
        pkk <- lambda_partable[ (lambda_partable$class==kk ) & (lambda_partable$free==1),
                                        'par_index']
        if (length(pkk)>0){
            bb <- bb+1
            lambda_index_blocks[[bb]] <- pkk
            lambda_partable[ lambda_partable$par_index %in% pkk, 'block' ] <- bb
        }
    }
    lambda_partable$block <- lambda_partable$block * (lambda_partable$free > 0)
    if (item_con){
        lambda_partable$block <- lambda_partable$par_index * (lambda_partable$free > 0)
        lambda_index_blocks <- NULL
    }

    #-- parameter names
    lambda_partable$par_name <- paste0( rep(items, K), '_Cl', rep(1:K, each=I))

    #-- output
    res <- list(lambda.index=lambda.index, lambda_partable=lambda_partable,
                lambda_index_blocks=lambda_index_blocks)
    return(res)
}

## File Name: sirt_matrix_names.R
## File Version: 0.07

sirt_matrix_names <- function(x, row_names=NULL, col_names=NULL, extract_names=NULL)
{
    if ( ! is.null(extract_names) ){
        row_names <- rownames(extract_names)
        col_names <- colnames(extract_names)
    }
    if ( ! is.null(row_names) ){
        rownames(x) <- row_names
    }
    if ( ! is.null(col_names) ){
        colnames(x) <- col_names
    }
    return(x)
}

## File Name: xxirt_nr_pml_more_arguments.R
## File Version: 0.129

xxirt_nr_pml_more_arguments <- function(em_args, grad_fun)
{
    parindex <- em_args$partable$parindex
    parindex1 <- setdiff(parindex,0)
    if( length(parindex1)>length(unique(parindex1))){
        grad_fun <- NULL
        no_itempar_constraints <- FALSE
    } else {
        no_itempar_constraints <- TRUE
    }
    parindex_items <- em_args$parindex_items
    parindex_Theta <- em_args$parindex_Theta
    em_args$pml_args$in_par_Theta <- sapply(1L:em_args$NP, FUN=function(pp){
                                                pp %in% parindex_Theta },
                                                simplify=TRUE )
    #- index handling
    I <- em_args$I
    NPI <- em_args$NPI
    NP <- em_args$NP
    NI2 <- em_args$pml_args$NI2

    index_freq2 <- index_freq1 <- as.list(1L:NP)
    W2_long <- em_args$pml_args$W2_long

    for (pp in 1L:NP){
        if (pp<=NPI){
            item_pp <- em_args$item_index[[pp]]
            index_freq1[[pp]] <- item_pp - 1
            v1 <- sort( union( which( W2_long[,1] %in% item_pp ),
                                which( W2_long[,2] %in% item_pp ) ) )
            index_freq2[[pp]] <- v1 - 1
        } else {
            index_freq1[[pp]] <- (1L:I) - 1
            index_freq2[[pp]] <- (1L:NI2) - 1
        }
    }
    em_args$pml_args$index_freq1 <- index_freq1
    em_args$pml_args$index_freq2 <- index_freq2
    em_args$pml_args$no_itempar_constraints <- no_itempar_constraints

    #-- output
    res <- list(em_args=em_args, grad_fun=grad_fun)
    return(res)
}

## File Name: dif.strata.variance.R
## File Version: 0.153



#* stratified DIF variance
dif.strata.variance <- function( dif, se.dif, itemcluster )
{
    # stratified DIF variance
    # means in differential item functioning
    # itemcluster is a vector of strata corresponding to items
    stratadif <- stats::aggregate( 1+0*dif, list(itemcluster), sum, na.rm=TRUE )
    colnames(stratadif) <- c('strata', 'N.Items' )
    stratadif <- data.frame(stratadif)
    stratadif$M.DIF <- stats::aggregate( dif, list(itemcluster), mean, na.rm=TRUE )[,2]
    # DIF in strata
    SS <- nrow(stratadif)
    for (ss in 1L:SS){
        items.ss <- which( itemcluster==stratadif[ss,'strata']  )
        dif.ss <- dif[ items.ss ]
        difv.ss <- dif.variance( dif=dif.ss, se.dif=se.dif[ items.ss ] )
        stratadif$weighted.tau[ss] <- difv.ss$weighted.DIFSD
        stratadif$unweighted.tau[ss] <- difv.ss$unweighted.DIFSD
    }
    stratadif[ is.na(stratadif ) ] <- 0

    sd_ni1 <- stratadif$N.Items-1
    NI <- sum(stratadif$N.Items)
    weighted.DIFSD <- sum(stratadif$N.Items/NI*stratadif$weighted.tau)
    unweighted.DIFSD <- sum( sd_ni1/(NI-1)*stratadif$unweighted.tau)

    #-- output
    res <- list( stratadif=stratadif, weighted.DIFSD=weighted.DIFSD,
                    unweighted.DIFSD=unweighted.DIFSD)
    return(res)
}

## File Name: personfit.stat.R
## File Version: 0.10
personfit.stat <-
function( dat, abil, b ){
    dfr <- data.frame( "case"=seq( 1, nrow(dat)) )
    dfr$NItems <- rowSums( 1 - is.na(dat))
    dfr$mean <- rowMeans( dat, na.rm=TRUE )
    dfr$abil <- abil
    # person fit statistics
    pval <- colMeans( dat, na.rm=TRUE )
    dfr$caution <- pf.caution( data=dat, pval=pval )
    dfr$depend <- pf.dependability( data=dat, pval=pval )
    h1 <- pf.eci(data=dat, pval=pval, wle=abil, itemdiff=b)
    dfr <- cbind( dfr, h1$ECI )
    h1 <- pf.l0(data=dat, wle=abil, itemdiff=b)
    dfr$l0 <- h1$l0
    dfr$lz <- h1$lz
    h1 <- pf.outfit.infit(data=dat, wle=abil, itemdiff=b)
    dfr$outfit <- h1$Outfit
    dfr$infit <- h1$Infit
    dfr$rpbis <- pf.rpbis(data=dat, pval=pval)
    dfr$rpbis.itemdiff <- pf.rpbis.itemdiff(data=dat, itemdiff=b)
    dfr$U3 <- pf.U3( data=dat, pval=pval )
    return( dfr )
            }

## File Name: xxirt_parTheta_extract_freeParameters.R
## File Version: 0.08


xxirt_parTheta_extract_freeParameters <- function( customTheta )
{
    ind <- customTheta$est
    p1 <- customTheta$par[ ind ]
    names(p1) <- names(customTheta$par)[ind]
    return(p1)
}

## File Name: rm_grouped_expected_likelihood.R
## File Version: 0.06

rm_grouped_expected_likelihood <- function(pjk, n.ik, diffindex=NULL, eps=1E-30)
{
    ll0 <- rowSums( n.ik * log(pjk+eps) )
    if ( ! is.null(diffindex) ){
        ll0 <- rowsum(ll0, diffindex )[,1]
    }
    return(ll0)
}

## File Name: summary.invariance_alignment_constraints.R
## File Version: 0.12



summary.invariance_alignment_constraints <- function( object, digits=3, file=NULL, ...)
{
    # open sink for a file
    sirt_osink(file=file)

    display_string <- sirt_summary_print_display(symbol="-", len=65)
    cat(display_string)

    #- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    cat("Invariance Alignment with Post-Hoc Item Parameter Constraints\n\n")
    invariance_alignment_summary_optimization(object=object$model, digits=digits)

    cat(display_string)
    cat("Alignment Results Lambda Parameters \n\n")
    invariance_alignment_constraints_summary_print_item_summary(
            x=object$lambda_list, digits=digits)

    cat(display_string)
    cat("Alignment Results Nu Parameters \n\n")
    invariance_alignment_constraints_summary_print_item_summary(
            x=object$nu_list, digits=digits)

    sirt_csink(file=file)
}

## File Name: xxirt_compute_itemprobs.R
## File Version: 0.226


# compute item probabilities
xxirt_compute_itemprobs <- function( item_list, items, Theta, ncat,
        partable, partable_index, item_index=NULL )
{
    person_covariates <- attr(partable, 'person_covariates')
    TP <- nrow(Theta)
    maxK <- max(ncat)
    if ( is.null(item_index) ){
        I <- length(items)
        item_index <- 1L:I
    }
    I <- length(item_index)
    # compute item probabilities as a function of theta
    if (person_covariates){
        N <- nrow( (item_list[[1]])$X )
        dim_probs <- c(I,maxK,TP,N)
    } else {
        dim_probs <- c(I,maxK,TP)
    }
    probs <- array( 0, dim=dim_probs )
    for (jj in 1L:I){
        ii <- item_index[jj]
        item_ii <- item_list[[ii]]
        par_ii <- partable[ partable_index[[ii]], 'value' ]
        ncat_ii <- ncat[ii]
        arg_ii <- list( par=par_ii, Theta=Theta, ncat=ncat_ii )
        if (person_covariates){
            arg_ii$X <- item_ii$X
        }
        probs_ii <- do.call( what=item_ii$P, args=arg_ii )
        if (!person_covariates){
            probs[ jj, 1L:ncat_ii,] <- t(probs_ii)
        } else {
            probs[jj,,,] <- aperm(probs_ii, perm=c(2,1,3) )
        }
    }  # end jj
    return(probs)
}

## File Name: sia_sirt_remove_transitive.R
## File Version: 0.141


#**** remove transitive relations
sia_sirt_remove_transitive <- function(I1){
    I <- ncol(I1)
    diag(I1) <- 0
    BB <- 1
    IS <- I1
    iter <- 0
    while (BB > .0001){
        I0 <- IS
        iter <- iter + 1
        for (ii in 1L:I ){
            for (jj in 1L:I){
                if (ii!=jj ){
                    if ( sum( IS[ii,] * IS[,jj] ) > 0 ){
                            I1[ii,jj] <- 0
                    }
                }
            }
        }
        IS <- IS + IS %*% I1
        IS <- 1*(IS > 0 )
        BB <- sum( abs( I0 - IS ) )
    }
    return(I1)
}


.sia.remove.transitive <- sia_sirt_remove_transitive

## File Name: invgamma2.R
## File Version: 0.201



#---- inverse gamma distribution for variance
rinvgamma2 <- function( n, n0, var0 )
{
    res <- sirt_rinvgamma( n, shape=n0 / 2,  scale=n0 * var0 / 2 )
    return(res)
}
dinvgamma2 <- function( x, n0, var0 ){
    res <- sirt_dinvgamma( x, shape=n0 / 2,  scale=n0 * var0 / 2 )
    return(res)
}

#--- copied from MCMCpack package
sirt_dinvgamma <- function (x, shape, scale=1)
{
    if (shape <=0 | scale <=0) {
        stop('Shape or scale parameter negative in dinvgamma().\n')
    }
    alpha <- shape
    beta <- scale
    log.density <- alpha * log(beta) - lgamma(alpha) - (alpha + 1) * log(x) - (beta/x)
    return(exp(log.density))
}

sirt_rinvgamma <- function (n, shape, scale=1)
{
    return(1/stats::rgamma(n=n, shape=shape, rate=scale))
}

## File Name: dmlavaan_se_sandwich.R
## File Version: 0.116

dmlavaan_se_sandwich <- function(mod1, mod2, partable, label_parnames="parnames0",
        label_NPU="NPU", label_B="B", is_dmlavaan=TRUE)
{
    parnames1 <- mod1[[ label_parnames ]]
    NP1 <- mod1[[ label_NPU ]]
    parnames2 <- mod2[[ label_parnames ]]
    NP2 <- mod2[[ label_NPU ]]

    #-- compute standard error for difference
    scores <- cbind( mod1$scores, mod2$scores )
    N <- mod1$N
    A <- crossprod(scores)
    if (! is_dmlavaan){
        A <- A / N^4
    }
    B <- matrix(0, nrow=NP1+NP2, ncol=NP1+NP2)
    B[ 1L:NP1, 1L:NP1 ] <- mod1[[ label_B ]]
    B[ NP1 + 1L:NP2, NP1 + 1L:NP2 ] <- mod2[[ label_B ]]
    parnames0 <- c( paste0(parnames1, '_mod1'), paste0(parnames2, '_mod2') )
    res <- dmlavaan_sandwich_formula(A=A, B=B, parnames=parnames0)
    V <- res$V

    #-- now compute standard error for inference
    pars_sel <- which( ! is.na( partable$diff ) )

    pp <- pars_sel[1]
    for (pp in pars_sel){
        partable_pp <- partable[pp,]
        parname_pp <- partable_pp$parname
        ind_pp <- c( partable_pp$in_mod1, NP1 + partable_pp$in_mod2 )
        V_pp <- V[ ind_pp, ind_pp ]
        H_pp <- matrix( c(1,-1), nrow=1, ncol=2)
        DV_pp <- H_pp %*% V_pp %*% t(H_pp)
        partable$se_diff[pp] <- sqrt( DV_pp[1,1] )
    }
    partable$t <- partable$diff / partable$se_diff
    partable$p <- 2*stats::pnorm(-abs(partable$t))

    #--- output
    res <- list(partable=partable, V=V)
    return(res)
}

## File Name: lsem_fitsem_verbose_progress.R
## File Version: 0.03

lsem_fitsem_verbose_progress <- function(gg, G, pr, verbose)
{
    if (verbose){
        if (gg %in% pr ){
            cat('-')
        }
        if (gg>G){
            cat('|\n')
        }
        utils::flush.console()
    }
}

## File Name: gom.jml.R
## File Version: 0.26

########################################
# GOM JML
gom.jml <- function( dat, K=2, seed=NULL, globconv=.001,
        maxdevchange=.001, maxiter=600, min.lambda=.001,
        min.g=.001 ){
    s1 <- Sys.time()
    dev <- 0
    e1 <- environment()
    N <- dat.resp <- p.item <- score <- g.mean <- g1 <- gcut.distr <- NULL
    # data processing
    datproc <- res <- .gom.proc(dat, seed=seed,K)
    .sirt.attach.environment( res, envir=e1 )
    iter <- 0 ; conv <- 10
    devchange <- -100
        disp <- "...........................................................\n"

    #****
    # BEGIN Algorithm
    while( ( ( maxdevchange < devchange ) | (globconv < conv) ) &
                ( iter < maxiter )
                            ){
        cat(disp)
        cat("Iteration", iter+1, "   ", paste( Sys.time() ), "\n" )
        g0 <- g
        lambda0 <- lambda
        dev0 <- dev
        # update g
        g <- .gom.jml.est.g( lambda, g, N, I, K, dat, dat.resp, min.g )
        # update lambda
        lambda <- .gom.jml.est.lambda( lambda, g, N, I, K,  dat, dat.resp,
            min.lambda )
        # compute deviance
        dev <- .gom.deviance( lambda, g, K, dat, dat.resp )
        conv <- max( max( abs( g0-g) ), max( abs( lambda-lambda0) ) )
        devchange <- abs( ( dev - dev0 ) / dev0 )
        cat( paste( "   Deviance=", round( dev, 4 ),
            if (iter > 1 ){ " | Deviance change=" } else {""},
            if( iter>1){round( - dev + dev0, 6 )} else { ""},"\n",sep="") )
        cat( paste( "    Maximum membership parameter change=",
                    paste( round(max(abs(g-g0)),6), collapse=" " ), "\n", sep=""))
        cat( paste( "    Maximum probability change=",
                    paste( round(max(abs(lambda-lambda0)),6), collapse=" " ), "\n", sep=""))
        flush.console()
        iter <- iter+1
            }
    colnames(g) <- colnames(lambda) <- paste0("Class",1:K)
    #*****
    # information criteria
    ic <- list( "deviance"=dev, "n"=nrow(dat) )
    ic$K <- K
    ic$I <- I
    ic$np.item <- K*I
    ic$np.person <- (K-1)*N
    ic$np <- ic$np.item + ic$np.person
    # AIC
    ic$AICi <- dev + 2*ic$np.item
    ic$AICip <- dev + 2*ic$np
    ic$BICi <- dev + ( log(ic$n) )*ic$np.item
    # BIC
    ic$BIC <- dev + ( log(ic$n) )*ic$np
    # CAIC (conistent AIC)
    ic$CAIC <- dev + ( log(ic$n) + 1 )*ic$np
    # corrected AIC
    ic$AICc <- ic$AIC + 2*ic$np * ( ic$np + 1 ) / ( ic$n - ic$np - 1 )
    #***
    lambda <- data.frame("item"=colnames(dat), "p"=p.item, lambda )
    #***
    # discretizing membership functions

    res <- .gom.discret.membership(g,K,score)
    .sirt.attach.environment( res, envir=e1 )

    s2 <- Sys.time()
    res <- list( "lambda"=lambda, "g"=g, "g.mean"=g.mean,
            "gcut"=g1, "gcut.distr"=gcut.distr, "K"=K,
            "deviance"=dev, "ic"=ic, "N"=N, "score"=score, "iter"=iter,
            "datproc"=datproc,
            "s1"=s1, "s2"=s2)
    class(res) <- "gom.jml"
    return(res)
    }
#####################################################
#####################################################
summary.gom.jml <- function( object, ... ){
    cat("-----------------------------------------------------------------\n")
        d1 <- packageDescription("sirt")
        cat( paste( d1$Package, " ", d1$Version, " (", d1$Date, ")", sep=""), "\n\n" )
        cat( "Date of Analysis:", paste( object$s2 ), "\n" )
        cat("Computation time:", print(object$s2 - object$s1), "\n\n")
    cat("Grade of Membership Model (Joint Maximum Likelihood Estimation) \n")
    cat("   Function 'gdm.jml'\n")
    cat("-----------------------------------------------------------------\n")
    cat( "Number of iterations=", object$iter, "\n" )
    cat( "Deviance=", round( object$deviance, 2 ), " | " )
    cat( "Log Likelihood=", round( -object$deviance/2, 2 ), "\n" )
    cat( "Number of persons=", object$ic$n, "\n" )
    cat( "Number of items=", object$ic$I, "\n" )
    cat( "Number of classes=", object$ic$K, "\n" )
    cat( "Number of estimated parameters=", object$ic$np, "\n" )
    cat( "    Item parameters (ni)=", object$ic$np.item, "\n" )
    cat( "    Person parameters (np)=", object$ic$np.person, "\n" )
    cat( "AICi=", round( object$ic$AICi, 2 ), " | penalty=", round( object$ic$AICi - object$ic$deviance,2 ),
            "   | AICi=-2*LL + 2*(ni)  \n" )
    cat( "AICip=", round( object$ic$AICip, 2 ), " | penalty=", round( object$ic$AICip - object$ic$deviance,2 ),
            "   | AICip=-2*LL + 2*(ni+np)  \n" )
#    cat( "AICc=", round( object$ic$AICc, 2 )," | penalty=", round( object$ic$AICc - object$ic$deviance,2 ) )
#        cat("    | AICc=-2*LL + 2*p + 2*p*(p+1)/(n-p-1)  (bias corrected AIC)\n" )
    cat( "BICi=", round( object$ic$BICi, 2 ), " | penalty=", round( object$ic$BICi - object$ic$deviance,2 ),
            "   | BICi=-2*LL + log(n)*ni  \n" )
#    cat( "CAIC=", round( object$ic$CAIC, 2 )," | penalty=", round( object$ic$CAIC - object$ic$deviance,2 ) )
#        cat("   | CAIC=-2*LL + [log(n)+1]*p  (consistent AIC)\n\n" )
#    cat( "Trait Distribution\n",
#              "Mean=", 0, " SD=", round( object$sigma, 3) )
#    cat( "\nEAP Reliability=")
#    cat(round( object$EAP.rel,3 ) )
#    cat( "\n")
    cat("-----------------------------------------------------------------\n")
    cat("Item Parameters \n")
    obji <- object$lambda
    rownames(obji) <- NULL
    rvars <- seq( 2, ncol(obji ) )
    for (vv in rvars ){ obji[,vv] <- round( obji[,vv], 3 ) }
    print(obji)
    cat("-----------------------------------------------------------------\n")
    cat("Membership Scores\n\n")
    cat( "Class Proportions:\n" )
    print( round( object$g.mean,3)  )
    cat( "\nDistribution membership scores\n" )
    obji <- object$gcut.distr
#    obji <- object$rater
    rvars <- seq( 1, ncol(obji ) )
    for (vv in rvars ){ obji[,vv] <- round( obji[,vv], 2 ) }
    print(obji)
                }
#*******************************************************

## File Name: lsem_wald_test.R
## File Version: 0.04

lsem_wald_test <- function(theta, V, A)
{
    requireNamespace('MASS')
    r <- ( A %*% theta )[,1]
    W <- A %*% V %*% t(A)
    W1 <- MASS::ginv( X=W )
    chisq <- ( t(r) %*% W1 %*% r )[1,1]
    df <- nrow(A)
    p <- 1 - stats::pchisq(q=chisq, df=df)

    #-- output
    res <- list(chisq=chisq, df=df, p=p)
    return(res)
}

## File Name: noharm_sirt_preproc_pattern_matrix.R
## File Version: 0.182

noharm_sirt_preproc_pattern_matrix <- function(pattmat, minval=0, symm=FALSE)
{
    I <- nrow(pattmat)
    D <- ncol(pattmat)
    if (symm){
        pattmat0 <- pattmat
        pattmat1 <- t(pattmat)
        pattmat <- ifelse( pattmat0 < pattmat1, pattmat1, pattmat0)
        pattmat <- ( pattmat + t(pattmat) ) / 2
    }
    mp <- max(pattmat)
    ind <- ( pattmat > 0 ) & ( pattmat <=1 )
    if (sum(ind) > 0 ){
        DI <- prod(dim(pattmat))
        v1 <- matrix( 1L:DI, nrow=I )
        if (symm){
            v1 <- sirt_matrix_lower_to_upper(x=v1)
        }
        pattmat[ind] <- mp + v1[ind]
    }
    pattmat[ pattmat==0] <- NA
    elem <- stats::na.omit(as.vector(pattmat))
    if (length(elem)>0){
        pattmat <- matrix( match( pattmat, unique(elem) ), nrow=I) + minval
    }
    return(pattmat)
}

## File Name: xxirt_compute_posterior.R
## File Version: 0.301


##-- xxirt: compute posterior
xxirt_compute_posterior <- function( prior_Theta, p.xi.aj, group,
                G, weights, dat1, dat_resp, maxK, group_index, dat1_resp,
                eps=1e-100, customTheta=NULL)
{
    N <- nrow(dat_resp)
    TP <- ncol(p.xi.aj)
    I <- ncol(dat1)
    # posterior distribution
    if (customTheta$person_covariates){
        prior1 <- t(prior_Theta)
    } else {
        prior1 <- t( prior_Theta[, group ] )
    }

    # p.xi.aj[ is.na(p.xi.aj) ] <- eps
    p.aj.xi <- prior1 * p.xi.aj
    p1 <- p.aj.xi
    p.aj.xi <- p.aj.xi / rowSums( p.aj.xi )

    # expected counts
    n.ik <- array( 0, dim=c(I,maxK, TP,G) )
    N.ik <- array( 0, dim=c(I,maxK, TP) )
    pi.k <- matrix( 0, nrow=TP, ncol=G )
    for (gg in 1L:G){
        ind_gg <- group_index[[gg]]
        p.aj.xi.gg <- as.matrix( p.aj.xi[ind_gg, ] )
        dat1_resp_gg <- dat1_resp[ ind_gg,, ]
        weights_gg <- weights[ind_gg]
        for (kk in 1L:maxK){
            dat1_resp_gg_kk <- as.matrix(dat1_resp_gg[,,kk])
            n.ik[,kk,,gg] <- sirt_rcpp_xxirt_compute_posterior_expected_counts(
                                    dat1_resp_gg=dat1_resp_gg_kk, p_aj_xi_gg=p.aj.xi.gg,
                                    weights_gg=weights_gg)
        }
        N.ik <- N.ik + n.ik[,,,gg]
        pi.k[,gg] <- colSums( p.aj.xi.gg * weights[ ind_gg ] )
    }  # end gg
    res <- list( p.aj.xi=p.aj.xi, n.ik=n.ik, N.ik=N.ik, N.k=pi.k, post_unnorm=p1 )
    return(res)
}

## File Name: lsem.MGM.stepfunctions.R
## File Version: 0.132

#**** define object with LSEM step functions
lsem.MGM.stepfunctions <- function( object, moderator.grid )
{

    # object <- lsem.object
    moderator.grouped <- object$moderator.grouped
    parameters <- object$parameters

    dfr <- NULL
    G <- length(moderator.grid)

    for (gg in 1L:G){
        mod.gg <- moderator.grid[gg]
        ind.gg <- which( ( moderator.grouped$min <=mod.gg ) &
                            ( moderator.grouped$max > mod.gg ) )
        grouped.gg <-  object$moderator.grid[ind.gg]
        parameters.gg <- parameters[ parameters$moderator==grouped.gg, ]
        parameters.gg$moderator <- mod.gg
        dfr <- rbind( dfr, parameters.gg )
    }
    dfr <- dfr[ order( 10E20*dfr$parindex + dfr$moderator ), ]
    return(dfr)
}

## File Name: invariance_alignment_summary_optimization.R
## File Version: 0.083

invariance_alignment_summary_optimization <- function(object, digits)
{
    align.pow <- object$align.pow
    align.scale <- object$align.scale
    cat('Optimization Function Value', '=', round(object$fopt[1],digits), '\n' )
    cat('Optimizer', '=', object$res_optim$optimizer, '\n' )
    cat('Fixed', '=', object$fixed, '\n' )
    cat('Converged', '=', object$res_optim$converged, '\n' )
    cat('Alignment Power Values','=', round(align.pow[1], digits),
                            round(align.pow[2], digits), '\n')
    cat('Alignment Scale Values','=', round(align.scale[1], digits),
                            round(align.scale[2], digits), '\n')
    cat('Epsilon Value', '=', object$eps, '\n' )
    cat('Optimization method meth', '=', object$meth, '\n' )
}

## File Name: noharm_sirt_compute_chi_square_statistics.R
## File Version: 0.04

noharm_sirt_compute_chi_square_statistics <- function(res, residuals, pm,
    N, I, sumwgtm, modesttype, Nestpars)
{
    if (modesttype==1){
        RM <- residuals
        PV <- diag(pm)
        g1 <- sqrt( TAM::tam_outer( PV * (1-PV), PV * ( 1-PV ) ) )
        rM <- ( RM / g1  )
        zM <- 0.5 * log( 1 + rM ) - 0.5 * log( 1 - rM )
        # chi square
        res$chisquare <- X2 <- ( N - 3 ) * sum( zM^2 )
        res$df <- df <- I + sumwgtm - Nestpars$total
        res$chisquare_df  <- res$chisquare / res$df
        # calculate RMSEA
        res$rmsea <- rmsea <- sqrt(max(c( (X2 / res$Nobs ) / df - 1/res$Nobs, 0)))
        # calculate p values
        res$p.chisquare <- 1 - stats::pchisq( res$chisquare, df=res$df )
    }
    #--- output
    return(res)
}

## File Name: mgsem_modify_model.R
## File Version: 0.04


mgsem_modify_model <- function(model, group, entry, type, value)
{
    for (gg in group){
        for (ee in entry){
            for (tt in type){
                model[[gg+1]][[ee]][[tt]] <- value+0*model[[gg+1]][[ee]][[tt]]
            }  # end tt
        }  # end ee
    }  # end gg
    #--- output
    return(model)
}

## File Name: linking_2groups_haebara_grad.R
## File Version: 0.02


linking_2groups_haebara_grad <- function(x, pars, Theta, wgt, type="asymm",
        pow=2, eps=0.001, simultan=FALSE)
{
    res <- linking_2groups_haebara_fun(x=x, pars=pars, Theta=Theta, wgt=wgt, type=type,
        pow=pow, eps=eps, simultan=simultan, deriv=TRUE )
    return(res)
}

## File Name: sirt_pem_algorithm_compute_Pnew.R
## File Version: 0.03

sirt_pem_algorithm_compute_Pnew <- function( tt, P0, P1, P2)
{
    Pnew <- (1-tt)^2 * P0 + 2*tt*(1-tt)*P1 + tt^2 * P2
    return(Pnew)
}

## File Name: lsem.bootstrap.R
## File Version: 0.438


lsem.bootstrap <- function(object, R=100, verbose=TRUE, cluster=NULL,
        repl_design=NULL, repl_factor=NULL, use_starting_values=TRUE,
        n.core=1, cl.type="PSOCK")
{
    # fix seed locally
    s1 <- Sys.time()

    #* do not fix seed
    if (FALSE){
        old <- .Random.seed
        on.exit({.Random.seed <<- old})
        # set.seed(seed)
    }

    if (!is.null(repl_design)){
        R <- ncol(repl_design)
    }

    lsem_args <- object$lsem_args
    lsem_args$se <- 'none'
    lsem_args$verbose <- FALSE
    lsem_args$sampling_weights <- object$sampling_weights

    #- use starting values
    if (use_starting_values){
        partable_joint <- object$partable_joint
        if ( !is.null(partable_joint)){
            partable_joint$start <- partable_joint$est
            lsem_args$partable_joint <- partable_joint
        }
    }

    sampling_weights <- object$sampling_weights
    data <- object$data
    est_joint <- object$est_joint
    fitstats_joint <- object$fitstats_joint

    #-- bootstrap parameters_summary
    parameters_summary <- object$parameters_summary
    parameters_var_boot <- matrix(NA, nrow=nrow(parameters_summary), ncol=R)

    #-- create output arguments
    parameters <- object$parameters
    NP <- nrow(parameters)
    parameters_boot <- matrix(NA, nrow=NP, ncol=R)
    rownames(parameters_boot) <- rownames(parameters)
    if (est_joint){
        fitstats_joint_boot <- matrix(NA, nrow=nrow(fitstats_joint), ncol=R)
        rownames(fitstats_joint_boot) <- paste(fitstats_joint$stat)
    } else {
        fitstats_joint_boot <- NULL
    }

    #- moderator grid
    NG <- nrow(object$moderator.density)
    moderator_density_boot <- matrix(NA, nrow=NG, ncol=R)

    #-- loop over bootstrap samples
    lsem_bootstrap_print_start(verbose=verbose)

    repl_design_used <- matrix(NA, nrow=attr(data, 'N'), ncol=R)
    Nimp <- attr(data, 'Nimp')

    if (n.core==1){
        rr <- 1
        while (rr<=R){
            lsem_bootstrap_print_progress(rr=rr, verbose=verbose, R=R)
            #- draw bootstrap sample
            res <- lsem_bootstrap_draw_bootstrap_sample(data=data,
                                sampling_weights=sampling_weights, lsem_args=lsem_args,
                                cluster=cluster, repl_design=repl_design, rr=rr)
            lsem_args1 <- res$lsem_args1
            #- fit model
            mod1 <- try( do.call(what=lsem.estimate, args=lsem_args1), silent=TRUE)

            #- output collection
            if ( ! inherits(mod1,'try-error' ) ){
                parameters_boot[,rr] <- mod1$parameters$est
                fitstats_joint_boot[,rr] <- mod1$fitstats_joint$value
                parameters_var_boot[,rr] <- mod1$parameters_summary$SD^2
                repl_design_used[,rr] <- res$repl_vector
                moderator_density_boot[,rr] <- mod1$moderator.density$wgt
                rr <- rr + 1
            }
        }
    }
    if (n.core>1){
        m <- parallel::detectCores()
        if(n.core>m-1){
            n.core <- m-1
        }
        cl <- parallel::makeCluster(spec=n.core, type=cl.type)

        arglist <- list( data=data, sampling_weights=sampling_weights,
                        lsem_args=lsem_args, cluster=cluster, repl_design=repl_design )

        varlist <- c('arglist', 'lsem_bootstrap_draw_bootstrap_sample')
        parallel::clusterExport(cl=cl, varlist=varlist, envir=environment() )
        parallel::clusterEvalQ(cl, expr=library(sirt))

        fun_lsem_bootstrap <- function(i, arglist){
            iterate <- TRUE
            data <- arglist$data
            sampling_weights <- arglist$sampling_weights
            lsem_args <- arglist$lsem_args
            cluster <- arglist$cluster
            repl_design <- arglist$repl_design
            #-- estimate LSEM
            while (iterate){
                res <- lsem_bootstrap_draw_bootstrap_sample(data=data,
                                sampling_weights=sampling_weights,
                                lsem_args=lsem_args,
                                cluster=cluster, repl_design=repl_design, rr=i)
                lsem_args1 <- res$lsem_args1

                mod1 <- try( do.call(what=lsem.estimate, args=lsem_args1),
                                silent=TRUE)
                res_out <- list()
                if ( ! inherits(mod1,'try-error' ) ){
                    res_out$parameters_boot <- mod1$parameters$est
                    res_out$fitstats_joint_boot <- mod1$fitstats_joint$value
                    res_out$parameters_var_boot <- mod1$parameters_summary$SD^2
                    res_out$moderator_density_boot <- mod1$moderator.density$wgt
                    res_out$repl_design_used <- res$repl_vector
                    iterate <- FALSE
                }
            }
            return(res_out)
        }

        res_all <- sirt_parlapply(cl=cl, X=1L:R, FUN=fun_lsem_bootstrap,
                        verbose=verbose, arglist=arglist)
        parallel::stopCluster(cl)

        for (rr in 1L:R){
            res_out_rr <- res_all[[rr]]
            parameters_boot[,rr] <- res_out_rr$parameters_boot
            fitstats_joint_boot[,rr] <- res_out_rr$fitstats_joint_boot
            parameters_var_boot[,rr] <- res_out_rr$parameters_var_boot
            repl_design_used[,rr] <- res_out_rr$repl_design_used
            moderator_density_boot[,rr] <- res_out_rr$moderator_density_boot
        }

    }

    #- modify output objects
    res <- lsem_bootstrap_postproc_output( parameters=parameters,
                parameters_boot=parameters_boot, fitstats_joint=fitstats_joint,
                fitstats_joint_boot=fitstats_joint_boot, est_joint=est_joint,
                repl_factor=repl_factor, parameters_summary=parameters_summary,
                parameters_var_boot=parameters_var_boot )
    parameters <- res$parameters
    fitstats_joint <- res$fitstats_joint
    object$parameters_summary <- res$parameters_summary

    #- include new objects in output
    object$parameters_boot <- parameters_boot
    object$fitstats_joint_boot <- fitstats_joint_boot
    object$parameters <- parameters
    object$moderator_density_boot <- moderator_density_boot
    object$R <- R
    object$class_boot <- TRUE
    object$fitstats_joint <- fitstats_joint
    object$repl_design <- repl_design
    object$repl_factor <- repl_factor
    object$repl_design_used <- repl_design_used
    object$n.core <- n.core
    object$cl.type <- cl.type
    s2 <- Sys.time()
    object$s1 <- s1
    object$s2 <- s2
    object$time <- s2-s1

    #--- output
    class(object) <- c('lsem','lsem.boot')
    return(object)
}

## File Name: xxirt_coef.R
## File Version: 0.09

##########################################
# coef S3 method for xxirt objects
coef.xxirt <- function(object,...)
{
    par1 <- xxirt_partable_extract_freeParameters( object$partable )
    par2 <- xxirt_parTheta_extract_freeParameters( object$customTheta )
    par <- c(par1, par2)
    return(par)
}
##########################################

## File Name: plot.linking.robust.R
## File Version: 0.04


#--- linking.robust S3 plot method
plot.linking.robust <- function( x,  ... )
{
    graphics::par( mfrow=c(2,1))
    KK <- length(x$k.robust)
    graphics::plot( x$k.robust, x$meanpars[1:KK], type="l", xlab="k",
            ylab="Linking constant", main="Linking constant")
    graphics::points( 0, x$meanpars[1], pch=16, col=3, cex=1.4 )
    graphics::points( x$kopt, x$meanpars.kopt, pch=17, col=2, cex=1.4 )
    #****
    graphics::plot( x$k.robust, x$se[1:KK], type="l",
            main=paste0( "Standard error of linking constant (k_opt=", round(x$kopt, 3 ),")" ),
            xlab="k", ylab="Standard error")
    graphics::points( 0, x$se[1], pch=16, col=3, cex=1.4 )
    graphics::points( x$kopt, x$se.kopt, pch=17, col=2, cex=1.4 )
    graphics::par( mfrow=c(1,1) )
}

## File Name: sirt_matrix_lower_to_upper.R
## File Version: 0.03

sirt_matrix_lower_to_upper <- function(x)
{
    N <- ncol(x)
    for (ii in 1L:N){
        for (jj in ii:N){
            x[ii,jj] <- x[jj,ii]
        }
    }
    return(x)
}

## File Name: rm_sdt_evaluate_prior.R
## File Version: 0.07

rm_sdt_evaluate_prior <- function(partable)
{
    partable <- partable[ partable$est, ]
    m1 <- partable$prior_M
    sd1 <- partable$prior_SD
    y1 <- sirt_dnorm(partable$value, mean=m1, sd=sd1, log=TRUE)
    y2 <- sirt_dnorm(m1, mean=m1, sd=sd1, log=TRUE)
    res <- - ( y1 - y2 )
    return(res)
}

## File Name: modelfit.cor.R
## File Version: 2.38

#############################################################################
modelfit.cor.sirt.pml <-
function( data=NULL, posterior=NULL, probs=NULL, pmlobject=NULL ){

        data <- pmlobject$data
        itempairs <- as.data.frame( pmlobject$itempairs )
        ip0 <- itempairs
        itempairs$n11 <- itempairs$f11
        itempairs$n10 <- itempairs$f10
        itempairs$n01 <- itempairs$f01
        itempairs$n00 <- itempairs$f00
        itempairs$n <- rowSums( itempairs[, c("n11","n10", "n01","n00") ] )
        itempairs$Exp00 <- ip0$p00 * itempairs$n
        itempairs$Exp10 <- ip0$p10 * itempairs$n
        itempairs$Exp01 <- ip0$p01 * itempairs$n
        itempairs$Exp11 <- ip0$p11 * itempairs$n

        n <- itempairs$n
        m1 <- ( itempairs$n10 + itempairs$n11 ) / n
        m2 <- ( itempairs$n01 + itempairs$n11 ) / n
        t1 <- itempairs$n11 / n  - m1 * m2
        itempairs$corObs <- t1 / sqrt( m1 * ( 1 - m1 ) * m2 * ( 1-m2 ) )

        # observed correlation
        m1 <- ( itempairs$Exp10 + itempairs$Exp11 ) / n
        m2 <- ( itempairs$Exp01 + itempairs$Exp11 ) / n
        t1 <- itempairs$Exp11 / n  - m1 * m2
        itempairs$corExp <- t1 / sqrt( m1 * ( 1 - m1 ) * m2 * ( 1-m2 ) )

    #    m1 <- matrix( c(1,1,1,0,0,1,0,0), 4, 2, byrow=T )

        # define further quantities
        itempairs$X2 <- NA
        itempairs$RESIDCOV <- NA

        ##############################
        itempairs$X2 <- ( itempairs$n00 - itempairs$Exp00 )^2 / itempairs$Exp00 +
                        ( itempairs$n10 - itempairs$Exp10 )^2 / itempairs$Exp10    +
                        ( itempairs$n01 - itempairs$Exp01 )^2 / itempairs$Exp01    +
                        ( itempairs$n11 - itempairs$Exp11 )^2 / itempairs$Exp11

        itempairs$RESIDCOV <- ( itempairs$n11 * itempairs$n00 - itempairs$n10 * itempairs$n01 ) / itempairs$n^2 -
                    ( itempairs$Exp11 * itempairs$Exp00 - itempairs$Exp10 * itempairs$Exp01 ) / itempairs$n^2
        ##############################

        # labels
        itempairs$item1 <- colnames(data)[ itempairs$item1 ]
        itempairs$item2 <- colnames(data)[ itempairs$item2 ]

        # fisherz from psych package
        # residual of correlation
        itempairs$fcor <- sirt_fisherz( itempairs$corObs ) - sirt_fisherz( itempairs$corExp )

        #----
        # p values and p value adjustments adjustments

        # X2 statistic
        itempairs$X2_df <- 1
        itempairs$X2_p <- 1 - stats::pchisq(itempairs$X2, df=1 )
        itempairs$X2_p.holm <- stats::p.adjust( itempairs$X2_p, method="holm")
        itempairs$X2_sig.holm <- 1 * ( itempairs$X2_p.holm < .05 )
        itempairs$X2_p.fdr <- stats::p.adjust( itempairs$X2_p, method="fdr")
        # fcor statistic
        itempairs$fcor_se <- ( itempairs$n - 3 )^(-1/2)
        itempairs$fcor_z <- itempairs$fcor / itempairs$fcor_se
        itempairs$fcor_p <- 1 - stats::pnorm( abs(itempairs$fcor_z ) )
        itempairs$fcor_p.holm <- stats::p.adjust( itempairs$fcor_p, method="holm")
        itempairs$fcor_p.fdr <- stats::p.adjust( itempairs$fcor_p, method="fdr")

        #**********************
        # model fit
        modelfit <- data.frame( "est"=c(
                mean( abs( itempairs$corObs - itempairs$corExp ) ),
                sqrt( mean( ( itempairs$corObs - itempairs$corExp )^2 ) ),
                mean( itempairs$X2 ), # mean( itempairs$G2),
                mean( 100*abs(itempairs$RESIDCOV ) ) )
                            )
        rownames(modelfit) <- c("MADcor", "SRMSR", "MX2", # "MG2",
                    "100*MADRESIDCOV" )

    #*****
    # summary statistics
    modelfit.test <- data.frame("type"=c("max(X2)","abs(fcor)"),
            "value"=c( max( itempairs$X2), max( abs(itempairs$fcor) )  ),
            "p"=c( min( itempairs$X2_p.holm), min( itempairs$fcor_p.holm)  ) )
    #****
    # print results
    res <- list( "modelfit.stat"=modelfit, "itempairs"=itempairs,
        "modelfit.test"=modelfit.test  )
    return(res)
    }
#######################################################################

#######################################################################
# auxiliary function: weighted correlation
.corr.wt <- function( x, y, w=rep(1,length(x))) {
#  stopifnot(length(x)==dim(y)[2] )
  w <- w / sum(w)
  # Center x and y, using the weighted means
  x <- x - sum(x * w)
  ty <- y - sum( y * w)
  # Compute the variance
  vx <- sum(w * x * x)
  vy <- sum(w * ty * ty)
  # Compute the covariance
  vxy <- sum(ty * x * w)
  # Compute the correlation
  vxy / sqrt(vx * vy)
}
##################################################################################

## File Name: invariance.alignment.R
## File Version: 4.019


invariance.alignment <- function( lambda, nu, wgt=NULL,
    align.scale=c(1,1), align.pow=c(.5,.5), eps=1e-3,
    psi0.init=NULL, alpha0.init=NULL, center=FALSE, optimizer="optim",
    fixed=NULL, meth=1, vcov=NULL, eps_grid=seq(0,-10, by=-.5),
    num_deriv=FALSE, ... )
{
    CALL <- match.call()
    s1 <- Sys.time()
    type <- 'AM'
    align.pow0 <- align.pow
    align.pow <- align.pow / 2
    overparam <- FALSE
    le <- FALSE

    #-- labels for groups and items
    lambda0 <- lambda <- invariance_alignment_proc_labels(x=lambda)
    nu0 <- nu <- invariance_alignment_proc_labels(x=nu)

    #-- weights
    G <- nrow(lambda)   # number of groups
    I <- ncol(lambda)   # number of items
    if ( is.null(wgt) ){
        wgt <- 1+0*nu
    }

    #- reparametrization of meth argument
    meth0 <- meth
    if (meth0==4){ meth <- 0}     # Mplus FREE
    if (meth0==3){ meth <- 0.5}      # Mplus FIXED

    #- choose fixed value
    fixed <- invariance_alignment_choose_fixed(fixed=fixed, G=G, Gmax=999)
    reparam <- ! fixed
    if (meth%in%c(0,0.5)){
        constraint <- 'prod'
        reparam <- TRUE
        num_deriv <- TRUE
    }
    if (overparam){
        num_deriv <- TRUE
    }

    W1 <- dim(wgt)
    wgtM <- matrix( colSums(wgt,na.rm=TRUE), nrow=W1[1], ncol=W1[2], byrow=TRUE )
    # wgtM <- matrix( 1, nrow=W1[1], ncol=W1[2], byrow=TRUE )
    wgtM <- wgt / wgtM

    wgt <- wgtM

    # missing indicator matrix: 1 - no missings
    missM <- 0.5 * ( (1-is.na(lambda))+ (1- is.na(nu)) )
    wgt <- wgt * missM
    wgt[ missM==0 ] <- 0
    numb_items <- rowSums(missM)

    lambda[ missM==0 ] <- mean( lambda, na.rm=TRUE )
    nu[ missM==0 ] <- mean( nu, na.rm=TRUE )
    group.combis <- t( utils::combn(x=G, m=2))
    group.combis <- rbind( group.combis, group.combis[,c(2,1) ] )
    group_combis <- group.combis

    #--- initial estimates means and SDs
    psi0 <- apply(lambda, 1, stats::median)
    alpha0 <- apply(nu, 1, stats::median, na.rm=TRUE)
    psi0 <- psi0 / psi0[1]
    alpha0 <- alpha0 - alpha0[1]
    if ( ! is.null( psi0.init) ){ psi0 <- psi0.init }
    if ( ! is.null( alpha0.init) ){ alpha0 <- alpha0.init }
    lambda <- as.matrix(lambda)
    wgt <- as.matrix(wgt)

    wgt_combi <- matrix(NA, nrow=nrow(group.combis), ncol=ncol(lambda) )
    for (ii in 1L:I){
        wgt_combi[,ii] <- wgt[ group.combis[,1], ii]*wgt[ group.combis[,2], ii]
    }

    group_combis <- group.combis-1
    G1 <- G-1
    if (overparam){
        G1 <- G
    }
    ind_alpha <- seq_len(G1)
    ind_psi <- G1 + ind_alpha
    if (meth==0){
        ind_alpha <- seq_len(G)
        ind_psi <- G + seq_len(G1)
    }

    #-- define optimization functions
    ia_fct_optim <- function(x, lambda, nu, overparam, eps, meth_,
                        item_wise=FALSE ){
        res <- invariance_alignment_define_parameters(x=x, ind_alpha=ind_alpha,
                        ind_psi=ind_psi, reparam=reparam, meth=meth_)
        alpha0 <- res$alpha0
        psi0 <- res$psi0
        val <- sirt_rcpp_invariance_alignment_opt_fct( nu=nu, lambda=lambda,
                        alpha0=alpha0, psi0=psi0, group_combis=group_combis,
                        wgt=wgt, align_scale=align.scale,
                        align_pow=align.pow, eps=eps, wgt_combi=wgt_combi, type=type,
                        reparam=FALSE, meth=meth_)
        if (item_wise){
            val <- val$fopt_item
        } else {
            val <- val$fopt
        }
        if (overparam | meth==0 ){
            G <- nrow(lambda)
            fac <- sum(wgt[,1]) / 1000
            val <- val+fac*sum(x[1L:G]^2)
        }
        return(val)
    }

    ia_grad_optim <- function(x, lambda, nu, overparam, eps, meth_){
        res <- invariance_alignment_define_parameters(x=x, ind_alpha=ind_alpha,
                            ind_psi=ind_psi, reparam=reparam)
        alpha0 <- res$alpha0
        psi0 <- res$psi0
        grad <- sirt_rcpp_invariance_alignment_opt_grad( nu=nu, lambda=lambda,
                        alpha0=alpha0, psi0=psi0, group_combis=group_combis, wgt=wgt,
                        align_scale=align.scale, align_pow=align.pow, eps=eps,
                        wgt_combi=wgt_combi, type=type, reparam=reparam, meth=meth_)
        grad <- grad[-c(1,G+1)]
        return(grad)
    }
    if ( align.pow[1]==0){
        ia_grad_optim <- NULL
    }
    if ((meth>=2) | num_deriv ){
        ia_grad_optim <- NULL
    }

    #** evaluate optimization function at initial solution
    x0 <- c( alpha0[-1], psi0[-1] )
    if (overparam){
        x0 <- c(alpha0, psi0)
    }
    if (meth==0){
        x0 <- c( alpha0, psi0[-1] )
    }
    fct_optim_inits <- ia_fct_optim(x=x0, lambda=lambda, nu=nu,
                            overparam=overparam, eps=eps, meth=meth )

    #* estimate alignment parameters
    min_val <- .01
    GL <- G1
    par <- c( alpha0[-1], psi0[-1] )
    if (overparam){
        GL <- G
        par <- c( alpha0, psi0 )
    }
    if (meth==0){
        par <- c( alpha0, psi0[-1] )
    }
    lower <- c(rep(-Inf,GL), rep(min_val, GL))
    if (reparam){
        grad_optim <- NULL
    }
    if (meth==0){
        lower <- c(rep(-Inf,G), rep(min_val, GL))
    }
    #* define sequence of epsilon values
    eps_vec <- 10^eps_grid
    eps_vec <- sirt_define_eps_sequence(eps=eps, eps_vec=eps_vec)

    #- optimize (with useful starting values)
    lambda1 <- lambda
    nu1 <- nu
    est_loop <- 1
    psi_list <- list()

    while(est_loop>=1){
        for (eps in eps_vec){
            res_optim <- sirt_optimizer(optimizer=optimizer, par=par, fn=ia_fct_optim,
                                grad=ia_grad_optim, lower=lower, hessian=FALSE,
                                lambda=lambda1, nu=nu1, overparam=overparam,
                                eps=eps, meth_=meth, ...)
            par <- res_optim$par
            res <- invariance_alignment_define_parameters(x=res_optim$par,
                            ind_alpha=ind_alpha, ind_psi=ind_psi, reparam=reparam,
                            meth=meth)
            alpha0 <- res$alpha0
            psi0 <- res$psi0
        } # end eps grid loop
        if (meth%in%c(0,0.5,1,2)){
            est_loop <- 0
        }
        if (meth>=3){
            psi_list[[est_loop]] <- psi0
            if (est_loop==2){
                est_loop <- 0
            }
            if (est_loop==1){
                est_loop <- est_loop + 1
                # nu1 <- nu/sqrt(mod1)*psi_list[[1]]
                #=> likely dead code
            }
        }
    }  # end while loop

    if (meth==3){
        psi0 <- psi_list[[1]]
    }

    #-- standard error computation (if requested)
    TEjkbc <- LEjkbc <- TEjk <- LEjk <- TE <- TEbc <- LEbc <- LE <- SE <- NULL
    vcov0 <- vcov
    V_TEjkbc <- V_TEjk <- V_LEjk <- V_TEbc <- V_TE <- V_LEbc <- V_LE <- NULL
    V_LEjkbc <- NULL

    if (! is.null(vcov)){

        names(par) <- c( paste0('alpha',2L:G), paste0('psi',2L:G) )
        NP <- length(par)

        #- gradient computation
        ia_grad_optim_num <- function(x, lambda, nu, overparam, eps, meth=1,
                                h=1e-4, item_wise=FALSE){
            NP <- length(x)
            par <- x
            if (!item_wise){
                grad <- rep(0,NP)
                names(grad) <- names(x)
            } else {
                grad <- matrix(0,nrow=NP, ncol=I)
                rownames(grad) <- names(x)
            }
            args <- list(x=par, lambda=lambda, nu=nu, overparam=overparam, eps=eps,
                                meth_=meth, item_wise=item_wise)
            for (pp in 1L:NP){
                args$x <- mgsem_add_increment(x=par, h=h, i1=pp)
                f1 <- do.call(what=ia_fct_optim, args=args)
                args$x <- mgsem_add_increment(x=par, h=-h, i1=pp)
                f2 <- do.call(what=ia_fct_optim, args=args)
                der <- (f1-f2)/(2*h)
                if (!item_wise){
                    grad[pp] <- der
                } else {
                    grad[pp,] <- der
                }
            }
            return(grad)
        }

        #- compute hessian with respect to par
        h <- 1e-4
        hess_par <- matrix(NA, nrow=NP, ncol=NP)
        rownames(hess_par) <- colnames(hess_par) <- names(par)
        args <- list(x=par, lambda=lambda, nu=nu, overparam=overparam, eps=eps,
                        meth=meth)
        pp <- 1
        for (pp in 1L:NP){
            args$x <- mgsem_add_increment(x=par, h=h, i1=pp)
            f1 <- do.call( what=ia_grad_optim_num, args=args)
            args$x <- mgsem_add_increment(x=par, h=-h, i1=pp)
            f2 <- do.call( what=ia_grad_optim_num, args=args)
            hess_par[,pp] <- (f1-f2)/(2*h)
        }
        #- compute hessian with respect to lambda and nu
        hess_theta <- matrix(NA, nrow=NP, ncol=2*G*I)
        rownames(hess_theta) <- names(par)
        colnames(hess_theta) <- rownames(vcov)

        pp <- 1

        args <- list(x=par, lambda=lambda, nu=nu, overparam=overparam, eps=eps)

        for (gg in 1L:G){
            for (subs in c('lambda','nu')){
                for (ii in 1L:I){
                    if (subs=='lambda'){
                        z <- lambda
                    } else {
                        z <- nu
                    }
                    args[[subs]] <- mgsem_add_increment(x=z, h=h, i1=gg, i2=ii)
                    f1 <- do.call( what=ia_grad_optim_num, args=args)
                    args[[subs]] <- mgsem_add_increment(x=z, h=-h, i1=gg, i2=ii)
                    f2 <- do.call( what=ia_grad_optim_num, args=args)
                    hess_theta[,pp] <- (f1-f2)/(2*h)
                    pp <- pp+1
                }
            }
        }

        H1 <- MASS::ginv(hess_par)
        A <- H1 %*% hess_theta
        vcov <- A %*% vcov0 %*% t(A)
        rownames(vcov) <- colnames(vcov) <- names(par)
        SE <- sqrt_diag(x=vcov)

        #--- linking error based on M-estimation
        if (le){
            args1 <- list(x=par, lambda=lambda, nu=nu, overparam=overparam, eps=eps,
                            meth=meth, item_wise=TRUE)

            # item-wise gradient
            grad_item <- do.call( what=ia_grad_optim_num, args=args1)
            # variance matrix (meat matrix) for linking error estimation
            M <- 0*vcov
            Mbc <- M
            for (ii in 1L:I){
                gii <- grad_item[,ii]
                Mii <- outer(gii,gii)
                ind <- seq(ii,2*I*G,I)
                vcov0_ii <- vcov0[ind, ind]
                hess_theta_ii <- hess_theta[,ind]
                Mbcii <- hess_theta_ii %*% vcov0_ii %*% t(hess_theta_ii)
                M <- M + Mii
                Mbc <- Mbc + Mbcii
            }
            V_LE <- I/(I-1) * H1 %*% M %*% t(H1)
            V_LEbc <- I/(I-1) * H1 %*% (M-Mbc) %*% t(H1)
            LE <- sqrt_diag(x=V_LE, names=names(par))
            LEbc <- sqrt_diag(x=V_LEbc, names=names(par))

            V_TE <- vcov + V_LE
            V_TEbc <- vcov + V_LEbc
            TE <- sqrt(SE^2 + LE^2)
            TEbc <- sqrt(SE^2 + LEbc^2)

            #--- jackknife linking error

            hess_par_item <- list()
            for (ii in 1L:I){
                hess_par_item[[ii]] <- 0*hess_par
            }

            H0 <- 0*hess_par
            pp <- 1
            for (pp in 1L:NP){
                args1$x <- mgsem_add_increment(x=par, h=h, i1=pp)
                f1 <- do.call( what=ia_grad_optim_num, args=args1)
                args1$x <- mgsem_add_increment(x=par, h=-h, i1=pp)
                f2 <- do.call( what=ia_grad_optim_num, args=args1)
                der <- (f1-f2)/(2*h)
                for (ii in 1L:I){
                    hess_par_item[[ii]][,pp] <- der[,ii]
                }
            }
            for (ii in 1L:I){
                H0 <- H0 + hess_par_item[[ii]]
            }
            # hess_par=H0

            # jk le
            jkfac <- (I-1)/I
            estdiff_jk <- matrix(NA, nrow=I, ncol=NP)
            V_LEjkbc <- 0*V_LE

            for (ii in 1L:I){
                cii <- grad_item[,ii]
                Aii <- hess_par - hess_par_item[[ii]]
                A1ii <- MASS::ginv(Aii)
                parii <- A1ii %*% cii
                estdiff_jk[ii,] <- parii[,1]
                # correction: see LESL paper
                U <- A
                ind <- seq(ii, 2*I*G, I )
                HTii <- hess_theta
                HTii[,ind] <- 0
                Ui <- A1ii %*% HTii
                Wi <- Ui-U
                V_LEjkbc <- V_LEjkbc + jkfac*( Wi %*% vcov0 %*% t(Wi) )
            }
            V_LEjk <- jkfac*crossprod(estdiff_jk)
            LEjk <- sqrt_diag(x=V_LEjk, names=names(par))
            V_LEjkbc <- V_LEjk - V_LEjkbc
            LEjkbc <- sqrt_diag(x=V_LEjkbc, names=names(par))

            TEjk <- sqrt( SE^2 + LEjk^2)
            TEjkbc <- sqrt( SE^2 + LEjkbc^2)

        }  # end le

    } # end vcov

    # center parameters
    res <- invariance_alignment_center_parameters(alpha0=alpha0, psi0=psi0,
                    center=center, meth=meth )
    alpha0 <- res$alpha0
    psi0 <- res$psi0

    # define aligned parameters
    res <- sirt_rcpp_invariance_alignment_opt_fct( nu=nu, lambda=lambda, alpha0=alpha0,
                        psi0=psi0, group_combis=group_combis, wgt=wgt,
                        align_scale=align.scale, align_pow=align.pow, eps=eps,
                        wgt_combi=wgt_combi, type=type, reparam=reparam, meth=1)
    lambda.aligned <- invariance_alignment_process_parameters(par.aligned=res$lambda,
                            par=lambda0)
    nu.aligned <- invariance_alignment_process_parameters(par.aligned=res$nu, par=nu0)
    fopt <- res$fopt

    #**** calculate item statistics and R-squared measures
    # groupwise aligned loading
    # average aligned parameter
    c1 <- invariance_alignment_aligned_parameters_summary(x=lambda.aligned,
                    label='lambda')
    c2 <- invariance_alignment_aligned_parameters_summary(x=nu.aligned, label='nu')
    itempars.aligned <- data.frame(c1, c2, row.names=colnames(lambda) )
    M.lambda_matr <- sirt_matrix2( itempars.aligned$M.lambda, nrow=G)
    M.nu_matr <- sirt_matrix2( itempars.aligned$M.nu, nrow=G)
    lambda.resid <- lambda.aligned - M.lambda_matr
    nu.resid <- nu.aligned - M.nu_matr

    # R-squared measures
    Rsquared.invariance <- c(NA,NA)
    names(Rsquared.invariance) <- c('loadings', 'intercepts' )
    expl <- psi0 * M.lambda_matr
    Rsquared.invariance['loadings'] <- sirt_rsquared(x=lambda, expl=expl)
    expl <- M.nu_matr + alpha0 * M.lambda_matr
    Rsquared.invariance['intercepts'] <- sirt_rsquared(x=nu, expl=expl)

    # correlations aligned parameters
    rbar <- c( invariance_alignment_calc_corr(t(lambda.aligned)),
                    invariance_alignment_calc_corr(t(nu.aligned)) )
    es.invariance <- rbind( Rsquared.invariance, sqrt(1-Rsquared.invariance), rbar)
    rownames(es.invariance) <- c('R2', 'sqrtU2', 'rbar')

    pars <- data.frame(alpha0=alpha0, psi0=psi0)
    rownames(pars) <- rownames(lambda)

    s2 <- Sys.time()
    time_diff <- s2-s1

    #----- OUTPUT:
    res <- list( pars=pars, itempars.aligned=itempars.aligned,
            es.invariance=es.invariance, center=center, lambda.aligned=lambda.aligned,
            lambda.resid=lambda.resid, nu.aligned=nu.aligned, lambda=lambda0,
            nu=nu0, nu.resid=nu.resid, fopt=fopt, align.scale=align.scale,
            align.pow=align.pow0, res_optim=res_optim, eps=eps, wgt=wgt,
            miss_items=missM, numb_items=numb_items, vcov=vcov, V_LE=V_LE,
            V_LEbc=V_LEbc, V_TE=V_TE, V_LEjk=V_LEjk, V_LEjkbc=V_LEjkbc,
            SE=SE, LE=LE, LEbc=LEbc, LEjk=LEjk, LEjkbc=LEjkbc,
            TE=TE, TEbc=TEbc, TEjk=TEjk, TEjkbc=TEjkbc,
            fct_optim_inits=fct_optim_inits, fixed=fixed, meth=meth0,
            meth_internal=meth, s1=s1, s2=s2, time_diff=time_diff, CALL=CALL)
    class(res) <- 'invariance.alignment'
    return(res)
}

## File Name: dm_m_est.R
## File Version: 0.106

dm_m_est <- function(mod1, mod2)
{
    #*** create joint parameter table
    res <- dmlavaan_joint_parameterTable(mod1=mod1, mod2=mod2,
                    label_parnames='parnames')
    partable <- res$partable
    parnames <- res$parnames
    NP <- res$NP

    #*** sandwich estimate
    res <- dmlavaan_se_sandwich(mod1=mod1, mod2=mod2, partable=partable,
                    label_parnames='parnames', label_NPU='NP', label_B='B',
                    is_dmlavaan=TRUE)
    partable <- res$partable
    V <- res$partable

    #--- output
    res <- list(partable=partable, V=V, parnames=parnames, NP=NP)
    return(res)
}

## File Name: sirt_pem_include_ll_args.R
## File Version: 0.06

sirt_pem_include_ll_args <- function(ll_args, pem_parm, pem_pars, pem_parameter_index)
{
    for (pp in pem_pars){
        ll_args[[ pp ]] <- sirt_pem_extract_parameters( parm=pem_parm, parmgroup=pp,
                                    pem_parameter_index=pem_parameter_index )
    }
    return(ll_args)
}

## File Name: mgsem_partable2coef.R
## File Version: 0.05

mgsem_partable2coef <- function(partable)
{
    dfr <- partable
    ind <- which(dfr$unique==1)
    coef <- dfr[ ind,'est']
    names(coef) <- dfr[ind,'name']
    return(coef)
}

## File Name: print.xxirt.R
## File Version: 0.11


print.xxirt <- function(x, ... )
{
    object <- x
    d1 <- utils::packageDescription("sirt")
    cat( paste( d1$Package, " ", d1$Version, " (", d1$Date, ")", sep=""), "\n\n" )

    cat("Call:\n", paste(deparse(object$CALL), sep="\n", collapse="\n"),
                "\n\n", sep="")

    cat( object$ic$n, "Cases, ", object$ic$I, "Items, ",
                object$G, "Group(s)", "\n")

    #*** parameters
    cat( "Number of estimated parameters=", object$ic$np, "\n" )
    cat( "  Number of estimated item parameters=", object$ic$np.item,
                "\n" )
    cat( "  Number of estimated distribution parameters=", object$ic$np.Theta,
                "\n\n" )

    #*** likelihood
    cat( paste0( "Log-Likelihood=", round( x$loglike,2 ), "\n") )
    #*** information criteria
    cat( paste0( "AIC=", round( x$ic$AIC,0 ), "\n") )
    cat( paste0( "BIC=", round( x$ic$BIC,0 ), "\n") )
    invisible(x)
}

## File Name: lsem.estimate.R
## File Version: 1.113

# estimate LSEM model
lsem.estimate <- function( data, moderator, moderator.grid,
        lavmodel, type="LSEM", h=1.1, bw=NULL, residualize=TRUE,
        fit_measures=c("rmsea","cfi","tli","gfi","srmr"), standardized=FALSE,
        standardized_type="std.all", lavaan_fct="sem", sufficient_statistics=TRUE,
        pseudo_weights=0, sampling_weights=NULL,
        loc_linear_smooth=TRUE, est_joint=FALSE, par_invariant=NULL, par_linear=NULL,
        par_quadratic=NULL, partable_joint=NULL, pw_linear=1,
        pw_quadratic=1, pd=TRUE, est_DIF=FALSE, se=NULL, kernel="gaussian", eps=1E-8,
        verbose=TRUE, ... )
{
    lsem_args <- c(as.list(environment()), list(...))
    CALL <- match.call()
    s1 <- Sys.time()
    lavaan.args <- list(...)
    if (standardized){
        if ( type=='MGM'){
            stop('standardized=TRUE cannot be applied for type=\'MGM\'')
        }
    }
    use_lavaan_survey <- FALSE

    #- check if list of imputed datasets is available
    is_imputed <- ! ( is.list(data) & is.data.frame(data) )

    #- data cleaning
    if (!is_imputed){
        data <- as.data.frame(data)
        data <- data[ ! is.na(data[,moderator]), ]
        moderator_variable <- data[,moderator]
        Nimp <- 0
    } else {
        moderator_variable <- NULL
        Nimp <- length(data)
    }

    #- process arguments
    res <- lsem_estimate_proc_args( lavaan.args=lavaan.args,
                sufficient_statistics=sufficient_statistics,
                pseudo_weights=pseudo_weights, lavmodel=lavmodel, data=data,
                use_lavaan_survey=use_lavaan_survey, est_joint=est_joint,
                par_invariant=par_invariant, par_linear=par_linear,
                par_quadratic=par_quadratic, partable_joint=partable_joint,
                moderator.grid=moderator.grid, se=se, verbose=verbose,
                is_imputed=is_imputed)
    sufficient_statistics <- res$sufficient_statistics
    use_lavaan_survey <- res$use_lavaan_survey
    variables_model <- res$variables_model
    use_pseudo_weights <- res$use_pseudo_weights
    variables_ordered <- res$variables_ordered
    est_joint <- res$est_joint
    partable <- res$partable
    has_meanstructure <- res$has_meanstructure
    se <- res$se
    compute_se <- res$compute_se
    pseudo_weights <- res$pseudo_weights
    some_ordinal <- res$some_ordinal

    # group moderator if type='MGM'
    out <- lsem_group_moderator( data=data, type=type, moderator.grid=moderator.grid,
                moderator=moderator, residualize=residualize, h=h,
                is_imputed=is_imputed, Nimp=Nimp )
    data <- out$data
    moderator.grouped <- out$moderator.grouped
    h <- out$h
    residualize <- out$residualize
    moderator.grid <- out$moderator.grid

    # residualize input data
    out <- lsem_residualize( data=data, moderator=moderator,
                    moderator.grid=moderator.grid, lavmodel=lavmodel, h=h, bw=bw,
                    residualize=residualize, eps=eps, verbose=verbose,
                    sampling_weights=sampling_weights, kernel=kernel,
                    variables_model=variables_model, is_imputed=is_imputed,
                    Nimp=Nimp)
    G <- out$G
    data <- out$data
    weights <- out$weights
    residualized_intercepts <- out$residualized_intercepts
    N <- out$N
    bw <- out$bw
    h <- out$h
    moderator.density <- out$moderator.density
    sampling_weights <- out$sampling_weights
    no_sampling_weights <- out$no_sampling_weights
    m.moderator <- out$m.moderator
    sd.moderator <- out$sd.moderator
    data$index <- seq(1,N)

    # unweighted fit of lavaan model
    dat <- data
    lavmodel__ <- lavmodel

    #* extract estimation function
    lavaan_est_fun <- lsem_define_lavaan_est_fun(lavaan_fct=lavaan_fct)

    #* fit initial lavaan model
    lavfit <- lsem_fit_initial_model( lavmodel__=lavmodel__,
                    lavaan_est_fun=lavaan_est_fun, dat=dat,
                    variables_model=variables_model, sampling_weights=sampling_weights,
                    has_meanstructure=has_meanstructure,
                    sufficient_statistics=sufficient_statistics, est_joint=est_joint,
                    se=se, use_lavaan_survey=use_lavaan_survey,
                    is_imputed=is_imputed, Nimp=Nimp, ... )
    nobs <- unlist(lavfit@Data@nobs)

    # extract variables which are in model and data frame
    pars <- sirt_import_lavaan_parameterEstimates(object=lavfit)
    fM <- sirt_import_lavaan_fitMeasures(object=lavfit)
    fit_measures <- intersect( fit_measures, names(fM) )
    fitstat <- fM[ fit_measures ]
    NF <- length(fit_measures)

    if (standardized){
        sol <- sirt_import_lavaan_standardizedSolution( object=lavfit,
                            type=standardized_type)
        colnames(sol)[ which( colnames(sol)=='est.std' ) ] <- 'est'
        sol$lhs <- paste0( 'std__', sol$lhs)
        pars <- sirt_rbind_fill( x=pars, y=sol )
    }
    pars <- apply( pars[, c('lhs', 'op', 'rhs' ) ], 1, FUN=function(ll){
                        paste0( ll[1], ll[2], ll[3] ) } )

    # fit LSEM for all moderator groups
    out2 <- lsem_fitsem( dat=dat, weights=weights, lavfit=lavfit,
                    fit_measures=fit_measures, NF=NF, G=G, moderator.grid=moderator.grid,
                    verbose=verbose, pars=pars, standardized=standardized,
                    variables_model=variables_model,
                    sufficient_statistics=sufficient_statistics,
                    lavaan_fct=lavaan_fct, lavmodel=lavmodel,
                    use_lavaan_survey=use_lavaan_survey,
                    pseudo_weights=pseudo_weights, est_joint=est_joint,
                    par_invariant=par_invariant, par_linear=par_linear,
                    par_quadratic=par_quadratic, partable_joint=partable_joint,
                    pw_linear=pw_linear, pw_quadratic=pw_quadratic,
                    se=se, moderator_variable=moderator_variable,
                    loc_linear_smooth=loc_linear_smooth, pd=pd,
                    residualized_intercepts=residualized_intercepts,
                    has_meanstructure=has_meanstructure, est_DIF=est_DIF,
                    residualize=residualize, is_imputed=is_imputed,
                    Nimp=Nimp, moderator=moderator, ... )
    dif_effects <- out2$dif_effects
    parameters <- out2$parameters
    is_meanstructure <- out2$is_meanstructure
    fitstats_joint <- out2$fitstats_joint
    partable_joint <- out2$partable_joint
    sample_stats <- out2$sample_stats

    #**** parameter and fit statistics summary
    parameters_summary <- lsem_parameter_summary( parameters=parameters,
                                moderator.density=out$moderator.density,
                                verbose=verbose )
    weights0 <- weights
    if (is_imputed){
        weights0 <- lsem_aggregate_statistics(x=weights)
    }
    out$moderator.density$Neff <- colSums(weights0)
    obji0 <- obji <- out$moderator.density
    obji$moderator <- obji$moderator
    obji$wgt <- obji$wgt
    obji$Neff <- obji$Neff
    Y <- obji0[,-1]
    dfr <- data.frame( M=colMeans(Y), SD=apply( Y, 2, stats::sd ),
                            min=apply( Y, 2, min ), max=apply( Y, 2, max ) )
    if (is_imputed){
        x <- (data[[1]])[, moderator]
    } else {
        x <- data[,moderator]
    }
    dfr0 <- data.frame(M=mean(x, na.rm=TRUE ), SD=out$sd.moderator,
                        min=min(x, na.rm=TRUE ), max=max(x, na.rm=TRUE ) )
    obji <- rbind( dfr0, dfr)
    rownames(obji) <- NULL
    moderator.stat <- data.frame(variable=c('moderator','wgt', 'Neff'), obji )

    #-- model parameters
    model_parameters <- setdiff( paste(parameters_summary$par), fit_measures)

    #- add attr 'Nimp' to data
    attr(data, 'Nimp') <- Nimp
    attr(data, 'N') <- out$N

    #-- output
    s2 <- Sys.time()
    time <- s2-s1
    res <- list( parameters=parameters, weights=weights,
                    parameters_summary=parameters_summary,
                    bw=out$bw, h=h, N=out$N, nobs=nobs,
                    moderator.density=out$moderator.density,
                    moderator.stat=moderator.stat, moderator.grouped=moderator.grouped,
                    m.moderator=m.moderator, sd.moderator=sd.moderator,
                    moderator=moderator, moderator.grid=moderator.grid,
                    lavmodel=lavmodel, residualize=residualize,
                    data=data, residualized.intercepts=residualized_intercepts,
                    lavaan.args=lavaan.args, lsem_args=lsem_args,
                    fit_measures=fit_measures, model_parameters=model_parameters,
                    s1=s1, s2=s2, time=time,
                    standardized=standardized, standardized_type=standardized_type,
                    lavaan_fct=lavaan_fct, use_lavaan_survey=use_lavaan_survey,
                    pseudo_weights=pseudo_weights, use_pseudo_weights=use_pseudo_weights,
                    sufficient_statistics=sufficient_statistics,
                    variables_ordered=variables_ordered,
                    moderator_variable=moderator_variable,
                    sampling_weights=sampling_weights,
                    no_sampling_weights=no_sampling_weights,
                    is_meanstructure=is_meanstructure,
                    par_invariant=par_invariant, par_linear=par_linear,
                    par_quadratic=par_quadratic,
                    est_joint=est_joint, fitstats_joint=fitstats_joint,
                    partable_joint=partable_joint,
                    dif_effects=dif_effects, sample_stats=sample_stats,
                    loc_linear_smooth=loc_linear_smooth,
                    se=se, compute_se=compute_se, is_imputed=is_imputed, Nimp=Nimp,
                    class_boot=FALSE, type=type, CALL=CALL )
    class(res) <- 'lsem'
    return(res)
}

## File Name: soft_thresholding.R
## File Version: 0.07


soft_thresholding <- function( x, lambda )
{
    x_abs <- abs(x)
    x <- ifelse( x_abs > lambda, x - sign(x) * lambda, 0 )
    return(x)
}


mlnormal_soft_thresholding <- soft_thresholding

## File Name: linking_haberman_lq_pw_create_design.R
## File Version: 0.141

linking_haberman_lq_pw_create_design <- function(y, ind_studies, ind_items, method)
{
    G <- max(ind_studies)
    I <- max(ind_items)

    des <- data.frame(study=ind_studies, item=ind_items)
    dfr <- NULL
    y_long <- NULL
    X <- NULL
    w <- NULL

    for (gg in 1L:(G-1)){
        for (hh in (gg+1):G){
            items_sel <- intersect( des[ des$study==gg, 'item' ],
                                    des[ des$study==hh, 'item' ] )
            if (length(items_sel)>0){
                dfr1 <- data.frame(study1=gg, study2=hh, item=items_sel)
                I1 <- nrow(dfr1)
                X1 <- matrix(0, I1, G-1)
                if (gg>1){
                    X1[,gg-1] <- 1
                }
                X1[,hh-1] <- -1
                for (ii in items_sel){
                    i1 <- which( ind_studies==gg & ind_items==ii )
                    i2 <- which( ind_studies==hh & ind_items==ii )
                    y1 <- y[i1] - y[i2]
                    y_long <- c(y_long, y1)
                }
                X <- rbind(X, X1)
                dfr <- rbind(dfr, dfr1)
            }
        }  # end hh
    }  # end gg

    #- define weights
    w <- rep(1, nrow(X) )
    if (method=='pw2'){
        a1 <- stats::aggregate(1+0*des$item, list(des$item), sum)
        a2 <- stats::aggregate(1+0*dfr$item, list(dfr$item), sum)
        a1 <- a1[ a1[,1] %in% a2[,1], ]
        w0 <- a1[,2] / a2[,2]
        w <- w0[ match( dfr$item, a1[,1]) ]
    }

    #-- output
    res <- list(y=y_long, X=X, w=w, ind_studies=ind_studies, ind_items=ind_items,
                    design=dfr, G=G, I=I)
    return(res)
}

## File Name: m_est.R
## File Version: 0.152

m_est <- function(data, par, optfun_case=NULL, gradfun_case=NULL,
                bread=TRUE, optimizer="optim",
                method="L-BFGS-B", control=list(), ... )
{
    requireNamespace('MASS')

    is_optfun_case <- ! is.null(optfun_case)
    is_gradfun_case <- ! is.null(gradfun_case)
    optfun_case0 <- optfun_case
    gradfun_case0 <- gradfun_case
    N <- nrow(data)

    #-- define case-wise gradient
    if (! is_gradfun_case ){
        gradfun_case <- function(par, data, h=1e-4)
        {
            what_optfun <- 'optfun_case'
            args <- list(par=par, data=data)
            val1 <- do.call(what=what_optfun, args=args)
            N <- length(val1)
            NP <- length(par)
            grad_case <- matrix(NA, nrow=N, ncol=NP)
            colnames(grad_case) <- names(par)
            args1 <- args
            for (pp in 1L:NP){
                par1 <- m_est_add_increment(x=par, pos=pp, h=h)
                args1$par <- par1
                val2 <- do.call(what=what_optfun, args=args1)
                grad_case[,pp] <- (val2-val1)/h
            }
            return(grad_case)
        }
    }

    #-- define gradient
    gradfun <- function(x, data)
    {
        args <- list(par=x, data=data)
        res <- do.call(what='gradfun_case', args=args)
        res <- colSums(res)
        return(res)
    }

    #-- define optimization function
    if (is_optfun_case){
        optfun <- function(x, data)
        {
            args <- list(par=x, data=data)
            val <- do.call(what='optfun_case', args=args)
            val <- sum(val)
            return(val)
        }
    } else {
        optfun <- function(x, data)
        {
            args <- list(par=x, data=data)
            val <- do.call(what='gradfun_case', args=args)
            val <- sum( colSums(val)^2 )/2
            return(val)
        }
        gradfun <- NULL
    }

    #***** optimization
    mod <- sirt_optimizer(optimizer=optimizer, par=par, fn=optfun, data=data,
                    grad=gradfun, method=method, hessian=TRUE, control=control, ... )
    coef <- mod$par
    parnames <- names(par)
    NP <- length(parnames)

    #-- vcov based on observed information
    V_obs <- MASS::ginv(X=mod$hessian)
    V_obs <- rowcolnames(x=V_obs, names=parnames)
    B <- mod$hessian / N

    #-- compute scores and sandwich vcov
    scores <- -gradfun_case(data=data, par=coef)
    A <- crossprod(scores)
    if (bread){
        NP <- ncol(scores)
        hess <- array(0, dim=c(N,NP,NP))
        h <- 1e-4
        pp <- 1
        for (pp in 1L:NP){
            par_pp <- m_est_add_increment(x=coef, pos=pp, h=h)
            scores2 <- -gradfun_case(data=data, par=par_pp)
            hess[,pp,] <- (scores2-scores)/h
        }
        B <- matrix(0, nrow=NP, ncol=NP)
        for (pp in 1L:NP){
            for (hh in 1L:NP){
                B[pp,hh] <- mean( hess[,pp,hh] )
            }
        }
        B <- -B
        # B <- ( B + t(B) ) / 2
        A1 <- A / N^2
        res <- dmlavaan_sandwich_formula(A=A1, B=B, parnames=parnames)
        V_sw <- res$V
        V_obs <- NULL
    } else {
        V_sw <- V_obs %*% A %*% V_obs
    }

    #-- create parameter table
    partable <- data.frame(id=1L:NP, parname=parnames, est=coef)
    if (!bread){
        partable$se_obs <- sqrt_diag(x=V_obs)
    }
    partable$se_sw <- sqrt_diag(x=V_sw)
    rownames(partable) <- NULL

    #--- output
    res <- list(coef=coef, partable=partable, vcov=V_sw, parnames=parnames, mod=mod,
                    scores=scores, V_obs=V_obs, V_sw=V_sw, A=A, B=B, NP=NP,
                    is_optfun_case=is_optfun_case, is_gradfun_case=is_gradfun_case,
                    optfun_case=optfun_case0, gradfun_case=gradfun_case0,
                    optimizer=optimizer, method=method,
                    control=control, data=data, N=N)
    class(res) <- 'm_est'
    return(res)
}

## File Name: tam2mirt.R
## File Version: 0.292


# convert a fitted tam object into a mirt object
tam2mirt <- function( tamobj )
{
    est.mirt <- FALSE
    # extract intercept
    AXsi <- tamobj$AXsi
    # extract loadings
    B <- tamobj$B
    # number of dimensions
    D <- dim(B)[3]
    # extract trait distribution
    mean.trait <- tamobj$beta
    cov.trait <- tamobj$variance
    # extract data
    dat <- tamobj$resp
    # factors
    if (D==1){
        factors <- 'F1'
    }
    if (D>1){
        factors <- dimnames(tamobj$B)[[3]]
    }
    # lavaan syntax with fixed values
    lavsyn <- tam2mirt_fix( D=D, factors=factors, B=B, dat=dat, AXsi=AXsi,
                    mean.trait=mean.trait, cov.trait=cov.trait, tamobj=tamobj )
    # lavaan syntax with freed values
    lavsyn.freed <- tam2mirt_freed( D=D, factors=factors, B=B, dat=dat,
            AXsi=AXsi, mean.trait=mean.trait, cov.trait=cov.trait, tamobj=tamobj )
    # pseudo-estimate model in mirt: just create mirt object structure
    res <- lavaan2mirt( dat=dat, lavmodel=lavsyn, est.mirt=TRUE )
    #--- include parameters in mirt object
    res$mirt@Model$nest <- as.integer(tamobj$ic$np ) # number of estimated parameters
    # recalculate AIC, BIC, AICc and SABIC
    res$mirt@Fit$AIC <- tamobj$ic$AIC
    res$mirt@Fit$BIC <- tamobj$ic$BIC
    res$mirt@Fit$AICc <- tamobj$ic$AICc
    res$mirt@Fit$SABIC <- tamobj$ic$aBIC
    # use theta grid from estimation in TAM
    res$mirt@Model$Theta <- tamobj$theta
    res$mirt@Options$quadpts <- nrow(tamobj$theta)
    # output
    res$lavaan.syntax.fixed <- lavsyn
    res$lavaan.syntax.freed <- lavsyn.freed
    # res$tamobj <- tamobj
    return(res)
}

## File Name: btm_fit_statistics.R
## File Version: 0.212


#**** item outfit and infit statistic
btm_fit_statistics <- function( probs, dat0, ind1, ind2, TP, judge=NULL,
    compute_agreement=TRUE, wgt.ties=.5)
{
    multiple_judges <- TRUE
    if (is.null(judge)){
        multiple_judges <- FALSE
    }
    # first individual
    X_exp1 <- probs[,1]*1 + wgt.ties*probs[,3]
    X_var1 <- 1*probs[,1] + wgt.ties^2*probs[,3]
    X_var1 <- X_var1 - X_exp1^2
    H_1 <- dat0[,3] - X_exp1
    Z_1 <- H_1 / sqrt( X_var1 )
    dat0$resid_home <- H_1
    dat0$stand_resid_home <- Z_1

    # second individual
    X_exp2 <- probs[,2]*1 + wgt.ties*probs[,3]
    X_var2 <- 1*probs[,2] + wgt.ties^2*probs[,3]
    X_var2 <- X_var2 - X_exp2^2
    H_2 <- 1 - dat0[,3] - X_exp2
    Z_2 <- H_2 / sqrt( X_var2 )
    dat0$resid_away <- H_2
    dat0$stand_resid_away <- Z_2

    # compute outfit statistic
    out1 <- rowsum( Z_1^2, dat0[,1] )
    N1 <- rowsum( 1+0*Z_1, dat0[,1] )
    out2 <- rowsum( Z_1^2, dat0[,2] )
    N2 <- rowsum( 1+0*Z_1, dat0[,2] )

    # compute infit statistic
    wvar1 <- rowsum( X_var1, dat0[,1] )
    wvar2 <- rowsum( X_var2, dat0[,2] )
    win1 <- rowsum( X_var1*Z_1^2, dat0[,1] )
    win2 <- rowsum( X_var2*Z_1^2, dat0[,2] )

    out <- btm_fit_combine_tables( win1=out1, win2=out2, ind1=ind1, ind2=ind2, TP=TP)
    N <- btm_fit_combine_tables( win1=N1, win2=N2, ind1=ind1, ind2=ind2, TP=TP)
    wvar <- btm_fit_combine_tables( win1=wvar1, win2=wvar2, ind1=ind1, ind2=ind2, TP=TP)
    win <- btm_fit_combine_tables( win1=win1, win2=win2, ind1=ind1, ind2=ind2, TP=TP)
    outfit <- out/N
    infit <- win/wvar

    #--- fit statistics in case of multiple judges
    fit_judges <- NULL
    if (multiple_judges){
        # outfit and infit statistic
        out1 <- rowsum( Z_1^2, judge )
        N1 <- rowsum( 1+0*Z_1, judge )
        wvar1 <- rowsum( X_var1, judge )
        win1 <- rowsum( X_var1*Z_1^2, judge)
        fit_judges <- data.frame( judge=rownames(out1), outfit=out1/N1,
                            infit=win1/wvar1)

        #* compute agreement statistics
        if (compute_agreement){
            dat1 <- data.frame(judge=judge)
            colnames(dat0) <- c('id1','id2','result')
            ind <- dat0$id1 < dat0$id2
            dat1$id1 <- ifelse(ind, dat0$id1, dat0$id2)
            dat1$id2 <- ifelse(ind, dat0$id2, dat0$id1)
            dat1$result <- ifelse(ind, dat0$result, 1-dat0$result)
            dat1$dyad <- paste0(dat1$id1, '-', dat1$id2)
            a1 <- stats::aggregate( dat1$result, list(dat1$dyad), median )
            colnames(a1) <- c('dyad', 'mode')
            a1$N_dyad <- stats::aggregate( 1+0*dat1$result, list(dat1$dyad), sum )[,2]
            a1 <- a1[ ( a1$N_dyad > 2 ) & ( a1$mode %in% c(0,1) ), ]
            dat1$mode <- a1[ match(dat1$dyad, a1$dyad), 'mode' ]
            a2 <- stats::aggregate( dat1$result==dat1$mode, list(dat1$judge),
                                        mean, na.rm=TRUE)
            fit_judges$agree <- a2[ match(fit_judges$judge, a2[,1]), 2]
        }
    }

    #--- output
    res0 <- list( outfit=outfit, infit=infit, multiple_judges=multiple_judges,
                fit_judges=fit_judges, residuals=dat0)
    return(res0)
}

## File Name: rasch_mml2_calc_prob.R
## File Version: 0.09

rasch_mml2_calc_prob <- function( theta.k, b, fixed.a, fixed.c, fixed.d,
    alpha1, alpha2, Qmatrix, eps=1e-40)
{
    pjk <- prob_genlogis_4pl(theta=theta.k, b=b, a=fixed.a, c=fixed.c, d=fixed.d,
                        alpha1=alpha1, alpha2=alpha2, Qmatrix=Qmatrix)
    pjk <- ( pjk + eps ) / ( 1 + 2*eps )
    pjk.M <- t(pjk)
    return(pjk.M)
}

## File Name: rm_eap_reliability.R
## File Version: 0.05

rm_eap_reliability <- function( EAP, SE_EAP )
{
    EAP.rel <- 1 - mean( SE_EAP^2 ) / ( mean( SE_EAP^2 ) + stats::var( EAP ) )
    return(EAP.rel)
}

## File Name: rm_sdt_calc_expected_likelihood_item.R
## File Version: 0.16


rm_sdt_calc_expected_likelihood_item <- function( nik.item, a.item, tau.item, Qmatrix,
        theta.k, VV, K, TP, eps=1E-10)
{
    prob_dim <- c(VV, K+1, TP)
    nik_item <- as.vector(nik.item)
    probs <- rm_sdt_calc_probs_gpcm_rcpp( a.item=a.item, tau.item=tau.item,
                Qmatrix=Qmatrix, theta.k=theta.k, VV=VV, K=K, TP=TP, eps=eps, use_log=TRUE,
                as_vector=TRUE)
    ll <- sirt_rcpp_rm_sdt_calc_gradient_likelihood_item_llgrad( logprob_D1=probs,
                    prob_D1_dim=prob_dim, nik_item=nik_item )
    return(ll)
}

## File Name: sirt_remove_arguments_function.R
## File Version: 0.02

sirt_remove_arguments_function <- function(fun, args)
{
    fun_formals <- formals(fun=fun)
    rem <- setdiff( names(args), names(fun_formals))
    args <- sirt_remove_list_entries(list=args, rem=rem)
    return(args)
}

## File Name: locpolycor_est_polycor_opt_fun.R
## File Version: 0.084


locpolycor_est_polycor_opt_fun <- function(x, w, x1, x0,
        thresh_ii1, thresh_ii2, thresh_jj1, thresh_jj2,
        model="const", eps=1e-10,
        package="pbivnorm", maxcor=0.99)
{
    requireNamespace(package)
    if (package=='pbivnorm'){
        pbvnorm_fun <- pbivnorm::pbivnorm
    }
    if (package=='pbv'){
        pbvnorm_fun <- pbv::pbvnorm
    }
    if (model=='const'){
        rho1 <- x
    }
    if (model=='lin'){
        rho1 <- x[1] + x[2] * (x1-x0)
        rho1 <- pmin( maxcor, pmax(-maxcor, rho1) )
    }

    d11 <- pbvnorm_fun(x=thresh_ii1, y=thresh_jj1, rho=rho1)
    d10 <- pbvnorm_fun(x=thresh_ii1, y=thresh_jj2, rho=rho1)
    d01 <- pbvnorm_fun(x=thresh_ii2, y=thresh_jj1, rho=rho1)
    d00 <- pbvnorm_fun(x=thresh_ii2, y=thresh_jj2, rho=rho1)
    val <- d11 - d10 - d01 + d00
    val <- ifelse(val<eps, eps, val)
    val <- - sum( w*log(val+eps) )
    return(val)
}

## File Name: dmlavaan_joint_parameterTable.R
## File Version: 0.07

dmlavaan_joint_parameterTable <- function(mod1, mod2, label_parnames="parnames0")
{
    parnames1 <- mod1[[ label_parnames ]]
    parnames2 <- mod2[[ label_parnames ]]
    #-- create joint parameter table
    parnames <- union(parnames1, parnames2)
    NP <- length(parnames)
    partable <- data.frame(id=1L:NP, parname=parnames)
    partable$in_mod1 <- match(partable$parname, parnames1)
    partable$in_mod2 <- match(partable$parname, parnames2)
    #- first model
    partable <- dmlavaan_joint_parameterTable_merge_table(partable=partable,
                        mod=mod1, model_index=1)
    #- second model
    partable <- dmlavaan_joint_parameterTable_merge_table(partable=partable,
                        mod=mod2, model_index=2)
    partable$diff <- partable$est1 - partable$est2
    partable$se_diff <- NA
    rownames(partable) <- NULL

    #--- output
    res <- list(partable=partable, parnames=parnames, NP=NP)
    return(res)
}

## File Name: mgsem_opt_fun.R
## File Version: 0.275


mgsem_opt_fun <- function(x, opt_fun_args, output_all=FALSE)
{

    estimator <- opt_fun_args$estimator
    partable <- mgsem_coef2partable(coef=x, partable=opt_fun_args$partable)
    model <- mgsem_partable2model(partable=partable,
                        model=opt_fun_args$model, index=FALSE)
    G <- opt_fun_args$G
    eval_pen_res <- NULL
    if (output_all){
        implied_list <- list()
        est_tot_list <- list()
        S1_list <- list()
        mean_residual_list <- list()
    }
    suffstat <- opt_fun_args$suffstat

    ll <- 0
    chisq <- 0
    N <- 0
    for (gg in seq_len(G) ){
        est0 <- model[[1]]$est
        est_gg <- model[[gg+1]]$est
        est_tot_gg <- mgsem_add_list_entries(list1=est0, add_list=est_gg,
                            output_list=est0)
        implied <- mgsem_compute_model_implied_moments(est=est_tot_gg,
                            is_B=opt_fun_args$is_B, calc_Sigma=TRUE, calc_Mu=TRUE)
        if (output_all){
            implied_list[[gg]] <- implied
            est_tot_list[[gg]] <- est_tot_gg
        }
        S_gg <- suffstat[[gg]]$S
        p <- nrow(S_gg)

        #- function evaluation
        eval_args <- list(suffstat=opt_fun_args$suffstat[[gg]],
                            Mu=implied$Mu, Sigma=implied$Sigma,
                            output_all=output_all)
        if (estimator=='ME'){
            eval_args$p <- opt_fun_args$p_me
            eval_args$eps <- opt_fun_args$eps_approx
            eval_args$deriv <- FALSE
            eval_args$approx_method <- opt_fun_args$technical$approx_method
        }
        ll_gg <- do.call(what=opt_fun_args$eval_fun, args=eval_args)
        if (output_all & (estimator=='ML') ){
            ll0_gg <- ll_gg
            S1_list[[gg]] <- ll_gg$S1
            mean_residual_list[[gg]] <- ll_gg$mean_residual
            ll_gg <- ll_gg$loglike
            N_gg <- suffstat[[gg]]$N
            S_gg <- suffstat[[gg]]$S
            p <- nrow(S_gg)
            ll_gg_adj <- ll_gg + N_gg/2*p*log(2*pi)
            chi_gg <- -2*( (N_gg/2)*( sirt_logdet(x=S_gg) + p ) + ll_gg_adj )
            chisq <- chisq + chi_gg
            N <- N + N_gg
        }
        ll <- ll+ll_gg
    }
    ll0 <- ll
    #-- penalty function
    if (opt_fun_args$use_penalty){
        res <- mgsem_evaluate_penalties(x=x, partable=partable,
                        prior_list=opt_fun_args$prior_list,
                        technical=opt_fun_args$technical,
                        h=opt_fun_args$technical$h,
                        p=opt_fun_args$p_pen,
                        eps_approx=opt_fun_args$eps_approx,
                        deriv=FALSE, difflp_info=opt_fun_args$difflp_info,
                        loop_parms=opt_fun_args$loop_parms,
                        pen_type=opt_fun_args$pen_type, a_scad=opt_fun_args$a_scad)
        eval_pen_res <- res
        ll <- ll + res$pen_all
    }

    #-- negative function (minimization problem)
    ll <- - ll

    #- whole output
    res <- ll
    if (output_all){
        # chi square statistic and RMSEA
        p_mu <- 0
        for (gg in 1L:G){
            mu1 <- suffstat[[gg]]$M
            p_mu <- p_mu + sum(abs(mu1)>1e-14)
        }
        chisq_df <- p_mu + G*p*(p+1)/2 - max(partable$index)
        chisq_p <- 1-stats::pchisq(chisq, df=chisq_df)
        lambda <- max( chisq - chisq_df, 0 )
        fac <- chisq_df * N
        RMSEA <- sqrt( lambda / fac )
        # fit proportion
        pen_all <- abs(eval_pen_res$pen_all)
        pen_prop <- pen_all / ( chisq/2 + pen_all )

        #-- output
        res <- list(loglike=ll0, eval_pen_res=eval_pen_res, opt_val=ll,
                pen_all=pen_all, implied=implied_list, est_tot=est_tot_list,
                S1=S1_list, suffstat=opt_fun_args$suffstat,
                mean_residual=mean_residual_list, G=G, estimator=estimator,
                chisq=chisq, chisq_df=chisq_df, chisq_p=chisq_p,
                RMSEA=RMSEA, pen_prop=pen_prop)
    }

    #-- output
    return(res)
}

## File Name: regpolca_penalty_fun.R
## File Version: 0.05


regpolca_penalty_fun <- function(x, regular_grouped, I, partable,
    combis_classes_list, regular_lam, eps, penalty_used, Ni,
    combis_categ_list, fuse_categories, K)
{
    pen <- 0
    if (regular_grouped=='none'){
        #* fused probabilities among classes
        for (ii in 1L:I){
            x_ii <- x[ partable$itemnr==ii ]
            a1 <- regpolca_penalty_fun_value_nongrouped(x_ii=x_ii,
                                combis_ii=combis_classes_list[[ii]],
                                regular_lam=regular_lam[1],
                                eps=eps, penalty_used=penalty_used)
            pen <- pen + Ni[ii]*sum(a1)
        }
        #* fixed probabilities among categories
        for (ii in 1L:I){
            if (fuse_categories[ii]){
                x_ii <- x[ partable$itemnr==ii ]
                a1 <- regpolca_penalty_fun_value_nongrouped(x_ii=x_ii,
                            combis_ii=combis_categ_list[[ii]],
                            regular_lam=regular_lam[2],
                            eps=eps, penalty_used=penalty_used)
                pen <- pen + Ni[ii]*sum(a1)
            }
        }
    } else {
        for (ii in 1L:I){
            x_ii <- x[ partable$itemnr==ii ]
            combis_ii <- combis_categ_list[[ii]]
            nc <- nrow(combis_ii)
            a1 <- 0
            if (regular_grouped=='class'){
                K1 <- max(combis_ii$cat_pair)
                for (kk in 1:K1){
                    cii_kk <- combis_ii[ seq(kk,nc,by=nc/K), ]
                    a2 <- regpolca_penalty_fun_value_grouped(x_ii=x_ii,
                                combis_ii=cii_kk, regular_lam=regular_lam[1],
                                eps=eps, penalty_used=penalty_used)
                    a1 <- a1 + a2
                }
            }
            if (regular_grouped=='categ'){
                for (kk in 1:K){
                    cii_kk <- combis_ii[ combis_ii$class==kk, ]
                    a2 <- regpolca_penalty_fun_value_grouped(x_ii=x_ii,
                                combis_ii=cii_kk, regular_lam=regular_lam[1],
                                eps=eps, penalty_used=penalty_used)
                    a1 <- a1 + a2
                }
            }
            pen <- pen + Ni[ii]*sum(a1)
        }
    }
    return(pen)
}

## File Name: rm_facets_pp_mle.R
## File Version: 0.187


#*** person parameter estimation in partial credit model
rm_facets_pp_mle <- function( data, a, b, theta, WLE=FALSE,
    maxiter=30, maxincr=3, h=.001, convP=.0001, maxval=9.99,
    progress=TRUE )
{

    theta0 <- theta
    N <- length(theta)
    I <- ncol(data)
    iter <- 1
    conv <- 1E5

    args <- list(data=data, a=a, b=b, theta=theta)
    args_change <- function(args, theta){
        args$theta <- theta
        return(args)
    }

    #-------- begin algorithm
    while ( ( conv > convP ) & (  iter <=maxiter ) ){

        theta0 <- theta
        ll0 <- do.call(rm_facets_pp_mle_calc_ll_theta, args_change(args, theta))
        llP1 <- do.call(rm_facets_pp_mle_calc_ll_theta, args_change(args, theta+h))
        llM1 <- do.call(rm_facets_pp_mle_calc_ll_theta, args_change(args, theta-h))
        if (WLE){
            llP2 <- do.call(rm_facets_pp_mle_calc_ll_theta, args_change(args, theta+2*h))
            llM2 <- do.call(rm_facets_pp_mle_calc_ll_theta, args_change(args, theta-2*h))
        }

        ll1 <- (llP1 - llM1) / (2*h)
        ll2 <- (llP1 - 2*ll0 + llM1) / h^2
        incr <- - ll1 / ll2
        if (WLE){
            ll3 <- ( llP2 - 2*llP1 + 2*llM1 - llM2 ) / (2*h^3 )
            incr <- - ll1 / ll2 - ll3 / ( 2*ll2^2 )
        }

        maxincr <- maxincr / 1.05
        incr <- rm_numdiff_trim_increment( increment=incr, max.increment=maxincr, eps2=0 )
        theta <- theta + incr
        theta <- ifelse( abs(theta) > maxval, sign(theta)*maxval, theta )
        conv <- max( abs( theta - theta0) )
        if (progress){
            cat('* Iteration', iter, ':', 'maximum parameter change',
                    round( conv, 5), '\n')
            utils::flush.console()
        }
        iter <- iter + 1
    }

    #-- output
    se <- sqrt( abs( - 1/ll2 ) )
    se <- ifelse( abs(theta)==maxval, NA, se )
    theta <- ifelse( theta==maxval, Inf, theta )
    theta <- ifelse( theta==- maxval, -Inf, theta )
    res <- data.frame( est=theta, se=se )
    return(res)
}

## File Name: fit.adisop.R
## File Version: 2.256


#---- Fit ADISOP model
fit.adisop <- function( freq.correct, wgt, conv=.0001,
        maxit=100, epsilon=.01, progress=TRUE, calc.ll=TRUE )
{
    eps2 <- epsilon
    M1 <- as.matrix( freq.correct )
    wgt <- as.matrix(wgt)
    freq.correct1 <-  ( freq.correct + eps2/2 ) / ( 1 + eps2 )
    wgt2 <- wgt / sqrt( ( freq.correct1 * ( 1 - freq.correct1 ) ) )
    RR <- nrow(freq.correct)
    CC <- ncol(freq.correct)
    # auxiliary variables
    scores <- 0L:(RR-1)
    item.psx <- colSums( freq.correct * wgt ) / colSums( wgt )
    # initialization
    dfr0 <- data.frame( stud.index=rep(1L:RR, CC),
                        item.index=rep(1L:CC,each=RR),
                        stud.p=rep( scores,CC),
                        item.p=rep( item.psx, each=RR ),
                        wgt2=matrix( as.matrix( wgt2 ), RR*CC, 1 ),
                        p0=matrix( as.matrix(freq.correct ), RR*CC, 1 ) )
    dfr0$studPitem.p <- dfr0$stud.p + dfr0$item.p
    dfr0 <- dfr0[ order( dfr0$p0), ]
    dfr0$psort.index <- rank( dfr0$p0 )
    # define matrix I
    I.matrix <- 0*M1
    I.matrix[ as.matrix(dfr0[, c('stud.index', 'item.index' ) ] ) ] <-
                dfr0$psort.index
    IS <- sum(I.matrix)
    Y <- Y0 <- I.matrix
    iter <- 0
    deviation <- 1000
    if (progress){
        cat('\n*******ADISOP Model*********\n')
    }

    #**** begin algorithm
    while( ( iter < maxit) & ( deviation > conv ) ){
        #*** start algorithm
        Y0 <- Y
        # row and column averages
        X1 <- rowMeans(Y0)
        X1 <- X1 - X1[1]
        X2 <- colMeans(Y0)
        X2 <- X2 - X2[1]
        # calculate sum
        Y <- outer( X1, X2, '+' )
        # preparation isotonic regression
        dfr0 <- data.frame( stud.index=rep(1L:RR, CC),
                            item.index=rep(1L:CC,each=RR),
                            stud.X=rep( X1,CC),
                            item.X=rep( X2, each=RR ),
                            wgt2=matrix( as.matrix( wgt2 ), RR*CC, 1 ),
                            Y=matrix( as.matrix( Y), RR*CC, 1 ),
                            I=matrix( as.matrix( I.matrix), RR*CC, 1 ) )
            # Y=X1 + X2
        dfr0 <- dfr0[ order( dfr0$I), ]
        dfr0$yf <- monoreg.rowwise( matrix( dfr0$Y, nrow=1 ),
                                    matrix(dfr0$wgt2,nrow=1) )[1,]
        Y <- 0*M1
        Y[ as.matrix(dfr0[, c('stud.index', 'item.index' ) ] ) ] <- dfr0$yf
        Y <- Y / sum(Y) * IS
        # calculate deviation
        deviation <- max( abs( Y - Y0 ))
        iter <- iter + 1
        if (progress){
            cat( 'Iteration', iter, '- Deviation=',  round( deviation, 6 ), '\n')
            utils::flush.console()
        }
    }
    #-------------- end algorithm

    #**** calculate link function
    dfr0 <- data.frame( stud.index=rep(1L:RR, CC),
                            item.index=rep(1L:CC,each=RR),
                            stud.p=rep( X1,CC),
                            item.p=rep( X2, each=RR ),
                            wgt=matrix( as.matrix(wgt), RR*CC, 1 ),
                            wgt2=matrix( as.matrix(wgt2), RR*CC, 1 ),
                            p0=matrix( as.matrix(freq.correct), RR*CC, 1 ) )
    dfr0$X1PX2 <- dfr0$stud.p + dfr0$item.p
    dfr0 <- dfr0[ order( dfr0$X1PX2), ]
    dfr0$yf <- monoreg.rowwise( matrix( dfr0$p0, nrow=1 ), matrix(dfr0$wgt2,nrow=1) )[1,]
    dfr0 <- dfr0[ order( dfr0$stud.index * 1000 + dfr0$item.index ), ]

    Y <- 0*M1
    Y[ as.matrix(dfr0[, c('stud.index', 'item.index' ) ] ) ] <- dfr0$yf
    G <- Y
    colnames(dfr0)[ colnames(dfr0)=='stud.p' ] <- 'person.sc'
    colnames(dfr0)[ colnames(dfr0)=='item.p' ] <- 'item.sc'
    colnames(dfr0)[ colnames(dfr0)=='p0' ] <- 'freq'
    colnames(dfr0)[ colnames(dfr0)=='yf' ] <- 'freq.fitted'
    # deviation criterion
    wgt1 <- ( wgt / colSums( wgt ) ) / ncol(wgt)
    fit <- sqrt( sum( ( M1-G  )^2 * wgt1  ) )
    #****
    # calculate likelihood
    ll <- NULL
    if (calc.ll){
        ll <- list( ll.ind=.calc.ll.isop( freq.correct, wgt, irtfitted=freq.correct) )
        ll$ll.adisop <- .calc.ll.isop( freq.correct, wgt, irtfitted=G )
        NW <- mean( colSums(wgt) )
        ll$llcase.ind <- ll$ll.ind/NW
        ll$llcase.adisop <- ll$ll.adisop/NW
            }
    # output
    res <- list( fX=G, ResX=M1-G, fit=fit, item.sc=X2,
                        person.sc=X1, ll=ll, freq.fitted=dfr0)
    return(res)
}

## File Name: sirt_probs_dichotomous_to_array.R
## File Version: 0.02

sirt_probs_dichotomous_to_array <- function(probs)
{
    I <- nrow(probs)
    TP <- ncol(probs)
    ncat <- 2
    probs1 <- array(0, dim=c(I,ncat,TP) )
    probs1[,2,] <- probs
    probs1[,1,] <- 1 - probs
    return(probs1)
}

## File Name: lsem_fitsem_joint_estimation_prepare_partable_include_group_label.R
## File Version: 0.111


lsem_fitsem_joint_estimation_prepare_partable_include_group_label <- function(
    partable, gg, label_list)
{
    hh <- 'lhs'
    for (hh in c('lhs','rhs')){
        ind <- which( partable[,hh] %in% setdiff(label_list, '') )
        if (length(ind)>0){
            partable[ ind, hh] <- paste0(partable[ ind, hh], 'g', gg)
        }
    }
    partable$plabel <- paste0(label_list,'g', gg)
    partable$plabel[ paste(label_list)=='' ] <- ''
    return(partable)
}

## File Name: sqrt_diag_positive.R
## File Version: 0.02

sqrt_diag_positive <- function(x)
{
    y <- diag(x)
    y <- ifelse(y<0,0,y)
    names(y) <- rownames(x)
    res <- sqrt(y)
    return(res)
}

## File Name: lsem_fitsem_verbose_start.R
## File Version: 0.05

lsem_fitsem_verbose_start <- function(G, verbose)
{
    pr <- NULL
    if (verbose){
        cat( '** Fit lavaan model\n')
        G1 <- min(G,10)
        pr <- round( seq(1, G, len=G1) )
        cat('|')
        cat( paste0( rep('*',G1), collapse='') )
        cat('|\n')
        cat('|')
    }
    return(pr)
}

## File Name: rm_summary_trait_distribution.R
## File Version: 0.07

rm_summary_trait_distribution <- function(object)
{
    cat( "Trait Distribution\n" )
    cat( "Mean=", round( object$mu, 3), " SD=", round( object$sigma, 3) )
    cat( "\n\nEAP Reliability=")
    cat(round( object$EAP.rel,3 ) )
    cat( "\n")
}

## File Name: regpolca_define_customItems.R
## File Version: 0.06

regpolca_define_customItems <- function(ncats, K, dat, par_item_max)
{
    cats_u <- unique(ncats)
    customItems <- list(1:length(cats_u))
    for (ncat in cats_u){
        par <- xxirt_irf_lca_init_par(K=K, ncat=ncat, random_sd=0)
        item_lc <- xxirt_createDiscItem( name=paste0('LC',ncat), par=par,
                     est=rep(TRUE,K*(ncat-1)), P=xxirt_irf_lca )
        ii <- which(ncat==cats_u)
        customItems[[ii]] <- item_lc
    }
    itemtype <- paste0( 'LC', ncats )
    partable <- xxirt_createParTable( dat=dat, itemtype=itemtype,
                    customItems=customItems)
    partable$lower <- -par_item_max
    partable$upper <- par_item_max

    #- output
    res <- list(customItems=customItems, partable=partable,
                    itemtype=itemtype)
    return(res)
}

## File Name: xxirt_classprobs_lca.R
## File Version: 0.09

xxirt_classprobs_lca <- function(par, Theta, G)
{
    K <- nrow(Theta)
    probs <- matrix(NA, nrow=K, ncol=G)
    for (gg in 1L:G){
        logitprobs_gg <- par[ (K-1)*(gg-1) + ( 1L:(K-1) ) ]
        probs[,gg] <- xxirt_classprobs_lca_compute_probs(logitprobs=logitprobs_gg)
    }
    return(probs)
}

## File Name: summary.rasch.copula3.R
## File Version: 0.131


#* summary for rasch.copula object
summary.rasch.copula3 <- function( object, file=NULL, digits=3, ...)
{
    summary.rasch.copula2( object=object, file=file, digits=digits, ... )
}

## File Name: meas_inv_cfa_modify_partable.R
## File Version: 0.137

meas_inv_cfa_modify_partable <- function(partable, mimod, critval)
{
    pars_mi <- meas_inv_compute_lavaan_parnames(object=mimod)
    pars_pt <- meas_inv_compute_lavaan_parnames(object=partable)
    free_parameter <- ( mimod$mi[1] > critval )
    if (free_parameter){
        ind <- which( pars_pt==pars_mi[1] & partable$group==mimod$group[1] )
        joint_label <- partable[ind,'label']
        plabel <- partable[ind,'plabel']
        is_joint_label <- joint_label==plabel
        partable[ind,'label'] <- ''
        if (is_joint_label){
            ind0 <- setdiff(which(pars_pt==pars_mi[1]),ind)
            partable0 <- partable[ind0, ]
            partable0 <- partable0[ partable0$label !='', ]
            plabel_new <- partable0$plabel[1]
            partable0[,'label'] <- plabel_new
            partable[ partable0$id, 'label' ] <- partable0$label
            ind2 <- which( partable$op=='==' & partable$lhs==plabel)
            partable[ind2,'lhs'] <- plabel_new

            ind2 <- which( partable$op=='==' & (partable$lhs==partable$rhs) )
            if (length(ind2)>0){
                partable <- partable[-ind2, ]
            }

        }

        ind2 <- which( partable$op=='==' & (partable$lhs==plabel | partable$rhs==plabel))
        if (!is_joint_label){
            if (length(ind2)>0){
                partable <- partable[ -ind2, ]
            }
        }
    }
    #--- output
    res <- list(partable=partable, free_parameter=free_parameter, par_changed=pars_mi[1])
    return(res)
}

## File Name: locpolycor_est_polycor_itempair.R
## File Version: 0.089


locpolycor_est_polycor_itempair <- function(y, ii, jj, data.mod, moderator.grid,
        weights_grid, thresh_ind_list, x0, model="const",
        lower=-.99, upper=.99, EE=99.99, par_init=0.2,
        eps=1e-10, optimizer="optim")
{
    y1 <- y[,ii]
    y2 <- y[,jj]
    data_mod <- data.mod
    G <- length(moderator.grid)
    par0 <- par_init
    if (model=='lin'){
        par0 <- c(par0, 0)
    }
    K1 <- max(y1)
    K2 <- max(y2)
    N <- length(y1)
    x1 <- data_mod

    W <- polycor <- rep(NA,G)

    for (gg in 1L:G){
        x0 <- moderator.grid[gg]
        w <- weights_grid[,gg]
        thresh_ii <- thresh_ind_list[[ii]]
        thresh_jj <- thresh_ind_list[[jj]]
        thresh_ii <- cbind(-EE, thresh_ii, EE)
        thresh_jj <- cbind(-EE, thresh_jj, EE)
        thresh_ii1 <- thresh_ii[ cbind(1L:N, y1+2) ]
        thresh_ii2 <- thresh_ii[ cbind(1L:N, y1+1) ]
        thresh_jj1 <- thresh_jj[ cbind(1L:N, y2+2) ]
        thresh_jj2 <- thresh_jj[ cbind(1L:N, y2+1) ]

        res_optim <- sirt_optimizer(optimizer=optimizer, par=par0,
                            fn=locpolycor_est_polycor_opt_fun,
                            w=w, thresh_ii1=thresh_ii1,
                            thresh_ii2=thresh_ii2, thresh_jj1=thresh_jj1,
                            thresh_jj2=thresh_jj2, eps=eps,
                            x1=data.mod, x0=x0, model=model, method='L-BFGS-B',
                            lower=lower, upper=upper, hessian=FALSE,
                            package='pbivnorm')
        par0 <- res_optim$par
        polycor[gg] <- par0[1]
        W[gg] <- sum(w)
    }

    #-- output
    res <- list( polycor=polycor, ii=ii, jj=jj, W=W, N=N)
    return(res)
}

## File Name: xxirt_nr_pml_grad_fun.R
## File Version: 0.129

xxirt_nr_pml_grad_fun <- function(x, em_args, output_all=FALSE, use_rcpp=TRUE)
{

    Theta <- em_args$Theta
    pml_args <- em_args$pml_args
    h <- em_args$h

    #*** compute prior distribution
    prior_Theta <- xxirt_compute_prior_Theta_from_x(x=x, em_args=em_args)

    #* compute item response probabilities
    probs_items <- xxirt_compute_prob_item_from_x(x=x, em_args=em_args)
    eps <- 1e-14

    I <- pml_args$I
    K <- pml_args$K
    G <- pml_args$G
    TP <- pml_args$TP
    W1 <- pml_args$W1
    freq1 <- pml_args$freq1
    NI2 <- pml_args$NI2
    W2_long <- as.matrix(pml_args$W2_long)
    freq2 <- pml_args$freq2

    NP <- em_args$NP
    NPI <- em_args$NPI
    NPT <- em_args$NPT

    #- define more arguments
    parindex_items <- em_args$parindex_items
    parindex_Theta <- em_args$parindex_Theta
    pml_args$in_par_Theta <- em_args$pml_args$in_par_Theta

    #- define gradient
    eval_fun_args <- list(prior_Theta=prior_Theta, probs_items=probs_items, freq1=freq1,
                            freq2=freq2, W1=W1, W2_long=W2_long, G=G, K=K,
                            I=I, TP=TP, NI2=NI2, eps=eps )
    fun_pml_eval <- sirt_rcpp_xxirt_nr_pml_opt_fun
    pml_fun0 <- do.call( what=fun_pml_eval, args=eval_fun_args)

    grad <- 0*x
    eval_grad_args <- eval_fun_args
    eval_grad_args$NP <- NP
    eval_grad_args$val1 <- pml_fun0$val1
    eval_grad_args$val2 <- pml_fun0$val2

    eval_grad_args$der_prior_Theta <- 0*prior_Theta
    eval_grad_args$der_probs_items <- der_probs_items0 <- 0*probs_items

    for (tpp in 1L:NP){
        pp_Theta <- pml_args$in_par_Theta[tpp]

        x1 <- sirt_add_increment(x=x, pos=tpp, value=h)
        if (pp_Theta){
            prior_Theta1 <- xxirt_compute_prior_Theta_from_x(x=x1, em_args=em_args)
            der_prior_Theta <- ( (prior_Theta1-prior_Theta)/h )
            eval_grad_args$der_prior_Theta <- der_prior_Theta
        } else {
            itpp <- em_args$item_index[[tpp]]
            probs_items1 <- xxirt_compute_prob_item_from_x(x=x1, em_args=em_args,
                                    item_index=itpp)
            der_probs_items <- der_probs_items0
            der_probs_items[itpp,,] <- ( (probs_items1-probs_items[itpp,,,drop=FALSE])/h )
            eval_grad_args$der_probs_items <- der_probs_items
        }

        eval_grad_args$pp_Theta <- pp_Theta
        eval_grad_args$index_freq1 <- em_args$pml_args$index_freq1[[tpp]]
        eval_grad_args$index_freq2 <- em_args$pml_args$index_freq2[[tpp]]

        grad[tpp] <- do.call( what=sirt_rcpp_xxirt_nr_pml_grad_fun_eval,
                                    args=eval_grad_args)

    }

    #-- output
    return(grad)
}

## File Name: rasch.jml.biascorr.R
## File Version: 0.12


#########################################################################################
# analytical bias correction of JML estimated Rasch model
# rasch.jml.biascorr <- function( data, data.resp, theta, b, adjmin=.5 ){
rasch.jml.biascorr <- function( jmlobj, itemfac=NULL #, adjmin=.5
                ){
    mod <- jmlobj
    m1 <- mod$data.proc
    data <- m1$data
    data.resp <- m1$data.resp
    theta <- m1$theta
    b <- mod$item$itemdiff

    I <- length(b)
    I1 <- mean( rowSums( data.resp ) )
    if (! is.null( itemfac) ){ I1 <- itemfac }
    N <- nrow(data)
    p_it <- stats::plogis( outer( theta, b, "-" ) )
    q_it <- 1 - p_it
    u_it <- - data + p_it   # d/d(theta) f
    v_it <- - u_it      # d/d(alpha) f
    # auxiliary calculations
    u_it_theta <- - p_it * q_it
    u_it_alpha <- p_it * q_it
    v_it_alpha <- - p_it * q_it
    u_it_alpha2 <- p_it * q_it^2 - p_it^2 * q_it
    v_it_alpha2 <- - p_it * q_it^2 + p_it^2 * q_it

    # terms
    s2 <- rowWgtMeans( v_it_alpha, data.resp )
    t00 <- rowWgtMeans( v_it^2, data.resp )
    t1z <- - 1 / t00
    t0 <- - t00 / s2
    t1b <- rowWgtMeans( v_it * v_it_alpha, data.resp )
    t2b <- rowWgtMeans( v_it_alpha2, data.resp )
    t2c2 <- t1c2 <- t2c2 <- s2
    t2z <- 1/2 / t1c2

    b.bias2 <- b.bias1 <- rep(NA, I )
    M1 <- matrix( 1, nrow=N,ncol=I)

    for (ii in 1:I){
        # ii <- 1
        M1.ii <- M1
        M1.ii[,ii] <- 0
        M1.ii <- M1 - M1.ii
        data.resp.ii <- data.resp[,ii]

        #******
        # bias correction method 1
        s1 <- rowWgtMeans( u_it_alpha * M1.ii, data.resp )
        s0 <- rowWgtMeans( u_it_theta * M1.ii, data.resp )
        I_i <- sum( - ( s0 - s1^2 / s2 ) * data.resp.ii / sum(data.resp.ii) )
        t1a <- rowWgtMeans( v_it * u_it_alpha * M1.ii, data.resp )
        t2c1 <- t1c1 <- rowWgtMeans( u_it_alpha* M1.ii, data.resp )
        t2a <- rowWgtMeans( u_it_alpha2* M1.ii, data.resp )
        # term
        btheta <- t0 * ( - t1z * ( t1a - t1b * t1c1 / t1c2 )
                        - t2z * ( t2a - t2b  * t2c1 / t2c2 ) )
        btheta <- sum( btheta * data.resp.ii / sum(data.resp.ii ) )
        b.bias1[ii] <- btheta / I_i

        #******
        # bias correction method 2
        a1 <- rowWgtMeans( u_it^2 * M1.ii, data.resp )
        a4 <- a2 <- rowWgtMeans( u_it * v_it * M1.ii, data.resp )
        a3 <- rowWgtMeans( v_it^2, data.resp )
        I_i <- sum( - ( a1 - a2 * a3 * a4 ) )
        U_it <- u_it * M1.ii - a2 / a3 * v_it
        V_2it <- v_it^2 + v_it_alpha
        t1 <- rowSums( U_it * V_2it  * data.resp )
        t2 <- 2*rowSums( v_it_alpha  * data.resp )
        b.bias2[ii] <- - sum( t1/t2  ) / I_i

            }

    b.analytcorr1 <- b - b.bias1 / I1
    b.analytcorr2 <- b - b.bias2 / I1

#    ind <- which( abs(b) > adjmin )
#    b.analytcorr1b <- mean( ( b.analytcorr1a / b )[ind] )*b
#    b.analytcorr2b <- mean( ( b.analytcorr2a / b )[ind] )*b

    dfr <- data.frame( "b.JML"=b, "b.JMLcorr"=(I1-1)/I1*b,
        b.analytcorr1, # b.analytcorr1b,
        b.analytcorr2  #, b.analytcorr2b
        )

    res <- list( "b.biascorr"=dfr, "b.bias1"=b.bias1, "b.bias2"=b.bias2,
            "itemfac"=I1 )
    return(res)
        }
#########################################################################################
# auxiliary function for bias correction
rowWgtMeans <- function( M, W ){
    rowSums( M*W ) / rowSums(W)
            }
#........................................................................................

## File Name: xxirt_em_args_extract.R
## File Version: 0.02

xxirt_em_args_extract <- function(em_args, em_out, object)
{
    v1 <- em_out[[object]]
    if (is.null(v1)){
        v1 <- em_args[[object]]
    }
    return(v1)
}

## File Name: truescore.irt.R
## File Version: 0.252


#--- true score item response function
truescore.irt <- function( A, B, c=NULL, d=NULL, theta=seq(-3,3,len=21),
        error=NULL, pid=NULL, h=.001 )
{
    if ( is.vector(B) ){ B <- matrix( B, ncol=1 ) }
    if ( is.vector(A) ){ A <- matrix( A, ncol=1 ) }
    nB <- ncol(B)
    maxK <- nB - rowSums( is.na( B ))
    I <- nrow(B)
    if ( is.null(c) ){ c <- rep(0,I ) }
    if ( is.null(d) ){ d <- rep(1,I ) }
    if ( is.null(pid) ){ pid <- 1L:(length(theta)) }
    # true score
    truescore <- truescore_irt_irf( A=A, B=B, c=c, d=d, theta=theta )
    # calculate standard error of true score
    if ( is.null( error ) ){
        truescore.error <- NULL } else {
        truescore1 <- truescore_irt_irf( A=A, B=B, c=c, d=d, theta=theta + h )
        truescore.error <- sqrt( ( ( truescore1 - truescore ) /  h )^2 ) * error
    }
    percscore <- truescore / sum( maxK )
    percscore.error <- NULL
    if ( ! is.null( error ) ){
        percscore.error <- truescore.error / sum( maxK )
    }
    # optimization function values
    ind <- intersect( which( !is.na( theta ) ), which( !is.na(percscore) ) )
    x0 <- theta[ind]
    y0 <- percscore[ind]
    minf <- sum( ifelse( maxK==1, c, 0 ) ) / sum(maxK)
    maxf <- sum( ifelse( maxK==1, d, maxK ) ) / sum(maxK)
    critfct <- function(x) {
        a <- x[1]
        b <- x[2]
        # define fit function
        sum( ( y0 - minf - (maxf - minf) * stats::plogis( a*x0 + b ) )^2 )
    }
    h1 <- stats::optim( c( .5, 0 ), critfct )

    #--- OUTPUT
    res <- data.frame( pid=pid, truescore=truescore )
    res$truescore.error <- truescore.error
    res2 <- data.frame( percscore=percscore )
    res2$percscore.error <- percscore.error
    res3 <- data.frame( lower=minf, upper=maxf, a=h1$par[1], b=h1$par[2] )
    res <- cbind( res, res2, res3 )
    return(res)
}

## File Name: invariance_alignment_center_parameters.R
## File Version: 0.130

invariance_alignment_center_parameters <- function(alpha0, psi0, center,
    reparam=FALSE, convert=FALSE, meth=1)
{
    if (reparam & convert){
        alpha0 <- alpha0 * psi0
    }
    if (center){
        alpha0 <- alpha0 - mean(alpha0)
        log_psi <- log(psi0)
        psi0 <- exp(log_psi - mean(log_psi))
    }
    if (meth %in% c(0,0.5) ){
        fac <- 1/psi0[1]
        psi0 <- fac*psi0
        alpha0 <- fac*alpha0
        if (meth==0){
            alpha0 <- alpha0 - alpha0[1]
        }
    }
    #--- output
    res <- list(alpha0=alpha0, psi0=psi0)
    return(res)
}

## File Name: lsem_group_moderator.R
## File Version: 0.178


#***** grouping a moderator variable
lsem_group_moderator <- function( data, type, moderator.grid,
            moderator, residualize, h, is_imputed=FALSE, Nimp=0 )
{
    data0 <- data
    moderator.grouped <- NULL
    if (type=='MGM'){
        G1 <- length(moderator.grid)
        moderator.grouped <- data.frame( min=moderator.grid[-G1],
                                            max=moderator.grid[-1] )
        moderator.grouped$mid <- rowMeans( moderator.grouped)
        if (! is_imputed){
            v1 <- data[, moderator ]
            v2 <- moderator.grouped$mid[1]
            for (gg in 2L:G1){
                v2 <- ifelse( v1 > moderator.grouped$max[gg-1],
                            moderator.grouped$mid[gg], v2 )
            }
            data[,moderator] <- v2
        }
        if (is_imputed){
            for (ii in 1L:Nimp){
                data <- data0[[ii]]
                v1 <- data[, moderator ]
                v2 <- moderator.grouped$mid[1]
                for (gg in 2L:G1){
                    v2 <- ifelse( v1 > moderator.grouped$max[gg-1],
                                moderator.grouped$mid[gg], v2 )
                }
                data[,moderator] <- v2
                data0[[ii]] <- data
            }
        }
        # residualize <- FALSE
        h <- 1E-5
        moderator.grid <- moderator.grouped$mid
    }
    res <- list( data=data, moderator.grouped=moderator.grouped,
                residualize=residualize, h=h,
                moderator.grid=moderator.grid )
    return(res)
}


lsem.group.moderator <- lsem_group_moderator

## File Name: weighted_stats_extend_wgt.R
## File Version: 0.09

weighted_stats_extend_wgt <- function( wgt, mat )
{
    N1 <- nrow(mat)
    N2 <- ncol(mat)
    if ( is.null(wgt) ){
        wgt <- rep( 1, N1 )
    }
    if ( is.vector(wgt) ){
        wgt <- matrix( wgt, nrow=N1, ncol=N2 )
    }
    return(wgt)
}

## File Name: pgenlogis.R
## File Version: 1.11

#-- generalized logistic distribution
pgenlogis <- function( x, alpha1=0, alpha2=0 )
{
    x <- as.vector(x)
    xp <- x[ x >=0 ]
    xn <- x[ x < 0 ]
    indp <- which( x >=0 )
    indn <- which( x < 0 )
    # positive part of x
    if( ( alpha1 > 0 ) ){ y <-  ( ( exp( alpha1 * xp ) - 1  ) / alpha1  ) }
    if( ( alpha1==0 ) ) { y <-  xp  }
    if( ( alpha1 < 0 ) ) { y <- ( - log( 1 - alpha1 * xp )/alpha1 ) }
    # negative part of x
    if( ( alpha2 > 0 ) ){ y1 <-  - ( ( exp( alpha2 * abs(xn) ) - 1  ) / alpha2  ) }
    if( ( alpha2==0 ) ) { y1 <-  xn  }
    if( ( alpha2 < 0 ) ) { y1 <- (  log( 1 - alpha2 * abs(xn) )/alpha2 ) }
    # rearrange entries
    w <- rep( 0, length(x) )
    if ( length(indp) >  0 ){ w[ indp ] <- y }
    if ( length(indn) >  0 ){ w[ indn ] <- y1 }
    # transform to inverse logistic probability
    y <- stats::plogis(w)
    return(y)
}

## File Name: rm_sdt_mstep_rater_function_gradient.R
## File Version: 0.11


rm_sdt_mstep_rater_function_gradient <- function(x, par_index, partable_rater,
        pargroup_rater, I, K, numdiff.parm, nik.rater, eps=1E-10)
{
    probs_fun <- rm_sdt_calc_probs_grm_rcpp
    K1 <- K+1
    probs_dim <- c(I, K1, K1)
    probs_args <- list( I=I, K=K, eps=eps, use_log=TRUE )
    update_probs_args <- c('c.rater', 'd.rater')
    grad_post <- rm_sdt_mstep_type_function_gradient( x=x,
            par_index=par_index, partable=partable_rater, type='rater',
            pargroup_type=pargroup_rater, probs_args=probs_args, probs_fun=probs_fun,
            probs_dim=probs_dim, update_probs_args=update_probs_args, nik=nik.rater,
            numdiff.parm=numdiff.parm )
    return(grad_post)
}

## File Name: xxirt_nr_grad_fun_R.R
## File Version: 0.141

xxirt_nr_grad_fun_R <- function(x, em_args, eps=1e-100)
{
    NP <- em_args$NP
    NPI <- em_args$NPI
    NPT <- em_args$NPT
    free_pars_design <- em_args$free_pars_design
    h <- em_args$h

    grad <- 0*x

    #*** compute prior distribution
    prior_Theta0 <- xxirt_compute_prior_Theta_from_x(x=x, em_args=em_args)

    #* compute item response probabilities
    probs_items0 <- xxirt_compute_prob_item_from_x(x=x, em_args=em_args)
    p.xi.aj0 <- xxirt_compute_likelihood( probs_items=probs_items0,
                            dat=em_args$dat, dat_resp_bool=em_args$dat_resp_bool )
    ll_case0 <- xxirt_compute_casewise_likelihood(prior_Theta=prior_Theta0,
                            group=em_args$group, p.xi.aj=p.xi.aj0)
    ll0 <- sum( em_args$weights*log(ll_case0) )

    partable <- xxirt_partable_include_freeParameters( partable=em_args$partable, x=x )

    #---
    # derivatives of probabilities
    MIGC <- em_args$MIGC
    probs_items_der <- list()

    for (mm in 1L:MIGC){
        free_pars_design_mm <- free_pars_design[ free_pars_design$item_group_comp==mm, ]
        x1 <- x[ em_args$parindex_items ]
        x1 <- sirt_add_increment(x=x1, pos=free_pars_design_mm$pid, value=h)
        partable1 <- xxirt_partable_include_freeParameters( partable=em_args$partable,
                                x=x1 )
        probs_temp <- xxirt_compute_itemprobs( item_list=em_args$item_list,
                                items=em_args$items, Theta=em_args$Theta,
                                ncat=em_args$ncat, partable=partable1,
                                partable_index=em_args$partable_index )
        probs_items_der[[mm]] <- (probs_temp-probs_items0)/h
    }

    #--- loop over item parameters
    for (pp in 1L:NPI){

        free_pars_design_pp <- free_pars_design[pp,]
        partable_pp <- partable[ partable$parlabel==free_pars_design_pp$parlabel, ]
        item_pp <- free_pars_design_pp$itemnr
        item_group_comp_pp <- free_pars_design_pp$item_group_comp

        #- shortcut for item parameter per one item
        if (free_pars_design_pp$one_item){
            probs_items_der_pp <- ( probs_items_der[[item_group_comp_pp]] )[item_pp,,]
            probs_items0_pp <- probs_items0[item_pp,,]
            ratio <- probs_items_der_pp / ( probs_items0_pp + eps)
            ind_pp <- em_args$dat_resp_bool[,item_pp]
            p_xi_aj_der <- ratio[ em_args$dat[, item_pp ]+1, ] * p.xi.aj0
            p_xi_aj_der[! em_args$dat_resp_bool[,item_pp],] <- 0
            ll_case_der <- xxirt_compute_casewise_likelihood(prior_Theta=prior_Theta0,
                                    group=em_args$group, p.xi.aj=p_xi_aj_der)
            grad[pp] <- -sum( em_args$weights*ll_case_der / ( ll_case0+eps ) )
        } else {
            x1 <- x[ em_args$parindex_items ]
            x1 <- sirt_add_increment(x=x1, pos=pp, value=h)
            probs_items_temp <- xxirt_compute_prob_item_from_x(x=x1, em_args=em_args)
            p.xi.aj.temp <- xxirt_compute_likelihood( probs_items=probs_items_temp,
                                dat=em_args$dat, dat_resp_bool=em_args$dat_resp_bool)
            ll_case_temp <- xxirt_compute_casewise_likelihood(prior_Theta=prior_Theta0,
                                group=em_args$group, p.xi.aj=p.xi.aj.temp)
            ll_temp <- sum( em_args$weights*log(ll_case_temp) )
            grad[pp] <- -( ll_temp - ll0 ) / h

        }

    }        # end pp

    #--- loop over distribution parameters
    if (NPT>0){
        for (pp in em_args$parindex_Theta){
            x1 <- sirt_add_increment(x=x, pos=pp, value=h)
            prior_Theta <- xxirt_compute_prior_Theta_from_x(x=x1, em_args=em_args)
            prior_Theta_der <- ( prior_Theta - prior_Theta0 ) / h
            ll_case <- xxirt_compute_casewise_likelihood(prior_Theta=prior_Theta_der,
                                group=em_args$group, p.xi.aj=p.xi.aj0)
            grad[pp] <- -sum( em_args$weights*ll_case/(ll_case0 + eps) )
        }
    }
    return(grad)
}

## File Name: xxirt_partable_extract_freeParameters.R
## File Version: 0.121


xxirt_partable_extract_freeParameters <- function( partable )
{
    partable <- partable[ partable$est, ]
    partable <- partable[ order(partable$parindex), ]
    partable <- partable[ ! duplicated( partable$parindex), ]
    x <- partable$value
    names(x) <- partable$parlabel
    return(x)
}

## File Name: sqrt_diag.R
## File Version: 0.03

sqrt_diag <- function(x, names=NULL)
{
    v1 <- diag(x)
    v1 <- v1*(v1>0)
    res <- sqrt(v1)
    names(res) <- colnames(x)
    if (!is.null(names)){
        names(res) <- names
    }
    return(res)
}

## File Name: xxirt_compute_casewise_likelihood.R
## File Version: 0.085

xxirt_compute_casewise_likelihood <- function(prior_Theta, group, p.xi.aj,
        customTheta=NULL)
{
    if (is.null(customTheta)){
        customTheta <- list(person_covariates=FALSE)
    }
    if (customTheta$person_covariates){
        prior1 <- t( prior_Theta )
    } else {
        prior1 <- t( prior_Theta[, group ] )
    }
    p.aj.xi <- prior1 * p.xi.aj
    ll_case <- rowSums(p.aj.xi)
    return(ll_case)
}

## File Name: noharm_sirt_preproc.R
## File Version: 0.409


#**** data preprocessing noharm.sirt
noharm_sirt_preproc <- function( dat, pm, N, weights, Fpatt, Fval,
    Ppatt, Pval, Psipatt, Psival, wgtm, dimensions, pos.loading,
    pos.variance, pos.residcorr, input_pm, lower, upper )
{

    res <- NULL
    res$dat0 <- dat

    if ( ! input_pm ){
        N <- nrow(dat)
        I <- ncol(dat)
        items <- colnames(dat)
    } else {
        I <- ncol(pm)
        items <- colnames(pm)
        pm <- as.matrix(pm)
    }

    if (length(lower)==1){
        lower <- rep(lower, I)
    }
    if (length(upper)==1){
        upper <- rep(upper, I)
    }

    #---- data processing
    res$N <- N
    res$I <- I

    if (! input_pm){
        if (is.null(weights)){
            weights <- rep(1, N)
        }
        res$weights <- weights
        dat.resp <- 1-is.na(dat)
        dat0 <- dat
        dat[dat.resp==0] <- 0
        # calculate (weighted) product moment correlation
        ss <- as.matrix( crossprod( weights*dat.resp ) )
        eps <- 1e-6
        pm <- as.matrix( crossprod( as.matrix(dat*weights*dat.resp) ) / ( ss+eps ) )
    } else {
        dat.resp <- NULL
        ss <- 1
    }
    res$dat <- dat
    res$dat.resp <- dat.resp
    res$pm <- pm
    res$pm0 <- pm
    res$ss <- ss
    res$lower <- lower
    res$upper <- upper

    # CFA or EFA?
    if ( is.null(dimensions) ){
        model.type <- 'CFA'
        modtype <- 3    # 3 - multidimensional CFA
    } else {
        model.type <- 'EFA'
        modtype <- 2    # 2 - multidimensional EFA
        D <- dimensions
        Pval <- diag(D)
        Ppatt <- 0*diag(D)
        Fpatt <- matrix(1,nrow=I,ncol=D)
        if (D>1){
            for (dd in 2L:D){
                Fpatt[dd,1L:(dd-1)] <- 0
            }
        }
        Fval <- .5*(Fpatt>0)
        if ( D==1 ){ # 1 dimension
            model.type <- 'CFA'
            modtype <- 3
        }
    }

    res$model.type <- model.type
    res$modtype  <- modtype
    # initial values if they are not provided
    if ( is.null(Psival) ){ Psival <- 0*diag(res$I) }
    if ( is.null(Psipatt) ){ Psipatt <- 0*diag(res$I) }
    if ( is.null(Fval) ){ Fval <- .5*(Fpatt>0) }
    if ( is.null(Pval) ){ Pval <- diag( ncol(Ppatt) ) }
    # weight matrix
    wgtm.default <- FALSE
    if ( is.null(wgtm) ){
        wgtm <- matrix(1, nrow=I, ncol=I)
        wgtm.default <- TRUE
    }
    diag(wgtm) <- 0
    wgtm <- ( wgtm + t(wgtm) ) / 2
    wgtm <- wgtm * ( ss > 0 )
    res$wgtm <- wgtm
    res$sumwgtm <- ( sum( wgtm > 0 ) - sum( diag(wgtm) > 0 ) ) / 2

    #*** column names
    D <- ncol(Ppatt)
    cn <- paste0('F',1L:D)
    if (is.null(colnames(Fpatt) ) ){
        colnames(Fpatt) <- cn
    }
    if (is.null(colnames(Fval) ) ){
        colnames(Fval) <- colnames(Fpatt)
    }

    if (is.null(colnames(Pval))){
        colnames(Pval) <- colnames(Ppatt)
    }
    if (is.null(colnames(Pval))){
        colnames(Pval) <- colnames(Fval)
    }
    rownames(Pval) <- colnames(Pval)

    #--- create parameter table

    # F
    parm_table <- noharm_sirt_preproc_parameter_table_matrix(pattmat=Fpatt, valmat=Fval,
            patt_id=1, patt_label='F', minval=0, symm=FALSE)
    # P
    parm1 <- noharm_sirt_preproc_parameter_table_matrix(pattmat=Ppatt, valmat=Pval,
            patt_id=2, patt_label='P', minval=sirt_max(parm_table$index), symm=TRUE)
    parm_table <- rbind(parm_table, parm1)

    # Psi
    parm1 <- noharm_sirt_preproc_parameter_table_matrix(pattmat=Psipatt, valmat=Psival,
            patt_id=3, patt_label='Psi', minval=sirt_max(parm_table$index), symm=TRUE)
    parm_table <- rbind(parm_table, parm1)
    parm_table <- parm_table[ parm_table$nonnull_par==1, ]
    rownames(parm_table) <- NULL
    npar <- max(parm_table$index, na.rm=TRUE)

    # indices
    ip <- parm_table$index
    ip[duplicated(ip)] <- NA
    extract_index <- match(1L:npar, ip)
    extract_index <- intersect( which( ! is.na( parm_table$index ) ),
                            which( ! duplicated( parm_table$index ) ) )
    parm_table$est <- parm_table$starts

    parm_table$lower <- -Inf

    ind <- NULL
    if (pos.variance){
        ind1 <- which((parm_table$mat=='P') & (parm_table$row==parm_table$col ))
        ind <- union(ind, ind1)
    }
    if (pos.loading){
        ind1 <- which((parm_table$mat=='F') )
        ind <- union(ind, ind1)
    }
    if (pos.residcorr){
        ind1 <- which((parm_table$mat=='Psi') )
        ind <- union(ind, ind1)
    }
    parm_table[ind, 'lower'] <- 0
    parm_table[ is.na(parm_table$index), 'lower'] <- NA
    non_fixed <- ! parm_table$fixed
    include_index <- parm_table$index[ non_fixed ]
    parm_table$nonnull_par <- NULL
    attr(parm_table, 'extract_index') <- extract_index
    attr(parm_table, 'non_fixed') <- non_fixed
    attr(parm_table, 'include_index') <- include_index
    attr(parm_table, 'npar') <- npar
    attr(parm_table, 'NH') <- sum(non_fixed)
    attr(parm_table, 'est_par_index') <- which(parm_table$est_par==1)
    attr(parm_table, 'parm_table_free_index') <- which(parm_table$fixed==0)

    parm_index <- list()
    for (mat_label in c('F', 'P', 'Psi') ){
        ind_mat <- which(parm_table$mat==mat_label)
        parm_index[[ mat_label ]] <- list()
        parm_index[[ mat_label ]][[ 'row_parm_table' ]] <- ind_mat
        parm_index[[ mat_label ]][[ 'entries' ]] <- parm_table[ ind_mat, c('row','col')]
        parm_index[[ mat_label ]][[ 'len' ]] <- length(ind_mat)
        nrow <- I
        ncol <- D
        if (mat_label=='P'){ nrow <- D}
        if (mat_label=='Psi'){ ncol <- I}
        parm_index[[ mat_label ]][['nrow']] <- nrow
        parm_index[[ mat_label ]][['ncol']] <- ncol
    }

    res$parm_table <- parm_table
    res$npar <- npar
    res$parm_index <- parm_index

    #*****
    # matrix conversion
    #****
    res$Fpatt <- as.matrix(Fpatt)
    res$Fval <- as.matrix(Fval)
    res$Ppatt <- as.matrix(Ppatt)
    res$Pval <- as.matrix(Pval)
    res$Psipatt <- as.matrix(Psipatt)
    res$Psival <- as.matrix(Psival)
    res$Psipatt <- ( Psipatt + t(Psipatt) )/2
    res$Ppatt <- ( Ppatt + t(Ppatt) )/2
    res$D <- ncol(Ppatt)
    res$Fpatt <- 1*(res$Fpatt>0)
    res$Ppatt <- 1*(res$Ppatt>0)
    res$Psipatt <- 1*(res$Psipatt>0)
    res$wgtm.default <- wgtm.default
    res$F_dimnames <- colnames(Fpatt)
    res$items <- items
    return(res)
}

## File Name: summary.rasch.pairwise.R
## File Version: 0.09



# Summary for rasch.pairwise objects
summary.rasch.pairwise <- function( object, digits=3, file=NULL, ...)
{
    # open sink for a file
    sirt_osink(file=file)

    display_string <- sirt_summary_print_display(symbol="-", len=65)
    cat(display_string)

    #- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    cat(paste0("Function '", object$fct, "'"), "\n\n")

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object)

    cat(display_string)
    cat("Pairwise likelihood estimation \n")
    cat("Rasch Model \n")
    cat(display_string)
    cat("Item Parameters \n")
    obji <- object$item
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    sirt_csink(file=file)
}

## File Name: xxirt_summary_parts.R
## File Version: 0.088
#
xxirt_summary_parts <- function(object, digits, len_disp=66)
{
    sirt_display_function(length=len_disp)
    #- package and R session
    sirt_summary_print_package_rsession(pack='sirt')

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object)

    #    modeltype <- object$irtmodel
    cat( '   ', object$ic$n, 'Cases, ', object$ic$I, 'Items, ',
                    object$G, 'Group(s)',     '\n')

    sirt_display_function(length=len_disp)

    est_ml <- object$estimator=='ML'

    cat( 'Estimator','=', object$estimator, '\n' )
    if (est_ml){
        cat( 'Number of EM iterations','=', object$iter_em, '\n' )
    }
    cat( 'Number of Newton-Raphson iterations','=', object$iter_nr, '\n' )
    if (est_ml){
        cat( 'Deviance','=', round( object$deviance, 2 ), ' | ' )
        cat( 'Log Likelihood','=', round( -object$deviance/2, 2 ), '\n' )
        cat( 'Penalty function','=', round( object$pen_val, 4 ), '\n' )
    } else {
        cat( 'Optimization function','=', round( object$res_opt_nr$objective, 2), '\n')
    }
    cat( 'Number of persons','=', object$ic$n, '\n' )

    cat( 'Number of estimated parameters','=', object$ic$np, '\n' )
    cat( '  Number of estimated item parameters','=', object$ic$np.item, '\n' )
    cat( '  Number of estimated distribution parameters','=', object$ic$np.Theta, '\n\n' )

    #--- information criteria
    if (est_ml){
        rm_summary_information_criteria(object=object)
    }

    #- trait parameters
    sirt_display_function(length=len_disp)
    cat('Trait Parameters\n')
    obji <- object$customTheta$par
    sirt_summary_print_objects(obji=obji, digits=digits, from=1)

    #- item parameters
    sirt_display_function(length=len_disp)
    cat('Item Parameters \n')
    obji <- object$par_items_summary
    sirt_summary_print_objects(obji=obji, digits=digits, from=2)
}

## File Name: modelfit.cor.poly.R
## File Version: 0.08


##########################################################################
# model fit for polytomous item responses
modelfit.cor.poly <- function( data, probs, theta.k, f.qk.yi){
    # create input for tam.modelfit
    tamobj <- list()
    resp.ind <- 1 - is.na(data)
    resp <- data
    resp[ is.na(data) ] <- 0
    tamobj$resp <- as.matrix(resp)
    tamobj$resp.ind <- resp.ind
    tamobj$rprobs <- probs
    tamobj$theta <- vec2mat.sirt(theta.k)
    tamobj$hwt <- f.qk.yi
    # apply tam.modelfit
    tmod <- TAM::tam.modelfit( tamobj)
    # tmod <- tam.modelfit( tamobj)
    return(tmod)
            }
##########################################################################

## File Name: rasch.pairwise.itemcluster.R
## File Version: 0.35


#***** Pairwise estimation with itemclusters
rasch.pairwise.itemcluster <- function( dat, itemcluster=NULL,
            b.fixed=NULL, weights=NULL, conv=.00001, maxiter=3000,
            progress=TRUE, b.init=NULL, zerosum=FALSE)
{
    s1 <- Sys.time()
    CALL <- match.call()
    if ( is.null(b.init) ){
        b.init <- - stats::qlogis( colMeans( dat, na.rm=TRUE ) )
    }
    if (is.null(weights)){
        weights <- rep(1,nrow(dat))
    }
    weights0 <- weights
    I <- ncol(dat)
    dat <- as.matrix(dat)
    dat00 <- dat
    dat[ is.na(dat) ] <- 9
    b <- b.init
    if ( ! is.null(b.fixed) ){
        b[ b.fixed[,1] ] <- b.fixed[,2]
        b.fixed <- cbind( b.fixed, exp( b.fixed[,2] ) )
        zerosum <- FALSE
    }
    # create count tables
#    weights <- weights / sum(weights) * nrow(dat)
    sw <- sqrt(weights)
    dat0 <- sw*(dat==0)
    dat1 <- sw*(dat==1)
    Aij <- crossprod(dat0, dat1)
    Aji <- t(Aij)
    # set some entries to zero for itemclusters
    clusters <- unique( itemcluster[ itemcluster !=0 ] )
    CC <- length(clusters)
    for (cc in clusters){
        icc <- which( itemcluster==cc )
        Aji[icc,icc] <- Aij[icc,icc] <- 0
    }
    nij <- Aij + Aji
    Aij_rowsums <- rowSums(Aij)
    eps0 <- eps <- exp(b)
    max.change <- 10
    iter <- 1
    tol <- 1e-10
    #**** start algorithm
    while( max.change > conv ){
        b0 <- b
        eps0 <- eps
        m1 <- sirt_matrix2( eps0, nrow=I) + matrix( eps0, nrow=I, ncol=I )
        m1 <- m1+tol
        g1 <- rowSums(nij/m1)
        eps <- Aij_rowsums/(g1+tol)
        b <- log(eps+tol)
        # include item parameter constraints
        if ( ! is.null(b.fixed) ){
            eps[ b.fixed[,1] ] <- b.fixed[,3]
        }
        if (zerosum){
            b1 <- -b
            b2 <- b1-mean(b1)
            eps <- exp(-b2)
            b <- log(eps)
        }
        max.change <- max(abs( b - b0 ))
        if (progress){
            cat( "PL Iter.", iter, ": max. parm. change=", round( max.change, 6 ), "\n")
            utils::flush.console()
        }
        iter <- iter + 1
    }  #** end algorithm

    #** post-processing
    item <- data.frame( N=colSums(1-is.na(dat00)),
                    p=colMeans(dat00, na.rm=TRUE), b=log(eps) )
    if ( is.null(itemcluster) ){
        itemcluster <- rep(0,I)
    }
    item$itemcluster <- itemcluster

    #-- output
    s2 <- Sys.time()
    res <- list( b=b, eps=eps, iter=iter, conv=conv, dat=dat00,item=item,
            fct='rasch.pairwise.itemcluster', itemcluster=itemcluster,
            s1=s1, s2=s2, CALL=CALL )
    class(res) <- 'rasch.pairwise'
    return(res)
}

## File Name: rasch_mml2_rmsd.R
## File Version: 0.02

rasch_mml2_rmsd <- function(n.jk, r.jk, rprobs, pi.k, dat)
{
    requireNamespace("miceadds")
    dims <- dim(n.jk)
    n.ik <- array(0, dim=c(dims[1],2,dims[2],dims[3]))
    n.ik[,2,,] <- r.jk
    n.ik[,1,,] <- n.jk - r.jk
    n.ik <- aperm(n.ik, perm=c(3,1,2,4))
    probs <- aperm(rprobs, perm=c(3,1,2))
    dim(probs)[[2]] <- ncol(dat)
    rmsd <- CDM::IRT_RMSD_calc_rmsd( n.ik=n.ik, pi.k=pi.k, probs=probs, eps=1E-30 )
    args <- list( n.ik=n.ik, pi.k=pi.k, probs=probs, eps=1E-30 )
    miceadds::Reval( paste0("rmsd$MD <- do.call( CDM:::","IRT_RMSD_calc_md, args=args)"),
                        print.string=FALSE, n.eval.parent=1)
    items <- colnames(dat)
    for (vv in c("RMSD","RMSD_bc","MD","MAD")){
        names(rmsd[[vv]]) <- items
    }
    return(rmsd)
}

## File Name: IRT.mle.R
## File Version: 0.351


#--- IRT.mle
IRT.mle <- function(data, irffct, arg.list, theta=rep(0,nrow(data)), type="MLE",
        mu=0, sigma=1, maxiter=20, maxincr=3, h=0.001, convP=1e-04, maxval=9,
        progress=TRUE)
{
    N <- length(theta)
    I <- ncol(data)
    iter <- 1
    MAP <- WLE <- FALSE
    conv <- 1E3
    eps <- 1E-6

    if ( type=="WLE" ){ WLE <- TRUE }
    if ( type=="MAP" ){ MAP <- TRUE }

    #-------- begin algorithm
    while ( ( conv > convP ) & (  iter <=maxiter  ) ){
        theta0 <- theta
        ll0 <- ll1 <- ll2 <- rep(0,N)
        llP2 <- llP1 <- llP0 <- llM1 <- llM2 <- rep(0,N)
        for (ii in 1:I){
            # ii <- 2
            arg.list$ii <- ii
            # theta
            arg.list$theta <- theta
            llP0 <- llP0 + calc.ll( do.call( irffct, arg.list ), data, ii )
            # theta + h
            arg.list$theta <- theta+h
            llP1 <- llP1 + calc.ll( do.call( irffct, arg.list ), data, ii )
            # theta - h
            arg.list$theta <- theta-h
            llM1 <- llM1 + calc.ll( do.call( irffct, arg.list ), data, ii )
            if (WLE){
                # theta + 2*h
                arg.list$theta <- theta+2*h
                llP2 <- llP2 + calc.ll( do.call( irffct, arg.list ), data, ii )
                # theta - 2*h
                arg.list$theta <- theta-2*h
                llM2 <- llM2 + calc.ll( do.call( irffct, arg.list ), data, ii )
            }
        }# end item ii
        #-----
        # prior distribution
        if (MAP){
           llP0 <- llP0 + sirt_dnorm( theta, mean=mu,sd=sigma, log=TRUE)
           llP1 <- llP1 + sirt_dnorm( theta+h, mean=mu,sd=sigma, log=TRUE)
           llP2 <- llP2 + sirt_dnorm( theta+2*h, mean=mu,sd=sigma, log=TRUE)
           llM1 <- llM1 + sirt_dnorm( theta-h, mean=mu,sd=sigma, log=TRUE)
           llM2 <- llM2 + sirt_dnorm( theta-2*h, mean=mu,sd=sigma, log=TRUE)
        }
        #-----
        #*** likelihood
        ll0 <- llP0
        ll1 <- ( llP1 - llP0 ) / h
        ll2 <- ( llP1 - 2*llP0 + llM1 ) / (h^2)
        # ability increments
        M1 <- ll1
        ll2 <- ll2 + eps
        incr <- - ll1 / ll2
        # WLE bias correction term
        if (WLE){
            ll3 <- ( llP2 - 2*llP1 + 2*llM1 - llM2 ) / (2*h^3 )
            incr <- - ll1 / ll2 - ll3 / ( 2*ll2^2 )
        }

        maxincr <- maxincr / 1.05
        incr <- ifelse( abs(incr) > maxincr, sign(incr)*maxincr, incr )
        theta <- theta + incr
        theta <- ifelse( abs(theta) > maxval, sign(theta)*maxval, theta )
        conv <- max( abs( theta - theta0) )
        if (progress){
            cat("* Iteration", iter, ":", "maximum parameter change",
                round( conv, 5 ), "\n")
            utils::flush.console()
        }
        iter <- iter + 1
    }
    #------------- end algorithm

    #--- output
    se <- sqrt( abs( - 1 / ll2 ) )
    se <-  ifelse( abs(theta)==maxval, NA, se )
    theta <- ifelse( theta==maxval, Inf, theta )
    theta <- ifelse( theta==- maxval, -Inf, theta )
    res <- data.frame( est=theta, se=se )

    #--- calculate reliability
    attr(res, "type") <- type
    if ( type %in% c("MLE","WLE") ){
        attr(res, "reliability") <- mle.reliability( res$est, res$se )
    }
    if ( type %in% c("MAP") ){
        attr(res, "reliability") <- eap.reliability( res$est, res$se )
    }
    #--- output
    return(res)
}


#--- calculate individual likelihood for item ii
calc.ll <- function( probs, data, ii, eps=1e-50 )
{
    N <- nrow(data)
    probs <- log(probs + eps)
    m1 <- matrix(1:N, nrow=N, ncol=2)
    m1[,2] <- data[,ii] + 1
    h1 <- probs[ m1 ]
    h1[ is.na(h1) ] <- 0
    return(h1)
}

## File Name: unidim.csn.R
## File Version: 0.206

##*** unidimensionality test
unidim.test.csn <- function( dat, RR=400, prop.perm=.75,
                progress=TRUE)
{
    dat <- as.matrix(dat)
    dat <- stats::na.omit(dat)
    score <- rowSums(dat)
    dat <- dat[ order(score), ]
    score <- rowSums(dat)
    ds <- diff(score)
    ind <- c(1,which( ds > 0 )+1 )
    score_index <- cbind( ind, c( ind[-1]-1, nrow(dat) ) )
    score_index <- cbind( score_index, score_index[,2] - score_index[,1] + 1)
    N <- nrow(dat)
    dat_perm <- matrix(NA, N, ncol(dat) )
    SS <- nrow(score_index)
    for (ss in 1L:SS){
        for (ii in 1L:ncol(dat)){
            iss <- score_index[ss,1]:score_index[ss,2]
            dat_perm[iss,ii] <- sample( dat[iss,ii] )
        }
    }
    score_index <- cbind( score_index, round( score_index[,3] * prop.perm ) )
    progress <- 1*progress
    progress_vec <- c(0,which( diff( floor( 10 * ( 1L:RR ) / ( RR+1 ) ) )==1 ) )
    if (progress==1){
        cat('|**********|\n')
    }
    res <- gooijer_csn_table( dat=dat, dat_perm=dat_perm, RR=RR, NS=0, progress=progress,
                    progress_vec=progress_vec, score_index=as.matrix(score_index) )
    res$p <- mean(res$stat < res$stat_perm)
    quants <- c(.05, .25, .50, .75, .95, .99, .999 )
    res$H0_quantiles <- stats::quantile(res$stat_perm, probs=quants )
    if ( progress==1 ){
        cat( paste0('CSN Statistic=', round( res$stat, 5) ) )
        cat( ', p=', round( res$p, 5 ), '\n')
    }
    return(res)
}

## File Name: rm_sdt_create_partable.R
## File Version: 0.502


rm_sdt_create_partable <- function( item.index, rater.index,
    est.c.rater, est.d.rater, tau.item, c.rater, diffindex,
    tau.prior, a.prior, d.prior, c.prior, est.a.item, tau.item.fixed,
    a.item.fixed, c.rater.fixed, index_no_est_value=-9 )
{
    I <- nrow(tau.item)
    ND <- nrow(c.rater)
    K <- ncol(c.rater)

    #--- tau.item
    v1 <- 1:(K*I) + 0*tau.item
    partable <- rm_sdt_create_parm_index_modify_elements(x=v1, start_index=1, type="tau")

    partable <- rm_sdt_create_partable_include_fixed_item_category_parameters(
                        partable=partable, tif=tau.item.fixed,
                        index_no_est_value=index_no_est_value )

    #--- a.item
    v1 <- seq_len(I)
    if (! est.a.item){
        v1 <- -9 + 0*v1
    }
    start <- max(partable$parindex) + 1
    if ( ! is.null(a.item.fixed) ){
        NT <- nrow(a.item.fixed)
        v1[ a.item.fixed[,1] ] <- index_no_est_value
    }
    v1 <- rm_sdt_create_parm_index_modify_elements(x=v1, start_index=start, type="a" )
    partable <- rbind( partable, v1)
    partable_item <- partable

    #--- c.rater
    v1 <- 0*c.rater
    g1 <- rm_sdt_create_parm_index_rater( est.rater=est.c.rater, ND=ND,
                item.index=item.index, rater.index=rater.index )
    M <- 0
    for (kk in 1:K){
        v1[,kk] <- g1 + M
        M <- max(v1[,kk])
    }
    start <- 1
    partable <- rm_sdt_create_parm_index_modify_elements(x=v1, start_index=start, type="c")

    partable <- rm_sdt_create_partable_include_fixed_item_category_parameters(
                        partable=partable, tif=c.rater.fixed, index_no_est_value=index_no_est_value )

    #--- d.rater
    v1 <- rm_sdt_create_parm_index_rater( est.rater=est.d.rater, ND=ND,
                item.index=item.index, rater.index=rater.index )
    start <- max(partable$parindex) + 1
    v1 <- rm_sdt_create_parm_index_modify_elements(x=v1, start_index=start, type="d")
    partable <- rbind( partable, v1)
    partable_rater <- partable

    partable_item <- rm_sdt_create_partable_include_index(partable=partable_item)
    partable_rater <- rm_sdt_create_partable_include_index(partable=partable_rater)

    #--- index lists
    par_index <- list()
    par_index$tau.item <- which(partable_item$type=="tau")
    par_index$a.item <- which(partable_item$type=="a")
    par_index$c.rater <- which(partable_rater$type=="c")
    par_index$d.rater <- which(partable_rater$type=="d")

    #--- parameter groups for differentiation
    partable_item <- rm_sdt_create_partable_define_pargroups(partable=partable_item,
                            pg1="tau", pg2="a")
    partable_rater <- rm_sdt_create_partable_define_pargroups(partable=partable_rater,
                            pg1="c", pg2="d")
    #--- diffindex pargroups
    pargroup_item <- rm_sdt_create_partable_pargroup_indices( partable=partable_item,
                    item.index=item.index, diffindex=diffindex )
    pargroup_rater <- rm_sdt_create_partable_pargroup_indices( partable=partable_rater,
                    item.index=item.index, diffindex=diffindex )

    #--- prior distributions
    partable$prior_M <- NA
    partable$prior_SD <- NA
    partable_item <- rm_sdt_create_partable_include_priors(partable=partable_item,
                                type="tau", obj=tau.prior)
    partable_item <- rm_sdt_create_partable_include_priors(partable=partable_item,
                                type="a", obj=a.prior)
    partable_rater <- rm_sdt_create_partable_include_priors(partable=partable_rater,
                                type="c", obj=c.prior)
    partable_rater <- rm_sdt_create_partable_include_priors(partable=partable_rater,
                                type="d", obj=d.prior)
    #--- output
    res <- list( partable_item=partable_item, partable_rater=partable_rater,
                par_index=par_index, pargroup_item=pargroup_item,
                pargroup_rater=pargroup_rater )
    return(res)
}

## File Name: likelihood_adjustment_aux.R
## File Version: 0.201


#######################################################
# compute adjusted likelihood
likelihood_adjustment_compute <- function( likelihood, theta,
            thetaM, adjfac, tuningfac=1)
{
    TP <- length(theta)
    # compute weighted mean and weighted SD
    res0 <- likelihood_moments( likelihood, theta=theta )
    M1 <- res0$M
    SD1 <- res0$SD
    w1 <- rowSums(likelihood)
    # compute adjusted likelihood
    like2 <- 0*likelihood
    for (tt in 1L:TP){
        like2[,tt] <- sirt_dnorm( theta[tt], mean=M1, sd=SD1*adjfac*tuningfac )
    }
    like2 <- like2 / rowSums(like2) * w1
    return(like2)
}
#############################################################
# EAP reliability
like_adj_EAP_reliability <- function( M, SD )
{
    stats::var(M) / ( stats::var(M) + mean( SD^2 ) )
}
#########################################################
# likelihood adjustment with a tuning parameter
likelihood_adjustment_tuning <- function( likelihood, theta, thetaM, adjfac,
        tuningfac, probs, maxiter=100, conv=5*1E-5, normal_approx=TRUE )
{

    like2 <- likelihood_adjustment_compute( likelihood, theta, thetaM, adjfac,
                tuningfac=tuningfac)
    change <- 1
    iter <- 0

    if (normal_approx){
        probs <- trait_normal_approx( probs, theta )
    }

    while( ( iter < maxiter ) & ( change > conv ) ){

        probsM <- matrix( probs, nrow(likelihood), ncol(likelihood), byrow=TRUE )
        post <- like2 * probsM
        post <- post / rowSums(post)
        probs_new <- colMeans( post )
        if (normal_approx){
            probs_new <- trait_normal_approx( probs_new, theta )
        }

        change <- max( abs( probs_new - probs ))
        probs <- probs_new
        iter <- iter + 1
    }

    res0 <- likelihood_moments( likelihood=like2 * probsM, theta=theta  )
    EAP.rel <- like_adj_EAP_reliability( res0$M, res0$SD )
    res <- list( likelihood=like2, EAP.rel=EAP.rel )
    return(res)
}

#*** normal approximation of trait distribution
trait_normal_approx <- function( probs, theta )
{
    M <- sum( theta*probs )
    SD <- sqrt( sum( theta^2*probs ) - M^2 )
    probs1 <- sirt_dnorm_discrete( theta, mean=M, sd=SD)
    return(probs1)
}

## File Name: rasch_mml2_mstep_one_step.R
## File Version: 1.076


rasch_mml2_mstep_one_step <- function(args0, prob_fun, entry, n.ik,
        diffindex, max.increment, numdiff.parm)
{

    # gg0 <- Sys.time()

    h <- numdiff.parm
    val0 <- args0[[entry]]
    pjk <- do.call(what=prob_fun, args=args0)
    args1 <- rasch_mml2_modify_list_element( x=args0, entry=entry, value=val0+h )
    pjk1 <- do.call(what=prob_fun, args=args1)
    args1 <- rasch_mml2_modify_list_element( x=args0, entry=entry, value=val0-h )
    pjk2 <- do.call(what=prob_fun, args=args1)

    # numerical differentiation
    res <- rasch_mml2_numdiff_index( pjk=pjk, pjk1=pjk1, pjk2=pjk2, n.ik=n.ik,
                    diffindex=diffindex, max.increment=max.increment,
                    numdiff.parm=numdiff.parm )

    # update
    args0[[entry]] <- args0[[entry]] + res$increment
    res$args0 <- args0
    #- output
    return(res)
}

# cat( " @@@@@ calc probs") ; gg1 <- Sys.time(); print(gg1-gg0) ; gg0 <- gg1

## File Name: conf.detect.R
## File Version: 1.212


# Confirmatory DETECT analysis
conf.detect <- function( data, score, itemcluster, bwscale=1.1, progress=TRUE,
                thetagrid=seq( -3,3,len=200), smooth=TRUE, use_sum_score=FALSE,
                bias_corr=TRUE)
{
    CALL <- match.call()
    cat('-----------------------------------------------------------\n' )
    cat('Confirmatory DETECT Analysis \n' )
    utils::flush.console()
    h1 <- is.matrix(score)
    if (h1){
        PP <- ncol(score)
    }
    is_one_score <- TRUE
    if (! h1 ){
        cat('Conditioning on 1 Score\n' )
    } else {
        cat(paste('Conditioning on ',PP, ' Scores\n', sep='') )
        is_one_score <- FALSE
    }
    cat(paste('Bandwidth Scale:', bwscale, '\n' ) )
    utils::flush.console()
    scale_score <- TRUE
    if (!smooth){
        scale_score <- FALSE
    }
    args_ccov_np <- list( data=data, score=score, bwscale=bwscale,
                            progress=progress, thetagrid=thetagrid, smooth=smooth,
                            scale_score=scale_score, use_sum_score=use_sum_score,
                            bias_corr=bias_corr)
    if ( ! h1 ){
        ccovtable <- do.call( what=ccov.np, args=args_ccov_np)
        res <- detect.index( ccovtable=ccovtable, itemcluster=itemcluster )
    } else {
        ccovtable.list <- list()
        args_ccov_np$progress <- FALSE
        for (pp in 1L:PP){
            cat( paste( 'DETECT Calculation Score ', pp, '\n', sep='') ) ;
            utils::flush.console()
            args_ccov_np$score <- score[,pp]
            ccovtable.list[[pp]] <- do.call( what=ccov.np, args=args_ccov_np)
        }
        detect.list <- lapply( ccovtable.list, FUN=function(ccovtable){
                    detect.index( ccovtable, itemcluster=itemcluster ) } )
        detect.matrix <- matrix( unlist( lapply( detect.list, FUN=function( ll){
                                c( ll[1,], ll[2,], ll[3,] ) } ) ), nrow=PP, byrow=TRUE)
        detect.summary <- data.frame( NScores=PP, Mean=colMeans( detect.matrix ),
                                SD=apply( detect.matrix, 2, stats::sd ),
                                Min=apply( detect.matrix, 2, min ),
                                Max=apply( detect.matrix, 2, max ) )
        rownames(detect.summary) <- c('DETECT Unweighted', 'DETECT Weighted',
                                        'ASSI Unweighted', 'ASSI Weighted',
                                        'RATIO Unweighted', 'RATIO Weighted' )
    }
    cat('-----------------------------------------------------------\n' )
    if ( ! h1){
        res <- list( detect=res, ccovtable=ccovtable, detect.summary=res )
    } else {
        res <- list( detect=detect.list, ccovtable=ccovtable.list,
                        detect.summary=detect.summary )
    }
    res$is_one_score <- is_one_score
    res$CALL <- CALL
    res$bwscale <- bwscale
    res$itemcluster <- itemcluster
    res$smooth <- smooth
    #--- print
    print(round(res$detect.summary,3))
    #--- return
    class(res) <- 'conf.detect'
    return(res)
}

## File Name: mgsem_proc_model_update_penalties_matrix.R
## File Version: 0.073

mgsem_proc_model_update_penalties_matrix <- function(partable, entries, model)
{

    ND <- nrow(partable)
    for (entry in entries){
        for (dd in 1L:ND){
            if (partable[dd,'unique']==1){
                group <- partable$group[dd]+1
                type_dd <- paste(partable$type[dd])
                mat_gg_ee <- model[[group]][[entry]]
                if (!is.null(mat_gg_ee)){
                    mat_gg_ee <- mat_gg_ee[[type_dd]]
                    if (!is.null(mat_gg_ee)){
                        i1 <- partable$i1[dd]
                        i2 <- partable$i2[dd]
                        mat_gg_ee[ i1, i2] <- partable[dd,entry]
                        model[[group]][[entry]][[type_dd]] <- mat_gg_ee
                    }
                }
            }  # end is entry to be changed
        }  # end dd
    }  # end entry
    return(model)
}

## File Name: lsem_parameter_summary.R
## File Version: 0.236


## lsem parameter summary
lsem_parameter_summary <- function( parameters, moderator.density, verbose )
{
    mod.density <- moderator.density
    NP <- max(parameters$parindex)
    if (verbose){
        cat('** Parameter summary \n\n')
        utils::flush.console()
    }
    parameters_summary <- NULL
    for (pp in 1L:NP){
        par.pp <- parameters[ parameters$parindex==pp, ]
        pars1 <- data.frame( par=paste(par.pp$par[1]), parindex=pp)
        x <- par.pp[,'est']
        pars1$M <- stats::weighted.mean( x, mod.density[,2] )
        pars1$SD <- lsem_wtdSD( x, mod.density[,2] )
        # pars1$Var <- pars1$SD^2
        pars1$MAD <- sum( mod.density[,2] * abs( x - pars1$M ) )
        pars1$Min <- min(x)
        pars1$Max <- max(x)
        mod1 <- stats::lm( x ~ mod.density[,1], weights=mod.density[,2] )
        pars1$lin_int <- stats::coef(mod1)[1]
        pars1$lin_slo <- stats::coef(mod1)[2]
        pars1$SD_nonlin <- lsem_wtdSD( stats::resid(mod1), mod.density[,2] )
        parameters_summary <- rbind( parameters_summary, pars1 )
    }
    return(parameters_summary)
}

lsem.parameter.summary <- lsem_parameter_summary

## File Name: invariance_alignment_define_parameters.R
## File Version: 0.215

invariance_alignment_define_parameters <- function(x, ind_alpha, ind_psi,
        fix_first_psi=TRUE, reparam=FALSE, constraint="prod", overparam=FALSE,
        meth=1)
{
    # meth=0=> free optimization with ridge regularization
    alpha0 <- c(0, x[ind_alpha])
    if (meth==0){
        alpha0 <- alpha0[-c(1)]
    }
    if (fix_first_psi){
        psi0 <- c(1, x[ind_psi])
    } else {
        psi0 <- c(x[ind_psi])
    }
    if (( reparam & ( constraint=='prod') ) | (meth %in% c(0,0.5) ) ){
        prod_psi <- prod(psi0)
        NX <- length(psi0)
        log_psi0 <- log(psi0)
        psi0 <- exp( log_psi0 - mean(log_psi0) )
        # psi0 <- psi0 / ( prod_psi^(1/NX) )
    }
    if (reparam & ( constraint=='sum') ){
        psi0 <- psi0 - mean(psi0) + 1
    }
    if (meth==0){
        # alpha0[1] <- -sum(alpha0[-c(1)])
        # alpha0 <- alpha0 - mean(alpha0)
        # Implementing the constraints leads to bias
    }
    #--- output
    res <- list(alpha0=alpha0, psi0=psi0)
    return(res)
}

## File Name: gom_em_numdiff_index.R
## File Version: 0.15


#--- general function for numerical differentiation
#--- diffindex aggregates across super items
gom_em_numdiff_index <- function( pjk, pjk1, pjk2, an.ik, diffindex,
        max.increment, numdiff.parm, eps=1e-5 )
{
    h <- numdiff.parm
    ll0 <- rowSums( an.ik * log(pjk+eps) )
    ll1 <- rowSums( an.ik * log(pjk1+eps) )
    ll2 <- rowSums( an.ik * log(pjk2+eps) )

    #- derivatives
    res <- rasch_mml2_difference_quotient(ll0=ll0, ll1=ll1, ll2=ll2, h=numdiff.parm)
    d1 <- res$d1
    d2 <- res$d2
    increment <- - d1 / d2
    increment <- sirt_trim_increment(increment=increment, max_increment=max.increment)
    #-- output
    res <- list(increment=increment, d2=d2, d1=d1, ll0=ll0)
    return(res)
}


.gom.numdiff.index <- gom_em_numdiff_index

## File Name: rasch_jml_itemfit.R
## File Version: 0.05



#*** item fit Rasch model
rasch_jml_itemfit <- function( theta0, b, dat )
{
    dat9 <- dat
    dat9[ is.na(dat)] <- 9
    ind <- is.finite(theta0)
    dat2 <- dat9[ ind, ]
    dat2.resp <- 1 * ( dat2 !=9 )
    theta0 <- theta0[ ind ]
    p <- .prob.rasch( theta0, b )
    v <- p * (1 - p)
    z2 <- dat2.resp * (dat2 - p)/v
    z2d <- z2 * dat2.resp
    infit <- colSums( z2d * v ) / colSums( v * dat2.resp )
    outfit <- colSums(z2d) / colSums( dat2.resp )
    res <- data.frame(infit, outfit )
    return(res)
}

rasch.itemfit <- rasch_jml_itemfit

## File Name: mirt_prodterms.R
## File Version: 0.091


# prodterms function from mirt package
# This function is not exported and hence redefined in sirt
mirt_prodterms <- function (theta0, prodlist)
{
    products <- matrix(1, ncol=length(prodlist), nrow=nrow(theta0))
    for (i in 1L:length(prodlist)) {
        tmp <- prodlist[[i]]
        for (j in 1L:length(tmp)){
            products[, i] <- products[, i] * theta0[, tmp[j]]
        }
    }
    ret <- cbind(theta0, products)
    return(ret)
}

## File Name: lsem_fitsem_raw_data_lavaan.R
## File Version: 0.136

lsem_fitsem_raw_data_lavaan <- function(dat, pseudo_weights, survey.fit,
        lavaan_est_fun, se, ...)
{
    #- define pseudo weights if requested
    res <- lsem_fitsem_raw_data_define_pseudo_weights(dat=dat,
                        pseudo_weights=pseudo_weights)
    dat1 <- res$dat
    sampling_weights <- res$sampling_weights  #=NULL for pseudo weights
    nobs_pseudo <- res$nobs_pseudo
    sum_weight <- res$sum_weight
    # use starting values
    partable <- sirt_import_lavaan_parameterTable(object=survey.fit)
    partable$start <- partable$est
    #- fit model with sampling weights
    survey.fit <- lavaan_est_fun(model=partable, data=dat1,
                        sampling.weights=sampling_weights, se=se, ... )

    #- adjust sample size
    survey.fit <- lavaan_object_adjust_sample_size(object=survey.fit, n_used=sum_weight)

    #-- output
    return(survey.fit)
}

## File Name: lsem_bootstrap_print_progress.R
## File Version: 0.091

lsem_bootstrap_print_progress <- function(rr, verbose, R)
{
    if (verbose){
        cat(rr, ' ')
        if (rr %% 10==0){ cat('\n')}
        if (rr==R){ cat('\n') }
        utils::flush.console()
    }
}

## File Name: xxirt_simulate.R
## File Version: 0.145

xxirt_simulate <- function(partable, customItems, Theta, customTheta, N=1e4,
                    method="random")
{

    #---- simulate theta distribution
    P_Theta <- customTheta$P
    args_Theta <- list(par=customTheta$par, Theta=Theta, G=1)
    probs_Theta <- do.call(what=customTheta$P, args=args_Theta)[,1]
    TP <- nrow(Theta)
    indices <- as.data.frame(matrix(NA, nrow=TP, ncol=3))
    colnames(indices) <- c('start', 'end', 'N')
    if (method=='random'){
        Theta_sim <- stats::rmultinom(n=N, size=1, prob=probs_Theta)
        N_Theta <- rowSums(Theta_sim)
        indices$N <- N_Theta
        indices$start <- c(1,cumsum(N_Theta)[-TP]+1)
        indices$end <- cumsum(N_Theta)
    }
    if (method=='quasiexact'){
        N_Theta <- N*probs_Theta
        N_Theta1 <- cumsum(N_Theta)
        N_Theta2 <- floor(N_Theta1)
        dec <- N_Theta1 - N_Theta2
        indices$end <- round(N_Theta1)
        indices$start <- c(1, indices$end[-TP]+1 )
        indices$N <- indices$end - indices$start + 1
    }
    #-- define index table for theta
    if( indices$start[2]==0){
        indices$start[1] <- 0
    }
    indices$start <- ifelse( indices$start==0 & indices$N > 0, 1, indices$start )
    indices$N <- ifelse( indices$N<1, 0, indices$N )

    #---- simulate item responses
    I <- max(partable$itemnr)
    items <- unique(paste(partable$item))
    ncat <- stats::aggregate(partable$ncat, list(partable$itemnr), max)[,2]

    dat <- as.data.frame( matrix(NA, nrow=N, ncol=I) )
    colnames(dat) <- items

    #- simulate random numbers
    mu <- rep(0,I)
    Sigma <- diag(I)
    exact <- TRUE
    if (method=='random'){ exact <- FALSE }
    rn <- rmvn(N=N, mu=mu, Sigma=Sigma, exact=exact)
    colnames(rn) <- items
    rn <- stats::pnorm(q=rn)
    # rn <- matrix( stats::runif(N*I), nrow=N, ncol=I)

    #--- simulate items
    TP_list <- which( indices$N > 0 )

    for (ii in 1L:I){

        item_ii <- items[ii]
        ncat_ii <- ncat[ii]
        partable_ii <- partable[ partable$itemnr==ii, ]
        type_ii <- paste(partable_ii$type[1])
        LC <- length(customItems)
        ll0 <- NA
        for (ll in 1L:LC){
            if (customItems[[ll]]$name==type_ii){
                ll0 <- ll
                customItems_ll <- customItems[[ll]]
            }
        }
        par <- partable_ii$value
        names(par) <- paste(partable_ii$parname)
        args_fun <- list( par=par, Theta=Theta, ncat=ncat_ii)
        probs <- do.call(what=customItems_ll$P, args=args_fun)

        for (tt in TP_list){
            indices_tt <- indices[tt,]
            N_tt <- indices_tt$N
            probs_tt <- probs[tt,]
            index_tt <- seq( indices_tt$start, indices_tt$end )
            sizes_tt <- probs_tt * N_tt
            # csizes_tt <- round(cumsum(sizes_tt))
            csizes_tt <- cumsum(sizes_tt)
            probs_tt <- csizes_tt/N_tt
            rn_tt <- as.vector(rn[ index_tt, ii ])
            eps <- 1e-7
            V <- 1-eps
            probs_tt[ probs_tt > V ] <- V
            quants <- stats::quantile( rn_tt, probs=probs_tt)
            if (method=='random'){
                quants <- cumsum(probs[tt,])
            }
            vals <- rep(0,N_tt)
            rn_ii <- rn[index_tt, ii ]
            hh <- 1
            for (hh in 1L:(ncat_ii-1)){
                vals <- ifelse( rn_ii > quants[hh], vals+1, vals )
            }
            dat[index_tt,ii] <- vals
        }  # end tt
    } # end ii

    #--- output
    return(dat)
}

## File Name: rasch_mirtlc_estep_lc.R
## File Version: 0.24


#--- E Step rasch_mirtlc
rasch_mirtlc_estep_lc <- function( dat1, dat2, dat2.resp, pi.k, pjk, I,
        group, G, theta.k,  f.qk.yi=NULL  )
{
    #--- array notation of probabilities
    TP <- nrow(pjk)
    pjkt <- t(pjk)
    pjkL <- array( NA, dim=c( I, 2, TP ))
    pjkL[,1,] <- 1 - pjkt
    pjkL[,2,] <- pjkt
    probsM <- matrix( aperm( pjkL, c(2,1,3) ), nrow=I*2, ncol=TP )
    f.yi.qk <- mml_calc_like( dat2=dat2, dat2resp=dat2.resp, probs=probsM )$fyiqk
    f.qk.yi <- 0 * f.yi.qk
    if (G==1){
        pi.k <- matrix( pi.k, ncol=1 )
    }
    if (G>1){
        for (gg in 1:G){
            ind.gg <- group==gg
            ngg <- length(ind.gg)
            pikM <- sirt_matrix2(x=pi.k[,gg], nrow=ngg)
            f.qk.yi[ ind.gg, ] <- f.yi.qk[ ind.gg, ] * pikM
        }
    }
    if (G==1){
        pikM <- sirt_matrix2( x=pi.k[,1], nrow=nrow(f.yi.qk))
        f.qk.yi <- f.yi.qk * pikM
    }
    f.qk.yi <- f.qk.yi / rowSums( f.qk.yi )
    # expected counts at theta.k
    n.k <- matrix( 0, length(theta.k), G )
    r.jk <- n.jk <- array( 0, dim=c( ncol(dat2), length(theta.k), G) )
    ll <- rep(0,G)
    for (gg in 1:G){
        ind.gg <- group==gg
        n.k[,gg] <- colSums( dat1[ind.gg,2] * f.qk.yi[ind.gg,,drop=FALSE]  )
        # expected counts at theta.k and item j
        f.qk.yi.gg <- f.qk.yi[ ind.gg, ]
        dat1.gg <- matrix( dat1[ind.gg,2], nrow=I, ncol=sum(ind.gg), byrow=TRUE )
        M1 <- ( t(dat2.resp[ind.gg,]) * dat1.gg )
        n.jk[,,gg] <-  M1 %*% f.qk.yi.gg
        # compute r.jk (expected counts for correct item responses at theta.k for item j
        r.jk[,,gg] <- ( t(dat2[ind.gg,]) * M1 ) %*% f.qk.yi.gg
        # compute log-Likelihood
        ngg <- length(ind.gg)
        pikM <- sirt_matrix2(x=pi.k[,gg], nrow=ngg)
        ll[gg] <- sum( dat1[ind.gg,2] * log( rowSums( f.yi.qk[ind.gg,]*pikM)))
    }
    res <- list( n.k=n.k, n.jk=n.jk, r.jk=r.jk, f.qk.yi=f.qk.yi, pjk=pjk,
                f.yi.qk=f.yi.qk, ll=sum(ll) )
    return(res)
}


.e.step.mirtlc.lc <- rasch_mirtlc_estep_lc

## File Name: sirt_summary_cat_label_equal_value.R
## File Version: 0.05



sirt_summary_cat_label_equal_value <- function( label, value, label2="", digits=NULL )
{
    res <- sirt_summary_label_equal_value(label=label, value=value,
                label2=label2, digits=digits)
    cat(res)
}

## File Name: sirt_summary_print_packages.R
## File Version: 0.04

sirt_summary_print_packages <- function(packages)
{
    for (pack in packages){
        sirt_summary_print_package(pack=pack)
    }
}

## File Name: mcmc.3pno.testlet_alg.R
## File Version: 2.24

##############################################
# draw latent response data W
.draw.W.3pno.testlet <- function( aM, bM, theta, gamma.testlet,
                N, I, threshlow, threshupp, testletgroups, param,
                dat, guess, tau.ni ){
    # P(Y=1)=c_i + ( 1 - c_i ) Phi ( tau )
    #        =Phi(tau) + c_i ( 1 - Phi(tau) )
    # calculate probabilities P(W=1|Y=1)
    p1 <- stats::pnorm( tau.ni )
    # calculate P(W=0|Y=1)
    p0 <- matrix( guess, N, I, byrow=TRUE ) * ( 1 - p1 )
    p1 <- p1 / ( p1 + p0 )
    # simulate random draw
    rij <- matrix( stats::runif( N*I ), nrow=N, ncol=I )
    # calculate corresponding value
    W <- 1 * ( p1 > rij  ) * dat
    return(W)
        }

###############################################
# draw latent responses Z
.draw.Z.3pno.testlet <- function( aM, bM, theta, gamma.testlet,
                N, I, threshlow, threshupp, testletgroups,
                param, W, dat.resp, tau.ni){
    # calculate indicator testletgroups
    gamma.testletM <- gamma.testlet[, testletgroups ]
    # calculate means
#    mij <- aM * theta + gamma.testletM + bM
    mij <- tau.ni
    # simulate uniform data
    rij <- matrix( stats::runif( N*I ), nrow=N, ncol=I )
    # calculate thresholds if they should calculated
        # define lower and upper thresholds
        ZZ <- 1000
        threshlow <- -ZZ + ZZ*W
        threshlow <- threshlow - (1-dat.resp)*ZZ
        threshupp <- ZZ*W
        threshupp <- threshupp + (1-dat.resp)*ZZ
    # calculate corresponding value
    pl <- stats::pnorm( threshlow, mean=mij)
    pu <- stats::pnorm( threshupp, mean=mij)
    pij <- pl + (pu-pl)*rij
    # simulate Z
    Zij <- stats::qnorm( pij, mean=mij )
    return(Zij)
        }
##################################################

#############################################
# draw theta from 3PNO testlet model
.draw.theta.3pno.testlet <- function( aM, bM, N, I, Z, param,
        gamma.testletM, sigma, a.testletM ){
    if (param==1){ V <- Z - gamma.testletM }
    if (param==2){ V <- Z - aM*gamma.testletM }
    if (param==3){ V <- Z - a.testletM*gamma.testletM }
    vtheta <- 1 / ( rowSums( aM^2 ) + 1 / sigma^2  )
    mtheta <- rowSums( aM * V ) * vtheta
    theta <- stats::rnorm( N, mean=mtheta, sd=sqrt( vtheta ) )
    theta <- theta - mean( theta )
    if (param==3){ theta <- theta / stats::sd( theta ) }
    return(theta)
            }
#############################################

#############################################
# draw gamma from 3PNO testlet model
.draw.gamma.3pno.testlet <- function( aM, bM, N, I, Z, param,
        theta, testletgroups, TT, N.items, gamma.testlet,
        sigma.testlet, a.testletM){
    Zast <- Z - aM * theta - bM
    for (tt in 1:TT){
#        tt <- 1
        ind.tt <- which( testletgroups==tt)
        #*** param=1
        if (param==1){
            Zast.tt <- Zast[, ind.tt ]
            nu.tt <- 1 / N.items[tt]
            vtheta <- 1 / ( 1 / nu.tt + 1 / sigma.testlet[tt]^2 )
            mtheta <- rowMeans( Zast.tt ) * 1 / nu.tt * vtheta
                    }
        #*** param=2
        if (param==2){
            Zast.tt <- Zast[, ind.tt ]
            nu.tt <- rowSums( aM[,ind.tt]^2 )
            vtheta <- 1 / ( 1 / nu.tt + 1 / sigma.testlet[tt]^2 )
            mtheta <- rowSums( aM[,ind.tt] * Zast.tt ) * 1 / nu.tt * vtheta
                    }
        #*** param=3
        if (param==3){
            Zast.tt <- Zast[, ind.tt ]
            nu.tt <- rowSums( a.testletM[,ind.tt]^2 )
            vtheta <- 1 / ( 1 / nu.tt + 1 / sigma.testlet[tt]^2 )
            mtheta <- rowSums( a.testletM[,ind.tt] * Zast.tt ) * 1 / nu.tt * vtheta
                    }
        # sample gamma parameter
        gamma.testlet[,tt] <- stats::rnorm(N, mean=mtheta, sd=sqrt(vtheta) )
        gamma.testlet[,tt] <- gamma.testlet[,tt] - mean( gamma.testlet[,tt] )

#        gamma.testlet[, 1:TT] <- gamma.testlet[, 1:TT ] - rowMeans( gamma.testlet[,1:TT] )
        if (param==3){
             gamma.testlet[,tt] <- gamma.testlet[,tt] / stats::sd( gamma.testlet[,tt] )
                    }
            }
    return(gamma.testlet)
            }
#############################################



#############################################
# draw b from 3PNO testlet model if slopes are fixed at one
.draw.est.b.3pno.testlet <- function( aM, bM, N, I, Z, param,
        gamma.testletM, sigma, weights, theta ){
    #************************
    V <- Z - aM * theta - gamma.testletM
    if ( is.null(weights) ){
        mb <- colMeans( V )
        vb <- 1 / N
            }
    if ( ! is.null(weights) ){
        mb <- colSums( V * weights ) / N
        vb <- sum( weights^2 ) / N^2
            }
    b <- stats::rnorm( I, mean=mb, sd=sqrt( vb ) )
    return(b)
            }
#############################################


#############################################
# draw a.testlet from 3PNO testlet model
.draw.est.a.testlet.3pno.testlet <- function( aM, bM, N, I, Z, param,
        gamma.testletM, sigma, weights, theta, testletgroups,
        gamma.testlet, TT    ){
    #************************
    # Z=a * theta + a.testlet *gamma + bM + eps
    # Z - a*theta - bM=a.testlet * gamma.testletM + esp

    V <- Z - aM * theta - bM
    a.testlet <- rep(0,I)
    for (tt in 1:TT){
    # tt <- 1
        ind.tt <- which( testletgroups==tt )
        Itt <- length(ind.tt)
        Vtt <- V[,ind.tt]
        v.ik <- 1 / ( colSums( Vtt^2 ) )
        if ( is.null(weights ) ){
            m.ik <- colSums(  gamma.testlet[,tt] * Vtt  )
                        }
        if ( ! is.null(weights ) ){
            m.ik <- colSums( weights*gamma.testlet[,tt] * Vtt  )
                        }
        m.ik <- m.ik * v.ik
        a.testlet[ind.tt] <- stats::rnorm( Itt, mean=m.ik, sd=sqrt(v.ik ) )
                }
# cat(".....\n")
    return(a.testlet)
            }
#############################################


##########################################
# draw guessing parameter
.draw.guess.3pno.testlet <- function( guess.prior, W,
        dat, dat.resp, weights, I ){
    if ( is.null(weights) ){
        ri <- colSums( (W==0) )
        si <- colSums( (W==0) * (dat==1) * dat.resp)
                }
    if ( ! is.null(weights) ){
        ri <- colSums( (W==0) * weights )
        si <- colSums( (W==0) * (dat==1) * dat.resp * weights)
                }
    guess <- stats::rbeta(I, si + guess.prior[,1], ri-si+guess.prior[,2] )
    return( guess )
            }
#################################################
# draw theta variance
.draw.theta.variance.3pno.testlet <- function( theta, weights, N ){
        if ( is.null(weights) ){  sig2 <- stats::var(theta) }
        if ( ! is.null(weights) ){
            sig2 <- ( sum( theta^2 * weights ) - ( sum( theta*weights )/N )^2 )/ N
                        }
        sigma <- sqrt( .mcmc.draw.variance( 1, w0=1, sig02=1, n=N, sig2=sig2 ) )
        return(sigma)
            }
############################################################
# draw testlet variance
.draw.testlet.variance.3pno.testlet <- function( gamma.testlet, N,
    sigma.testlet, testlet.variance.prior, weights, TT){
    for (tt in 1:TT){
        if ( is.null(weights)){ sig2 <- sum( gamma.testlet[,tt]^2 ) / N }
        if ( ! is.null(weights)){ sig2 <- sum( weights*gamma.testlet[,tt]^2 ) / N }
        sigma.testlet[tt] <- sqrt( .mcmc.draw.variance( 1, w0=testlet.variance.prior[1],
                sig02=testlet.variance.prior[2], n=N, sig2=sig2 ) )
                    }
    return(sigma.testlet)
            }
######################################################

## File Name: lsem_residualize.R
## File Version: 0.478


#**** residualize data
lsem_residualize <- function( data, moderator, moderator.grid,
        lavmodel, h=1.1, bw=NULL, residualize=TRUE, sampling_weights=NULL,
        eps=1E-10, verbose=TRUE, kernel="gaussian", variables_model,
        is_imputed=FALSE, Nimp=0 )
{
    # lavaanify model
    lavaanstr <- sirt_import_lavaan_lavaanify(model=lavmodel)
    vars <- variables_model

    # values of moderator variable
    if (! is_imputed){
        data.mod <- data[, moderator, drop=TRUE ]
        Nimp <- 1
    } else {  # use first dataset for imputed data
        data.mod <- ( data[[1]] )[, moderator, drop=TRUE ]
    }

    # compute local weights
    res <- lsem_local_weights(data.mod=data.mod, moderator.grid=moderator.grid,
                        h=h, sampling_weights=sampling_weights, bw=bw, kernel=kernel,
                        is_imputed=is_imputed, Nimp=Nimp, data=data,
                        moderator=moderator)
    weights <- res$weights
    modgrid_index <- res$modgrid_index
    N <- res$N
    G <- res$G
    m.moderator <- res$m.moderator
    sd.moderator <- res$sd.moderator
    bw <- res$bw
    h <- res$h
    moderator.density <- res$moderator.density
    sampling_weights <- res$sampling_weights
    no_sampling_weights <- res$no_sampling_weights

    res0 <- as.list(1L:Nimp)
    data0 <- data

    # residualize
    for (ii in 1L:Nimp){
        dat2 <- data
        if (is_imputed){
            dat2 <- data <- data0[[ii]]
        }

        V <- length(vars)
        residualized_intercepts <- matrix( 0, nrow=G, ncol=V)
        colnames(residualized_intercepts) <- vars
        rownames(residualized_intercepts) <- round( moderator.grid, 3 )
        if (residualize){
            if (verbose & ii==1){
                cat('** Residualize Data\n')
                utils::flush.console()
            }
            N <- nrow(data)
            for (vv in 1L:V){
                var.vv <- vars[vv]
                ind_vv <- which( ! is.na( data[,var.vv] ) )
                y0 <- rep(NA,N)
                for (gg in 1L:G){
                    x <- dat2[,moderator]
                    data1 <- data
                    data1$x <- x
                    res_formula <- paste0( var.vv, ' ~ x + I(x^2)' )
                    if (!is_imputed){
                        weights_gg <- weights[,gg]
                    } else {
                        weights_gg <- (weights[[ii]])[,gg, drop=TRUE]
                    }
                    data1$weights_gg <- weights_gg
                    mod <- stats::lm( formula=res_formula, data=data1,
                                            weights=weights_gg )
                    dfr_pred <- data.frame( x=moderator.grid[gg] )
                    m1 <- stats::predict( mod, dfr_pred )
                    residualized_intercepts[gg,vv] <- m1
                    y <- stats::resid(mod)
                    y0[ ind_vv ] <- y
                    if (!is_imputed){
                        modgrid_index1 <- modgrid_index
                    } else {
                        modgrid_index1 <- modgrid_index[[ii]]
                    }

                    dat2[, var.vv] <- ifelse( modgrid_index1==gg, y0, dat2[, var.vv] )
                }
            }  # end vv
        }  # end residualize=TRUE
        res <- list( resid_vars=vars, data=dat2, weights_grid=weights, bw=bw, h=h,
                moderator.density=moderator.density, sd.moderator=sd.moderator, G=G, N=N,
                residualized_intercepts=residualized_intercepts,
                sampling_weights=sampling_weights,
                no_sampling_weights=no_sampling_weights,
                m.moderator=m.moderator, residualize=residualize,
                is_imputed=is_imputed, Nimp=Nimp)
        res0[[ii]] <- res

    }  # end imputation loop ii

    #--- process output
    if (! is_imputed){
        res <- res0[[1]]
    } else {
        res <- res0[[1]]
        entries <- c('data','residualized_intercepts')
        for (ee in entries){
            v1 <- list()
            for (ii in 1L:Nimp){
                v1[[ii]] <- res0[[ii]][[ee]]
            }
            res[[ee]] <- v1
        }

        #- aggregate residualized intercepts
        ee <- 'residualized_intercepts'
        res[[ee]] <- lsem_aggregate_statistics(x=res[[ee]])

    }  # end process output

    #-- out
    return(res)
}

lsem.residualize <- lsem_residualize

## File Name: fit.gradedresponse_alg.R
## File Version: 1.175


#####################################################################
# calculate log likelihood function in graded response model
.calc.ll.grm <- function( theta, b, b.cat, freq.categories)
{
    eps <- 10^(-10)
    TP <- length(theta)
    I <- length(b)
    K <- length(b.cat)
    prob1 <- prob <- array( 1, dim=c(TP,I,K+1) )
    for (kk in 1L:K){
        prob1[,,kk+1] <- stats::plogis( theta +
                            matrix( b, nrow=TP, ncol=I, byrow=TRUE) + b.cat[kk] )
        prob[,,kk] <- prob1[,,kk]-prob1[,,kk+1]
    }
    kk <- K+1
    prob[,,kk] <- prob1[,,kk]
    prob[ prob < eps ] <- eps
    # calculate log-likelihood
    ll <- freq.categories * log( prob )
    res <- list(ll=ll, prob=prob )
    return(res)
}

#####################################################################
# update theta parameter in logistic distribution
.update.theta.grm <- function( theta, b, b.cat, freq.categories,
        numdiff.parm, max.increment)
{
    h <- numdiff.parm
    # update theta parameter
    ll0 <- .calc.ll.grm( theta, b, b.cat, freq.categories)
    prob.grm <- ll0$prob
    ll0 <- ll0$ll
    ll1 <- .calc.ll.grm( theta+h, b, b.cat, freq.categories)$ll
    ll2 <- .calc.ll.grm( theta-h, b, b.cat, freq.categories)$ll
    ll0 <- rowSums(ll0)
    ll1 <- rowSums(ll1)
    ll2 <- rowSums(ll2)
    # derivative
    d1 <- ( ll1 - ll2  ) / ( 2 * h )    # negative sign?
    # second order derivative
    # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
    d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
    # change in item difficulty
    d2[ abs(d2) < 10^(-10) ] <- 10^(-10)
    increment <- - d1 / d2
    increment <- ifelse( abs( increment) > abs(max.increment),
                                     sign(increment)*max.increment, increment )
    theta <- theta + increment
    res <- list(theta=theta, ll=sum(ll0), prob.grm=prob.grm )
    return(res)
}

#####################################################################
# update b parameter in logistic distribution
.update.b.grm <- function( theta, b, b.cat, freq.categories,
        numdiff.parm, max.increment)
{
    h <- numdiff.parm
    # update theta parameter
    ll0 <- .calc.ll.grm( theta, b, b.cat, freq.categories)
    ll0 <- ll0$ll
    ll1 <- .calc.ll.grm( theta, b+h, b.cat, freq.categories)$ll
    ll2 <- .calc.ll.grm( theta, b-h, b.cat, freq.categories)$ll
    ll0 <- rowSums(colSums(ll0))
    ll1 <- rowSums(colSums(ll1))
    ll2 <- rowSums(colSums(ll2))
    # derivative
    d1 <- ( ll1 - ll2  ) / ( 2 * h )    # negative sign?
    # second order derivative
    # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
    d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
    # change in item difficulty
    d2[ abs(d2) < 10^(-10) ] <- 10^(-10)
    increment <- - d1 / d2
    increment <- ifelse( abs(increment) > abs(max.increment),
                                     sign(increment)*max.increment, increment )
    b <- b + increment
    #-- output
    res <- list(b=b, ll=sum(ll0) )
    return(res)
    }
##############################################################

#####################################################################
# update b parameter in logistic distribution
.update.bcat.grm <- function( theta, b, b.cat, freq.categories,
        numdiff.parm, max.increment)
{
    h <- numdiff.parm
    b.catN <- 0*b.cat
    for (kk in seq(1,length(b.cat))){
        e1 <- b.catN
        e1[kk] <- 1
        # update theta parameter
        ll0 <- .calc.ll.grm( theta, b, b.cat, freq.categories)
        ll0 <- ll0$ll
        ll1 <- .calc.ll.grm( theta, b, b.cat+h*e1, freq.categories)$ll
        ll2 <- .calc.ll.grm( theta, b, b.cat-h*e1, freq.categories)$ll
        ll0 <- sum(ll0)
        ll1 <- sum(ll1)
        ll2 <- sum(ll2)
        # derivative
        d1 <- ( ll1 - ll2  ) / ( 2 * h )    # negative sign?
        # second order derivative
        # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
        d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
        # change in item difficulty
        d2[ abs(d2) < 10^(-10) ] <- 10^(-10)
        increment <- - d1 / d2
        increment <- ifelse( abs( increment) > abs(max.increment),
                                         sign(increment)*max.increment, increment )
        b.cat[kk] <- b.cat[kk] + increment
    }
    res <- list(b.cat=b.cat, ll=sum(ll0) )
    return(res)
}
##############################################################

## File Name: xxirt_modifyParTable.R
## File Version: 0.201

#**** modify parameter table
xxirt_modifyParTable <- function( partable, parname, item=NULL, value=NULL,
        est=NULL, parlabel=NULL, parindex=NULL, lower=NULL,
        upper=NULL, prior=NULL, prior_par1=NULL, prior_par2=NULL )
{
    if ( is.null(item) ){
        item <- paste( unique(partable$item ))
    }
    ind <- which( (partable$parname %in% parname) & (partable$item %in% item ) )
    vv <- value ; vv_name <- 'value'
        if ( ! is.null(vv) ){ partable[ ind, vv_name ] <- vv }
    vv <- est ; vv_name <- 'est'
        if ( ! is.null(vv) ){ partable[ ind, vv_name ] <- vv }
    vv <- lower ; vv_name <- 'lower'
        if ( ! is.null(vv) ){ partable[ ind, vv_name ] <- vv }
    vv <- upper ; vv_name <- 'upper'
        if ( ! is.null(vv) ){ partable[ ind, vv_name ] <- vv }
    vv <- prior ; vv_name <- 'prior'
        if ( ! is.null(vv) ){ partable[ ind, vv_name ] <- vv }
    vv <- prior_par1 ; vv_name <- 'prior_par1'
        if ( ! is.null(vv) ){ partable[ ind, vv_name ] <- vv }
    vv <- prior_par2 ; vv_name <- 'prior_par2'
        if ( ! is.null(vv) ){ partable[ ind, vv_name ] <- vv }
    vv <- parlabel ; vv_name <- 'parlabel'
        if ( ! is.null(vv) ){ partable[ ind, vv_name ] <- vv }
    vv <- parindex ; vv_name <- 'parindex'
        if ( ! is.null(vv) ){ partable[ ind, vv_name ] <- vv }
    #*** output
    return(partable)
}

## File Name: parmsummary_extend.R
## File Version: 0.131


#--- extend parameter summary
parmsummary_extend <- function( dfr, level=.95, est_label="est", se_label="se",
            df_label="df" )
{
    dfr <- as.data.frame(dfr)
    #-- compute t values and p values
    dfr$t <- dfr[,est_label] / dfr[,se_label]
    dfr$p <- 2 * stats::pnorm( - abs(dfr$t) )
    #-- compute confidence intervals
    if ( ! is.null(level) ){
        if ( 'df' %in% colnames(dfr) ){
            quant <- - stats::qt( (1-level)/2, df=dfr$df)
        } else {
            quant <- - stats::qnorm( (1-level)/2 )
        }
        dfr[, paste0('lower', 100*level ) ] <-
            dfr[,est_label] - quant * dfr[, se_label ]
        dfr[, paste0('upper', 100*level ) ] <-
            dfr[,est_label] + quant * dfr[, se_label ]
    }
    #-- output
    return(dfr)
}

## File Name: lc.2raters.aux.R
## File Version: 0.10

#########################################
# data preparation exchanegable raters
lc2.data.prep <- function(data){
    maxK <- max( data )
    m1 <- matrix( NA, maxK+1, maxK+1 )
    rownames(m1) <- colnames(m1) <- paste0("Cat", 0:maxK)
    for (kk1 in 0:maxK){
    for (kk2 in 0:maxK){
        m1[kk1+1,kk2+1] <- sum( ( data[,1]==kk1 ) * ( data[,2]==kk2 ) )
                        }
                }
    m1 <- m1+t(m1)
    m1 <- m1/2
    res <- list("m1"=m1, "maxK"=maxK)
    return(res)
    }
###############################################

###################################################
# agreement measures: input is a frequency table
lc2.agreement <- function( m1 ){
    # agreement and kappa measures
    #***
    # agreement
    res <- list( "agree0"=sum( diag( m1 ) ) / sum( m1 ) )
    K <- ncol(m1)
    qk <- outer( 1:K, 1:K, "-")
    res$agree1 <- sum( m1 * ( abs(qk) <=1 ) ) / sum(m1)
    p1 <- m1 / sum(m1)
    #***
    # kappa
    pk <- colSums( m1 )
    pk <- pk/sum(pk)
    pek <- sum( pk^2 )
    res$kappa <- ( sum( diag(p1) ) - pek ) / ( 1 - pek )

    #***
    # weighted kappa

    # define weights
    wgt <- 1 - abs( qk ) / ( K - 1 )
    pa <- sum( wgt * p1 )
    pek <- sum( outer( pk, pk )*wgt )
    res$wtd.kappa.linear <- ( pa - pek) / ( 1 - pek )

    wgt <- 1 - qk^2 / ( K - 1 )^2
    pa <- sum( wgt * p1 )
    pek <- sum( outer( pk, pk )*wgt )
    res$wtd.kappa.quadratic <- ( pa - pek) / ( 1 - pek )

    #***
    # Gwet's AC(1) statistic
    pek <- sum( pk*(1-pk) ) / ( K - 1 )
    pa <- sum( diag(p1))
    res$AC1 <- ( pa - pek) / ( 1 - pek )
    res$alpha.aickin <- alpha.aickin(m1)
    res <- unlist(res)
    }
################################################################

######################################################
# Aickin's alpha
# Aickin (1990) Biometrics
alpha.aickin <- function(m1){
    # Aickin's alpha statistic
    p1 <- m1 / sum(m1)
    pk <- colSums(p1)
    pek <- sum( pk^2 )
    pa <- sum( diag(p1) )
    ckappa <- ( pa - pek ) / ( 1 - pek )
    Q <- ncol(m1)
    alpha <- ckappa
    pkH <- pk
    pet <- sum( pkH^2 )
    conv <- .001
    iter <- 0
    maxit <- 100
    par.change <- 1
    while( ( par.change > conv ) & ( iter < maxit ) ){
        alpha0 <- alpha
        pkH0 <- pkH
        # update hard-to-classify probabilities
        pkH <- pk / ( ( 1 - alpha ) + alpha * pkH / pet )
        # update alpha
        pet <- sum( pkH^2 )
        alpha <- ( pa - pet ) / ( 1 - pet )
        par.change <- max( abs( c( alpha - alpha0, pkH - pkH0 ) ) )
        iter <- iter + 1
#        print( par.change )
                }
    return(alpha)
        }
############################################################

## File Name: mirt.wrapper.posterior.R
## File Version: 0.370


# calculation of posterior
mirt.wrapper.posterior <- function( mirt.obj, weights=NULL, group=NULL )
{
    TAM::require_namespace_msg("mirt")
    mirt_class <- class(mirt.obj)
    is_mg <- mirt_class=="MultipleGroupClass"
    data_group <- mirt.obj@Data$group
    orig_data <- mirt.obj@Data$data
    item_names <- colnames(orig_data)
    N_orig <- nrow(orig_data)
    G <- length(unique(data_group))
    N <- length(data_group)
    ind_group <- 1:N
    id_group <- 1
    groupNames <- NULL
    if (!is.null(group)){
        groupNames <- mirt.obj@Data$groupNames
        ind_group <- which(paste(data_group)==group)
        id_group <- which(groupNames==group)
    }
    mobj <- mirt.obj
    # extract theta
    Theta <- mobj@Model$Theta
    # extract theta distribution
    pi.k <- mobj@Internals$Prior[[id_group]]

    # extract full data
    fulldata <- mobj@Data$fulldata[[id_group]]
    #load items
    I <- ncol( mobj@Data$data )
    items <- vector('list', I)
    for(ii in 1:I){
        items[[ii]] <- mirt::extract.item(x=mobj, item=ii, group=id_group)
    }
    # check whether prodlist exists
    prodlist <- mobj@Model$prodlist
    Theta1 <- Theta
    if ( length(prodlist) > 0 ){
        Theta1 <- mirt_prodterms(theta0=Theta, prodlist=prodlist)
    }
    # item-wise probabilities for each Theta
    traces <- Probtrace_sirt(items=items, Theta=Theta1)
    # log-Likelihood
    f.yi.qk <- exp( fulldata %*% t(log(traces)) )
    # compute individual posterior
    N <- nrow(fulldata)
    TP <- length(pi.k)
    piM <- matrix( pi.k, nrow=N, ncol=TP, byrow=TRUE )
    f.qk.yi <- f.yi.qk * piM
    f.qk.yi <- f.qk.yi / matrix( rowSums(f.qk.yi), nrow=N, ncol=TP, byrow=FALSE )
    # maximum category
    maxK <- apply( mobj@Data$data, 2, max, na.rm=TRUE)+1
    resp <- mobj@Data$data
    resp <- resp[ ind_group, ]
    resp.ind <- 1 - is.na(resp)
    resp[ resp.ind==0 ] <- 0
    # calculate counts
    if (is.null(weights) ){
        pweights <- rep(1,N)
    } else {
        pweights <- weights
    }
    # Theta is only used for calculating dimension size
    n.ik <- mirt.wrapper.calc.counts( resp=resp, theta=Theta, resp.ind=resp.ind,
                group=group, maxK=max(maxK), pweights=pweights, hwt=f.qk.yi )
    dimnames(n.ik)[[2]] <- item_names
    exp_counts <- aperm(n.ik, perm=c(2,3,1,4))
    probs <- traces
    probs <- array( probs, dim=c(TP,max(maxK),I) )
    probs <- aperm( probs, perm=c(3,2,1) )
    dimnames(probs)[[1]] <- item_names
    # result list
    res <- list( theta.k=Theta, pi.k=pi.k, f.yi.qk=f.yi.qk, f.qk.yi=f.qk.yi,
            n.ik=n.ik, exp_counts=exp_counts, probs=probs, N=N, TP=TP, I=I, data=resp,
            maxK=maxK, G=G, groupNames=groupNames, group=group,
            orig_data=orig_data, ind_group=ind_group, N_orig=N_orig,
            mirt_class=mirt_class, pweights=pweights)
    class(res) <- "mirt"
    return(res)
}

## File Name: mcmc_rename_define_symbols.R
## File Version: 0.061

mcmc_rename_define_symbols <- function()
{
    trans <- c('X', 'Z', 'M')
    orig <- c('[', ']', ',')
    res <- list(trans=trans, orig=orig)
    return(res)
}

## File Name: zzz.R
## File Version: 1.242


#  zzz.R
#
# This function is simply copied from the mice package.


# on attach sirt
.onAttach <- function(libname,pkgname)
{
    d <- utils::packageDescription('sirt')
    d1 <- d$Version
    packageStartupMessage(
        paste('- ', d$Package,' ', d1,' (',d$Date,')',sep='')  )
}


version <- function(pkg='sirt')
{
    lib <- dirname(system.file(package=pkg))
    d <- utils::packageDescription(pkg)
    return( paste(d$Package,d$Version,d$Date,lib))
}


xx <- function(f1=1, f2=1)
{
    v1 <- paste0( rep(' ',f1), collapse='' )
    v2 <- paste0( rep(' ',f2), collapse='' )
    res <- paste0( v1, '=', v2)
    return(res)
}

## File Name: isop.test.R
## File Version: 0.20

############################################################
# testing the ISOP model
isop.test <- function( data, jackunits=20, weights=rep(1,nrow(data)) )
{
    dat <- as.matrix( data )
    dat.resp <- 1 - is.na( dat )
    dat[ is.na(dat) ] <- 0
    weights <- rep( 1, nrow(dat) )
    N <- nrow(dat)
    if ( length(jackunits==1)){
        jackunits <- ( 1:N ) %% jackunits
    }
    JJ <- max( jackunits )+1

    # apply Rcpp routine
    res <- isop_tests_cpp( dat=dat, dat.resp=dat.resp, weights=weights,
                jackunits=jackunits, JJ=JJ )

    #**********************
    # arrange output
    I <- ncol(dat)
    # Jackknife estimates
    itemstat <- as.data.frame( matrix( NA, nrow=I+1, ncol=3 ) )
    colnames(itemstat) <- c("parm", "est", "se" )
    W1test <- res$W1test
    itemstat[1,1] <- "test"
    itemstat[1,2] <- W1test[1]
    itemstat[1,3] <- sqrt( sum( ( W1test[-1] - mean( W1test[-1] ) )^2 ) * (JJ-1) / JJ )
    W1i <- res$W1i
    itemstat[-1,1] <- colnames(dat)
    itemstat[-1,2] <- W1i[,1]
    itemstat[-1,3] <- sqrt( rowSums( ( W1i[,-1] - rowMeans( W1i[,-1] ) )^2 ) * (JJ-1) / JJ )
    # include N's and p values
    itemstat <- data.frame("parm"=itemstat$parm, "N"=c(N, colSums( dat.resp * weights ) ),
            "M"=c(NA, colSums( dat.resp * weights * dat ) / colSums( dat.resp * weights ) ),
            itemstat[,-1] )
    itemstat$t <- itemstat$est / itemstat$se
    rownames(itemstat) <- NULL
    res <- list("itemstat"=itemstat, "Es"=res$Esi[,1], "Ed"=res$Edi[,1], JJ=JJ)
    class(res) <- "isop.test"
    return(res)
}

## File Name: plot.isop.R
## File Version: 1.16

#############################################################
# plot ISOP item response functions
plot.isop <- function( x, ask=TRUE, ... ){
    if ( x$model=="isop.dich"){
        plotisopdich( x, ask=ask, ... )
                        }
    if ( x$model=="isop.poly"){
        plotisoppoly( x, ask=ask, ... )
                        }
                }
#############################################################

##################################################
# plot isop class for polytomous data
plotisoppoly <- function(x, ask=TRUE, ... ){
        mod3 <- x
        mod <- x
        pers <- mod3$person
        pers <- pers[ order( pers$mpsc ), ]
        I <- ncol( mod3$dat )
        K <- ncol(x$p.itemcat)
        for (kk in 2:K){
        #kk <- 2
        #********************
        # plot all item response functions
        graphics::par( mfrow=c(2,2))
            # Labels for plots
            kv <- substitute( expression( a ), list(a=kk) )
            xlabplot <- expression( paste( "Modified P Score ", rho[p] ))
#            ylabplot <- expression(paste(P,"(",X[i], ">=", kv, "|", rho[p], ")" ))
#            ylabplot <- substitute( paste(P,"(",X[i], ">=", kv, "|", rho[p], ")" ),
#                            list(kv=kk-1) )
            ylabplot <- substitute( paste(P,"(",X[i] >=kv, "|", rho[p], ")" ),
                            list(kv=kk-1) )
            # saturated IRT model
            p1 <- mod$surv.saturated
            mainplot1 <- paste0( "Survivor Functions Category ", kk - 1 )
            mainplot <- paste0( mainplot1,  "\nSaturated Model")
            plot.all.icc.isop.poly( pers, p1, xlabplot, ylabplot, mainplot,I,kk)
            # isop
            p1 <- mod$surv.isop
            mainplot <- paste0( mainplot1,  "\nISOP Model")
            plot.all.icc.isop.poly( pers, p1, xlabplot, ylabplot, mainplot,I,kk)
            # adisop
            p1 <- mod$surv.adisop
            mainplot <- paste0( mainplot1,  "\nADISOP Model")
            plot.all.icc.isop.poly( pers, p1, xlabplot, ylabplot, mainplot,I,kk)
            # logistic function
            p1 <- mod$surv.grm
            mainplot <- paste0( mainplot1,  "\nGraded Response Model")
            plot.all.icc.isop.poly( pers, p1, xlabplot, ylabplot, mainplot,I,kk)
        graphics::par( mfrow=c(1,1))
        graphics::par(ask=ask)
                }
    }
#########################################################################

##################################################
# plot isop class for dichotomous data
plotisopdich <- function(x, ask=TRUE, ... ){
        mod3 <- x
        mod <- x
        pers <- mod3$person
        pers <- pers[ order( pers$mpsc ), ]
        I <- ncol( mod3$dat )
        #********************
        # plot all item response functions
        graphics::par( mfrow=c(2,2))
            # Labels for plots
            xlabplot <- expression( paste( "Modified P Score ", rho[p] ))
            ylabplot <- expression(paste(P,"(",X[i]==1, "|", rho[p], ")" ))
            # saturated IRT model
            p1 <- mod$prob.saturated
            mainplot <- paste0( "Item Response Functions \nSaturated Model")
            plot.all.icc.isop.dich( pers, p1, xlabplot, ylabplot, mainplot,I)
            # isop
            p1 <- mod$prob.isop
            mainplot <- paste0( "Item Response Functions \nISOP Model")
            plot.all.icc.isop.dich( pers, p1, xlabplot, ylabplot, mainplot,I)
            # adisop
            p1 <- mod$prob.adisop
            mainplot <- paste0( "Item Response Functions \nADISOP Model")
            plot.all.icc.isop.dich( pers, p1, xlabplot, ylabplot, mainplot,I)
            # logistic function
            p1 <- mod$prob.logistic
            mainplot <- paste0( "Item Response Functions \nLogistic Model (Rasch Model)")
            plot.all.icc.isop.dich( pers, p1, xlabplot, ylabplot, mainplot,I)
        graphics::par( mfrow=c(1,1))
        graphics::par(ask=ask)
        #*********************
        # separate item response functions
        for (ii in 1:I){
        # ii <- 1
        graphics::par( mfrow=c(2,2))
            xlabplot <- expression( paste( "Modified P Score ", rho[p] ))
            ylabplot <- expression(paste(P,"(",X[i]==1, "|", rho[p], ")" ))
            mainplot <- paste0("Item ", colnames(mod3$dat)[ii], " | Saturated Model" )
            graphics::plot( pers$mpsc, mod3$prob.saturated[ii,2, pers$scoregroup ], type="l",
                         xlab=xlabplot, ylab=ylabplot, main=mainplot, ylim=c(0,1)  )
            mainplot <- paste0("Item ", colnames(mod3$dat)[ii], " | ISOP Model" )
            graphics::plot( pers$mpsc, mod3$prob.isop[ii,2, pers$scoregroup ], type="l",
                         xlab=xlabplot, ylab=ylabplot, main=mainplot, ylim=c(0,1)  )
            mainplot <- paste0("Item ", colnames(mod3$dat)[ii], " | ADISOP Model" )
            graphics::plot( pers$mpsc, mod3$prob.adisop[ii,2, pers$scoregroup ], type="l",
                         xlab=xlabplot, ylab=ylabplot, main=mainplot, ylim=c(0,1)  )
            mainplot <- paste0("Item ", colnames(mod3$dat)[ii], " | Logistic Model" )
            graphics::plot( pers$mpsc, mod3$prob.logistic[ii,2, pers$scoregroup ], type="l",
                         xlab=xlabplot, ylab=ylabplot, main=mainplot, ylim=c(0,1)  )
           graphics::par( mfrow=c(1,1))
           graphics::par(ask=ask)
            }
    }
#########################################################################

#########################################################################
# aux plot function all item response functions
plot.all.icc.isop.dich <- function( pers, p1,
    xlabplot, ylabplot, mainplot, I ){
    # begin plot
    graphics::plot( pers$mpsc, p1[1,2, pers$scoregroup ], type="l", ylim=c(0,1),
         xlab=xlabplot, ylab=ylabplot, main=mainplot)
    for (ii in 2:I){
        graphics::lines( pers$mpsc, p1[ii,2, pers$scoregroup ], lty=ii, col=ii )
                    }
                }
#########################################################################
# aux plot function all item response functions
plot.all.icc.isop.poly <- function( pers, p1,
    xlabplot, ylabplot, mainplot, I, kk){
    # begin plot
    graphics::plot( pers$mpsc, p1[1,kk, pers$scoregroup ], type="l", ylim=c(0,1),
         xlab=xlabplot, ylab=ylabplot, main=mainplot)
    for (ii in 2:I){
        graphics::lines( pers$mpsc, p1[ii,kk, pers$scoregroup ], lty=ii, col=ii )
                    }
                }

## File Name: monoreg.rowwise.R
## File Version: 0.11


# monotone regression for all rows in a matrix
monoreg.rowwise <- function(yM, wM)
{
    yM <- as.matrix(yM)
    wM <- as.matrix(wM)
    res <- sirt_rcpp_monoreg_rowwise( YM=yM, WM=wM )
    return(res)
}

## File Name: sirt_define_eps_sequence.R
## File Version: 0.01

sirt_define_eps_sequence <- function(eps, eps_vec)
{
    eps_vec <- unique(c(eps_vec, eps))
    eps_vec <- eps_vec[ order(eps_vec, decreasing=TRUE) ]
    eps_vec <- eps_vec[ eps_vec >=eps ]
    return(eps_vec)
}

## File Name: summary.regpolca.R
## File Version: 0.09


#--- summary for regpolca object
summary.regpolca <- function( object, digits=3, file=NULL, ...)
{
    # open sink
    sirt_osink( file=file )
    len_disp <- 66

    # print summary based on xxirt object
    res <- xxirt_summary_parts(object=object, digits=digits, len_disp=len_disp)

    #-------- output regpolca specific -------
    sirt_display_function(length=len_disp)
    cat("Regularized polytomous latent class analysis (regpolca)\n\n")
    cat( "Regularization type","=", object$regular_type, "\n" )
    cat( "Group regularization","=", object$regular_grouped, "\n" )
    cat( "Regularization parameter(s)","=", object$regular_lam, "\n" )
    cat( "Number of regularized parameters","=", object$n_reg, "\n\n" )

    #* class probabilities
    sirt_display_function(length=len_disp)
    cat("Class Probabilities\n")
    obji <- object$probs_Theta
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    #* item response probabilities
    sirt_display_function(length=len_disp)
    cat("Item Response Functions \n")
    obji <- object$item
    sirt_summary_print_objects(obji=obji, digits=digits, from=3-object$lca_dich,
                rownames_null=TRUE)

    # close sink
    sirt_csink( file=file )
}

## File Name: mml_calc_like.R
## File Version: 0.04


#-- calculation of the likelihood
mml_calc_like <- function (dat2, dat2resp, probs, pseudoll=0)
{
    if ( pseudoll==0 ){
        res <- MML2_CALCPOST_V1( DAT2=dat2, DAT2RESP=dat2resp, PROBS=probs)
    }
    if ( pseudoll==1 ){
        res <- sirt_rcpp_rasch_mml2_calcpost_pseudoll( DAT2=dat2, DAT2RESP=dat2resp,
                    PROBS=probs)
    }
    return(res)
}

## File Name: xxirt_createParTable.R
## File Version: 0.271

#*** create parameter table
xxirt_createParTable <- function( dat, itemtype, customItems=NULL )
{
    I <- ncol(dat)
    ncat1 <- apply( dat, 2, max, na.rm=TRUE ) + 1
    items <- colnames(dat)
    if ( length(itemtype)==1 ){
        itemtype <- rep( itemtype, I )
    }
    dfr <- NULL
    CI <- length(customItems)
    pc <- 0
    for (ii in 1L:I){
        type_ii <- itemtype[ii]
        item_ii <- NULL
        for ( vv in 1L:CI){
            ci_ii <- customItems[[vv]]
            if ( ci_ii$name==type_ii ){
                item_ii <- ci_ii
            }
        }
        if ( is.null(item_ii) ){
            stop( paste0( 'Item type ', type_ii, ' not found!') )
        }
        NP <- length( item_ii$par )
        dfr1 <- data.frame( item=rep( items[ii], NP ) )
        dfr1$itemnr <- ii
        dfr1$ncat <- ncat1[ii]
        dfr1$class <- class(item_ii)
        dfr1$type <- item_ii$name
        dfr1$parname <- names(item_ii$par)
        dfr1$value <- item_ii$par
        dfr1$est <- item_ii$est
        dfr1$lower <- item_ii$lower
        dfr1$upper <- item_ii$upper
        dfr1$prior <- NA
        dfr1$prior_par1 <- NA
        dfr1$prior_par2 <- NA
        if ( ! is.null( item_ii$prior ) ){
            item_ii_prior <- names(item_ii$prior)
            ind_ii <- match( item_ii_prior, dfr1$parname )
            dfr1[ ind_ii, 'prior' ] <- item_ii$prior
            dfr1[ ind_ii, 'prior_par1' ] <- item_ii$prior_par1
            dfr1[ ind_ii, 'prior_par2' ] <- item_ii$prior_par2
        }
        if (item_ii$person_covariates){
            dfr1[,'person_covariates'] <- TRUE
            pc <- pc + 1
        }

        dfr <- rbind( dfr, dfr1 )
    }  # end ii
    #**** create parameter indices
    NP <- nrow(dfr)
    dfr$rowindex <- 1L:NP
    # parameter index
    dfr$parindex <- cumsum( dfr$est )
    #*** parameter label
    dfr$parlabel <- paste0( dfr$item, '_', dfr$parname )
    attr(dfr, 'ncat' ) <- ncat1
    attr(dfr, 'items' ) <- items
    attr(dfr, 'person_covariates') <- (pc>0)
    return(dfr)
}

## File Name: sirt_trim_increment.R
## File Version: 0.02

sirt_trim_increment <- function(increment, max_increment)
{
    eps <- 1E-10
    ci <- ceiling( abs(increment) / ( max_increment + eps ) )
    change <- ifelse( abs(increment) > max_increment, increment/(2*ci), increment )
    return(change)
}

## File Name: rm_sdt_calc_gradient_likelihood_item.R
## File Version: 0.08

rm_sdt_calc_gradient_likelihood_item <- function(logprob1, logprob2,
    numdiff.parm, nik.item, diffindex)
{
    K <- dim(nik.item)[2] - 1
    logprob_D1 <- ( logprob1 - logprob2 ) / (2*numdiff.parm)
    ll <- rowSums( logprob_D1[,1,] * nik.item[,1,] )
    for (kk in 2:(K+1) ){
        ll <- ll + rowSums( logprob_D1[,kk,] * nik.item[,kk,] )
    }
    ll_grad <- ll
    ll_grad <- rowsum(ll_grad, diffindex )
    ll_grad <- ll_grad[ rownames(ll_grad) > 0, 1 ]
    return(ll_grad)
}

## File Name: summary.mcmc.sirt.R
## File Version: 1.33


# summary for MCMC item analysis in sirt
summary.mcmc.sirt <- function( object, digits=3, file=NULL, ... ){

    # open sink
    sirt_osink(file=file)

    cat("-----------------------------------------------------------------\n")

    #- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    #- computation time
    sirt_summary_print_computation_time(object=object)

    #- information criteria
    mcmc_summary_print_information_criteria(object=object)

    #- sample characteristics
    cat( "Number of persons=", nrow(object$dat), "\n" )
    if ( object$model=="2pno.ml"){
        cat( "Number of groups=", object$ic$G, "\n")
        cat( "  Group sizes: M=", round(object$ic$M.n,3),
            " | SD=", round(object$ic$SD.n,3), "\n")
    }
    cat( "Number of items=", ncol(object$dat), "\n" )

    cat( "\nEAP Reliability=")
    cat(round( object$EAP.rel,3 ) )
    cat( "\n")

    cat("-----------------------------------------------------------------\n")
    cat("Item Parameters \n")
    obji <- object$summary.mcmcobj
    obji <- obji[ obji$parameter !="deviance", ]
    vars <- c("parameter", "Mean", "SD", "MAP", "Rhat", "effSize", "Q5", "Q95" )
    obji <- obji[, vars ]
    digits_vec <- sirt_vector_with_names(value=digits, names=vars)
    digits_vec["Rhat"] <- 2
    digits_vec["effSize"] <- 1
    sirt_summary_print_objects(obji=obji, digits=digits_vec, rownames_null=TRUE)

    #- close sink
    sirt_csink(file=file)
}

## File Name: xxirt_sandwich_pml.R
## File Version: 0.118

xxirt_sandwich_pml <- function(object, h=1e-4)
{
    requireNamespace('MASS')
    weights <- object$weights
    partable <- object$partable
    customTheta <- object$customTheta
    par1 <- xxirt_partable_extract_freeParameters( partable=partable )
    par2 <- xxirt_parTheta_extract_freeParameters( customTheta=customTheta )
    par0 <- par <- x <- c(par1, par2)
    NP <- length(x)
    parnames <- names(x)

    # compute gradient function
    opt_fun <- xxirt_nr_opt_fun_pml_casewise
    args <- list(x=x, object=object, eps=1e-14)
    grad <- xxirt_nr_grad_fun_pml_casewise(x=x, args=args, opt_fun=opt_fun, h=h)

    #- compute variance matrix
    G1 <- grad*sqrt(weights)
    A <- crossprod(G1)
    rownames(A) <- colnames(A) <- parnames

    #- compute hessian
    hess <- matrix(NA, nrow=NP, ncol=NP, dimnames=list(parnames,parnames))

    pp <- 1
    for (pp in 1L:NP){
        x1 <- sirt_add_increment(x=x, pos=pp, value=h)
        grad1 <- xxirt_nr_grad_fun_pml_casewise(x=x1, args=args, opt_fun=opt_fun, h=h)
        hess_pp <- (grad1-grad)/h
        hess[,pp] <- colSums(hess_pp*weights)
    }
    hess <- ( hess + t(hess) ) / 2
    B1 <- MASS::ginv(hess)

    V <- B1 %*% A %*% t(B1)
    se <- sqrt(diag(V))
    names(se) <- parnames

    #-- output
    res <- list(V=V, A=A, hess=hess, se=se)
    return(res)
}

## File Name: sirt_import_coda_as.mcmc.list.R
## File Version: 0.02

sirt_import_coda_as.mcmc.list <- function(...)
{
    TAM::require_namespace_msg('coda')
    res <- coda::as.mcmc.list(...)
    return(res)
}

## File Name: sirt_add_list_elements.R
## File Version: 0.04

sirt_add_list_elements <- function(res, res2)
{
    NR <- length(res2)
    for (rr in 1L:NR){
        res[[ names(res2)[[rr]] ]] <- res2[[rr]]
    }
    return(res)
}

## File Name: IRT.irfprob.mirt.R
## File Version: 0.27


#--- irfprob singleGroup
IRT.irfprob.mirt.singleGroup <- function( object, ... )
{
    object <- mirt.wrapper.posterior(mirt.obj=object)
    ll <- object$probs
    attr(ll,"theta") <- object$theta.k
    attr(ll,"prob.theta") <- object$pi.k
    attr(ll,"G") <- 1
    return(ll)
}
IRT.irfprob.ConfirmatoryClass <- IRT.irfprob.mirt.singleGroup
IRT.irfprob.ExploratoryClass <- IRT.irfprob.mirt.singleGroup
IRT.irfprob.SingleGroupClass <- IRT.irfprob.mirt.singleGroup


#--- irfprob multipleGroup
#--- by default, only item response functions from the first group are extracted
IRT.irfprob.mirt.multipleGroup <- function( object, group=1, ... )
{
    mobj <- object
    groups <- object@Data$groupNames
    G <- length(groups)
    prob.theta <- list()
    for (gg in 1:G){
        object <- mirt.wrapper.posterior(mirt.obj=mobj, group=groups[gg])
        if (gg==group){
            ll <- object$probs
            theta <- object$theta.k
            TP <- object$TP
        }
        prob.theta[[gg]] <- object$pi.k
    }
    prob.theta <- matrix( unlist(prob.theta), nrow=TP, ncol=G)
    attr(ll,"theta") <- theta
    attr(ll,"prob.theta") <- prob.theta
    attr(ll,"G") <- G
    return(ll)
}
IRT.irfprob.MultipleGroupClass <- IRT.irfprob.mirt.multipleGroup

## File Name: latent.regression.em.raschtype.R
## File Version: 2.62


#----------------------------------------------------------------------------

#-- Latent Regression: EM algorithm
#-- Rasch type model: based on individual likelihood
latent.regression.em.raschtype <- function( data=NULL, f.yi.qk=NULL, X,
        weights=rep(1,nrow(X)), beta.init=rep(0,ncol(X)), sigma.init=1,
        b=rep(0,ncol(X)), a=rep(1, length(b)), c=rep(0, length(b)),
        d=rep(1, length(b)), alpha1=0, alpha2=0, max.parchange=.0001,
        theta.list=seq(-5,5,len=20), maxiter=300, progress=TRUE )
{
    X <- as.matrix(X)   # matrix format
    # init parameters
    beta0 <- beta.init
    sig0 <- sigma.init
    # normalize sample weights
    weights <- length(weights) * weights / sum(weights )
    # initialize iteration index and value of parameter change
    iter <- 1 ; parchange <- 1000
    #***
    # data preparation
    if ( ! is.null(data)){
        data <- as.matrix(data)
        y <- data
        y[ is.na(y)] <- 0
        dat2.resp <- 1-is.na(data)
        I <- ncol(y)    # number of items
        dat1 <- data.frame( 1+0*y[,1], 1 )
                    }
    LT <- length( theta.list )
    pi.k <- sirt_dnorm( theta.list )
    theta.kM <- matrix( theta.list, nrow=nrow(X), ncol=length(theta.list), byrow=TRUE )
    theta.kM2 <- theta.kM^2
    n <- nrow(X)
    # calculate likelihood if not provided
    if ( is.null( f.yi.qk) ){
        f.yi.qk <- .e.step.raschtype( dat1=dat1, dat2=y, dat2.resp=dat2.resp,
            theta.k=theta.list, pi.k=pi.k, I=I,
            n=nrow(y), b=b, fixed.a=a, fixed.c=c,  fixed.d=d,
            alpha1=alpha1, alpha2=alpha2, group=NULL, pseudoll=0, f.qk.yi=NULL )$f.yi.qk
                    }
    post <- f.yi.qk / rowSums( f.yi.qk )
    #....................................
    # begin iterations
    while( ( parchange > max.parchange ) & ( iter < maxiter ) ){

        EAP <- rowSums( theta.kM * post )
        SE.EAP <- sqrt( rowSums( theta.kM2 * post ) - EAP^2     )
        # estimate latent regression model (linear model)
        mod <- stats::lm( EAP ~ 0 + X, weights=weights)
        cmod <- stats::coef(mod)
        # Calculation of SD
        sigma <- sqrt( mean( weights* ( SE.EAP^2 + stats::resid(mod)^2 ) ) )
        # fitted values
        fmod <- stats::fitted(mod)
        sigma.res <- stats::sd( stats::resid(mod) )
        h1 <- theta.kM - matrix( fmod,  nrow=n, ncol=LT )
        prior <- sirt_dnorm( h1, sd=sigma )
        prior <- prior / rowSums( prior )
        post <- f.yi.qk * prior
        post <- post / rowSums( post )
        parchange <- max( abs(sigma - sig0), abs( cmod - beta0) )
        if (progress){
            cat( paste("Iteration ", iter,": max parm. change ", round( parchange, 6 ),sep=""),
    #                " # Regr. Coeff. ", round(as.vector(cmod),5),
                     "\n") ; flush.console()
                     }
        # parameter update
        sig0 <- sigma ; beta0 <- cmod ; iter <- iter + 1
                    }
    # standard errors for regression coefficients
    V <- ncol(X)        # number of X variables (predictors)
    W <- diag(weights)    # diagonal matrix of weights
    # simple covariance matrix
#    h1 <- t(X) %*% W %*% X
    h1 <- crossprod( X, W ) %*% X
    h2 <- solve(h1)
    vcov.simple <- sigma^2 * h2
    # covariance matrix which includes measurement error
    error.weights <- 1 - SE.EAP^2 / sigma^2
#    EW <- outer( sqrt( error.weights ), rep(1,ncol(X)))
#    X1 <- X * EW
    X1 <- X * sqrt(error.weights)
#    h1 <- t(X1) %*% W %*% X1
    h1 <- crossprod( X1, W ) %*% X1
    h2 <- solve(h1)
    vcov.latent <- sigma^2 * h2
    scoefs <- matrix( 0, nrow=V, ncol=9 )
    scoefs <- data.frame(scoefs)
    colnames(scoefs) <- c("est", "se.simple", "se", "t", "p",
                            "beta", "fmi", "N.simple", "pseudoN.latent" )
    rownames(scoefs) <- names(beta0)
    scoefs[,1] <- beta0
    scoefs[, "se.simple"] <- sqrt( diag( vcov.simple) )
    scoefs[, "se"] <- sqrt( diag( vcov.latent) )
    # explained variance
    explvar <- stats::var( fmod )
#    totalvar <- explvar + sigma^2
    totalvar <- explvar + sigma^2
    rsquared <- explvar / totalvar
    scoefs$beta <- scoefs$est / sqrt( totalvar ) * apply( X, 2, stats::sd )
    scoefs$fmi <- 1 -  scoefs$se.simple^2 / scoefs$se^2
    scoefs[,"N.simple" ] <- nrow(X)
    scoefs[,"pseudoN.latent"] <- sum( error.weights )
    scoefs$t <- scoefs$est / scoefs$se
    scoefs$p <- 2 * ( 1 - stats::pnorm( abs( scoefs$t ) ) )
    if ( ! is.null( colnames(X) ) ){ rownames(scoefs) <- colnames(X) } # use column names of X
    if (progress){
        cat("\nRegression Parameters\n\n")
        .prnum(scoefs,4)        # print results
        cat( paste( "\nResidual Variance=", round( sigma^2, 4 ) ), "\n" )
        cat( paste( "Explained Variance=", round( explvar, 4 ) ), "\n" )
        cat( paste( "Total Variance=", round( totalvar, 4 ) ), "\n" )
        cat( paste( "            R2=", round( rsquared, 4 ) ), "\n" )
                }
    #********
    # list of results
    res <- list( "iterations"=iter - 1, "maxiter"=maxiter,
                "max.parchange"=max.parchange,
                "coef"=beta0, "summary.coef"=scoefs, "sigma"=sigma,
                "vcov.simple"=vcov.simple, "vcov.latent"=vcov.latent,
                "post"=post, "EAP"=EAP, "SE.EAP"=SE.EAP,
                "explvar"=explvar, "totalvar"=totalvar,
                "rsquared"=rsquared )
    class(res) <- "latent.regression"
    return(res)
    }
#-----------------------------------------------------------------------------------------------------

## File Name: rm_facets_print_progress.R
## File Version: 0.08

rm_facets_print_progress <- function(dev, dev0, b.rater, b.rater0, a.rater, a.rater0,
        tau.item, tau.item0, a.item, a.item0, mu, sigma, iter,
        digits_parm=6, digits_trait=3, digits_deviance=4)
{
    #--- print deviance
    rm_facets_print_progress_deviance( dev=dev, dev0=dev0, digits_deviance=digits_deviance, iter=iter )
    #--- item and rater parameters
    rm_facets_print_progress_parameter( parm=b.rater, parm0=b.rater0, parmlabel="b.rater", digits_parm=digits_parm )
    rm_facets_print_progress_parameter( parm=a.rater, parm0=a.rater0, parmlabel="a.rater", digits_parm=digits_parm )
    rm_facets_print_progress_parameter( parm=tau.item, parm0=tau.item0, parmlabel="tau.item", digits_parm=digits_parm )
    rm_facets_print_progress_parameter( parm=a.item, parm0=a.item0, parmlabel="a.item", digits_parm=digits_parm )
    #--- trait distribution
    rm_facets_print_progress_trait_distribution( parm=mu, parmlabel="Trait M  ", digits_trait=digits_trait )
    rm_facets_print_progress_trait_distribution( parm=sigma, parmlabel="Trait SD ", digits_trait=digits_trait )
    utils::flush.console()
}

## File Name: rm_sdt_create_partable_pargroup_indices.R
## File Version: 0.08


rm_sdt_create_partable_pargroup_indices <- function(partable, item.index,
    diffindex)
{
    pargroup_diffindex <- list()
    pargroup_index <- list()
    pargroup_type <- list()
    max_pargroup <- max(partable$pargroup)
    for (pp in seq_len(max_pargroup) ){
        pt_pp <- partable[ partable$pargroup==pp, ]
        if (pt_pp$type[1]=="tau"){
            I <- max(partable$row)
            g1 <- seq_len(I)
            item0 <- which( pt_pp$parindex < 0 )
            g1[ g1 %in% item0 ] <- 0
            pargroup_diffindex[[pp]] <- g1
        }
        if (pt_pp$type[1]=="a"){
            g1 <- seq_len(I)
            partable_a <- partable[ partable$type=="a", ]
            if ( sd( partable_a$parindex)==0 ){
                g1 <- 1+0*g1
            }
            item0 <- which( pt_pp$parindex < 0 )
            g1[ g1 %in% item0 ] <- 0
            pargroup_diffindex[[pp]] <- g1
        }
        if (pt_pp$type[1]=="c"){
            g1 <- diffindex$c.rater
            item0 <- which( pt_pp$parindex < 0 )
            g1[ item.index %in% item0 ] <- 0
            pargroup_diffindex[[pp]] <- g1
        }
        if (pt_pp$type[1]=="d"){
            g1 <- diffindex$d.rater
            item0 <- which( pt_pp$parindex < 0 )
            g1[ item.index %in% item0 ] <- 0
            pargroup_diffindex[[pp]] <- g1
        }
        p1 <- partable[ partable$est & partable$pargroup==pp, "parindex" ]
        pargroup_index[[pp]] <- p1
        pargroup_type[[pp]] <- pt_pp$type[1]
    }
    max_pargroup <- max( partable$pargroup )
    np <- max(partable$parindex)

    #--- output
    res <- list( pargroup_diffindex=pargroup_diffindex, np=np,
                pargroup_index=pargroup_index, pargroup_type=pargroup_type,
                max_pargroup=max_pargroup)
    return(res)
}

## File Name: dirichlet.mle.R
## File Version: 0.13



#--- Maximum likelihood estimation of distribution parameters
dirichlet.mle <- function( x, weights=NULL, eps=10^(-5), convcrit=.00001,
        maxit=1000, oldfac=.3, progress=FALSE)
{
    N <- nrow(x)
    K <- ncol(x)
    # compute log pbar
    x <- ( x+eps ) / ( 1 + 2*eps )
    x <- x / rowSums(x)
    N <- nrow(x)
    if ( is.null(weights) ){
        weights <- rep(1,N)
    }
    weights <- N * weights / sum( weights )
    log.pbar <- colMeans( weights * log( x ) )
    # compute inits
    alphaprob <- colMeans( x * weights )
    p2 <- mean( x[,1]^2 * weights )
    xsi <- ( alphaprob[1] - p2 ) / ( p2 - ( alphaprob[1] )^2 )
    alpha <- xsi * alphaprob
    K1 <- matrix(1,K,K)
    conv <- 1
    iter <- 1

    #--- BEGIN iterations
    while( ( conv > convcrit ) & (iter < maxit) ){
        alpha0 <- alpha
        g <- N * digamma( sum(alpha ) ) - N * digamma(alpha) + N * log.pbar
        z <- N * sirt_digamma1( sum(alpha ))
        H <- diag( -N * sirt_digamma1( alpha ) ) + z
        alpha <- alpha0 - solve(H, g )
        alpha[ alpha < 0 ] <- 1e-10
        alpha <- alpha0 + oldfac*( alpha - alpha0 )
        conv <- max( abs( alpha0 - alpha ) )
        if (progress){
            print( paste( iter, sum(alpha), conv) )
            utils::flush.console()
        }
        iter <- iter+1
    }
    alpha0 <- sum(alpha)
    xsi <- alpha / alpha0
    res <- list( alpha=alpha, alpha0=alpha0, xsi=xsi )
    return(res)
}

## File Name: mgsem_loss_function_suffstat.R
## File Version: 0.184

mgsem_loss_function_suffstat <- function(suffstat, Mu, Sigma, p=2, eps=1e-3,
        deriv=FALSE, approx_method="lp", only_deriv=FALSE, output_all=FALSE )
{
    N <- suffstat$N
    M <- suffstat$M
    S <- suffstat$S
    if (missing(Sigma)){
        res <- Mu
        Mu <- res$Mu
        Sigma <- res$Sigma
    }
    I <- length(Mu)

    #** mean structure
    t1 <- 0
    m1 <- M-Mu
    w <- suffstat$weights_M
    if(!only_deriv){
        y <- mgsem_power_fun_differentiable_approx(x=m1, p=p, eps=eps, deriv=FALSE,
                approx_method=approx_method)
        t1 <- sum(y*w)
    }
    if (deriv){
        y <- mgsem_power_fun_differentiable_approx(x=m1, p=p, eps=eps,
                deriv=TRUE, approx_method=approx_method)
        t1_der <- -as.vector(y)*w
    }

    #** covariance structure
    t2 <- NULL
    v1 <- S-Sigma
    v1 <- mgsem_vech(x=v1)
    w <- suffstat$vech_weights_S
    if (! only_deriv ){
        y <- mgsem_power_fun_differentiable_approx(x=v1, p=p, eps=eps,
                deriv=FALSE, approx_method=approx_method)
        t2 <- sum(y*w)
    }
    if (deriv){
        y <- mgsem_power_fun_differentiable_approx(x=v1, p=p, eps=eps,
                deriv=TRUE, approx_method=approx_method)
        t2_der <- -y*w
    }

    # total loss function
    res <- -N/2*(t1+t2)

    if (deriv){
        res <- list(loss_fun=res, dermean=-N/2*t1_der, dercov=-N/2*t2_der)
    }
    #-- arrange output
    return(res)
}

## File Name: rm_sdt_pem_inits.R
## File Version: 0.06

rm_sdt_pem_inits <- function( tau.item, a.item, c.rater, d.rater, est.a.item, a.item.fixed, skillspace, PEM )
{
    parmlist <- list( tau.item=tau.item, a.item=a.item, c.rater=c.rater, d.rater=d.rater, mu=0, sigma=1)
    pem_parameter_index <- sirt_pem_create_parameter_index( parmlist=parmlist )
    pem_parameter_sequence <- list()
    center_log_a <- ( ! is.null( a.item.fixed ) ) & ( est.a.item )
    pem_pars <- c("c.rater","d.rater","a.item","tau.item","mu","sigma")
    if ( skillspace=="discrete" ){
        PEM <- FALSE
    }
    #--- output
    res <- list(PEM=PEM, pem_pars=pem_pars, center_log_a=center_log_a, pem_parameter_index=pem_parameter_index,
                pem_parameter_sequence=pem_parameter_sequence )
    return(res)
}

## File Name: xxirt_ic_compute_criteria.R
## File Version: 0.07

xxirt_ic_compute_criteria <- function(ic, compute_np=TRUE, estimator="ML")
{
    dev <- ic$deviance
    if (compute_np){
        ic$np <- ic$np.item + ic$np.Theta
    }
    if (estimator=='ML'){
        ic$AIC <- dev + 2*ic$np
        log_n <- log(ic$n)
        ic$BIC <- dev + log_n * ic$np
        ic$CAIC <- dev + ( log_n + 1 )*ic$np
        ic$AICc <- ic$AIC + 2*ic$np * ( ic$np + 1 ) / ( ic$n - ic$np - 1 )
    }
    return(ic)
}

## File Name: plot.rasch.mml.R
## File Version: 0.15



##############################################################
# S3 plot method for objects of class rasch.mml2
plot.rasch.mml <- function( x, items=NULL, xlim=NULL, main=NULL,
        ...){
    object <- x
    theta <- object$theta.k
    if ( is.matrix(theta) ){
       stop("Plot function is only applicable for unidimensional models")
                        }
    probs <- t(object$pjk)
    if ( is.null(items) ){ items <- 1:(nrow(probs) ) }
    I <- length(items)
    if (is.null(xlim)){ xlim <- c( min(theta), max(theta) ) }
    xlabplot <- expression( paste( theta ))
    ylabplot <- expression(paste(P,"(",X[i]==1, "|", theta, ")" ))
    if (object$irtmodel=="ramsay.qm"){
            xlabplot <- expression( paste( "log ",theta ))
                        }
    graphics::plot( theta, as.vector(probs[1,]), type="l", lty=1, xlab=xlabplot,
        ylab=ylabplot, xlim=xlim, ylim=c(0,1), main=main, ... )
    for (ii in 2:I){
        graphics::lines( theta, as.vector(probs[ii,]), type="l", lty=ii)
                    }
            }
##############################################################

## File Name: sirtcat.R
## File Version: 0.091


sirtcat <- function( label, time0, active )
{
    if (active){
        z0 <- time0
        cat( label, '  ' )
        z1 <- Sys.time()
        print(z1-z0)
        z0 <- z1
        zout <- z0
    } else {
        zout <- NULL
    }
    return(zout)
}

## File Name: plot.lsdm.R
## File Version: 0.02


plot.lsdm <- function(x, ...)
{
    graphics::matplot( x=x$theta, y=t(x$attr.curves), xlab=expression(theta),
        ylab="Attribute response curve", ylim=c(0,1), ... )
}

## File Name: summary.lsem.permutationTest.R
## File Version: 0.298


summary.lsem.permutationTest <- function( object, file=NULL, digits=3, ... )
{

    # open sink for a file
    sirt_osink( file=file )

    cat('-----------------------------------------------------------------\n')
    cat('Permutation Test for Local Structural Equation Model \n\n')

    packages <- c('sirt', 'lavaan')
    if (object$use_lavaan_survey){
        packages <- c(packages, 'lavaan.survey')
    }
    sirt_summary_print_packages(packages=packages)

    cat('\nFunction \'sirt::lsem.permutationTest\' \n\n')

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    cat( 'Date of Analysis:', paste( object$s2 ), '\n' )
    cat('Computation Time:', print(object$s2 - object$s1), '\n\n')

    cat( 'Number of permutations', '=', object$B, '\n')
    cat( 'Percentage of non-converged datasets',  '=',
                round(object$nonconverged_rate, digits=digits), '\n')
    cat( paste0( 'Number of observations',  '=', round(object$N,digits) ), '\n')
    cat( paste0( 'Bandwidth factor',  '=', round(object$h,digits) ), '\n')
    cat( paste0( 'Bandwidth',  '=', round(object$bw,digits) ), '\n')
    cat( paste0( 'Number of focal points for moderator',  '=',
                        length(object$moderator.grid ) ), '\n')

    cat('\nlavaan Model\n')
    cat(object$lavmodel)

    cat('\n\n')

    cat('Global Test Statistics\n\n')
    obji <- object$teststat
    VV <- ncol(obji)
    for (vv in 2L:VV){
        obji[,vv] <- round( obji[,vv], digits )
    }
    print(obji)

    cat('\n')
    cat('Pointwise Test Statistics\n\n')
    obji <- object$parameters_pointwise_test
    vars <- c('est', 'p')
    for (vv in vars){
        obji[,vv] <- round( obji[,vv], digits )
    }
    obji <- obji[, c('par', 'parindex', 'moderator', 'est', 'p') ]
    rownames(obji) <- NULL
    print(obji)

    # close file
    sirt_csink(file=file)

}

## File Name: nedelsky.sim.R
## File Version: 0.154


#---- simulate data according to the Nedelsky model
nedelsky.sim <- function( theta, b, a=NULL, tau=NULL )
{
    Theta <- matrix( theta, ncol=1 )
    N <- length(theta)
    # generate all combinations
    nodes <- c(0,1)
    ndim <- K <- ncol(b)
    I <- nrow(b)
    if ( is.null(a) ){
        a <- rep(1,I)
    }
    combis <- as.matrix( expand.grid( as.data.frame(
                        matrix( rep(nodes, ndim), ncol=ndim))))
    if ( is.null(tau) ){
        tau <- matrix( 1, nrow=I, ncol=K+1 )
    }
    # dataset
    dat <- matrix( NA, nrow=N, ncol=I)
    for (ii in 1L:I){
        b0 <- as.vector(b[ii,])
        a0 <- a[ii]
        thdim <- 1
        tau0 <- as.vector(tau[ii,])
        probs <- nedelsky.irf( Theta, K, b=b0, a=a0, tau=tau0, combis )$probs
        cprobs <- rowCumsums.sirt(matr=probs)
        rn1 <- stats::runif(N)
        dat[,ii] <- rowIntervalIndex.sirt(matr=cprobs,rn=rn1) - 1
    }
    colnames(dat) <- paste0( 'I', 100+1L:I )
    return(dat)
}

## File Name: mirt_summary.R
## File Version: 0.151


mirt_summary <- function(object, digits=4, file=NULL, ...)
{
    TAM::require_namespace_msg("mirt")

    sirt_osink(file=file)

    is_mg <- inherits(object,"MultipleGroupClass")

    #- print mirt object
    print(object)
    cat("\n\n")

    # print estimated coefficients
    res <- mirt.wrapper.coef(mirt.obj=object)
    cat("*** Item Parameters ***\n\n")
    sirt_summary_print_objects(obji=res$coef, digits=digits, from=2+is_mg)

    cat("\n*** Trait Distribution ***\n\n")
    G <- res$G
    if (G==1){
        sirt_summary_print_objects(obji=res$GroupPars, digits=digits)
    } else {
        for (gg in 1:G){
            groups <- res$groups
            cat(paste0("Group ", groups[gg],":\n"))
            sirt_summary_print_objects(obji=res$GroupPars[[gg]], digits=digits)
            cat("\n")
        }
    }

    sirt_csink(file=file)
}

## File Name: stratified.cronbach.alpha.R
## File Version: 0.257


# stratified Cronbach's Alpha
stratified.cronbach.alpha <- function( data, itemstrata=NULL )
{
    stratcomp <- TRUE
    if ( is.null(itemstrata) ){
        itemstrata <- cbind( colnames(data), 1 )
        stratcomp <- FALSE
    }
    #---------------------------------------
    dfr <- data.frame( scale="total", stratified_cronbach_alpha_compute_alpha(data) )
    # calculation of stratified alpha
    itemstrata.u <- sort(unique( itemstrata[,2] ))
    for (gg in itemstrata.u){
        data_gg <- data[, itemstrata[ itemstrata[,2]==gg, 1] ]
        dfr1 <- data.frame( scale=gg,
            stratified_cronbach_alpha_compute_alpha(data=data_gg) )
        dfr <- rbind( dfr, dfr1 )
    }
    # stratified alpha
    dfr$alpha.stratified <- NA
    var_tot <- dfr[ -1, "var.tot" ]
    dfr_alpha <- dfr[ -1, "alpha" ]
    # dfr$alpha.stratified[1] <- 1 - sum (( 1 - dfr_alpha )*var_tot ) / var_tot
    dfr$alpha.stratified[1] <- 1 - sum (( 1 - dfr_alpha )*var_tot ) / dfr[ 1, "var.tot"]
    obji <- dfr
    obji[, -c(1:2)] <- round( obji[,-c(1:2) ], 3 )
    if ( ! stratcomp ){
        obji <- obji[1,]
    }
    print(obji)
    invisible(dfr)
}

## File Name: sirt_optimizer_summary_print.R
## File Version: 0.141

sirt_optimizer_summary_print <- function(res, msg="Information about optimization")
{
    digits <- 6
    cat('\n---', msg, '---\n\n')
    cat( 'Optimizer', '=', res$optimizer, '\n')
    cat( 'Converged', '=', res$converged, '\n')
    cat('Optimization Function Value', '=', round(res$value,digits), '\n' )
    cat( 'Number of iterations', '=', res$iter, '\n')
    cat( 'Elapsed time', '=', ' ')
    print(res$time)
    cat('\n')
}

## File Name: sirt_dmvnorm_discrete.R
## File Version: 0.07

sirt_dmvnorm_discrete <- function(x, mean=NULL, sigma=NULL, as_matrix=FALSE,
    eps=0, ...)
{
    y <- sirt_dmvnorm(x=x, mean=mean, sigma=sigma, ... )
    if (as_matrix){
        y <- matrix(y, nrow=length(y), ncol=1)
    }
    y <- y + eps
    y <- y / sum(y)
    return(y)
}

## File Name: read.fwf2.R
## File Version: 1.11


# This function reads fwf files
read.fwf2 <- function( file, format, variables=NULL)
{
    ff <- readLines( file )
    ind.ff1 <- c( 1, cumsum(format)[- length(format) ] + 1 )
    ind.ff2 <- cumsum(format)
    I <- length(format)
    n <- length( ff )
    dfr <- data.frame( matrix(0, nrow=n, ncol=I ) )
    for (ii in 1:I){
        dfr[,ii ] <- as.numeric( substring( ff, ind.ff1[ii], ind.ff2[ii] )  )
    }
    if (!is.null(variables)){
        colnames(dfr) <- variables
    }
    return(dfr)
}

## File Name: R2noharm.EAP.R
## File Version: 0.26

R2noharm.EAP <- function( noharmobj, theta.k=seq(-6,6,len=21), print.output=TRUE )
{
    mod <- noharmobj
    mod$guess <- guess <- mod$guesses
    f0 <- mod$final.constants
    FL <- mod$loadings.theta
    D <- mod$dimensions
    P <- mod$factor.cor
    I <- length(mod$guess)
    dat <- mod$dat
    N <- nrow(dat)
    theta.k <- as.matrix( expand.grid( as.data.frame( matrix( rep( theta.k, D), ncol=D ) ) ) )
    TT <- nrow(theta.k)
    if ( TT > 1000 ){
        theta.k <- qmc.nodes( snodes=1000, ndim=D )
    }
    TT <- nrow(theta.k)
    #***
    # calculate probabilities
    probs <- matrix( f0, nrow=I, ncol=TT, byrow=F)
    for (dd in 1:D){
        probs <- probs + outer( FL[,dd], theta.k[,dd] )
    }
    guessM <- matrix( mod$guess, nrow=I, ncol=TT )
    upperM <- matrix( mod$upper, nrow=I, ncol=TT )
    probs1 <- guessM + ( upperM - guessM) * stats::pnorm( probs )
    probs <- array( 0, dim=c(I, TT, 2 ) )
    probs[,,1] <- 1-probs1
    probs[,,2] <- probs1

    #****
    # evaluate posterior distribution
    prior.density <- sirt_dmvnorm_discrete( x=as.matrix(theta.k), mean=rep(0,D), sigma=as.matrix(P) )
    prior.density <- matrix( prior.density, nrow=N, ncol=TT, byrow=TRUE )
    like <- matrix( 1, nrow=N, ncol=TT )
    for (ii in 1:I){
        ind.ii <- which( ! is.na( dat[,ii] ) )
        like[ind.ii,] <- like[ ind.ii, ] * t( probs[ ii,, dat[ ind.ii,ii ] + 1 ] )
    }
    posterior <- like * prior.density
    # posterior <- like
    posterior <- posterior / rowSums( posterior )

    #***
    # calculate EAP of all dimensions and reliabilities
    person <- data.frame( "case"=1:N)
    nstudl <- rep(1,N)
    EAP.rel <- rep(0,D)
    hwtE <- posterior
    pweights <- nstudl
        for ( dd in 1:D ){
        # dd <- 1  # dimension
        person$EAP <- rowSums( hwtE * outer( nstudl, theta.k[,dd] ) )
        person$SD.EAP <- sqrt(rowSums( hwtE * outer( nstudl, theta.k[,dd]^2 ) ) - person$EAP^2)
        #***
        # calculate EAP reliability
        # EAP variance
        EAP.variance <- stats::weighted.mean( person$EAP^2, pweights ) -
                            ( stats::weighted.mean( person$EAP, pweights ) )^2
        EAP.error <- stats::weighted.mean( person$SD.EAP^2, pweights )
        EAP.rel[dd] <- EAP.variance / ( EAP.variance + EAP.error )
        colnames(person)[ which( colnames(person)=="EAP" ) ] <- paste("EAP.Dim", dd, sep="")
        colnames(person)[ which( colnames(person)=="SD.EAP" ) ] <- paste("SD.EAP.Dim", dd, sep="")
        }
    if ( print.output ){
        cat("EAP Reliabilities:\n")
        print( round (EAP.rel,3) )
    }
    res <- list( person=person, theta=theta.k, posterior=posterior, like=like,
                        EAP.rel=EAP.rel, probs=probs )
    return(res)
}

## File Name: ccov_np_score_density.R
## File Version: 0.07

ccov_np_score_density <- function(score, thetagrid, smooth=TRUE)
{
    if (smooth){
        thg_dens <- stats::density(x=score, from=min(thetagrid), to=max(thetagrid))
        wgt_thetagrid <- ccov_np_regression(x=thg_dens$x, y=thg_dens$y,
                                xgrid=thetagrid, bwscale=.1)
        wgt_thetagrid <- sirt_sum_norm(x=wgt_thetagrid)
    } else {
        a1 <- stats::aggregate(x=1+0*score, by=list(score), FUN=sum, na.rm=TRUE)
        wgt_thetagrid <- sirt_sum_norm(x=a1[,2])
    }
    return(wgt_thetagrid)
}

## File Name: xxirt_parTheta_include_freeParameters.R
## File Version: 0.02


xxirt_parTheta_include_freeParameters <- function(customTheta, x)
{
    if (!is.null(x)){
        customTheta$par[ customTheta$est ] <- x
    }
    return(customTheta)
}

## File Name: gom_em_loglike_parameter_conversion.R
## File Version: 0.192


gom_em_loglike_parameter_conversion <- function(x, ind_lambda, ind_pi, I, K,
            ind_mu, ind_sigma, model, theta_grid, lambda_partable)
{
    mu <- NULL
    Sigma <- NULL
    #- item response probabilities
    lambda_logit <- x[ind_lambda]
    lambda_logit <- lambda_logit[ lambda_partable$par_index ]
    lambda <- gom_em_extract_lambda_matrix(lambda_logit=lambda_logit, I=I, K=K)
    # trait distribution
    if ( model=="GOM" ){
        pi_k_logit <- x[ind_pi]
        pi.k <- sirt_logit_to_probs(y=pi_k_logit)
    }
    if (model=="GOMnormal"){
        mu <- x[ind_mu]
        sigma_chol <- x[ind_sigma]
        K1 <- K-1
        sc <- matrix(0, nrow=K1, ncol=K1)
        sc[!upper.tri(sc)] <- sigma_chol
        Sigma <- tcrossprod(x=sc)
        pi.k <- sirt_dmvnorm_discrete(x=theta_grid, mean=mu, sigma=Sigma, eps=1e-10)
    }
    #--- output
    res <- list(lambda=lambda, pi.k=pi.k, mu=mu, Sigma=Sigma)
    return(res)
}

## File Name: rm_summary_information_criteria_print_one_criterium.R
## File Version: 0.12

rm_summary_information_criteria_print_one_criterium <- function(ic, crit, desc, digits_crit=0, digits_penalty=2)
{
    val <- ic[[crit]]
    deviance <- ic[["deviance"]]
    crit_label0 <- crit
    if ( nchar(crit_label0)==3){
        crit_label0 <- paste0( crit_label0, " " )
    }
    crit_label <- paste0( crit_label0, " ", "=", " ")
    penalty <- val - deviance
    cat( crit_label, round( val, digits_crit ), " | penalty","=", round( penalty,digits_penalty ),
            "   |", desc, "  \n" )
}

## File Name: summary.linking.haberman.R
## File Version: 0.284


summary.linking.haberman <- function( object, digits=3, file=NULL, ... )
{
    # open sink
    sirt_osink(file=file)

    display_string <- sirt_summary_print_display(symbol="-", len=65)
    cat(display_string)

    #- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    cat("\n")
    cat(object$description, "")

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object$time)

    cat(display_string)
    cat("Estimation information item slopes\n")
    linking_haberman_summary_estimation_information(res_opt=object$res_opt_slopes)

    cat(display_string)
    cat("Estimation information item intercepts\n")
    linking_haberman_summary_estimation_information(res_opt=object$res_opt_intercepts)

    cat(display_string)
    cat("Transformation parameters (Haberman linking)\n")
    obji <- object$transf.pars
    sirt_summary_print_objects(obji=obji, digits=digits, from=2, rownames_null=FALSE)

    cat("\nLinear transformation for item parameters a and b\n")
    obji <- object$transf.itempars
    sirt_summary_print_objects(obji=obji, digits=digits, from=2, rownames_null=FALSE)

    cat("\nLinear transformation for person parameters theta\n")
    obji <- object$transf.personpars
    sirt_summary_print_objects(obji=obji, digits=digits, from=2, rownames_null=FALSE)

    cat("\n")
    cat(display_string)
    cat("R-Squared Measures of Invariance (all items and unweighted)\n")
    obji <- object$es.invariance
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    cat("\nR-Squared Measures of Invariance (weighted)\n")
    obji <- object$es.robust
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    cat("\n-----------------------------------------------------------------\n")
    if ( ! object$linking_slopes ){
        cat("Estimated DIF effects in logarithms of item slopes \n")
        obji <- object$a.resid[ object$selitems, ]
        sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)
        cat("\n")
    }
    cat("Estimated DIF effects of item intercepts \n")
    obji <- object$b.resid[ object$selitems, ]
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    cat("\n-----------------------------------------------------------------\n")
    digits2 <- 1
    if ( ! object$linking_slopes ){
        cat("Used items in linking item slopes \n")
        obji <- object$a.item_stat
        sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)
        cat("\n")
    }
    cat("Used items in linking item intercepts \n")
    obji <- object$b.item_stat
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    # close sink
    sirt_csink( file=file )
}

## File Name: xxirt_hessian_compute_loglike.R
## File Version: 0.03


xxirt_hessian_compute_loglike <- function(p.xi.aj, prior1, weights)
{
    post_unnorm <- prior1 * p.xi.aj
    # compute log-likelihood
    dev <- sum( weights * log( rowSums( post_unnorm ) ) )
    return(dev)
}

## File Name: regpolca_postproc_prob_Theta.R
## File Version: 0.08

regpolca_postproc_prob_Theta <- function(probs_Theta)
{
    K <- nrow(probs_Theta)
    G <- ncol(probs_Theta)
    rownames(probs_Theta) <- paste0('Class', 1:K)
    colnames(probs_Theta) <- paste0('Gr', 1:G)
    return(probs_Theta)
}

## File Name: rm_sdt_calc_probs_gpcm_rcpp.R
## File Version: 0.12

rm_sdt_calc_probs_gpcm_rcpp <- function(a.item, tau.item, Qmatrix, theta.k, VV, K, TP,
    eps=0, use_log=FALSE, as_vector=FALSE)
{
    K1 <- K+1
    prob_dim <- c(VV, K1, TP)
    res <- sirt_rcpp_rm_sdt_calc_probs_gpcm( a=a.item, tau=tau.item,
                theta_k=theta.k, VV=VV, K1=K1, TP=TP, eps=eps, use_log=use_log )
    if ( ! as_vector ){
        res <- array(res, dim=prob_dim)
    }
    return(res)
}

## File Name: mirt.wrapper.calc.counts.R
## File Version: 0.12


#######################################################
# calculate counts
mirt.wrapper.calc.counts <- function( resp, theta, resp.ind,
    group, maxK, pweights, hwt )
{
    TP <- nrow(theta)
    I <- ncol(resp)
    if ( is.null( group )){
        group <- rep(1, nrow(resp))
    }
    G <- length( unique( group ))
    # n.ik [ 1:TP, 1:I, 1:(K+1), 1:G ]
    n.ik <- array( 0, dim=c(TP,I,maxK, G ))
    for (gg in 1:G){
    ind.gg <- which( group==gg )
        for (kk in 1:(maxK)){
            dkk2 <- ( resp[ ind.gg, ]==(kk-1) ) * resp.ind[ ind.gg ] * pweights[ind.gg]
            # t( t(A) * B )=t(B) * A=crossprod(B,A)
            n.ik[,,kk,gg] <- crossprod( hwt[ind.gg,], dkk2 )
        }
    }
    return(n.ik)
}
#####################################

## File Name: noharm.sirt.R
## File Version: 0.940


########################################
# NOHARM implementation in R
########################################

#---------------
# NOHARM
# Normal-Ogive Harmonic Analysis Robust Method
#---------------
# The other function with linear and loglinear models of parameters
# is denoted as 'noharm.loglm.sirt'.
#---------------

noharm.sirt <- function(dat, pm=NULL, N=NULL, weights=NULL, Fval=NULL, Fpatt=NULL,
    Pval=NULL, Ppatt=NULL, Psival=NULL, Psipatt=NULL, dimensions=NULL,
    lower=0, upper=1, wgtm=NULL, pos.loading=FALSE, pos.variance=FALSE,
    pos.residcorr=FALSE, maxiter=1000, conv=1e-6, optimizer="nlminb",
    par_lower=NULL, reliability=FALSE, ... )
{
    v1 <- s1 <- Sys.time()
    CALL <- match.call()
    #*** data preprocessing
    if (missing(dat)){
        input_pm <- TRUE
        dat <- NULL
    } else {
        input_pm <- FALSE
    }

    res <- noharm_sirt_preproc( dat=dat, pm=pm, N=N, weights=weights, Fpatt=Fpatt,
                Fval=Fval, Ppatt=Ppatt, Pval=Pval, Psipatt=Psipatt, Psival=Psival,
                wgtm=wgtm, dimensions=dimensions, pos.loading=pos.loading,
                pos.variance=pos.variance, pos.residcorr=pos.residcorr,
                input_pm=input_pm, lower=lower, upper=upper)
    ss <- res$ss
    pm0 <- res$pm0
    sumwgtm <- res$sumwgtm
    model.type <- res$model.type
    modtype <- res$modtype
    N <- res$N
    I <- res$I
    dat0 <- res$dat0
    Fpatt <- res$Fpatt
    Fval <- res$Fval
    Ppatt <- res$Ppatt
    Pval <- res$Pval
    Psipatt <- res$Psipatt
    Psival <- res$Psival
    D <- res$D
    pm <- res$pm
    dat <- res$dat
    dat.resp <- res$dat.resp
    wgtm <- res$wgtm
    F_dimnames <- res$F_dimnames
    items <- res$items
    wgtm.default <- res$wgtm.default
    parm_table <- res$parm_table
    npar <- res$npar
    parm_index <- res$parm_index
    lower <- res$lower
    upper <- res$upper

    modesttype <- 1   # NOHARM estimation

    #----
    # compute betaj
    if (modesttype==1){
        betaj <- - stats::qnorm( ( diag(pm) - lower ) / ( upper - lower ) )
    }
    # compute bjk coefficients
    # include lower and upper asymptotes here

    b0 <- lower + (upper-lower) * stats::pnorm(-betaj)
    b1 <- (upper-lower) * sirt_dnorm(betaj)
    b2 <- betaj * b1 / sqrt(2)
    b3 <- ( betaj^2 -1 ) * b1 / sqrt(6)

    # create fixed cofficients
    b0_jk <- b0.jk <- noharm_sirt_outer_coefs(x=b0)
    b1_jk <- b1.jk <- noharm_sirt_outer_coefs(x=b1)
    b2_jk <- b2.jk <- noharm_sirt_outer_coefs(x=b2)
    b3_jk <- b3.jk <- noharm_sirt_outer_coefs(x=b3)

    par0 <- noharm_sirt_partable_extract_par(parm_table=parm_table)
    parm_table_free <- parm_table[ parm_table$fixed==0, ]

    args_optim <- list(parm_table=parm_table, parm_index=parm_index, I=I, D=D,
                    b0.jk=b0.jk, b1.jk=b1.jk, b2.jk=b2.jk, b3.jk=b3.jk,
                    pm=pm, wgtm=wgtm, use_rcpp=TRUE)
    optim_fn <- function(x){
        args_optim$x <- x
        val <- do.call(what=noharm_sirt_optim_function, args=args_optim)
        return(val)
    }
    optim_gr <- function(x){
        args_optim$x <- x
        grad <- do.call(what=noharm_sirt_optim_gradient, args=args_optim)
        return(grad)
    }

    v2 <- Sys.time()
    time <- list( time_pre=v2-v1)

    #* optimization
    if (is.null(par_lower)){
        par_lower <- noharm_sirt_partable_extract_par(parm_table=parm_table, col='lower')
    }
    res <- sirt_optimizer(optimizer=optimizer, par=par0, fn=optim_fn,
                grad=optim_gr, lower=par_lower, method='L-BFGS-B', hessian=FALSE, ...)
    res_opt <- res
    par0 <- res$par
    v3 <- Sys.time()
    time$time_opt <- v3-v2

    #* include in parameter table
    parm_table <- noharm_sirt_partable_include_par(par=par0, parm_table=parm_table)
    Fval <- Fmat <- noharm_sirt_create_parameter_matrices('F', parm_table=parm_table,
                                parm_index=parm_index)
    Pval <- Pmat <- noharm_sirt_create_parameter_matrices('P', parm_table=parm_table,
                                parm_index=parm_index)
    Psival <- Psimat <- noharm_sirt_create_parameter_matrices('Psi',
                                parm_table=parm_table, parm_index=parm_index)
    estpars <- list( estF=sum(Fpatt>0), estP=sum(Ppatt>0), estPsi=sum(Psipatt>0) )

    #--- include colnames in estimated matrices
    colnames(Fval) <- F_dimnames
    rownames(Fval) <- items
    rownames(Pval) <- colnames(Pval) <- F_dimnames
    rownames(Psival) <- colnames(Psival) <- items

    #* compute final constants
    res <- noharm_sirt_compute_final_constants( Fval=Fval, Pval=Pval, betaj=betaj,
                        modesttype=modesttype )
    f0 <- res$f0
    uqn <- res$uqn
    loadingsF <- res$loadingsF

    #* compute residuals
    res <- noharm_sirt_est_residuals( Fval=Fval, Pval=Pval, Fpatt=Fpatt, Ppatt=Ppatt, I=I,
                D=D, b0.jk=b0.jk, b1.jk=b1.jk, b2.jk=b2.jk, b3.jk=b3.jk, wgtm=wgtm, pm=pm,
                Psival=Psival, Psipatt=Psipatt, modesttype=modesttype )
    residuals <- res$residuals
    rmsr <- res$rmsr
    tanaka <- res$tanaka

    #**** arrange output list
    res <- list( tanaka=tanaka, rmsr=rmsr, N.itempair=ss, pm=pm0, wgtm=wgtm,
                sumwgtm=sumwgtm, lower=lower, upper=upper, residuals=residuals,
                final.constants=f0, factor.cor=Pval, thresholds=-betaj,
                uniquenesses=uqn, loadings=loadingsF, loadings.theta=Fval,
                residcorr=Psival, model.type=model.type, modtype=modtype,
                Nobs=N, Nitems=I, Fpatt=Fpatt, Ppatt=Ppatt, Psipatt=Psipatt,
                dat=dat0, systime=Sys.time(), dimensions=D, display.fit=5,
                res_opt=res_opt, parm_table=parm_table, b0=b0, b1=b1, b2=b2, b3=b3,
                par_lower=par_lower)

    #*** number of estimated parameters
    Nestpars <- noharm_sirt_number_estimated_parameters( I=I, Fpatt=Fpatt,
                    Ppatt=Ppatt, Psipatt=Psipatt )
    res$Nestpars <- Nestpars

    #*** compute chi square statistics
    res <- noharm_sirt_compute_chi_square_statistics( res=res,
                residuals=residuals, pm=pm, N=N, I=I, sumwgtm=sumwgtm,
                modesttype=modesttype, Nestpars=Nestpars )

    #*** compute Green-Yang reliability
    v1 <- 1 * ( ( wgtm - diag(wgtm) ) > 0 )
    # ensure positive definiteness
    res$omega.rel <- NA
    L0 <- sqrt( diag(Pval) )
    N1 <- length(L0)
    L1inv <- L1 <- matrix( 0, nrow=N1, ncol=N1 )
    rownames(L1) <- rownames(L1inv) <- colnames(L1) <- colnames(L1inv) <- rownames(Pval)
    diag(L1) <- L0
    diag(L1inv) <- 1 / L0
    Fval2 <- Fval %*% L1
    Pval2 <- L1inv %*% Pval %*% L1inv
    dj <- sqrt( diag( Fval2 %*% Pval2 %*% t(Fval2) ) )
    ej <- sqrt( 1+dj^2 )
    Fval2 <- Fval2 / ej
    standardized.solution <- list( Fval=Fval2, Pval=Pval2 )
    if (reliability){
        res0 <- reliability.nonlinearSEM(facloadings=Fval2,
            thresh=res$thresholds, resid.cov=res$residcorr, cor.factors=Pval2 )
        res$omega.rel <- res0$omega.rel
    }
    if ( sum(v1) + I < I^2 ){
        res$omega.rel <- NA
    }

    #*** rotated solution for EFA
    if (model.type=='EFA'){
        res <- noharm_sirt_efa_rotated_solution( res=res, items=items,
                    F_dimnames=F_dimnames )
    }
    if (modesttype==2){
        res$tetracor <- pm
    }

    #***----- more output
    res$estpars <- estpars
    res$modesttype <- modesttype
    res$guesses <- res$lower
    res$wgtm.default <- wgtm.default
    res$standardized.solution <- standardized.solution
    s2 <- Sys.time()
    res$s1 <- s1
    res$s2 <- s2
    res$CALL <- CALL
    v4 <- Sys.time()
    time$time_post <- v4-v3
    res$time <- time
    class(res) <- 'noharm.sirt'
    return(res)
}

## File Name: prob.guttman.R
## File Version: 1.17



####################################################
# probabilistic Guttman analysis
prob.guttman <- function( dat, pid=NULL, guess.equal=FALSE,
        slip.equal=FALSE, itemlevel=NULL, conv1=.001,
        glob.conv=.001, mmliter=500 ){
    # data prep
    dat <- as.matrix(dat)
    I <- ncol(dat)
    dat2.resp <- 1 - is.na( dat )
    dat2 <- dat
    dat2[ is.na(dat2) ] <- 0
    if ( is.null( pid) ){ pid <- seq(1,nrow(dat)) }
    # p values
    p <- colMeans( dat, na.rm=TRUE )
    # items and p value levels
    itemtable <- data.frame( "index"=seq(1,ncol(dat)),
                    "item"=colnames(dat), "p"=p )
    if ( is.null( itemlevel )){
        itemtable <- itemtable[ order( itemtable$p, decreasing=TRUE ), ]
        itemtable$level <- match( itemtable$p, unique(itemtable$p ) )
            } else { itemtable$level <- itemlevel }
    itemtable <- itemtable[ order( itemtable$index ), ]
    theta.k <- c( 0, sort(unique(itemtable$level ) ))
    # design matrices for items
    itemdes <- matrix(0, nrow=length(theta.k), ncol=ncol(dat))
    colnames(itemdes) <- colnames(dat)
    rownames(itemdes) <- paste( "theta_level_", theta.k, sep="")
    items <- itemtable$item
    for (ii in items){
        itemdes[,ii] <- 1 * ( theta.k >=itemtable[ itemtable$item==ii, "level" ] )
                    }
    # init slipping and guessing parameters
    guess <- slip <- rep(.2,I)

    # prior distribution pi(k)
    LK <- length(theta.k)
    pi.k <- rep( 1 /LK, LK )
    iter <- 0
    dev0 <- dev.change <- 1000
    par.change <- 1000
    cat("---------------------------------------------------------------------------------------------------------- \n")
    cat("Probabilistic Guttman Model \n")
    cat("---------------------------------------------------------------------------------------------------------- \n")
    while ( ( dev.change > glob.conv | par.change > conv1 ) & ( iter < mmliter ) ){
        #*****
        # E step Guttman scaling
        res <- .e.step.guttman( dat2, dat2.resp, theta.k, pi.k, I, guess, slip, itemdes )

        # M step Guttman scaling
        pi.k <- ( res$n.k / sum(res$n.k ) )[,1]

        guess.old <- guess
        slip.old <- slip
        # update guessing parameters

        c1 <- colSums( t( (res$r.jk)[,,1]  ) * ( 1 - itemdes ) )
        c2 <- colSums( t( (res$n.jk)[,,1]  ) * ( 1 - itemdes ) )
        if ( ! guess.equal ){    guess <- c1/c2 } else {
            guess <-  rep( sum(c1) / sum(c2), I )
                        }
        # update slipping parameters
        c1 <- colSums( t( (res$r.jk)[,,1]  ) * (  itemdes ) )
        c2 <- colSums( t( (res$n.jk)[,,1]  ) * ( itemdes ) )
#        slip <- 1- c1/c2
        if ( ! slip.equal ){    slip <- 1 - c1/c2 } else {
            slip <-  rep( 1 - sum(c1) / sum(c2), I )
                        }
        par.change <- max( abs( c( guess - guess.old, slip - slip.old) ) )
        dev <- -2*res$ll
        dev.change <- abs( dev - dev0 )
        cat( paste( "Iteration ", iter + 1,  " |  Deviance=", round( dev, 4 ),
                                    if (iter > 0 ){ " | Deviance change=" } else {""},
                                    if( iter>0){round( - dev + dev0, 6 )} else { ""}," |",sep=""))
        cat( paste( " Maximum parameter change=",
                                    round( max(abs( par.change )), 6 ),  " \n"   )  )
        utils::flush.console()
        dev0 <- dev
        iter <- iter + 1
            }

    # MAP
    map <- theta.k[ whichrowMaxs( res$f.qk.yi )$arg ]
    # MLE
    mle <- theta.k[ whichrowMaxs( res$f.yi.qk )$arg ]

    # person data frame
    person <- data.frame( "pid"=pid,
                "score"=rowSums( dat, na.rm=T), "max"=rowSums(dat2.resp),
                "MLE"=mle, "MAP"=map )
    # Information criteria
    if (guess.equal){ np <- 1 } else {
        np <- length(guess) }
    if (slip.equal){ np <- np + 1 } else { np <- np + length(slip) }
    np <- np + length(theta.k) - 1
    ic <- data.frame( "np"=np,"n"=nrow(dat),
        "deviance"=dev )
    # AIC
    ic$AIC <- dev + 2*ic$np
    # BIC
    ic$BIC <- dev + ( log(ic$n) )*ic$np
    # CAIC (conistent AIC)
    ic$CAIC <- dev + ( log(ic$n) + 1 )*ic$np
    # corrected AIC
    ic$AICc <- ic$AIC + 2*ic$np * ( ic$np + 1 ) / ( ic$n - ic$np - 1 )
    # item data frame
    item <- itemtable
    item$guess <- guess
    item$slip <- slip
    item$discrim <- 1-slip-guess
    #**********
    # trait distribution
    trait <- data.frame( "level"=theta.k, "prob"=pi.k )
    trait$freq.MLE <- sapply( theta.k, FUN=function(tt){ mean( person$MLE==tt ) } )
    trait$freq.MAP <- sapply( theta.k, FUN=function(tt){ mean( person$MAP==tt ) } )
    rownames(trait) <- rownames(itemdes)
    # collect results
    res <- list( "person"=person, "item"=item, "theta.k"=theta.k,
            "trait"=trait, "pi.k"=pi.k, "f.yi.qk"=res$f.yi.qk,
            "f.qk.yi"=res$f.qk.yi, "probs"=res$probs,
            "ic"=ic, "G"=1, "deviance"=dev, "iter"=iter,
            "itemdesign"=itemdes )
    class(res) <- "prob.guttman"
    return(res)
        }
###########################################################################




#*******************************************************
# Summary for prob.guttman object                         *
summary.prob.guttman <- function( object, ... ){
    # object      ... object from rasch.mml                #

    cat("---------------------------------------------------------------------------------------------------------- \n")
        d1 <- utils::packageDescription("sirt")
        cat( paste( d1$Package, " ", d1$Version, " (", d1$Date, ")", sep=""), "\n\n" )
#        cat( "Date of Analysis:", paste( object$s2 ), "\n" )
#        cat("Computation time:", print(object$s2 - object$s1), "\n\n")
    cat("Probabilistic Guttman Model \n")
    cat("---------------------------------------------------------------------------------------------------------- \n")
    cat( "Number of iterations=", object$iter, "\n" )
    cat( "Deviance=", round( object$deviance, 2 ), "\n" )
    cat( "Number of persons=", object$ic$n, "\n" )
    cat( "Number of estimated parameters=", object$ic$np, "\n" )
    cat( "AIC=", round( object$ic$AIC, 2 ), " | penalty=",
                round( object$ic$AIC - object$ic$deviance,2 ),
            "   | AIC=-2*LL + 2*p  \n" )
    cat( "AICc=", round( object$ic$AICc, 2 )," | penalty=", round( object$ic$AICc - object$ic$deviance,2 ) )
        cat("    | AICc=-2*LL + 2*p + 2*p*(p+1)/(n-p-1)  (bias corrected AIC)\n" )
    cat( "BIC=", round( object$ic$BIC, 2 ), " | penalty=", round( object$ic$BIC - object$ic$deviance,2 ),
            "   | BIC=-2*LL + log(n)*p  \n" )
    cat( "CAIC=", round( object$ic$CAIC, 2 )," | penalty=", round( object$ic$CAIC - object$ic$deviance,2 ) )
        cat("   | CAIC=-2*LL + [log(n)+1]*p  (consistent AIC)\n\n" )

    #cat( "\n")
    cat("---------------------------------------------------------------------------------------------------------- \n")
    cat("Trait Distribution \n")
    obji <- object$trait
    roundvars <- c("prob", "freq.MLE", "freq.MAP" )
    for (vv in roundvars ){
        obji[,vv] <- round( obji[,vv], 3 )
                }
#    rownames(obji) <- NULL
    print( obji )


    cat( "\n")
    cat("---------------------------------------------------------------------------------------------------------- \n")
    cat("Item Parameter \n")
    obji <- object$item
    roundvars <- c("p","guess","slip","discrim" )
    for (vv in roundvars ){
        obji[,vv] <- round( obji[,vv], 3 )
                }
    rownames(obji) <- NULL
    print( obji )
                }
#*******************************************************





#*************************************************************************************
.e.step.guttman <- function( dat2, dat2.resp, theta.k, pi.k, I, guess, slip, itemdes ){
    #...................................
    # probabilities of correct item at theta_k
    guessM <- matrix( guess, byrow=T, nrow=length(theta.k), ncol=I )
    slipM <- matrix( slip, byrow=T, nrow=length(theta.k), ncol=I )
    pjk <- itemdes * ( 1 - slipM ) +  (  1 - itemdes ) * guessM
    TP <- dim(pjk)[1]
#    pjk <- t(pjk)
        #***
        pjkt <- t(pjk)
        pjkL <- array( NA, dim=c( I, 2, TP  ) )
        pjkL[,1,] <- 1 - pjkt
        pjkL[,2,] <- pjkt
        probsM <- matrix( aperm( pjkL, c(2,1,3) ), nrow=I*2, ncol=TP )
        f.yi.qk <- mml_calc_like( dat2=dat2, dat2resp=dat2.resp,
                    probs=probsM )$fyiqk
    #******
    f.qk.yi <- 0 * f.yi.qk
    pi.k <- matrix( pi.k, ncol=1 )
    f.qk.yi <- f.yi.qk * outer( rep( 1, nrow(dat2) ), pi.k[,1] )
    f.qk.yi <- f.qk.yi / rowSums( f.qk.yi )

    # expected counts at theta.k
    G <- 1
    n.k <- matrix( 0, nrow=length(theta.k), ncol=G )
    r.jk <- n.jk <- array( 0, dim=c( ncol(dat2), length(theta.k), G) )
    ll <- rep(0,G)
    group <- rep(1,nrow(dat2))
    dat1 <- matrix( 1, nrow=nrow(dat2), ncol=2 )
    for (gg in 1:G){
        ind.gg <- which( group==gg )
        res <- mml_raschtype_counts( dat2=dat2[ind.gg,], dat2resp=dat2.resp[ind.gg,],
                    dat1=dat1[ind.gg,2], fqkyi=f.qk.yi[ind.gg,],
                    pik=pi.k[,gg], fyiqk=f.yi.qk[ind.gg,]  )
        n.k[,gg] <- res$nk
        n.jk[,,gg] <- res$njk
        r.jk[,,gg] <- res$rjk
        ll[gg] <- res$ll
        }
    res <- list( "n.k"=n.k, "n.jk"=n.jk, "r.jk"=r.jk,
            "f.qk.yi"=f.qk.yi, "pjk"=pjk,
            "f.yi.qk"=f.yi.qk, "ll"=sum(ll), "probs"=pjkL )
    return(res)
    }
#*************************************************************************************

## File Name: rm_sdt_postproc_ic.R
## File Version: 0.12

rm_sdt_postproc_ic <- function(dev, dat2, VV, RR, est.mean, est.sigma,
    partable_item, partable_rater, skillspace, theta.k)
{
    ic <- list( deviance=dev, n=nrow(dat2) )
    ic$VV <- VV
    ic$RR <- RR
    #****
    # skill parameters
    ic$np.skill <- 0
    if ( skillspace=="normal" ){
        ic$np.skill <- est.sigma + est.mean
    }
    if ( skillspace=="discrete" ){
        ic$np.skill <- length(theta.k) - 1
    }
    #*****
    # item parameters
    ic$np.item <- 0
    ic$np.item <- ic$np.item + sum( partable_item[ partable_item$type=="tau", "est"] )
    ic$np.item <- ic$np.item + sum( partable_item[ partable_item$type=="a", "est"] )

    #*****
    # rater parameters
    ic$np.rater <- 0
    ic$np.rater <- ic$np.rater + sum( partable_rater[ partable_rater$type=="c", "est"] )
    ic$np.rater <- ic$np.rater + sum( partable_rater[ partable_rater$type=="d", "est"] )
    ic$np <- ic$np.skill + ic$np.item + ic$np.rater

    #-- compute information criteria
    ic <- rm_ic_criteria(ic=ic)
    return(ic)
}

## File Name: lsem_fitsem.R
## File Version: 0.661

lsem_fitsem <- function( dat, weights, lavfit, fit_measures, NF, G, moderator.grid,
                verbose, pars, standardized, variables_model, sufficient_statistics,
                lavaan_fct, lavmodel, use_lavaan_survey=TRUE, pseudo_weights=0,
                est_joint=FALSE, par_invariant=NULL, par_linear=NULL, par_quadratic=NULL,
                partable_joint=NULL, pw_linear=1, pw_quadratic=1,
                se="standard", moderator_variable=NULL,
                loc_linear_smooth=NULL, pd=FALSE, has_meanstructure=FALSE,
                est_DIF=FALSE, residualized_intercepts=NULL, residualize=TRUE,
                is_imputed=FALSE, Nimp=0, moderator=NULL, ... )
{
    parameters <- NULL
    fits <- NULL
    fitstats_joint <- NULL
    sample_stats <- NULL
    data_joint <- NULL
    survey.fit <- lavfit
    pars0 <- pars
    lavaan_est_fun <- lsem_define_lavaan_est_fun(lavaan_fct=lavaan_fct)
    est_separate <- ! est_joint
    dif_effects <- NULL

    #- verbose
    pr <- lsem_fitsem_verbose_start(G=G, verbose=verbose)

    #- estimate model meanstructure
    is_meanstructure <- lavfit@Model@meanstructure

    #- sufficient statistics
    if (sufficient_statistics){

        sample_stats <- lsem_fitsem_compute_sufficient_statistics(G=G, dat=dat,
                            variables_model=variables_model, weights=weights,
                            moderator_variable=moderator_variable,
                            loc_linear_smooth=loc_linear_smooth,
                            moderator.grid=moderator.grid,
                            pd=pd, residualized_intercepts=residualized_intercepts,
                            has_meanstructure=has_meanstructure,
                            residualize=residualize, is_imputed=is_imputed, Nimp=Nimp,
                            moderator=moderator)

    }
    if (est_joint & (! sufficient_statistics)){
        N <- nrow(dat)
        ind <- rep(1L:N, G)
        data_joint <- dat[ind, ]
        data_joint$group__ <- rep(1L:G, each=N)
        data_joint$weight <- as.vector(weights)
        data_joint$case__ <- ind
    }

    #-- joint estimation
    if (est_joint){
        #- parameter table
        if ( is.null(partable_joint) ){
            partable_joint <- lsem_fitsem_joint_estimation_partable(lavfit=lavfit, G=G,
                                par_invariant=par_invariant, par_linear=par_linear,
                                par_quadratic=par_quadratic, pw_linear=pw_linear,
                                pw_quadratic=pw_quadratic)
        }

        #- fit model
        fit_args <- list( partable_joint=partable_joint,
                            is_meanstructure=is_meanstructure, data_joint=data_joint,
                            sample_stats=sample_stats, lavaan_est_fun=lavaan_est_fun,
                            verbose=verbose, se=se,
                            sufficient_statistics=sufficient_statistics,
                            G=G, pseudo_weights=pseudo_weights, ... )
        survey.fit <- do.call(what=lsem_fitsem_joint_estimation, args=fit_args)
        partable_joint <- lavaan::parameterTable(object=survey.fit)
        fitstats_joint <- lsem_lavaan_fit_measures(object=survey.fit,
                                fit_measures=fit_measures)

        #*** assessment of DIF effects
        partable_dif <- partable_joint
        partable_dif[ partable_dif$free > 0, 'free'] <- 0
        partable_dif[ partable_dif$par %in% par_invariant, 'free'] <- 1
        ind1 <- intersect( grep('=~', partable_dif$par), which( partable_joint$free==0) )
        ind2 <- NULL
        if (has_meanstructure){
            ind2 <- intersect( grep('~1', partable_dif$par),
                                    which( partable_joint$free==0) )
        }
        ind3 <- intersect( grep('~~', partable_dif$par), which( partable_joint$free==0) )
        partable_dif[ c(ind1,ind2, ind3), 'free'] <- 1

        partable_dif <- partable_dif[ - grep( '_con', partable_dif$par), ]
        partable_dif$start <- partable_dif$est
        partable_dif$free <- cumsum(partable_dif$free)*(partable_dif$free>0)
        if ( sum(partable_dif$free)==0 ){
            est_DIF <- FALSE
        }
        if ( est_DIF ){
            fit_args$partable_joint <- partable_dif
            fit_dif <- do.call(what=lsem_fitsem_joint_estimation, args=fit_args)
            dif_effects <- lavaan::parameterTable(object=fit_dif)
            dif_effects <- dif_effects[ order(dif_effects$par), ]
            dif_effects$DIF <- 1*( dif_effects$free > 0 )
        }

        #--- output joint estimation
        fitstats_joint <- data.frame(stat=names(fitstats_joint), value=fitstats_joint)
    }

    #-- separate estimation: loop over groups
    for (gg in 1L:G){

        if (! is_imputed){
            dat$weight <- weights[,gg]
        } else {
            dat$weight <- weights[[1]][,gg]
        }

        #***** fit the model using weighted data
        if (( ! sufficient_statistics) & ( est_separate)){
            if (use_lavaan_survey){   # fit in lavaan.survey
                survey.fit <- lsem_fitsem_raw_data_lavaan_survey(dat=dat,
                                    lavmodel=lavmodel, lavfit=lavfit)
            }
            if (! use_lavaan_survey){  # fit in lavaan
                survey.fit <- lsem_fitsem_raw_data_lavaan(dat=dat,
                                    pseudo_weights=pseudo_weights,
                                    survey.fit=survey.fit, lavaan_est_fun=lavaan_est_fun,
                                    se=se, ...)
            }
        }
        #***** fit the model using sufficient statistics
        if (sufficient_statistics & est_separate){
            survey.fit <- lsem_fitsem_sufficient_statistics_lavaan( gg=gg,
                                lavmodel=lavmodel, lavaan_est_fun=lavaan_est_fun,
                                survey.fit=survey.fit, sample_stats=sample_stats,
                                is_meanstructure=is_meanstructure, se=se, ... )
        }

        dfr.gg <- pars <- sirt_import_lavaan_parameterEstimates(object=survey.fit)

        if (standardized){
            sol <- sirt_import_lavaan_standardizedSolution(object=survey.fit)
            colnames(sol)[ which( colnames(sol)=='est.std' ) ] <- 'est'
            sol$lhs <- paste0( 'std__', sol$lhs)
            pars <- sirt_rbind_fill( x=pars, y=sol )
            dfr.gg <- pars
        }

        pars <- sirt_lavaan_partable_parnames(partable=pars)
        NP <- length(pars0)
        if (est_separate){
            ind <- match(pars0, pars)
            grid_index <- gg
            parindex <- 1L:NP
            par_gg <- pars0
        } else {
            ind <- NULL
            par_gg <- NULL
            for (pp in 1L:NP){
                ind_pp <- which(pars==pars0[pp])
                npp <- length(ind_pp)
                ind <- c( ind, ind_pp)
                par_gg <- c(par_gg, rep(pars0[pp], npp) )
            }
            grid_index <- dfr.gg$group[ ind ]
            parindex <- rep(1L:NP, each=G)
            dfr.gg$block <- dfr.gg$group <- NULL
        }
        dfr.gg <- dfr.gg[ ind, ]
        dfr.gg <- data.frame(grid_index=grid_index, moderator=moderator.grid[grid_index],
                            par=par_gg, parindex=parindex, dfr.gg )
        if (est_separate){
            est_fit <- lsem_lavaan_fit_measures(object=survey.fit,
                                fit_measures=fit_measures)
            dfr.gg0 <- data.frame(grid_index=gg, moderator=moderator.grid[gg],
                              par=fit_measures, parindex=NP+1L:NF, est=est_fit, op='fit')
            vars <- setdiff( colnames(dfr.gg), colnames(dfr.gg0) )
            for (vv in vars){ dfr.gg0[,vv] <- NA }
            dfr.gg <- rbind( dfr.gg, dfr.gg0[, colnames(dfr.gg) ] )
        }
        parameters <- rbind( parameters, dfr.gg )

        if (est_DIF){
            dif1 <- dif_effects[ dif_effects$DIF==1, ]
            pars1 <- intersect(paste(dif1$par), paste(parameters$par))
            dif1 <- dif1[ dif1$par %in% pars1, ]
            NP <- max(parameters$parindex)
            parameters1 <- parameters[ parameters$par %in% pars1, ]
            parameters1$est <- dif1$est
            parameters1$par <- paste0('dif__', parameters1$par)
            parameters1$parindex <- match( parameters1$parindex,
                                            unique( parameters1$parindex ) ) + NP
            parameters <- rbind( parameters, parameters1)
        }

        #- verbose
        res <- lsem_fitsem_verbose_progress(gg=gg, G=G, pr=pr, verbose=verbose)
        if (est_joint){ break }
    }
    if ( sum(colnames(parameters)=='se')==0){
        parameters$se <- NA
    }

    res <- lsem_fitsem_verbose_progress(gg=G+1, G=G, pr=pr, verbose=verbose)
    parameters <- parameters[ order(parameters$parindex), ]
    rownames(parameters) <- paste0( parameters$par, '__', parameters$grid_index )

    #--- OUTPUT
    res <- list( parameters=parameters, is_meanstructure=is_meanstructure,
                    partable_joint=partable_joint, fitstats_joint=fitstats_joint,
                    sample_stats=sample_stats, dif_effects=dif_effects)
    return(res)
}

lsem.fitsem <- lsem_fitsem

## File Name: lsem_local_weights.R
## File Version: 0.222

lsem_local_weights <- function(data.mod, moderator.grid, h,
        sampling_weights=NULL, bw=NULL, kernel="gaussian",
        is_imputed=FALSE, Nimp=0, data=NULL, moderator=NULL)
{
    eps <- 1E-8
    N <- length(data.mod)
    no_sampling_weights <- FALSE
    if (is.null(sampling_weights)){
        sampling_weights <- rep(1,N)
        no_sampling_weights <- TRUE
    }
    bw0 <- bw

    # define list for imputed datasets
    Nimp <- max(1, Nimp)
    res0 <- list()

    for (ii in 1L:Nimp){
        if (is_imputed){
            data.mod <- ( data[[ii]] )[, moderator ]
        }
        bw <- bw0

        sampling_weights <- sampling_weights / sum(sampling_weights) * N
        # select nearest neighbor in moderator group for calculating residuals
        G <- length(moderator.grid)
        modgrid_index <- rep(1,N)
        for (gg in 2L:G){
            modgrid_index <- ifelse( abs( data.mod - moderator.grid[ modgrid_index ] ) <
                        abs( data.mod - moderator.grid[ gg ] ),
                        modgrid_index, gg )
        }
        res0$modgrid_index[[ii]] <- modgrid_index

        # compute weights for every grid point gg
        weights <- matrix( NA, nrow=N, ncol=G )
        sd.moderator <- TAM::weighted_sd(x=data.mod, w=sampling_weights)
        m.moderator <- TAM::weighted_mean(x=data.mod, w=sampling_weights)
        bw_basis <- sd.moderator * N^(-1/5)
        if (is.null(bw)){
            bw <- h * bw_basis
        } else {
            h <- bw / bw_basis
        }
        weights1 <- sampling_weights / sum(sampling_weights)

        moderator.density <- stats::density( data.mod, weights=weights1,
                                from=min(moderator.grid), to=max(moderator.grid ), n=G )$y
        moderator.density <- data.frame( moderator=moderator.grid,
                                wgt=moderator.density / sum(moderator.density) )
        for (gg in 1L:G){
            xgg <- moderator.grid[gg]
            wgt <- lsem_kernel_weights(x=data.mod, x0=xgg, bw=bw, kernel=kernel)
            weights[,gg] <- ifelse( wgt < eps, eps, wgt )
        }

        sampling_weights <- sampling_weights / sum(sampling_weights) * N
        weights <- weights * sampling_weights

        res0$weights[[ii]] <- weights
        res0$moderator.density[[ii]] <- moderator.density
        res0$m.moderator[[ii]] <- m.moderator
        res0$sd.moderator[[ii]] <- sd.moderator
        res0$bw[[ii]] <- bw

    } # end ii
    if (is_imputed){
        m.moderator <- lsem_aggregate_statistics(x=res0$m.moderator)
        sd.moderator <- lsem_aggregate_statistics(x=res0$sd.moderator)
        moderator.density <- lsem_aggregate_statistics(x=res0$moderator.density)
        bw <- lsem_aggregate_statistics(x=res0$bw)
        weights <- res0$weights
        modgrid_index <- res0$modgrid_index
    }

    #--- output
    res <- list(weights=weights, N=N, G=G, modgrid_index=modgrid_index,
                m.moderator=m.moderator, sd.moderator=sd.moderator, bw=bw, h=h,
                moderator.density=moderator.density,
                sampling_weights=sampling_weights,
                no_sampling_weights=no_sampling_weights)
    return(res)
}

## File Name: equating.rasch.R
## File Version: 0.247


#---- Equating (linking) in the Rasch model
equating.rasch <- function( x, y, theta=seq( -4, 4, len=100),
        alpha1=0, alpha2=0 )
{
    # Data preparation
    x[,1] <- gsub( ' ', '', paste( x[,1] ) )
    y[,1] <- gsub( ' ', '', paste( y[,1] ) )
    b.xy <- data.frame( merge( x, y, by.x=1, by.y=1 ) )
    colnames(b.xy) <- c('item', 'Itempar.Gr1', 'Itempar.Gr2' )
    b.xy <- stats::na.omit( b.xy )
    # mean-mean method
    B.mm <- mean(b.xy[,3]) - mean(b.xy[,2])
    g1 <- .prob.raschtype.genlogis( theta=theta, b=b.xy[,2], alpha1=0, alpha2=0 )
    opt_interval <- 10*c(-1,1)
    #-- Haebara function
    ha <- function(B){
                fct1 <- .prob.raschtype.genlogis( theta=theta, b=b.xy[,2], alpha1=alpha1,
                                    alpha2=alpha2 )
                fct2 <- .prob.raschtype.genlogis( theta=theta, b=b.xy[,3] - B,
                                    alpha1=alpha1, alpha2=alpha2 )
                sum( (fct1 - fct2)^2 )
            }
    B.ha <- stats::optimize( f=ha, interval=opt_interval )$minimum
    # Stocking and Lord Approach
    sl <- function(B){
                fct1 <- .prob.raschtype.genlogis( theta=theta, b=b.xy[,2],
                                alpha1=alpha1, alpha2=alpha2 )
                fct2 <- .prob.raschtype.genlogis( theta=theta, b=b.xy[,3] - B,
                                alpha1=alpha1, alpha2=alpha2 )
                sum( (rowSums( fct1 - fct2 ) )^2 )
            }
    B.sl <- stats::optimize( f=sl, interval=opt_interval )$minimum
    # collect all parameter estimates
    B.est <- data.frame( B.mm, B.ha, B.sl )
    colnames(B.est) <- c('Mean.Mean', 'Haebara', 'Stocking.Lord')
    # Transformation of item parameters (according to Stocking-Lord)
    b.xy$TransfItempar.Gr1 <- b.xy[,2] + B.est[1,'Stocking.Lord']
    x[,2] <- x[,2] + B.est[1,'Stocking.Lord']
    # transformed parameters
    transf.par <- merge( x=x, y=y, by.x=1, by.y=1, all=TRUE )
    colnames(transf.par) <- c('item', 'TransfItempar.Gr1', 'Itempar.Gr2'  )
    transf.par <- transf.par[ order( paste(transf.par$item ) ), ]
    # calculate variance and linking error
    des <- data.frame( N.Items=nrow(b.xy),
                            SD=stats::sd( b.xy$TransfItempar.Gr1 - b.xy$Itempar.Gr2 ) )
    des$Var <- des$SD^2
    des$linkerror <- sqrt( des['SD']^2 / des['N.Items'] )[1,1]
    #--- output
    res <- list( B.est=B.est, descriptives=des, anchor=b.xy[, c(1,2,4,3)],
                        transf.par=transf.par )
    return(res)
}

## File Name: mgsem_eval_lp_penalty_matrix.R
## File Version: 0.084

mgsem_eval_lp_penalty_matrix <- function(x, fac, p, n, h, eps_approx,
    pen_type="lasso", a_scad=3.7)
{
    x1 <- x
    I1 <- length(x1)
    y <- matrix(x1, nrow=I1, ncol=I1)-sirt_matrix2(x=x1, nrow=I1)
    y <- mgsem_power_fun_differentiable_approx(x=y, p=p,
                            eps=eps_approx, deriv=FALSE, approx_method='lp')
    if (pen_type=='lasso'){
        val <- fac*y
    }
    if (pen_type=='scad'){
        val <- mgsem_scad_penalty(x=y, lambda=fac, a=a_scad)
    }
    res <- sum(n*val)
    return(res)
}

## File Name: mgsem_evaluate_penalties_evaluate_entry.R
## File Version: 0.189


mgsem_evaluate_penalties_evaluate_entry <- function(x, res, dd, index, partable,
            technical, h, p, eps_approx, prior_list, deriv=FALSE, difflp_info=NULL)
{
    x0 <- x
    x <- x0[index]

    if (deriv){
        ind_add <- index
        null_vec <- rep(0, length(x))
    } else {
        ind_add <- 1
        null_vec <- 0
    }

    # res[['pen_l2']] <- null_vec

    # fun_eval='none' produces a value of zero

    #*** priors
    if (technical$is_prior){
        pen_entry <- 'pen_prior'
        prior <- partable[dd,'prior']
        if (prior!='none'){
            fun_eval <- prior_list[[prior]]
            args_eval <- list()
            val <- mgsem_evaluate_penalties_evaluate_entry_fun_eval(x=x,
                        args_eval=args_eval, fun_eval=fun_eval, h=h, deriv=deriv)
            res[[pen_entry]][ind_add] <- res[[pen_entry]][ind_add] + val
        }
    }  # end is_prior

    # --- output
    return(res)
}

## File Name: mcmc_confint.R
## File Version: 0.121


#*** confidence interval
mcmc_confint <- function( mcmcobj, parm, level=.95, exclude="deviance" )
{
    mcmcobj <- mcmcobj[, ! ( colnames(mcmcobj) %in% exclude ) ]
    if ( ! missing(parm) ){
        mcmcobj <- mcmcobj[,parm]
    }
    q1 <- ( 1 - level ) / 2
    h1 <- apply( mcmcobj, 2, stats::quantile, q1 )
    q2 <- 1 - ( 1 - level ) / 2
    h2 <- apply( mcmcobj, 2, stats::quantile, q2 )
    res <- data.frame( h1, h2)
    colnames(res)[1] <- paste0( round( 100*q1,1 ), ' %')
    colnames(res)[2] <- paste0( round( 100*q2,1 ), ' %')
    rownames(res) <- colnames(mcmcobj)
    return(res)
}

## File Name: IRT.modelfit.sirt.R
## File Version: 0.21


###########################################################
# general model fit function for sirt objects
IRT.modelfit.sirt <- function( object, mod, ... )
{
    res <- modelfit.sirt(object)
    res$IRT.IC <- CDM::IRT.IC(object)
    res$objname <- mod
    class(res) <- paste0("IRT.modelfit.", class(object) )
    return(res)
}
###########################################################
# summary IRT.modelfit.xxx
summary.IRT.modelfit.sirt <- function( object, ... )
{
    class(object) <- "modelfit.sirt"
    summary(object)
}
###########################################################

###########################################################
# IRT.modelfit for objects of class rasch.mml
IRT.modelfit.rasch.mml <- function( object, ... )
{
    cl <- paste(match.call())[2]
    res <- IRT.modelfit.sirt( object, mod=cl )
    return(res)
}
summary.IRT.modelfit.rasch.mml <- summary.IRT.modelfit.sirt
###########################################################
# IRT.modelfit smirt
IRT.modelfit.smirt <- IRT.modelfit.rasch.mml
summary.IRT.modelfit.smirt <- summary.IRT.modelfit.sirt
###########################################################
# IRT.modelfit rasch.mirtlc
IRT.modelfit.rasch.mirtlc <- IRT.modelfit.rasch.mml
summary.IRT.modelfit.rasch.mirtlc <- summary.IRT.modelfit.sirt
###########################################################
# IRT.modelfit gom
IRT.modelfit.gom <- IRT.modelfit.rasch.mml
summary.IRT.modelfit.gom <- summary.IRT.modelfit.sirt
###########################################################


###########################################################
# summary IRT.modelfit2
summary.IRT.modelfit.sirt2 <- function( object, ... )
{
    class(object) <- "tam.modelfit"
    summary(object)
}
###########################################################

###########################################################
# modelfit rm.facets
IRT.modelfit.rm.facets <- function(object, ... )
{
    mod <- paste(match.call())[2]
    data <- object$procdata$dat2.NA
    probs <- object$probs
    theta.k <- object$theta.k
    f.qk.yi <- object$f.qk.yi
    res <- modelfit.cor.poly( data, probs, theta.k, f.qk.yi )
    res$IRT.IC <- CDM::IRT.IC(object)
    res$objname <- mod
    class(res) <- paste0("IRT.modelfit.", class(object) )
    return(res)
}
summary.IRT.modelfit.rm.facets <- summary.IRT.modelfit.sirt2
############################################################
# model fit rm.sdt
IRT.modelfit.rm.sdt <- IRT.modelfit.rm.facets
summary.IRT.modelfit.rm.sdt <- summary.IRT.modelfit.sirt2
##########################################################



###########################################################
# modelfit xxirt
IRT.modelfit.xxirt <- function(object, ... )
{
    mod <- paste(match.call())[2]
    data <- as.matrix( object$dat )
    probs <- object$probs_items
    theta.k <- as.matrix(object$Theta)
    f.qk.yi <- as.matrix(object$p.aj.xi)
    res <- modelfit.cor.poly( data, probs, theta.k, f.qk.yi )
    res$IRT.IC <- CDM::IRT.IC(object)
    res$objname <- mod
    class(res) <- paste0("IRT.modelfit.", class(object) )
    return(res)
}
summary.IRT.modelfit.xxirt <- summary.IRT.modelfit.sirt2
############################################################

## File Name: matrixfunctions_sirt.R
## File Version: 0.22

##########################################################################
# rowwise maximum and minimum function
#****************************
# rowMaxs function
rowMaxs.sirt <- function(matr){
    rowMaxsCPP_source( matr )
}
#*****************************
# rowMins function
rowMins.sirt <- function(matr){
    matr2 <- - matr
    res2 <- rowMaxs.sirt( matr2 )
    res <- list( "minval"=- res2$maxval,
                    "minind"=res2$maxind )
    return(res)
}
##########################################################################
# rowwise cumulative sum
rowCumsums.sirt <- function(matr){
    rowCumsums2_source( matr )
                    }

# The C code was posted by Romain Francois at
# http://lists.r-forge.r-project.org/pipermail/rcpp-devel/2010-October/001198.html



##########################################################################
# rowwise cumulative sum
colCumsums.sirt <- function(matr){
    t( rowCumsums.sirt( t(matr) ) )
                    }

##########################################################################
#****
# 'interval_index' searches an index when a frequency is exceeded
# -> used in plausible value imputation
rowIntervalIndex.sirt <- function(matr,rn){
    interval_index_C( matr, rn)
                    }

##########################################################################
# extract k smallest elements in a row of a matrix
rowKSmallest.sirt <- function( matr, K, break.ties=TRUE){
    M1 <- matr
    N1 <- dim(M1)[1] ; N2 <- dim(M1)[2]
    # generate random number matrix
    rM1 <- matrix( round( stats::runif( N1*N2 ) ), N1, N2 )
    if ( ! break.ties ){ rM1 <- 0*rM1 }
    # define integer matrix
    indexmatr <- matrix( 1:N2, N1, N2, byrow=TRUE )
    # apply function for extracting k smallest elements
    a1 <- rowKSmallest_C( matr, K, indexmatr, rM1)
    ## OUTPUT:
    ## return List::create(_["smallval"]=SMALLVAL,
    ##                      _["smallind"]=SMALLIND ) ;
    return(a1)
}
##########################################################################
# Ksmallest -> different implementation
rowKSmallest2.sirt <- function(matr, K ){
    Nmis <- nrow(matr)
    disty <- matr
    donors <- K
    indvec <- 1:Nmis
    M1 <- max(disty)+1
    smallval <- donor.ind <- matrix( 0, nrow=Nmis, ncol=donors )
    res1 <- rowMins.sirt(matr=disty)
    donor.ind[,1] <- res1$minind
    smallval[,1] <- res1$minval
    for (ii in 2:donors){
        disty[ cbind(indvec, donor.ind[,ii-1] ) ] <- M1
        res1 <- rowMins.sirt(matr=disty)
        donor.ind[,ii] <- res1$minind
        smallval[,ii] <- res1$minval
                        }
    res <- list( "smallval"=smallval, "smallind"=donor.ind )
    return(res)
    }

## File Name: rasch_jml_update_b.R
## File Version: 0.053



# update item difficulty estimation (Rasch model)
rasch_jml_update_b <- function( b, theta, freq.thetapattern, freq.dat.resp.thetapattern,
            constraints=NULL, conv=.0001, suffB, progress=progress, bsteps=4)
{
    b.change <- 1
    iter <- 0
    while( max( abs( b.change  ) ) > conv & ( iter < bsteps ) ){
        p.ia <- stats::plogis( theta,
                        matrix( b, nrow=length(theta), length(b), byrow=TRUE ) )
        deriv <- colSums( - freq.thetapattern * p.ia * ( 1- p.ia ) )
        diff <- suffB + colSums( freq.dat.resp.thetapattern *  p.ia  )
        b.change <-  diff / deriv
        if (! is.null(constraints)){
            b.change[ constraints[,1] ] <- 0
        }
        if (progress){
            cat('-')
            utils::flush.console()
        }
        b <- b - b.change
        iter <- iter + 1
    }
    return(b)
}


.update.b.rasch.jml2 <- rasch_jml_update_b

## File Name: noharm_sirt_optim_gradient_R_der_gamma_item_pair.R
## File Version: 0.192


noharm_sirt_optim_gradient_R_der_gamma_item_pair <- function(parm_table_free, Fmat,
    Pmat, Psimat, FP, npar, NH, I, gamma_val, grad_gamma_diag1, pm, b0.jk, b1.jk, b2.jk,
    b3.jk, wgtm, ii, jj )
{
    grad <- rep(0,npar)
    if (wgtm[ii,jj]>0){
        for (hh in 1L:NH){
            mat_hh <- parm_table_free[hh,'mat']
            par_index_hh <- parm_table_free[hh,'index']
            row <- parm_table_free[hh,'row']
            col <- parm_table_free[hh,'col']
            # F
            if (mat_hh=='F'){
                der <- 0
                if (row==ii){
                    der <- FP[jj,col]
                    grad[par_index_hh] <- grad[par_index_hh] + der
                }
                if (row==jj){
                    der <- FP[ii,col]
                    grad[par_index_hh] <- grad[par_index_hh] + der
                }
            }
            # P
            if (mat_hh=='P'){
                der <- 0
                if (row==col){
                    der <- Fmat[ii,col]*Fmat[jj,col]
                    grad[par_index_hh] <- grad[par_index_hh] + der
                } else {
                    der <- Fmat[ii,row]*Fmat[jj,col]
                    grad[par_index_hh] <- grad[par_index_hh] + der
                }
            }
            # Psi
            if (mat_hh=='Psi'){
                if (row==ii){
                    if (col==jj){
                        der <- 1
                        grad[par_index_hh] <- grad[par_index_hh] + der
                    }
                }
            }
        }    # end hh
        # grad_gamma_nondiag <- grad

        # g_ij / sqrt( (1+g_ii)*(1+g_jj) )
        gii <- (1+gamma_val[ii,ii])
        gjj <- (1+gamma_val[jj,jj])
        gij <- gamma_val[ii,jj]
        za <- sqrt(gii*gjj)
        val <- gij / za

        # grad <- rep(0, npar)
        # first term
        grad <- grad / za
        # second term
        grad <- grad - 0.5*val*grad_gamma_diag1[ii,]
        # third term
        grad <- grad - 0.5*val*grad_gamma_diag1[jj,]

        #-- discrepancy function
        x_ij <- val
        pm_exp <- b0.jk[ii,jj] + b1.jk[ii,jj]*x_ij + b2.jk[ii,jj]*x_ij^2 +
                        b3.jk[ii,jj]*x_ij^3
        pm_exp_der <- b1.jk[ii,jj] + 2*b2.jk[ii,jj]*x_ij + 3*b3.jk[ii,jj]*x_ij^2
        grad <- pm_exp_der*grad
        temp1 <- -2*wgtm[ii,jj] * ( pm[ii,jj] - pm_exp )
        grad <- temp1*grad
    }

    #--- output
    return(grad)
}

## File Name: linking_haberman_lq_pw_le_arrange_Vgamma.R
## File Version: 0.04


linking_haberman_lq_pw_le_arrange_Vgamma <- function(vcov_list, par_gamma, I, G,
        ind_items, ind_studies)
{

    NPG <- 2*I*G
    Vgamma <- matrix(0, nrow=NPG, ncol=NPG,
                    dimnames=list(names(par_gamma), names(par_gamma) ) )
    if (!is.null(vcov_list)){
        for (gg in 1L:G){
            items_gg <- ind_items[ ind_studies==gg ]
            Igg <- length(items_gg)
            items_gg <- rep(items_gg, each=2)
            ind2 <- 2*I*(gg-1) + 2*(items_gg-1)+rep(1L:2, Igg)
            Vgamma[ind2, ind2] <- vcov_list[[gg]]
        }
    }
    return(Vgamma)
}

## File Name: rasch.va.R
## File Version: 0.13


# estimating the Rasch model with variational approximation
rasch.va <- function( dat, globconv=.001, maxiter=1000)
{
    # data preparation
    N <- nrow(dat)
    I <- ncol(dat)
    dat2 <- dat
    dat2.resp <- 1 - is.na(dat)
    dat2[ dat2.resp==0 ] <- 0
    # initial values
    b <- - stats::qlogis( colMeans( dat, na.rm=T ) )
    sig2 <- 1
    mu.i <- rep(0,N)
    mu.i <- .8*stats::qnorm( rowMeans( ( dat + .1 ) / 1.2 ) )
    sigma2.i <- rep( sig2, N )
    disp <- "...........................................................\n"
    iter <- 0
    conv <- 1000

    #***************
    # begin algorithm
    while( (globconv < conv)  &    ( iter < maxiter )    ){
        cat(disp)
        cat("Iteration", iter+1, "   ", paste( Sys.time() ), "\n" )
        b0 <- b
        # compute xsi.ij: formula (18)
        xsi.ij <- matrix( b^2, N, I, byrow=T) -       # (x'beta)^2
                    2 * matrix( b, N, I, byrow=T ) * mu.i +  # 2nd term
                    mu.i^2 + sigma2.i
        xsi.ij <- sqrt( xsi.ij )
        lam.xsi.ij <- tanh( xsi.ij / 2 ) / ( 4*xsi.ij )
        # update beta
        t1 <- colSums( 2 * lam.xsi.ij  * dat2.resp )
        b <- colSums( - ( dat2 - .5 ) * dat2.resp + 2 * lam.xsi.ij * mu.i ) / t1
        # update sigma2
        sig2 <- mean( mu.i^2 + sigma2.i )
        # update sigma2.i
        sigma2.i <- 1 / ( 2 * rowSums( lam.xsi.ij ) + 1 / sig2  )
        # update mu.i
        mu.i <- rowSums( ( dat2 - 1/2 + 2*lam.xsi.ij * matrix( b, N, I, byrow=TRUE ) )
                                * dat2.resp  ) * sigma2.i
        # display progress
        conv <- max( abs(b-b0))
        iter <- iter+1
        cat( paste( "    Maximum b parameter change=",
                paste( round(max(abs(b-b0)),6), collapse=" " ), "\n", sep=""))
        cat( paste( "    SD Trait=",
                paste( round(sqrt(sig2),3), collapse=" " ), "\n", sep=""))
        utils::flush.console()
    }
    item <- data.frame("item"=colnames(dat), "b"=b )
    res <- list(sig=sqrt(sig2), item=item, xsi.ij=xsi.ij, mu.i=mu.i, sigma2.i=sigma2.i)
    return(res)
}

## File Name: noharm_sirt_preproc_parameter_table_matrix.R
## File Version: 0.105

noharm_sirt_preproc_parameter_table_matrix <- function(pattmat, valmat, patt_id,
    patt_label, minval, symm=FALSE)
{
    if (!is.finite(minval)){
        minval <- 0
    }
    pattmat <- noharm_sirt_preproc_pattern_matrix(pattmat=pattmat, minval=minval,
                    symm=symm)
    I <- nrow(pattmat)
    D <- ncol(pattmat)
    parm1 <- data.frame(mat=patt_label, matid=patt_id, row=rep(1L:I,D),
                            col=rep(1L:D, each=I) )
    parm1$index <- as.vector(pattmat)
    parm1$starts <- as.vector(valmat)
    parm1$est_par <- 1 - is.na(parm1$index)
    parm1$nonnull_par <- 1 - (1-parm1$est)*(parm1$starts==0)
    parm1$fixed <- 1*is.na(parm1$index)
    if (symm){
        parm1$est_par <- parm1$est_par * (parm1$row >=parm1$col )
    }
    return(parm1)
}

## File Name: mgsem_list_elements_est_total_implied.R
## File Version: 0.03

mgsem_list_elements_est_total_implied <- function(model, is_B)
{
    G <- length(model)-1
    est_total0 <- list()
    implied0 <- list()
    for (gg in 1L:G){
        est0 <- model[[1]]$est
        est_gg <- model[[gg+1]]$est
        est_total0[[gg]] <- mgsem_add_list_entries(list1=est0, add_list=est_gg,
                                output_list=est0)
        implied0[[gg]] <- mgsem_compute_model_implied_moments(est=est_total0[[gg]],
                                    is_B=is_B, calc_Sigma=TRUE, calc_Mu=TRUE)
    }
    #--- output
    res <- list(est_total0=est_total0, implied0=implied0)
    return(res)
}

## File Name: invariance_alignment_find_parameter_constraints.R
## File Version: 0.416

invariance_alignment_find_parameter_constraints <- function(parm,
    parm_tol, miss_items, wgt=NULL, maxiter=10, conv=1E-4)
{
    parm_est <- parm
    G <- nrow(parm)
    I <- ncol(parm)
    parm_names <- colnames(parm)
    parm_joint <- sirt_colMedian(x=parm)
    if (is.null(wgt)){
        wgt <- 1+0*parm
    }
    iter <- 1
    iterate <- TRUE

    while(iterate){
        parm_est_old <- parm_est
        parm_est <- sirt_matrix2( x=parm_joint, nrow=G)
        ind <- abs(parm-parm_est) > parm_tol
        ind0 <- ! ind
        if ( sum(ind) > 0){
            parm_est[ind] <- parm[ind]
        }
        parm_joint <- colSums(parm_est*ind0*wgt) / colSums(ind0*wgt)
        parm_joint2 <- colSums(parm_est*wgt) / colSums(wgt)
        parm_joint <- ifelse(is.na(parm_joint), parm_joint2, parm_joint)
        change <- max(abs(parm_est-parm_est_old))
        if ((change < conv) | (iter >=maxiter)){
            iterate <- FALSE
        }
        iter <- iter + 1
        if (iter==2){ iterate <- TRUE }
    }
    parm_est <- sirt_matrix_names(x=parm_est, extract_names=parm)
    parm_est[ miss_items==0 ] <- NA

    parm_dif <- parm_est - sirt_matrix2( x=parm_joint, nrow=G)

    #--- count number of estimated parameters
    eps <- 1E-5
    pj <- sirt_matrix2(x=parm_joint, nrow=G)
    N_unique_parm_items <- colSums( abs(parm_est-pj)>eps, na.rm=TRUE )
    names(N_unique_parm_items) <- parm_names

    N0 <- ( colSums( abs(parm_est-pj)<=eps, na.rm=TRUE ) > 0 )
    N_parm_items <- N0 + N_unique_parm_items
    names(N_parm_items) <- parm_names

    N_parm_all <- sum(N_parm_items)
    N_unique_parm_groups <- rowSums( abs(parm_est-pj)>eps, na.rm=TRUE )
    names(N_unique_parm_groups) <- rownames(parm)

    N_total <- sum(miss_items)
    prop_noninvariance <- 100*sum(N_unique_parm_groups)/N_total

    #--- output
    res <- list( parm_est=parm_est, parm_joint=parm_joint, parm=parm, parm_tol=parm_tol,
                    parm_dif=parm_dif, N_parm_items=N_parm_items, N_parm_all=N_parm_all,
                    N_unique_parm_items=N_unique_parm_items,
                    N_unique_parm_groups=N_unique_parm_groups,
                    prop_noninvariance=prop_noninvariance, iter=iter, maxiter=maxiter,
                    G=G, I=I, wgt=wgt, N_total=N_total, miss_items=miss_items)
    return(res)
}

## File Name: sirt_pem_collect_parameters.R
## File Version: 0.07

sirt_pem_collect_parameters <- function( parmlist, pem_parameter_index )
{
    NV <- pem_parameter_index[['__length_parameter_vector']]
    parm <- rep(0, NV)
    NP <- pem_parameter_index[['__n_parameters']]
    NPL <- length(parmlist)
    for (pp in 1L:NPL){
        var_pp <- names(parmlist)[pp]
        parm_index_pp <- pem_parameter_index[[ var_pp ]]
        x <- as.vector(parmlist[[pp]])
        parm[ parm_index_pp$index ] <- x
    }
    return(parm)
}

## File Name: rasch_mml2_raschtype_mstep_parameter_group_evaluate_prior.R
## File Version: 0.06


rasch_mml2_raschtype_mstep_parameter_group_evaluate_prior <- function(parm,
    h, prior, ll0, ll1, ll2, prior_fct )
{
    if (!is.null(prior)){
        prior_dens_args <- list(parm, prior[1], prior[2], log=TRUE)
        m0 <- do.call(prior_fct, args=prior_dens_args)
        prior_dens_args[[1]] <- parm + h
        m1 <- do.call(prior_fct, args=prior_dens_args)
        prior_dens_args[[1]] <- parm - h
        m2 <- do.call(prior_fct, args=prior_dens_args)
        ll0 <- ll0 + m0
        ll1 <- ll1 + m1
        ll2 <- ll2 + m2
    }
    res <- list(ll0=ll0, ll1=ll1, ll2=ll2)
    return(res)
}

## File Name: amh_plot.R
## File Version: 0.342


#*** plot results of objects of class amh
amh_plot <- function( x, conflevel=.95, digits=3, lag.max=.1, col.smooth="red",
    lwd.smooth=2, col.split="blue", lwd.split=2, lty.split=1,
    col.ci="orange", cex.summ=1, ask=FALSE, ... )
{

    object <- x    # rename x into object
    mcmcobj <- object$mcmcobj
    lag.max <- round( nrow(mcmcobj) * lag.max )
    round.summ <- digits

    # mcmcobj <- (object$mcmcobj)[[1]]
    lag.max <- min( nrow(mcmcobj), lag.max )
    # index vector
    a1 <- attr(mcmcobj,'mcpar')
    iterindex <- seq(a1[1], a1[2], a1[3] )

    # smcmcobj <- object$summary.mcmcobj
    smcmcobj <- object$amh_summary
    VV <- ncol(mcmcobj)
    ci.quant <- - stats::qnorm( (1-conflevel)/2 )
    graphics::par( mfrow=c(2,2))

    for (vv in 1L:VV){
        x.vv <- as.vector( mcmcobj[,vv] )
        parm.vv <- colnames(mcmcobj)[vv]
        sparm.vv <- smcmcobj[ smcmcobj$parameter==parm.vv, ]

        #*** traceplot
        graphics::plot( iterindex, x.vv, type='l',
                            main=paste0( 'Traceplot of ', parm.vv ),
                            xlab='Iterations', ylab='', ... )
        x1 <- as.numeric( x.vv )
        xmin <- min(x1)
        xmax <- max(x1)

        # 3 splits for chain
        NX <- length(x.vv)
        NS <- NX / 3
        g1 <- round(seq( 0, NX, length=4 ))

        for (ii in 1L:3){
            i1 <- seq(g1[ii]+1, g1[ii+1] )
            m1 <- mean( x1[ i1 ] )
            graphics::lines( iterindex[ i1 ], rep( m1, length(i1) ), col=col.split,
                                lwd=lwd.split, lty=lty.split )
        }
        # include moving average here!!
        l1 <- sirt_moving_average(x1, B=round( lag.max / 2 ), fill=FALSE)
        graphics::lines( iterindex, l1, col=col.smooth, lwd=lwd.smooth )

        #*** density estimate
        graphics::plot( stats::density( x.vv ), main=paste0( 'Density of ', parm.vv ) )

        c1 <- stats::quantile( x1, ( 1 - conflevel  ) / 2 )
        c2 <- stats::quantile( x1, 1 - ( 1 - conflevel  ) / 2 )
        graphics::lines( c(c1,c2), c(0,0), col=col.ci, lwd=3 )
        graphics::points( sparm.vv$Mean, 0, pch=17, col=col.ci, cex=1.5)

        #*** plot autocorrelation function
        mtitle <- paste0( 'Autocorrelation of ', parm.vv )
        m1 <- stats::acf( x.vv, lag.max=lag.max, plot=FALSE)
        acf1 <- m1$acf[,1,1]
        thin_lag <- round( mean( diff(iterindex) ) )
        iter_vv <- seq( 0, lag.max )*thin_lag
        # blue dashed line at
        bd <- .05
        ylim <- c( min( acf1, - bd), 1 )
        graphics::plot( iter_vv, acf1, xlab='Lag', ylab='ACF',
                            main=mtitle, type='n', ylim=ylim)
        NL <- length(iter_vv)
        for (hh in 1L:NL){
            graphics::lines( rep( iter_vv[hh],2), c(0, acf1[hh]) )
        }
        graphics::abline( h=bd, col='blue', lty=2)
        graphics::abline( h=-bd, col='blue', lty=2)

        #***
        # numerical summary
        graphics::plot( c(0,1), c(0,1), axes=FALSE, xlab='', ylab='',
                    main=paste0( 'Summary of ', parm.vv ), type='n', ...)
        x0 <- 0
        y0 <- 0
        heights.summ <- c( .05,  .15, .25,  .35, .45, .55, .65, .75)
        graphics::text( x0 + .0015, y0 + heights.summ[8], 'Posterior Mean=',
                                cex=cex.summ, pos=4)
        graphics::text( x0 + .5, y0 + heights.summ[8],
                paste0( sirt_format_numb( x=mean( x1 ), digits=round.summ) ), pos=4 )
        hvv <- heights.summ[7]
        graphics::text( x0 + .0015, y0 + hvv, 'Posterior Mode=', cex=cex.summ, pos=4)
        graphics::text( x0 + .5, y0 + hvv,
                paste0( sirt_format_numb( x=sparm.vv$MAP, digits=round.summ) ), pos=4 )

        graphics::text( x0 + .0015, y0 + heights.summ[6], 'Posterior SD=',
                            cex=cex.summ, pos=4)
        graphics::text( x0 + .5, y0 + heights.summ[6],
                paste0( sirt_format_numb( x=stats::sd( x1 ), digits=round.summ) ), pos=4 )

        hvv <- heights.summ[5]
        graphics::text( x0 + .0015, y0 + hvv,
                        paste( round(100*conflevel ), '% Credibility Interval=',sep=''),
                        cex=cex.summ, pos=4 )

        hvv <- heights.summ[4]
        ci.lower <- sirt_format_numb( stats::quantile( x1, ( 1 - conflevel  ) / 2 ),
                                            digits=round.summ )
        ci.upper <- sirt_format_numb( stats::quantile( x1, 1-( 1 - conflevel  ) / 2 ),
                                            digits=round.summ )
        graphics::text( x0 + .25, y0 + hvv,
                            paste( '[', ci.lower,    ',', ci.upper, ']',  sep=''),
                            cex=cex.summ, pos=4)
        hvv <- heights.summ[3]
        graphics::text( x0 + .0015, y0 + hvv, 'Rhat=', cex=cex.summ, pos=4)
        graphics::text( x0 + .5, y0 + hvv,
                paste0( sirt_format_numb( x=sparm.vv$Rhat, digits=3)  ), pos=4 )
        hvv <- heights.summ[2]
        graphics::text( x0 + .0015, y0 + hvv, 'SERatio=', cex=cex.summ, pos=4)
        graphics::text( x0 + .5, y0 + hvv,
                        paste0( sirt_format_numb( x=sparm.vv$SERatio, digits=3)), pos=4 )

        hvv <- heights.summ[1]
        graphics::text( x0 + .0015, y0 + hvv, 'Effective Sample Size=',
                        cex=cex.summ, pos=4)
        graphics::text( x0 + .705, y0 + hvv,
                        paste0( sirt_format_numb( x=sparm.vv$effSize, digits=1)), pos=4 )
        graphics::par(ask=ask)
    }
    graphics::par(mfrow=c(1,1))
}

## File Name: lsem.permutationTest.R
## File Version: 0.594


#*** permutation test for LSEM model
lsem.permutationTest <- function( lsem.object, B=1000, residualize=TRUE,
            verbose=TRUE, n.core=1, cl.type="PSOCK" )
{

    s1 <- Sys.time()
    CALL <- match.call()

    lavaan.args <- lsem.object$lavaan.args
    entr <- c( 'lavmodel', 'data', 'h', 'moderator.grid', 'moderator', 'eps',
                    'fit_measures', 'par_invariant', 'est_joint',
                    'par_linear', 'par_quadratic', 'pw_linear',
                    'pw_quadratic')
    object <- lsem.object
    arglist <- list()
    EE <- length(entr)
    for (ee in 1L:EE){
        arglist[[ entr[ee] ]] <- object[[ entr[ee] ]]
    }
    arglist2 <- lsem.object$lavaan.args
    NL <- length(arglist2)
    if (NL > 0){
        for (ll in 1L:NL){
            arglist[[ names(arglist2)[ll] ]] <- arglist2[[ names(arglist2)[ll] ]]
        }
    }
    arglist$residualize <- residualize
    arglist$verbose <- FALSE
    arglist$standardized <- lsem.object$standardized
    arglist$standardized_type <- lsem.object$standardized_type
    arglist$lavaan_fct <- lsem.object$lavaan_fct
    arglist$sufficient_statistics <- lsem.object$sufficient_statistics
    use_lavaan_survey <- FALSE
    arglist$pseudo_weights <- lsem.object$pseudo_weights

    data1 <- data0 <- object$data
    parameters <- object$parameters
    parameters_summary <- object$parameters_summary
    moderator <- object$moderator

    #******************************************
    # start permutation test

    parameters_permutation <- matrix( NA, nrow(parameters), ncol=B)
    parameters_summary_M <- matrix( NA, nrow(parameters_summary), ncol=B)
    rownames(parameters_summary_M) <- parameters_summary$par
    parameters_summary_MAD <- parameters_summary_SD <- parameters_summary_M
    parameters_summary_lin_slo <- parameters_summary_M

    if ( verbose ){
        cat('Permutation test LSEM \n')
    }
    #** estimate model for each permutation
    bb <- 1
    nn <- 0

    N <- nrow(data0)

    # sampled values
    l1 <- list()

    #--- nonparallel version
    if (n.core==1){
        while (bb<=B){
            data1[, moderator ] <- sample( data0[, moderator ] )
            arglist$data <- data1
            res0 <- try( do.call( what='lsem.estimate', args=arglist ), silent=TRUE )
            res0_out <- list(est=res0$parameters$est, M=res0$parameters_summary$M,
                                SD=res0$parameters_summary$SD,
                                MAD=res0$parameters_summary$MAD,
                                lin_slo=res0$parameters_summary$lin_slo)
            res0_out <- lsem_permutationTest_collect_output(res0=res0)
            if ( ! inherits(res0,'try-error') ){
                parameters_permutation[, bb] <- res0_out$est
                parameters_summary_M[,bb] <- res0_out$M
                parameters_summary_SD[,bb] <- res0_out$SD
                parameters_summary_MAD[,bb] <- res0_out$MAD
                parameters_summary_lin_slo[,bb] <- res0_out$lin_slo
                if (verbose){
                    cat( bb, ' ')
                    if ( bb %% 20==0 ){
                        cat('\n')
                    }
                    utils::flush.console();
                }
                bb <- bb+1
            } else {
                nn <- nn + 1
                if (verbose){
                    cat( '\n Delete permutation dataset due to non-convergence\n')
                    utils::flush.console()
                }
            }
        }  # end bb
        if (verbose){ cat('\n') }
    }

    #------
    #* parallel version
    if (n.core>1){
        m <- parallel::detectCores()
        if(n.core>m-1){
            n.core <- m-1
        }
        cl <- parallel::makeCluster(spec=n.core, type=cl.type)
        varlist <- c('arglist')
        parallel::clusterExport(cl=cl, varlist=varlist, envir=environment() )
        parallel::clusterEvalQ(cl, expr=library(sirt))

        fun_lsem_permutation_test <- function(i, arglist)
        {
            data1 <- data0 <- arglist$data
            moderator <- arglist$moderator
            iterate <- TRUE
            while(iterate){
                data1[, moderator ] <- sample( data0[, moderator ] )
                arglist1 <- arglist
                arglist1$data <- data1
                res0 <- try( do.call( what=lsem.estimate, args=arglist1,
                                envir=environment() ), silent=TRUE )
                if ( ! inherits(res0,'try-error') ){
                    iterate <- FALSE
                    res0_out <- list(est=res0$parameters$est,
                                        M=res0$parameters_summary$M,
                                        SD=res0$parameters_summary$SD,
                                        MAD=res0$parameters_summary$MAD,
                                        lin_slo=res0$parameters_summary$lin_slo)
                }
            }
            return(res0_out)
        }


        res_all <- sirt_parlapply(cl=cl, X=1L:B, FUN=fun_lsem_permutation_test,
                        verbose=verbose, arglist=arglist)

        parallel::stopCluster(cl)

        for (bb in 1L:B){
            parameters_permutation[, bb] <- res_all[[bb]]$est
            parameters_summary_M[,bb] <- res_all[[bb]]$M
            parameters_summary_SD[,bb] <- res_all[[bb]]$SD
            parameters_summary_MAD[,bb] <- res_all[[bb]]$MAD
            parameters_summary_lin_slo[,bb] <- res_all[[bb]]$lin_slo
        }

        nn <- NA
    }
    #-----------------

    #*** create global test statistics
    teststat <- data.frame( par=parameters_summary$par )
    teststat$M <- parameters_summary$M
    teststat$SD <- parameters_summary$SD
    teststat$SD_p <- rowMeans( parameters_summary_SD >=parameters_summary$SD )
    teststat$MAD <- parameters_summary$MAD
    teststat$MAD_p <- rowMeans( parameters_summary_MAD >=parameters_summary$MAD )
    teststat$lin_slo <- parameters_summary$lin_slo
    p1 <- rowMeans( parameters_summary_lin_slo >=parameters_summary$lin_slo )
    p2 <- rowMeans( parameters_summary_lin_slo <=parameters_summary$lin_slo )
    teststat$lin_slo_p <- 2*ifelse( p1 < p2, p1, p2 )
    teststat$lin_slo_p <- ifelse( teststat$lin_slo_p > 1, 1, teststat$lin_slo_p )
    #********************
    # pointwise statistics
    rownames(parameters_permutation) <- rownames(parameters)

    parameters_pointwise_test <- parameters[, c('grid_index',
                                    'moderator','par','parindex' ) ]
    parindex_ <- parameters$parindex
    parameters_pointwise_test$est <- parameters$est - parameters_summary$M[ parindex_ ]
    par_pointwise_perm <- parameters_permutation -     parameters_summary_M[ parindex_, ]

    p1 <- rowMeans( parameters_pointwise_test$est >=par_pointwise_perm )
    p2 <- rowMeans( parameters_pointwise_test$est <=par_pointwise_perm )
    parameters_pointwise_test$p <- 2*ifelse( p1 < p2, p1, p2 )
    parameters_pointwise_test$p <- ifelse( parameters_pointwise_test$p > 1,
                                        1, parameters_pointwise_test$p )

    #- non-convergence rates
    nonconverged_rate <- 100*nn / (B+nn)

    s2 <- Sys.time()

    res <- list( teststat=teststat,
                    parameters_pointwise_test=parameters_pointwise_test,
                    parameters=parameters,
                    parameters_permutation=parameters_permutation,
                    parameters_summary=parameters_summary,
                    parameters_summary_M=parameters_summary_M,
                    parameters_summary_SD=parameters_summary_SD,
                    parameters_summary_MAD=parameters_summary_MAD,
                    parameters_summary_lin_slo=parameters_summary_lin_slo,
                    par_pointwise_perm=par_pointwise_perm,
                    moderator.density=object$moderator.density,
                    moderator=object$moderator,
                    moderator.grid=object$moderator.grid,
                    h=object$h, bw=object$bw, N=object$N,
                    nonconverged_rate=nonconverged_rate,
                    use_lavaan_survey=use_lavaan_survey,
                    B=B, s1=s1, s2=s2, lavmodel=object$lavmodel,
                    n.core=n.core, cl.type=cl.type, CALL=CALL )
    class(res) <- 'lsem.permutationTest'
    return(res)
}

## File Name: rm_facets_string_part_extract.R
## File Version: 0.06

rm_facets_string_part_extract <- function( x, split, part)
{
    vec <- strsplit( paste(x), split=split )
    vec <- unlist( lapply( vec, FUN=function(vv){ vv[part] } ) )
    return(vec)
}

## File Name: btm_fit_combine_tables.R
## File Version: 0.07


btm_fit_combine_tables <- function( win1, win2, ind1, ind2, TP )
{
    win <- rep( 0, TP )
    win[ind1] <- win1[, 1]
    win[ind2] <- win[ind2] + win2[, 1]
    return(win)
}

## File Name: sirt_rmvnorm.R
## File Version: 0.09


sirt_rmvnorm <- function (n, mean=NULL, sigma, ...)
{
    if (is.null(mean)){
        mean <- rep(0,ncol(sigma) )
    }
    CDM::CDM_rmvnorm( n=n, mean=mean, sigma=sigma )
}

## File Name: tetrachoric2.R
## File Version: 1.346

tetrachoric2 <- function( dat, method="Ol",  delta=.007, maxit=1000000,
    cor.smooth=TRUE, progress=TRUE)
{
    # process data
    dat <- as.matrix(dat)
    if (method=='Ol'){
        res <- polychoric2( dat=dat, cor.smooth=cor.smooth, maxiter=maxit)
    }
    if ( method !='Ol' ){

        # calculate tau
        tau <- - stats::qnorm( colMeans(dat,na.rm=TRUE ) )
        dat.resp <- 1 - is.na(dat)
        dat[ dat.resp==0] <- 9
        I <- ncol(dat)
        # calculate frequencies
        dfr <- data.frame( item1=rep(1L:I,I), item2=rep(1L:I, each=I ) )
        h1 <- t( ( dat==1 ) ) %*% ( dat==0 )
        dfr$f11 <- matrix( t( ( dat==1 ) ) %*% ( dat==1 ), ncol=1, byrow=TRUE ) + .5
        dfr$f10 <- matrix( t( ( dat==1 ) ) %*% ( dat==0 ), ncol=1, byrow=TRUE ) + .5
        dfr$f01 <- matrix( t( ( dat==0 ) ) %*% ( dat==1 ), ncol=1, byrow=TRUE ) + .5
        dfr$f00 <- matrix( t( ( dat==0 ) ) %*% ( dat==0 ), ncol=1, byrow=TRUE ) + .5

        dfr$ftot <- dfr$f11 + dfr$f10 + dfr$f01 + dfr$f00
        dfr$p11 <- dfr$f11 / dfr$ftot
        dfr$pi1 <- ( dfr$f11 + dfr$f10 - 1 ) / dfr$ftot
        dfr$pi2 <- ( dfr$f11 + dfr$f01 - 1) / dfr$ftot
        # subdata of dfr
        dfr <- dfr[ dfr$item1 > dfr$item2, ]
        dfr <- dfr[ dfr$ftot > 0, ]

        dfr$qi1 <- stats::qnorm( dfr$pi1)
        dfr$qi2 <- stats::qnorm( dfr$pi2)
        #******************
        # method of Bonett
        if ( method %in% c('Bo','Di') ){
            dfr$pmin <- ifelse( dfr$pi1 < dfr$pi2, dfr$pi1, dfr$pi2 )
            dfr$c <- ( 1 - abs( dfr$pi1 - dfr$pi2 ) / 5 - ( 0.5 - dfr$pmin)^2  ) / 2
            dfr$omega <- ( dfr$f00 * dfr$f11 ) / ( dfr$f01 * dfr$f10)
            dfr$r0 <- cos( pi / ( 1 + dfr$omega^( dfr$c )  ) )
        }
        #*****
        # method of Divgi
        if ( method=='Di'){
            dfr2 <- as.matrix(dfr)
            numdiffparm <- 1e-5
            dfr$r0 <- sirt_rcpp_tetrachoric2_rcpp( dfr=dfr2, h=numdiffparm,
                                maxiter=maxit )
        }
        #******************
        # method of Tucker
        if ( method=='Tu'){
            # functions defined by Cengiz Zopluoglu
            L <- function(r,h,k)
                {
                        (1/(2*pi*sqrt(1-r^2)))*exp(-((h^2-2*h*k*r+k^2)/(2*(1-r^2))))
                }
            S <- function(x,r,h,k)
                { x-(L(r,h,k)*delta)
                }
            A0 <- dfr$A0 <- dfr$p11 - dfr$pi1 * dfr$pi2
            dfr$r0 <- delta / 2
            dfr$iter <- 0

            dfr$iter <- 0
            dfr$conv <- 0
            ii <- 0

            vars <-  c('A0','r0','iter','conv')
            if (progress){ cat('    0 : ' ) }
            while( ( mean( dfr$conv) < 1 ) & ( ii < maxit ) ){
                # iterations
                dfr0 <- dfr
                #***
                ii <- ii+1
                dfr$A0 <- dfr$A0 - delta * L( dfr$r0, dfr$qi1, dfr$qi2 )
                dfr$r0 <- dfr$r0 + delta
                dfr$iter <- ii
                ind <- which( dfr$A0 < 0 )
                if ( length(ind) > 0 ){
                    h1 <- dfr$r0 - delta/2 + dfr$A0 / L( dfr0$r0, dfr$qi1, dfr$qi2 )
                    dfr$r0[ind] <- h1[ind]
                    dfr$conv[ind] <- 1
                }
                i2 <- which( dfr$conv==1 & dfr0$conv==1 )
                if (length(i2) > 0){    dfr[ i2, vars] <- dfr0[ i2, vars] }
                if (progress){
                    if (ii %% 100==0 ){ cat('.') ; utils::flush.console() }
                    if (ii %% 1000==0 ){ cat('\n',ii, ': ') }
                }
            }
        #******************
            cat('\n')
        }
        TC <- matrix(NA, I, I )
        diag(TC) <- 1
        TC[ as.matrix(dfr[, c('item1','item2') ] ) ] <- dfr$r0
        TC[ as.matrix(dfr[, c('item2','item1') ] ) ] <- dfr$r0
        if (cor.smooth){
            TC <- sirt_import_psych_cor.smooth(x=TC)
        }
        rownames(TC) <- colnames(TC) <- colnames(dat)
        res <- list(tau=tau, rho=TC )
    }   # method !='Ol'
    return(res)
}

## File Name: R2noharm-utility.R
## File Version: 1.14




#..............................................................................
# utility functions for R2noharm

#---------------------------------------------------------------
# Read Tanaka Index
.noharm.tanaka <- function( noharmout1 ){
    i1 <- grep( "Tanaka", noharmout1 )
    tanaka <- as.numeric( strsplit( paste(noharmout1[i1]), split="=" )[[1]][2] )
    return(tanaka)
    }
#---------------------------------------------------------------

#-----------------------------------------------------------------
# Root mean square of residuals
.noharm.rmsr <- function( noharmout1 ){
    i1 <- grep( "Root mean square of residuals", noharmout1 )
    RMSR <- as.numeric( strsplit( paste(noharmout1[i1]), split="=" )[[1]][2] )
    return(RMSR)
    }
#-----------------------------------------------------------------


#------------------------------------------------------------------
# UNIQUE VARIANCES, thresholds, 1-dimensional parametrizations    #
.noharm.itemlevel <- function( noharmout1, itemlevel.type, I=I,dat=dat){
        i1 <- grep( itemlevel.type, noharmout1 )
        VV <- floor(I/9)+1 * ( I/9 !=floor( I/9) )
        uniquevar <- noharmout1[ seq( i1+5, i1 + 5 + 4*(VV-1), 4 ) ]
        #*** addition ARb 2014-01-29
            uniquevar <- uniquevar[ substring( uniquevar, 1, 1 )==" " ]
        #***
        uniquevar <- paste( uniquevar, collapse="  " )
        uniquevar <- strsplit( uniquevar, split=" " )[[1]]
        uniquevar <- as.numeric( paste( uniquevar[ uniquevar !="" ] ) )
        uniquevar <- uniquevar[1:I]
        names(uniquevar) <- colnames(dat)
        return( uniquevar )
        }
#------------------------------------------------------------------


#-------------------------------------------------------------------
# Function for extracting factor loadings
.noharm.loadings <- function( noharmout1, type1, type2, dimensions=dimensions, I=I, dat=dat){
    i1 <- grep( type1, noharmout1 )[1]
    i2 <- grep( type2, noharmout1 )[1]
    rows <- seq( i1+2, i2-4  )
    if ( ( dimensions==1 ) & ( type1=="Factor Loadings" ) ){
        rows <- seq(i1+2, i2-6 )
            }
    r1 <- intersect( which( noharmout1=="" ), rows )
    VV <- length(r1)/2
    r2 <- stats::aggregate( r1, list(rep( 1:VV, each=2 )), mean )
    colnames(r2) <- c("index", "index.row" )
    index.m <- data.frame( r2, "begin"=r2[,2] + 2,
                    "end"=c( r2[,2][-1] - 2, i2-4 ) )
        if ( dimensions==1 & type1=="Factor Loadings"){
                index.m$end <- i2 - 6
                    }
    loading.matrix <- matrix(0,I,dimensions)
    rownames(loading.matrix) <- colnames(dat)
    colnames(loading.matrix) <- paste( "F", 1:dimensions, sep="")
    for (vv in 1:VV){
        ind.vv.col <- noharmout1[ index.m$index.row[vv] ]
        ind.vv.col <- strsplit( ind.vv.col, split=" " )[[1]]
        ind.vv.col <- as.numeric( ind.vv.col[ ind.vv.col !="" ] )
        ind.vv <- seq( index.m$begin[vv], index.m$end[vv] )
        loading.vv <- noharmout1[ ind.vv ]
        loading.vv <- sapply( loading.vv, FUN=function(ll){
                            ll1 <- strsplit( paste(ll), split=" " )[[1]]
                            as.numeric( ll1[ ll1 !=ll1[1] ] )
                                                } )
        loading.vv <- as.matrix( loading.vv )
        ind.vv.row <- as.vector( loading.vv[1,] )
        loading.vv <-  loading.vv[-1,]
        loading.matrix[ ind.vv.row, ind.vv.col ] <- t(loading.vv)
        }
        return( loading.matrix )
        }
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------
# Function for extracting Factor Correlations
.noharm.correlations <- function( noharmout1,
                        type1="Factor Correlations",
                        type2="Residual Matrix",
                                    dimensions=dimensions, dat=dat ){
    i1 <- grep( type1, noharmout1 )[1]
    i2 <- grep( type2, noharmout1 )[1]
    rows <- seq( i1+2, i2-4 )
    r1 <- intersect( which( noharmout1=="" ), rows )
    VV <- length(r1)/2
    r2 <- stats::aggregate( r1, list(rep( 1:VV, each=2 )), mean )
    colnames(r2) <- c("index", "index.row" )
    index.m <- data.frame( r2, "begin"=r2[,2] + 2,
                    "end"=c( r2[,2][-1] - 2, i2-4 ) )
    correlation.matrix <- matrix(NA,dimensions,dimensions)
    rownames(correlation.matrix) <- colnames(correlation.matrix) <- paste( "F", 1:dimensions, sep="")
    for (vv in 1:VV){
        ind.vv.col <- noharmout1[ index.m$index.row[vv] ]
        ind.vv.col <- strsplit( ind.vv.col, split=" " )[[1]]
        ind.vv.col <- as.numeric( ind.vv.col[ ind.vv.col !="" ] )
        ind.vv <- seq( index.m$begin[vv], index.m$end[vv] )
        correlation.vv <- noharmout1[ ind.vv ]
        correlation.vv <- sapply( correlation.vv, FUN=function(ll){
                            ll1 <- strsplit( paste(ll), split=" " )[[1]]
                            as.vector( as.numeric( ll1[ ll1 !="" ] ))
                                                } )
        if ( is.list( correlation.vv ) ){
            ind.vv.row <- as.vector(unlist( lapply( correlation.vv, FUN=function(ll){ ll[1] } ) ))
            correlation.vv <-  lapply( correlation.vv, FUN=function(ll){ ll[-1] } )
            } else {
            correlation.vv <- as.vector( correlation.vv )
            ind.vv.row <- correlation.vv[1]
            correlation.vv <- as.list( correlation.vv[-1] )
            }
        RVV <- length(correlation.vv)
        for (rvv in 1:RVV){
            correlation.vv.rvv <- correlation.vv[[rvv]]
            i1.rvv <- ind.vv.row[rvv]
            i2.rvv <- ind.vv.col[ seq( 1, length(correlation.vv.rvv)) ]
            correlation.matrix[ i2.rvv, i1.rvv ] <- correlation.matrix[ i1.rvv, i2.rvv ] <- correlation.vv.rvv
                }
        }
    correlation.matrix
        }
#---------------------------------------------------------------------------------------------



#--------------------------------------------------------------------------#
# Extracting Residual Matrix                                               #
.noharm.residuals <- function(noharmout1, I=I, dat=dat ){
    i1 <- grep( "Residual Matrix", noharmout1 )
    i2 <- grep( "Sum of squares of residuals", noharmout1 )
    rows <- seq( i1+2, i2-4 )
    r1 <- intersect( which( noharmout1=="" ), rows )
    VV <- length(r1)/2
    r2 <- stats::aggregate( r1, list(rep( 1:VV, each=2 )), mean )
    colnames(r2) <- c("index", "index.row" )
    index.m <- data.frame( r2, "begin"=r2[,2] + 2,
#                    "end"=c( r2[,2][-1] - 2, i2-4 ) )
                    "end"=c( r2[,2][-1] - 2, i2-3 ) )
    resid.matrix <- matrix(0,I,I)
    rownames(resid.matrix) <- colnames(resid.matrix) <- colnames(dat)
    for (vv in 1:VV){
        ind.vv.col <- noharmout1[ index.m$index.row[vv] ]
        ind.vv.col <- strsplit( ind.vv.col, split=" " )[[1]]
        ind.vv.col <- as.numeric( ind.vv.col[ ind.vv.col !="" ] )
        ind.vv <- seq( index.m$begin[vv], index.m$end[vv] )
        resid.vv <- noharmout1[ ind.vv ]
        resid.vv <- sapply( resid.vv, FUN=function(ll){
                            ll1 <- strsplit( paste(ll), split=" " )[[1]]
                            as.numeric( ll1[ ll1 !="" ] )
                                                } )
    if (is.list( resid.vv)){
            gh1 <- lapply( resid.vv, FUN=function(ll){ length(ll) } )
            ind1 <- which (gh1==0)
            if ( length(ind1) > 0 ){
                for (ii in ind1 ){    resid.vv[[ii]] <- NULL }
                        }
            ind.vv.row <- as.vector(unlist( lapply( resid.vv, FUN=function(ll){ ll[1] } ) ))
            resid.vv <-  lapply( resid.vv, FUN=function(ll){ ll[-1] } )
                        } else {
                        resid.vv <- as.vector(resid.vv)
                        ind.vv.row <- resid.vv[1]
                        resid.vv <- as.list( resid.vv[-1] )
                        }
        RVV <- length(resid.vv)
        for (rvv in 1:RVV){
            resid.vv.rvv <- resid.vv[[rvv]]
            i1.rvv <- ind.vv.row[rvv]
            i2.rvv <- ind.vv.col[ seq( 1, length(resid.vv.rvv)) ]
            resid.matrix[ i2.rvv, i1.rvv ] <- resid.matrix[ i1.rvv, i2.rvv ] <- resid.vv.rvv
                }
        }
        return( resid.matrix )
        }
#-------------------------------------------------------------------------------------

## File Name: linking_2groups_stocking_lord_fun.R
## File Version: 0.136



#- Stocking-Lord optimization function
linking_2groups_stocking_lord_fun <- function(x, pars, Theta, wgt, type="asymm",
        pow=2, eps=1e-3, simultan=FALSE, deriv=FALSE )
{
    TP <- length(Theta)
    I <- nrow(pars)
    mat <- matrix(NA, nrow=TP, ncol=I)
    mu <- x[1]
    sigma <- x[2]
    a1M <- sirt_matrix2(pars[,1], nrow=TP)
    b1M <- sirt_matrix2(pars[,2], nrow=TP)
    a2M <- sirt_matrix2(pars[,3], nrow=TP)
    b2M <- sirt_matrix2(pars[,4], nrow=TP)

    val <- 0
    if (deriv){
        val <- rep(0,2)
    }

    if (simultan){
        aM <- sirt_matrix2(x[ 2 + 1L:I ], nrow=TP)
        bM <- sirt_matrix2(x[ 2 + I + 1L:I ], nrow=TP)
        ind_a <- 2+1L:I
        ind_b <- 2+I+1L:I
        if (deriv){
            val <- rep(0,2+2*I)
        }
    }

    if (type=='asymm' | type=='symm'){
        Theta1 <- sigma*Theta+mu
        Theta2 <- Theta
        P1 <- stats::plogis( a1M*(Theta1-b1M))
        P2 <- stats::plogis( a2M*(Theta2-b2M))
        rowSums_P1 <- rowSums(P1)
        rowSums_P2 <- rowSums(P2)
        if (! simultan){ # simultan=FALSE
            z <- rowSums_P1 - rowSums_P2
            d <- linking_2groups_power_loss(x=z, pow=pow, eps=eps, deriv=deriv)
            if (!deriv){  # deriv=FALSE
                val <- val + sum( d*wgt )
            } else {  # deriv=TRUE
                fac0 <- P1*(1-P1)*a1M
                val[1] <- val[1] + sum(wgt*d*rowSums(fac0) )  # mu
                val[2] <- val[2] + sum(wgt*d*rowSums(fac0*Theta) )  # sigma
            }
        } else {  # simultan=TRUE
            P0 <- stats::plogis( aM * (Theta2-bM) )
            rowSums_P0 <- rowSums(P0)
            z1 <- rowSums_P1 - rowSums_P0
            d1 <- linking_2groups_power_loss(x=z1, pow=pow, eps=eps, deriv=deriv)
            z2 <- rowSums_P2 - rowSums_P0
            d2 <- linking_2groups_power_loss(x=z2, pow=pow, eps=eps, deriv=deriv)
            if (!deriv){ # deriv=FALSE
                val <- val + sum( (d1+d2)*wgt )
            } else {  # deriv=TRUE
                fac0 <- P1*(1-P1)*a1M
                val[1] <- val[1] + sum(wgt*d1*rowSums(fac0) )  # mu
                val[2] <- val[2] + sum(wgt*d1*rowSums(fac0*Theta) )  # sigma
                #- derivatives item discriminations
                fac1 <- P0*(1-P0)*(Theta2-bM)
                val[ind_a] <- val[ind_a] - colSums(wgt*(d1+d2)*fac1)
                #- derivatives item difficulties
                fac1 <- P0*(1-P0)*aM
                val[ind_b] <- val[ind_b] + colSums(wgt*(d1+d2)*fac1)
            }
        }
    }
    if (type=='symm'){
        Theta1 <- Theta
        Theta2 <- 1/sigma*Theta-mu/sigma
        P1 <- stats::plogis( a1M*(Theta1-b1M))
        P2 <- stats::plogis( a2M*(Theta2-b2M))
        rowSums_P1 <- rowSums(P1)
        rowSums_P2 <- rowSums(P2)
        if (!simultan){ # simultan=FALSE
            z <- rowSums_P1 - rowSums_P2
            d <- linking_2groups_power_loss(x=z, pow=pow, eps=eps, deriv=deriv)
            if (!deriv){ # deriv=FALSE
                val <- val + sum( d*wgt )
            } else {  # deriv=TRUE
                fac0 <- P2*(1-P2)*a2M
                val[1] <- val[1] + sum(wgt*d*rowSums(fac0)/sigma )  # mu
                val[2] <- val[2] + sum(wgt*d*rowSums(fac0*Theta2)/sigma )  # sigma
            }
        } else { # simultan=TRUE
            P0 <- stats::plogis( aM * (Theta2-bM) )
            rowSums_P0 <- rowSums(P0)
            z1 <- rowSums_P1 - rowSums_P0
            d1 <- linking_2groups_power_loss(x=z1, pow=pow, eps=eps, deriv=deriv)
            z2 <- rowSums_P2 - rowSums_P0
            d2 <- linking_2groups_power_loss(x=z2, pow=pow, eps=eps, deriv=deriv)
            wgt_d <- wgt*(d1+d2)
            if (!deriv){  # deriv=FALSE
                val <- val + sum(wgt_d)
            } else {  # deriv=TRUE
                fac1 <- - P0*(1-P0)*aM
                fac2 <- P2*(1-P2)*a2M + fac1
                #* mu
                h1 <- d1*rowSums(fac1)+d2*rowSums(fac2)
                val[1] <- val[1] - sum(wgt*h1/sigma )
                #* sigma
                der <- sum(wgt*h1*Theta2/sigma )
                val[2] <- val[2] - der
                #* derivatives item discriminations
                fac_P0 <- P0*(1-P0)
                fac1 <- fac_P0*(Theta2-bM)
                der <- -colSums(wgt_d*fac1)
                val[ind_a] <- val[ind_a] + der
                #* derivatives item difficulties
                fac1 <- fac_P0*aM
                der <- colSums(wgt_d*fac1)
                val[ind_b] <- val[ind_b] + der
            }
        } # end simultan
    } # end symm

    return(val)
}

## File Name: fuzdiscr.R
## File Version: 0.093

fuzdiscr <- function( X, theta0=NULL, maxiter=200, conv=.0001 )
{
    if ( is.null(theta0) ){
        theta0 <- rep( 1/ ncol(X), ncol(X) )
    }
    theta <- theta0
    iter <- 0
    change <- 1000
    while( ( iter < maxiter ) & ( change > conv ) ){
        # update xsi
        thetaM <- matrix( theta,  nrow=nrow(X), ncol=ncol(X), byrow=TRUE)
        xsi <- thetaM * X  / rowSums( thetaM * X )
        # update theta
        theta <- colMeans(xsi)
        change <- max( abs( theta - theta0 ) )
        theta0 <- theta
        iter <- iter + 1
    }
    return(theta)
}

## File Name: rm_sdt_create_parm_index_rater.R
## File Version: 0.06


rm_sdt_create_parm_index_rater <- function( est.rater, ND, item.index,
    rater.index)
{
    g1 <- NULL
    if ( est.rater=="a"){ g1 <- seq_len(ND) }
    if ( est.rater=="r"){ g1 <- rater.index }
    if ( est.rater=="i"){ g1 <- item.index }
    if ( est.rater=="e"){ g1 <- rep(1,ND) }
    if ( est.rater=="n"){ g1 <- rep(-999,ND) }
    return(g1)
}

## File Name: regpolca_run_xxirt_random_starts.R
## File Version: 0.151

regpolca_run_xxirt_random_starts <- function(args, random_starts, sd_noise_init,
        par_item_init=NULL)
{
    partable1 <- partable <- args$partable
    customTheta1 <- customTheta <- args$customTheta
    NR <- nrow(partable)
    CP <- length(customTheta$par)
    opt0 <- Inf
    partable_opt <- NULL
    par_Theta_opt <- NULL
    rr_opt <- NULL
    if (!is.null(par_item_init)){
        args$partable$value <- par_item_init
    }
    if (random_starts>0){
        for (rr in 1L:random_starts){
            fac_rr <- sqrt(rr-1)
            sd_rr <- sd_noise_init*fac_rr/random_starts
            partable1$value <- partable$value + partable$est*stats::rnorm(NR, sd=sd_rr)
            args$verbose_index <- paste0('Random start ',rr)
            args$partable <- partable1
            customTheta1$par <- customTheta$par + stats::rnorm( CP, sd=sd_rr )
            args$customTheta <- customTheta1
            cat(paste0( '*** Random start ', rr, ' ***'), '\n')
            mod <- do.call(what=xxirt, args=args)
            par_Theta0 <- mod$customTheta$par
            partable0 <- mod$partable
            opt_val <- mod$opt_val
            if (opt_val < opt0){
                opt0 <- opt_val
                par_Theta_opt <- par_Theta0
                partable_opt <- partable0
                rr_opt <- rr
            }
        }
        partable1$value <- partable_opt$value
        args$partable <- partable1
        customTheta1$par <- par_Theta_opt
        args$customTheta <- customTheta1
    }
    return(args)
}

## File Name: rm_sdt_mstep_numdiff_diffindex.R
## File Version: 0.04


rm_sdt_mstep_numdiff_diffindex <- function(ll1, ll2, numdiff.parm,
        diffindex )
{
    ll_grad <- ( ll1 - ll2 ) / (2*numdiff.parm)
    ll_grad <- rowsum(ll_grad, diffindex )
    ll_grad <- ll_grad[ rownames(ll_grad) > 0, 1 ]
    return(ll_grad)
}

## File Name: rm_sdt_fill_init_partable.R
## File Version: 0.11


rm_sdt_fill_init_partable <- function(partable, par_index, tau.item, a.item,
    c.rater, d.rater, type)
{
    #--- collect parameters
    if( type=='item' ){
        pars <- list( tau.item=tau.item, a.item=a.item)
    } else {
        pars <- list( c.rater=c.rater, d.rater=d.rater)
    }
    #--- inits
    for (pp in names(pars)){
        par_pp <- par_index[[pp]]
        if ( length(par_pp) > 0 ){
            partable[ par_pp, "value" ] <- as.vector( pars[[ pp ]] )
        }
    }
    #-- output
    return(partable)
}

## File Name: mgsem_proc_model_include_missing_entries.R
## File Version: 0.11


mgsem_proc_model_include_missing_entries <- function(model_hh, types,
        entries=c("est","index"), I, D)
{
    for (tt in types){
        val <- 0
        if (tt=='NU'){ NR <- I; NC <- 1 }
        if (tt=='ALPHA'){ NR <- D; NC <- 1 }
        if (tt=='LAM'){ NR <- I; NC <- D }
        if (tt=='PHI'){ NR <- D; NC <- D }
        if (tt=='PSI'){ NR <- I; NC <- I }
        if (tt=='B'){ NR <- D; NC <- D }
        for (ee in entries){
            if (is.null(model_hh[[ee]][[tt]])){
                model_hh[[ee]][[tt]] <- matrix(val, nrow=NR, ncol=NC)
            }
        }
    }
    return(model_hh)
}

## File Name: lsem_lavaan_fit_measures.R
## File Version: 0.07

lsem_lavaan_fit_measures <- function(object, fit_measures)
{
    fM <- sirt_import_lavaan_fitMeasures(object=object, fit_measures=fit_measures)
    fit_measures <- intersect( fit_measures, names(fM))
    fM <- fM[ fit_measures ]
    return(fM)
}

## File Name: lsem.test.R
## File Version: 0.140

#**** test LSEM model based on bootstrap
lsem.test <- function( mod, bmod, models=NULL )
{
    parameters <- mod$parameters
    repl_factor <- bmod$repl_factor
    R <- bmod$R
    if (is.null(repl_factor)){
        repl_factor <- 1/(R-1)
    }

    parnames <- unique(paste(parameters$par))
    w <- mod$moderator.density$wgt
    NG <- length(w)
    bmod_missing <- missing(bmod)
    if (bmod_missing){
        parameters_boot <- NULL
    } else {
        parameters_boot <- bmod$parameters_boot
    }

    #* define design matrix
    A <- matrix(0, nrow=NG-1, ncol=NG)
    for (gg in 1L:(NG-1)){
        A[gg,gg] <- -1
        A[gg,gg+1] <- 1
    }

    wald_test_global <- NULL
    dfr <- NULL

    if (! bmod_missing){
        for (pp in parnames){
            ind_pp <- which( parameters$par==pp )
            parameters_pp <- parameters[ind_pp, ]
            theta <- parameters_pp$est
            par_boot_pp <- t( parameters_boot[ind_pp, ] )
            V <- stats::cov( par_boot_pp )*(R-1)*repl_factor
            M <- TAM::weighted_mean(x=theta, w=w)
            SD <- TAM::weighted_sd(x=theta, w=w)
            dfr1 <- data.frame(par=pp, M=M, SD=SD, chisq=NA, df=NA, p=NA)
            if (SD>1e-10){
                res <- lsem_wald_test(theta=theta, V=V, A=A)
                dfr1$chisq <- res$chisq
                dfr1$df <- res$df
                dfr1$p <- res$p
            }
            dfr <- rbind(dfr, dfr1)
        }
        wald_test_global <- dfr
    }


    #** run models
    R <- ncol(parameters_boot)
    NM <- length(models)
    test_models <- NULL

    if (NM>0){
        for (mm in 1L:NM){
            model_mm <- models[[mm]]
            pp <- names(models)[mm]
            ind_pp <- which( paste(parameters$par)==pp )
            y <- parameters[ind_pp,'est']
            dat <- data.frame(m=mod$moderator.grid, y=y, w=w)
            mod11 <- stats::lm(formula=model_mm, data=dat, weights=w)
            coef11 <- coef(mod11)
            parameters[ind_pp,'est'] <- predict(mod11)
            # compute residuals
            resid_pp <- resid(mod11)
            NC <- length(coef11)
            if (! bmod_missing){
                est_boot <- matrix(NA, nrow=NC, ncol=R)
                NRES <- length(resid_pp)
                resid_boot <- matrix(NA, nrow=NRES, ncol=R)
                rr <- 1
                for (rr in 1L:R){
                    dat$y <- parameters_boot[ind_pp,rr]
                    dat$w <- bmod$moderator_density_boot[,rr]
                    mod12 <- stats::lm(formula=model_mm, data=dat)
                    parameters_boot[ind_pp,rr] <- predict(mod12)
                    est_boot[,rr] <- coef(mod12)
                    resid_boot[,rr] <- resid(mod12)
                }
            }
            dfr1 <- data.frame(par=pp, coef=names(coef11), est=coef11)
            if (! bmod_missing){
                dfr1$se <- apply(est_boot, 1, stats::sd)
                dfr1$t <- dfr1$est / dfr1$se
                dfr1$p <- 2*stats::pnorm( -abs(dfr1$t) )
                # global Wald test for all parameters without intercept
                V <- stats::cov(t(est_boot))
                A <- matrix(0, nrow=NC-1, ncol=NC)
                for (cc in 1L:(NC-1)){
                    A[cc,cc+1] <- 1
                }
                res <- lsem_wald_test(theta=coef11, V=V, A=A)
                dfr1[1,'chisq_het'] <- res$chisq
                dfr1[1,'df_het'] <- res$df
                dfr1[1,'p_het'] <- res$p

                ## test whether all residuals are zero
                VR <- stats::cov(t(resid_boot))
                A0 <- diag(NRES)
                res1 <- lsem_wald_test(theta=resid_pp, V=VR, A=A0)
                dfr1[1,'chisq_fit'] <- res1$chisq
                dfr1[1,'df_fit'] <- NRES-NC
                dfr1[1,'p_fit'] <- res1$p

            }
            rownames(dfr1) <- NULL

            test_models <- rbind(test_models, dfr1)
        }  # end mm
    }

    #--- output
    res <- list(wald_test_global=wald_test_global, test_models=test_models,
                    parameters=parameters, parameters_boot=parameters_boot)
    return(res)
}

## File Name: mcmc_WaldTest.R
## File Version: 0.261


#** Wald Test for a set of hypotheses
mcmc_WaldTest <- function( mcmcobj, hypotheses )
{
    mcmcobj <- mcmc_extract_samples_first_chain(mcmcobj=mcmcobj)
    NH <- length(hypotheses)
    n1 <- ncol(mcmcobj)
    mcmcobj <- mcmc_derivedPars( mcmcobj=mcmcobj, derivedPars=hypotheses)
    n2 <- ncol(mcmcobj)
    mcmcobj <- mcmcobj[, seq(n1+1,n2) ]
    v1 <- mcmc_vcov(mcmcobj)
    s1 <- mcmc_summary(mcmcobj)
    c1 <- s1$MAP
    # compute test statistic
    v1a <- sirt_import_MASS_ginv(X=v1)
    v1_svd <- svd(v1)
    eps <- 1E-10
    NH <- sum( v1_svd$d > eps )
    W <- t(c1) %*% v1a %*% c1
    stat <- c( chi2=W, df=NH)
    stat['p'] <- 1 - stats::pchisq( W, df=stat['df'])
    res <- list( hypotheses_summary=s1, chisq_stat=stat)
    class(res) <- 'mcmc_WaldTest'
    return(res)
}

## File Name: gom_em_inits_lambda.R
## File Version: 0.05

gom_em_inits_lambda <- function(I, K, lambda.inits=NULL, lambda_partable=NULL)
{
    if (is.null(lambda.inits)){
        lambda <- matrix( seq( 1/(2*K), 1, 1/K), I, K, byrow=TRUE )
    } else {
        lambda <- lambda.inits
    }
    a1 <- stats::aggregate( as.vector(lambda), list(lambda_partable$par_index), mean )
    lambda <- matrix( a1[lambda_partable$par_index,2], nrow=I, ncol=K)
    return(lambda)
}

## File Name: rm_facets_ic.R
## File Version: 0.20

#########################################################################
# computation information criteria
rm_facets_ic <- function( dev, dat2, VV, RR, maxK, a.item.center,
        est.a.item, est.b.rater, est.a.rater, est.mean,
        b.rater.center, a.rater.center, b.rater.fixed,
        a.rater.fixed, tau.item.fixed_val, a.item.fixed
        )
{
    # Information criteria
    ic <- list( "deviance"=dev, "n"=nrow(dat2) )
    ic$VV <- VV
    ic$RR <- RR
    # item parameters
    # tau.item
    ic$np.item <- sum(maxK)
    if ( ! is.null( tau.item.fixed_val ) ){
        ic$np.item <- ic$np.item - sum( 1-is.na( tau.item.fixed_val ) )
    }
    # a.item
    b1 <- VV-a.item.center
    if ( ! is.null(a.item.fixed) ){
        b1 <- b1 - sum( ! is.na( a.item.fixed ) )
    }
    ic$np.item <- ic$np.item + est.a.item*b1
    #****
    #-- rater parameters
    ic$np.rater <- 0
    # b.rater
    b1 <- RR-b.rater.center
    if ( ! is.null(b.rater.fixed) ){
        b1 <- b1 - sum( ! is.na( b.rater.fixed ) )
    }
    ic$np.rater <- est.b.rater*b1
    # a.rater
    b1 <- RR-a.rater.center
    if ( ! is.null(a.rater.fixed) ){
        b1 <- b1 - sum( ! is.na( a.rater.fixed ) )
    }
    ic$np.rater <- ic$np.rater + est.a.rater*b1
    # distribution parameters
    ic$np.trait <- 1 + est.mean
    # estimated parameters
    ic$np <- ic$np.trait + ic$np.item + ic$np.rater

    #-- compute information criteria
    ic <- rm_ic_criteria(ic=ic)
    return(ic)
}
#########################################################################

## File Name: lsem_aggregate_statistics.R
## File Version: 0.02


lsem_aggregate_statistics <- function(x)
{
    Nimp <- length(x)
    w <- 0
    for (ii in 1L:Nimp){
        w <- w + x[[ii]]
    }
    w <- w / Nimp
    return(w)
}

## File Name: dmlavaan_extract_logLik_casewise.R
## File Version: 0.03

dmlavaan_extract_logLik_casewise <- function(mod)
{
    requireNamespace('lavaan')
    res <- unlist( lavaan::lavInspect(mod, what='loglik.casewise', list.by.group=FALSE) )
    return(res)
}

## File Name: mgsem_proc_technical.R
## File Version: 0.182

mgsem_proc_technical <- function(technical, control, p_me, p_pen, eps_approx, suffstat,
            estimator, diffpar_pen=NULL, cd_control=NULL)
{
    #*** defaults technical; update list
    technical0 <- list(maxiter=1000, h=1e-5, eps_zero=1e-12, optimizer='nlminb',
                        use_deriv=TRUE,    num_approx=FALSE,
                        eps_count_penal=1e-2,
                        approx_method='lp', use_rcpp_penalty=TRUE)
    use_approx_method <- ! is.null(technical$approx_method)
    technical <- mgsem_update_list_entries(add_list=technical, output_list=technical0)

    #*** update list cd_control
    cd_control0 <- list(maxiter=20, tol=5*1e-4, interval_length=0.05,
                            method='exact')
    cd_control <- mgsem_update_list_entries(add_list=cd_control, output_list=cd_control0)

    if ((p_pen==2) & (p_me==2) & ( ! use_approx_method) ){
        technical$approx_method    <- 'l2'
    }

    if (technical$approx_method=='l2' & ( (p_pen!=2) | ( p_me!=2) ) ){
        technical$approx_method    <- 'lp'
    }
    # if (technical$approx_method=='l2'){
    #     eps_approx <- 0
    # }

    #- optimizer
    if (technical$optimizer=='nlminb'){
        control$iter.max <- technical$maxiter
    }
    if (technical$optimizer=='optim'){
        control$maxit <- technical$maxiter
    }
    if (technical$optimizer=='bobyqa'){
        fac <- max(15, 3*nrow(suffstat[[1]][[2]]))
        technical$maxiter <- technical$maxiter*fac
        control$maxfun <- technical$maxiter
    }
    if (technical$optimizer=='nloptr'){
        control$maxeval <- technical$maxiter
    }

    p <- p_pen
    if (estimator=='ME'){
        p <- p_me
    }

    #--- output
    res <- list(control=control, technical=technical, eps_approx=eps_approx, p=p,
                    cd_control=cd_control)
    return(res)
}

## File Name: sia.sirt.R
## File Version: 0.304


#--- statistical implicative analysis
#--- simplified algorithm
sia.sirt <- function(dat, significance=.85 )
{
    TAM::require_namespace_msg('igraph')
    dat1 <- dat
    # order items
    dat1 <- dat1[, order( colMeans(dat1, na.rm=TRUE ) ) ]
    p <- colMeans(dat1, na.rm=TRUE )

    # handle missing responses
    dat1.resp <- 1*(1-is.na(dat1) )
    dat1[ is.na(dat1) ] <- 0
    I <- ncol(dat1)

    # total sample size
    ntot <- crossprod( dat1.resp )
    # n10
    n10 <- crossprod( dat1.resp * ( dat1==1 ), dat1.resp * ( dat1==0 ) )
    # n11
    n11 <- crossprod( dat1.resp * ( dat1==1 ), dat1.resp * ( dat1==1 ) )
    # calculate some probabilities
    p10 <- n10 / ntot       # X_i=1, X_j=0
    p11 <- n11 / ntot
    # item p values in every cell
    p1 <- diag( n11 ) / diag( ntot )
    p1M <- outer( p1, p1 )
    p0 <- 1-p1
    p0M <- 1 - p1M
    # probability under independence
    p10_ind <- outer( p1, p0 )
    # define effect size
    impl_int_es <- ( p10 - p10_ind  ) / sqrt( p10_ind )
    diag(impl_int_es) <- NA
    # implicative intensity t value
    impl_int_t <- sqrt( ntot ) * impl_int_es
    # significance value: implicative intensity
    impl_int <- 1 - stats::pnorm( impl_int_t )
    # look at significant implications
    impl_significance <- 1 * ( impl_int > significance )
    conf_loev <- p11 / outer( p1, rep(1,I) )
    diag( conf_loev) <- colMeans( dat1, na.rm=TRUE )

    #***
    # compute arrows: remove symmetric and transitive paths
    I1 <- impl_significance
    # remove symmetric implications
    #    I1[ ( I1==t(I1) ) & ( I1==1 ) ] <- 0
    for (ii in 1L:(I-1) ){
        for (jj in (ii+1):I){
            l2 <- I1[ii,jj] + I1[jj,ii]
            if (l2==2){
                if (impl_int[ii,jj] > impl_int[jj,ii] ){
                        I1[jj,ii] <- 0 } else { I1[ii,jj] <- 0 }
            } # end l2=2
        }
    }

    # remove transitive relations
    I1 <- sia_sirt_remove_transitive(I1=I1)

    # calculate potencies to look for connected graphs
    I1.pot <- I1

    # calculate matrix potence
    IS <- IS0 <- I1
    pp <- 2
    dev <- 1
    while ( dev > .5 ){
        IS.old <- IS
        G1 <-  1 * ( IS0 %*% I1 > 0 )
        IS0 <- G1
        I1.pot[ G1 > 0 ] <- pp
        IS <- 1 * ( IS + G1  > 0 )
        pp <- pp + 1
        dev <- sum ( abs( IS - IS.old ) )
    }

    # descriptives
    desc <- list( nodes=I  )
    # unconnected nodes
    ind.uc <- intersect( which( rowSums(I1)==0 ), which( colSums(I1)==0 ) )
    UC <- length( ind.uc )
    vars.uc <- rownames(I1)[ ind.uc ]
    if (  UC > 0 ){
        desc$nodes.unconnected <- UC
    }
    desc$edges <- sum(I1)
    desc$sign.implications <- sum( impl_significance, na.rm=TRUE)
    # descriptives on item level
    desc.item <- data.frame( item=colnames(dat1), p=p )
    desc.item$level <- g1 <- apply( I1.pot, 1, max )
# cat('\nc100')
    #****
    # recalculate levels
    g0 <- g1
    g0 <- sort( g0, decreasing=TRUE )
    g1 <- g0
    for (ii in 1L:(I-1) ){
        # ii <- 1
        vii <- names(g1)[ii]
        levii <- g1[ vii ]
        hzu <- which( I1[vii,] > 0 )
        if ( length( hzu) > 0 ){
            g1[ colnames(I1)[hzu] ] <- levii - 1
            g1 <- sort( g1, decreasing=TRUE )
        }
    }
    desc.item[ names(g1), 'level' ]  <- g1
    if (UC > 0 ){
        desc.item[vars.uc,'level'] <- NA
        g1[vars.uc] <- -1
    }
    # from objects and to objects
    desc.item$from <- colSums(I1)
    desc.item$to <- rowSums(I1)
    desc.item <- desc.item[ order( desc.item$level, decreasing=TRUE ), ]

    # create graph matrix
    dfr <- NULL
    dfr2 <- NULL
    g1 <- g1[ colnames(I1) ]
    CI1 <- paste0( colnames(I1), '\n(L', g1, ')')
    CI2 <- colnames(I1)
    for (ii in 1L:I){
        v1 <- which( I1[ii, ] > 0 )
        if ( length(v1) > 0 ){
            dfr.ii <- cbind( CI1[ii], CI1[ v1] )
            dfr <- rbind( dfr, dfr.ii )
            dfr2.ii <- cbind( CI2[ii], CI2[ v1] )
            dfr2 <- rbind( dfr2, dfr2.ii )
        }
    }
    if (UC > 0 ){
        v1 <- ind.uc
        dfr.ii <- cbind( CI1[v1], CI1[ v1] )
        dfr <- rbind( dfr, dfr.ii )
        dfr2.ii <- cbind( CI2[v1], CI2[ v1] )
        dfr2 <- rbind( dfr2, dfr2.ii )
    }

    # create igraph object
    igraph.obj <-  igraph::graph.edgelist(dfr)
    # create Rgraphviz object

    #*****
    # OUTPUT
    res <- list( adj.matrix=I1, adj.pot=I1.pot, adj.matrix.trans=IS,
        desc=desc, desc.item=desc.item, impl.int=impl_int, impl.t=impl_int_t,
        impl.significance=impl_significance, conf.loev=conf_loev,
        graph.matr=dfr2, graph.edges=unique( c( dfr2[,1], dfr2[,2]  ) ),
        igraph.matr=dfr, igraph.obj=igraph.obj)
    return(res)
}

## File Name: mgsem_partable2model.R
## File Version: 0.152


mgsem_partable2model <- function(partable, model, index=FALSE)
{
    ND <- nrow(partable)
    entries <- c('est')
    if (index){
        entries <- c('est','index')
    }
    for (entry in entries){
        for (dd in 1L:ND){
            hh <- partable[dd,'group']+1
            type <- paste(partable[dd,'type'])
            mat <- model[[hh]][[entry]][[type]]
            mat[ partable[dd,'i1'], partable[dd,'i2'] ] <- partable[dd,entry]
            if (type %in% c('PHI','PSI') ){
                mat[ partable[dd,'i2'], partable[dd,'i1'] ] <- partable[dd,entry]
            }
            model[[hh]][[entry]][[ type ]] <- mat
        }
    }
    return(model)
}

## File Name: rm_sdt_print_progress.R
## File Version: 0.05

rm_sdt_print_progress <- function( dev, dev0, c.rater, c.rater0, d.rater, d.rater0,
    tau.item, tau.item0, a.item, a.item0, mu, sigma, iter, digits_parm=6, digits_trait=3, digits_deviance=4 )
{
    #--- print deviance
    rm_facets_print_progress_deviance( dev=dev, dev0=dev0, digits_deviance=digits_deviance, iter=iter )
    #--- item and rater parameters
    rm_facets_print_progress_parameter( parm=c.rater, parm0=c.rater0, parmlabel="c.rater", digits_parm=digits_parm )
    rm_facets_print_progress_parameter( parm=d.rater, parm0=d.rater0, parmlabel="d.rater", digits_parm=digits_parm )
    rm_facets_print_progress_parameter( parm=tau.item, parm0=tau.item0, parmlabel="tau.item", digits_parm=digits_parm )
    rm_facets_print_progress_parameter( parm=a.item, parm0=a.item0, parmlabel="a.item", digits_parm=digits_parm )
    #--- trait distribution
    rm_facets_print_progress_trait_distribution( parm=mu, parmlabel="Trait M  ", digits_trait=digits_trait )
    rm_facets_print_progress_trait_distribution( parm=sigma, parmlabel="Trait SD ", digits_trait=digits_trait )
    utils::flush.console()
}

## File Name: mcmc_plot.R
## File Version: 0.222


#-- mcmc plot
mcmc_plot <- function(mcmcobj, ...)
{
    mcmcobj <- mcmc_extract_samples_first_chain(mcmcobj=mcmcobj)
    x <- list( mcmcobj=mcmcobj )
    x$amh_summary <- mcmc_summary(mcmcobj)
    class(x) <- 'amh'
    amh_plot(x, ... )
}

## File Name: xxirt_nr_pml_opt_fun_R.R
## File Version: 0.06


xxirt_nr_pml_opt_fun_R <- function(prior_Theta, probs_items, freq1, freq2, W1,
                                W2_long, G, K, I, TP, NI2, eps )
{
    val <- 0

    #** first-order frequencies
    for (gg in 1L:G){
        for (ii in 1L:I){
            t1 <- 0
            for (hh in 1L:K){
                p1 <- sum( prior_Theta[,gg] * probs_items[ii,hh,] )
                t1 <- t1 + freq1[ii,hh,gg]*log(p1+eps)
            }
            val <- val + W1[ii]*t1
        }
    }

    #*** second-order frequencies
    for (gg in 1L:G){
        for (nn in 1L:NI2){
            ii1 <- W2_long[nn,'item1']
            ii2 <- W2_long[nn,'item2']
            w2ii <- W2_long[nn,'w2']
            t1 <- 0
            for (hh in 1L:K){
                for (kk in 1L:K){
                    p1 <- sum( prior_Theta[,gg] * probs_items[ii1,hh,] *
                                                probs_items[ii2,kk,])
                    t1 <- t1 + freq2[nn,hh,kk,gg]*log(p1+eps)
                }
            }
            val <- val + w2ii*t1
        }
    }

    #- output
    res <- list(vcal=val)
    return(res)
}

## File Name: rm_sdt_calc_probs_grm_rcpp.R
## File Version: 0.05


rm_sdt_calc_probs_grm_rcpp <- function(c.rater, d.rater, I, K, eps=0, use_log=FALSE)
{
    res <- sirt_rcpp_rm_sdt_calc_probs_grm_rater( c_rater=c.rater, d_rater=d.rater, I=I, K=K,
                    eps=eps, use_log=use_log)
    K1 <- K + 1
    res <- array(res, dim=c(I,K1,K1))
    return(res)
}

## File Name: sirt_pem_extract_dimension.R
## File Version: 0.03

sirt_pem_extract_dimension <- function(x)
{
    x_dim <- NULL
    if ( is.vector(x) ){
        x_dim <- length(x)
    }
    if ( is.matrix(x) | is.array(x) | is.data.frame(x) ){
        x_dim <- dim(x)
    }
    return(x_dim)
}

## File Name: mcmc_summary_print_information_criteria.R
## File Version: 0.141

mcmc_summary_print_information_criteria <- function(object)
{
    cat( 'Number of iterations','=', object$iter, '\n' )
    cat( 'Number of burnin iterations','=', object$burnin, '\n\n' )
    if ( ! is.null(object$description) ){
        cat( object$description, '\n')
    }
    cat('-----------------------------------------------------------------\n')
    cat( 'Dbar','=', round( object$ic$Dbar, 2 ), '\n' )#, ' | ' )
    cat( 'Dhat','=', round( object$ic$Dhat, 2 ), '\n' )#, ' | ' )
    cat( 'pD','=', round( object$ic$pD, 2 ),  ' | pD=Dbar - Dhat \n' )
    cat( 'DIC','=', round( object$ic$DIC, 2 ),  ' | DIC=Dhat + pD \n\n' )
}

## File Name: mgsem_smoothic_penalty.R
## File Version: 0.04


mgsem_smoothic_penalty <- function(x, eps, deriv=FALSE)
{
    if (deriv){  ## derivative
        res <- 2*x*eps / (x^2+eps)^2
        # 2*x*(x^2+eps) - x^2*(2*x)
    } else {  # no derivative
        res <- x^2 / (x^2 + eps)
    }
    return(res)
}

## File Name: sirt_import_psych_omega.R
## File Version: 0.02

sirt_import_psych_omega <- function(m, nfactors, ...)
{
    TAM::require_namespace_msg('psych')
    y <- psych::omega( m=m, nfactors=nfactors, ...)
    return(y)
}

## File Name: lsem_fitsem_joint_estimation_partable_id.R
## File Version: 0.01

lsem_fitsem_joint_estimation_partable_id <- function(partable_gg, partable_mg, vv)
{
    partable_gg[,vv] <- partable_gg[,vv] + max(partable_mg[,vv], na.rm=TRUE)
    return(partable_gg)
}

## File Name: sirt_summary_print_computation_time_s1.R
## File Version: 0.07

sirt_summary_print_computation_time_s1 <- function(object)
{
    cat('Date of Analysis:', paste(object$s2 ), '\n' )
    cat('Computation Time:', print(object$s2 - object$s1), '\n\n')
}

## File Name: mgsem_ic.R
## File Version: 0.133

mgsem_ic <- function(opt_fun_output, opt_fun_args, partable, technical)
{

    ic <- list(estimator=opt_fun_args$estimator)
    ic$deviance <- -2*opt_fun_output$loglike
    ic$loglike <- opt_fun_output$loglike
    ic$pen <- -opt_fun_output$pen_all
    ic$n <- opt_fun_args$N

    ic$np_all <- max(partable$index)

    #-- extract free and penalized parameters
    partable <- partable[ partable$unique==1, ]
    ic$np_penal <- 0
    eps <- technical$eps_count_penal
    ic$np_penal <- sum( ( partable$pen_lp > 0 ) * ( abs(partable$est) < eps ) )
    ic$np <- ic$np_all-ic$np_penal

    # compute information criteria
    ic <- xxirt_ic_compute_criteria(ic=ic, compute_np=FALSE)

    #-- output
    return(ic)
}

## File Name: rasch_mml2_create_theta_k.R
## File Version: 0.06

rasch_mml2_create_theta_k <- function(Qmatrix, theta.k)
{
    if ( ( ! is.null( Qmatrix ) ) &  is.vector(theta.k) ){
        D <- ncol(Qmatrix)
        if ( D==2){ theta.k <- expand.grid( theta.k, theta.k ) }
        if ( D==3){ theta.k <- expand.grid( theta.k, theta.k, theta.k) }
        if ( D==4){ theta.k <- expand.grid( theta.k, theta.k, theta.k, theta.k) }
        if ( D==5){ theta.k <- expand.grid( theta.k, theta.k, theta.k, theta.k, theta.k) }
        if ( D==6){ theta.k <- expand.grid( theta.k, theta.k, theta.k, theta.k, theta.k,
                                    theta.k) }
        if ( D==7){ theta.k <- expand.grid( theta.k, theta.k, theta.k, theta.k, theta.k,
                                    theta.k, theta.k) }
        if ( D==8){ theta.k <- expand.grid( theta.k, theta.k, theta.k, theta.k, theta.k,
                                    theta.k, theta.k, theta.k) }
        if ( D==9){ theta.k <- expand.grid( theta.k, theta.k, theta.k, theta.k, theta.k,
                                    theta.k, theta.k, theta.k, theta.k) }
    }
    return(theta.k)
}

## File Name: write.fwf2.R
## File Version: 1.124


write.fwf2 <- function( dat, format.full, format.round, savename )
{
    if (is.null( colnames(dat) ) ){
        colnames(dat) <- paste( 'V', 1L:(ncol(dat)), sep='')
    }
    matr <- matrix( ' ', nrow=nrow(dat), ncol=ncol(dat))
    ind1 <- which( format.round <=0  )
    format.full[ ind1 ] <- format.full[ind1]
    format.round[ ind1 ] <- format.round[ind1]
    for (vv in 1L:(ncol(matr)) ){
        fvv <- format.round[vv]
        fff <- format.full[vv]
        matr[,vv] <- write.format2( vec1=dat[,vv], ff=fff, fr=fvv )
    }
    matr <- apply( matr, 1, FUN=function(ll){ paste( ll, collapse='' ) } )
    if ( is.vector(matr) ){
        writeLines( matr, paste( savename, '.dat', sep='') )
    } else {
        utils::write.table( matr, paste(savename, '.dat', sep=''),
                    row.names=FALSE, col.names=FALSE)
    }
    dfr <- data.frame( variable=colnames(dat),
                    begin=c( 1, cumsum( format.full )[ - ncol(dat) ] + 1 ),
                    end=cumsum( format.full), length=format.full )
    utils::write.table( dfr, paste( savename, '__LEGEND.txt', sep=''),
                quote=FALSE, row.names=FALSE, col.names=TRUE)
    return(dfr)
}

## File Name: rm_sdt_mstep_item_function_gradient.R
## File Version: 0.16


rm_sdt_mstep_item_function_gradient <- function(x, par_index, partable_item, Qmatrix, theta.k,
    VV, K, TP, pargroup_item, nik.item, numdiff.parm, eps=1E-10, conv_array=TRUE )
{
    res <- rm_sdt_fill_par_to_partable( par_index=par_index, partable=partable_item,
                parm0=x, type="item" )
    partable1 <- res$partable
    tau.item <- res$parm_list$tau.item
    a.item <- res$parm_list$a.item
    args <- list(a.item=a.item, tau.item=tau.item, Qmatrix=Qmatrix, theta.k=theta.k, VV=VV, K=K,
                TP=TP, eps=eps, use_log=FALSE )
    prob0 <- do.call( what=rm_sdt_calc_probs_gpcm_rcpp, args=args )
    args$use_log <- TRUE
    prob_D1_dim <- c(VV, K+1, TP)
    prob_D1 <- array(0, dim=prob_D1_dim )

    #*** loop over pargroups
    grad_out <- rep(NA, pargroup_item$np)
    for (pp in seq_len(pargroup_item$max_pargroup) ){
        pt_pp <- partable1[ partable1$pargroup==pp, ][1,]
        type_pp <- pt_pp$type
        #-- tau parameter
        if ( type_pp=="tau"){
            cat_pp <- pt_pp$col
            prob_D1 <- sirt_rcpp_rm_sdt_calc_gradient_item_deriv_tau( prob0=as.vector(prob0),
                                    prob_D1_dim=prob_D1_dim, cat_pp=cat_pp)
            prob_D1 <- array(prob_D1, dim=prob_D1_dim)
        }
        #-- a parameter
        if (type_pp=="a" ){
            prob_D1 <- sirt_rcpp_rm_sdt_calc_gradient_item_deriv_a( prob0=as.vector(prob0),
                            prob_D1_dim=prob_D1_dim, theta_k=theta.k )
            prob_D1 <- array(prob_D1, dim=prob_D1_dim)
        }
        #*** compute gradient
        ll_grad <- rm_sdt_calc_gradient_likelihood_item_llgrad2( logprob_D1=prob_D1,
                        nik.item=nik.item, diffindex=pargroup_item$pargroup_diffindex[[pp]], K=K,
                        prob_D1_dim=prob_D1_dim)
        grad_out[ pargroup_item$pargroup_index[[pp]] ] <- ll_grad
    }
    grad_ll <- - grad_out
    grad_prior <- rm_sdt_evaluate_prior_derivative(partable=partable1, h=numdiff.parm)
    grad_post <- grad_ll + grad_prior
    return(grad_post)
}

## File Name: isop.dich.R
## File Version: 3.17
######################################################################
# Fitting the ISOP and ADISOP model
isop.dich <- function( dat, score.breaks=NULL, merge.extreme=TRUE,
            conv=.0001, maxit=1000, epsilon=.025,
            progress=TRUE ){
    #*****************************************
    # convert matrix into matrix format
    dat <- as.matrix(dat)
    I <- ncol(dat)
    # define response matrix
    dat.resp <- 1-is.na( dat )
    if( max( dat, na.rm=TRUE ) > 1 ){
        stop("Use isop.poly for polytomous data!")
                    }
    if ( ! is.null(score.breaks) ){
        merge.extreme <- FALSE }
    # different scores of students
    stud.p <- rowMeans( dat, na.rm=TRUE )
    if (merge.extreme){
        score.breaks <- ( c(0,2:(I-1), I+1 ) -.5 ) / I
                      }
    if ( ! is.null( score.breaks) ){
        stud.p <- cut( stud.p, breaks=score.breaks, labels=FALSE)
                        }
    # different items
    item.p <- colMeans( dat, na.rm=TRUE )
    item.ps <- sort( item.p, index.return=TRUE)
    dat <- dat[,  item.ps$ix ]
    # score groups students
    scores <- sort( unique( stud.p ) )
    SC <- length(scores)
    # scoring of students
    res <- isop.scoring( dat )
    person <- res$person
    person$scoregroup <- stud.p
    item <- res$item
    p.itemcat <- res$p.itemcat
    score.itemcat <- res$score.itemcat
    # create item groups
    # create table
    freq.correct <- matrix( NA, SC, I )
    rownames(freq.correct) <- paste0("score_", scores )
    colnames(freq.correct) <- colnames(dat)
    wgt <- freq.correct
    # percent correct
    a1 <- stats::aggregate( dat==1, list( stud.p ), mean, na.rm=TRUE )
    freq.correct <- a1[,-1]
    # weights
    a1 <- stats::aggregate( dat.resp, list( stud.p ), sum, na.rm=TRUE )
    wgt <- a1[,-1]
    # ISOP Model
    res.isop <- fit.isop( freq.correct, wgt, conv=conv,
            maxit=maxit, progress=progress )
    f1 <- freq.correct
    f1 <- res.isop$fX
    # ADISOP Model (start with the fitted isop solution)
    res.adisop <- fit.adisop( freq.correct=f1, wgt, conv=conv, maxit=maxit,
            epsilon=epsilon, progress=progress  )
    wgt1 <- ( wgt / colSums( wgt ) ) / ncol(wgt)
    res.adisop$fit <- sqrt( sum( (  freq.correct - res.adisop$fX  )^2 * wgt1  ) )
    # logistic function
    res.logistic <- fit.logistic( freq.correct, wgt,
                        scores=scores / max( scores), item.p=item.ps$x,
                        conv, maxit, progress=progress )
    # fit generalized logistic function
#    res.genlogistic <- fit.genlogistic( freq.correct, wgt, scores=scores / max( scores), item.p=item.ps$x,
#                        conv, maxit, progress=progress )
    # fit alpha1 parameter
    #*******
    # collect likelihood values
    ll <- data.frame( "model"=c("saturated", "isop", "adisop", "logistic"),
       "ll"=c(res.isop$ll$ll.ind, res.isop$ll$ll.isop,
                res.adisop$ll$ll.adisop, res.logistic$ll$ll.logistic ) )
    ll$llcase <- c( res.isop$ll$llcase.ind, res.isop$ll$llcase.isop,
                      res.adisop$ll$llcase.adisop,
                      res.logistic$ll$llcase.logistic  )
    #*****
    # model fit
    fit <- c( res.isop$fit, res.adisop$fit, res.logistic$fit )
    names(fit) <- c("isop","adisop","logistic")
    # probability tables
    prob.isop <- calc.prob.from.freq.dich( fittedfreq=res.isop$fX )
    prob.adisop <- calc.prob.from.freq.dich( fittedfreq=res.adisop$fX )
    prob.logistic <- calc.prob.from.freq.dich( fittedfreq=res.logistic$fX )
    prob.saturated <- calc.prob.from.freq.dich( fittedfreq=freq.correct )

    #*******
    #* collect results
    res <- list( "freq.correct"=freq.correct, "wgt"=wgt,
              "prob.saturated"=prob.saturated,
              "prob.isop"=prob.isop, "prob.adisop"=prob.adisop,
              "prob.logistic"=prob.logistic,
              "ll"=ll, "fit"=fit,
              "person"=person, "item"=item,
              "p.itemcat"=p.itemcat, "score.itemcat"=score.itemcat,
              "fit.isop"=res.isop, "fit.adisop"=res.adisop,
              "fit.logistic"=res.logistic, "s2"=Sys.time(), "dat"=dat,
              "model"="isop.dich")
    class(res) <- "isop"
    return(res)
        }
##########################################################
# calculate probability table from fitted frequencies
calc.prob.from.freq.dich <- function( fittedfreq ){
         SC <- nrow(fittedfreq)
         I <- ncol(fittedfreq)
         prob1 <- array( NA, dim=c(I,2,SC ) )
         prob1[,2,] <- t(fittedfreq)
         prob1[,1,] <- 1-t(fittedfreq)
         return(prob1)
                }
#############################################################

## File Name: sirt_dmvnorm.R
## File Version: 0.114

sirt_dmvnorm <- function(x, mean=NULL, sigma=NULL, ... )
{
    TAM::require_namespace_msg('mvtnorm')
    if (is.matrix(x)){
        p <- ncol(x)
    } else {
        p <- 1
    }
    if (is.null(mean)){
        mean <- rep(0,p)
    }
    if (is.null(sigma)){
        sigma <- diag(p)
        if (p==1){
            sigma <- 1
        }
    }
    if (( p>1 ) | (is.matrix(x)) ){
        y <- mvtnorm::dmvnorm(x=x, mean=mean, sigma=sigma, ...)
    } else {
        y <- stats::dnorm(x=x, mean=mean, sd=sigma, ...)
    }
    return(y)
}

## File Name: rm_sdt_mstep_include_probs_args.R
## File Version: 0.04

rm_sdt_mstep_include_probs_args <- function(probs_args, parm_list,
    update_probs_args )
{
    for ( uu in update_probs_args){
        probs_args[[ uu ]] <- parm_list[[ uu ]]
    }
    return(probs_args)
}

## File Name: fit.logistic_alg.R
## File Version: 0.14


#####################################################################
# calculate log likelihood function in logistic model
.calc.ll.logistic <- function( theta, b, freq.correct, wgt ){
    eps <- 10^(-10)
    TP <- length(theta)
    I <- length(b)
    # calculate probabilities
    prob <- stats::plogis( theta + matrix( b, nrow=TP, ncol=I, byrow=TRUE) )
    # calculate log-likelihood
    ll <- freq.correct*wgt*log( prob + eps) + (1-freq.correct)*wgt*log( 1-prob+eps )
    res <- ll
    return(res)
    }
#####################################################################
# update theta parameter in logistic distribution
.update.theta.logistic <- function( theta, b, freq.correct, wgt,
        numdiff.parm, max.increment){
    #*****
    h <- numdiff.parm
    # update theta parameter
    ll0 <- .calc.ll.logistic( theta, b, freq.correct, wgt )
    ll1 <- .calc.ll.logistic( theta+h, b, freq.correct, wgt )
    ll2 <- .calc.ll.logistic( theta-h, b, freq.correct, wgt )
    ll0 <- rowSums(ll0)
    ll1 <- rowSums(ll1)
    ll2 <- rowSums(ll2)
    # derivative
    d1 <- ( ll1 - ll2  ) / ( 2 * h )    # negative sign?
    # second order derivative
    # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
    d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
    # change in item difficulty
    d2[ abs(d2) < 10^(-10) ] <- 10^(-10)
    increment <- - d1 / d2
#    ci <- ceiling( abs(increment) / ( abs( max.increment) + 10^(-10) ) )
#    increment <- ifelse( abs( increment) > abs(max.increment),
#                                 increment/(2*ci), increment )
    increment <- ifelse( abs( increment) > abs(max.increment),
                                     sign(increment)*max.increment, increment )
    theta <- theta + increment
    res <- list("theta"=theta, "ll"=sum(ll0) )
    return(res)
    }
##############################################################
#####################################################################
# update b parameter in logistic distribution
.update.b.logistic <- function( theta, b, freq.correct, wgt,
        numdiff.parm, max.increment){
    #*****
    h <- numdiff.parm

        # update theta parameter
        ll0 <- .calc.ll.logistic( theta, b, freq.correct, wgt )
        ll1 <- .calc.ll.logistic( theta, b+h, freq.correct, wgt )
        ll2 <- .calc.ll.logistic( theta, b-h, freq.correct, wgt )
        ll0 <- colSums(ll0)
        ll1 <- colSums(ll1)
        ll2 <- colSums(ll2)
        # derivative
        d1 <- ( ll1 - ll2  ) / ( 2 * h )    # negative sign?
        # second order derivative
        # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
        d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
        # change in item difficulty
        d2[ abs(d2) < 10^(-10) ] <- 10^(-10)
        increment <- - d1 / d2
#        ci <- ceiling( abs(increment) / ( abs( max.increment) + 10^(-10) ) )
#        increment <- ifelse( abs( increment) > abs(max.increment),
#                                     increment/(2*ci), increment )
        increment <- ifelse( abs( increment) > abs(max.increment),
                                     sign(increment)*max.increment, increment )

        b <- b + increment

    res <- list("b"=b, "ll"=sum(ll0) )
    return(res)
    }
##############################################################

## File Name: sirt_lavaan_partable_parnames.R
## File Version: 0.02

sirt_lavaan_partable_parnames <- function(partable)
{
    res <- paste0( partable$lhs, partable$op, partable$rhs )
    return(res)
}

## File Name: sirt_abs_smooth.R
## File Version: 0.07

sirt_abs_smooth <- function(x, deriv=0, eps=1e-4)
{
    .expr2 <- x^2 + eps
    y <- sqrt(.expr2)
    if (deriv==0){
        z <- y
    }
    if (deriv==1){
        z <- x / y
    }
    if (deriv==2){
        .expr4 <- 2 * x
        .expr5 <- 1 / y
        z <- .expr5 - x^2 * .expr2^(-1.5)
    }
    return(z)
}

## File Name: gom_em_ic.R
## File Version: 0.06

gom_em_ic <- function(dev, dat2, I, K, TP, model, weights, lambda_partable=NULL)
{
    ic <- list( deviance=dev, n=sum(weights) )
    ic$np.item <- I*K
    if (!is.null(lambda_partable)){
        ic$np.item <- max(lambda_partable$par_index)
    }
    if (model=="GOMRasch"){
        ic$np.item <- I
    }
    # trait matrix
    ic$np.trait <- TP - 1
    if (model=="GOMnormal"){
        ic$np.trait <- 2*(K-1) + (K-1)*(K-2)/2
    }
    if (model=="GOMRasch"){
        ic$np.trait <- 3 + 1
    }
    ic$np <- ic$np.item + ic$np.trait
    # AIC
    ic$AIC <- dev + 2*ic$np
    # BIC
    ic$BIC <- dev + ( log(ic$n) )*ic$np
    # CAIC (consistent AIC)
    ic$CAIC <- dev + ( log(ic$n) + 1 )*ic$np
    # corrected AIC
    ic$AICc <- ic$AIC + 2*ic$np * ( ic$np + 1 ) / ( ic$n - ic$np - 1 )
    #--- output
    return(ic)
}

## File Name: rm_sdt_mstep_type_function_value.R
## File Version: 0.16


rm_sdt_mstep_type_function_value <- function(x, par_index, partable, type,
    probs_args, probs_fun, probs_dim, update_probs_args, nik)
{
    res <- rm_sdt_fill_par_to_partable( par_index=par_index, partable=partable,
                                parm0=x, type=type )
    partable1 <- res$partable
    parm_list <- res$parm_list
    probs_args <- rm_sdt_mstep_include_probs_args(probs_args=probs_args,
                    parm_list=parm_list, update_probs_args=update_probs_args )
    probs <- do.call( what=probs_fun, args=probs_args )
    ll <- sirt_rcpp_rm_sdt_calc_gradient_likelihood_item_llgrad( logprob_D1=probs,
                        prob_D1_dim=probs_dim, nik_item=nik )
    ll <- -sum(ll)
    prior <- rm_sdt_evaluate_prior(partable=partable1)
    post <- ll + sum(prior)
    return(post)
}

## File Name: L0_polish_one_iteration.R
## File Version: 0.354


L0_polish_one_iteration <- function(x, tol, eps=0.01, maxiter=30, type=1)
{
    nr <- nrow(x)
    nc <- ncol(x)
    mod <- NULL
    num0 <- Inf
    elim0 <- NULL
    val0 <- Inf
    ind_x <- ! is.na(x)
    x_update <- x
    for (i in 1L:nr){
        for (j in 1L:nc){
            elim <- c(i,j)
            update <- FALSE
            x1 <- x
            if (ind_x[i,j]){
                x1[elim[1],elim[2]] <- NA
                mod1a <- L1_polish(x=x1, eps=eps, maxiter=maxiter, type=type)
                resid1 <- mod1a$residuals
                num <- sum( abs(resid1) > tol, na.rm=TRUE)
                val_tol <- mean( abs(resid1)*(abs(resid1) <=tol), na.rm=TRUE )
                val <- mean( abs(resid1), na.rm=TRUE )
                if (num < num0){ update <- TRUE }
                if ((num <=num0)&(val<val0)){ update <- TRUE  }
                if (update){
                    num0 <- num
                    val0 <- val
                    elim0 <- elim
                    mod <- mod1a
                    x_update <- x1
                }
            }
        }
    }
    #-- processing
    iterate_further <- TRUE
    max_resid <- max( abs(mod$residuals), na.rm=TRUE)
    if( max_resid <=tol ){
        iterate_further <- FALSE
    }

    #* final least squares estimate
    res1 <- L2_polish(x=x_update)
    row_fin_ls <- res1$row
    col_fin_ls <- res1$col
    wgt <- res1$wgt
    N_elim <- sum(1-wgt)

    #--- output
    res <- list(num=num0, elim=elim0, val=val0, overall=mod$overall,
                    row=mod$row, row_fin_ls=row_fin_ls, col=mod$col,
                    col_fin_ls=col_fin_ls, residuals=mod$residuals,
                    x_update=x_update, tol=tol, iterate_further=iterate_further,
                    max_resid=max_resid, wgt=wgt, N_elim=N_elim)
    return(res)
}

## File Name: brm.irf.R
## File Version: 0.121

#--- item response function (discretized) beta response model
brm.irf <- function( Theta, delta, tau, ncat, thdim=1, eps=1E-10 )
{
    TP <- nrow(Theta)
    K <- ncat
    # compute mid points
    mp <- seq( 1 / (2*(ncat) ), 1, 1/ncat  )
    probs <- matrix( 0, nrow=TP, ncol=K )
    eps <- 1E-10
    # compute beta shape parameters
    m1 <- Theta[,thdim] - delta + tau
    m1 <- exp( m1 / 2 )
    m2 <- - Theta[,thdim] + delta + tau
    m2 <- exp( m2 / 2 )
    for (cc in 1L:ncat){
        probs[,cc] <- stats::dbeta( mp[cc], shape1=m1, shape2=m2 )
    }
    probs <- probs + eps
    probs <- probs / rowSums(probs)
    return(probs)
}

## File Name: invariance_alignment_aligned_parameters_summary.R
## File Version: 0.07

invariance_alignment_aligned_parameters_summary <- function(x, label=NULL)
{
    dfr <- data.frame(Med=sirt_colMedians(x=x), M=colMeans(x=x, na.rm=TRUE),
                        SD=sirt_colSDs(x=x), Min=sirt_colMins(x=x),
                        Max=sirt_colMaxs(x=x) )
    if ( ! is.null(label) ){
        colnames(dfr) <- paste0( colnames(dfr), '.', label )
    }
    return(dfr)
}

## File Name: rasch.copula2.R
## File Version: 6.321



#----- Copula estimation in a Rasch type model
rasch.copula2 <- function( dat, itemcluster, weights=NULL,
        copula.type="bound.mixt", progress=TRUE, mmliter=1000, delta=NULL,
        theta.k=seq(-4,4,len=21), alpha1=0, alpha2=0, numdiff.parm=.000001,
        est.b=seq(1,ncol(dat)), est.a=rep(1,ncol(dat)), est.delta=NULL,
        b.init=NULL, a.init=NULL, est.alpha=FALSE,
        glob.conv=.0001, alpha.conv=.0001, conv1=.001,
        dev.crit=.2, increment.factor=1.01)
{
    s1 <- Sys.time()
    CALL <- match.call()
    group <- NULL
    # arrange item clusters item clusters
    t1 <- table(itemcluster)
    itemcluster[ which( itemcluster %in% names(t1)[ t1==1 ] ) ] <- 0
    itemcluster <- match( itemcluster, unique( sort(itemcluster[itemcluster!=0]) ) )
    itemcluster[ is.na( itemcluster ) ] <- 0
    t1 <- table(itemcluster)
    t1c <- t1[ names(t1) !=0 ]
    t1b <- as.numeric( names(t1))
    t1c <- t1[ names(t1) !=0 ]

    if ( any( t1c==1) ){
           stop( "There should be at least two items in an item cluster\n" )
                                    }
            x1 <- seq( 1, max(t1b) )
            x2 <- sort(setdiff( as.numeric(sort(names(t1))), 0 ))
            if ( sum( abs( x1-x2) ) > 1e-10 ){
               stop( "Item cluster identifiers must be recoded to 1, ..., C\n" )
                                 }
    CC <- length(x1) # number of clusters
    # calculation of number of itemclusters
    if (progress){
        cat("-----------------------------------------------------------------\n")
        cat("Marginal Maximum Likelihood Estimation \n")
        cat(paste( "Raschtype Copula Model with generalized logistic link function: Estimation of alpha1 and alpha2 \n") )
        cat("Function 'rasch.copula2'\n")
        cat("-----------------------------------------------------------------\n")
        utils::flush.console()
      }
     # arrange copula types
    if ( length( copula.type )==1 ){
        copula.type <- rep( copula.type, CC )
    }

    I <- ncol(dat)
    if ( is.null( colnames(dat))){
        colnames(dat) <- paste("Item", 1:I, sep="")
                }
    # remove cases where all responses are missings
    ind <- which( rowMeans( is.na(dat) ) < 1 )
    N0 <- nrow(dat)
    dat <- dat[ ind, ]
    Nmiss <- N0 - nrow(dat)
    if (Nmiss > 0){
        cat(paste("Removed ", Nmiss, " cases with only missing reponses\n",sep=""))
                }
    # groups
    if ( ! is.null( group) ){
        cat("Multiple groups are ignored!\n")
        cat("This option is only implemented in 'rasch.copula'\n")
#        groups <- unique( group )
#        G <- length( groups )
        G <- 1 ; group <- NULL
            } else { G <- 1 }
    GG <- G
    # data preparation (frequencies)
    dat10 <- dat00 <- dat
    dat10[ is.na( dat10) ] <- 9
    patt <- paste("P",dat10[,1],sep="")
    for (ii in 2:I){ patt <- paste( patt, dat10[,ii],sep="") }
    pattern <- data.frame( table(patt) )
    colnames(pattern) <- c("pattern", "freqwgt")

    use_weights <- FALSE
    if (!is.null(weights)){
        use_weights <- TRUE
        rs <- rowsum(weights, patt)
        ind <- match(rownames(rs), pattern$pattern)
        pattern$freqwgt <- rs[ind,1]
    }


    # pattern in data
#    pattern.in.data <- data.frame(match( patt, pattern$pattern )
    pattern.in.data <- patt

    # calculate frequencies in multiple group case
    if (G > 1){
        for (gg in 1:G){
        # gg <- 1
        t1 <- table( patt[ group==gg  ] )
        pattern <- merge( pattern, t1, by.x=1, by.y=1, all=TRUE )
                    }
        pattern[ is.na(pattern) ] <- 0
        colnames(pattern)[-c(1:2)]     <- paste("freqwgt", 1:G, sep="")
                }
    dat0 <- matrix( 0, nrow=nrow(pattern), I )
    for (ii in 1:I){
            dat0[,ii] <- as.numeric( substring( paste( pattern[,1] ), ii+1, ii+1 ) )
                }
    dat0[ dat0==9 ] <- NA
    colnames(dat0) <- colnames(dat)
    # define mu and sigma
    mu <- rep(0,G)
    sigma <- rep(1,G)
    if ( G > 1){
#            mu[2] <- 1 ; sigma[2] <- 1.2
                }

    #------------------------
    dat <- dat0
    dat2 <- dat
    dat2.resp <- 1 * ( !is.na( dat2) )
    dat2[ is.na(dat2) ] <- 0
        # data preparation in case of item clusters
        dat2.ld <- NULL        # dat2.ld is NULL if there are no item clusters
            if ( is.null(delta)){
                    delta <- stats::runif( CC, .3, .7 )
                    delta <- ifelse( copula.type=="frank", 1.3, delta )
#                    delta <- ifelse( copula.type=="cook.johnson", 1, delta )
                            }    # initial estimate of delta
            if ( is.null(est.delta)){ est.delta <- 1:CC }
            dp.ld <- as.list( 1:CC )

            # item pattern
            for (cc in 1:CC){
    #            cc <- 1

                icl.cc <- which( itemcluster==cc )

                # !!! This is the most time consuming function!
#                dp.ld.cc <- .calc.copula.itemcluster( D=length(icl.cc) )
#cat("   ***  calc copula itemcluster") ; aa1 <- Sys.time(); print(aa1-aa0) ; aa0 <- aa1
                dp.ld.cc <- .calc.copula.itemcluster2( D=length(icl.cc) )
#cat("   ***  calc copula itemcluster2") ; aa1 <- Sys.time(); print(aa1-aa0) ; aa0 <- aa1
# stop()
                dp.ld.cc$items <- icl.cc
                dp.ld.cc$N.items <- NCC <- length(icl.cc)
                dp.ld.cc$itemlabels <- colnames(dat)[icl.cc]
                # item selection for independent items
                m1 <- outer( rep(1,2^NCC), icl.cc)
                m2 <- dp.ld.cc$patt * m1   + ( 1 - dp.ld.cc$patt ) * ( m1 + I )
                m2 <- matrix( t(m2),  nrow=1, byrow=T )[1,]
                res <- list( "items"=m2 )
                res1 <- rep(NCC, 2^NCC )
                names(res1) <- rownames( dp.ld.cc$patt )
                res$N.Index1 <- res1
                dp.ld.cc$independent <- res
                # item selection for dependent items
                m2 <- ( 1 - dp.ld.cc$patt ) * ( m1 + I )
                m2 <- matrix( t(m2),  nrow=1, byrow=T )[1,]
                m2 <- c( m2[ m2> 0 ], 2*I + 1 )
                res <- list( "items"=m2 )
                res$N.Index1 <- rowSums( dp.ld.cc$patt==0 )
                res$N.Index1[ length(res$N.Index1) ] <- 1
                dp.ld.cc$dependent <- res
                dp.ld[[cc]] <- dp.ld.cc
                            }
            # create data frame with item response pattern
            dat2.ld <- matrix(0, nrow(dat2), CC )
            dat3.ld <- as.list( 1:CC )
            for (cc in 1:CC){
                # cc <- 1
                dp.cc <- dp.ld[[cc]]
                dat2.cc <- dat2[, dp.cc$items ]
                l1 <- "P"
                for ( vv in seq(1,ncol(dat2.cc))){
                    l1 <- paste( l1, dat2.cc[,vv], sep="")
                        }
                dat2.ld[, cc ] <- match( l1, rownames( dp.cc$patt ) )
                dat2.ld[ rowSums( dat2.resp[, dp.cc$items ]    ) < length(dp.cc$items), cc ] <- NA
                NRR <- nrow( dp.cc$patt )
                dat3.ld.cc <- sapply( seq( 1, NRR), FUN=function(aa){ 1*(dat2.ld[,cc]==aa ) } )
                dat3.ld.cc[ is.na(dat3.ld.cc) ] <- 0
                dat3.ld[[cc]] <- dat3.ld.cc
                            }
                # response indicator
                dat2.ld.resp <- 1 - is.na( dat2.ld )
                # set missings in dat2 to some arbitrary category
                dat2.ld[ is.na( dat2.ld ) ] <- 1
                # create dat2 data sets for local independence items
                itemcluster0 <- ind2 <- which( itemcluster==0 )
                bdat2.li.resp <- dat2.li.resp <- bdat2.li <- dat2.li <- NULL
                if ( length(ind2) > 0 ){
                        dat2.li <- dat2[, ind2, drop=FALSE]
                        dat2.li.resp <- dat2.resp[, ind2, drop=FALSE]
                        bdat2.li <- cbind( dat2.li, 1 - dat2.li )
                        bdat2.li.resp <- cbind( dat2.li.resp, dat2.li.resp )
                                        }
        # descriptives itemcluster
        Ncat.ld <- max( unlist( lapply( dp.ld, FUN=function(ll){ nrow(ll$patt) } ) ))
        # design table for estimating item difficulties
        b.design <- NULL
        if (length(itemcluster0) > 0){
            b.design <- data.frame( "itemcluster"=0,
                    "item"=itemcluster0, "b.indexgroup"=1 )
                                    }
        for (cc in 1:CC){
        #    cc <- 1
            g.cc <- ( dp.ld[[cc]] )$items
            bb <- data.frame("itemcluster"=cc,
                        "item"=g.cc, "b.indexgroup"=seq( 1, length(g.cc) ) )
            b.design <- rbind( b.design, bb )
                    }
        b.design <- b.design[ order( b.design$item ), ]
        b.design$est.b <- est.b
        t1 <- table( setdiff( est.b, 0 ) )
        if ( max(t1) > 1 ){    b.design$b.indexgroup <- 1:I }
        if ( sum( est.b==0 ) > 0 ){
                b.design[ est.b==0, "b.indexgroup"  ] <- b.design
                            }
        b1 <- setdiff( 1:max(b.design$b.indexgroup),
                        setdiff( b.design$b.indexgroup, 0 ) )
        if ( length(b1) > 0 ){     b.design$b.indexgroup <- 1:I }
    #########################################################################
    #--------------------------------------------------
    # initial estimate of item difficulty
#    b <- rasch.pairwise( dat, progress=FALSE)$item$itemdiff
    I <- ncol(dat2)
    if ( is.null( b.init) ){
        b <- - stats::qlogis( ( colMeans( dat00, na.rm=T ) + .005 ) / 1.01 )
                } else { b <- b.init }
    # initial estimate of (mean) item discrimination
    if ( is.null(a.init) ){ a <- rep( 1, I ) } else { a <- a.init }
    # density weights
    wgt.theta <- stats::dnorm(theta.k)
    wgt.theta <- wgt.theta / sum( wgt.theta )
    if ( G > 1){
        wgt.theta <- matrix(0, length(theta.k), G )
        for ( gg in 1:G){
            wgt.theta[,gg] <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg] )
            wgt.theta[,gg] <- wgt.theta[,gg] / sum( wgt.theta[,gg] )
                        }
                }
    iter <- 0
    # prelimanaries
    M2 <- outer( rep(1,nrow(pattern)), wgt.theta )
    # maximum increments
    hstep_b <- .6
    hstep_delta <- .2

    #************************************************************************
    #************************************************************************
    # BEGIN MARGINAL MAXIMUM LIKELIHOOD ESTIMATION
    dev <- 1 ; absdev.change <- par.change <- dev.change <- 1000
    res.posterior <- NULL
    maxalphachange <- 0
    while ( ( ( absdev.change > dev.crit | dev.change > glob.conv |
                    par.change > conv1 | maxalphachange > alpha.conv ) & iter < mmliter ) ){

# zz0 <- Sys.time()
    if (progress){
        cat( paste(rep("-", 70), collapse=""), "\n")
        k1 <- floor( log10(iter+1) )
        x1 <- "        |"
        x1 <- substring( x1, k1+1 )
        s1c <- Sys.time()
        cat( paste( paste( "MML EM Iter.", iter + 1 ), x1, paste( rep( "*", 10  ), collapse=""), "|  ",
                        s1c, "  ",
                        if ( iter > 0 ){ paste( round(difftime(s1c,s1b, units='secs' ),4), "secs" ) },
                        "\n",sep="") ) #
                        }
        s1b <- Sys.time()
        h <- numdiff.parm
        dev0 <- dev

        #************************************
        # estimation of b parameters
        b0 <- b
        # identify different b parameter groups
#        bG <- setdiff( unique( est.b ), 0 )
        bG <- setdiff( unique( b.design$b.indexgroup ), 0 )
        prbar <- seq( 1, 10, len=length(bG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        if (progress){
            cat(" Estimation of b:     |")
                }
        for (bb in bG){
# vv0 <- Sys.time()
            est.bb <- 1 * ( b.design$b.indexgroup==bb )
            b.design.bb <- b.design[ b.design$b.indexgroup==bb, ]
            if (bb==1 ){
                rescop <- .ll.rasch.copula20( theta.k, b0, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type,
                            Ncat.ld)
                res.posterior <- rescop
            }
            # is this really necessary?
#            wgt.theta <- rescop$pik

            rest1 <- .update.ll.rasch.copula21( theta.k, b0 + h*est.bb, alpha1, alpha2,
                a, dat2.li, itemcluster0,
                    CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                    bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type)

            rest2 <- .update.ll.rasch.copula21( theta.k, b0 - h*est.bb, alpha1, alpha2,
                    a, dat2.li, itemcluster0,
                    CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                    bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type)
            ll0 <- ll1 <- ll2 <- rep(0,I)
            # numerical derivatives independent items
            if ( rescop$calc.ind ){
            for (jj in 1:(length(itemcluster0) ) ){
                ll0[ itemcluster0[jj] ] <- sum( rescop$rjk0.1[,jj] * log( rescop$pjk.theta.k0[,jj] ) +
                                rescop$rjk0.0[,jj] * log( 1-  rescop$pjk.theta.k0[,jj] ) )
                ll1[ itemcluster0[jj] ] <- sum( rescop$rjk0.1[,jj] * log( rest1$pjk.theta.k0[,jj] ) +
                                rescop$rjk0.0[,jj] * log( 1-  rest1$pjk.theta.k0[,jj] ) )
                ll2[ itemcluster0[jj] ] <- sum( rescop$rjk0.1[,jj] * log( rest2$pjk.theta.k0[,jj] ) +
                                rescop$rjk0.0[,jj] * log( 1-  rest2$pjk.theta.k0[,jj] ) )
                                        } }
            itemcluster1 <- b.design.bb[ b.design.bb$itemcluster > 0, "item"]
            rjkCC <- rescop$rjkCC
            for (jj in 1:(length(itemcluster1) ) ){
                    cc <- b.design.bb[ b.design.bb$item==itemcluster1[jj], "itemcluster" ]
                    ll0[itemcluster1[jj]] <- sum( rjkCC[[cc]] * log( rescop$pjk.theta.kCC[[cc]] ) )
                    ll1[itemcluster1[jj]] <- sum( rjkCC[[cc]] * log( rest1$pjk.theta.kCC[[cc]] ) )
                    ll2[itemcluster1[jj]] <- sum( rjkCC[[cc]] * log( rest2$pjk.theta.kCC[[cc]] ) )
                                }
            a1 <- stats::aggregate( cbind( ll0, ll1, ll2 ), list(est.b), sum, na.rm=TRUE)
            ll0 <- a1[,2]
            ll1 <- a1[,3]
            ll2 <- a1[,4]
            b.change <- nr.numdiff( ll0=ll0, ll1=ll1, ll2=ll2, h=h )
#            hstep <- .5^( log(iter) )
            if (bb==bG[1] ){
                hstep_b <- hstep <- hstep_b * ( 1 / increment.factor )
                            }
            b.change <- ifelse( abs( b.change ) > hstep, hstep*sign(b.change), b.change )
            b.change <- b.change[ match( est.b, a1[,1] ) ]
            b <- b + b.change
            if (progress){
                    cat( paste( rep( "-", prbar[bb]), collapse="") )
                            }
            flush.console()

# cat("end b item") ; vv1 <- Sys.time(); print(vv1-vv0) ; vv0 <- vv1
                    }
        a1b <- max( abs( b - b0 ) )
        if (progress){
            cat("|     max. parm. change", round( a1b, 5),"\n")
                    }

        wm1 <- sum( theta.k * rescop$pik )
        wsd <- sqrt( sum( ( theta.k - wm1 )^2 * rescop$pik ) )
# cat("end b") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1
# stop()


        #******************************************************************************
        # estimation of a parameters
        a0 <- a
        # identify different a parameter groups
        aG <- setdiff( unique( est.a ), 0 )
        prbar <- seq( 1, 10, len=length(aG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        if ( progress ){ cat(" Estimation of a:     |") }
        for (aa in aG){
            est.aa <- 1 * (est.a==aa )
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp,
                            delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld)
            ll0 <- rescop$ll
            ll1 <- .update.ll.rasch.copula20( theta.k, b, alpha1, alpha2, a + h*est.aa,
                        dat2.li, itemcluster0,
                        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                        bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type )$ll
            ll2 <- .update.ll.rasch.copula20( theta.k, b, alpha1, alpha2, a - h*est.aa,
                        dat2.li, itemcluster0, CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,    bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type)$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
             if ( abs(d2) < 10^(-20) ){ d2 <- 10^20 }
            a.change <- - d1 / d2
            a.change <- ifelse( abs( a.change ) > .3, .3*sign(a.change), a.change )
            a.change <- a.change * est.aa
            a <- a + a.change
            a[ a < 0 ] <- .01
            if (progress){
                cat( paste( rep( "-", prbar[aa]), collapse="") )
                flush.console()
                        }
                            }

        if (progress){
            if ( length(aG) < 2 ){ cat( paste( rep( "-", 10 - length(aG) ), collapse="") ) }
                    }
        a1a <- max( abs( a - a0 ) )
        if (progress){
            cat("|     max. parm. change", round( a1a, 5),"\n")
                        }
# cat("end a") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1
        #******************************************************************************
        # estimation of delta parameters
        delta0 <- delta
        dG <- setdiff( unique( est.delta ), 0 )
        prbar <- seq( 1, 10, len=length(dG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        if (progress){
            cat(" Estimation of delta: |")
                }
        if ( length(dG)==0 ){ cat( paste(rep("-",10),collapse="") ) }
        if ( length(dG) > 0 ){
        # identify different a parameter groups
            est.cc <- 1 # * ( est.delta==cc )
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                                CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp,
                                delta, wgt.theta, I,
                                bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld )
#            ll0 <- rescop$ll
            rest1 <- .update.ll.rasch.copula21( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta + h*est.cc, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type)
            rest2 <- .update.ll.rasch.copula21( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta - h*est.cc, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type)
            ll0 <- ll1 <- ll2 <- rep(0,length(dG))
            rjkCC <- rescop$rjkCC
            for (cc in 1:CC){
                    ll0[cc] <- sum( rjkCC[[cc]] * log( rescop$pjk.theta.kCC[[cc]] ) )
                    ll1[cc] <- sum( rjkCC[[cc]] * log( rest1$pjk.theta.kCC[[cc]] ) )
                    ll2[cc] <- sum( rjkCC[[cc]] * log( rest2$pjk.theta.kCC[[cc]] ) )
                                }
            a1 <- stats::aggregate( cbind( ll0, ll1, ll2 ), list(est.delta), sum, na.rm=T)
            ll0 <- a1[,2]
            ll1 <- a1[,3]
            ll2 <- a1[,4]
            delta.change <- nr.numdiff( ll0=ll0, ll1=ll1, ll2=ll2, h=h )

            ct <- sapply( dG, FUN=function(dd){
                    ( copula.type[ est.delta==dd ] )[1] } )
            maxstep <- ifelse( copula.type=="bound.mixt", .2, .9 )
            #hstep <- maxstep^( log( 2+iter))

            hstep_delta <- hstep <- hstep_delta * ( 1 / increment.factor )

            delta.change <- ifelse( abs( delta.change ) > hstep,
                            hstep*sign(delta.change), delta.change )
            delta.change <- delta.change[ match( est.delta, a1[,1] ) ]
            delta <- delta + delta.change
            delta[ delta <=0 ] <- 2*numdiff.parm
            delta <- ifelse( copula.type=="bound.mixt" & ( delta > 1 ),
                        1 - 2 * numdiff.parm, delta )
            if (progress){
                cat( paste( rep( "-", 10 ), collapse="") )
                flush.console()
                        }
                            }
        a1d <- max( abs( delta - delta0 ) )
        if (progress){
                cat("|     max. parm. change", round( a1d, 5),"\n")
                        }
# cat("end delta") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1
        #******************************************************************************
        # estimation of alpha parameters
        alpha10 <- alpha1
        alpha20 <- alpha2
        prbar <- 5
        if (progress){ cat(" Estimation of alpha: |")         }
        # alpha1
        if (est.alpha){
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld)
            ll0 <- rescop$ll
            ll1 <- .update.ll.rasch.copula20( theta.k, b, alpha1 + h, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type)$ll
            ll2 <- .update.ll.rasch.copula20( theta.k, b, alpha1 - h, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type)$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            alpha.change <- - d1 / d2
            a1k1 <- alpha.change <- ifelse( abs( alpha.change ) > .1, .1*sign(alpha.change), alpha.change )
            alpha1 <- alpha1 + alpha.change
            }
            if (progress){
                cat( paste( rep( "-", prbar), collapse="") )
                flush.console()
                        }
        # alpha2
        if (est.alpha){
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld)
            ll0 <- rescop$ll
            ll1 <- .update.ll.rasch.copula20( theta.k, b, alpha1, alpha2+h, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster,pattern, GG, copula.type)$ll
            ll2 <- .update.ll.rasch.copula20( theta.k, b, alpha1, alpha2 -h, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster,pattern, GG, copula.type)$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            alpha.change <- - d1 / d2
            a1k2 <- alpha.change <- ifelse( abs( alpha.change ) > .1, .1*sign(alpha.change), alpha.change )
            alpha2 <- alpha2 + alpha.change
            }
            if (progress){
                cat( paste( rep( "-", prbar), collapse="") )
                flush.console()
                        }
        a1k <- max( abs( c( alpha1 - alpha10, alpha2 - alpha20 )) )
        if (progress){
            cat("|     max. parm. change", round( a1k, 5),"\n")
                        }
        #******************************************************************************
        # estimation of mu parameters
        a1m <- 0
        if (G>1){
        mu0 <- mu
        # identify different a parameter groups
        muG <- 1:(GG-1)
        prbar <- seq( 1, 10, len=length(muG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        if (progress){ cat(" Estimation of mu:    |") }
        for (gg in 2:G){
#            est.aa <- est.a * (est.a==aa )
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld)
            ll0 <- rescop$ll
            # mu + h
            w1 <- wgt.theta
            w2 <- stats::dnorm( theta.k, mean=mu[gg] + h, sd=sigma[gg] )
            w1[,gg] <- w2 / sum(w2)
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
                            wgt.theta=w1, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld)
            ll1 <- rescop$ll
            # mu - h
            w1 <- wgt.theta
            w2 <- stats::dnorm( theta.k, mean=mu[gg] - h, sd=sigma[gg] )
            w1[,gg] <- w2 / sum(w2)
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
                            wgt.theta=w1, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld )
            ll2 <- rescop$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            mu.change <- - d1 / d2
            mu.change <- ifelse( abs( mu.change ) > .3, .3*sign(mu.change), mu.change )
            mu.change <- mu.change * ( ( 1:G )==gg )
            mu <- mu + mu.change
            w2 <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg] )
            wgt.theta[,gg] <- w2 / sum(w2)
            cat( paste( rep( "-", prbar[gg-1]), collapse="") )
            flush.console()
                            }
        if ( length(muG) < 2 ){ cat( paste( rep( "-", 10 - length(muG) ), collapse="") ) }
        a1m <- max( abs( mu - mu0 ) )
        if (progress){ cat("|     max. parm. change", round( a1m, 5),"\n") }
                }            # end mu
        ######################################################################
        #******************************************************************************
        # estimation of sigma parameters
        a1s <- 0
        if (G>1){
        sigma0 <- sigma
        # identify different a parameter groups
#        h <- 10 * numdiff.parm
        sigmaG <- seq(1,GG-1)
        prbar <- seq( 1, 10, len=length(sigmaG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        if (progress){ cat(" Estimation of sigma: |") }
        for (gg in 2:G){
#            est.aa <- est.a * (est.a==aa )
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG,copula.type,Ncat.ld )
            ll0 <- rescop$ll
            # sigma + h
            w1 <- wgt.theta
            w2 <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg] +h)
            w1[,gg] <- w2 / sum(w2)
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
                            wgt.theta=w1, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type,Ncat.ld)
            ll1 <- rescop$ll
            # sigma - h
            w1 <- wgt.theta
            w2 <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg]-h )
            w1[,gg] <- w2 / sum(w2)
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
                            wgt.theta=w1, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type,Ncat.ld)
            ll2 <- rescop$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            sigma.change <- - d1 / d2
            sigma.change <- ifelse( abs( sigma.change ) > .3, .3*sign(sigma.change), sigma.change )
            sigma.change <- sigma.change * ( ( 1:G )==gg )
            sigma <- sigma + sigma.change
            w2 <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg] )
            wgt.theta[,gg] <- w2 / sum(w2)
#            cat( aa, " ") ;
            cat( paste( rep( "-", prbar[gg-1]), collapse="") )
            flush.console()
                            }
        if ( length(sigmaG) < 2 ){ cat( paste( rep( "-", 10 - length(sigmaG) ), collapse="") ) }
        a1s <- max( abs( sigma - sigma0 ) )
        if (progress){
                cat("|     max. parm. change", round( a1s, 5),"\n")
                    }
                }            # end sigma
        ######################################################################

# cat("other pars") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1
        iter <- iter + 1
#        thetawidth <- diff( theta.k )[1]
        #**********************
        # deviance rasch.mml
        # ll[gg] <- sum( dat1[,2] * log( rowSums( f.yi.qk[,] *
        #            outer( rep(1,nrow(f.yi.qk[,])), pi.k ) ) ) )
        dev <- - 2 * sum( pattern$freqwgt * log( rowSums(rescop$post.unnorm * M2 ) ) )
        #**********************
        # deviance tam
        # deviance <- - 2 * sum( pweights * log( res.hwt$rfx * thetawidth ) )
#        dev <- - 2 * sum( pattern$freqwgt * log( rowSums(rescop$post.unnorm) * thetawidth ) )
#        dev <- - 2 * sum( pattern$freqwgt * log( rowSums(rescop$post.unnorm * M2 ) ))
        dev.change <- abs( ( dev - dev0)/ dev0 )
        absdev.change <- abs( dev- dev0 )
        par.change <- max( a1a, a1b, a1d, a1k, a1m, a1s)
        if (progress){
            cat( "Deviance=",   round( dev, 5 ),
                    " | Deviance change=", - round( dev- dev0, 4 ),
                    "| max. parm. change=",  round( par.change, 6 ),  " \n"   )
            if ( ( dev > dev0 ) & ( iter > 4 ) ){
                cat("   Deviance has increased! Convergence Problems?\n")
                    }
# cat("rest") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1
            flush.console()
                }
            }
    # end MML iterations
    #**********************************************************************************
    #**********************************************************************************
    # Standard error estimation (This is a TO DO!)
    iterend <- iter

    ################################################
    # evaluation of posterior distribution
    post <- rescop$post
    M3 <- outer( rep(1,nrow(post)), theta.k )
    pattern$EAP <- rowSums( M3 * post )
    pattern$PostVar <- rowSums( M3^2 * post ) - pattern$EAP^2
    M.EAP <- weighted.mean( pattern$EAP, pattern$freqwgt )
    Var.EAP <- sum(( ( pattern$EAP - M.EAP )^2 * pattern$freqwgt )) / ( sum( pattern$freqwgt ) )
    MVar.EAP <- weighted.mean( pattern$PostVar, pattern$freqwgt )
    EAP.Rel <- Var.EAP / ( Var.EAP + MVar.EAP )

    #**** information criteria
    n <- nrow(dat00)
    w <- rep(1,n)
    if (use_weights){
        w <- weights
        n <- sum(w)
    }
    ic <- list( deviance=dev, n=n )
    bG <- setdiff( unique(est.b), 0 )
    aG <- setdiff( unique(est.a), 0 )
    dG <- setdiff( unique(est.delta ), 0 )
    ic[[ "np" ]] <- length(bG) + length(aG) + length(dG) + 2*est.alpha
    ic$AIC <- dev + 2*ic$np
    ic$BIC <- dev + ( log(ic$n) )*ic$np
    ic$CAIC <- dev + ( log(ic$n) + 1 )*ic$np
    ic$AICc <- ic$AIC + 2*ic$np * ( ic$np + 1 ) / ( ic$n - ic$np - 1 )

    #---- results item parameters
    N_item <- colSums((!is.na(dat00))*w)
    item <- data.frame( item=colnames(dat), N=N_item,
                p=colSums( w*dat00, na.rm=TRUE )/N_item,
                b=b, est.b=est.b, a=a, est.a=est.a )
    item$thresh <- item$a * item$b
    # add results dependency parameter for item clusters
    item$itemcluster <- itemcluster
    item$delta <- 0
    if (progress){
            cat("---------------------------------------------------------------------------------------------------------- \n")
                        }
    for (cc in 1:CC){
        # cc <- 1
        dcc <- dp.ld[[cc]]
        item[ dcc$items, "delta"] <- delta[cc]
                }
    item$est.delta <- 0
    i1 <- which( item$itemcluster > 0 )
    icl <- item$itemcluster[ i1 ]
    item$est.delta[ i1 ] <- est.delta[ icl ]
    # print item summary
    if (progress){
        cat("Parameter summary\n")
        sirt_print_helper( item, digits=3 )
    }
    # dependency parameter
    if (progress){
        cat("\nDependency parameters\n")
    }
    summary.delta <- data.frame( "cluster"=1:CC, "delta"=delta,
                "est.delta"=est.delta, "copula.type"=copula.type )
    summary.delta$items <- sapply( 1:CC, FUN=function(cc){
        paste( colnames(dat)[ itemcluster==cc ], collapse="-" )
                } )
    s2 <- Sys.time()
    if (progress){
        sirt_print_helper(summary.delta, digits=3)
        cat(paste("\nEAP Reliability:", round( EAP.Rel,3)),"\n\n")
        cat("Generalized logistic link function\n")
        cat("alpha1=",round(alpha1,3)," alpha2=", round(alpha2,3), " \n\n")
            # computational time
       cat("-----------------------------------------------------------------\n")
       cat("Start:", paste( s1), "\n")
       cat("End:", paste(s2), "\n")
       cat("Difference:", print(s2 -s1), "\n")
       cat("-----------------------------------------------------------------\n")
                }

    datalist <- list( pattern.in.data=pattern.in.data, dat0=dat0,
                    dat2=dat2, dat2.resp=dat2.resp, dat2.li=dat2.li,
                    dat2.ld=dat2.ld, dat2.li.resp=dat2.li.resp,
                    dat2.ld.resp=dat2.ld.resp, dp.ld=dp.ld, CC=CC,
                    bdat2.li=bdat2.li, bdat2.li.resp=bdat2.li.resp,
                    itemcluster0=itemcluster0, dat3.ld=dat3.ld
                            )


    # collect results
    v1 <- datalist$pattern.in.data
    patternindex <- match( v1, pattern$pattern )
    # person parameters
    person <- pattern[ patternindex, ]
    person$freqwgt <- NULL

    res <- list( "N.itemclusters"=CC, "item"=item, "iter"=iterend, "dev"=dev,
                    "delta"=delta, "b"=b, "a"=a, "mu"=mu, "sigma"=sigma,
                    "alpha1"=alpha1, "alpha2"=alpha2, "ic"=ic, "theta.k"=theta.k,
                    "pi.k"=wgt.theta, "deviance"=dev,
                    "pattern"=pattern, "person"=person,
                    "datalist"=datalist, "EAP.Rel"=EAP.Rel,
                    "copula.type"=copula.type, "summary.delta"=summary.delta,
                    "f.qk.yi"=(res.posterior$post)[ patternindex,],
                    "f.yi.qk"=(res.posterior$post.unnorm)[ patternindex,],
                    weights=weights, "s2"=s2, "s1"=s1, CALL=CALL )
    class(res) <- "rasch.copula2"
    return(res)
}

## File Name: rasch.evm.pcm.R
## File Version: 1.59


# estimation of the partial credit model employing the eigenvector method
rasch.evm.pcm <- function( dat, jackunits=20, weights=NULL,
    pid=NULL, group=NULL, powB=2, adj_eps=.3, progress=TRUE )
{
    CALL <- match.call()
    s1 <- Sys.time()
    dat0 <- dat <- as.matrix(dat)
    N <- nrow(dat)
    if ( is.null(weights) ){
        weights <- rep(1,N)
    }
    dat.resp <- 1-is.na(dat)
    dat[ is.na(dat) ] <- 0
    if ( length(jackunits)==1 ){
        jackunits <- ( 1:N ) %% jackunits
    }
    JJ <- length(unique(jackunits))
    maxK <- sirt_colMaxs(x=dat)
    I <- ncol(dat)
    # row indices of B
    dfr <- NULL
    for (ii in 1:I){
        dfr <- rbind( dfr, cbind( ii-1, 1:maxK[ii] )    )
    }
    row_index <- as.matrix(dfr)
    # column indices of B
    dfr <- NULL
    for (ii in 1:I){
        dfr <- rbind( dfr, cbind( ii-1, 1:maxK[ii] - 1) )
    }
    col_index <- as.matrix(dfr)
    powD <- powB
    if (progress){ progress <- 1 }

    #**************
    # subroutine multiple groups
    nogroup <- TRUE
    G <- 1
    if ( ! is.null(group) ){
        group0 <- group
        group.unique <- sort(unique(group))
        group <- match( group, group.unique)
        G <- length(group.unique)
        nogroup <- FALSE
    }

    #*** descriptives
    if (nogroup){
        group.unique <- NULL
        desc <- data.frame( Nobs=nrow(dat), G=1 )
        desc$N.items <- ncol(dat)
        desc$N.pars <- nrow(dfr)
        desc$sum.weights <- sum(weights)
    }
    if (!nogroup){
        desc <- data.frame( Nobs=as.numeric(table(group)), G=G )
        desc$N.items <- ncol(dat)
        desc$N.pars <- nrow(dfr)
        desc$sum.weights <- stats::aggregate( weights, list(group), sum)[,2]
    }
    m1 <- rowSums(dat.resp)
    desc$M.Nitems <- mean(m1)
    desc$SD.Nitems <- stats::sd(m1)
    desc$min.Nitems <- min(m1)
    desc$max.Nitems <- max(m1)

    #**** run Rcpp subroutine
    # no group
    if ( nogroup ){
        res1 <- sirt_rcpp_evm_comp_poly( dat=dat, dat_resp=dat.resp,
                    weights=weights, JJ=JJ, jackunits=jackunits, powD=powD, progress_=progress,
                    row_index=row_index, col_index=col_index )
    }  else {
        # groups
        res1 <- as.list(1:G)
        for (gg in 1:G){
            if (progress){
                cat("\n------- Group", group.unique[gg], " ------\n")
            }
            ind.gg <- which(group==gg)
            res1[[gg]] <- sirt_rcpp_evm_comp_poly( dat=dat[ind.gg,],
                                dat_resp=dat.resp[ind.gg,], weights=weights[ind.gg], JJ=JJ,
                                jackunits=jackunits, powD=powD, progress_=progress, row_index=row_index,
                                col_index=col_index )
        }
    }
    #************************
    # collect item parameters
    item_index <- row_index[,1]+1
    items <- colnames(dat)[ item_index ]
    ri <- paste0( items, "_Cat", row_index[,2]  )
    # item parameters
    if (nogroup){
        PP <- nrow(res1$PARS_vcov) - 2
    } else {
        PP <- nrow(res1[[1]]$PARS_vcov) - 2
    }
    item <- data.frame(parmlabel=ri, item=item_index, itemlabel=items,
                category=row_index[,2] )
    item$parmindex <- seq(1,PP)
    if ( nogroup){
        b_evm <- res1$b_evm
        item$est <- res1$b_evm
        item$se <- sqrt( diag( res1$PARS_vcov )[1:PP] )
    } else {
        for (gg in 1:G){
            rgg <- res1[[gg]]
            item[,paste0("est.Gr",group.unique[gg])] <- rgg$b_evm
            item[,paste0("se.Gr",group.unique[gg])] <- sqrt( diag( rgg$PARS_vcov )[1:PP] )
        }
    }
    if (nogroup){
        colnames(res1$B) <- rownames(res1$B) <- ri
        # jackknife bias corrected estimation
            # http://statweb.stanford.edu/~susan/courses/s208/node16.html
        item$est_jack <- item$est - (res1$JJadj - 1 ) * ( res1$PARS_means[1:PP] - item$est )
        PARS_vcov <- res1$PARS_vcov[ 1:PP, 1:PP ]
    }
    rownames(item) <- NULL
    #*************************
    # include person parameter estimation
    #   person parameter estimation: no multiple groups
    #   use function mle.pcm.group
    person <- NULL
    if ( nogroup){   # no groups
        row_index[,1] <- row_index[,1] + 1
        b <- matrix( NA, nrow=I, ncol=max( maxK) )
        rownames(b) <- colnames(dat)
        colnames(b) <- paste0("Cat", 1:max(maxK) )
        b[ row_index ] <- item$est
        #****
        # estimation subroutine
        person <- mle.pcm.group(dat, b, a=rep(1, ncol(dat)), pid=pid,
                        adj_eps=adj_eps, conv=1e-04, maxiter=30)$person
        B <- res1$B
        D <- res1$D
        #  PARS_vcov <- PARS_vcov
        JJadj <- res1$JJadj
    }
    if ( ! nogroup){   # groups
        row_index[,1] <- row_index[,1] + 1
        b0 <- as.list(1:G)
        for (gg in 1:G){
            b <- matrix( NA, nrow=I, ncol=max( maxK) )
            rownames(b) <- colnames(dat)
            colnames(b) <- paste0("Cat", 1:max(maxK) )
            b[ row_index ] <- item[,paste0("est.Gr",group.unique[gg])]
            b0[[gg]] <- b
        }
        b <- b0
        b_evm <- B <- D <- PARS_vcov <- JJadj <- as.list(1:G)
        for (gg in 1:G){
            rgg <- res1[[gg]]
            B[[gg]] <- rgg$B
            D[[gg]] <- rgg$D
            PARS_vcov[[gg]] <- rgg$PARS_vcov[ 1:PP, 1:PP ]
            JJadj[[gg]] <- rgg$JJadj
            b_evm[[gg]] <- rgg$b_evm
        }
    }

    #*** assess differential item functioning
    difstats <- NULL
    if ( ! nogroup ){
        difstats <- rasch_evm_pcm_dif( b_evm=b_evm, item=item,
                        PARS_vcov=PARS_vcov, I=I, G=G, group.unique=group.unique, dat=dat,
                        dat.resp=dat.resp )
    }

    # collect coefficients in case one group and multiple groups
    # create results for multiple groups
    if (G>1){
        ni <- nrow(item)
        Npars <- ni*G
        coeff <- rep( NA, Npars )
        pcov <- matrix( 0, nrow=Npars, ncol=Npars )
        for (gg in 1:G){
            ind.gg <- (gg-1)*ni + 1:ni
            coeff[ind.gg] <- item[, paste0("est.Gr", group.unique[gg] ) ]
            names(coeff)[ind.gg] <- paste0( item$parmlabel, "_Gr", group.unique[gg] )
            pcov[ind.gg,ind.gg] <- PARS_vcov[[gg]]
        }
        rownames(pcov) <- colnames(pcov) <- names(coeff)
        PARS_vcov <- pcov
    } else {
        coeff <- as.vector(item$est)
        names(coeff) <- item$parmlabel
        rownames(PARS_vcov) <- colnames(PARS_vcov) <- names(coeff)
    }
    #************************* output *******************************
    s2 <- Sys.time()
    res <- list( item=item, b=b, person=person, B=B, D=D, coef=coeff, vcov=PARS_vcov,
                JJ=JJ, JJadj=JJadj, powB=powB, maxK=maxK, G=G, desc=desc,
                difstats=difstats, b_evm=b_evm, I=I, group.unique=group.unique,
                dat=dat0, CALL=CALL, s1=s1, s2=s2)
    class(res) <- 'rasch.evm.pcm'
    return(res)
}

## File Name: gom_em_extract_lambda_matrix.R
## File Version: 0.01

gom_em_extract_lambda_matrix <- function(lambda_logit, I, K)
{
    lambda <- matrix( stats::plogis(lambda_logit), nrow=I, ncol=K)
    return(lambda)
}

## File Name: sirt_import_psych_cor.smooth.R
## File Version: 0.03

sirt_import_psych_cor.smooth <- function(x, ...)
{
    TAM::require_namespace_msg('psych')
    y <- psych::cor.smooth(x=x, ...)
    return(y)
}

## File Name: mcmc_summary.R
## File Version: 0.225



#**** mcmclist descriptives
mcmc_summary <- function( mcmcobj, quantiles=c(.025,.05,.50,.95,.975) )
{
    summary.mcmcobj <- summary(mcmcobj, quantile=quantiles)
    dat.bugs <- mcmc_extract_samples_first_chain(mcmcobj=mcmcobj)
    vars <- colnames(dat.bugs)
    n.iter <- nrow(dat.bugs)
    discret <- round( seq( 1, n.iter, length=4 ) )
    MAP <- Rhat <- rep(0,length(vars) )
    for (ii in seq( 1, length(vars )) ){
        vv <- vars[ii]
        dat.vv <- as.vector( dat.bugs[, vv ] )
        # extract 3 subchains
        l1 <- dat.vv[ discret[1]:discret[2] ]
        l2 <- dat.vv[ (discret[2]+1):discret[3] ]
        l3 <- dat.vv[ (discret[3]+1):discret[4] ]
        W <- ( stats::var(l1) + stats::var(l2) + stats::var(l3) ) / 3
        S1 <- n.iter / 3
        est.chains <- c( mean(l1), mean(l2), mean(l3) )
        B <-  S1 / 2 * sum( ( est.chains - mean( est.chains ) )^2 )
        Rhat[ii] <- ( ( S1-1 ) / S1 * W + B / S1 ) / W
        # mode estimation
        m1 <- stats::density( dat.vv, from=min(dat.vv),
                            to=max(dat.vv) )
        MAP[ii] <- m1$x[ which( m1$y==max( m1$y) ) ]
    }
    res <- data.frame( MAP=MAP, Rhat=Rhat )
    rownames(res) <- vars
    smc3  <- res
    smc2 <- summary.mcmcobj$statistics
    colnames(summary.mcmcobj$quantiles) <- paste0( 'Q', 100*quantiles )
    # calculate effective sample size
    effSize <- sirt_import_coda_effectiveSize( mcmcobj )
    statis <- summary.mcmcobj$statistics
    statis <- cbind( statis[, c(1,2) ],
                apply( as.matrix(mcmcobj), 2, stats::mad ),
                apply( as.matrix(mcmcobj), 2, skewness.sirt ),
                statis[,c(3,4) ]    )
    colnames(statis)[3:4] <- c('MAD', 'skewness' )
    dfr <- data.frame( parameter=rownames(smc3), statis, smc3,
                    SERatio=smc2[,4] / smc2[,2], sampSize=nrow(as.matrix(mcmcobj)),
                    effSize=effSize, summary.mcmcobj$quantiles )
    rownames(dfr) <- NULL
    return(dfr)
}

## File Name: gom_em_item_parameters.R
## File Version: 0.09


gom_em_item_parameters <- function(dat2, dat2.resp, model, b, lambda, K,
    weights, progress)
{
    item <- data.frame("item"=colnames(dat2))
    item$N <- colSums( weights*dat2.resp )
    item$p <- colSums( weights*dat2, na.rm=TRUE) / item$N
    item$b <- b
    if (model !="GOMRaschxxx"){
        for (kk in 1:K){
            item[,paste0("lam.Cl",kk)] <- lambda[,kk]
        }
    }
    obji <- item
    for (vv in seq(2,ncol(obji) )){
        obji[,vv] <- round( obji[,vv], 3 )
    }
    if (progress){
        cat("*********************************\n")
        cat("Item Parameters\n")
        print(obji)
    }
    #-- output
    return(item)
}

## File Name: sirt_pem_adjust_dimension.R
## File Version: 0.06

sirt_pem_adjust_dimension <- function(x, x_dim )
{
    if ( length(x_dim)==2 ){
        x <- matrix(x, nrow=x_dim[1], ncol=x_dim[2] )
    }
    if ( length(x_dim) >=3){
        x <- array(x, dim=x_dim )
    }
    return(x)
}

## File Name: noharm_sirt_est_residuals.R
## File Version: 0.183


#**** estimate residuals
noharm_sirt_est_residuals <- function( Fval, Pval, Fpatt, Ppatt,
    I, D,  b0.jk, b1.jk, b2.jk, b3.jk, wgtm, pm, Psival, Psipatt, modesttype )
{
    # compute dj
    dj <- sqrt( diag( Fval %*% Pval %*% t(Fval) ) )

    # compute ej
    ej <- sqrt( 1 + dj^2 )
    ej.ek <- 1 / TAM::tam_outer( ej, ej )
    diag( ej.ek ) <- 0
    v0.jk <- b0.jk
    v1.jk <- b1.jk * ej.ek
    v2.jk <- b2.jk * ej.ek^2
    v3.jk <- b3.jk * ej.ek^3
    # compute gamma.jk=f_j' P f_k
    gamma.jk <- Fval %*% Pval %*% t(Fval) + Psival
    # gamma.jk <- gamma.jk * ej.ek
    # compute p_d ' f_k
    pd.fk <- Fval %*% Pval
    Fval_old <- Fval
    Pval_old <- Pval
    residuals <- ( wgtm * ( pm - v0.jk - v1.jk*gamma.jk -
                                    v2.jk*gamma.jk^2 - v3.jk*gamma.jk^3 ) )

    #* fit statistics
    rmsr <- sqrt( sum( residuals^2 * wgtm ) / sum(wgtm) )
    RMW <- residuals * wgtm
    PMW <- pm * wgtm
    tanaka <- 1 - sum( diag( RMW %*% RMW ) ) /  sum(diag( PMW %*% PMW ))
    if (modesttype==2){
        tanaka <- rmsr <- NA
    }

    #--- output
    res <- list(residuals=residuals, rmsr=rmsr, tanaka=tanaka)
    return(res)
}

## File Name: rasch_pairwise_optimize.R
## File Version: 0.223


rasch_pairwise_optimize <- function(n.ij, n.ji, beta, zerosum, optimizer="nlminb",
        h=1e-4, maxiter=1000, estimator="MINCHI")
{
    I <- nrow(n.ij)
    ctl <- list(iter.max=maxiter)
    opt_fun1 <- function(x)
    {
        eps <- rasch_pairwise_compute_eps(x=x)
        t3 <- rasch_pairwise_optimize_opt_fun_terms(eps_horiz=eps,
                    eps_vert=eps, n.ij=n.ij, n.ji=n.ji)
        val <- sum(t3)
        return(val)
    }
    tol <- 1e-7
    if (estimator=='MINCHI'){
        y.ij <- n.ij / sqrt(n.ij+n.ji+tol)
        y.ji <- n.ji / sqrt(n.ij+n.ji+tol)
    }
    if (estimator=='ULS'){
        y.ij <- n.ij
        y.ji <- n.ji
    }
    opt_fun2 <- function(x)
    {
        eps <- rasch_pairwise_compute_eps(x=x)
        t3 <- rasch_pairwise_optimize_opt_fun_terms2(eps_horiz=eps,
                    eps_vert=eps, y.ij=y.ij, y.ji=y.ji, estimator=estimator)
        val <- sum(t3)
        return(val)
    }

    par_init <- beta - beta[1]
    par_init <- par_init[-1]

    grad_fun2 <- function(x)
    {
        eps <- rasch_pairwise_compute_eps(x=x)
        eps_h <- rasch_pairwise_compute_eps(x=x+h)
        eps_h2 <- rasch_pairwise_compute_eps(x=x-h)

        t2 <- rasch_pairwise_optimize_opt_fun_terms2(eps_horiz=eps_h,
                    eps_vert=eps, y.ij=y.ij, y.ji=y.ji, estimator=estimator)
        t3 <- rasch_pairwise_optimize_opt_fun_terms2(eps_horiz=eps,
                    eps_vert=eps_h, y.ij=y.ij, y.ji=y.ji, estimator=estimator)
        t21 <- rasch_pairwise_optimize_opt_fun_terms2(eps_horiz=eps_h2,
                    eps_vert=eps, y.ij=y.ij, y.ji=y.ji, estimator=estimator)
        t31 <- rasch_pairwise_optimize_opt_fun_terms2(eps_horiz=eps,
                    eps_vert=eps_h2, y.ij=y.ij, y.ji=y.ji, estimator=estimator)

        val2a <- rowSums(t2)
        val3a <- colSums(t3)
        val21 <- rowSums(t21)
        val31 <- colSums(t31)

        val2 <- val2a[-1]+val3a[-1]
        val1 <- val21[-1]+val31[-1]
        grad <- (val2-val1)/(2*h)
        return(grad)
    }

    res_opt <- sirt_optimizer(optimizer=optimizer, par=par_init, fn=opt_fun2,
                    grad=grad_fun2, control=ctl, hessian=FALSE)
    b <- c(0,-res_opt$par)
    if (zerosum){
        b <- b - mean(b)
    }
    eps <- exp(b)
    iter <- res_opt$iterations

    #--- output
    res <- list(eps=eps, b=b, iter=iter, res_opt=res_opt)
    return(res)
}

## File Name: dmlavaan_se_bootstrap.R
## File Version: 0.136

dmlavaan_se_bootstrap <- function(mod1, mod2, partable, R)
{
    parnames1 <- mod1$parnames
    NP1 <- mod1$NP
    parnames2 <- mod2$parnames
    NP2 <- mod2$NP
    est_boot1 <- matrix(NA, nrow=R, ncol=NP1)
    colnames(est_boot1) <- parnames1
    est_boot2 <- matrix(NA, nrow=R, ncol=NP2)
    colnames(est_boot2) <- parnames2
    partable1 <- mod1$partable
    partable1$start <- partable1$est
    partable2 <- mod2$partable
    partable2$start <- partable2$est

    #- reestimate models
    N <- mod1$N
    data <- mod1$args$data
    args1a <- mod1$args
    args1a$model <- partable1
    args2a <- mod2$args
    args2a$model <- partable2
    boot_sample <- matrix(0, nrow=N, ncol=R)
    rr <- 1
    while (rr<=R){
        # bootstrap data
        ind <- sort(sample(1L:N, N, replace=TRUE))
        t1 <- table(ind)
        boot_sample[ as.numeric( names(t1) ), rr ] <- as.numeric(t1)
        # half sampling
        data_boot <- data[ind,]
        args1a$data <- data_boot
        args2a$data <- data_boot
        # estimate models
        mod1a <- try( do.call(what=mod1$fun, args=args1a), silent=TRUE)
        mod2a <- try( do.call(what=mod2$fun, args=args2a), silent=TRUE)
        accept <- ( ! inherits(mod1a,'try-error')) & ( ! inherits(mod1a,'try-error'))
        if (accept){
            rr <- rr + 1
            est_boot1[rr-1,1L:NP1] <- coef(mod1a)
            est_boot2[rr-1,1L:NP2] <- coef(mod2a)
        }
    }
    est_boot1 <- dmlavaan_remove_duplicated_columns(x=est_boot1)
    est_boot2 <- dmlavaan_remove_duplicated_columns(x=est_boot2)

    #-- now compute standard error for inference
    # pars_sel <- which( ! is.na( partable$diff ) )
    pars_sel <- 1L:nrow(partable)
    pp <- pars_sel[1]
    for (pp in pars_sel){
        partable_pp <- partable[pp,]
        parname_pp <- partable_pp$parname
        ind_pp1 <- which( colnames(est_boot1)==parname_pp)[1]
        ind_pp2 <- which( colnames(est_boot2)==parname_pp)[1]
        if (length(ind_pp1)>0){
            partable$se1[pp] <- stats::sd(est_boot1[,ind_pp1])
        }
        if (length(ind_pp2)>0){
            partable$se2[pp] <- stats::sd(est_boot2[,ind_pp2])
        }
        if (length(ind_pp1)*length(ind_pp2)>0){
            z <- est_boot1[,ind_pp1]-est_boot2[,ind_pp2]
            partable$se_diff[pp] <- stats::sd(z)
        }
    }
    partable$t <- partable$diff / partable$se_diff
    partable$p <- 2*stats::pnorm(-abs(partable$t))

    #-- create bootstrap output objects
    res <- dmlavaan_se_bootstrap_create_est_boot(est_boot1=est_boot1,
                    est_boot2=est_boot2)
    est_boot <- res$est_boot
    V <- res$V

    #--- output
    res <- list(partable=partable, V=V, est_boot=est_boot,
                    boot_sample=boot_sample)
    return(res)
}

## File Name: noharm_sirt_optim_gradient.R
## File Version: 0.467


noharm_sirt_optim_gradient <- function(x, parm_table, parm_index, I, D,
        b0.jk, b1.jk, b2.jk, b3.jk, pm, wgtm, use_rcpp=FALSE )
{

    parm_table <- noharm_sirt_partable_include_par(par=x, parm_table=parm_table)
    Fmat <- noharm_sirt_create_parameter_matrices('F', parm_table=parm_table,
                            parm_index=parm_index)
    Pmat <- noharm_sirt_create_parameter_matrices('P', parm_table=parm_table,
                            parm_index=parm_index)
    Psimat <- noharm_sirt_create_parameter_matrices('Psi', parm_table=parm_table,
                            parm_index=parm_index)
    npar <- attr(parm_table, 'npar')

    # gamma values (diagonal)
    gamma_val <- noharm_sirt_implied_cov(Fmat=Fmat, Pmat=Pmat, Psimat=Psimat)
    FP <- Fmat %*% Pmat

    npar <- attr(parm_table, 'npar')
    NH <- attr(parm_table, 'NH')

    #- extract parameter table with free parameters
    parm_table_free <- parm_table[ attr(parm_table, 'parm_table_free_index'), ]

    #* computations
    if (use_rcpp){
        pt_index <- parm_table_free$index - 1
        pt_row <- parm_table_free$row -1
        pt_col <- parm_table_free$col - 1
        pt_matid <- parm_table_free$matid
        res <- sirt_rcpp_noharm_sirt_optim_gr_rcpp( gamma_val=gamma_val, NH=NH,
                    I=I, wgtm=wgtm, pm=pm, b0_jk=b0.jk, b1_jk=b1.jk, b2_jk=b2.jk,
                    b3_jk=b3.jk, npar=npar, pt_matid=pt_matid, pt_index=pt_index,
                    pt_row=pt_row, pt_col=pt_col, FP=FP, Fmat=Fmat, Pmat=Pmat,
                    Psimat=Psimat )
        grad0 <- res$grad0
    } else {
        grad0 <- noharm_sirt_optim_gradient_R( parm_table_free=parm_table_free,
                        Fmat=Fmat, Pmat=Pmat, Psimat=Psimat, FP=FP, npar=npar, NH=NH,
                        I=I, gamma_val=gamma_val, pm=pm, wgtm=wgtm, b0.jk=b0.jk,
                        b1.jk=b1.jk, b2.jk=b2.jk, b3.jk=b3.jk )
    }

    #-- output
    return(grad0)
}

## File Name: mgsem_create_index.R
## File Version: 0.04


mgsem_create_index <- function(x, all=TRUE, start=0, symm=FALSE, onlydiag=FALSE)
{
    ND <- prod(dim(x))
    if (start>0){
        v <- matrix( start + ( 1L:ND ) - 1, nrow=dim(x)[1], ncol=dim(x)[2])
        if (symm){
            v <- v + t(v)
            v <- as.vector(v)
            g1 <- match( v, unique(v) )
            v <- matrix( g1 + start - 1, nrow=dim(x)[1], ncol=dim(x)[2] )
        }
    } else {
        v <- 1+0*x
        if (!all){
            v <- 0*v
        }
    }
    #*** only diagonal entries
    if (onlydiag){
        ND <- dim(x)[1]
        if (start>0){
            v <- diag( start - 1 + 1L:ND )
        } else {
            v <- diag( rep(1, dim(x)[1]))
        }
    }
    return(v)
}

## File Name: mirt.wrapper.itemplot.R
## File Version: 0.08



mirt.wrapper.itemplot <- function( mirt.obj, ask=TRUE, ...)
{
    TAM::require_namespace_msg("mirt")
    I <- ncol( mirt.obj@Data$data )
    for (ii in 1:I){
        main <- paste0("Trace Lines of Item ", colnames( mirt.obj@Data$data )[ii] )
        print( mirt::itemplot(mirt.obj, item=ii, main=main, ...) )
        graphics::par(ask=ask)
    }
}

## File Name: class.accuracy.rasch.R
## File Version: 0.181


#*** classification accuracy in the Rasch model
class.accuracy.rasch <- function( cutscores, b, meantheta, sdtheta, theta.l,
        n.sims=0)
{
    dat0 <- matrix( 1, ncol=length(b), nrow=length(theta.l))
    cat("Cut Scores \n")
    print( cutscores, digits=3 )

    CC <- length(cutscores)+1 # number of classes
    L <- length(theta.l)    # number of theta points
    cutscores.long <- c(-Inf, cutscores, Inf )
    # MLE standard error
    semle <- rasch.info.mle(dat=dat0, theta=theta.l, b=b )
    wgttheta <- sirt_dnorm_discrete( theta.l, mean=meantheta, sd=sdtheta )
    # distribution of estimated theta values
    m1 <- matrix( NA, nrow=L, ncol=L )
    for (tt in 1:L ){
        w.tt <- sirt_dnorm_discrete( theta.l, mean=theta.l[tt], sd=semle[tt] )
        m1[tt,] <- w.tt
    }
    # calculate probability classification
    m2 <- matrix( NA, CC, CC )
    classes <- paste("Class", 1:CC, sep="")
    rownames(m2) <- paste("True", classes,sep="_" )
    colnames(m2) <- paste("Est", classes,sep="_")
    m0 <- matrix( wgttheta, L, L, byrow=FALSE  )
    for (cc in 1:CC){
        for (ee in 1:CC){
            ind.cc <- which( ( theta.l >=cutscores.long[cc] ) & ( theta.l < cutscores.long[cc+1] ) )
            ind.ee <- which( ( theta.l >=cutscores.long[ee] ) & ( theta.l < cutscores.long[ee+1] ) )
            m2[cc,ee] <- sum( m0[ind.cc, ind.ee ] * m1[ ind.cc, ind.ee ] )
        }
    }
    stats <- data.frame( agree0=sum( diag(m2)) )
    stats$agree1 <- sum( m2 * ( abs( outer( 1:CC, 1:CC, "-" ) ) <=1 ) )

    # calculate kappa calculation
    chance.prob <- rep(NA, CC)
    for (cc in 1:CC){
        ind.cc <- which( ( theta.l >=cutscores.long[cc] ) & ( theta.l < cutscores.long[cc+1] ) )
        chance.prob[cc] <- sum( m0[ ind.cc, 1 ] )
    }
    phi.c <- sum( chance.prob^2 )
    stats$kappa <- ( stats$agree0 - phi.c ) / ( 1 - phi.c )
    rownames(stats) <- "analytical"

    #*** accuracy by simulation
    if ( n.sims > 0 ){
        theta0 <- stats::rnorm(n.sims, mean=meantheta, sd=sdtheta )
        theta.sim <- data.frame( theta.true=theta0 )
        theta.sim$true.class <- as.numeric( paste( cut( theta0, breaks=cutscores.long, labels=1:CC) ))
        theta.sim$random.class <- sample( theta.sim$true.class )
        dat.sim <- sim.raschtype( theta=theta0, b=b )
        dat.sim2 <- sim.raschtype( theta=theta0, b=b )
        wle1 <- wle.rasch( dat.sim, b=b )
        theta.sim$WLE <- wle1$theta
        theta.sim$WLE.class <- as.numeric( paste( cut( theta.sim$WLE, breaks=cutscores.long, labels=1:CC )) )
        wle2 <- wle.rasch( dat.sim2, b=b )
        theta.sim$WLE2 <- wle2$theta
        theta.sim$WLE2.class <- as.numeric( paste( cut( theta.sim$WLE2, breaks=cutscores.long, labels=1:CC )) )
        # calculate WLE reliability
        v2 <- mean( ( theta.sim$WLE - theta0)^2 )
        wle.rel <- stats::var( theta0) / ( var( theta0) + v2 )
        stats2 <- stats
        rownames(stats2) <- "simulated"
        stats2$agree0 <- mean( theta.sim$true.class==theta.sim$WLE.class )
        stats2$agree1 <- mean( abs( theta.sim$true.class - theta.sim$WLE.class ) <=1 )
        phi.c <- mean( theta.sim$true.class==theta.sim$random.class )
        stats2$kappa <- ( stats$agree0 - phi.c ) / ( 1 - phi.c )
        stats <- round( rbind( stats, stats2 ), 3 )
        cat("\nWLE reliability (by simulation)=", round( wle.rel, 3), "\n" )
        stats[2,"consistency"] <- mean( theta.sim$WLE2.class==theta.sim$WLE.class )
        cat("WLE consistency (correlation between two parallel forms)=" )
        cat( round( stats::cor( theta.sim$WLE, theta.sim$WLE2 ), 3 ), "\n")
    }
    cat("\nClassification accuracy and consistency\n")
    print( stats, digits=3 )
    cat("\nProbability classification table \n")
    print( round(m2,3), digits=3 )
    res <- list( class.stats=stats, class.prob=m2)
    invisible(res)
}

## File Name: summary.linking.haberman.lq.R
## File Version: 0.123



summary.linking.haberman.lq <- function( object, digits=3, file=NULL, ... )
{
    # open sink
    sirt_osink(file=file)

    object$linking_slopes <- TRUE

    display_string <- sirt_summary_print_display(symbol='-', len=65)
    cat(display_string)

    #- package and R session
    sirt_summary_print_package_rsession(pack='sirt')

    cat('\n')
    cat(object$description, '\n')

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object$time)

    cat('Converged', '=', object$converged, '\n' )
    cat('Estimated power values','=', object$est_pow, '\n')
    cat('Loss function power slopes','=', object$pow_slopes, '\n')
    cat('Loss function power intercepts','=', object$pow_intercepts, '\n')
    cat('Epsilon Value', '=', object$eps, '\n' )
    cat('a_log', '=', object$a_log, '\n' )
    cat('use_nu', '=', object$use_nu, '\n' )

    # cat(display_string)
    # cat('Transformation parameters (Haberman linking)\n')
    # obji <- object$transf.pars
    # sirt_summary_print_objects(obji=obji, digits=digits, from=2, rownames_null=FALSE)

    cat('\nLinear transformation for item parameters a and b\n')
    obji <- object$transf.itempars
    sirt_summary_print_objects(obji=obji, digits=digits, from=2, rownames_null=FALSE)

    cat('\nLinear transformation for person parameters theta\n')
    obji <- object$transf.personpars
    sirt_summary_print_objects(obji=obji, digits=digits, from=2, rownames_null=FALSE)

    if (FALSE){
        cat('\n-----------------------------------------------------------------\n')
        if ( ! object$linking_slopes ){
            cat('Estimated DIF effects in logarithms of item slopes \n')
            obji <- object$a.resid[ object$selitems, ]
            sirt_summary_print_objects(obji=obji, digits=digits, from=1,
                            rownames_null=FALSE)
            cat('\n')
        }
        cat('Estimated DIF effects of item intercepts \n')
        obji <- object$b.resid[ object$selitems, ]
        sirt_summary_print_objects(obji=obji, digits=digits, from=1,
                            rownames_null=FALSE)

        cat('\n-----------------------------------------------------------------\n')
        digits2 <- 1
        if ( ! object$linking_slopes ){
            cat('Used items in linking item slopes \n')
            obji <- object$a.item_stat
            sirt_summary_print_objects(obji=obji, digits=digits, from=1,
                            rownames_null=FALSE)
            cat('\n')
        }
        cat('Used items in linking item intercepts \n')
        obji <- object$b.item_stat
        sirt_summary_print_objects(obji=obji, digits=digits, from=1,
                            rownames_null=FALSE)
    }

    # close sink
    sirt_csink( file=file )
}

## File Name: wle.rasch.jackknife.R
## File Version: 1.101


###############################################################################################
# Jackknife standard error estimation of WLE
wle.rasch.jackknife <- function( dat, b, itemweights=1+0*b, pid=NULL,
            testlet=NULL, stratum=NULL,
            size.itempop=NULL )
{
        # INPUT:
        # size.itempop ... finite sampling correction
        #
        #*************************************************
        # define person ID
        if( is.null(pid) ){ pid <- seq( 1, nrow(dat)) }
        # WLE estimate
        wle <- wle.rasch( dat=dat, b=b, itemweights=itemweights )
        dat.resp <- 1-is.na( dat )
        dfr <- est.bias <- NULL

        if ( ! is.null( size.itempop ) ){
            if ( is.null( names( size.itempop ) ) ){
                    names(size.itempop) <- sort( unique( stratum ) )
                            }}

        cat("Jackknife standard of WLE\n")
        #*********************************
        # stratified item sampling
        if ( ! is.null(stratum)){
            strata <- unique(stratum)
            cat("Stratified item sampling\n")
            S <- length(strata)
            wlevarstrata <- matrix( 0, nrow=nrow(dat), ncol=S )
            colnames(wlevarstrata) <- strata
            items.in.strata <- wlevarstrata
            cat(paste(S, "strata\n") )
            # save data response matrix
            dat.resp <- wle$dat.resp
            sk <- 0
            for ( ss in strata ){
#                ss <- strata[3]
                items.ss <- colnames(dat)[ stratum==ss ]
                II <- length(items.ss)
                wlejack <- matrix( 0, nrow=nrow(dat), ncol=II)
                for (ii in 1L:II){   # begin loop: items within a stratum
#                    ii <- 1
                    # calculate item index
                    ind.ii <- which( colnames(dat)==items.ss[ii] )
                    # calculate weight matrix of stratification
                    ind.ss <- which( stratum==ss )
                    dat.resp.ss <- dat.resp[, -ind.ii ]
                    # sum of itemweights within a stratum
                    sumitemweights <- rowSums( dat.resp[, ind.ss ] )
                    ind1 <- intersect( colnames(dat.resp.ss), items.ss )
                    h1 <- as.matrix( dat.resp.ss[, ind1 ], ncol=length(ind1))
                    dat.resp.ss[, ind1] <- sumitemweights / rowSums( h1 ) * dat.resp.ss[, ind1 ]
                    dat.resp.ss[ is.na(dat.resp.ss) ] <- 0
                    # WLE estimation
                    wlejack[,ii] <- wle.rasch( dat=dat[, - ind.ii ], dat.resp=dat.resp.ss,
                                            b=b[ - ind.ii  ],
                                            theta=wle$theta  )$theta
                                }       # end loop: items within a stratum
                    ind.resp.ss <- 1*( dat.resp[, items.ss ] > 0 )
                    wlejack <- wlejack * ind.resp.ss
                    N.wlejack <- rowSums( ind.resp.ss )
                    items.in.strata[,ss] <- N.wlejack

                    mean.wlejack <- rowSums( wlejack ) / N.wlejack
                    i1 <- rowSums( ( wlejack - outer( mean.wlejack, rep(1, ncol(wlejack) ) ) )^2 )
                    # finite sampling correction
                    if ( is.null(size.itempop[ss]) ){ fcs <- rep(1,nrow(dat)) } else
                                {
                                    fcs <- 1 - N.wlejack / size.itempop[ss]
                            cat( "\nStratum ", ss, "(Mean) Correction Factor", round(mean(fcs),5), "\n")
                                }
                    wlevarstrata[,ss] <- fcs * (N.wlejack -1 )/ N.wlejack  * i1
                    sk <- sk+1
                    cat(paste(sk, ".",sep="")) ;
                    utils::flush.console() ; if( sk %% 10==0 | sk==S ){ cat("\n") }
                        }   # end loop strata
            jack.se <- sqrt( rowSums(wlevarstrata )   )
            jackunits <- NA
                    }
        #*********************************
        # random sampling of testlets
        if ( ! is.null(testlet) ){
            testlets <- unique(testlet)
            cat("Simple random sampling of testlets\n")
            I <- length(testlets)
            wlejack <- matrix( 0, nrow=nrow(dat), ncol=I )
            testlet.ind <- wlejack
            cat(paste(I, "testlets\n") )
            testletcount <- rep(0,nrow(dat))
                for (ii in 1L:I){
                    # ii <- 1
                    wlejack[,ii] <- wle.rasch( dat=dat[, testlet !=testlets[ii] ], b=b[ testlet !=testlets[ii]  ],
                                            itemweights=itemweights[ testlet !=testlets[ii]  ],
                                            theta=wle$theta  )$theta
                    testletcount <- testletcount + ( rowSums( wle$dat.resp[, testlet==testlets[ii] ] ) > 0 )
                    testlet.ind[,ii] <- ( rowSums( wle$dat.resp[, testlet==testlets[ii] ] ) > 0 )
                    cat(paste(ii, ".",sep="")) ;
                    utils::flush.console() ; if( ii %% 10==0 | ii==I ){ cat("\n") }
                            }
                # number of Jackknife units
                jackunits <- testletcount
            # finite sampling correction
            N.items <- rowSums( wle$dat.resp > 0 )
            if ( is.null(size.itempop ) ){ fcs <- rep(1,nrow(dat)) } else
                    { fcs <- 1 - N.items / size.itempop }
        # include indicator matrix of testlet responses
                jack.se <- sqrt( fcs * (jackunits - 1) / jackunits * rowSums( testlet.ind*( wlejack - outer( wle$theta, rep(1,I) ) )^2  ) )
                        }
        #*********************************
        # Simple random item sampling
        if ( is.null(testlet) & ( is.null(stratum) ) ){     # begin simple random sampling
            I <- ncol(dat)
            wlejack <- matrix( 0, nrow=nrow(dat), ncol=I )
            cat("Simple random sampling of items\n")
            cat(paste(I, "items\n") )
            for (ii in 1L:I){
                # ii <- 1
                wlejack[,ii] <- wle.rasch( dat=dat[, -ii ], b=b[-ii], itemweights=itemweights[-ii],
                            theta=wle$theta  )$theta
                cat(paste(ii, ".",sep="")) ;
                utils::flush.console() ; if( ii %% 10==0 | ii==I ){ cat("\n") }
                        }
            # estimate WLE bias
            dfr <- data.frame( "wle"=wle$theta )
            UJ <- rowSums( dat.resp )
            mean.jack <- rowSums( wlejack, na.rm=T ) / UJ
            dfr$wle.jack <- UJ * wle$theta - (UJ-1)*mean.jack
            est.bias <- dfr$est.bias <- - dfr$wle.jack + dfr$wle
            # number of Jackknife units
            jackunits <- rowSums( wle$dat.resp > 0 )
            # finite sampling correction
            if ( is.null(size.itempop ) ){ fcs <- rep(1,nrow(dat)) } else
                    { fcs <- 1 - jackunits / size.itempop }
            jack.se <- sqrt( fcs * (jackunits - 1) / jackunits * rowSums( ( wle$dat.resp>0)*( wlejack - outer( wle$theta, rep(1,I) ) )^2  ) )
                            }   # end simple random sampling
        #*********************************
        # calculate and print reliability
        v1 <- stats::var(wle$theta)
        v2 <- mean( jack.se^2    )
        wle.rel <-     ( v1 - v2 ) / v1
        cat("WLE Reliability=", round(wle.rel,3), "\n")
        #*********************************
        # collect results
        dfr <- data.frame( "pid"=pid, "N.jackunits"=jackunits,
                            "sum.itemweights"=rowSums( wle$dat.resp),
                            "score"=rowSums( dat, na.rm=T), "max"=rowSums( 1 -is.na(dat) ),
                            "wle"=wle$theta, "wle.jackse"=jack.se,
                            "wlejack"=wlejack,
                            "est.bias"=ifelse( is.null( est.bias), NA, est.bias )
                            )
        res <- list( "wle"=dfr, "wle.rel"=wle.rel )
        return(res)
}
###############################################################################################

## File Name: linking_haberman_compute_median.R
## File Version: 0.02

linking_haberman_compute_median <- function(x, w)
{
    res <- linking_haberman_remove_missings_vector(x=x,w=w)
    x <- res$x
    w <- res$w
    res <- stats::quantile(x=x, weights=w,    probs=.5, na.rm=TRUE, ties=TRUE)
    return(res)
}

## File Name: rm_proc_create_pseudoraters.R
## File Version: 0.133

rm_proc_create_pseudoraters <- function( dat, rater, pid, reference_rater=NULL )
{
    dat0 <- dat
    rater0 <- paste0(rater)
    pid0 <- pid
    items <- colnames(dat)
    dat <- NULL
    pid <- NULL
    rater <- NULL
    I <- length(items)

    m0 <- as.data.frame( matrix(NA, nrow=nrow(dat0), ncol=I) )
    colnames(m0) <- colnames(dat0)

    for (ii in 1L:I){
        dat_ii <- m0
        dat_ii[, ii ] <- dat0[,ii]
        rater_ii <- paste0( rater0, '-', items[ii] )
        rater <- c( paste(rater), paste(rater_ii) )
        dat <- rbind( dat, dat_ii)
        pid <- c( pid, pid0)
    }

    if ( ! is.null(reference_rater) ){
        reference_rater <- paste0(reference_rater, '-', items )
    }

    #-- output
    res <- list( dat=dat, rater=rater, pid=pid, reference_rater=reference_rater)
    return(res)
}

## File Name: rm_sdt_fill_init_partables.R
## File Version: 0.04


rm_sdt_fill_init_partables <- function(partable_item, partable_rater,
    par_index, tau.item, a.item, c.rater, d.rater, type)
{
    partable_item <- rm_sdt_fill_init_partable( partable=partable_item,
                par_index=par_index, tau.item=tau.item, a.item=a.item, c.rater=c.rater,
                d.rater=d.rater, type='item' )
    partable_rater <- rm_sdt_fill_init_partable( partable=partable_rater,
                par_index=par_index, tau.item=tau.item, a.item=a.item, c.rater=c.rater,
                d.rater=d.rater, type='rater' )
    #--- output
    res <- list(partable_item=partable_item, partable_rater=partable_rater)
    return(res)
}

## File Name: mgsem_proc_model_add_specs_all.R
## File Version: 0.097


mgsem_proc_model_add_specs_all <- function(model, entries, type, ii, jj, dfr1,
        group, N_group, names_prior_list=NULL, pen_type="scad")
{
    # N1 <- N_group[group]
    # if (group==0){
    #     N1 <- sum(N_group)
    # }
    N1 <- 1
    NE <- length(entries)
    for (entry in entries){
        if (entry=='lower'){ default <- -Inf }
        if (entry=='upper'){ default <- Inf }
        if (entry=='prior'){ default <- 'none' }
        if (entry=='pen_l2'){ default <- 0 }
        if (entry=='pen_lp'){ default <- 0 }
        if (entry=='pen_difflp'){ default <- 0 }

        val <- mgsem_proc_model_add_specs(model=model, entry=entry, type=type,
                    ii=ii, jj=jj, default=default)
        if (entry=='prior'){ val <- paste(val) }
        if (entry %in% c('pen_l2','pen_lp','pen_difflp')){
            val <- N1*val
            if ( (entry=='pen_difflp') & (group==0)){
                val <- 0
            }
            #- no penalty
            if (pen_type=='none'){
                val <- 0
            }
        }

        #- correctness checks
        if (entry=='prior'){
            if ( ( val!='none') & ( ! val %in% names_prior_list ) ){
                v1 <- paste0( 'Specified prior in ', type, '[',
                        ii, ',', jj, '] in group ', group, ' not in \'prior_list\'!\n')
                stop(v1)
            }
        }
        dfr1[1,entry] <- val
    }
    return(dfr1)
}

## File Name: xxirt_mstep_ThetaParameters.R
## File Version: 0.249


xxirt_mstep_ThetaParameters <- function( customTheta, G, eps,
            mstep_iter, N.k, par1, mstep_reltol, Theta, penalty_fun_theta=NULL,
            weights=NULL, p.aj.xi=NULL)
{
    like_Theta <- function( x, ... )
    {
        par1 <- customTheta$par
        par1[ customTheta$est ] <- x
        arg_list <- list( par=par1, Theta=Theta, G=G )
        if (customTheta$person_covariates){
            arg_list$X <- customTheta$X
            prior1 <- t( do.call( what=customTheta$P, args=arg_list )     )
            # multiply by individual log-likelihood
            ll2 <-  - sum( weights * p.aj.xi * log( prior1 + eps ) )
        } else {
            mod1 <- do.call( what=customTheta$P, args=arg_list )
            ll2 <- - sum( N.k * log( mod1 + eps ) )
        }

        NP <- length(customTheta$prior)
        pen <- 0
        if ( NP > 0 ){
            for (pp in 1L:NP){
                if ( ! is.na( customTheta$prior[pp] ) ) {
                    prior_pp <- customTheta$prior[pp]
                    # val <- par1[ names(customTheta$prior) ]
                    val <- par1[pp]
                    arg_list <- list( val,customTheta$prior_par1[pp],
                                                customTheta$prior_par2[pp]  )
                    prior_val <- do.call( prior_pp, arg_list )
                    pen <- pen - log( prior_val + eps )
                }  # end if prior
            }  # end pp
        }
        ll2 <- ll2 + pen
        if (!is.null(penalty_fun_theta)){
            pen <- penalty_fun_theta(x=x)
            ll2 <- ll2+pen
        }
        return(2*ll2)
    }
    #----- end definition likelihood function

    mstep_method <- 'BFGS'
    lower <- upper <- NULL
    if (customTheta$some_bound){
        mstep_method <- 'L-BFGS-B'
        lower <- customTheta$lower
        upper <- customTheta$upper
    }
    arg_control <- list(maxit=mstep_iter)
    if ( mstep_method %in% c( 'BFGS')){
        arg_control$reltol <- mstep_reltol
    }
    if ( mstep_method %in% c( 'L-BFGS-B')){
        arg_control$factr <- mstep_reltol
    }
    # method L-BFGS-B uses 'factr' (and 'pgtol') instead of 'reltol' and 'abstol'
    arg_list <- list( par=par1, fn=like_Theta, method=mstep_method, control=arg_control)
    if (customTheta$some_bound){
        arg_list$lower <- lower
        arg_list$upper <- upper
    }
    mod <- do.call( what=stats::optim, args=arg_list )
    par1 <- mod$par
    ll2 <- mod$value
    customTheta$par[ customTheta$est ] <- par1
    par1 <- xxirt_ThetaDistribution_extract_freeParameters( customTheta=customTheta )

    pen_theta <- 0
    if (!is.null(penalty_fun_theta)){
        pen_theta <- penalty_fun_theta(x=par1)
    }

    #--- output
    res <- list( par1=par1, ll2=ll2, customTheta=customTheta, pen_theta=pen_theta)
    return(res)
}

## File Name: mgsem_proc_suffstat.R
## File Version: 0.099


mgsem_proc_suffstat <- function(suffstat)
{
    G <- length(suffstat)
    N_group <- rep(0,G)
    for (gg in 1L:G){
        suffstat_gg <- suffstat[[gg]]
        N_group[gg] <- suffstat_gg$N
        # rename entries
        ns_gg <- names(suffstat[[gg]])
        if( ! ( 'M' %in% ns_gg ) ){
            ind <- min( which( unlist( lapply(suffstat_gg, is.vector) ) ))
            ns_gg[ind] <- 'M'
        }
        if( ! ( 'S' %in% ns_gg ) ){
            ind <- min( which( unlist( lapply(suffstat_gg, is.matrix) ) ))
            ns_gg[ind] <- 'S'
        }
        names(suffstat_gg) <- ns_gg
        I <- length(suffstat_gg$M)

        #*** weights for moment-based estimation
        if ( ! ( 'weights_M' %in% ns_gg ) ){
            suffstat_gg$weights_M <- 1+0*suffstat_gg$M
        }
        if ( ! ( 'weights_S' %in% ns_gg ) ){
            suffstat_gg$weights_S <- 1+0*suffstat_gg$S
        }
        suffstat_gg$vech_weights_S <- mgsem_vech(x=suffstat_gg$weights_S)
        suffstat[[gg]] <- suffstat_gg
    }  # end gg
    N <- sum(N_group)

    #--- output
    res <- list(G=G, N=N, I=I, N_group=N_group, suffstat=suffstat)
    return(res)
}

## File Name: noharm_sirt_partable_include_par.R
## File Version: 0.02

noharm_sirt_partable_include_par <- function(par, parm_table)
{
    non_fixed <- attr(parm_table, 'non_fixed')
    include_index <- attr(parm_table, 'include_index')
    parm_table$est[ non_fixed ] <- par[ include_index ]
    return(parm_table)
}

## File Name: data.recode.sirt.R
## File Version: 0.02


#*** utility function for recoding a raw dataset
data.recode.sirt <- function( data.raw, keys )
{
    item.stat <- keys
    V <- ncol(data.raw)
    data.scored <- matrix( 0, nrow(data.raw), ncol(data.raw) )
    colnames(data.scored) <- colnames(data.raw )
    for (vv in 1L:V){
        data.scored[,vv] <- 1* ( paste(data.raw[,vv])==
                    paste(item.stat[ item.stat$item==colnames(data.raw)[vv], 'key' ]) )
        data.scored[ paste( data.raw[,vv] )=='NA', vv ] <- NA
    }
    return(data.scored)
}

## File Name: lsem_bootstrap_draw_bootstrap_sample.R
## File Version: 0.076

lsem_bootstrap_draw_bootstrap_sample <- function(data, sampling_weights,
    lsem_args, cluster=NULL, repl_design=NULL, rr=NULL)
{
    lsem_args1 <- lsem_args
    ind <- NULL
    used_repl_design <- ! is.null(repl_design)
    repl_vector <- NULL
    N <- attr(data, 'N')
    N1 <- N
    Nimp <- attr(data, 'Nimp')

    if (! used_repl_design){
        # no replication design
        if (is.null(cluster)){
            ind <- sample(1L:N, N, replace=TRUE)
        } else {
            cluster1 <- data[,cluster]
            t1 <- unique(cluster1)
            N <- length(t1)
            ind0 <- sort(sample(t1, size=N, replace=TRUE))
            ind <- NULL
            for (nn in 1L:N){
                v1 <- which(cluster1==ind0[nn])
                ind <- c(ind, v1)
            }
        }
        if (Nimp==0){
            lsem_args1$data <- data[ind,]
        } else {
            for (ii in 1L:Nimp){
                dat1 <- data[[ii]]
                dat1 <- dat1[ind,]
                data[[ii]] <- dat1
            }
            lsem_args1$data <- data
        }
        lsem_args1$sampling_weights <- sampling_weights[ind]

        # define replication design
        t1 <- table(ind)
        t1 <- data.frame( index=as.numeric(names(t1)),
                                freq=as.numeric(t1) )
        rownames(t1) <- NULL

        t2 <- data.frame(index=1L:N1, weight=sampling_weights)
        t1 <- merge(x=t1, y=t2, by='index', all=TRUE)
        t1$freq <- ifelse( is.na(t1$freq), 0, t1$freq )
        t1$repl_vector <- t1$freq * t1$weight
        repl_vector <- t1$repl_vector

    } else { # replication design
        lsem_args1$sampling_weights <- repl_design[,rr]
        repl_vector <- lsem_args1$sampling_weights
    }

    #--- arrange output
    res <- list(lsem_args1=lsem_args1, repl_vector=repl_vector,
                    used_repl_design=used_repl_design)

    return(res)
}

## File Name: lsem_fitsem_joint_estimation_partable.R
## File Version: 0.05


lsem_fitsem_joint_estimation_partable <- function(lavfit, G, par_invariant=NULL,
    par_linear=NULL, par_quadratic=NULL, pw_linear=1, pw_quadratic=1)
{
    partable <- sirt_import_lavaan_parameterTable(lavfit)
    partable$start <- partable$est
    partable_joint <- lsem_fitsem_joint_estimation_prepare_partable(partable=partable,
                                G=G, par_invariant=par_invariant, par_linear=par_linear,
                                par_quadratic=par_quadratic, pw_linear=pw_linear,
                                pw_quadratic=pw_quadratic)
    return(partable_joint)
}

## File Name: sirt_add_names.R
## File Version: 0.02

sirt_add_names <- function(x, names)
{
    if (! is.null(names)){
        if (is.vector(x)){
            names(x) <- names
        }
        if (is.matrix(x)){
            rownames(x) <- names
            colnames(x) <- names
        }
    }
    return(x)
}

## File Name: marginal.truescore.reliability.R
## File Version: 0.171


marginal.truescore.reliability <- function(  b, a=1+0*b, c=0*b, d=1+0*b,
        mean.trait=0, sd.trait=1, theta.k=seq( -6, 6, len=200) )
{
    TT <- length(theta.k)
    I <- length(b)
    phi.k <- sirt_dnorm_discrete( theta.k, mean=mean.trait, sd=sd.trait )
    phi.kM <- matrix( phi.k, nrow=TT, ncol=I )

    aM <- sirt_matrix2( a, nrow=TT)
    bM <- sirt_matrix2( b, nrow=TT)
    cM <- sirt_matrix2( c, nrow=TT)
    dM <- sirt_matrix2( d, nrow=TT)

    # item response functions
    theta.kM <- matrix( theta.k, nrow=TT, ncol=I )
    icc.theta <- cM + (dM-cM)* stats::plogis( aM*(theta.kM - bM ) )

    # calculate pi_i (predicted probabilities)
    pi.i <- colSums( icc.theta * phi.kM )

    #  expected number correct
    mu <- sum(pi.i)

    # compute error variance
    sig2.error <- colSums( icc.theta * ( 1 - icc.theta ) * phi.k )

    # compute total error variance
    sig2.total.error <- sum( sig2.error )

    # compute total true score variance
    iccmean <- rowMeans( icc.theta )
    sig2.total.tau <- sum( ( I*iccmean )^2 * phi.k ) - ( sum( I*iccmean*phi.k ) )^2

    # item level true score variance
    sig2.tau <- pi.i * ( 1 - pi.i ) - sig2.error

    # collect all formulas for item
    item <- data.frame( item=1:I, pi=pi.i, sig2.tau=sig2.tau, sig2.error=sig2.error )
    item$rel.item <- item$sig2.tau /  ( item$sig2.tau + item$sig2.error )

    rel.test <- sig2.total.tau / ( sig2.total.tau + sig2.total.error )
    cat('Reliability', '=', round( rel.test,3 ),'\n')
    # Formula (15)
    # sum( sqrt( outer( sig2.tau, sig2.tau ) ) )
    res <- list(rel.test=rel.test, item=item, pi=mu/I, sig2.tau=sig2.total.tau,
                    sig2.error=sig2.total.error )
    invisible(res)
}

## File Name: sirt_summary_label_equal_value.R
## File Version: 0.07

sirt_summary_label_equal_value <- function( label, value, label2="", digits=NULL )
{
    if ( ! is.null(digits) ){
        value <- round( value, digits)
    }
    res <- paste0( label, ' ', '=', ' ', value, ' ', label2 )
    return(res)
}

## File Name: summary.fuzcluster.R
## File Version: 0.13
#*******************************************************
summary.fuzcluster <- function( object,... ){
    # object      ... object from rasch.mml                #
    cat("---------------------------------------------------------------------------------------------------------- \n")
    d1 <- utils::packageDescription("sirt")
    cat( paste( d1$Package, " ", d1$Version, " (", d1$Date, ")", sep=""), "\n\n" )
    cat( "Date of Analysis:", paste( object$s2 ), "\n" )
    cat( "Random starts=", object$nstarts, "\n" )
    cat("Computation time:", print(object$s2 - object$s1), "\n\n")
    cat("---------------------------------------------------------------------------------------------------------- \n")
    cat( "Number of iterations=", object$iter,
          "(Seed=", object$seed, ")", "\n" )
    cat( "Deviance=", round( object$deviance, 2 ), " | " )
    cat( "Log Likelihood=", round( -object$deviance/2, 2 ), "\n" )
    cat( "Number of estimated parameters=", object$ic$np, "\n" )

    cat( "AIC=", round( object$ic$AIC, 2 ), " | penalty=", round( object$ic$AIC - object$ic$deviance,2 ),
            "   | AIC=-2*LL + 2*p  \n" )
    cat( "AICc=", round( object$ic$AICc, 2 )," | penalty=", round( object$ic$AICc - object$ic$deviance,2 ) )
        cat("    | AICc=-2*LL + 2*p + 2*p*(p+1)/(n-p-1)  (bias corrected AIC)\n" )
    cat( "BIC=", round( object$ic$BIC, 2 ), " | penalty=", round( object$ic$BIC - object$ic$deviance,2 ),
            "   | BIC=-2*LL + log(n)*p  \n" )
    cat( "CAIC=", round( object$ic$CAIC, 2 )," | penalty=", round( object$ic$CAIC - object$ic$deviance,2 ) )
        cat("   | CAIC=-2*LL + [log(n)+1]*p  (consistent AIC)\n\n" )

        cat("--------------------------------------------------- \n")
        cat(paste0( "Class probabilities (", object$K, " Classes)"),
                "\n")
        obji <- object$pi_est
        obji <- round( obji, 4)
        print( obji )
        cat("\nMeans\n")
        obji <- object$mu_est
        obji <- round( obji, 4)
        print( obji )
        cat("\nStandard deviations\n")
        obji <- object$sd_est
        obji <- round( obji, 4)
        print( obji )

            }
#*******************************************************

## File Name: pcm.conversion.R
## File Version: 0.12


#*** conversion of the partial credit parametrization
pcm.conversion <- function( b )
{
    maxK <- rowSums( 1 - is.na(b) )
    I <- nrow(b)
    # define delta
    delta <- b[ cbind( 1:I, maxK ) ] / maxK
    # define tau
    tau <- matrix( NA, nrow=I, ncol=ncol(b) )
    for (kk in 1:I){
        if ( maxK[kk]==1 ){
            tau[kk,1] <- 0
        }
        if ( maxK[kk] > 1){
            for (vv in 1:maxK[kk] ){
                t1 <- 0
                if (vv > 1){
                    t1 <- sum( tau[kk,seq(1,vv-1)] )
                }
                tau[kk,vv] <- b[kk,vv] - vv*delta[kk] - t1
            }
        }
    }
    res <- list( delta=delta, tau=tau )
    return(res)
}

## File Name: lq_fit.R
## File Version: 0.165

lq_fit <- function(y, X, w=NULL, pow=2, eps=1e-3, beta_init=NULL,
        est_pow=FALSE, optimizer="optim", eps_vec=10^seq(0,-10, by=-.5),
        conv=1e-4, miter=20, lower_pow=.1, upper_pow=5)
{
    N <- length(y)
    if (is.null(w)){
        w <- rep(1,N)
    }
    ind <- ! is.na(y)
    y <- y[ind]
    X <- X[ind,,drop=FALSE]
    w <- w[ind]
    if (is.null(beta_init)){
        mod <- stats::lm.wfit(y=y, x=X, w=w)
        beta_init <- mod$coefficients
    }

    #-- define optimization functions
    Xs <- sirt_rcpp_lq_fit_analyze_matrix(X=X)

    fct_optim <- function(x, y, X, pow, eps, w, Xs=NULL)
    {
        beta <- x
        if (pow==0){
            e <- y - X %*% beta
            val <- sum( e^2 / ( e^2 + eps ) )
        } else {
            val <- sirt_rcpp_lq_fit_fct_optim(Z=Xs, y=y, beta=beta, pow=pow, w=w,
                    eps=eps)
        }
        return(val)
    }

    if (pow>0){
        grad_optim <- function(x, y, X, Xs, pow, eps, w)
        {
            beta <- x
            # e <- ( y - X %*% beta )[,1]
            e <- sirt_rcpp_lq_fit_matrix_mult( Z=Xs, y=y, beta=beta)
            pow2 <- pow/2-1
            h1 <- pow*exp(pow2 * log( e^2 + eps ))*e*w
            # der <- - colSums(X*h1)
            px <- ncol(X)
            der <- sirt_rcpp_lq_fit_sparse_matrix_derivative(Z=Xs, h1=h1, px=px)
            return(der)
        }
    }

    if (pow==0){
        grad_optim <- function(x, y, X, Xs, pow, eps, w)
        {
            NP <- length(x)
            grad <- rep(NA, NP)
            e <- y - X %*% x
            for (pp in 1L:NP){
                grad[pp] <- sum(-2*e*eps/(e^2+eps)^2*X[,pp])
            }
            return(grad)
        }
    }

    #- define epsilon sequence
    eps_vec <- sirt_define_eps_sequence(eps=eps, eps_vec=eps_vec)
    pow00 <- pow

    #-- optimize
    for (eps in eps_vec){
        iterate_powers <- TRUE
        iter <- 0
        pow <- pow00
        while (iterate_powers){
            pow0 <- pow
            res_optim <- sirt_optimizer(optimizer=optimizer, par=beta_init, fn=fct_optim,
                            grad=grad_optim, y=y, X=X, Xs=Xs, pow=pow, eps=eps,w=w)
            beta <- res_optim$par
            converged <- res_optim$converged
            e <- ( y - X %*% beta )[,1]
            beta_init <- beta
            if (est_pow){
                pow <- lq_fit_estimate_power(e=e, pow_init=pow, lower_pow=lower_pow,
                            upper_pow=upper_pow)
            }
            pow_change <- abs(pow-pow0)
            if (( pow_change < conv ) | ( iter > miter ) ){
                iterate_powers <- FALSE
            }
            iter <- iter + 1
            # cat( paste0('eps=',eps, ' | pow=',pow,'\n'))
        }
    }

    #-- output
    res <- list(coefficients=beta, residuals=e, pow=pow, eps=eps,
                converged=converged, res_optim=res_optim, eps_vec=eps_vec)
    return(res)
}

## File Name: sim.raschtype.R
## File Version: 0.242



#---- simulation of Rasch type models
sim.raschtype <- function( theta, b, alpha1=0, alpha2=0, fixed.a=NULL,
        fixed.c=NULL, fixed.d=NULL )
{
    if ( is.null(fixed.a)){ fixed.a <- 1+0*b }
    if ( is.null(fixed.c)){ fixed.c <- 0*b }
    if ( is.null(fixed.d)){ fixed.d <- 1 + 0*b}
    # latent response (subtraction)
    latresp <- TAM::tam_outer( theta, b, op='-' )
    # include slope simulation
    TP <- length(theta)
    latresp <- sirt_matrix2( fixed.a, nrow=TP ) * latresp
    # transformed response
    cM <- sirt_matrix2( fixed.c, nrow=TP )
    dM <- sirt_matrix2( fixed.d, nrow=TP )
    trlat <- pgenlogis( latresp, alpha1=alpha1, alpha2=alpha2 )
    trlat <- cM + ( dM - cM )*trlat
    expprob <- trlat
    N <- nrow(expprob)
    I <- ncol(expprob)
    # define response matrix
    dat.resp <- 1 * ( expprob > matrix( stats::runif(N*I), ncol=I) )
    pot <- max( 2, floor(log10(I) + 1 ) )
    colnames(dat.resp) <- paste( 'I', substring( 10^pot + 1L:I,2), sep='')
    dat.resp <- as.data.frame(dat.resp)
    return(dat.resp)
}

## File Name: rasch.pml3_aux.R
## File Version: 5.12



######################################################################
# calculation of pairwise marginal likelihood
.ll.rasch.pml3.probit.est.b <- function( b, a, sigma, Q,eps.corr, itempairs, IP, eps=10^(-14),
        h, desb00, desb01, desb10, desb11, b.items ){
    cor.Sigma <- NULL
    # unidimensional case
    t1 <- a^2*sigma^2
    # define different b parameters
    xib0 <-   - b  / sqrt( 1 + t1 )
    xib1 <-   - (b+h)  / sqrt( 1 + t1 )
    xib2 <-   - (b-h)  / sqrt( 1 + t1 )
# a00 <- Sys.time()
    # 00
    res00 <- .pml3.est.b.aux( xib1=xib0, xib2=xib0, itempairs, a, sigma,
                eps.corr, cor.Sigma, eps )
    ll00 <- res00$ll

    # 10 / 01
    # 10: b + h ; b
    # 01: b ; b + h
    # 11: b+h ; b+h
    ll10 <- .pml3.est.b.aux( xib1=xib1, xib2=xib0, itempairs, a, sigma,
                eps.corr, cor.Sigma, eps )$ll
    ll01 <- .pml3.est.b.aux( xib1=xib0, xib2=xib1, itempairs, a, sigma,
                eps.corr, cor.Sigma, eps )$ll
    ll11 <- .pml3.est.b.aux( xib1=xib1, xib2=xib1, itempairs, a, sigma,
                eps.corr, cor.Sigma, eps )$ll
    # 20 / 02
    # 20: b-h;b
    # 02: b;b-h
    # 22: b-h;b-h
    ll20 <- .pml3.est.b.aux( xib1=xib2, xib2=xib0, itempairs, a, sigma,
                eps.corr, cor.Sigma, eps )$ll
    ll02 <- .pml3.est.b.aux( xib1=xib0, xib2=xib2, itempairs, a, sigma,
                eps.corr, cor.Sigma, eps )$ll
    ll22 <- .pml3.est.b.aux( xib1=xib2, xib2=xib2, itempairs, a, sigma,
                eps.corr, cor.Sigma, eps )$ll
# cat("calc all ll") ; a11 <- Sys.time(); print(a11-a00) ; a00 <- a11
#     ll.temp <- as.matrix( cbind( ll00, ll10, ll01, ll20, ll02 ) )
    ll0 <- t(ll00) %*% desb00
    ll1 <- t(ll01) %*% desb01 +  t(ll10) %*% desb10 + t(ll11) %*% desb11
    ll2 <- t(ll02) %*% desb01 +  t(ll20) %*% desb10 + t(ll22) %*% desb11
    # calculate increment
    incr <- nr.numdiff( ll0=as.vector(ll0), ll1=as.vector(ll1),
            ll2=as.vector(ll2), h=h, eps=10^(-10) )
# cat("calc incr") ; a11 <- Sys.time(); print(a11-a00) ; a00 <- a11
    change <- rep( 0, length(b) )
    change <- incr[ b.items ]
    change[ is.na(change ) ] <- 0
    change <- ifelse( abs(change) > 1, 1*sign(change), change )
    b <- b + change
    ll <- sum( ll00 )
# cat("add and mult") ; a1 <- Sys.time(); print(a1-a0) ; a0 <- a1
    res <-list( "ll"=ll, "itempairs"=res00$itempairs, "b"=b, "sigma"=sigma #,
#                "ind1"=ind1, "ind2"=ind2
                )
    return(res)
        }

######################################################################


#*********
.pml3.est.b.aux <- function( xib1, xib2, itempairs, a, sigma,
            eps.corr, cor.Sigma, eps ){
# a0 <- Sys.time()
    xi1 <- xib1[ itempairs[,"item1"] ]
    xi2 <- xib2[ itempairs[,"item2"] ]
    a1 <- a[ itempairs[,"item1"] ]
    a2 <- a[ itempairs[,"item2"] ]
    t1 <- a1*a2*sigma^2
    cor.Sigma <- ( t1 + eps.corr ) / ( 1 + t1 )
    cor.Sigma[ cor.Sigma > 1 ] <- .99
# cat("cor sigma") ; a1 <- Sys.time(); print(a1-a0) ; a0 <- a1
    pxi1 <- stats::pnorm( xib1 )
    pxi2 <- stats::pnorm( xib2 )
# cat("pnorm") ; a1 <- Sys.time(); print(a1-a0) ; a0 <- a1
    itempairs$p1.item1 <- pxi1[ itempairs$item1 ]
    itempairs$p1.item2 <- pxi2[ itempairs$item2 ]
    itempairs$p11 <- pbivnorm2( x=xi1, y=xi2, rho=cor.Sigma )
# cat("pbivnorm") ; a1 <- Sys.time(); print(a1-a0) ; a0 <- a1
    itempairs$p10 <- itempairs$p1.item1 - itempairs$p11
    itempairs$p01 <- itempairs$p1.item2 - itempairs$p11
    itempairs$p00 <- 1 - itempairs$p11 - itempairs$p01 - itempairs$p10
    ind1 <- which( colnames(itempairs) %in% c( "p11", "p10", "p01", "p00" ) )
# cat("ip2") ; a1 <- Sys.time(); print(a1-a0) ; a0 <- a1
#    itempairs[ itempairs[, ind1 ] < eps, ind1 ] <- eps
    for (ii in ind1){
        itempairs[, ii ] <- ifelse( itempairs[,ii] < eps, eps, itempairs[,ii] )
                }
# cat("ifelse") ; a1 <- Sys.time(); print(a1-a0) ; a0 <- a1
    ind2 <- which( colnames(itempairs) %in% c( "f11", "f10", "f01", "f00" ) )
#    ll <-  sum( log( itempairs[,ind1] ) * itempairs[,ind2]  )
    ll <-  rowSums( log( itempairs[,ind1] ) * itempairs[,ind2]  )
# cat("ll") ; a1 <- Sys.time(); print(a1-a0) ; a0 <- a1
    res <- list("ll"=ll, "itempairs"=itempairs )
# cat("rest") ; a1 <- Sys.time(); print(a1-a0) ; a0 <- a1
# stop("here")
    return(res)
        }
#********************************






######################################################################
# calculation of pairwise marginal likelihood
.ll.rasch.pml3.probit.est.a <- function( b, a, sigma, Q,eps.corr, itempairs, IP, eps=10^(-14),
        h, desa00, desa01, desa10, desa11, a.items ){
    cor.Sigma <- NULL
    a0 <- a
    a1 <- a+h
    a2 <- a-h

    # 00
    ll00 <- .pml3.est.a.aux( b, itempairs, a01=a0, a02=a0, sigma,
            eps.corr, cor.Sigma, eps )
    # 10 / 01 / 11
    ll10 <- .pml3.est.a.aux( b, itempairs, a01=a1, a02=a0, sigma,
            eps.corr, cor.Sigma, eps )
    ll01 <- .pml3.est.a.aux( b, itempairs, a01=a0, a02=a1, sigma,
            eps.corr, cor.Sigma, eps )
    ll11 <- .pml3.est.a.aux( b, itempairs, a01=a1, a02=a1, sigma,
            eps.corr, cor.Sigma, eps )
    # 20 / 02 / 22
    ll20 <- .pml3.est.a.aux( b, itempairs, a01=a2, a02=a0, sigma,
            eps.corr, cor.Sigma, eps )
    ll02 <- .pml3.est.a.aux( b, itempairs, a01=a0, a02=a2, sigma,
            eps.corr, cor.Sigma, eps )
    ll22 <- .pml3.est.a.aux( b, itempairs, a01=a2, a02=a2, sigma,
            eps.corr, cor.Sigma, eps )
    ll0 <- t(ll00) %*% desa00
    ll1 <- t(ll01) %*% desa01 +  t(ll10) %*% desa10 + t(ll11) %*% desa11
    ll2 <- t(ll02) %*% desa01 +  t(ll20) %*% desa10 + t(ll22) %*% desa11
    # calculate increment
    incr <- nr.numdiff( ll0=as.vector(ll0), ll1=as.vector(ll1), ll2=as.vector(ll2), h=h, eps=10^(-10) )
    change <- rep( 0, length(a) )
    change <- incr[ a.items ]
    change[ is.na(change ) ] <- 0
    change <- ifelse( abs(change) > .3, .3*sign(change), change )
    a <- a + change
    ll <- sum( ll00 )
    res <-list( "ll"=ll, "itempairs"=itempairs, "a"=a, "sigma"=sigma #,
#                "ind1"=ind1, "ind2"=ind2
                )
    return(res)
        }

######################################################################


#*********
.pml3.est.a.aux <- function( b, itempairs, a01, a02, sigma,
            eps.corr, cor.Sigma, eps ){

    # unidimensional case
    t11 <- a01^2*sigma^2
    t12 <- a02^2*sigma^2

    # define different b parameters
    xib1 <-  - b  / sqrt( 1 + t11 )
    xib2 <-  - b  / sqrt( 1 + t12 )

    xi1 <- xib1[ itempairs[,"item1"] ]
    xi2 <- xib2[ itempairs[,"item2"] ]

    a1 <- a01[ itempairs[,"item1"] ]
    a2 <- a02[ itempairs[,"item2"] ]

    t1 <- a1*a2*sigma^2
    cor.Sigma <- ( t1 + eps.corr ) / ( 1 + t1 )
    cor.Sigma[ cor.Sigma > 1 ] <- .99

    pxi1 <- stats::pnorm( xib1 )
    pxi2 <- stats::pnorm( xib2 )

    itempairs$p1.item1 <- pxi1[ itempairs$item1 ]
    itempairs$p1.item2 <- pxi2[ itempairs$item2 ]
    itempairs$p11 <- pbivnorm2( x=xi1, y=xi2, rho=cor.Sigma )
    itempairs$p10 <- itempairs$p1.item1 - itempairs$p11
    itempairs$p01 <- itempairs$p1.item2 - itempairs$p11
    itempairs$p00 <- 1 - itempairs$p11 - itempairs$p01 - itempairs$p10
    ind1 <- which( colnames(itempairs) %in% c( "p11", "p10", "p01", "p00" ) )
    itempairs[ itempairs[, ind1 ] < eps, ind1 ] <- eps
    ind2 <- which( colnames(itempairs) %in% c( "f11", "f10", "f01", "f00" ) )
#    ll <-  sum( log( itempairs[,ind1] ) * itempairs[,ind2]  )
    ll <-  rowSums( log( itempairs[,ind1] ) * itempairs[,ind2]  )
    return(ll)
        }
#********************************







######################################################################
# calculation of pairwise marginal likelihood
.ll.rasch.pml3.probit.est.corr <- function( b, a, sigma, Q,eps.corr, itempairs,
        IP, eps=10^(-14), h, deseps00, eps.items ){
    cor.Sigma <- NULL
    # 00
    ll00 <- .pml3.est.eps.aux( b, itempairs, a, sigma,
            eps.corr, cor.Sigma, eps )
    # 11
    ll11 <- .pml3.est.eps.aux( b, itempairs, a, sigma,
            eps.corr+h, cor.Sigma, eps )
    # 20 / 02 / 22
    ll22 <- .pml3.est.eps.aux( b, itempairs, a, sigma,
            eps.corr-h, cor.Sigma, eps )
    ll0 <- t(ll00) %*% deseps00
    ll1 <- t(ll11) %*% deseps00
    ll2 <- t(ll22) %*% deseps00
    # calculate increment
    incr <- nr.numdiff( ll0=as.vector(ll0), ll1=as.vector(ll1), ll2=as.vector(ll2), h=h, eps=10^(-10) )
    change <- rep( 0, IP )
    change <- incr[ eps.items ]
    change[ is.na(change ) ] <- 0
    change <- ifelse( abs(change) > .3, .3*sign(change), change )
    eps.corr <- eps.corr + change
    ll <- sum( ll00 )
    res <-list( "ll"=ll, "itempairs"=itempairs, "eps.corr"=eps.corr, "sigma"=sigma #,
#                "ind1"=ind1, "ind2"=ind2
                )
    return(res)
        }

######################################################################


#*********
.pml3.est.eps.aux <- function( b, itempairs, a, sigma,
            eps.corr, cor.Sigma, eps ){

    # unidimensional case
    t11 <- a^2*sigma^2
    t12 <- a^2*sigma^2

    # define different b parameters
    xib1 <-  - b  / sqrt( 1 + t11 )
    xib2 <-  - b  / sqrt( 1 + t12 )

    xi1 <- xib1[ itempairs[,"item1"] ]
    xi2 <- xib2[ itempairs[,"item2"] ]

    a1 <- a[ itempairs[,"item1"] ]
    a2 <- a[ itempairs[,"item2"] ]

    t1 <- a1*a2*sigma^2
    cor.Sigma <- ( t1 + eps.corr ) / ( 1 + t1 )
    cor.Sigma[ cor.Sigma > 1 ] <- .99

    pxi1 <- stats::pnorm( xib1 )
    pxi2 <- stats::pnorm( xib2 )

    itempairs$p1.item1 <- pxi1[ itempairs$item1 ]
    itempairs$p1.item2 <- pxi2[ itempairs$item2 ]
    itempairs$p11 <- pbivnorm2( x=xi1, y=xi2, rho=cor.Sigma )
    itempairs$p10 <- itempairs$p1.item1 - itempairs$p11
    itempairs$p01 <- itempairs$p1.item2 - itempairs$p11
    itempairs$p00 <- 1 - itempairs$p11 - itempairs$p01 - itempairs$p10
    ind1 <- which( colnames(itempairs) %in% c( "p11", "p10", "p01", "p00" ) )
    itempairs[ itempairs[, ind1 ] < eps, ind1 ] <- eps
    ind2 <- which( colnames(itempairs) %in% c( "f11", "f10", "f01", "f00" ) )
#    ll <-  sum( log( itempairs[,ind1] ) * itempairs[,ind2]  )
    ll <-  rowSums( log( itempairs[,ind1] ) * itempairs[,ind2]  )
    return(ll)
        }
#********************************

## File Name: sirt_logit_to_probs.R
## File Version: 0.04


sirt_logit_to_probs <- function(y)
{
    K1 <- length(y)
    x <- rep(0,K1+1)
    x[1L:K1] <- y
    x <- exp(x)
    x <- x / sum(x)
    return(x)
}

## File Name: penalty_D1_mcp.R
## File Version: 0.03


penalty_D1_mcp <- function(x, lambda, eps, a=2.7)
{
    x <- abs(x)
    res <- ifelse( x < a*lambda, lambda*sqrt(x^2 + eps) - x^2 / (2*a), .5*a*lambda^2)
    return(res)
}

## File Name: predict_scale_group_means.R
## File Version: 0.03


predict_scale_group_means <- function(object, M, SD)
{
    a <- object$a
    b <- object$b
    M_trafo <- a*M+b
    SD_trafo <- a*SD
    #-- output
    res <- list(M_trafo=M_trafo, SD_trafo=SD_trafo)
    return(res)
}

## File Name: lsem_wtdSD.R
## File Version: 0.16


lsem_wtdSD <- function( x, w )
{
    res1 <- sum( x*w )
    res2 <- sum( x^2*w)
    res12 <- res1^2
    if( res2 >=res12 ){
        res <- sqrt( res2 - res12 )
    } else {
        res <- 0
    }
    return(res)
}

## File Name: mle.pcm.group.R
## File Version: 0.182


#***** MLE of person parameters for individuals and groups
mle.pcm.group <- function( dat, b, a=rep(1,ncol(dat)), group=NULL, pid=NULL,
        adj_eps=.3, conv=.0001, maxiter=30)
{
    # indicator for groupwise estimation
    est_group <- ! is.null(group)
    if ( is.null( pid )){
        pid <- 1:(nrow(dat))
    }

    #*** data preparation
    data <- dat
    dat_resp <- 1 - is.na(data)
    dat <- data
    dat[ is.na(data) ] <- 0
    dat <- as.matrix(dat)
    theta0 <- rep( 0, nrow(dat) )
    dat_resp <- as.matrix( dat_resp )

    #**** group indicators
    if (est_group){
        ind <- order(group)
        dat_resp <- dat_resp[ ind, ]
        dat <- dat[ind,]
        group <- group[ind]
        idgroup <- match( group, unique( group ))
        # group indices
        gr1 <- rowsum( 1+0*idgroup, idgroup )
        c1 <- cumsum( gr1[, 1] )
        groupM <- as.matrix( cbind( c( 1, c1[ - length(c1) ] +1 ), c1 ) )
    }

    #**** extract maximum values
    maxK <- apply( dat, 2, max )
    # compute person scores
    personScore <- rowSums( dat * dat_resp )
    # compute person maximum score
    personMax <- rowSums( matrix( maxK, nrow=nrow(dat),
                                ncol=ncol(dat), byrow=TRUE ) * dat_resp )

    #******
    # epsilon adjustment for groups
    if ( est_group){
        groupMax <- rowsum( personMax, idgroup )
        groupScore <- rowsum( personScore, idgroup )
        grstat <- data.frame( groupScore, groupMax )
        fac <- ( groupMax - 2*adj_eps ) / groupMax
        maxKM <- dat_resp*matrix( maxK, nrow(dat), ncol(dat), byrow=TRUE )
        # adjustment matrix
        adjM <- maxKM / groupMax[ idgroup ] * adj_eps * dat_resp
        dat2 <- adjM + dat_resp * ( dat * fac[ idgroup ] )
        grstat$groupScoreAdj <- rowsum( rowSums( dat2*dat_resp ), idgroup )
        theta0 <- rep( 0, nrow(groupM) )
    }
    if ( ! est_group ){ # person estimation
        fac <- ( personMax - 2*adj_eps ) / personMax
        maxKM <- dat_resp*matrix( maxK, nrow(dat), ncol(dat), byrow=TRUE )
        # adjustment matrix
        adjM <- maxKM / personMax * adj_eps * dat_resp
        dat2 <- adjM + dat_resp * ( dat * fac )
        personScoreAdj <- rowSums( dat2 * dat_resp )
        theta0 <- rep(0, nrow(dat2) )
    }
    b <- as.matrix(b)
    dat <- dat2

    if ( est_group){
        res1 <- mle_pcm_group_CR( dat, dat_resp, groupM, b, a,
                maxK+1, theta0, conv, maxiter )
        res <- data.frame( "group"=unique(group), "idgroup"=rownames(gr1),
                "N.group"=gr1[,1],
                grstat, "theta"=res1$theta, "setheta"=res1$setheta,
                "Niter"=res1$Niter )
    }
    if ( ! est_group){
        res1 <- mle_pcm_CR( dat, dat_resp, b, a,
                maxK+1, theta0, conv, maxiter )
        res <- data.frame( "pid"=pid, "personScore"=personScore,
                "personMax"=personMax, "personScoreAdj"=personScoreAdj,
                "theta"=res1$theta, "setheta"=res1$setheta,
                "Niter"=res1$Niter )
    }
    #--- output
    res <- list( "person"=res, "data_adjeps"=dat )
    return(res)
}

#----- Partial Credit Model: group-wise likelihoods
mle_pcm_group_CR <- function(dat_, dat_resp_, groupM_, b_, a_, maxK_, theta0_,
            conv_, maxiter_ )
{
    mle_pcm_group_C( dat_, dat_resp_, groupM_, b_, a_, maxK_, theta0_, conv_, maxiter_ )
}


#---- Partial Credit Model: individual likelihoods
mle_pcm_CR <- function(dat_, dat_resp_, b_, a_, maxK_, theta0_, conv_, maxiter_ ){
    mle_pcm_C( dat_, dat_resp_, b_, a_, maxK_, theta0_, conv_, maxiter_ )
}

## File Name: fuzcluster_estimate.R
## File Version: 0.265


fuzcluster_estimate <- function(K, dat_m, dat_s, dat_resp,
        maxiter=1000, parmconv=.0001, progress=TRUE, seed=NULL, fac.oldxsi=0)
{

    dat.resp <- dat_resp
    # inits
    m1 <- colMeans( dat_m, na.rm=TRUE)
    s1 <- apply( dat_m, 2, stats::sd, na.rm=TRUE )

    # items
    I <- ncol(dat_m)
    N <- nrow(dat_m)
    if ( ! is.null(seed) ){
        set.seed( seed )
    }
    # initial mu estimate
    mu_est <- sirt_rmvnorm( K, mean=m1, sigma=1.0*diag(s1) )

    # initial SD estimate
    sd_est <- t(sapply( 1:K, FUN=function(kk){
                stats::runif( I, 1.05*s1, 2*s1 ) } ))
    # initial pi estimate
    pi_est <- stats::runif(K)
    pi_est <- pi_est / sum( pi_est )
    dev <- 0
    iter <- 0
    eps <- 1e-10
    parmchange <- 1

    mu_estmin <- mu_est
    sd_estmin <- sd_est
    pi_estmin <- pi_est
    dev_min <- 1E90
    itermin <- iter
    if (progress){
        cat("****************************************************************\n")
    }
    #*************************************************
    # begin algorithm
    while ( ( iter < maxiter ) & ( parmchange > parmconv) ){
        mu_est0 <- mu_est
        sd_est0 <- sd_est
        pi_est0 <- pi_est
        dev0 <- dev

        #*** E-step
        # matrix inits
        t_ik <- matrix( 0, nrow=N, ncol=K)
        phi_ik <- xi_ik <- sig2_ik <- mu_ik <- array( 0, dim=c(N,I,K) )

        for (kk in 1:K){
            # mu_k
            mu_k <- matrix( mu_est[kk,], nrow=N, ncol=I, byrow=TRUE )
            # sd_k
            sd_k <- matrix( sd_est[kk,], nrow=N, ncol=I, byrow=TRUE )
            # numerator
            num1 <- dat.resp*(dat_s^2 + sd_k^2 + eps)
            # mu estimate
            mu_ik[,,kk]  <- dat.resp*( mu_k * dat_s^2 + dat_m * sd_k^2  ) / num1
            # sigma2 estimate
            sig2_ik[,,kk] <- dat.resp*(dat_s^2 * sd_k^2 / num1)
            # xi estimate
            xi_ik[,,kk] <- mu_ik[,,kk]^2 + sig2_ik[,,kk]
            # t_ik: posterior distribution
            phi_ik[,,kk] <- sirt_dnorm( dat.resp*( dat_m - mu_k ) / sqrt( sd_k^2 + dat_s^2 + eps) )
            t_ik[,kk] <- pi_est[kk] * rowProds2( phi_ik[,,kk] )
                    }
        t_ik <- t_ik / rowSums( t_ik )

        #*** M-step

        # sum t_ik
        sum_tik <- colSums(t_ik)+eps

        # pi estimate
        pi_est <- colMeans(t_ik )+eps
        pi_est <- pi_est / sum( pi_est )
        if ( fac.oldxsi > 0 ){
            pi_est <- (1-fac.oldxsi) * pi_est + fac.oldxsi *pi_est0
            mu_est <- (1-fac.oldxsi) * mu_est + fac.oldxsi *mu_est0
            sd_est <- (1-fac.oldxsi) * sd_est + fac.oldxsi *sd_est0
        }
        # mu estimate
        for (kk in 1:K){
            mu_est[kk,] <- colSums( mu_ik[,,kk] * t_ik[,kk] ) / sum_tik[kk]
        }

        # sd estimate
        for (kk in 1:K){
            mu_k <- matrix( mu_est[kk,], nrow=N, ncol=I, byrow=TRUE )
            h1 <- colSums( t_ik[,kk] * ( xi_ik[,,kk] - 2*mu_k*mu_ik[,,kk] + mu_k^2 ) ) / sum_tik[kk]
            h1 <- ifelse( h1 < eps, eps, sqrt(h1) )
            sd_est[kk,] <- h1
        }
        sd_est[ sd_est < .01 ] <- .01
        # calculate deviance
        dev <- sum( colSums( t_ik ) * log( pi_est +eps) )
        dev <- dev - N*I / 2 * log(2*pi)
        for (kk in 1:K){
            dev <- dev - sum( t_ik[,kk] * sum( log( sd_est[kk,] + eps) ) )
        }
        for (kk in 1:K){
            sd2_k <- matrix( sd_est[kk,]^2, nrow=N, ncol=I, byrow=TRUE )
            mu_k <- matrix( mu_est[kk,], nrow=N, ncol=I, byrow=TRUE )
            dev <- dev - .5 * sum( t_ik[,kk] * rowSums( ( xi_ik[,,kk] -
                                    2*mu_k*mu_ik[,,kk] + mu_k^2 ) / (sd2_k+eps) ) )
        }
        dev <- -2*dev
        iter <- iter+1
        devchange <- abs( dev - dev0 )
        devchange1 <- - ( dev - dev0 )
        parmchange <- max( abs( mu_est - mu_est0 ), abs( sd_est - sd_est0) )
        if (progress){
            cat("............ seed=", seed, " | Iteration",iter, ".............\n")
            cat(paste("Deviance=", round( dev,3) ))
            cat(paste(" | Deviance Change=", round(devchange1,3), "\n"))
            cat("Class Probabilities=", paste(round( pi_est,4) ))
            cat(paste(" | Maximum Parameter Change=", round(parmchange,3),"\n"))
            utils::flush.console()
        }
        if ( dev < dev_min){
            t_ikmin <- t_ik
            mu_estmin <- mu_est
            sd_estmin <- sd_est
            pi_estmin <- pi_est
            itermin <- iter
            dev_min <- dev
        }
    }
    #** end algorithm
    res <- list( deviance=dev_min, iter=itermin, pi_est=pi_estmin,
        mu_est=mu_estmin, sd_est=sd_estmin, posterior=t_ikmin, seed=seed )
    return(res)
}

## File Name: sirt_EAP.R
## File Version: 0.02

sirt_EAP <- function(post, theta)
{
    TP <- ncol(post)
    N <- nrow(post)
    D <- ncol(theta)
    EAP <- matrix(NA, nrow=N, ncol=D)
    for (tt in 1L:D){
        thetaM <- sirt_matrix2(theta[,tt], nrow=N)
        EAP[,tt] <- rowSums(thetaM * post)
    }
    return(EAP)
}

## File Name: linking.haberman.R
## File Version: 2.655


#**** Linking Haberman: ETS Research Report 2009
linking.haberman <- function( itempars, personpars=NULL,
        estimation="OLS", a_trim=Inf, b_trim=Inf, lts_prop=.5,
        a_log=TRUE, conv=.00001, maxiter=1000, progress=TRUE,
        adjust_main_effects=TRUE, vcov=TRUE)
{
    CALL <- match.call()
    s1 <- Sys.time()

    #--- process item parameters
    res <- linking_proc_itempars(itempars=itempars)
    itempars <- res$itempars
    NS <- res$NS
    NI <- res$NI
    items <- res$items
    studies <- res$studies
    wgtM <- res$wgtM
    aM <- res$aM
    bM <- res$bM
    est_pars <- res$est_pars
    weights_exist <- res$weights_exist

    a.orig <- aM
    b.orig <- bM

    #**** estimation of A
    if (a_log){
        logaM <- log(aM)
    } else {
        logaM <- aM
    }
    est_type <- 'A (slopes)'
    resA <- linking_haberman_als(logaM=logaM, wgtM=wgtM, maxiter=maxiter, conv=conv,
                    progress=progress, est.type=est_type, cutoff=a_trim,
                    reference_value=1-a_log, adjust_main_effects=adjust_main_effects,
                    estimation=estimation, lts_prop=lts_prop, vcov=vcov)
    if (a_log){
        aj <- exp(resA$logaj)
        At <- exp(resA$logaAt)
    } else {
        aj <- resA$logaj
        At <- resA$logaAt
    }
    aj_resid <- resA$loga_resid
    aj_wgt_adj <- resA$loga_wgt_adj
    aj_wgtM <- resA$loga_wgt
    aj_vcov <- resA$vcov
    aj_se <- resA$se
    aj_item_stat <- resA$item_stat

    ## vcov of transformed parameters
    # H1 <- diag( At[-1] )
    H1 <- diag2( At[-1] )
    res <- linking_haberman_vcov_transformation( H1=H1, aj_vcov=aj_vcov )
    aj_vcov <- res$vcov
    aj_se <- c( NA, res$se )

    #**** estimation of B
    est_type <- 'B (intercepts)'
    At_m <- sirt_matrix2( At, nrow=NI)
    bMadj <- bM * At_m
    resB <- linking_haberman_als(logaM=bMadj, wgtM=wgtM, maxiter=maxiter,
                    conv=conv, progress=progress, est.type=est_type,
                    cutoff=b_trim, reference_value=0,
                    adjust_main_effects=adjust_main_effects,
                    estimation=estimation, lts_prop=lts_prop, vcov=vcov)
    Bj <- resB$logaj
    Bt <- resB$logaAt
    Bj_resid <- resB$loga_resid
    Bj_wgt_adj <- resB$loga_wgt_adj
    Bj_wgtM <- resB$loga_wgt
    Bj_vcov <- resB$vcov
    Bj_se <- resB$se
    Bj_item_stat <- resB$item_stat

    #*****
    # transformations
    transf.pars <- data.frame( study=studies, At=At, se_At=aj_se, Bt=Bt, se_Bt=Bj_se )
    rownames(transf.pars) <- NULL
    transf.itempars <- data.frame( study=studies, At=1/At, se_At=NA, At2=At,
            se_At2=transf.pars$se_At, Bt=Bt, se_Bt=transf.pars$se_Bt )
    rownames(transf.itempars) <- NULL

    #*** transformation 1/At
    H1 <- diag2( - 1 / ( At[-1] )^2 )

    res <- linking_haberman_vcov_transformation( H1=H1, aj_vcov=aj_vcov )
    transf.itempars$se_At <- c( NA, res$se )

    #**** tranbsformation for person parameters
    transf.personpars <- transf.itempars[,c('study','At','se_At2','Bt', 'se_Bt')]
    transf.personpars$At <- transf.pars$At
    transf.personpars$Bt <- -transf.pars$Bt
    colnames(transf.personpars) <- c('study', 'A_theta',
                'se_A_theta', 'B_theta', 'se_B_theta' )
    colnames(transf.itempars) <- c('study', 'A_a', 'se_A_a',
                'A_b', 'se_A_b', 'B_b', 'se_B_b' )
    # new item parameters
    joint.itempars <- data.frame(item=items, aj=aj, bj=Bj )
    # transformations for item parameters
    AtM <- sirt_matrix2( At, nrow=NI)
    BtM <- sirt_matrix2( Bt, nrow=NI)
    aM <- aM / AtM
    bM <- bM * AtM - BtM
    #****
    # transform person parameters
    if ( ! is.null( personpars) ){
        for (ll in 1L:NS){
            pp0 <- pp1 <- personpars[[ll]]
            pp1 <- transf.personpars$A_theta[ll] * pp1 + transf.personpars$B_theta[ll]
            ind <- which( substring( colnames(pp0),1,2) %in% c('se', 'SE') )
            if ( length(ind) > 0 ){
                pp1[,ind] <- transf.personpars$A_theta[ll] * pp0[,ind]
            }
            ind <- which( substring( colnames(pp0),1,3) %in% c('pid') )
            if ( length(ind) > 0 ){
                pp1[,ind] <- pp0[,ind]
            }
            personpars[[ll]] <- pp1
        }
    }

    #****** calculate R-squared measures of invariance
    # select items for R2 calculation for which at least
    # two studies are available.
    selitems <- which( rowSums( 1 - is.na( a.orig ) ) > 1 )
    Rsquared.partial.invariance <- Rsquared.invariance <- c(NA,NA)
    names(Rsquared.invariance) <- c('slopes', 'intercepts' )
    names(Rsquared.partial.invariance) <- c('slopes', 'intercepts' )

    # retransformed parameters
    aj1 <- aj * AtM
    a.res <- a.orig - aj1

    Rsquared.invariance['slopes'] <- 1 - sirt_sum( a.res[ selitems,]^2 ) /
                                            sirt_sum( a.orig[ selitems, ]^2 )
    Rsquared.partial.invariance['slopes'] <- 1 -
        sirt_sum( a.res[ selitems,]^2 * aj_wgtM[selitems,]  ) /
        sirt_sum( a.orig[ selitems, ]^2 * aj_wgtM[selitems, ]  )
    bj1 <- 1 / AtM *( Bj + BtM )
    b.res <- b.orig - bj1

    Rsquared.partial.invariance['intercepts'] <- 1 -
        sirt_sum( b.res[ selitems,]^2 * Bj_wgtM[selitems,] ) /
        sirt_sum( b.orig[ selitems, ]^2 * Bj_wgtM[selitems,] )
    Rsquared.invariance['intercepts'] <- 1 -
        sirt_sum( b.res[ selitems,]^2  ) / sirt_sum( b.orig[ selitems, ]^2 )
    es.invariance <- rbind( Rsquared.invariance,
            sqrt( 1- Rsquared.invariance ) )
    rownames(es.invariance) <- c('R2', 'sqrtU2')
    es.partial.invariance <- rbind( Rsquared.partial.invariance,
            sqrt( 1- Rsquared.partial.invariance ) )
    rownames(es.partial.invariance) <- c('R2', 'sqrtU2')

    # logical indicating whether item slopes are linked
    linking_slopes <- stats::sd( transf.pars$At ) < 1E-10

    s2 <- Sys.time()
    time <- list(s1=s1, s2=s2)

    #--- output
    res <- list( transf.pars=transf.pars, transf.itempars=transf.itempars,
            transf.personpars=transf.personpars, joint.itempars=joint.itempars,
            a.trans=aM, b.trans=bM, a.orig=a.orig, b.orig=b.orig,
            a.resid=aj_resid, b.resid=Bj_resid, personpars=personpars,
            es.invariance=es.invariance, es.robust=es.partial.invariance,
            selitems=selitems, a_trim=a_trim, b_trim=b_trim,
            a.wgt=aj_wgtM, b.wgt=Bj_wgtM, a.wgt.adj=aj_wgt_adj, b.wgt.adj=Bj_wgt_adj,
            a.vcov=aj_vcov, b.vcov=Bj_vcov, a.item_stat=aj_item_stat,
            b.item_stat=Bj_item_stat, linking_slopes=linking_slopes,
            description='Linking according to Haberman (2009)',
            res_opt_slopes=resA, res_opt_intercepts=resB, CALL=CALL, time=time )
    class(res) <- 'linking.haberman'
    return(res)
}

## File Name: lsdm.R
## File Version: 1.389


# LSDM - Least Squares Distance Method
# LSDM -- Least Squares Distance Method of Cognitive Validation
# Reference: Dimitrov, D. (2007) Applied Psychological Measurement, 31, 367-387.
lsdm <- function( data, Qmatrix, theta=seq(-3,3,by=.5), wgt_theta=rep(1, length(theta)),
            distance="L2", quant.list=c(.5,.65,.8), b=NULL, a=rep(1,nrow(Qmatrix)),
            c=rep(0,nrow(Qmatrix)) )
{
    CALL <- match.call()
    s1 <- Sys.time()
    # generate sequence for display
    display.separate <- paste( rep('.', each=65 ), collapse='' )
    # display progress
    cat(display.separate, '\n' )
    cat('LSDM -- Least Squares Distance Method of Cognitive Validation \n')
    cat('Dimitrov, D. (2007) Applied Psychological Measurement, 31, 367-387.\n')
    cat(display.separate, '\n')
    TP <- length(theta)
    I <- nrow(Qmatrix)
    thetaM <- sirt_matrix2(theta, nrow=I)
    wgt_theta <- sirt_sum_norm(x=wgt_theta)
    if (! is.null(b) ){
        eins <- rep(1, TP)
        cM <- matrix(c, nrow=I, ncol=TP)
        bM <- matrix(b, nrow=I, ncol=TP)
        aM <- matrix(a, nrow=I, ncol=TP)
        data <- cM + ( 1 - cM )* stats::plogis( aM * ( thetaM - bM ) )
    }
    Qmatrix <- as.matrix(Qmatrix)

    # print Q matrix
    cat('\nQmatrix\n\n')
    cmax <- apply( Qmatrix, 2, max )
    Qmatrix <- Qmatrix / sirt_matrix2(cmax, nrow=I)
    print(Qmatrix)
    cat('\n')
    d1 <- det( crossprod(Qmatrix) )
    # warning for singular Q matrices
    if (abs(d1) < 1E-8){
        stop('You used a singular Q matrix as an input. LSDM cannot be computed.\n')
    }
    est.icc <- TRUE
    I <- nrow(data)
    L <- ncol(data)
    K <- ncol(Qmatrix)
    if ( is.null(rownames(Qmatrix)) ){
        rownames(Qmatrix) <- rownames(data)
    }
    # log probability functions
    data1 <- data
    eps <- 1e-3
    logdata <- log( data1 + eps)
    # estimate item parameter and item quantiles
    cat('Estimation of Item Parameters \n')
    utils::flush.console()
    icc.pars <- lsdm_est_logist_quant( probcurves=data, theta=theta,
                    quantiles=quant.list, wgt_theta=wgt_theta, est.icc=est.icc,
                    b=b, a=a)
    cat(display.separate, '\n' )

    #***** Estimate attribute response curves
    cat('Estimation of Attribute Parameters \n')
    utils::flush.console()

    Q <- Qmatrix
    eps_obj <- 1e-4
    if (distance=='L2'){ pow_obj <- 2 }
    if (distance=='L1'){ pow_obj <- 1 }

    #** define optimization function for LSDM method
        # P=A1 * A2
        # log(P)=log(A1) + log(A2) for every tt
        #- apply restricted linear model
    lsdm_obj_fn <- function(x, y_tt){
        dist_log <- y_tt - Q %*% x
        if (pow_obj==2){
            val <- sum(dist_log^2)
        }
        if (pow_obj==1){
            val <- sum(sirt_abs_smooth(x=dist_log, eps=eps_obj))
        }
        return(val)
    }
    lsdm_obj_gr <- function(x, y_tt){
        K <- length(x)
        I <- length(y_tt)
        dist_log <- y_tt - Q %*% x
        dist_log <- matrix(dist_log, nrow=I, ncol=K)
        if (pow_obj==2){
            grad <- -2*colSums(dist_log*Q)
        }
        if (pow_obj==1){
            der1 <- sirt_abs_smooth(x=dist_log, eps=eps_obj, deriv=1 )
            grad <- -colSums( der1*Q )
        }
        return(grad)
    }

    log.arc0 <- matrix(NA, nrow=K, ncol=L)
    par0 <- rep(-2,K)
    upper <- rep(0,K)
    for (tt in 1L:L){
        y_tt <- logdata[,tt]
        res <- stats::optim(par=par0, fn=lsdm_obj_fn, gr=lsdm_obj_gr, method='L-BFGS-B',
                        upper=upper, y_tt=y_tt)
        par0 <- res$par
        log.arc0[,tt] <- par0
    }

    # exponentiate attribute response curve
    arc0 <- exp(log.arc0)
    rownames(arc0) <- colnames(Qmatrix)

    #--- estimate 'ordinary' LLTM
    lltm.res1 <- stats::lm( as.numeric(icc.pars$b.1PL) ~ 0 + as.matrix(Qmatrix) )
    slltm.res1 <- summary(lltm.res1)
    cat(display.separate, '\n')

    # calculate Rasch data predicted by LLTM
    b_lltm <- lltm.res1$fitted
    data.lltm <- stats::plogis(thetaM - b_lltm)
    # estimate attribute parameter and attribute quantiles
    arc0.pars <- lsdm_est_logist_quant( probcurves=arc0, theta=theta,
                quantiles=quant.list, est.icc=est.icc, wgt_theta=wgt_theta )
    arc0.pars$eta.LLTM <- coef(lltm.res1)
    arc0.pars$se.LLTM <- slltm.res1[[4]][,2]
    arc0.pars$pval.LLTM <- slltm.res1[[4]][,4]
    W <- matrix( NA, nrow=I, ncol=K )
    for (ii in 1L:I){
        index.ii <- which( Qmatrix[ii,] > 0 )
        L.ii <- length(index.ii)
        x.ii <- t( log.arc0[ index.ii, ] )
        x.ii <- matrix( x.ii, ncol=length(index.ii) )
        y.ii <- as.numeric(logdata[ii,])
        mod1.ii <- stats::lm( y.ii ~ 0 + x.ii )
        W[ii,index.ii] <- coef( mod1.ii )
    }

    #---- evaluate goodness of fit
    data0.fitted <- exp( Qmatrix %*% log.arc0 )
    rownames(data.lltm) <- rownames(data0.fitted) <- rownames(data)
    # MAD for original model (Dimitrov)
    mad0 <- lsdm_irf_distance_mad(data=data, data_fitted=data0.fitted,
                            wgt_theta=wgt_theta, use_abs=TRUE)
    md0 <- lsdm_irf_distance_mad(data=data, data_fitted=data0.fitted,
                            wgt_theta=wgt_theta, use_abs=FALSE)
    mm0 <- mean(mad0)

    mad.lltm <- lsdm_irf_distance_mad(data=data, data_fitted=data.lltm,
                            wgt_theta=wgt_theta, use_abs=TRUE)
    md.lltm <- lsdm_irf_distance_mad(data=data, data_fitted=data.lltm,
                            wgt_theta=wgt_theta, use_abs=FALSE)
    mm.lltm <- mean(mad.lltm)

    #* model fit LSDM
    cat(paste( 'Model Fit LSDM   -  Mean MAD:',
                    formatC( round( mm0, 3 ),digits=3, width=6),
                    '    Median MAD:', formatC( round( median(mad0), 3 ),
                                digits=3, width=6), '\n') )
    cat(paste( 'Model Fit LLTM   -  Mean MAD:',
                    formatC( round( mm.lltm, 3 ),digits=3, width=6),
                    '    Median MAD:', formatC( round( median(mad.lltm), 3 ),
                                    digits=3, width=6),
                    '   R^2=', format( round( slltm.res1$r.squared, 3 ),digits=3), '\n') )
    item <- data.frame( N.skills=rowSums(Qmatrix), mad.lsdm=mad0,
                        mad.lltm=mad.lltm, md.lsdm=md0, md.lltm=md.lltm, icc.pars )
    colnames(W) <- colnames(Qmatrix)
    rownames(W) <- rownames(Qmatrix)

    #--- output
    s2 <- Sys.time()
    time <- list(s1=s1, s2=s2, time_diff=s2-s1)
    res <- list( mean.mad.lsdm0=mm0,  mean.mad.lltm=mm.lltm, attr.curves=arc0,
                attr.pars=arc0.pars, data.fitted=data0.fitted, theta=theta,
                item=item, data=data, Qmatrix=Qmatrix, lltm=lltm.res1, W=W,
                distance=distance, CALL=CALL, time=time )
    class(res) <- 'lsdm'
    return(res)
}

## File Name: IRT.expectedCounts.mirt.R
## File Version: 0.082

# IRT.expectedCounts.mirt

#- counts single group
IRT.expectedCounts.SingleGroupClass <- function( object, ... )
{
    type <- 'exp_counts'
    object <- mirt.wrapper.posterior(mirt.obj=object)
    ll <- object[[type]]
    attr(ll,'theta') <- object$theta.k
    attr(ll,'prob.theta') <- object$pi.k
    attr(ll,'G') <- 1
    attr(ll,'pweights') <- object$pweights
    return(ll)
}


#--- counts multipleGroup
IRT.expectedCounts.MultipleGroupClass <- function( object, ... )
{
    mobj <- object
    groups <- object@Data$groupNames
    G <- length(groups)
    prob.theta <- list()
    ll_list <- list()
    ind_group <- list()
    pweights <- list()
    type <- 'exp_counts'
    for (gg in 1L:G){
        object <- mirt.wrapper.posterior(mirt.obj=mobj, group=groups[gg])
        if (gg==1){
            theta <- object$theta.k
            TP <- object$TP
        }
        prob.theta[[gg]] <- object$pi.k
        ll_list[[gg]] <- object[[type]]
        ind_group[[gg]] <- object$ind_group
        pweights[[gg]] <- object$pweights
    }
    dims <- dim(ll_list[[1]])[1L:3]
    ll <- array(NA, dim=c(dims, G))
    ll_pw <- rep(NA, object$N_orig)
    for (gg in 1L:G){
        ll[,,,gg] <- ll_list[[gg]]
        ll_pw[ ind_group[[gg]] ] <- pweights[[gg]]
    }
    prob.theta <- matrix( unlist(prob.theta), nrow=TP, ncol=G)
    attr(ll,'theta') <- theta
    attr(ll,'prob.theta') <- prob.theta
    attr(ll,'G') <- G
    attr(ll,'pweights') <- ll_pw
    return(ll)
}

## File Name: regpolca_proc_data.R
## File Version: 0.03

regpolca_proc_data <- function(dat, group)
{
    ncats <- apply(dat, 2, max, na.rm=TRUE)+1
    lca_dich <- max(ncats)==2
    I <- ncol(dat)
    N <- nrow(dat)
    if (is.null(group)){
        group <- rep(1, N)
    }
    groups <- unique(sort(group))
    group <- match(group, groups)
    G <- length(groups)
    Ni <- colSums(1-is.na(dat))
    #- output
    res <- list(ncats=ncats, lca_dich=lca_dich, I=I, N=N, group=group,
                groups=groups, G=G, Ni=Ni)
    return(res)
}

## File Name: rm_sdt_create_partable_define_pargroups.R
## File Version: 0.09


rm_sdt_create_partable_define_pargroups <- function(partable, pg1, pg2)
{
    partable$pargroup <- 0
    # K <- max( partable$row, na.rm=TRUE)
    K <- max( partable$col, na.rm=TRUE)  # changed ARb 2019-07-21
    for (kk in 1:K){
        m1 <- max(partable$pargroup) + 1
        ind <- ( partable$type==pg1 ) & ( partable$col==kk)
        partable[ ind, "pargroup"] <- m1 * ( sum( partable[ind,"est"] ) > 0 )
    }
    m1 <- max(partable$pargroup) + 1
    ind <- ( partable$type==pg2 )
    partable[ ind, "pargroup"] <- m1 * ( sum( partable[ind,"est"] ) > 0 )
    return(partable)
}

## File Name: gom_em_loglike_opt_fun.R
## File Version: 0.271



gom_em_loglike_opt_fun <- function(x, ind_lambda, ind_pi, I, K, theta.k, theta0.k,
        dat2, dat2.resp, TP, model, ind_mu, ind_sigma, theta_grid, weights,
        lambda_partable, lambda_index_blocks)
{
    ncat <- 2
    res <- gom_em_loglike_parameter_conversion( x=x, ind_lambda=ind_lambda,
                ind_pi=ind_pi, I=I, K=K, ind_mu=ind_mu, ind_sigma=ind_sigma, model=model,
                theta_grid=theta_grid, lambda_partable=lambda_partable )
    lambda <- res$lambda
    pi.k <- res$pi.k
    mu <- res$mu
    Sigma <- res$Sigma

    res <- gom_em_calc_probs( lambda=lambda, theta.k=theta.k, b=NULL, theta0.k=theta0.k )
    #- convert probabilities to array
    probs1 <- sirt_probs_dichotomous_to_array(probs=res$probs)
    #- compute individual likelihood
    fyiqk <- sirt_rcpp_gom_em_likelihood( probs=probs1, ncat=ncat, TP=TP,
                        dat2=dat2, dat2resp=dat2.resp)
    #- compute total log-likelihood
    ll <- sirt_rcpp_gom_em_log_likelihood( fyiqk=fyiqk, pik=pi.k, weights=weights)
    return(ll)
}

## File Name: dmlavaan_joint_parameterTable_merge_table.R
## File Version: 0.04

dmlavaan_joint_parameterTable_merge_table <- function(partable, mod, model_index=1)
{
    y1 <- mod$partable[, c('parname','est','se_sw')]
    colnames(y1) <- c('parname',paste0(c('est','se'), model_index ) )
    y1 <- y1[ ! duplicated(y1$parname), ]
    partable <- merge(x=partable, y=y1, by='parname', all.x=TRUE)
    partable <- partable[ order(partable$id), ]
    return(partable)
}

## File Name: rasch_mml2_estep_missing1.R
## File Version: 1.119



#*** E step missing data IRT model
rasch_mml2_estep_missing1 <- function( dat2, dat2.resp, theta.k, b, beta, delta.miss, I, CC,
        TP, group_, pi.k, pjk, weights, fixed.a=fixed.a, est.a=NULL, irtmodel="missing1" )
{
    eps <- 1e-16
    if (is.null(est.a)){
        est.a <- rep(0,I)
    }
# a0 <- Sys.time()
    # probability correct response
    pjk <- rasch_mml2_calcprob_missing1( theta.k=theta.k, b=b, beta=beta,
                delta.miss=delta.miss, pjk=pjk, fixed.a=fixed.a, irtmodel=irtmodel )
# cat( " *** calc probs") ; a1 <- Sys.time(); print(a1-a0) ; a0 <- a1


    #** calculate likelihood
    probs_ <- as.matrix( array( pjk, dim=c(I,CC*TP) ) )
    f.yi.qk <- sirt_rcpp_probs_pcm_groups_C( dat=dat2, dat_resp=dat2.resp, group=group_,
                    probs=probs_, CC=CC, TP=TP )
#  cat( " *** calc like") ; a1 <- Sys.time(); print(a1-a0) ; a0 <- a1

    #*** calculate expected counts
    e1 <- sirt_rcpp_calccounts_pcm_groups_C( dat=dat2, dat_resp=dat2.resp, group=group_,
                fyiqk=f.yi.qk, pik=pi.k, CC=CC, weights=weights )
# cat( " *** calc counts") ; a1 <- Sys.time(); print(a1-a0) ; a0 <- a1

    e1$f.yi.qk <- f.yi.qk
    v1 <- array( e1$nik, dim=c(I,CC,TP) )
    e1$pjk <- pjk
    e1$n.k <- e1$count_pik
    e1$r.jk <- e1$n.jk <- NULL
    e1$n.ik <- v1
    e1$f.qk.yi <- e1$fqkyi
    e1$theta.k <- theta.k
    e1$irtmodel <- irtmodel
    return(e1)
}




.e.step.missing1 <- rasch_mml2_estep_missing1

## File Name: rasch_mml2_estep_raschtype.R
## File Version: 0.136


#--- E Step Raschtype Model
rasch_mml2_estep_raschtype <- function( dat1, dat2, dat2.resp,
        theta.k, pi.k, I, n, b, fixed.a, fixed.c,  fixed.d, alpha1, alpha2,
        group, pseudoll, f.qk.yi=NULL, weights=NULL )
{
    # arrange groups
    if ( is.null(group) ){
        group <- rep( 1, nrow(dat1))
    }
    G <- length( unique(group) )
    TP <- length(theta.k)

    # probabilities of correct item at theta_k
    pjk <- .prob.raschtype.genlogis( theta.k, b, alpha1, alpha2, fixed.a )
# cat("   probs") ; vv1 <- Sys.time(); print(vv1-vv0) ; vv0 <- vv1

    fixed.c.M <- sirt_matrix2( x=fixed.c, nrow=nrow(pjk) )
    fixed.d.M <- sirt_matrix2( x=fixed.d, nrow=nrow(pjk) )
    pjk <- fixed.c.M + ( fixed.d.M - fixed.c.M) * pjk
# cat("   probs compute") ; vv1 <- Sys.time(); print(vv1-vv0) ; vv0 <- vv1

    if ( is.null( f.qk.yi ) ){
        pjkt <- t(pjk)
        pjkL <- array( NA, dim=c( I, 2, TP ) )
        pjkL[,1,] <- 1 - pjkt
        pjkL[,2,] <- pjkt
        probsM <- matrix( aperm( pjkL, c(2,1,3) ), nrow=I*2, ncol=TP )
        f.yi.qk <- mml_calc_like( dat2=dat2, dat2resp=dat2.resp, probs=probsM,
                            pseudoll=pseudoll )$fyiqk
#cat("   calc likelihood") ; vv1 <- Sys.time(); print(vv1-vv0) ; vv0 <- vv1
        # f.qk.yi <- 0 * f.yi.qk
        f.qk.yi <- matrix( 0, nrow=nrow(f.yi.qk), ncol=ncol(f.yi.qk) )
        if ( G==1 ){
            pi.k <- matrix( pi.k, ncol=1 )
        }
        for ( gg in 1:G){
            ind_gg <- which(group==gg)
            f.qk.yi[ind_gg,] <- f.yi.qk[ind_gg,]*sirt_matrix2(x=pi.k[,gg], nrow=length(ind_gg))
        }
        f.qk.yi <- f.qk.yi / rowSums(f.qk.yi)
    }

#cat("   calc posterior") ; vv1 <- Sys.time(); print(vv1-vv0) ; vv0 <- vv1

    # expected counts at theta.k
    n.k <- matrix( 0, length(theta.k), G )
    r.jk <- n.jk <- array( 0, dim=c( ncol(dat2), length(theta.k), G) )
    ll <- rep(0,G)
    for (gg in 1:G){
        ind.gg <- which(group==gg)
        res <- mml_raschtype_counts( dat2=dat2[ind.gg,], dat2resp=dat2.resp[ind.gg,],
                    dat1=dat1[ind.gg,2], fqkyi=f.qk.yi[ind.gg,], pik=pi.k[,gg],
                    fyiqk=f.yi.qk[ind.gg,])
        n.k[,gg] <- res$nk
        n.jk[,,gg] <- res$njk
        r.jk[,,gg] <- res$rjk
        ll[gg] <- res$ll
    }
#cat("   calc expected counts") ; vv1 <- Sys.time(); print(vv1-vv0) ; vv0 <- vv1

    #-- output
    res <- list( n.k=n.k, n.jk=n.jk, r.jk=r.jk, f.qk.yi=f.qk.yi, pjk=pjk,
                f.yi.qk=f.yi.qk, ll=sum(ll), theta.k=theta.k )
    return(res)
}

.e.step.raschtype <- rasch_mml2_estep_raschtype

## File Name: rasch.prox.R
## File Version: 1.14



#------------------------------------------------------------------------------------------------------------------------------
# PROX routine for Rasch models
rasch.prox <- function( dat, dat.resp=1 - is.na(dat),
                freq=rep(1,nrow(dat)),
                conv=0.001, maxiter=30, progress=FALSE ){
        # INPUT:
        # dat       ... data frame
        # dat.resp  ... missing response pattern
        # freq      ... frequencies of item response pattern
        # conv      ... convergence criterion (maximal change for item parameter estimates)
        # maxiter   ... maximal number of PROX iterations
        # print     ... should estimation process being displayed
        I <- ncol( dat )
        IP <- nrow(dat )
        # calculate logit frequency for every item
        s.i <- colSums( dat * dat.resp * freq )
        n.i <- colSums( dat.resp * freq )
        logit.freq.i <- stats::qlogis( ( s.i / n.i + .01)/1.02 )
        # calculate logit frequency for every person (response pattern)
        r.n <- rowSums( dat * dat.resp * freq )
        n.n <- rowSums( dat.resp * freq )
        logit.freq.n <- stats::qlogis( ( r.n / n.n + .01 )/1.02 )
        # initial mean and SD of logit abilities for persons which answers item i
        d.i <- mu.i <- rep(0,I)
        sigma.i <- rep(1,I)
        # initial distribution of logit item diffuculties encountered by person n (respective pattern n)
        b.n <- mu.n <- rep( 0, IP )
        sigma.n <- rep( 1, IP )
        # Number of subjects per item
        N.i <- colSums( dat.resp * freq )
        # Number of items per subject pattern
        N.n <- rowSums( dat.resp  )
        iter <- 0
        par.change <- 1
        Mdf <- dat.resp * freq
        # PROX algorithm                            #
        while ( ( par.change > conv ) & ( iter < maxiter )) {
                d.i0 <- d.i
                b.n0 <- b.n
                # update item difficulty
                d.i <- mu.i - sqrt( ( 1 + sigma.i^2 / 2.9 ) ) * logit.freq.i
                # update ability estimate for pattern
                b.n <- mu.n + sqrt( ( 1 + sigma.n^2 / 2.9 ) ) * logit.freq.n
                # center persons
                b.n <- b.n - stats::weighted.mean( b.n, freq )
                # update mu.i and sigma.i
                # (mean and standard deviation of the logit abilities of the person encountering item i)
                mu.i <- colSums( Mdf * b.n ) / N.i
                g1 <- ( colSums( b.n^2 * Mdf ) - N.i * mu.i^2  ) / ( N.i - 1 )
                sigma.i <- sqrt( ifelse( g1 < 0, .0001, g1 ) )
                # update mu.n and sigma.n
                # (mean and standard deviation of the logit difficulties encountered by person n)
#                d.i.m <- outer( rep(1, IP ), d.i )
                d.i.m <- matrix( d.i, nrow=IP, ncol=I, byrow=T )
                mu.n <- rowSums( dat.resp *  d.i.m ) / N.n
                g1 <- ( rowSums( d.i.m^2 * dat.resp ) - N.n * mu.n^2  ) / ( N.n - 1 )
                sigma.n <- sqrt( ifelse( g1 < 0, .0001, g1 ) )
                iter <- iter + 1
                par.change <- max( abs(d.i - d.i0) )
                if ( progress){
                    cat( "PROX Iter.", iter, ": max. parm. change=", round( par.change, 6 ), "\n")
                    utils::flush.console()
                        }
        }
    return( list( "b"=d.i, "theta"=b.n, "iter"=iter,
                    "sigma.i"=sigma.i, "sigma.n"=sigma.n ) )
    }
#------------------------------------------------------------------------------------------------------------------------------

## File Name: lsem_bootstrap_postproc_output.R
## File Version: 0.104

lsem_bootstrap_postproc_output <- function(parameters, parameters_boot,
    fitstats_joint, fitstats_joint_boot, est_joint=FALSE, repl_factor=NULL,
    parameters_summary, parameters_var_boot)
{
    #* parameters
    res <- lsem_bootstrap_inference(parameters_boot=parameters_boot, est=parameters$est,
                repl_factor=repl_factor, bc_square=NULL)
    parameters$est_bc <- res$est_bc
    parameters$se <- res$se_boot
    parameters$z <- parameters$est / parameters$se
    parameters$pvalue <- 2*stats::pnorm(abs(-parameters$z))
    quant <- stats::qnorm(.975)
    parameters$ci.lower <- parameters$est - quant * parameters$se
    parameters$ci.upper <- parameters$est + quant * parameters$se

    #* fitstats_joint: bootstrap inference for fit statistics
    if (est_joint){
        fjb <- fitstats_joint_boot
        # bc_square <- which( rownames(fjb) %in% c('rmsea','srmr') )
        bc_square <- which( rownames(fjb) %in% c('srmr') )
        val <- fitstats_joint$value
        res <- lsem_bootstrap_inference(parameters_boot=fjb,
                        est=val, repl_factor=repl_factor,
                        bc_square=bc_square)
        fitstats_joint$value_bc <- res$est_bc
        fitstats_joint$se <- res$se_boot
    }

    #* adapt parameters_summary
    vt1 <- rowMeans(parameters_var_boot)
    vt0 <- parameters_summary$SD^2
    parameters_summary$SD_se <- apply(sqrt(parameters_var_boot), 1, stats::sd)
    w <- vt0 - (vt1-vt0)
    parameters_summary$SD_bc <- ifelse( w < 0, 0, sqrt(w) )
    h <- parameters_summary$SD_bc / (parameters_summary$SD_se+1e-100)
    h <- ifelse( abs(parameters_summary$SD) < 1e-5*abs(parameters_summary$M), 0, h )
    parameters_summary$SD_t <- h
    parameters_summary$SD_p <- stats::pnorm(-abs(h))

    parameters_summary <- move_variables_df(x=parameters_summary, after_var='SD',
                                move_vars=c('SD_bc', 'SD_se', 'SD_t', 'SD_p') )

    #-- output
    res <- list(parameters=parameters, fitstats_joint=fitstats_joint,
                    parameters_summary=parameters_summary)
    return(res)
}

## File Name: penalty_D1_scad.R
## File Version: 0.05


penalty_D1_scad <- function(x, lambda, eps, a=3.7)
{
    x <- abs(x)
    res <- ifelse( x < lambda, lambda * sqrt( x^2 + eps ), 0)
    res <- res + ifelse( ( x >=lambda ) & ( x < a*lambda),
                        - ( x^2 - 2*a*lambda*sqrt(x^2+eps)+lambda^2) / ( 2*(a-1)),0 )
    res <- res + ifelse( x>=a*lambda, (a+1)*lambda^2 / 2, 0 )
    return(res)
}

## File Name: dmlavaan_est_model.R
## File Version: 0.298

dmlavaan_est_model <- function(fun, args, h=1e-5, use_observed_info_lavaan=FALSE,
            method="sandwich")
{
    requireNamespace('lavaan')
    requireNamespace('MASS')
    remove_duplicated <- TRUE

    #- reorder groups
    if ('group' %in% names(args) ){
        data <- args$data
        group <- args$group
        data <- data[ order(data[,group]), ]
        args$data <- data
    }

    #- estimate model
    mod <- do.call(what=fun, args=args)
    data <- args$data
    N <- nrow(data)
    coef1 <- coef(mod)
    parnames <- names(coef1)
    NP <- length(coef1)
    #- standard errors
    vcov1 <- vcov(mod)

    #- parameter table
    res <- dmlavaan_est_model_parameterTable(mod=mod, parnames=parnames,
                    coef1=coef1, vcov1=vcov1)
    partable <- res$partable
    parnames <- res$parnames
    parnames0 <- res$parnames0
    coef1 <- res$coef1
    vcov1 <- res$vcov1
    NPU <- res$NPU
    se1 <- sqrt(diag(vcov1))

    #- score function
    if (method=='sandwich'){
        scores <- lavaan::lavScores(object=mod, remove.duplicated=remove_duplicated)
        colnames(scores) <- parnames0
        #--- compute derivative of score function
        use_observed_info_score_deriv <- TRUE
        if (use_observed_info_score_deriv){
            B <- dmlavaan_est_model_bread_matrix_score_derivatives(fun=fun, args=args,
                        partable=partable, scores=scores, h=h, symmetric=FALSE,
                        remove_duplicated=remove_duplicated, parnames=parnames0)
        }

        #- observed information matrix
        use_observed_info_lavaan <- FALSE
        if (use_observed_info_lavaan){
            BI <- lavaan::lavInspect( object=mod, what='information.observed')
            BI <- as.matrix(BI)
            sel <- partable[ partable$unique==1, 'free' ]
            B <- BI[sel,sel]
        }

        A <- crossprod(scores)

        res <- dmlavaan_sandwich_formula(A=A, B=B, parnames=parnames0)
        V <- res$V
        V_sw <- V
        se_sw <- res$se_sw

        #-- include results in partable
        partable <- dmlavaan_est_model_include_partable(partable=partable, se_sw=se_sw)
    } else {
        partable$se_sw <- partable$se
        A <- NULL
        B <- NULL
        scores <- NULL
        se_sw <- NULL
        V_sw <- NULL
    }

    #-- output
    res <- list(mod=mod, coef=coef1, vcov=vcov1, parnames=parnames, NP=NP,
                    se=se1, se_sw=se_sw, scores=scores, partable=partable,
                    V_sw=V_sw, A=A, B=B, N=N, NPU=NPU, data=data, fun=fun, args=args,
                    parnames0=parnames0)
    return(res)
}

## File Name: rm_facets_print_progress_parameter.R
## File Version: 0.08


rm_facets_print_progress_parameter <- function( parm, parm0, parmlabel, digits_parm )
{
    cat( paste( "    Maximum ", parmlabel, " parameter change=",
            paste( round(max(abs(parm0-parm)), digits_parm), collapse=" " ), "\n", sep=""))
}

## File Name: sirt_import_lavaan_standardizedSolution.R
## File Version: 0.05

sirt_import_lavaan_standardizedSolution <- function(...)
{
    TAM::require_namespace_msg('lavaan')
    res <- lavaan::standardizedSolution(...)
    return(res)
}

## File Name: summary.xxirt.R
## File Version: 0.275


#--- summary for xxirt object
summary.xxirt <- function( object, digits=3, file=NULL, ...)
{
    # open sink
    sirt_osink( file=file )

    # print summary
    res <- xxirt_summary_parts(object=object, digits=digits)

    # close sink
    sirt_csink( file=file )
}

## File Name: meas_inv_compute_lavaan_parnames.R
## File Version: 0.01


meas_inv_compute_lavaan_parnames <- function(object)
{
    pars <- paste0( object$lhs, object$op, object$rhs )
    return(pars)
}

## File Name: sirt_colMedians.R
## File Version: 0.04

sirt_colMedians <- function(x, na.rm=TRUE)
{
    res <- apply(x, 2, stats::median, na.rm=na.rm)
    return(res)
}

sirt_colMedian <- sirt_colMedians

## File Name: xxirt_nr_grad_fun_Rcpp.R
## File Version: 0.233

xxirt_nr_grad_fun_Rcpp <- function(x, em_args, eps=1e-100)
{

    NP <- em_args$NP
    NPI <- em_args$NPI
    NPT <- em_args$NPT
    customTheta <- em_args$customTheta
    free_pars_design <- em_args$free_pars_design
    h <- em_args$h
    eps2 <- 1e-300

    grad <- 0*x

    #*** compute prior distribution
    prior_Theta0 <- xxirt_compute_prior_Theta_from_x(x=x, em_args=em_args)
    #* compute item response probabilities
    probs_items0 <- xxirt_compute_prob_item_from_x(x=x, em_args=em_args)

    p.xi.aj0 <- xxirt_compute_likelihood( probs_items=probs_items0,
                            dat=em_args$dat, dat_resp_bool=em_args$dat_resp_bool )
    ll_case0 <- xxirt_compute_casewise_likelihood(prior_Theta=prior_Theta0,
                            group=em_args$group, p.xi.aj=p.xi.aj0,
                            customTheta=customTheta)
    ll0 <- sum( em_args$weights*log(ll_case0+eps2) )

    partable <- xxirt_partable_include_freeParameters( partable=em_args$partable, x=x )

    #---
    # derivatives of probabilities
    MIGC <- em_args$MIGC
    probs_items_der <- list()
    ratio_list <- list()

    for (mm in seq_len(MIGC) ){
        free_pars_design_mm <- free_pars_design[ free_pars_design$item_group_comp==mm, ]
        x1 <- x[ em_args$parindex_items ]
        x1 <- sirt_add_increment(x=x1, pos=free_pars_design_mm$pid, value=h)
        partable1 <- xxirt_partable_include_freeParameters( partable=em_args$partable,
                                x=x1 )
        probs_temp <- xxirt_compute_itemprobs( item_list=em_args$item_list,
                                items=em_args$items, Theta=em_args$Theta,
                                ncat=em_args$ncat, partable=partable1,
                                partable_index=em_args$partable_index )
        probs_items_der[[mm]] <- (probs_temp-probs_items0)/h
        ratio_list[[mm]] <- probs_items_der[[mm]] / ( probs_items0 + eps )
    }

    #--- loop over item parameters
    for (pp in seq_len(NPI) ){

        free_pars_design_pp <- free_pars_design[pp,]
        partable_pp <- partable[ partable$parlabel==free_pars_design_pp$parlabel, ]
        item_pp <- free_pars_design_pp$itemnr
        item_group_comp_pp <- free_pars_design_pp$item_group_comp

        #- shortcut for item parameter per one item
        shortcut <- free_pars_design_pp$one_item
        if (attr(partable,'person_covariates')){
            shortcut <- FALSE
        }

        if (shortcut){
            RL1 <- ratio_list[[item_group_comp_pp]]
            person_covariates <- customTheta$person_covariates
            person_covariates_items <- FALSE
            if (! attr(partable,'person_covariates')){
                ratio <- RL1[item_pp,,]
            } else {
                ratio <- RL1[item_pp,,,]
                maxK <- dim(ratio)[1]
                TP <- dim(ratio)[2]
                N <- dim(ratio)[3]
                ratio <- matrix(ratio, nrow=maxK, ncol=N*TP)
                person_covariates_items <- TRUE
            }
            grad[pp] <- sirt_rcpp_xxirt_newton_raphson_derivative_par(
                                dat=em_args$dat, dat_resp_bool=em_args$dat_resp_bool,
                                ratio=ratio, p_xi_aj=p.xi.aj0, item=item_pp,
                                prior_Theta=prior_Theta0, group0=em_args$group0,
                                weights=em_args$weights, ll_case0=ll_case0, eps=eps,
                                person_covariates=person_covariates,
                                person_covariates_items=person_covariates_items)
        } else {
            x1 <- x[ em_args$parindex_items ]
            x1 <- sirt_add_increment(x=x1, pos=pp, value=h)
            probs_items_temp <- xxirt_compute_prob_item_from_x(x=x1, em_args=em_args)
            p.xi.aj.temp <- xxirt_compute_likelihood( probs_items=probs_items_temp,
                                dat=em_args$dat, dat_resp_bool=em_args$dat_resp_bool)
            ll_case_temp <- xxirt_compute_casewise_likelihood(prior_Theta=prior_Theta0,
                                group=em_args$group, p.xi.aj=p.xi.aj.temp,
                                customTheta=customTheta)
            # ll_temp <- sum( em_args$weights*log(ll_case_temp+eps2) )
            # grad[pp] <- -( ll_temp - ll0 ) / h
            grad[pp] <- -sum( em_args$weights* (ll_case_temp - ll_case0)/(h*ll_case0) )
        }

    }        # end pp

    #*** prior distributions
    if (NPI > 0){
        index_items <- 1L:NPI
        x1 <- x[ em_args$parindex_items ]
        partable1 <- xxirt_partable_include_freeParameters( partable=em_args$partable,
                            x=x1)
        par_prior <- xxirt_mstep_itemParameters_evalPrior(partable1, h=0)
        par_prior1 <- xxirt_mstep_itemParameters_evalPrior(partable1, h=h)
        par_prior_der <- ( par_prior1 - par_prior) / h
        grad[index_items] <- grad[index_items] + par_prior_der
    }

    #*** penalty function
    if (NPI>0){
        if (!is.null(em_args$penalty_fun_item)){
            pen0 <- em_args$penalty_fun_item(x=x1)
            pen1 <- rep(0,NPI)
            for (pp in 1L:NPI){
                xh <- sirt_add_increment(x=x1, pos=pp, value=h)
                pen1[pp] <- em_args$penalty_fun_item(x=xh)
            }
            grad[index_items] <- grad[index_items] + (pen1-pen0)/h
        }
    }

    #--- loop over distribution parameters
    if (NPT>0){
        for (pp in em_args$parindex_Theta){
            x1 <- sirt_add_increment(x=x, pos=pp, value=h)
            prior_Theta <- xxirt_compute_prior_Theta_from_x(x=x1, em_args=em_args)
            prior_Theta_der <- ( prior_Theta - prior_Theta0 ) / h
            ll_case <- xxirt_compute_casewise_likelihood(prior_Theta=prior_Theta_der,
                                group=em_args$group, p.xi.aj=p.xi.aj0,
                                customTheta=customTheta)
            grad[pp] <- -sum( em_args$weights*ll_case/(ll_case0 + eps) )
        }
    }

    return(grad)
}

## File Name: rasch.pairwise.R
## File Version: 0.446


#------ Rasch estimation (Approximate Method)
# also called MINCHI method
# Handbook of Statistics Vol. 26
# Chapter of G. Fischer: p. 544 ff.
rasch.pairwise <- function( dat, weights=NULL, conv=0.0001, maxiter=3000,
        progress=TRUE, b.init=NULL, zerosum=FALSE, power=1,
        direct_optim=TRUE)
{
    s1 <- Sys.time()
    CALL <- match.call()
    version <- 1
    # should items be excluded?
    item.elim <- which( colMeans( dat, na.rm=TRUE ) %in% c(0,1))
    if (length(item.elim)>0){
        dat <- dat[, - item.elim ]
    }
    I <- ncol(dat)
    dat <- as.matrix(dat)
    dat00 <- dat
    if (direct_optim){
        p <- colMeans(dat, na.rm=TRUE)
        p <- p[ order(abs(p-.5)) ]
        ind0 <- match(names(p),colnames(dat))
        dat <- dat[,ind0]
    }

    # data preparation
    dat.resp <- 1 - is.na(dat)
    dat.9 <- dat
    dat.9[ is.na(dat) ] <- 9
    # calculate n_{ij}
    if (is.null(weights)){
        weights <- rep(1,nrow(dat))
    }
    sw <- sqrt(weights)
    n.ij <- crossprod( dat.9 * dat.resp * sw, ( 1 - dat.9 ) * dat.resp *sw )
    # which item pairs occur in estimation procedure?
    delta.ij <- 1 * ( n.ij + t(n.ij) > 0 )

    # initial values for beta
    if( is.null(b.init) ){
        beta <- - stats::qlogis( colMeans( dat, na.rm=TRUE ) )
    } else {
        beta <- b.init
    }
    # calculate y_{ij} values
    if (power==1){
        y.ij <- n.ij / ( n.ij + t(n.ij) )
    }
    if (power>1){
        m.ij <- n.ij
        for (hh in 2L:power){
            m.ij <- m.ij %*% n.ij
        }
        y.ij <- m.ij / ( n.ij + t(n.ij) )  # Fischer (2007)
    }
    if (version==2){
        y.ij <- n.ij^2 / ( n.ij + t(n.ij) )
    }
    y.ij[ delta.ij==0 ] <- 0
    y.ji <- t(y.ij)
    eps <- exp(-beta)
    b <- -log(eps)

    if (direct_optim){
        #- direct optimization
        n.ji <- t(n.ij)
        res <- rasch_pairwise_optimize(n.ij=n.ij, n.ji=n.ji, beta=beta,
                    zerosum=zerosum, maxiter=maxiter, estimator="MINCHI")
    } else {
        #- iterations
        fix_first <- version==2
        res <- rasch_pairwise_iterations( eps=eps, y.ij=y.ij, delta.ij=delta.ij,
                    conv=conv, maxiter=maxiter, zerosum=zerosum, progress=progress,
                    fix_first=fix_first)
    }
    eps <- res$eps
    b <- res$b
    iter <- res$iter

    #* post processing
    item <- data.frame( item=colnames(dat), N=colSums(1-is.na(dat)),
                            p=colMeans(dat, na.rm=TRUE), b=b, eps=eps,
                            itemcluster=rep(0,I) )
    item <- item[ match(colnames(dat00), item$item), ]

    #-- output
    s2 <- Sys.time()
    res <- list( b=item$b, eps=item$eps, iter=iter, conv=conv, dat=dat, I=I,
                freq.ij=n.ij, item=item, power=power, direct_optim=direct_optim,
                fct='rasch.pairwise', s1=s1, s2=s2, CALL=CALL )
    class(res) <- 'rasch.pairwise'
    return(res)
}

## File Name: rasch_mml2_numdiff_index.R
## File Version: 1.127



#** general function for numerical differentiation
#** diffindex aggregates across super items
rasch_mml2_numdiff_index <- function( pjk, pjk1, pjk2, n.ik, diffindex,
        max.increment, numdiff.parm, eps=1e-16, shortcut=TRUE )
{
    eps2 <- 1e-10
    h <- numdiff.parm
    an.ik <- n.ik

    ll0 <- rowSums( an.ik * log(pjk+eps) )
    ll0 <- rowsum(ll0, diffindex)[,1]

    if (! shortcut){

        ll1 <- rowSums( an.ik * log(pjk1+eps) )
        ll2 <- rowSums( an.ik * log(pjk2+eps) )
        # ll0 <- stats::aggregate( ll0, list(diffindex), sum )[,2]
        # ll1 <- stats::aggregate( ll1, list(diffindex), sum )[,2]
        # ll2 <- stats::aggregate( ll2, list(diffindex), sum )[,2]
        ll1 <- rowsum(ll1, diffindex)[,1]
        ll2 <- rowsum(ll2, diffindex)[,1]

        d1 <- ( ll1 - ll2  ) / ( 2 * h )    # negative sign?

        # second order derivative
        # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
        d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2

    }

    if (shortcut){

        p1 <- ( pjk1 - pjk2 ) / (2*h)
        ll1a <- rowSums( an.ik * p1 / pjk )
        ll1a <- rowsum(ll1a, diffindex)[,1]
        d1 <- ll1a

        p2 <- ( pjk1 + pjk2 - 2*pjk) / h^2
        ll1a <- rowSums( an.ik * (p2*pjk-p1^2)/pjk^2 )
        ll1a <- rowsum(ll1a, diffindex)[,1]
        d2 <- ll1a

    }

    # change in item difficulty
    d2[ abs(d2) < eps2 ] <- eps2
    increment <- - d1 / d2
    ci <- ceiling( abs(increment) / ( abs( max.increment) + eps2 ) )
    increment <- ifelse( abs( increment) > abs(max.increment),
                                increment/(2*ci), increment )

    # handle issues
    ind <- which( ! is.finite(increment) )
    if (length(ind)>0){
        increment[ind] <- 0
        d2[ind] <- 1e10
    }

    max_increment <- max( abs( increment ) )
    se <- sqrt( abs( 1 / d2 ) )
    increment <- increment[ match(diffindex, sort(unique(diffindex))) ]
    increment[ is.na(increment) ] <- 0
    #-- output
    res <- list(increment=increment, d2=d2, ll0=ll0, max_increment=max_increment,
                    se=se)
    return(res)
}


.mml2.numdiff.index <- rasch_mml2_numdiff_index

## File Name: rasch_mml2_difference_quotient.R
## File Version: 0.12

rasch_mml2_difference_quotient <- function(ll0, ll1, ll2, h, eps=1e-6)
{
    # first order derivative
    # f(x+h) - f(x-h)=2*f'(x)*h
    d1 <- ( ll1 - ll2  ) / ( 2 * h )    # negative sign?
    # second order derivative
    # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
    d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
    # change in item difficulty
    d2[ abs(d2) < eps ] <- eps
    #--- output
    res <- list( d1=d1, d2=d2)
    return(res)
}

## File Name: dmlavaan_se_bootstrap_create_est_boot.R
## File Version: 0.05


dmlavaan_se_bootstrap_create_est_boot <- function(est_boot1, est_boot2)
{
    #-- est_boot
    est_boot1 <- dmlavaan_add_suffix_column_names(x=est_boot1, suffix='_mod1')
    est_boot2 <- dmlavaan_add_suffix_column_names(x=est_boot2, suffix='_mod2')
    est_boot <- data.frame( est_boot1, est_boot2)
    colnames(est_boot) <- c(colnames(est_boot1), colnames(est_boot2) )

    #-- compute covariance matrix
    V <- stats::cov(est_boot)

    #--- output
    res <- list(est_boot=est_boot, V=V)
    return(res)
}

## File Name: Probtrace_sirt.R
## File Version: 0.05


# auxiliary function for mirt package
# trace function for all items
Probtrace_sirt <- function(items, Theta)
{
    TAM::require_namespace_msg("mirt")
    traces <- lapply(items, mirt::probtrace, Theta=Theta)
    ret <- do.call(cbind, traces)
    return(ret)
}

## File Name: linking_haberman_als_residual_weights.R
## File Version: 0.373


linking_haberman_als_residual_weights <- function( logaj, logaAt,
        logaM, cutoff, wgtM0, eps, estimation="OLS", lts_prop=.5 )
{
    loga_expected <- TAM::tam_outer( x=logaj, y=logaAt, op='+' )
    loga_resid <- logaM - loga_expected
    NS <- ncol(wgtM0)
    NI <- nrow(wgtM0)
    wgt_adj <- matrix(1, nrow=NI, ncol=NS)
    wgtM <- wgtM0
    k_estimate <- FALSE
    eps_k <- 1e-5

    #*--- settings
    if (estimation=='BSQ'){
        k_fac <- 4.685
        wgt_fn <- linking_haberman_bisquare_weight
    }
    if (estimation=='HUB'){
        k_fac <- 1.345
        wgt_fn <- linking_haberman_huber_weight
    }

    #********************
    #-- estimation BSQ or HUB
    if (estimation %in% c('BSQ','HUB') ){
        if (cutoff==Inf ){
            k <- k_fac*mad_normalized(x=loga_resid)
            k_estimate <- TRUE
            cutoff <- k + eps_k
        }
        if (cutoff<Inf){
            min_x <- apply(abs(loga_resid), 2, min, na.rm=TRUE)
            min_x <- 2*max(min_x)
            cutoff <- max(cutoff, min_x)
            args <- list(x=loga_resid, cutoff=cutoff)
            wgt_adj <- do.call(wgt_fn, args=args )
        }
    }
    #-- estimation LTS
    if (estimation=='LTS'){
        for (ss in 1L:NS){
            e <- loga_resid[,ss]
            e <- e - median(e, na.rm=TRUE)
            dfr_resid <- data.frame(item=1L:NI, e=e )
            dfr_resid <- na.omit(dfr_resid)
            dfr_resid <- dfr_resid[ order(abs(dfr_resid$e), decreasing=TRUE), ]
            wgt_adj[ is.na(loga_resid[,ss]), ss ] <- 0
            n <- nrow(dfr_resid)
            n_del <- floor( (1-lts_prop)*n)
            m1 <- dfr_resid[ 1L:n_del, c(2,1) ]
            wgt_adj[ dfr_resid[ 1L:n_del, 1 ], ss ] <- 0
        }
    }

    #*** update weights
    wgt_adj[ is.na(wgt_adj) ] <- 0
    wgtM <- wgtM0 * wgt_adj + eps

    #--- output
    res <- list(loga_resid=loga_resid, wgt_adj=wgt_adj, wgtM=wgtM,
                    cutoff=cutoff, k_estimate=k_estimate)
    return(res)
}

## File Name: data.wide2long.R
## File Version: 0.263


#--- converts a data frame in wide format into long format
data.wide2long <- function( dat, id=NULL, X=NULL, Q=NULL)
{
    items <- setdiff( colnames(dat), id )
    I <- length(items)
    N <- nrow(dat)
    dat1 <- matrix( t(dat[,items]), nrow=N*I, ncol=1, byrow=FALSE )
    dat2 <- data.frame( dat1 )
    colnames(dat2) <- 'resp'
    dat1 <- data.frame( id_index=rep(1L:N, each=I ) )
    if ( ! is.null(id) ){
        dat1 <- cbind( dat1, rep( dat[, id], each=I ) )
        colnames(dat1)[2] <- id
    }
    dat1 <- cbind( dat1, item=rep( items, N ), item_index=rep(1L:I, N),
                    resp=dat2$resp )
    if ( ! is.null(X) ){
        dat1 <- cbind( dat1, X[ rep(1L:N, each=I ), ] )
    }
    rownames(dat1) <- NULL
    if ( ! is.null(Q) ){
        if ( is.null(colnames(Q) ) ){
            colnames(Q) <- paste0('q',1L:ncol(Q) )
        }
        if ( sum( colnames(Q) %in% 'item' )==0 ){
            Q <- as.data.frame(Q)
            Q$item <- colnames(dat)
        }
        dat1 <- merge( x=dat1, y=Q, by='item', all.x=TRUE )
    }
    dat1 <- dat1[ order( 1E9*dat1$id_index + dat1$item_index ), ]
    dat1 <- data.frame( rowindex=1:(N*I), dat1 )
    return(dat1)
}

## File Name: greenyang.reliability.R
## File Version: 1.236


#---- reliability from a multidimensional nonlinear SEM for dichotomous data
greenyang.reliability <- function( object.tetra, nfactors)
{
    cat("Reliability Estimation Based on a Nonlinear SEM\n\n")
    cat("Green & Yang (2009, Psychometrika). Reliability of summed item scores\n")
    cat("  using structural equation modeling: An alternative to coefficient alpha\n\n")
    mod.omega1 <- sirt_import_psych_omega( m=object.tetra$rho, nfactors=1)

    facloadings <- matrix( mod.omega1$schmid$sl[,1], ncol=1 )
    thresh <- object.tetra$tau

    # reliability for one factor
    rel1 <- reliability.nonlinearSEM( facloadings=facloadings, thresh=thresh )$omega.rel
    # reliability for f factors
    mod.omega <- sirt_import_psych_omega( m=object.tetra$rho, nfactors=nfactors)
    rel1h <- reliability.nonlinearSEM( facloadings=facloadings, thresh=thresh )
    relf <- reliability.nonlinearSEM( facloadings=mod.omega$schmid$orthog,
                                        thresh=thresh )

    #--- calculate Omega Hierarchical Asymptotic
        pthresh <- relf$pthresh
        I <- length(pthresh)
        # create matrix of multiplied facloadings (expected correlation)
        rho.exp <- matrix( 0, nrow=I, ncol=I )
        # colnames(rho.exp) <- rownames(rho.exp) <- rownames(facloadings)
        # reliability matrix
        rel.matrix3 <- rel.matrix2 <- rel.matrix <- rho.exp
        for (ii1 in 1:I){
            for (ii2 in 1:ii1){
                rho.exp[ii1,ii2] <- rel1h$rho.exp[ii1,ii2]
                rho.exp[ii2,ii1] <- rho.exp[ii1,ii2]
                r1 <- rho.exp[ii1,ii2]
                rel.matrix[ii1,ii2] <- sirt_pmvnorm( c(-Inf,-Inf), pthresh[c(ii1,ii2)],
                                         corr=matrix( c( 1, r1, r1, 1),2,2 ) ) - stats::pnorm( pthresh[ii1] ) * pnorm( pthresh[ii2] )
                rel.matrix[ii2,ii1] <- rel.matrix[ii1,ii2]
                # multidimensional analysis
                rho.exp[ii1,ii2] <- relf$rho.exp[ii1,ii2]
                rho.exp[ii2,ii1] <- rho.exp[ii1,ii2]
                r1 <- rho.exp[ii1,ii2]
                rel.matrix2[ii1,ii2] <- sirt_pmvnorm( c(-Inf,-Inf), pthresh[c(ii1,ii2)],
                                         corr=matrix( c( 1, r1, r1, 1),2,2 ) ) -
                                         stats::pnorm( pthresh[ii1] ) * stats::pnorm( pthresh[ii2] )
                rel.matrix2[ii2,ii1] <- rel.matrix2[ii1,ii2]
                rel.matrix3[ii1,ii2] <- rel.matrix2[ii1,ii2]
                rel.matrix3[ii2,ii1] <- rel.matrix3[ii1,ii2]
                if (ii1==ii2 ){
                    r1 <- 1
                    rel.matrix3[ii1,ii2] <- sirt_pmvnorm( c(-Inf,-Inf), pthresh[c(ii1,ii2)],
                                             corr=matrix( c( 1, r1, r1, 1),2,2 ) ) -
                                                 stats::pnorm( pthresh[ii1] ) * stats::pnorm( pthresh[ii2] )
                    rel.matrix3[ii2,ii1] <- rel.matrix3[ii1,ii2]
                                        }
                                }
                        }
        # calculation of reliability
        omega.relha <- sum( rel.matrix ) / sum( rel.matrix2 )
       rel1h <- sum( rel.matrix ) / sum( rel.matrix3 )
    #'''''''''
    # eigenvalue decomposition
    eigenval.rho <- svd( object.tetra$rho )$d
    #'''''''''
#    rel1h <- rel1h$omega.rel
    relf <- relf$omega.rel
    dfr <- data.frame( "coefficient"=c( "omega_1", "omega_h", "omega_t",
                                        "omega_ha","ECV", "ExplVar", "EigenvalRatio"),
                    "dimensions"=c(1,nfactors, nfactors,nfactors,
                        nfactors, NA, NA),
                    "estimate"=c( rel1, rel1h, relf, omega.relha,
                       mod.omega$omega.lim,
                       round( 100*eigenval.rho[1]/sum(eigenval.rho),3),
                       round( eigenval.rho[1]/eigenval.rho[2],3)
                                )
                        )
     dfr <- dfr[ c(1,3,2,4,5,6,7), ]
    rownames(dfr)[1] <- c("Omega Total (1D)")
    rownames(dfr)[2] <- paste("Omega Total (",nfactors,"D)",sep="")
    rownames(dfr)[3] <- paste("Omega Hierarchical (",nfactors,"D)",sep="")
    rownames(dfr)[4] <- paste("Omega Hierarchical Asymptotic (",nfactors,"D)",sep="")
    rownames(dfr)[5] <- paste("Explained Common Variance (",nfactors,"D)",sep="")
    rownames(dfr)[6] <- "Explained Variance (First Eigenvalue)"
    rownames(dfr)[7] <- "Eigenvalue Ratio (1st to 2nd Eigenvalue)"

    dfr1 <- dfr
    dfr1[,"estimate"] <- round( dfr1[,"estimate"], 3)
    cat( paste( rep( "-", 70), collapse="") )
    cat("\n\n")
    print(dfr1)
    cat("\n\n")
    cat( paste( rep( "-", 70), collapse="") )
    cat("\n\nOutput from Hierarchical Factor Analysis (psych package)\n\n" )
    cat( paste( nfactors, "-dimensional model\n\n",sep="") )
    print(mod.omega)
    cat("\n\n")
    cat( paste( rep( ".", 45), collapse="") )
    cat( paste( "\n", 1, "-dimensional model\n\n",sep="") )
    mod.omega1 <- sirt_import_psych_omega( m=object.tetra$rho, nfactors=1)
    print(mod.omega1)
    invisible(dfr1)
}

## File Name: rasch_mml2_mstep_raschtype.R
## File Version: 0.154


#--- M-Step Rasch Type Model
rasch_mml2_mstep_raschtype <- function( theta.k, b, n.k, n, n.jk, r.jk, pi.k, I,
                    conv1, constraints, mitermax, pure.rasch, trait.weights, fixed.a,
                    fixed.c, fixed.d, alpha1, alpha2, h=.0025, designmatrix, group,
                    numdiff.parm, Qmatrix=NULL, old_increment, est.b, center.b,
                    min.b, max.b, prior.b=NULL )
{
    abs.change <- 1
    miter <- 0
    # group estimation
    G <- ncol(n.k)
    b00 <- b
    # number of subjects within groups
    n <- colSums(n.k)
    if ( is.null(Qmatrix) ){
            NT <- length(theta.k)
    } else {
        if ( is.matrix(theta.k) ){
            NT <- nrow(theta.k)
        } else {
            NT <- length(theta.k)
        }
    }

    #*** begin loop
    eps <- numdiff.parm
    h <- h1 <- eps
    h2 <- 1 + 2*eps

    #-- start iterations M-step
    while( abs.change > conv1 & miter < mitermax ){

        b0 <- b

        probs_args <- list( theta.k=theta.k, b=b,
                    fixed.a=fixed.a, fixed.c=fixed.c, fixed.d=fixed.d, alpha1=alpha1,
                    alpha2=alpha2, Qmatrix=Qmatrix, h=0, h1=h1, h2=h2, incr="b" )
        res <- do.call(what=rasch_mml2_prob_genlogis_4pl_evaluate, args=probs_args )
        pjk.M <- res$pjk.M
        qjk.M <- res$qjk.M

        probs_args$h <- h
        res <- do.call(what=rasch_mml2_prob_genlogis_4pl_evaluate, args=probs_args )
        pjk1.M <- res$pjk.M
        qjk1.M <- res$qjk.M

        probs_args$h <- -h
        res <- do.call(what=rasch_mml2_prob_genlogis_4pl_evaluate, args=probs_args )
        pjk2.M <- res$pjk.M
        qjk2.M <- res$qjk.M

        # update item parameters
        ll0 <- ll1 <- ll2 <- matrix( 0, nrow=nrow(n.jk), ncol=G)
        for ( gg in 1:G){
            exp_r <- r.jk[,,gg]
            exp_n <- n.jk[,,gg]
            ll0[,gg] <- rasch_mml2_mstep_calc_loglike( exp_r=exp_r, prob1=pjk.M,
                                exp_n=exp_n, prob0=qjk.M)
            ll1[,gg] <- rasch_mml2_mstep_calc_loglike( exp_r=exp_r, prob1=pjk1.M,
                                exp_n=exp_n, prob0=qjk1.M)
            ll2[,gg] <- rasch_mml2_mstep_calc_loglike( exp_r=exp_r, prob1=pjk2.M,
                                exp_n=exp_n, prob0=qjk2.M)
        }
        # sum across all groups
        ll0 <- rowSums(ll0)
        ll1 <- rowSums(ll1)
        ll2 <- rowSums(ll2)

        # evaluate prior distribution
        parm <- b
        prior_fct <- stats::dnorm
        prior <- prior.b
        res <- rasch_mml2_raschtype_mstep_parameter_group_evaluate_prior(parm=parm,
                h=h, prior=prior, ll0=ll0, ll1=ll1, ll2=ll2, prior_fct=prior_fct )
        ll0 <- res$ll0
        ll1 <- res$ll1
        ll2 <- res$ll2

        #- derivatives
        res <- rasch_mml2_difference_quotient(ll0=ll0, ll1=ll1, ll2=ll2, h=h)
        d1 <- res$d1
        d2 <- res$d2

        #-- sum over contributions
        if ( ! is.null(est.b) ){
            d1 <- stats::aggregate( d1, list(est.b), sum )
            i1 <- d1[,1]
            d1 <- d1[,2]
            d2 <- stats::aggregate( d2, list(est.b), sum )[,2]
        }
        increment <- - d1 / d2
        increment <- sirt_trim_increment(increment=increment,
                            max_increment=max(abs(old_increment)))
        # define old_increment here
        old_increment <- increment
        if ( ! is.null(est.b) ){
            increment <- increment[ match( est.b, i1 ) ]
        }
        b <- b + increment
        # linear parameter constraints
        if ( ! is.null( designmatrix ) & is.null(est.b) ){
            mod <- stats::lm( b ~ 0 + designmatrix  )
            b <- stats::fitted(mod)
        }
        # last item is the sum of all other item difficulties
        center <- is.null(constraints)
        if ( !is.null( constraints) ){
            b[ constraints[,1] ] <- constraints[,2]
        }
        abs.change <- max( abs( b0 - b ) )
        miter <- miter+1
    }
    #*** end iteration loop

    #*** squeeze parameter estimates
    b <- squeeze.mml2( b, c( min.b, max.b ) )

    #-- center b
    if ( center.b ){
        D <- ncol(Qmatrix)
        for (dd in 1:D){
            ind.dd <- which( Qmatrix[,dd] > 0 )
            b[ind.dd] <- b[ind.dd] - sum( Qmatrix[ind.dd,dd] * b[ ind.dd] ) /
                    sum( Qmatrix[ind.dd,dd]  )
        }
    }
    #-- recompute probabilities
    pjk <- prob_genlogis_4pl(theta=theta.k, b=b, a=fixed.a, c=fixed.c, d=fixed.d,
                alpha1=alpha1, alpha2=alpha2, Qmatrix=Qmatrix)

    # calculate log likelihood
    ll <- sapply( 1:G, FUN=function(gg){
            sum( rasch_mml2_mstep_calc_loglike( exp_r=r.jk[,,gg], prob1=t(pjk), exp_n=n.jk[,,gg]) )
            } )
    ll <- sum(ll)
    #-- output
    res <- list( b=b, pi.k=pi.k, ll=ll, miter=miter, center=center, G=G,
                    old_increment=increment, se.b=sqrt( 1 /abs(d2) ) )
    return(res)
}


.m.step.raschtype <- rasch_mml2_mstep_raschtype

## File Name: rasch.mirtlc.R
## File Version: 91.651


#*** Multidimensional Latent Class IRT models
rasch.mirtlc <- function( dat, Nclasses=NULL, modeltype="LC",
    dimensions=NULL, group=NULL, weights=rep(1,nrow(dat)),
    theta.k=NULL, ref.item=NULL, distribution.trait=FALSE,  range.b=c(-8,8), range.a=c(.2, 6 ),
    progress=TRUE, glob.conv=10^(-5), conv1=10^(-5), mmliter=1000, mstep.maxit=3,
    seed=0, nstarts=1, fac.iter=.35 )
{
    #--- preliminaries
    dat <- as.matrix(dat)
    theta.normal <- FALSE
    a <- c(1)
    # handle warnings
    warn_temp <- options()$warn
    if ( is.null(theta.k) ){
        theta.fixed<-FALSE
    } else {
        theta.fixed <- TRUE
        if( is.vector(theta.k) ){ Nclasses <- length(theta.k) }
        if( is.matrix(theta.k) ){ Nclasses <- nrow(theta.k) }
    }
    s1 <- Sys.time()
    if ( is.null(group)){ group <- rep(1, nrow(dat)) }
    G <- length( unique( group ))

    dat2 <- dat
    dat2.resp <- 1* ( 1- is.na(dat ))
    dat2[ is.na(dat) ] <- 0
    # dat1: calculate matrix of frequencies
    dat1 <- matrix( NA, nrow(dat), Nclasses + 1 )
    dat1[,2] <- weights
    I <- ncol(dat2)     # number of items
    n <- nrow(dat2)     # number of persons

    #***initialize probabilities
    if ( modeltype=="LC"){
        D <- 1
        pi.k <- rep( 1/Nclasses, Nclasses )
        if (G>1){ pi.k <- matrix( pi.k, nrow=Nclasses, ncol=G ) }
        lc.probs <- matrix( NA, nrow=Nclasses, ncol=I )
        b <- - stats::qnorm( colMeans( dat, na.rm=T ) )
        theta.k <- stats::qnorm( ( seq( 1, Nclasses, 1 ) - .5 ) / Nclasses )
        for (cc in 1:Nclasses ){
            lc.probs[ cc, ] <- stats::plogis( theta.k[cc] - b )
        }
        pjk <- lc.probs
    }  # end LC

    #*** MLC
    if ( modeltype %in% c("MLC1","MLC2") ){
        if ( is.null( dimensions ) ){
            D <- 1
        } else {
            D <- length( unique( dimensions ) )
            dimensions <- match( dimensions, unique( dimensions ) )
            distribution.trait <- FALSE
        }
        inut <- is.null(theta.k)
        if (inut){
            theta.k <- 2* stats::qnorm( seq( 1 / Nclasses / 2, 1, 1/Nclasses ) )
        } else {
            if (D==1){ Nclasses <- length(theta.k )  }    # works for D=1
            if (D>1){ Nclasses <- nrow(theta.k )  }
        }
        if ( D==1){
            pi.k <- stats::dnorm(theta.k)
            Qmatrix <- NULL
        }
        if ( D > 1 ){
            Qmatrix <- matrix( 0, I, D )
            Qmatrix[ cbind( 1:I, dimensions ) ] <- 1
            if ( inut ){
                theta.k <- matrix( theta.k, nrow=Nclasses, ncol=D, byrow=FALSE)
                theta.kstart <- theta.k
                if ( seed[1] !=0 ){
                    if (seed[1] >0){  set.seed( seed[1] ) } else{ set.seed( Sys.time() ) }
                    theta.k <- theta.k + matrix( rnorm( Nclasses*D, sd=2 ), ncol=D )
                }
            } # end inut (is.null(theta.k))
            if ( D > 1 ){ Nclasses <- nrow( theta.k ) }
            pi.k <- rep(1/Nclasses, Nclasses )
        } # end if D > 1
        pi.k <- pi.k / sum( pi.k )
        b <- - stats::qlogis( colMeans( dat, na.rm=T ) )
        a <- rep(1,I)
        if (G>1){ pi.k <- matrix( pi.k, nrow=Nclasses, ncol=G ) }
        lc.probs <- matrix( NA, nrow=Nclasses, ncol=I )
        if (D==1){
            for (cc in 1:Nclasses ){
                lc.probs[ cc, ] <- stats::plogis( theta.k[cc] - b )
            }
        }
        pjk <- lc.probs
        # design matrix theta
        des1 <- matrix( 0, nrow=I*Nclasses, ncol=Nclasses )
        g1 <- cbind( seq(1,I*Nclasses), rep( 1:Nclasses, each=I ) )
        des1[g1] <- 1
        des1 <- rbind( des1, des1 )
        if ( D==1 ){ des.theta <- des1 }
        if ( D > 1 ){
            des.theta <- NULL
            for (dd in 1:D){
                ind.dd <- which( dimensions !=dd )
                sel.dd <- ( 1:( nrow(des1) ) ) %% I
                sel.dd[ sel.dd==0 ] <- I
                des1.dd <- des1
                des1.dd[ sel.dd %in% ind.dd, ] <- 0
                des.theta <- cbind( des.theta, des1.dd )
            }
        }
        # design matrix item difficulties
        des2 <- matrix( 0, nrow=I*Nclasses, ncol=I )
        g1 <- cbind( seq(1,I*Nclasses), rep( 1:I, Nclasses ) )
        des2[g1] <- -1
        des2 <- rbind( des2, des2 )
        if ( theta.normal ){ theta.fixed <- TRUE }
        # select reference item
        pval <- colMeans( dat, na.rm=T )
        p1 <- sort( pval, index.return=TRUE)$ix
        if ( D==1 & is.null(ref.item) ){
                ref.item <- p1[round(I/2)]
                        }
        if (D>1 & is.null(ref.item) ){
            ref.item <- NULL
            for (dd in 1:D){
                # dd <- 1
                h1 <- which( dimensions==dd )
                ind.dd <- h1[ which.min( pval[h1] - 1/2 )[1] ]
                ref.item <- c( ref.item, ind.dd )
                    }
        }

#        if ( theta.normal ){ ref.item <- NULL }
        des2 <- des2[, - ref.item ]
        # reference item
        b[ ref.item ] <- 0
        est.a <- seq(1,I)
        est.a[ref.item] <- 0
        des.b <- des2
    }  # end MLC
    #***************

    #** iterate over different starts
    if ( ( seed[1] < 0) | ( length(seed) < nstarts) ){
        seed <- round( stats::runif( nstarts, 1, 10000 ))
    }
    devL <- rep(NA, nstarts )
    NNdev <- NN1dev <-  1e90

    #---- different starts
    for (nn in 1:nstarts ){
        seed.nn <- seed[nn]
        if ( seed[nn] > 0 ){
            set.seed( seed.nn )
            pjk <- matrix( stats::runif( I*Nclasses ), nrow=Nclasses, ncol=I )
            theta.k <- theta.k + matrix( rnorm( Nclasses*D, sd=.7 ), ncol=D)
            if (modeltype%in%c("MLC1","MLC2") & ( nstarts > 1 ) & ( ! theta.fixed ) & (D>1) ){
                pi.k <- stats::runif( Nclasses, 0, 1 )
                pi.k <- pi.k / sum( pi.k )
                theta.k <- .7*theta.k + matrix( stats::rnorm( Nclasses*D, sd=.97 ), ncol=D )
            }
            if (modeltype%in%c("MLC1","MLC2") & ( nstarts > 1 ) & ( ! theta.fixed ) & (D==1) ){
                    pi.k <- stats::runif( Nclasses, 0, 1 )
                    pi.k <- pi.k / sum( pi.k )
                    theta.k <- .7*theta.k + stats::rnorm( Nclasses*D, sd=.97 )
            }
        }
        dev <- 1
        iter <- 0
        dev.change <- par.change <- 1000
        # display
        disp <- "...........................................................\n"
        NN1dev <-  1e90

        #--------------------------------------------------------------
        #------------------ begin EM algorithm -------------------------
        while ( ( dev.change > glob.conv | par.change > conv1 ) & iter < mmliter ){
            if (progress){
                cat(disp)
                cat("Iteration", iter+1, "   ", paste( Sys.time() ), "\n" )
                utils::flush.console()
            }
            pjk0 <- pjk
            pi.k0 <- pi.k
            theta.k0 <- theta.k
            dev0 <- dev
            # E step latent class analysis
            if ( modeltype=="LC"){
                res1 <- rasch_mirtlc_estep_lc( dat1=dat1, dat2=dat2, dat2.resp=dat2.resp,
                            pi.k=pi.k, pjk=pjk, I=I, group=group, G=G, theta.k=theta.k,
                            f.qk.yi=NULL )
            }
            if ( modeltype %in% c("MLC1","MLC2")){
                res1 <- rasch_mirtlc_estep_mlc1( dat1=dat1, dat2=dat2, dat2.resp=dat2.resp,
                                pi.k=pi.k, pjk=pjk, I=I, b=b, a=a, group=group, G=G,
                                theta.k=theta.k, D=D, dimensions=dimensions,
                                Qmatrix=Qmatrix, f.qk.yi=NULL )
            }
    # a1 <- Sys.time() ; adiff <- a1-a0 ; cat("\ne step", adiff ) ; a0 <- a1
            n.k <- res1$n.k
            n.jk <- res1$n.jk
            r.jk <-  res1$r.jk
            f.qk.yi <- res1$f.qk.yi
            pjk <- res1$pjk
            f.yi.qk <- res1$f.yi.qk
            ll <- res1$ll
            dev <- -2*ll
            # Mstep:
            if ( modeltype=="LC"){
                res2 <- rasch_mirtlc_mstep_lc( pjk=pjk, n.k=n.k, r.jk=r.jk, n.jk=n.jk, G=G,
                                Nclasses=Nclasses )
            }
            if ( modeltype %in% c("MLC1","MLC2") ){
                options(warn=-1)
                res2 <- rasch_mirtlc_mstep_mlc1( pjk=pjk, n.k=n.k, r.jk=r.jk, n.jk=n.jk, G=G,
                            Nclasses=Nclasses, theta.k=theta.k, b=b, a=a, I=I, ref.item=ref.item,
                            mstep.maxit=mstep.maxit, des.theta=des.theta, des.b=des.b,
                            theta.fixed=theta.fixed, theta.normal=theta.normal, f.qk.yi=f.qk.yi, D=D,
                            distribution.trait=distribution.trait, est.a=est.a, Qmatrix=Qmatrix,
                            modeltype=modeltype, range.b=range.b, range.a=range.a, iter=iter,
                            fac.iter=fac.iter, dimensions=dimensions )
                b <- res2$b
                theta.k <- res2$theta.k
                pi.k <- res2$pi.k
                a <- res2$a
                options(warn=warn_temp)
            }

    # a1 <- Sys.time() ; adiff <- a1-a0 ; cat("\nm step", adiff ) ; a0 <- a1
            pi.k <- res2$pi.k
            pjk <- res2$pjk
            # prevent label switching (modeltype=="LC")
            if ( modeltype=="LC"){
                ind <- order( rowMeans( pjk ) )
                pjk <- pjk[ ind,, drop=FALSE]
                if (G==1){
                    pi.k <- pi.k[ ind ]
                }
                if (G>1){
                    pi.k <- pi.k[ ind, ]
                }
            }

            # convergence criteria
            a1 <- max( abs( pjk - pjk0 ) )
            a2 <- max( abs( pi.k - pi.k0 ) )
            if ( modeltype %in% c("MLC1","MLC2") ){
                a3 <- max( abs( theta.k - theta.k0 ))
            } else {
                a3 <- 0
            }
            dev.change <- abs( ( dev - dev0)/ dev0 )
            par.change <- max( c(a1,a2,a3))
            iter <- iter + 1

            # settings
            if ( dev < NN1dev ){
                NN1pjk <- pjk
                NN1pi.k <- pi.k
                NN1dev <- dev
                NN1ll <- ll
                NN1res1 <- res1
                NN1theta.k <- theta.k
                NN1a <- a
                NN1b <- b
                NN1iter <- iter
            }
            if (progress){
                cat( paste( "   Deviance=",
                        round( dev, 4 ),
                        if (iter > 0 ){ " | Deviance change=" } else {""},
                        if( iter>0){round( - dev + dev0, 6 )} else { ""},
                        "  START ", nn, " (Seed ", seed.nn, ")\n",sep="") )
                # maximum probability change
                cat( paste( "    Maximum item parameter change=",
                                paste( round(a1,6), collapse=" " ), "\n", sep=""))
                # maximum probability distribution parameter change
                cat( paste( "    Maximum probability distribution change=",
                                paste( round(a2,6), collapse=" " ), "\n", sep=""))
                cat( paste( "    Maximum theta parameter change=",
                                paste( round(a3,6), collapse=" " ), "\n", sep=""))
            }
        }
        #---------------- end EM algorithm  -------------------------------

        NN1pjk -> pjk
        NN1pi.k -> pi.k
        NN1dev -> dev
        NN1ll -> ll
        NN1res1 -> res1
        NN1theta.k -> theta.k
        NN1a -> a
        NN1b -> b
        NN1iter -> iter

        #----- collect results of nstarts
        if ( ( nn==1 ) | ( ( nn > 1 ) & ( dev < NNdev ) )  ){
            NNpjk <- pjk ;         NNpi.k <- pi.k
            NNdev <- dev ;         NNll <- ll ;
            NNres1 <- res1 ; NNtheta.k <- theta.k
            NNa <- a ; NNb <- b ; NNiter <- iter
        }
        devL[nn] <- dev
    } #-- end multiple starts


    if (nstarts > 1){
        NNpjk -> pjk ;         NNpi.k -> pi.k
        NNdev -> dev ;         NNll -> ll
        NNres1 -> res1 ; NNtheta.k -> theta.k
        NNa -> a ; NNb -> b ; NNiter -> iter
    }
    #############################################
    # labels for pjk
    colnames(pjk) <- colnames(dat)
    rownames(pjk) <- paste("Class", 1:Nclasses, sep="")
    if (G==1 ){ names(pi.k) <- rownames(pjk) }
    if (G>1 ){ rownames(pi.k) <- rownames(pjk) }

    # Information criteria
    # calculations for information criteria
    ic <- list( "deviance"=dev, "n"=nrow(dat) )
    # number of parameters to be estimated
    if ( modeltype=="LC" ){
        ic[[ "itempars" ]] <- Nclasses * I
        ic[[ "traitpars" ]] <- ( Nclasses - 1 ) * G
                        }
    if ( modeltype%in%c("MLC1","MLC2") ){
        ic[[ "itempars" ]] <- I - D
        if ( modeltype=="MLC2"){ ic$itempars <- 2*(I-D) }
        ic[[ "traitpars" ]] <- D*Nclasses + ( Nclasses - 1 ) * G
        # trait distribution + probabilities
        ic$traitpars <- ic$traitpars - D*theta.fixed*Nclasses
        if ( distribution.trait=="normal" ){
            ic$traitpars <- 2*G
                                        }
        if ( distribution.trait=="smooth2" ){    ic$traitpars <- 2*G    }
        if ( distribution.trait=="smooth3" ){    ic$traitpars <- 3*G    }
        if ( distribution.trait=="smooth4" ){    ic$traitpars <- 4*G    }
                                        }

    ic$np <- ic$itempars + ic$traitpars

    ic$n <- n # number of persons
    # AIC
    ic$AIC <- dev + 2*ic$np
    # BIC
    ic$BIC <- dev + ( log(ic$n) )*ic$np
    # CAIC (conistent AIC)
    ic$CAIC <- dev + ( log(ic$n) + 1 )*ic$np
    # corrected AIC
    ic$AICc <- ic$AIC + 2*ic$np * ( ic$np + 1 ) / ( ic$n - ic$np - 1 )

    #---- item and trait parameter
    cor.trait <- skewness.trait <- mean.trait <- sd.trait <- trait <- item <- NULL
    if (modeltype%in%c("MLC1","MLC2") ){
        mean.trait <- rep(NA,G)
        if (D>1){ skewness.trait <- sd.trait <- mean.trait <- matrix(NA, D, G ) }
        item <- data.frame("b"=b, "a"=a )
        rownames(item) <- colnames(dat)
        item$thresh <- b*a

        trait <- data.frame( "theta"=theta.k )
        if ( D>1){ colnames(trait) <- paste( "theta.Dim", 1:D, sep="") }
        # mean
        if (D==1){
            mean.trait[1] <- M <- sum( pi.k[,1] * theta.k )
#            item$b.cent <- item$b - M
#            trait$theta.cent <- theta.k - M
            mean.trait <- sapply(1:G, FUN=function(gg){
                                    sum( pi.k[,gg] * theta.k )  } )
            sd.trait <- sqrt( sapply(1:G, FUN=function(gg){
                            sum( pi.k[,gg] * theta.k^2 )  } ) - mean.trait^2 )

            skewness.trait <- sapply(1:G, FUN=function(gg){
                            m3 <- sum( pi.k[,gg] * ( theta.k - mean.trait[gg] )^3 )
                            m3 / sd.trait[gg]^3
                                } )
                        }
            # mean and SD trait in multidimensional case
            if (D> 1){
                cor.trait <- array( NA, dim=list(D,D,G) )
                for (dd in 1:D){
                for (gg in 1:G){
#                    dd <- 1 ;                 gg <- 1
                    mean.trait[ dd, gg ] <- sum( theta.k[,dd] * pi.k[,gg] )
                    sd.trait[dd,gg] <- sqrt( sum( theta.k[,dd]^2 * pi.k[,gg] ) - mean.trait[dd,gg]^2 )
                     m3 <- sum( pi.k[,gg] * ( theta.k[,dd] - mean.trait[gg] )^3 )
                    skewness.trait[dd,gg] <- m3 / sd.trait[gg]^3
                                }
                            }
                for (dd1 in 1:D){
                for (dd2 in 1:D){
                for (gg in 1:G){
#                    dd <- 1 ;                 gg <- 1
                    mdd1 <- mean.trait[ dd1, gg ]
                    mdd2 <- mean.trait[dd2,gg]
                    sdd1 <- sd.trait[ dd1, gg ]
                    sdd2 <- sd.trait[dd2,gg]
                    cor.trait[dd1,dd2,gg] <- sum( theta.k[,dd1] * theta.k[,dd2] * pi.k[,gg] )
                    cor.trait[dd1,dd2,gg] <- ( cor.trait[dd1,dd2,gg] - mdd1 * mdd2 ) / ( sdd1 * sdd2 )
                                }
                            } }

                        } # end D>1
            if (G==1){ trait$prob <- pi.k }
            if (G>1){ trait <- cbind( trait, pi.k ) }
            trait <- data.frame( trait )
            if (G>1){
                colnames(trait)[ seq( ncol(trait) -G+1, ncol(trait), 1) ] <-
                            paste("prob.Group", 1:G, sep="") }
            # standardized theta dimensions
            if (D>1){
             for (dd in 1:D){
#            dd <- 1
                trait[, paste("theta.stand.Dim", dd,sep="") ] <-
                        ( trait[, dd ] - mean.trait[dd,1] ) / sd.trait[dd,1]
                            } # end dd in 1:D
                        } # end D>1
                    }  # end MLC1
    #####################################
    # output LC
    if ( modeltype=="LC"){
        theta.k <- diag( length( theta.k) )
    }

    ##################################################
    # item response probabilities
    d1 <- dim(pjk)
    rprobs <- array( 0, dim=c( d1[2], 2, d1[1] ) )
    dimnames(rprobs)[[1]] <- colnames(dat)
    rprobs[,2,] <- t( pjk )
    rprobs[,1,] <- 1 - t(pjk)

    #--- OUTPUT
    s2 <- Sys.time()
    res <- list( pjk=pjk, rprobs=rprobs, pi.k=pi.k, theta.k=theta.k,
            item=item, trait=trait, mean.trait=mean.trait,
            sd.trait=sd.trait, skewness.trait=skewness.trait,
            cor.trait=cor.trait, ic=ic, D=D, G=G,
            deviance=dev, ll=ll, Nclasses=Nclasses,
            modeltype=modeltype, estep.res=res1, dat=dat,
            devL=devL, seedL=seed,  iter=iter, s1=s1, s2=s2,
            distribution.trait=distribution.trait)
    class(res) <- "rasch.mirtlc"
    return(res)
}

## File Name: mcmc.2pno.R
## File Version: 1.271
##############################################
# MCMC estimation 2PNO model
mcmc.2pno <- function(dat, weights=NULL, burnin=500, iter=1000, N.sampvalues=1000,
        progress.iter=50, save.theta=FALSE )
{
    s1 <- Sys.time()
    # data preparation
    dat0 <- dat
    dat <- as.matrix(dat)
    dat[ is.na(dat0) ] <- 0
    dat.resp <- 1-is.na(dat0)
    N <- nrow(dat)
    I <- ncol(dat)
    eps <- 10^(-10)
    # redefine weights
    if (! is.null(weights) ){
        weights <- N * weights / sum(weights )
                    }
    # set initial values
    a <- rep(1,I)
    b <- - stats::qnorm( (colMeans(dat0, na.rm=TRUE) + .01 )/1.02 )
    # item parameters in matrix form
    aM <- matrix( a, nrow=N, ncol=I, byrow=TRUE)
    bM <- matrix( b, nrow=N, ncol=I, byrow=TRUE)
    theta <- stats::qnorm( ( rowMeans( dat0,na.rm=TRUE ) + .01 ) / 1.02 )
    # define lower and upper thresholds
    ZZ <- 1000
    threshlow <- -ZZ + ZZ*dat
    threshlow[ is.na(dat0) ] <- -ZZ
    threshupp <- ZZ*dat
    threshupp[ is.na(dat0) ] <- ZZ
    # saved values
    SV <- min( N.sampvalues, iter - burnin )
    svindex <- round( seq( burnin, iter, len=SV )     )
    a.chain <- matrix( NA, SV, I )
    b.chain <- matrix( NA, SV, I )
    theta.chain <- matrix( NA, SV, N )
    deviance.chain <- rep(NA, SV)
    zz <- 0
    #**********************
    # begin iterations
    for (ii in 1:iter){

        #****
        # draw latent data Z
        Z <- .draw.Z.2pl( aM, bM, theta, N, I, threshlow, threshupp )

        #***
        # draw latent traits theta
        res <- .draw.theta.2pl( aM, bM, N, I, Z )
        theta <- res$theta

        #***
        # draw item parameters
        res <- .draw.itempars.2pl( theta, Z, I, N, weights)
        a <- res$a
        b <- res$b

        # item parameters in matrix form
        aM <- matrix( a, nrow=N, ncol=I, byrow=TRUE)
        bM <- matrix( b, nrow=N, ncol=I, byrow=TRUE)

        # save parameters
        if ( ii %in% svindex ){
            zz <- zz+1
            a.chain[ zz, ] <- a
            b.chain[ zz, ] <- b
            theta.chain[ zz, ] <- theta
            deviance.chain[zz] <- .mcmc.deviance.2pl( aM, bM, theta, dat,
                     dat.resp,     weights, eps )
                        }
        # print progress
        if ( ( ii %% progress.iter )==0 ){
            cat( "Iteration", ii, " | ", paste(Sys.time()), "\n")
            flush.console() }

                }
    ##############################
    # output
    # Information criteria
    ic <- .mcmc.ic.2pl( a.chain, b.chain, theta.chain, N, I,
        dat, dat.resp, weights, eps,
        deviance.chain )

    # EAP reliability and person parameter estimates
    res <- .mcmc.person.2pno( theta.chain, weights )
    EAP.rel <- res$EAP.rel
    person <- res$person

    #-----
    # create MCMC object
    a <- a.chain
    b <- b.chain
    theta <- theta.chain
    colnames(a) <- paste0("a[", 1:I, "]")
    colnames(b) <- paste0("b[", 1:I, "]")
    colnames(theta) <- paste0("theta[", 1:N, "]")
    mcmcobj <- cbind( deviance.chain, a, b )
    colnames(mcmcobj)[1] <- "deviance"
    if (save.theta){ mcmcobj <- cbind( mcmcobj, theta ) }
    class(mcmcobj) <- "mcmc"
    attr(mcmcobj, "mcpar") <- c( burnin+1, burnin+SV, 1 )
    mcmcobj <- sirt_import_coda_as.mcmc.list( mcmcobj )

    #----
    # summary of the MCMC output
    summary.mcmcobj <- mcmc.list.descriptives( mcmcobj )
    # number of estimated parameters
#    np <- 2*I

    # time
    s2 <- Sys.time()
    time <- list( "start"=s1, "end"=s2, "timediff"=s2-s1 )
    #----
    # result list
    res <- list( "mcmcobj"=mcmcobj, "summary.mcmcobj"=summary.mcmcobj,
            "ic"=ic,
            "burnin"=burnin, "iter"=iter,
            "a.chain"=a.chain, "b.chain"=b.chain, "theta.chain"=theta.chain,
            "deviance.chain"=deviance.chain,
            "EAP.rel"=EAP.rel, "person"=person, "dat"=dat0, "weights"=weights,
            "time"=time, "model"="2pno", "description"="2PNO Model" )
    class(res) <- "mcmc.sirt"
    return(res)
    }

## File Name: xxirt_ic.R
## File Version: 0.195


#-- information criteria xxirt
xxirt_ic <- function( dev, N, par_items, par_Theta, I, par_items_bounds, np_item=NULL,
            np_theta=NULL, estimator="ML")
{
    # Information criteria
    ic <- list( deviance=dev, n=N, I=I )
    ic$np.items <- sum(par_items_bounds$active)
    if ( ! is.null(np_item) ){
        ic$np.items <- np_item
    }
    NPT <- length(par_Theta)
    if (!is.null(np_theta)){
        NPT <- np_theta
    }
    ic$np.Theta <- NPT
    ic <- xxirt_ic_compute_criteria(ic=ic, estimator=estimator)
    return(ic)
}

## File Name: plot.mcmc.sirt.R
## File Version: 0.29

######################################################
# plot results of objects of class mcmc.sirt

plot.mcmc.sirt <- function( x, layout=1, conflevel=.90,
    round.summ=3, lag.max=.1, col.smooth="red", lwd.smooth=2,
    col.ci="orange", cex.summ=1, ask=FALSE, ... )
{

    object <- x    # rename x into object
    mcmcobj <- (object$mcmcobj)[[1]]
    lag.max <- round( nrow(mcmcobj) * lag.max )

    # layout type
    # layout=1 : standard output from coda package
    if (layout==1){
        graphics::plot(object$mcmcobj, ask=ask, ...)
    }

    #***************
    # layout=2
    if (layout==2){
        mcmcobj <- (object$mcmcobj)[[1]]
        lag.max <- min( nrow(mcmcobj), lag.max )
        # index vector
        a1 <- attr(mcmcobj,"mcpar")
        iterindex <- seq(a1[1], a1[2], a1[3] )

        smcmcobj <- object$summary.mcmcobj
        VV <- ncol(mcmcobj)
        ci.quant <- - stats::qnorm( (1-conflevel)/2 )
        graphics::par( mfrow=c(2,2))
        for (vv in 1:VV){
            x.vv <- as.vector( mcmcobj[,vv] )
            parm.vv <- colnames(mcmcobj)[vv]
            sparm.vv <- smcmcobj[ smcmcobj$parameter==parm.vv, ]
            #***
            # traceplot
            graphics::plot( iterindex, x.vv, type="l",  main=paste0( "Traceplot of ", parm.vv ),
                xlab="Iterations", ylab="", ... )
            x1 <- as.numeric( x.vv )
            xmin <- min(x1)
            xmax <- max(x1)
            # l1 <- loess( x1 ~ iterindex )$fitted
            # include moving average here!!
            l1 <- sirt_moving_average(x1, B=round( lag.max / 2 ), fill=FALSE)
            graphics::lines( iterindex,l1, col=col.smooth, lwd=lwd.smooth )
            #***
            # density estimate
            graphics::plot( stats::density( x.vv ), main=paste0( "Density of ", parm.vv ) )

            c1 <- stats::quantile( x1, ( 1 - conflevel  ) / 2 )
            c2 <- stats::quantile( x1, 1 - ( 1 - conflevel  ) / 2 )
#            lines( sparm.vv$Mean + c(-1,1)*ci.quant * sparm.vv$SD, c(0,0), col=col.ci, lwd=3 )
            graphics::lines( c(c1,c2), c(0,0), col=col.ci, lwd=3 )
            graphics::points( sparm.vv$Mean, 0, pch=17, col=col.ci, cex=1.5)
            #***
            # autocorrelation function
            stats::acf( x.vv, lag.max=lag.max,
                main=paste0( "Autocorrelation of ", parm.vv ) )
            #***
            # numerical summary
            graphics::plot( c(0,1), c(0,1), axes=FALSE, xlab="", ylab="",
                    main=paste0( "Summary of ", parm.vv ), type="n", ...)
            x0 <- 0 ; y0 <- 0
            # heights.summ=c( .05,  .20, .35,  .5, .65, .8, .95)
            heights.summ=c( .05,  .15, .25,  .35, .45, .55, .65, .75)
            graphics::text( x0 + .0015, y0 + heights.summ[8], "Posterior Mean=", cex=cex.summ, pos=4)
            graphics::text( x0 + .5, y0 + heights.summ[8],
                paste0( sirt_format_numb( x=mean( x1 ), digits=round.summ)  ), pos=4 )
            hvv <- heights.summ[7]
            graphics::text( x0 + .0015, y0 + hvv, "Posterior Mode=", cex=cex.summ, pos=4)
            graphics::text( x0 + .5, y0 + hvv,
                paste0( sirt_format_numb( x=sparm.vv$MAP, digits=round.summ)  ), pos=4 )

            graphics::text( x0 + .0015, y0 + heights.summ[6], "Posterior SD=", cex=cex.summ, pos=4)
            graphics::text( x0 + .5, y0 + heights.summ[6],
                paste0( sirt_format_numb( x=stats::sd( x1 ), digits=round.summ)  ), pos=4 )

            hvv <- heights.summ[5]
            graphics::text( x0 + .0015, y0 + hvv,
                            paste( round(100*conflevel ), "% Credibility Interval=",sep=""),
                            cex=cex.summ, pos=4 )

            hvv <- heights.summ[4]
                ci.lower <- sirt_format_numb( stats::quantile( x1, ( 1 - conflevel  ) / 2 ), digits=round.summ )
                ci.upper <- sirt_format_numb( stats::quantile( x1, 1- ( 1 - conflevel  ) / 2 ), digits=round.summ )
            graphics::text( x0 + .25, y0 + hvv,
                            paste( "[", ci.lower,    ",", ci.upper, "]",  sep=""),
                            cex=cex.summ, pos=4)
            hvv <- heights.summ[3]
            graphics::text( x0 + .0015, y0 + hvv, "Rhat=", cex=cex.summ, pos=4)
            graphics::text( x0 + .5, y0 + hvv,
                paste0( sirt_format_numb( x=sparm.vv$Rhat, digits=2)  ), pos=4 )
            hvv <- heights.summ[2]
            graphics::text( x0 + .0015, y0 + hvv, "PercSERatio=", cex=cex.summ, pos=4)
            graphics::text( x0 + .5, y0 + hvv,
                paste0( sirt_format_numb( x=sparm.vv$PercSERatio, digits=1)  ), pos=4 )

            hvv <- heights.summ[1]
            graphics::text( x0 + .0015, y0 + hvv, "Effective Sample Size=", cex=cex.summ, pos=4)
            graphics::text( x0 + .705, y0 + hvv,
                paste0( sirt_format_numb( x=sparm.vv$effSize, digits=1)  ), pos=4 )
            graphics::par(ask=ask)
                    }
            graphics::par(mfrow=c(1,1))
    }
}

## File Name: invariance_alignment_cfa_config_estimate.R
## File Version: 0.202

invariance_alignment_cfa_config_estimate <- function(dat_gg, N, weights_gg=NULL,
    model="2PM", ...)
{
    requireNamespace('MASS')
    is_data <- sirt_is_data(dat=dat_gg)
    #-- create lavaan model
    if (is_data){
        I_gg <- ncol(dat_gg)
        items_gg <- colnames(dat_gg)
    } else {
        mu <- dat_gg[[1]]
        Sigma <- dat_gg[[2]]
        I_gg <- length(mu)
        items_gg <- names(mu)
        if (is.null(items_gg)){
            items_gg <- paste0('I',1L:I)
        }
        names(mu) <- items_gg
        rownames(Sigma) <- items_gg
        colnames(Sigma) <- items_gg
    }
    lavmodel <- invariance_alignment_cfa_config_estimate_define_lavaan_model(
                    items_gg=items_gg, label_F='F', model=model)
    if (is.null(weights_gg)){
        weights_name <- NULL
    } else {
        dat_gg$weights <- weights_gg
        weights_name <- 'weights'
    }

    #-- estimate lavaan model
    args <- list( model=lavmodel, std.lv=TRUE, meanstructure=TRUE, ...)
    if (is_data){
        args$data <- dat_gg
        args$sampling.weights <- weights_name
    } else {
        args$sample.cov <- Sigma
        args$sample.mean <- mu
        args$sample.nobs <- min(1e20,N)
    }
    mod <- do.call(what='sirt_import_lavaan_cfa', args=args)
    partable <- sirt_import_lavaan_parameterTable(object=mod)
    lambda <- partable[ partable$op=='=~', 'est']
    nu <- partable[ partable$op=='~1', 'est'][1L:I_gg]
    err_var <- partable[ partable$op=='~~', 'est'][1L:I_gg]
    nobs <- mod@Data@nobs[[1]]
    # vcov <- lavaan::lavInspect(object=mod, what='information')
    vcov <- mod@vcov$vcov
    ind <- c(1L:I_gg, 2*I_gg+1L:I_gg)
    vcov <- vcov[ ind, ind ]

    #--- output
    res <- list(lambda=lambda, nu=nu, err_var=err_var, nobs=nobs, vcov=vcov)
    return(res)
}

## File Name: mgsem_compute_model_implied_moments.R
## File Version: 0.169


mgsem_compute_model_implied_moments <- function(est, is_B=FALSE, calc_Sigma=TRUE,
    calc_Mu=TRUE)
{
    Mu <- NULL
    Sigma <- NULL
    if (is_B){
        D <- ncol(est$PHI)
        ID <- diag(rep(1,D))
        B <- res$B
        B1 <- mgsem_ginv(ID-B)
        LAMB <- est$LAM %*% B1
        Mu <- LAMB %*% est$ALPHA + est$NU
        # Sigma <- LAMB %*% tcrossprod(est$PHI, LAMB) + est$PSI
        if (calc_Sigma){
            Sigma <- sirt_rcpp_mgsem_compute_cov(LAM=LAMB, PHI=est$PHI, PSI=est$PSI)
        }
    } else {
        if (calc_Mu){
            Mu <- est$LAM %*% est$ALPHA + est$NU
        }
        # Sigma <- est$LAM %*% tcrossprod(est$PHI, est$LAM) + est$PSI
        if (calc_Sigma){
            Sigma <- sirt_rcpp_mgsem_compute_cov(LAM=est$LAM, PHI=est$PHI, PSI=est$PSI)
        }
    }
    #*** output
    res <- list(Mu=Mu, Sigma=Sigma)
    return(res)
}

## File Name: lq_fit_estimate_power.R
## File Version: 0.15


lq_fit_estimate_power <- function(e, pow_init=2, lower_pow=.1, upper_pow=10)
{
    pow0 <- pow_init
    me <- mean(e)
    n <- length(e)
    est_pow <- TRUE
    if (stats::var(e)<1e-5){
        est_pow <- FALSE
        p <- pow0
    }
    if (est_pow){
        I <- sum( abs(e-me) / n ) / sqrt( sum( (e-me)^2 ) / (n-1) )
        vi <- 1/I
        # copied part from normalp::estimatep function
        fvi <- function(p){
            (vi - sqrt(gamma(1/p) * gamma(3/p))/gamma(2/p))^2
        }
        res <- stats::optim(par=pow0, fn=fvi, lower=lower_pow, upper=upper_pow,
                            method='L-BFGS-B')
        p <- res$par
    }
    return(p)
}

## File Name: mcmc_vcov.R
## File Version: 0.13


#--- variance covariance matrix
mcmc_vcov <- function( mcmcobj, exclude="deviance" )
{
    mcmcobj <- mcmc_extract_samples_first_chain(mcmcobj=mcmcobj)
    mcmcobj <- mcmcobj[, ! ( colnames(mcmcobj) %in% exclude ) ]
    res <- stats::var(mcmcobj)
    colnames(mcmcobj) -> colnames(res) -> rownames(res)
    return(res)
}

## File Name: regpolca_penalty_fun_value_grouped.R
## File Version: 0.01

regpolca_penalty_fun_value_grouped <- function(x_ii, combis_ii,
    regular_lam, eps, penalty_used)
{
    diff_ii <- x_ii[ combis_ii[,1] ] - x_ii[ combis_ii[,2] ]
    norm_ii <- regpolca_grouped_norm(x=diff_ii)
    a1 <- penalty_used( x=norm_ii, lambda=regular_lam, eps=eps )
    return(a1)
}

## File Name: rm_facets_pp_mle_calc_pcm.R
## File Version: 0.13


#--- calculate item response probabilities
rm_facets_pp_mle_calc_pcm <- function( theta, a, b, ii )
{
    K <- ncol(b)
    N <- length(theta)
    matrK <- sirt_matrix2( x=0:K, nrow=N)
    eta <- a[ii] * theta * matrK - sirt_matrix2( x=c(0,b[ii,]), nrow=N)
    eta <- exp(eta)
    probs <- eta / rowSums(eta, na.rm=TRUE)
    return(probs)
}

## File Name: sirt_summary_print_elapsed_time.R
## File Version: 0.04

sirt_summary_print_elapsed_time <- function(object)
{
    cat( 'Elapsed time pre-processing', '=', ' ')
    print(object$time$time_pre)
    cat( 'Elapsed time optimization', '=', ' ')
    print(object$time$time_opt)
    cat( 'Elapsed time post-processing', '=', ' ')
    print(object$time$time_post)
    cat('\n')
}

## File Name: regpolca.R
## File Version: 0.195


#- Regularized polytomous latent class analysis
regpolca <- function(dat, nclasses, weights=NULL, group=NULL,
    regular_type="scad", regular_lam=0, regular_grouped="none",
    sd_noise_init=1, par_item_init=NULL, par_Theta_init=NULL, random_starts=1,
    random_iter=20, random_sd=0, conv=1e-5, h=1e-4, mstep_iter=10, maxit=1000,
    verbose=TRUE, par_item_max=10, set_equal=.01, eps=1e-3)
{
    #*** preliminaries
    CALL <- match.call()
    s1 <- Sys.time()

    #** analyze response patterns
    res <- regpolca_proc_data(dat=dat, group=group)
    ncats <- res$ncats
    lca_dich <- res$lca_dich
    I <- res$I
    N <- res$N
    group <- res$group
    groups <- res$groups
    G <- res$G
    Ni <- res$Ni

    #* initial parameters
    if (! is.null(par_item_init)){
        # random_starts <- 0
    }

    #- define theta class distribution
    K <- nclasses
    par_Theta <- xxirt_classprobs_lca_init_par(K=K, G=G, random_sd=random_sd,
                    par_Theta_init=par_Theta_init)
    customTheta  <- xxirt_createThetaDistribution( par=par_Theta,
                            est=rep(TRUE,G*(K-1)), P=xxirt_classprobs_lca)
    Theta <- diag(K)

    #- define item response functions
    res <- regpolca_define_customItems( ncats=ncats, K=K, dat=dat,
                    par_item_max=par_item_max )
    customItems <- res$customItems
    partable <- res$partable
    itemtype <- res$itemtype

    #-- include penalty function
    penalty_fun_item <- NULL
    if (length(regular_lam)==1){
        regular_lam <- c(regular_lam, 0)
    }
    if (regular_grouped!='none'){
        regular_lam <- c(regular_lam[1], 0)
    }
    if (sum(regular_lam)>0){
        combis <- t( utils::combn(K,2) )
        combis_classes_list <- list()
        combis_categ_list <- list()
        for (ii in 1L:I){
            ncats1_ii <- ncats[ii]-1
            #- classes
            combis_ii <- combis
            if (ncats1_ii>1){
                for (cc in 2:ncats1_ii){
                    combis_ii <- rbind(combis_ii, K*(cc-1) + combis)
                }
            }
            combis_classes_list[[ii]] <- combis_ii
            #- fusing categories
            combis_ii <- NULL
            if (ncats1_ii>1){
                combis0 <- t( utils::combn(ncats1_ii,2) )
                combis2 <- K*( combis0 - 1 )
                for (cc in 1L:K){
                    combis_ii <- rbind( combis_ii, combis2+cc )
                }
                combis_ii <- data.frame(combis_ii)
                colnames(combis_ii) <- c('par1','par2')
                nri <- nrow(combis_ii)
                n1 <- nri/nrow(combis0)
                combis_ii$cat1 <- rep(combis0[,1], n1)
                combis_ii$cat2 <- rep(combis0[,2], n1)
                combis_ii$class <- rep(1:K, each=nri/K)
                cat_pair <- 100*combis_ii$cat1+combis_ii$cat2
                combis_ii$cat_pair <- match(cat_pair, unique(cat_pair))
                #rep(1:(ncats_ii), each=nri
            }
            combis_categ_list[[ii]] <- combis_ii

        }

        if (regular_type=='scad'){ penalty_used <- penalty_D1_scad }
        if (regular_type=='mcp'){ penalty_used <- penalty_D1_mcp }
        if (regular_type=='lasso'){ penalty_used <- penalty_D1_abs }
        fuse_categories <- rep(FALSE,I)
        if (regular_lam[2]>0){
            for (ii in 1L:I){
                if (ncats[ii]>2){
                    fuse_categories[ii] <- TRUE
                }
            }
        }

        #- define penalty function
        penalty_fun_item <- function(x, ...)
        {
            pen <- 0
            pen <- regpolca_penalty_fun( x=x, regular_grouped=regular_grouped, I=I,
                        partable=partable, combis_classes_list=combis_classes_list,
                        regular_lam=regular_lam, eps=eps, penalty_used=penalty_used,
                        Ni=Ni, combis_categ_list=combis_categ_list,
                        fuse_categories=fuse_categories, K=K)
            return(pen)
        }
    }

    #-- create argument list for xxirt
    args <- list( dat=dat, Theta=Theta, partable=partable, customItems=customItems,
                customTheta=customTheta, maxit=random_iter, mstep_iter=mstep_iter,
                penalty_fun_item=penalty_fun_item, h=h, use_grad=TRUE, verbose=2 )

    #-- random starts if required
    args <- regpolca_run_xxirt_random_starts( args=args, random_starts=random_starts,
                    sd_noise_init=sd_noise_init, par_item_init=par_item_init )

    #-- arguments for final xxirt model
    args$verbose <- TRUE
    args$maxit <- maxit

    #-- run xxirt in a final model
    res <- do.call(what=xxirt, args=args)
    res$iter <- res$iter + random_iter*(random_starts>0)

    #- process output
    res$probs_Theta <- regpolca_postproc_prob_Theta(probs_Theta=res$probs_Theta)
    item <- regpolca_postproc_irf(probs_items=res$probs_items, dat=dat,
                    lca_dich=lca_dich)
    res0 <- regpolca_postproc_count_regularized_parameters(item=item,
                    set_equal=set_equal, lca_dich=lca_dich, probs_items=res$probs_items,
                    nclasses=nclasses, ncats=ncats)
    item1_index <- res0$item1_index
    n_reg <- res0$n_reg
    item <- res0$item
    res$probs_items <- res0$probs_items

    #- adapt information criteria
    res$ic <- regpolca_postproc_ic(ic=res$ic, n_reg=n_reg)
    res$ic$opt_val <- res$opt_val
    res$ic$pen_val <- res$pen_val

    #-- arrange output
    res$CALL <- CALL
    res2 <- list(s1=s1, s2=Sys.time(), lca_dich=lca_dich, nclasses=nclasses,
                    item=item, regular_lam=regular_lam, regular_type=regular_type,
                    item1_index=item1_index, n_reg=n_reg,
                    regular_grouped=regular_grouped)
    res <- sirt_add_list_elements(res=res, res2=res2)

    class(res) <- 'regpolca'
    return(res)
}

## File Name: mgsem_proc_model_partable_define_index.R
## File Version: 0.123


mgsem_proc_model_partable_define_index <- function(partable)
{
    dfr <- partable
    ind1 <- which(dfr$index==1)
    N1 <- length(ind1)
    M1 <- max(dfr$index)+10
    if (length(ind1)>0){
        dfr$index[ind1] <- M1+1L:N1
    }
    dfr$index <- match(dfr$index, unique(dfr$index))
    dfr$unique <- 1 * (! duplicated(dfr$index) )
    #** find duplicated parameters
    ND <- nrow(dfr)
    for (dd in 1L:ND){
        if (dfr[dd,'unique']==0){
            i1 <- which( dfr[,'index']==dfr[dd,'index'])
            i2 <- which( dfr[,'unique']==1)
            i3 <- intersect(i1,i2)
            dfr[dd,'name'] <- dfr[i3,'name']
        }
        #- recycled parameters
        group_dd <- dfr[dd,'group']
        if (group_dd>0){
            name_dd <- dfr[dd,'name']
            name0_dd <- gsub(paste0('_G',group_dd),'_G0', name_dd)
            ind <- match(name0_dd, dfr$name)
            if (length(ind)==1){
                dfr[dd,'recycle'] <- ind
            }
        }
    }
    dfr$recycle <- ifelse(is.na(dfr$recycle), 0, dfr$recycle )

    dfr <- data.frame(rid=1:ND, dfr)
    NP <- max(dfr$index)
    #--- output
    res <- list(partable=dfr, ND=ND, NP=NP)
    return(res)
}

## File Name: rm_sdt_prepare_diffindex.R
## File Version: 0.06

rm_sdt_prepare_diffindex <- function( item.index, rater.index, I, est.c.rater, est.d.rater )
{
    res <- NULL

    #---- est.c.rater
    if (est.c.rater=="r"){ diffindex <- rater.index }
    if (est.c.rater=="i"){ diffindex <- item.index }
    if (est.c.rater=="e"){ diffindex <- rep(1,I) }
    if (est.c.rater=="a"){ diffindex <- 1:I }
    if (est.c.rater=="n"){ diffindex <- NA }
    res$c.rater <- diffindex

    #---- est.d.rater
    if (est.d.rater=="r"){ diffindex <- rater.index }
    if (est.d.rater=="i"){ diffindex <- item.index }
    if (est.d.rater=="e"){ diffindex <- rep(1,I) }
    if (est.d.rater=="a"){ diffindex <- 1:I }
    if (est.d.rater=="n"){ diffindex <- NA }
    res$d.rater <- diffindex

    return(res)
}

## File Name: rasch_mirtlc_est_a.R
## File Version: 0.172


#-- estimation of a parameter (discrimination parameter)
rasch_mirtlc_est_a <- function( theta.k, b, fixed.a, pjk, alpha1, alpha2, h,
        G, I, r.jk, n.jk, est.a, Qmatrix, iter, fac.iter, dimensions=NULL )
{
    eps <- 0.000000005
    #** a
    pjk <- prob_raschtype_genlogis( theta.k, b, alpha1, alpha2, fixed.a, Qmatrix)
    pjk <- sirt_squeeze_probs(probs=pjk, eps=eps)
    pjk.M <- t(pjk)
    qjk.M <- 1 - pjk.M
    pjk1 <- prob_raschtype_genlogis( theta.k, b, alpha1, alpha2, fixed.a + h,Qmatrix)
    pjk1 <- sirt_squeeze_probs(probs=pjk1, eps=eps)
    pjk1.M <- t(pjk1)
    qjk1.M <- 1 - pjk1.M
    pjk2 <- prob_raschtype_genlogis( theta.k, b, alpha1, alpha2, fixed.a - h,Qmatrix)
    pjk2 <- sirt_squeeze_probs(probs=pjk2, eps=eps)
    pjk2.M <- t(pjk2)
    qjk2.M <- 1 - pjk2.M
    # first order derivative
    # f(x+h) - f(x-h)=2* f'(x) * h
    ll0 <- ll1 <- ll2 <- matrix(0, I, G)
    #-- groups
    for (gg in 1:G){
        ll0[,gg] <- rowSums( r.jk[,,gg] * log( pjk.M ) +
                                ( n.jk[,,gg] - r.jk[,,gg]  ) * log( qjk.M ) )
        ll1[,gg] <- rowSums( r.jk[,,gg] * log( pjk1.M ) +
                                ( n.jk[,,gg] - r.jk[,,gg]  ) * log( qjk1.M ) )
        ll2[,gg] <- rowSums( r.jk[,,gg] * log( pjk2.M ) +
                                ( n.jk[,,gg] - r.jk[,,gg]  ) * log( qjk2.M )     )
    }
    ll0 <- rowSums(ll0)
    ll1 <- rowSums(ll1)
    ll2 <- rowSums(ll2)
    # aggregate with respect to estimation of a
    a1 <- stats::aggregate( cbind( ll0, ll1, ll2 ), list(est.a), sum, na.rm=TRUE)
    a1 <- a1[ a1[,1] > 0, ]
    ll0 <- a1[,2]
    ll1 <- a1[,3]
    ll2 <- a1[,4]
    d1 <- ( ll1 - ll2  ) / ( 2 * h )    # negative sign?
    # second order derivative
    # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
    d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
    d2[ abs(d2) < 1e-10 ] <- 1e-10
    # change in item difficulty
    a.change <- - d1 / d2
    # dampening parameter as in tam
    old_increment <- .2^( iter^fac.iter )
    ci <- ceiling( abs(a.change) / ( abs( old_increment) + 10^(-10) ) )
    a.change <- ifelse( abs( a.change) > abs(old_increment),
                                        a.change/(2*ci), a.change )
    a.change <- a.change[ match( est.a, a1[,1] ) ]
    if ( any( est.a==0 ) ){
        a.change[ est.a==0 ] <- 0
    }
    fixed.a <- fixed.a + a.change
    fixed.a[ fixed.a < 0 ] <- 0
    return(fixed.a)
}


.mirtlc.est.a <- rasch_mirtlc_est_a

## File Name: rm_facets_est_b_rater.R
## File Version: 0.22


#########################################
# estimation of rater severity
rm_facets_est_b_rater <- function( b.item, b.rater, Qmatrix, tau.item,
        VV, K, I, TP, a.item, a.rater, item.index, rater.index,
        n.ik, numdiff.parm=.001, max.b.increment=1, theta.k, msteps,
        mstepconv, b.rater.center, b.rater.fixed  )
{
    h <- numdiff.parm
    diffindex <- rater.index
    RR <- length(b.rater)
    cat("  M steps b.rater parameter  |")
    it <- 0
    conv1 <- 1000
    #--- input for rm_facets_calcprobs
    args <- list( b.item=b.item, b.rater=b.rater, Qmatrix=Qmatrix, tau.item=tau.item, VV=VV, K=K, I=I, TP=TP,
                    a.item=a.item, a.rater=a.rater, item.index=item.index, rater.index=rater.index,
                    theta.k=theta.k, RR=RR )
    #---- begin algorithm
    while( ( it < msteps ) & ( conv1 > mstepconv ) ){
        b0 <- b.rater
        args$b.rater <- b.rater
        pjk <- do.call( what=rm_facets_calcprobs, args=args)
        args$b.rater <- b.rater + h
        pjk1 <- do.call( what=rm_facets_calcprobs, args=args)
        args$b.rater <- b.rater - h
        pjk2 <- do.call( what=rm_facets_calcprobs, args=args)
        #-- increment
        res <- rm_numdiff_index( pjk=pjk, pjk1=pjk1, pjk2=pjk2, n.ik=n.ik, diffindex=diffindex,
                    max.increment=max.b.increment, numdiff.parm=numdiff.parm )
        increment <- res$increment
        b.rater <- b.rater + increment
        if ( ! is.null( b.rater.fixed) ){
            ind <- which( ! is.na( b.rater.fixed  ) )
            b.rater[ind] <- b.rater.fixed[ind]
            res$d2[ ind ] <- -1E10
        }
        brc <- mean( b.rater )

        #-- centering
        b.rater <- rm_center_vector( vec=b.rater, center_type=b.rater.center)
        conv1 <- max( abs( b.rater - b0 ) )
        it <- it+1
        cat("-")
    }
    cat(" ", it, "Step(s) \n")
    res <- list(b.rater=b.rater, se.b.rater=sqrt( abs(-1/res$d2 ) ),
                            ll=sum(res$ll0), brc=brc     )
    return(res)
}


.rm.facets.est.b.rater <- rm_facets_est_b_rater

## File Name: write.format2.R
## File Version: 1.133



#*** utility function for formatting output in write.fwf2
write.format2 <- function( vec1, ff, fr )
{
    if (fr==0){
        vec2 <- round( vec1, fr )
        blank.vv <- paste( rep( ' ', ff ), collapse='' )
        vec2 <- paste( substring( blank.vv, 1, ff - nchar(vec2) ), vec2, sep='')
    } else {
        d.vv <- round( vec1, fr ) + 10^(-(fr+1))
        # generate blank
        blank.vv <- paste( rep( ' ', ff+1 ), collapse='' )
        d.vv <- paste( substring( blank.vv, 1, ff+1 - nchar(d.vv) ), d.vv, sep='')
        g.vv <- grep('NA',d.vv)
        d.vv[ g.vv  ] <- ifelse( ff > 1,  gsub( 'NA', ' .', d.vv[g.vv] ),
                                        gsub( 'NA', '.', d.vv[g.vv] ) )
        vec2 <- substring( d.vv, 1, ff )
        vec2
    }
    return(vec2)
}

.write.format2 <- write.format2

## File Name: mgsem_output_proc_casewise_likelihood.R
## File Version: 0.08


mgsem_output_proc_casewise_likelihood <- function(data_proc, implied, estimator="ML")
{
    case_ll <- NULL
    if (estimator=='ML' & ( ! is.null(data_proc) ) ){
        requireNamespace('mvtnorm')
        N <- data_proc$N
        G <- data_proc$G
        data <- data_proc$data
        idgroup <- data_proc$idgroup
        case_ll <- rep(NA,N)
        for (gg in 1L:G){
            implied_gg <- implied[[gg]]
            ind_gg <- which(idgroup==gg)
            Mu <- as.vector(implied_gg$Mu[,1])
            Sigma <- implied_gg$Sigma
            y <- mvtnorm::dmvnorm(x=data[ind_gg,], mean=Mu, sigma=Sigma, log=TRUE)
            case_ll[ind_gg] <- y
        }
    }
    #-- output
    return(case_ll)
}

## File Name: dmlavaan_est_model_include_partable.R
## File Version: 0.06

dmlavaan_est_model_include_partable <- function(partable, se_sw)
{
    ind <- which(partable$free>0)
    partable1 <- partable[ind,]
    partable$se_sw <- 0
    partable1$se_sw <- se_sw[ partable1$pnid ]
    partable[ind,] <- partable1
    #-- output
    return(partable)
}

## File Name: rm_numdiff_index.R
## File Version: 0.252


####################################################################
# general function for numerical differentiation
# diffindex aggregates across super items
rm_numdiff_index <- function( pjk, pjk1, pjk2, n.ik, diffindex,
        max.increment, numdiff.parm, eps=1E-80, eps2=1E-10, prior=NULL, value=NULL )
{
    h <- numdiff.parm
    an.ik <- aperm( n.ik, c(2,3,1) )
    # str("pjk")
    # [items, categories, nodes]

    #--- evaluate expected likelihood
    ll0 <- rm_grouped_expected_likelihood(pjk=pjk, n.ik=an.ik,
                        diffindex=diffindex, eps=eps)
    ll1 <- rm_grouped_expected_likelihood(pjk=pjk1, n.ik=an.ik,
                        diffindex=diffindex, eps=eps)
    ll2 <- rm_grouped_expected_likelihood(pjk=pjk2, n.ik=an.ik,
                        diffindex=diffindex, eps=eps)

    if (! is.null(prior)){
        if (length(value) > length(ll0)){
            value <- stats::aggregate( matrix(value,ncol=1), list(diffindex), mean )[,2]
        }
        M <- prior[1]
        SD <- sqrt(prior[2])
        ll0 <- ll0 + sirt_dnorm( value, mean=M, sd=SD, log=TRUE)
        ll1 <- ll1 + sirt_dnorm( value+h, mean=M, sd=SD, log=TRUE)
        ll2 <- ll2 + sirt_dnorm( value-h, mean=M, sd=SD, log=TRUE)
    }

    #-- discrete differences
    res <- rm_numdiff_discrete_differences(ll0=ll0, ll1=ll1, ll2=ll2, h=h)
    d1 <- res$d1
    d2 <- res$d2

    #-- compute incremental change in item parameters
    d2[ abs(d2) < eps2 ] <- eps2
    increment <- - d1 / d2


    #-- trim increment
    increment <- rm_numdiff_trim_increment( increment=increment,
                        max.increment=max.increment, eps2=eps2 )
    #--- output
    res <- list(increment=increment, d1=d1, d2=d2, ll0=ll0, h=h)
    return(res)
}


.rm.numdiff.index <- rm_numdiff_index

## File Name: xxirt_compute_priorDistribution.R
## File Version: 0.148


xxirt_compute_priorDistribution <- function( Theta, customTheta, G )
{
    P_Theta <- customTheta$P
    arg_Theta <- list( Theta=Theta, par=customTheta$par, G=G )
    if (customTheta$person_covariates){
        arg_Theta$X <- customTheta$X
    }
    prior_Theta <- do.call( what=P_Theta, args=arg_Theta )
    return(prior_Theta)
}

## File Name: summary.btm.R
## File Version: 0.268


#--- summary.btm
summary.btm <- function( object, file=NULL, digits=4,... )
{
    # open sink
    sirt_osink( file=file )

    res <- object

    cat("------------------------------------------------------------\n")
    #- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object)
    cat("Computation Time Algorithm", print(object$time_alg), "\n\n")

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    cat("Bradley-Terry Model with Ties and Home Advantage Parameters\n")

    cat("------------------------------------------------------------\n")
    cat( "Log-likelihood value", "=", round(object$ll,2), "\n" )
    cat( "Number of iterations", "=", object$iter, "\n" )
    #    cat( "Deviance=", round( object$deviance, 2 ), " | " )
    #    cat( "Log Likelihood=", round( -object$deviance/2, 2 ), "\n" )
    cat( "Number of individuals", "=", object$ic$n, "\n" )
    cat( "Number of pairwise comparisons", "=", object$ic$D, "\n" )
    cat( "Epsilon value", "=", object$eps, "\n" )
    cat( "ignore.ties", "=", object$ignore.ties, "\n" )
    cat( "wgt.ties", "=", round(object$wgt.ties,3), "\n" )
    #    cat( "Number of estimated parameters=", object$ic$np, "\n" )

    cat("------------------------------------------------------------\n")
    cat("Ties and Home advantage parameters\n")
    obji <- res$pars
    sirt_summary_print_objects(obji=obji, digits=digits, from=3)

    cat("------------------------------------------------------------\n")
    cat("Summary of individual effects parameters\n")
    obji <- res$summary.effects
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=TRUE)

    cat("------------------------------------------------------------\n")
    cat("MLE reliability (separation reliability)\n")
    cat(paste0("MLE Rel", "=", round( res$mle.rel, digits ), "\n") )
    cat(paste0("Separation index", "=", round( res$sepG, digits ), "\n") )

    cat("------------------------------------------------------------\n")
    cat("Individual effects parameters\n")
    obji <- res$effects
    sirt_summary_print_objects(obji=obji, digits=digits, from=3)

    if (res$multiple_judges){
        cat("------------------------------------------------------------\n")
        cat("Fit statistics judges\n")
        obji <- res$fit_judges
        sirt_summary_print_objects(obji=obji, digits=digits, from=2)
    }

    # close sink
    sirt_csink( file=file )
}

## File Name: rasch_mirtlc_mstep_lc.R
## File Version: 0.04


#---- calculate class probabilities
rasch_mirtlc_mstep_lc <- function( pjk, n.k, r.jk, n.jk, G, Nclasses )
{
    if (G==1){
        pi.k <- n.k / sum( n.k )
        pi.k <- matrix( pi.k, nrow=ncol(pjk), ncol=nrow(pjk) )
    }
    if ( G> 1){
        pi.k <- n.k / matrix( colSums(n.k ), nrow=Nclasses, ncol=G, byrow=TRUE)
    }
    for (cc in 1:Nclasses ){
        if (G==1){
            pjk[ cc, ] <- r.jk[, cc, 1] / n.jk[, cc, 1]
        }
        if (G>1){
            pjk[ cc, ] <- rowSums( r.jk[, cc, ] ) / rowSums( n.jk[, cc, ]  )
        }
    }
    res <- list( pi.k=pi.k, pjk=pjk )
    return(res)
}


.m.step.mirtlc.lc <- rasch_mirtlc_mstep_lc

## File Name: expl.detect.R
## File Version: 1.317


#**** Exploratory DETECT analysis
expl.detect <- function( data, score, nclusters, N.est=NULL, seed=NULL,
        bwscale=1.1, smooth=TRUE, use_sum_score=FALSE, hclust_method="ward.D",
        estsample=NULL)
{
    if ( ! is.null(seed) ){
        set.seed(seed)
    }
    # number of items
    I <- ncol(data)
    if (use_sum_score){
        smooth <- FALSE
        score <- rowSums(data)
    }
    scale_score <- TRUE
    if (!smooth){
        scale_score <- FALSE
    }
    # sample for estimation
    N <- nrow(data)
    if ( is.null( N.est ) ){
        N.est <- floor(N/2)
    }
    if (is.null(estsample)){
        estsample <- sort( sample( 1L:N, floor( N.est ) ) )
    }
    # validation sample
    valsample <- setdiff( 1L:N, estsample )

    #--- Maximizing DETECT index
    # nonparametric estimation of conditional covariance
    ccov_np_args <- list( data=data[ estsample,], score=score[estsample],
                        bwscale=bwscale, smooth=smooth, use_sum_score=use_sum_score,
                        scale_score=scale_score)
    cc_est <- cc <- do.call( what=ccov.np, args=ccov_np_args)
    ccov.matrix <- create.ccov( cc=cc, data=data[ estsample,]  )
    # create distance matrix
    cc1 <- max(ccov.matrix) - ccov.matrix
    # Ward Hierarchical Clustering
    d <- stats::as.dist(cc1)
    fit <- stats::hclust(d, method=hclust_method)         # hierarchical cluster analysis
    clusterfit <- fit
    itemcluster <- data.frame( matrix( 0, nrow=I, ncol=nclusters ) )
    itemcluster[,1] <- colnames(data)
    colnames(itemcluster) <- c( 'item', paste( 'cluster', 2L:nclusters, sep='') )
    detect.unweighted <- detect.weighted <- NULL
    for (k in 2L:nclusters){
        itemcluster[,k] <- stats::cutree( fit, k=k )
        h1 <- detect.index( ccovtable=cc, itemcluster=itemcluster[,k] )
        detect.unweighted <- rbind( detect.unweighted, h1$unweighted )
        detect.weighted <- rbind( detect.weighted, h1$weighted )
    }
    parnames <- c( 'DETECT', 'ASSI', 'RATIO', 'MADCOV100', 'MCOV100')
    colnames(detect.unweighted) <- paste( parnames, '.est', sep='')
    colnames(detect.weighted) <- paste( parnames, '.est', sep='')
    dfr1 <- data.frame( N.Cluster=2L:nclusters )
    dfr1$N.items <- I
    dfr1$N.est <- N.est
    dfr1$N.val <- length(valsample)
    dfr1$size.cluster <- sapply( 2L:nclusters, FUN=function(tt){
                            paste( table( itemcluster[,tt] ), collapse='-' )
                        } )
    detu <- data.frame( dfr1, detect.unweighted )
    detw <- data.frame( dfr1, detect.weighted )

    #--- Validating DETECT index
    if ( length(valsample) > 0 ){
        ccov_np_args$data=data[ valsample,]
        ccov_np_args$score=score[valsample]
        cc <- do.call( what=ccov.np, args=ccov_np_args)
        detect.unweighted <- detect.weighted <- NULL
        for (k in 2L:nclusters){
            h1 <- detect.index( ccovtable=cc, itemcluster=itemcluster[,k] )
            detect.unweighted <- rbind( detect.unweighted, h1$unweighted )
            detect.weighted <- rbind( detect.weighted, h1$weighted )
        }
        colnames(detect.unweighted) <- paste( parnames, '.val', sep='')
        colnames(detect.weighted) <- paste( parnames, '.val', sep='')
        detu <- data.frame( detu, detect.unweighted )
        detw <- data.frame( detw, detect.weighted )
    }
    rownames(detect.unweighted) <- paste0('Cl', 2L:nclusters)
    rownames(detect.weighted) <- rownames(detect.unweighted)
    cat('\n\nDETECT (unweighted)\n\n')
    clopt <- which.max( detu$DETECT.est ) + 1
    cat('Optimal Cluster Size is ', clopt, ' (Maximum of DETECT Index)\n\n' )
    detu1 <- detu
    for (vv in 6L:ncol(detu)){
        detu1[,vv] <- round( detu1[,vv], 3)
    }
    print(detu1)
    res <- list( detect.unweighted=detect.unweighted, detect.weighted=detect.weighted,
                    clusterfit=clusterfit, itemcluster=itemcluster )
    # plot cluster solution
    graphics::plot( res$clusterfit, main=paste( 'Cluster Dendogram with ',
                            clopt, ' Clusters', sep='') )
    stats::rect.hclust(res$clusterfit, k=clopt, border='red')
    class(res) <- 'expl.detect'
    return(res)
}

## File Name: rasch_jml_centerpersons.R
## File Version: 0.04

rasch_jml_centerpersons <- function(theta, dat1, centerpersons)
{
    if (centerpersons){
        theta <- theta - stats::weighted.mean( theta, dat1[,2] )
    }
    return(theta)
}

## File Name: mgsem_loss_function_suffstat_derivative_parameter.R
## File Version: 0.115

mgsem_loss_function_suffstat_derivative_parameter <- function(est, dermoments, suffstat,
        type, i1, i2, h, is_B=FALSE, num_approx=FALSE, eps=1e-12, p=2, vech_rcpp=TRUE,
        output_all=FALSE)
{

    #** derivative of moments with respect to parameter
    res <- mgsem_moments_derivative_parameter( est=est, type=type, i1=i1, i2=i2,
                h=h, is_B=is_B, num_approx=num_approx, eps=eps )
    Mu_der <- res$Mu_der
    Sigma_der <- res$Sigma_der
    Sigma_der_logical <- res$Sigma_der_logical
    t1 <- 0
    t2 <- 0

    #*** means
    if (res$calc_Mu){
        t1 <- sum(Mu_der*dermoments$dermean)
    }
    #*** covariance
    if (res$calc_Sigma){
        if (!vech_rcpp){
            vech_sigma_der <- mgsem_vech(x=Sigma_der)
            vech_sigma_der_logical <- mgsem_vech(x=Sigma_der_logical)
        } else {
            vech_sigma_der <- sirt_rcpp_mgsem_vech_numeric(A=Sigma_der)
            vech_sigma_der_logical <- sirt_rcpp_mgsem_vech_logical(A=Sigma_der_logical)
        }
        # t2 <- sum(vech_sigma_der*dermoments$dercov)
        t2 <- sirt_rcpp_mgsem_sumproduct_logical(y=vech_sigma_der,
                    x=dermoments$dercov, y_logical=vech_sigma_der_logical)
    }

    #*** output
    res <- t1+t2
    return(res)
}

## File Name: rm_center_vector.R
## File Version: 0.09

rm_center_vector <- function( vec, center_type, do_log=FALSE )
{
    # log metric
    if (do_log){
        vec <- log(vec)
    }
    #--- center_type=1
    if ( center_type==1){
        RR <- length(vec)
        vec[RR] <- - sum( vec[-RR] )
    }
    #--- center_type=2
    if ( center_type==2){
        vec <- vec - mean(vec)
    }
    # reconvert to exp metric
    if (do_log){
        vec <- exp(vec)
    }
    #--- output
    return(vec)
}

## File Name: mle.rasch.R
## File Version: 1.03



#---------------------------------------------------------
# Maximum Likelihood Estimation (Rasch model)

mle.rasch <- function( dat, dat.resp=1-is.na(dat), b, theta, conv=.001,
            progress=FALSE, prior_sd=NULL)
{
    theta.change <- 1
    if ( progress){ cat("\n  MLE estimation  |" ) }
    I <- length(b)
    bM <- matrix( b, nrow=length(theta), length(b), byrow=TRUE )

    # prior for ability
    if (! is.null(prior_sd) ){
        is_prior <- TRUE
    } else {
        is_prior <- FALSE
    }

    while( max( abs( theta.change) > conv )){
        # calculate P and Q
        p.ia <- stats::plogis( theta - bM )
        q.ia <- 1 - p.ia
        # Likelihood
        l1 <- rowSums( dat.resp* ( dat - p.ia ) )
        # derivative of the objective function
        f1.obj <- rowSums( - dat.resp * p.ia * q.ia  )
        # add prior
        if (is_prior){
            l1 <- l1 - theta / prior_sd^2
            f1.obj <- f1.obj - 1 / prior_sd^2
        }

        # theta change
        theta.change <- - l1 / f1.obj
        theta <- theta + theta.change
        if ( progress){  cat("-") }
    }
    res <- list( "theta"=theta, "p.ia"=p.ia )
    return(res)
}

## File Name: sim.rasch.dep.R
## File Version: 0.232


#**** simulation of Rasch copula model
sim.rasch.dep <- function( theta, b, itemcluster, rho )
{
    probmat <- stats::plogis( outer( theta, b, '-' ) )
    I <- length(b)
    n <- length(theta)
    # covariance matrix of dependencies
    cov.dep <- diag(1, I)
    clusters <- unique(itemcluster[ itemcluster > 0 ] )
    CC <- length(clusters)
    for (cc in 1L:CC){
        v1 <- which( itemcluster==cc )
        for (ii in v1){
            for (jj in v1) {
                if ( ii!=jj ){
                    cov.dep[ii,jj] <- rho[cc]
                }
            }
        }
    }
    random.gen <- stats::pnorm( sirt_rmvnorm( n, mean=rep(0,I), sigma=cov.dep ) )
    dat <- 1 * ( probmat > random.gen )
    colnames(dat) <- paste0( 'I', substring(100+1:I,2))
    return(dat)
}

## File Name: mcmc_rename_helper.R
## File Version: 0.051

mcmc_rename_helper <- function( string, rep_string=3, pre=3, suff=3)
{
    string <- paste0(rep( string, rep_string ), collapse='')
    pre <- paste0(rep( '_', pre ), collapse='')
    suff <- paste0(rep( '_', suff ), collapse='')
    trans <- paste0( pre, string, suff )
    return(trans)
}

## File Name: linking_haberman_itempars_convert.R
## File Version: 0.092


linking_haberman_itempars_convert <- function(itempars=NULL, lambda=NULL,
    nu=NULL, a=NULL, b=NULL)
{
    if (!is.null(nu)){
        b <- -nu/lambda
        a <- lambda
    }
    if (!is.null(b)){
        if (is.null(a)){
            a <- 1+0*b
        }
        itempars <- linking_haberman_itempars_prepare(b=t(b), a=t(a))
    }

    if (! is.null(itempars)){
        itempars_list <- linking_proc_itempars(itempars=itempars)
    }

    #- reconvert into a and b
    a <- linking_haberman_itempars_convert_process_matrices(
                mat=itempars_list$aM, est_pars=itempars_list$est_pars)
    b <- linking_haberman_itempars_convert_process_matrices(
                mat=itempars_list$bM, est_pars=itempars_list$est_pars)

    #- reconvert into lambda and mu
    lambda <- t(a)
    nu <- -t(b/a)

    #-- output
    res <- list(itempars=itempars, lambda=lambda, nu=nu, a=a, b=b,
                    itempars_list=itempars_list)
    return(res)
}

## File Name: rm_facets_pp_mle_calc_ll_theta.R
## File Version: 0.11



rm_facets_pp_mle_calc_ll_theta <- function( data, a, b, theta )
{
    N <- length(theta)
    I <- ncol(data)
    ll0 <- rep(0,N)
    for (ii in 1L:I){
        probs.ii <- rm_facets_pp_mle_calc_pcm( theta=theta, a=a, b=b, ii=ii )
        res <- rm_facets_pp_mle_calc_ll( probs=probs.ii, data=data, ii=ii )
        ll0 <- ll0 + res
    }
    return(ll0)
}

## File Name: fuzcluster.R
## File Version: 0.163


#*** Clustering for continuous fuzzy data
fuzcluster <- function(dat_m, dat_s, K=2, nstarts=7,
            seed=NULL, maxiter=100, parmconv=.001, fac.oldxsi=0.75, progress=TRUE )
{
    s1 <- Sys.time()
    dev0 <- 1e200
    dat_resp <- 1 - is.na(dat_m)
    if ( ! is.null(seed) ){
        nstarts <- 1
    }
    for ( rr in 1:nstarts ){
        res1 <- fuzcluster_estimate(K, dat_m, dat_s, dat_resp,
                    maxiter=maxiter, parmconv=parmconv, progress=progress,
                    seed=seed, fac.oldxsi=fac.oldxsi)
        if ( res1$deviance < dev0 ){
            res <- res1
            dev0 <- res1$deviance
        }
    }
    s2 <- Sys.time()
    ### end random starts
    ### computation of information criteria
    dev <- res$deviance
    ic <- list( deviance=dev, n=nrow(dat_m) )
    I <- ncol(dat_m)
    ic$np <- (K-1) + 2*K*I
    ic$AIC <- dev + 2*ic$np
    ic$BIC <- dev + ( log(ic$n) )*ic$np
    ic$CAIC <- dev + ( log(ic$n) + 1 )*ic$np
    ic$AICc <- ic$AIC + 2*ic$np * ( ic$np + 1 ) / ( ic$n - ic$np - 1 )
    res$ic <- ic
    res$K <- K
    res$s1 <- s1
    res$s2 <- s2
    res$nstarts <- nstarts
    class(res) <- "fuzcluster"
    return(res)
}

## File Name: mgsem_proc_model_single_group.R
## File Version: 0.081

mgsem_proc_model_single_group <- function(model)
{
    H <- length(model)
    if (H==1){
        entries <- c('est','index')
        group1 <- list()
        group0 <- model[[1]]
        for (ee in entries){
            v1 <- list()
            for (vv in names(group0[[ee]]) ){
                v1[[vv]] <- 0*group0[[ee]][[vv]]
            } # end vv
            group1[[ee]] <- v1
        } # end ee
        res <- list( group0=group0, group1=group1)
    } else {
        res <- model
    }
    #-- output
    return(res)
}

## File Name: regpolca_postproc_ic.R
## File Version: 0.05


regpolca_postproc_ic <- function(ic, n_reg)
{
    ic$n_reg <- n_reg
    ic$np.items <- ic$np.items - ic$n_reg
    ic <- xxirt_ic_compute_criteria(ic=ic)
    return(ic)
}

## File Name: lsem_lavaan_modify_lavaan_object_test.R
## File Version: 0.03

lsem_lavaan_modify_lavaan_object_test <- function(object)
{
    # object@test
    test <- object@test
    test[[1]]$test <- 'standard'
    object@test <- test

    #-- output
    return(object)
}

## File Name: rm_sdt_fill_par_to_partable.R
## File Version: 0.18


rm_sdt_fill_par_to_partable <- function( par_index, partable, parm0, type)
{

    not_fixed <- ! partable$fixed
    partable_parindex <- partable$parindex[ not_fixed ]
    partable[ not_fixed, "value"] <- parm0[ partable_parindex ]
    #--- fill objects
    parm_list <- NULL
    if (type=="item"){
        pars <- c("tau.item", "a.item")
    } else {
        pars <- c("c.rater", "d.rater")
    }
    for (pp in pars){
        index_pp <- par_index[[pp]]
        n_pp <- index_pp[ length(index_pp) ]
        n_row <- partable[n_pp, "row"]
        n_col <- partable[n_pp, "col"]
        if ( ! is.na(n_col) ){
            x <- matrix( partable[ index_pp, "value"], nrow=n_row, ncol=n_col )
        } else {
            x <- partable[ index_pp, "value"]
        }
        parm_list[[ pp ]] <- x
    }
    #--- output
    res <- list(partable=partable, parm_list=parm_list)
    return(res)
}

## File Name: bounds_parameters.R
## File Version: 0.08


bounds_parameters <- function( pars, lower=NULL, upper=NULL)
{
    if ( ! is.null(lower)){
        pars <- ifelse( pars < lower, lower, pars )
    }
    if ( ! is.null(upper)){
        pars <- ifelse( pars > upper, upper, pars )
    }
    return(pars)
}

## File Name: sirt_rbind_fill.R
## File Version: 0.06

## reimplementation of plyr::rbind.fill

sirt_rbind_fill <- function( x, y )
{
    nx <- nrow(x)
    ny <- nrow(y)
    vars <- c( colnames(x), setdiff( colnames(y), colnames(x) ) )
    z <- matrix( NA, nrow=nx+ny, ncol=length(vars) )
    colnames(z) <- vars
    z <- as.data.frame(z)
    z[ 1L:nx, colnames(x) ] <- x
    z[ nx + 1L:ny, colnames(y) ] <- y
    return(z)
}

## File Name: lsdm_extract_probquantile.R
## File Version: 0.14


# auxiliary function for extracting quantiles of curves
lsdm_extract_probquantile <- function( vec, theta, quant )
{
    x2 <- theta[ vec>=quant ][1]
    x1 <- sort( theta[ vec<quant ], decreasing=TRUE)[1]
    value <- - Inf
    if ( (1-is.na(x2)) * (1-is.na(x1))==1 ){
        y1 <- vec[theta==x1]
        y2 <- vec[theta==x2]
        value <- x1 + ( quant - y1 ) * ( x2 - x1 ) / ( y2 - y1 )
    }
    if ( is.na(x2) ){
        value <- Inf
    }
    return(value)
}

## File Name: invariance_alignment_process_parameters.R
## File Version: 0.03

invariance_alignment_process_parameters <- function(par.aligned, par)
{
    res <- sirt_matrix_names(x=par.aligned, extract_names=par)
    res[is.na(par)] <- NA
    return(res)
}

## File Name: sirt_dnorm_discrete.R
## File Version: 0.04

sirt_dnorm_discrete <- function(x, mean=0, sd=1, ...)
{
    fx <- stats::dnorm(x, mean=mean, sd=sd, ...)
    fx <- fx / sum(fx)
    return(fx)
}

## File Name: sirt_parlapply.R
## File Version: 0.02

sirt_parlapply <- function(cl, X, FUN, verbose=FALSE, ...)
{
    args <- list(...)
    args$cl <- cl
    args$X <- X
    if (verbose){
        what <- pbapply::pblapply
        args$FUN <- FUN
    } else {
        what <- parallel::parLapply
        args$fun <- FUN
    }
    res_all <- do.call(what=what, args=args)
    return(res_all)
}

## File Name: lsem_fitsem_joint_estimation.R
## File Version: 0.184

lsem_fitsem_joint_estimation <- function(partable_joint,
    is_meanstructure, sample_stats, lavaan_est_fun, se,
    sufficient_statistics=TRUE, data_joint=NULL,
    group="group__", verbose=FALSE, G=NULL, pseudo_weights=NULL, ...)
{
    wmean <- sample_stats$wmean
    wcov <- sample_stats$wcov
    Nobs <- sample_stats$Nobs

    if (is_meanstructure){
        sample_mean <- wmean
    } else {
        sample_mean <- NULL
    }
    if (sufficient_statistics){
        survey.fit <- lavaan_est_fun(partable_joint, sample.cov=wcov,
                            sample.mean=sample_mean,
                            sample.nobs=Nobs, verbose=verbose, se=se, ... )
    } else {
        # pseudo weights
        res <- lsem_fitsem_raw_data_define_pseudo_weights(dat=data_joint,
                            pseudo_weights=pseudo_weights)
        dat1 <- res$dat
        ## this function seems to be buggy, test statistics cannot be computed
        survey.fit <- lavaan_est_fun(partable_joint, data=dat1, verbose=verbose,
                            group=group, se=se, test='standard',... )
    }
    if (verbose){
        cat('\nUse lsem.bootstrap() for computing standard errors!\n\n')
    }
    return(survey.fit)
}

## File Name: np.dich.R
## File Version: 0.30


#----------------------------------------------------------------------------------------------------
np.dich <- function( dat, theta, thetagrid, progress=FALSE,
            bwscale=1.1, method="normal"){
    #.....................................................................
    # INPUT:
    # dat       ... data frame
    # theta     ... estimated theta values
    # thetagrid ... grid on which ability is nonparametrically evaluated        #
    # method    ... binomial and normal (nonparametric binomial or normal nonparametric regression)
    # bwscale   ... bandwidth scale h=bwscale * ^N^{-1/5)                     #
    #......................................................................
    # recode values of infinity to missing
    TAM::require_namespace_msg("sm")
    theta[ theta %in% c( -Inf, Inf ) ] <- NA
    I <- ncol(dat)
    # display
    cat("Nonparametric ICC Estimation for Dichotomous Data  \n" )
    cat(I, "Items \n" )

    # matrix of lower and upper confidence limits and estimates
    lower <- upper <- estimate <- matrix( 0, nrow=I, ncol=length(thetagrid) )
    missing.resp <- dat
    if (progress){ cat( "Item")}
    for (ii in 1:I){
        dfr <- data.frame( dat[,ii ], theta )
        missing.resp[, ii] <- 1* ( rowSums( is.na(dfr) ) > 0 )
        dfr <- stats::na.omit(dfr)
        y <- dfr[,1]
        x <- dfr[,2]
        if (method=="normal" ){
            h1 <- stats::ksmooth( x, y, bandwidth=bwscale * sd( x ) * length(x)^(-1/5),
                        x.points=thetagrid, kernel="normal")
            estimate[ii,] <- h1$y
                    }
        if (method=="binomial"){
                hsel <- sm::h.select(x=x, y=y )
                h1 <- sm::sm.binomial( x, y, N=rep(1, length(x) ),  h=hsel,
                                eval.points=thetagrid, display="none" )
                estimate[ ii,] <- h1$estimate
                lower[ii,] <- h1$lower
                upper[ii,] <- h1$upper
                }
    if (progress ){ if ( ii %% 20==0) { cat("", ii, "\n" ) } else { cat( "", ii ) }
        utils::flush.console() }
        }
    if (method=="normal"){ lower <- upper <- NULL }
    if (progress){ cat( "\n")}
    res <- list( "dat"=dat, "thetagrid"=thetagrid, "theta"=theta,
            "estimate"=estimate, "lower"=lower, "upper"=upper,
            "missing.resp"=missing.resp )
    class(res) <- "np.dich"
    return(res)
    }
#----------------------------------------------------------------------------------------------------



#--------------------------------------------------------------------------------------------------
# plotting function for shades between 2 curves
.shade.area <- function( xval, yval1, yval2, shadecol="gray",eps=0){
    # INPUT:
    # xval .. x values
    # yval1/yval2 ... vectors of y curves
    # shadecol ... color for shading the area
    yvaldiff <- 1*( yval1 - yval2 )
    xx <- c( xval, rev(xval) )
    yy <- c( yval1 - eps*yvaldiff, rev( yval2 ) - eps * yvaldiff )
    graphics::polygon(xx, yy, col=shadecol, border=shadecol )
    }
#--------------------------------------------------------------------------------------------------

##NS # S3method(plot, rasch.np)
#-------------------------------------------------------------------------------------------#
# plotting Rasch ICCs and their corresponding nonparametric estimates                       #
plot.np.dich <- function( x, b, infit=NULL, outfit=NULL, nsize=100,
        askplot=TRUE, progress=TRUE,
        bands=FALSE, plot.b=FALSE, shade=FALSE, shadecol="burlywood1", ...){
    # INPUT:                                                                    #
    # np.dich.obj... object generated by np.dich                                #
    # theta     ... ability estimates obtained from Rasch analysis              #
    # b         ... Rasch item difficulties                                     #
    # thetalim  ... limits of theta values                                      #
    # nsize     ... number of grid points                                       #
    # askplot   ... should before every plot asked to go to the next one?       #
    # progress  ... display iteration process of nonparametric estimation       #
    # bands     ... should confidence bands be drawn?                           #
    # plot.b    ... plot item difficulties                                      #
    P <- NULL
    np.dich.obj <- object <- x
    # extract informations from object np.dich
    thetagrid <- np.dich.obj$thetagrid
    theta <- np.dich.obj$theta
    nsize <- length(thetagrid)
    dat <- np.dich.obj$dat
    estimate <- np.dich.obj$estimate
    lower <- np.dich.obj$lower
    upper <- np.dich.obj$upper
    missing.resp <- np.dich.obj$missing.resp
    I <- ncol(dat)
    n <- nrow(dat)
    p <- .prob.rasch( thetagrid, b )
    for (ii in 1:I){
        plotname <- paste( colnames(dat)[ii], ": N=",
                sum((1-is.na(dat))[,ii]),
                ";  b=", formatC( b[ii], 2, format="f"), sep="")
        if ( !is.null(infit) ){ plotname <- paste( plotname, "\n Infit=", formatC( infit[ii],2, format="f") ) }
        if ( !is.null(infit) ){ plotname <- paste( plotname, " Outfit=", formatC( outfit[ii],2, format="f") ) }
        graphics::plot( thetagrid, estimate[ii,], type="n", ylim=c( - .02, 1.02 ),
                    xlab=expression( theta ), ylab=expression( P( theta )),
                    main=plotname )
        graphics::abline( h=0, lty=3 )
        graphics::abline( h=1, lty=3 )
        # graphics::rug( theta[ missing.resp[,ii]==0 ], col="cyan" )
        # shade area
        if (shade ){
            .shade.area( xval=thetagrid, yval1=p[,ii], yval2=estimate[ii,],
                            eps=0, shadecol=shadecol )
                      }
        if (bands & !is.null(upper) ){
            graphics::segments( thetagrid, lower[ii,], thetagrid,
                    upper[ii,], col="gray")
                                }
        # select display
        theta.ii <- theta[  missing.resp[,ii]==0 ]
        ind <- which( thetagrid > min(theta.ii) & thetagrid < max(theta.ii ) )
        graphics::lines( thetagrid[ind], estimate[  ii, ind ], lwd=2, col=1,  lty=5 )
        graphics::lines( thetagrid, p[, ii ], lwd=2, col=2 )
        if (plot.b ){
            graphics::lines( c( b[ii], b[ii] ), c( 0, .5), col=3, lty=4, lwd=2)
            graphics::lines( c( -90, b[ii] ), c( .5, .5), col=3, lty=4,  lwd=2 )
            th80 <- stats::qlogis(.8) + b[ii]
            graphics::lines( c( th80, th80 ), c( 0, .8 ), col=3, lty=3,  lwd=2 )
            graphics::lines( c(  th80, 90 ), c( .8, .8 ), col=3, lty=3,  lwd=2 )
            graphics::lines( c( b[ii], th80), c(0,0), col=3, lwd=2 )
                         }
        graphics::par(ask=askplot)
        }
    graphics::par(! askplot )
    }
#-------------------------------------------------------------------------------------------#

## File Name: sirt_print_helper.R
## File Version: 2.145


# This is an auxiliary function which helps for printing some progress
sirt_print_helper <- function( object, digits)
{
    ow <- options()$warn
    if ( length(dim(object))==2 ){
        options( warn=-1 )
        if ( nrow(object) >=1 ){
            g1a <- apply( object, 2, as.numeric )
        } else {
            g1a <- object
        }
        g1a <- matrix(g1a, nrow=nrow(object), ncol=ncol(object))
        colnames(g1a) <- colnames(object)
        g1 <- colMeans( g1a )
        g1 <- which( ! is.na( g1 ) )
        options( warn=ow )
        object1 <- object
        object1[, g1 ] <- round( object1[, g1 ], digits )
        print( object1 )
    } else {
        print( round( object, digits ) )
    }
}


.pr <- sirt_print_helper

## File Name: linking_haberman_huber_weight.R
## File Version: 0.04

linking_haberman_huber_weight <- function(x, cutoff)
{
    eps <- 1e-10
    wgt_adj <- (abs(x) >=cutoff)*cutoff / abs(x) + (abs(x) < cutoff)*1
    return(wgt_adj)
}

## File Name: rm_facets_center_value_aggregate.R
## File Version: 0.071

rm_facets_center_value_aggregate <- function(x, index, value=0)
{
    y <- rep(NA, length(x) )
    index_unique <- unique(index)
    NI <- length(index_unique)
    for (ii in 1L:NI){
        index_ii <- index_unique[ii]
        ind_ii <- which( index==index_ii )
        y[ind_ii] <- rm_facets_center_value(x=x[ind_ii], value=value)
    }
    return(y)
}

## File Name: lsdm_est_logist_2pl.R
## File Version: 0.225


lsdm_est_logist_2pl <- function( y, theta, wgt_theta, method="L-BFGS-B",
        b0=NULL, a0=NULL)
{
    y <- as.numeric(y)

    #-- define optimization function
    dist_irf_val <- function(x){
        a <- x[2]
        b <- x[1]
        irf_est <- stats::plogis(a*(theta-b))
        diff_irf <- y - irf_est
        val <- sum( diff_irf^2 * wgt_theta )
        return(val)
    }
    dist_irf_grad <- function(x){
        grad <- rep(0,2)
        a <- x[2]
        b <- x[1]
        irf_est <- stats::plogis(a*(theta-b))
        diff_irf <- y - irf_est
        der1 <- 2*diff_irf*irf_est*(1-irf_est)*wgt_theta
        grad[1] <- sum( der1*a )
        grad[2] <- -sum( der1*(theta-b) )
        return(grad)
    }

    #* perform optimization
    if (is.null(b0)){
        b0 <- theta[ which.min( abs(y-0.5) ) ]
        par0 <- c(b0, 1)
    } else {
        par0 <- c( b0, a0)
    }
    res <- stats::optim( par=par0, fn=dist_irf_val, gr=dist_irf_grad, method=method)

    #--- output
    res <- c( res$par, sqrt(res$value) )
    return(res)
}

## File Name: rm_facets_pem_acceleration.R
## File Version: 0.17

rm_facets_pem_acceleration <- function( iter, pem_parameter_index, pem_parameter_sequence,
        a.rater, Qmatrix, tau.item, VV, K, I, TP, a.item, b.item, b.rater, item.index, rater.index, theta.k, RR,
        dat2, dat2.resp, pi.k, dat2.ind.resp, ll, mu, sigma, pem_pars, a_center_type,
        PEM_itermax,  b.rater.center, a.rater.center, a.item.center, a_lower, a_upper )
{
    PEM <- TRUE
    #-- collect all parameters in a list
    parmlist <- list( mu=mu, tau.item=tau.item, a.item=a.item, b.rater=b.rater, a.rater=a.rater, sigma=sigma)
    #-- transform into a vector
    pem_parm <- sirt_pem_collect_parameters( parmlist=parmlist, pem_parameter_index=pem_parameter_index )
    #-- collect parameters in initial iterations
    pem_parameter_sequence <- sirt_pem_parameter_sequence_initial_iterations( pem_parm=pem_parm,
                                            pem_parameter_sequence=pem_parameter_sequence, iter=iter )

    if ( ( iter %% 2==0 ) & ( iter > 0 ) & ( iter < PEM_itermax ) ){

        pem_parameter_sequence$P2 <- pem_parm

        ll_args <-  list( tau.item=tau.item, a.rater=a.rater, Qmatrix=Qmatrix, b.item=b.item, VV=VV, K=K, I=I, TP=TP,
                        a.item=a.item, b.rater=b.rater, item.index=item.index, rater.index=rater.index,
                        theta.k=theta.k, RR=RR, dat2=dat2, dat2.resp=dat2.resp, pi.k=NULL,
                        dat2.ind.resp=dat2.ind.resp, mu=mu, sigma=sigma, b.rater.center=b.rater.center,
                        a.rater.center=a.rater.center, a.item.center=a.item.center, a_lower=a_lower, a_upper=a_upper
                        )
        #** baseline likelihood
        ll_args <- sirt_pem_include_ll_args( ll_args=ll_args, pem_parm=pem_parm, pem_pars=pem_pars,
                            pem_parameter_index=pem_parameter_index )
        res <- do.call( what=rm_facets_calc_loglikelihood, args=ll_args )
        ll0 <- ll <- res$ll
        P0 <- pem_parameter_sequence$P0
        P1 <- pem_parameter_sequence$P1
        P2 <- pem_parameter_sequence$P2
        iterate <- TRUE
        ii <- 0
        #--- begin PEM iterations
        while (iterate){
            ll_args0 <- ll_args
            res0 <- res
            ll0 <- ll
            tt <- sirt_pem_algorithm_compute_t( i=ii )
            Pnew <- sirt_pem_algorithm_compute_Pnew( tt=tt, P0=P0, P1=P1, P2=P2 )
            ll_args <- sirt_pem_include_ll_args( ll_args=ll_args, pem_parm=Pnew, pem_pars=pem_pars,
                            pem_parameter_index=pem_parameter_index )
            res <- do.call( what=rm_facets_calc_loglikelihood, args=ll_args )
            ll <- res$ll
            if ( is.na(ll) ){
                ll <- - Inf
                iterate <- FALSE
            }
            if ( ll < ll0 ){
                iterate <- FALSE
            }
            ii <- ii + 1
        }
        #-- end PEM iterations

        tau.item <- ll_args0$tau.item
        mu <- ll_args0$mu
        sigma <- ll_args0$sigma
        pi.k <- res0$pi.k
        b.rater <- res0$b.rater
        a.rater <- res0$a.rater
        a.item <- res0$a.item
        ll <- res0$ll
        pem_parameter_sequence$P0 <- P1
        pem_parameter_sequence$P1 <- P2
    }
    if (iter > PEM_itermax){
        PEM <- FALSE
    }
    #--- output
    res <- list(ll=ll, pem_parameter_sequence=pem_parameter_sequence, a.rater=a.rater, b.rater=b.rater, a.item=a.item,
                    tau.item=tau.item, pi.k=pi.k, mu=mu, sigma=sigma, PEM=PEM)
    return(res)
}

## File Name: linking_haberman_itempars_prepare.R
## File Version: 0.132


linking_haberman_itempars_prepare <- function(b, a=NULL, wgt=NULL)
{
    I <- nrow(b)
    NS <- ncol(b)
    if ( is.null(rownames(b) )){
        i0 <- ceiling( log10(I) + 1 )
        rownames(b) <- paste0('I', 10^i0 + 1L:I)
    }
    if ( is.null(colnames(b) )){
        colnames(b) <- 1L:NS
    }
    if (is.null(a)){
        a <- matrix(1, nrow=I, ncol=NS)
    }
    if (is.null(wgt)){
        wgt <- matrix(1, nrow=I, ncol=NS)
    }
    #-- arrange table
    itempars <- data.frame(study=rep(colnames(b), each=I),
                        item=rep(rownames(b), NS), a=as.vector(a),
                        b=as.vector(b), wgt=as.vector(wgt) )
    return(itempars)
}

## File Name: nedelsky.latresp.R
## File Version: 0.113


#---- latent responses for Nedelsky function
nedelsky.latresp <- function(K)
{
    nodes <- c(0,1)
    ndim <- K
    combis <- as.matrix( expand.grid( as.data.frame(
                            matrix( rep(nodes, ndim), ncol=ndim ))))
    return(combis)
}

## File Name: f1d.irt.R
## File Version: 1.296

#--- Functional unidimensional model (Ip et al., 2013)
f1d.irt <- function( dat=NULL, nnormal=1000, nfactors=3,
            A=NULL, intercept=NULL, mu=NULL, Sigma=NULL, maxiter=100,
            conv=10^(-5), progress=TRUE )
{
    if ( ! is.null(dat) ){
        # estimate tetrachoric correlation matrix
        if (progress){
            cat('*** Estimate tetrachoric correlation\n')
        }
        tetra <- res <- tetrachoric2(dat=dat, progress=progress)
        # estimate factor analysis
        fac1 <- sirt_import_psych_fa( r=res$rho, nfactors=nfactors, rotate='none' )
        fac0 <- sirt_import_psych_fa( r=res$rho, nfactors=1, rotate='none' )
        # extract standardized loadings
        A_stand <- as.matrix( fac1$loadings )
        # calculate communality
        h2 <- rowSums( A_stand^2 )
        # unstandardized loadings
        A <- A_stand / sqrt( 1-h2 )
        # intercepts
        intercept <- - res$tau / sqrt( 1 - h2 )
        names.dat <- colnames(dat)
    } else {
        a0 <- NA
        d0 <- NA
        A[ is.na(A) ] <- 0
        names.dat <- names(intercept)
        tetra <- NULL
    }

    #***************************************
    # approximation of normal distribution using quasi Monte Carlo integration nodes
    theta <- qmc.nodes( nnormal, nfactors )
    if ( is.null(mu) ){
        mu <- rep(0,nfactors)
    }
    if ( is.null(Sigma) ){
        Sigma <- diag(1,nfactors)
    }

    wgt_theta <- sirt_dmvnorm_discrete(x=theta, mean=mu, sigma=Sigma )

    I <- length(intercept)
    TP <- nrow(theta)
    W1 <- diag(wgt_theta )

    D <- ncol(A)
    # a_i ' theta_p
    Zpi <- matrix( 0, TP, I )
    for (dd in 1L:D){
        Zpi <- Zpi + theta[,dd] * matrix( A[,dd], TP, I, byrow=TRUE )
    }
    # Z_pi=a_i theta_p + d_i
    Zpi <- Zpi + matrix( intercept, TP, I, byrow=TRUE )

    # starting values for a_i and d_i
    di <- intercept
    diast <- di  # diast=d_i*
    aiast <- rep(.7,I)

    thetaast <- rep(0,TP)
    iter <- 0
    parchange <- 1

    #***** begin algorithm
    while( ( iter < maxiter ) & ( parchange > conv ) ){

        thetaast0 <- thetaast
        aiast0 <- aiast
        diast0 <- diast

        # (1) update theta_p using ( a_i, theta_p, d_i, a_i*, d_i* )
        Ypi <- Zpi - matrix( diast, nrow=TP, ncol=I, byrow=TRUE)
        aiastM <- matrix( aiast, nrow=TP, ncol=I, byrow=TRUE )
        thetaast <- rowSums( Ypi * aiastM ) / rowSums( aiastM^2 )
        wM <- stats::weighted.mean( thetaast, wgt_theta )
        sdM <- sqrt( sum( ( thetaast - wM )^2 * wgt_theta ) )
        thetaast <- ( thetaast - wM ) / sdM

        X <- cbind( 1, thetaast )
        c1 <- solve( crossprod( X, W1 ) %*% X, crossprod( X, W1 ) %*% Zpi )
        diast <- c1[1,]
        aiast <- c1[2,]

        # compute approximation error
        errpi <- Zpi - thetaast * matrix(aiast,TP, I, byrow=TRUE )  -
                            matrix(diast,TP, I, byrow=TRUE )
        approx.error <- sum( errpi^2 * wgt_theta ) / I
        # parameter change
        parchange <- max( abs( c(diast - diast0,aiast-aiast0,thetaast-thetaast0) ))
        iter <- iter + 1
        if (progress){
            cat( paste0( 'Iteration ', iter,
                ' | Approximation error ', '=', ' ', round( approx.error, 5 ),
                ' | Max. parameter change ', '=', ' ', round( parchange, 5),
                '\n') )
            utils::flush.console()
        }
    }
    #***** end algorithm

    if ( ! is.null(dat) ){
        # unstandardized loadings 1 factor model
        A0_stand <- fac0$loadings
        a0 <- A0_stand[,1] / sqrt( 1 - A0_stand[,1]^2 )
        d0 <- - res$tau / sqrt( 1 - A0_stand[,1]^2 )
    }
    if ( is.null(dat) ){
        a0 <- NULL
        d0 <- NULL
    }

    item <- data.frame( item=names.dat )
    item$ai.ast <- aiast
    item$ai0 <- a0
    item$di.ast <- diast
    item$di0 <- d0
    person <- data.frame( theta.ast=thetaast, wgt=wgt_theta )
    #--- output
    res <- list( item=item, person=person, A=A, intercept=intercept,
                dat=dat, tetra=tetra )
    return(res)
}

## File Name: regpolca_postproc_irf.R
## File Version: 0.05


regpolca_postproc_irf <- function(probs_items, dat, lca_dich)
{
    K <- dim(probs_items)[3]
    NC <- dim(probs_items)[2]
    I <- dim(probs_items)[1]
    ci <- paste0('Cl',1:K)
    if (lca_dich){
        item <- probs_items[,2,]
        colnames(item) <- ci
        item <- data.frame( item=colnames(dat), item)
    } else {
        item <- NULL
        for (cc in 1:NC){
            dfr1 <- probs_items[,cc,]
            colnames(dfr1) <- ci
            dfr1 <- data.frame(item=1:I, cat=paste0('Cat',cc-1), dfr1)
            item <- rbind(item, dfr1)
        }
        item <- item[ order(item$item), ]
        item$item <- colnames(dat)[ item$item ]
    }
    return(item)
}

## File Name: dmlavaan_create_coef.R
## File Version: 0.01

dmlavaan_create_coef <- function(partable)
{
    coef1 <- partable$est1
    names(coef1) <- paste0(partable$parname, '_mod1')
    coef2 <- partable$est2
    names(coef2) <- paste0(partable$parname, '_mod2')
    coef <- c( coef1, coef2 )
    coef <- coef[ ! is.na(coef) ]
    return(coef)
}

## File Name: mgsem_grad_fun_numeric_approx.R
## File Version: 0.06


mgsem_grad_fun_numeric_approx <- function(x, opt_fun_args)
{
    h <- opt_fun_args$technical$h
    NP <- opt_fun_args$NP
    grad <- rep(0,NP)
    for (pp in seq_len(NP) ){
        coef1 <- mgsem_add_increment(x=x,h=h, i1=pp)
        coef2 <- mgsem_add_increment(x=x,h=-h, i1=pp)
        ll1 <- mgsem_opt_fun(x=coef1, opt_fun_args=opt_fun_args)
        ll2 <- mgsem_opt_fun(x=coef2, opt_fun_args=opt_fun_args)
        D1 <- (ll1-ll2)/(2*h)
        grad[pp] <- grad[pp] + D1
    }
    return(grad)
}

## File Name: linking.haberman.lq.R
## File Version: 0.257

linking.haberman.lq <- function(itempars, pow=2, eps=1e-3, a_log=TRUE,
    use_nu=FALSE, est_pow=FALSE, lower_pow=.1, upper_pow=3, method="joint",
    le=FALSE, vcov_list=NULL)
{
    CALL <- match.call()
    s1 <- Sys.time()

    use_pw <- method %in% c('pw1','pw2')
    if ( (!use_pw) | (pow!=2) | (!a_log) ){ le <- FALSE }

    #--- process item parameters
    res <- linking_proc_itempars(itempars=itempars)
    itempars <- res$itempars
    NS <- res$NS
    NI <- res$NI
    items <- res$items
    studies <- res$studies
    wgtM <- res$wgtM
    aM <- res$aM
    bM <- res$bM
    est_pars <- res$est_pars
    weights_exist <- res$weights_exist
    a.orig <- aM
    b.orig <- bM
    itempars1 <- itempars
    items <- sirt_sup(x=itempars1[,2])
    studies <- sirt_sup(x=itempars1[,1])
    G <- length(studies)
    I <- length(items)
    ind_items <- match(itempars1[,2], items)
    ind_studies <- match(itempars1[,1], studies)
    IP <- nrow(itempars1)
    X0 <- matrix(0, nrow=IP, ncol=G-1+I)
    colnames(X0) <- c(studies[-1], items)
    w <- itempars1[,5]

    res_optim <- list()
    eps_vec <- 10^seq(0,-10, by=-.5)
    item <- data.frame(item=items)

    #**** estimation of A
    y <- itempars1[,3]
    if (a_log){
        y <- log(y)
    }
    X <- X0
    for (gg in 2L:G){
        X[ ind_studies==gg, gg-1] <- 1
    }
    for (ii in 1L:I){
        X[ ind_items==ii, ii+G-1] <- 1
    }

    if ( use_pw ){
        res <- linking_haberman_lq_pw_create_design(y=y, ind_studies=ind_studies,
                    ind_items=ind_items, method=method)
        y <- res$y
        X <- res$X
        w <- res$w
        des_pw_slopes <- res
    }

    #- fit
    res_optim$slopes <- mod0 <- lq_fit(y=y, X=X, w=w, pow=pow, eps=eps,
                eps_vec=eps_vec, est_pow=est_pow, lower_pow=lower_pow,
                upper_pow=upper_pow)
    coef0 <- mod0$coefficients
    pow_slopes <- mod0$pow
    ind_groups <- 1L:(G-1)
    coef0_A <- coef0[ind_groups]
    if (!use_pw){
        a_joint <- coef0[-c(ind_groups)]
    } else {
        a_joint <- NULL
    }
    if (!use_pw){
        ar <- y - X %*% coef0
    } else {
        ar <- NULL
    }

    if (a_log){
        coef0_A <- exp(c(0,coef0_A))
        if (!use_pw){
            a_joint <- exp(a_joint)
            ar <- a_joint[ind_items]*( exp(ar) - 1 )
        }
    } else {
        coef0_A <- 1+c(0,coef0_A)
    }
    resid <- itempars1
    if (!use_pw){
        resid$adif <- ar
    }

    At <- coef0_A
    transf.personpars <- data.frame(study=studies, A_theta=coef0_A, se_A_theta=NA)
    item$a <- a_joint

    #**** estimation of B
    y <- itempars1[,4] * coef0_A[ ind_studies ]
    if (use_nu){
        y <- -itempars1[,4]*itempars1[,3]
        for (gg in 2L:G){
            ind_gg <- which(ind_studies==gg)
            X[ ind_gg, gg-1] <- itempars1[ ind_gg,3]/coef0_A[gg]
        }
    }
    if ( use_pw ){
        res <- linking_haberman_lq_pw_create_design(y=y, ind_studies=ind_studies,
                    ind_items=ind_items, method=method)
        y <- res$y
        X <- res$X
        w <- res$w
    }
    #- fit
    res_optim$intercepts <- mod0 <- lq_fit(y=y, X=X, w=w, pow=pow, eps=eps,
                eps_vec=eps_vec, est_pow=est_pow, lower_pow=lower_pow,
                upper_pow=upper_pow)
    coef0 <- mod0$coefficients

    pow_intercepts <- mod0$pow
    coef0_B <- -coef0[ind_groups]

    if (!use_pw){
        b_joint <- coef0[-c(ind_groups)]
    }
    if (use_nu){
        coef0_B <- -coef0_B
        if (!use_pw){
            b_joint <- -b_joint
        }
    }
    if (!use_pw){
        resid$b_resid <- y - X %*% coef0
    }
    Bt <- coef0_B <- c(0, coef0_B)
    transf.personpars$B_theta <- coef0_B
    transf.personpars$se_B_theta <- NA
    rownames(transf.personpars) <- NULL
    if (!use_pw){
        item$b <- b_joint
    }

    res_vcov <- NULL
    if (le){
        res_vcov <- linking_haberman_lq_pw_le(des=des_pw_slopes, res_optim=res_optim,
                        vcov_list=vcov_list, itempars=itempars)
    }  # end if le==TRUE

    #- include joint item parameters
    if (!use_pw){
        resid$a_joint <- item[ind_items, 'a']
        resid$b_joint <- item[ind_items, 'b']
    }

    # transformation for item parameters
    transf.itempars <- data.frame( study=studies, At=1/At, se_At=NA,
                            Bt=-Bt, se_Bt=NA )

    converged <- res_optim$intercepts$converged & res_optim$slopes$converged
    s2 <- Sys.time()
    time <- list(s1=s1, s2=s2)

    #-- output list
    description <- 'Linking based on L_q distance according to Haberman (2009)'
    res <- list( transf.personpars=transf.personpars, transf.itempars=transf.itempars,
                    pow=pow, eps=eps, item=item, resid=resid, description=description,
                    converged=converged, a_log=a_log, use_nu=use_nu, est_pow=est_pow,
                    pow_slopes=pow_slopes, pow_intercepts=pow_intercepts,
                    vcov=res_vcov, method=method, CALL=CALL, time=time)
    class(res) <- 'linking.haberman.lq'
    return(res)

}

## File Name: L0_polish.R
## File Version: 0.131


L0_polish <- function(x, tol, conv=0.01, maxiter=30, type=1, verbose=TRUE)
{
    res <- list(x_update=x, iterate_further=TRUE)
    #-- iterate
    while(res$iterate_further){
        res <- L0_polish_one_iteration(x=res$x_update, tol=tol, type=type, eps=conv)
        if (verbose){
            v1 <- paste0('Interactions detected: ', res$N_elim)
            v2 <- paste0(' | Absolute value residual: ', round(res$max_resid,3) )
            cat(v1, v2, '\n')
            utils::flush.console()
        }
    }
    #--- output
    return(res)
}

## File Name: invariance_alignment_cfa_config_estimate_define_lavaan_model.R
## File Version: 0.091


invariance_alignment_cfa_config_estimate_define_lavaan_model <-
        function(items_gg, label_F="F", model="2PM")
{
    while (label_F %in% items_gg){
        label_F <- paste0( label_F, 'F')
    }
    if (substring(model,1,1)=='2'){
        lavmodel <- paste0(label_F, '=~', paste0(items_gg, collapse='+') )
    } else {
        lavmodel <- paste0(label_F, '=~', paste0( paste0('a*',items_gg), collapse='+') )
        lavmodel1 <- paste0(paste0(items_gg, '~~b*', items_gg), collapse='\n' )
        lavmodel <- paste0( lavmodel, '\n', lavmodel1)
    }
    return(lavmodel)
}

## File Name: sirt_import_coda_effectiveSize.R
## File Version: 0.02


sirt_import_coda_effectiveSize <- function(...)
{
    TAM::require_namespace_msg('coda')
    res <- coda::effectiveSize(...)
    return(res)
}

## File Name: diag2.R
## File Version: 0.04

diag2 <- function( vec)
{
    if ( length(vec) > 1){
        res <- diag(vec)
    } else {
        res <- matrix(vec, nrow=1,ncol=1)
    }
    return(res)
}

## File Name: sirt_summary_print_display.R
## File Version: 0.04

sirt_summary_print_display <- function(symbol="-", len=65)
{
    res <- paste0( paste0(rep(symbol, len), collapse=''), '\n' )
    return(res)
}

## File Name: lsem_permutationTest_collect_output.R
## File Version: 0.02

lsem_permutationTest_collect_output <- function(res0)
{
    res0_out <- list(est=res0$parameters$est, M=res0$parameters_summary$M,
                            SD=res0$parameters_summary$SD,
                            MAD=res0$parameters_summary$MAD,
                            lin_slo=res0$parameters_summary$lin_slo)
    return(res0_out)
}

## File Name: rm_sdt_calc_probs_grm_item_rcpp.R
## File Version: 0.05


rm_sdt_calc_probs_grm_item_rcpp <- function( tau.item, a.item, theta.k, VV,
    K, TP, eps=0, use_log=FALSE)
{
    prob_item <- sirt_rcpp_rm_sdt_calc_probs_grm_item( tau_item=tau.item, a_item=a.item,
                    theta_k=theta.k, VV=VV, K=K, TP=TP, eps=eps, use_log=use_log)
    prob.item <- array(prob_item, dim=c(VV,K+1,TP) )
    return(prob.item)
}

## File Name: rasch_mml2_mstep_calc_loglike.R
## File Version: 0.02


rasch_mml2_mstep_calc_loglike <- function( exp_r, prob1, exp_n, prob0=NULL)
{
    if (is.null(prob0)){
        prob0 <- 1 - prob1
    }
    y <- rowSums( exp_r * log(prob1) + (exp_n - exp_r) * log(prob0) )
    return(y)
}

## File Name: md.pattern.sirt.R
## File Version: 0.121

#----- Function for analyzing response patterns
md.pattern.sirt <- function(dat)
{
    dat <- as.matrix(dat)
    if ( ncol(dat)>1000 ){
        stop('Function only works for datasets with fewer than 1000 variables!\n')
    }
    res <- md_pattern_rcpp( dat_=dat )
    rp_unique <- unique(res$unique_resp_patt)
    res$unique_resp_patt <- match( res$unique_resp_patt, rp_unique )
    res$resp_patt <- match( res$resp_patt, rp_unique )
    res$dat.ordered <- res$dat[ order( res$resp_patt ), ]
    return(res)
}


#----- calling the Rcpp function
md_pattern_rcpp <- function(dat_){
    md_pattern_csource( dat_ )
}

## File Name: rm_summary_information_criteria.R
## File Version: 0.09

rm_summary_information_criteria <- function(object, digits_crit=0, digits_penalty=2)
{
    ic <- object$ic
    rm_summary_information_criteria_print_one_criterium(ic=ic, crit="AIC",
            desc="AIC=-2*LL + 2*p ", digits_crit=digits_crit, digits_penalty=digits_penalty)
    rm_summary_information_criteria_print_one_criterium(ic=ic, crit="AICc",
            desc="AICc=-2*LL + 2*p + 2*p*(p+1)/(n-p-1)  (bias corrected AIC)", digits_crit=digits_crit, digits_penalty=digits_penalty)
    rm_summary_information_criteria_print_one_criterium(ic=ic, crit="BIC",
            desc="BIC=-2*LL + log(n)*p ", digits_crit=digits_crit, digits_penalty=digits_penalty)
    rm_summary_information_criteria_print_one_criterium(ic=ic, crit="CAIC",
            desc="CAIC=-2*LL + [log(n)+1]*p  (consistent AIC)", digits_crit=digits_crit, digits_penalty=digits_penalty)
    cat("\n")
}

## File Name: Q3.testlet.R
## File Version: 1.21


################################################################################
# Summarizing testlet effect using Q3 statistic
Q3.testlet <- function( q3.res, testlet.matrix, progress=TRUE )
{
    N.item.testlet <- stats::aggregate( rep(1, nrow(testlet.matrix) ),
                              list( testlet.matrix[,1]), sum )
    testlet.matrix <- testlet.matrix[ testlet.matrix[,1] %in%
                                N.item.testlet[ N.item.testlet[,2] > 1, 1 ], ]
    testlets <- sort( unique( testlet.matrix[,1] ) )
    testlet.q3 <- t( sapply( testlets, FUN=function(testlet){
            testlet.items <- testlet.matrix[ testlet.matrix[,1]==testlet, 2 ]
            ti.ind <- colnames(q3.res$q3.matrix) %in% testlet.items
            # c( sum( ti.ind), mean( q3.res$q3.matrix[ ti.ind, ti.ind ], na.rm=T ) )
            # correction thanks to Thomas Kiefer (2014-03-06)
                c( sum( ti.ind), mean(
                     q3.res$q3.matrix[ ti.ind, ti.ind ][
                            lower.tri( diag( sum(ti.ind))) ],
                     na.rm=TRUE ) )
                } ) )
    colnames(testlet.q3) <- c("N.Items", "Mean.Q3" )
    testlet.q3 <- data.frame( "Testlet"=testlets, testlet.q3,
                            "mean"=mean(q3.res$q3.long[,3]) )
    rownames(testlet.q3) <- NULL
    # mean Q3-statistics between testlets
    TT <- length(testlets)
    matr <- matrix( 1, nrow=TT, ncol=TT )
    colnames(matr) <- rownames(matr) <- testlets
    for (ii1 in seq(1,TT-1)){
        for (ii2 in seq(ii1+1, TT )){
            tt1 <- paste(testlets[ii1])
            tt2 <- paste(testlets[ii2])
            itt1 <- testlet.matrix[ testlet.matrix[,1]==tt1,2 ]
            itt2 <- testlet.matrix[ testlet.matrix[,1]==tt2,2 ]
            q.tt <- q3.res$q3.matrix[ paste( itt1 ), paste( itt2) ]
            matr[ tt1, tt2 ] <- matr[tt2,tt1] <- mean( q.tt, na.rm=TRUE )
        }
    }
    diag(matr) <- testlet.q3$Mean.Q3
    if (progress){
        cat( "\nMean Q3 Testlets:", round( mean(testlet.q3$Mean.Q3), 5 ),"\n\n")
        print( testlet.q3, digits=3 )
        cat( "\n\nMean Q3 between testlets \n\n")
        matr1 <- round( matr, 3 )
        print( matr1, digits=3 )
    }
    #--- OUTPUT
    res <- list( "testlet.q3"=testlet.q3, "testlet.q3.korr"=matr )
    return(res)
}
##########################################################################

# Q3.testlet <- testlet.yen.q3

## File Name: logLik_sirt.R
## File Version: 0.152


#--- log-likelihood function rasch.copula2
logLik.rasch.copula2 <- function (object, ...)
{
    # extract log-likelihood
    out <- - object$ic$deviance / 2
    # number of parameters
    attr(out, 'df') <- object$ic$np
    # extract number of observations
    attr(out, 'nobs') <- object$ic$n
    class(out) <- 'logLik'
    return(out)
}
logLik.rasch.copula3 <- logLik.rasch.copula2

#-- logLik.rasch.mml
logLik.rasch.mml <- logLik.rasch.copula2

#-- smirt
logLik.smirt <- logLik.rasch.copula2

#-- rasch.mirtlc
logLik.rasch.mirtlc <- logLik.rasch.copula2

#-- gom
logLik.gom <- logLik.rasch.copula2

#-- rm.facets
logLik.rm.facets <- logLik.rasch.copula2

#-- rm.sdt
logLik.rm.sdt <- logLik.rasch.copula2

#-- prob.guttman
logLik.prob.guttman <- logLik.rasch.copula2

#-- xxirt
logLik.xxirt <- logLik.rasch.copula2

## File Name: pcm.fit.R
## File Version: 0.22

#############################################
# fit partial credit (or Rasch model)
pcm.fit <- function( b, theta, dat )
{
    N <- length(theta)
    if ( is.vector(b) ){
        b <- matrix( b, ncol=1 )
    }
    K <- ncol(b)
    I <- nrow(b)
    b[ is.na(b) ] <- 999
    dat.ind <-  1 - is.na(dat)
    dat[ is.na(dat) ] <- 0

    # create probability matrix
    rprobs <- array( 0, dim=c( N, K +1, I ) )
    score_vec <- 0:K
    M0 <- TAM::tam_outer( theta, score_vec )
    for (ii in 1:I){
        M1 <- M0
        M1[,-1] <- M1[,-1] - TAM::tam_matrix2( b[ii,], nrow=N )
        M1_max <- rowMaxs.sirt(matr=M1)$maxval
        M1 <- M1 - M1_max
        M1 <- exp(M1)
        M1 <- M1 / rowSums(M1)
        rprobs[,,ii] <- M1
    }

    # expected response
    Eni <- array( 0, dim=c(N,I) )

    M2 <- TAM::tam_matrix2( 0:K, nrow=N)
    for (ii in 1:I){
        Eni[,ii] <- rowSums( M2*rprobs[,,ii]  )
    }
    # calculate residuals
    Yni <- dat - Eni
    # calculate variances
    Wni <- array( 0, dim=c(N,I) )
    for (ii in 1:I){
        Wni[,ii] <- rowSums( ( M2  - Eni[,ii] )^2 *rprobs[,,ii]  )
    }
    # calculate kurtosis
    Cni <- array( 0, dim=c(N,I) )
    for (ii in 1:I){
        Cni[,ii] <- rowSums( ( M2  - Eni[,ii] )^4 *rprobs[,,ii]  )
    }
    # standardized residual
    zni <- Yni / sqrt( Wni )
    #************************************
    # item fit statistics
    N.item <- colSums( dat.ind )
    #--- Outfit
    outfit <- colSums( zni^2 * dat.ind ) / N.item
    itemfit <- data.frame( "item"=colnames(dat), "outfit"=outfit )
    qi <- sqrt( colSums( dat.ind * Cni / Wni^2  ) / N.item^2 - 1 / N.item )
    itemfit$outfit.t <- ( itemfit$outfit^(1/3) - 1 ) * ( 3 / qi ) + qi / 3
    #--- Infit
    itemfit$infit <- colSums( dat.ind * Wni * zni^2 ) / colSums( dat.ind * Wni )
    qi <- sqrt( colSums( dat.ind * ( Cni - Wni^2 )  ) / ( colSums(Wni*dat.ind)  )^2  )
    itemfit$infit.t <- ( itemfit$infit^(1/3) - 1 ) * ( 3 / qi ) + qi / 3
    itemfit0 <- itemfit
    #************************************
    # person fit statistics
    N.item <- rowSums( dat.ind )
    #--- Outfit
    outfit <- rowSums( zni^2 * dat.ind ) / N.item
    personfit <- data.frame( "person"=1:N, "outfit"=outfit )
    qi <- sqrt( rowSums( dat.ind * Cni / Wni^2  ) / N.item^2 - 1 / N.item )
    personfit$outfit.t <- ( personfit$outfit^(1/3) - 1 ) * ( 3 / qi ) + qi / 3
    #--- Infit
    personfit$infit <- rowSums( dat.ind * Wni * zni^2 ) / rowSums( dat.ind * Wni )
    qi <- sqrt( rowSums( dat.ind * ( Cni - Wni^2 )  ) / ( rowSums(Wni*dat.ind)  )^2  )
    personfit$infit.t <- ( personfit$infit^(1/3) - 1 ) * ( 3 / qi ) + qi / 3
    # output
    res <- list("itemfit"=itemfit, "personfit"=personfit)
    return(res)
}
#############################################################

## File Name: xxirt_compute_prob_item_from_x.R
## File Version: 0.06

xxirt_compute_prob_item_from_x <- function(x, em_args, item_index=NULL)
{
    partable <- xxirt_partable_include_freeParameters( partable=em_args$partable,
                                        x=x[ em_args$parindex_items ] )
    probs_items <- xxirt_compute_itemprobs( item_list=em_args$item_list,
                                items=em_args$items, Theta=em_args$Theta,
                                ncat=em_args$ncat, partable=partable,
                                partable_index=em_args$partable_index,
                                item_index=item_index)
    return(probs_items)
}

## File Name: lsdm_irf_distance_mad.R
## File Version: 0.04

lsdm_irf_distance_mad <- function(data, data_fitted, wgt_theta, use_abs=TRUE)
{
    dist <- data - data_fitted
    if (use_abs){
        dist <- abs(dist)
    }
    res <- rowSums(dist*wgt_theta)
    return(res)
}

## File Name: summary.mcmc_WaldTest.R
## File Version: 0.01

# summary of Wald Test based on MCMC output
summary.mcmc_WaldTest <- function( object, digits=3, ... )
{
    cat("Wald Test\n")
    W1 <- sprintf( paste0("%.", digits, "f" ), object$chisq_stat["chi2"] )

    v1 <- paste0("Chi^2=",  W1, ", df=", object$chisq_stat["df"])
    v1 <- paste0( v1, ", p=", sprintf( paste0("%.", digits, "f" ),
                    object$chisq_stat["p"] ) )
    cat(v1)

    cat("\n\nSummary Hypotheses\n")
    obji <- object$hypotheses_summary
    vars <- c("parameter","MAP","SD", "Q2.5", "Q97.5", "Rhat","SERatio",
                    "effSize" )
    obji <- obji[,vars]
    NO <- ncol(obji)
    obji[,NO] <- round(obji[,NO])
    sirt_summary_print_objects(obji=obji, digits=digits, from=2)
}

## File Name: mgsem_proc_model_is_B.R
## File Version: 0.04


mgsem_proc_model_is_B <- function(model)
{
    H <- length(model)
    is_B <- 0
    for (hh in 1L:H){
        is_B <- is_B + ( ! is.null( model[[hh]][['est']][['B']] ) )
    }
    is_B <- ( is_B > 0 )
    return(is_B)
}

## File Name: rasch.copula3.R
## File Version: 6.542



#------------------------------------------------------------------
# Copula estimation in a rasch type model
rasch.copula3 <- function( dat, itemcluster, dims=NULL,
                        copula.type="bound.mixt",
                        progress=TRUE, mmliter=1000, delta=NULL,
                        theta.k=seq(-4,4,len=21), alpha1=0, alpha2=0,
                        numdiff.parm=.000001,
                        est.b=seq(1,ncol(dat)), est.a=rep(1,ncol(dat)), est.delta=NULL,
                        b.init=NULL, a.init=NULL,
                        est.alpha=FALSE,
                        glob.conv=.0001, alpha.conv=.0001, conv1=.001,
                        dev.crit=.2, rho.init=.5, increment.factor=1.01
#                        pattern.off=FALSE
                                        ){
    ###############################################################
    # INPUT:
    # dat    ... data frame
    # itemcluster ... vector of integers
    # progress    ... display progress?
    # mmliter     ... see rasch.mml
    # delta ... initial delta estimate (is relevant for fixing delta parameters)
    # theta.k... number of grid theta points
    # alpha1, alpha2 ... rasch type parameter
    # numdiff.parm ... numerical differentiation parameter
    # est.b    ... which b parameters shall be estimated
    ###############################################################
    s1 <- Sys.time()
    CALL <- match.call()
    group <- NULL
    N <- nrow(dat)
    if (is.null(dims) ){
        dims <- rep( 1, ncol(dat) )
                    }

    # arrange item clusters item clusters
    t1 <- table(itemcluster)
    itemcluster[ which( itemcluster %in% names(t1)[ t1==1     ] ) ] <- 0
    itemcluster <- match( itemcluster, unique( sort(itemcluster[itemcluster!=0]) ) )
    itemcluster[ is.na( itemcluster ) ] <- 0
    t1 <- table(itemcluster)
    t1c <- t1[ names(t1) !=0 ]
    t1b <- as.numeric( names(t1))
    t1c <- t1[ names(t1) !=0 ]
    theta.k <- as.matrix(theta.k)
    I <- ncol(dat)


    if ( any( t1c==1) ){
           stop( "There should be at least two items in an item cluster\n" )
                                    }
            x1 <- seq( 1, max(t1b) )
            x2 <- sort(setdiff( as.numeric(sort(names(t1))), 0 ))
            if ( sum( abs( x1-x2) ) > 10^(-10) ){
               stop( "Item cluster identifiers must be recoded to 1, ..., C\n" )
                                 }
    CC <- length(x1) # number of clusters
    # calculation of number of itemclusters
    if ( progress  ){
        cat("-----------------------------------------------------------------\n")
        cat("Marginal Maximum Likelihood Estimation \n")
        cat(paste( "Raschtype Copula Model with generalized logistic link function: Estimation of alpha1 and alpha2 \n") )
        cat("Function 'rasch.copula3'\n")
        cat("-----------------------------------------------------------------\n")
        flush.console()
      }
    # dimensions
    dims <- match( dims, unique( dims ) )
    D <- length(unique(dims))
    theta.k <- as.matrix( expand.grid( as.data.frame( matrix( rep( theta.k, D), ncol=D ) ) ) )
#    D <- ncol(theta.k)

    G <- 1    # function can only be applied for single groups


     # arrange copula types
    if ( length( copula.type )==1 ){ copula.type <- rep( copula.type, CC ) }

    I <- ncol(dat)
    if ( is.null( colnames(dat))){
        colnames(dat) <- paste("Item", 1:I, sep="")
                }
    # remove cases where all responses are missings
    ind <- which( rowMeans( is.na(dat) ) < 1 )
    N0 <- nrow(dat)
    dat <- dat[ ind, ]
    Nmiss <- N0 - nrow(dat)
    if (Nmiss > 0){
        cat(paste("Removed ", Nmiss, " cases with only missing reponses\n",sep=""))
                }
    # groups
    if ( ! is.null( group) ){
        cat("Multiple groups are ignored!\n")
        cat("This option is only implemented in 'rasch.copula'\n")
#        groups <- unique( group )
#        G <- length( groups )
        G <- 1 ; group <- NULL
            } else { G <- 1 }
    GG <- G
    # data preparation (frequencies)
    dat10 <- dat00 <- dat
    dat10[ is.na( dat10) ] <- 9
    patt <- paste("P",dat10[,1],sep="")
    for (ii in 2:I){ patt <- paste( patt, dat10[,ii],sep="") }
    pattern <- data.frame( table(patt) )
    colnames(pattern) <- c("pattern", "freqwgt")
    # patttern in data
#    pattern.in.data <- data.frame(match( patt, pattern$pattern )
    pattern.in.data <- patt
    # calculate frequencies in multiple group case
    if (G > 1 ){
        for (gg in 1:G){
        # gg <- 1
        t1 <- table( patt[ group==gg  ] )
        pattern <- merge( pattern, t1, by.x=1, by.y=1, all=T )
                    }
        pattern[ is.na(pattern) ] <- 0
        colnames(pattern)[-c(1:2)]     <- paste("freqwgt", 1:G, sep="")
                }
    dat0 <- matrix( 0, nrow=nrow(pattern), I )
    for (ii in 1:I){
            dat0[,ii] <- as.numeric( substring( paste( pattern[,1] ), ii+1, ii+1 ) )
                }
    dat0[ dat0==9 ] <- NA
    colnames(dat0) <- colnames(dat)
    # define mu and sigma
    mu <- rep(0,G)
    sigma <- rep(1,G)
    if ( G > 1){
#            mu[2] <- 1 ; sigma[2] <- 1.2
                }
    # in multidimensional case
    if (D > 0){
        mu <- rep(0,D)
        rho <- rho.init
        sigma <- rho + (1-rho) *diag(1,D)
            }


    #------------------------
    dat <- dat0
    dat2 <- dat
    dat2.resp <- 1 * ( !is.na( dat2) )
    dat2[ is.na(dat2) ] <- 0
        # data preparation in case of item clusters
        dat2.ld <- NULL        # dat2.ld is NULL if there are no item clusters
            if ( is.null(delta)){
                    delta <- stats::runif( CC, .3, .7 )
                    delta <- ifelse( copula.type=="frank", 1.3, delta )
#                    delta <- ifelse( copula.type=="cook.johnson", 1, delta )
                            }    # initial estimate of delta
            if ( is.null(est.delta)){ est.delta <- 1:CC }
            dp.ld <- as.list( 1:CC )
            # item pattern
            for (cc in 1:CC){
    #            cc <- 1
                icl.cc <- which( itemcluster==cc )
                dp.ld.cc <- .calc.copula.itemcluster2( D=length(icl.cc) )
                dp.ld.cc$items <- icl.cc
                dp.ld.cc$N.items <- NCC <- length(icl.cc)
                dp.ld.cc$itemlabels <- colnames(dat)[icl.cc]
                # item selection for independent items
                m1 <- outer( rep(1,2^NCC), icl.cc)
                m2 <- dp.ld.cc$patt * m1   + ( 1 - dp.ld.cc$patt ) * ( m1 + I )
                m2 <- matrix( t(m2),  nrow=1, byrow=T )[1,]
                res <- list( "items"=m2 )
                res1 <- rep(NCC, 2^NCC )
                names(res1) <- rownames( dp.ld.cc$patt )
                res$N.Index1 <- res1
                dp.ld.cc$independent <- res
                # item selection for dependent items
                m2 <- ( 1 - dp.ld.cc$patt ) * ( m1 + I )
                m2 <- matrix( t(m2),  nrow=1, byrow=T )[1,]
                m2 <- c( m2[ m2> 0 ], 2*I + 1 )
                res <- list( "items"=m2 )
                res$N.Index1 <- rowSums( dp.ld.cc$patt==0 )
                res$N.Index1[ length(res$N.Index1) ] <- 1
                dp.ld.cc$dependent <- res
                dp.ld[[cc]] <- dp.ld.cc
                            }
            # create data frame with item response pattern
            dat2.ld <- matrix(0, nrow(dat2), CC )
            dat3.ld <- as.list( 1:CC )
            for (cc in 1:CC){
                # cc <- 1
                dp.cc <- dp.ld[[cc]]
                dat2.cc <- dat2[, dp.cc$items ]
                l1 <- "P"
                for ( vv in seq(1,ncol(dat2.cc))){
                    l1 <- paste( l1, dat2.cc[,vv], sep="")
                        }
                dat2.ld[, cc ] <- match( l1, rownames( dp.cc$patt ) )
                dat2.ld[ rowSums( dat2.resp[, dp.cc$items ]    ) < length(dp.cc$items), cc ] <- NA
                NRR <- nrow( dp.cc$patt )
                dat3.ld.cc <- sapply( seq( 1, NRR), FUN=function(aa){ 1*(dat2.ld[,cc]==aa ) } )
                dat3.ld.cc[ is.na(dat3.ld.cc) ] <- 0
                dat3.ld[[cc]] <- dat3.ld.cc
                            }
                # response indicator
                dat2.ld.resp <- 1 - is.na( dat2.ld )
                # set missings in dat2 to some arbitrary category
                dat2.ld[ is.na( dat2.ld ) ] <- 1
                # create dat2 data sets for local independence items
                itemcluster0 <- ind2 <- which( itemcluster==0 )
                bdat2.li.resp <- dat2.li.resp <- bdat2.li <- dat2.li <- NULL
                if ( length(ind2) > 0 ){
                        dat2.li <- dat2[, ind2, drop=FALSE]
                        dat2.li.resp <- dat2.resp[, ind2, drop=FALSE]
                        bdat2.li <- cbind( dat2.li, 1 - dat2.li )
                        bdat2.li.resp <- cbind( dat2.li.resp, dat2.li.resp )
                                        }
        # descriptives itemcluster
        Ncat.ld <- max( unlist( lapply( dp.ld, FUN=function(ll){ nrow(ll$patt) } ) ))

        # design table for estimating item difficulties
        b.design <- NULL
        if (length(itemcluster0) > 0){
            b.design <- data.frame( "itemcluster"=0,
                    "item"=itemcluster0, "b.indexgroup"=1 )
                                    }
        for (cc in 1:CC){
        #    cc <- 1
            g.cc <- ( dp.ld[[cc]] )$items
            bb <- data.frame("itemcluster"=cc,
                        "item"=g.cc, "b.indexgroup"=seq( 1, length(g.cc) ) )
            b.design <- rbind( b.design, bb )
                    }
        b.design <- b.design[ order( b.design$item ), ]
        b.design$est.b <- est.b
        t1 <- table( setdiff( est.b, 0 ) )
        if ( max(t1) > 1 ){    b.design$b.indexgroup <- 1:I }
        if ( sum( est.b==0 ) > 0 ){
                b.design[ est.b==0, "b.indexgroup"  ] <- b.design
                            }
        b1 <- setdiff( 1:max(b.design$b.indexgroup),
                        setdiff( b.design$b.indexgroup, 0 ) )
        if ( length(b1) > 0 ){     b.design$b.indexgroup <- 1:I }

    #########################################################################
    #--------------------------------------------------
    # initial estimate of item difficulty
#    b <- rasch.pairwise( dat, progress=FALSE)$item$itemdiff
    I <- ncol(dat2)
    if ( is.null( b.init) ){
        b <- - qlogis( ( colMeans( dat00, na.rm=T ) + .005 ) / 1.01 )
                } else { b <- b.init }
    # initial estimate of (mean) item discrimination
    if ( is.null(a.init) ){ a <- rep( 1, I ) } else { a <- a.init }
    # density weights
    wgt.theta <- sirt_dmvnorm_discrete( theta.k, mean=mu, sigma=sigma)

    if ( G > 1){
        wgt.theta <- matrix(0, length(theta.k), G )
        for ( gg in 1:G){
            wgt.theta[,gg] <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg] )
            wgt.theta[,gg] <- wgt.theta[,gg] / sum( wgt.theta[,gg] )
                        }
                }
    iter <- 0
    # preliminaries
    M2 <- outer( rep(1,nrow(pattern)), wgt.theta )
    # define Qmatrix
    Qmatrix <- matrix( 0, I, D )

    Qmatrix[ cbind(1:I, dims ) ] <- 1
    # define mu.fixed
    mu.fixed <- cbind( 1:D, rep(0,D) )
    # maximum increments
    hstep_b <- .6
    hstep_delta <- .2



    #************************************************************************
    #************************************************************************
    # BEGIN MARGINAL MAXIMUM LIKELIHOOD ESTIMATION
    dev <- 1 ; absdev.change <- par.change <- dev.change <- 1000
    res.posterior <- NULL
    maxalphachange <- 0
    while ( ( ( absdev.change > dev.crit | dev.change > glob.conv |
                    par.change > conv1 | maxalphachange > alpha.conv ) & iter < mmliter ) ){


# zz0 <- Sys.time()
    if (progress){
        cat( paste(rep("-", 70), collapse=""), "\n")
        k1 <- floor( log10(iter+1) )
        x1 <- "        |"
        x1 <- substring( x1, k1+1 )
        s1c <- Sys.time()
        cat( paste( paste( "MML EM Iter.", iter + 1 ), x1, paste( rep( "*", 10  ), collapse=""), "|  ",
                        s1c, "  ",
                        if ( iter > 0 ){ paste( round(difftime(s1c,s1b, units='secs' ),4), "secs" ) },
                        "\n",sep="") ) #
                        }
        s1b <- Sys.time()
        h <- numdiff.parm
        dev0 <- dev
        #************************************
        # estimation of b parameters
        b0 <- b
        # identify different b parameter groups
#        bG <- setdiff( unique( est.b ), 0 )
        bG <- setdiff( unique( b.design$b.indexgroup ), 0 )
        prbar <- seq( 1, 10, len=length(bG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        if (progress){ cat(" Estimation of b:     |")         }
        for (bb in bG){
#vv0 <- Sys.time()
            est.bb <- 1 * ( b.design$b.indexgroup==bb )
            b.design.bb <- b.design[ b.design$b.indexgroup==bb, ]
            if (bb==1 ){
                rescop <- .ll.rasch.copula320( theta.k, b0, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
                            wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type,
                            Ncat.ld    , Qmatrix=Qmatrix )
                res.posterior <- rescop
                            }
            # is this really necessary?
#            wgt.theta <- rescop$pik

            rest1 <- .update.ll.rasch.copula321( theta.k, b0 + h*est.bb, alpha1, alpha2,
                a, dat2.li, itemcluster0,
                    CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                    bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type,
                    Qmatrix=Qmatrix)
## continue here the code rewriting!!!


            rest2 <- .update.ll.rasch.copula321( theta.k, b0 - h*est.bb, alpha1, alpha2,
                    a, dat2.li, itemcluster0,
                    CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                    bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type,
                    Qmatrix=Qmatrix)
            ll0 <- ll1 <- ll2 <- rep(0,I)
            # numerical derivatives independent items
            if ( rescop$calc.ind ){
            for (jj in 1:(length(itemcluster0) ) ){
                ll0[ itemcluster0[jj] ] <- sum( rescop$rjk0.1[,jj] * log( rescop$pjk.theta.k0[,jj] ) +
                                rescop$rjk0.0[,jj] * log( 1-  rescop$pjk.theta.k0[,jj] ) )
                ll1[ itemcluster0[jj] ] <- sum( rescop$rjk0.1[,jj] * log( rest1$pjk.theta.k0[,jj] ) +
                                rescop$rjk0.0[,jj] * log( 1-  rest1$pjk.theta.k0[,jj] ) )
                ll2[ itemcluster0[jj] ] <- sum( rescop$rjk0.1[,jj] * log( rest2$pjk.theta.k0[,jj] ) +
                                rescop$rjk0.0[,jj] * log( 1-  rest2$pjk.theta.k0[,jj] ) )
                                        } }
            itemcluster1 <- b.design.bb[ b.design.bb$itemcluster > 0, "item"]
            rjkCC <- rescop$rjkCC
            for (jj in 1:(length(itemcluster1) ) ){
                    cc <- b.design.bb[ b.design.bb$item==itemcluster1[jj], "itemcluster" ]
                    ll0[itemcluster1[jj]] <- sum( rjkCC[[cc]] * log( rescop$pjk.theta.kCC[[cc]] ) )
                    ll1[itemcluster1[jj]] <- sum( rjkCC[[cc]] * log( rest1$pjk.theta.kCC[[cc]] ) )
                    ll2[itemcluster1[jj]] <- sum( rjkCC[[cc]] * log( rest2$pjk.theta.kCC[[cc]] ) )
                                }
            a1 <-stats::aggregate( cbind( ll0, ll1, ll2 ), list(est.b), sum, na.rm=TRUE)
            ll0 <- a1[,2]
            ll1 <- a1[,3]
            ll2 <- a1[,4]
            b.change <- nr.numdiff( ll0=ll0, ll1=ll1, ll2=ll2, h=h )
#            hstep <- .5^( log(iter) )
            if (bb==bG[1] ){
                hstep_b <- hstep <- hstep_b * ( 1 / increment.factor )
                            }
            b.change <- ifelse( abs( b.change ) > hstep, hstep*sign(b.change), b.change )
            b.change <- b.change[ match( est.b, a1[,1] ) ]
            b <- b + b.change
            if (progress){
                cat( paste( rep( "-", prbar[bb]), collapse="") )
                flush.console()
                        }
# cat("end b item") ; vv1 <- Sys.time(); print(vv1-zz0) ; vv0 <- vv1
                    }
        a1b <- max( abs( b - b0 ) )
        if (progress){
            cat("|     max. parm. change", round( a1b, 5),"\n")
                    }

        wm1 <- sum( theta.k * rescop$pik )
        wsd <- sqrt( sum( ( theta.k - wm1 )^2 * rescop$pik ) )
# cat("end b") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1
        #******************************************************************************
        # estimation of a parameters
        a0 <- a
        # identify different a parameter groups
        aG <- setdiff( unique( est.a ), 0 )
        prbar <- seq( 1, 10, len=length(aG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        if (progress){ cat(" Estimation of a:     |") }
        for (aa in aG){
            est.aa <- 1 * (est.a==aa )
            rescop <- .ll.rasch.copula320( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp,
                            delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld,
                            Qmatrix=Qmatrix)
            ll0 <- rescop$ll
            ll1 <- .update.ll.rasch.copula320( theta.k, b, alpha1, alpha2, a + h*est.aa,
                        dat2.li, itemcluster0,
                        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                        bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type,
                        Qmatrix=Qmatrix)$ll
            ll2 <- .update.ll.rasch.copula320( theta.k, b, alpha1, alpha2, a - h*est.aa,
                        dat2.li, itemcluster0, CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,    bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type, Qmatrix=Qmatrix)$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
             if ( abs(d2) < 10^(-20) ){ d2 <- 10^20 }
            a.change <- - d1 / d2
            a.change <- ifelse( abs( a.change ) > .3, .3*sign(a.change), a.change )
            a.change <- a.change * est.aa
            a <- a + a.change
            a[ a < 0 ] <- .01
#            cat( aa, " ") ;
        if (progress){
                cat( paste( rep( "-", prbar[aa]), collapse="") )
                flush.console()

        if ( length(aG) < 2 ){ cat( paste( rep( "-", 10 - length(aG) ), collapse="") ) }
                        }
                    }
        a1a <- max( abs( a - a0 ) )
        if (progress){
            cat("|     max. parm. change", round( a1a, 5),"\n")
                }
# cat("end a") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1

        #******************************************************************************
        # estimation of delta parameters
        delta0 <- delta
        dG <- setdiff( unique( est.delta ), 0 )
        prbar <- seq( 1, 10, len=length(dG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        if (progress){ cat(" Estimation of delta: |") }
        if ( length(dG)==0 ){ cat( paste(rep("-",10),collapse="") ) }
        if ( length(dG) > 0 ){
        # identify different a parameter groups
            est.cc <- 1 # * ( est.delta==cc )
            rescop <- .ll.rasch.copula320( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                                CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp,
                                delta, wgt.theta, I,
                                bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld,
                            Qmatrix=Qmatrix        )
#            ll0 <- rescop$ll
            rest1 <- .update.ll.rasch.copula321( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta + h*est.cc, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type,
                            Qmatrix=Qmatrix )
            rest2 <- .update.ll.rasch.copula321( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta - h*est.cc, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type,
                            Qmatrix=Qmatrix )
            ll0 <- ll1 <- ll2 <- rep(0,length(dG))
            rjkCC <- rescop$rjkCC
            for (cc in 1:CC){
                    ll0[cc] <- sum( rjkCC[[cc]] * log( rescop$pjk.theta.kCC[[cc]] ) )
                    ll1[cc] <- sum( rjkCC[[cc]] * log( rest1$pjk.theta.kCC[[cc]] ) )
                    ll2[cc] <- sum( rjkCC[[cc]] * log( rest2$pjk.theta.kCC[[cc]] ) )
                                }
            a1 <- stats::aggregate( cbind( ll0, ll1, ll2 ), list(est.delta), sum, na.rm=T)
            ll0 <- a1[,2]
            ll1 <- a1[,3]
            ll2 <- a1[,4]
            delta.change <- nr.numdiff( ll0=ll0, ll1=ll1, ll2=ll2, h=h )

            ct <- sapply( dG, FUN=function(dd){
                    ( copula.type[ est.delta==dd ] )[1] } )
            maxstep <- ifelse( copula.type=="bound.mixt", .2, .9 )
            # hstep <- maxstep^( log( 2+iter))
            hstep_delta <- hstep <- hstep_delta * ( 1 / increment.factor )

            delta.change <- ifelse( abs( delta.change ) > hstep,
                            hstep*sign(delta.change), delta.change )
            delta.change <- delta.change[ match( est.delta, a1[,1] ) ]
            delta <- delta + delta.change
            delta[ delta <=0 ] <- 2*numdiff.parm
            delta <- ifelse( copula.type=="bound.mixt" & ( delta > 1 ),
                        1 - 2 * numdiff.parm, delta )
            if (progress){
                cat( paste( rep( "-", 10 ), collapse="") )
                flush.console()
                        }
                            }
        a1d <- max( abs( delta - delta0 ) )
        if (progress){ cat("|     max. parm. change", round( a1d, 5),"\n") }
# cat("end delta") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1
        #******************************************************************************
        # estimation of alpha parameters
        alpha10 <- alpha1
        alpha20 <- alpha2
        prbar <- 5
        if (progress){ cat(" Estimation of alpha: |")         }
        # alpha1
        if (est.alpha){
            rescop <- .ll.rasch.copula320( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld,
                            Qmatrix=Qmatrix )
            ll0 <- rescop$ll
            ll1 <- .update.ll.rasch.copula320( theta.k, b, alpha1 + h, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type,
                            Qmatrix=Qmatrix )$ll
            ll2 <- .update.ll.rasch.copula320( theta.k, b, alpha1 - h, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG, copula.type,
                            Qmatrix=Qmatrix )$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            alpha.change <- - d1 / d2
            a1k1 <- alpha.change <- ifelse( abs( alpha.change ) > .1, .1*sign(alpha.change), alpha.change )
            alpha1 <- alpha1 + alpha.change
            }
            if (progress){
                cat( paste( rep( "-", prbar), collapse="") )
                flush.console()
                        }
        # alpha2
        if (est.alpha){
            rescop <- .ll.rasch.copula320( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld,
                            Qmatrix=Qmatrix )
            ll0 <- rescop$ll
            ll1 <- .update.ll.rasch.copula320( theta.k, b, alpha1, alpha2+h, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster,pattern, GG, copula.type,
                            Qmatrix=Qmatrix )$ll
            ll2 <- .update.ll.rasch.copula320( theta.k, b, alpha1, alpha2 -h, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster,pattern, GG, copula.type,
                            Qmatrix=Qmatrix )$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            alpha.change <- - d1 / d2
            a1k2 <- alpha.change <- ifelse( abs( alpha.change ) > .1, .1*sign(alpha.change), alpha.change )
            alpha2 <- alpha2 + alpha.change
            }
            if (progress){
                cat( paste( rep( "-", prbar), collapse="") )
                flush.console()
                        }
        a1k <- max( abs( c( alpha1 - alpha10, alpha2 - alpha20 )) )
        if (progress){ cat("|     max. parm. change", round( a1k, 5),"\n")    }
        #******************************************************************************
        # estimation of mu parameters

        a1m <- 0
        if (G>1){
        mu0 <- mu
        # identify different a parameter groups
        muG <- 1:(GG-1)
        prbar <- seq( 1, 10, len=length(muG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        if (progress){ cat(" Estimation of mu:    |") }
        for (gg in 2:G){
#            est.aa <- est.a * (est.a==aa )
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld)
            ll0 <- rescop$ll
            # mu + h
            w1 <- wgt.theta
            w2 <- stats::dnorm( theta.k, mean=mu[gg] + h, sd=sigma[gg] )
            w1[,gg] <- w2 / sum(w2)
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
                            wgt.theta=w1, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld)
            ll1 <- rescop$ll
            # mu - h
            w1 <- wgt.theta
            w2 <- stats::dnorm( theta.k, mean=mu[gg] - h, sd=sigma[gg] )
            w1[,gg] <- w2 / sum(w2)
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
                            wgt.theta=w1, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld )
            ll2 <- rescop$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            mu.change <- - d1 / d2
            mu.change <- ifelse( abs( mu.change ) > .3, .3*sign(mu.change), mu.change )
            mu.change <- mu.change * ( ( 1:G )==gg )
            mu <- mu + mu.change
            w2 <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg] )
            wgt.theta[,gg] <- w2 / sum(w2)
            if (progress){
                cat( paste( rep( "-", prbar[gg-1]), collapse="") )
                flush.console()
                        }
                            }
        if ( length(muG) < 2 ){ cat( paste( rep( "-", 10 - length(muG) ), collapse="") ) }
        a1m <- max( abs( mu - mu0 ) )
        if (progress){ cat("|     max. parm. change", round( a1m, 5),"\n") }
                }            # end mu
        ######################################################################
        #******************************************************************************
        # estimation of sigma parameters
        a1s <- 0
        if (G>1){
        sigma0 <- sigma
        # identify different a parameter groups
#        h <- 10 * numdiff.parm
        sigmaG <- seq(1,GG-1)
        prbar <- seq( 1, 10, len=length(sigmaG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        if (progress){ cat(" Estimation of sigma: |") }
        for (gg in 2L:G){
#            est.aa <- est.a * (est.a==aa )
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG,copula.type,Ncat.ld )
            ll0 <- rescop$ll
            # sigma + h
            w1 <- wgt.theta
            w2 <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg] +h)
            w1[,gg] <- w2 / sum(w2)
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
                            wgt.theta=w1, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type,Ncat.ld)
            ll1 <- rescop$ll
            # sigma - h
            w1 <- wgt.theta
            w2 <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg]-h )
            w1[,gg] <- w2 / sum(w2)
            rescop <- .ll.rasch.copula20( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
                            wgt.theta=w1, I,
                            bdat2.li, bdat2.li.resp, pattern, GG, copula.type,Ncat.ld)
            ll2 <- rescop$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            sigma.change <- - d1 / d2
            sigma.change <- ifelse( abs( sigma.change ) > .3, .3*sign(sigma.change), sigma.change )
            sigma.change <- sigma.change * ( ( 1:G )==gg )
            sigma <- sigma + sigma.change
            w2 <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg] )
            wgt.theta[,gg] <- w2 / sum(w2)
#            cat( aa, " ") ;
        if (progress){
                cat( paste( rep( "-", prbar[gg-1]), collapse="") )
                flush.console()
                        }
                            }
        if ( length(sigmaG) < 2 ){ cat( paste( rep( "-", 10 - length(sigmaG) ), collapse="") ) }
        a1s <- max( abs( sigma - sigma0 ) )
        if (progress){ cat("|     max. parm. change", round( a1s, 5),"\n") }
                }            # end sigma
        ######################################################################
# cat("end rest") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1
        # estimate covariance matrix

        res0 <- rasch.copula3.covariance( f.qk.yi=rescop$post, Sigma=sigma, theta.k, N,
                    mu.fixed=mu.fixed, variance.fixed=NULL, D=D, est.corr=TRUE, irtmodel=NULL,
                    freqwgt=pattern$freqwgt)
        mu <- res0$mu
        sigma <- res0$Sigma
        sigma0 <- res0$Sigma0
        wgt.theta <- res0$pi.k[,1]
# cat("end covariance") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1


#cat("other pars") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1
        iter <- iter + 1
#        thetawidth <- diff( theta.k )[1]
        #**********************
        # deviance rasch.mml
        # ll[gg] <- sum( dat1[,2] * log( rowSums( f.yi.qk[,] *
        #            outer( rep(1,nrow(f.yi.qk[,])), pi.k ) ) ) )

        M2 <- outer( rep(1,nrow(pattern)), wgt.theta )

        dev <- - 2 * sum( pattern$freqwgt * log( rowSums(rescop$post.unnorm * M2 ) ) )
        #**********************
        # deviance tam
        # deviance <- - 2 * sum( pweights * log( res.hwt$rfx * thetawidth ) )
#        dev <- - 2 * sum( pattern$freqwgt * log( rowSums(rescop$post.unnorm) * thetawidth ) )
#        dev <- - 2 * sum( pattern$freqwgt * log( rowSums(rescop$post.unnorm * M2 ) ))
        dev.change <- abs( ( dev - dev0)/ dev0 )
        absdev.change <- abs( dev- dev0 )
        par.change <- max( a1a, a1b, a1d, a1k, a1m, a1s)
        if (progress){
            cat( "Deviance=",   round( dev, 5 ),
                    " | Deviance change=", - round( dev- dev0, 4 ),
                    "| max. parm. change=",  round( par.change, 6 ),  " \n"   )
            if ( ( dev > dev0 ) & ( iter > 4 ) ){ cat("   Deviance has increased! Convergence Problems?\n") }
    #cat("rest") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1
            flush.console()
                    }
            }
    # end MML iterations
    #**********************************************************************************
    # Standard error estimation (This is a TO DO!)
    iterend <- iter

    ################################################
    # evaluation of posterior distribution
    # use functions from smirt

    post <- rescop$post
    EAP.Rel <- rep(0,D)
    names(EAP.Rel) <- paste0("Dim",1:D)
    for (dd in 1:D){
        #    dd <- 1
        M3 <- outer( rep(1,nrow(post)), theta.k[,dd] )
        pattern[, paste0("EAP.Dim",dd)] <- rowSums( M3 * post )
        pattern[, paste0("PostVar.Dim",dd)] <-
            rowSums( M3^2 * post ) -  pattern[, paste0("EAP.Dim",dd)]^2
        M.EAP <- weighted.mean( pattern[, paste0("EAP.Dim",dd)], pattern$freqwgt )
        Var.EAP <- sum(( ( pattern[, paste0("EAP.Dim",dd)] - M.EAP )^2 *
                         pattern$freqwgt )) / ( sum( pattern$freqwgt ) )
        MVar.EAP <- weighted.mean( pattern[, paste0("PostVar.Dim",dd)], pattern$freqwgt )
        EAP.Rel[dd] <- Var.EAP / ( Var.EAP + MVar.EAP )
                    }

    #********


    #********************************************************
    # information criteria
        # calculations for information criteria
        ic <- list( "deviance"=dev, "n"=nrow(dat00) )
        # number of parameters to be estimated
        # these formulas hold when assuming normal distributions
        bG <- setdiff( unique( est.b ), 0 )
        aG <- setdiff( unique( est.a ), 0 )
        dG <- setdiff( unique( est.delta ), 0 )
        ic[[ "np" ]] <- length(bG) + length(aG) + length(dG) + 2*est.alpha + D*(D-1)/2
        # AIC
        ic$AIC <- dev + 2*ic$np
        # BIC
        ic$BIC <- dev + ( log(ic$n) )*ic$np
        # CAIC
        ic$CAIC <- dev + ( log(ic$n) + 1 )*ic$np
        # corrected AIC
        ic$AICc <- ic$AIC + 2*ic$np * ( ic$np + 1 ) / ( ic$n - ic$np - 1 )
    #**********************************************************************************
    # results item parameters
    item <- data.frame( "item"=colnames(dat),
                "N"=colSums(!is.na(dat00)),
                "p"=colMeans( dat00, na.rm=T ),
                "b"=b, "est.b"=est.b, "a"=a, "est.a"=est.a )
    item$thresh <- item$a * item$b
    # add results dependency parameter for item clusters
    item$itemcluster <- itemcluster
    item$delta <- 0
    if (progress){
        cat("--------------------------------------------------------------------------------\n")
                    }
    for (cc in 1:CC){
        # cc <- 1
        dcc <- dp.ld[[cc]]
        item[ dcc$items, "delta"] <- delta[cc]
                }
    item$est.delta <- 0
    i1 <- which( item$itemcluster > 0 )
    icl <- item$itemcluster[ i1 ]
    item$est.delta[ i1 ] <- est.delta[ icl ]
    # print item summary
    if (progress){
        cat("Parameter summary\n")
        sirt_print_helper( item, digits=3)        # print item statistics
        # dependency parameter
        cat("\nDependency parameters\n")
                }
    summary.delta <- data.frame( "cluster"=1:CC, "delta"=delta,
                "est.delta"=est.delta, "copula.type"=copula.type     )
    summary.delta$items <- sapply( 1:CC, FUN=function(cc){
        paste( colnames(dat)[ itemcluster==cc ], collapse="-" )
                } )
            s2 <- Sys.time()
    if (progress){
        sirt_print_helper(summary.delta, digits=3)
        cat(paste("\nEAP Reliability:\n "))
        print( round( EAP.Rel,3))
        cat("\n")
        cat("Generalized logistic link function\n")
        cat("alpha1=",round(alpha1,3)," alpha2=", round(alpha2,3), " \n\n")
            # computational time
           cat("-----------------------------------------------------------------\n")
           cat("Start:", paste( s1), "\n")
           cat("End:", paste(s2), "\n")
           cat("Difference:", print(s2 -s1), "\n")
           cat("-----------------------------------------------------------------\n")
                }
    datalist <- list( pattern.in.data=pattern.in.data, dat0=dat0,
                    dat2=dat2, dat2.resp=dat2.resp, dat2.li=dat2.li,
                    dat2.ld=dat2.ld, dat2.li.resp=dat2.li.resp,
                    dat2.ld.resp=dat2.ld.resp, dp.ld=dp.ld, CC=CC,
                    bdat2.li=bdat2.li, bdat2.li.resp=bdat2.li.resp,
                    itemcluster0=itemcluster0, dat3.ld=dat3.ld
                            )
    # collect results
    v1 <- datalist$pattern.in.data
    patternindex <- match( v1, pattern$pattern )
    names(mu) <- rownames(sigma) <- colnames(sigma) <- paste0("Dim",1:D)
    # person parameters
    person <- pattern[ patternindex, ]
    person$freqwgt <- NULL

    res <- list( "N.itemclusters"=CC, "item"=item, "iter"=iterend, "dev"=dev,
                    "delta"=delta, "b"=b, "a"=a, "mu"=mu, "sigma"=sigma,
                    "alpha1"=alpha1, "alpha2"=alpha2, "ic"=ic, "theta.k"=theta.k,
                    "pi.k"=wgt.theta, "deviance"=dev,
                    "pattern"=pattern    , "person"=person,
                    "datalist"=datalist    , "EAP.Rel"=EAP.Rel    ,
                    "copula.type"=copula.type    , "summary.delta"=summary.delta,
                    "f.qk.yi"=(res.posterior$post)[ patternindex,],
                    "f.yi.qk"=(res.posterior$post.unnorm)[ patternindex,],
                    "Qmatrix"=Qmatrix, "D"=D,
                    "s2"=s2, "s1"=s1, CALL=CALL
                                )
    class(res) <- "rasch.copula3"
    return(res)
        }
#----------------------------------------------------------------------------------

## File Name: summary.R2noharm.R
## File Version: 0.14

#..........................................................
# R2Noharm summary function

#----------------------------------------------------------
# summary function for R2noharm
summary.R2noharm <- function( object, logfile=NULL, ...){
    #**********************
    # global summary
    if ( ! is.null(logfile) ){ sink( paste0(logfile,".Rout"), split=TRUE ) }
    cat( paste( "-------------------------------------------------"), "\n" )
    cat( paste( "NOHARM4 ", object$systime, sep=""), "\n" )
    d <- utils::packageDescription("sirt")
    cat(paste(d$Package," ", d$Version," (",d$Date,")",sep=""))
    cat( paste( "\n-------------------------------------------------"), "\n\n" )
    #**********************
    # model type
    if (object$modtype==2){ cat( "Multidimensional Exploratory Factor Analysis\n\n") }
    if (object$modtype==3){ cat( "Multidimensional Confirmatory Factor Analysis\n\n") }
    if (object$modtype==4){ cat( "Unidimensional Confirmatory Factor Analysis\n\n") }
    #*********************
    # descriptives
    cat( paste( "Number of Observations: ", object$Nobs, sep=""), "\n" )
    cat( paste( "Number of Items       : ", object$Nitems, sep=""), "\n" )
    cat( paste( "Number of Dimensions  : ", object$dimensions, sep=""), "\n" )
    cat( paste( "Tanaka Index          : ", round(object$tanaka,object$display.fit), sep=""), "\n" )
    cat( paste( "RMSR                  : ", round(object$rmsr,object$display.fit), sep=""), "\n\n" )

    # chi square statistic
    if (object$modtype %in% 2:4){
        cat("Chi Square Statistic of Gessaroli & De Champlain (1996)\n\n")
        cat( paste( "Number of Estimated Parameters : ", object$Nestpars, sep=""), "\n" )
        cat( paste( "Chi2                           : ", round(object$chisquare,3), sep=""), "\n" )
        cat( paste( "Degrees of Freedom (df)        : ", object$df, sep=""), "\n" )
        cat( paste( "p(Chi2,df)                     : ", round(object$p.chisquare,3), sep=""), "\n" )
        cat( paste( "Chi2 / df                      : ", round(object$chisquare / object$df,3), sep=""), "\n" )
        cat( paste( "RMSEA                          : ", round(object$rmsea,3), sep=""), "\n\n" )
                        }
    if ( object$modtype %in% 2:4){
        #***
        # factor correlation
        cat( "Factor Covariance Matrix\n")
        print( object$factor.cor )
        if ( object$modtype %in% 3){
            cat( "\nFactor Correlation Matrix\n")
            print( round( stats::cov2cor(object$factor.cor), 3 ))
                                    }
                                }
    if ( object$modtype %in% 3:4){
        # item parameter
        l1 <- object$loadings.theta
        cat("\nItem Parameters - Latent Trait Model Parametrization\n",
                "Loadings, Constants, Guessing Parameters and Decriptives\n\n")
        m1 <- l1 %*% as.matrix(object$factor.cor) %*% t( l1 )
        v1 <- 1 + diag( m1 )
        l1 <- data.frame( l1, "final.constants"=object$final.constants, "guess"=object$guesses,
                    "item.variance"=round( v1, 3), "N"=diag(object$N.itempair), "p"=diag(object$pm) )
        print( l1 )
        res <- list( "itempars"=l1 )
        # factor analysis parametrization
        l1 <- object$loadings
        cat("\nItem Parameters - Common Factor Parametrization\n",
            "Loadings, Thresholds, Uniquenesses and Guessing Parameters\n\n")
        l1 <- data.frame( l1, "threshold"=object$thresholds, "uniqueness"=object$uniquenesses,
                        "guess"=object$guesses )
        print( l1 )
        res[["itempars.faparm" ]] <- l1
        #***
        # Lord's parametrization for unidimensional models
        if (object$modtype==4){
            cat("\nItem Parameters - Lord's Parametrization\n",
                "Difficulty and Discrimination\n\n")
            l1 <- data.frame( "difficulty"=object$difficulties,
                               "discrimination"=object$discriminations )
            print(l1)
        res[["itempars.lord" ]] <- l1
                    }
            }
    if ( object$modtype %in% 2){
        # item parameter
        l1 <- object$loadings.theta
        cat("\nItem Parameters - Promax Rotated Parameters\n",
                "Loadings, Constants, Guessing Parameters and Descriptives\n\n")
        l1 <- object$promax.theta
        l1 <- data.frame( l1, "final.constants"=object$final.constants,
                    "guess"=object$guesses, "N"=diag(object$N.itempair), "p"=diag(object$pm) )
        print( l1 )
        res <- list( "itempars.promax"=l1 )
                }
    if ( ! is.null(logfile) ){ sink() }
        invisible(res)
    }
#--------------------------------------------------------------------------------------

## File Name: fit.isop.R
## File Version: 2.15

############################################################
# Fit ISOP Model
fit.isop <- function( freq.correct, wgt, conv=.0001,
              maxit=100, progress=TRUE, calc.ll=TRUE){
    #****
    # initializations
    # monotone regression
    M1 <- as.matrix(freq.correct)
    I <- ncol(M1)
    PP <- nrow(M1)
    wgt <- as.matrix(wgt)
    # isotonic row regression
    res <- fit.isotonic.rows(X=M1, wgt=wgt )
    RX <- res$RX
    IR <- res$IR
    # isotonic column regression
    res <- fit.isotonic.cols(X=RX, wgt=wgt )
    CX <- res$CX
    IC <- res$IC
    X <- M1
    deviation <- 1
    iter <- 0
    if (progress){ cat("\n*******ISOP Model***********\n") }
    #****
    # ISOP algorithm
    while( ( iter < maxit) & ( deviation > conv ) ){
        Xold <- X
        # isotonic rows
        res <- fit.isotonic.rows(X=X-IC, wgt=wgt )
        RX <- res$RX
        IR <- res$IR
        # isotonic columns
        res <- fit.isotonic.cols(X=X-IR, wgt=wgt )
        CX <- res$CX
        IC <- res$IC
        # updated X
        X <- Xold - IR - IC
        # calculate deviation
        deviation <- sum( ( X - Xold )^2*wgt )
        iter <- iter + 1
        if (progress){
            cat( "Iteration", iter, "- Deviation=",  round( deviation, 6 ), "\n")
            utils::flush.console()
                    }
                    }  # end algorithm
    ###############################################
    RR <- nrow(freq.correct)
    CC <- ncol(freq.correct)
    if ( calc.ll ){
        CX[ CX > 1 ] <- 1
        CX[ CX < 0 ] <- 0
                }
    dfr0 <- data.frame( "stud.index"=rep(1:RR, CC),
                "item.index"=rep(1:CC,each=RR),
                "wgt"=matrix( as.matrix( wgt ), RR*CC, 1 ),
                "freq"=matrix( as.matrix(freq.correct ), RR*CC, 1 ),
                "freq.fitted"=matrix( as.matrix( CX ), RR*CC, 1 )
                        )
    dfr0 <- dfr0[ order( dfr0$stud.index * 1000 + dfr0$item.index ), ]
    # deviation criterion
    wgt1 <- ( wgt / colSums( wgt ) ) / ncol(wgt)
    fit <- sqrt( sum( ( M1-CX  )^2 * wgt1  ) )
    #****
    # calculate likelihood
    ll <- NULL
    if (calc.ll){
        ll <- list(
            "ll.ind"=.calc.ll.isop( freq.correct, wgt, irtfitted=freq.correct )
                        )
        ll$ll.isop <- .calc.ll.isop( freq.correct, wgt, irtfitted=CX )
        NW <- mean( colSums(wgt) )
        ll$llcase.ind <- ll$ll.ind /NW
        ll$llcase.isop <- ll$ll.isop /NW
        }
    # collect item and person scores
    item.sc <- ( seq( 0, I-1 ) + .5 ) / I
    person.sc <- ( seq( 0, PP-1 ) + .5 ) / PP
    # output
    res <- list( "fX"=CX, "ResX"=M1 - CX, "fit"=fit,
        "item.sc"=item.sc, "person.sc"=person.sc, "ll"=ll,
        "freq.fitted"=dfr0)
    return(res)
        }
############################################################
# calculate likelihood for saturated and fitted models
.calc.ll.isop <- function( freq.correct, wgt, irtfitted, eps=10^(-20) ){
    ll <- sum( wgt * freq.correct * log( irtfitted + eps ) +
            wgt * (1-freq.correct) * log( 1 - irtfitted + eps ) )
    return(ll)
        }
########################################################


#****************************************
# fit isotonic rows
fit.isotonic.rows <- function( X, wgt ){
    M2 <- X
    RR <- nrow(M2)
#    for (rr in 1:RR){
#        M2[rr,] <- monoreg(x=X[rr,], w=wgt[rr,] )$yf
#                }
#    RX <- M2
    RX <- monoreg.rowwise(X,wgt)
    IX <- X - RX
    res <- list( "X"=X, "RX"=RX, "IR"=IX )
    return(res)
        }

#****************************************
# fit isotonic columns
fit.isotonic.cols <- function( X, wgt ){
    M2 <- X
    RR <- ncol(M2)
#    for (rr in 1:RR){
#        M2[,rr] <- monoreg(x=X[,rr], w=wgt[,rr] )$yf
#                }
#    RX <- M2
    RX <- monoreg.colwise(X,wgt)
    IX <- X - RX
    res <- list( "X"=X, "CX"=RX, "IC"=IX )
    return(res)
        }
#******************************************

## File Name: mad_normalized.R
## File Version: 0.03

mad_normalized <- function(x)
{
    x <- stats::na.omit(x)
    res <- stats::median( abs( x - stats::median(x) ) )
    res <- res/0.6745
    return(res)
}

## File Name: rasch_jml_emp_discrim.R
## File Version: 0.08



# Function for calculating empirical discrimination
# slope estimation (WINSTEPS manual p. 300)
rasch_jml_emp_discrim <- function( theta, b, dat, dat.resp=1-is.na(dat.resp), freq )
{
    N <- length(theta)
    I <- length(b)
    pni <- .prob.rasch( theta=theta, b=b )
    bM <- matrix(b, nrow=N, ncol=I, byrow=TRUE)
    thetaM <- matrix(theta, nrow=N, ncol=I)
    tbdiff <- thetaM - bM
    tdf <- tbdiff * dat.resp * freq
    t1 <- colSums( ( dat - pni ) * tdf )
    t2 <- colSums( pni * ( 1 - pni ) * tbdiff * tdf)
    res <- 1 + t1/t2
    return(res)
}

# emp.discr <- rasch_jml_emp_discrim

## File Name: btm.R
## File Version: 1.541


#--- Bradley-Terry model in sirt
btm <- function( data, judge=NULL, ignore.ties=FALSE, fix.eta=NULL, fix.delta=NULL,
            fix.theta=NULL, maxiter=100, conv=.0001, eps=.3, wgt.ties=.5)
{
    s1 <- Sys.time()
    CALL <- match.call()
    admiss <- c(0,1,.5)
    est.delta <- TRUE
    est.eta <- TRUE
    delta <- -1
    eta <- 0
    center.theta <- TRUE
    if ( ! is.null( fix.eta ) ){
        eta <- fix.eta
        est.eta <- FALSE
    }

    if ( ignore.ties ){
        admiss <- admiss[c(1,2)]
        delta <- -99
        est.delta <- FALSE
    }
    data <- data[ ! is.na( data[,3] ), ]
    data <- data[ data[,3] %in% admiss, ]
    dat0 <- dat <- data

    # teams
    teams <- unique( sort( c( paste(dat[,1] ), paste(dat[,2] ) ) ) )
    teams1 <- as.numeric(paste(teams))

    if ( sum( is.na(teams1))==0 ){
        teams <- sort(teams1)
    }

    dat0[,1] <- match( paste( dat0[,1] ), teams )
    dat0[,2] <- match( paste( dat0[,2] ), teams )
    dfr <- data.frame( res1=1*( dat0[,3]==1 ), res0=1*( dat0[,3]==0 ),
                    resD=1*( dat0[,3]==1/2 ) )

    # calculate scores for each team
    TP <- length(teams)
    r1 <- rowsum( dfr, dat0[,1] )
    ind1 <- as.numeric(rownames(r1))
    r2 <- rowsum( dfr, dat0[,2] )
    ind2 <- as.numeric(rownames(r2))
    r2 <- r2[, c(2,1,3) ]
    r0 <- as.data.frame( matrix( 0, nrow=TP, ncol=3 ))
    r0[ ind1, ] <- r0[ ind1,] + r1
    r0[ ind2, ] <- r0[ ind2,] + r2
    r3 <- r0
    colnames(r3) <- colnames(dfr)
    score <- r3[,1]*1 + r3[,3]*wgt.ties
    maxscore <- rowSums(r3)
    score_delta <- sum(r1[,3])
    score_eta <- sum(r1[,1]+r1[,3]*wgt.ties)

    # epsilon adjustment
    raw <- score
    score <- eps + ( maxscore-2*eps)*score / maxscore
    propscore <- score / maxscore
    # initial ability for each team
    theta <- stats::qlogis( ( propscore + .1 ) / 1.2 )
    if (center.theta){
        theta <- theta - mean(theta)
    }
    # eliminate individuals with extreme scores
    elim_persons <- FALSE
    if ( sum( propscore %in% c(0,1) ) > 0 ){
        elim_persons <- TRUE
        elim_persons_index <- which( propscore %in% c(0,1) )
        theta_elim <- theta[ elim_persons_index ]
        # define matrix with sets probabilities to zero for
        # comparisons which are excluded
        indicator_elim <- dat0
        indicator_elim[,3] <- 0
        indicator_elim[ indicator_elim[,1] %in% elim_persons_index, 3 ] <- 1
        indicator_elim[ indicator_elim[,2] %in% elim_persons_index, 3 ] <- 1
    }

    some.fixed.theta <- FALSE
    if ( ! is.null( fix.theta) ){
        fix.theta.index <- match( names(fix.theta), teams )
        if ( sum( is.na( fix.theta.index ) ) > 0 ){
            stop( paste0( 'Cannot find all individuals with fixed values\n',
                    '  in \'fixed.theta\'\n') )
        }
        some.fixed.theta <- TRUE
        center.theta <- FALSE
    }

    # number of dyads
    ND <- nrow(dat0)
    max.change <- 1E5
    iter <- 0
    incrfac <- .98
    maxincr <- 1
    se.delta <- NA
    se.eta <- NA
    c0 <- Sys.time()

    while( ( iter < maxiter ) & ( max.change > conv) ) {

        theta0 <- theta
        delta0 <- delta
        eta0 <- eta

        M1 <- matrix(0, nrow=ND, ncol=3)
        M1[,1] <- theta[ dat0[,1] ] + eta
        M1[,2] <- theta[ dat0[,2] ]
        M1[,3] <- delta + ( theta[ dat0[,1] ] + theta[ dat0[,2] ] + eta ) * wgt.ties

        M1 <- exp(M1)
        M1 <- M1 / rowSums(M1)

        if ( elim_persons ){
            M1[ indicator_elim[,3]==1, ] <- 0
        }

        maxincr <- maxincr * incrfac

        #--- pre-calculations
        M13w <- M1[,3]*wgt.ties
        M11_M13_w <- M1[,1] + M13w
        wm1 <- wgt.ties - M1[,1] - M1[,3]*wgt.ties
        wm2 <- wgt.ties - M1[,2] - M1[,3]*wgt.ties

        #**** derivatives with respect to delta
        if ( est.delta ){
            d1 <- score_delta - sum( M1[,3] )
            d2 <- sum( M1[,3] * ( 1 - M1[,3] ) )
            incr <- d1 / d2
            incr <- btm_trim_increment(incr=incr, maxincr=maxincr )
            delta <- delta + incr
            se.delta <- sqrt( 1 / d2 )
        }

        #*** derivatives with respect to eta
        if ( est.eta ){
            d1 <- score_eta - sum( M11_M13_w)
            d2 <- sum( M1[,1] * ( 1 - M11_M13_w ) +    M13w*wm1)
            incr <-  d1 / d2
            incr <- btm_trim_increment(incr=incr, maxincr=maxincr )
            eta <- eta + incr
            se.eta <- sqrt( 1 / d2 )
        }

        #*** derivatives with respect to theta
        # first derivative
        fac1 <-  M1[,1] + M13w
        deriv_theta_i1 <- fac1
        fac2 <-  M1[,2] + M13w
        deriv_theta_i2 <- fac2
        # d1a <- ( rowsum( deriv_theta_i1, dat0[,1] )[,1] +
        #             rowsum( deriv_theta_i2, dat0[,2] )[,1] )
        h1 <- rowsum( deriv_theta_i1, dat0[,1] )
        h2 <- rowsum( deriv_theta_i2, dat0[,2] )
        d1a <- rep(0,TP)
        d1a[ ind1 ] <- d1a[ ind1 ] + h1
        d1a[ ind2 ] <- d1a[ ind2 ] + h2
        d1 <- score -  d1a
        # second derivative
        d2a <- M1[,1]*(1 - fac1) + M13w*wm1
        d2b <- M1[,2]*(1 - fac2) + M13w*wm2
        h1 <- rowsum( d2a, dat0[,1] )
        h2 <- rowsum( d2b, dat0[,2] )
        d2 <- rep(1E-20,TP)
        d2[ ind1 ] <- d2[ ind1 ] + h1
        d2[ ind2 ] <- d2[ ind2 ] + h2
        incr <- d1/d2
        incr <- btm_trim_increment(incr=incr, maxincr=maxincr )
        theta <- theta + incr

        theta2 <- theta
        se.theta <- sqrt( 1 / d2 )
        if ( elim_persons ){
            theta[ elim_persons_index ] <- theta_elim
            se.theta[ elim_persons_index ] <- NA
            theta2[ abs(theta) %in% Inf ] <- NA
        }

        if (center.theta){
            theta <- theta - mean(theta2, na.rm=TRUE)
        }

        if (some.fixed.theta){
            theta[ fix.theta.index ] <- fix.theta
            se.theta[ fix.theta.index ] <- NA
        }

        # assess convergence
        iter <- iter + 1
        theta_ch <- abs( theta - theta0 )

        if (elim_persons){
            theta_ch <- ifelse( theta_ch==Inf, NA, theta_ch )
        }

        theta.change <- max( theta_ch, na.rm=TRUE)
        delta.change <- max( abs( delta - delta0 ))
        eta.change <- max( abs( eta - eta0 ))
        max.change <- max( c(theta.change,delta.change, eta.change) )

        cat( paste0('**** Iteration ', iter,
                        ' | Maximum parameter change=', round(max.change, 7), '\n') )
        utils::flush.console()

    }  #---- end algorithm
    time_alg <- Sys.time() - c0

    # arrange output
    pars <- data.frame(parlabel=c('Ties', 'Home'), par=c('delta', 'eta') )
    pars$est <- c( delta, eta )
    pars$se <- c( se.delta, se.eta )

    # estimated individual effect
    effects <- data.frame( individual=teams, id=seq( 1, length(teams) ) )
    effects$Ntot <- rowSums(r3)
    effects$N1 <- r3[,1]
    effects$ND <- r3[,3]
    effects$N0 <- r3[,2]
    effects$raw <- raw
    effects$score <- score
    effects$propscore <- propscore
    effects$theta <- theta
    effects$se.theta <- se.theta
    # effects <- effects[ order( effects$theta, decreasing=TRUE), ]
    rownames(effects) <- NULL

    # summary of effects parameters
    theta2 <- theta
    theta2[ abs(theta) %in% Inf ] <- NA
    summary.effects <- data.frame( M=mean(theta2,na.rm=TRUE),
                    median=stats::median(theta), SD=stats::sd(theta2,na.rm=TRUE),
                    min=min(theta), max=max(theta) )

    # probabilities
    probs <- M1
    colnames(probs) <- c('p1', 'p0', 'pD')

    # log-likelihood
    NObs <- nrow(dat0)
    dat0$result <- dat0[,3]
    ll <- rep(0,NObs)
    ll <- ll+log(probs[,1])*(dat0$result==1)
    ll <- ll+log(probs[,2])*(dat0$result==0)
    ll <- ll+log(probs[,3])*(dat0$result==.5)
    ll <- sum(ll)

    # fit statistics
    res0 <- btm_fit_statistics( probs=probs, dat0=dat0, ind1=ind1, ind2=ind2,
                    TP=TP, judge=judge, wgt.ties=wgt.ties )
    effects$outfit <- res0$outfit
    effects$infit <- res0$infit
    multiple_judges <- res0$multiple_judges
    fit_judges <- res0$fit_judges
    residuals <- res0$residuals

    # MLE reliability
    v2 <- mean(effects$se.theta^2)
    v0 <- stats::var(effects$theta)
    mle.rel <- 1 - v2 / v0
    sep.rel <- sqrt( v0 / v2 )
    #--- output list
    effects <- effects[ order(effects$propscore, decreasing=TRUE), ]
    res <- list( effects=effects, pars=pars, summary.effects=summary.effects,
                    mle.rel=mle.rel, sepG=sep.rel, probs=probs, data=dat0,
                    multiple_judges=multiple_judges, fit_judges=fit_judges,
                    residuals=residuals, eps=eps, ignore.ties=ignore.ties,
                    wgt.ties=wgt.ties, time_alg=time_alg, ll=ll, dat=dat)
    res$CALL <- CALL
    res$iter <- iter
    ic <- list( n=length(teams), D=nrow(dat0) )
    res$ic <- ic
    s2 <- Sys.time()
    res$s1 <- s1
    res$s2 <- s2
    class(res) <- 'btm'
    return(res)
}

## File Name: dmlavaan_sandwich_formula.R
## File Version: 0.06

dmlavaan_sandwich_formula <- function(A, B, parnames=NULL)
{
    requireNamespace('MASS')
    B1 <- MASS::ginv(X=B)
    V <- B1 %*% A %*% B1
    if (!is.null(parnames)){
        rownames(V) <- colnames(V) <- parnames
    }
    se_sw <- sqrt( diag(V) )
    #-- output
    res <- list(V=V, se_sw=se_sw)
    return(res)
}

## File Name: smirt_alg_noncomp.R
## File Version: 2.421

############################################
# probability in noncompensatory model
calcprob.noncomp <- function (a,b,Q,thetak,cc,dd)
{
    res <- SMIRT_CALCPROB_NONCOMP( a,b,Q,thetak,cc,dd)
    return(res)
}

#--- calculation of posterior distribution
calcpost <- function (dat2, dat2resp, probs, dat2ind, pik, K)
{
    res0 <- SMIRT_CALCPOST( dat2, dat2resp, probs, dat2ind, PIK=pik, K )
    res <- list(fyiqk=res0$fyiqk, f.qk.yi=res0$fqkyi, pi.k=res0$pik,
                    n.ik=res0$nik, N.ik=res0$NIK )
    return(res)
}



######################################
# estimation of covariance
.smirt.est.covariance <- function( f.qk.yi, Sigma, theta.k, N,
        mu.fixed, variance.fixed, D, est.corr, irtmodel ){
        Sigma.cov <- Sigma
        delta.theta <- 1
        hwt <- f.qk.yi
#        if (qmcnodes){
#            hwt <- hwt / nrow(theta.k)
#            hwt <- hwt / rowSums( hwt )
#                }
        thetabar <- hwt%*%theta.k
        # calculation of mu
        mu <- colSums( thetabar ) / N
        if ( ! is.null(mu.fixed ) ){
              if (is.matrix(mu.fixed) ){
                mu0 <- mu
                mu[ mu.fixed[,1] ] <- mu.fixed[,2]
                                    }
                            }
        # calculation of the covariance matrix
        theta.k.adj <- theta.k - matrix( mu, nrow=nrow(theta.k),
                                    ncol=ncol(theta.k), byrow=TRUE)
        for (dd1 in 1:D){
            for (dd2 in dd1:D){
                tk <- theta.k.adj[,dd1]*theta.k.adj[,dd2]
                h1 <- ( hwt %*% tk ) * delta.theta
                Sigma.cov[dd1,dd2] <- sum( h1 ) / N
                if (dd1 < dd2 ){ Sigma.cov[dd2,dd1] <- Sigma.cov[dd1,dd2] }
                                    }
                                }
        if ( est.corr ){ Sigma.cov <- stats::cov2cor(Sigma.cov ) }
        if ( ! is.null(variance.fixed ) ){
                Sigma.cov[ variance.fixed[,1:2,drop=FALSE] ] <- variance.fixed[,3]
                Sigma.cov[ variance.fixed[,c(2,1),drop=FALSE] ] <- variance.fixed[,3]
#
                                    }
        diag(Sigma.cov) <- diag(Sigma.cov) + 10^(-10)
        pi.k <- sirt_dmvnorm_discrete( theta.k, mean=mu, sigma=Sigma.cov, as_matrix=TRUE)
        res <- list("mu"=mu, "Sigma"=Sigma.cov, "pi.k"=pi.k )
        return(res)
                    }
#################################################################
# convert matrix with probabilities
problong2probarray <- function( probres, I, TP ){
        probs <- array(0, dim=c(I,2,TP))
        probs[,1,] <- 1 - probres
        probs[,2,] <- probres
        return(probs)
        }

###########################################
# estimation of b
.smirt.est.b.noncomp <- function(   b, a, c, d, Qmatrix, est.b, theta.k,
        n.ik, I, K, TP, D, numdiff.parm=.001, max.increment=1,
        msteps,  mstepconv, increment.factor){
    h <- numdiff.parm
    diffindex <- est.b        # zeros are allowed!
    cat("  M steps b parameter   |")
    it <- 0
    conv1 <- 1000
    Q2 <- Q1 <- 0*Qmatrix
    Q <- Qmatrix
    se.b <- b
    b00 <- b
    while( ( it < msteps ) & ( conv1 > mstepconv ) ){
        b0 <- b
        for (dd in 1:D){
#                dd <- 2
            Q2 <- Q1
            Q2[,dd] <- 1 * ( Qmatrix[,dd] !=0 )

            probres <- calcprob.noncomp( a, b, Q, thetak=theta.k, c, d )
            pjk <- problong2probarray( probres, I, TP )

            probres <- calcprob.noncomp( a, b + h*Q2, Q, thetak=theta.k, c, d )
            pjk1 <- problong2probarray( probres, I, TP )

            probres <- calcprob.noncomp( a, b - h*Q2, Q, thetak=theta.k, c, d )
            pjk2 <- problong2probarray( probres, I, TP )

            # numerical differentiation
            res <- .rm.numdiff.index( pjk, pjk1, pjk2, n.ik, diffindex[,dd],
                    max.increment=max.increment, numdiff.parm )
            ind <- match( diffindex[,dd], sort(unique( diffindex[,dd] )) )
            b[,dd] <- b[,dd] + (res$increment)[ind]
            se.b[,dd] <- (sqrt(  1 / abs(res$d2) ))[ind]
                        }   # end dd
        conv1 <- max( abs( b - b0 ) )
        it <- it+1
        cat("-") # ; flush.console()
            }
    cat(" ", it, "Step(s) \n")    #; flush.console()
    if ( increment.factor > 1){
        b <- .adj.maxincrement.parameter( oldparm=b00, newparm=b,
                    max.increment=max.increment )
                        }
    res <- list("b"=b, "se.b"=se.b,
            "ll"=sum(res$ll0) )
    return(res)
            }

###########################################
# estimation of a
.smirt.est.a.noncomp <- function(   b, a, c, d, Qmatrix, est.a, theta.k,
        n.ik, I, K, TP, D, numdiff.parm=.001, max.a.increment,
        msteps,  mstepconv, increment.factor){
    h <- numdiff.parm
    diffindex <- est.a        # zeros are allowed!
    cat("  M steps a parameter   |")
    it <- 0 ;    conv1 <- 1000
    Q2 <- Q1 <- 0*Qmatrix
    Q <- Qmatrix
    se.a <- a
    a00 <- a
    while( ( it < msteps ) & ( conv1 > mstepconv ) ){
        a0 <- a
        for (dd in 1:D){
#                dd <- 2
            Q2 <- Q1
            Q2[,dd] <- 1 * ( Qmatrix[,dd] !=0 )

            probres <- calcprob.noncomp( a, b, Q, thetak=theta.k, c, d )
            pjk <- problong2probarray( probres, I, TP )

            probres <- calcprob.noncomp( a + h*Q2, b, Q, thetak=theta.k, c, d )
            pjk1 <- problong2probarray( probres, I, TP )

            probres <- calcprob.noncomp( a - h*Q2, b, Q, thetak=theta.k, c, d )
            pjk2 <- problong2probarray( probres, I, TP )

            # numerical differentiation
            res <- .rm.numdiff.index( pjk, pjk1, pjk2, n.ik, diffindex[,dd],
                    max.increment=max.a.increment[,dd], numdiff.parm )
            ind <- match( diffindex[,dd], sort(unique( diffindex[,dd] )) )
            a[,dd] <- a[,dd] + (res$increment)[ind]
#            max.a.increment[,dd] <- abs( (res$increment)[ind] )
            se.a[,dd] <- (sqrt(  1 / abs(res$d2) ))[ind]
                        }   # end dd
        conv1 <- max( abs( a - a0 ) )
        it <- it+1
        cat("-") # ; flush.console()
            }
    cat(" ", it, "Step(s) \n")    #; flush.console()
    #****
    # post-processing of a parameters
    if ( increment.factor > 1){
        a <- .adj.maxincrement.parameter( oldparm=a00, newparm=a,
                    max.increment=max.a.increment )
                        }
    res <- list("a"=a, "se.a"=se.a, "ll"=sum(res$ll0) )
    return(res)
            }



###########################################
# estimation of c
.smirt.est.c.noncomp <- function(   b, a, c, d, Qmatrix, est.c, theta.k,
        n.ik, I, K, TP, D, numdiff.parm=.001, max.increment=max.increment,
        msteps,  mstepconv, increment.factor){
    h <- numdiff.parm
    diffindex <- est.c        # zeros are allowed!
    Q1 <- rep(1,I)
    Q1[ est.c==0 ] <- 0
    Q <- Qmatrix
    c00 <- c
    cat("  M steps c parameter   |")
    it <- 0 ;    conv1 <- 1000
    while( ( it < msteps ) & ( conv1 > mstepconv ) ){
        c0 <- c
            probres <- calcprob.noncomp( a, b, Q, thetak=theta.k, c, d )
            pjk <- problong2probarray( probres, I, TP )

            probres <- calcprob.noncomp( a, b, Q, thetak=theta.k, c+h*Q1, d )
            pjk1 <- problong2probarray( probres, I, TP )

            probres <- calcprob.noncomp( a, b, Q, thetak=theta.k, c-h*Q1, d )
            pjk2 <- problong2probarray( probres, I, TP )

            # numerical differentiation
            res <- .rm.numdiff.index( pjk, pjk1, pjk2, n.ik, diffindex,
                    max.increment=max.increment, numdiff.parm )
            ind <- match( diffindex, sort(unique( diffindex ))    )
            incr <- res$increment
            incr[ is.na(incr ) ] <- 0
            c <- c + incr[ind]
            c[ c< 0 ] <- .001
#            max.a.increment[,dd] <- abs( (res$increment)[ind] )
            se.c <- (sqrt(  1 / abs(res$d2) ))[ind]
        conv1 <- max( abs( c - c0 ) )
        it <- it+1
        cat("-") # ; flush.console()
            }
    cat(" ", it, "Step(s) \n")    #; flush.console()
    if ( increment.factor > 1){
        c <- .adj.maxincrement.parameter( oldparm=c00, newparm=c,
                    max.increment=max.increment )
                        }
    res <- list("c"=c, "se.c"=se.c,
            "ll"=sum(res$ll0) )
    return(res)
            }


###########################################
# estimation of c
.smirt.est.d.noncomp <- function(   b, a, c, d, Qmatrix, est.d, theta.k,
        n.ik, I, K, TP, D, numdiff.parm=.001, max.increment=max.increment,
        msteps,  mstepconv, increment.factor){
    h <- numdiff.parm
    diffindex <- est.d        # zeros are allowed!
    Q1 <- rep(1,I)
    Q1[ est.d==0 ] <- 0
    Q <- Qmatrix
    d00 <- d
    cat("  M steps d parameter   |")
    it <- 0 ;    conv1 <- 1000
    while( ( it < msteps ) & ( conv1 > mstepconv ) ){
        d0 <- d
            probres <- calcprob.noncomp( a, b, Q, thetak=theta.k, c, d )
            pjk <- problong2probarray( probres, I, TP )

            probres <- calcprob.noncomp( a, b, Q, thetak=theta.k, c, d +h*Q1)
            pjk1 <- problong2probarray( probres, I, TP )

            probres <- calcprob.noncomp( a, b, Q, thetak=theta.k, c, d-h*Q1 )
            pjk2 <- problong2probarray( probres, I, TP )

            # numerical differentiation
            res <- .rm.numdiff.index( pjk, pjk1, pjk2, n.ik, diffindex,
                    max.increment=max.increment, numdiff.parm )
            ind <- match( diffindex, sort(unique( diffindex ))        )
            incr <- res$increment
            incr[ is.na(incr ) ] <- 0
            d <- d + incr[ind]
            d[ d>1 ] <- .999
#            max.a.increment[,dd] <- abs( (res$increment)[ind] )
            se.d <- (sqrt(  1 / abs(res$d2) ))[ind]
        conv1 <- max( abs( d - d0 ) )
        it <- it+1
        cat("-") # ; flush.console()
            }
    cat(" ", it, "Step(s) \n")    #; flush.console()
    if ( increment.factor > 1){
        d <- .adj.maxincrement.parameter( oldparm=d00, newparm=d,
                    max.increment=max.increment )
                        }
    res <- list("d"=d, "se.d"=se.d, "ll"=sum(res$ll0) )
    return(res)
            }

## File Name: btm_sim.R
## File Version: 0.04

btm_sim <- function(theta, eta=0, delta=-99, repeated=FALSE)
{
    N <- length(theta)
    dat <- t( utils::combn(N,2) )
    colnames(dat) <- c('id1', 'id2')
    dat <- as.data.frame(dat)
    if (repeated){
        dat2 <- dat[, c('id2','id1') ]
        colnames(dat2) <- colnames(dat)
        dat <- rbind(dat, dat2 )
    }
    N1 <- nrow(dat)
    probs <- matrix(0, nrow=N1, ncol=3)
    probs[,1] <- eta + theta[ dat$id1 ]
    probs[,2] <- theta[ dat$id2 ]
    probs[,3] <- delta + ( eta + theta[ dat$id1 ] + theta[dat$id2 ] ) / 2
    probs <- exp(probs)
    probs <- probs / rowSums(probs)
    probs1 <- rowCumsums.sirt(matr=probs)
    rn <- stats::runif(N1)
    result <- rowIntervalIndex.sirt(matr=probs1,rn=rn)
    result <- 1*(result==1) + 0.5*(result==3) + 0*(result==2)
    dat <- data.frame( dat, result=result)
    rownames(dat) <- NULL
    return(dat)
}

## File Name: xxirt_postproc_parameters.R
## File Version: 0.249


xxirt_postproc_parameters <- function( partable, customTheta,
        items, probs_items, np_fun_item=NULL, np_fun_theta=NULL )
{
    #**** item parameters
    p1 <- partable[ partable$parfree==1, ]
    par_items <- p1$value
    names(par_items) <- p1$parlabel
    #*** theta distribution parameters
    cs <- customTheta
    par_Theta <- cs$par[ cs$est ]
    #*** structured form of parameters
    I <- length(items)
    parnames <- sort( unique( paste( partable$parname) ) )
    PN <- length(parnames)
    m1 <- matrix(NA, nrow=I, ncol=PN)
    rownames(m1) <- items
    colnames(m1) <- parnames
    for (pp in 1L:PN){
        p1 <- partable[ partable$parname==parnames[pp], ]
        m1[ p1$item, parnames[pp] ] <- p1$value
    }
    p1 <- partable[ ! duplicated(partable$item ), ]
    dfr <- data.frame( item=items, type=paste(p1$type), m1  )
    rownames(dfr) <- NULL
    #*** probabilities item parameters
    pi_dim <- dim(probs_items)
    dimnames(probs_items)[[1]] <- items
    dimnames(probs_items)[[2]] <- paste0('Cat', seq(0,pi_dim[2]-1) )
    #*** lower and upper bounds
    p1 <- partable[ partable$parfree==1, c('item', 'type', 'parname',
                        'value', 'lower', 'upper', 'parindex', 'parlabel' ) ]
    p1$active <- 1 * ( p1$value > p1$lower )
    p1$active <- p1$active * ( p1$value < p1$upper )
    par_items_bounds <- p1
    np_item <- NULL
    np_theta <- NULL
    if ( ! is.null(np_fun_item) ){
        np_item <- np_fun_item(x=par_items)
    }
    if ( ! is.null(np_fun_theta) ){
        np_theta <- np_fun_theta(x=customTheta$par)
    }

    #*** output
    res <- list( par_items=par_items, par_Theta=par_Theta,
                    probs_items=probs_items, par_items_summary=dfr,
                    par_items_bounds=par_items_bounds, np_item=np_item,
                    np_theta=np_theta)
    return(res)
}

## File Name: lsdm_est_logist_rasch.R
## File Version: 0.283


lsdm_est_logist_rasch <- function( y, theta, wgt_theta, method="L-BFGS-B" )
{
    y <- as.numeric(y)

    #-- define optimization function
    dist_irf_val <- function(x){
        irf_est <- stats::plogis(theta-x)
        diff_irf <- y - irf_est
        val <- sum( diff_irf^2 * wgt_theta )
        return(val)
    }
    dist_irf_grad <- function(x){
        irf_est <- stats::plogis(theta-x)
        diff_irf <- y - irf_est
        grad1 <- 2*diff_irf*irf_est*(1-irf_est)
        grad <- sum(grad1*wgt_theta)
        return(grad)
    }

    #* optimization
    par0 <- theta[ which.min( abs(y-0.5) ) ]
    res <- stats::optim(par=par0, fn=dist_irf_val, gr=dist_irf_grad, method=method)

    #--- output
    res <- c( res$par, sqrt(res$value) )
    return(res)
}

## File Name: sirt_MAP.R
## File Version: 0.02

sirt_MAP <- function(post, theta)
{
    TP <- ncol(post)
    maxval <- post[,1]
    indval <- 1
    for (tt in 2L:TP){
        m0 <- maxval
        maxval <- ifelse( post[,tt] > m0, post[,tt], m0 )
        indval <- ifelse( post[,tt] > m0, tt, indval )
    }
    MAP <- theta[ indval, ]
    return(MAP)
}

## File Name: mgsem_cd_opt.R
## File Version: 0.178

mgsem_cd_opt <- function(x, opt_fun_args, tol=1e-4, eps_approx=1e-20,
                maxiter=100, h=1e-4, verbose=TRUE, interval_length=0.025,
                method="lqa")
{
    cd_fun_args <- opt_fun_args
    cd_fun_args$eps_approx <- eps_approx
    cd_fun_args$partable_start <- cd_fun_args$partable
    NP <- max(cd_fun_args$partable$index)

    verbose_print <- round( seq( 1,NP, length=min(NP,10)) )

    if (verbose){
        cat('\n----- Start coordinate descent ---- \n')
    }

    iter <- 1
    x00 <- x
    pp <- 1
    maxdiff <- 1

    while ( ( iter < (maxiter+1) ) & ( maxdiff > tol ) ){

        if (verbose){
            cat('Iter.', iter, ' ')
        }

        x0 <- x

        for (pp in 1L:NP){
            partable <- cd_fun_args$partable
            x1 <- x[pp]
            interval <- x1 + interval_length*c(-1,1)
            cd_opt_fun <- function(x1)
            {
                x[pp] <- x1
                ll <- mgsem_opt_fun(x=x, opt_fun_args=cd_fun_args)
                return(ll)
            }
            if (method=='exact'){
                res <- stats::optimize(f=cd_opt_fun, interval=interval, tol=tol)
            }
            if (method=='lqa'){
                cda_fun_args <- cd_fun_args
                cda_fun_args$pen_type <- 'none'

                ll0 <- mgsem_opt_fun(x=x, opt_fun_args=cda_fun_args)
                ll1 <- mgsem_opt_fun(x=mgsem_add_increment(x=x, h=h, i1=pp),
                                        opt_fun_args=cda_fun_args)
                ll2 <- mgsem_opt_fun(x=mgsem_add_increment(x=x, h=-h, i1=pp),
                                        opt_fun_args=cda_fun_args)
                a0 <- ll0
                a1 <- (ll1-ll2)/(2*h)
                a2 <- 0.5*(ll1-2*ll0+ll2)/h^2
                x10 <- x1
                cda_opt_fun <- function(x1)
                {
                    x[pp] <- x1
                    res <- mgsem_cda_opt_evaluate_penalties(x=x, opt_fun_args=cd_fun_args)
                    val0 <- -res$pen_all
                    val <- a0+a1*(x1-x10)+a2*(x1-x10)^2+val0
                    return(val)
                }
                res <- stats::optimize(f=cda_opt_fun, interval=interval, tol=tol)
            }

            v1 <- res$minimum
            x[pp] <- v1
            cd_fun_args$partable <- mgsem_coef2partable(coef=x, partable=partable)
            verbose_pp <- verbose & (pp %in% verbose_print)
            if (verbose_pp){
                cat('.')
                utils::flush.console()
            }
        } # end pp

        maxdiff <- max( abs(x-x0) )
        if (verbose){
            cat( ' | Max. diff.=', round(maxdiff, 7), '\n')
        }

        iter <- iter + 1

    }

    value <- cd_opt_fun(x=x1)

    #--- output
    res <- list(par=x, value=value)
    return(res)
}

## File Name: xxirt_classprobs_lca_compute_probs.R
## File Version: 0.04

xxirt_classprobs_lca_compute_probs <- function(logitprobs)
{
    v1 <- c( logitprobs, 0 )
    v1 <- v1 - max(v1)
    l1 <- exp(v1)
    res <- sirt_sum_norm(x=l1)
    return(res)
}

## File Name: prior_model_parse.R
## File Version: 0.19

#############################################################
prior_model_parse <- function( prior_model ){
    ps <- strsplit( prior_model, split="\n", fixed=TRUE)[[1]]
    # clean string
    ps <- prior_model_pars_CleanString( ps )
    NP <- length(ps)
    prior <- as.list( 1:NP )
    for (pp in 1:NP){
        # pp <- 1
        ps_pp <- ps[pp]
        ps_pp1 <- strsplit( ps_pp, split="~", fixed=TRUE)[[1]]
        # extract name
        names(prior)[pp] <- ps_pp1[1]
        prior[[pp]] <- as.list(1:2)
        # extract distribution
        ps_pp2 <- ps_pp1[2]
        ps_pp2a <- strsplit( ps_pp2, split="(", fixed=TRUE)[[1]]
        prior[[pp]][[1]] <- ps_pp2a[1]
        ps_pp3 <- ps_pp2a[2]
        ps_pp3 <- strsplit( ps_pp3, split=")", fixed=TRUE)[[1]][1]
        ps_pp3 <- strsplit( ps_pp3, split=",", fixed=TRUE)[[1]]
        NV <- length(ps_pp3)
        prior_pp2 <- as.list( 1:NV )
        for (vv in 1:NV){
            # vv <- 1
            ps_vv <- ps_pp3[vv]
            h_vv <- strsplit(ps_vv, split="=", fixed=TRUE)[[1]]
            len_h_vv <- length(h_vv)
            if ( len_h_vv==1){
                prior_pp2[[vv]] <- suppressWarnings( as.numeric(h_vv) )
            }
            if ( len_h_vv==2){
                prior_pp2[[vv]] <- suppressWarnings( as.numeric(h_vv[2]) )
                names(prior_pp2)[vv] <- h_vv[1]
            }
        }
        prior[[pp]][[2]] <- prior_pp2
    }
    return(prior)
}
###############################################################

## File Name: xxirt_ThetaDistribution_extract_freeParameters.R
## File Version: 0.10


xxirt_ThetaDistribution_extract_freeParameters <- function( customTheta )
{
    est <- customTheta$est
    if ( sum(est)==0 ){
        par1 <- NULL
    } else {
        par1 <- customTheta$par[ est ]
    }
    return(par1)
}

## File Name: mcmc.list.descriptives.R
## File Version: 0.19

########################################################
# mcmclist descriptives
mcmc.list.descriptives <- function( mcmcobj, quantiles=c(.025,.05,.1,.50,.9,.95,.975) ){
     summary.mcmcobj <- summary(mcmcobj, quantile=quantiles)
    dat.bugs <- mcmcobj[[1]]
    vars <- colnames(dat.bugs)
    n.iter <- nrow(dat.bugs)
        discret <- round( seq( 1, n.iter, length=4 ) )
        MAP <- Rhat <- rep(0,length(vars) )
        for (ii in seq( 1, length(vars ))){
            vv <- vars[ii]
            dat.vv <- as.vector( dat.bugs[, vv ] )
            # extract 3 subchains
            l1 <- dat.vv[ discret[1]:discret[2] ]
            l2 <- dat.vv[ (discret[2]+1):discret[3] ]
            l3 <- dat.vv[ (discret[3]+1):discret[4] ]
            W <- ( var(l1) + var(l2) + var(l3) ) / 3
            S1 <- n.iter / 3
            est.chains <- c(mean(l1), mean(l2), mean(l3) )
            B <-  S1 / 2 * sum( ( est.chains - mean( est.chains ) )^2 )
            Rhat[ii] <- ( ( S1-1 ) / S1 * W + B / S1 ) / W
            # mode estimation
            m1 <- stats::density( dat.vv, from=min(dat.vv), to=max(dat.vv) )
            MAP[ii] <- m1$x[ which( m1$y==max( m1$y) ) ]
                            }
    res <- data.frame( "MAP"=MAP, "Rhat"=Rhat )
    rownames(res) <- vars
    smc3  <- res
    smc2 <- summary.mcmcobj$statistics
    colnames(summary.mcmcobj$quantiles) <- paste0( "Q", 100*quantiles )
    # calculate effective sample size
    effSize <- sirt_import_coda_effectiveSize( mcmcobj )
    statis <- summary.mcmcobj$statistics
    statis <- cbind( statis[, c(1,2) ], apply( as.matrix(mcmcobj), 2, stats::mad ),
                apply( as.matrix(mcmcobj), 2, skewness.sirt ),
                statis[,c(3,4) ]    )
    colnames(statis)[3:4] <- c("MAD", "skewness" )
    dfr <- data.frame( "parameter"=rownames(smc3),
        statis, smc3, "PercSERatio"=100*smc2[,4] / smc2[,2],
        "sampSize"=nrow(as.matrix(mcmcobj)), "effSize"=effSize,
        summary.mcmcobj$quantiles )
    return(dfr)
        }


#**********************************************
# function for calculation skewness
skewness.sirt <- function(x){
    n <- length(x)
    x <- x - mean(x)
    y <- sqrt(n) * sum(x^3)/(sum(x^2)^(3/2))
    y <- y * ((1 - 1/n))^(3/2)
    return(y)
    }

## File Name: tam2mirt_fix.R
## File Version: 0.154


#--- return lavaan syntax with fixed parameters
tam2mirt_fix <- function( D, factors, B, dat, AXsi,
        mean.trait, cov.trait, tamobj )
{
    # create lavaan syntax with constraints
    lavsyn <- NULL
    for (dd in 1L:D){
        fac.dd <- factors[dd]
        # create terms for loadings
        B2.dd <- round( B[,2,dd], 4)
        syn0 <- paste0( paste0( B2.dd[ B2.dd!=0], '*',
                        colnames(dat)[ B2.dd!=0] ), collapse='+' )
        syn0 <- paste0( fac.dd, '=~ ', syn0, '\n')
        lavsyn <- paste0( lavsyn, syn0 )
    }
    # create syntax for intercepts
    maxK <- ncol(AXsi) - 1
    for (kk in 1L:maxK){
        t1 <- round( AXsi[,kk+1], 4 )
        string1 <- paste0('t', kk )
        syn0 <- paste0( colnames(dat), ' | ', t1, '*', string1)
        syn0 <- paste0( syn0, collapse='\n')
        hh <- ''
        if (kk !=maxK){ hh <- '\n' }
        lavsyn <- paste0( lavsyn, syn0, hh)
    }
    # guessing and slipping parameters
    itemg <- colnames(dat)[ maxK==1 ]
    if ( length(itemg) > 0 ){
        lavsyn <- paste0( lavsyn, '\n', paste0( paste0( itemg, ' ?=0*g1' ),
                                    collapse='\n') )
        lavsyn <- paste0( lavsyn, '\n', paste0( paste0( itemg, ' ?=0*s1' ),
                                    collapse='\n') )
    }
    # syntax for means
    syn0 <- paste0( factors, ' ~ ', round(as.vector(mean.trait),4), '*1'  )
    syn0 <- paste0( syn0, collapse='\n')
    lavsyn <- paste0( lavsyn, '\n', syn0 )
    # syntax for variances
    syn0 <- paste0( factors, ' ~~ ', round( as.vector(diag(cov.trait)),4), '*',factors  )
    syn0 <- paste0( syn0, collapse='\n')
    lavsyn <- paste0( lavsyn, '\n', syn0 )
    # syntax for covariances
    if (D>1){
        for (dd in 1L:(D-1)){
            for (ee in (dd+1):D ){
                syn0 <- paste0( factors[dd], ' ~~ ',
                            round( cov.trait[dd,ee],4), '*',factors[ee]  )
                syn0 <- paste0( syn0, collapse='\n')
                lavsyn <- paste0( lavsyn, '\n', syn0 )
            }
        }
    }
    # finalize lavaan syntax
    lavsyn <- paste0( lavsyn, ' \n')
    return(lavsyn)
}

## File Name: sirt_vector_with_names.R
## File Version: 0.06

sirt_vector_with_names <- function(value, names)
{
    vec <- rep( value, length(names) )
    if ( ! is.numeric(names) ){
        names(vec) <- names
    }
    return(vec)
}

## File Name: rasch_mirtlc_mstep_mlc1.R
## File Version: 0.17


#-- M-step rasch mirtlc
rasch_mirtlc_mstep_mlc1 <- function( pjk, n.k, r.jk, n.jk, G, Nclasses,
            theta.k, b, a, I, ref.item, mstep.maxit,
            des.theta, des.b, theta.fixed, theta.normal, f.qk.yi,D,
            distribution.trait, est.a, Qmatrix,modeltype, range.b, range.a,
            iter, fac.iter, dimensions=NULL)
{
    if (G==1){
        pi.k <- n.k / sum(n.k)
    }
    if ( G> 1){
        pi.k <- n.k / matrix( colSums(n.k), nrow=Nclasses, ncol=G, byrow=TRUE)
    }

    #*** perform logistic regression
    if (G==1){
        c1 <- r.jk[,,1]
        i1 <- n.jk[,,1] - c1
    }
    if (G > 1){
        c1 <- apply( r.jk, c(1,2), sum )
        i1 <- apply( n.jk - r.jk, c(1,2), sum )
    }

    # class weights correct response
    wc1 <- matrix( c1, nrow=I*Nclasses, 1 )
    wi1 <- matrix( i1, nrow=I*Nclasses, 1 )
    # outcome
    y <- c( rep(1,I*Nclasses), rep(0,I*Nclasses) )
    wgt <- c( wc1, wi1 )

    if ( theta.fixed ){
        if (D==1){
            theta.offset <- rep( theta.k, each=I )
            theta.offset <- c( theta.offset, theta.offset )
        }
        if (D>1){
            tk1 <- matrix( theta.k, nrow=1, ncol=ncol(des.theta) )
            tk1 <- matrix( tk1, nrow=nrow(des.theta), ncol=ncol(tk1), byrow=TRUE)
            theta.offset <- des.theta * tk1
            theta.offset <- rowSums( theta.offset )
        }
    }
    if ( ( ! theta.fixed ) ){
        b_old <- b
        theta_old <- theta.k
        # structure theta
        # theta_1: class_1, ..., class_D
        # theta_2: class_1, ..., class_D
        # ... theta_D: class_1, ..., class_D
        # unconstrained theta optimization
        if (modeltype=="MLC2"){
            des1 <- matrix( a, nrow=nrow(des.theta), ncol=1)
            des.theta <- des.theta * outer( des1[,1], rep(1,ncol(des.theta) ) )
            des.b <- des.b * outer( des1[,1], rep(1,ncol(des.b) ) )
        }
            mod2 <- stats::glm( y ~ 0 + des.theta + des.b, weights=wgt, family="binomial",
                        control=list(maxit=mstep.maxit ) )
        if (D==1){
            theta.k <- coef(mod2)[ 1:Nclasses ]    # theta
            b0 <- coef(mod2)[ -c( 1:Nclasses ) ]
        }
        if (D>1){
            theta.k0 <- theta.k
            theta.k <- coef(mod2)[ 1:(D*Nclasses) ]    # theta
            theta.k <- matrix( theta.k, nrow=Nclasses, ncol=D )
            theta.change <- theta.k - theta_old
            increment <- .9^( iter^fac.iter)
            theta.change <- ifelse( abs( theta.change ) > increment, sign(theta.change)*increment, theta.change )
            theta.k <- theta_old + theta.change
            b0 <- coef(mod2)[ -c( 1:(D*Nclasses) ) ]
        }
        b[ setdiff( 1:I, ref.item ) ] <- b0
        b.change <- b - b_old
        b.increment <- .9^( iter^fac.iter)
        b.change <- ifelse( abs( b.change ) > b.increment, sign(b.change)*b.increment, b.change )
        b <- b_old + b.change
    }
    if ( theta.fixed ){
        # constrained theta optimization
        if (modeltype=="MLC2"){
            des1 <- matrix( a, nrow=nrow(des.theta), ncol=1)
            theta.offset <- des1[,1] * theta.offset
            des.b <- des.b * outer( des1[,1], rep(1,ncol(des.b) ) )
        }
        mod2 <- stats::glm( y ~ 0 + offset(theta.offset) + des.b, weights=wgt, family="binomial",
                        control=list(maxit=mstep.maxit ) )
        b0 <- coef(mod2)
        b[ setdiff( 1:I, ref.item ) ] <- b0

        #***** normal distribution assumption
        if (distribution.trait=="normal" & ( D==1) ){ # D=1
            for (gg in 1:G){
                pik1 <-    pi.k[,gg]
                m1 <- sum( theta.k * pik1 )
                sd1 <- sqrt( sum( theta.k^2 * pik1 ) - m1^2 )
                pi.k[,gg] <- sirt_dnorm_discrete( theta.k, mean=m1, sd=sd1 )
            }
        }

        #--- log linear smoothing
        if (distribution.trait %in% c("smooth2", "smooth3", "smooth4") & ( D==1) ){ # D=1
            for (gg in 1:G){
                pik1 <-    pi.k[,gg]
                pik1 <- pik1 + 1e-10
                lpik1 <- log( pik1 )
                tk <- theta.k
                if ( distribution.trait=="smooth2"){
                    formula1 <- lpik1 ~ tk + I(tk^2)
                }
                if ( distribution.trait=="smooth3"){
                    formula1 <- lpik1 ~ tk + I(tk^2) + I(tk^3)
                }
                if ( distribution.trait=="smooth4"){
                    formula1 <- lpik1 ~ tk + I(tk^2) + I(tk^3) + I(tk^4)
                }
                mod <- stats::lm( formula1, weights=pik1 )
                pik2 <- exp( fitted(mod))
                pi.k[,gg] <- pik2 / sum(pik2)
            }
        }
    }

    # range restrictions
    b[ b < range.b[1] ] <- range.b[1]
    b[ b > range.b[2] ] <- range.b[2]
    ## 2PL estimation
    if (modeltype=="MLC2" ){
        a <- rasch_mirtlc_est_a( theta.k=theta.k, b=b, fixed.a=a,
                pjk=pjk, alpha1=0, alpha2=0, h=1e-4, G=G, I=I, r.jk=r.jk,
                n.jk=n.jk, est.a=est.a, Qmatrix=Qmatrix,
                iter=iter, fac.iter=fac.iter, dimensions=dimensions )
        a[ a < range.a[1] ] <- range.a[1]
        a[ a > range.a[2] ] <- range.a[2]
    }

    #-- output
    res <- list( pi.k=pi.k, pjk=pjk, theta.k=theta.k, b=b, a=a )
    return(res)
}


.m.step.mirtlc.mlc1 <- rasch_mirtlc_mstep_mlc1

## File Name: lsem_weighted_mean.R
## File Version: 0.173

lsem_weighted_mean <- function( x, weights, x_resp=NULL,
        moderator_variable=NULL, loc_linear_smooth=NULL, moderator_value=NULL )
{
    if (is.null(loc_linear_smooth)){
        loc_linear_smooth <- FALSE
    }
    x <- as.matrix(x)
    if ( is.null(x_resp)){
        x_resp <- 1 - is.na(x)
    }
    weights_m <- weights * x_resp
    x[ ! x_resp ] <- 0
    weightsN <- colSums(weights_m)
    wm_raw <- wm <- colSums( x * weights_m ) / weightsN

    if (loc_linear_smooth){
        V <- length(wm)
        wm2 <- rep(NA, V)
        colnames(wm2) <- colnames(wm)
        for (vv in 1L:V){
            mod <- stats::lm(x[,vv]~moderator_variable, weights=weights_m[,vv])
            cmod <- mod$coefficients
            wm2[vv] <- cmod[1]+cmod[2]*moderator_value
        }
        wm <- wm2
    }

    #-- output
    res <- list( weightsN=weightsN, mean=wm, raw_mean=wm_raw )
    return(res)
}

## File Name: invariance_alignment_simulate.R
## File Version: 0.123

invariance_alignment_simulate <- function(nu, lambda, err_var, mu, sigma, N,
    output="data", groupwise=FALSE, exact=FALSE)
{
    if (length(N)==1 & nrow(nu)>1){
        N <- rep(N, nrow(nu))
    }
    nu <- as.matrix(nu)
    lambda <- as.matrix(lambda)
    err_var <- as.matrix(err_var)
    N_tot <- sum(N)
    G <- nrow(nu)
    I <- ncol(nu)
    items <- colnames(nu)
    if (is.null(items)){
        items <- paste0('I',1L:I)
    }
    n_end <- cumsum(N)
    n_start <- c(1,n_end+1)[-c(G+1)]
    #* simulate data
    if (N[1]<Inf){
        group <- rep(1L:G, N)
        dat <- matrix(NA, nrow=N_tot, ncol=I+1)
        colnames(dat) <- c('group',items)
        dat <- as.data.frame(dat)
        dat$group <- group
        for (gg in 1L:G){
            N_gg <- N[gg]
            ind_gg <- seq(n_start[gg], n_end[gg])
            fac <- ruvn(N=N_gg, mean=mu[gg], sd=sigma[gg], exact=FALSE)
            for (ii in 1L:I){
                err_ii <- ruvn(N=N_gg, mean=0, sd=sqrt(err_var[gg,ii]), exact=FALSE )
                dat[ind_gg, ii+1] <- nu[gg,ii] + lambda[gg,ii]*fac + err_ii
            }
            #* exact distribution
            if (exact){
                mu_gg <- nu[gg,] + lambda[gg,] * mu[gg]
                Sigma_gg <- sigma[gg]^2 *tcrossprod(lambda[gg,]) + diag(err_var[gg,])
                dat_gg <- rmvn(N=N_gg, mu=mu_gg, Sigma=Sigma_gg, exact=TRUE )
                dat[ind_gg,-1] <- dat_gg
            }

        }
        #-- output
        res <- dat
        if (output=='suffstat'){
            res <- list(mu=list(), Sigma=list(), N=list() )
            for (gg in 1L:G){
                ind_gg <- which(dat$group==gg)
                res$mu[[gg]] <- colMeans(dat[ ind_gg, -1])
                res$Sigma[[gg]] <- stats::cov.wt(dat[ ind_gg, -1], method='ML')$cov
                res$N[[gg]] <- N[gg]
            }
        }
    }
    #*** only compute covariance matrices
    if (N[1]==Inf){
        res <- list(mu=list(), Sigma=list(), N=list() )
        for (gg in 1L:G){
            lam_gg <- lambda[gg,]
            sig2_gg <- sigma[gg]^2
            res$mu[[gg]] <- nu[gg,] + lam_gg*mu[gg]
            lam_gg <- matrix(lam_gg, ncol=1)
            res$Sigma[[gg]] <- lam_gg %*% sig2_gg %*% t(lam_gg) + diag(err_var[gg,])
            res$N[[gg]] <- N[gg]
        }
        output <- 'suffstat'
    }
    #*** group-wise output
    if ( (output=='suffstat') & groupwise ){
        res1 <- res
        res <- as.list(1L:G)
        for (gg in 1L:G){
            res[[gg]] <- list(mu=res1$mu[[gg]], Sigma=res1$Sigma[[gg]], N=res1$N[[gg]])
        }
    }

    #--- output
    return(res)
}

## File Name: mi_inv_lavaan_modification_indices.R
## File Version: 0.06

mi_inv_lavaan_modification_indices <- function(mod, op=c("~1","=~"))
{
    requireNamespace('lavaan')
    res <- lavaan::modificationIndices(object=mod, free.remove=FALSE,
                    op=op, sort=TRUE)
    return(res)
}

## File Name: mcmc_extract_samples_first_chain.R
## File Version: 0.04

mcmc_extract_samples_first_chain <- function(mcmcobj)
{
    if ( ! is.matrix(mcmcobj) ){
        dat.bugs <- mcmcobj[[1]]
    } else {
        dat.bugs <- mcmcobj
    }
    return(dat.bugs)
}

## File Name: sirt_Sapply.R
## File Version: 0.075


sirt_Sapply <- function(...)
{
    args <- list(...)
    fun <- utils::getFromNamespace(x='mySapply', ns='mirt')
    return( do.call(what=fun, args=args) )
}

## File Name: gom_em_calc_probs.R
## File Version: 0.13



#--- gom calcprobs
gom_em_calc_probs <- function( lambda, theta.k, b=NULL, theta0.k=NULL )
{
    if ( ! is.null( b ) ){
        lambda <- stats::plogis( - b + matrix( theta0.k, length(b), length(theta0.k),
                    byrow=TRUE ) )
    }
    probs <- tcrossprod( lambda, theta.k)
    probsL <- array( 0, dim=c( nrow(lambda), 2, nrow(theta.k) ) )
    probsL[,2,] <- probs
    probsL[,1,] <- 1-probs
    res <- list(probs=probs, probsL=probsL)
    return(res)
}

.gom.calcprobs <- gom_em_calc_probs

## File Name: sirt_pem_create_parameter_index.R
## File Version: 0.121


sirt_pem_create_parameter_index <- function( parmlist )
{
    NP <- length(parmlist)
    parm_index <- list()
    parmlist_names <- names(parmlist)
    parm_index_names <- rep('', NP)
    last_index <- 0
    for (pp in 1L:NP){
        x <- parmlist[[pp]]
        x_pp <- list()
        parm_index_names[pp] <- parmlist_names[pp]
        x_dim <- sirt_pem_extract_dimension(x=x)
        NX <- prod(x_dim)
        x_index <- last_index + seq( 1, NX )
        x_pp$index <- x_index
        last_index <- max(x_index)
        x_pp$dim <- x_dim
        parm_index[[pp]] <- x_pp
    }
    names(parm_index) <- parm_index_names
    #-- more meta-information about parameters
    parm_index[['__n_parameters']] <- NP
    parm_index[['__length_parameter_vector']] <- last_index
    return(parm_index)
}

## File Name: mgsem_proc_model_add_specs.R
## File Version: 0.07


mgsem_proc_model_add_specs <- function(model, entry, type, ii, jj, default)
{
    val <- default
    mat <- model[[entry]][[type]]
    if (!is.null(mat)){
        val <- mat[ii,jj]
    }
    return(val)
}

## File Name: xxirt_nr_opt_fun_pml_casewise.R
## File Version: 0.03


#-- case-wise optimization function for PML
xxirt_nr_opt_fun_pml_casewise <- function(x, object, eps=1e-14)
{
    em_args <- object$em_args
    pml_args <- em_args$pml_args
    prior_Theta <- xxirt_compute_prior_Theta_from_x(x=x, em_args=em_args)
    probs_items <- xxirt_compute_prob_item_from_x(x=x, em_args=em_args)
    I <- pml_args$I
    K <- pml_args$K
    G <- pml_args$G
    TP <- pml_args$TP
    W1 <- pml_args$W1
    freq1 <- pml_args$freq1
    NI2 <- pml_args$NI2
    W2_long <- as.matrix(pml_args$W2_long)
    freq2 <- pml_args$freq2
    group0 <- object$group - 1
    weights <- object$weights
    dat1 <- object$dat1
    dat_resp <- object$dat_resp==1
    eval_fun_args <- list(prior_Theta=prior_Theta, probs_items=probs_items,
                            W1=W1, W2_long=W2_long, G=G, K=K,
                            I=I, TP=TP, NI2=NI2, eps=eps, group0=group0,
                            weights=weights, dat1=dat1, dat_resp=dat_resp)
    res <- do.call( what=sirt_rcpp_xxirt_nr_pml_casewise_opt_fun, args=eval_fun_args)
    return(-res$case_val)
}

## File Name: summary.gom.em.R
## File Version: 0.179


#--- summary for gom object
summary.gom <- function( object, file=NULL, ...)
{

    # open sink
    sirt_osink( file=file )

    cat("-----------------------------------------------------------------\n")
    #- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object)

    cat("  Function 'gom.em' \n")

    #- Newton-Raphson optimization
    if (object$newton_raphson){
        sirt_optimizer_summary_print(res=object$optimization,
                msg="Information about Newton-Raphson algorithm of optimization")
    }

    if (object$model=="GOM"){
        cat("   Discrete Grade of Membership Model\n\n")
    }
    if (object$model=="GOMnormal"){
        cat("   Grade of Membership Model Based on Multivariate Normal Distribution\n\n")
    }
    if (object$model=="GOMRasch"){
        cat("   Rasch Grade of Membership Model\n\n")
    }
    modeltype <- object$irtmodel
    cat( "   ", object$ic$n, "Cases, ", ncol(object$dat2), "Items, ",
                object$K, "Classes", ",", object$TP, "Discrete Integration Points\n")

    cat("-----------------------------------------------------------------\n")
    cat( "Number of EM iterations", "=", object$iter, "\n" )
    cat( "Deviance", "=", round( object$deviance, 2 ), " | " )
    cat( "Log Likelihood", "=", round( -object$deviance/2, 2 ), "\n" )
    cat( "Number of persons", "=", object$ic$n, "\n" )

    cat( "Number of estimated parameters", "=", object$ic$np, "\n" )
    cat( "  Number of estimated item parameters", "=", object$ic$np.item, "\n" )
    cat( "  Number of estimated distribution parameters", "=",
                                        object$ic$np.trait, "\n\n" )

    #--- information criteria
    rm_summary_information_criteria(object=object)

    #--- trait distribution
    if (object$model %in% c("GOMRasch","GOMnormal") ){
        cat("-----------------------------------------------------------------\n")
        if (object$model=="GOMRasch"){
            cat("Trait Distribution (Location, Variability)\n")
        }
        if (object$model=="GOMnormal"){
            cat("Underlying multivariate normal distribution\n")
        }
        cat( " Means: ", round( object$mu, 3 ), "\n")
        cat( " Standard deviations: ", round( sqrt(diag(object$Sigma)), 3 ), "\n")
        if (object$model=="GOMRasch"){
            c1 <- stats::cov2cor(object$Sigma)
            cat( " Correlation ", round( c1[lower.tri(c1)], 3 ), "\n")
            cat("EAP Reliability: ", round(object$EAP.rel,3), "\n")
        }
        if (object$model=="GOMnormal"){
            c1 <- object$Sigma
            cat("Covariance matrix:\n")
            print(round(c1,3))
        }
    }

    #--- membership descriptives
    if ( ! ( object$plmat) ){
        cat("-----------------------------------------------------------------\n")
        cat("Membership Function Descriptives \n")
        obji <- object$classdesc
        sirt_summary_print_objects(obji=obji, digits=3, rownames_null=FALSE)
    }

    #-- item parameters
    cat("-----------------------------------------------------------------\n")
    cat("Item Parameters \n")
    obji <- object$item
    sirt_summary_print_objects(obji=obji, digits=3, from=2)

    # close sink
    sirt_csink( file=file )
}

## File Name: rasch_mml2_mstep_calc_likelihood.R
## File Version: 0.06

rasch_mml2_mstep_calc_likelihood <- function(G, pjk.M, n.jk, r.jk )
{
    qjk.M <- 1 - pjk.M
    ll0 <- rep(0,G)
    for (gg in 1:G){
        ll0[gg] <- sum( r.jk[,,gg] * log(pjk.M) + (n.jk[,,gg]-r.jk[,,gg])*log(qjk.M))
    }
    res <- sum(ll0)
    return(res)
}

## File Name: rasch_mml2_prior_information.R
## File Version: 0.04

rasch_mml2_prior_information <- function(prior.a, prior.b, prior.c,
    prior.d)
{
    a <- rasch_mml2_prior_information_generate_string(prior=prior.a,
                    distribution="N")
    b <- rasch_mml2_prior_information_generate_string(prior=prior.b,
                    distribution="N")
    c <- rasch_mml2_prior_information_generate_string(prior=prior.c,
                    distribution="Beta")
    d <- rasch_mml2_prior_information_generate_string(prior=prior.d,
                    distribution="Beta")
    priors <- list( a=a, b=b, c=c, d=d)
    return(priors)

}

## File Name: resp_groupwise.R
## File Version: 0.121

resp_groupwise <- function(resp, group, items_group)
{
    dat <- resp
    if (is.null(names(items_group))){
        names(items_group) <- colnames(resp)
    }
    NI <- length(items_group)
    N <- nrow(resp)
    for (ii in seq_len(NI)){
        item_ii <- paste(names(items_group)[ii])
        group_ii <- items_group[[ii]]
        NG <- length(group_ii)
        for (gg in seq_len(NG)){
            ind_gg <- which(group==group_ii[gg])
            v1 <- rep(NA, N)
            v1[ ind_gg ] <- resp[ ind_gg, item_ii]
            dat[ ind_gg, item_ii ] <- NA
            name_gg <- paste0(item_ii, '_Gr', group_ii[gg])
            dat <- data.frame(dat, v1)
            colnames(dat)[ ncol(dat) ] <- name_gg
        }
    }
    dat <- dat[, colSums(!is.na(dat)) > 0 ]
    dat <- dat[, sort(colnames(dat)) ]
    return(dat)
}

## File Name: decategorize.R
## File Version: 0.124

#* decategorize
decategorize <- function( dat, categ_design=NULL )
{
    # preliminaries
    dat4 <- dat3 <- dat
    dfr <- categ_design

    #** handle categories
    if ( ! is.null( dfr ) ){
        vars <- sort(unique(paste(dfr$variable)))
        VV <- length(vars)
        for (vv in 1L:VV){
            dfr.vv <- dfr[ paste(dfr$variable)==vars[vv], ]
            dat4[, vars[vv] ] <- dfr.vv[ match( dat3[,vars[vv]], dfr.vv$recode ), 'orig']
        }
    }
    return(dat4)
}

## File Name: linking_haebara_gradient_function_R.R
## File Version: 0.294


linking_haebara_gradient_function_R <- function(NI, NS, dist, aM, bM, theta,
    prob_theta, est_pars, wgtM, a, b, mu, sigma, eps, index_a, index_b,
    index_mu, index_sigma, parnames, NP, pow=1 )
{

    # logit(p)=a*(th-b)
    # th=SIG*TH+MU=> logit(p)=a*(SIG*TH+MU-b)=a*SIG*(TH-(-MU)/SIG-b/SIG)
    grad <- rep(0, 2*NI+2*(NS-1) )
    names(grad) <- parnames
    for (ii in 1L:NI){
        for (ss in 1L:NS){
            if (est_pars[ii,ss]){
                p_obs <- stats::plogis( aM[ii,ss] * (theta - bM[ii,ss] ) )
                a_exp <- a[ii] * sigma[ss]
                b_exp <- ( b[ii] - mu[ss] ) / sigma[ss]
                p_exp <- stats::plogis( a_exp * (theta - b_exp ) )
                der <- p_exp*(1-p_exp)
                if (dist=='L2'){
                    der_basis <- -2*(p_obs - p_exp)*prob_theta*der
                }
                if (dist=='L1'){
                    diff2 <- p_obs - p_exp
                    dist2 <- diff2^2
                    der_basis <- -(dist2+eps)^(-.5)*diff2*prob_theta*der
                }
                if (dist=='Lp'){
                    diff2 <- p_obs - p_exp
                    dist2 <- diff2^2
                    der_basis <- -pow*(dist2+eps)^(pow/2-1)*diff2*prob_theta*der
                }
                w_t <- wgtM[ii,ss]
                #- a
                der_t <- sigma[ss]*theta + mu[ss] - b[ii]
                ind <- index_a[ii]
                grad[ind] <- grad[ind] + w_t*sum(der_basis*der_t )
                #- b
                der_t <- -a[ii]
                ind <- index_b[ii]
                grad[ind] <- grad[ind] + w_t*sum(der_basis*der_t )
                #- mu
                if (ss>1){
                    der_t <- a[ii]
                    ind <- index_mu[ss-1]
                    grad[ind] <- grad[ind] + w_t*sum(der_basis*der_t )
                }
                #- sigma
                if (ss>1){
                    der_t <- a[ii]*theta
                    ind <- index_sigma[ss-1]
                    grad[ind] <- grad[ind] + w_t*sum(der_basis*der_t )
                }
            }  # end est_parm[ii,ss]
        }  # end ss
    }  # end ii

    #--- output
    return(grad)
}

## File Name: mgsem.R
## File Version: 0.554

mgsem <- function(suffstat, model, data=NULL, group=NULL, weights=NULL,
        estimator="ML", p_me=2, p_pen=1, pen_type="scad",
        diffpar_pen=NULL, pen_sample_size=TRUE, a_scad=3.7, eps_approx=1e-3, comp_se=TRUE,
        se_delta_formula=FALSE, prior_list=NULL, hessian=TRUE,
        fixed_parms=FALSE, cd=FALSE,
        cd_control=list(maxiter=20, tol=5*1e-4, interval_length=0.05, method="exact"),
        partable_start=NULL, num_approx=FALSE, technical=NULL, control=list() )
{
    #- pen_type: lasso, scad or none

    #*** preliminaries
    CALL <- match.call()
    s1 <- Sys.time()

    test <- FALSE
    # test <- TRUE

    #** coordinate decent
    if (cd){
        fixed_parms <- TRUE
    }

    #*** process data if suffstat not available
    if (missing(suffstat)){
        data_proc <- mgsem_proc_data(data=data, group=group, weights=weights)
        groups <- data_proc$groups
        suffstat <- data_proc$suffstat
        weights <- data_proc$weights
        is_data <- TRUE
        G <- length(groups)
    } else {
        groups <- names(suffstat)
        G <- length(groups)
        if (is.null(groups)){
            groups <- paste0('Group',1L:G)
        }
        data_proc <- NULL
        is_data <- FALSE
    }
    technical$is_data <- is_data

    #*** compute covariance matrix of sufficient statistics
    suffstat_vcov <- mgsem_suffstat_covariance_matrix(suffstat=suffstat)

    #*** process technical defaults
    res <- mgsem_proc_technical(technical=technical, control=control, p_me=p_me,
                p_pen=p_pen, eps_approx=eps_approx, suffstat=suffstat,
                estimator=estimator, diffpar_pen=diffpar_pen, cd_control=cd_control)
    technical <- res$technical
    technical$estimator <- estimator
    control <- res$control
    eps_approx <- res$eps_approx
    p <- res$p
    cd_control <- res$cd_control

    #*** process sufficient statistics
    res <- mgsem_proc_suffstat(suffstat=suffstat)
    suffstat <- res$suffstat
    G <- res$G
    I <- res$I
    N <- res$N
    N_group <- res$N_group
    W <- res$W
    random_sd <- -9
    if (test){
        random_sd <- 1e-1
    }

    #*** process model specification
    res <- mgsem_proc_model(model=model, G=G, prior_list=prior_list,
                    technical=technical, N_group=N_group, random_sd=random_sd,
                    pen_type=pen_type, fixed_parms=fixed_parms,
                    partable_start=partable_start, diffpar_pen=diffpar_pen,
                    pen_sample_size=pen_sample_size, W=W)
    model <- res$model
    partable <- res$partable
    NP <- res$NP
    coef <- res$coef
    ND <- res$ND
    D <- res$D
    is_B <- res$is_B
    technical <- res$technical
    types <- res$types
    difflp_info <- res$difflp_info
    loop_parms <- res$loop_parms

    if (estimator=='ML'){
        eval_fun <- 'mgsem_loglike_suffstat'
        grad_param_fun <- 'mgsem_loglike_suffstat_derivative_parameter'
        grad_suffstat_fun <- 'mgsem_loglike_suffstat_derivative'
    }
    if (estimator=='ME'){
        eval_fun <- 'mgsem_loss_function_suffstat'
        grad_param_fun <- 'mgsem_loss_function_suffstat_derivative_parameter'
        grad_suffstat_fun <- 'mgsem_loss_function_suffstat'
    }

    if (technical$use_deriv){
        grad_fun <- mgsem_grad_fun
    } else {
        grad_fun <- mgsem_grad_fun_numeric_approx
    }

    x <- coef
    opt_fun_args <- list(NP=NP, ND=ND, suffstat=suffstat, model=model,
                        partable=partable, G=G, I=I, D=D, is_B=is_B,
                        N=N, N_group=N_group,
                        estimator=estimator, eval_fun=eval_fun,
                        grad_param_fun=grad_param_fun,
                        grad_suffstat_fun=grad_suffstat_fun,
                        technical=technical, p_pen=p_pen, p_me=p_me,
                        p=p, eps_approx=eps_approx, num_approx=num_approx,
                        prior_list=prior_list, use_penalty=TRUE, types=types,
                        difflp_info=difflp_info, loop_parms=loop_parms,
                        pen_type=pen_type, a_scad=a_scad )

    #- test function
    res <- mgsem_test_fun(test=test, coef=coef, opt_fun_args=opt_fun_args)

    #**** estimation
    if (! fixed_parms){

        #- define lower and upper bounds
        ind <- which(partable$unique>0)
        lower <- partable[ ind, 'lower' ]
        upper <- partable[ ind, 'upper' ]

        #- use optimizer
        opt_res <- sirt_optimizer(optimizer=technical$optimizer, par=x, fn=mgsem_opt_fun,
                        grad=grad_fun, opt_fun_args=opt_fun_args,
                        method='L-BFGS-B', lower=lower, upper=upper,
                        hessian=hessian, control=control )
    } else {
        #**** no estimation
        ll <- mgsem_opt_fun(x=x, opt_fun_args=opt_fun_args)
        opt_res <- list(par=x, opt_value=ll)

        #--- coordinate descent
        if (cd){
            opt_res <- mgsem_cd_opt(x=x, opt_fun_args=opt_fun_args, tol=cd_control$tol,
                                maxiter=cd_control$maxiter, method=cd_control$method,
                                interval_length=cd_control$interval_length)
        }
    }

    #-- collect results
    coef <- opt_res$par
    opt_value <- opt_res$value
    partable <- mgsem_coef2partable(coef=coef, partable=partable)
    model <- mgsem_partable2model(partable=partable, model=model)

    #-- evaluate optimization function and gradient
    opt_fun_args$partable <- partable
    opt_fun_args$model <- model
    opt_fun_output <- mgsem_opt_fun(x=coef, opt_fun_args=opt_fun_args, output_all=TRUE)

    implied <- opt_fun_output$implied
    est_tot <- opt_fun_output$est_tot
    grad_fun_output <- mgsem_grad_fun(x=coef, opt_fun_args=opt_fun_args, output_all=TRUE)

    #-- vcov for estimator='ME'
    res <- mgsem_vcov_me(coef=coef, opt_fun_args=opt_fun_args,
                            suffstat_vcov=suffstat_vcov, comp_se=comp_se,
                            se_delta_formula=se_delta_formula)
    vcov <- res$vcov
    se <- res$se
    comp_se_me <- res$comp_se_me
    me_delta_method <- res

    #-- residual statistics
    residuals <- mgsem_output_proc_residuals(implied=implied, suffstat=suffstat)

    #-- compute case-wise log-likelihood
    case_ll <- mgsem_output_proc_casewise_likelihood(data_proc=data_proc,
                    implied=implied, estimator=estimator)

    #-- computation of standard errors
    res <- mgsem_observed_information(coef=coef, opt_fun_args=opt_fun_args,
                    technical=technical, comp_se=comp_se, comp_se_me=comp_se_me)
    if (!comp_se_me){
        vcov <- res$vcov
        comp_se <- res$comp_se
        se <- res$se
    }
    info_loglike <- res$info_loglike
    info_loglike_pen <- res$info_loglike_pen

    #-- information criteria
    ic <- mgsem_ic(opt_fun_output=opt_fun_output, opt_fun_args=opt_fun_args,
                        partable=partable, technical=technical)

    #--- output
    s2 <- Sys.time()
    res <- list(coef=coef, vcov=vcov, se=se,
                    partable=partable, model=model, suffstat=suffstat,
                    opt_res=opt_res, opt_value=opt_value, implied=implied,
                    est_tot=est_tot, residuals=residuals, opt_fun_output=opt_fun_output,
                    ic=ic, info_loglike=info_loglike, info_loglike_pen=info_loglike_pen,
                    estimator=estimator, p_pen=p_pen, p_me=p_me,
                    pen_type=pen_type, eps_approx=eps_approx,
                    technical=technical, comp_se=comp_se, groups=groups, group=group,
                    data=data, data_proc=data_proc, case_ll=case_ll,
                    suffstat_vcov=suffstat_vcov, me_delta_method=me_delta_method,
                    CALL=CALL, s1=s1, s2=s2)
    class(res) <- 'mgsem'
    return(res)
}

## File Name: rasch_pairwise_zerosum.R
## File Version: 0.02


rasch_pairwise_zerosum <- function(eps)
{
    b1 <- - log(eps)
    b2 <- b1 - mean(b1)
    eps <- exp(-b2)
    #- output
    return(eps)
}

## File Name: rm_facets_calc_loglikelihood.R
## File Version: 0.14

rm_facets_calc_loglikelihood <- function( tau.item, a.rater, Qmatrix, b.item, VV,
            K, I, TP, a.item, b.rater, item.index, rater.index, theta.k, RR, dat2,
            dat2.resp, pi.k=NULL, dat2.ind.resp, mu=NULL, sigma=NULL, b.rater.center,
            a.rater.center, a.item.center, a_lower, a_upper )
{
    #--- adjust pi.k probabilities
    if ( ( ! is.null(mu) ) | ( ! is.null(sigma) ) ){
        pi.k <- sirt_dnorm_discrete( x=theta.k, mean=mu, sd=sigma )
    }
    #--- center parameters
    a.item <- rm_squeeze(x=a.item, lower=a_lower, upper=a_upper )
    a.rater <- rm_squeeze(x=a.rater, lower=a_lower, upper=a_upper )
    a.rater <- rm_center_vector( vec=a.rater, center_type=a.rater.center, do_log=TRUE )
    a.item <- rm_center_vector( vec=a.item, center_type=a.item.center, do_log=TRUE )
    b.rater <- rm_center_vector( vec=b.rater, center_type=b.rater.center, do_log=FALSE )

    #--- calculate probabilities
    probs <- rm_facets_calcprobs( tau.item=tau.item, b.rater=b.rater, Qmatrix=Qmatrix,
                        b.item=b.item, VV=VV, K=K, I=I, TP=TP, a.item=a.item,
                        a.rater=a.rater, item.index=item.index,
                        rater.index=rater.index, theta.k=theta.k, RR=RR )

    #--- calculate posterior
    res <- rm_posterior( dat2=dat2, dat2.resp=dat2.resp, TP=TP, pi.k=pi.k, K=K, I=I,
                            probs=probs, dat2.ind.resp=dat2.ind.resp )

    #--- output
    res <- list(ll=res$ll, a.item=a.item, pi.k=pi.k, a.rater=a.rater,
                    b.rater=b.rater, a.item=a.item)
    return(res)
}

## File Name: linking_haberman_als.R
## File Version: 0.659



#--- alternating least squares for Haberman linking
linking_haberman_als <- function(logaM, wgtM, maxiter, conv,
            progress, est.type, cutoff, reference_value=0, adjust_main_effects=FALSE,
            estimation="OLS", lts_prop=.5, vcov=TRUE)
{
    non_null <- sum( abs(logaM) > 0, na.rm=TRUE)
    iter <- 0
    parchange <- 1000
    NS <- ncol(logaM)
    NI <- nrow(logaM)
    #-- initial values study parameters
    logaAt <- rep(0,NS)
    At_inits <- TRUE
    if ( At_inits ){
        logaAt <- weighted_colMeans( mat=logaM, wgt=wgtM )
        logaAt <- logaAt - logaAt[1]
    }
    wgtM0 <- wgtM
    wgt_adj <- 1 + 0*wgtM
    eps <- 1E-5
    wgtM <- wgtM + eps
    #- initial OLS estimation of item parameters
    logaAt_M <- sirt_matrix2( x=logaAt, nrow=NI)
    logaM_adj1 <- logaM - logaAt_M
    logaj <- weighted_rowMeans( mat=logaM_adj1, wgt=wgtM )
    if (! ( estimation %in% c('L0','BSQ','HUB') ) ) {
        cutoff <- Inf
    }
    if (estimation=='L0'){
        res1 <- L0_polish(x=logaM, tol=cutoff, type=1)
        wgtM0 <- wgtM <- res1$wgt
        logaM1 <- res1$x_update
    }
    I <- nrow(wgtM)
    NS <- ncol(wgtM)
    abs_a_change <- 1

    #*** begin algorithm
    while( ( parchange > conv ) & (iter < maxiter) ){
        logaAt0 <- logaAt
        logaj_old <- logaj
        #--- estimate item parameters
        res <- linking_haberman_als_residual_weights( logaj=logaj, logaAt=logaAt,
                    logaM=logaM, cutoff=cutoff, wgtM0=wgtM0, eps=eps,
                    estimation=estimation, lts_prop=lts_prop)
        loga_resid <- res$loga_resid
        wgtM <- res$wgtM
        wgt_adj <- res$wgt_adj

        # estimation
        logaAt_M <- sirt_matrix2( x=logaAt, nrow=NI)
        logaM_adj1 <- logaM - logaAt_M
        logaj <- rep(NA,I)
        if (estimation %in% c('OLS','BSQ','HUB')){
            logaj <- weighted_rowMeans( mat=logaM_adj1, wgt=wgtM )
        }
        if (estimation %in% c('MED')){
            for (ii in 1L:I){
                logaj[ii] <- linking_haberman_compute_median(x=logaM_adj1[ii,],
                                            w=wgtM[ii,])
            }
        }
        if (estimation %in% c('L0','L1')){
            if (estimation=='L1'){
                logaM1 <- logaM
            }
            res1 <- L1_polish(x=logaM1, type=1)
            logaj <- res1$row
            logaAt <- res1$col
        }
        if (estimation %in% c('LTS')){
            for (ii in 1L:I){
                logaj[ii] <- linking_haberman_compute_lts_mean(x=logaM_adj1[ii,],
                                        w=wgtM[ii,], lts_prop=lts_prop)
            }
        }

        #--- estimate linking parameters
        logaMadj <- logaM - logaj
        res <- linking_haberman_als_residual_weights( logaj=logaj, logaAt=logaAt,
                    logaM=logaM, cutoff=cutoff, wgtM0=wgtM0, eps=eps,
                    estimation=estimation, lts_prop=lts_prop)
        wgtM <- res$wgtM
        wgt_adj <- res$wgt_adj
        loga_resid <- res$loga_resid
        cutoff_used <- res$cutoff
        k_estimate <- res$k_estimate

        #* estimation of parameters
        if (estimation %in% c('OLS','BSQ','LTS','HUB')){
            logaAt <- weighted_colMeans( mat=logaMadj, wgt=wgtM )
        }
        if (estimation %in% c('MED')){
            for (ss in 1L:NS){
                logaAt[ss] <- linking_haberman_compute_median(x=logaMadj[,ss],
                                            w=wgtM[,ss])
            }
        }
        if ( ! adjust_main_effects){
            logaAt[1] <- reference_value
        } else {
            ma <- logaAt[1]
            logaAt <- logaAt - ma + reference_value
        }
        a_change <- logaAt - logaAt0

        #- stabilize convergence
        if (iter>50){
            if (max(abs(a_change)) >=abs_a_change ){
                a_change <- ifelse( abs(a_change) >=abs_a_change, .95*abs_a_change,
                                        a_change )
                logaAt <- logaAt0 + a_change
            }
        }
        parchange <- abs_a_change <- max(abs(a_change))
        if (progress){
            cat( paste0( '** ', est.type, ' estimation | Iteration ', iter+1, ' | ',
                'Max. parameter change=', round( parchange, 6 ) ), '\n')
            utils::flush.console()
        }
        iter <- iter + 1

        #-- stop iterations
        if (estimation %in% c('L0','L1')){
            break
        }

    }
    if (progress){
        cat('\n')
    }

    #------- summary of regression

    # residual SD
    selitems <- which( rowSums( 1 - is.na( loga_resid ) ) > 1 )

    #--- calculation of standard errors of regression coefficients
    sd_loga <- stats::sd(logaAt)
    if ( sd_loga < 1E-10 ){
        res <- list( vcov=0*diag(NS-1), se=rep(0,NS-1)  )
    } else {
        res <- linking_haberman_als_vcov( regr_resid=loga_resid,
                    regr_wgt=wgtM, selitems=selitems, transf_pars=logaAt,
                    estimation=estimation, vcov=vcov, NS=NS)
    }
    #--- item statistics
    item_stat <- data.frame( study=colnames(wgtM0) )
    item_stat$N_items <- colSums( wgtM0 > 0, na.rm=TRUE)
    item_stat$sumwgt_items <- colSums( (wgtM0 > 0)*wgt_adj, na.rm=TRUE )
    if (estimation!='LTS'){
        lts_prop <- 1
    }
    #*** end algorithm
    res <- list( logaAt=logaAt, logaj=logaj, loga_resid=loga_resid, loga_wgt=wgtM,
                loga_wgt_adj=wgt_adj, vcov=res$vcov, se=c(NA, res$se),
                item_stat=item_stat, iter=iter, cutoff=cutoff_used,
                estimation=estimation, k_estimate=k_estimate, lts_prop=lts_prop)
    return(res)
}

## File Name: mgsem_output_proc_residuals.R
## File Version: 0.06


mgsem_output_proc_residuals <- function(implied, suffstat)
{
    G <- length(implied)
    residuals_groupwise <- list()
    fit_total <- list(fit_Mu=0, fit_Sigma=0)
    for (gg in 1L:G){
        N <- suffstat[[gg]]$N
        res1 <- list()
        res1$Mu <- suffstat[[gg]]$M-implied[[gg]]$Mu
        res1$Sigma <- suffstat[[gg]]$S-implied[[gg]]$Sigma

        fit_Mu <- N*sum(suffstat[[gg]]$weights_M*res1$Mu^2)
        res1$fit_Mu <- fit_Mu
        fit_total$fit_Mu <- fit_total$fit_Mu+res1$fit_Mu

        fit_Sigma <- sum(res1$Sigma^2*suffstat[[gg]]$weights_M)
        res1$fit_Sigma <- N*sum( mgsem_vech(x=fit_Sigma) )
        fit_total$fit_Sigma <- fit_total$fit_Sigma+res1$fit_Sigma

        residuals_groupwise[[gg]] <- res1
    }
    fit_total$fit_both <- fit_total$fit_Mu + fit_total$fit_Sigma

    #--- output
    res <- list(fit_total=fit_total, residuals_groupwise=residuals_groupwise)
    return(res)
}

## File Name: fit.logistic.R
## File Version: 2.15



#-- fit logistic model
fit.logistic <- function( freq.correct, wgt, scores, item.p,
    conv=.0001, maxit=100, progress=TRUE )
{
    sc.adj <- min( diff(scores) ) / 3
    scores1 <- scores
    freq.correct <- as.matrix(freq.correct)
    RR <- nrow(freq.correct)
    CC <- ncol(freq.correct)
    scores1[ scores==0 ] <- sc.adj
    scores1[ scores==1 ] <- 1 - sc.adj
    dfr0 <- data.frame( stud.index=rep(1:RR, CC),
                item.index=rep(1:CC,each=RR),
                stud.p=rep( scores1,CC),
                item.p=rep( item.p, each=RR ),
                wgt=matrix( as.matrix( wgt ), RR*CC, 1 ),
                p0=matrix( as.matrix(freq.correct ), RR*CC, 1 ) )
    if (progress){
        cat("\n*******Logistic Model***********\n")
    }
    theta <- stats::qlogis( scores1 )
    b <- stats::qlogis( item.p )

    #*** begin algorithm
    numdiff.parm <- .001
    max.increment <- 1
    deviation <- 1000
    iter <- 0
    # adjust freq.correct if necessary
    adj <- min( c( min( freq.correct[ freq.correct > 0 ] ),
            1-max( freq.correct[ freq.correct < 1 ] ) ) ) / 3
    freq.correct1 <- freq.correct
    freq.correct1[ freq.correct==0 ] <- adj
    freq.correct1[ freq.correct==1 ] <- 1-adj
    while( ( iter < maxit) & ( deviation > conv ) ){
        b0 <- b
        theta0 <- theta
        # update theta
        res <- .update.theta.logistic( theta, b, freq.correct=freq.correct1, wgt,
                    numdiff.parm, max.increment=max.increment)
        theta <- res$theta
        ll <- res$ll
        # update b
        res <- .update.b.logistic( theta, b, freq.correct=freq.correct1, wgt,
                    numdiff.parm, max.increment=max.increment)
        b <- res$b
        b <- b - mean(b)
        deviation <- max( abs( c( theta-theta0, b - b0 )) )
        if (progress){
            cat( "Iteration", iter, "- Deviation=",  round( deviation, 6 ), "\n")
            utils::flush.console()
        }
        iter <- iter + 1
    }  # end algorithm
    #*************************
    dfr0$irtfitted <- matrix( plogis( outer(theta, b, "+") ), RR*CC, 1 )[,1]
    Y <- 0*freq.correct
    Y <- as.matrix(Y)
    Y[ as.matrix(dfr0[, c("stud.index", "item.index" ) ] ) ] <- dfr0$irtfitted
    G <- Y
    # fitted frequency table
    dfr0$stud.p <- rep( theta,CC)
    dfr0$item.p <- rep( b, each=RR )
    colnames(dfr0)[ colnames(dfr0)=="stud.p" ] <- "person.sc"
    colnames(dfr0)[ colnames(dfr0)=="item.p" ] <- "item.sc"
    colnames(dfr0)[ colnames(dfr0)=="p0" ] <- "freq"
    colnames(dfr0)[ colnames(dfr0)=="irtfitted" ] <- "freq.fitted"
    dfr0 <- dfr0[ order( dfr0$stud.index * 1000 + dfr0$item.index ), ]
    # deviation criterion
    wgt1 <- ( wgt / colSums( wgt ) ) / ncol(wgt)
    fit <- sqrt( sum( ( freq.correct - G  )^2 * wgt1  ) )
    # calculate likelihood
    ll <- list( ll.ind=.calc.ll.isop( freq.correct, wgt, irtfitted=freq.correct ) )
    ll$ll.logistic <- .calc.ll.isop( freq.correct, wgt, irtfitted=G )
    NW <- mean( colSums(wgt) )
    ll$llcase.ind <- ll$ll.ind/NW
    ll$llcase.logistic <- ll$ll.logistic/NW

    # output
    res <- list( "fX"=G, "ResX"=freq.correct - G,
                "fit"=fit, "item.sc"=b,
                "person.sc"=theta, "ll"=ll, "freq.fitted"=dfr0)
    return(res)
}

## File Name: modelfit.sirt.R
## File Version: 1.138


# model fit in sirt
modelfit.sirt <- function( object )
{
    #****
    # object of class tam.mml, tam.mml.2pl or tam.fa
    # Note that only dichotomous responses are allowed
    if (inherits(object,c("tam.mml","tam.mml.2pl")) ){
        mod <- object
        posterior <- mod$hwt
        probs <- mod$rprobs
        dat <- mod$resp
        dat[ mod$resp.ind==0 ] <- NA
    }

    #--- rasch.mml
    if (inherits(object,"rasch.mml") ){
        mod <- object
        posterior <- mod$f.qk.yi
        prob1 <- mod$pjk
        probs <- array( NA, dim=c( ncol(prob1), 2, nrow(prob1)) )
        probs[, 2, ] <- t(prob1)
        probs[, 1, ] <- 1 - t(prob1)
        dat <- mod$dat
    }

    #--- rasch.mirtlc
    if (inherits(object,"rasch.mirtlc")){
        mod <- object$estep.res
        posterior <- mod$f.qk.yi
        prob1 <- mod$pjk
        probs <- array( NA, dim=c( ncol(prob1), 2, nrow(prob1)) )
        probs[, 2, ] <- t(prob1)
        probs[, 1, ] <- 1 - t(prob1)
        dat <- object$dat
    }

    #--- rasch.pml
    if ( ! inherits(object,"rasch.pml") ){
        pmlobject <- NULL
    } else {
        data <- NULL
        posterior <- NULL
        probs <- NULL
        pmlobject <- object
    }

    #--- smirt
    if (inherits(object,"smirt") ){
        # note that for polytomous response data some adaptations are
        # necessary: see modelfit in the CDM package
        mod <- object
        probs <- mod$probs
        posterior <- mod$f.qk.yi
        dat <- mod$dat
    }

    #--- smirt
    if (inherits(object,"gom") ){
        mod <- object
        probs <- mod$probs
        posterior <- mod$f.qk.yi
        dat <- mod$dat
    }

    #--- rm.facets
#    if (inherits(object,"rm.facets") ){
#        mod <- object
#        probs <- mod$probs
#        posterior <- mod$f.qk.yi
#        dat <- mod$procdata$dat2.NA
#    }

    #--- mirt
    if (inherits(object, c("ConfirmatoryClass","ExploratoryClass","SingleGroupClass"))){
        mod <- object
        mod <- mirt.wrapper.posterior(mod)
        probs <- mod$probs
        posterior <- mod$f.qk.yi
        dat <- mod$dat
    }

    #--- R2noharm, noharm.sirt
    if ( inherits(object, c("R2noharm","noharm.sirt")) ){
        # exclusion criteria for noharm.sirt
        if ( inherits(object,"noharm.sirt")) {
            if ( object$estpars$estPsi > 0 ){
                stop("Model fit cannot be calculated because of correlated residuals")
            }
            if ( ! ( object$wgtm.default ) ){
                stop( paste0( "Model fit cannot be calculated because not all",
                                    "item pairs are used for estimation") )
            }
        }
        # evaluation of posterior
        mod <- R2noharm.EAP(noharmobj=object, theta.k=seq(-6, 6, len=15 ),
                        print.output=FALSE )
        probs <- aperm( mod$probs, c(1,3,2) )
        posterior <- mod$posterior
        dat <- object$dat
    }
    # calculate modelfit.cor
    if ( inherits(object,"rasch.pml") ){
        res <- modelfit.cor.sirt.pml( data=dat, posterior=posterior, probs=probs,
                        pmlobject=pmlobject)
    } else {
        res <- CDM::modelfit.cor2( data=dat, posterior=posterior, probs=probs )
    }

    #--- model output
    class(res) <- "modelfit.sirt"
    return(res)
}

## File Name: rasch.mml2.R
## File Version: 7.717


# Semiparametric Maximum Likelihood Estimation in the Rasch type Model
# item discrimination and guessing parameter can be fixed
rasch.mml2 <- function( dat, theta.k=seq(-6,6,len=21), group=NULL, weights=NULL,
        constraints=NULL, glob.conv=10^(-5), parm.conv=10^(-4), mitermax=4,
        mmliter=1000, progress=TRUE, fixed.a=rep(1,ncol(dat)),
        fixed.c=rep(0,ncol(dat)), fixed.d=rep(1,ncol(dat)), fixed.K=rep(3,ncol(dat)),
        b.init=NULL, est.a=NULL, est.b=NULL, est.c=NULL, est.d=NULL, min.b=-99,
        max.b=99, min.a=-99, max.a=99, min.c=0, max.c=1, min.d=0, max.d=1,
        prior.b=NULL, prior.a=NULL, prior.c=NULL, prior.d=NULL, est.K=NULL,
        min.K=1, max.K=20, min.delta=-20, max.delta=20, beta.init=NULL, min.beta=-8,
        pid=1:(nrow(dat)), trait.weights=NULL, center.trait=TRUE, center.b=FALSE, alpha1=0, alpha2=0,
        est.alpha=FALSE, equal.alpha=FALSE, designmatrix=NULL, alpha.conv=parm.conv,
        numdiff.parm=0.00001, numdiff.alpha.parm=numdiff.parm,
        distribution.trait="normal", Qmatrix=NULL,
        variance.fixed=NULL, variance.init=NULL,
        mu.fixed=cbind(seq(1,ncol(Qmatrix)),rep(0,ncol(Qmatrix))),
        irtmodel="raschtype", npformula=NULL, npirt.monotone=TRUE,
        use.freqpatt=is.null(group), delta.miss=0, est.delta=rep(NA,ncol(dat)),
        nimps=0, ... )
{

    # specifications
    conv1 <- parm.conv
    nplausible=5
    dat <- as.matrix(dat)
    adaptive.quadrature <- FALSE
    CALL <- match.call()
# a0 <- Sys.time()
    # models
    npirt <- ramsay.qm <- FALSE
    variance_fixed_input <- ! is.null(variance.fixed)
    I <- ncol(dat)

    irtmodel_missing <- c("missing1","missing2")
#    if ( esttype=="pseudoll" ){
#        if ( is.null( group) ){
#                group <- rep(1,nrow(dat) )
#                            }
#                    }

#    m1r <- FALSE
#    if (irtmodel=="missing1r"){
#        m1r <- TRUE
#        irtmodel <- "missing1"
#            }
    if ((nimps>0) | (irtmodel=="pseudoll") ){
        use.freqpatt <- FALSE
    }
    if ( irtmodel=="ramsay.qm" ){
        ramsay.qm <- TRUE
        kG <- NULL
        }
    if ( irtmodel=="npirt" ){
            npirt <- TRUE
            I <- ncol(dat)
            if ( ! is.null(npformula) ){
                if ( length( npformula)==1 ){
                    npformula <- rep( npformula, I )
                                }
                npformula0 <- npformula
                npformula <- list( 1:I)
                for (ii in 1:I){
                    npformula[[ii]] <- stats::as.formula( npformula0[ii] )
                                }
                                }
                npmodel <- list(1:I)
                }
    dat.resp <- 1 - is.na(dat)
    if (is.null(Qmatrix)){
        D <- 1
    } else {
        D <- ncol(Qmatrix)
    }
    if (irtmodel %in% c(irtmodel_missing) ){
        D <- 2
        if (is.vector(theta.k)){
            theta.k <- expand.grid( theta.k, theta.k )
        }
        dat[ dat==9 ] <- 2
        # init b parameters
        b.init <- - stats::qlogis( colMeans( dat==1, na.rm=TRUE ) )
        # init beta parameters
        if ( is.null(beta.init) ){
            beta <-  stats::qlogis( colMeans( dat==2, na.rm=TRUE ) +  1E-3 )
        } else {
            beta <- beta.init
        }
    }
    # multidimensional model
    theta.k <- rasch_mml2_create_theta_k(Qmatrix=Qmatrix, theta.k=theta.k)

    if (is.data.frame(theta.k)){
        theta.k <- as.matrix(theta.k)
    }
    if (D > 1){
        if ( is.null(variance.fixed) & ( sum(est.a) > 0) ){
            variance.fixed <- as.matrix( cbind( 1:D, 1:D, 1 ) )
                }
            }
    Sigma.cov <- diag(D)
    if ( ! is.null(variance.init) ){
        Sigma.cov <- variance.init
                    }
    mu <- rep(0,D)
# cat("114") ; a1 <- Sys.time(); print(a1-a0) ; a0 <- a1
#    ramsay.qm <- FALSE
    if ( ! ramsay.qm) { raschtype <- TRUE      }
    if (ramsay.qm | npirt ){
        raschtype <- FALSE
        # no alpha, a, c or d parameters can be estimated
        est.alpha <- FALSE
        est.a <- est.c <- est.d <- NULL
        pow.qm <- 1    # This parameter is ignored in analyses
                    }
    # computation time
    s1 <- Sys.time()
    if (est.alpha){
            if (is.null(alpha1) ){ alpha1 <- 0 }
            if (is.null(alpha2) ){ alpha2 <- 0 }
                }

    #*** some data checks
    ag1 <- NULL
    if( max( colMeans( is.na( dat ) ) )==1 ){
        stop("Remove items which have no observations!")
    }
    if ( ! is.null(group) ){
            t1 <- table(sort(group) )
            group.orig <- group
            group <- match( group.orig, sort(unique( group.orig)) )
            ag1 <- stats::aggregate( group, list( group.orig), mean )
            colnames(ag1) <- c("group", "groupindex" )
                            }
    # center trait: if there exists constraints, then do not center

    if ( is.null( colnames(dat) ) ){
            colnames(dat) <- paste( "I", 1:ncol(dat), sep="")
                        }
      if ( ! is.null( constraints ) ){
            center.trait <- FALSE
          if( ! is.numeric( constraints[,1] ) ){
            constraints[,1] <- match( paste(constraints[,1]), colnames(dat) )
                                       }
             constraints <- na.omit(constraints)
             constraints <- constraints[ constraints[,1] <=ncol(dat),, drop=FALSE]
                                }
    if ( ! is.null( designmatrix) ){
            if ( ncol(dat) !=nrow(designmatrix) ){
                    stop( "Row dimension of designmatrix should be equal to number of items")
                                }
                        }
    # est.b parameters
    if ( ! is.null(est.b) ){
#        bG <- unique( est.b )
        bG <- unique( setdiff( est.b,0 ))
        if ( is.null( b.init) ){
            b <- rep(0, I ) }  else { b <- b.init }

        designmatrix <- matrix( 0, ncol(dat), length(bG) )
        for (bb in bG){
            # bb <- bG[1]
#            designmatrix[ which( est.b==bb ), bb ] <- 1
            designmatrix[ which( est.b==bb ), match(bb,bG) ] <- 1
                        }
                }
    # set starting values for estimated c and d parameters
    if ( ( sum(est.c) > 0 ) & is.null(fixed.c) ){
        fixed.c[ est.c > 0 ] <- .10
    }
    if ( ( sum(est.d) > 0 ) & is.null(fixed.d) ){
        fixed.d[ est.d > 0 ] <- .95
    }

    #* estimate SD
    est_sd <- FALSE
    if (! is.null(est.a)){
        est_sd <- ( sum(est.a==0,na.rm=TRUE)+sum(is.na(est.a)) > 0 )
    }

    #****************************************************************************************
     WLE <- FALSE
     pure.rasch <- -9   # this parameter is only included for historical reasons of this program.
    # specify weights
    if ( is.null(weights) ){ weights <- rep( 1, nrow(dat) ) }
    # display
    if ( progress & ( npirt ) ){
            cat("------------------------------------------------------------\n")
        cat("Semiparametric Marginal Maximum Likelihood Estimation \n")
        cat("Nonparametric IRT Model (Rossi, Wang & Ramsay, 2002) \n")
            cat("------------------------------------------------------------\n")
        flush.console()
      }
    if ( progress & ( npirt ) ){
            cat("------------------------------------------------------------\n")
            cat("Semiparametric Marginal Maximum Likelihood Estimation \n")
            cat("Missing Data Item Response Model (Mislevy & Wu, 1996) \n")
            cat("------------------------------------------------------------\n")
        flush.console()
      }
    if ( progress & ( ramsay.qm ) ){
            cat("------------------------------------------------------------\n")
        cat("Semiparametric Marginal Maximum Likelihood Estimation \n")
        cat("Quotient Model (Ramsay, 1989) \n")
#        if (normal.trait){ cat("Normal trait distribution \n") } else { cat("Nonparametric trait distribution \n") }
#        if (ramsay.qm){ cat("Log Normal Distribution of Theta with Power of", pow.qm, "\n") }
            cat("------------------------------------------------------------\n")
        flush.console()
      }
    if ( progress & (raschtype) ){
           cat("------------------------------------------------------------\n")
        cat("Semiparametric Marginal Maximum Likelihood Estimation \n")
        if ( est.alpha ){
            cat(paste( "Raschtype Model with generalized logistic link function: Estimation of alpha1 and alpha2 \n") )
                    } else {
            cat(paste( "Raschtype Model with generalized logistic link function: alpha1=",alpha1, ", alpha2=", alpha2, " \n") )
                        }
        if ( sum(est.c) > 0){ cat(paste( "Estimated guessing parameter groups \n") )}  ## estimated guessing parameters
        if ( sum(est.d) > 0){ cat(paste( "Estimated slipping parameter groups \n") )}  ## estimated slipping parameters
            cat("------------------------------------------------------------\n")
        flush.console()
      }
    # revise guessing parameter (if necessary)
    if ( !is.null(fixed.c) ){
        # calculate item means
        itemmean <- colMeans( dat,  na.rm=TRUE )
        if ( any( itemmean < fixed.c) ){
                cat ( "revise fixed guessing estimates\n")
                fixed.c[ itemmean < fixed.c] <- 0
                }
            }

    # data preparations
    if ( ! is.null(group) ){
        use.freqpatt <- FALSE
    }
    if ( ! ( irtmodel %in% irtmodel_missing ) ){
        dp <- data.prep( dat=dat, weights=weights, use.freqpatt=use.freqpatt,
                    standardize_weights=FALSE)
        dat1 <- dp$dat1
        dat2 <- dp$dat2
        dat2.resp <- dp$dat2.resp
        freq.patt <- dp$freq.patt
        n <- dp$n
        I <- dp$I
    }
    se.delta <- NULL
    if ( irtmodel %in% irtmodel_missing ){
        dat1 <- as.data.frame( cbind( "P", weights ) )
        for (ii in 1:I){
            l1 <- dat[,ii]
            l1 <- ifelse ( dat.resp[,ii]==0, 9, l1 )
            dat1[, 1 ] <- paste0( dat1[,1], l1 )
        }
        colnames(dat1) <- c("pattern","Freq")
        dat1 <- as.data.frame(dat1)
        freq.patt <- dat1$pattern
        dat1$Freq <- weights
        dat1$mean <- rowMeans( dat==1 )
        dat2 <- dat
        dat2.resp <- dat.resp
        n <- nrow(dat2)
        I <- ncol(dat2)
        # adjust variance.fixed
        if (!variance_fixed_input){
            if (!is.null(variance.fixed)){
                ind <- which( variance.fixed[,1]==2 & variance.fixed[,2]==2 )
                if (length(ind)>0){
                    variance.fixed <- variance.fixed[-ind,,drop=FALSE]
                }
            }
        }
    }

    #*** pseudolikelihood estimation?
    fracresp <- "pseudoll"
    pseudoll <- 0
    i1 <- sum( ( dat2 > 0 ) * ( dat2 < 1), na.rm=TRUE )
    if (i1 > 1e-10 ){
        if ( fracresp=="pseudoll"){
            pseudoll <- 1
        }
        irtmodel <- "pseudoll"
        if ( is.null(group) ){
            group <- rep( 1, nrow(dat2) )
        }
    }

    # probability weights at theta.k
    if (D==1){
        pi.k <- sirt_dnorm_discrete( x=theta.k )
    }
    if (D > 1){
        pi.k <- sirt_dmvnorm_discrete( x=theta.k, mean=rep(0,D), sigma=Sigma.cov )
    }
    G <- 1
    pi.k <- matrix( pi.k, nrow=length(pi.k), ncol=G )
        # group calculations
        if ( !is.null( group )){
            G <- length( unique( group ) )
            pi.k0 <- pi.k
            pi.k <- matrix( 0, nrow=length(pi.k0), ncol=G)
            for (gg in 1:G){
                pi.k[,gg] <- pi.k0
                              }
                    }
        sd.trait <- mean.trait <- rep(0,G)
        # initial estimates for item difficulties
        if ( is.null(b.init) & is.null(est.b) ){
            b <- - stats::qlogis( colMeans( dat, na.rm=T ) )
                if ( FALSE ){
#                if ( ramsay.qm ){
                        b <-   - log( ( fixed.K * colSums( dat, na.rm=TRUE ) ) /
                                    ( colSums( 1 - dat, na.rm=TRUE ) ) )
                                }
                        }
        if ( (!is.null(b.init) ) & is.null(est.b) ){
                    b <- b.init
                    }
        if ( G==1 ){     group <- rep(1, nrow(dat1)) }
        # missing data indicators
        ind.ii.list <- list(1:I)
        for (ii in 1:I){
            ind.ii.list[[ii]] <- which( dat2.resp[,ii]==1 )
                        }
        mean.trait <- rep(0,G)
        sd.trait <- rep(1,G)
        # initial iteration index
        iter <- 0
        par.change <- dev.change <- 3
        dev <- 99
        apmax <- 0
        maxalphachange <- 1
          # display
          disp <- "...........................................................\n"

#        old_increment.d <- old_increment.c <- rep( .2, I )
    if( sum( est.d ) > 0 ){
        old_increment.d <- rep( .2, length( unique( est.d[ est.d > 0 ] ) ) )
            }
    if( sum( est.c ) > 0 ){
        old_increment.c <- rep( .2, length( unique( est.c[ est.c > 0 ] ) ) )
            }
    old_increment_b <- rep( 2, I )

    h <- numdiff.parm

    # initialize standard errors
    se.alpha <- se.K <- se.b <- se.a <- se.c <- se.d <- NULL

    # Ramsay QM
#    if ( irtmodel=="ramsay.qm" ){ normal.trait <- TRUE }

    #****
    # preparations npirt and npformula
    ICC_model_matrix <- NULL
    if ( ( npirt) & ( !is.null(npformula) ) ){
        for (ii in 1:I){
                    dfr1 <- data.frame( "theta"=theta.k, "y"=1, "wgt"=NA )
                    dfr0 <- data.frame( "theta"=theta.k, "y"=0, "wgt"=NA )
                    dafr <- data.frame( rbind( dfr0, dfr1 ) )
                    theta <- dafr$theta.k
                    ICC_model_matrix[[ii]] <- stats::model.matrix( npformula[[ii]], dafr )
                        }
                }
        # inits theta.k
        theta.k0 <- as.matrix(theta.k)
    dat2 <- as.matrix(dat2)
    dat2.resp <- as.matrix(dat2.resp)
    # inits probs
    if ( irtmodel %in% irtmodel_missing){
        TP <- nrow( theta.k)
        CC <- 3
        pjk <- array( 0, dim=c(I,CC,TP ) )

#        if ( is.null(group) ){
            group_ <- rep(0,nrow(dat2) )
#                    } else {
#            group_ <- group
#                    }


        raschtype <- FALSE
        G <- length(unique(group))
                                }

        #***** module missing1
        # if ( ! is.null( est.delta ) ){
        est_delta <- sum( 1-is.na(est.delta) ) > 0

    if ( sum(est.a) > 0 & (raschtype | irtmodel %in% irtmodel_missing ) ){
        fixed.a0 <- fixed.a
        aG <- setdiff(unique( est.a ), 0 )
    }
    if ( center.b & is.null(Qmatrix) ){
        Qmatrix <- matrix( 1, nrow=I, ncol=1)
        theta.k <- matrix( theta.k, ncol=1 )
        center.trait <- FALSE
    }

    #-- indicators of estimated parameters
    est_parameters <- list( a=sum(est.a)>0, c=sum(est.c)>0, d=sum(est.d)>0)

    aa0 <- Sys.time()

    #******************************************************
    #*************** MML Iteration Algorithm **************
    while ( ( dev.change > glob.conv | par.change > conv1 | maxalphachange > alpha.conv ) & iter < mmliter ){
        if (progress){
            cat(disp)
            cat("Iteration", iter+1, "   ", paste( Sys.time() ), "\n" )
            utils::flush.console()
        }
        zz0 <- Sys.time()

        b0 <- b
        if ( irtmodel %in% irtmodel_missing ){
            beta0 <- beta
        }
        dev0 <- dev
        #-------------- E-step  --------------
        if ( irtmodel %in% irtmodel_missing ){
            e1 <- rasch_mml2_estep_missing1( dat2=dat2, dat2.resp=dat2.resp, theta.k=theta.k,
                        b=b, beta=beta, delta.miss=delta.miss, I=I, CC=CC, TP=TP,
                        group_=group_, pi.k=pi.k, pjk=pjk, weights=weights,
                        fixed.a=fixed.a, est.a=est.a, irtmodel=irtmodel)
            n.ik <- e1$n.ik
            e1$ll <- e1$LL
        }
        if ( ramsay.qm ){
            e1 <- .e.step.ramsay( dat1, dat2, dat2.resp, theta.k, pi.k, I, n, b,
                            fixed.K, group, pow.qm=pow.qm, ind.ii.list )
        }
        if (raschtype & D==1){
            e1 <- rasch_mml2_estep_raschtype( dat1=dat1, dat2=dat2, dat2.resp=dat2.resp,
                        theta.k=theta.k, pi.k=pi.k, I=I, n=n, b=b, fixed.a=fixed.a, fixed.c=fixed.c,
                        fixed.d=fixed.d, alpha1=alpha1, alpha2=alpha2, group=group, pseudoll=pseudoll,
                        weights=weights)
        }
        if (raschtype & D>1){
            e1 <- rasch_mml2_estep_raschtype_mirt( dat1=dat1, dat2=dat2,
                        dat2.resp=dat2.resp, theta.k=theta.k, pi.k=pi.k, I=I, n=n, b=b,
                        fixed.a=fixed.a, fixed.c=fixed.c, fixed.d=fixed.d, alpha1=alpha1,
                        alpha2=alpha2, group=group, mu=mu, Sigma.cov=Sigma.cov,
                        Qmatrix=Qmatrix, pseudoll=pseudoll )
        }
        if (npirt){
            if (iter==0){
                pjk <- stats::plogis( outer( theta.k, b, "-" ) )
            }
            e1 <- .e.step.ramsay( dat1, dat2, dat2.resp, theta.k, pi.k, I, n, b,
                                    fixed.K, group, pow.qm=pow.qm, ind.ii.list,
                                    pjk=pjk )
        }
        n.k <- e1$n.k
        n.jk <- e1$n.jk
        r.jk <- e1$r.jk
        pjk <- e1$pjk
        f.qk.yi <- e1$f.qk.yi
        f.yi.qk <- e1$f.yi.qk
        dev <- -2*e1$ll

# cat("e step") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1

        #-------------- M-step  --------------

        # Ramsay QM
        if ( ramsay.qm ){
            m1 <- .m.step.ramsay( theta.k, b, n.k, n, n.jk, r.jk, I,
                            conv1, constraints,
                            mitermax, pure.rasch,  trait.weights, fixed.K,
                            designmatrix=designmatrix, group=group,
                            numdiff.parm=numdiff.parm, pow.qm=pow.qm, max.b=max.b )
            se.b <- m1$se.b
        }

        # generalized Rasch type model
        if (raschtype){
            m1 <- rasch_mml2_mstep_raschtype( theta.k=theta.k, b=b, n.k=n.k, n=n, n.jk=n.jk,
                        r.jk=r.jk, pi.k=pi.k, I=I, conv1=conv1, constraints=constraints,
                        mitermax=mitermax, pure.rasch=pure.rasch, trait.weights=trait.weights,
                        fixed.a=fixed.a, fixed.c=fixed.c, fixed.d=fixed.d, alpha1=alpha1,
                        alpha2=alpha2, designmatrix=designmatrix, group=group,
                        numdiff.parm=numdiff.parm, Qmatrix=Qmatrix, old_increment=old_increment_b,
                        est.b=est.b, center.b=center.b, min.b=min.b, max.b=max.b,
                        prior.b=prior.b)
            se.b <- m1$se.b
        }

        # nonparametric IRT model
        if (npirt){
            pjk0 <- pjk
            res <- .mstep.mml.npirt( pjk, r.jk, n.jk, theta.k,
                                    npformula, npmodel, G, I, npirt.monotone,
                                    ICC_model_matrix )
            pjk <- res$pjk
            npmodel <- res$npmodel
            apmax <- max( pi.k[,1]*abs( pjk - pjk0)/.40 )
            m1 <- list( "b"=b, "G"=G, "pi.k"=pi.k, "center"=FALSE )
        }
        # missing data IRT model
        if ( irtmodel %in% irtmodel_missing ){
            m1 <- rasch_mml2_mstep_missing1( theta.k=theta.k, n.ik=n.ik,
                        mitermax=mitermax, conv1=conv1, b=b, beta=beta,
                        delta.miss=delta.miss, pjk=pjk,
                        numdiff.parm=numdiff.parm, constraints=constraints,
                        est.delta=est.delta, min.beta=min.beta, est_delta=est_delta,
                        fixed.a=fixed.a, est.a=est.a, min.delta=min.delta,
                        max.delta=max.delta, irtmodel=irtmodel)
            b <- m1$b
            se.b <- m1$se.b
            beta <- m1$beta
            se.beta <- m1$se.beta
            delta.miss <- m1$delta.miss
            se.delta <- m1$se.delta
            m1$dev <- dev
            a1beta <- max( abs( beta - beta0 ) )
            fixed.a <- m1$fixed.a
            se.a <- m1$se.a
            a_change <- m1$a_change
        }
# cat("m step") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1


        #***************************************
        # update mean and covariance in multidimensional models
        if ( D > 1){
            theta.k <- as.matrix(theta.k)
#            delta.theta <- (theta.k[2,1] - theta.k[1,1])^D
            delta.theta <- 1
                hwt <- e1$f.qk.yi
                hwt <- hwt / rowSums(hwt)
                thetabar <- hwt%*%theta.k
                # calculation of mu
                mu <- colSums( thetabar * dat1$Freq ) / sum( dat1$Freq )
                if ( ! is.null(mu.fixed ) ){
                  if (is.matrix(mu.fixed) ){
                    mu0 <- mu
                    mu[ mu.fixed[,1] ] <- mu.fixed[,2]
                    if ( ( sum( as.vector(mu.fixed[1,1:2]) - c(1,0))==0 ) &
                            (    nrow(mu.fixed)==1    )     ){
                        mu[-1] <- -mu0[1] + mu[-1]
                                            }
                                        }
#                  if ( mu.fixed=="center"){
#                        mu <- mu - mean(mu)
#                                        }
                                    }
                # calculation of the covariance matrix
                theta.k.adj <- theta.k - matrix( mu, nrow=nrow(theta.k),
                                    ncol=ncol(theta.k), byrow=TRUE)
                for (dd1 in 1:D){
                    for (dd2 in dd1:D){
                        tk <- theta.k.adj[,dd1]*theta.k.adj[,dd2]
                        h1 <- dat1$Freq * ( hwt %*% tk ) * delta.theta
                        Sigma.cov[dd1,dd2] <- sum( h1 ) / sum( dat1$Freq )
                        if (dd1 < dd2 ){ Sigma.cov[dd2,dd1] <- Sigma.cov[dd1,dd2] }
                                        }
                                    }
                if ( ! is.null(variance.fixed ) ){
                    Sigma.cov[ variance.fixed[,1:2,drop=FALSE] ] <- variance.fixed[,3]
                    Sigma.cov[ variance.fixed[,c(2,1),drop=FALSE] ] <- variance.fixed[,3]
                                    }
                diag(Sigma.cov) <- diag(Sigma.cov) + 1e-10
#                if (m1r){
#                    d11 <- sqrt( Sigma.cov[1,1]*Sigma.cov[2,2] )- .001
#                    Sigma.cov[2,1] <- Sigma.cov[1,2] <- d11
#                        }

                # adaptive estimation
                if ( adaptive.quadrature ){
                    theta.k <- mu + theta.k0 %*% chol(Sigma.cov)
                            }
                pi.k <- sirt_dmvnorm_discrete( theta.k, mean=mu, sigma=Sigma.cov, as_matrix=TRUE )
                m1$pi.k <- pi.k

                            }
            # end MIRT
            #*****
                b <- m1$b
                # distribution
                G <- m1$G
                pi.k <- m1$pi.k
                if (!is.null( trait.weights) ){
                        pi.k <- matrix( trait.weights, ncol=1 )
                            }
                #****************************************************
                # latent ability distribution
                if (distribution.trait=="normal" & D==1){

                    delta.theta <- 1
#                    delta.theta <- theta.k[2] - theta.k[1]
#                    sd.trait <- mean.trait <- rep(0,G)
                  h <- .0001
                    for (gg in 1:G){
                        pi.k0 <- pi.k
                        f.yi.qk.gg <- e1$f.yi.qk[group==gg,]
                        dat1.gg <- dat1[group==gg,2]
                        X1 <- rep(1,nrow(f.yi.qk.gg) )
                    if ( gg > 1 | ( ! center.trait ) ){
                        #*********************************
                        # mean estimation
                        # d.change <- .est.mean( dat1.gg, f.yi.qk.gg, X1, pi.k, pi.k0, gg,
                        #          mean.trait, sd.trait, theta.k, h)
                        # mean.trait[gg] <- mean.trait[gg] + d.change

                        hwt <- e1$f.qk.yi[ group==gg, ]
                        hwt <- hwt / rowSums(hwt)
                        thetabar <- hwt%*%theta.k

                        # calculation of mu
                        mu <- colSums( thetabar * dat1.gg ) / sum( dat1.gg )
                        mean.trait[gg] <- mu

                        pi.k[,gg] <- sirt_dnorm_discrete( theta.k, mean=mean.trait[gg], sd=sd.trait[gg] )
                    }
                    if (!is.null(variance.fixed)){
                            sd.trait[1] <- sqrt(variance.fixed[1,3])

                    }

                        if (center.trait){ mean.trait[1] <- 0 }

                        #*********************************
                        # SD estimation
                        if ( ( gg > 1 ) | ( sum(est.a)==0 ) | est_sd ){

                            #        d.change <- .est.sd( dat1.gg, f.yi.qk.gg, X1, pi.k, pi.k0, gg,
                            #        mean.trait, sd.trait, theta.k, h )
                            #    sd.trait[gg] <- sd.trait[gg] + d.change
                            hwt <- e1$f.qk.yi
                            theta.k.adj <- theta.k - matrix( mu, nrow=length(theta.k), ncol=1, byrow=TRUE)
                            tk <- theta.k.adj[,1]*theta.k.adj[,1]
                            h1 <- dat1.gg * ( hwt %*% tk ) * delta.theta
                            Sigma.cov <- sum( h1 ) / sum( dat1.gg )
                            if (is.null(variance.fixed)){
                                sd.trait[gg] <- sqrt(Sigma.cov)
                            }
                        }
                    if ( ( ( ! is.null(est.a) ) | ( irtmodel=="npirt" ) ) & ( ! est_sd ) ){
                            sd.trait[1] <- 1
                                    }
                        pi.k[,gg] <- sirt_dnorm_discrete( theta.k, mean=mean.trait[gg], sd=sd.trait[gg] )
                                }
                        }  # end normal distribution
        #######################################
               if (distribution.trait!="normal" & D==1){
               for (gg in 1:G){
                    pik1 <-    n.k[,gg] / sum(n.k[,gg] )
                    pik1 <- pik1 + 10e-10
                    if (distribution.trait!="multinomial"){
                        lpik1 <- log( pik1 )
                        tk <- theta.k
                        if ( distribution.trait=="smooth2"){
                                formula1 <- lpik1 ~ tk + I(tk^2)
                                                }
                        if ( distribution.trait=="smooth3"){
                                formula1 <- lpik1 ~ tk + I(tk^2) + I(tk^3)
                                                }
                        if ( distribution.trait=="smooth4"){
                                formula1 <- lpik1 ~ tk + I(tk^2) + I(tk^3)+I(tk^4)
                                                }
                        mod <- stats::lm( formula1, weights=pik1 )
                        pik2 <- exp( stats::fitted(mod))
                    } else {
                        pik2 <- pik1
                    }
                    pi.k[,gg] <- pik2 / sum(pik2)
                    if (center.trait & gg==1){
                        mmm1 <- stats::weighted.mean( theta.k, pik2 )
                        theta.k <- theta.k - mmm1
                                            }
                    if ( ( ! is.null(est.a) ) | ( irtmodel=="npirt" )  ){
                       if (gg==1){
                         sd1 <- sqrt( sum( theta.k^2 * pi.k[,1] ) - sum( theta.k * pi.k[,1] )^2 )
                         theta.k <- theta.k / sd1
                                    }
                                    }
                            }
                         }  # end non-normal distribution
#  cat("trait distribution estimation") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1

        #---- estimation of alpha, c and d parameters
        alpha.change <- 0
        maxalphachange <- 0
        a1a <- a1b <- 0
        a1K <- a1c <- 0

        #--- estimation of a parameters
        if ( sum(est.a) > 0 & raschtype ){
            fixed.a0 <- fixed.a
            aG <- setdiff(unique( est.a ), 0 )
            res <- rasch_mml2_raschtype_mstep_parameter_group( theta.k=theta.k,
                        b=b, fixed.a=fixed.a, fixed.c=fixed.c, fixed.d=fixed.d,
                        pjk=pjk, alpha1=alpha1, alpha2=alpha2, h=numdiff.parm, G=G, I=I,
                        r.jk=r.jk, n.jk=n.jk, est_val=est.a, min_val=min.a,
                        max_val=max.a, iter=iter, old_increment=.3,
                        Qmatrix=Qmatrix, parameter="a", prior=prior.a)
            fixed.a <- res$parm
            se.a <- res$se
            a1a <- max( abs( fixed.a - fixed.a0 ) )
        }
        if ( irtmodel %in% irtmodel_missing ){
            a1a <- a_change
        }


        #--- estimation of c parameter
        if ( sum(est.c) > 0 & raschtype ){
            fixed.c0 <- fixed.c
            cG <- setdiff( unique(est.c), 0 )
            res <- rasch_mml2_raschtype_mstep_parameter_group( theta.k=theta.k,
                        b=b, fixed.a=fixed.a, fixed.c=fixed.c, fixed.d=fixed.d,
                        pjk=pjk, alpha1=alpha1, alpha2=alpha2, h=numdiff.parm, G=G, I=I,
                        r.jk=r.jk, n.jk=n.jk, est_val=est.c, min_val=min.c,
                        max_val=max.c, iter=iter, old_increment=old_increment.c,
                        Qmatrix=Qmatrix, parameter="c", prior=prior.c)
            fixed.c <- res$parm
            se.c <- res$se
            a1b <- max( abs( fixed.c - fixed.c0 ) )
        }
        #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        # estimation of d parameters
        if ( sum( est.d ) > 0 & raschtype ){
            fixed.d0 <- fixed.d
            dG <- setdiff( unique( est.d ), 0 )
            res <- rasch_mml2_raschtype_mstep_parameter_group( theta.k=theta.k,
                        b=b, fixed.a=fixed.a, fixed.c=fixed.c, fixed.d=fixed.d,
                        pjk=pjk, alpha1=alpha1, alpha2=alpha2, h=numdiff.parm, G=G, I=I,
                        r.jk=r.jk, n.jk=n.jk, est_val=est.d, min_val=min.d,
                        max_val=max.d, iter=iter, old_increment=old_increment.d,
                        Qmatrix=Qmatrix, parameter="d", prior=prior.d)
            fixed.d <- res$parm
            se.d <- res$se
            a1c <- max( abs( fixed.d - fixed.d0 ) )
        }
        #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        # estimation of K parameters in Ramsay's quotient model
        if ( sum( est.K ) > 0 & ramsay.qm ){
                h <- numdiff.parm
                fixed.K0 <- fixed.K
                # identify different c parameter groups
                kG <- setdiff( unique( est.K ), 0 )

                res <- .mml.ramsay.est.K( theta.k, b, fixed.a, fixed.c, fixed.d,
                    fixed.K, pjk, alpha1, alpha2, h, G, I, r.jk, n.jk, est.K,
                    min.K, max.K, iter, pow.qm )
                fixed.K <- res$fixed.K
                se.K <- res$se.K
                # convergence is indicated in metric guess.K=1 / ( fixed.K + 1 )
                a1K <- max( abs( 1/(1+fixed.K) - 1/(1+fixed.K0) ) )
                        }
        #***************************
        # estimation of alpha
        if ( est.alpha ){
            alpha1.old <- alpha1
            h <- numdiff.alpha.parm

            #-- alpha1
            calc_prob_args <- list( theta.k=theta.k, b=b, fixed.a=fixed.a, fixed.c=fixed.c,
                            fixed.d=fixed.d, alpha1=alpha1, alpha2=alpha2, Qmatrix=Qmatrix )
            pjk.M <- do.call( "rasch_mml2_calc_prob", args=calc_prob_args )
            #-- alpha1 + h
            calc_prob_args$alpha1 <- alpha1 +  h
            pjk1.M <- do.call( "rasch_mml2_calc_prob", args=calc_prob_args )
            #-- alpha1 - h
            calc_prob_args$alpha1 <- alpha1 -  h
            pjk2.M <- do.call( "rasch_mml2_calc_prob", args=calc_prob_args )

            #-- log likelihood
            ll0a1 <- ll0 <- rasch_mml2_mstep_calc_likelihood( G=G, pjk.M=pjk.M, n.jk=n.jk, r.jk=r.jk )
            ll1a1 <- ll1 <- rasch_mml2_mstep_calc_likelihood( G=G, pjk.M=pjk1.M, n.jk=n.jk, r.jk=r.jk )
            ll2a1 <- ll2 <- rasch_mml2_mstep_calc_likelihood( G=G, pjk.M=pjk2.M, n.jk=n.jk, r.jk=r.jk )
            #--- derivatives
            res <- rasch_mml2_difference_quotient( ll0=ll0, ll1=ll1, ll2=ll2, h=h )
            d1 <- res$d1
            d2 <- res$d2

            # change in item difficulty
            alpha.change <- - d1 / d2
            alpha.change <- ifelse( abs( alpha.change ) > .1, .1*sign(alpha.change), alpha.change )
            alpha1 <- alpha1 + alpha.change
            a1 <- abs(alpha.change )
            se.alpha <- sqrt( 1 / abs(d2) )

            #-- alpha2
            calc_prob_args$alpha1 <- alpha1
            pjk.M <- do.call( "rasch_mml2_calc_prob", args=calc_prob_args )
            #-- alpha2 + h
            calc_prob_args$alpha2 <- alpha2 + h
            pjk1.M <- do.call( "rasch_mml2_calc_prob", args=calc_prob_args )
            #-- alpha2 - h
            calc_prob_args$alpha2 <- alpha2 - h
            pjk2.M <- do.call( "rasch_mml2_calc_prob", args=calc_prob_args )

            #-- log likelihood
            ll0a1 <- ll0 <- rasch_mml2_mstep_calc_likelihood( G=G, pjk.M=pjk.M, n.jk=n.jk, r.jk=r.jk )
            ll1a1 <- ll1 <- rasch_mml2_mstep_calc_likelihood( G=G, pjk.M=pjk1.M, n.jk=n.jk, r.jk=r.jk )
            ll2a1 <- ll2 <- rasch_mml2_mstep_calc_likelihood( G=G, pjk.M=pjk2.M, n.jk=n.jk, r.jk=r.jk )
            #--- derivatives
            res <- rasch_mml2_difference_quotient( ll0=ll0, ll1=ll1, ll2=ll2, h=h )
            d1 <- res$d1
            d2 <- res$d2

            alpha.change <- - d1 / d2
            alpha.change <- ifelse( abs( alpha.change ) > .1, .1*sign(alpha.change), alpha.change )
            alpha2 <- alpha2 + alpha.change
            a2 <- abs(alpha.change)
            maxalphachange <- max(a1, a2)
            se.alpha <- c( se.alpha, sqrt( 1 / abs(d2) ) )

            if (equal.alpha){
                ll0 <- ll0a1 + ll0
                ll1 <- ll1a1 + ll1
                ll2 <- ll2a1 + ll2
                d1 <- ( ll1 - ll2  ) / ( 2 * h )    # negative sign?
                d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
                alpha.change <- - d1 / d2
                alpha.change <- ifelse( abs( alpha.change ) > .1, .1*sign(alpha.change), alpha.change )
                alpha2 <- alpha1 <- alpha1.old + alpha.change
                a2 <- abs(alpha.change)
                maxalphachange <- max(a2)
                se.alpha <- sqrt( 1 / abs(d2) )
            }
        }
# cat("distribution / rest") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1
#


        ##***** output
        # iteration index
        dev.change <- abs( ( dev - dev0)/ dev0 )
        par.change <- max( c( abs(b - b0 ), abs(alpha.change ), a1a, a1b,
                        a1c, a1K, apmax) )
        if (irtmodel %in% irtmodel_missing){
            par.change <- max( c( par.change, a1beta ))
        }
        # display convergence
        if (progress){
            cat( paste( "   Deviance=", round( dev, 4 ),
                             if (iter > 0 ){ " | Deviance change=" } else {""},
                             if( iter>0){round( - dev + dev0, 6 )} else { ""}    ,"\n",sep=""))
                    if ( ! npirt ){
                        cat( paste0( "    Maximum b parameter change", "=",
                             round( max(abs(b - b0 )), 6 ),  " \n"   )  )
                                    }
                                    if ( est.alpha ){
                                        cat( paste0( "    alpha1=", round(alpha1,3), " | alpha2=", round( alpha2,3),
                                                    " | max alpha change ", round( maxalphachange,7 ), "\n", sep=""))
                                                }
                                    if ( sum(est.a) > 0  ){
                                        cat( paste0( "    Maximum a parameter change", "=",
                                                paste( round(a1a,6), collapse=" " ), "\n", sep=""))
                                                }
                                    if ( irtmodel %in% irtmodel_missing ){
                                        cat( paste0( "    Maximum beta parameter change=",
                                                paste0( round(a1beta,6), collapse=" " ), "\n", sep=""))
                                                }
                                    if ( sum(est.c) > 0  ){
                                        cat( paste0( "    Maximum c parameter change=",
                                                paste( round(a1b,6), collapse=" " ), "\n", sep=""))
                                                }
                                    if ( sum(est.d) > 0  ){
                                        cat( paste0( "    Maximum d parameter change=",
                                                paste( round(a1c,6), collapse=" " ), "\n", sep=""))
                                                }
                                    if ( npirt  ){
                                        cat( paste0( "    Maximum weighted ICC change=",
                                                paste( round(apmax,6), collapse=" " ), "\n", sep=""))
                                                }
                                    if ( sum(est.K) > 0  ){
                                        cat( paste0( "    Maximum K parameter change=",
                                                paste( round(a1K,6), collapse=" " ), "\n", sep=""))
                                                }
                                if ( D > 1 ){
                                  cat("    Mean              | " )
                                  cat( round(as.vector(mu),3))
                                  cat("\n    Covariance Matrix | " )
                                  cat( round(Sigma.cov[!upper.tri(Sigma.cov)],3))
                                  cat("\n")
                                                            }
                                if ( irtmodel %in% irtmodel_missing ){
                                  cat("    Delta=" )
                                  r1 <- sort( unique( as.vector(delta.miss) ) )
                                  h1 <- ""
                                  if ( length(r1) > 5 ){
                                    r1 <- r1[1:5]
                                    h1 <- " ... "
                                                }

                                  cat( round( r1,3))
                                  cat( h1, "\n")
                                                            }


            utils::flush.console()
        }
        iter <- iter + 1
    }
        ####################################### end iterations #####################
        ############################################################################
        ##**************************************************************************

# cat(" ++++ total estimation time") ; aa1 <- Sys.time(); print(aa1-aa0) ; aa0 <- aa1

            if ( irtmodel %in% irtmodel_missing){
                m1$center <- FALSE
                G <- 1
                            }


            if (npirt & ( ! is.null(npformula ) ) ){
                item <- NULL
                for (ii in 1:I){
                    item.ii <- data.frame( "item"=colnames(dat)[ii] )
                    smod.ii <-  summary(npmodel[[ii]])
                    item.ii <- data.frame( cbind( item.ii, rownames(smod.ii$coef),
                                        smod.ii$coef[,1:2] ) )
                    colnames(item.ii)[-1] <- c("par", "est", "se" )
                    rownames(item.ii) <- NULL
                    item <- rbind( item, item.ii )
                            }
                        }

    # information criteria
    ic <- list( "deviance"=dev, "n"=nrow(dat) )
    if ( distribution.trait=="normal"){
        ic[[ "np" ]] <-  ( G - 1 ) + ncol(dat) + ( G - 0 )
    }
    if ( distribution.trait=="smooth2"){
        ic[[ "np" ]] <-  ( G - 1 ) + ncol(dat) + ( G - 0 )
    }
        if ( distribution.trait=="smooth3"){
                ic[[ "np" ]] <- ( G - 1 ) + ncol(dat) + ( G - 0 ) + G
                            }
        if ( distribution.trait=="smooth4"){
                ic[[ "np" ]]<-  ( G - 1 ) + ncol(dat) + ( G - 0 ) + 2*G
                            }
#        ic$itempars <- ic$traitpars - ncol(dat)
        # subtract fixed constraints
        if ( ! is.null( constraints) ){
            ic$np <- ic$np - nrow(constraints)
#            ic$itempars <- ic$itempars - nrow(constraints)
                }
        # subtract constraints due to designmatrix
        if ( ! is.null( designmatrix ) ){
                ic$np <- ic$np - ncol(dat) + ncol(designmatrix)
#                ic$itempars <- ic$itempars - ncol(dat) + ncol(designmatrix)
                        }
        # alpha estimation
        ic$np <- ic$np + est.alpha * 2 - equal.alpha *1
#        ic$itempars <- ic$itempars + est.alpha * 2 - equal.alpha *1
        # guessing, slipping and discrimination parameter estimation
        if ( sum(est.c) > 0 ){
                    ic$np <- ic$np + length(cG)
                        }
        if ( sum(est.d) > 0 ){
                ic$np <- ic$np + length(dG)
                            }
        if ( sum(est.a) > 0 ){
                ic$np <- ic$np + length(aG) - 1
                        }
        if ( sum(est.K) > 0 ){
                ic$np <- ic$np + length(kG)
                            }
        if ( irtmodel %in% irtmodel_missing){
            ic$np <- ic$np + I
            if ( est_delta ){
                v1 <- unique( est.delta )
                v1 <- v1[ ! is.na(v1) ]
                ic$np <- ic$np + length(v1)
                            }
                    }
        # parameters for multiple dimensions
        if (D>1){
            # mean vector
            MM <- nrow(mu.fixed )
#            if ( mu.fixed=="center" ){ MM <- 1 }
            if ( is.null(mu.fixed) ){ MM <- 0 }
            ic$np <- ic$np + length(mu) - MM
            # covariance matrix
            ic$np <- ic$np - 1*(sum(est.a)==0) + D*(D+1)/2        # SD's
            if ( ! is.null(variance.fixed)){ ic$np <- ic$np - nrow( variance.fixed ) }
                }

        # item parameter for nonparametric models
        if (npirt & ( ! is.null(npformula ) ) ){
                ic$np <- nrow(item) }
        if (npirt & (  is.null(npformula ) ) ){
                ic$np <- prod( dim(pjk)) }
        # AIC
        ic$AIC <- dev + 2*ic$np
        # BIC
        ic$BIC <- dev + ( log(ic$n) )*ic$np
        # CAIC (conistent AIC)
        ic$CAIC <- dev + ( log(ic$n) + 1 )*ic$np
        # corrected AIC
        ic$AICc <- ic$AIC + 2*ic$np * ( ic$np + 1 ) / ( ic$n - ic$np - 1 )
        # log penalty
        ic$GHP <- ic$AIC / ( sum(weights*(!is.na(dat)))) / 2

    #---- item statistics
    if ( npirt & ( ! is.null(npformula ) ) ){
        item0 <- item
    }
    item <- data.frame( item=colnames(dat),
                        N=colSums( weights*(1 - is.na(dat)) ),
                        p=colSums( weights*(dat==1), na.rm=TRUE) / colSums( weights*(1-is.na(dat))),
                        b=b)
    if ( ! is.null( constraints) ){
        est.b <- 1:I
        est.b[ constraints[,1] ] <- 0
        item$est.b <- est.b
    }
    if ( npirt & ( ! is.null(npformula ) ) ){ item <- merge( x=item[,1:3], y=item0, by="item" ) }
    if ( ! npirt ){
        if (is.null(est.b)){ item$est.b=seq(1,I) } else { item$est.b <- est.b }
            # fixed parameters
            item$a <- fixed.a
            if ( ! is.null( est.a) ){  item$est.a <- est.a } else { item$est.a <- rep(0,I) }
            # include threshold
            item$thresh <- item$a*item$b
            # guessing parameter
            item$c <- fixed.c
            if ( ! is.null( est.c) ){ item$est.c <- est.c } else { item$est.c <- rep(0,I) }
            item$d <- fixed.d
            if ( ! is.null( est.d) ){  item$est.d <- est.d } else { item$est.d <- rep(0,I) }
            if (m1$center){  if ( is.null(constraints) ){ #    item[I,4] <- NA
                                                                        }
                        else { item[ constraints[,1],4] <- NA } }
            rownames(item) <- colnames(dat)
                }

        # latent ability distribution
        skewness.trait <- sd.trait <- mean.trait <- rep(0,G)
        if ( D==1){
            for (gg in 1:G){
                mean.trait[gg] <- weighted.mean( theta.k, pi.k[,gg] )
                sd.trait[gg] <- sqrt( weighted.mean( ( theta.k - mean.trait[gg] )^2, pi.k[,gg] ) )
                skewness.trait[gg] <- sum( ( theta.k - mean.trait[gg]  )^3 * pi.k[,gg] ) / sd.trait[gg]^3
                if (gg==1 & npirt ){ sd.trait[gg] <- 1 }
                        }
                    }
            # center trait distribution
#            if ( center.trait & G < 1 ){
#                    theta.k <- theta.k - mean.trait
#                    b <- b - mean.trait
#                    item$itemdiff <- b
#                    mean.trait <- 0
#                            }
            trait.distr <- data.frame( "theta.k"=theta.k, "pi.k"=pi.k )
        # item response pattern
        if ( D==1 ){
            if ( is.matrix(theta.k) ){
                theta.k <- as.vector( theta.k)
                        }
            ability.est <- data.frame( dat1, theta.k[ whichrowMaxs( f.qk.yi )$arg ] )
            colnames(ability.est) <- c("pattern", "AbsFreq", "mean", "MAP" )
                    }
        if (D>1){
            ability.est <- data.frame( dat1, theta.k[ whichrowMaxs( f.qk.yi )$arg,] )
            colnames(ability.est) <- c("pattern", "AbsFreq", "mean",
                    paste("MAP.Dim",1:D,sep="") )
                }
        if (D==1){
            ability.est$EAP <- rowSums( f.qk.yi * outer( rep(1,nrow(ability.est)), theta.k  )  )
            ability.est$SE.EAP <- sqrt( rowSums( f.qk.yi * outer( rep(1,nrow(ability.est)),
                            theta.k^2  )  ) - ability.est$EAP^2 )
                 }
        if (D>1){
           for (dd in 1:D){
            ability.est[, paste("EAP.Dim",dd,sep="")] <-
                    rowSums( f.qk.yi * outer( rep(1,nrow(ability.est)), theta.k[,dd]  )  )
            ability.est[, paste("SE.EAP.Dim",dd,sep="")] <-
                sqrt( rowSums( f.qk.yi * outer( rep(1,nrow(ability.est)), theta.k[,dd]^2  )  ) -
                            ability.est[,paste("EAP.Dim",dd,sep="")]^2 )
                                }
                        }
        # posterior distribution
        rownames(f.qk.yi) <- dat1[,1]
        # merging ability estimates
#        if ( ! is.null(group)){
        if ( G > 1 ){
                    ability.est2 <- cbind( freq.patt, ability.est[,-1] )
                            } else {
               if (!(irtmodel %in% irtmodel_missing)  ){
                    ability.est2 <- merge( freq.patt, ability.est, 1, 1 )
                        } else {
                    ability.est2 <- ability.est
                    ability.est2$index <- seq(1, nrow(ability.est) )
                                }
                            }
        ability.est2 <- ability.est2[ order(ability.est2$index), -c(3:5) ]

        # EAP reliability estimate
        reliability <- NULL
        if (D==1){
            reliability$eap.reliability <-
                    1 - mean(ability.est2$SE.EAP^2) / ( mean(ability.est2$SE.EAP^2) + var(ability.est2$EAP) )
                        }
        if (D>1){
            r1 <- rep(0,D)
            for (dd in 1:D){
            r1[dd] <- 1 - mean(ability.est2[,paste("SE.EAP.Dim",dd,sep="")]^2) /
                                ( mean(ability.est2[,paste("SE.EAP.Dim",dd,sep="")]^2) +
                                    stats::var(ability.est2[,paste("EAP.Dim",dd,sep="")]) )
                            }
          if ( is.null( colnames(Qmatrix) ) ){
            dimnamesPars <- paste( "Dim",1:D, sep="")
                            } else { dimnamesPars <- colnames(Qmatrix) }
          names(r1) <- dimnamesPars
          reliability$eap.reliability <- r1
          names(mu) <- dimnamesPars
          rownames(Sigma.cov) <- colnames(Sigma.cov) <- dimnamesPars
                        }
        # include person ID
        ability.est2$pid <- pid
        # match ability patterns
        if (!(irtmodel %in% irtmodel_missing )){
            ind1 <- match( ability.est2$freq.patt, ability.est$pattern  )
            ability.est <- ability.est[ ind1, ]
            f.qk.yi <- f.qk.yi[ind1,]
            f.yi.qk <- f.yi.qk[ind1,]
                        }

        #*****
        # item table for missing data IRT model
        if ( irtmodel %in% irtmodel_missing){
            item$thresh <- item$est.c <- item$est.d <- NULL
            # missing proportion
            item$pmiss <- colSums( dat2.resp * ( dat2==2 ), na.rm=TRUE) / colSums( dat2.resp, na.rm=TRUE)
            item$beta <- beta
            item$est.delta <- est.delta
            NI <- nrow(item)
            delta1 <- delta.miss
            if (est_delta & ( length(delta1) !=NI) ){
                ind_delta <- match(est.delta, sort(unique(est.delta)))
                delta1 <- delta.miss[ ind_delta ]
            }
            item$delta.miss <- delta1
        }
        # output fixed.a and fixed.c
        if ( is.null(fixed.a ) & is.null(fixed.c) ){  fixed.a <- rep(1,I) ; fixed.c <- rep(0,I) }
        # include item discrimination
        if (D==1){
            i1 <- item$emp.discrim <- round( item.discrim( dat,  ability.est2$MAP ), 3 )
                }
        if (npirt){
            i1 <- data.frame( "item"=colnames(dat), "emp.discrim"=i1 )
            item$emp.discrim <- NULL
            item <- merge( x=item, y=i1, by="item" )
                }
        if ( ! npirt ){
            item$alpha1 <- alpha1
            item$alpha2 <- alpha2
                    }
        #---------------------------------------------------------
        # item summary Ramsay QM
        item2 <- NULL
        if ( ramsay.qm){
        if ( is.null(est.K) ){ est.K <- rep(0,I) }
            item2 <- data.frame( "item"=item$item, "N"=item$N, "p"=item$p,
                            "K"=fixed.K, "est.K"=est.K,
                            "b"=exp(b), "log_b"=b, "est.b"=item$est.b,
                            "guess.K"=1/(fixed.K+1),
                            "emp.discrim"=item$emp.discrim )
                            }

    #--- imputed data for irtmodel="missing1" or "raschtype"
    dat_implist <- NULL
    if ( ( irtmodel %in% c(irtmodel_missing,"raschtype") ) & (nimps>0) ){
        dat_implist <- rasch_mml2_impute_data( e1=e1, b=b, beta=beta,
                            delta.miss=delta.miss, pjk=pjk, fixed.a=fixed.a, dat=dat,
                            dat.resp=dat.resp, irtmodel=irtmodel, nimps=nimps )
    }

    #--- item response probabilities
    d1 <- dim(pjk)
    if ( length(d1)==2 ){
        rprobs <- array( 0, dim=c( d1[2], 2, d1[1] ) )
        rprobs[,2,] <- t( pjk )
        rprobs[,1,] <- 1 - t(pjk)
    } else {
        rprobs <- pjk
    }
    dimnames(rprobs)[[1]] <- colnames(dat)

    #- collect information about priors
    priors <- rasch_mml2_prior_information( prior.a=prior.a, prior.b=prior.b,
                    prior.c=prior.c, prior.d=prior.d )

    #--- result
    res <- list( dat=dat, weights=weights, item=item, item2=item2, trait.distr=trait.distr,
                mean.trait=mean.trait, sd.trait=sd.trait, skewness.trait=skewness.trait,
                deviance=dev, pjk=pjk, rprobs=rprobs, person=ability.est2, pid=pid,
                ability.est.pattern=ability.est, f.qk.yi=f.qk.yi, f.yi.qk=f.yi.qk,
                pure.rasch=pure.rasch, fixed.a=fixed.a, fixed.c=fixed.c, G=G, alpha1=alpha1,
                alpha2=alpha2, se.b=se.b, se.a=se.a, se.c=se.c, se.d=se.d, se.alpha=se.alpha,
                se.K=se.K, se.delta=se.delta, iter=iter, reliability=reliability,
                ramsay.qm=ramsay.qm, irtmodel=irtmodel, D=D, mu=mu, Sigma.cov=Sigma.cov,
                est_parameters=est_parameters, priors=priors,
                theta.k=theta.k, trait.weights=trait.weights, pi.k=pi.k,
                dat_implist=dat_implist, variance.fixed=variance.fixed, CALL=CALL )
    class(res) <- "rasch.mml"
    res$ic <- ic
    res$est.c <- est.c
    res$groupindex <- ag1
    res$n.jk <- n.jk
    res$r.jk <- r.jk
    res$esttype <- "ll"
    if ( pseudoll ){ res$esttype <- "pseudoll" }
    # computation time
    s2 <- Sys.time()
    res$s1 <- s1
    res$s2 <- s2
    res$Rfcttype <- "rasch.mml2"
    if (progress){
        cat("------------------------------------------------------------\n")
        cat("Start:", paste( s1), "\n")
        cat("End:", paste(s2), "\n")
        cat("Difference:", print(s2 -s1), "\n")
        cat("------------------------------------------------------------\n")
    }
    return(res)
}

## File Name: sirt_format_numb.R
## File Version: 0.141


#------ format numbers
sirt_format_numb <- function( x, digits )
{
    a1 <- round( x, digits ) + 10^{-(digits +1 ) }
    a1 <- substring( a1, 1, nchar(a1) - 1 )
    return(a1)
}

format_numb <- sirt_format_numb

## File Name: linking_haberman_itempars_convert_process_matrices.R
## File Version: 0.01


linking_haberman_itempars_convert_process_matrices <- function(mat, est_pars)
{
    mat[ ! est_pars ] <- NA
    return(mat)
}

## File Name: dif.variance.R
## File Version: 0.193



#*** Routine for calculating DIF variance   (Camilli & Penfield, 1997)      #
dif.variance <- function( dif, se.dif, items=paste("item",1:length(dif),sep="") )
{
    # calculating the weights
    wi <-  1/se.dif^2
    # mean DIF
    md <- mean(dif)
    # weighted tau2 estimator
    # formula (5) of PENFIELD & ALGINA (2006)
    weighted.tauq <- sum( wi^2 * ( dif - md )^2 - wi  ) / sum( wi^2 )
    # unweighted tau2 estimator
    # formula (5) of PENFIELD & ALGINA (2006)
    unweighted.tauq <- sum( ( dif - md )^2 - se.dif^2 ) / ( length(items) - 1 )
    # calculation of variances v_i
    vi <- weighted.tauq + se.dif^2
    weighted.tauq[ weighted.tauq < 0 ] <- 0
    unweighted.tauq[ unweighted.tauq < 0 ] <- 0
    # Empirical Bayes DIF estimate
    lambda.i <- weighted.tauq / vi
    eb.dif <- lambda.i * ( dif - md ) + ( 1 - lambda.i) *md

    #* output
    res <- list( weighted.DIFSD=sqrt(weighted.tauq),
                unweighted.DIFSD=sqrt(unweighted.tauq),
                mean.se.dif=sqrt( mean( se.dif^2 ) ), eb.dif=eb.dif  )
    return(res)
}

## File Name: summary_round_helper.R
## File Version: 0.101

summary_round_helper <- function( obji, digits, exclude=NULL, print=TRUE)
{
    NC <- ncol(obji)
    ind <- 1L:NC
    if ( ! is.null(exclude) ){
        ind2 <- which( colnames(obji) %in% exclude )
        ind <- setdiff( ind, ind2 )
    }
    obji[,ind] <- round( obji[,ind], digits )
    rownames(obji) <- NULL
    print(obji)
    invisible(obji)
}

## File Name: sirt_import_lavaan_parameterEstimates.R
## File Version: 0.07

sirt_import_lavaan_parameterEstimates <- function(...)
{
    TAM::require_namespace_msg('lavaan')
    res <- lavaan::parameterEstimates(...)
    return(res)
}

## File Name: mcmc.2pno_alg.R
## File Version: 1.242


######################
# draw latent responses Z
.draw.Z.2pl <- function( aM, bM, theta, N, I, threshlow, threshupp ){
    # calculate means
    mij <- aM * theta - bM
    # simulate uniform data
    rij <- matrix( stats::runif( N*I ), nrow=N, ncol=I )
    # calculate corresponding value
    pl <- stats::pnorm( threshlow, mean=mij)
    pu <- stats::pnorm( threshupp, mean=mij)
    pij <- pl + (pu-pl)*rij
    # simulate Z
    Zij <- stats::qnorm( pij, mean=mij )
    return(Zij)
}

########################################
# draw theta
.draw.theta.2pl <- function( aM, bM, N, I, Z ){
    vtheta <- 1 / ( rowSums( aM^2 ) + 1  )
    mtheta <- rowSums( aM * ( Z + bM ) ) * vtheta
    theta <- stats::rnorm( N, mean=mtheta, sd=sqrt( vtheta ) )
#    v1 <- var(mtheta)
#    EAP.rel <- v1 / ( mean(vtheta) + v1 )
#    res <- list("theta"=theta, "EAP.rel"=EAP.rel )
    res <- list("theta"=theta )
    return(res)
            }

##########################################
# draw item parameters
.draw.itempars.2pl <- function( theta, Z, I, N, weights){
    #--------------
    # sampling without weights
    Xast <- as.matrix( cbind( theta, -1 ) )
    if ( is.null(weights) ){
#        Sigma <- solve( t(Xast) %*% Xast )
        Sigma <- solve( crossprod(Xast) )
        # calculate mean
#        mj <- Sigma %*% t(Xast) %*% Z
        mj <- Sigma %*% crossprod(Xast, Z )
        mj <- as.matrix( t(mj))
                        }
    #--------------
    # sampling with weights
    if ( ! is.null( weights ) ){
        # compute elements of Xast
        Xast11 <- sum( theta^2 * weights )
        Xast12 <- - sum( theta * weights )
        Xast22 <- sum( weights )
        # compute inverse of Xast
        Xastdet <- Xast11*Xast22 - Xast12^2
        Xastinv11 <- Xast22 / Xastdet
        Xastinv22 <- Xast11 / Xastdet
        Xastinv12 <- - Xast12 / Xastdet
        Sigma <- matrix( c(Xastinv11, Xastinv12, Xastinv12, Xastinv22), 2,2 )
        # compute t(Xast) %*% Z (weighted)
#        mj <- Sigma %*% t( Xast * weights ) %*% Z
        mj <- Sigma %*% crossprod( Xast * weights, Z )
        mj <- as.matrix( t(mj))
            }
    #--------------
    # draw item parameters
    ipars <- sirt_rmvnorm( I, sigma=Sigma ) + mj
    a <- ipars[,1]
    b <- ipars[,2]
    return( list( "a"=a, "b"=b) )
}


#################################################
# compute deviance
.mcmc.deviance.2pl <- function( aM, bM, theta, dat, dat.resp,
                weights, eps ){
    pij <- stats::pnorm( aM * theta - bM )
    llij <- log( dat.resp * ( dat*pij + ( 1-dat )*(1-pij) ) + eps )
    if ( is.null( weights ) ){ deviance <- -2*sum( llij ) }
    if ( ! is.null( weights ) ){
        deviance <- -2*sum( rowSums(llij) * weights )
                    }
    return(deviance)
                }

######################################################
# subfunction for calculating the DIC
.mcmc.ic.2pl <- function( a.chain, b.chain, theta.chain, N, I,
        dat, dat.resp, weights, eps,
        deviance.chain ){
    #************
    aM <- matrix( colMeans( a.chain ), nrow=N, ncol=I, byrow=TRUE )
    bM <- matrix( colMeans( b.chain ), nrow=N, ncol=I, byrow=TRUE )
    theta <- colMeans( theta.chain )
    Dhat <- .mcmc.deviance.2pl( aM, bM, theta, dat, dat.resp,
                weights, eps )
    Dbar <- mean( deviance.chain )
    pD <- Dbar - Dhat
    ic <- list( "Dhat"=Dhat, "Dbar"=Dbar, "pD"=pD, "DIC"=Dhat + 2*pD )
    return(ic)
        }
#######################################################



####################################################################
# calculate EAP reliability, person parameter EAPs
# and corresponding posterior SDs
.mcmc.person.2pno <- function( theta.chain, weights ){
    ###################
    # EAP reliability
        v1 <- stats::var( colMeans( theta.chain ) )
        if ( is.null(weights) ){ EAP.rel <- v1 / 1 }
        if ( ! is.null(weights) ){
                w1 <- weights / sum(weights )
                m1 <- colMeans( theta.chain )
                v1 <- sum( m1^2 * w1 ) - ( sum( m1*w1 ) )^2
                wM <- matrix( w1, nrow=nrow(theta.chain), ncol=ncol(theta.chain),
                                byrow=TRUE )
                h1 <- rowSums( wM * theta.chain^2 ) - ( rowSums( wM * theta.chain ) )^2
                h1 <- mean(h1)
                EAP.rel <- v1 / h1
                        }
    ###################
    # person parameter estimates
    person <- data.frame( "EAP"=colMeans( theta.chain ),
                        "SD"=colSds(theta.chain) )
    # output
    res <- list( "EAP.rel"=EAP.rel, "person"=person )
    return(res)
            }
###############################################################

## File Name: sirt_import_lavaan_lavaanify.R
## File Version: 0.06

sirt_import_lavaan_lavaanify <- function(...)
{
    TAM::require_namespace_msg('lavaan')
    res <- lavaan::lavaanify(...)
    return(res)
}

## File Name: create.ccov.R
## File Version: 1.091


#**** auxiliary function for creating a covariance matrix
create.ccov <- function( cc, data )
{
    ccc <- cc$ccov.table
    I <- max( ccc$item1ID, ccc$item2ID )
    ccov.matrix <- matrix( 0, nrow=I, ncol=I)
    rownames(ccov.matrix) <- colnames(ccov.matrix) <- colnames(data)
    LL <- nrow(ccc)
    for (ll in 1L:LL){
        ccov.matrix[ ccc$item1ID[ll], ccc$item2ID[ll] ] <- ccc$ccov[ll]
        ccov.matrix[ ccc$item2ID[ll], ccc$item1ID[ll] ] <-
                        ccov.matrix[ ccc$item1ID[ll], ccc$item2ID[ll] ]
    }
    return(ccov.matrix)
}


.create.ccov <- create.ccov

## File Name: IRT.likelihood.mirt.R
## File Version: 0.284


IRT_likelihood_mirt_singleGroup <- function( object, type, ... )
{
    object0 <- object
    object <- mirt.wrapper.posterior(mirt.obj=object)
    ll <- object[[ type ]]
    attr(ll,"theta") <- object$theta.k
    attr(ll,"prob.theta") <- object$pi.k
    attr(ll,"G") <- 1
    attr(ll,"pweights") <- object$pweights
    return(ll)
}


#--- likelihood singleGroup
IRT.likelihood.mirt.singleGroup <- function( object, ... )
{
    ll <- IRT_likelihood_mirt_singleGroup( object=object, type="f.yi.qk", ...)
    return(ll)
}
IRT.likelihood.ConfirmatoryClass <- IRT.likelihood.mirt.singleGroup
IRT.likelihood.ExploratoryClass <- IRT.likelihood.mirt.singleGroup
IRT.likelihood.SingleGroupClass <- IRT.likelihood.mirt.singleGroup


#--- likelihood multipleGroup
IRT_likelihood_mirt_multipleGroup <- function( object, type, ... )
{
    mobj <- object
    groups <- object@Data$groupNames
    G <- length(groups)
    prob.theta <- list()
    ll_list <- list()
    ind_group <- list()
    pweights <- list()
    for (gg in 1:G){
        object <- mirt.wrapper.posterior(mirt.obj=mobj, group=groups[gg])
        if (gg==1){
            theta <- object$theta.k
            TP <- object$TP
        }
        prob.theta[[gg]] <- object$pi.k
        ll_list[[gg]] <- object[[type]]
        ind_group[[gg]] <- object$ind_group
        pweights[[gg]] <- object$pweights
    }
    ll <- matrix(NA, nrow=object$N_orig, ncol=object$TP)
    ll_pw <- rep(NA, object$N_orig)
    for (gg in 1:G){
        ll[ ind_group[[gg]], ] <- ll_list[[gg]]
        ll_pw[ ind_group[[gg]] ] <- pweights[[gg]]
    }
    prob.theta <- matrix( unlist(prob.theta), nrow=TP, ncol=G)
    attr(ll,"theta") <- theta
    attr(ll,"prob.theta") <- prob.theta
    attr(ll,"G") <- G
    attr(ll,"pweights") <- ll_pw
    return(ll)
}


IRT.likelihood.mirt.multipleGroup <- function( object, ... )
{
    ll <- IRT_likelihood_mirt_multipleGroup( object=object, type="f.yi.qk", ... )
    return(ll)
}
IRT.likelihood.MultipleGroupClass <- IRT.likelihood.mirt.multipleGroup

## File Name: linking_haberman_remove_missings_vector.R
## File Version: 0.01

linking_haberman_remove_missings_vector <- function(x,w)
{
    ind <- ! is.na(x)
    x <- x[ind]
    w <- w[ind]
    res <- list(x=x, w=w)
    return(res)
}

## File Name: equating.rasch.jackknife.R
## File Version: 0.158



#--- computation of linking error using Jackknife
equating.rasch.jackknife <- function( pars.data, display=TRUE, se.linkerror=FALSE,
            alpha1=0, alpha2=0 )
{
    pars.data <- as.data.frame( stats::na.omit( pars.data ) )
    itemunits <- unique( pars.data[,1] )
    N.units <- length( itemunits )
    N.items <- nrow( pars.data )
    pars.data[,4] <- paste('I', 1L:N.items,sep='')
    # display
    if (display){
        cat( paste( 'Jackknife Equating Procedure (Stocking-Lord)\n',
                        N.items, ' Items in ', N.units, ' Units\n', sep='') )
    }
    # equating without jackknife
    mod1 <- equating.rasch( pars.data[, c( 4, 2) ], pars.data[, c(4, 3) ] )
    res1 <- data.frame( unit=itemunits, shift=0, SD=0, linkerror=0)

    # perform jackknife
    for (nn in 1L:N.units){
        pars.data1 <- pars.data[ pars.data[,1] !=itemunits[nn], ]
        mod.nn <- equating.rasch( x=pars.data1[, c(4,2) ], y=pars.data1[, c(4,3) ] )
        res1[ nn, 'shift' ] <- mod.nn$B.est$Stocking.Lord
        res1[ nn, 'SD' ] <- mod.nn$descriptives$SD

        # Jackknife of the linking error
        if (se.linkerror){
            itemunits.nn <- itemunits[ - nn ]
            l1 <- NULL
            for (ii in itemunits.nn){
                pars.data1.ii <- pars.data1[ paste(pars.data1[,1]) !=ii, ]
                mod.ii <- equating.rasch( x=pars.data1.ii[,c(4,2)],
                                            y=pars.data1.ii[,c(4,3)],
                                            alpha1=alpha1, alpha2=alpha2)
                l1 <- c(l1, mod.ii$B.est$Stocking.Lord )
            }
            res1[ nn, 'linkerror'] <-  sqrt( ( N.units - 2 ) / ( N.units -1 ) *
                                            sum( ( l1 - res1[ nn, 'shift' ]  )^2 ) )
        }
        # display progress
        if (display){
            cat( paste( nn, ' ', sep='' ) )
            utils::flush.console()
            if ( nn%%10==0){ cat('\n') }
        }
    }
    cat('\n')
    le_fac <- ( N.units - 1 ) / N.units
    linkerror <- sqrt(  le_fac * sum( ( res1[,2] - mod1$B.est$Stocking.Lord )^2 ) )
    se.sd <- sqrt( le_fac * sum( ( res1[,3] - mod1$descriptives$SD )^2 ) )
    if (se.linkerror){
        se.linkerror <- sqrt( le_fac * sum( ( res1[,4] - linkerror )^2 ) )
    } else {
        se.linkerror <- NA
    }
    #--- output
    descriptives <- data.frame( N.items=N.items, N.units=N.units,
                            shift=mod1$B.est$Stocking.Lord,
                            SD=mod1$descriptives$SD, linkerror.jackknife=linkerror,
                            SE.SD.jackknife=se.sd, se.linkerror.jackknife=se.linkerror )
    res <- list( pars.data=pars.data, itemunits=itemunits, descriptives=descriptives )
    return(res)
}

## File Name: ruvn.R
## File Version: 0.02

ruvn <- function(N, mean=0, sd=1, exact=TRUE)
{
    x <- stats::rnorm(N, mean=mean, sd=sd)
    y <- x
    if (exact){
        v1 <- sirt_var(x)
        m1 <- mean(x)
        y <- mean+(x-m1)/sqrt(v1)*sd
    }

    #--- output
    return(y)
}

## File Name: xxirt_proc_ParTable.R
## File Version: 0.465

#-- process parameter table
xxirt_proc_ParTable <- function( itemtype, partable, items )
{
    #*** extract item types from partable
    itemtype <- unlist( sapply( items, FUN=function(ii){
                        partable$type[ paste(partable$item)==ii ][1]
                        } )  )
    I <- length(items)
    partable$rowindex <- seq( 1, nrow(partable) )
    #*** parameter index
    partable[ ! partable$est, 'parindex' ] <- 0
    # indices <- sort( unique( partable$parindex ) )
    indices <- unique( partable$parindex )
    indices <- c( 0, setdiff( indices, c(0) ) )
    IN <- length(indices)
    partable$parindex <- match( partable$parindex, indices ) - 1
    partable$parindex[ partable$parindex==0 ] <- NA
    #*** set prior distributions of fixed parameters to NA
    partable[ ! partable$est, 'prior' ] <- NA
    #*** list with parameter table indices
    partable$parfree <- 1*partable$est
    partable_index <- as.list( 1L:I )
    for (ii in 1L:I){
        partable_index[[ii]] <- which( partable$itemnr==ii )
    }
    ind <- which( duplicated( partable$parindex ) & ( ! is.na( partable$prior ) ) )
    if ( length(ind) > 0 ){
        partable[ind, c('prior','prior_par1','prior_par2', 'parlabel') ] <- NA
    }
    ind <- which( duplicated( partable$parindex ) )
    if ( length(ind) > 0 ){
        partable[ind,'parfree'] <- 0
    }
    #*** extract ncat and maxK
    p1 <- partable[ ! duplicated( partable$item), ]
    ncat <- p1$ncat
    names(ncat) <- paste(p1$item)
    maxK <- max(ncat)
    #*** extract M-step method
    p1 <- partable[ partable$parfree==1, ]
    m1 <- TRUE
    d1 <- sum( ( c('lower','upper') %in% colnames(partable) ) )
    if ( ( nrow(p1) > 0 ) & (d1>0) ){
        m1 <- ( mean( p1$lower==- Inf ) < 1 ) | ( mean( p1$upper==Inf ) < 1 )
    }
    mstep_method <- if (m1){ 'L-BFGS-B' } else { 'BFGS' }

    #**** item indices per parameter
    NP <- -Inf
    if ( sum(partable$est) > 0 ){
        NP <- max( partable$parindex, na.rm=TRUE )
    }
    if ( NP > -Inf){
        item_index <- as.list( 1L:NP )
        for (pp in 1L:NP){
            p1 <- partable[ paste0( partable$parindex)==pp, ]
            names(item_index)[pp] <- p1[1,'parlabel']
            item_index[[pp]] <- p1$itemnr
        }
    } else {
        item_index <- list()
    }

    #- include lower and upper columns if not available
    if ( ! ( 'lower' %in% colnames(partable) ) ){
        partable$lower <- rep(-Inf, nrow(partable))
    }
    if ( ! ( 'upper' %in% colnames(partable) ) ){
        partable$upper <- rep(Inf, nrow(partable))
    }

    #----- output
    res <- list( itemtype=itemtype, partable=partable,
                    partable_index=partable_index, ncat=ncat,
                    maxK=maxK, mstep_method=mstep_method,
                    item_index=item_index )
    return(res)
}

## File Name: xxirt_irf_lca.R
## File Version: 0.071

xxirt_irf_lca <- function(par, Theta, ncat)
{
    K <- nrow(Theta)
    P <- matrix( NA, nrow=K, ncol=ncat)
    P[,1] <- 0
    for (hh in 2L:ncat){
        b <- par[ K*(hh-2) + seq(1,K) ]
        P[,hh] <- b
    }
    a1M <- matrix(apply(P, 1, max), nrow=K, ncol=ncat)
    P <- exp(P-a1M)
    P <- P / rowSums(P)
    return(P)
}

## File Name: rm.facets.R
## File Version: 4.645

#################################################################
# Facets Model for Raters:
# MML estimation
rm.facets <- function( dat, pid=NULL, rater=NULL,
    Qmatrix=NULL, theta.k=seq(-9,9,len=30),
    est.b.rater=TRUE, est.a.item=FALSE, est.a.rater=FALSE, rater_item_int=FALSE,
    est.mean=FALSE, tau.item.fixed=NULL, a.item.fixed=NULL, b.rater.fixed=NULL, a.rater.fixed=NULL,
    b.rater.center=2, a.rater.center=2, a.item.center=2,  a_lower=.05, a_upper=10,
    reference_rater=NULL, max.b.increment=1, numdiff.parm=.00001, maxdevchange=.10,
    globconv=.001, maxiter=1000, msteps=4, mstepconv=.001, PEM=FALSE, PEM_itermax=maxiter)
{

    CALL <- match.call()
    s1 <- Sys.time()

    skillspace <- "normal"
    dat0 <- dat <- as.matrix(dat)
    rater0 <- rater
    if ( is.null(rater)){
        rater <- rep(1,nrow(dat))
        est.b.rater <- FALSE
        est.a.rater <- FALSE
    }
    if ( is.null(pid)){
        pid <- seq(1,nrow(dat) )
    }
    pcm.param <- FALSE
    theta.k0 <- theta.k
    pi.k <- sirt_dnorm_discrete(x=theta.k, mean=0, sd=1)

    # process data
    res <- rm_proc_data( dat=dat, rater=rater, pid=pid, rater_item_int=rater_item_int,
                            reference_rater=reference_rater)
    procdata <- res
    dat2 <- as.matrix(res$dat2)
    dat2.resp <- as.matrix(res$dat2.resp)
    rater.index1 <- res$rater.index
    dataproc.vars <- res$dataproc.vars
    VV <- res$VV
    RR <- res$RR
    item.index <- res$dataproc.vars$item.index
    rater.index <- res$dataproc.vars$rater.index
    dat2.ind.resp <- res$dat2.ind.resp
    rater <- res$rater
    dat <- res$dat
    pid <- res$pid
    reference_rater <- res$reference_rater

    deviance.history <- rep(NA, maxiter)

    #--- fixed values for raters
    res <- rm_proc_fixed_values_reference_rater( rater.index1=rater.index1,
                b.rater.fixed=b.rater.fixed, a.rater.fixed=a.rater.fixed,
                rater_item_int=rater_item_int, reference_rater=reference_rater,
                est.b.rater=est.b.rater, est.a.rater=est.a.rater )
    a.rater.fixed <- res$a.rater.fixed
    b.rater.fixed <- res$b.rater.fixed

    #-- maximum categories
    maxK <- sirt_colMaxs(x=dat)

    K <- max( maxK )
    if ( is.null(Qmatrix) ){
        Qmatrix <- matrix( 1L:K, nrow=VV, ncol=K, byrow=TRUE)
    }
    TP <- length(theta.k)
    I <- VV*RR

    # center parameters
    if ( ! is.null( b.rater.fixed) ){
        b.rater.center <- 0
    }
    if ( ! is.null( a.rater.fixed) ){
        a.rater.center <- 0
    }
    if ( ! is.null( a.item.fixed) ){
        a.item.center <- 0
    }

    if ( skillspace=="loglinear" ){
        est.mean <- TRUE
    }

    # define constraints on tau.item parameters
    # if not all categories are observed
    tau.item.fixed_val <- tau.item.fixed
    tau.item.fixed <- NULL
    if ( min(maxK) < K ){
        tau.item.fixed <- rm_determine_fixed_tau_parameters( K=K, maxK=maxK, VV=VV )
    }

    # starting values for item difficulties
    b.item <- - stats::qlogis( colMeans( dat, na.rm=TRUE ) / maxK  )
    if ( ! pcm.param ){
        b.item <- 0*b.item
    }
    #--- tau parameters
    tau.item <- matrix( 0, nrow=VV, ncol=K )
    rownames(tau.item) <- colnames(dat)
    tau.item <- matrix( seq( -2, 2, len=K ), nrow=VV, ncol=K, byrow=TRUE )
    #--- rater parameters
    M1 <- colSums( dat2 ) / colSums( dat2.resp )
    N <- colSums( dat2.resp )
    N <- stats::aggregate( N, list( rater.index ), sum, na.rm=TRUE )[,2]
    M1 <- stats::aggregate( M1, list( rater.index ), mean, na.rm=TRUE )[,2]
    b.rater <- - stats::qlogis( M1 / K )
    b.rater <- b.rater - mean( b.rater )
    a.item <- rep(1,VV)
    a.rater <- rep(1,RR)
    if ( ! est.b.rater ){
        b.rater <- rep(0,RR)
    }

    # init standard errors
    se.b.rater <- NA*b.rater
    se.a.rater <- NA*a.rater
    se.a.item <- NA*a.item

    #-- preliminaries PEM acceleration
    if (PEM){
        res <- rm_facets_pem_inits( tau.item=tau.item, a.item=a.item, a.rater=a.rater,
                        b.rater=b.rater, skillspace=skillspace, PEM=PEM, a.item.fixed=a.item.fixed,
                        est.a.item=est.a.item )
        PEM <- res$PEM
        pem_pars <- res$pem_pars
        pem_parameter_index <- res$pem_parameter_index
        pem_parameter_sequence <- res$pem_parameter_sequence
    }

    # inits
    iter <- 0
    dev0 <- dev <- 0
    conv <- devchange <- 1000
    sigma <- 1
    disp <- "...........................................................\n"

    b.rater.incr <- max.b.increment
    tau.item.incr  <- max.b.increment

    active <- TRUE
    active <- FALSE

    #****************************************************
    # start EM algorithm
    while( ( ( maxdevchange < devchange ) | (globconv < conv) ) &
            ( iter < maxiter )    ){
        cat(disp)
        cat("Iteration", iter+1, "   ", paste( Sys.time() ), "\n" )

        # previous values
        b.item0 <- b.item
        b.rater0 <- b.rater
        tau.item0 <- tau.item
        dev0 <- dev
        sigma0 <- sigma
        a.item0 <- a.item
        a.rater0 <- a.rater
        # calculate probabilities
        probs <- rm_facets_calcprobs( b.item=b.item, b.rater=b.rater, Qmatrix=Qmatrix,
                        tau.item=tau.item, VV=VV, K=K,
                        I=I, TP=TP, a.item=a.item, a.rater=a.rater, item.index=item.index,
                        rater.index=rater.index, theta.k=theta.k, RR=RR )
        # calculate posterior
        res <- rm_posterior( dat2=dat2, dat2.resp=dat2.resp, TP=TP, pi.k=pi.k, K=K, I=I,
                    probs=probs, dat2.ind.resp=dat2.ind.resp )
        f.yi.qk <- res$f.yi.qk
        f.qk.yi <- res$f.qk.yi
        n.ik <- res$n.ik
        N.ik <- res$N.ik
        pi.k <- res$pi.k
        ll <- res$ll

        #--- estimate b.rater parameter
        if( est.b.rater ){
            res <- rm_facets_est_b_rater( b.item=b.item, b.rater=b.rater, Qmatrix=Qmatrix, tau.item=tau.item, VV=VV, K=K,
                            I=I, TP=TP, a.item=a.item, a.rater=a.rater, item.index=item.index,
                            rater.index=rater.index, n.ik=n.ik, numdiff.parm=numdiff.parm,
                            max.b.increment=1, theta.k=theta.k, msteps=msteps,
                            mstepconv=mstepconv, b.rater.center=b.rater.center,
                            b.rater.fixed=b.rater.fixed )
            b.rater <- res$b.rater
            se.b.rater <- res$se.b.rater
            b.rater.incr <- abs( b.rater0 - b.rater )
            # b.rater.incr <- 1
        }

        #--- estimate tau.item parameters
        res <- rm_facets_est_tau_item( b.item=b.item, b.rater=b.rater, Qmatrix=Qmatrix, tau.item=tau.item, VV=VV, K=K,
                    I=I, TP=TP, a.item=a.item, a.rater=a.rater, item.index=item.index,
                    rater.index=rater.index, n.ik=n.ik, numdiff.parm=numdiff.parm,
                    max.b.increment=1, theta.k=theta.k, msteps=msteps,
                    mstepconv=mstepconv, tau.item.fixed=tau.item.fixed,
                    tau.item.fixed_val=tau.item.fixed_val )
        tau.item <- res$tau.item
        se.tau.item <- res$se.tau.item
        tau.item.incr  <- abs( tau.item0 - tau.item )
        # tau.item.incr  <- 1

        #--- estimate a.item parameter
        if (est.a.item){
            res <- rm_facets_est_a_item( b.item=b.item, b.rater=b.rater, Qmatrix=Qmatrix,
                        tau.item=tau.item, VV=VV, K=K, I=I, TP=TP,
                        a.item=a.item, a.rater=a.rater, item.index=item.index, rater.index=rater.index,
                        n.ik=n.ik, numdiff.parm=numdiff.parm, max.b.increment=1, theta.k=theta.k, msteps=msteps,
                        mstepconv=mstepconv, a.item.center=a.item.center, a.item.fixed=a.item.fixed, a_lower=a_lower,
                        a_upper=a_upper )
            a.item <- res$a.item
            se.a.item <- res$se.a.item
        }

        #--- estimate a.rater parameter
        if (est.a.rater){
            res <- rm_facets_est_a_rater( b.item=b.item, b.rater=b.rater, Qmatrix=Qmatrix,
                        tau.item=tau.item, VV=VV, K=K, I=I, TP=TP,
                        a.item=a.item, a.rater=a.rater, item.index=item.index, rater.index=rater.index,
                        n.ik=n.ik, numdiff.parm=numdiff.parm, max.b.increment=1, theta.k=theta.k, msteps=msteps,
                        mstepconv=mstepconv, a.rater.center=a.rater.center, a.rater.fixed=a.rater.fixed, a_lower=a_lower,
                        a_upper=a_upper )
            a.rater <- res$a.rater
            se.a.rater <- res$se.a.rater
        }
        utils::flush.console()

        #-- update distribution
        res <- rm_smooth_distribution( theta.k=theta.k, pi.k=pi.k, est.mean=est.mean, skillspace=skillspace )
        pi.k <- res$pi.k
        mu <- res$mu
        sigma <- res$sigma
        #-- PEM acceleration
        if (PEM){
            res <- rm_facets_pem_acceleration( iter=iter, pem_parameter_index=pem_parameter_index,
                        pem_parameter_sequence=pem_parameter_sequence, a.rater=a.rater, Qmatrix=Qmatrix,
                        tau.item=tau.item, VV=VV, K=K, I=I, TP=TP, a.item=a.item, b.item=b.item, b.rater=b.rater,
                        item.index=item.index, rater.index=rater.index, theta.k=theta.k, RR=RR, dat2=dat2,
                        dat2.resp=dat2.resp, pi.k=pi.k, dat2.ind.resp=dat2.ind.resp, ll=ll, mu=mu, sigma=sigma,
                        pem_pars=pem_pars, a_center_type=a.rater.center, PEM_itermax=PEM_itermax,
                        b.rater.center=b.rater.center, a.rater.center=a.rater.center,
                        a.item.center=a.item.center, a_lower=a_lower, a_upper=a_upper )
            ll_pem <- res$ll
            pem_parameter_sequence <- res$pem_parameter_sequence
            a.rater <- res$a.rater
            b.rater <- res$b.rater
            a.item <- res$a.item
            tau.item <- res$tau.item
            pi.k <- res$pi.k
            mu <- res$mu
            sigma <- res$sigma
            PEM <- res$PEM
        }

        #-- save deviance values
        deviance.history[iter+1] <- dev <- -2*ll

        #-- convergence criteria
        conv <- max( abs(b.rater-b.rater0), abs( a.rater-a.rater0), abs( tau.item0-tau.item), abs( a.item - a.item0 ) )
        iter <- iter+1
        devchange <- abs( ( dev - dev0 ) / dev0  )

        #-- print progress
        res <- rm_facets_print_progress( dev=dev, dev0=dev0, b.rater=b.rater, b.rater0=b.rater0,
                    a.rater=a.rater, a.rater0=a.rater0,
                    tau.item=tau.item, tau.item0=tau.item0, a.item=a.item, a.item0=a.item0, mu=mu, sigma=sigma,
                    iter=iter )
    }

    #*********
    # arrange OUTPUT
    #---
    ic <- rm_facets_ic( dev=dev, dat2=dat2, VV=VV, RR=RR, maxK=maxK, a.item.center=a.item.center,
                est.a.item=est.a.item, est.b.rater=est.b.rater, est.a.rater=est.a.rater,
                est.mean=est.mean, b.rater.center=b.rater.center, a.rater.center=a.rater.center,
                b.rater.fixed=b.rater.fixed, a.rater.fixed=a.rater.fixed,
                tau.item.fixed_val=tau.item.fixed_val, a.item.fixed=a.item.fixed )
    #---
    # person
    res <- rm_facets_postproc_person( dat2=dat2, dat2.resp=dat2.resp, procdata=procdata, maxK=maxK, RR=RR, theta.k=theta.k,
                f.qk.yi=f.qk.yi )
    person <- res$person
    EAP.rel <- res$EAP.rel

    #---
    # item
    if ( ! is.null(tau.item.fixed) ){
        tau.item[ tau.item.fixed[,1L:2,drop=FALSE] ] <- NA
        se.tau.item[ tau.item.fixed[,1L:2,drop=FALSE] ] <- NA
    }
    item <- data.frame( item=colnames(dat),
                        N=colSums( 1-is.na(dat)),
                        M=colMeans( dat, na.rm=TRUE ) )
    for (kk in 1L:K){
        item[, paste0("tau.Cat",kk) ] <- tau.item[,kk]
    }
    item$a <- a.item

    delta.item <- pcm.conversion(tau.item)$delta
    item$delta <- delta.item
    item$delta_cent <- rm_facets_center_value(x=item$delta, value=0)

    cat("*********************************\n")
    cat("Item Parameters\n")
    sirt_summary_print_objects(obji=item, digits=3, from=2)

    #--- table rater parameters
    rater <- rm_facets_postproc_rater_parameters( rater.index=rater.index, dat2=dat2,
                    dat2.resp=dat2.resp, b.rater=b.rater,
                    a.rater=a.rater, rater.index1=rater.index1, rater_item_int=rater_item_int )

    #--- dimnames probs
    dimnames(probs)[[1]] <- colnames(dat2)

    #--- expanded item parameters
    ipars.dat2 <- rm_facets_itempar_expanded( b.item=b.item, b.rater=b.rater, Qmatrix=Qmatrix,
                        tau.item=tau.item, VV=VV, K=K, I=I, TP=TP,
                        a.item=a.item, a.rater=a.rater, item.index=item.index, rater.index=rater.index,
                        theta.k=theta.k, RR=RR )

    cat("*********************************\n")
    cat("Rater Parameters\n")
    sirt_summary_print_objects(obji=rater, digits=3, from=2)

    cat("*********************************\n")
    cat("EAP Reliability=", round(EAP.rel,3), "\n")

    s2 <- Sys.time()
    res <-  list( deviance=dev, ic=ic, item=item, rater=rater, person=person,
                    EAP.rel=EAP.rel, mu=mu, sigma=sigma, theta.k=theta.k, pi.k=pi.k, G=1,
                    tau.item=tau.item, se.tau.item=se.tau.item,
                    a.item=a.item, se.a.item=se.a.item, delta.item=delta.item,
                    b.rater=b.rater, se.b.rater=se.b.rater, a.rater=a.rater,
                    se.a.rater=se.a.rater, f.yi.qk=f.yi.qk, f.qk.yi=f.qk.yi,
                    probs=probs, n.ik=n.ik, maxK=maxK, procdata=procdata, iter=iter,
                    s1=s1, s2=s2, tau.item.fixed=tau.item.fixed, item.index=item.index,
                    rater.index=rater.index, ipars.dat2=ipars.dat2,
                    rater_item_int=rater_item_int, CALL=CALL,
                    deviance.history=deviance.history )
    class(res) <- "rm.facets"
    return(res)
}

## File Name: rm_facets_calcprobs.R
## File Version: 0.102


#--- cpp implementation of calculation of facets probabilities
rm_facets_calcprobs <- function( b.item, b.rater, Qmatrix, tau.item,
        VV, K, I, TP, a.item, a.rater, item.index, rater.index,
        theta.k, RR )
{
    probs <- rm_facets_calcprobs_cpp( b_item=b.item, b_rater=b.rater,
                    Qmatrix=Qmatrix, tau_item=tau.item, K=K, I=I,
                    TP=TP, a_item=a.item, a_rater=a.rater, item_index=item.index-1,
                    rater_index=rater.index-1, theta_k=theta.k )
    probs <- array( probs, dim=c(I, K+1, TP ) )
    return(probs)
}

.rm.facets.calcprobs2 <- rm_facets_calcprobs

## File Name: cfa_meas_inv.R
## File Version: 0.161


cfa_meas_inv <- function(dat, group, weights=NULL, alpha=0.01, verbose=FALSE,
                    op=c("~1","=~") )
{
    requireNamespace('lavaan')
    #--- define data input
    items <- colnames(dat)
    if (is.null(weights)){
        weights <- rep(1, nrow(dat))
    }
    dat <- data.frame(group=group, weight=weights, dat)
    args <- list(data=dat, group='group')
    args$sampling.weights <- 'weight'

    #--- define model under scalar invariance
    lavmodel <- invariance_alignment_cfa_config_estimate_define_lavaan_model(
                        items_gg=items, label_F='F', model='2PM')
    args$model <- lavmodel
    args$meanstructure <- TRUE
    args$std.lv <- TRUE
    args$group.equal <- c('loadings','intercepts')

    mod1 <- lavaan::cfa(data=dat, model=lavmodel, std.lv=TRUE, meanstructure=TRUE,
                    group.equal=args$group.equal, group='group',
                    sampling.weights='weight')
    partable1 <- lavaan::parameterTable(object=mod1)
    mimod1 <- mi_inv_lavaan_modification_indices(mod=mod1, op=op)
    partable0 <- partable1
    partable <- partable1
    mod0 <- mod1

    pars_mi <- meas_inv_cfa_proc_partable(partable=partable1, items=items)

    #--- change model by subsequently freeing parameters
    nfp <- 0
    critval <- stats::qchisq(1-alpha, df=1)
    free <- TRUE
    while(free){

        res <- meas_inv_cfa_modify_partable(partable=partable1, mimod=mimod1,
                            critval=critval)
        free <- res$free_parameter

        if (free){
            nfp <- nfp+1
            partable2 <- res$partable
            mod2 <- lavaan::cfa(data=dat, model=partable2, group='group',
                        sampling.weights='weight')
            partable1 <- lavaan::parameterTable(object=mod2)
            mimod1 <- mi_inv_lavaan_modification_indices(mod=mod2, op=op)
            if (verbose){
                cat(paste0('freed ', nfp, ' parameters\n' ) )
                utils::flush.console()
            }

        }
        mod1 <- mod2
    }

    pars_pi <- meas_inv_cfa_proc_partable(partable=partable1, items=items)

    #--- output
    res <- list(pars_mi=pars_mi, pars_pi=pars_pi, alpha=alpha, critval=critval,
                    nfp=nfp, partable=partable1, dat=dat, items=items,
                    mod_mi=mod0, mod_pi=mod1)
    return(res)
}

## File Name: rm_numdiff_trim_increment.R
## File Version: 0.12

rm_numdiff_trim_increment <- function( increment, max.increment, eps2 )
{
    aincr <- abs(increment)
    amaxincr <- abs(max.increment)
    ci <- ceiling( aincr / ( amaxincr + eps2 ) )
    increment <- ifelse( aincr > amaxincr, increment/(2*ci), increment)
    return(increment)
}

## File Name: rasch.jml.R
## File Version: 3.315


rasch.jml <- function( dat, method="MLE", b.init=NULL, constraints=NULL, weights=NULL,
                    center="persons", glob.conv=10^(-6), conv1=0.00001, conv2=0.001, progress=TRUE,
                    bsteps=4, thetasteps=2, wle.adj=0, jmliter=100, prox=TRUE, proxiter=30, proxconv=0.01,
                    dp=NULL, theta.init=NULL, calc.fit=TRUE, prior_sd=NULL)
{
    CALL <- match.call()
    s1 <- Sys.time()
    # warning messages
    if ( max( dat, na.rm=TRUE )  > 1){
        stop( "You should only have dichotomous data")
    }
    # zz0 <- Sys.time()

    # display
    if ( progress ){
        cat("-----------------------------------------------------------------\n")
        cat("Joint Maximum Likelihood Estimation \n")
        cat("Rasch Model \n")
        cat("-----------------------------------------------------------------\n")
    }

    # prior for ability
    if (! is.null(prior_sd) ){
        is_prior <- TRUE
    } else {
        is_prior <- FALSE
    }

    #--- centering of parameters
    centeritems <- FALSE
    centerpersons <- FALSE
    if (center=="persons"){
        centerpersons <- TRUE
    }
    if (center=="items"){
        centeritems <- TRUE
    }
    if (! is.null(constraints) ){
        prox <- FALSE
        centerpersons <- FALSE
        center <- "none"
    }

    # data preparations
    use_weights <- FALSE
    if (!is.null(weights)){
        use_weights <- TRUE
        prox <- FALSE
    }
    if ( is.null(dp) ){
        dp <- data.prep( dat, weights=weights, use.freqpatt=FALSE,
                standardize_weights=FALSE)
    }
    dat1 <- dp$dat1
    dat2 <- dp$dat2
    dat2.resp <- dp$dat2.resp
    freq.patt <- dp$freq.patt
    I <- dp$I
    N <- n <- dp$n

    # exclude extreme item response pattern (if method==MLE)
    if ( ( method=="MLE" ) & ( ! is_prior ) ){
        ind <- which( ! dat1$mean %in% c(0,1) )
        dat1 <- dat1[ ind,]
        dat2 <- dat2[ ind,]
        dat2.resp <- dat2.resp[ ind,]
        N <- nrow(dat1)
    }

    #**** exist some missings?
    some.missings <- 1 * ( mean( rowMeans( dat2.resp) ) < 1 )
    # compute missing response pattern
    dat1$mp <- paste("MP", paste(rep(1,I), collapse=""), sep="")
    dat1$mp.index <- 1
    if ( some.missings ){
        dat1$mp.index <- resp.pattern2( dat2.resp )$mp.index
    }
    dat1$sumscore <- rowSums( dat2 * dat2.resp )
    ti <- paste( dat1$mp.index, "-", dat1$sumscore, sep="")
    dat1$theta.index <- match( ti, unique( ti ))
    dat1$caseid <- seq( 1, nrow(dat1) )

    theta.pattern <- as.data.frame( stats::aggregate( dat1$caseid, list(dat1$theta.index), min ))
    colnames(theta.pattern) <- c("theta.index", "caseid")

    theta.pattern[,"Freq"] <- rowsum( dat1$Freq, dat1$theta.index )[,1]
    TPsapp <- 1:nrow(theta.pattern)
    freq.thetapattern <- rowsum( dat1$Freq, dat1$theta.index )[,1]
    if ( some.missings ){
        freq.dat.resp.thetapattern <- rowsum( dat1$Freq  * dat2.resp, dat1$theta.index )
    } else {
        freq.dat.resp.thetapattern <- matrix( freq.thetapattern, nrow=length(freq.thetapattern), ncol=I )
    }

    #** compute sufficient statistic
    suffB <- colSums(  - dat1[,2] * dat2.resp * dat2 )
    d2d2r <- dat2 * dat2.resp
    d2d2m <- ( 1 - dat2 ) * dat2.resp

    # starting values item parameter
    if ( is.null(b.init) ){
        b <- - stats::qlogis( colSums( dat2 * dat2.resp * dat1[,2] ) / colSums( dat1[,2] * dat2.resp ) )
    } else {
        b <- b.init
    }
    if (!is.null(constraints)){
        b[ constraints[,1] ] <- constraints[,2]
    }

    # starting values person paramters
    if ( is.null(theta.init)){
        theta <- stats::qlogis( dat1$mean * ( 1  - 1 / I ) + 1 / ( 2*I ) )
    } else {
        theta <- theta.init
        theta[ theta==Inf ] <- 20
        theta[ theta==-Inf ] <- -20
    }

    #--- PROX estimation
    if (prox){
        if (method=="WLE" ){
            ind <- which( ! dat1$mean %in% c(0,1) )
        } else {
            ind <- 1:( nrow(dat1 ))
        }
        prox.res <- rasch.prox( dat=dat2[ind,], dat.resp=dat2.resp[ind,], freq=dat1[ind,2],
                        conv=proxconv, progress=progress, maxiter=proxiter )
        b <- prox.res$b
        theta[ind] <- prox.res$theta
    }
    # centering
    theta <- rasch_jml_centerpersons(theta=theta, dat1=dat1, centerpersons=centerpersons)
    b <- rasch_jml_centeritems(b=b, centeritems=centeritems)

    # initial settings
    dev0 <- 1
    dev.change <- 1
    iter <- 0
    b0 <- 0
    bconv <- 10^10
    disp <- "...............................\n"

    #------------------------------------------------------
    # JML Iteration Algorithm
    while ( ( ( dev.change > glob.conv ) & ( iter < jmliter ) ) | ( bconv > conv1 ) ){
        b0 <- b
        # update item difficulties
        if ( progress ){
            cat(disp)
            cat("JML Iteration", iter +1, "\n")
            cat("  Item parameters |")
        }
        b <- rasch_jml_update_b( b0, theta=theta[theta.pattern$caseid],
                        freq.thetapattern=freq.thetapattern,
                        freq.dat.resp.thetapattern=freq.dat.resp.thetapattern,
                        constraints=constraints, conv=conv2,
                        suffB=suffB, progress=progress, bsteps=bsteps)
        b <- rasch_jml_centeritems(b=b, centeritems=centeritems)
        # ability estimate (method==MLE or method==WLE)
        ind <- theta.pattern$caseid
        if (method=="WLE" ){
            m1 <- wle.rasch( dat=dat2[ind,], b=b, theta=theta[ind],
                        dat.resp=dat2.resp[ind, ], conv=conv2, progress=progress,
                        maxit=thetasteps, wle.adj=wle.adj )
        }
        if (method=="MLE" ){
            m1 <- mle.rasch( dat=dat2[ind,], b=b, theta=theta[ind], dat.resp=dat2.resp[ind,],
                            conv=conv2,    progress=progress, prior_sd=prior_sd )
        }
        theta <- m1$theta
        theta <- theta[ dat1$theta.index ]
        # centering
        theta <- rasch_jml_centerpersons(theta=theta, dat1=dat1, centerpersons=centerpersons)
        # calculate Log-Likelihood and Deviance
        p.ia <- (m1$p.ia)[ dat1$theta.index, ]
        dev1 <- -2*sum( rowSums( ( d2d2r*log(p.ia) + d2d2m*log( 1 - p.ia ))) * dat1[,2] )
        # relative deviance change
        dev1a <- - (dev1 - dev0)
        dev.change <- abs( dev1 - dev0 ) / dev0 ;
        dev0 <- dev1
        # iteration index
        iter <- iter + 1
        # display convergence
        bconv <- max( abs(b - b0 ) )
        if ( progress ){
            cat( paste( "\n  Deviance", sirt_sign_space(), round( dev1, 5 ) ))
            if (iter > 1){
                cat( paste0( " | Deviance change", sirt_sign_space(), round( dev1a, 5 ) ) )
            }
            cat( paste0("\n  Max. parm. change", sirt_sign_space(), round( bconv, 6 ), "\n"  ) )
            utils::flush.console()
        }
    }
    #------------- end algorithm

    if ( progress ){
        if ( iter < jmliter ){
            cat( paste( "Convergence reached in", iter, "JML Iterations \n" ) )
        } else {
            cat( paste( "Analysis terminated at", iter, "JML Iterations \n" ) )
        }
        cat("-----------------------------------------------------------------\n\n")
    }
    dat1 <- dat1[,1:3]
    # arrange ability estimates
    abil <- data.frame( dat1, theta )
    abil <- merge( freq.patt, abil[ c(1,4)], 1, 1, all=T )
    abil <- abil[ order(abil[,3]), c( 3,1,2,4) ]
    colnames(abil) <- c("person", "pattern", "meancorrect", "theta" )
    if (method=="MLE"){
        abil[abil$meancorrect==0, 4 ] <- -Inf
        abil[abil$meancorrect==1, 4 ] <- Inf
    }

    # standard error of person parameter
    abil$se.theta <- rasch.info.mle( dat=dat, theta=abil$theta, b=b * (I-1)/I)

    #- summary ability parameters
    theta_summary <- rasch_jml_person_parameters_summary(x=abil[ is.finite(abil$theta), "theta"])

    # empirical discrimination
    a.i <- rasch_jml_emp_discrim( theta=theta, b=b, dat=dat2, dat.resp=dat2.resp, freq=dat1[,2] )

    # calculate itemfit statistics
    if (calc.fit){
        fit <- rasch.itemfit( theta0=abil$theta, b=b, dat )
    } else {
        fit <- NULL
    }


    # standard error for item parameter
    se.b <- .se.item.rasch( theta=theta, b=b, dat.resp=dat2.resp, freq=dat1[,2],
                    constraints=constraints)

    # Data frame for "item side"
    I <- length(b)
    # item parameter constraints and correction formula
    UJJ <- 0
    # item summary
    dfr <- data.frame( N=colSums(dat2.resp*dat1[,2]),
                p=colMeans( dat, na.rm=TRUE ), itemdiff=b,
                itemdiff.correction=b*(I-1)/I, se=se.b, discr=a.i )
    if ( ! is.null(constraints)    ){
        dfr$itemdiff.correction <- dfr$itemdiff
    }


    # include item fit statistics                              #
    if (calc.fit){
        dfr <- data.frame( dfr, fit )
    }
    rownames(dfr) <- colnames(dat)

    # processed data
    data.proc <- list( "data"=dat2, "data.resp"=dat2.resp, "theta"=theta )
    s2 <- Sys.time()
    time_diff <- s2-s1
    #--- output
    res <- list( item=dfr, person=abil, theta_summary=theta_summary,
                    method=method,dat=dat, deviance=dev1,
                    data.proc=data.proc, dp=dp, constraints=constraints, N=N, I=I,
                    method=method, center=center, iter=iter,
                    CALL=CALL, s1=s1, s2=s2, time_diff=time_diff )
    class(res) <- "rasch.jml"
    return(res)
}

# cat("fit") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1

#---------------------------------------------------------------------------------
# Functions for calculating standard errors in the Rasch Model                   #
.se.item.rasch <- function( theta, b, dat.resp, freq, constraints ){
        # standard error for item parameter
        pij <- .prob.rasch( theta=theta, b=b )
        se.b <- sqrt( 1/colSums( freq * pij * ( 1 - pij ) * dat.resp )) # calculate information function
        if (!is.null(constraints)){ se.b[ constraints[,1]] <- NA }
        se.b
        }
#---------------------------------------------------------------------------------

#-----------------------------------------------------
# Calculate of standard measurement error using
# test information function from the Rasch model
##NS export(rasch.info.mle)
rasch.info.mle <- function( dat, theta, b){
    #......................
    # INPUT:
    # dat   ... original data
    # theta ... ability estimates
    # b     ... item difficulties
    dat.resp <- 1 - is.na(dat)
    p.ia <- stats::plogis( outer( theta, b, "-" ) )
    # calculate information function
    info <- rowSums( dat.resp * p.ia * ( 1- p.ia ) )
    # calculate standard error
    sqrt( 1 / info )
    }
#------------------------------------------------------

# auxiliary functions
#-----------------------------------------------------------------------------------------------------------#
# gradient Rasch function
.b.gradient.rasch <- function( b, theta, freq, dat, dat.resp ){
    colSums(  - freq * dat.resp * ( dat - stats::plogis(  outer( theta, b, "-" ) ) ) )
    }
# Information Matrix item difficulties
.infb.rasch1 <- function( b, theta, freq ){
#    p.ia <- plogis( outer( theta, b, "-" ) )
    p.ia <- stats::plogis( theta, matrix( b, nrow=length(theta), length(b), byrow=T ) )
    colSums( - freq * p.ia * ( 1- p.ia ) )
    }
.infb.rasch <- function( b, theta, freq ){
    p.ia <- stats::plogis( outer( theta, b, "-" ) )
    colSums( - freq * p.ia * ( 1- p.ia ) )
    }

#---- calculate P_i ( theta)
.prob.rasch <- function( theta, b )
{
    bM <- matrix( b, nrow=length(theta), length(b), byrow=TRUE )
    res <- stats::plogis( theta - bM )
    return(res)
}

# calculate P_i(theta) for 3PL model
.prob.3pl <- function( theta, b, a, c){
    l1  <- rep( 1, length(theta) )
    cM <- outer( l1, c )
    aM <- outer( l1, a )
    bM <- outer( l1, b )
    thetaM <- outer( theta, rep(1,length(b) ) )
    cM + (1-cM) * stats::plogis( aM*(thetaM - bM ) )
    }
#-----------------------------------------------------------------------------------------------------------#

#-------------------------------------------------------------------
# calculate information function in the Rasch model
.info.rasch <- function( theta, b ){
    pij <-  .prob.rasch( theta=theta, b=b )
    sqrt(  1 / rowSums( pij * ( 1 - pij ) ) )
        }
#-------------------------------------------------------------------#
# calculate (mean) reliability for known variance and mean for      #
# latent trait distribution                                         #
##NS # export(mle.reliability.rasch)
mle.reliability.rasch <- function(  b, mean.abil=NULL, var.abil=NULL, npoints=200,
            theta=NULL){
    # INPUT:
    # b ... item difficulties
    # mean.abil ... mean of ability distribution
    # var.abil ... variance of ability distribution
    # npoints ... number of discretization points
    # theta grid ... optional
    #...................................
    q.grid <- seq( 1 /npoints, 1 - 1/npoints, len=npoints )
    if ( is.null(theta)){
            theta <- stats::qnorm( q.grid, mean=mean.abil, sd=sqrt( var.abil ) )
                        }
    info <- .info.rasch( theta=theta, b=b )
    # mean estimate of variance
    mean.var.est <- mean( info^2 )
    # Note: info.grid is the standard error
    list( "theta.grid"=theta, "info.grid"=info, "mean.var.est"=mean.var.est,
            "I"=length(b),     "mle.rel"=var.abil /( var.abil + mean.var.est ) )
    }
#-------------------------------------------------------------------#

#--------------------------------------------------------------------------------------------------------#
# update item difficulty estimation (Rasch model)
.update.b.rasch.jml1 <- function( b, theta, freq, dat, dat.resp,  constraints=NULL, conv=.0001,
                suffB, thetaindex=NULL, progress=FALSE ){
    #-------------------------------------------------------
    # INPUT:
    # b     ... initial item difficulties
    # theta ... ability estimate
    # freq  ... frequency of item response pattern (theta estimate)
    # dat   ... data frame
    # update item difficulties
    b.change <- 1
    while( max( abs( b.change  ) ) > conv ){
        p.ia <- stats::plogis( theta, matrix( b, nrow=length(theta), length(b), byrow=T ) )
        if ( ! is.null( thetaindex ) ){
            p.ia <- p.ia[ thetaindex, ]
                                    }
        deriv <- colSums( - freq * p.ia * ( 1- p.ia ) )
        diff <- suffB + colSums( freq * dat.resp *  p.ia  )
        b.change <-  diff / deriv
        if (! is.null(constraints)){
                b.change[ constraints[,1] ] <- 0
                    }
        if (progress){    cat("-"); utils::flush.console() }
        b <- b - b.change
        }
    b
    }
#-----------------------------------------------------------------------------------------------------------#

#--------------------------------------------------------------------------------------------------------#
# update item difficulty estimation (Rasch model)
.update.b.rasch.jml <- function( b, theta, freq, dat, dat.resp,  constraints=NULL, conv=.0001 ){
    #-------------------------------------------------------
    # INPUT:
    # b     ... initial item difficulties
    # theta ... ability estimate
    # freq  ... frequency of item response pattern (theta estimate)
    # dat   ... data frame
    # update item difficulties
    b.change <- 1
    while( max( abs( b.change  ) ) > conv ){
        deriv <- .infb.rasch( b, theta, freq )
        b.change <-  .b.gradient.rasch( b, theta, freq, dat, dat.resp ) / deriv
        if (! is.null(constraints)){  b.change[ constraints[,1] ] <- 0 }
        b <- b - b.change
        }
    b
    }
#-----------------------------------------------------------------------------------------------------------#

#-------------------------------------------------------------------------------------------------------
# calculate empirical item discrimination
item.discrim <- function( dat, score ){
    # INPUT:
    # dat       ... data frame
    # score     ... score with which items are to be correlated (e.g. sum score, WLE, ...)
    apply( dat, 2, FUN=function(variable){ cor( variable,  score, use="complete.obs") } )
    }
#------------------------------------------------------------------------------------------------------




##########################################################################################
rasch.jml.jackknife1 <- function( jmlobj  ){
    #, jackunits=NULL
    jackunits <- NULL
    mod <- jmlobj
    # define jackknife units
    cat("Joint Maximum Likelihood Estimation \nJackknife Estimation\n")
    dat <- jmlobj$dat
    Nobs <- nrow( dat )
    I <- ncol(dat)
    if ( is.null(jackunits) ){ jackunits <- 1:I }
    UJ <- max( jackunits)
    # define jackknife units
    if ( length(jackunits) > 1){
            jackunits <- match( jackunits, unique( jackunits) )
                                }
    # define progress bar
    u.jackunits <- unique( jackunits )
    cat( UJ, "Jackknife Units are used\n" )
    prbar <- paste( "|", paste(rep("-", 20 ), collapse=""), "PROGRESS",
                    paste(rep("-",20 ), collapse=""), "|", sep="")
    cat(prbar,"\n")
    #***************
    # initialize matrices
    cat("|")
    j.itemdiff <- matrix(NA,, nrow=I, ncol=UJ )
    # calculate jackunits indices when progress should be displayed
    prbar.display <- floor( seq( 1, max(u.jackunits), len=48 ) )
    # collect constraints
    constraints <- jmlobj$constraints
    #**************************
    # BEGIN JACKKNIFE
    for ( uu in u.jackunits ){
        # uu <- 10
        dat.uu <- dat[, - uu]
        if ( is.null(constraints) ){
            constraints.uu <- NULL } else {
            constraints.uu <- constraints[ constraints[,1] !=uu, ]
            M1 <- cbind( setdiff( 1:I, uu ), seq(1,ncol(dat.uu) ) )
            constraints.uu <- merge( x=constraints.uu, y=M1, by=1, all.x=TRUE )
            constraints.uu <- constraints.uu[, c(3,2) ]
                        }

        mod.uu <- rasch.jml( dat=dat.uu, b.init=(mod$item$itemdiff)[-uu],
                            prox=FALSE, progress=FALSE,
#                            dp=mod$dp,
                            constraints=constraints.uu,
                            method=mod$method, calc.fit=FALSE )
        j.itemdiff[-uu,uu] <- mod.uu$item$itemdiff
        if ( uu %in% prbar.display ){
                    suu <- sum( prbar.display==uu )
                    cat(paste( rep("-",suu), collapse="")) ; utils::flush.console()
                                }
                    }
    cat("|\n\n")
    # END JACKKNIFE
    #*************************
    dfr <- data.frame( mod$item[,1:2]   )
    # item difficulty
    dfr$b.JML <- mod$item$itemdiff
    dfr$b.JMLcorr <- mod$item$itemdiff.correction
    mean.jack <- rowMeans( j.itemdiff, na.rm=T)
    # calculate correction factor
    UJJ <- UJ
    if ( ! is.null(constraints) ){
            UJJ <- UJJ - nrow(constraints)
                }
    dfr$b.jack <- UJJ * mod$item$itemdiff - (UJJ-1)*mean.jack
    dfr$est.bias <- - dfr$b.jack + dfr$b.JML
    dfr$b.JMLse <- mod$item$se
    print( round(dfr,3)  )
    return( list( "item"=dfr, "jack.itemdiff"=j.itemdiff ))
    }
##########################################################################################

## File Name: rm_sdt_mstep_type_function_gradient.R
## File Version: 0.11



rm_sdt_mstep_type_function_gradient <- function(x, par_index, partable, type,
    pargroup_type, probs_args, probs_fun, probs_dim, update_probs_args, nik,
    numdiff.parm)
{
    res <- rm_sdt_fill_par_to_partable( par_index=par_index, partable=partable,
                                parm0=x, type=type )
    partable1 <- res$partable
    probs_nr <- as.list(1:2)
    grad_out <- rep(NA, pargroup_type$np)
    numdiff_fac <- c(1,-1)
    K <- probs_dim[2] - 1
    for (pp in seq_len(pargroup_type$max_pargroup) ){
        type_pp <- paste( pargroup_type$pargroup_type[[pp]] )
        #- x0 + h and x0 - h
        for ( vv in 1:2){
            probs_args1 <- probs_args
            parm1 <- rm_sdt_extract_par_from_partable_add_increment(partable=partable1,
                            pargroup=pp, increment=numdiff.parm*numdiff_fac[vv] )
            res <- rm_sdt_fill_par_to_partable( par_index=par_index, partable=partable,
                                        parm0=parm1, type=type )
            probs_args1 <- rm_sdt_mstep_include_probs_args(probs_args=probs_args1,
                    parm_list=res$parm_list, update_probs_args=update_probs_args )
            probs_nr[[vv]] <- do.call( what=probs_fun, args=probs_args1)
        }
        logprobs <- ( probs_nr[[1]] - probs_nr[[2]] ) / ( 2 * numdiff.parm)
        ll_grad <- rm_sdt_calc_gradient_likelihood_item_llgrad2( logprob_D1=logprobs,
                            nik.item=nik, diffindex=pargroup_type$pargroup_diffindex[[pp]], K=K,
                            prob_D1_dim=probs_dim)
        grad_out[ pargroup_type$pargroup_index[[pp]] ] <- ll_grad
    }
    grad_ll <- - grad_out
    grad_prior <- rm_sdt_evaluate_prior_derivative(partable=partable1, h=numdiff.parm)
    grad_post <- grad_ll + grad_prior
    grad_post <- ifelse(is.na(grad_post), 0, grad_post )
    return(grad_post)
}

## File Name: mgsem_cda_opt_evaluate_penalties.R
## File Version: 0.02

mgsem_cda_opt_evaluate_penalties <- function(x, opt_fun_args)
{
    res <- mgsem_evaluate_penalties(x=x,
                    partable=opt_fun_args$partable,
                    prior_list=opt_fun_args$prior_list,
                    technical=opt_fun_args$technical,
                    h=opt_fun_args$technical$h,
                    p=opt_fun_args$p_pen,
                    eps_approx=opt_fun_args$eps_approx,
                    deriv=FALSE, difflp_info=opt_fun_args$difflp_info,
                    loop_parms=opt_fun_args$loop_parms,
                    pen_type=opt_fun_args$pen_type, a_scad=opt_fun_args$a_scad)
    return(res)
}

## File Name: mcmc_3pno_testlet_draw_itempars.R
## File Version: 0.137


#---- draw item parameters a and b
mcmc_3pno_testlet_draw_itempars <- function( theta, Z, I, N, weights,
    gamma.testlet, testletgroups, param, TT, a.testletM)
{
    # define adjusted Z values
    gamma.testletM <- gamma.testlet[, testletgroups ]
    if (param==1){
        Z <- Z - gamma.testletM
    }
    if (param==3){
        Z <- Z - a.testletM*gamma.testletM
    }
    if (param==2){ # Z <- Z
        theta0 <- theta
        Z0 <- Z
    }
    # for parametrization 2, this function must be rewritten
    # because 'the theta' is now item specific
    # loop over testlets tt=1,...,TT
    # maybe for TT+1 some adjustment has to be done
    #--- parametrization param=1
    if ( (param==1) | (param==3) ){
        #--------------
        # sampling without weights
        Xast <- as.matrix( cbind( theta, 1 ) )
        if ( is.null(weights) ){
            Sigma <- solve( crossprod(Xast) )
            # calculate mean
            mj <- Sigma %*% crossprod( Xast, Z )
            mj <- as.matrix( t(mj))
        }
        #--------------
        # sampling with weights
        if ( ! is.null( weights ) ){
            # compute elements of Xast
            Xast11 <- sum( theta^2 * weights )
            Xast12 <- - sum( theta * weights )
            Xast22 <- sum( weights )
            # compute inverse of Xast
            Xastdet <- Xast11*Xast22 - Xast12^2
            Xastinv11 <- Xast22 / Xastdet
            Xastinv22 <- Xast11 / Xastdet
            Xastinv12 <- - Xast12 / Xastdet
            Sigma <- matrix( c(Xastinv11, Xastinv12, Xastinv12, Xastinv22), 2,2 )
            mj <- Sigma %*% crossprod( Xast * weights, Z )
            mj <- as.matrix( t(mj))
        }
        #--------------
        # draw item parameters
        ipars <- sirt_rmvnorm( I, sigma=Sigma ) + mj
        a <- ipars[,1]
        b <- ipars[,2]
    }
    #--- parametrization param=2
    if (param==2){
        a <- rep(NA,I)
        b <- rep(NA,I)
        TTT <- TT
        if ( sum( testletgroups==TT+1 ) > 0 ){
            TTT <- TT + 1
        }
        for (tt in 1L:TTT){

            theta <- theta0
            Z <- Z0
            ind.tt <- which( testletgroups==tt)
            Itt <- length(ind.tt)
            theta <- theta0 + gamma.testlet[, tt]
            Z <- Z[, ind.tt, drop=FALSE]
            #--------------
            # sampling without weights
            Xast <- as.matrix( cbind( theta, 1 ) )
            if ( is.null(weights) ){
                Sigma <- solve( crossprod(Xast) )
                # calculate mean
                mj <- Sigma %*% crossprod(Xast, Z )
                mj <- as.matrix( t(mj))
                                }
            #--------------
            # sampling with weights
            if ( ! is.null( weights ) ){
                # compute elements of Xast
                Xast11 <- sum( theta^2 * weights )
                Xast12 <- - sum( theta * weights )
                Xast22 <- sum( weights )
                # compute inverse of Xast
                Xastdet <- Xast11*Xast22 - Xast12^2
                Xastinv11 <- Xast22 / Xastdet
                Xastinv22 <- Xast11 / Xastdet
                Xastinv12 <- - Xast12 / Xastdet
                Sigma <- matrix( c(Xastinv11, Xastinv12, Xastinv12, Xastinv22), 2,2 )
                # compute t(Xast) %*% Z (weighted)
                mj <- Sigma %*% crossprod( Xast * weights, Z )
                mj <- as.matrix( t(mj))
            }
            #--------------
            # draw item parameters
            ipars <- sirt_rmvnorm( Itt, sigma=Sigma ) + mj
            a[ind.tt] <- ipars[,1]
            b[ind.tt] <- ipars[,2]
        }  # end testlet tt
    }  # end param=2
    #-- output
    res <- list( a=a, b=b)
    return(res)
}


.draw.itempars.3pno.testlet <- mcmc_3pno_testlet_draw_itempars

## File Name: summary.rasch.mml2.R
## File Version: 1.252


#-- Summary for rasch.mml object
summary.rasch.mml <- function( object, file=NULL, ... )
{
    # open sink
    sirt_osink( file=file )

    npirt <- object$irtmodel=="npirt"
    D <- object$D

    a5 <- 1*( npirt & ncol( object$item)==5 )
    disp <- "------------------------------------------------------------\n"

    cat(disp)
    d1 <- utils::packageDescription("sirt")
    cat( paste( d1$Package, " ", d1$Version, " (", d1$Date, ")", sep=""), "\n\n" )
    cat( "Date of Analysis:", paste( object$s2 ), "\n" )
    cat("Computation time:", print(object$s2 - object$s1), "\n\n")

    cat("Call:\n", paste(deparse(object$CALL), sep="\n", collapse="\n"),
                "\n\n", sep="")

    cat("Semiparametric Marginal Maximum Likelihood Estimation \n")
    if ( object$Rfcttype=="rasch.mml" ){ cat("Function 'rasch.mml' \n\n") }
    if ( object$Rfcttype=="rasch.mml2" ){ cat("Function 'rasch.mml2' \n\n") }
    cat("irtmodel=", object$irtmodel, "\n")
    if ( ! object$ramsay.qm ){
        cat("Rasch Type Model with Fixed Discrimination, Guessing and Slipping Parameters \n")
        cat("alpha1", "=", round(object$alpha1,3)," alpha2", "=", round(object$alpha2,3), " \n")
        moments <- genlogis.moments( alpha1=object$alpha1, alpha2=object$alpha2)
        cat("Moments: \n" )
        print(round(moments,2))
        cat("\n")
    }
    if ( object$ramsay.qm ){ cat("Quotient Model (Ramsay, 1989) \n")     }
    if ( object$irtmodel=="npirt" ){  cat("Nonparametric IRT \n")     }
    if ( object$irtmodel=="missing1"){ cat("Missing Data IRT Model \n")        }

    #- prior distributions
    if (object$irtmodel=="raschtype"){
        est_parameters <- object$est_parameters
        cat("Prior distributions\n")
        cat(paste0(" b parameters: ", object$priors$b, "\n"))
        for (pp in c("a", "c", "d")){
            if (est_parameters[[pp]]){
                cat(paste0(" ", pp, " parameters: ", object$priors[[pp]], "\n"))
            }
        }
        cat("\n")
    }

    if ( object$G > 1 ){ cat("\nMultiple Group Estmation with",object$G, "Groups \n")
        print(object$groupindex) ; cat("\n")
    }
    cat(disp)
    cat( "Number of iterations", "=", object$iter, "\n" )
    cat( "Deviance", "=", round( object$deviance, 2 ), " | " )
    cat( "Log Likelihood", "=", round( -object$deviance/2, 2 ), "\n" )
    cat( "Number of persons", "=", object$ic$n, "\n" )
    cat( "Number of estimated parameters", "=", object$ic$np, "\n" )
    cat( "AIC", "=", round( object$ic$AIC, 2 ), " | penalty=",
                round( object$ic$AIC - object$ic$deviance,2 ),
            "   | AIC=-2*LL + 2*p  \n" )
    cat( "AICc", "=", round( object$ic$AICc, 2 )," | penalty=",
                round( object$ic$AICc - object$ic$deviance,2 ) )
        cat("    | AICc=-2*LL + 2*p + 2*p*(p+1)/(n-p-1)  (bias corrected AIC)\n" )
    cat( "BIC", "=", round( object$ic$BIC, 2 ), " | penalty=",
                    round( object$ic$BIC - object$ic$deviance,2 ),
            "   | BIC=-2*LL + log(n)*p  \n" )
    cat( "CAIC", "=", round( object$ic$CAIC, 2 )," | penalty=",
                round( object$ic$CAIC - object$ic$deviance,2 ) )
        cat("   | CAIC=-2*LL + [log(n)+1]*p  (consistent AIC)\n\n" )

    #------------------------------------
    if ( object$D==1){
        if ( is.null( object$trait.weights) ){
            cat( "Trait Distribution (", length(object$trait.distr[,1]), " Knots )\n",
                "Mean", "=", round( object$mean.trait,3),
                "\n SD", "=", round( object$sd.trait, 3),
                "\n Skewness", "=", round( object$skewness.trait, 3))
        }
        if ( ! is.null( object$trait.weights) ){
            M1 <- stats::weighted.mean( object$theta.k,object$trait.weights )
            S1 <- sqrt( stats::weighted.mean( object$theta.k^2,object$trait.weights ) - M1^2    )
            cat( "Fixed Trait Distribution (", length(object$trait.distr[,1]), " Knots )\n",
                    "Mean", "=", round( M1,3 ),
                    " SD", "=", round( S1, 3) )
        }
    }
    if ( object$ramsay.qm ){
        cat( "      Note: log theta distribution is parametrized!")
    }
    cat("\n")
    if ( D > 1){
        cat("Mean Vector\n") ; print( round( object$mu, 3 ) )
        cat("\nCovariance Matrix\n") ; print( round( object$Sigma.cov, 3 ) )
        cat("\n")
        covmat <- object$Sigma.cov
        covmat2 <- stats::cov2cor( covmat )
        diag(covmat2) <- sqrt( diag(covmat) )
        cat("\nStandard Deviations / Correlation Matrix\n") ; print( round( covmat2, 3 ) )
        cat("\n")
    }

    if ( object$irtmodel !="npirt" ){
        cat( "Item Difficulty Distribution (", nrow(object$item), " Items )\n",
                    "Mean", "=", round( mean(object$item$b),3),
                    " SD", "=", round( stats::sd(object$item$b), 3), "\n")
    }
    cat( "Distribution of Items Administered (", nrow(object$item), " Items )\n",
                "Mean", "=", round( mean(rowSums( 1 - is.na(object$dat) )),3),
                " SD", "=", round( stats::sd(rowSums( 1 - is.na(object$dat) )),3), "\n\n")
    cat( "EAP Reliability: ")
    cat(round( object$reliability$eap.reliability,3 ) )
    cat("\n")
    if ( a5==0 ){
        cat(disp)
        cat("Item Parameter \n")
        if ( ! object$ramsay.qm ){
            obji <- object$item
        } else {
            obji <- object$item2
        }
        rvars <- seq( 2, ncol(obji) )
        ind <- which( colMeans( is.na( obji ))==1 )
        roundvars <- setdiff( rvars, ind )
        if (npirt){
            roundvars <- c("p","est","se" )
        }
        for (vv in roundvars){
            obji[,vv] <- round( obji[,vv], 3 )
        }
        rownames(obji) <- NULL
        print( obji )
    }

    # close sink
    sirt_csink( file=file )
}

## File Name: prior_model_pars_CleanString.R
## File Version: 0.09


#-- clean string
prior_model_pars_CleanString <- function( ps ){
    ps <- gsub( " ", "", ps )
    ps <- ps[ ps !="" ]
    NP <- length(ps)
    for (pp in 1:NP){
        ps_pp <- ps[pp]
        # locate comment symbol
        h1 <- gregexpr(pattern='#', text=ps_pp)
        if ( h1 > 0){
            ps[pp] <- substring( ps_pp, 1, h1[[1]] -1)
        }
    }
    ps <- ps[ ps !="" ]
    return(ps)
}

## File Name: package_version_date.R
## File Version: 0.091

package_version_date <- function(package)
{
    d1 <- utils::packageDescription(pkg=package)
    res <- paste( d1$Package, ' ', d1$Version,
                ' (', d1$Date, ')', sep='')
    return(res)
}

## File Name: R2noharm.R
## File Version: 2.24

#------------------------------------------------------------------
# NOHARM exploratory factor analysis
R2noharm <- function( dat=NULL, pm=NULL, n=NULL,model.type, weights=NULL,
            dimensions=NULL, guesses=NULL, noharm.path,
                            F.pattern=NULL, F.init=NULL, P.pattern=NULL, P.init=NULL,
                            digits.pm=4, writename=NULL,
                            display.fit=5, dec=".", display=TRUE  ){

    # INPUT:
    # dat ... dataframe
    # model.type ... CFA or EFA
    # dimensions ... dimensions for exploratory factor analysis
    # guesses ... fixed guessing paramters
    # noharm.path ... path for NOHARM console (console must have the name NOHARM87.exe)
    # digits.pm ... digits for calculation of product moment matrix
    # writename ... name of NOHARM Output file
    # dec ... "." or "," for decimal separator
    # display ... display noharm settings (TRUE or FALSE)
    #............................................................................
    # The matrices F.pattern, F.init, P.pattern and P.init correspond to the
    # definition in the NOHARM manual
#    noharm.path <- shQuote(noharm.path)
    if (model.type=="CFA" ){ dimensions <- ncol(F.pattern) }
    if ( display ){
        cat("Multidimensional Normal Ogive by Harmonic Analysis (NOHARM 4) \n" )
        cat("C. Fraser & R. P. McDonald (2012)\n" )
        cat("For more informations please look at \n ")
#        cat( "    http://people.niagaracollege.ca/cfraser/download/nhCLwindl.html\n\n")
        cat( "  http://noharm.niagararesearch.ca/nh4cldl.html\n\n")
        cat( paste( "Path of NOHARMCL file: \n  ** ", noharm.path, "\n" ) )
        cat( paste( "Path of NOHARM input and output files: \n  ** ", getwd(), "\n\n" ) )
        if (model.type=="EFA"){
                cat(paste( "Exploratory Item Factor Analysis with", dimensions, "Dimensions" ), "\n\n" )
                } else {             cat("Confirmatory Item Factor Analysis \n\n" ) }
        flush.console()
                }
    ########################################################
    # data input
    if ( ! is.null(dat) ){
        # allow also for input of moment matrix data
        dat <- as.matrix(dat)
        I <- ncol(dat)     # number of items
        n <- nrow(dat)     # number of subjects
        if ( is.null(weights) ){
            weights <- rep(1,n) } else {
            weights <- weights / sum(weights) * n
                }

        # calculate raw product moment matrix
        dat9 <- dat
        dat.resp <- is.na( dat)
        dat9[ dat.resp ] <- 9
        # pairwise product moment matrix
    #    BM <- t( dat9 * (1- dat.resp ) ) %*% ( dat9 * (1- dat.resp  ) )
         BM <- crossprod( dat9 * (1- dat.resp  )*weights  )
        # number of persons on an item pair
    #    NM <- t((1- dat.resp ) ) %*% (  (1- dat.resp  ) )
        NM <- crossprod(  (1- dat.resp)*weights  )
        BM <- round( BM/NM, digits.pm )
                    }
    inputdat <- TRUE
    ########################################################
    if ( ! is.null(pm) ){
      if ( is.vector(pm) ){
        I2 <- length(pm)
        I <- sqrt( 2*I2+1/4 ) - .5
        BM <- matrix( 0, I, I )
        colnames(BM) <- paste0("I",1:I)
        vv <- 0
        for (ii in 1:I){
#        ii <- 1
            BM[ ii, 1:ii ] <- pm[ seq(vv+1,vv+ii) ]
            vv <- vv + ii
                    }
        BM <-  BM + t(BM)
        diag(BM) <- diag(BM)/2
             } else {
                pm <- BM
                I <- ncol(pm)
                    }
        weights <- rep(1,n)
        NM <- matrix(n, I,I)
        dat <- BM[1:2,,drop=FALSE]
        inputdat <- FALSE
                        }
    # Exploratory analysis requested?
    EX <- 1 * ( model.type=="EFA" )
    # supply starting values
    IV <- 1 * (model.type=="CFA" )

    if ( is.null(guesses) ){
        guesses <- rep(0, I )
                    }

    # arrange NOHARM Input Files
    s1 <- Sys.time()
    noharm.input <- c( paste( "R2noharm Input file", s1 ),
                        paste( I, dimensions, n, 1,  EX, IV, 0, 0,  sep=" ")
                                        )
    # add guessing parameter
    noharm.input <- c( noharm.input, "   ", guesses, "  " )

    if (model.type=="CFA"){
        # pattern matrices
        noharm.input <- c( noharm.input,
            apply( F.pattern, 1, FUN=function(ll){ paste( ll, collapse=" " ) } ), " ",
            sapply( 1:dimensions, FUN=function(ss){ paste( P.pattern[ ss, 1:ss ], collapse=" " ) }  ),
            "   "     )
        # add initial matrices
        if ( is.null(F.init) ){ F.init <- .5*F.pattern }
        if ( is.null(P.init) ){ P.init <- 0.1  + diag( .9, dimensions) }
        noharm.input <- c( noharm.input,
                apply( F.init, 1, FUN=function(ll){ paste( ll, collapse=" " ) } ), " ",
                sapply( 1:dimensions, FUN=function(ss){ paste( P.init[ ss, 1:ss ], collapse=" " ) }  )
                        )
         }

        # product moment matrix
        pm <- unlist( sapply( 1:I, FUN=function( ii){  BM[ ii, 1:ii ]  } ) )
        LPM <- length(pm)
        h1 <- seq( 1, LPM, 10 )
        h2 <- c( seq( 10, LPM, 10 ) )
        if (length(h1) !=length(h2) ){ h2 <- c( h2, LPM ) }
        h3 <- data.frame( h1, h2 )
        pm <- apply( h3, 1, FUN=function(ll){ paste( pm[ ll[1]:ll[2] ], collapse=" " ) } )


    # add product moment matrix to NOHARM input
    noharm.input <- c( noharm.input, " ", pm   )
    if (dec=="," ){ noharm.input <- gsub( "\\.", ",", noharm.input ) }
    # path specifications and starting NOHARM
    current.path <- getwd()
    setwd( noharm.path )
    writeLines( noharm.input, "mymodel.inp" )
#    writeLines( "NoharmCL mymodel.inp mymodel.out 800 0.00001", "noharm_analysis.bat" )
    writeLines( "NoharmCL mymodel.inp mymodel.out", "noharm_analysis.bat" )
    # working without DOSBox
    system( "noharm_analysis.bat", show.output.on.console=F  )
    # read NOHARM output
    noharmout0 <- readLines( "MYMODEL.out" )
    if (dec=="," ){ noharmout0 <- gsub( ",", "\\.", noharmout0 ) }
    noharmout1 <- noharmout0
    noharmout1 <- c( noharmout1, rep("",3), "ENDE" )
    # set current working directory
    setwd( current.path )
    if (!is.null( writename ) ){
                writeLines( noharmout0, paste( writename, ".out", sep="") )
                writeLines( noharm.input, paste( writename, ".inp", sep="") )
                }
    # results for 1-dimensional IRT analysis
    if (dimensions==1 & model.type=="EFA" ){
            modtype <- 1
        res <- list( "tanaka"=.noharm.tanaka( noharmout1 ),
            "rmsr"=.noharm.rmsr( noharmout1 ),
            "N.itempair"=NM,
            "pm"=BM,
            "weights"=weights,
            "guesses"=guesses,
            "residuals"=.noharm.residuals( noharmout1, I=I, dat=dat),
            "final.constants"=.noharm.itemlevel( noharmout1, "Final Constants", I=I, dat=dat),
            "thresholds"=.noharm.itemlevel( noharmout1, "Threshold Values", I=I, dat=dat),
            "uniquenesses"=.noharm.itemlevel( noharmout1, "Unique Variances", I=I, dat=dat),
            "difficulties"=.noharm.itemlevel( noharmout1, "Vector B", I=I, dat=dat),
            "discriminations"=.noharm.itemlevel( noharmout1, "Vector A", I=I, dat=dat)
                )  }
    # results for noharm.efa with more than one dimension
    if (dimensions > 1 & model.type=="EFA" ){
            modtype <- 2
        res <- list( "tanaka"=.noharm.tanaka( noharmout1 ),
                "rmsr"=.noharm.rmsr( noharmout1 ),
                "N.itempair"=NM,
                "pm"=BM,
                "guesses"=guesses,
                "residuals"=.noharm.residuals( noharmout1, I=I, dat=dat),
                "final.constants"=.noharm.itemlevel( noharmout1, "Final Constants",I=I, dat=dat),
                "factor.cor"=.noharm.correlations( noharmout1, "Factor Correlations",
                                "Promax Rotated", dimensions=dimensions, dat=dat),
                "thresholds"=.noharm.itemlevel( noharmout1, "Threshold Values", I=I, dat=dat),
                "uniquenesses"=.noharm.itemlevel( noharmout1, "Unique Variances", I=I, dat=dat),
                "unrotated"=.noharm.loadings( noharmout1, "Factor Loadings",
                                "Varimax Rotated Factor Loadings",  dimensions=dimensions, I=I, dat=dat),
                "varimax"=.noharm.loadings( noharmout1, "Varimax Rotated Factor Loadings",
                                "Varimax Rotated Coefficients of Theta",   dimensions=dimensions, I=I, dat=dat),
                "varimax.theta"=.noharm.loadings( noharmout1, "Varimax Rotated Coefficients of Theta",
                                    "oblique",   dimensions=dimensions, I=I, dat=dat),
                "promax"=.noharm.loadings( noharmout1, "oblique",
                                "Factor Correlations",   dimensions=dimensions, I=I, dat=dat),
                "promax.theta"=.noharm.loadings( noharmout1, "Promax Rotated Coefficients of Theta",
                                        "ENDE",   dimensions=dimensions, I=I, dat=dat)
                )
            }

    # results for noharm.cfa with more than one dimension
    if (dimensions > 1 & model.type=="CFA" ){
            modtype <- 3
        res <- list( "tanaka"=.noharm.tanaka( noharmout1 ),
                "rmsr"=.noharm.rmsr( noharmout1 ),
                "N.itempair"=NM,
                "pm"=BM,
                "guesses"=guesses,
                "residuals"=.noharm.residuals( noharmout1, I=I, dat=dat),
                "final.constants"=.noharm.itemlevel( noharmout1, "Final Constants",I=I, dat=dat),
                "factor.cor"=.noharm.correlations( noharmout1, "Final Correlations", "Residual", dimensions=dimensions, dat=dat),
                "thresholds"=.noharm.itemlevel( noharmout1, "Threshold Values", I=I, dat=dat),
                "uniquenesses"=.noharm.itemlevel( noharmout1, "Unique Variances", I=I, dat=dat),
                "loadings"=.noharm.loadings( noharmout1, "Factor Loadings",
                                "ENDE",  dimensions=dimensions, I=I, dat=dat),
                "loadings.theta"=.noharm.loadings( noharmout1, "Final Coefficients of Theta",
                                "Final Correlations of Theta",   dimensions=dimensions, I=I, dat=dat)
                    )


            if( !is.null( colnames(F.pattern) ) ){
                    colnames(res$loadings) <- colnames(F.pattern)
                    colnames(res$loadings.theta) <- colnames(F.pattern)
                    colnames(res$factor.cor) <- rownames(res$factor.cor) <- colnames(F.pattern)
                                }
                 }
    # CFA 1 dimension
    if ( ( dimensions==1 ) & ( model.type=="CFA" ) ){
                    modtype <- 4
            if ( is.null(colnames(F.pattern) ) ){ colnames(F.pattern) <- "F1" }
        res <- list( "tanaka"=.noharm.tanaka( noharmout1 ),
                "rmsr"=.noharm.rmsr( noharmout1 ),
                "N.itempair"=NM,
                "pm"=BM,
                "guesses"=guesses,
                "residuals"=.noharm.residuals( noharmout1, I=I, dat=dat),
                "final.constants"=.noharm.itemlevel( noharmout1, "Final Constants",I=I, dat=dat),
                "thresholds"=.noharm.itemlevel( noharmout1, "Threshold Values", I=I, dat=dat),
                "uniquenesses"=.noharm.itemlevel( noharmout1, "Unique Variances", I=I, dat=dat),
                "loadings.theta"=.noharm.loadings( noharmout1, "Final Coefficients of Theta",
                                "Residual",   dimensions=dimensions, I=I, dat=dat),
                "factor.cor"=as.data.frame(matrix(1,1, 1 )),
                "difficulties"=.noharm.itemlevel( noharmout1, "Vector B", I=I, dat=dat),
                "discriminations"=.noharm.itemlevel( noharmout1, "Vector A", I=I, dat=dat),
                "loadings"=.noharm.loadings( noharmout1, "Factor Loadings",
                                "LORD",   dimensions=dimensions, I=I, dat=dat)
                #                ,
#                "loadings"=.noharm.loadings( noharmout1, "Factor Loadings",
#                                "LORD",  dimensions=dimensions, I=I, dat=dat)
                )
            if( !is.null( colnames(F.pattern) ) ){
                    colnames(res$loadings) <- colnames(F.pattern)
                    rownames(res$factor.cor) <- colnames(res$factor.cor) <- colnames(F.pattern)
                    colnames(res$loadings.theta) <- colnames(F.pattern)
                              } }
    # collect arguments in output
    res$model.type <- model.type
#    res$Nobs <- nrow(dat)
#    res$Nitems <- ncol(dat)
    res$Nobs <- n
    res$Nitems <- I
    res$modtype <- modtype
    res$F.init <- F.init
    res$F.pattern <- F.pattern
    res$P.init <- P.init
    res$P.pattern <- P.pattern
    res$dat <- dat
    res$systime <- s1
    res$guesses <- guesses
    res$noharm.path <- noharm.path
    res$digits.pm <- digits.pm
    res$dec <- dec
    res$display.fit <- display.fit
    if ( modtype %in% c(1,4)){ res$dimensions <- 1 }
    if ( modtype %in% c(2,3)){ res$dimensions <- ncol(res$factor.cor) }
    #***
    # calculate fit statistic
    if ( modtype %in% 2:4){
        RM <- res$residuals
        PV <- diag( res$pm )
        g1 <- sqrt( outer( PV * (1-PV), PV * ( 1-PV ) ) )
        rM <- ( RM / g1  )
        zM <- 0.5 * log( 1 + rM ) - 0.5 * log( 1 - rM )
        # chi square
        res$chisquare <- X2 <- ( res$Nobs - 3 ) * sum( zM^2 )
        # calculate number of estimated parameters and degrees of freedom
        I <- res$Nitems
        if (modtype %in% 3:4){
            Nestpars <- I + sum( F.pattern==1 ) + length( unique( intersect( F.pattern, 2:9999 ) ) )
            Nestpars <- Nestpars + sum( diag(P.pattern)==1 ) +
                            length( unique( diag(P.pattern)[ diag(P.pattern) > 1] ) )
            g2 <- P.pattern[ lower.tri(P.pattern) ]
            Nestpars <- Nestpars + sum( g2==1 ) + length( unique( intersect( g2, 2:9999 ) ))
            res$Nestpars <- Nestpars
                            }
        if ( modtype %in% 2){
            cs <- 1:(dimensions-1)
            res$Nestpars <- Nestpars <- I + I*dimensions - sum(cs)
                    }
        res$df <- df <- 0.5*I*(I+1) - Nestpars
        res$chisquare_df  <- res$chisquare / res$df
        # calculate RMSEA
        res$rmsea <- rmsea <- sqrt(max(c( (X2 / res$Nobs ) / df - 1/res$Nobs, 0)))
        # calculate p values
        res$p.chisquare <- 1 - pchisq( res$chisquare, df=res$df )
            }
    # display
    if (display){
        cat( paste( "Tanaka Index=", round(res$tanaka,display.fit), sep=""), "\n" )
        cat( paste( "RMSR=", round(res$rmsr,display.fit), sep=""), "\n" )
        if ( ! inputdat ){
           cat("\n**** Note that Jackknife does not work for pm input! **** \n")
                }
            }
    if ( ! inputdat ){
           res$dat <- NULL
            }
    res$inputdat <- inputdat
    res$upper <- 1+0*res$guess
    res$lower <- res$guess
    class(res) <- "R2noharm"
    return( res )
    }
#----------------------------------------------------------

## File Name: xxirt_nr_grad_fun_numapprox.R
## File Version: 0.092

xxirt_nr_grad_fun_numapprox <- function(x, em_args, opt_fun)
{
    grad <- 0*x
    ll1 <- opt_fun(x=x, em_args=em_args)
    h <- em_args$h
    for (pp in 1L:em_args$NP){
        x1 <- sirt_add_increment(x=x, pos=pp, value=h)
        ll2 <- opt_fun(x=x1, em_args=em_args)
        x1 <- sirt_add_increment(x=x, pos=pp, value=-h)
        ll2b <- opt_fun(x=x1, em_args=em_args)
        grad[pp] <- (ll2-ll2b) / (2*h)
    }
    return(grad)
}

## File Name: lsem_weighted_cov.R
## File Version: 0.252

lsem_weighted_cov <- function( x, weights, x_resp=NULL,
        moderator_variable=NULL, loc_linear_smooth=NULL, moderator_value=NULL,
        pd=FALSE, residualized_intercepts=NULL,    has_meanstructure=FALSE,
        residualize=TRUE)
{
    if (pd){
        requireNamespace('Matrix')
    }
    if (is.null(loc_linear_smooth)){
        loc_linear_smooth <- FALSE
    }
    x <- as.matrix(x)
    if ( is.null(x_resp)){
        x_resp <- 1 - is.na(x)
    }
    eps0 <- 1e-200
    eps <- eps0 * max(weights)
    weights_m <- sqrt( weights + eps ) * x_resp
    x[ ! x_resp ] <- 0

    #*** estimate mean for covariance structure
    x0 <- x
    res <- lsem_weighted_mean( x=x0, weights=weights, x_resp=x_resp,
                        moderator_variable=moderator_variable,
                        loc_linear_smooth=loc_linear_smooth,
                        moderator_value=moderator_value)
    x_center <- res$mean
    XC <- matrix( x_center, nrow=nrow(x), ncol=ncol(x), byrow=TRUE )
    x <- x - XC
    weightsN <- crossprod(weights_m)
    xw <- as.matrix( x * weights_m)
    covw <- crossprod(xw) / weightsN
    covw_raw <- covw
    covw2 <- NA*covw

    if (loc_linear_smooth){
        V <- ncol(x)
        for (vv in 1L:V){
            for (ww in vv:V){
                weights1 <- weights
                mod_vv <- stats::lm(x[,vv]~moderator_variable, weights=weights1)
                if (ww==vv){
                    mod_ww <- mod_vv
                } else {
                    mod_ww <- stats::lm(x[,ww]~moderator_variable, weights=weights1)
                }
                rvv <- resid(mod_vv)
                rww <- resid(mod_ww)
                mod <- stats::lm(rvv*rww ~ moderator_variable, weights=weights1)
                cmod <- mod$coefficients
                temp1 <- cmod[1]+cmod[2]*moderator_value
                covw2[vv,ww] <- covw2[ww,vv] <- temp1
            }
        }
        covw <- covw2
    }  # end fit loc_linear_smooth

    if (pd){
        covw <- as.matrix(Matrix::nearPD(x=covw)$mat)
    }

    Nobs <- mean( weightsN[ ! upper.tri(weightsN) ] )


    #-- output
    res <- list( weightsN=weightsN, cov=covw, raw_cov=covw_raw,
                    Nobs=Nobs, mean=x_center)
    return(res)
}

## File Name: prob_genlogis_4pl.R
## File Version: 0.08


prob_genlogis_4pl <- function(theta, b, a, c, d, alpha1, alpha2, Qmatrix)
{
    pjk <- prob_raschtype_genlogis( theta=theta, b=b, alpha1=alpha1,
                alpha2=alpha2, fixed.a=a, Qmatrix=Qmatrix)
    if ( (any(c>0)) | (any(d<1)) ){
        np <- nrow(pjk)
        cM <- sirt_matrix2( x=c, nrow=np )
        dM <- sirt_matrix2( x=d, nrow=np )
        pjk <- cM + (dM - cM) * pjk
    }
    #--- output
    return(pjk)
}

## File Name: brm.sim.R
## File Version: 0.124


#-- brm.sim
brm.sim <- function( theta, delta, tau, K=NULL)
{
    I <- length(delta)
    N <- length(theta)
    dat <- matrix( 0, nrow=N, ncol=I )
    colnames(dat) <- paste0( 'I', 1L:9 )
    if ( ! is.null(K) ){
        br <- seq( 0, 1, len=K+1 )
    }
    for (ii in 1L:I){
        m1 <- exp( ( theta - delta[ii] + tau[ii] ) / 2 )
        n1 <- exp( ( - theta + delta[ii] + tau[ii] ) / 2 )
        dat[,ii] <- stats::rbeta( N, shape1=m1, shape2=n1 )
        if ( ! is.null(K) ){
            dat[,ii] <- as.numeric(cut( dat[,ii], breaks=br )) - 1
        }
    }
    return(dat)
}

## File Name: sirt_pem_extract_parameters.R
## File Version: 0.03

sirt_pem_extract_parameters <- function( parm, parmgroup, pem_parameter_index )
{
    info <- pem_parameter_index[[ parmgroup ]]
    x_dim <- info$dim
    x <- parm[ info$index ]
    x <- sirt_pem_adjust_dimension(x=x, x_dim=x_dim )
    return(x)
}

## File Name: move_variables_df.R
## File Version: 0.02

##-- move variables in a data frame
move_variables_df <- function(x, after_var, move_vars)
{
    cnx <- colnames(x)
    i1 <- which(cnx==after_var)
    vars1 <- cnx[ seq(1,i1) ]
    vars2 <- move_vars
    vars3 <- setdiff( cnx, c(vars1, vars2) )
    x <- x[, c(vars1, vars2, vars3) ]
    return(x)
}

## File Name: mgsem_proc_data.R
## File Version: 0.057

mgsem_proc_data <- function(data, group, weights)
{
    N <- nrow(data)
    if (is.null(group)){
        group <- rep(1,N)
    }
    groups <- unique(group)
    G <- length(groups)
    if (is.null(weights)){
        weights <- rep(1,N)
    }
    idgroup <- match(group, groups)
    suffstat <- list()
    for (gg in 1L:G){
        ind_gg <- which(group==groups[gg])
        dat_gg <- data[ ind_gg, ]
        w <- weights[ind_gg]
        res <- stats::cov.wt( x=dat_gg, w=w, method='ML')
        s_gg <- list(M=res$center, S=res$cov, N=sum(w) )
        suffstat[[gg]] <- s_gg
    }
    #-- output
    res <- list(suffstat=suffstat, weights=weights, groups=groups,
                    G=G, N=N, idgroup=idgroup, data=data)
    return(res)
}

## File Name: lsem_fitsem_sufficient_statistics_lavaan.R
## File Version: 0.06

lsem_fitsem_sufficient_statistics_lavaan <- function(gg, lavmodel, lavaan_est_fun,
    survey.fit, sample_stats, is_meanstructure, se, ...)
{
    wmean <- sample_stats$wmean
    wcov <- sample_stats$wcov
    Nobs <- sample_stats$Nobs
    if (gg==1){
        input_model <- lavmodel
    } else {
        input_model <- sirt_import_lavaan_parameterTable(survey.fit)
    }
    if (is_meanstructure){
        wmean_gg <- wmean[[gg]]
    } else {
        wmean_gg <- NULL
    }
    survey.fit <- lavaan_est_fun(input_model, sample.cov=wcov[[gg]],
                                sample.mean=wmean_gg, sample.nobs=Nobs[[gg]],
                                se=se, ... )
    return(survey.fit)
}

## File Name: rasch.mml.ramsay.R
## File Version: 2.181


######################################################
# probability ramsay qm
.prob.ramsay <- function( theta, b,  fixed.K=3 + 0*b, pow=1)
{
    XXT <- matrix( theta, nrow=length(theta), ncol=length(b))
    XXb <- matrix( b, nrow=length(theta), ncol=length(b), byrow=T )
    XX2 <- exp( (exp( XXT ))^pow / exp(XXb) )
    XX2[ XX2 > 10^15 ] <- 10^15
    KM <- matrix( fixed.K, nrow=length(theta), ncol=length(b), byrow=T )
    pm <- XX2 / ( KM + XX2 )
    pm[ pm==Inf ] <- 1
    return(pm)
}
#-----------------------------------------------------------------


##################################################################
# simulate ramsay quotient model
##NS export(sim.qm.ramsay)
sim.qm.ramsay <- function( theta, b, K  ){
    N <- length(theta)
    I <- length(b)
    p1 <- exp( outer( theta, 1/b ) )
    p1 <- p1 / ( outer( rep(1,length(theta)), K ) + p1 )
    dat <- 1* ( p1 > matrix( stats::runif( N*I ), ncol=I) )
    colnames(dat) <- paste( "I", substring(100+1:I,2), sep="")
    return(dat )
    }
##################################################################



#*************************************************************************************
# E Step Ramsay QM
.e.step.ramsay <- function( dat1, dat2, dat2.resp, theta.k, pi.k, I, n, b, fixed.K,
            group, pow.qm, ind.ii.list, pjk=NULL){
    #...................................
    # arrange groups
    if ( is.null(group) ){ group <- rep( 1, nrow(dat1)) }
    G <- length( unique( group) )
    # probabilities of correct item at theta_k
    if ( is.null( pjk )){
        pjk <- .prob.ramsay( theta.k, b,  fixed.K, pow=pow.qm )
                    }
    TP <- dim(pjk)[1]

    #***
    # array notation of probabilities
#    pjkL <- array( NA, dim=c(2, nrow(pjk), ncol(pjk) ) )
#    pjkL[1,,] <- 1 - pjk
#    pjkL[2,,] <- pjk
#    f.yi.qk <- matrix( 1, nrow(dat2), length(theta.k) )
#    for (ii in 1:ncol(dat2)){
#    #    ii <- 1
#        ind.ii <- ind.ii.list[[ii]]
#        if ( length(ind.ii)==nrow(dat2) ){
#            f.yi.qk <- f.yi.qk * pjkL[ dat2[,ii]+1,,ii]
#                    } else {
#            f.yi.qk[ind.ii,] <- f.yi.qk[ind.ii,] * pjkL[ dat2[ind.ii,ii]+1,,ii]
#                            }
#                    }
        pjkt <- t(pjk)
        pjkL <- array( NA, dim=c( I, 2, TP  ) )
        pjkL[,1,] <- 1 - pjkt
        pjkL[,2,] <- pjkt
        probsM <- matrix( aperm( pjkL, c(2,1,3) ), nrow=I*2, ncol=TP )
        f.yi.qk <- mml_calc_like( dat2=dat2, dat2resp=dat2.resp,
                    probs=probsM )$fyiqk
#cat("   calc likelihood") ; vv1 <- Sys.time(); print(vv1-vv0) ; vv0 <- vv1

    #******
    f.qk.yi <- 0 * f.yi.qk
    if ( G==1 ){ pi.k <- matrix( pi.k, ncol=1 ) }
    for ( gg in 1:G){
        f.qk.yi[ group==gg, ] <- f.yi.qk[ group==gg, ] * outer( rep( 1, nrow(dat2[ group==gg,]) ), pi.k[,gg] )
                    }
    f.qk.yi <- f.qk.yi / rowSums( f.qk.yi )
    # expected counts at theta.k
    n.k <- matrix( 0, TP, G )
    r.jk <- n.jk <- array( 0, dim=c( ncol(dat2), TP, G) )
    ll <- rep(0,G)
    for (gg in 1:G){
        ind.gg <- which( group==gg )
        res <- mml_raschtype_counts( dat2=dat2[ind.gg,], dat2resp=dat2.resp[ind.gg,],
                    dat1=dat1[ind.gg,2], fqkyi=f.qk.yi[ind.gg,],
                    pik=pi.k[,gg], fyiqk=f.yi.qk[ind.gg,]  )
        n.k[,gg] <- res$nk
        n.jk[,,gg] <- res$njk
        r.jk[,,gg] <- res$rjk
        ll[gg] <- res$ll

        }
    res <- list( "n.k"=n.k, "n.jk"=n.jk, "r.jk"=r.jk, "f.qk.yi"=f.qk.yi, "pjk"=pjk,
            "f.yi.qk"=f.yi.qk, "ll"=sum(ll) )
    return(res)
    }
#*************************************************************************************



#*************************************************************************************
# M Step Ramsay's quotient model
.m.step.ramsay <- function( theta.k, b, n.k, n, n.jk, r.jk, I, conv1, constraints,
            mitermax, pure.rasch,  trait.weights, fixed.K,
            designmatrix=designmatrix, group=group,
            numdiff.parm=numdiff.parm, pow.qm=1, max.b=NULL){
    abs.change <- 1
    miter <- 0
    # group estimation
    G <- ncol(n.k)
    # number of subjects within groups
    n <- colSums(n.k)
    # update pi.k
    pi.k <- matrix( 0, nrow=length(theta.k), ncol=G)
    for (gg in 1:G){
        if (is.null( trait.weights) | G > 1 ){
                pi.k[,gg] <- n.k[,gg] / n[gg]  } else  {
                pi.k <- sirt_dnorm_discrete( theta.k)
                pi.k <- matrix( pi.k, ncol=1 )
                        }
               }
    #*****
    # begin loop
    eps <- numdiff.parm
    # eps <- .00005
    h <- h1 <- eps
    h2 <- 1 + 2*eps
    while( abs.change > conv1 & miter < mitermax ){
        b0 <- b
            pjk <- .prob.ramsay( theta.k, b, fixed.K, pow=pow.qm )
            pjk <- ( pjk + h1 ) / h2
            pjk.M <- t(pjk)
            qjk.M <- 1 - pjk.M
            pjk1 <- .prob.ramsay( theta.k, b + h, fixed.K, pow=pow.qm )
            pjk1 <- ( pjk1 + h1 ) / h2
            pjk1.M <- t(pjk1)
            qjk1.M <- 1 - pjk1.M
            pjk2 <- .prob.ramsay( theta.k, b - h, fixed.K, pow=pow.qm )
            pjk2 <- ( pjk2 + h1 ) / h2
            pjk2.M <- t(pjk2)
            qjk2.M <- 1 - pjk2.M
            # update item parameter
            # first order derivative
            # f(x+h) - f(x-h)=2* f'(x) * h
            ll0 <- ll1 <- ll2 <- matrix( 0, nrow=nrow(n.jk), ncol=G)
            for ( gg in 1:G){
                ll0[,gg] <- rowSums( r.jk[,,gg] * log( pjk.M ) + ( n.jk[,,gg] - r.jk[,,gg]  ) * log( qjk.M ) )
                ll1[,gg] <- rowSums( r.jk[,,gg] * log( pjk1.M ) + ( n.jk[,,gg] - r.jk[,,gg]  ) * log( qjk1.M ) )
                ll2[,gg] <- rowSums( r.jk[,,gg] * log( pjk2.M ) + ( n.jk[,,gg] - r.jk[,,gg]  ) * log( qjk2.M ) )
                            }
            # sum across all groups
            ll0 <- rowSums(ll0)
            ll1 <- rowSums(ll1)
            ll2 <- rowSums(ll2)
            d1 <- ( ll1 - ll2  ) / ( 2 * h )    # negative sign?
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
        # change in item difficulty
        eps <- 10^(-15)
        b.change <- - d1 / d2
        b.change[ abs(d2) < eps ] <- 0
        bstep <- .5
        b.change <- ifelse( abs( b.change ) > bstep, sign(b.change)*bstep, b.change )
        b <- b + b.change
        # linear parameter constraints
        if ( ! is.null( designmatrix )){
            mod <- stats::lm( b ~ 0 + designmatrix  )
            b <- stats::fitted(mod)
                }
        # last item is the sum of all other item difficulties
        center <- is.null(constraints)
        if ( !is.null( constraints) ){   b[ constraints[,1] ] <- constraints[,2] }
        if (!is.null(max.b)){
            b <- ifelse(b>max.b, max.b, b)
        }
        abs.change <- max( abs( b0 - b ) )
        miter <- miter+1
        }   #*** end loop
       pjk <- .prob.ramsay( theta.k, b, fixed.K )
        # calculate log likelihood
       ll <- sapply( 1:G, FUN=function(gg){
                    sum( rowSums( r.jk[,,gg]  * t( log( pjk ) ) + ( n.jk[,,gg] - r.jk[,,gg] ) * ( t( log(1 - pjk)) ) ) )
                            } )
       ll <- sum(ll)
       list( "b"=b, "pi.k"=pi.k, "ll"=ll, "miter"=miter, "center"=center, "G"=G,
            "se.b"=sqrt(1/abs(d2)) )
    }
#*************************************************************************************
# estimation of K parameter in Ramsay's QM
.mml.ramsay.est.K <- function( theta.k, b, fixed.a, fixed.c, fixed.d,
                    fixed.K, pjk, alpha1, alpha2, h, G, I, r.jk, n.jk, est.K,
                    min.K, max.K, iter, pow.qm ){

                    #.prob.ramsay <- function( theta, b,  fixed.K=3 + 0*b)
                    #****
                    # kk
                    pjk <- .prob.ramsay( theta.k, b, fixed.K, pow=pow.qm )
                    pjk <- ( pjk + .000000005 ) / 1.00000001
                    pjk.M <- t(pjk) ;  qjk.M <- 1 - pjk.M
                    pjk1 <- .prob.ramsay( theta.k, b, fixed.K+h, pow=pow.qm  )
                    pjk1 <- ( pjk1 + .000000005 ) / 1.00000001
                    pjk1.M <- t(pjk1) ; qjk1.M <- 1 - pjk1.M
                    pjk2 <- .prob.ramsay( theta.k, b, fixed.K-h, pow=pow.qm  )
                    pjk2 <- ( pjk2 + .000000005 ) / 1.00000001
                    pjk2.M <- t(pjk2) ; qjk2.M <- 1 - pjk2.M

                    ll0 <- ll1 <- ll2 <- matrix(0, I, G)
                # was ist hier das G?=> G ist hier Anzahl der Gruppen
                    for (gg in 1:G){
                        ll0[,gg] <- rowSums( r.jk[,,gg] * log( pjk.M ) + ( n.jk[,,gg] - r.jk[,,gg]  ) * log( qjk.M ) )
                        ll1[,gg] <- rowSums( r.jk[,,gg] * log( pjk1.M ) + ( n.jk[,,gg] - r.jk[,,gg]  ) * log( qjk1.M ) )
                        ll2[,gg] <- rowSums( r.jk[,,gg] * log( pjk2.M ) + ( n.jk[,,gg] - r.jk[,,gg]  ) * log( qjk2.M )     )
                                   }
                    ll0 <- rowSums(ll0)
                    ll1 <- rowSums(ll1)
                    ll2 <- rowSums(ll2)
                    # aggregate with respect to estimation of a
                    a1 <- stats::aggregate( cbind( ll0, ll1, ll2 ), list(est.K), sum, na.rm=T)
                    a1 <- a1[ a1[,1] > 0, ]
                    ll0 <- a1[,2]
                    ll1 <- a1[,3]
                    ll2 <- a1[,4]
                    d1 <- ( ll1 - ll2  ) / ( 2 * h )    # negative sign?
                    # second order derivative
                    # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
                    d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
                    # change in item difficulty
                    K.change <- - d1 / d2
#                    a.change <- ifelse( abs( a.change ) > .2, .2*sign(a.change), a.change )
                    # dampening parameter as in tam
#                    ci <- ceiling( abs(K.change) / ( abs( old_increment.d) + 10^(-10) ) )
                    K.change <- ifelse( abs( K.change) > 1, 1, K.change )
                    K.change <- K.change[ match( est.K, a1[,1] ) ]
                    if ( any( est.K==0 ) ){
                        K.change[ est.K==0 ] <- 0
                                    }
                    fixed.K <- fixed.K + K.change
                    fixed.K[ fixed.K < min.K ] <- min.K
                    fixed.K[ fixed.K > max.K ] <- max.K
                    res <- list("fixed.K"=fixed.K, "se.K"=sqrt( 1 /abs(d2) ) )
                    return( res )
                                            }
#####################################################################

## File Name: plot.lsem.R
## File Version: 0.367

#**** plot function for objects of class lsem
plot.lsem <- function( x, parindex=NULL, ask=TRUE, ci=TRUE,
            lintrend=TRUE, parsummary=TRUE,
            ylim=NULL, xlab=NULL, ylab=NULL, main=NULL, digits=3, ... )
{

    #--- change by Johanna Hartung 2016-06-20
    if (is.null(parindex)) {
        NP <- max(x$parameters$parindex)
        parindex <- 1:NP
    } else {
        NP <- parindex
    }
    NP <- max( parindex )

    ylim0 <- ylim
    if ( ! is.null(ylim) ){
        if ( ! is.list(ylim) ){
            ylim <- list()
            for (pp in 1:NP){
                ylim[[pp]] <- ylim0
            }
        }
    }
    xlab1 <- xlab
    if ( is.null(xlab)){
        xlab1 <- x$moderator
    }

    modgrid <- x$moderator.density

    #--- loop over all parameters
    for (pp in parindex ){
        graphics::par( mfrow=c(1,1))
        ind.pp <- which( parindex==pp)
        x.pp <- x$parameters
        x.pp <- x.pp[ x.pp$parindex==pp, ]
        t1 <- paste( x.pp$par[1] )
        if (! is.null(main) ){
            t1 <- main[ind.pp]
        }
        if (parsummary){
            t1 <- paste0( t1, "\n M=", round(x$parameters_summary$M[pp],digits),
                        ", SD=", round(x$parameters_summary$SD[pp],digits),
                        " | y="    , round(x$parameters_summary$lin_int[pp],digits)    ,
                        ifelse( x$parameters_summary$lin_slo[pp] < 0, "", "+" ),
                        round(x$parameters_summary$lin_slo[pp],digits),
                        "*m"
                                )
        }
        if ( is.null(ylim0) ){
            ylim1 <- NULL
            if (!is.null(x.pp$ci.lower)){
                ylim1 <- c( min( x.pp$ci.lower), max( x.pp$ci.upper) )
            }
        } else {
                ylim1 <- ylim[[ind.pp]]
        }
        ylim1_null <- is.null(ylim1)
        if (!ylim1_null){
            if (max(ylim1) %in% c("-Inf","Inf") ){
                ylim1_null <- TRUE
            }
        }
        if (ylim1_null){
            ylim1 <- range(x.pp$est)
        }
        moderator.grid <- x$moderator.grid
        xlim1 <- range( moderator.grid )
        if ( x$type=="MGM" ){
            moderator.grouped <- x$moderator.grouped
            xlim1 <- c( min( moderator.grouped$min), max(moderator.grouped$max ) )
        }
        ylab1 <- paste(x.pp$par[1])
        if ( ! is.null(ylab) ){
            ylab1 <- ylab[ind.pp]
        }

        x1 <- modgrid[,1]
        y1 <- x.pp$est

        graphics::plot( x1, y1, xlab=xlab1, ylab=ylab1,
                    main=t1, type="p", pch=16, ylim=ylim1, xlim=xlim1 )
        if ( x$type=="LSEM" ){
            graphics::lines( lsem_spline( x=x1, y=y1 )  )
        }
        if ( x$type=="MGM" ){
            G <- nrow(moderator.grouped)
            for (gg in 1:G){
                graphics::lines( moderator.grouped[gg,1:2], rep( x.pp$est[gg], 2 ) )
            }
            graphics::points( modgrid[,1], y=x.pp$est, pch=16 )
        }

        if (lintrend){
            x1a <- xlim1
            graphics::lines( x1a, x$parameters_summary$lin_int[pp] +
            x1a*x$parameters_summary$lin_slo[pp], lty=2, col=2)
        }
        if (ci & ( x.pp$op[1] !="fit" )  ){
            if ( x$type=="LSEM" ){
                graphics::lines( lsem_spline( x=x1, y=x.pp$ci.lower), lty=4)
                graphics::lines( lsem_spline( x=x1, y=x.pp$ci.upper), lty=4)
            }
            if ( x$type=="MGM" ){
                tick_length <- diff(xlim1) / 100
                for (gg in 1:G){
                    graphics::lines( rep( x1[gg], 2 ),
                                c( x.pp$ci.lower[gg], x.pp$ci.upper[gg]),
                                col="gray", lty=1)
                    graphics::points( x1[gg], x.pp$est[gg], pch=16 )
                    graphics::lines( x1[gg] + c(-1,1)*tick_length,  rep(x.pp$ci.lower[gg],2),
                                    col="gray", lty=1)
                    graphics::lines( x1[gg] + c(-1,1)*tick_length,  rep(x.pp$ci.upper[gg],2),
                                    col="gray", lty=1)
                }
            }
        }
        graphics::par(mfrow=c(1,1))
        graphics::par(ask=ask)
    }
    #***************** end loop
}

## File Name: sirt_import_function_value.R
## File Version: 0.091

sirt_import_function_value <- function(fun, pkg, ...)
{
    TAM::require_namespace_msg(pkg)
    fun1 <- NULL
    eval(parse(text=paste0('fun1 <- ', pkg,'::', fun)))
    res <- fun1(...)
    return(res)
}

## File Name: mgsem_proc_model_difflp_information.R
## File Version: 0.187


mgsem_proc_model_difflp_information <- function(partable)
{
    ND <- nrow(partable)
    NP <- max(partable$index)
    is_lpdiff_entry <- rep(FALSE,ND)
    lpdiff_diff_indices <- list()
    pen_difflp_factor <- list()

    pen_entry <- 'pen_difflp'
    loop_parms <- which(partable$unique==1 & partable$group>0)

    for (dd in loop_parms){
        index_dd <- partable[dd,'index']
        ind <- which( partable$type==partable$type[dd] & partable$i1==partable$i1[dd] &
                partable$i2==partable$i2[dd] & (partable$group > 0) & (partable$rid!=dd) &
                partable$unique==1)
        if (partable$group[dd]==0){
            ind <- NULL
        }
        if ( (length(ind)>0) & (partable$pen_difflp[dd]>0)  ){
            is_lpdiff_entry[dd] <- TRUE
            lpdiff_diff_indices[[dd]] <- partable[ind, 'index']
            fac <- sqrt(partable[dd,pen_entry]*partable[ind,pen_entry])
            pen_difflp_factor[[dd]] <- fac
        } else {
            partable$pen_difflp[dd] <- 0
        }
    }

    #--- design matrices
    cp <- partable[ partable$unique==1, 'name']
    lpdiff_facmat <- matrix(0, nrow=NP, ncol=NP)
    rownames(lpdiff_facmat) <- colnames(lpdiff_facmat) <- cp
    lpdiff_n <- lpdiff_facmat
    for (dd in loop_parms){
        if (is_lpdiff_entry[dd]){
            i1 <- partable[dd,'index']
            i2 <- lpdiff_diff_indices[[dd]]
            lpdiff_facmat[i1,i2] <- pen_difflp_factor[[dd]]
            n1 <- partable[partable$index==i1,'N_group']
            n2 <- partable[partable$index%in%i2,'N_group']
            lpdiff_n[i1,i2] <- 0.5*sqrt(n1*n2)
        }
    }

    difflp_indices <- which( rowSums(lpdiff_facmat)>0 )
    lpdiff_facmat <- lpdiff_facmat[difflp_indices, difflp_indices]
    lpdiff_n <- lpdiff_n[difflp_indices, difflp_indices]
    is_pen_difflp <- nrow(lpdiff_facmat)>0
    lpdiff_facmat_logical <- (lpdiff_facmat>0)

    #--- output
    difflp_info <- list(is_lpdiff_entry=is_lpdiff_entry,
                        lpdiff_diff_indices=lpdiff_diff_indices,
                        pen_difflp_factor=pen_difflp_factor,
                        lpdiff_facmat=lpdiff_facmat,
                        difflp_indices=difflp_indices,
                        is_pen_difflp=is_pen_difflp,
                        lpdiff_facmat_logical=lpdiff_facmat_logical,
                        lpdiff_n=lpdiff_n )
    res <- list(partable=partable, difflp_info=difflp_info)
    return(res)
}

## File Name: rexppow.R
## File Version: 0.232


rexppow <- function (n, mu=0, sigmap=1, pow=2, xbound=100, xdiff=.01)
{
    use_rcpp <- TRUE
    xmin <- -xbound
    xmax <- xbound
    x <- sigmap*( mu + seq(xmin, xmax, by=xdiff) )
    nx <- length(x)
    dx <- dexppow(x=x, mu=mu, sigmap=sigmap, pow=pow)
    dx <- dx / sum(dx)
    dfr <- data.frame( x=x, min=x-xdiff/2, max=x+xdiff/2, cum=cumsum(dx))
    rn <- stats::runif(n)
    if (!use_rcpp){
        res <- rep(NA,n)
        for (nn in 1L:n){
            ind_nn <- min( which( rn[nn] <=dfr$cum ) )
            dfr_nn <- dfr[ind_nn,]
            ind_nn1 <- ind_nn-1
            if (ind_nn1==0){
                res[nn] <- dfr_nn[1,'x']
            } else {
                res[nn] <- dfr_nn$min+xdiff*(rn[nn]-dfr[ind_nn1,'cum'])/
                                    (dfr[ind_nn,'cum']-dfr[ind_nn1,'cum'])
            }
        }
    } else {
        res <- sirt_rcpp_discrete_inverse(x0=dfr$x, y0=dfr$cum, y=rn)$x
    }
    return(res)
}

## File Name: sirt_summary_print_computation_time.R
## File Version: 0.06

sirt_summary_print_computation_time <- function( object )
{
    t1 <- object$time$start
    t2 <- object$time$end
    cat( 'Date of Analysis:', '\n' )
    cat( '   Start:', paste( t1 ), '\n' )
    cat( '   End  :', paste( t2 ), '\n' )
    cat('Computation time:', print( t2 - t1 ), '\n\n')
}

## File Name: linking_haberman_als_vcov.R
## File Version: 0.167

linking_haberman_als_vcov <- function( regr_resid, regr_wgt, transf_pars,
        selitems, estimation="OLS", vcov=TRUE, NS=NULL )
{
    transf_parsM <- matrix( transf_pars, nrow=nrow(regr_resid),
            ncol=ncol(regr_resid), byrow=TRUE )
    regr_resid <- regr_resid + transf_parsM
    regr_resid <- regr_resid[ selitems, ]
    regr_wgt <- regr_wgt[selitems,]
    if ( is.null(NS) ){
        NS <- ncol(regr_resid)
    }
    N <- nrow(regr_resid)
    data <- data.frame( y=matrix( regr_resid, ncol=1 ),
                        study=rep(1L:NS, each=N) )
    data$wgt <- matrix( regr_wgt, ncol=1 )
    data <- stats::na.omit(data)
    for (ss in 2L:NS){
        data[, paste0('X', ss) ] <- 1*(data$study==ss)
    }
    if (vcov){
        form <- paste0( 'y ~ ', paste0('X', 2L:NS, collapse=' + ' ) )
        mod <- stats::lm( stats::as.formula(form), data=data, weights=data$wgt )
        mod_vcov <- stats::vcov(mod)
        vcov <- mod_vcov[-1,-1]
        se <- sqrt( diag( mod_vcov) )[-1]
        if (estimation!='OLS'){
            vcov <- NA*vcov
            se <- NA*se
        }
    } else {
        vcov <- NULL
        se <- NA
    }
    #-- output
    res <- list( vcov=vcov, se=se )
    return(res)
}

## File Name: fit.gradedresponse.R
## File Version: 1.162


#--- fit logistic model
fit.gradedresponse <- function( freq.categories, SC, I, K,
    conv=.0001, maxit=100, progress=TRUE )
{
    if (progress){ cat('\n*******Graded Response Model***********\n') }
    theta <- stats::qlogis( seq( .5, SC-1, len=SC ) / SC )
    # item parameters
    b <- rep(0,I)
    b.cat <- seq(1.5, -1.5, len=K)
    #*** begin algorithm
    numdiff.parm <- .001
    max.increment <- 1
    increment.factor <- 1.01
    deviation <- 1000
    iter <- 0
    ########
    # algorithm
    while( ( iter < maxit) & ( deviation > conv ) ){
        b0 <- b
        b.cat0 <- b.cat
        theta0 <- theta
        max.increment <- max.increment * increment.factor^(-iter)
        # update theta
        res <- .update.theta.grm( theta, b, b.cat, freq.categories,
                        numdiff.parm, max.increment=max.increment)
        theta <- res$theta
        ll <- res$ll
        prob <- res$prob
        # update b
        res <- .update.b.grm( theta, b, b.cat, freq.categories,
                        numdiff.parm, max.increment)
        b <- res$b
        b <- b - mean(b)
        # update b.cat
        res <- .update.bcat.grm( theta, b, b.cat, freq.categories,
                        numdiff.parm, max.increment)
        b.cat <- res$b.cat
        b.cat <- b.cat - mean(b.cat )
        deviation <- max( abs( c( theta-theta0, b - b0, b.cat - b.cat0 )) )
        if (progress){
            cat( 'Iteration', iter, '- Deviation=',  round( deviation, 6 ), '\n')
            utils::flush.console()
        }
        iter <- iter + 1
    }  # end algorithm
    #*************************
    llcase.grm <- ll / mean( rowSums(colSums( freq.categories )))
    # output
    res <- list( item.sc=b, cat.sc=b.cat, person.sc=theta, ll=ll, llcase=llcase.grm,
                    prob=prob )
    return(res)
}

## File Name: reliability.nonlinear.sem.R
## File Version: 1.19



#********************************************************************
# Function calculates reliability from nonlinear SEM
reliability.nonlinearSEM <- function( facloadings, thresh,
        resid.cov=NULL, cor.factors=NULL ){
        # INPUT:
        # loadings      ... matrix of factor loadings
        # thresh        ... vector of thresholds
        # cor.factors   ... correlation matrix of factors
        #.............................................
        facloadings <- as.matrix( facloadings )
        NF <- ncol(facloadings ) # number of factor facloadings
        I <- nrow(facloadings)
        # correlation matrix
        if ( is.null( cor.factors )){
                cor.factors <- matrix( 0, NF, NF )
                diag( cor.factors ) <- rep(1,NF)
                        }
        if ( is.null(resid.cov) ){
            resid.cov <- diag(I )
                }
        # number of items
        I <- nrow(facloadings)
        # transform thresholds
        pthresh <- stats::pnorm( thresh )
        # create matrix of multiplied facloadings (expected correlation)
        rho.exp <- matrix( 0, I, I )
        colnames(rho.exp) <- rownames(rho.exp) <- rownames(facloadings)
        # reliability matrix
        rel.matrix2 <- rel.matrix <- rho.exp
        for (ii1 in 1:I){
            for (ii2 in 1:ii1){
                #        ii1 <- 2
                #        ii2 <- 4
                rho.exp[ii1,ii2] <- as.vector(facloadings[ii1,]) %*% cor.factors  %*%
                                            matrix( as.vector(facloadings[ii2,]), ncol=1 )
                rho.exp[ii2,ii1] <- rho.exp[ii1,ii2]
                r1 <- rho.exp[ii1,ii2]
                rel.matrix[ii1,ii2] <- sirt_pmvnorm( c(-Inf,-Inf), pthresh[c(ii1,ii2)],
                                         corr=matrix( c( 1, r1, r1, 1),2,2 ) ) -
                                         stats::pnorm( pthresh[ii1] ) * stats::pnorm( pthresh[ii2] )
                r1 <- max( -1, min( r1 + resid.cov[ii1,ii2], 1 ) )
                rel.matrix2[ii1,ii2] <- sirt_pmvnorm( c(-Inf,-Inf), pthresh[c(ii1,ii2)],
                                         corr=matrix( c( 1, r1, r1, 1),2,2 ) ) -
                                         stats::pnorm( pthresh[ii1] ) * stats::pnorm( pthresh[ii2] )
#                rel.matrix2[ii1,ii2] <- rel.matrix[ii1,ii2]
                rel.matrix[ii2,ii1] <- rel.matrix[ii1,ii2]
                rel.matrix2[ii2,ii1] <- rel.matrix2[ii1,ii2]
                if (ii1==ii2){
                    r1 <- 1
#                    r1 <- 1 - 1e-6
                    rel.matrix2[ii1,ii2] <- sirt_pmvnorm( c(-Inf,-Inf), pthresh[c(ii1,ii2)],
                                               corr=matrix( c( 1, r1, r1, 1),2,2 ) ) -
                                                    stats::pnorm( pthresh[ii1] ) * stats::pnorm( pthresh[ii2] )
                    rel.matrix2[ii2,ii1] <- rel.matrix2[ii1,ii2]
                                    }
                                }
                        }
        # calculation of reliability
        omega.rel <- sum( rel.matrix ) / sum( rel.matrix2 )
        # output
        res <- list( "omega.rel"=omega.rel, "NF"=NF, "cor.factors"=cor.factors,
                    "pthresh"=pthresh, "rho.exp"=rho.exp, "rel.matrix"=rel.matrix,
                        "rel.matrix2"=rel.matrix2)
        return(res)
            }
#********************************************************************

## File Name: mcmc_as_formula.R
## File Version: 0.121

mcmc_as_formula <- function( string )
{
    string <- paste0( string, collapse=' ' )
    string <- gsub('___ ', '___', string, fixed=TRUE )
    form <- stats::as.formula(string)
    return(form)
}

## File Name: mgsem_add_increment.R
## File Version: 0.04


mgsem_add_increment <- function(x, h, i1, i2=NULL, symm=FALSE )
{
    x1 <- x
    if (is.vector(x)){
        x1[i1] <- x[i1] + h
    } else {
        x1[i1,i2] <- x[i1,i2] + h
        if (symm & (i1!=i2) & (!is.null(i2)) ){
            x1[i2,i1] <- x1[i2,i1] + h
        }
    }
    return(x1)
}

## File Name: rasch_pairwise_iterations.R
## File Version: 0.111


rasch_pairwise_iterations <- function(eps, y.ij, delta.ij, conv, maxiter,
    zerosum, progress, fix_first=FALSE)
{
    change <- 1
    iter <- 0

    #* start estimation algorithm
    while( change > conv & iter < maxiter ){
        eps0 <- eps
        b0 <- b
        eps <- sqrt( rowSums( y.ij * eps * delta.ij ) / colSums( y.ij / eps ) )
        if (zerosum){
            eps <- rasch_pairwise_zerosum(eps=eps)
        }
        if (fix_first){
            eps[1] <- 1
        }
        b <- -log(eps)
        change <- max( abs( eps0 - eps ) )
        change_b <- max(abs( -b0 + b))
        iter <- iter + 1
        if ( progress ){
            cat( 'PL Iter.', iter, ': max. parm. change=', round( change_b, 6 ), '\n')
            utils::flush.console()
        }
    } #* end estimation algorithm
    if (zerosum){
        eps <- rasch_pairwise_zerosum(eps=eps)
        b <- -log(eps)
    }
    #--- output
    res <- list(eps=eps, b=b, iter=iter)
    return(res)
}

## File Name: automatic.recode.R
## File Version: 1.194

#*** automatic recoding of a dataset
automatic.recode <- function( data, exclude=NULL, pstart.min=.6,
    allocate=200, maxiter=20, progress=TRUE)
{
    # data table with frequencies and discriminations
    item.stat <- data.frame( item=colnames(data), old.key=-99, new.key=NA,
                                p=NA, discrim=NA )
    items <- colnames(data)
    # compute frequencies
    fstart <- TAM::tam.ctt3( data, allocate=allocate, progress=FALSE)
    I <- ncol(data)
    prbar <- floor( 10 * ( 1L:I ) / (I+1) )
    prbar <- c(1,which( diff(prbar)==1 )  )

    fstart1 <- fstart
    fstart1 <- fstart1[ ! ( fstart$Categ %in% exclude ), ]
    fstart1 <- fstart1[ fstart1$RelFreq > pstart.min, ]
    fstart1 <- fstart1[ order( fstart1$itemno + fstart1$RelFreq, decreasing=TRUE), ]
    fstart1 <- fstart1[ ! duplicated( fstart1$itemno), ]
    item.stat[ fstart1$itemno, 'new.key' ] <- fstart1[, 'Categ' ]
    item.stat[ fstart1$itemno, 'p' ] <- fstart1[, 'RelFreq' ]
    iter <- 1
    changed.keys <- I

    #--- start iterations
    while ( ( iter < maxiter ) & ( changed.keys > 0 ) ){
        # select items which should be recoded
        is1 <- paste( item.stat[ ! is.na( item.stat$new.key ), 'item' ] )

        # scoring responses
        data.raw <- data[, is1 ]
        item.stat$key <- item.stat$new.key
        # scoring data
        data.scored <- data.recode.sirt( data.raw, keys=item.stat )

        score <- rowMeans( data.scored, na.rm=TRUE )
        # compute frequencies in iterations
    #    if (progress){ cat( '   apply tam.ctt3\n') }
        fiter <- TAM::tam.ctt3( data, allocate=allocate, wlescore=score, progress=FALSE)
    #    if (progress){ cat( '   |') }
        item.stat$old.key <- item.stat$new.key

        fstart1 <- fiter
        fstart1 <- fstart1[ ! ( fstart$Categ %in% exclude ), ]
        fstart1 <- fstart1[ order( fstart1$itemno + fstart1$rpb.WLE, decreasing=TRUE), ]
        fstart1 <- fstart1[ ! duplicated( fstart1$itemno), ]

        item.stat[ fstart1$itemno, 'new.key' ] <- fstart1[, 'Categ' ]
        item.stat[ fstart1$itemno, 'p' ] <- fstart1[, 'RelFreq' ]
        item.stat[ fstart1$itemno, 'discrim' ] <- fstart1[, 'rpb.WLE' ]

        changed.keys <- sum( paste(item.stat$new.key) !=paste(item.stat$key) )
        if (progress){
            cat( paste0( 'Iteration ', iter,     ' | ',
                            'Changed ', changed.keys, ' Keys\n' ) )
            utils::flush.console()
        }
        iter <- iter+1
    }
    #********* end iterations
    item.stat$old.key <- item.stat$new.key <- NULL
    res <- list( item.stat=item.stat, data.scored=data.scored, categ.stats=fiter)
    return(res)
}

## File Name: mgsem_ginv.R
## File Version: 0.04

mgsem_ginv <- function(X)
{
    requireNamespace('MASS')
    res <- MASS::ginv(X=X)
    rownames(res) <- colnames(res) <- colnames(X)
    return(res)
}

## File Name: xxirt_compute_likelihood.R
## File Version: 0.290


##-- xxirt compute likelihood
xxirt_compute_likelihood <- function( probs_items, dat, resp_index=NULL,
        dat_resp_bool=NULL, person_covariates_items=FALSE )
{
    N <- nrow(dat)
    TP <- dim(probs_items)[3]
    I <- dim(probs_items)[1]
    maxK <- dim(probs_items)[2]
    # p.xi.aj <- matrix( 1, nrow=N, ncol=TP )
    if (!person_covariates_items){
        probs <- matrix( probs_items, nrow=I, ncol=maxK*TP )
        p.xi.aj <- sirt_rcpp_xxirt_compute_likelihood( dat=dat,
                            dat_resp_bool=dat_resp_bool,
                            probs=probs, TP=TP, maxK=maxK )
    } else {
        probs <- as.vector( probs_items )
        p.xi.aj <- sirt_rcpp_xxirt_compute_likelihood_person_covariates( dat=dat,
                        dat_resp_bool=dat_resp_bool, probs=probs, TP=TP, maxK=maxK )
    }
    #-- output
    return(p.xi.aj)
}

## File Name: rm_sdt_calc_gradient_likelihood_item_llgrad2.R
## File Version: 0.18

rm_sdt_calc_gradient_likelihood_item_llgrad2 <- function(logprob_D1, nik.item, diffindex, K,
    prob_D1_dim )
{
    nik_item <- as.vector(nik.item)
    logprob_D1 <- as.vector(logprob_D1)
    ll_grad <- sirt_rcpp_rm_sdt_calc_gradient_likelihood_item_llgrad( logprob_D1=logprob_D1,
                    prob_D1_dim=prob_D1_dim, nik_item=nik_item )
    ll_grad <- rowsum(ll_grad, diffindex )
    ll_grad <- ll_grad[ rownames(ll_grad) > 0, 1 ]
    return(ll_grad)
}

## File Name: mml_raschtype_counts.R
## File Version: 0.08



# calculation of counts
mml_raschtype_counts <- function (dat2, dat2resp, dat1, fqkyi, pik, fyiqk)
{
    res <- MML2_RASCHTYPE_COUNTS( DAT2=dat2, DAT2RESP=dat2resp, DAT1=dat1,
                FQKYI=fqkyi, PIK=pik, FYIQK=fyiqk)
    return(res)
}

## File Name: smirt.R
## File Version: 7.302


#--- smirt model
#--- noncompensatory and compensatory IRT models
smirt <- function( dat, Qmatrix, irtmodel="noncomp",
    est.b=NULL, est.a=NULL,
    est.c=NULL, est.d=NULL, est.mu.i=NULL, b.init=NULL, a.init=NULL,
    c.init=NULL, d.init=NULL, mu.i.init=NULL, Sigma.init=NULL,
    b.lower=-Inf, b.upper=Inf, a.lower=-Inf, a.upper=Inf,
    c.lower=-Inf, c.upper=Inf, d.lower=-Inf, d.upper=Inf,
    theta.k=seq(-6,6,len=20),     theta.kDES=NULL, qmcnodes=0,
    mu.fixed=NULL, variance.fixed=NULL,
    est.corr=FALSE, max.increment=1, increment.factor=1,
    numdiff.parm=.0001, maxdevchange=.10,
    globconv=.001, maxiter=1000, msteps=4, mstepconv=.001)
{

    s1 <- Sys.time()
    if (is.null(colnames(dat))){
        colnames(dat) <- paste0("I", 1:ncol(dat))
    }
    dat0 <- dat <- as.matrix(dat)
    N <- nrow(dat)
    # check inits for item parameters
    res1 <- .smirt.check.inits( a.init, b.init, irtmodel, Qmatrix )
    a.init <- res1$a.init
    b.init <- res1$b.init

    #***
    # process data
    # Q matrix and theta design matrix
    Q <- Qmatrix
    if ( is.null( theta.kDES) ){
        D <- ncol(Qmatrix)
        TD <- D
            } else {
            D <- ncol(theta.k)
            TD <- ncol(theta.kDES )
                }
    if ( is.null(colnames(Qmatrix) ) ){
        colnames(Qmatrix) <- paste0( "Dim", 1:D )
            }
    if ( is.null(theta.kDES) ){
        nodes <- theta.k
        theta.k <- as.matrix( expand.grid( as.data.frame( matrix( rep(nodes, D),
                ncol=D ) ) ) )
                }
    theta.k <- as.matrix( theta.k )
    Q <- as.matrix(Q)
    Qmatrix <- as.matrix(Qmatrix)
    QMC <- FALSE
    if (qmcnodes>0){
        QMC <- TRUE
        theta.k <- qmc.nodes( snodes=qmcnodes, ndim=D )
#        if (! is.null(Sigma.init) ){
#            theta.k <- theta.k %*% chol(Sigma.init)
#                        }
        colnames(theta.k) <- paste0("Dim",1:D)
        theta.k0 <- theta.k
            }
    TP <- nrow(theta.k)
    # theta design matrix
    if ( is.null(theta.kDES) ){ theta.kDES <- theta.k }
    theta.kDES <- as.matrix( theta.kDES )
    # K <- max( dat, na.rm=T )
    K <- 1
    dat2 <- dat
    I <- ncol(dat)
    dat2.resp <- 1 - is.na(dat)
    dat2[ dat2.resp==0 ] <- 0
    # indicator matrix
    dat2.ind0 <- dat2.resp * 1*(dat2==0)
    dat2.ind1 <- dat2.resp * 1*(dat2==1)
    dat2.ind <- as.matrix( cbind( dat2.ind0, dat2.ind1 ) )
    #*** indicators for calculating probabilities
#    index_ind <- matrix( 0, N, I )
#    index_indL <- rep(0,I)
#    for (ii in 1:I){
        #ii <- 1
#        vii <- which( dat2.resp[,ii]==1 )
#        index_indL[ii] <- length(vii)
#        index_ind[ 1:(index_indL[ii]), ii] <- vii - 1
#                }
    #****
    # init b parameters
    if ( is.null(b.init)){
        if (irtmodel!="comp"){
            b <- Qmatrix * matrix( - stats::qlogis( colMeans(dat, na.rm=TRUE ) ),I, D )
                } else {    # irtmodel="comp"s
            b <-  - stats::qlogis( colMeans(dat, na.rm=TRUE ) )
                        }
                    } else {
                b <- b.init
                    }
    if ( ! is.null(est.b) & is.null(b.init) ){ b <- 0*b }
    e1 <- matrix( 1:I, I, ncol(Qmatrix) )
    if ( is.null(est.b)){
        if (irtmodel!="comp"){ est.b <- Qmatrix*e1  }
        if (irtmodel=="comp"){ est.b <- 1:I  }
                }
    #****
    # init a parameters
    if ( is.null(a.init) ){
        a <- matrix( 1, I, ncol(Qmatrix) )
        a[ Qmatrix==0 ] <- 0
            } else { a <- a.init }
#    max.a.increment <- a*max.increment
    max.a.increment <- matrix( .4, nrow=1, ncol=ncol(a) )
    if ( ! is.null( est.a)){
        if ( est.a=="2PL"){
            est.a <- matrix(1:I, I, ncol(Qmatrix) ) * ( Qmatrix !=0 )
            est.corr <- TRUE
                }
            }
    if ( is.null(est.a)){
        est.a <- 0 * Qmatrix*e1
                }
    if ( ! is.null(est.a) & is.null(a.init) ){ a <- 1*a }
    #***
    # init c and d parameters
    if ( is.null(c.init)){  c <- rep(0,I)  } else { c <- c.init }
    if ( is.null(d.init)){     d <- rep(1,I) } else { d <- d.init }
    # init mu parameters for partially compensatory model
    if ( is.null(mu.i.init)){
            mu.i <- rep(.5,I)
            mu.i[ ( rowSums(Qmatrix>0)==1 ) ] <- 0
                } else {
            mu.i <- mu.i.init
            }
    if ( irtmodel=="partcomp"){
        if ( is.null(est.mu.i) ){
            est.mu.i <- ( 1:I ) * ( rowSums(Qmatrix>0) > 1 )
                                }
                            }

    # calculate pi.k
    mu <- rep(0,D)
    if ( is.null(mu.fixed) ){     mu.fixed <- cbind( 1:D, 0 ) }
    if ( is.null(Sigma.init) ){
            Sigma <- .4+.6*diag(1,D) } else { Sigma <- Sigma.init }
    if ( ! is.null(variance.fixed ) ){
            Sigma[ variance.fixed[,1:2,drop=FALSE] ] <- variance.fixed[,3]
            Sigma[ variance.fixed[,c(2,1),drop=FALSE] ] <- variance.fixed[,3]
                            }

    pi.k <- sirt_dmvnorm_discrete( theta.k, mean=mu, sigma=Sigma )
    # init counts
    n.ik <- array( 0, dim=c(TP,I, K+1) )
    probs <- array(0, dim=c(I,K+1,TP))
    se.mu.i <- se.d <- se.c <- se.b <- se.a <- NULL
    # inits
    iter <- 0
    dev0 <- dev <- 0
    conv <- devchange <- 1000
    disp <- "...........................................................\n"
    #****************************************************
    # start EM algorithm
    while( ( ( maxdevchange < devchange ) | (globconv < conv) ) &
            ( iter < maxiter )
                        ){
# zz0 <- Sys.time()
        cat(disp)
        cat("Iteration", iter+1, "   ", paste( Sys.time() ), "\n" )

        # previous values
        dev0 <- dev
        b0 <- b
        a0 <- a
        c0 <- c
        d0 <- d
        mu0 <- mu
        mu.i0 <- mu.i
        Sigma0 <- Sigma
        # calculate probabilities
        probres <- .smirt.calcprob( a, b, Q=Qmatrix, thetak=theta.kDES, c, d,
                    mu.i,     irtmodel )
        probs <- problong2probarray( probres, I, TP )
# cat("smirt calcprob") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1
        # calculate likelihood and posterior
        if (QMC){ pi.k <- 1+0*pi.k }
        probsM <- matrix( aperm( probs, c(2,1,3) ), nrow=I*2, ncol=TP )
        res1 <- calcpost( dat2, dat2.resp, probsM, dat2.ind, pi.k, K )
# cat("smirt calcpost") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1
#        res2 <- calcpost2( dat2, dat2.resp, probsM, dat2.ind, pi.k, K,
#                index_ind, index_indL)
#cat("smirt calcpost2") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1
        f.yi.qk <- res1$fyiqk
        f.qk.yi <- res1$f.qk.yi
        pi.k <- res1$pi.k
        n.ik <- array( res1$n.ik, dim=dim(n.ik) )
        N.ik <- res1$N.ik
        # estimate b parameters
        res2 <- .smirt.est.b(   b, a, c, d, mu.i, Qmatrix, est.b,
                theta.k=theta.kDES,
                n.ik, I, K, TP, D,  numdiff.parm=numdiff.parm,
                max.increment=max.increment,
                msteps,  mstepconv, irtmodel, increment.factor )
        b <- res2$b
        se.b <- res2$se.b
        b <- smirt_squeeze( b, b.lower, b.upper, est.b )

#        ll <- res2$ll
# cat("smirt est.b") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1

        # estimate a parameters
        if ( sum (est.a) > 0 ){
            res2 <- .smirt.est.a(   b, a, c, d, mu.i, Qmatrix, est.a,
                theta.k=theta.kDES,
                n.ik, I, K, TP, D=TD,  numdiff.parm=numdiff.parm,
                max.a.increment=max.a.increment,
                msteps,  mstepconv, irtmodel, increment.factor )
            a <- res2$a
            se.a <- res2$se.a
            a <- smirt_squeeze( a, a.lower, a.upper, est.a )
                            }


        # estimate c parameters
        if ( sum (est.c) > 0 ){
            res2 <- .smirt.est.c(   b, a, c, d, mu.i, Qmatrix, est.c,
                theta.k=theta.kDES,
                n.ik, I, K, TP, D,  numdiff.parm=numdiff.parm,
                max.increment=max.increment,
                msteps,  mstepconv, irtmodel, increment.factor)
            c <- res2$c
            se.c <- res2$se.c
            c <- smirt_squeeze( c, c.lower, c.upper, est.c )
                            }

        # estimate d parameters
        if ( sum (est.d) > 0 ){
            res2 <- .smirt.est.d(   b, a, c, d, mu.i, Qmatrix, est.d,
                theta.k=theta.kDES,
                n.ik, I, K, TP, D,  numdiff.parm=numdiff.parm,
                max.increment=max.increment,
                msteps,  mstepconv, irtmodel, increment.factor )
            d <- res2$d
            se.d <- res2$se.d
            d <- smirt_squeeze( d, d.lower, d.upper, est.d )
                            }

        # estimate mu.i parameters
        if ( sum(est.mu.i) > 0 ){
            res2 <- .smirt.est.mu.i.partcomp(   b, a, c, d, mu.i, Qmatrix, est.mu.i, theta.k,
                n.ik, I, K, TP, D, numdiff.parm, max.increment=max.increment,
                msteps,  mstepconv, increment.factor)
            mu.i <- res2$mu.i
            se.mu.i <- res2$se.mu.i
                            }
#         print(a1-a0) ; a0 <- a1 ;
        flush.console()

        #***
        # decrease increments in every iteration
        if( increment.factor > 1){
            max.increment <-  max.increment / increment.factor
            max.a.increment <-  max.a.increment / increment.factor
                    }

        # Covariance estimation
        m1 <- .smirt.est.covariance( f.qk.yi, Sigma, theta.k, N,
            mu.fixed, variance.fixed, D, est.corr, irtmodel )
        mu <- m1$mu
        Sigma <- m1$Sigma
        pi.k <- m1$pi.k
# cat("smirt est.covariance") ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1

        #**
        # calculate deviance
        pi.k <- matrix( pi.k, ncol=1 )
#        ll <- sum( log( rowSums( f.yi.qk * outer( rep(1,nrow(f.yi.qk)), pi.k ) ) ) )
        ll <- sum( log( rowSums( f.yi.qk * outer( rep(1,nrow(f.yi.qk)), pi.k[,1] ) ) ) )
        if (QMC){
            ll <- sum( log( rowMeans( f.yi.qk ) ) )
            # beta[ rep(1,snodes), ] +  t ( t(chol(variance)) %*% t(theta0.samp) )
            theta.k <- matrix( mu, nrow=TP, ncol=D, byrow=T ) + t ( t(chol(Sigma)) %*% t(theta.k0) )
            theta.kDES <- theta.k
                }
        dev <- -2*ll
        # convergence criteria
        conv <- max( abs(a-a0), abs(b-b0), abs(c-c0), abs(d-d0), abs(mu.i - mu.i0) )
        iter <- iter+1
        devchange <- abs( ( dev - dev0 ) / dev0  )
        #****
        # print progress
        cat( paste( "   Deviance=", round( dev, 4 ),
            if (iter > 1 ){ " | Deviance change=" } else {""},
            if( iter>1){round( - dev + dev0, 6 )} else { ""}    ,"\n",sep="") )
        cat( paste( "    Maximum b parameter change=",
                paste( round(max(abs(b-b0)),6), collapse=" " ), "\n", sep=""))
        cat( paste( "    Maximum a parameter change=",
                paste( round(max(abs(a-a0)),6), collapse=" " ), "\n", sep=""))
        cat( paste( "    Maximum c parameter change=",
                paste( round(max(abs(c-c0)),6), collapse=" " ), "\n", sep=""))
        cat( paste( "    Maximum d parameter change=",
                paste( round(max(abs(d-d0)),6), collapse=" " ), "\n", sep=""))
        if (irtmodel=="partcomp"){
            cat( paste( "    Maximum mu.i parameter change=",
                paste( round(max(abs(mu.i-mu.i0)),6), collapse=" " ), "\n", sep=""))
                    }

        #--- distribution parameters
        cat( " Means: ", round( mu, 3 ), "\n")
        cat( " Standard deviations: ", round( sqrt(diag(Sigma)), 3 ), "\n")
        c1 <- stats::cov2cor(Sigma)
        cat( " Correlations ", round( c1[lower.tri(c1)], 3 ), "\n")
        # flush.console()
                }


    #*** arrange OUTPUT

    #-- Information criteria
    ic <- list( "deviance"=dev, "n"=nrow(dat2) )
    ic$np.item <- 0
    ic$np.item.b <- ic$np.item.a <- ic$np.item.c <- ic$np.item.d <- 0
    if (irtmodel=="noncomp"){
        for (dd in 1:D){
            ic$np.item.b <- ic$np.item.b + length( setdiff( unique( est.b[,dd] ), 0 )  )
        }
    } else {
        ic$np.item.b <- ic$np.item.b + length( setdiff( unique( est.b ), 0 )  )
    }
    for (dd in 1:TD){
        ic$np.item.a <- ic$np.item.a + length( setdiff( unique( est.a[,dd] ), 0 ) )
    }
    ic$np.item.c <- length( setdiff( unique( est.c ), 0 ) )
    ic$np.item.d <- length( setdiff( unique( est.d ), 0 ) )
    ic$np.item.mu.i <- length( setdiff( unique( est.mu.i ), 0 ) )
    # sum over all item parameters
    ic$np.item <- ic$np.item.b + ic$np.item.a + ic$np.item.c + ic$np.item.d + ic$np.item.mu.i
    # covariance matrix
    ic$np.cov <- 0

    ic$np.cov.covM <- D*(D+1)/2
    if ( est.corr ){
        ic$np.cov.covM <- ic$np.cov.covM - D
    }
    if ( ! is.null(variance.fixed) ){
        ic$np.cov.covM <- ic$np.cov.covM - nrow(variance.fixed)
    }
    ic$np.cov.mu <- D
    if ( ! is.null(mu.fixed) ){
        ic$np.cov.mu <- ic$np.cov.mu - nrow(mu.fixed)
    }
    ic$np.cov <- ic$np.cov.covM + ic$np.cov.mu
    ic$np <- ic$np.item + ic$np.cov
    ic$AIC <- dev + 2*ic$np
    ic$BIC <- dev + ( log(ic$n) )*ic$np
    ic$CAIC <- dev + ( log(ic$n) + 1 )*ic$np
    ic$AICc <- ic$AIC + 2*ic$np * ( ic$np + 1 ) / ( ic$n - ic$np - 1 )

    #*** person parameters
    pers <- .smirt.person.parameters( data=dat2, D=D, theta.k=theta.k,
        p.xi.aj=f.yi.qk, p.aj.xi=f.qk.yi, weights=rep(1,N) )
    person <- pers$person
    EAP.rel <- pers$EAP.rel

    #--- item
    item <- data.frame("item"=colnames(dat))
    item$N <- colSums( dat2.resp )
    item$p <- colSums( dat2, na.rm=TRUE) / item$N
    if ( irtmodel !="comp"){
        for (dd in 1:( ncol(Qmatrix)) ){
            b[ Qmatrix[,dd]==0, dd ] <- NA
            se.b[ est.b[,dd]==0, dd ] <- NA
            item[, paste0("b.",colnames(Qmatrix)[dd]) ] <- b[,dd]
        }
    } else {    # end irtmodel=="noncomp"
        item$b <- b
        se.b[ est.b==0 ] <- NA
    } # end irtmodel !="noncomp"
    for (dd in 1:( ncol(Qmatrix)) ){
        a[ Qmatrix[,dd]==0, dd ] <- NA
        if ( sum( est.a) !=0 ){
            se.a[ est.a[,dd]==0, dd ] <- NA
        }
        item[, paste0("a.",colnames(Qmatrix)[dd]) ] <- a[,dd]
    }
    item$c <- c
    item$d <- d
    if ( irtmodel=="partcomp" ){
        mu.i[ rowSums( Qmatrix > 0 )==1 ] <- NA
        item$mu.i <- mu.i
    }

    obji <- item
    for (vv in seq(2,ncol(obji) )){
        obji[,vv] <- round( obji[,vv],3 )
    }
    cat("*********************************\n")
    cat("Item Parameters\n")
    print( obji )

    cat("*********************************\n")
    cat("EAP Reliability=", round(EAP.rel,3), "\n")

    s2 <- Sys.time()

    res <- list(deviance=dev, ic=ic, item=item,
                person=person, EAP.rel=EAP.rel,
                mean.trait=mu, sd.trait=sqrt(diag(Sigma)),
                Sigma=Sigma, cor.trait=stats::cov2cor(Sigma),
                b=b, se.b=se.b, a=a, se.a=se.a,
                c=c, se.c=se.c, d=d, se.d=se.d,
                mu.i=mu.i, se.mu.i=se.mu.i,
                f.yi.qk=f.yi.qk, f.qk.yi=f.qk.yi, probs=probs,
                n.ik=n.ik,  iter=iter, dat2=dat2, dat2.resp=dat2.resp,
                dat=dat0, I=I, D=D, K=K,  G=1, theta.k=theta.k, pi.k=pi.k,
                irtmodel=irtmodel, Qmatrix=Qmatrix, s1=s1, s2=s2 )
    class(res) <- "smirt"
    return(res)
}

## File Name: linking_haberman_lq_pw_le_hess_delta.R
## File Version: 0.06


linking_haberman_lq_pw_le_hess_delta <- function(par_delta, par_gamma, des, h=1e-4)
{
    I <- des$I
    G <- des$G
    NPD <- length(par_delta)
    hess <- matrix(0, nrow=NPD, ncol=NPD)
    rownames(hess) <- colnames(hess) <- names(par_delta)
    grad_fun <- linking_haberman_lq_pw_le_grad
    for (pp in 1L:NPD){
        par_delta2 <- mgsem_add_increment(x=par_delta, h=h, i1=pp)
        args <- list(par_delta=par_delta2, par_gamma=par_gamma, des=des,h=h)
        opt1 <- do.call(what=grad_fun, args=args)
        args$par_delta <- mgsem_add_increment(x=par_delta, h=-h, i1=pp)
        opt2 <- do.call(what=grad_fun, args=args)
        hess[,pp] <- (colSums(opt1)-colSums(opt2))/(2*h)
    }
    return(hess)
}

## File Name: summary.R2noharm.jackknife.R
## File Version: 1.06

#---------------------------------------------------
# summary of jackknife statistic
summary.R2noharm.jackknife <- function(object,logfile=NULL,...){
    # INPUT:
    # object ... object of class jackknife.R2noharm
    #.........................
    if ( ! is.null(logfile) ){ sink( paste0(logfile,".Rout"), split=TRUE ) }
    NJ <- length(object$u.jackunits)
    cat("Jackknife NOHARM Model with", NJ, "Jackknife Units\n\n")
    dfr <- object$partable
    dfr[,1] <- gsub( "\\.stat", "", paste( dfr[,1] ))
    dfr1 <- dfr
    dfr1[,3] <- round( dfr1[,3], 4 )
    dfr1[,4] <- round( dfr1[,4], 4 )
    dfr1[,5] <- round( dfr1[,5], 4 )
    print( dfr1 )
    if ( ! is.null(logfile) ){ sink() }
    invisible(dfr)
    }
#----------------------------------------------------

## File Name: ccov_np_print_progress.R
## File Version: 0.03

ccov_np_print_progress <- function(progress, i, ii, display)
{
    if ( i < 20 ){
        if ( ii==display[i] & progress ){
            cat( paste( 5*i, '% ', sep='' ) )
            i <- i + 1
            if (i==11){
                cat('\n' )
            }
            utils::flush.console()
        }
    }
    return(i)
}

## File Name: gom.jml_alg.R
## File Version: 0.15

###############################
# attach objects in the local environment
.sirt.attach.environment <- function( res, envir ){
#    e1 <- environment()
    CC <- length(res)
    for (cc in 1:CC){
        assign( names(res)[cc], res[[cc]], envir=envir )
                    }
            }
#####################################
# Data processing GOM JML
.gom.proc <- function( dat, seed=NULL,K ){
    dat.resp <- 1-is.na(dat)
    dat[ is.na(dat) ] <- 0
    N <- nrow(dat)
    I <- ncol(dat)
    # create starting values for person parameters
    score <- rowSums( dat ) / rowSums( dat.resp )
    theta0 <- seq( -1.5,1.5, len=K )
    g <- exp( - ( outer( stats::qlogis( ( score + .1 ) / 1.2 ), theta0, "-" ) )^2 )
    g <- g / rowSums(g)
    # create starting values item parameters
    p.item <- colSums( dat ) / colSums( dat.resp )
    theta0 <- seq( -2,2, len=K )
    lambda <- t( stats::plogis( outer( theta0, - stats::qlogis( p.item ), "-" ) ) )
    # random start
    if ( ! is.null(seed) ){
        g <- matrix( stats::runif( N*K ), N, K )
        g <- g / rowSums( g)
        lambda <- matrix( stats::runif(I*K), nrow=I, ncol=K)
            }

    res <- list( "dat"=dat, "dat.resp"=dat.resp, "N"=N, "K"=K,
        "lambda"=lambda, "g"=g, "I"=I, "score"=score, "p.item"=p.item )
    return(res)
        }
######################################
# GOM JML update g
.gom.jml.est.g <- function( lambda, g, N, I, K,  dat, dat.resp, min.g){
    #***
    # update g
    # sum lambda
    lambda0.sum <- lambda1.sum <- 0
    for (kk in 1:K){
        # kk <- 1
        lambda1.sum <- lambda1.sum + matrix( lambda[,kk], N, I, byrow=T ) * g[,kk ]
        lambda0.sum <- lambda0.sum + matrix( 1-lambda[,kk], N, I, byrow=T ) * g[,kk ]
                    }
    for (kk in 1:K){
        # kk <- 1
        lambda.kk <- matrix( lambda[,kk], N, I, byrow=T )
        w1.kk <- lambda.kk * g[,kk] / lambda1.sum
        w0.kk <- ( 1 - lambda.kk ) * g[,kk]  / lambda0.sum
        g[,kk] <- rowSums( dat * dat.resp * w1.kk + ( 1- dat ) * dat.resp * w0.kk ) / rowSums( dat.resp )
                }
    g[ g < min.g ] <- min.g
    g[ g > 1 - min.g ] <- 1 - min.g
    g <- g / rowSums(g)
    return( g )
        }
#################################################
# update lambda
.gom.jml.est.lambda <- function( lambda, g, N, I, K, dat, dat.resp, min.lambda ){
    lambda0.sum <- lambda1.sum <- 0
    for (kk in 1:K){
        # kk <- 1
        lambda1.sum <- lambda1.sum + matrix( lambda[,kk], N, I, byrow=T ) * g[,kk ]
        lambda0.sum <- lambda0.sum + matrix( 1-lambda[,kk], N, I, byrow=T ) * g[,kk ]
                    }
    for (kk in 1:K){
        # kk <- 1
        lambda.kk <- matrix( lambda[,kk], N, I, byrow=T )
        w1.kk <- lambda.kk * g[,kk] / lambda1.sum
        w0.kk <- ( 1 - lambda.kk ) * g[,kk]  / lambda0.sum
        sum1 <- colSums( dat * dat.resp * w1.kk )
        sum0 <- colSums( (1-dat) * dat.resp * w0.kk )
        lambda[,kk] <- sum1 / ( sum1 + sum0 )
                }
    lambda[ lambda < min.lambda ] <- min.lambda
    lambda[ lambda > 1 - min.lambda ] <- 1 - min.lambda
    return(lambda)
        }
###############################################
# calculation of deviance
.gom.deviance <- function( lambda, g, K, dat, dat.resp ){
    prob0 <- prob1 <- 0
    for (kk in 1:K){
        # kk <- 1
        prob1 <- prob1 + outer( g[,kk], lambda[,kk] )
        prob0 <- prob0 + outer( g[,kk], 1-lambda[,kk] )
                }
    dev <- -2*sum( log( dat*dat.resp*prob1 + (1-dat)*dat.resp*prob0 ) )
    return(dev)
            }
##################################################
# Discrteizing Membership Function
.gom.discret.membership <- function(g,K,score){
    g1 <- g
    b1 <- seq(0,1,len=5)
    for (kk in 1:K){    g1[,kk] <- cut( g[,kk], breaks=b1) }
    pattern <- apply( g1, 1, FUN=function(kk){ paste( kk, collapse="-") } )
    gcut.distr <- matrix(0,K, 2*(length(b1)-1) )
    for (kk in 1:K){
        for (ii in 1:(length(b1)-1) ){
            gcut.distr[kk,ii] <- sum( g1[,kk]==ii )
            gcut.distr[kk,ii+4] <- mean( score[ g1[,kk]==ii ] )
                    }
            }
    gcut.distr[,1:4] <- 100*gcut.distr[,1:4] / rowSums( gcut.distr[,1:4] )
    gcut.distr <- data.frame(gcut.distr)
    rownames(gcut.distr) <- paste0("Class",1:K)
    colnames(gcut.distr) <- c( paste0("PrGr", seq(0,75,len=4) )    ,
                    paste0("MGr", seq(0,75,len=4) )     )
    gcut.distr$cor.score <- NA
    for (kk in 1:K){
        gcut.distr$cor.score[kk] <- cor( g[,kk], score )
            }
    g1 <- data.frame(g1)
    g1$pattern <- pattern
    g.mean <- colMeans(g)
    res <- list("g1"=g1, "gcut.distr"=gcut.distr,  "g.mean"=g.mean )
    return(res)
                }

## File Name: rm_proc_fixed_values_reference_rater.R
## File Version: 0.09

rm_proc_fixed_values_reference_rater <- function( rater.index1, b.rater.fixed, a.rater.fixed, rater_item_int,
        reference_rater, est.b.rater, est.a.rater )
{
    if ( ! is.null(reference_rater) ){
        #----- no item-rater-interactions
        if ( ! rater_item_int ){
            RR <- nrow(rater.index1)
            #** b parameters
            if (est.b.rater){
                if ( is.null(b.rater.fixed) ){
                    val <- 0
                    b.rater.fixed <- rep(NA, RR)
                    b.rater.fixed[ rater.index1[ paste(rater.index1$rater)==reference_rater, "rater.id" ] ] <- val
                }
            }
            #** a parameters
            if (est.a.rater){
                if ( is.null(a.rater.fixed) ){
                    val <- 1
                    a.rater.fixed <- rep(NA, RR)
                    a.rater.fixed[ rater.index1[ paste(rater.index1$rater)==reference_rater, "rater.id" ] ] <- val
                }
            }

        }
        #---- item-rater-interactions
        if ( rater_item_int ){
            RR <- nrow(rater.index1)
            ind <- match( reference_rater, paste(rater.index1$rater) )
            #** b parameters
            if (est.b.rater){
                val <- 0
                if ( is.null(b.rater.fixed) ){
                    b.rater.fixed <- rep(NA, RR)
                    b.rater.fixed[ ind ] <- val
                }
            }
            #** a parameters
            if (est.a.rater){
                if ( is.null(a.rater.fixed) ){
                    val <- 1
                    a.rater.fixed <- rep(NA, RR)
                    a.rater.fixed[ ind ] <- val
                }
            }
        }
    }
    #--- output
    res <- list( a.rater.fixed=a.rater.fixed, b.rater.fixed=b.rater.fixed )
    return(res)
}

## File Name: data.prep.R
## File Version: 1.153

#----- data preparations for rasch.jml and rasch.mml
data.prep <- function( dat, weights=NULL, use.freqpatt=TRUE,
        standardize_weights=TRUE)
{
    item.means <- colMeans( dat, na.rm=TRUE )
    item.elim <- which( item.means %in% c(0,1))
    if ( length( item.elim ) > 0 ){
        stop( cat( paste( 'There are', length(item.elim), 'Items with no variance!') ) )
    }
    if ( any( is.na(item.means)) ){
        stop( 'There are items which contains only missings!')
    }
    n <- nrow(dat)
    I <- ncol(dat)
    if( is.null(weights) ){  weights <- rep( 1, n ) }
    # indicator for nonmissing response
    dat.9 <- dat
    dat.9[ is.na(dat) ] <- 9

    #* pattern
    if ( use.freqpatt ){
        freq.patt <- apply( dat.9, 1, FUN=function(ll){ paste(ll, collapse='' ) } )  #
        dat1 <- data.frame( table( freq.patt ) )
    } else {
        freq.patt <- paste('FP', 1000000 + 1L:n, sep='')
        dat1 <- data.frame( freq.patt )
        colnames(dat1)[1] <- 'freq.patt'
    }
    # weighting the frequencies if survey weights are supplied
    if( !is.null(weights) ){
        # standardize weights
        if (standardize_weights){
            weights <- weights / sum(weights) * n
        }
        if ( use.freqpatt ){
            dat1[,2] <- stats::aggregate( weights, list( freq.patt), sum )[,2]
        } else {
            dat1[,'Freq'] <- weights
        }
    }
    # item pattern corresponding to frequency pattern
    if ( use.freqpatt){
        dat2 <- matrix( as.numeric( unlist( strsplit( paste(dat1[,1]), '' ) ) ),
                            ncol=ncol(dat), byrow=TRUE)
    } else {
        dat2 <- dat.9 }
        dat2.resp <- 1*( dat2 !=9 )
        dat2[ dat2==9 ] <- 0
        # mean right
        dat1$mean <- rowSums( dat2 * dat2.resp )  / rowSums( dat2.resp )
        freq.patt <- data.frame(  freq.patt, rowMeans( dat, na.rm=TRUE ), 1L:n )
        colnames(freq.patt)[2L:3] <- c('mean', 'index' )
        list( dat=dat, dat2=dat2, dat2.resp=dat2.resp, dat1=dat1,
                freq.patt=freq.patt, I=I, n=n, dat9=dat.9 )
}


.data.prep <- data.prep



#*** Small function which helps for printing purposes
.prnum <- function( matr, digits )
{
    VV <- ncol(matr)
    for (vv in 1L:VV){
        if ( is.numeric( matr[,vv]) ){
            matr[,vv] <- round( matr[,vv], digits )
        }
    }
    print(matr)
}


#-- Function for calculation of a response pattern
#   for dichotomous responses
resp.pattern2 <- function(x)
{
    n <- nrow(x)
    p <- ncol(x)
    mdp <- (x %*% (2^((1L:ncol(x)) - 1))) + 1
    misspattern <- mdp[,1]
    misspattern <- list( miss.pattern=mdp[,1],
                        mp.index=match( mdp[,1], sort( unique(mdp[,1] ) ) ) )
    return(misspattern)
}

## File Name: summary.modelfit.sirt.R
## File Version: 1.07


##############################################################
# summary modelfit.sirt
summary.modelfit.sirt <- function( object, ... )
{
    cat("Test of Global Model Fit\n")
    sirt_summary_print_objects(obji=object$modelfit.test, digits=5, from=2)

    cat("\nFit Statistics\n")
    sirt_summary_print_objects(obji=object$modelfit.stat, digits=5, from=1,
                rownames_null=FALSE)
}
#################################################################

## File Name: rasch.pml3.R
## File Version: 6.131



#####################################################
# Pairwise marginal likelihood (PML) estimation
##NS export(rasch.pml)
rasch.pml3 <- function( dat, est.b=seq( 1, ncol(dat) ),
            est.a=rep( 0, ncol(dat) ),
            est.sigma=TRUE,
            itemcluster=NULL,
#            Q=NULL,
#            zero.corrs=NULL,
            weight=rep(1,nrow(dat)),
            numdiff.parm=.001, b.init=NULL,
            a.init=NULL, sigma.init=NULL,
            error.corr=0*diag( 1, ncol(dat) ),
            err.constraintM=NULL, err.constraintV=NULL,
            glob.conv=10^(-6), conv1=10^(-4), pmliter=300,
            progress=TRUE, use.maxincrement=TRUE ){
    ##################################
    # multidimensional version does not work
    Q <- NULL ; combs <- zero.corrs <- NULL
    V <- err.constraintM
    ##################################
    # load libraries
# a0 <- Sys.time()
    # extract information from data
    link <- "probit"
    s1 <- Sys.time()
    I <- ncol(dat)
    dat <- as.matrix(dat)
    if( is.null( colnames(dat)) ){ colnames(dat) <- paste("I", 1:I, sep="") }
    dat <- dat[ rowMeans( is.na( dat) ) < 1, ]
    dat0 <- dat
    dat[ is.na(dat) ] <- 9
    N <- nrow(dat)
        if ( progress  ){
        cat("---------------------------------------------------------------------------------------------------------- \n")
        cat("Pairwise Marginal Likelihood Estimation \n")
        cat(paste( "Raschtype Model with", link, "link" ), "\n")
        cat("---------------------------------------------------------------------------------------------------------- \n")
        flush.console()
      }
    # extract frequencies from items
    p1 <- t(( dat==1 )) %*% ( dat==1 )
    itemfreq <- data.frame( "item"=1:I, "p1"=diag(p1)  )
    # extract frequencies from item pairs
    itempairs <- as.data.frame( t( combn(1:I, 2 ) ) )
    colnames(itempairs) <- c("item1", "item2")
    itempairs <- itempairs[ order( 1000 * itempairs[,2]  + itempairs[,1]  ), ]
    # create weight matrix
    weight <- nrow(dat) * weight / sum(weight)
#    weight <- rep(1,nrow(dat))
    # WM <- sqrt( outer( weight, rep(1,I) ) )
    WM <- matrix( sqrt(weight), length(weight), I )
    WM1 <- ( dat==1 )*WM
    WM0 <- ( dat==0 )*WM
    # p11
    p11 <- t(WM1) %*% (WM1)
    itempairs$f11 <- p11[ upper.tri(p11) ]
    # p10
    p10 <- t(WM1) %*% (WM0)
    itempairs$f10 <- p10[ upper.tri(p10) ]
    # p01
    p10 <- t(WM0) %*% (WM1)
    itempairs$f01 <- p10[ upper.tri(p10) ]
    # p01
    p10 <- t(WM0) %*% (WM0)
    itempairs$f00 <- p10[ upper.tri(p10) ]
    # error correlations
    est.eps.corr <- itempairs$est.eps.corr <- error.corr[ upper.tri( error.corr ) ]
    est.corrs <- FALSE
    if ( any( itempairs$est.eps.corr !=0 ) ){ est.corrs <- TRUE }
    if ( link=="logit" ){ est.corrs <- FALSE }
    eps.corr <- 0.2 * ( itempairs$est.eps.corr !=0 )

    #*********
    # exclude some item pairs from calculation because they are
    # located in the same itemclusters (local dependence)
    # if error calculations are estimated, then no itemclusters
    # can be selected
    if ( sum( error.corr) > 0 ){ itemcluster <- NULL }

    itemclusters <- unique( itemcluster[ itemcluster !=0 ] )
    IC <- length( itemclusters)
    for ( cc in itemclusters ){
    #    cc <- 1
        icc <- which( itemcluster==cc )
        elim <- intersect( which( itempairs[, "item1" ] %in% icc ), which( itempairs[, "item2" ] %in% icc ) )
        if ( length(elim) > 0 ){ itempairs <- itempairs[ - elim, ] }
                }
    if ( ! is.null( itemcluster ) ){
        eps.corr <- rep(0,nrow(itempairs ))
                            }
    #*******
    # evaluate pairwise likelihood
    if ( is.null(b.init)){
        b <- b0 <- - stats::qnorm( colMeans( dat0, na.rm=T) )
     if ( sum( est.b !=    seq( 1, ncol(dat) ))>0 ){
        b <- 0*b
                }
            } #    if ( link=="logit"){
#        b <- b0 <- - qlogis( colMeans( dat0, na.rm=T) )
#        p.ki <- c( .25220, .58522, .16257 )
#        S.ki <- c( .90793, .57778, .36403 )
#                }
    if ( sum(est.a) >0 ){
                a <- rep(.5,I)
                    } else { a <- rep(1,I) }
    a1s <- a1b <- a1a <- 0
    if ( ! is.null( b.init) ){ b <- b0 <- b.init }
    if ( ! is.null( a.init) ){ a <- a0 <- a.init }
    if ( is.null(sigma.init)){ sigma <- 1 } else { sigma <- sigma.init }
    #******
    # mutidimensional model
#    if ( ! is.null(Q) ){
#        D <- ncol(Q)
#        sigma <- diag(D)
#        combs <- t( combn( D, 2 ) )
#        combs <- rbind( combs, cbind(1:D, 1:D) )
#        combs <- combs[ order(  combs[,1]*1000+combs[,2] ), ]
#        if ( ! is.null(zero.corrs)){
#            for (ii in seq(1, nrow(zero.corrs)) ){
#                ind <- which( ( combs[,1]==zero.corrs[ii,1] ) & ( combs[,2]==zero.corrs[ii,2] ) )
#                combs <- combs[-ind, ]
#                                    }
#                            }
#                } else {
                D <- 1
#                }

    IP <- nrow(itempairs )
    itempairs$p1.item2 <- itempairs$p1.item1 <- rep(0,IP)
    itempairs$p11 <- rep(0,IP)
    #******************
    # design matrices for faster calculation
    #*******************
    # design matrix for item difficulties b
    est.b_unique <- unique( est.b[ est.b !=0 ] )
    IB <- length( est.b_unique)
    desb00 <- desb10 <- desb01 <- desb11 <- matrix( 0, nrow=IP, ncol=IB )
    b.items <- match( est.b, est.b_unique )
    for (bb in 1:IB){    # ii <- 1
        ii.bb <- which( est.b==est.b_unique[bb] )
        desb00[ itempairs$item1 %in% ii.bb | itempairs$item2 %in% ii.bb, bb ] <- 1
        desb01[ itempairs$item2 %in% ii.bb & ! ( itempairs$item1 %in% ii.bb ), bb ] <- 1
        desb10[ itempairs$item1 %in% ii.bb & ! ( itempairs$item2 %in% ii.bb ), bb ] <- 1
        desb11[ itempairs$item1 %in% ii.bb & itempairs$item2 %in% ii.bb, bb ] <- 1
                }
    #*******************
    # design matrix for item slopes a
    if ( sum(est.a) > 0 ){
        est.a_unique <- unique( est.a[ est.a !=0 ] )
        IA <- length( est.a_unique)
        desa00 <- desa10 <- desa01 <- desa11 <- matrix( 0, nrow=IP, ncol=IA )
        a.items <- match( est.a, est.a_unique )
        for (bb in 1:IA){    # ii <- 1
            ii.bb <- which( est.a==est.a_unique[bb] )
            desa00[ itempairs$item1 %in% ii.bb | itempairs$item2 %in% ii.bb, bb ] <- 1
            desa01[ itempairs$item2 %in% ii.bb & ! ( itempairs$item1 %in% ii.bb ), bb ] <- 1
            desa10[ itempairs$item1 %in% ii.bb & ! ( itempairs$item2 %in% ii.bb ), bb ] <- 1
            desa11[ itempairs$item1 %in% ii.bb & itempairs$item2 %in% ii.bb, bb ] <- 1
                    }
                }
    #************************
    # design matrix for correlations
    epsG <- setdiff( unique( est.eps.corr ), 0 )
    if ( length(epsG) > 0 ){
        epsG_unique <- epsG
        IG <- length( epsG_unique)
        deseps00 <- deseps10 <- deseps01 <- deseps11 <- matrix( 0, nrow=IP, ncol=IG )
        eps.items <- match( est.eps.corr, epsG_unique )
        for (bb in 1:IG){    # ii <- 1
            ii.bb <- which( est.eps.corr==epsG_unique[bb] )
            deseps00[ ii.bb, bb ] <- 1
                     }
                }

    max.increment.a <- .3
#    use.maxincrement <- TRUE

    #............................
    iter <- 0
    #**********************************
    # BEGIN MARGINAL MAXIMUM LIKELIHOOD ESTIMATION
    dev <- 1 ; par.change <- dev.change <- 1000
    while ( ( dev.change > glob.conv | par.change > conv1  ) & ( iter < pmliter )    ){
        cat( paste(rep("-", 70), collapse=""), "\n")
# a00 <- Sys.time()
        k1 <- floor( log10(iter+1) )
        x1 <- "        |"
        x1 <- substring( x1, k1+1 )
        s1c <- Sys.time()
        cat( paste( paste( "PML EM Iter.", iter + 1 ), x1, paste( rep( "*", 10  ), collapse=""), "|  ",
                        s1c, "  ",
                        if ( iter > 0 ){ paste( round(difftime(s1c,s1b, units='secs' ),4), "secs" ) },
                        "\n",sep="") ) #
        s1b <- Sys.time()
        h <- numdiff.parm
        dev0 <- dev
        #************************************
        # estimation of b parameters
        b0 <- b
        # identify different b parameter groups
        bG <- setdiff( unique( est.b ), 0 )
 #       prbar <- seq( 1, 10, len=length(bG) )
#        prbar <- floor( prbar )
#        prbar <- c( prbar[1], diff(prbar) )
        cat(" Estimation of b:     |")
        respml0 <- .ll.rasch.pml3.probit.est.b( b,a,  sigma, Q,
                            eps.corr,itempairs, IP, eps=10^(-14), h=h,
                            desb00, desb01, desb10, desb11, b.items)
        b <- respml0$b
        itempairs0 <- respml0$itempairs
        dev <- -2*respml0$ll
        a1b <- max( abs( b - b0 ) )
        cat("----------|     max. parm. change", round( a1b, 5),"\n")
        flush.console()
# cat("est.b") ; a11 <- Sys.time(); print(a11-a00) ; a00 <- a11

        #************************************
        # estimation of a parameters

        a0 <- a
        # identify different a parameter groups
        aG <- setdiff( unique( est.a ), 0 )
        if ( sum(est.a) > 0 ){
            cat(" Estimation of a:     |")
            respml0 <- .ll.rasch.pml3.probit.est.a( b, a, sigma, Q,eps.corr, itempairs, IP, eps=10^(-14),
                h, desa00, desa01, desa10, desa11, a.items )
            a <- respml0$a
            dev <- -2*respml0$ll
            if ( use.maxincrement ){
                incr <- a - a0
                incr <- ifelse( abs(incr) > max.increment.a, sign(incr)*max.increment.a, incr )
                a <- a0 + incr
                max.increment.a <- max.increment.a / 1.2
                        }
            a1a <- max( abs( a - a0 ) )
            cat("----------|     max. parm. change", round( a1a, 5),"\n")
            flush.console()
                        }
        ######################################
        # estimation sigma
        sigma0 <- sigma
        cat(" Estimation of sigma: |")
        if (est.sigma & is.null(Q) ){
                  respml0 <- .ll.rasch.pml.probit( b, a, sigma,
                                Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll0 <- respml0$ll
                  respml0 <- .ll.rasch.pml.probit( b, a, sigma + h,
                                Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll1 <- respml0$ll
                  respml0 <- .ll.rasch.pml.probit( b, a, sigma - h,
                            Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll2 <- respml0$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            alpha.change <- - d1 / d2
            a1k2 <- alpha.change <- ifelse( abs( alpha.change ) > .2, .2*sign(alpha.change), alpha.change )
            sigma <- sigma + alpha.change
                                }
            prbar <- 10
            flush.console()
        if (est.sigma & (!is.null(Q)) ){
            for ( zz in seq(1,nrow(combs)) ){
                ii <- combs[zz,1]
                jj <- combs[zz,2]
                sigma.hh <- 0*sigma
                sigma.hh[ii,jj] <- sigma.hh[jj,ii] <- 1
                  respml0 <- .ll.rasch.pml.probit( b, a, sigma,
                                Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll0 <- respml0$ll
                  respml0 <- .ll.rasch.pml.probit( b, a, sigma + h*sigma.hh,
                                Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll1 <- respml0$ll
                  respml0 <- .ll.rasch.pml.probit( b, a, sigma - h*sigma.hh,
                            Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll2 <- respml0$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            alpha.change <- - d1 / d2
            a1k2 <- alpha.change <- ifelse( abs( alpha.change ) > .12, .12*sign(alpha.change), alpha.change )
            alpha.change <- alpha.change*sigma.hh

            sigma <- sigma + alpha.change
                                    }
        diag(sigma)[ diag(sigma) < .001 ] <- .0001
                                }
        cat( paste( rep( "-", prbar), collapse="") )
        a1s <- max( abs( c( sigma - sigma0 )) )
        cat("|     max. parm. change", round( a1s, 5),"\n")
# cat("est.sigma") ; a11 <- Sys.time(); print(a11-a00) ; a00 <- a11

        #************************************
        # estimation of error correlation parameters
        a1e <- 0
        epsG <- NULL
        if ( est.corrs ){
            eps.corr0 <- eps.corr
            # identify different eps parameter groups
            epsG <- setdiff( unique( est.eps.corr ), 0 )
            prbar <- seq( 1, 10, len=length(epsG) )
            prbar <- floor( prbar )
            prbar <- c( prbar[1], diff(prbar) )
            cat(" Estimation of eps:   |")
            respml0 <- .ll.rasch.pml3.probit.est.corr( b, a, sigma, Q,eps.corr, itempairs,
                    IP, eps=10^(-14),h, deseps00, eps.items )
            dev <- -2*respml0$ll
            eps.corr <- respml0$eps.corr
            # linear constraints: Let e be the vector of error
            # correlations, V a design matrix and c a vector.
            # The constraints can be written in the form
            # c=V * e . Then V*e - c=0.
            # See the Neuhaus paper:
            # e_cons=e + V'* (V'V)^(-1) * ( c - V * e )
            if ( ! is.null( err.constraintM ) ){
                V1 <- solve( t(V) %*% V )
                e1 <- matrix( eps.corr, ncol=1 )
                eps.corr2 <- eps.corr + V %*% V1 %*% ( err.constraintV - t(V) %*% e1 )
#                itempairs$eps.corr <- eps.corr2
                eps.corr <- eps.corr2
                                        }
            a1e <- max( abs( eps.corr - eps.corr0 ) )
            cat("---------|     max. parm. change", round( a1e, 5),"\n")
                        }
        ######################################
        # convergence display
#        a1 <- aggregate( b, list( est.b), mean )
#        a1aa <- aggregate( a, list( est.a), mean )
#        cat("   b parameters: ", paste( round( a1[,2], 3 ), collapse=" " ), "\n" )
#        cat("   a parameters: ", paste( round( a1aa[,2], 3 ), collapse=" " ), "\n" )
#        if ( D==1){
#            cat("   sigma parameter:  ", paste( round( sigma, 3 ), collapse=" " ), "\n" )
#                }
        if (D>1){
            cat("   sigma parameters:  ", paste( round( sigma[ ! lower.tri(sigma) ], 3 ), collapse=" " ), "\n" )
                    }
#        if (est.corrs){
#            a1 <- aggregate( eps.corr, list( est.eps.corr), mean )
#            cat("   eps parameters: ", paste( round( a1[,2], 3 ), collapse=" " ), "\n" )
#                    }
        #******************************************************************************
        iter <- iter + 1
#        dev <- -2*ll0
        dev.change <- abs( ( dev - dev0)/ dev0 )
#        par.change <- max( a1a, a1b, a1d, a1k, a1m, a1s)
        par.change <- max( a1b, a1s, a1e, a1a )
        cat( "Pseudolikelihood=",   round( dev, 5 ), "| max. parm. change=",
                                        round( par.change, 6 ),  " \n"   )
        if ( ( dev > dev0 ) & ( iter > 4 ) ){ cat("   Objective Function has increased! Convergence Problems?\n") }
        flush.console()
                }
        #********************************************************
    # information criteria
        # calculations for information criteria
        ic <- list( "deviance"=dev, "n"=nrow(dat0) )
        # number of parameters to be estimated
        # these formulas hold when assuming normal distributions
        ic[[ "np" ]] <- length(bG) + est.sigma + length( epsG ) + length(aG)
        if ( ! is.null(err.constraintM) ){
                ic$np <- ic$np - ncol( err.constraintM )
                                }
        # AIC
   #     ic$AIC <- dev + 2*ic$np
        # BIC
        ic$PLIC <- dev + ( log(ic$n) )*ic$np
        # CAIC
     #   ic$CAIC <- dev + ( log(ic$n) + 1 )*ic$np
    #**********************************************************************************
    # results item parameters
    item <- data.frame( "item"=colnames(dat0),
                "N"=colSums(!is.na(dat0)),
#                "sumWeights"=colSums( ( !is.na(dat0)) * WM ),
                "p"=colMeans( dat0, na.rm=T ),
                "b"=b, "est.b"=est.b,
                "a"=a, "est.a"=est.a ) #, "aG"=aG, "bG"=bG)
    if (D==1){ item$sigma <- sigma }
    item$est.sigma=1*est.sigma
#    item$link"=link
    if ( ! is.null( itemcluster) ){ item$itemcluster <- itemcluster }
    item$b.logit <- item$b * 1.701
    item$a.logit <- item$a*1.701
    if (D==1){    item$sigma.logit <- item$sigma * 1.701 }
    # add results dependency parameter for item clusters
    #item$itemcluster <- itemcluster
    #item$delta <- 0

    cat("---------------------------------------------------------------------------------------------------------- \n")
    # print item summary
    cat("Item Parameter Summary\n")
    cat( " Estimated", length(bG), "Item Parameters\n\n")
    sirt_print_helper( item, digits=3 )        # print item statistics
    cat("---------------------------------------------------------------------------------------------------------- \n")

    #....................................................................
    # print Trait parameter summary
#    cat("Trait Distribution Summary\n")
#    cat("\nCovariance Matrix (Probit Link)\n")
    cat("Trait SD (Probit Link): ")
    cat( round(sigma, 3 ), "\n")
    cat("Trait SD (Logit Link) : ")
    cat( round( item$sigma[1] * 1.701, 3 ), "\n")
    if (D>1){
        cat("\nCorrelation Matrix\n")
        print( cov2cor(sigma), digits=3 )
            }
   cat("---------------------------------------------------------------------------------------------------------- \n")
    # print correlation parameter summary
    error.corr0 <- error.corr
    if ( est.corrs){
        cat("Residual Correlation Parameter Summary\n")
        cat( " Estimated", length(epsG), "Residual Correlation Parameters\n\n")
        diag(error.corr0) <- 0
        colnames(error.corr) <- rownames(error.corr) <- colnames(dat)
        error.corr[ upper.tri( error.corr) ] <- eps.corr
        error.corr[ lower.tri( error.corr) ] <- 0
        error.corr <- error.corr + t(error.corr)
#        error.corr[ lower.tri( error.corr) ] <- eps.corr
        diag(error.corr) <- 1
       sirt_print_helper( error.corr, digits=3 )        # print item statistics
       cat("---------------------------------------------------------------------------------------------------------- \n")
            }
        # computational time
        s2 <- Sys.time()
        if (progress){
                cat("---------------------------------------------------------------------------------------------------------- \n")
                cat("Start:", paste( s1), "\n")
                cat("End:", paste(s2), "\n")
                cat("Difference:", print(s2 -s1), "\n")
                cat("---------------------------------------------------------------------------------------------------------- \n")
                    }
    res <- list( "item"=item, "iter"=iter, "deviance"=dev,
                    "b"=b, "sigma"=sigma,
                    "dat"=dat    ,  "ic"=ic    ,
                    "link"=link, "itempairs"=itempairs0,
                    "error.corr"=error.corr0         ,
                    "eps.corr"=eps.corr, # NEW!!!
                    "bG"=bG, "aG"=aG, "epsG"=epsG,
                    "est.b"=est.b, "est.a"=est.a, "est.corrs"=est.corrs,
                    "Q"=Q, "D"=D )
    if (est.corrs){
            res$eps.corr <- eps.corr
            res$eps.corrM <- error.corr
                }
    #***
    # reliabiity

    thresh <- - matrix( item$a * item$b, I, 1 )
    A <- matrix( item$a * item$sigma, I, 1 )
    # extract estimated correlation matrix
#    corM <- mod1$eps.corrM
    if (is.null(res$eps.corrM) ){
        corM <- diag(I)
            } else {
        corM <- res$eps.corrM
                }
    # compute standardized factor loadings
    facA <- 1 / sqrt( A^2 + 1 )
    resvar <- 1 - facA^2
    covM <- outer( sqrt(resvar[,1]), sqrt(resvar[,1] ) ) * corM
    facloadings <- A *facA
    # estimate reliability
    rel1 <- reliability.nonlinearSEM( facloadings=facloadings,
            thresh=thresh, resid.cov=covM)
    res$omega.rel <- rel1$omega.rel


    res$fct <- "rasch.pml3"
    res$s1 <- s1 ; res$s2 <- s2
    class(res) <- "rasch.pml"
    return(res)
               }
##########################################################################

## File Name: rm_numdiff_discrete_differences.R
## File Version: 0.08

rm_numdiff_discrete_differences <- function(ll0, ll1, ll2, h)
{
    # first derivative
    # f(x+h)-f(x-h)=2*f'(x)*h
    d1 <- ( ll1 - ll2  ) / ( 2 * h )
    # second order derivative
    # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
    d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
    #--- output
    res <- list(d1=d1, d2=d2)
    return(res)
}

## File Name: noharm_sirt_create_parameter_matrices.R
## File Version: 0.041

noharm_sirt_create_parameter_matrices <- function(mat_label, parm_table, parm_index)
{
    nrow <- parm_index[[mat_label]]$nrow
    ncol <- parm_index[[mat_label]]$ncol
    mat <- matrix(0, nrow=nrow, ncol=ncol)
    x1 <- parm_table[ parm_index[[mat_label]]$row_parm_table, 'est']
    mat[ as.matrix(parm_index[[mat_label]]$entries) ] <- x1
    return(mat)
}

## File Name: xxirt_EAP.R
## File Version: 0.117


#--- compute EAP and its standard deviation
xxirt_EAP <- function(p.aj.xi, Theta )
{
    D <- ncol(Theta)
    e1 <- p.aj.xi %*% Theta
    colnames(e1) <- paste0('EAP.Dim', 1L:D)
    e2 <- p.aj.xi %*% Theta^2
    colnames(e2) <- paste0('SD.EAP.Dim', 1L:D)
    e2 <- e2 - e1^2
    res <- cbind( e1, e2)
    res <- res[, rep( c(0,D), D ) + 1L:D ]
    return(res)
}

## File Name: sirt_summary_print_objects.R
## File Version: 0.21

sirt_summary_print_objects <- function(obji, from=NULL, to=NULL,
        digits=3, rownames_null=TRUE, grep_string=NULL)
{
    #------ data frame or matrix
    if ( is.data.frame(obji) | is.matrix(obji)){
        include_names <- FALSE
        if ( is.null(colnames(obji) ) ){
            include_names <- TRUE
            colnames(obji) <- paste0('V',seq_len(ncol(obji)) )
        }
        if ( ! is.null(grep_string) ){
            obji <- obji[, grep( grep_string, colnames(obji)) ]
        }
        if (is.null(from)){
            from <- 1
        }
        if (is.null(to)){
            to <- ncol(obji)
        }
        if ( length(digits)==1){
            rvars <- seq( from, to)
            names1 <- colnames(obji)[rvars]
            if (is.null(names1)){
                names1 <- rvars
            }
            digits <- sirt_vector_with_names(value=digits, names=names1 )
        }
        if ( length(digits) > 1){
            rvars <- names(digits)
        }
        for (vv in rvars ){
            obji[,vv] <- sirt_round_vector(x=obji[,vv], digits=digits[vv])
        }
        if (rownames_null){
            rownames(obji) <- NULL
        }
        if (include_names){
            colnames(obji) <- NULL
        }
    }
    #------ vector
    if ( is.vector(obji) ){
        obji <- sirt_round_vector( x=obji, digits=digits )
    }
    #--- print object
    print(obji)
}

## File Name: locpolycor_est_thresh_grad_fun.R
## File Version: 0.141


locpolycor_est_thresh_grad_fun <- function(x, y, x1, w, x0, model, K, eps=1e-10)
{
    if (model=='const'){
        par1 <- c(-Inf,x,Inf)
        pred1 <- par1[y+2]
        pred2 <- par1[y+1]
    }
    if (model=='lin'){
        par1 <- c(-Inf,x[1L:K],Inf)
        par2 <- c(-Inf,x[K+(1L:K)],Inf)
        pred1 <- par1[y+2] + par2[y+2]*(x1-x0)
        pred1 <- ifelse(y==K, par1[y+2], pred1)
        pred2 <- par1[y+1] + par2[y+1]*(x1-x0)
        pred2 <- ifelse(y==0, par2[y+1], pred2)
    }
    arg <- stats::pnorm(pred1) - stats::pnorm(pred2)
    dpred1 <- stats::dnorm(pred1)
    dpred2 <- stats::dnorm(pred2)
    if (model=='lin'){
        lpred1 <- stats::dnorm(pred1)*(x1-x0)
        lpred2 <- stats::dnorm(pred2)*(x1-x0)
    }

    arg <- ifelse( arg < eps, eps, arg )
    NP <- length(x)
    grad <- rep(NA, NP)
    for (pp in 1L:NP){
        # P(Y=0)=Phi(tau1) - 0
        # P(Y=1)=P(Y<=1) - P(Y=0)=Phi(tau2) - Phi(tau1), ...
        arg1 <- dpred1
        arg2 <- dpred2
        uu <- pp
        if (model=='lin' & (pp>(NP/2) )){
            arg1 <- lpred1
            arg2 <- lpred2
            uu <- pp - NP/2
        }
        # d log(arg) / d par=d par / arg
        a1 <- ifelse(y==uu-1, arg1, 0)
        a2 <- ifelse(y==uu, -arg2, 0)
        ad <- a1+a2
        grad[pp] <- - sum(w*ad/arg)
    }
    return(grad)
}

## File Name: mirt.wrapper.coef.R
## File Version: 3.154

mirt.wrapper.coef <- function(mirt.obj)
{
    TAM::require_namespace_msg("mirt")
    tmod <- mirt::mod2values(mirt.obj)
    items <- colnames(mirt.obj@Data$data)
    I <- length(items)
    itempars <- NULL
    pars <- list()
    for (ii in 1:I) {
        tmod_ii <- tmod[ tmod$item==items[ii], ]
        pars[[ii]] <- tmod_ii
        itempars <- unique(c(itempars, tmod_ii$name ))
        itempars <- sort(itempars)
        IP <- length(itempars)
    }
    #-- single group case
    if (inherits(mirt.obj,"SingleGroupClass")){
        dfr <- matrix(NA, nrow=I, ncol=IP)
        colnames(dfr) <- itempars
        dfr <- as.data.frame(dfr)
        dfr <- data.frame(item=items, dfr)
        rownames(dfr) <- items
        for (ii in 1:I){
            pars_ii <- pars[[ii]]
            cii <- pars_ii$value
            names(cii) <- pars_ii$name
            dfr[ii, names(cii)] <- cii
        }
        rownames(dfr) <- NULL
        tmod_group <- tmod[ tmod$item=="GROUP", ]
        h2 <- tmod_group$value
        names(h2) <- tmod_group$name
        res <- list(coef=dfr, GroupPars=h2, G=1)
    }
    #-- multiple group case
    if (inherits(mirt.obj,"MultipleGroupClass") ) {
        groups <- sort(unique(tmod$group))
        G <- length(groups)
        dfr0 <- NULL

        dfr <- matrix(NA, nrow=1, ncol=IP)
        colnames(dfr) <- itempars
        dfr <- as.data.frame(dfr)
        dfr0 <- NULL
        for (ii in 1:I){
            for (group in groups){
                pars_ii <- pars[[ii]]
                pars_ii <- pars_ii[ pars_ii$group==group, ]
                cii <- pars_ii$value
                names(cii) <- pars_ii$name
                if (length(cii)>0){
                    dfr1 <- data.frame(group=group, item=items[ii], dfr )
                    dfr1[1, names(cii)] <- cii
                    dfr0 <- rbind(dfr0, dfr1)
                }
            }
        }
        dfr <- dfr0
        dfr <- dfr[ order(dfr$item), ]
        rownames(dfr) <- NULL

        tmod2 <- tmod[ tmod$item=="GROUP", ]
        h1 <- list()
        for (group in groups){
            tmod_group <- tmod2[ tmod2$group==group, ]
            h2 <- tmod_group$value
            names(h2) <- tmod_group$name
            h1[[group]] <- h2
        }
        res <- list(coef=dfr, GroupPars=h1, G=G, groups=groups)
    }
    res$class <- class(mirt.obj)
    return(res)
}

## File Name: isop_tests_cpp.R
## File Version: 0.06


isop_tests_cpp <- function( dat, dat.resp, weights, jackunits, JJ )
{
    res <- isop_tests_C( dat=dat, dat_resp=dat.resp, weights=weights,
                jackunits=jackunits, JJ=JJ )
    return(res)
}

## File Name: R2conquest.R
## File Version: 1.41



#******************************************************************************************************************
# Function R2Conquest for Rasch Model
##NS export(rasch.conquest)
R2conquest <- function( dat, path.conquest, conquest.name="console",
                            converge=0.001, deviancechange=0.0001,
                            iter=800, nodes=20,
                            minnode=-6, maxnode=6, show.conquestoutput=FALSE,
                            name="rasch",   pid=1:(nrow(dat) ),
                            wgt=NULL, X=NULL, set.constraints=NULL,
                            model="item", regression=NULL, itemcodes=seq( 0, max(dat,na.rm=TRUE) ),
                            constraints=NULL, digits=5, onlysyntax=FALSE, qmatrix=NULL,
                            import.regression=NULL, anchor.regression=NULL, anchor.covariance=NULL,
                            pv=TRUE, designmatrix=NULL,
                            only.calibration=FALSE, init_parameters=NULL, n_plausible=10,
                            persons.elim=TRUE, est.wle=TRUE, save.bat=TRUE, use.bat=FALSE,
                            read.output=TRUE, ignore.pid=FALSE
                                ){
        # INPUT:
        # pv ... estimation of plausible values?
#        pv <- TRUE
        s1 <- Sys.time()
        if ( is.null(wgt) ){ wgt <- 1 + 0 * pid }
        if (onlysyntax==FALSE){
            cat("---------------------------------------------------------------------------------------------------------- \n")
            cat("Marginal Maximum Likelihood Estimation \n")
            cat("Estimation of the Rasch Model in ConQuest\n")
            cat("---------------------------------------------------------------------------------------------------------- \n")
            utils::flush.console()
                }
        # shQuote() for path.conquest
        path.conquest <- shQuote( path.conquest )
        # pv estimation?
        if (only.calibration){ pv <- FALSE }

        # calculate score
        score1 <- rowMeans( dat, na.rm=T )
        # character vector for items in constraints matrix
        if (! is.null( constraints) ){
                constraints[,1] <- paste( constraints[,1] )
                constraints <- constraints[ constraints[,1] %in% colnames(dat), ]
                                }
        # data preparation (eliminate items)
        items.elim <- which( colSums( 1 - is.na(dat) )==0 )
        # initial warnings (no responses on some items)
        if ( length(items.elim) > 0 & is.null(constraints) ){
                    cat( "\nSome items have been removed because they have no responses!\n" )
                    utils::flush.console()
                    dat <- dat[, - items.elim ]
                    }
        # data preparation (eliminate persons)
        if ( ! is.null(X) ){ persons.elim <- FALSE }

        if ( persons.elim ){
            persons.elim <- which( rowSums( 1 - is.na(dat) )==0 )
            if ( length(persons.elim) > 0 ){
                        cat( "\nSome persons have been removed because they have no responses!\n" )
                        utils::flush.console()
                        dat <- dat[ - persons.elim,  ]
                        pid <- pid[ - persons.elim ]
                        wgt <- wgt[ - persons.elim ]
                        }
                }
        # number of items
        I <- ncol(dat)
        p1 <- path.conquest
        # remove files
        h1 <- c( "analysis.bat", "rasch.cqc",  "rasch.nam", "rasch.shw", "rasch.itn", "rasch.wle", "rasch.des",
                        "rasch.par", "rasch.constrpar", "rasch.regr", "rasch.cov", "rasch.pv", "rasch.dat", "rasch.log",
                        "rasch.designmatrix"    )
        h1 <- gsub( "rasch", name, h1 )
        f1 <- getwd()
        h1 <- intersect( list.files( f1 ), h1 )
        file.remove(h1 )
        # generate label file for ConQuest
        writeLines( c( "===> item", paste( 1:I, colnames(dat) ) ),
                        paste( name,".nam",sep="") )


        # which items are being constrained
        if ( ! is.null( constraints) ){
            constraints$itemnr <- match( constraints[,1], colnames(dat) )
            constraints <- constraints[ order( constraints$itemnr), ]
            c1 <- paste( constraints[,3],  constraints[,2],       paste(  " /* ", constraints[,1], "*/" ) )
            utils::write.table( c1,  paste(  name,".constrpar",sep=""), quote=F, row.names=F, col.names=F )
                    }
        # extract decimals for person ID
        PP <- round( max( floor( log(  pid, 10 ) + 1 ) ) )
        # extract decimals for weights
        WW <- round( max( floor( log( wgt, 10 )) + 1 ) + digits )
        # do covariates exist? If, then add constant column
        if (! is.null(X)){
            X <- data.frame( "one"=1, X )
            XNcols <- apply( abs( X ), 2, FUN=function(ll){ floor(max( log( ll, 10 ) )+1)+1 } )
            rdcols <- digits * ( 1 - colMeans( X==round(X) ) )
            XNcols <- XNcols + rdcols + 1
            # possible correction
            XNcols <- ifelse( floor(XNcols ) !=XNcols, floor(XNcols)+1, XNcols )

            # write data
            dat2 <- data.frame( pid, wgt,  X, dat )
            i1 <- write.fwf2( dat=dat2, format.full=c( PP, WW, XNcols,  rep(1,I)),
                   format.round=c( 0, digits, rdcols, rep(1,I) ), savename=name )
                        }
        if (is.null(X)){
            dat2 <- data.frame( pid, wgt,   dat )
            i1 <- write.fwf2( dat=dat2,
                    format.full=c( PP, WW,   rep(1,I)),
                    format.round=c( 0, digits,  rep(1,I) ),
                    savename=name )
                    }
        # statement data input
        state1 <-   paste0( "format " )
        if ( ! ignore.pid ){
            state1 <- paste0( state1,     "pid ", 1, "-", PP )
                            }
            state1 <- paste0( state1, " wgt ", PP+1, "-", PP+WW,
                            " responses ", PP+1+WW, "-", PP + I + WW,";" )

        if ( ! is.null(X) ){
            i1a <- i1[ seq( 1, 2 + ncol(X) ), ]
            state2 <- paste( paste( i1a$variable, " ", i1a$begin, "-", i1a$end, sep="" ), collapse=" " )
            state1 <- paste( "format ", state2, paste( "responses ", i1[ 3 + ncol(X), "begin" ], "-", i1[ nrow(i1), "end" ], ";", sep=""), sep=" " )
                }

        # input designmatrix
        if ( ! is.null(designmatrix) ){
            utils::write.table( designmatrix, paste( name, ".des", sep=""), col.names=F, row.names=F, quote=F )
            desformat <- readLines(  paste( name, ".des", sep="")  )
            desformat <- gsub( "0", "-0", desformat )
            desformat <- c( ncol(designmatrix), desformat )
            writeLines( desformat, paste( name, ".des", sep="") )
                                    }
        # regression statement
        if (is.null(X)){ regrstate <- "" } else {
                    regrstate <- paste( "regression ", paste( colnames(X)[-1], collapse=" " ), " ;", sep="") }
        if ( ! is.null(regression) ){ regrstate <- paste( "regression ", regression, " ;", sep="") }
        # constraints statement
        constrstate <- ""
        if( ! is.null(constraints)  ){ constrstate <-  "set constraints=none;" } else
            {   if( is.null(constraints) & is.null(X) ) {
                    constrstate <- "set constraints=cases;" } else
                    { constrstate <- "set constraints=items;" }
            }
        if ( ! is.null( set.constraints )){
                constrstate <- paste( "set constraints=", set.constraints, ";", sep="")
                        }
        # generate score statement
        if (! is.null(qmatrix)){
#                q1 <- qmatrix[  match( colnames(dat), qmatrix[,1]), ]
                q1 <- qmatrix
                DD <- ncol(q1)
                h1 <- paste( "score (", paste( itemcodes, collapse=" " ), ")", sep="" )
                for (dd in 1:DD){
                    # dd <- 1
                    h1.dd <- outer( q1[,dd], itemcodes,  FUN=function(x,y){ x * y } )
                    h1.dd <- apply( h1.dd, 1, FUN=function(ll){ paste( ll, collapse=" " ) } )
                    h1 <- paste( h1, "(", h1.dd, ")", sep="")
                        }
                scorestate <- paste( h1, " !items(", 1:I, ");   /* ", q1[,1], " */", sep="" )
                    } else {
                                DD <- 1
                                scorestate <- NULL
                                }


        # change number of nodes in monte carlo integration
        if (DD > 2 & nodes < 100 ){ nodes <- 2500 }
        # generate ConQuest Syntax for Rasch model
        cqc <- c(
                paste( "title ", name, " ", Sys.time(), " ;",  sep=""),
                paste( "let name=", name, ";", sep=""),
                "datafile %name%.dat ;",
                state1,
                "caseweight wgt;",
                paste( "codes ", paste( itemcodes,collapse=","),";",sep=""),
                ifelse( is.null(designmatrix), "labels << %name%.nam;", "" ),          "",
                scorestate, "",
                # initial item parameters
                ifelse( ! is.null(init_parameters), paste( "import init_parameters << ", init_parameters, " ; ", sep=""), ""),
                # import designmatrix
                ifelse( is.null(designmatrix), "", "import designmatrix << %name%.des ;" ),
                # anchor item parameters
                ifelse( is.null(constraints), "", "import anchor_parameters << %name%.constrpar ;" ),
                ifelse( is.null(import.regression), "", paste( "import init_reg_coefficients << ", import.regression, " ;", sep="") ),
                ifelse( is.null(anchor.regression), "", paste( "import anchor_reg_coefficients << ", anchor.regression, " ;", sep="") ),
                ifelse( is.null(anchor.covariance), "", paste( "import anchor_covariance << ", anchor.covariance, " ;", sep="") ),
                constrstate,
                paste( "model ", model, ";",sep=""),
                regrstate,
                paste( "estimate !iter=", iter, ", nodes=", nodes,
                    ", minnode=", minnode,  ", maxnode=", maxnode,
                    ifelse( DD > 2, ", method=montecarlo ", "" ),
                    ", converge=", gsub( " ", "", formatC(converge,8)),
                    ", deviancechange=", gsub( " ", "", formatC(deviancechange,8)), "; ",  sep="" ),
                "export parameters              >> %name%.par;",
                "export reg_coefficients        >> %name%.regr;",
                "export covariance              >> %name%.cov;",
                "export logfile                 >> %name%.log;",
                "export designmatrix >> %name%.designmatrix ;",
                ifelse( ( ! only.calibration ) &  est.wle, "show cases ! estimates=wle >> %name%.wle ;", "" ),
                ifelse( pv, paste( "set n_plausible=", n_plausible, ";",sep=""), "" ),
                ifelse( pv, "show cases !estimates=latent >> %name%.pv;", "" ),
                "show >> %name%.shw ;",
                ifelse(!only.calibration, "itanal >> %name%.itn;",""),
                "quit;"               )
        # write ConQuest syntax
        writeLines( cqc, paste( name, ".cqc", sep="") )
        # write bat file
        if ( save.bat ){
            writeLines( paste( path.conquest, "\\", conquest.name, " ",
                    paste( name, ".cqc", sep=""), sep=""), "analysis.bat" )
                        }
        if ( onlysyntax ){
                cat( paste( "Conquest Input Syntaxes are in ", getwd(),  "\n", sep="") )
                res <- NULL
                        }
                else {
        # link to conquest console
        if ( use.bat ){
#            system( "analysis.bat", show.output.on.console=show.conquestoutput, invisible=FALSE )
            system( "analysis.bat"  )
                    } else {
        system(paste(path.conquest,"\\", conquest.name, ".exe ", name,".cqc",sep=""),
                    show.output.on.console=show.conquestoutput, invisible=FALSE)
                            }




        #######################################################
        # READING CONQUEST OUTPUT
        if (read.output){
            if ( ( ! only.calibration ) & est.wle  ){
                # read WLEs and add pid
                wle <- utils::read.table( paste( name, ".wle", sep="")  )
                v1 <- c("score", "max", "wle", "se.wle" )
                if (DD > 1 ){
                    v1 <- rep( v1, each=DD )
                    v1 <- paste( rep( paste( "dim", 1:DD, sep=""), 4 ), v1, sep="")
                            }
                if ( ignore.pid ){
                    colnames(wle) <- c("case",  v1 )
                    wle <- data.frame( "case"=wle[,"case"], "pid"=wle[,"case"],
                                wle[,-1] )
                                } else {
                    colnames(wle) <- c("case", "pid", v1 )
                                }
                wle <- data.frame(  wle )

                        } else { wle <- NA }

            # read shw file
            shw <- readLines( paste( name, ".shw", sep="") )
            deviance <- as.numeric( substring( shw[ grep( "Final Deviance:", shw ) ], 17 ) )
            numbiter <- as.numeric( substring( shw[ grep( "The number of iterations:", shw ) ], 26 ) )
            ind1 <- grep("TERM 1: item", shw ) + 6
            ind2 <- grep("An asterisk next to a", shw )[1] - 2

            # extract trait mean and trait variance
#            m3 <- read.table( paste( name, ".regr", sep=""), header=F)
            m3 <- utils::read.table( paste( name, ".regr", sep="") )
            mean.trait <- m3[ 1, 3]
            trait.variance <- as.numeric( substring( shw[ grep( "Variance ", shw ) ], 25 ) )
            wle.rel <-  as.numeric( substring( shw[ grep( " WLE Person separation RELIABILITY:", shw ) ], 37 ) )
            itemdiff <- as.numeric( substring( shw[ ind1:ind2 ], 20, 27 ) )

            # read pv file
            if ( pv  ) {
                    if (DD==1){
                        pv1 <- .read.pv( pvfile=paste( name, ".pv", sep=""), npv=n_plausible )
                                } else {
                                    pv1 <- .read.multidimpv( pvfile=paste( name, ".pv", sep=""), ndim=DD,
                                                npv=n_plausible  )
                                        }
                if ( mean( is.na( pv1$pid ) )==1 ){
                            pv1$pid <- pv1$case
                                    }
                    pv1 <- pv1[, - grep( "case", colnames(pv1) ) ]
                    wle <- merge( x=wle, y=pv1, by="pid", all=T  )
                                 }




            #*****
            # reliability estimation
            reliability <- NULL
            if ( ! only.calibration & DD==1){
                # WLE reliability
                reliability$wle.reliability <- 1 - mean( wle$se.wle^2 ) / stats::var( wle$wle )
                # EAP reliability
                reliability$eap.reliability <- 1 - mean( wle$se.eap^2 ) / ( stats::var( wle$eap ) +  mean( wle$se.eap^2 ) )
                                    }
            # calculate expected probability
            if (DD==1 & pv ){
                thetagrid <- seq( -10, 10, .01 )
                mean.trait <- mean( pv1$PV1)
                sd.trait <- stats::sd( pv1$PV1 )
                dens.thetagrid <- sirt_dnorm( thetagrid, mean=mean.trait, sd=sd.trait )
                dens.thetagrid <- dens.thetagrid / sum( dens.thetagrid )
                p.exp <- sapply( itemdiff, FUN=function(bii) {
                       stats::weighted.mean( stats::plogis( thetagrid - bii ), dens.thetagrid )
                    } )    }
        if ( DD > 1 |  ( !pv )) { p.exp <- rep(NA,I)  }
        if ( DD==1 & pv  ) {
    #        emp.discrim <- round( item.discrim( dat, wle$wle ),3) }
            emp.discrim <- stats::cor( dat, wle$wle, use="pairwise.complete.obs") }
                    else { emp.discrim <- NA  }


            diffs <- as.numeric( substring( shw[ ind1:ind2 ], 20, 27 ) )
            diffs <- diffs - mean(diffs)

        item <- data.frame( "item"=colnames(dat),
                        "N"=colSums( 1 -is.na(dat), na.rm=T),
                        "p"=round(colMeans( dat, na.rm=T ),3),
                        "p.exp"=p.exp,
                        "itemdiff"=itemdiff,
                        "itemdiff.cent"=diffs,
                        "se.itemdiff"=as.numeric( substring( shw[ ind1:ind2 ], 29, 35 ) ),
                        "emp.discrim"=round(emp.discrim,3),
                        "outfit"=as.numeric( substring( shw[ ind1:ind2 ], 38, 43 ) ),
                        "t.outfit"=as.numeric( substring( shw[ ind1:ind2 ], 58, 62 ) ),
                        "infit"=as.numeric( substring( shw[ ind1:ind2 ], 64, 69 ) ),
                        "t.infit"=as.numeric( substring( shw[ ind1:ind2 ], 84, 88 ) )
                                )
            res <- list( "person"=wle, "item"=item, "deviance"=deviance, "numbiter"=numbiter, "mean.trait"=mean.trait,
                    "sd.trait"=sqrt(trait.variance), "wle.rel"=wle.rel, "itemmean"=mean( item$itemdiff),
                        "reliability"=reliability )
            s2 <- Sys.time()
            res$sys.time <- list( "start"=s1, "end"=s2, "timediff"=s2-s1 )


            if ( DD==1){
                    res$shw.itemparameter <- read.show( showfile=paste( name, ".shw", sep="") )
                    res$shw.regrparameter <- read.show.regression( showfile=paste( name, ".shw", sep="") )
                    res$shw.pimap <- read.pimap( showfile=paste( name, ".shw", sep="") )
                            }
            class(res) <- "rasch.conquest"
                    }
            return(res)
            }
        }
#**************************************************************************************************************************


#...............................................................#
# function for reading conquest files of plausible values
.read.pv <- function( pvfile, npv=5 ){
    pv <- readLines( pvfile )   # read file with plausible values
    nl <- length(pv)            # length of pv file
    nseq <- npv + 3             # number of lines corresponding to one person
    N <- nl/nseq                # number of persons
    # all plausible values
    pv.all <- as.numeric( substring( pv[  - c( nseq * ( 1:N  ) - 1, nseq * ( 1:N  ) - 0, nseq * ( 1:N -1 ) +1  ) ], 11 ) )
    pv1 <- matrix( pv.all, ncol=npv, byrow=T )
    colnames(pv1) <- paste("PV", 1:npv, sep="")

    pid <- as.numeric( substring( pv[ nseq * ( 1:N  ) - npv -2 ], 6, 40 ) )
    pv1 <- data.frame(  "case"=1:N,
                        "pid"=pid,
                        "eap"=as.numeric( pv[ nseq * ( 1:N  ) - 1  ] ),
                        "se.eap"=as.numeric( pv[ nseq * ( 1:N  )  ] ),
                        pv1
                            )
    return(pv1)
    }
#................................................................#



#...............................................................#
# function for reading conquest files of plausible values
.read.multidimpv <- function( pvfile, ndim, npv=5  ){
    # INPUT:
    # pvfile    ... name of file with plausible values
    # ndim      ... number of dimensions for which plausible values were drawn
    pv <- scan( pvfile )
    pv1 <- readLines( pvfile )
    a1 <- pv1[1]
    a1 <- gsub( " ", "", a1 )
    pid.pres <- 1
#    if ( a1=="1" ){ pid.pres <- 0 } else { pid.pres <- 1 }
    nl <- length(pv1)            # length of pv file
    nseq <- npv + 3             # number of lines corresponding to one person
    N <- nl/nseq                # number of persons

    # numbers for each person
    L <- npv * ( ndim + 1 ) + 2 * ndim +  1   + pid.pres
    # arrange matrix
    dfr <- matrix( pv, ncol=L, byrow=T)
    dfr <- cbind( dfr[,1], seq(1,N), dfr[,-1] )
    # which columns should be deleted from dfr

#    dfr <- data.frame(dfr[, - c( 1 + seq( 2, 1 + (ndim+1)*npv, ndim+1 )) ])
    del <- c( 3,  ( 0:(npv-1))*(ndim+1) +1+3 )
    dfr <- data.frame(dfr[, - del ] )
    colnames(dfr)[1:2] <- c( "case", "pid" )
    cdim <- matrix( sapply( 1:npv, FUN=function(pp){ paste( "dim", 1:ndim, "pv", pp, sep="") } ), ncol=1)[,1]
    colnames(dfr)[ seq( 1, length(cdim) ) + 2 ] <- cdim
    cdim <- c( paste( "dim", 1:ndim, "eap", sep=""), paste( "dim", 1:ndim, "seeap", sep="") )
    colnames(dfr)[ seq( 1 + ncol(dfr) - 2* ndim, ncol(dfr) )  ] <- cdim
    return(dfr)
    }
#................................................................#
# R2conquest <- rasch.conquest


read.pv <- .read.pv
read.multidimpv <- .read.multidimpv

#*******************************************************
# Summary for rasch.conquest object                         *
##NS S3method(summary,rasch.conquest)
summary.R2conquest <- function( object, ... ){
    # object      ... object from rasch.conquest                #
    cat("---------------------------------------------------------------------------------------------------------- \n")
    cat("Marginal Maximum Likelihood Estimation (Normal Distribution) \n")
    cat("Estimation of the Rasch Model in ConQuest\n" )
    cat("---------------------------------------------------------------------------------------------------------- \n")
    cat(paste( "Rasch Model with ", nrow(object$item), " Items (Average Number of Items per Person: ",
                 round( mean( object$person$max ), 1 ),  " )\n", sep="") )
    cat( "Deviance=", round( object$deviance, 2 ), "\n" )
    cat( "Trait Distribution: ",
              "Mean=", round(object$mean.trait,3), " SD=", round( object$sd.trait, 3), "  WLE Reliability=", round( object$wle.rel, 3), "\n")
    cat("Mean of Item Difficulty Parameters: ", round( object$itemmean, 3 ), "\n" )
    cat("---------------------------------------------------------------------------------------------------------- \n")
    cat( paste( "WLE Reliability:", round( object$reliability$wle.reliability,3 )), "\n")
    cat( paste( "EAP Reliability:", round( object$reliability$eap.reliability,3 )), "\n")
    cat("---------------------------------------------------------------------------------------------------------- \n")
    cat("Item Parameter \n")
    print( object$item[, c("N", "p", "itemdiff", "emp.discrim", "outfit", "infit" )] )
                }
#*******************************************************
# summary.R2conquest <- summary.rasch.conquest



##   #*******************************************************
##   # plot rasch.conquest object
##   ##NS S3method(plot,rasch.conquest)
##   plot.rasch.conquest <- function( x, plottitle="Person-Item-Map",
##       n.gridpoints=41, itemcex=1, ...){
##       # calculate empirical WLE distribution
##       object <- x
##       trait.grid <- seq( -4, 4,len=n.gridpoints )
##       h <- diff(trait.grid)[1]
##       breaks.grid <- c( - 1000,  ( trait.grid + h / 2 )[ - n.gridpoints ], 1000 )
##       freq <- hist( object$person$wle, breaks=breaks.grid, plot=F)
##       object$trait.distr <- cbind( trait.grid, freq$counts / sum( freq$counts)  )
##       object$fixed.a <- 1 + 0 * 1:(nrow(object$item))
##   #    class(object) <- "rasch.mml"
##       plot.rasch.mml( object, plottitle=plottitle, itemcex=itemcex)
##       }
##   #******************************************************




#-----------------------------------------------------------------------------------------------------------#
# Read parameter for one term from a ConQuest output shw file                                                     #
##NS export(read.show.term)
read.show.term <- function( showfile, term ){
        shw <- readLines( showfile )
        ind <- intersect( grep( "TERM", shw ), grep( term, shw ) )[1]
        i1 <- ind+6
        i2 <- grep( "--------------", shw )
        i2 <- i2[ i2 > i1 ][1] - 1
        table1 <- shw[i1:i2]
        h1 <- regexpr( "ESTIMAT", shw[i1-2] )[1]
        h2 <- h1 - 21
        dfr1 <- data.frame(     "term"=shw[ind],
                        "parameter"=substring( table1, 6, h1- 1 ),
                        "parameter2"=gsub( " ", "", substring( table1, 6, h1- 1 ) ),
                        "itemdiff"=as.numeric( substring( table1, 21+h2,27+h2 ) ),
                        "constrained"=1* ( substring( table1, 28+h2,28+h2 )=="*" ),
                        "se.itemdiff"=as.numeric( substring( table1, first=31+h2, last=35+h2 )),
                                "outfit"=as.numeric( substring( table1, first=39+h2, last=43+h2 )),
                                "t.outfit"=as.numeric( substring( table1, first=58+h2, last=62+h2 )),
                                "infit"=as.numeric( substring( table1, first=65+h2, last=69+h2 ) ),
                                "t.infit"=as.numeric( substring( table1, first=84+h2, last=88+h2 ) )
                                        )
        p2 <- paste(dfr1$parameter)
        p2 <- unlist( lapply( strsplit( p2, split=" " ), FUN=function(ll){
                    paste( ll[ ll !="" ], collapse="-") } ) )
        dfr1$parameter2 <- p2
        return(dfr1)
            }
#-----------------------------------------------------------------------------------------------------------#



#.............................................................................
# read all terms from a ConQuest shw file
##NS export(read.show)
read.show <- function( showfile){
    shw <- readLines( showfile )
    # liste alle verschiedenen auftretenden Terme auf
    terms.shw <- shw[ grep( "TERM", shw ) ]
    terms.shw <- strsplit( shw[ grep( "TERM", shw ) ], split=": ")
    terms.shw <- unlist( lapply( terms.shw, FUN=function(ll){ paste(ll[1],sep=":") } ) )
    dfr <- NULL
    for (tt in terms.shw ){
        dfr1 <- read.show.term( showfile=showfile, term=tt )
        dfr <- rbind( dfr, dfr1 )
            }
    return(dfr)
    }
#.............................................................................






#......................................................................................
# read regression coefficients and residual variance from a ConQuest output showfile
##NS export(read.show.regression)
read.show.regression <- function( showfile ){
    shw <- readLines( showfile )
    # Regression Coefficients
    i1 <- grep("Regression Variable", shw )+2
    i2 <- grep("--------------------------", shw )
    i2 <- i2[ i2>i1][1] - 1
    dfr1 <- data.frame( "parameter"=gsub( " ", "", substring( shw[i1:i2], 1, 25 ) ),
                "estimate"=as.numeric( substring( shw[i1:i2], 26, 32 ) ),
                "se"=as.numeric( substring( shw[i1:i2], 35, 40 ) )
                    )
    # Variance
    i1 <- grep("Variance", shw )
    dfr2 <- data.frame( "parameter"="Variance",
                "estimate"=as.numeric( substring( shw[i1], 26, 33 ) ),
                "se"=NA
                    )
    dfr <- rbind( dfr1, dfr2 )
    dfr$par.index <- seq( 1, nrow(dfr) )
    return(dfr)
        }
#......................................................................................




#---------------------------------------------------------------------------
# read person-item map from ConQuest output shw file
##NS export(read.pimap)
read.pimap <- function( showfile ){
    shw <- readLines( showfile )
    ind1 <- grep( "MAP OF LATENT ", shw )[1] + 5
    ind2 <- grep( "===========================", shw )
    ind2 <- ( ind2[ ind2 > ind1 ] )[1] - 1
    ZZ <- ind2 - ind1
    dfr <- data.frame( matrix(NA, nrow=ZZ, ncol=6 ) )
    colnames(dfr) <- c( "row", "first.bar", "second.bar",
                "ylab",  "X", "items" )
    dfr$row <- seq( 1, ZZ )
    for (zz in 1:ZZ){
        # zz <- 25
        shw.zz <- unlist( strsplit( shw[ind1+zz], split="") )
        a1 <- grep( "\\|", shw.zz )
        dfr$first.bar[zz] <- a1[1]
        dfr$second.bar[zz] <- a1[2]
        dfr$X[zz] <- sum( shw.zz[ seq(1,a1[1]) ]=="X" )
        items.zz <- paste( shw.zz[ seq(a1[1]+1,a1[2]-1) ], collapse="" )
        items.zz <- unlist( strsplit( items.zz, split=" " ) )
        items.zz <- items.zz[ items.zz !="" ]
        dfr$items[zz] <- paste( items.zz, collapse="-" )
        dfr$ylab[zz] <- as.numeric(shw.zz[4])
            }
    return(dfr)
        }
#---------------------------------------------------------------------------

## File Name: rm_proc_data.R
## File Version: 0.663


#--- Data preprocessing rater models
rm_proc_data <- function( dat, pid, rater, rater_item_int=FALSE, reference_rater=NULL )
{
    #--- define reference rater
    if ( is.null(reference_rater) ){
        reference_rater0 <- sort( paste( rater ))[1]
    }
    I0 <- ncol(dat)
    #--- item-specific rater parameters and rearrange dataset
    if (rater_item_int){
        if (is.null(reference_rater)){
            reference_rater <- reference_rater0
        }
        res <- rm_proc_create_pseudoraters( dat=dat, rater=rater, pid=pid,
                    reference_rater=reference_rater )
        dat <- res$dat
        rater <- res$rater
        pid <- res$pid
        reference_rater <- res$reference_rater
    }

    #-- create rater indices
    rater <- paste(rater)
    # create table of rater indizes
    rater.index <- data.frame( rater=sort( unique( rater )) )
    rater.index$rater.id <- seq( 1, nrow(rater.index) )
    RR <- nrow(rater.index)

    # create table of person indizes
    person.index <- data.frame( pid=sort( unique( pid )) )
    person.index$person.id <- seq( 1, nrow(person.index) )
    PP <- nrow( person.index )

    # number of variables
    VV <- ncol(dat)
    vars <- colnames(dat)
    # create data frame with crossed items and raters
    dat2 <- data.frame( matrix( NA, nrow=PP, ncol=RR*VV ) )
    colnames_dat2 <- colnames(dat2) <- paste0( rep(vars, RR ), "-",
                rep( rater.index$rater, each=VV) )
    rownames(dat2) <- person.index$pid

    # create expanded dataset
    rater0 <- match( rater, rater.index$rater) - 1
    pid0 <- match( pid, person.index$pid) - 1
    dat2 <- sirt_rcpp_rm_proc_expand_dataset(dat=as.matrix(dat), rater0=rater0,
                    pid0=pid0, N=PP, R=RR)
    colnames(dat2) <- colnames_dat2
    dat20 <- dat2

    # variable list
    dataproc.vars <- list( item.index=rep( 1L:VV, RR ),
                            rater.index=rep(1L:RR, each=VV ) )

    # arrange response data
    N <- nrow(dat2)
    K <- max(dat2, na.rm=TRUE)
    I <- ncol(dat2)
    res <- sirt_rcpp_rm_proc_datasets_na_indicators(dat=as.matrix(dat2), K=K )
    dat2 <- res$dat2
    dat2.resp <- res$dat_resp
    colnames(dat2) <- colnames_dat2
    dat2.ind.resp <- array( res$dat2_ind_resp, dim=c(N,I,K+1) )

    #--- output
    res <- list( dat2=dat2, dat2.resp=dat2.resp, dat2.NA=dat20, dat=dat,
                person.index=person.index, rater.index=rater.index, VV=VV, N=PP,
                RR=RR, dataproc.vars=dataproc.vars, dat2.ind.resp=dat2.ind.resp,
                rater=rater, pid=pid, dat=dat, reference_rater=reference_rater, I0=I0)
    return(res)
}


rm_proc <- rm_proc_data
.prep.data.rm <- rm_proc

## File Name: dexppow.R
## File Version: 0.054


dexppow <- function (x, mu=0, sigmap=1, pow=2, log=FALSE)
{
    p <- pow
    cost <- 2 * p^(1/p) * gamma(1 + 1/p) * sigmap
    expon1 <- (abs(x - mu))^p
    expon2 <- p * sigmap^p
    dsty <- (1/cost) * exp(-expon1/expon2)
    if (log){
        dsty <- log(dsty)
    }
    return(dsty)
}

## File Name: lsem_bootstrap_print_start.R
## File Version: 0.02

lsem_bootstrap_print_start <- function(verbose)
{
    if (verbose){
        cat('Compute LSEM for bootstrap samples:\n')
        utils::flush.console()
    }
}

## File Name: mgsem_moments_derivative_parameter.R
## File Version: 0.395

mgsem_moments_derivative_parameter <- function(est, est_add=NULL, type,
        i1, i2, h, is_B, eps=1e-12, num_approx=FALSE)
{

    calc_Sigma <- TRUE
    calc_Mu <- TRUE
    symm <- FALSE
    I <- nrow(est$LAM)
    log0 <- Sigma_der_logical <- matrix(FALSE, nrow=I, ncol=I)

    if (type %in% c('PHI','PSI')){
        symm <- TRUE
        calc_Mu <- FALSE
    }
    if (type %in% c('NU','ALPHA')){
        calc_Sigma <- FALSE
    }


    if (is_B){
        num_approx <- TRUE
    }
    if (type %in% c('LAM','PHI')){
        num_approx <- FALSE
    }

    # leave always numerical approximation because est is a function of
    # different components

    num_approx <- TRUE

    if (type %in% c('ALPHA','NU','PSI') ) {
        num_approx <- FALSE
    }


    # issue is for LAM and PHI
    if (type %in% c('PHI') ) {
        if (i1==i2){
            num_approx <- FALSE
        }
    }

    if (type %in% c('LAM') ) {
        # num_approx <- FALSE
    }


    #*** numerical approximation
    if (num_approx){
        est1 <- est
        est1[[type]] <- mgsem_add_increment(x=est[[type]],h=h,i1=i1,i2=i2, symm=symm)
        est2 <- est
        est2[[type]] <- mgsem_add_increment(x=est[[type]],h=-h,i1=i1,i2=i2, symm=symm)

        ## here: est_group=est_between + est_within_group
        # add to est_add!

        # est3 <- mgsem_add_list_entries(list1=est1, add_list=est2, elements=type)

        #* compute implied moments
        # implied <- mgsem_compute_model_implied_moments(est=est)
        implied1 <- mgsem_compute_model_implied_moments(est=est1, is_B=is_B,
                        calc_Mu=calc_Mu, calc_Sigma=calc_Sigma)
        implied2 <- mgsem_compute_model_implied_moments(est=est2, is_B=is_B,
                        calc_Mu=calc_Mu, calc_Sigma=calc_Sigma)

        # compute numerical derivatives with respect to Mu and Sigma
        if (calc_Mu){
            Mu_der <- (implied1$Mu-implied2$Mu)/(2*h)
        } else {
            Mu_der <- matrix(0, nrow=I, ncol=1)
        }
        if (calc_Sigma){
            Sigma_der <- (implied1$Sigma-implied2$Sigma)/(2*h)
            if (type %in% c('B','PHI')){
                Sigma_der_logical <- mgsem_differ_from_zero(x=Sigma_der, eps=eps)
            }
            if (type %in% c('PSI')){
                Sigma_der_logical[i1,i2] <- TRUE
                Sigma_der_logical[i2,i1] <- TRUE
            }
            if (type %in% c('LAM')){
                Sigma_der_logical[i1,] <- TRUE
                Sigma_der_logical[,i1] <- TRUE
            }
        } else {
            Sigma_der <- matrix(0, nrow=I, ncol=I)
        }


    } else {
    ## this part must be cleaned!!
        I <- nrow(est$LAM)
        mat0 <- matrix(0, nrow=I, ncol=I)
        Mu_der <- matrix(0, nrow=I, ncol=1)
        Sigma_der <- mat0
        Sigma_der_logical <- log0

        if (type=='ALPHA'){
            Mu_der <- est$LAM[,i1,drop=FALSE]
        }
        if (type=='NU'){
            Mu_der[i1,1] <- 1
        }

        if (type=='PSI'){
            Sigma_der <- mgsem_add_increment(x=mat0,h=1,i1=i1,i2=i2, symm=symm)
            Sigma_der_logical[i1,i2] <- TRUE
            Sigma_der_logical[i2,i1] <- TRUE
        }

        if (type=='LAM'){
            Mu_der[i1,1] <- est$ALPHA[i2,1]
            # D1 <- matrix(0, nrow=I, ncol=ncol(est$LAM))
            # D1[i1,i2] <- 1
            # LP <- est$LAM %*% est$PHI
            # Sigma_der <- D1 %*% est$PHI %*% t(est$LAM) + est$LAM %*% est$PHI %*% t(D1)
            # Sigma_der <- D1 %*% t(LP) + LP %*% t(D1)
            # G1 <- D1 %*% t(LP)
            # G1 <- tcrossprod(D1, LP)
            # Sigma_der <- G1 + t(G1)

            H1 <- est$LAM %*% est$PHI
            H2 <- t(H1)
            ONE_H1 <- matrix(0, nrow=ncol(est$LAM), ncol=nrow(est$LAM))
            ONE_H2 <- matrix(0, nrow=nrow(est$LAM), ncol=ncol(est$LAM))
            ONE_H1[i2,i1] <- 1
            ONE_H2[i1,i2] <- 1
            t1 <- H1 %*% ONE_H1
            t2 <- ONE_H2 %*% H2
            Sigma_der <- t1 + t2

            Sigma_der_logical[i1,] <- TRUE
            Sigma_der_logical[,i1] <- TRUE
        }
        if (type=='PHI'){
            # D <- nrow(est$PHI)
            # D1 <- matrix(0, nrow=D, ncol=D)
            # D1[i1,i2] <- 1
            # D1[i2,i1] <- 1
            # Sigma_der <- est$LAM %*% D1 %*% t(est$LAM)
            # Sigma_der <- est$LAM %*% tcrossprod(D1, est$LAM)
            # H1 <- D1 + t(D1) - D1*t(D1)
            # H1 <- D1 + t(D1) - D1*t(D1)
            #    Sigma_der <- est$LAM %*% H1 %*% t(est$LAM)
            l1 <- est$LAM[,i1]
            l2 <- est$LAM[,i2]
            A1 <- outer(l1,l2)
            if (i1==i2){
                Sigma_der <- A1
            }
            if (i1!=i2){
                Sigma_der <- A1+outer(l2,l1)
            }
            Sigma_der_logical <- mgsem_differ_from_zero(x=Sigma_der, eps=eps)
        }
    }

    #--- output
    res <- list(Mu_der=Mu_der, Sigma_der=Sigma_der, Sigma_der_logical=Sigma_der_logical,
                    calc_Sigma=calc_Sigma, calc_Mu=calc_Mu)
    return(res)
}

## File Name: mcmcirt_compute_mh.R
## File Version: 0.04


mcmcirt_compute_mh <- function(ll_old, ll_new)
{
    NL <- length(ll_new)
    mh <- ll_new - ll_old
    prob_mh <- ifelse(mh<0, exp(mh), 1)
    accept <- stats::runif(NL) < prob_mh
    #--- output
    ll_recent <- ifelse(accept, ll_new, ll_old)
    res <- list(accept=accept, ll_recent=ll_recent)
    return(res)
}

## File Name: mgsem_proc_model_extract_dimension.R
## File Version: 0.09


mgsem_proc_model_extract_dimension <- function(model, entry="est", type, nrow=TRUE)
{
    G <- length(model)
    gg <- 1
    while (gg<=G){
        mat <- model[[gg]][[entry]][[type]]
        if (!is.null(mat)){
            val <- ifelse(nrow, nrow(mat), ncol(mat) )
            gg <- G+1
        }
        gg <- gg + 1
    }
    return(val)
}

## File Name: mgsem_evaluate_penalties_evaluate_entry_fun_eval.R
## File Version: 0.10


mgsem_evaluate_penalties_evaluate_entry_fun_eval <- function(x, fun_eval,
        args_eval, h, deriv=FALSE)
{
    if (deriv){
        args_eval$x <- x+h
        val1 <- do.call(what=fun_eval, args=args_eval )
        args_eval$x <- x-h
        val2 <- do.call(what=fun_eval, args=args_eval )
        val <- (val1-val2)/(2*h)
    } else {
        args_eval$x <- x
        val <- do.call(what=fun_eval, args=args_eval )
    }
    return(val)
}

## File Name: pbivnorm2.R
## File Version: 1.166


#*** approximation of the bivariate normal integral
#*** using the method of Cox and Wermuth (1991)
pbivnorm2 <- function( x, y, rho )
{
    #  (X,Y) ~ N_2(0, rho)
    #  P( X > a1, Y > b1, rho )
    #=P( -X < - a1, -Y < -b1, rho )
    # if ( any( rho < 0 )){stop("only positive correlations are allowed!")}
    a <- x
    b <- y
    a11 <- a1 <- - a
    b1 <- - b
    # APPROXIMATION for negative correlations
    ind.neg <- which( rho < 0 )
    if ( length(ind.neg) > 0 ){
        rho[ind.neg] <- - rho[ind.neg]
        b1[ind.neg] <- -b1[ind.neg]
    }
    # APPROX. (ii)
    ind2 <- which( a1 < 0 & b1 < 0 )
    if ( length(ind2) > 0 ){
        a1[ ind2 ] <- - a1[ind2]
        b1[ ind2 ] <- - b1[ind2]
    }
    # APPROX. (i):
    # a > 0 and b > 0=> a > b
    ind <- which( a1 < b1 )
    t1 <- b1
    if ( length(ind) > 0 ){
        b1[ ind ] <- a1[ind]
        a1[ind] <- t1[ind]
    }
    t1 <- stats::pnorm( - a1 )
    mu <- sirt_dnorm( a1 ) / stats::pnorm( - a1 )
    xi <-  ( rho * mu - b1 ) / sqrt( 1 - rho^2 )
    # see Hong (1999)
    sig2 <- 1 + a1*mu - mu^2
    #  Formula (3)
    # t1 * pnorm( xi )
    # Formula (4): correction in Hong (1999)
    prob1 <- t1 * ( stats::pnorm(  xi ) - 1/2 * rho^2 / ( 1 - rho^2 ) * xi *
                sirt_dnorm( xi ) * sig2  )
    # adjust formula in case of APPROX. (ii)
    if ( length(ind2) > 0 ){
        # CW. Formula in (ii), p. 264
        prob1[ind2] <- 1-stats::pnorm(-a1[ind2])-stats::pnorm(-b1[ind2])+prob1[ind2]
    }
#    # negative correlations
    if ( length(ind.neg) > 0 ){
        prob1[ind.neg] <- stats::pnorm( - a11[ind.neg] ) - prob1[ ind.neg]
    }
    return( prob1 )
}

## File Name: lc.2raters.R
## File Version: 0.25


##############################################################
# latent class model for two exchangeable raters
lc.2raters <- function( data, conv=.001, maxiter=1000, progress=TRUE ){
    #**********************
    data <- stats::na.omit(data)
    # data preparation and creation of a frequency table
    res <- lc2.data.prep(data)
    maxK <- res$maxK
    m1 <- res$m1

    m2.combs <- t( utils::combn( maxK+1, 2 ) )
    N2 <- nrow(m2.combs)
    m2.combs <- rbind( m2.combs, m2.combs[,c(2,1)], cbind( 1:(maxK+1), 1:(maxK+1 ) ) )
    m2.combs <- m2.combs[ order(paste( m2.combs[,1],m2.combs[,2])), ]
    m2 <- cbind( m2.combs, NA )
    m2[, 3 ] <- m1[ m2[,1:2] ]
    rownames(m2) <- paste0( rownames(m1)[ m2[,1] ], "-",  rownames(m1)[ m2[,2] ] )
    pi.k <- colSums( m1 )
    pi.k0 <- pi.k <- pi.k / sum( pi.k )
    TP <- maxK+1
    KK <- nrow(m2)
    # init probabilities
    probs <- matrix( 1/TP, nrow=TP, ncol=TP)
    rownames(probs) <- colnames(probs) <- rownames(m1)
    diag(probs) <- 1
    probs <- probs / rowSums( probs )
    counts <- probs
    par.change <- 1
    iter <- 0
    #********************
    #**** begin iterations
    while ( ( par.change > conv ) & ( iter < maxiter) ){
        probs0 <- probs
        # compute likelihood
        f.yi.qk <- probs[ m2[,1], ] *  probs[ m2[,2], ]
        rownames(f.yi.qk) <- rownames(m2)
        # compute posterior
        f.qk.yi <- matrix( pi.k, nrow=KK, ncol=TP, byrow=TRUE ) * f.yi.qk
        f.qk.yi <- f.qk.yi / rowSums( f.qk.yi )
        # compute expected counts
        nik <- f.qk.yi * m2[,3 ]
        Nk <- colSums( nik )
        pi.k <- Nk / sum( Nk )
        # compute updated probabilities
        for (zz in 1:TP){      # zz <- 2
            counts[zz,] <- colSums( m2[, 3 ] * 1 * ( m2[,1]==zz ) * f.qk.yi )
                    }
        # update probabilities
        probs <- counts / matrix( colSums( counts ), TP, TP, byrow=TRUE )
        #---
        par.change <- max( abs(probs - probs0) )
        iter <- iter + 1
        if (progress){
           cat( paste0("Iteration ", iter, " | Max. probability change=",
                round( par.change, 6 ), "\n") )
            utils::flush.console()
                    }
            } # end iter
    #******* end iterations
    #*******************************
    colnames(probs) <- paste0("Lat", colnames(probs))

    #*** posterior probabilities
    classprob.2raters.like <- f.yi.qk[ m2[,1] <=m2[,2], ]
    classprob.2raters.like <- classprob.2raters.like / rowSums( classprob.2raters.like )

    classprob.2raters.post <- f.yi.qk[ m2[,1] <=m2[,2], ]
    classprob.2raters.post <- classprob.2raters.post / rowSums( classprob.2raters.post )

    classprob.1rater.post <- classprob.1rater.like <- 0*probs

    for ( kk in 1:(maxK+1) ){
        f1 <- colSums( f.yi.qk[ ( m2[,1]==kk ) | ( m2[,2]==kk ),, drop=FALSE] )
        classprob.1rater.like[kk,] <- f1
        classprob.1rater.like[kk,] <- classprob.1rater.like[kk,] / sum( classprob.1rater.like[kk,] )
        classprob.1rater.post[kk,] <- f1 * pi.k
        classprob.1rater.post[kk,] <- classprob.1rater.post[kk,] / sum( classprob.1rater.post[kk,] )
                    }

    # arrange output
    f.yi.qk <- f.yi.qk[ m2[,1] <=m2[,2], ]
    f.qk.yi <- f.qk.yi[ m2[,1] <=m2[,2], ]

    m2 <- as.data.frame(m2)
    colnames(m2) <- c("cat1", "cat2", "AbsFreq" )
    m2$RelFreq <- m2$AbsFreq / sum( m2$AbsFreq )
    s2 <- Sys.time()

    # agreement statistics
    agree.stats <- lc2.agreement( m1 )

    #*** output list
    res <- list( "classprob.1rater.like"=classprob.1rater.like,
        "classprob.1rater.post"=classprob.1rater.post,
        "classprob.2raters.like"=classprob.2raters.like,
        "classprob.2raters.post"=classprob.2raters.post,
        "f.yi.qk"=f.yi.qk, "f.qk.yi"=f.qk.yi, "probs"=probs,
        "pi.k"=pi.k, "pi.k.obs"=pi.k0, "freq.long"=m2, "freq.table"=m1,
        "agree.stats"=agree.stats,
        "data"=data, "N.categ"=maxK+1, "s2"=s2)
    class(res) <- "lc.2raters"
    return(res)
    }
###############################################################
#     S3 summary method for lc.2raters
summary.lc.2raters <- function( object, ... ){

    cat("---------------------------------------------------------------\n")
        d1 <- utils::packageDescription("sirt")
        cat( paste( d1$Package, " ", d1$Version, " (", d1$Date, ")", sep=""), "\n\n" )
        cat( "Date of Analysis:", paste( object$s2 ), "\n" )
    cat("Latent Class Model for Two Exchangeable Raters \n")
    cat("---------------------------------------------------------------\n")
    cat( "Number of persons=", nrow(object$data), "\n" )
    cat( "Number of item categories=", object$N.categ, "\n" )
    cat("\n********************************************\n")
    cat("Symmetrized Frequency Table\n")
    obji <- object$freq.table
    print(obji)
# include agreement statistics here

    cat("\n********************************************\n")
    cat("Rater agreement statistics\n\n")
    cat( "Percentage agreement=", round(object$agree.stats["agree0"],4), "\n" )
    cat( "Percentage agreement (up to 1 category)=", round(object$agree.stats["agree1"],4), "\n" )
    cat( "Cohen's kappa=", round(object$agree.stats["kappa"],4), "\n" )
    cat( "Weighted kappa (linear weights)=", round(object$agree.stats["wtd.kappa.linear"],4), "\n" )
    cat( "Gwet's AC1=", round(object$agree.stats["AC1"],4), "\n" )
    cat( "Aickin's alpha=", round(object$agree.stats["alpha.aickin"],4), "\n" )
    cat("\n********************************************\n")
    cat("Item probabilities given latent classes\n")
    obji <- object$probs
    for (vv in 1:( ncol(obji))){
        obji[,vv] <- round( obji[,vv], 4 )
                    }
    print(obji)
    cat("\n********************************************\n")
    cat("Latent class classification probability for one rater (likelihood estimate)\n")
    obji <- object$classprob.1rater.like
    for (vv in 1:( ncol(obji))){
        obji[,vv] <- round( obji[,vv], 4 )
                    }
    print(obji)
    cat("\n********************************************\n")
    cat("Latent class classification probability for two raters (likelihood estimate)\n")
    obji <- object$classprob.2raters.like
    for (vv in 1:( ncol(obji))){
        obji[,vv] <- round( obji[,vv], 4 )
                    }
    print(obji)
                }
#*******************************************************

## File Name: mcmc.3pno.testlet_output.R
## File Version: 1.21

######################################################
# compute deviance
.mcmc.deviance.3pno.testlet <- function( aM, bM, a.testletM, theta,
        guess, gamma.testlet, testletgroups, dat, dat.resp,
        weights, eps, param)
{
    guessM <- matrix( guess, nrow=nrow(aM), ncol=length(guess) )
    gamma.testletM <- gamma.testlet[, testletgroups ]
    if (param==1){ tau.ni <- aM * theta + gamma.testletM + bM }
    if (param==2){ tau.ni <- aM * theta + aM*gamma.testletM + bM }
    if (param==3){ tau.ni <- aM * theta + a.testletM*gamma.testletM + bM }
    pij <- guessM + ( 1 - guessM )* stats::pnorm( tau.ni )
    llij <- log( dat.resp * ( dat*pij + ( 1-dat )*(1-pij) ) + eps )
    if ( is.null( weights ) ){ deviance <- -2*sum( llij ) }
    if ( ! is.null( weights ) ){
        deviance <- -2*sum( rowSums(llij) * weights )
    }
    return(deviance)
}

######################################################
# subfunction for calculating the DIC
.mcmc.ic.3pno.testlet <- function( a.chain, b.chain,
        a.testlet.chain, N, I,
        theta.chain, c.chain, gamma.testlet.chain, TT,
        testletgroups, dat, dat.resp, weights, eps, param,
        deviance.chain )
{
    aM <- matrix( colMeans( a.chain ), nrow=N, ncol=I, byrow=TRUE )
    bM <- matrix( colMeans( b.chain ), nrow=N, ncol=I, byrow=TRUE )
    if (param==3){
        a.testletM <- matrix( colMeans( a.testlet.chain ), nrow=N, ncol=I, byrow=TRUE )
    }
    theta <- colMeans( theta.chain )
    guess <- colMeans( c.chain )
    gamma.testlet <- colMeans( gamma.testlet.chain )
    gamma.testlet <- matrix( gamma.testlet, nrow=N, ncol=TT+1 )
    Dhat <- .mcmc.deviance.3pno.testlet( aM, bM, a.testletM, theta,
                    guess,    gamma.testlet, testletgroups, dat, dat.resp,
                    weights, eps, param )
    Dbar <- mean( deviance.chain )
    pD <- Dbar - Dhat
    ic <- list( "Dhat"=Dhat, "Dbar"=Dbar, "pD"=pD, "DIC"=Dhat + 2*pD )
    return(ic)
}
#################################################################
# output mcmc.list
.mcmc.list.3pno.testlet <- function( a.chain, b.chain, a.testlet.chain,
        I, deviance.chain, est.slope, c.chain, sigma.chain,
        est.guess, theta.chain, sigma.testlet.chain, TT,
        burnin, SV, save.theta, testletgroups, param )
{
    a <- a.chain
    b <- b.chain
    theta <- theta.chain
    N <- ncol(theta.chain )
    colnames(a) <- paste0("a[", 1:I, "]")
    colnames(b) <- paste0("b[", 1:I, "]")
    mcmcobj <- cbind( deviance.chain, b )
    colnames(mcmcobj)[1] <- "deviance"
    if ( est.slope ){
        mcmcobj <- cbind( mcmcobj, a )
    }
    if ( est.guess ){
        colnames(c.chain) <- paste0("c[", 1:I, "]")
        mcmcobj <- cbind( mcmcobj, c.chain )
    }
    if ( ! est.slope ){
        mcmcobj <- cbind( mcmcobj, sigma.chain )
        colnames(mcmcobj)[ncol(mcmcobj)] <- "sigma"
    }
    if ( param==3 ){
        colnames(a.testlet.chain) <- paste0("a.testlet[", 1:I, "]")
        ind <- which( testletgroups <=TT )
        mcmcobj <- cbind( mcmcobj, a.testlet.chain[,ind]  )
    }
    colnames(theta) <- paste0("theta[", 1:N, "]")
    if ( ( TT>0 ) & (param!=3 ) ){    # save only testlet effects if they are present
        colnames(sigma.testlet.chain) <- paste0("sigma.testlet[", 1:TT, "]")
        mcmcobj <- cbind( mcmcobj, sigma.testlet.chain )
    }

    #**** compute marginal effects
    if (TT>0){
        sigma.testlet.chain <- cbind( sigma.testlet.chain, 0 )
        ind <- which( testletgroups <=TT )
        # intercept parameters
        st2 <- sqrt( 1 + sigma.testlet.chain[, testletgroups[ind] ]^2 )
        b_marg <- b.chain[,ind] / st2
        colnames(b_marg) <- paste0("b_marg[", (1:I)[ind], "]")
        mcmcobj <- cbind( mcmcobj, b_marg  )
        # slope parameters
        a_marg <- a.chain[,ind] / st2
        colnames(a_marg) <- paste0("a_marg[", (1:I)[ind], "]")
        mcmcobj <- cbind( mcmcobj, a_marg  )
    }
    if (save.theta){ mcmcobj <- cbind( mcmcobj, theta ) }
    class(mcmcobj) <- "mcmc"
    attr(mcmcobj, "mcpar") <- c( burnin+1, burnin+SV, 1 )
    mcmcobj <- sirt_import_coda_as.mcmc.list( mcmcobj )
    res <- list( "mcmcobj"=mcmcobj, "theta"=theta )
    return(res)
}
#################################################################
# description
.mcmc.description.3pno.testlet <- function( TT, param, est.guess, est.slope)
{
    if ( est.guess ){ mod <- "3PNO" } else { mod <- "2PNO" }
    if ( (!est.guess)& (!est.slope) ){ mod <- "1PNO" }
    if (TT>0){
        description <- paste0( mod, " Testlet Model (param=", param, ")" )
    }
    if (TT==0){
        description <- paste0( mod," Model"  )
    }
    return(description)
}
##################################################################

####################################################################
# calculate EAP reliability, person parameter EAPs
# and corresponding posterior SDs
.mcmc.person.3pno.testlet <- function( theta.chain, weights, gamma.testlet.chain)
{
    #*** EAP reliability
    N <- ncol(theta.chain )
    v1 <- stats::var( colMeans( theta.chain ) )
    SV <- nrow(theta.chain)
    if ( is.null(weights) ){
        h1 <- ( rowSums( theta.chain^2 ) - ( N * rowMeans( theta.chain ) )^2 ) / N
        h1 <- mean(h1)
        EAP.rel <- v1 / h1
    }
    if ( ! is.null(weights) ){
        w1 <- weights / sum(weights )
        m1 <- colMeans( theta.chain )
        v1 <- sum( m1^2 * w1 ) - ( sum( m1*w1 ) )^2
        wM <- matrix( w1, nrow=nrow(theta.chain), ncol=ncol(theta.chain), byrow=TRUE )
        h1 <- rowSums( wM * theta.chain^2 ) - ( rowSums( wM * theta.chain ) )^2
        h1 <- mean(h1)
        EAP.rel <- v1 / h1
    }

    #*** person parameter estimates
    person <- data.frame( "EAP"=colMeans( theta.chain ),
                          "SD"=colSds(theta.chain) )
    # compute testlet effects
    TT <- ncol(gamma.testlet.chain) / N - 1
    if (TT>0){
        for (tt in 1:TT){
            gtc <- gamma.testlet.chain[, 1:N + (tt-1)*N ]
            person[, paste0("EAP.Testlet",tt)] <- colMeans(gtc)
            person[, paste0("SD.Testlet",tt)] <- colSds( gtc )
        }
    }
    # output
    res <- list( "EAP.rel"=EAP.rel, "person"=person )
    return(res)
}
###############################################################

## File Name: prob_raschtype_genlogis.R
## File Version: 0.13


# probability raschtype models
prob_raschtype_genlogis <- function( theta, b, alpha1, alpha2, fixed.a=1+0*b,
        Qmatrix=NULL, dimensions=NULL )
{
    LT <- length(theta)
    if (is.matrix(theta)){
        LT <- nrow(theta)
    }

    if ( is.null(Qmatrix) ){
        XX <- TAM::tam_outer(x=theta, y=b, op="-")
        XX <- as.vector(XX)
        aM <- sirt_matrix2(x=fixed.a, nrow=LT)
        XX <- aM * XX
    }
    if ( ! is.null(Qmatrix) ){
        XX0 <- tcrossprod( as.matrix(theta), Qmatrix )
        XX <- XX0 - outer( rep(1,nrow(theta)), b )
        XX <- as.vector(XX)
        aM <- sirt_matrix2(x=fixed.a, nrow=LT)
        XX <- aM * XX
    }
    pm <- pgenlogis(x=XX, alpha1=alpha1, alpha2=alpha2 )
    pm <- matrix( pm, ncol=length(b))
    return(pm)
}


.prob.raschtype.genlogis <- prob_raschtype_genlogis

## File Name: noharm_sirt_efa_rotated_solution.R
## File Version: 0.06


noharm_sirt_efa_rotated_solution <- function(res, items, F_dimnames)
{
    L1 <- res$loadings
    I <- nrow(L1)
    D <- ncol(L1)
    m1 <- stats::promax(L1)
    p1 <- matrix( 0, nrow=I, ncol=D)
    for (dd in 1L:D){
        p1[,dd] <- m1$loadings[,dd]
    }
    colnames(p1) <- F_dimnames
    rownames(p1) <- items
    res$promax <- p1
    res$factor.cor <- solve( crossprod( m1$rotmat ) )
    rownames(res$factor.cor) <- colnames(res$factor.cor) <- F_dimnames
    # conversion to THETA parametrization
    h1 <- rowSums(p1^2)
    p2 <- p1 / sqrt( max(1 - h1, 1e-4 ) )
    res$promax.theta <- p2
    #--- output
    return(res)
}

## File Name: mgsem_loglike_data.R
## File Version: 0.03


mgsem_loglike_data <- function(dat, Mu, Sigma)
{
    requireNamespace('mvtnorm')
    ll <- sum(mvtnorm::dmvnorm(dat, mean=Mu, sigma=Sigma, log=TRUE))
    return(ll)
}

## File Name: linking.robust.R
## File Version: 1.263


#*** Robust linking
linking.robust <- function(  itempars )
{
    itempars0 <- itempars
    itempars <- na.omit(itempars)
    pars <- itempars[,2] - itempars[,3]
    items <- paste(itempars[,1])
    names(pars) <- items
    I <- length(pars)
    x <- sort(pars)
    kvec <- seq(1, floor( (I-1)/2  ) )
    KK <- length(kvec)
    se <- meanpars <- rep(NA, KK )
    # define trimming factor
    for (kk in 1L:KK){
        # arrange calculations
        N <- length(x)
        k <- kk
        indkk <- seq( k+1,  N-k,1)
        x0 <- x[ indkk ]
        # compute winsorized mean
        trim.mean <- mean( x0 )
        swk2 <- k * ( x[ k] - trim.mean )^2 + sum( ( x0 - trim.mean )^2 )
                                    + k * ( x[ N - k + 1] - trim.mean )^2
        # standard error
        se.trimmean <- sqrt( swk2 ) / sqrt( (N-2*k) * ( N - 2*k - 1 ) )
        # output
        meanpars[kk] <- trim.mean
        se[kk] <- se.trimmean
    }

    v1 <- paste0('k', 0:KK)
    meanpars <- c( mean(x), meanpars )
    se <- c( stats::sd(x) / sqrt(I), se )
    names(meanpars) <- v1
    names(se) <- v1

    # arrange output
    res1 <- list()
    res1$ind.kopt <- ind.kopt <- which.min( se )
    res1$kopt <- kvec[ ind.kopt ] - 1
    res1$meanpars.kopt <- meanpars[ ind.kopt ]
    res1$se.kopt <- se[ ind.kopt ]
    res1$meanpars <- meanpars
    res1$se <- se
    res1$sd <- stats::sd(x)
    res1$mad <- stats::mad(x)
    res1$k.robust <- c(0,kvec)
    res1$I <- I
    res1$itempars <- itempars0
    class(res1) <- 'linking.robust'
    return(res1)
}

## File Name: mcmc.2pno.ml.R
## File Version: 3.20
##############################################
# MCMC estimation 2PNO model
mcmc.2pno.ml <- function(dat, group,
        link="logit",
        est.b.M="h", est.b.Var="n",
        est.a.M="f", est.a.Var="n",
        burnin=500, iter=1000, N.sampvalues=1000,
        progress.iter=50, prior.sigma2=c(1,.4 ),
        prior.sigma.b=c(1,1), prior.sigma.a=c(1,1),
        prior.omega.b=c(1,1), prior.omega.a=c(1,.4),
        sigma.b.init=.3 )
{
    ##############################
    # INPUT:
    # describe input in help files
    ##############################
    s1 <- Sys.time()
    # data preparation
    dat0 <- dat
    dat <- as.matrix(dat)
    dat[ is.na(dat0) ] <- 0
    dat.resp <- 1-is.na(dat0)
    N <- nrow(dat)
    I <- ncol(dat)
    eps <- 10^(-10)
    #***
    # groups
    groups <- sort( unique(group) )
    idgroup <- as.numeric( match( group, groups ) )
    G <- length(groups)
    groupsize <- as.numeric( rowsum( 1+0*group, idgroup )[,1] )

    #***
    # set initial values
    a <- rep(1,I)
    if (link=="logit"){
        b <- - stats::qnorm( (colMeans(dat0, na.rm=TRUE) + .01 )/1.02 )
                        }
    if (link=="normal"){b <- - colMeans( dat0, na.rm=TRUE ) }
    eps.bG <- bG <- matrix( stats::rnorm( G*I, sd=sigma.b.init ),  G, I )
    aG <- eps.aG <- 0*bG
    sigma.res <- rep(1,I)
    if (link=="normal"){
        sigma.res <- apply( dat0, 2, stats::sd, na.rm=TRUE)
        Z <- dat
                }
    sigma.b <- rep( sigma.b.init, I )
    sigma.a <- rep( .5, I )
    # hyperparameters for items
    omega.b <- if ( est.b.M=="h"){ 2 } else { 1000 }
    mu.b <- 0
    omega.a <- if ( est.a.M=="h"){ 1 } else { 1000 }
    mu.a <- 1
    # item parameters in matrix form
    aM <- matrix( a, nrow=N, ncol=I, byrow=TRUE)
    bM <- matrix( b, nrow=N, ncol=I, byrow=TRUE) + bG[ idgroup, ]
    aM.chainsum <- bM.chainsum <- 0*bM
    if (link=="logit"){
        theta <- stats::qnorm( ( rowMeans( dat0,na.rm=TRUE ) + .01 ) / 1.02 )
                        }
    if (link=="normal"){
        theta <-  ( rowMeans( dat0,na.rm=TRUE ) + .01 ) / 1.02
                        }
    theta <- theta - mean( theta )
    # theta level 2 part
    theta2 <- rep( stats::rnorm( G, sd=.2 ), groupsize )
    # inits Level 1 and Level 2 standard deviations
    sigma1 <- sqrt( .7 )
    sigma2 <- sqrt(.3)
    # residual standard deviation (only applies in model with normal data)
    sigma.res <- apply( dat0, 2, stats::sd, na.rm=TRUE ) * .8
    #***
    # define lower and upper thresholds
    ZZ <- 1000
    threshlow <- -ZZ + ZZ*dat
    threshlow[ is.na(dat0) ] <- -ZZ
    threshupp <- ZZ*dat
    threshupp[ is.na(dat0) ] <- ZZ
    # saved values
    SV <- min( N.sampvalues, iter - burnin )
    svindex <- round( seq( burnin, iter, len=SV ) )
    a.chain <- matrix( NA, SV, I )
    b.chain <- matrix( NA, SV, I )
    sigma.res.chain <- sigma.a.chain <- sigma.b.chain <- matrix( NA, SV, I )
    theta.chain <- matrix( NA, SV, N )
    theta2.chain <- matrix( NA, SV, G )
    # standard deviations
    sigma2.chain <- sigma1.chain <- rep( NA, SV )    # theta Level 1
    omega.a.chain <- mu.b.chain <- omega.b.chain <- rep(NA, SV )

    deviance.chain <- rep(NA, SV)
    zz <- 0
    #**********************
    # begin iterations
    for (ii in 1:iter){

        #****
        # draw latent data Z
        if (link=="logit"){
            Z <- .draw.Z.2pno.ml( aM, bM, theta, N, I, threshlow, threshupp )
                        }

        #***
        # draw latent traits theta (total)
        theta <- .draw.theta.2pno.ml( aM, b, bM, N, I, Z,
                sigma1, sigma2, sigma.res, link, theta2, idgroup  )

        #***
        # draw latent group means (level 2 effects)
        theta2 <- .draw.theta2.2pno.ml( theta, idgroup, groupsize,
                        sigma1, sigma2, G )

        #***
        # draw level 1 and level 2 theta variances
        res <- .draw.sigma12.2pno.ml( theta, theta2, idgroup, N,
                G, prior.sigma2 )
        sigma1 <- res$sigma1
        sigma2 <- res$sigma2


        #***
        # draw item groupwise parameters b (est.b.Var="j" or est.b.Var="i")
        if ( est.b.Var %in% c("i","j")){

            # sampling of b parameters
            b <- .mcmc.est.b.2pno.ml.v2( N,Z, aM, theta, idgroup, groupsize,
                                        b, bG, G, I, sigma.b, omega.b,
                                        mu.b, sigma.res, link )
            # sampling of groupwise b item parameters
            bG <- .mcmc.est.b.group.2pno.ml( Z, aM, theta, idgroup, groupsize,
                    b, G, I, sigma.b, sigma.res, link, N  )
            # sampling of sigma.b
            sigma.b <- .mcmc.sigma.b.2pno.ml( bG, mu.b, omega.b, G, I,
                est.b.Var, prior.sigma.b, sigma.b )
            # define item parameters in a matrix
            bM <- matrix(b,N,I,byrow=TRUE) + bG[ idgroup, ]

                }
        #***
        # draw item difficulties (est.b.Var="n")
        # single level case
        if ( est.b.Var=="n"){
            b <- .draw.est.b.sl(Z, aM, theta, N, I, omega.b, mu.b,
                sigma.res, link )
            bM <- matrix( b, nrow=N, ncol=I, byrow=TRUE)
                        }

        # draw hyperparameters of b (independent of est.b.Var)
        res <- .draw.est.b.hyperpars( b, mu.b, omega.b, I,
                  prior.omega.b, est.b.M  )
        mu.b <- res$mu.b
        omega.b <- res$omega.b

        ##################################
        # sampling of a parameters
        if ( est.a.M !="f" ){
            #****
            # draw a item parameters group wise
            if ( est.a.Var %in% c("i","j")){

                # sampling of a parameters
                a <- .mcmc.a.est.a.2pno.ml( Z, bM, aG, idgroup, theta,
                        mu.a, omega.a, I, link, sigma.res  )

                # sampling of groupwise a parameters
                aG <- .mcmc.est.aG.2pno.ml.v2( Z, bM, theta, idgroup, G, I,
                        a, sigma.a, N, link, sigma.res )

                # sampling of standard deviations of random item
                # discrimination effects
                sigma.a <- .mcmc.a.grouphier.2pno.ml( aG, mu.a, G, omega.a, I,
                            prior.sigma.a, est.a.Var, sigma.a )
                # calculate total item discrimination
                aM <- matrix( a, N, I, byrow=TRUE ) + aG[ idgroup, ]
                                            }
            #***
            # draw a item parameters (single level estimation)
            if ( est.a.Var=="n"){
                a <- .draw.est.a.sl( Z, bM, theta, mu.a, omega.a, I,
                        sigma.res, link)
                aM <- matrix( a, nrow=N, ncol=I, byrow=TRUE)
                        }

            # draw hyperparameters of a
            res <- .draw.est.a.hyperpars( a, mu.a, omega.a, I,
                    prior.omega.a, est.a.M  )
            omega.a <- res$omega.a
                            }

        # draw residual standard deviations for link="normal"
        if (link=="normal"){
            sigma.res <- .draw.sigma.res.2pno.ml( Z, aM, bM, theta, N, I )
                    }

        # save parameters
        if ( ii %in% svindex ){
            zz <- zz+1
            a.chain[ zz, ] <- a
            b.chain[ zz, ] <- b
            theta.chain[ zz, ] <- theta
            theta2.chain[ zz, ] <- theta2
            if (link=="logit"){
                deviance.zz <- .mcmc.deviance.2pl( aM, bM, theta, dat,
                        dat.resp,     weights=NULL, eps )
                            }
            if (link=="normal"){
                deviance.zz <- .mcmc.deviance.normallink.2pno.ml( aM, bM,
                        theta, N, I, dat, dat.resp, sigma.res )
                                }
            deviance.chain[zz] <- deviance.zz
            sigma1.chain[zz] <- sigma1
            sigma2.chain[zz] <- sigma2
            sigma.b.chain[zz,] <- sigma.b
            sigma.a.chain[zz,] <- sigma.a
            mu.b.chain[zz]    <- mu.b
            omega.b.chain[zz] <- omega.b
            omega.a.chain[zz] <- omega.a
            if (link=="normal"){
                sigma.res.chain[zz,] <- sigma.res
            }
            # sum b and a parameters
            bM.chainsum <- bM.chainsum + bM
            aM.chainsum <- aM.chainsum + aM
        }
        # print progress
        if ( ( ii %% progress.iter )==0 ){
            cat( "Iteration", ii, " | ", paste(Sys.time()), "\n")
            utils::flush.console()
        }
    }

    ##############################
    # output
    # Information criteria
    ic <- .mcmc.ic.2pno.ml( aM.chainsum, bM.chainsum, theta.chain, N, I,
                dat, dat.resp, eps, deviance.chain, groupsize,
                sigma.res.chain, link )

    # EAP reliability and person parameter estimates (okay!)
    res <- .mcmc.person.2pno.ml( theta.chain, weights=NULL )
    EAP.rel <- res$EAP.rel
    person <- res$person

    #-----
    # create MCMC object
    mcmcobj <- .mcmc.mcmclist.2pno.ml( a.chain, b.chain, deviance.chain,
        sigma1.chain, sigma2.chain, I, SV, burnin, iter,
        est.b.M, mu.b.chain, omega.b.chain, sigma.b.chain, est.b.Var,
        est.a.M, est.a.Var, omega.a.chain, sigma.a.chain,
        sigma.res.chain, link )

    #----
    # summary of the MCMC output
    summary.mcmcobj <- mcmc.list.descriptives( mcmcobj )

    # time
    s2 <- Sys.time()
    time <- list( "start"=s1, "end"=s2, "timediff"=s2-s1 )
    # description
    if ( link=="logit"){ description <- "2PNO Multilevel Model" }
    if ( link=="normal"){ description <- "Normal Multilevel Model" }
    #----
    # result list
    res <- list( mcmcobj=mcmcobj, summary.mcmcobj=summary.mcmcobj,
            ic=ic, burnin=burnin, iter=iter,
            theta.chain=theta.chain, theta2.chain=theta2.chain,
            deviance.chain=deviance.chain,
            EAP.rel=EAP.rel, person=person, dat=dat0,
            time=time, model='2pno.ml', description=description )
    class(res) <- "mcmc.sirt"
    return(res)
}

## File Name: eigenvalues.manymatrices.R
## File Version: 0.112

###################################################################
# computation of eigenvalues of many matrices
eigenvalues.manymatrices <- function( Sigma.all, itermax=10, maxconv=.001,
    inverse=FALSE )
{
    D <- sqrt( ncol(Sigma.all) )
    N <- nrow( Sigma.all )
    lambda <- matrix( 0, nrow=N, ncol=D )
    U <- matrix( 0, nrow=N, ncol=D^2 )
    Sigma1 <- Sigma.all
    for (dd in 1:D){
        res <- .eigenvalue.manymatrices( Sigma.all=Sigma1, itermax=itermax, maxconv=maxconv )
        lambda[,dd] <- res$lambda
        U[, D*(dd-1 )+1:D ] <- res$z
        z <- res$z
        for (zz in 1:D){
            Sigma1[, D*(zz-1) + 1:D  ] <- Sigma1[, D*(zz-1) + 1:D  ] - lambda[,dd] * z[,zz] * z
        }
   }
    res <- list(lambda=lambda, U=U )
    # calculate determinant
    res$logdet <- rowSums( log( lambda ) )
    res$det <- exp( res$logdet )
    # compute inverse matrix if required
    if (inverse){
        Sigma.inv <- 0*Sigma.all
        for (dd in 1:D){
        for (zz in 1:D){
            #zz <- 1
            #dd <- 1
            z <- U[, D*(dd-1) + 1:D  ]
            Sigma.inv[, D*(zz-1) + 1:D  ] <- Sigma.inv[, D*(zz-1) + 1:D  ] + 1 / lambda[,dd] * z[,zz] * z
                        }}
        res$Sigma.inv <- Sigma.inv
            }
    return(res)
        }
###############################################################################
# computation of the largest eigenvalue and the first eigenvector
.eigenvalue.manymatrices <- function( Sigma.all, itermax=10, maxconv=.001 ){
        D <- sqrt( ncol( Sigma.all) )
        N <- nrow(Sigma.all)
        z <- matrix(1,nrow=N, ncol=D )
        z <- z / sqrt( rowSums( z^2 ) )
        z0 <- z
        ii <- 0
        conv <- 2*maxconv
        while ( ( ii < itermax ) & ( conv > maxconv) ){
            for (dd in 1:D){ #        dd <- 1
                z[,dd] <- rowSums( Sigma.all[, D*(dd-1) + 1:D ] * z0 )
                        }
            v1 <- sqrt(rowSums( z^2 ))
            z <- z / v1
            res <- .rowmax.matrix( abs(z) )
            ind <- as.matrix( data.frame( 1:N, res ) )
            si <- sign( z[ ind ] / z0[ind] )
            z <- si*z
            lambda <- si*v1
            conv <- max( abs( z - z0 ) )
            z0 <- z
            ii <- ii+1
                        }
        res <- list( lambda=lambda, z=z )
        return(res)
            }
##########################################################################
# rowwise maximum in a matrix
.rowmax.matrix <- function(matr){
    ind <- rep(1, nrow(matr) )
    v1 <- matr[,1]
    for (dd in 2:( ncol(matr) ) ){
        ind <- ifelse( matr[,dd] > v1, dd, ind )
        v1 <- ifelse( matr[,dd] > v1, matr[,dd], v1 )
                            }
    return(ind)
        }
########################################################################

## File Name: rasch.copula2_aux.R
## File Version: 1.345



#--------------------------------------------------------------------------------------
# Function calculates necessary patterns for copula IRT models (Braeken, 2011)
.calc.copula.itemcluster <- function(D){
    res <- sirt_permutations(r=D, v=0:1)
    rownames(res) <- apply( res, 1, FUN=function(ll){ paste("P", paste( ll,collapse=""),sep="") } )
    RR <- nrow(res)
    matr <- matrix( 0, RR, RR )
    rownames(matr) <- colnames(matr) <- rownames(res)
    colnames(matr) <- gsub( "P", "F", colnames(matr) )
    vec <- 1:RR
    for (rr in vec){
        res.rr <- outer( rep(1,nrow(res)), res[rr,] ) - res
        a1.rr <- apply( res.rr, 1, FUN=function(ll){ paste("F", paste( ll,collapse=""),sep="") } )
        g1.rr <- ( (-1)^rowSums( res ))
        ind.rr <- which( apply( res.rr, 1, min ) > -1 )
        a1.rr <- a1.rr[ind.rr]
        g1.rr <- g1.rr[ind.rr]
        matr[ rr, a1.rr ]  <- g1.rr
    }
    res1 <- list( "patt"=res, "calc"=matr )
    return(res1)
}
#--------------------------------------------------------------------------------------



#--------------------------------------------------------------------------------------
# Function calculates necessary patterns for copula IRT models (Braeken, 2011)
.calc.copula.itemcluster2 <- function(D){
    res <- sirt_permutations(r=D, v=0:1)
    rownames(res) <- apply( res, 1, FUN=function(ll){ paste("P", paste( ll,collapse=""),sep="") } )
    RR <- nrow(res)
    matr <- matrix( 0, RR, RR )
    rownames(matr) <- colnames(matr) <- rownames(res)
    colnames(matr) <- gsub( "P", "F", colnames(matr) )
    matr <-  calc_copula_itemcluster_C( D, res )$matr
    res1 <- list( "patt"=res, "calc"=matr )
    return(res1)
}
#--------------------------------------------------------------------------------------




#----------------------------------------------------------------------------------------------
.update.ll.rasch.copula21 <- function( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
        bdat2.li, bdat2.li.resp, rescopula, itemcluster, pattern, GG, copula.type, ... ){
        #-------------------------------------------------------
        # use this function for log likelihood calculation
        # look for items which change parameters for necessary update
        G <- GG
        # calculation of terms of independent itemclusters?
        calc.ind <- length(itemcluster0) > 0
        eps1 <- 1E-14
        # calculate necessary updates
        ind.b <- which( b !=rescopula$b )
        ind.a <- which( a !=rescopula$a )
        ind.delta <- which( delta !=rescopula$delta )
        ind.alpha1 <- ( alpha1 !=rescopula$alpha1 )    + ( alpha2 !=rescopula$alpha2 )
        if (ind.alpha1 > 0){ ind.alpha <- seq(1, ncol(dat2.ld) ) } else { ind.alpha <- NULL }
        itemset <- union( ind.b, ind.a )
        itemset <- union( itemset, ind.alpha )
        # update term local independence
        li.update <- 1 * ( sum( itemcluster0 %in% itemset ) > 0 )
        # update terms item dependence parameters
        ld.update <- sapply( 1:CC, FUN=function(cc){
                g1 <- intersect( which( itemcluster==cc ), itemset )
                if ( length(g1)){ v1 <- cc } else { v1 <- NULL }
                v1
                    } )
        ld.update <- unique( union( ind.delta, unlist( ld.update) ) )
# cat("    ---- ld.update") ; vv1 <- Sys.time(); print(vv1-vv0) ; vv0 <- vv1
        ###########################################################################
        ndat2 <- nrow(dat2.ld)
        M1 <- rep(1,ndat2)
        ntheta <- length(theta.k)
        M2 <- rep( 1, ntheta)
        pjk.theta.k <- .prob.raschtype.genlogis( theta.k, b, alpha1, alpha2, a)
        pjk.theta.k01 <- cbind( pjk.theta.k, 1 - pjk.theta.k, 1  )
        #.............................................
        # probabilities for indepedent items
        if ( calc.ind ){
            pjk.theta.k0 <- pjk.theta.k01[, c( itemcluster0, itemcluster0 + I ) ]
                                } else  {
            pjk.theta.k0 <- NULL
                                }
# cat("    ---- probs independence") ; vv1 <- Sys.time(); print(vv1-vv0) ; vv0 <- vv1
        # probabilities for dependent items
        pjk.theta.kCC <- rescopula$pjk.theta.kCC
        for (cc in ld.update){
# cat("     ***", 'cc=', cc )
# hh0 <- Sys.time()
            # cc <- 2    # itemcluster cc
            dp.ld.cc <- dp.ld[[cc]]
            m1.cc <- pjk.theta.k01[, dp.ld.cc$independent$items ]
            v1.cc <- dp.ld.cc$independent$N.Index1
# cat("    ---- cc begin probs") ; hh1 <- Sys.time(); print(hh1-hh0) ; hh0 <- hh1
            #--------------------------------------------
            # Boundary Mixture Copula (Braeken, 2011)
            if (copula.type[cc]=="bound.mixt" ){
                # likelihood under independence
                F0pjk.cc <- .rowProds2.bundle( m1=m1.cc, v1=v1.cc)
                # likelihood under dependence
                m1.cc <- pjk.theta.k01[, dp.ld.cc$dependent$items ]
                v1.cc <- dp.ld.cc$dependent$N.Index1

#cat("    ---- cc before bundle") ; hh1 <- Sys.time(); print(hh1-hh0) ; hh0 <- hh1

                pjk.cc <- .rowMins2cpp.bundle( m1=m1.cc, v1=v1.cc)
#cat("    ---- cc after bundle") ; hh1 <- Sys.time(); print(hh1-hh0) ; hh0 <- hh1

# This function is most time consuming!!!!
#                F1pjk.cc <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                # t( A * t(B) )=B * t(A)
                F1pjk.cc <- tcrossprod(  pjk.cc,  dp.ld.cc$calc  )
                pjk.theta.kCC[[cc]] <- ( 1 - delta[cc] ) * F0pjk.cc + delta[cc] * F1pjk.cc
# cat("    ---- cc clac pjk.theta.kCC") ; hh1 <- Sys.time(); print(hh1-hh0) ; hh0 <- hh1
                                        }

            #-----------------------------------------------
            # Cook-Johnson Copula
            if (copula.type[cc]=="cook.johnson" ){
                F.Xr <-  1- pjk.theta.k[, dp.ld.cc$items ]
                R <- ncol(F.Xr)
                delta.cc <- delta[cc]
                patt.cc <- dp.ld.cc$patt
                pjk.cc  <- matrix( 0, nrow=ntheta, ncol=nrow(patt.cc) )
                for (pp in 1:( nrow(patt.cc) ) ){
                    ppcc <- 1 - patt.cc[pp,]
                    pjk.cc[, pp ] <- ( rowSums(
                        ( F.Xr^(-delta.cc))^( outer( rep(1,ntheta), ppcc )) )     - R + 1 )^(-1/delta.cc)
        #            temp1 <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                    temp1 <- tcrossprod( pjk.cc, dp.ld.cc$calc )
                    temp1[ temp1 < 0 ] <- eps1
                    pjk.theta.kCC[[cc]]    <- temp1

                                }
                            }
            #******************************************
            # Frank copula
            if (copula.type[cc]=="frank" ){
                F.Xr <-  1- pjk.theta.k[, dp.ld.cc$items ]
                R <- ncol(F.Xr)
                delta.cc <- delta[cc]
                prod.delta <- ( 1 - exp( - delta.cc ) )^(R-1)
                patt.cc <- dp.ld.cc$patt
                pjk.cc  <- matrix( 0, nrow=ntheta, ncol=nrow(patt.cc) )
                for (pp in 1:( nrow(patt.cc) ) ){
                    ppcc <- 1 - patt.cc[pp,]
                    g1 <- rowProds2( ( 1 - exp( - delta.cc * F.Xr^( outer( rep(1,ntheta), ppcc ))  ) ) )
                    pjk.cc[,pp] <- - log( 1 - g1 / prod.delta ) / delta.cc
                                }
#                    temp1 <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                    temp1 <- tcrossprod( pjk.cc, dp.ld.cc$calc )
                    temp1[ temp1 < 0 ] <- eps1
                    pjk.theta.kCC[[cc]]    <- temp1
                            } # end Frank copula
                        }
        #---------------------------------------------------------------
# cat("    ---- probs dependence") ; vv1 <- Sys.time(); print(vv1-vv0) ; vv0 <- vv1

        #####################################
        # rearrange output
                res <- list( "pjk.theta.kCC"=pjk.theta.kCC, "pjk.theta.k0"=pjk.theta.k0  )
                return(res)
                }
#----------------------------------------------------------------------------------------------



#----------------------------------------------------------------------------------------------
.update.ll.rasch.copula20 <- function( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
        bdat2.li, bdat2.li.resp, rescopula, itemcluster, pattern, GG, copula.type, ... ){
        #-------------------------------------------------------
        # use this function for log likelihood calculation
        # look for items which change parameters for necessary update
        G <- GG
        # calculation of terms of independent itemclusters?
        calc.ind <- length(itemcluster0) > 0
        eps1 <- 10^(-14)
        # calculate necessary updates
        ind.b <- which( b !=rescopula$b )
        ind.a <- which( a !=rescopula$a )
        ind.delta <- which( delta !=rescopula$delta )
        ind.alpha1 <- ( alpha1 !=rescopula$alpha1 )    + ( alpha2 !=rescopula$alpha2 )
        if (ind.alpha1 > 0){ ind.alpha <- seq(1, ncol(dat2.ld) ) } else { ind.alpha <- NULL }
        itemset <- union( ind.b, ind.a )
        itemset <- union( itemset, ind.alpha )
        # update term local independence
        li.update <- 1 * ( sum( itemcluster0 %in% itemset ) > 0 )
        # update terms item dependence parameters
        ld.update <- sapply( 1:CC, FUN=function(cc){
                g1 <- intersect( which( itemcluster==cc ), itemset )
                if ( length(g1)){ v1 <- cc } else { v1 <- NULL }
                v1
                    } )
        ld.update <- unique( union( ind.delta, unlist( ld.update) ) )
        ###########################################################################
        ndat2 <- nrow(dat2.ld)
        M1 <- rep(1,ndat2)
        ntheta <- length(theta.k)
        M2 <- rep( 1, ntheta)
        pjk.theta.k <- .prob.raschtype.genlogis( theta.k, b, alpha1, alpha2, a)
        pjk.theta.k01 <- cbind( pjk.theta.k, 1 - pjk.theta.k, 1  )
        #.............................................
        # probabilities for indepedent items
        if ( calc.ind ){
            pjk.theta.k0 <- pjk.theta.k01[, c( itemcluster0, itemcluster0 + I ) ]
                                } else  {
            pjk.theta.k0 <- NULL
                                }
        # probabilities for dependent items
        pjk.theta.kCC <- rescopula$pjk.theta.kCC
        for (cc in ld.update){
            # cc <- 2    # itemcluster cc
            dp.ld.cc <- dp.ld[[cc]]
            m1.cc <- pjk.theta.k01[, dp.ld.cc$independent$items ]
            v1.cc <- dp.ld.cc$independent$N.Index1
            #--------------------------------------------
            # Boundary Mixture Copula (Braeken, 2011)
            if (copula.type[cc]=="bound.mixt" ){
                # likelihood under independence
                F0pjk.cc <- .rowProds2.bundle( m1=m1.cc, v1=v1.cc)
                # likelihood under dependence
                m1.cc <- pjk.theta.k01[, dp.ld.cc$dependent$items ]
                v1.cc <- dp.ld.cc$dependent$N.Index1
                pjk.cc <- .rowMins2cpp.bundle( m1=m1.cc, v1=v1.cc)
#                F1pjk.cc <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                F1pjk.cc <- tcrossprod(  pjk.cc,  dp.ld.cc$calc  )
                pjk.theta.kCC[[cc]] <- ( 1 - delta[cc] ) * F0pjk.cc + delta[cc] * F1pjk.cc
                                        }
            #-----------------------------------------------
            # Cook-Johnson Copula
            if (copula.type[cc]=="cook.johnson" ){
                F.Xr <-  1- pjk.theta.k[, dp.ld.cc$items ]
                R <- ncol(F.Xr)
                delta.cc <- delta[cc]
                patt.cc <- dp.ld.cc$patt
                pjk.cc  <- matrix( 0, nrow=ntheta, ncol=nrow(patt.cc) )
                for (pp in 1:( nrow(patt.cc) ) ){
                    ppcc <- 1 - patt.cc[pp,]
                    pjk.cc[, pp ] <- ( rowSums( ( F.Xr^(-delta.cc))^( outer( rep(1,ntheta), ppcc )) )
                                                    - R + 1 )^(-1/delta.cc)
#                    temp1 <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                    temp1 <- tcrossprod( pjk.cc, dp.ld.cc$calc )
                    temp1[ temp1 < 0 ] <- eps1
                    pjk.theta.kCC[[cc]]    <- temp1

                                }
                            }
            #******************************************
            # Frank copula
            if (copula.type[cc]=="frank" ){
                F.Xr <-  1- pjk.theta.k[, dp.ld.cc$items ]
                R <- ncol(F.Xr)
                delta.cc <- delta[cc]
                prod.delta <- ( 1 - exp( - delta.cc ) )^(R-1)
                patt.cc <- dp.ld.cc$patt
                pjk.cc  <- matrix( 0, nrow=ntheta, ncol=nrow(patt.cc) )
                for (pp in 1:( nrow(patt.cc) ) ){
                    ppcc <- 1 - patt.cc[pp,]
                    g1 <- rowProds2( ( 1 - exp( - delta.cc * F.Xr^( outer( rep(1,ntheta), ppcc ))  ) ) )
                    pjk.cc[,pp] <- - log( 1 - g1 / prod.delta ) / delta.cc
                                }
#                    temp1 <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                    temp1 <- tcrossprod( pjk.cc, dp.ld.cc$calc )
                    temp1[ temp1 < 0 ] <- eps1
                    pjk.theta.kCC[[cc]]    <- temp1
                            } # end Frank copula
                        }
        #---------------------------------------------------------------
        # calculate log likelihood
        rjk0.1 <- rescopula$rjk0.1
        rjk0.0 <- rescopula$rjk0.0
        rjkCC <- rescopula$rjkCC
        #.......................
        # one group
        if (G==1){
            ll0 <-     sum( rescopula$nk * log(rescopula$pik) )
             # likelihood part from independent items
             if ( calc.ind ){
                if ( nrow( rjk0.1)==1 ){
                    rjk.temp <- cbind( t( rjk0.1), t(rjk0.0) )
                            } else {
                    rjk.temp <- cbind( rjk0.1, rjk0.0 )
                                }
                    ll0 <- ll0 + sum( log(pjk.theta.k0) * rjk.temp )
                            }
             # likelihood part from dependent items
            for (cc in 1:CC){
                ll0 <- ll0 + sum( rjkCC[[cc]] * log( pjk.theta.kCC[[cc]] + 10^(-15) ) )
                            }

                                }
#                    }
        #.......................
        # multiple groups
        #    ... to do    ...
            lli <- ll0

        #####################################
        # rearrange output
                res <- list( "ll"=ll0, "lli"=lli )
                return(res)
                }
#----------------------------------------------------------------------------------------------




#----------------------------------------------------------------------------------------------
.ll.rasch.copula20 <- function( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
        bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld, ... ){
        #-------------------------------------------------------
        # use this function for log likelihood calculation
        # calculation of terms of independent itemclusters?
        calc.ind <- length(itemcluster0) > 0
        G <- GG            # number of groups
        eps1 <- 10^(-14)
        ndat2 <- nrow(dat2.ld)
        M1 <- rep(1,ndat2)
        ntheta <- length(theta.k)
        M2 <- rep( 1, ntheta)
         pjk.theta.k <- .prob.raschtype.genlogis( theta.k, b, alpha1, alpha2, a)
        pjk.theta.k01 <- cbind( pjk.theta.k, 1 - pjk.theta.k, 1  )

        ################################################
        # E step
        ################################################
        #.............................................
        # probabilities for indepedent items
        if ( calc.ind ){
            pjk.theta.k0 <- pjk.theta.k01[, c( itemcluster0, itemcluster0 + I ) ]
                                } else  {
            pjk.theta.k0 <- NULL
                                }
        # probabilities for dependent items
        pjk.theta.kCC <- as.list( 1:CC )

        for (cc in 1:CC){
            # cc <- 2    # itemcluster cc
            dp.ld.cc <- dp.ld[[cc]]
            m1.cc <- pjk.theta.k01[, dp.ld.cc$independent$items ]
            v1.cc <- dp.ld.cc$independent$N.Index1
            #--------------------------------------------
            # Boundary Mixture Copula (Braeken, 2011)
            if (copula.type[cc]=="bound.mixt" ){
                # likelihood under independence
                F0pjk.cc <- .rowProds2.bundle( m1=m1.cc, v1=v1.cc)
                # likelihood under dependence
                m1.cc <- pjk.theta.k01[, dp.ld.cc$dependent$items ]
                v1.cc <- dp.ld.cc$dependent$N.Index1
                pjk.cc <- .rowMins2cpp.bundle( m1=m1.cc, v1=v1.cc)
#                F1pjk.cc <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                F1pjk.cc <- tcrossprod(  pjk.cc,  dp.ld.cc$calc  )
                pjk.theta.kCC[[cc]] <- ( 1 - delta[cc] ) * F0pjk.cc + delta[cc] * F1pjk.cc
                            }
            #-----------------------------------------------
            # include other Copula models here
            # Cook-Johnson copula (Braeken et al., 2007)
            if (copula.type[cc]=="cook.johnson" ){
                F.Xr <-  1- pjk.theta.k[, dp.ld.cc$items ]
                R <- ncol(F.Xr)
                delta.cc <- delta[cc]
                patt.cc <- dp.ld.cc$patt
                pjk.cc  <- matrix( 0, nrow=ntheta, ncol=nrow(patt.cc) )
                for (pp in 1:( nrow(patt.cc) ) ){
                    ppcc <- 1 - patt.cc[pp,]
                    pjk.cc[, pp ] <- ( rowSums( ( F.Xr^(-delta.cc))^( outer( rep(1,ntheta), ppcc )) )
                                                    - R + 1 )^(-1/delta.cc)
                                }
#                    pjk.theta.kCC[[cc]] <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
#                    temp1 <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                    temp1 <- tcrossprod( pjk.cc, dp.ld.cc$calc )
                    temp1[ temp1 < 0 ] <- eps1
                    pjk.theta.kCC[[cc]]    <- temp1
                            }
            #******************************************
            # Frank copula
            if (copula.type[cc]=="frank" ){
                F.Xr <-  1- pjk.theta.k[, dp.ld.cc$items ]
                R <- ncol(F.Xr)
                delta.cc <- delta[cc]
                prod.delta <- ( 1 - exp( - delta.cc ) )^(R-1)
                patt.cc <- dp.ld.cc$patt
                pjk.cc  <- matrix( 0, nrow=ntheta, ncol=nrow(patt.cc) )
                for (pp in 1:( nrow(patt.cc) ) ){
                    ppcc <- 1 - patt.cc[pp,]
                    g1 <- rowProds2( ( 1 - exp( - delta.cc * F.Xr^( outer( rep(1,ntheta), ppcc ))  ) ) )
                    pjk.cc[,pp] <- - log( 1 - g1 / prod.delta ) / delta.cc
                                }
#                    pjk.theta.kCC[[cc]] <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
#                    temp1 <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                    temp1 <- tcrossprod( pjk.cc, dp.ld.cc$calc )
                    temp1[ temp1 < 0 ] <- eps1
                    pjk.theta.kCC[[cc]]    <- temp1
                            }  # end Frank copula
                        }

#print( str(pjk.theta.kCC))
# cat( "\n probabilities \n" ) ; aa1 <- Sys.time() ; print(aa1-aa0) ; aa0 <- aa1

        #.............................................
        # Calculate posterior distribution
#        post0 <- matrix( 1, nrow=ndat2, ncol=ntheta )
        # posterior distribution independent items
#        if ( length(itemcluster0) > 0){
#            post0 <- sapply( 1:ntheta, FUN=function(tt){
#                    # tt <- 11
#                    g1 <- outer( M1, pjk.theta.k0[ tt, ] )
#                    rowProds2( g1^( bdat2.li * bdat2.li.resp ) )
#                        } )
#                            }

        #****
        post0 <- matrix( 1, nrow=ndat2, ncol=ntheta )
        I0 <- length(itemcluster0)
        if ( calc.ind ){
            pjkL <- array( NA, dim=c(2, length(theta.k), I0 ) )
            # pjkL: [ #categories, #thetagrid, #items ]
            pjkL[1,,] <- 1 - pjk.theta.k0[,1:I0]
            pjkL[2,,] <- pjk.theta.k0[,1:I0]
            for (ii in 1:I0 ){
                ind.ii <- which( bdat2.li.resp[,ii]==1 )
                post0[ind.ii,] <- post0[ind.ii,] * pjkL[ bdat2.li[ind.ii,ii]+1,,ii]
                        }
                        }

        # posterior distribution dependent items
#        post2 <- sapply( 1:ntheta, FUN=function(tt){
                # tt <- 11
#                h1 <- 1
#                for (cc in 1:CC){
                    # cc <- 1
#                    pcc <- pjk.theta.kCC[[cc]]
#                    h1 <- h1 * ( (pcc[tt,])[ dat2.ld[,cc] ] )^( dat2.ld.resp[,cc] )
#                                }
#                    h1
#                            } )
        #****
        post2 <- matrix( 1, nrow=ndat2, ncol=ntheta )
        pjkL <- array( NA, dim=c(Ncat.ld, length(theta.k), CC ) )
        for (cc in 1:CC){
            #    cc <- 1
            p1.cc <- t( pjk.theta.kCC[[cc]] )
            pjkL[ seq( 1, nrow(p1.cc) ),, cc ] <- p1.cc
                        }
        for (cc in 1:CC ){
            ind.ii <- which( dat2.ld.resp[,cc]==1 )
            post2[ind.ii,] <- post2[ind.ii,] * pjkL[ dat2.ld[ind.ii,cc],,cc]
                    }


        post <- post0 * post2        # product of independent and dependent parts
        post.unnorm <-  post
        post <- post * outer( M1, wgt.theta )
        post <- post / rowSums( post)    # standardization of posterior distribution

# cat( "\n posterior\n" ) ; aa1 <- Sys.time() ; print(aa1-aa0) ; aa0 <- aa1


        #....................................................
        # Calculate expected counts
        # expected counts independent item responses
        njk0 <- rjk0.0 <- rjk0.1 <- NULL
        # pattern[,gg+1] is the frequency weight of a response pattern in group gg
        gg <- 1
        if ( calc.ind ){
#            FW1 <- outer( pattern[,gg+1], rep( 1, ncol(dat2.li.resp) ) )
            # students at items
#            njk0 <- t( sapply( 1:ntheta, FUN=function(tt){
#                        colSums( FW1 * dat2.li.resp * outer( post[,tt], rep( 1, ncol(dat2.li.resp) ) ) )
#                            } )    )
#            njk0 <- t( post ) %*% ( pattern[,gg+1] * dat2.li.resp )
            njk0 <- crossprod( post, pattern[,gg+1] * dat2.li.resp )

            # how many students solved correctly items
#            rjk0.1 <- t( sapply( 1:ntheta, FUN=function(tt){
#                        colSums( dat2.li * FW1 * dat2.li.resp * outer( post[,tt], rep( 1, ncol(dat2.li.resp) ) ) )
#                            } )    )
#            rjk0.1 <- t( post ) %*% ( pattern[,gg+1] * dat2.li.resp * dat2.li )
            rjk0.1 <- crossprod( post,  pattern[,gg+1] * dat2.li.resp * dat2.li )
            rjk0.0 <- njk0 - rjk0.1
                                    }
        # use loops in case of multiple groups !!!


        #*****************
        # expected counts dependent items
        rjkCC <- as.list( 1:CC )
        gg <- 1
        for ( cc in 1:CC){
            #    cc <- 1
#            rjkCC[[cc]] <- t(post) %*% ( pattern[,gg+1] * dat3.ld[[cc]] * dat2.ld.resp[,cc] )
            rjkCC[[cc]] <- crossprod( post,  pattern[,gg+1] * dat3.ld[[cc]] * dat2.ld.resp[,cc] )
                    }

        # total counts
        gg <- 1
        tc <- outer( pattern[,gg+1], rep( 1, ntheta ) )
        nk <- colSums( tc * post)
        pik <- nk / sum(nk)

# cat( "\n expected counts \n" ) ; aa1 <- Sys.time() ; print(aa1-aa0) ; aa0 <- aa1

        #---------------------------------------------------------------
        # calculate log likelihood
        #.......................
        # one group
        if (G==1){
             ll0 <- sum( nk * log(pik) )
#             ll0 <- sum( nk * log(wgt.theta) )
             # likelihood part from independent items
             if ( calc.ind ){
                if ( nrow( rjk0.1)==1 ){
                    rjk.temp <- cbind( t( rjk0.1), t(rjk0.0) )
                            } else {
                    rjk.temp <- cbind( rjk0.1, rjk0.0 )
                                }
                    ll0 <- ll0 + sum( log(pjk.theta.k0) * rjk.temp )
                            }
             # likelihood part from dependent items
            for (cc in 1:CC){
                ll0 <- ll0 + sum( rjkCC[[cc]] * log( pjk.theta.kCC[[cc]] + 10^(-15) ) )
                                }

                    }
# cat( "\n Likelihood \n" ) ; aa1 <- Sys.time() ; print(aa1-aa0) ; aa0 <- aa1

        #.......................
        # multiple groups
        #    ... to do    ...

            lli <- ll0
        ################################
        # arrange output
        res <- list( "ll"=ll0,  "b"=b, "a"=a, "delta"=delta,
                        "alpha1"=alpha1, "alpha2"=alpha2, "lli"=lli,
                        "post"=post, "post.unnorm"=post.unnorm,
                        "rjk0.1"=rjk0.1,
                        "rjk0.0"=rjk0.0, "rjkCC"=rjkCC,
                        "pjk.theta.kCC"=pjk.theta.kCC,
                        "pjk.theta.k0"=pjk.theta.k0,
                        "nk"=nk, "pik"=pik,"calc.ind"=calc.ind
                                                    )
        return(res)
        #~~~~~~~~~~~~~~
        # Output Version < 1.0
        #                res <- list( "ll"=ll1, "g1"=res, "b"=b, "a"=a, "delta"=delta,
        #                                    "alpha1"=alpha1, "alpha2"=alpha2, "lli"=lli,
        #                                    "ll.theta.post"=ll.theta.post,
        #                                    "exp.theta.post"=exp.theta.post
        #                                            )
                }
#----------------------------------------------------------------------------------------------



###############################################################################################
# auxiliary function: person parameter estimation
.likelihood.rasch.copula <- function( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, I,
        bdat2.li, bdat2.li.resp, eps=10^(-20), ... ){
        pjk.theta.k.tt <- .prob.raschtype.genlogis( theta.k, b, alpha1, alpha2, a)
        g1 <- matrix( 0, nrow(pjk.theta.k.tt), CC + 1 )
        ndat2 <- nrow(dat2.ld)
        M1 <- rep(1,ndat2)
        pqjk.theta.k.tt <- cbind( pjk.theta.k.tt, 1 - pjk.theta.k.tt, 1 )
                    if ( length(itemcluster0) > 0 ){
                pqjk.theta.k.tt0 <- pqjk.theta.k.tt[, c( itemcluster0, itemcluster0+I) ]
                # likelihood for independent items at theta tt
                ll.tt <- ( pqjk.theta.k.tt0 ^ bdat2.li )^bdat2.li.resp
                g1[,1] <- rowProds2( ll.tt )        } else { g1[,1] <- 1 }

            g1i <- g1
            # likelihood for dependent items
            for (cc in 1:CC){
                #                cc <- 2
                dat3.ld.cc <- dat3.ld[[cc]]
                dp.ld.cc <- dp.ld[[cc]]
                m1.cc <- pqjk.theta.k.tt[, dp.ld.cc$independent$items ]
                v1.cc <- dp.ld.cc$independent$N.Index1
                # product under independence
                Fpjk.cc <- .rowProds2.bundle( m1=m1.cc, v1=v1.cc)
                # evaluate likelihood
                g1i[,cc+1] <- g1.tt <- ( rowSums(Fpjk.cc * dat3.ld.cc ) )^dat2.ld.resp[,cc]
                # product under dependence
                m1.cc <-  pqjk.theta.k.tt[, dp.ld.cc$dependent$items ]
                v1.cc <- dp.ld.cc$dependent$N.Index1
                F0pjk.cc <- .rowMins2cpp.bundle( m1=m1.cc, v1=v1.cc)
#                F1pjk.cc <- F0pjk.cc %*% t(dp.ld.cc$calc)
                F1pjk.cc <- tcrossprod( F0pjk.cc, dp.ld.cc$calc )
                g2.tt <- ( rowSums(F1pjk.cc * dat3.ld.cc) )^dat2.ld.resp[,cc]
                g3.tt <- ( 1 - delta[cc] ) * g1.tt + delta[cc] * g2.tt
                g1[,cc+1] <- g3.tt
                            }
                res <- g1
                g1 <- rowProds2( g1 )
                g1i <- rowProds2( g1i )
                # calculate log likelihood
                g1[ g1 < eps] <- eps
                g1i[ g1i < eps] <- eps
                g1 <- log( g1 )
                g1i <- log( g1i )
                res <- list( "loglike.dep"=g1, "loglike.ind"=g1i )
                return(res)
                    }
###############################################################################################



##################################################################
# function for person parameter estimation in rasch copula models
person.parameter.rasch.copula <- function( raschcopula.object, numdiff.parm=.001,
                    conv.parm=.001, maxiter=20, stepwidth=1,
                    print.summary=TRUE,                     ... ){
        dat2.li <- NULL
        dat2 <- raschcopula.object$datalist$dat2
        dat2.resp <- raschcopula.object$datalist$dat2.resp
        dat2.ld <- raschcopula.object$datalist$dat2.ld
        dat2.ld.resp <- raschcopula.object$datalist$dat2.ld.resp
        dat2.li.resp <- raschcopula.object$datalist$dat2.li.resp
        dat3.ld <- raschcopula.object$datalist$dat3.ld
        bdat2.li <- raschcopula.object$datalist$bdat2.li
        bdat2.li.resp <- raschcopula.object$datalist$bdat2.li.resp
        CC <- raschcopula.object$datalist$CC
        dp.ld <- raschcopula.object$datalist$dp.ld
        itemcluster0 <- raschcopula.object$datalist$itemcluster0
        b <- raschcopula.object$b
        a <- raschcopula.object$a
        alpha1 <- raschcopula.object$alpha1
        alpha2 <- raschcopula.object$alpha2
        delta <- raschcopula.object$delta
        pattern <- raschcopula.object$pattern
        ndat2 <- nrow(dat2)
        ntheta <- nrow(dat2)
        I <- ncol(dat2)
        ######################
        # missing response pattern
        mp <- paste("R", dat2.resp[,1], sep="")
        for (vv in seq( 2, ncol(dat2) ) ){
            mp <- paste( mp, dat2.resp[,vv], sep="")
                    }
        # initial estimate of theta
#        theta0 <- rowMeans( dat2 * dat2.resp  )
        dat20 <- dat2 * dat2.resp
        dat20[ dat2.resp==0 ] <- NA
        theta0 <- rowMeans( dat20, na.rm=T )
        theta0 <- stats::qlogis( theta0 )
        # theta0[1:2] <- c(0,1)
        theta0[ theta0==- Inf] <- -9999
        theta0[ theta0==Inf] <- 9999
        theta.init <- theta0i <- theta0
        ii <- 0
        h <- numdiff.parm
        a1m <- 990
        while( a1m > conv.parm & ii < maxiter ){
            # evaluation of likelihood at theta0
            rescop0 <- .likelihood.rasch.copula( theta.k=theta0, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, I,
                        bdat2.li, bdat2.li.resp  )
            rescop1 <- .likelihood.rasch.copula( theta.k=theta0 + h, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, I,
                        bdat2.li, bdat2.li.resp  )
            rescop2 <- .likelihood.rasch.copula( theta.k=theta0 - h, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, I,
                        bdat2.li, bdat2.li.resp  )
            rescop0i <- .likelihood.rasch.copula( theta.k=theta0i, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, I,
                        bdat2.li, bdat2.li.resp  )
            rescop1i <- .likelihood.rasch.copula( theta.k=theta0i + h, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, I,
                        bdat2.li, bdat2.li.resp  )
            rescop2i <- .likelihood.rasch.copula( theta.k=theta0i - h, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, I,
                        bdat2.li, bdat2.li.resp  )
            #******
            # estimation assuming dependence
            ll1 <- rescop1$loglike.dep
            ll2 <- rescop2$loglike.dep
            ll0 <- rescop0$loglike.dep
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2d <- d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            theta.change <- - d1 / d2
            theta.change[ abs( theta.init )==9999 ] <- 0
            theta.change[ is.na( theta.change ) ] <- 0
            a1t1 <- theta.change <- ifelse( abs( theta.change ) > stepwidth, stepwidth*sign(theta.change), theta.change )
            theta0 <- theta0 + theta.change
            ind1 <- ( abs( theta.change ) < conv.parm )

            #******
            # estimation assuming independence
            ll1 <- rescop1i$loglike.ind
            ll2 <- rescop2i$loglike.ind
            ll0 <- rescop0i$loglike.ind
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2i <- d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            theta.change <- - d1 / d2
            theta.change[ abs( theta.init )==9999 ] <- 0
            theta.change[ is.na( theta.change ) ] <- 0
            a1t2 <- theta.change <- ifelse( abs( theta.change ) > stepwidth, stepwidth*sign(theta.change), theta.change )
            theta0i <- theta0i + theta.change
            a1m <- max( abs(a1t1), abs(a1t2) )
            ii <- ii+1
            cat("Iteration", ii, ":   max. parm. change", round( a1m, 5))
            cat("   |", sum(ind1), "out of", ndat2, "cases converged")
            cat(" (", round(100*sum(ind1)/length(ind1),1), "%)\n")
            utils::flush.console()
                    }
        theta0[ abs( theta.init  )==9999 ] <- NA
        theta0i[ abs( theta.init  )==9999 ] <- NA
        res <- data.frame( "pattern"=pattern[,1],
                    "missing.pattern"=match( mp, unique(mp) ),
                    "freqwgt"=pattern$freqwgt,
                    "converged"=1*(ind1==1),
                "score"=rowSums(dat2), "max"=rowSums( dat2.resp),
                 "theta.dep"=theta0, "theta.ind"=theta0i )
        eps <- 1E-7
        res$setheta.dep <- sqrt( - 1 / abs(d2d+eps) )
        res$setheta.ind <- sqrt( - 1 / abs(d2i+eps) )
        res$setheta.dep[ is.na(theta0i) ] <- NA
        res$setheta.ind[ is.na(theta0i) ] <- NA
        res$seinflat <- res$setheta.dep / res$setheta.ind
        res[ is.na(res$theta.dep), "converged" ] <- NA
        x1 <- seq( grep( "theta", colnames(res) )[1], ncol(res) )
        for (vv in x1){
                res[ paste(res$converged)==0, vv] <- NA
                        }
        res0 <- res <- res[ order( paste( 10000+ res$missing.pattern, 10000+ res$score)), ]
        # calculate a summary
        index <- rep( seq(1,nrow(res0)), res$freqwgt )
        res <- res[ index, ]
        a1 <- stats::aggregate( res[, c("theta.dep", "theta.ind")], list( res$missing.pattern,  res$score, res$max), mean, na.rm=T )
        colnames(a1) <- c("missing.pattern", "score", "max", "M.theta.dep", "M.theta.ind" )
        a1$N <- stats::aggregate( 1+0*res[, c("theta.dep")], list( res$missing.pattern,res$score, res$max), sum, na.rm=T )[,4]
        a1 <- a1[, c(1:3,6,4,5) ]
        a1$SD.theta.dep <- stats::aggregate( res[, c("theta.dep")],
                list( res$missing.pattern,res$score, res$max), stats::sd, na.rm=T)[,4]
        a1$Min.theta.dep <- stats::aggregate( res[, c("theta.dep")],
                list( res$missing.pattern,res$score, res$max), min, na.rm=T)[,4]
        a1$Max.theta.dep <- stats::aggregate( res[, c("theta.dep" )],
                list( res$missing.pattern,res$score, res$max), max, na.rm=T )[,4]
        if ( abs(alpha1) + abs( alpha2) > 0 ){
            a1$SD.theta.ind <- stats::aggregate( res[, c("theta.ind")],
                    list( res$missing.pattern,res$score, res$max), stats::sd, na.rm=T)[,4]
            a1$Min.theta.ind <- stats::aggregate( res[, c("theta.ind")],
                        list( res$missing.pattern,res$score, res$max), min, na.rm=T )[,4]
            a1$Max.theta.ind <- stats::aggregate( res[, c("theta.ind" )],
                        list( res$missing.pattern,res$score, res$max), max, na.rm=T)[,4]
                    }
        a1$M.seinflat <- stats::aggregate( res[, c("seinflat")],
                list( res$missing.pattern,res$score, res$max), mean, na.rm=T)[,4]
        a1$M.setheta.dep <- stats::aggregate( res[, c("setheta.dep" )],
                    list( res$missing.pattern,res$score, res$max), mean, na.rm=T )[,4]
        a1$M.setheta.ind <- stats::aggregate( res[, c("setheta.ind" )],
                    list( res$missing.pattern,res$score, res$max), mean, na.rm=T)[,4]
        if (print.summary){
            cat("\n..................................................................\n")
            cat("Mean percentage standard error inflation\n\n")
            a4 <- stats::aggregate( res[,"seinflat"], list( res$missing.pattern), mean, na.rm=T)
            a4[,2] <- round( 100*(a4[,2] - 1), 2 )
            colnames(a4) <- c("missing.pattern", "Mperc.seinflat")
            print(a4)
            cat("\n..................................................................\n")
            cat("Summary theta estimation\n\n")
            a1b <- a1
            a1b[,seq(5,ncol(a1))] <- round( a1[, seq(5,ncol(a1)) ], 4 )
            print(a1b)
                }
        # person parameter estimates

        ind <- match( raschcopula.object$datalist$pattern.in.data, res0$pattern )

        # results
        res <- list( "person"=res0[ ind, ], "se.inflat"=a4,
                        "theta.table"=res0, "pattern.in.data"=raschcopula.object$datalist$pattern.in.data,
                        "summary.theta.table"=a1)
        return(res)
           }
#########################################################################################




##################################################################################
# product of rows in a matrix m1 | bundlewise calculated by a vector v1
.rowProds.bundle <- function( m1, v1 ){
    L1 <- length(v1)
    m1prod <- matrix( 0, nrow=nrow(m1), ncol=L1 )
    v1min <- c(1, cumsum(v1)[ - L1 ]+1 )
    v1max <- cumsum(v1)
    for (ll in 1:L1){
        if (v1[ll] > 1 ){
                m1prod[,ll] <- rowProds( m1[, v1min[ll]:v1max[ll] ] )
                    } else { m1prod[ll] <- m1[, v1min[ll] ] }
                        }
    m1prod
        }
##################################################################################
.rowProds2.bundle <- function( m1, v1 ){
    L1 <- length(v1)
    m1prod <- matrix( 0, nrow=nrow(m1), ncol=L1 )
    v1min <- c(1, cumsum(v1)[ - L1 ]+1 )
    v1max <- cumsum(v1)
    for (ll in 1:L1){
        if (v1[ll] > 1 ){
                m1prod[,ll] <- rowProds2( m1[, v1min[ll]:v1max[ll] ] )
                    } else { m1prod[ll] <- m1[, v1min[ll] ] }
                        }
    m1prod
        }
#*********************************************************************************
##################################################################################
# product of rows in a matrix m1 | bundlewise calculated by a vector v1
.rowMins2.bundle <- function( m1, v1 ){
    L1 <- length(v1)
    m1min <- matrix( 0, nrow=nrow(m1), ncol=L1 )
    v1min <- c(1, cumsum(v1)[ - L1 ]+1 )
    v1max <- cumsum(v1)
    m1min[, which(v1==1)] <- m1[, v1min[ v1==1 ] ]
    for (ll in (1:L1)[ v1 > 1] ){
                m1min[,ll] <- rowMins2( m1[, v1min[ll]:v1max[ll] ] )
                        }
    m1min
        }
##################################################################################

.rowMins2cpp.bundle <- function( m1, v1 ){
    m1min <- rowmins2_bundle_C( m1, v1 )
    return(m1min)
        }



#************************************************************************
# Function rowProds2
rowProds2 <- function(matr){
    y <- matr[,1]
    for (ii in 2:dim(matr)[2]){
        y <- y * matr[,ii] }
    return(y)
        }
#...................................................................

## File Name: rasch_mml2_calcprob_missing1.R
## File Version: 1.092



#**** calculation of probabilities in the missing data IRT model
rasch_mml2_calcprob_missing1 <- function( theta.k, b, beta, delta.miss, pjk,
        fixed.a=NULL, return_nonresponse_probs=FALSE, irtmodel="missing1")
{
    I <- length(b)
    if (is.null(fixed.a)){
        fixed.a <- rep(1,I)
    }
    TP <- nrow(theta.k)

    thetaM1 <- sirt_matrix2(theta.k[,1], nrow=I)
    thetaM2 <- sirt_matrix2(theta.k[,2], nrow=I)
    ab <- fixed.a*b
    M1 <- stats::plogis(fixed.a*thetaM1-ab)  # probability for correct item response

    if (irtmodel=="missing1"){
        # probability of a response for incorrect item responses
        M2a <- stats::plogis(thetaM2-beta)
        # probability of a response for correct item responses
        M2b <- stats::plogis(thetaM2-beta-delta.miss)
    } else {
        M2a <- stats::plogis(thetaM2-beta+delta.miss)
        M2b <- stats::plogis(thetaM2-beta)
    }
    pjk[,1,] <- 1 - M1
    pjk[,2,] <- M1
    # compute joint probability (R,Y)
    pjk[,1,] <- M2a*pjk[,1,]
    pjk[,2,] <- M2b*pjk[,2,]
    # P(R=0)=P(R=0|Y=0)P(Y=0)+P(R=0|Y=1)P(Y=1)
    m3 <- pjk[,3,] <- (1-M2a)*(1-M1) + (1-M2b)*M1
    if (return_nonresponse_probs){
        pjk[,1,] <- (1-M2a)*(1-M1) / m3
        pjk[,2,] <- (1-M2b)*M1 / m3
        pjk <- pjk[,1:2,]
    }
    return(pjk)
}


.calcprob.missing1 <- rasch_mml2_calcprob_missing1

## File Name: qmc.nodes.R
## File Version: 0.15

qmc.nodes <- function( snodes, ndim )
{
    TAM::require_namespace_msg("sfsmisc")
    r1 <- sfsmisc::QUnif(n=snodes, min=0, max=1, n.min=1, p=ndim, leap=409)
    theta <- as.matrix( stats::qnorm(r1) )
    return(theta)
}

## File Name: rm_smooth_distribution.R
## File Version: 0.261

rm_smooth_distribution <- function( theta.k, pi.k, est.mean=FALSE,
            skillspace="normal", est.sigma=TRUE, sigma=NULL )
{
    m2 <- 0
    if ( est.mean ){
        m2 <- sum( theta.k * pi.k )
    }
    if (est.sigma){
        w2 <- sum( theta.k^2 * pi.k ) - m2^2
        sigma <- sqrt(w2)
    }
    if ( skillspace=='normal' ){
        pi.k <- sirt_dnorm_discrete(x=theta.k, mean=m2, sd=sigma)
    }
    res <- list( mu=m2, sigma=sigma, pi.k=pi.k)
    return(res)
}

rm.smooth.distribution <- rm_smooth_distribution

## File Name: xxirt_compute_prior_Theta_from_x.R
## File Version: 0.07

xxirt_compute_prior_Theta_from_x <- function(x, em_args)
{
    # include parameters
    customTheta <- xxirt_parTheta_include_freeParameters(
                                customTheta=em_args$customTheta,
                                x=x[ em_args$parindex_Theta ])

    #*** compute prior distribution
    prior_Theta <- xxirt_compute_priorDistribution( Theta=em_args$Theta,
                                  customTheta=customTheta, G=em_args$G )

    return(prior_Theta)
}

## File Name: lsem_bootstrap_inference.R
## File Version: 0.112

lsem_bootstrap_inference <- function(parameters_boot, est, repl_factor=NULL,
        bc_square=NULL)
{
    R <- ncol(parameters_boot)
    est_boot <- rowMeans(parameters_boot, na.rm=TRUE)
    if (is.null(repl_factor)){
        repl_factor <- 1/(R-1)
    }
    se_boot <- sqrt( rowSums( ( parameters_boot - est_boot )^2 ) * repl_factor )
    bias_boot <- (est_boot - est)*repl_factor*(R-1)
    est_bc <- est - bias_boot
    if (!is.null(bc_square)){
        pb2 <- parameters_boot[bc_square,,drop=FALSE]^2
        est_boot <- rowMeans( pb2, na.rm=TRUE )
        bias_boot <- (est_boot - est[bc_square]^2)*repl_factor*(R-1)
        v1 <- est[bc_square]^2 - bias_boot
        est_bc[bc_square] <- ifelse( v1>0, sqrt(v1), 0 )
    }

    #-- output
    res <- list(mean_boot=est_boot, se_boot=se_boot, est_bc=est_bc,
                bias_boot=bias_boot, est=est)
    return(res)
}

## File Name: noharm_sirt_optim_gradient_R_der_gamma_item.R
## File Version: 0.07


noharm_sirt_optim_gradient_R_der_gamma_item <- function(parm_table_free, Fmat,
    Pmat, FP, npar, NH, I)
{
    grad_gamma_diag <- matrix(0, nrow=I, ncol=npar)
    grad_gamma_diag_bool <- matrix(FALSE, nrow=I, ncol=npar)
    for (iii in 1L:I){
        grad <- rep(0,npar)
        grad_bool <- rep(FALSE,npar)
        for (hh in 1L:NH){
            mat_hh <- parm_table_free[hh,'mat']
            par_index_hh <- parm_table_free[hh,'index']
            row <- parm_table_free[hh,'row']
            col <- parm_table_free[hh,'col']
            # F
            if (mat_hh=='F'){
                if (row==iii){
                    der <- 2*FP[iii,col]
                    grad[par_index_hh] <- grad[par_index_hh] + der
                    grad_bool[par_index_hh] <- TRUE
                }
            }
            # P
            if (mat_hh=='P'){
                der <- 0
                if (row==col){
                    der <- Fmat[iii,col]^2
                    grad[par_index_hh] <- grad[par_index_hh] + der
                } else {
                    der <- Fmat[iii,row]*Fmat[iii,col]
                    grad[par_index_hh] <- grad[par_index_hh] + der
                }
                if (der!=0){ grad_bool[par_index_hh] <- TRUE }
            }
        }
        grad_gamma_diag[iii,] <- grad
        grad_gamma_diag_bool[iii,] <- grad_bool
    }

    #-- output
    res <- list(grad_gamma_diag=grad_gamma_diag,
                    grad_gamma_diag_bool=grad_gamma_diag_bool)
    return(res)
}

## File Name: linking_haebara_optim_function_R.R
## File Version: 0.174


linking_haebara_optim_function_R <- function(NI, NS, dist, aM, bM, theta,
    prob_theta, est_pars, wgtM, a, b, mu, sigma, eps, pow=1)
{
    # logit(p)=a*(th-b)
    # th=SIG*TH+MU=> logit(p)=a*(SIG*TH+MU-b)=a*SIG*(TH-(-MU)/SIG-b/SIG)
    val <- 0
    for (ii in 1L:NI){
        for (ss in 1L:NS){
            if (est_pars[ii,ss]){
                p_obs <- stats::plogis( aM[ii,ss] * (theta - bM[ii,ss] ) )
                a_exp <- a[ii] * sigma[ss]
                b_exp <- ( b[ii] - mu[ss] ) / sigma[ss]
                p_exp <- stats::plogis( a_exp*theta - a[ii]*( b[ii] - mu[ss] ) )
                dist2 <- (p_obs - p_exp)^2
                if (dist=='L2'){
                    dist1 <- sum( dist2*prob_theta )
                }
                if (dist=='L1'){
                    dist1 <- sum( sqrt( dist2 + eps )*prob_theta )
                }
                if (dist=='Lp'){
                    dist1 <- sum( ( dist2 + eps )^(pow/2)*prob_theta )
                }
                val <- val + wgtM[ii,ss]*dist1
            }
        }
    }
    #--- output
    return(val)
}

## File Name: linking_2groups.R
## File Version: 0.139

linking_2groups <- function(pars, method, type="asymm", pow=2, eps=0.001,
            simultan=FALSE, Theta=seq(-6,6,len=101), wgt=NULL, par_init=NULL,
            optimizer="nlminb", control_optimizer=list() )
{
    if (is.null(wgt)){ wgt <- rep(1, length(Theta) ) }

    #- process item parameters
    pars <- pars[, c('a1', 'b1', 'a2', 'b2' ) ]
    pars <- as.matrix(na.omit(pars))
    I <- nrow(pars)

    #- initial parameters
    if (is.null(par_init)){
        par <- c(mu=0,sig=1)
        if (simultan){
            par1 <- linking_2groups_vector_with_names(x=pars[,1],
                            names=paste0('a_',1L:I))
            par2 <- linking_2groups_vector_with_names(x=pars[,2],
                            names=paste0('b_',1L:I))
            par <- c(par, par1, par2)
        }
    } else {
        par <- par_init
    }

    #-- define linking functions
    linking_functions <- list(
        SL=linking_2groups_stocking_lord_fun,
        Hae=linking_2groups_haebara_fun
    )

    grad_functions <- list(
        SL=linking_2groups_stocking_lord_grad,
        Hae=linking_2groups_haebara_grad
    )

    link_fun <- linking_functions[[method]]
    grad_fun <- grad_functions[[method]]  # Will be NULL if method !='Hae'

    #- check gradient
    check_gradient <- FALSE
    # check_gradient <- TRUE

    if (check_gradient){
        requireNamespace('miceadds')
        par[1] <- 1
        par[2] <- 1.2
        if (simultan){
            set.seed(78)
            ind <- 2+I+1L:I
            par2[ind] <- par2[ind] + stats::rnorm(I, sd=1)
        }

        args2 <- list(x=par, pars=pars, Theta=Theta, wgt=wgt, type=type,
                            pow=pow, eps=eps, simultan=simultan )
        f1 <- do.call( what=link_fun, args=args2)
        miceadds::Revalpr('f1')
        h <- 1e-4
        grad <- linking_2groups_numerical_gradient(fun=link_fun, args=args2, h=h)
        miceadds::Revalpr('grad')
        f0 <- do.call( what=grad_fun, args=args2)
        miceadds::Revalpr('f0')
        miceadds::Revalpr('f0-grad')
        miceadds::Revalpr_maxabs('f0','grad')
        stop()
    }

    #* arguments for optimizer
    args <- list( optimizer=optimizer, par=par, fn=link_fun, grad=grad_fun,
                        method='L-BFGS-B', hessian=FALSE, control=control_optimizer,
                        pars=pars, Theta=Theta, wgt=wgt, type=type,
                        pow=pow, eps=eps, simultan=simultan)

    #* optimize link function
    res <- do.call(what=sirt_optimizer, args=args)

    #-- output
    return(res)

}

## File Name: nedelsky.irf.R
## File Version: 0.122




#---- Item response function for the Nedelsky model
nedelsky.irf <- function( Theta, K, b, a, tau, combis, thdim=1)
{
    # probabilities for category response function
    C1 <- nrow(combis)
    TP <- nrow(Theta)
    prob.cats <- matrix(NA,nrow=TP, ncol=K)
    for (cc in 1L:K){
        prob.cats[,cc] <- stats::plogis( a*Theta[,thdim ] - b[cc] )
    }
    prob.latclasses <- matrix(1, nrow=TP, ncol=C1)
    # probabilities of latent classes
    for (kk in 1L:K){
        p1 <- prob.cats[,kk]
        p1 <- cbind( 1 - p1, p1 )
        p1M <- p1[, as.vector(combis[,kk])+1 ]
        prob.latclasses <- prob.latclasses * p1M
    }
    combs <- 1 - cbind( 0, combis )
    combs <- combs*matrix( tau, nrow=C1, ncol=K+1, byrow=TRUE )
    combs <- combs / rowSums(combs)
    # calculate category probabilities
    probs <- prob.latclasses %*% combs
    eps <- 1E-4
    probs[ probs < 0 ] <- eps
    probs[ probs > 1 ] <- 1 - eps
    probs <- probs + eps
    probs <- probs / rowSums(probs )
    res <- list(probs=probs, prob.latclasses=prob.latclasses )
    return(res)
}

## File Name: Q3.R
## File Version: 1.21

###############################################################
# Yen's Q3 statistic (1984)
Q3 <- function( dat, theta, b, progress=TRUE )
{
    I <- ncol(dat)
    if (progress){
        cat("Yen's Q3 Statistic based on an estimated theta score \n*** " )
        cat(I, "Items | " )
    }
    # expected probability
    expected <- .prob.rasch( theta, b )
    # residual
    residual <- dat - expected
    # initialize matrix of Q3 values
    I <- ncol(dat)
    q3.matr <- matrix( NA, nrow=I,ncol=I )
    q3.long <- matrix( NA, nrow=(I-1)*I/2, ncol=4 )
    colnames(q3.matr) <- rownames(q3.matr) <- colnames(dat)
    q3.matr <- stats::cor( residual, use="pairwise.complete.obs")
    nares <- 1 - is.na(residual)
    NIP <- crossprod(nares)
    itempairs <- t( utils::combn( I, 2 ) )
    q3.long[,3] <- q3.matr[ itempairs ]
    q3.long[,4] <- NIP[ itempairs ]
    q3.long[,1] <- colnames(q3.matr)[ itempairs[,1] ]
    q3.long[,2] <- colnames(q3.matr)[ itempairs[,2] ]
    q3.long <- as.data.frame( q3.long )
    q3.long[,3] <- as.numeric( paste( q3.long[,3] ))
    q3.long <- q3.long[ order( q3.long[,3] ), ]
    colnames(q3.long) <- c("Item1", "Item2", "Q3", "N" )
    q3.long <-   q3.long[ !is.na( q3.long[,3] ), ]
    MQ3 <- mean( q3.long[,3] )
    SDQ3 <- stats::sd( q3.long[,3] )
    Q3.stat <- stats::quantile( q3.long[,3], prob=c(  .10, .25, .50, .75, .90  ) )
    Q3.stat <- c("M"=MQ3, "SD"=SDQ3, "Min"=min(q3.long[,3] ),
                        Q3.stat, "Max"=max(q3.long[,3] ) )
    if (progress){
        cat( paste( nrow(q3.long), "item pairs\n" ) )
        cat("*** Q3 Descriptives\n")
        print( round(Q3.stat,3))
    }
    #--- OUTPUT
    res <- list( "q3.matrix"=q3.matr, "q3.long"=q3.long,
                "expected"=expected, "residual"=residual, "Q3.stat"=Q3.stat )
    return(res)
}
#*****************************************************************************************************************

# Q3 <- yen.q3

## File Name: smirt_squeeze.R
## File Version: 0.08

smirt_squeeze <- function( val, lower, upper, est)
{
    D <- 1
    is_matrix <- FALSE
    if ( is.matrix(est) ){
        D <- ncol(est)
        is_matrix <- TRUE
    }
    if( ! is.matrix(val) ){
        val <- matrix(val, ncol=1)
    }
    est <- matrix( est, ncol=D)
    val0 <- val
    for (dd in 1:D){
        val[,dd] <- ifelse( val[,dd] < lower, lower, val[,dd] )
        val[,dd] <- ifelse( val[,dd] > upper, upper, val[,dd] )
        ind_dd <- which(est[,dd]==0)
        if ( length(ind_dd) > 0 ){
            val[ ind_dd, dd] <- val0[ ind_dd,dd]
        }
    }
    if ( ! is_matrix){
        val <- val[,1]
    }
    return(val)
}

## File Name: xxirt_createThetaDistribution.R
## File Version: 0.171

xxirt_createThetaDistribution <- function( par, est, P, prior=NULL,
        prior_par1=NULL, prior_par2=NULL, lower=NULL, upper=NULL,
        X=NULL )
{
    res <- list()
    res$par <- par
    res$est <- est
    res$P <- P
    NP <- length(par)
    res$prior <- prior
    res$prior_par1 <- prior_par1
    res$prior_par2 <- prior_par2
    res$X <- X
    if (! is.null(X) ){
        res$person_covariates <- TRUE
    } else {
        res$person_covariates <- FALSE
    }

    NPT <- sum(est)
    np1 <- which(est)
    if (is.null(lower)){
        lower <- rep(-Inf,NPT)
    } else {
        lower <- lower[np1]
    }
    if (is.null(upper)){
        upper <- rep(Inf,NPT)
    } else {
        upper <- upper[np1]
    }
    names(lower) <- names(upper) <- np1
    res$lower <- lower
    res$upper <- upper
    res$some_bound <- ( sum(res$lower>-Inf)+sum(res$upper<Inf) )>0
    class(res) <- 'ThetaDistribution'
    return(res)
}

## File Name: xxirt_partable_include_freeParameters.R
## File Version: 0.11



xxirt_partable_include_freeParameters <- function( partable, x )
{
    vals <- x[ partable$parindex ]
    ind <- is.na(vals)
    vals[ ind ] <- partable$value[ ind ]
    partable$value <- vals
    return(partable)
}

## File Name: summary.rm.sdt.R
## File Version: 1.34


#*******************************************************
# Summary for rm.facets object                         *
summary.rm.sdt <- function( object, file=NULL, ...)
{

    # open sink for a file
    sirt_osink( file=file)

    cat("-----------------------------------------------------------------\n")
    #- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object)


    cat("Hierarchical Rater Model: Signal Detection Model \n")
    cat("-----------------------------------------------------------------\n")
    cat( "Iteration with minimal deviance","=", object$iter_opt, "\n" )
    cat( "Deviance","=", round( object$deviance, 2 ), " | " )
    cat( "Log Likelihood", "=",  round( -object$deviance/2, 2 ), "\n" )
    cat( "Number of persons", "=", object$ic$n, "\n" )
    cat( "Number of items  ", "=", object$ic$VV, "\n" )
    cat( "Number of raters ", "=", object$ic$RR, "\n" )
    cat( "Number of estimated parameters", "=", object$ic$np, "\n" )
    cat( "   Distribution parameters", "=", object$ic$np.skill, "\n" )
    cat( "   Item parameters        ", "=", object$ic$np.item, "\n" )
    cat( "   Rater parameters       ", "=", object$ic$np.rater, "\n" )
    cat("\n")

    #--- information criteria
    rm_summary_information_criteria(object=object)

    cat("-----------------------------------------------------------------\n")
    cat( "Trait Distribution\n",
            "Mean","=", round(object$mu,3), " SD=", round( object$sigma, 3) )

    if ( object$skillspace=="discrete" ){
        cat("\n\nDiscrete Skill Distribution\n")
        obji <- object$skill.distribution
        sirt_summary_print_objects(obji=obji, digits=4, from=1)
    }


    cat( "\n\nEAP Reliability=")
    cat(round( object$EAP.rel,3 ) )
    cat( "\n")

    cat("-----------------------------------------------------------------\n")
    cat("Item Parameters \n\n")
    cat("Item response function:", object$link_item, "\n\n")
    obji <- object$item
    sirt_summary_print_objects(obji=obji, digits=3, from=2)

    cat("-----------------------------------------------------------------\n")
    cat("Rater Parameters \n")
    obji <- object$rater
    sirt_summary_print_objects(obji=obji, digits=3, from=2)

    sirt_csink(file=file)
}
#*******************************************************

## File Name: summary.rasch.pml.R
## File Version: 0.161


#*******************************************************
# Summary for rasch.copula object                         *
##NS S3method(summary,rasch.pml)
summary.rasch.pml <- function( object, ...){
    cat("-----------------------------------------------------------------\n")
    d1 <- utils::packageDescription("sirt")
    cat( paste( d1$Package, " ", d1$Version, " (", d1$Date, ")", sep=""), "\n\n" )
    cat( "Date of Analysis:", paste( object$s2 ), "\n" )
    cat("Computation Time:", print(object$s2 - object$s1), "\n\n")
    cat("  Function", object$fct, " \n")
    cat("-----------------------------------------------------------------\n")
    cat("Pairwise Marginal Likelihood Estimation \n")
    cat(paste("Link function:", object$link, "\n"))
    cat("-----------------------------------------------------------------\n")
    cat( "Pseudolikelihood objective function=", round( object$ic$deviance, 2 ), "\n" )
    cat( "Number of persons=", object$ic$n, "\n" )
    cat( "Number of estimated parameters=", object$ic$np, "\n" )
    cat( "Number of used item pairs in PML estimation=", nrow(object$itempairs), "\n" )
#    cat( "AIC=", round( object$ic$AIC, 2 ), " ; penalty=", round( object$ic$AIC - object$ic$deviance,2 ), "\n" )
    cat( "PLIC=", round( object$ic$PLIC, 2 ), " ; penalty=",
                round( object$ic$PLIC - object$ic$deviance,2 ),
                " | PLIC=-2*PL + log(n)*p \n" )
    cat( "      (Pseudolikelihood information criterion)\n")
#    cat( "CAIC=", round( object$ic$CAIC, 2 )," ; penalty=", round( object$ic$CAIC - object$ic$deviance,2 ), "\n\n" )
#    cat( "Trait Distribution (", length(object$theta.k), " Knots )\n",
#              "Mean=", 0, " SD=", 1, "\n")

    cat("-----------------------------------------------------------------\n")
    # print item summary
    cat("Item Parameter Summary\n")
    cat( " Estimated", length(object$bG)+length(object$aG), "Item Parameters\n\n")
    sirt_print_helper( object$item, digits=3 )        # print item statistics
    #....................................................................
    # print Trait parameter summary
    cat("-----------------------------------------------------------------\n")
    cat("Trait SD (Probit Link): ")
    cat( round(object$sigma, 3 ), "\n")
    cat("Trait SD (Logit Link) : ")
    cat( round( object$item$sigma[1] * 1.701, 3 ), "\n")
    if( object$fct    =="rasch.pml3" ){
        cat("\nGreen-Yang Reliability \n")
        cat( "omega=", round( object$omega.rel, 3 ), "\n")
                }

    cat("-----------------------------------------------------------------\n")

    if ( object$est.corrs){
    cat("-----------------------------------------------------------------\n")
    cat("Residual Correlation Parameter Summary\n")
        cat( " Estimated", length(object$epsG), "Residual Correlation Parameters\n\n")
       sirt_print_helper( object$eps.corrM, digits=3 )        # print item statistics
            }
        }
#*******************************************************

## File Name: linking.haebara.R
## File Version: 0.434

linking.haebara <- function(itempars, dist="L2", theta=seq(-4,4, length=61),
        optimizer="optim", center=FALSE, eps=1e-3, par_init=NULL,
        use_rcpp=TRUE, pow=2, use_der=TRUE, ...)
{
    CALL <- match.call()
    s1 <- Sys.time()

    #--- process item parameters
    res <- linking_proc_itempars(itempars=itempars)
    itempars <- res$itempars
    NS <- res$NS
    NI <- res$NI
    items <- res$items
    studies <- res$studies
    wgtM <- res$wgtM
    aM <- res$aM
    bM <- res$bM
    est_pars <- res$est_pars
    weights_exist <- res$weights_exist
    is_1pl <- res$is_1pl
    a.orig <- aM
    b.orig <- bM
    prob_theta <- sirt_dnorm_discrete(x=theta, mean=0, sd=1)

    #* define parameter vector
    par <- c(rep(1,NI), rep(0,NI), rep(0,NS-1), rep(1, NS-1))
    parnames <- c( paste0('a_',items), paste0('b_',items), paste0('mu_',studies[-1]),
                    paste0('sigma_',studies[-1]) )
    names(par) <- parnames
    index_a <- 1L:NI
    index_b <- NI + 1L:NI
    index_mu <- 2*NI + 1L:(NS-1)
    index_sigma <- 2*NI + NS - 1 + 1L:(NS-1)
    NP <- length(par)

    #-- initial values
    b_mean <- colMeans(bM, na.rm=TRUE)
    par[index_mu] <- - ( b_mean - b_mean[1] )[-1]
    bM_centered <- apply( bM, 2, FUN=function(vv){ vv - mean(vv, na.rm=TRUE) } )
    par[index_b] <- rowMeans(bM_centered, na.rm=TRUE)
    if (!is.null(par_init)){
        par <- par_init
    }

    #-- define optimization function
    if (use_rcpp){
        fct_optim_call <- sirt_rcpp_linking_haebara_fct_optim
        grad_optim_call <- sirt_rcpp_linking_haebara_grad_optim
        index_a_ <- index_a - 1
        index_b_ <- index_b - 1
        index_mu_ <- index_mu - 1
        index_sigma_ <- index_sigma - 1
    } else {
        fct_optim_call <- linking_haebara_optim_function_R
        grad_optim_call <- linking_haebara_gradient_function_R
        index_a_ <- index_a
        index_b_ <- index_b
        index_mu_ <- index_mu
        index_sigma_ <- index_sigma
    }

    fct_optim <- function(x){
        a <- x[index_a]
        b <- x[index_b]
        mu <- c(0, x[index_mu])
        sigma <- c(1, x[index_sigma])
        if (is_1pl){
            a <- rep(1,NI)
            sigma <- 1+0*sigma
        }
        args <- list( NI=NI, NS=NS, dist=dist, aM=aM, bM=bM,
                    theta=theta, prob_theta=prob_theta, est_pars=est_pars, wgtM=wgtM,
                    a=a, b=b, mu=mu, sigma=sigma, eps=eps, pow=pow )
        val <- do.call(what=fct_optim_call, args=args)
        return(val)
    }
    grad_optim <- function(x){
        a <- x[index_a]
        b <- x[index_b]
        mu <- c(0, x[index_mu])
        sigma <- c(1, x[index_sigma])
        if (is_1pl){
            a <- rep(1,NI)
            sigma <- 1+0*sigma
        }
        args <- list( NI=NI, NS=NS, dist=dist, aM=aM, bM=bM, theta=theta,
                    prob_theta=prob_theta, est_pars=est_pars, wgtM=wgtM, a=a, b=b,
                    mu=mu, sigma=sigma, eps=eps, index_a=index_a_, index_b=index_b_,
                    index_mu=index_mu_, index_sigma=index_sigma_,
                    parnames=parnames, NP=NP, pow=pow )
        grad <- do.call(what=grad_optim_call, args=args)
        if (is_1pl){
            grad[index_a] <- 0
            grad[index_sigma] <- 0
        }
        names(grad) <- parnames
        return(grad)
    }
    if (!use_der){
        grad_optim <- NULL
    }

    #-- do optimization
    lower <- rep(-Inf, NP)
    upper <- rep(Inf, NP)
    b_max <- Inf
    a_min <- -Inf
    lower[index_sigma] <- 1e-3
    lower[index_a] <- a_min
    lower[index_b] <- -b_max
    upper[index_b] <- b_max
    args <- list(...)
    args$optimizer <- optimizer
    args$par <- par
    args$fn <- fct_optim
    args$grad <- grad_optim
    args$hessian <- FALSE
    if ( ! ( 'lower' %in% names(args) ) ){
        args$lower <- lower
    }
    if ( ! ( 'upper' %in% names(args) ) ){
        args$upper <- upper
    }
    if ('method' %in% (names(args)) ){
        if (! ( args$method %in% c('L-BFGS-B','Brent'))) {
            args$lower <- NULL
            args$upper <- NULL
        }
    }
    res_optim <- do.call(what=sirt_optimizer, args=args )
    x <- res_optim$par
    a <- x[index_a]
    b <- x[index_b]
    mu <- c(0, x[index_mu])
    sigma <- c(1, x[index_sigma])

    #-- estimate centered parameters
    res <- invariance_alignment_center_parameters(alpha0=mu, psi0=sigma, center=center)
    mu <- res$alpha0
    sigma <- res$psi0
    pars <- data.frame(study=studies, mu=mu, sigma=sigma)
    rownames(pars) <- NULL

    #-- estimated item parameters
    item <- data.frame(item=items, a=a, b=b)
    numb_items <- colSums(est_pars)

    #-- estimated DIF effects
    muM <- sirt_matrix2(x=mu, nrow=NI)
    sigmaM <- sirt_matrix2(x=sigma, nrow=NI)
    a.resid <- aM - a*sigmaM
    # th=SIG*TH+MU=> logit(p)=a*(SIG*TH+MU-b)=a*SIG*(TH-(-MU)/SIG-b/SIG)
    b.resid <- bM - ( b - muM )/sigmaM

    #-- output
    s2 <- Sys.time()
    time_diff <- s2-s1
    res <- list( pars=pars, item=item, mu=mu, sigma=sigma, a=a, b=b,
                    a.resid=a.resid, b.resid=b.resid, res_optim=res_optim,
                    est_pars=est_pars, numb_items=numb_items, center=center,
                    dist=dist, eps=eps, NP=NP, use_rcpp=use_rcpp,
                    s1=s1, s2=s2, time_diff=time_diff, CALL=CALL)
    class(res) <- 'linking.haebara'
    return(res)
}

## File Name: lsem_fitsem_raw_data_lavaan_survey.R
## File Version: 0.095

lsem_fitsem_raw_data_lavaan_survey <- function(dat, lavmodel, lavfit)
{
    TAM::require_namespace_msg('lavaan.survey')
    TAM::require_namespace_msg('survey')
    datsvy <- survey::svydesign(id=~index, weights=~weight, data=dat)
    assign_args <- list( x='lavmodel__', value=lavmodel, pos=1)
    res0 <- do.call( what='assign', args=assign_args)
    what <- paste0('lavaan.survey','::', 'lavaan.survey')
    args <- list(lavaan.fit=lavfit,    survey.design=datsvy )
    survey.fit <- do.call(what=what, args=args)
    # survey.fit <- lavaan.survey::lavaan.survey(lavaan.fit=lavfit,
    #                            survey.design=datsvy )
    return(survey.fit)
}

## File Name: gom_em_loglike_calc_probs.R
## File Version: 0.03

gom_em_loglike_calc_probs <- function(x, ind_pi, ind_lambda, I, K, theta.k,
    theta0.k)
{
    pi_k_logit <- x[ind_pi]
    lambda_logit <- x[ind_lambda]
    pi.k <- sirt_logit_to_probs(y=pi_k_logit)
    lambda <- gom_em_extract_lambda_matrix(lambda_logit=lambda_logit, I=I, K=K)
    res <- gom_em_calc_probs( lambda=lambda, theta.k=theta.k, b=NULL,
                                theta0.k=theta0.k )
    probs <- res$probs
    return(probs)
}

## File Name: mgsem_suffstat_covariance_matrix.R
## File Version: 0.101

mgsem_suffstat_covariance_matrix <- function(suffstat)
{
    G <- length(suffstat)
    dfr <- NULL

    for (gg in 1L:G){
        suffstat_gg <- suffstat[[gg]]
        names_gg <- names(suffstat_gg)
        mu <- suffstat_gg[[ intersect( c('mu','M'), names_gg) ]]
        Sigma <- suffstat_gg[[ intersect( c('S','Sigma'), names_gg) ]]
        N <- suffstat_gg$N
        vars <- names(mu)
        I1 <- length(mu)
        if (is.null(vars)){
            vars <- paste0('V',1L:I1)
        }
        #** mu
        label <- paste0('mu_G', gg, '_', vars)
        se <- sqrt( diag(Sigma) / N )
        dfr1 <- data.frame(type='mu', group=gg, index1=1L:I1, index2=NA, label=label,
                    par=mu, se=se)
        V1 <- Sigma / N
        rownames(V1) <- colnames(V1) <- label

        #** sigma
        c1 <- rbind( 1L:I1, 1L:I1)
        c2 <- utils::combn(x=I1, m=2)
        c3 <- rbind( t(c1), t(c2))
        c3 <- c3[ order(c3[,1]), ]
        label <- paste0('Sigma_G', gg, '_', vars[c3[,1]], '_', vars[c3[,2]] )
        par <- Sigma[ c3[,1L:2] ]

        #* compute duplication matrix
        K <- mgsem_duplication_matrix(x=Sigma)

        dfr2 <- data.frame(type='sigma', group=gg, index1=c3[,1], index2=c3[,2],
                        label=label, par=par)
        N2 <- nrow(dfr2)
        V2 <- matrix(0, nrow=N2, ncol=N2)
        V2 <- 2* ( K %*% ( kronecker(X=Sigma, Y=Sigma) ) %*% t(K) ) / N
        rownames(V2) <- colnames(V2) <- label
        dfr2$se <- sqrt(diag(V2))
        V3 <- mgsem_bdiag(x1=V1, x2=V2)
        dfr <- rbind(dfr, dfr1, dfr2)
        if (gg==1){
            V <- V3
        } else {
            V <- mgsem_bdiag(x1=V, x2=V3)
        }

    }    # end gg

    #--- output
    res <- list(suffstat_pars=dfr, suffstat_vcov=V)
    return(res)
}

## File Name: R2noharm.jackknife.R
## File Version: 1.13


#**** jackknife R2noharm
R2noharm.jackknife <- function( object, jackunits=NULL )
{
    # INPUT:
    # object     ... R2noharm object
    # jackunits ... units for jackknife
    #                    can be a number or a vector of jackknife units
    # modtype permitted?
    modtype <- object$modtype
    if (modtype %in% 1:2 ){
            stop("Jackknife not allowed for exploratory factor analysis.\n")
                }
    # define jackknife units
    cat("NOHARM4\nJackknife Estimation of Standard Errors\n")
    Nobs <- object$Nobs
    if ( length(jackunits) > 1){
            jackunits <- match( jackunits, unique( jackunits) )
                                }
    if ( is.null( jackunits ) ){ jackunits <- seq(1,Nobs) }
    if ( length(jackunits)==1){ jackunits <- 1 + floor( seq( 1, Nobs - .01 ) / ( Nobs/jackunits ) ) }
    # define progress bar
    u.jackunits <- unique( jackunits )
    UJ <- max( u.jackunits)
    cat( UJ, "Jackknife Units are used\n" )
    prbar <- paste( "|", paste(rep("-", 4 ), collapse=""), "PROGRESS",
                    paste(rep("-", 4 ), collapse=""), "|", sep="")
    cat(prbar,"\n")
    #***************
    # initialize matrices
    cat("|")
    j.tanaka <- rep(NA, UJ )
    j.rmsr <- rep(NA, UJ )
    if (modtype %in% 3:4){
        j.chisquare_df <- j.rmsea <- j.rmsr
                        }
    if ( modtype==4){
        j.difficulties <- matrix(NA, nrow=object$Nitems, ncol=UJ )
                            }
    j.uniquenesses <- j.final.constants <- j.thresholds <- j.discriminations <- matrix(NA, nrow=object$Nitems, ncol=UJ )
    j.pm <- matrix( NA, nrow=nrow(object$pm)*ncol(object$pm), ncol=UJ)
    if ( modtype==3){
        j.factor.cor <- matrix( NA, nrow=prod( dim( object$factor.cor) ), ncol=UJ )
        j.loadings <- matrix( NA, nrow=prod( dim( object$loadings) ), ncol=UJ )
        j.loadings.theta <- matrix( NA, nrow=prod( dim( object$loadings.theta) ), ncol=UJ )
                        }
    # calculate jackunits indices when progress should be displayed
    prbar.display <- floor( seq( 1, max(u.jackunits), len=16 ) )
    #**************************
    # BEGIN JACKKNIFE
    for ( uu in u.jackunits ){
        weights.uu <- object$weights[ jackunits !=uu ]
        mod.uu <- R2noharm( dat=object$dat[ jackunits !=uu, ],
                    model.type=object$model.type,
                    weights=weights.uu,
                    guesses=object$guesses, noharm.path=object$noharm.path,
                                F.pattern=object$F.pattern, F.init=object$F.init,
                                P.pattern=object$P.pattern, P.init=object$P.init, digits.pm=5, writename="model_jackknife",
                                display.fit=5, dec=object$dec, display=FALSE  )
        j.tanaka[uu] <- mod.uu$tanaka
        j.rmsr[uu] <- mod.uu$rmsr
        if (modtype %in% 3:4){
            j.rmsea[uu] <- mod.uu$rmsea
            j.chisquare_df[uu] <- mod.uu$chisquare / mod.uu$df
                            }
        if ( modtype==4){
            j.difficulties[,uu] <- mod.uu$difficulties
            j.discriminations[,uu] <- mod.uu$discriminations
                                }
        j.thresholds[,uu] <- mod.uu$thresholds
        j.uniquenesses[,uu] <- mod.uu$uniquenesses
        j.final.constants[,uu] <- mod.uu$final.constants
        if (modtype==3){
            j.factor.cor[,uu] <- matrix( mod.uu$factor.cor, ncol=1 )
            j.loadings[,uu] <- matrix( mod.uu$loadings, ncol=1 )
            j.loadings.theta[,uu] <- matrix( mod.uu$loadings.theta, ncol=1 )
                    }
        j.pm[,uu] <- matrix( as.matrix(mod.uu$pm), ncol=1 )
        if ( uu %in% prbar.display ){ cat("-") ; flush.console() }
                    }
    cat("|\n")
    #***********************
    # collect jackknife parameters
    jackknife.pars <- list( "j.tanaka"=j.tanaka, "j.rmsr"=j.rmsr )
    if (modtype %in% 3:4){
        jackknife.pars$rmsea <- j.rmsea
        jackknife.pars$chisquare_df <- j.chisquare_df
                        }
    if (modtype==4){
        jackknife.pars$j.difficulties=j.difficulties
        jackknife.pars$j.discriminations=j.discriminations
                        }
    jackknife.pars$j.pm=j.pm
    jackknife.pars$j.thresholds=j.thresholds
    if (modtype==3){
        jackknife.pars$j.factor.cor <- j.factor.cor
        jackknife.pars$j.loadings <- j.loadings
        jackknife.pars$j.loadings.theta <- j.loadings.theta
                    }
    #**********************
    # calculate jackknife statistics

    .est.jackknife.bias <- function( jvec, est, UJ ){
        # newly derived estimators
        if ( is.vector(jvec) ){
            Tn <- UJ * est - (UJ - 1) * jvec
            M <- mean(Tn)
#            se <- sqrt( sum( (Tn - M )^2 ) / (UJ-1 ) )
            se <- sqrt( sum( ( jvec - mean(jvec) )^2 ) * (UJ-1 )/UJ )
                            }
        if ( is.matrix(jvec) ){
            Tn <- UJ * est - (UJ - 1) * jvec
            M <- rowMeans(Tn)
#            se <- sqrt( rowSums( (Tn - M )^2 ) / (UJ-1 ) )
            se <- sqrt(  rowSums( ( jvec - rowMeans(jvec) )^2 ) * (UJ-1 )/UJ )
                            }
           res <- list( "est.jack"=M, "se"=se)
                }


    res <- .est.jackknife.bias( jvec=j.tanaka, est=object$tanaka, UJ )
    tanaka.stat <- data.frame( "est"=object$tanaka,
                                "est.jack"=res$est.jack,
                                "se"=res$se
#                                "se"=sqrt( ( UJ - 1 ) / UJ * sum( ( j.tanaka - object$tanaka )^2 ) )
                                        )
    rownames(tanaka.stat) <- "tanaka"

    res <- .est.jackknife.bias( jvec=j.rmsr, est=object$rmsr, UJ )
    rmsr.stat <- data.frame( "est"=object$rmsr,
                                "est.jack"=res$est.jack,
                                "se"=res$se )
    rownames(rmsr.stat) <- "rmsr"

    if (modtype %in% 3:4){

        res <- .est.jackknife.bias( jvec=j.rmsea, est=object$rmsea, UJ )
        rmsea.stat <- data.frame( "est"=object$rmsea,
                                "est.jack"=res$est.jack,
                                "se"=res$se )
        rownames(rmsea.stat) <- "rmsea"

        res <- .est.jackknife.bias( jvec=j.chisquare_df, est=object$chisquare / object$df, UJ )
        chisquare_df.stat <- data.frame( "est"=object$chisquare / object$df,
                                "est.jack"=res$est.jack,
                                "se"=res$se )
        rownames(chisquare_df.stat) <- "chisquare_df"

                        }


    if (modtype==4){


        jvec <- j.difficulties
        est <- object$difficulties
        res <- .est.jackknife.bias( jvec=jvec, est=est, UJ )
        difficulties.stat <- data.frame( "est"=est,
                                "est.jack"=res$est.jack,
                                "se"=res$se )

        jvec <- j.discriminations
        est <- object$discriminations
        res <- .est.jackknife.bias( jvec=jvec, est=est, UJ )
        discriminations.stat <-  data.frame( "est"=est,
                                "est.jack"=res$est.jack,
                                "se"=res$se )
                    }

        jvec <- j.thresholds
        est <- object$thresholds
        res <- .est.jackknife.bias( jvec=jvec, est=est, UJ )
        thresholds.stat <- data.frame( "est"=est, "est.jack"=res$est.jack, "se"=res$se )

        jvec <- j.final.constants
        est <- object$final.constants
        res <- .est.jackknife.bias( jvec=jvec, est=est, UJ )
    final.constants.stat <-data.frame( "est"=est, "est.jack"=res$est.jack, "se"=res$se )

        jvec <- j.uniquenesses
        est <- object$uniquenesses
        res <- .est.jackknife.bias( jvec=jvec, est=est, UJ )
    uniquenesses.stat <- data.frame( "est"=est, "est.jack"=res$est.jack, "se"=res$se )

    if (modtype==3){

        jvec <- j.factor.cor
        est <- as.vector(matrix( object$factor.cor, ncol=1 ))
        res <- .est.jackknife.bias( jvec=jvec, est=est, UJ )
        factor.cor.stat <- data.frame( "est"=est, "est.jack"=res$est.jack, "se"=res$se )
        a1a <- rep( rownames(object$factor.cor), each=ncol( object$factor.cor ) )
        a1b <- rep( colnames(object$factor.cor), nrow( object$factor.cor ) )
        a1 <- paste( a1a, a1b, sep="-" )
        rownames(factor.cor.stat) <- a1

        jvec <- j.loadings
        est <- as.vector(matrix( object$loadings, ncol=1 ))
        res <- .est.jackknife.bias( jvec=jvec, est=est, UJ )
        loadings.stat <- data.frame( "est"=est, "est.jack"=res$est.jack, "se"=res$se )
        a1a <- rep( rownames(object$loadings), ncol( object$loadings ) )
        a1b <- rep( colnames(object$loadings), each=nrow( object$loadings ) )
        a1 <- paste( a1a, a1b, sep="-" )
        rownames(loadings.stat) <- a1

        jvec <- j.loadings.theta
        est <- as.vector(matrix( object$loadings.theta, ncol=1 ))
        res <- .est.jackknife.bias( jvec=jvec, est=est, UJ )
        loadings.theta.stat <- data.frame( "est"=est, "est.jack"=res$est.jack, "se"=res$se )
        a1a <- rep( rownames(object$loadings.theta), ncol( object$loadings.theta ) )
        a1b <- rep( colnames(object$loadings.theta), each=nrow( object$loadings.theta ) )
        a1 <- paste( a1a, a1b, sep="-" )
        rownames(loadings.theta.stat) <- a1

                                }
    #*********************************************
    # collect objects with estimates and standard errors
    se.pars <- list( "tanaka.stat"=tanaka.stat, "rmsr.stat"=rmsr.stat )
    if (modtype %in% 3:4){
        se.pars$rmsea.stat <- rmsea.stat
        se.pars$chisquare_df.stat <- chisquare_df.stat
                }
    if (modtype==4){
        se.pars$difficulties.stat=difficulties.stat
        se.pars$discriminations.stat=discriminations.stat
                        }
    se.pars$thresholds.stat=thresholds.stat
    se.pars$final.constants.stat=final.constants.stat
    se.pars$uniquenesses.stat=uniquenesses.stat
    if (modtype==3){
        se.pars$factor.cor.stat <- factor.cor.stat
        se.pars$loadings.stat <- loadings.stat
        se.pars$loadings.theta.stat <- loadings.theta.stat
                    }
    # create parameter table
    res <- se.pars
    LL <- length(res)
    dfr <- NULL
    for (ll in 1:LL ){
        dfr.ll <- res[[ll]]
        dfr.ll <- data.frame( "type"=names(res)[ll], "parameter"=rownames(dfr.ll), dfr.ll )
        dfr <- rbind( dfr, dfr.ll )
            }
    rownames(dfr) <- NULL

    # output
    res <- list( "partable"=dfr, "se.pars"=se.pars, "jackknife.pars"=jackknife.pars,
            "u.jackunits"=u.jackunits )
    class(res) <- "R2noharm.jackknife"
    return(res)
}

## File Name: xxirt_newton_raphson.R
## File Version: 0.284


xxirt_newton_raphson <- function(em_out, em_args, maxit_nr, optimizer_nr,
            control_nr, verbose=TRUE, estimator="ML")
{
    partable <- xxirt_em_args_extract(em_args=em_args, em_out=em_out,
                            object='partable')
    customTheta <- xxirt_em_args_extract(em_args=em_args, em_out=em_out,
                            object='customTheta')
    G <- em_args$G
    Theta <- em_args$Theta
    items <- em_args$items
    item_list <- em_args$item_list
    ncat <- em_args$ncat
    partable_index <- em_args$partable_index
    dat <- em_args$dat
    dat_resp_bool <- em_args$dat_resp_bool
    resp_index <- em_args$resp_index
    group <- em_args$group
    weights <- em_args$weights

    par1 <- par_items <- xxirt_partable_extract_freeParameters( partable=partable )
    par2 <- par_Theta <- xxirt_parTheta_extract_freeParameters( customTheta=customTheta )
    par0 <- par <- c(par1, par2)

    NPI <- length(par_items)
    NPT <- length(par_Theta)
    NP <- NPI+NPT

    em_args$parindex_items <- 1L:NPI
    if (NPI==0){
        em_args$parindex_items <- NULL
    }

    em_args$parindex_Theta <- (NPI+1):(NPI+NPT)
    x <- par0
    # define parameter tables for estimation
    I <- ncol(dat)
    partable$item_group_comp <- 0
    for (ii in 1L:I){
        ind_ii <- which(partable$itemnr==ii)
        partable$item_group_comp[ ind_ii ] <- 1L:length(ind_ii)
    }
    partable$item_group_comp[ partable$parfree==0 ] <- 0
    t2 <- table(partable$parindex)
    i3 <- as.numeric( names(t2)[ t2 > 1] )
    partable$item_group_comp[ partable$parindex %in% i3 ] <- 0

    MIGC <- max(partable$item_group_comp)

    em_args$partable <- partable
    em_args$MIGC <- MIGC
    em_args$customTheta <- customTheta
    em_args$NPI <- NPI
    em_args$NPT <- NPT
    em_args$NP <- NP

    i1 <- stats::aggregate(partable$itemnr, list(partable$parindex), min )
    i2 <- stats::aggregate(partable$itemnr, list(partable$parindex), max )

    if (NPI>0){
        free_pars_design <- data.frame( pid=1L:NPI, type='item', parlabel=names(par1) )
        free_pars_design$one_item <- i1[,2]==i2[,2]
        free_pars_design$itemnr <- ifelse(free_pars_design$one_item, i1[,2], -9 )
        partable_free <- partable[ partable$parfree==1 & partable$est, ]
        free_pars_design$item_group_comp <- partable_free$item_group_comp
    } else {
        free_pars_design <- NULL
    }

    em_args$free_pars_design <- free_pars_design
    em_args$group0 <- em_args$group - 1

    test <- TRUE
    test <- FALSE

    ##** select functions
    if (estimator=='ML'){
        opt_fun <- xxirt_nr_optim_fun
        grad_fun <- xxirt_nr_grad_fun_Rcpp
        # grad_fun <- NULL
    }
    if (estimator=='PML'){
        opt_fun <- xxirt_nr_pml_opt_fun
        res <- xxirt_nr_pml_more_arguments( em_args=em_args,
                                            grad_fun=xxirt_nr_pml_grad_fun)
        em_args <- res$em_args
        grad_fun <- res$grad_fun
        pml_args <- em_args$pml_args
    }

    # test <- TRUE
    if (test){
        requireNamespace('miceadds')
        ll1 <- opt_fun(x=x, em_args=em_args)
        grad1 <- xxirt_nr_grad_fun_numapprox(x=x, em_args=em_args, opt_fun=opt_fun)
        grad2 <- grad_fun(x=x, em_args=em_args)
        miceadds::Revalpr_round('grad1-grad2',5)
        # stop()
    }

    #-- optimize
    control_nr$maxit <- maxit_nr
    partable_free <- partable[ partable$parfree==1, ]

    if (NPI>0){
        lower <- c( partable_free$lower, customTheta$lower )
        upper <- c( partable_free$upper, customTheta$upper )
    } else {
        lower <- c( customTheta$lower )
        upper <- c( customTheta$upper )
    }
    if (!is.null(lower)){ names(lower) <- names(x) }
    if (!is.null(upper)){ names(upper) <- names(x) }

    if (verbose){
        cat( paste0('****** Newton-Raphson Optimization ********\n'))
        utils::flush.console()
    }
    res_opt_nr <- sirt_optimizer(optimizer=optimizer_nr,
                            par=x, fn=opt_fun, grad=grad_fun, hessian=FALSE,
                            control=control_nr, em_args=em_args,
                            lower=lower, upper=upper )

    #-- collect output
    res_opt_nr$opt_fun <- opt_fun
    res_opt_nr$grad_fun <- grad_fun
    x <- res_opt_nr$par
    partable <- xxirt_partable_include_freeParameters( partable=em_args$partable,
                                        x=x[ em_args$parindex_items ] )
    customTheta <- xxirt_parTheta_include_freeParameters(
                                customTheta=em_args$customTheta,
                                x=x[ em_args$parindex_Theta ])

    #-- optimization function values
    opt_values <- opt_fun(x=x, em_args=em_args, output_all=TRUE)

    #-- item probabilities
    probs_items <- xxirt_compute_itemprobs( item_list=item_list,
                            items=items, Theta=Theta, ncat=ncat,
                            partable=partable, partable_index=partable_index )

    #--- output
    res <- list(res_opt_nr=res_opt_nr, partable=partable, customTheta=customTheta,
                    opt_values=opt_values, probs_items=probs_items,
                    em_args=em_args)
    return(res)
}

## File Name: invariance_alignment_choose_fixed.R
## File Version: 0.03

invariance_alignment_choose_fixed <- function(fixed, G, Gmax=6)
{
    if (is.null(fixed)){
        if (G>Gmax){
            fixed <- FALSE
        } else {
            fixed <- TRUE
        }
    }
    return(fixed)
}

## File Name: mcmc.2pnoh_alg.R
## File Version: 0.17


######################
# draw latent responses Z
.draw.Z.2pnoh <- function( aM, bM, theta, N, I, threshlow, threshupp ){
    # calculate means
    mij <- aM * ( theta - bM )
    # simulate uniform data
    rij <- matrix( stats::runif( N*I ), nrow=N, ncol=I )
    # calculate corresponding value
    pl <- stats::pnorm( threshlow, mean=mij)
    pu <- stats::pnorm( threshupp, mean=mij)
    pij <- pl + (pu-pl)*rij
    # simulate Z
    Zij <- stats::qnorm( pij, mean=mij )
    return(Zij)
        }

########################################
# draw theta
.draw.theta.2pnoh <- function( aM, bM, N, I, Z ){
    vtheta <- 1 / ( rowSums( aM^2 ) + 1  )
    mtheta <- rowSums( aM * ( Z + aM*bM ) ) * vtheta
    theta <- stats::rnorm( N, mean=mtheta, sd=sqrt( vtheta ) )
    v1 <- stats::var(mtheta)
    res <- list("theta"=theta )
    return(res)
            }

##########################################
# draw item parameters in the 2PNOH model
.draw.itempars.2pnoh <- function( theta, Z, I, N, a, b,
        xi, omega, sig, nu, itemgroup, K, Ik,  weights){

    aM <- matrix( a, nrow=N, ncol=I, byrow=TRUE )

    #************************
    # sampling beta parameters
    v.ik <- 1 / ( N * a^2 + 1 / sig^2 )
    if ( is.null(weights ) ){
        m.ik <- a * colSums( aM * theta - Z  ) + xi[ itemgroup ] / sig^2
                    }
    if ( ! is.null(weights ) ){
        m.ik <- a * colSums( weights*(aM * theta - Z)  ) + xi[ itemgroup ] / sig^2
                    }
    m.ik <- m.ik * v.ik
    b <- stats::rnorm( I, mean=m.ik, sd=sqrt(v.ik ) )
    bM <- matrix( b, nrow=N, ncol=I, byrow=TRUE )

    #************************
    # sampling alpha parameters
    v.ik <- 1 / ( colSums( ( theta - bM  )^2 ) + 1 /nu^2 )
    if ( is.null(weights ) ){
        m.ik <- colSums( ( theta - bM) *Z  ) + omega[ itemgroup ] / nu^2
                    }
    if ( ! is.null(weights ) ){
        m.ik <- colSums( weights*( theta - bM) *Z  ) + omega[ itemgroup ] / nu^2
                    }
    m.ik <- m.ik * v.ik
    a <- stats::rnorm( I, mean=m.ik, sd=sqrt(v.ik ) )

    #*********************************
    # sampling of xi parameter
    m.xi <- stats::aggregate( b, list(itemgroup), mean )[,2]
    xi <- stats::rnorm(K, mean=m.xi, sd=sqrt( sig^2 / Ik) )

    #*********************************
    # sampling of omega parameter
    m.xi <- stats::aggregate( a, list(itemgroup), mean )[,2]
    omega <- stats::rnorm(K, mean=m.xi, sd=sqrt( nu^2 / Ik) )

    # output
    return( list( "a"=a, "b"=b, "xi"=xi, "omega"=omega ) )
            }
#################################################
# compute deviance
.mcmc.deviance.2pnoh <- function( aM, bM, theta, dat, dat.resp,
                weights, eps ){
    pij <- stats::pnorm( aM * theta - bM )
    llij <- log( dat.resp * ( dat*pij + ( 1-dat )*(1-pij) ) + eps )
    if ( is.null( weights ) ){ deviance <- -2*sum( llij ) }
    if ( ! is.null( weights ) ){
        deviance <- -2*sum( rowSums(llij) * weights )
                    }
    return(deviance)
                }
####################################################
# draw item variances from inverse chi square distributions
.draw.itemvariances.2pnoh <- function(a,b,I, itemgroup, xi, omega, prior.variance ){
        # inverse gamma distribution
        # n <- 20
        # sig2 <- 1
        # 1 / stats::rgamma( 1, n/2, n*sig2/2 )
        a0 <- prior.variance[1]
        b0 <- prior.variance[2]
        #****
        # sample sig2
#        tau2 <- ( sum( ( b - xi[ itemgroup ] )^2 ) + 1 )
#        p1 <- 1 / rchisq( 1, df=I + 1 )
#        sig <- sqrt( tau2 * p1 )
        sig <- sqrt( 1 / stats::rgamma( 1, (I+a0)/2, ( sum( ( b - xi[ itemgroup ] )^2 ) + b0 ) / 2 )    )

        #****
        # sample sig2
#        tau2 <- ( sum( ( a - omega[ itemgroup ] )^2 ) + 1 )
#        p1 <- 1 / rchisq( 1, df=I + 1 )
#        nu <- sqrt( tau2 * p1 )
        nu <- sqrt( 1 / stats::rgamma( 1, (I+a0)/2, ( sum( ( a - omega[ itemgroup ] )^2 ) + b0 ) / 2 )    )
        res <- list("sig"=sig, "nu"=nu )
        return(res)
        }
#################################################
# calculation of mastery probabilities
.mcmc.mastery.2pnoh <- function( xi, omega, N, K, weights, theta, prob.mastery ){
    h1 <- stats::pnorm( matrix( omega, nrow=N, ncol=K, byrow=TRUE) *
            ( theta - matrix( xi, nrow=N, ncol=K, byrow=TRUE) ) )
    if ( is.null(weights)){
        nonmastery <- colMeans( h1 < prob.mastery[1] )
        transition <- colMeans( ( h1 < prob.mastery[2] ) * (h1> prob.mastery[1] ) )
        mastery <- colMeans( h1 > prob.mastery[2] )
                }
    if ( ! is.null(weights)){
        nonmastery <- colMeans( weights*(h1 < prob.mastery[1] ) )
        transition <- colMeans( weights*( h1 < prob.mastery[2] ) * (h1> prob.mastery[1] ) )
        mastery <- colMeans( weights*(h1 > prob.mastery[2] ) )
                }
    res <- list( "nonmastery"=as.vector(nonmastery), "transition"=as.vector(transition),
            "mastery"=as.vector(mastery) )
    return(res)
        }

## File Name: summary.isop.test.R
## File Version: 0.07

####################################################
# summary for ISOP test
summary.isop.test <- function( object, ... )
{
    obji <- object$itemstat
    VV <- ncol(obji)
    cat("*** Test for the W1 Axiom in the ISOP Model **** \n\n")
    for (vv in 2:VV){
        obji[,vv] <- round( obji[,vv],3)
    }
    print(obji)
    cat(paste0("\n-- Statistical inference is based on ", object$JJ,
            " jackknife units.\n"))
}

## File Name: lsdm_est_logist_quant.R
## File Version: 0.196


#--- Function for calculating logistic functions and probability quantiles
lsdm_est_logist_quant <- function( probcurves, theta, quantiles, wgt_theta,
        est.icc=TRUE, b=NULL, a=NULL)
{
    # estimate parameters of attribute response probcurves
    I <- nrow(probcurves)
    b0 <- NULL
    if (est.icc){
        pars.probcurves <- matrix( 0, nrow=I, ncol=5 )
        colnames(pars.probcurves) <- c('b.2PL', 'a.2PL', 'sigma.2PL', 'b.1PL',
                                            'sigma.1PL')
        rownames(pars.probcurves) <- rownames(probcurves)
        for (kk in 1L:I){
            if (!is.null(b)){
                b0 <- b[kk]
                a0 <- a[kk]
            }
            pars.probcurves[kk,1L:3] <- lsdm_est_logist_2pl( y=probcurves[kk,],
                                            theta=theta, wgt_theta=wgt_theta,
                                            b0=b0, a0=a0 )
            pars.probcurves[kk,4L:5] <- lsdm_est_logist_rasch( y=probcurves[kk,],
                                            theta=theta, wgt_theta=wgt_theta )
        }
    }
    # quantiles of Item Response Curves (Logistic Functions)
    probcurves.quant <- sapply( quantiles, FUN=function(ql){
            sapply( 1L:I, FUN=function(kk){
                    lsdm_extract_probquantile(vec=probcurves[kk,], theta=theta, quant=ql)
                } )
            } )
    probcurves.quant <- as.data.frame(probcurves.quant)
    colnames(probcurves.quant) <- paste( 'Q', 100*quantiles, sep='')
    rownames(probcurves.quant) <- rownames(probcurves)
    if (est.icc){
        pars.probcurves <- cbind( probcurves.quant, pars.probcurves )
    } else {
        pars.probcurves <- probcurves.quant
    }
    for (vv in 1L:(length(quantiles))){
        pars.probcurves[,vv] <- as.numeric( pars.probcurves[,vv] )
    }
    return(pars.probcurves)
}

## File Name: mgsem_observed_information.R
## File Version: 0.148


mgsem_observed_information <- function(coef, opt_fun_args, technical, comp_se=TRUE,
        comp_se_me=FALSE)
{
    is_ml <- technical$estimator=='ML'
    if (! is_ml){
        comp_se <- FALSE
    }
    res <- list(coef=coef, comp_se=comp_se, se=NA*coef,
                    vcov=NULL, info_loglike=NULL, info_loglike_pen=NULL,
                    is_ml=is_ml, comp_se_me=comp_se_me)

    if (comp_se){

        grad_fun <- 'mgsem_numerical_gradient'
        h <- technical$h

        #- observed information without penalty
        opt_fun_args1 <- opt_fun_args
        opt_fun_args1$use_penalty <- FALSE
        args_grad <- list(par=coef, FUN=mgsem_grad_fun, symmetrize=TRUE,
                                h=h, opt_fun_args=opt_fun_args1, output_all=FALSE)
        res$info_loglike <- do.call(what=grad_fun, args=args_grad)

        #- observed information with penalty
        args_grad$opt_fun_args <- opt_fun_args
        res$info_loglike_pen <- do.call(what=grad_fun, args=args_grad)

        #- standard errors
        res$vcov <- mgsem_ginv(X=res$info_loglike)
        res$se <- mgsem_sqrt_diag(x=res$vcov)

    }

    #-- output
    return(res)
}

## File Name: stratified_cronbach_alpha_compute_alpha.R
## File Version: 0.06


#** compute alpha
stratified_cronbach_alpha_compute_alpha <- function( data )
{
    # covariance
    c1 <- stats::cov( data, use="pairwise.complete.obs" )
    # mean covariance
    c1a <- c1
    diag(c1a) <- 0
    I <- ncol(data)
    mc <- sum(c1a) / ( I^2 - I )

    # mean and variance
    mv <- mean( diag(c1) )
    alpha <- I*mc / ( mv + (I-1)*mc )
    mean.tot <- mean( rowSums(data), na.rm=TRUE )
    var.tot <- stats::var( rowSums(data), na.rm=TRUE )
    res <- list( I=I, alpha=alpha, mean.tot=mean.tot, var.tot=var.tot )
    return(res)
}

## File Name: ginverse_sym.R
## File Version: 0.08


#--- code from Eugene Demidenko: book mixed effects models
ginverse_sym <- function(A, eps=1E-8)
{
    # generalized inverse of a symmetric matrix A
    PV <- eigen(A, symmetric=TRUE)
    V0 <- IV <- PV$values
    av0 <- abs(V0)
    IV[ av0 > eps] <- 1/V0[ av0 > eps]
    IV[ av0 <=eps] <- 0
    Ainv <- PV$vectors %*% ( IV*( t(PV$vectors) ) )
    return(Ainv)
}

## File Name: linking_haberman_summary_estimation_information.R
## File Version: 0.051

linking_haberman_summary_estimation_information <- function(res_opt)
{
    cat('Estimation type', '=', res_opt$estimation,'\n')
    cat('Number of iterations', '=', res_opt$iter,'\n')
    cat('Used trimming factor (\'BSQ\',\'HUB\')', '=', res_opt$cutoff,'\n')
    cat('Trimming factor estimated (\'BSQ\',\'HUB\')', '=', res_opt$k_estimate,'\n')
    cat('Proportion retained observation (\'LTS\')', '=', res_opt$lts_prop,'\n')
}

## File Name: gom_em_est_covariance.R
## File Version: 0.05


#-- GOM: estimation of covariance
gom_em_est_covariance <- function( f.qk.yi, Sigma, theta.kM, N  )
{
    Sigma.cov <- Sigma
    delta.theta <- 1
    hwt <- f.qk.yi
    theta.k <- theta.kM
    thetabar <- hwt %*% theta.k
    # calculation of mu
    mu <- colSums( thetabar ) / N
    mu[1] <- 0
    # calculation of the covariance matrix
    theta.k.adj <- theta.k - matrix( mu, nrow=nrow(theta.k),
                                            ncol=ncol(theta.k), byrow=TRUE)
    D <- 2
    for (dd1 in 1L:D){
        for (dd2 in dd1:D){
            tk <- theta.k.adj[,dd1]*theta.k.adj[,dd2]
            h1 <- ( hwt %*% tk ) * delta.theta
            Sigma.cov[dd1,dd2] <- sum( h1 ) / N
            if (dd1 < dd2 ){ Sigma.cov[dd2,dd1] <- Sigma.cov[dd1,dd2] }
        }
    }
    diag(Sigma.cov) <- diag(Sigma.cov) + 10^(-10)
    pi.k <- sirt_dmvnorm_discrete( theta.k, mean=mu, sigma=Sigma.cov, as_matrix=TRUE )
    res <- list(mu=mu, Sigma=Sigma.cov, pi.k=pi.k )
    return(res)
}

## File Name: summary.rasch.mirtlc.R
## File Version: 7.13
#*******************************************************
# Summary for rasch.mirtlc object                         *
summary.rasch.mirtlc <- function( object,... ){
    # object      ... object from rasch.mml                #
    cat("---------------------------------------------------------------------------------------------------------- \n")
    d1 <- utils::packageDescription("sirt")
    cat( paste( d1$Package, " ", d1$Version, " (", d1$Date, ")", sep=""), "\n\n" )
    cat( "Date of Analysis:", paste( object$s2 ), "\n" )
    cat("Computation time:", print(object$s2 - object$s1), "\n\n")
    cat("Multidimensional Item Response Latent Class Model \n\n")
    modeltype <- object$modeltype
    if (object$modeltype=="LC" ){
        cat("Latent Class Model with", object$Nclasses, "Classes",
            " - ", object$G, "Group(s)\n")
                    }
    if (object$modeltype %in% c("MLC1","MLC2") ){
        if (modeltype=="MLC1"){
                cat("Multidimensional Latent Class Rasch Model with\n     " )
                            }
        if (modeltype=="MLC2"){
                cat("Multidimensional Latent Class 2PL Model with\n     " )
                            }
            cat( object$Nclasses, "Classes",
                ", ", object$G, "Group(s)", ",",
                object$D, "Dimension(s)\n")
                    }
    if (object$distribution.trait=="normal" ){
        cat("      Normal distribution assumption\n" )
                    }
    if (object$distribution.trait=="smooth2" ){
        cat("      Log-linear Smoothing (2 Moments)\n" )
                    }
    if (object$distribution.trait=="smooth3" ){
        cat("      Log-linear Smoothing (3 Moments)\n" )
                    }
    if (object$distribution.trait=="smooth4" ){
        cat("      Log-linear Smoothing (4 Moments)\n" )
                    }
    cat("---------------------------------------------------------------------------------------------------------- \n")
    cat( "Number of iterations=", object$iter, "\n" )
    cat( "Deviance=", round( object$deviance, 2 ), " | " )
    cat( "Log Likelihood=", round( -object$deviance/2, 2 ), "\n" )
    cat( "Number of persons=", object$ic$n, "\n" )

    cat( "Number of estimated item parameters=", object$ic$itempars, "\n" )
    cat( "Number of estimated distribution parameters=", object$ic$traitpars, "\n" )
    cat( "Number of estimated parameters=", object$ic$np, "\n" )

    cat( "AIC=", round( object$ic$AIC, 2 ), " | penalty=", round( object$ic$AIC - object$ic$deviance,2 ),
            "   | AIC=-2*LL + 2*p  \n" )
    cat( "AICc=", round( object$ic$AICc, 2 )," | penalty=", round( object$ic$AICc - object$ic$deviance,2 ) )
        cat("    | AICc=-2*LL + 2*p + 2*p*(p+1)/(n-p-1)  (bias corrected AIC)\n" )
    cat( "BIC=", round( object$ic$BIC, 2 ), " | penalty=", round( object$ic$BIC - object$ic$deviance,2 ),
            "   | BIC=-2*LL + log(n)*p  \n" )
    cat( "CAIC=", round( object$ic$CAIC, 2 )," | penalty=", round( object$ic$CAIC - object$ic$deviance,2 ) )
        cat("   | CAIC=-2*LL + [log(n)+1]*p  (consistent AIC)\n\n" )

        cat("---------------------------------------------------------------------------------------------------------- \n")
        cat("Trait Distribution\n")
        obji <- object$pi.k
        if ( object$modeltype %in% c("MLC1","MLC2") ){ obji <- object$trait }
        obji <- round( obji, 3)
        print( obji )
        if ( object$modeltype %in% c("MLC1","MLC2") ){
            cat( "\nM Trait: ", round( object$mean.trait, 3 ), "\n")
            cat( "SD Trait: ", round( object$sd.trait, 3 ), "\n")
            cat( "Skewness Trait: ", round( object$skewness.trait, 3 ), "\n")
            if ( object$D > 1){
                    cat( "Correlations Trait: \n" )
                    for (gg in 1:object$G){
                        cat("Group", gg, "\n")
                        print( round( object$cor.trait[,,gg], 3 ) )
                                }
                                    }
                                    }
        cat("---------------------------------------------------------------------------------------------------------- \n")
        cat("Item Parameters \n")
        cat("Item Probabilities\n")
        obji <- t(object$pjk)
        obji <- round( obji, 3)
        print( obji )
        if ( object$modeltype %in% c("MLC1","MLC2") ){
            cat("\nItem Parameter \n")
            obji <- object$item
            obji <- round( obji, 3)
            print( obji )
                    }
            }
#*******************************************************

## File Name: mgsem_L0_penalty.R
## File Version: 0.03

mgsem_L0_penalty <- function(x=x, eps, gamma, deriv=FALSE, h=min(1e-4,eps/10) )
{
    if (deriv){
        y1 <- mgsem_L0_approx_ot(x=x+h, gamma=gamma, eps=eps)
        y2 <- mgsem_L0_approx_ot(x=x-h, gamma=gamma, eps=eps)
        y <- (y1-y2)/(2*h)
    } else {
        y <- mgsem_L0_approx_ot(x=x, gamma=gamma, eps=eps)
    }
    return(y)
}

## File Name: rasch_mml2_estep_raschtype_mirt.R
## File Version: 1.072


#*** E Step Raschtype Model: multidimensional version
rasch_mml2_estep_raschtype_mirt <- function( dat1, dat2, dat2.resp, theta.k, pi.k, I,
                n, b, fixed.a, fixed.c,  fixed.d, alpha1, alpha2, group,  mu,  Sigma.cov,
                Qmatrix, pseudoll )
{
    aa0 <- Sys.time()
    if ( is.null(group) ){
        group <- rep( 1, nrow(dat1))
    }
    G <- length( unique( group) )
    pjk <- prob_raschtype_genlogis( theta=theta.k, b=b, alpha1=alpha1,
                    alpha2=alpha2, fixed.a=fixed.a, Qmatrix=Qmatrix )
    TP <- nrow(pjk)
# cat("- probs") ; aa1 <- Sys.time(); print(aa1-aa0) ; aa0 <- aa1
    fixed.c.M  <- sirt_matrix2(fixed.c, nrow=TP)
    fixed.d.M  <- sirt_matrix2(fixed.d, nrow=TP)
    pjk <- fixed.c.M + ( fixed.d.M - fixed.c.M) * pjk

    TP <- dim(pjk)[1]
    pjkt <- t(pjk)
    pjkL <- array( NA, dim=c( I, 2, TP))
    pjkL[,1,] <- 1 - pjkt
    pjkL[,2,] <- pjkt
    probsM <- matrix( aperm( pjkL, c(2,1,3) ), nrow=I*2, ncol=TP )
    f.yi.qk <- mml_calc_like( dat2=dat2, dat2resp=dat2.resp,
                    probs=probsM, pseudoll=pseudoll)$fyiqk

    f.qk.yi <- 0 * f.yi.qk
    if ( G==1 ){
        pi.k <- matrix( pi.k, ncol=1 )
    }
    for ( gg in 1:G){
        f.qk.yi[ group==gg, ] <- f.yi.qk[ group==gg, ] * outer( rep( 1, nrow(dat2[ group==gg,]) ), pi.k[,gg] )
    }
    f.qk.yi <- f.qk.yi / rowSums( f.qk.yi )

    # expected counts at theta.k
    n.k <- matrix( 0, nrow(theta.k), G )
    r.jk <- n.jk <- array( 0, dim=c( ncol(dat2), nrow(theta.k), G) )
    ll <- rep(0,G)
    for (gg in 1:G){
        ind.gg <- which( group==gg )
        res <- mml_raschtype_counts( dat2=dat2[ind.gg,], dat2resp=dat2.resp[ind.gg,],
                    dat1=dat1[ind.gg,2], fqkyi=f.qk.yi[ind.gg,],
                    pik=pi.k[,gg], fyiqk=f.yi.qk[ind.gg,]  )
        n.k[,gg] <- res$nk
        n.jk[,,gg] <- res$njk
        r.jk[,,gg] <- res$rjk
        ll[gg] <- res$ll
    }

    #--- output
    res <- list( n.k=n.k, n.jk=n.jk, r.jk=r.jk, f.qk.yi=f.qk.yi, pjk=pjk,
                    f.yi.qk=f.yi.qk, ll=sum(ll) )
    return(res)
}



.e.step.raschtype.mirt <- rasch_mml2_estep_raschtype_mirt


# cat("- posterior") ; aa1 <- Sys.time(); print(aa1-aa0) ; aa0 <- aa1

## File Name: lsem_estimate_proc_args.R
## File Version: 0.416

lsem_estimate_proc_args <- function(lavaan.args, sufficient_statistics,
    pseudo_weights, lavmodel, data, use_lavaan_survey, est_joint=FALSE,
    par_invariant=NULL, par_linear=NULL, par_quadratic=NULL,
    partable_joint=NULL, se=NULL, G=NULL, moderator.grid=NULL, verbose=TRUE,
    is_imputed=FALSE)
{
    if (is_imputed){
        data <- data[[1]]
    }

    use_pseudo_weights <- pseudo_weights > 0
    if ( sufficient_statistics | use_pseudo_weights ){
        use_lavaan_survey <- FALSE
    }
    if (use_pseudo_weights){
        use_lavaan_survey <- FALSE
    }

    #- variables in model
    partable <- sirt_import_lavaan_lavaanify(model=lavmodel)
    variables_model <- intersect( union( partable$lhs, partable$rhs ), colnames(data) )
    has_meanstructure <- '~1' %in% paste(partable$op)

    #- joint estimation
    par_vec <- union(union(par_invariant, par_linear), par_quadratic)
    if (length(par_vec)>0){
        if (verbose){
            cat('Use joint estimation\n')
        }
        est_joint <- TRUE
        # sufficient_statistics <- TRUE
    }
    if ( ! is.null(partable_joint) ){
        est_joint <- TRUE
    }
    if (est_joint){
        sufficient_statistics <- TRUE
    }

    lavaan_args_names <- names(lavaan.args)
    if ( 'missing' %in% lavaan_args_names){
        if ( lavaan.args[['missing']]=='fiml' ){
            sufficient_statistics <- FALSE
        }
    }

    #- ordered variables
    variables_ordered <- NULL
    if ('ordered' %in% names(lavaan.args)){
        sufficient_statistics <- FALSE
        variables_ordered <- lavaan.args$ordered
    }

    data1 <- data[,variables_model,drop=FALSE]
    data_ordered <- rep(FALSE, ncol(data1))
    names(data_ordered) <- colnames(data1)
    NV <- ncol(data1)
    for (vv in 1L:NV){
        data_ordered[vv] <- is.factor(data1[,vv])
    }
    some_ordinal <- FALSE
    if (any(data_ordered)){
        sufficient_statistics <- FALSE
        variables_ordered <- union(variables_ordered, names(data_ordered)[data_ordered] )
        some_ordinal <- TRUE
    }
    if (est_joint & ( ! sufficient_statistics)){
        if (pseudo_weights==0){
            G <- length(moderator.grid)
            pseudo_weights <- 10*G*nrow(data)
            use_pseudo_weights <- TRUE
        }
        if (some_ordinal){
            stop('Cannot perform joint estimation for non-continuous data.')
        }
    }
    if (is.null(se)){
        if (est_joint){
            se <- 'none'
        } else {
            se <- 'standard'

        }
    }
    compute_se <- se!='none'

    #-- output
    res <- list(sufficient_statistics=sufficient_statistics,
                use_lavaan_survey=use_lavaan_survey,
                variables_model=variables_model, use_pseudo_weights=use_pseudo_weights,
                variables_ordered=variables_ordered, est_joint=est_joint,
                partable=partable, has_meanstructure=has_meanstructure, se=se,
                compute_se=compute_se, pseudo_weights=pseudo_weights,
                some_ordinal=some_ordinal, is_imputed=is_imputed)
    return(res)
}

## File Name: rm_calclike.R
## File Version: 0.07


#**** calculation of the likelihood
rm_calclike <- function(dat2, dat2resp, probs,K){
    RM_CALCPOST( DAT2=dat2, DAT2RESP=dat2resp, PROBS=probs, KK=K)
}

## File Name: lsem_kernel_weights.R
## File Version: 0.061

lsem_kernel_weights <- function(x, x0, bw, kernel="gaussian")
{
    if (kernel=='gaussian'){
        wgt <- exp( - (x - x0)^2 / (2*bw^2) )
    }
    if (kernel=='uniform'){
        wgt <- 1*(abs(x-x0) <=bw)
    }
    if (kernel=='epanechnikov'){
        z <- (x-x0)/bw
        wgt <- abs(z<1)*3/4*(1-z^2)
    }
    return(wgt)
}

## File Name: gom_em_prepare_data.R
## File Version: 0.05

gom_em_prepare_data <- function(dat, weights, model)
{
    dat0 <- dat
    dat.resp <- 1-is.na(dat)
    dat[ is.na(dat) ] <- 0
    N <- nrow(dat)
    I <- ncol(dat)
    if (is.null(weights)){
        weights <- rep(1,N)
    } else {
        if (model=="GOMRasch"){
            stop("'GOMRasch' cannot handle weights!\n")
        }
    }
    dat2 <- as.matrix(dat)
    dat2.resp <- as.matrix(dat.resp)
    # indicator matrix
    dat2.ind0 <- dat2.resp*(dat2==0)
    dat2.ind1 <- dat2.resp*(dat2==1)
    dat2.ind <- as.matrix( cbind( dat2.ind0, dat2.ind1 ) )

    #--- output
    res <- list( dat0=dat0, dat2=dat2, dat2.resp=dat2.resp, dat2.ind=dat2.ind, N=N,
                    I=I, weights=weights )
    return(res)
}

## File Name: rm_pcm_calcprobs.R
## File Version: 0.141



#- calculation of probabilities in the partial credit model
rm_pcm_calcprobs <- function( a, b, Qmatrix, theta.k, I, K, TP )
{
    probs <- array( 0, dim=c(I,K+1,TP) )   # categories 0, ..., K
    for (kk in 1L:K){
        l0 <- matrix( - b[,kk], nrow=I,ncol=TP)
        l0 <- l0 + TAM::tam_outer( a * Qmatrix[, kk], theta.k )
        probs[,kk+1,] <- l0
    }
    probs <- exp(probs)
    probs1 <- probs[,1,]
    for (kk in 2L:(K+1)){
        probs1 <- probs1 + probs[,kk,]
    }
    for (kk in 1L:(K+1)){
        probs[,kk,] <- probs[,kk,] / probs1
    }
    return(probs)
}

## File Name: xxirt_createDiscItem.R
## File Version: 0.174

xxirt_createDiscItem <- function( name, par, est, P, lower=-Inf,
            upper=Inf, prior=NULL, prior_par1=NULL, prior_par2=NULL,
            X=NULL)
{
    res <- list()
    res$name <- name
    res$par <- par
    res$est <- est
    res$P <- P
    res$lower <- lower
    res$upper <- upper
    res$prior <- prior
    res$prior_par1 <- prior_par1
    res$prior_par2 <- prior_par2
    res$X <- X
    if (is.null(X)){
        res$person_covariates <- FALSE
    } else {
        res$person_covariates <- TRUE
    }
    class(res) <- 'DiscItem'
    return(res)
}

## File Name: xxirt_mstep_itemParameters.R
## File Version: 0.458


#--- M-step item parameters
xxirt_mstep_itemParameters <- function( partable, item_list, items, Theta,
            ncat, partable_index, N.ik, mstep_iter, par0, eps,
            mstep_reltol, mstep_method, item_index, h, use_grad,
            penalty_fun_item=NULL, p.aj.xi=NULL, dat=NULL, dat_resp_bool=NULL,
            weights=NULL)
{

    TP <- ncol(p.aj.xi)
    maxK <- dim(N.ik)[2]

    #-------------------------------------------------
    #**** define likelihood function
    like_items1 <- function( x, ... )
    {
        partable <- xxirt_partable_include_freeParameters( partable=partable, x=x )
        probs1 <- xxirt_compute_itemprobs( item_list=item_list, items=items,
                            Theta=Theta, ncat=ncat, partable=partable,
                            partable_index=partable_index )
        log_probs1 <- log( probs1 + eps )
        # log likelihood value
        if (! attr(partable, 'person_covariates') ){
            ll <- - sum( N.ik * log_probs1 )
        } else {
            log_probs1 <- as.vector( log_probs1 )
            loglike_case_theta <- sirt_rcpp_xxirt_compute_loglike_case_theta_covariates(
                                            dat=dat, dat_resp_bool=dat_resp_bool,
                                            logprobs=log_probs1, TP=TP, maxK=maxK )
            ll <- -sum(weights*p.aj.xi*loglike_case_theta)
        }


        # add prior distributions
        pen <- xxirt_mstep_itemParameters_evalPrior(partable=partable, h=0)
        # add penalty
        ll <- ll + sum(pen)
        # add penalty function
        if (!is.null(penalty_fun_item)){
            ll <- ll + penalty_fun_item(x=x)
        }
        return(2*ll)
    }
    #**** end definition likelihood
    #-------------------------------------------------

    #-------------------------------------------------
    #*** define gradient here
    #### x <- partable[ partable$parfree==1, 'value']
    grad_items1 <- function(x, ...)
    {
        partable0 <- xxirt_partable_include_freeParameters( partable=partable, x=x )
        probs1 <- xxirt_compute_itemprobs( item_list=item_list, items=items,
                            Theta=Theta, ncat=ncat, partable=partable0,
                            partable_index=partable_index )
        logprobs1 <- log(probs1+eps)
        NP <- sum( partable$parfree==1)
        pen1 <- grad1 <- rep(0,NP)
        pen0 <- 0
        for (pp in 1L:NP){
            item_index_pp <- item_index[[pp]]
            xh <- x
            xh[pp] <- xh[pp] + h
            N.ik_pp <- N.ik[item_index_pp,,, drop=FALSE ]
            partable_h <- xxirt_partable_include_freeParameters( partable=partable0,
                                    x=xh )

            probs1h <- xxirt_compute_itemprobs( item_list=item_list, items=items,
                                    Theta=Theta, ncat=ncat, partable=partable_h,
                                    partable_index=partable_index,
                                    item_index=item_index_pp)
            logprobs1h <- log( probs1h + eps )
            if( ! attr(partable, 'person_covariates')    ){
                ll0 <- - sum( N.ik_pp * logprobs1[ item_index_pp,,,drop=FALSE] )
                ll1 <- - sum( N.ik_pp * logprobs1h )
                diff1 <- ll1-ll0
            } else {

                dat_pp <- dat[,item_index_pp,drop=FALSE]
                dat_resp_bool_pp <- dat_resp_bool[,item_index_pp,drop=FALSE]
                logprobs1_pp <- as.vector(logprobs1[item_index_pp,,,])

                fun <- sirt_rcpp_xxirt_compute_loglike_case_theta_covariates
                args1 <- list(dat=dat_pp, dat_resp_bool=dat_resp_bool_pp,
                                logprobs=logprobs1_pp, TP=TP, maxK=maxK )
                # loglike_case0 <- do.call(what=fun, args=args1 )
                # ll0 <- -sum(weights*p.aj.xi*loglike_case0)

                # args1$logprobs <- as.vector(logprobs1h)
                # loglike_case1 <- do.call(what=fun, args=args1 )
                # ll1 <- -sum(weights*p.aj.xi*loglike_case1)

                args1$logprobs <- as.vector(logprobs1h-logprobs1_pp)
                loglike_case2 <- do.call(what=fun, args=args1 )
                diff1 <- -sum(weights*p.aj.xi*loglike_case2)

            }
            grad1[pp] <- diff1
        }  # end pp

        #--- prior distributions
        pen <- xxirt_mstep_itemParameters_evalPrior(partable=partable0, h=0)
        pen_h <- xxirt_mstep_itemParameters_evalPrior(partable=partable0, h=h)
        #-- penalty function
        if (!is.null(penalty_fun_item)){
            pen0 <- penalty_fun_item(x=x)
            for (pp in 1L:NP){
                xh <- x
                xh[pp] <- xh[pp] + h
                pen1[pp] <- penalty_fun_item(x=xh)
            }
        }
        grad1 <- ( grad1 +  pen_h - pen + pen1 - pen0 )/h
        return(grad1)
    }
    #**** end definition gradient
    #-------------------------------------------------

    opt_fun <- stats::optim

    like_items <- function(x)
    {
        like_items1(x=x)
    }
    grad_items <- function(x)
    {
        grad_items1(x=x)
    }

    # mstep_method <- 'nloptr'
    use_nloptr <- mstep_method=='nloptr'
    if (use_nloptr){
        requireNamespace('nloptr')
        opt_fun <- nloptr::nloptr
    }

    if (use_nloptr){
        arg_control <- list(maxeval=mstep_iter)
    } else {
        arg_control <- list(maxit=mstep_iter)
    }
    if (! use_nloptr){
        if ( mstep_method=='BFGS'){
            arg_control$reltol <- mstep_reltol
        }
        if ( mstep_method=='L-BFGS-B'){
            arg_control$factr <- mstep_reltol
        }
    } else {
        arg_control <- list(ftol_rel=mstep_reltol, algorithm='NLOPT_LD_LBFGS')
    }
    # method L-BFGS-B uses 'factr' (and 'pgtol') instead of 'reltol' and 'abstol'
    if (use_nloptr){
        arg_list <- list(x0=par0, eval_f=like_items, opts=arg_control)
    } else {
        arg_list <- list( par=par0, fn=like_items, method=mstep_method,
                        control=arg_control)
    }
    if (use_grad){
        if (use_nloptr){
            arg_list$eval_grad_f <- grad_items
        } else {
            arg_list$gr <- grad_items
        }
    }
    lb <- partable[ partable$parfree==1, 'lower']
    ub <- partable[ partable$parfree==1, 'upper']
    if ( mstep_method=='L-BFGS-B'){
        arg_list$lower <- lb
        arg_list$upper <- ub
    }
    if (use_nloptr){
        arg_list$lb <- lb
        arg_list$ub <- ub
    }


    mod <- do.call( what=opt_fun, args=arg_list )

    partable <- xxirt_partable_include_freeParameters( partable=partable, x=mod$par )
    pen_val <- 0
    if (!is.null(penalty_fun_item)){
        pen_val <- penalty_fun_item(x=mod$par)
    }
    prior_par <- sum( xxirt_mstep_itemParameters_evalPrior(partable=partable, h=0) )

    #-- output
    res <- list( ll1=mod$value, partable=partable, par0=mod$par, pen_val=pen_val,
                    prior_par=prior_par)
    return(res)
}

## File Name: rm_sdt_extract_par_from_partable_add_increment.R
## File Version: 0.07


rm_sdt_extract_par_from_partable_add_increment <- function(partable,
        pargroup, increment )
{
    partable <- partable[ partable$est, ]
    res <- partable[ partable$est, "value" ]
    res <- res + increment * ( partable$pargroup==pargroup )
    return(res)
}

## File Name: summary.lsdm.R
## File Version: 0.199



#*** summary for LSDM function
summary.lsdm <- function( object, file=NULL, digits=3, ... )
{
    # open sink for a file
    sirt_osink(file=file)

    lsdmobj <- object
    # generate sequence for display
    display <- paste0( paste( rep("-", each=65 ), collapse="" ), "\n")
    cat(display)

    #- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object$time)

    cat("\nLSDM -- Least Squares Distance Method of Cognitive Validation \n")
    cat("Reference: Dimitrov, D. (2007). Applied Psychological Measurement, 31, 367-387.\n")

    cat(display)
    cat("Model Fit\n\n")
    cat("Distance function", "=", object$distance, "\n" )
    cat(paste( "Model Fit LSDM   -  Mean MAD:",
                    formatC( lsdmobj$mean.mad.lsdm0, digits=digits, width=digits+3),
                    "    Median MAD:", formatC( median(lsdmobj$item$mad.lsdm), digits=digits, width=digits+3), "\n") )
    cat(paste( "Model Fit LLTM   -  Mean MAD:", formatC( lsdmobj$mean.mad.lltm, digits=digits, width=digits+3),
                    "    Median MAD:", formatC( median(lsdmobj$item$mad.lltm), digits=digits, width=digits+3),
                    "   R^2=", format( summary(lsdmobj$lltm)$r.squared, digits=digits),   "\n") )

    cat(display)
    cat("Attribute Parameters\n\n")
    dfr.a <- data.frame( "N.Items"=colSums(lsdmobj$Qmatrix), round( lsdmobj$attr.pars, digits) )
    elim.a <- union( grep("Q", colnames(dfr.a) ), grep( "sigma", colnames(dfr.a) ) )
    print( dfr.a[, -elim.a ] )

    cat(display)
    cat("Item Parameters\n\n")
    dfr.i <- round( lsdmobj$item, digits)
    displ.i <- sort( union( which( colnames(dfr.i) %in% c(  "a.2PL", "b.2PL", "N.Items", "b.1PL" ) ),
                    grep( "mad", colnames(dfr.i))) )
    print( round( dfr.i[, displ.i ], digits) )

    cat(display)
    cat("Discrimination Parameters\n\n")
    dfr.i <- round( lsdmobj$W, digits)
    print(dfr.i)

    sirt_csink(file=file)
}

## File Name: ccov.np.R
## File Version: 1.225


#---- nonparametric estimation of conditional covariance
ccov.np <- function( data, score, bwscale=1.1, thetagrid=seq( -3,3,len=200),
        progress=TRUE, scale_score=TRUE, adjust_thetagrid=TRUE, smooth=TRUE,
        use_sum_score=FALSE, bias_corr=TRUE)
{
    # number of Items I
    I <- ncol(data)
    if (use_sum_score){
        smooth <- FALSE
        score <- rowSums(data)
    }
    # z-standardization of score
    if (scale_score){
        score <- scale(score)[,1]
    } else {
        bwscale <- bwscale*stats::sd(score, na.rm=TRUE)
    }
    # adjust thetagrid
    if (adjust_thetagrid & smooth){
        score_rg <- range(score)
        thetagrid <- seq(score_rg[1], score_rg[2], length=length(thetagrid))
    }
    if (!smooth){
        score <- round(score,3)
        thetagrid <- sort(unique(score))
    }
    # matrix of item response functions
    if (progress){
        cat('Pairwise Estimation of Conditional Covariances\n' )
        cat('...........................................................\n' )
        cat('Nonparametric ICC estimation \n ' )
    }
    icc_items <- matrix( 0, length(thetagrid), I )
    if ( I >=20 ){
        display <- seq( 1, I, floor( I/20 ) )[ 2L:20 ]
    } else {
        display <- 20
    }
    i <- 1
    for ( ii in 1L:I ){
        obs_ii <- ! is.na( data[,ii] )
        x <- score[ obs_ii ]
        y <- data[ obs_ii, ii ]
        icc_items[,ii] <- ccov_np_regression(x=x, y=y, xgrid=thetagrid,
                                bwscale=bwscale, smooth=smooth, score=score)
        i <- ccov_np_print_progress(progress=progress, i=i, ii=ii, display=display)
    }

    sirt_progress_cat(progress=progress)
    #-- weights thetagrid
    wgt_thetagrid <- ccov_np_score_density(score=score, thetagrid=thetagrid,
                            smooth=smooth)

    #-- display progress
    if (progress){
        cat('...........................................................\n' )
        cat('Nonparametric Estimation of conditional covariances \n ' )
        utils::flush.console()
    }
    # calculation of conditional covariance
    ccov.table <- data.frame( item1ID=rep(1L:I, I),
                                item2ID=rep( 1L:I, each=I) )
    ccov.table <- ccov.table[ ccov.table$item1ID < ccov.table$item2ID, ]
    ccov.table$N <- apply( ccov.table, 1, FUN=function(ll){
                            sum( rowSums( is.na( data[, c( ll[1], ll[2] ) ] ) )==0 ) } )
    ccov.table <- ccov.table[ ccov.table$N > 0, ]
    ccov.table$item1 <- colnames(data)[ ccov.table$item1ID ]
    ccov.table$item2 <- colnames(data)[ ccov.table$item2ID ]
    ccov.table$itempair <- paste( ccov.table$item1, ccov.table$item2, sep='-' )
    # smoothing all item pairs
    # calculate conditional covariances
    FF <- nrow(ccov.table)
    ccov.matrix <- prod.matrix <- matrix( 0, nrow=length(thetagrid), ncol=FF )
    ii <- 1
    ccov_sum_score <- rep(NA, FF)
    for (ff in 1L:FF){
        if (FF>20){
            display <- seq( 1, FF, floor( FF/20 ) )[ 2L:20 ]
        } else {
            display <- seq(1,FF)
        }
        data.ff <- data[, c( ccov.table[ff,1], ccov.table[ff,2] ) ]
        which.ff <- which( rowSums( is.na(data.ff) )==0  )
        data.ff <- data.ff[ which.ff, ]
        score.ff <- score[which.ff]
        prod.matrix[,ff] <- ccov_np_regression(x=score.ff, y=data.ff[,1]*data.ff[,2],
                        xgrid=thetagrid, bwscale=bwscale, smooth=smooth, score=score)
        m12 <- icc_items[, ccov.table[ff,1] ]*icc_items[, ccov.table[ff,2] ]
        ccov.matrix[,ff] <- prod.matrix[,ff] - m12

        #- computations based on sum score
        if (use_sum_score){
            res1 <- ccov_np_compute_ccov_sum_score(score=score.ff, data=data.ff)
            score_ff2 <- score.ff - data.ff[,1] - data.ff[,2]
            if (bias_corr){
                res2 <- ccov_np_compute_ccov_sum_score(score=score_ff2, data=data.ff)
            } else {
                res2 <- res1
            }
            ccov_sum_score[ff] <- ( res1$ccov_aggr + res2$ccov_aggr ) / 2
        }

        # print progress
        ii <- ccov_np_print_progress(progress=progress, i=ii, ii=ff, display=display)
    }
    # remove NAs from ccov.matrix
    ccov.matrix[ is.na(ccov.matrix) ] <- 0
    sirt_progress_cat(progress=progress)

    # calculate (weighted) conditional covariance
    ccov.table$ccov <- apply( ccov.matrix, 2, FUN=function(sp){
                                stats::weighted.mean( x=sp, w=wgt_thetagrid ) } )
    if (use_sum_score){
        ccov_sum_score[ is.na(ccov_sum_score) ] <- 0
        ccov.table$ccov <- ccov_sum_score
    }

    #--- output
    res <- list( ccov.table=ccov.table, ccov.matrix=ccov.matrix,
                    data=data, score=score, icc.items=icc_items,
                    wgt.thetagrid=wgt_thetagrid, use_sum_score=use_sum_score,
                    bias_corr=bias_corr, scale_score=scale_score)
    return(res)
}

## File Name: mcmc.aux.R
## File Version: 0.08

###################################################################
# draw variances from inverse chi square distribution
.mcmc.draw.variance <- function( N, w0, sig02, n, sig2 ){
    # INPUT:
    # N ... number of random draws
    # w0 ... sample size prior
    # sig02 ... prior variance
    # n ... empirical sample size
    # sig2 ... empirical variance
    res <- 1/ stats::rgamma( N, (w0+n) / 2, 0.5 * ( w0*sig02 + n*sig2 ) )
    return(res)
        }
#####################################################################

## File Name: locpolycor.R
## File Version: 0.271


locpolycor <- function(y, data.mod, moderator.grid, h=1.1,
                    model_thresh, model_polycor, sampling_weights=NULL,
                    kernel="gaussian",
                    eps=1e-10)
{
    #- compute weights
    llw <- lsem_local_weights(data.mod=data.mod, moderator.grid=moderator.grid, h=h,
                kernel=kernel, sampling_weights=sampling_weights)
    weights_grid <- llw$weights

    #- estimate thresholds
    y <- as.matrix(y)
    N <- nrow(y)
    I <- ncol(y)
    G <- length(moderator.grid)

    #- item-wise optimization
    thresh_list <- as.list(1L:I)
    K_list <- as.list(1L:I)
    thresh_ind_list <- as.list(1L:I)
    items <- colnames(y)
    names(thresh_list) <- items
    names(thresh_ind_list) <- items
    names(K_list) <- items
    Kmax <- max(y, na.rm=TRUE)
    thresh_stat <- list()
    thresh0 <- matrix(NA, nrow=Kmax, ncol=I)
    rownames(thresh0) <- paste0('t',1L:Kmax)
    colnames(thresh0) <- items
    for (gg in 1L:G){
        thresh_stat[[gg]] <- thresh0
    }

    for (ii in 1L:I){
        cat(paste0( '-- compute thresholds for item ', ii, '\n') )
        utils::flush.console()
        y1 <- y[,ii]
        ind <- which( ! is.na(y1) )
        K <- max(y1)
        thresh_item <- matrix(NA, nrow=K, ncol=G)
        thresh_ind_cbind <- NULL
        data_mod <- data.mod[ind]
        res_item <- list()
        par_init <- NULL
        for (gg in 1L:G){
            x0 <- moderator.grid[gg]
            w <- weights_grid[ind, gg]
            res <- locpolycor_est_thresh_item(y=y1, data.mod=data_mod, x0=x0,
                        w=w, model=model_thresh, par_init=par_init, eps=eps)
            par_init <- res$res_optim$par
            thresh_ind_cbind <- cbind( thresh_ind_cbind, res$thresh_ind)
            thresh_item[,gg] <- res$thresh
            res_item[[gg]] <- res
            thresh_stat[[gg]][1L:K,ii] <- res$thresh
        }

        thresh_list[[ii]] <- thresh_item

        K_list[[ii]] <- K

        #* individual predictions of thresholds
        thresh_ind <- matrix(NA, nrow=N, ncol=K)
        distmat <- abs( outer( data_mod, moderator.grid, '-' ) )
        smallest <- rowKSmallest2.sirt(matr=distmat, K=2)
        smallest <- smallest$smallind[,1L:2]

        # define predictions
        thresh_ind <- res_item[[1]]$thresh_ind
        ind <- which(data_mod > moderator.grid[G])
        if (length(ind)>0){
            thresh_ind[ind, ] <- ( res_item[[G]]$thresh_ind )[ ind,, drop=FALSE ]
        }

        ind <- which( ( data_mod <=moderator.grid[G]) & (data_mod >=moderator.grid[1]))
        sm1 <- smallest[,1]
        sm2 <- smallest[,2]
        mg1 <- moderator.grid[sm1]
        mg2 <- moderator.grid[sm2]
        t3 <- t2 <- t1 <- matrix(NA, nrow=N, ncol=K)
        for (kk in 1L:K){
            mat1 <- cbind( 1L:N, kk+K*(sm1-1) )
            t1[,kk] <- thresh_ind_cbind[ mat1 ]
            mat2 <- cbind( 1L:N, kk+K*(sm2-1) )
            t2[,kk] <- thresh_ind_cbind[ mat2 ]
            # linear interpolation of individual thresholds
            t3[,kk] <- t1[,kk] + ( t2[,kk]-t1[,kk]) * (data_mod-mg1) / (mg2-mg1)
        }
        thresh_ind[ind,] <- t3[ind,]
        thresh_ind_list[[ii]] <- thresh_ind

    }  # end ii

    #-- local polychoric correlations
    polycor0 <- diag(1,I)
    rownames(polycor0) <- colnames(polycor0) <- items
    polycor_stat <- list()
    for (gg in 1L:G){
        polycor_stat[[gg]] <- polycor0
    }
    for (ii in 1L:(I-1) ){
        for (jj in (ii+1):I){
            cat(paste0( '** compute polychoric correlation for item pair (',
                                ii, ',', jj, ')\n') )
            res <- locpolycor_est_polycor_itempair(y=y, ii=ii, jj=jj, data.mod=data.mod,
                            moderator.grid=moderator.grid, weights_grid=weights_grid,
                            model=model_polycor, thresh_ind_list=thresh_ind_list,
                            x0=x0, eps=eps)
            polycor1 <- res$polycor
            for (gg in 1L:G){
                polycor_stat[[gg]][ii,jj] <- polycor1[[gg]]
                polycor_stat[[gg]][jj,ii] <- polycor1[[gg]]
            }
        }
    }

    #-- output
    res <- list(thresh_list=thresh_list, thresh_stat=thresh_stat,
                    polycor_stat=polycor_stat, thresh_ind_list=thresh_ind_list,
                    K_list=K_list, I=I, moderator.grid=moderator.grid,
                    weights_grid=weights_grid)
    return(res)
}

## File Name: L2_polish.R
## File Version: 0.07

L2_polish <- function(x, eps=1e-4, maxiter=20)
{
    x0 <- x
    nr <- nrow(x)
    nc <- ncol(x)
    wgt <- 1*(! is.na(x))
    iterate <- TRUE
    xr <- 0
    xc <- 0
    iter <- 1
    while(iterate){
        xr_old <- xr
        xc_old <- xc
        x01 <- x0 - xr
        xc <- weighted_colMeans( mat=x01, wgt=wgt )
        xc <- xc - mean(xc)
        x01 <- x0 - sirt_matrix2(xc, nrow=nr)
        xr <- weighted_rowMeans( mat=x01, wgt=wgt )
        change <- max( abs(xc_old-xc), abs(xr_old-xr), na.rm=TRUE)
        if (iter > maxiter){ iterate <- FALSE }
        if (change < eps ){ iterate <- FALSE }
    }
    #--- output
    res <- list(row=xr, col=xc, iter=iter, wgt=wgt)
    return(res)
}

## File Name: mgsem_test_fun.R
## File Version: 0.161


mgsem_test_fun <- function(test, coef, opt_fun_args)
{
    if (test){
        requireNamespace('miceadds')
        #- function evaluation
        ll <- mgsem_opt_fun(x=coef, opt_fun_args=opt_fun_args)
        #- numerical gradient
        grad1 <- mgsem_grad_fun_numeric_approx(x=coef, opt_fun_args=opt_fun_args)
        #- analytical gradient
        args <- list(x=coef, opt_fun_args=opt_fun_args)
        grad <- do.call( what=mgsem_grad_fun, args=args)

        # dfr <- cbind( grad, grad1)

        #- print
        miceadds::Revalpr('ll')
        miceadds::Revalpr_maxabs('grad','grad1')
        stop()
    }
}

## File Name: xxirt_classprobs_lca_init_par.R
## File Version: 0.092


xxirt_classprobs_lca_init_par <- function(K, G, random_sd=0, par_Theta_init=NULL)
{
    if (G==1L){
        par_Theta <- xxirt_classprobs_lca_init_par_create(K=K, random_sd=random_sd)
        names(par_Theta) <- paste0('pi',1L:(K-1) )
        if ( !is.null(par_Theta_init) ){
            par_Theta <- par_Theta_init
        }
    } else {
        v1 <- NULL
        for (gg in 1L:G){
            par_Theta <- xxirt_classprobs_lca_init_par_create(K=K, random_sd=random_sd)
            names(par_Theta) <- paste0('pi',1L:(K-1), '_Gr', gg)
            v1 <- c(v1, par_Theta)
        }
        par_Theta <- v1
    }
    return(par_Theta)
}

## File Name: invariance_alignment_calc_corr.R
## File Version: 0.03


# auxiliary function for calculation of correlations
invariance_alignment_calc_corr <- function(parsM)
{
    cM <- stats::cor(parsM)
    I <- ncol(cM)
    rbar <- ( sum(cM) - I )/ ( I^2 - I)
    return(rbar)
}

## File Name: locpolycor_est_thresh_opt_fun.R
## File Version: 0.153



locpolycor_est_thresh_opt_fun <- function(x, y, x1, w, x0, model, K, eps=1e-10)
{
    if (model=='const'){
        par1 <- c(-Inf,x,Inf)
        pred1 <- par1[y+2]
        pred2 <- par1[y+1]
    }
    if (model=='lin'){
        par1 <- c(-Inf,x[1L:K],Inf)
        par2 <- c(-Inf,x[K+(1L:K)],Inf)
        pred1 <- par1[y+2] + par2[y+2]*(x1-x0)
        pred1 <- ifelse(y==K, par1[y+2], pred1)
        pred2 <- par1[y+1] + par2[y+1]*(x1-x0)
        pred2 <- ifelse(y==0, par2[y+1], pred2)
    }
    arg <- stats::pnorm(pred1) - stats::pnorm(pred2)
    arg <- ifelse( arg < eps, eps, arg )
    val <- - sum(w*log(arg))
    return(val)
}

## File Name: predict.btm.R
## File Version: 0.07

predict.btm <- function(object, data=NULL, ...)
{
    dat <- data
    if (is.null(dat)){
        dat <- object$dat
    }
    effects <- object$effects
    teams <- paste(effects$individual)
    pars <- object$pars
    ignore.ties <- object$ignore.ties
    delta <- pars[ pars$par=="delta", "est"]
    eta <- pars[ pars$par=="eta", "est"]
    wgt.ties <- object$wgt.ties
    #- identify teams
    ind1 <- match( paste(dat[,1]), teams)
    ind2 <- match( paste(dat[,2]), teams)

    #- compute probabilities
    theta <- effects$theta
    ND <- nrow(dat)
    M1 <- matrix(0, nrow=ND, ncol=3)
    M1[,1] <- theta[ ind1 ] + eta
    M1[,2] <- theta[ ind2 ]
    M1[,3] <- delta + ( theta[ ind1] + theta[ ind2 ] + eta ) * wgt.ties
    probs <- exp(M1)
    probs <- probs / rowSums(probs)
    #-- output
    return(probs)
}

## File Name: rasch_mirtlc_estep_mlc1.R
## File Version: 0.22


# E Step Multidimensional Latent Class Rasch Model
rasch_mirtlc_estep_mlc1  <- function( dat1, dat2, dat2.resp, pi.k, pjk, I,
            b, a, group, G, theta.k,  D, dimensions, Qmatrix, f.qk.yi=NULL  )
{
    #--- array notation of probabilities
    if ( D==1){
        pjk <- prob_raschtype_genlogis( theta=theta.k, b=b,    alpha1=0,
                    alpha2=0, fixed.a=a )
    }
    if (D>1){
        thetaPred <- tcrossprod( theta.k, Qmatrix)
        TP <- nrow(theta.k)
        bPred <- sirt_matrix2(x=b, nrow=TP)
        aPred <- sirt_matrix2(x=a, nrow=TP)
        pjk <- stats::plogis( aPred*(thetaPred - bPred ) )
    }
    if (D==1){
        NT <- length(theta.k)
    } else {
        NT <- nrow(theta.k)
    }
    TP <- NT
    pjkt <- t(pjk)
    pjkL <- array( NA, dim=c( I, 2, TP  ) )
    pjkL[,1,] <- 1 - pjkt
    pjkL[,2,] <- pjkt
    probsM <- matrix( aperm( pjkL, c(2,1,3) ), nrow=I*2, ncol=TP )
    f.yi.qk <- mml_calc_like( dat2=dat2, dat2resp=dat2.resp, probs=probsM )$fyiqk
    f.qk.yi <- 0 * f.yi.qk
    if ( G==1 ){ pi.k <- matrix( pi.k, ncol=1 ) }
    if (G>1){
        for ( gg in 1:G){
            ind.gg <- which( group==gg )
            pikM <- sirt_matrix2(x=pi.k[,gg], nrow=length(ind.gg))
            f.qk.yi[ ind.gg, ] <- f.yi.qk[ ind.gg, ] * pikM
        }
    }
    if (G==1){
        f.qk.yi <- f.yi.qk * sirt_matrix2( pi.k[,1], nrow=nrow(f.yi.qk) )
    }
    f.qk.yi <- f.qk.yi / rowSums(f.qk.yi)
    # expected counts at theta.k
    if (D==1){
        NT <- length(theta.k)
    } else {
        NT <- nrow(theta.k)
    }
    n.k <- matrix( 0, nrow=NT, ncol=G )
    r.jk <- n.jk <- array( 0, dim=c( ncol(dat2), NT, G) )
    ll <- rep(0,G)

    for (gg in 1:G){
        ind.gg <- which(group==gg)
        res <- mml_raschtype_counts( dat2=dat2[ind.gg,], dat2resp=dat2.resp[ind.gg,],
                    dat1=dat1[ind.gg,2], fqkyi=f.qk.yi[ind.gg,],
                    pik=pi.k[,gg], fyiqk=f.yi.qk[ind.gg,]  )
        n.k[,gg] <- res$nk
        n.jk[,,gg] <- res$njk
        r.jk[,,gg] <- res$rjk
        ll[gg] <- res$ll
    }
    res <- list( n.k=n.k, n.jk=n.jk, r.jk=r.jk, f.qk.yi=f.qk.yi, pjk=pjk,
                    f.yi.qk=f.yi.qk, ll=sum(ll) )
    return(res)
}


.e.step.mirtlc.mlc1 <- rasch_mirtlc_estep_mlc1

## File Name: rm_facets_est_tau_item.R
## File Version: 0.20


#####################################################
# estimation of tau.item parameters
rm_facets_est_tau_item <- function( b.item, b.rater, Qmatrix, tau.item,
        VV, K, I, TP, a.item, a.rater, item.index, rater.index,
        n.ik, numdiff.parm=.001, max.b.increment=1,theta.k, msteps,
        mstepconv, tau.item.fixed, tau.item.fixed_val )
{

    h <- numdiff.parm
    diffindex <- item.index
    RR <- length(b.rater)
    Q0 <- matrix(0,nrow=VV, ncol=K)
    se.tau.item <- Q0
    cat("  M steps tau.item parameter |")
    it <- 0
    conv1 <- 1000
    #--- input rm_facets_calcprobs
    args <- list( b.item=b.item, b.rater=b.rater, Qmatrix=Qmatrix, tau.item=tau.item, VV=VV, K=K,
                    I=I, TP=TP, a.item=a.item, a.rater=a.rater, item.index=item.index,
                    rater.index=rater.index, theta.k=theta.k, RR=RR )
    #--- begin M-steps
    while( ( it < msteps ) & ( conv1 > mstepconv ) ){
        tau.item11 <- tau.item0 <- tau.item
        for (kk in 1:K){
            Q1 <- Q0
            Q1[,kk] <- 1
            #-- compute expected likelihood
            args$tau.item <- tau.item11
            pjk <- do.call( what=rm_facets_calcprobs, args=args)
            args$tau.item <- tau.item11 + h*Q1
            pjk1 <- do.call( what=rm_facets_calcprobs, args=args)
            args$tau.item <- tau.item11 - h*Q1
            pjk2 <- do.call( what=rm_facets_calcprobs, args=args)
            #-- compute increments
            res <- rm_numdiff_index( pjk=pjk, pjk1=pjk1, pjk2=pjk2, n.ik=n.ik, diffindex=diffindex,
                        max.increment=max.b.increment, numdiff.parm=numdiff.parm )
            increment <- Q1*matrix( res$increment, nrow=VV, ncol=K)
            tau.item <- tau.item + increment
            se.tau.item[,kk] <- sqrt(abs(-1/res$d2)    )
        }

        if ( ! is.null( tau.item.fixed_val ) ){
            MK <- ncol( tau.item.fixed_val )
            for ( kk in 1:MK){
                ind <- which( ! is.na( tau.item.fixed_val[,kk]) )
                if ( length(ind) > 0 ){
                    tau.item[ ind, kk] <- tau.item.fixed_val[ ind, kk]
                }
            }
        }
        conv1 <- max( abs( tau.item - tau.item0 ) )
        it <- it+1
        cat("-")
        if ( ! is.null(tau.item.fixed) ){
            tau.item[ tau.item.fixed[,1:2,drop=FALSE] ] <- tau.item.fixed[,3]
        }
    }
    cat(" ", it, "Step(s) \n")
    #-- output
    res <- list(tau.item=tau.item, se.tau.item=se.tau.item, ll=sum(res$ll0) )
    return(res)
}

.rm.facets.est.tau.item <- rm_facets_est_tau_item

## File Name: sirt_summary_print_package_rsession.R
## File Version: 0.07

sirt_summary_print_package_rsession <- function(pack)
{
    res <- TAM::tam_packageinfo(pack=pack)
    cat(res,'\n')
    res <- TAM::tam_rsessinfo()
    cat(res, '\n')
}

## File Name: truescore_irt_irf.R
## File Version: 0.04


#---- true score IRT - item response function
truescore_irt_irf <- function( A, B, c, d, theta )
{
    TP <- length(theta)
    truescore <- rep(0, length(theta))
    # extract maximum item categories from B parameters
    nB <- ncol(B)
    maxK <- nB - rowSums( is.na( B ))
    I <- nrow(B)
    scoreM <- matrix( 1L:max(maxK), nrow=TP, ncol=max(maxK), byrow=TRUE )
    for (ii in 1L:I){
        prob.ii <- matrix( NA, TP, maxK[ii] )
        for (kk in seq(1, maxK[ii] ) ){
            prob.ii[,kk] <- exp( theta * A[ii,kk] + B[ii,kk] )
        }
        prob.ii <- prob.ii / ( rowSums( prob.ii ) + 1 )
        if ( ( maxK[ii]==1 ) & ( abs(c[ii])+abs(1-d[ii]) > 0 ) ){
            prob.ii <- c[ii] + ( d[ii] - c[ii] ) * prob.ii
        }
        truescore <- truescore + rowSums( prob.ii * scoreM[, seq( 1, maxK[ii] ),
                            drop=FALSE] )
    }
    return(truescore)
}

## File Name: linking_haberman_lq_pw_le_hess_gamma.R
## File Version: 0.04


linking_haberman_lq_pw_le_hess_gamma <- function(par_delta, par_gamma, des, h=1e-4)
{
    I <- des$I
    G <- des$G
    NPD <- length(par_delta)
    NPG <- length(par_gamma)
    hess <- matrix(0, nrow=NPD, ncol=NPG)
    rownames(hess) <- names(par_delta)
    colnames(hess) <- names(par_gamma)
    grad_fun <- linking_haberman_lq_pw_le_grad
    pgi <- is.na(par_gamma)
    if (sum(pgi)>0){
        par_gamma[ pgi ] <- 0
    }

    for (gg in 1L:G){
        for (ss in 1L:2){
            ind_gg <- 2*I*(gg-1)+seq(ss,2*I,by=2)
            par_gamma2 <- mgsem_add_increment(x=par_gamma, h=h, i1=ind_gg)
            args <- list(par_delta=par_delta, par_gamma=par_gamma2, des=des,h=h)
            opt1 <- do.call(what=grad_fun, args=args)
            args$par_gamma <- mgsem_add_increment(x=par_gamma, h=-h, i1=ind_gg)
            opt2 <- do.call(what=grad_fun, args=args)
            hess1 <- (opt1-opt2)/(2*h)
            hess[,ind_gg] <- t(hess1)
        }
    }

    return(hess)
}

## File Name: rm_facets_postproc_rater_parameters.R
## File Version: 0.09

rm_facets_postproc_rater_parameters <- function( rater.index, dat2, dat2.resp, b.rater, a.rater,
        rater.index1, rater_item_int )
{
    N <- colSums( dat2.resp )
    M1 <- colSums( dat2 ) / N
    N <- stats::aggregate( N, list( rater.index ), sum, na.rm=TRUE )[,2]
    M1 <- stats::aggregate( M1, list( rater.index ), mean, na.rm=TRUE )[,2]
    rater <- data.frame( "rater"=rater.index1[,1], "N"=N, "M"=M1,     "b"=b.rater,    "a"=a.rater )
    rater$thresh <- rater$a * rater$b
    if ( ! rater_item_int){
        rater$b.cent <- rm_facets_center_value(x=rater$b, value=0)
        rater$a.cent <- rm_facets_center_value(x=rater$a, value=1)
    }
    if ( rater_item_int){
        vec <- rm_facets_string_part_extract( x=rater$rater, split="-", part=2)
        rater$b.cent <- rm_facets_center_value_aggregate(x=rater$b, index=vec, value=0)
        rater$a.cent <- rm_facets_center_value_aggregate(x=rater$a, index=vec, value=1)
    }
    #--- output
    return(rater)
}

## File Name: sirt_pem_parameter_sequence_initial_iterations.R
## File Version: 0.07

sirt_pem_parameter_sequence_initial_iterations <- function( pem_parm,
            pem_parameter_sequence, iter )
{
    if (iter < 3){
        for (ii in 0:2){
            if (iter==ii){
                pem_parameter_sequence[[ paste0('P',ii) ]] <- pem_parm
            }
        }
    }
    return(pem_parameter_sequence)
}

## File Name: rm_sdt_create_parm_index_modify_elements.R
## File Version: 0.10


rm_sdt_create_parm_index_modify_elements <- function(x, start_index, type)
{
    is_fixed <- sum( x < 0 )
    ind2 <- start_index + match( x, unique(x[x>0]) ) - 1
    if (is.matrix(x) ){
        ind2 <- matrix(ind2, nrow=nrow(x), ncol=ncol(x))
    }
    x <- ind2
    if (is_fixed){
        x[ is.na(x) ] <- -9
    }
    if (is.matrix(x)){
        partable0 <- NULL
        x_dim <- dim(x)
        K <- x_dim[2]
        for (kk in 1:K){
            par1 <- data.frame(type=type, parindex=x[,kk], row=1:x_dim[1], col=kk)
            partable0 <- rbind(partable0, par1)
        }
    }
    if (is.vector(x)){
        x_dim <- dim(x)
        partable0 <- data.frame(type=type, parindex=x, row=seq_len(length(x)), col=NA)
    }
    partable0$est <- partable0$parindex > 0
    partable0$est <- partable0$est & ( ! duplicated(partable0$parindex) )
    partable0$fixed <- partable0$parindex < 0
    partable0$value <- NA
    return(partable0)
}

## File Name: sirt_max.R
## File Version: 0.02

sirt_max <- function(x, value=0)
{
    y <- x[ ! is.na(x) ]
    if (length(y)>0){
        z <- max(x, na.rm=TRUE)
    } else {
        z <- 0
    }
    return(z)
}

## File Name: mgsem_eval_lp_penalty_vector.R
## File Version: 0.211


mgsem_eval_lp_penalty_vector <- function(x, fac, n, p, eps_approx, deriv, h, a=3.7,
    pen_type="lasso")
{
    # smoothic penalty
    if (pen_type=='smoothic'){
        val <- mgsem_smoothic_penalty(x=x, eps=eps_approx, deriv=deriv)
        val <- fac*val
    }

    # L0 penalty
    if (pen_type=='L0'){
        h <- 1e-4
        gamma <- 50
        val <- mgsem_L0_penalty(x=x, eps=eps_approx, gamma=gamma, deriv=deriv, h=h)
        val <- n*fac*val
    }

    # Lasso penalty
    if (pen_type=='lasso'){
        val <- mgsem_power_fun_differentiable_approx(x=x, p=p,
                            eps=eps_approx, deriv=deriv, approx_method='lp')
        val <- n*fac*val
    }

    # SCAD penalty
    if (pen_type=='scad'){
        if (deriv){
            val <- mgsem_power_fun_differentiable_approx(x=x+h, p=p,
                                eps=eps_approx, deriv=FALSE, approx_method='lp')
            val1 <- mgsem_scad_penalty(x=val, lambda=fac, a=a)
            val <- mgsem_power_fun_differentiable_approx(x=x-h, p=p,
                                eps=eps_approx, deriv=FALSE, approx_method='lp')
            val2 <- mgsem_scad_penalty(x=val, lambda=fac, a=a)
            val <- n*(val1-val2)/(2*h)
        } else {
            val <- mgsem_power_fun_differentiable_approx(x=x, p=p,
                                eps=eps_approx, deriv=FALSE, approx_method='lp')
            val <- n*mgsem_scad_penalty(x=val, lambda=fac, a=a)
        }
    }

    #--- output
    return(val)
}

## File Name: linking_proc_itempars.R
## File Version: 0.09

linking_proc_itempars <- function(itempars)
{
    #*** convert itempars to data frame
    itempars <- as.data.frame( itempars )
    weights_exist <- TRUE
    # include wgt if there does not exist a fifth columm
    if ( ncol(itempars)==4){
        itempars$wgt <- 1
        weights_exist <- FALSE
    }
    # extract studies
    studies <- sort( paste( unique( itempars[,1] ) ) )
    NS <- length(studies)
    # extract items
    items <- sort( paste( unique( itempars[,2] ) ) )
    NI <- length(items)
    # define a and b matrices
    wgtM <- bM <- aM <- matrix(NA, nrow=NI, ncol=NS)
    rownames(wgtM) <- rownames(bM) <- rownames(aM) <- items
    colnames(wgtM) <- colnames(bM) <- colnames(aM) <- studies
    # define item parameters
    for (ss in studies){
        itempars.ss <- itempars[ itempars[,1]==ss, ]
        items_ss <- paste(itempars.ss[,2])
        aM[ items_ss, ss ] <- itempars.ss[,3]
        bM[ items_ss, ss ] <- itempars.ss[,4]
        wgtM[ items_ss, ss ] <- itempars.ss[,5]
    }
    wgtM <- wgtM / matrix( rowSums( wgtM, na.rm=TRUE ), nrow=NI, ncol=NS )
    est_pars <- ! is.na(wgtM)

    is_1pl <- ( stats::sd(aM, na.rm=TRUE) < 1e-5 )

    #--- output
    res <- list(itempars=itempars, NS=NS, NI=NI, items=items, studies=studies,
                wgtM=wgtM, aM=aM, bM=bM, est_pars=est_pars,
                weights_exist=weights_exist, is_1pl=is_1pl)
    return(res)
}

## File Name: latent.regression.em.normal.R
## File Version: 2.19


#----------------------------------------------------------------------------
# Latent Regression: EM algorithm
#    Rasch type model: based on individual likelihood
latent.regression.em.normal <- function( y, X, sig.e,
                                weights=rep(1,nrow(X)),
                                beta.init=rep(0,ncol(X)), sigma.init=1,
                                max.parchange=.0001,
                                maxiter=300,
                                progress=TRUE ){
    #.....................................................................................#
    # INPUT:                                                                              #
    # data          ... item response matrix                                              #
    # X             ... covariates (background variables)                                 #
    # weights       ... sample weights                                                    #
    # beta.init     ... initial estimate of beta coefficients                             #
    # sigma.init    ... initial estimate of residual standard deviation                   #
    # b     ... item difficulties                                                 #
    # a ... item discrimination
    # c ... guessing parameter
    # d ... 1 minus slipping parameter
    # alpha1, alpha2 ... alpha parameter in rasch type model                              #
    # max.parchange ... maximum parameter change                                          #
    # theta.list    ... grid of theta values for evaluation of posterior density          #
    # maxiter       ... maximum number of iterations                                      #
    #.....................................................................................#
    X <- as.matrix(X)   # matrix format
    # init parameters
    beta0 <- beta.init
    sig0 <- sigma.init
    # normalize sample weights
    weights <- length(weights) * weights / sum(weights )
    # initialize iteration index and value of parameter change
    iter <- 1 ; parchange <- 1000
    #***
    # data preparation
    n <- nrow(X)

    EAP <- EAP.like <- y
    SE.EAP <- SE.EAP.like <- sig.e

    #....................................
    # begin iterations
    while( ( parchange > max.parchange ) & ( iter < maxiter ) ){
        # estimate latent regression model (linear model)
        mod <- stats::lm( EAP ~ 0 + X, weights=weights)
        cmod <- stats::coef(mod)
        # Calculation of SD
        sigma <- sqrt( mean( weights* ( SE.EAP^2 + stats::resid(mod)^2 ) ) )
        # fitted values
        fmod <- stats::fitted(mod)
        sigma.res <- stats::sd( stats::resid(mod) )
        prec <- 1 / SE.EAP.like^2 + 1 / sigma^2
        EAP <- ( 1 / SE.EAP.like^2 * EAP.like  + 1/sigma^2 * fmod )/prec
        SE.EAP  <- 1 / sqrt( prec )
        parchange <- max( abs(sigma - sig0), abs( cmod - beta0) )
        if (progress){
            cat( paste("Iteration ", iter,": max parm. change ", round( parchange, 6 ),sep=""),
                     "\n") ; utils::flush.console()
                     }
        # parameter update
        sig0 <- sigma ; beta0 <- cmod ; iter <- iter + 1
                    }
    ############# END ALGORITHM ######################################
    ##################################################################
    # standard errors for regression coefficients
    V <- ncol(X)        # number of X variables (predictors)
    W <- diag(weights)    # diagonal matrix of weights
    # simple covariance matrix
    h1 <- crossprod( X, W ) %*% X
    h2 <- solve(h1)
    vcov.simple <- sigma^2 * h2
    # covariance matrix which includes measurement error
    error.weights <- 1 - SE.EAP^2 / sigma^2
    X1 <- X * sqrt(error.weights)
    h1 <- crossprod( X1, W ) %*% X1
    h2 <- solve(h1)
    vcov.latent <- sigma^2 * h2
    scoefs <- matrix( 0, nrow=V, ncol=9 )
    scoefs <- data.frame(scoefs)
    colnames(scoefs) <- c("est", "se.simple", "se", "t", "p",
                            "beta", "fmi", "N.simple", "pseudoN.latent" )
    rownames(scoefs) <- names(beta0)
    scoefs[,1] <- beta0
    scoefs[, "se.simple"] <- sqrt( diag( vcov.simple) )
    scoefs[, "se"] <- sqrt( diag( vcov.latent) )
    # explained variance
    explvar <- stats::var( fmod )
    totalvar <- explvar + sigma^2
    rsquared <- explvar / totalvar
    scoefs$beta <- scoefs$est / sqrt( totalvar ) * apply( X, 2, stats::sd )
    scoefs$fmi <- 1 -  scoefs$se.simple^2 / scoefs$se^2
#    scoefs[,"N.simple" ] <- nrow(data)
    scoefs[,"N.simple" ] <- nrow(X)

    scoefs[,"pseudoN.latent"] <- sum( error.weights )
    scoefs$t <- scoefs$est / scoefs$se
    scoefs$p <- 2 * ( 1 - stats::pnorm( abs( scoefs$t ) ) )
    if ( ! is.null( colnames(X) ) ){ rownames(scoefs) <- colnames(X) } # use column names of X
    if (progress){
        cat("\nRegression Parameters\n\n")
        .prnum(scoefs,4)        # print results
        cat( paste( "\nResidual Variance=", round( sigma^2, 4 ) ), "\n" )
        cat( paste( "Explained Variance=", round( explvar, 4 ) ), "\n" )
        cat( paste( "Total Variance=", round( totalvar, 4 ) ), "\n" )
        cat( paste( "            R2=", round( rsquared, 4 ) ), "\n" )
                }
    #********
    # list of results
    res <- list( "iterations"=iter - 1, "maxiter"=maxiter,
                "max.parchange"=max.parchange,
                "coef"=beta0, "summary.coef"=scoefs, "sigma"=sigma,
                "vcov.simple"=vcov.simple, "vcov.latent"=vcov.latent,
                "EAP"=EAP, "SE.EAP"=SE.EAP,
                "explvar"=explvar, "totalvar"=totalvar,
                "rsquared"=rsquared )
    class(res) <- "latent.regression"
    return(res)
    }
#-----------------------------------------------------------------------------------------------------

## File Name: xxirt_data_proc.R
## File Version: 0.215

#-- data processing xxirt
xxirt_data_proc <- function(dat, group=NULL, weights=NULL )
{
    ncat <- apply( dat, 2, max, na.rm=TRUE ) + 1
    I <- ncol(dat)
    items <- colnames(dat)
    N <- nrow(dat)
    if ( is.null(group) ){
        group <- rep(1,N)
    }
    if ( is.null(weights) ){
        weights <- rep(1,N)
    }
    W <- sum(weights)
    group0 <- group
    groups_unique <- sort( unique( group ) )
    G <- length(groups_unique)
    group <- match( group0, groups_unique )
    maxK <- max(ncat)
    #*** group_index
    group_index <- as.list( 1L:G )
    for (gg in 1L:G){
        group_index[[gg]] <- which( group==gg )
    }
    #*** data with response indices
    dat_na <- is.na(dat)
    dat_resp_bool <- ! dat_na
    dat_resp <- 1 - dat_na
    resp_index <- as.list( 1L:I )
    for ( ii in 1L:I){
        resp_index[[ii]] <- which( dat_resp[,ii]==1 )
    }
    dat1 <- as.matrix(dat)
    dat1[ dat_na ] <- 0
    #*** output
    res <- list( N=N, I=I, group=group, items=items,
                    group0=group0, G=G, groups_unique=groups_unique,
                    maxK=maxK, ncat=ncat, weights=weights,
                    group_index=group_index, dat_resp=dat_resp,
                    resp_index=resp_index, dat1=dat1, dat_resp_bool=dat_resp_bool,
                    W=W)
    return(res)
}

## File Name: mcmc.2pno.ml_output.R
## File Version: 1.15

######################################################
# subfunction for calculating the DIC
.mcmc.ic.2pno.ml <- function( aM.chainsum, bM.chainsum, theta.chain, N, I,
        dat, dat.resp,eps, deviance.chain, groupsize,
        sigma.res.chain, link ){
    #************
    weights <- NULL
#    aM <- matrix( colMeans( a.chain ), nrow=N, ncol=I, byrow=TRUE )
#    bM <- matrix( colMeans( b.chain ), nrow=N, ncol=I, byrow=TRUE )
    SV <- nrow(theta.chain )
    aM <- aM.chainsum / SV
    bM <- bM.chainsum / SV
    theta <- colMeans( theta.chain )
    if (link=="logit"){
        Dhat <- .mcmc.deviance.2pl( aM, bM, theta, dat, dat.resp,
                    weights, eps )
                        }
    if (link=="normal"){
        sigma.res <- as.vector(colMeans( sigma.res.chain ) )
        Dhat <- .mcmc.deviance.normallink.2pno.ml( aM, bM,
                        theta, N, I, dat, dat.resp, sigma.res )
                        }
    Dbar <- mean( deviance.chain )
    pD <- Dbar - Dhat
    ic <- list( "Dhat"=Dhat, "Dbar"=Dbar, "pD"=pD, "DIC"=Dhat + 2*pD )
    # group statistics
    ic$G <- length(groupsize)
    ic$M.n <- mean(groupsize)
    ic$SD.n <- stats::sd(groupsize)
    return(ic)
        }
#######################################################
# mcmc.list output object
.mcmc.mcmclist.2pno.ml <- function( a.chain, b.chain, deviance.chain,
    sigma1.chain, sigma2.chain, I, SV, burnin, iter, est.b.M,
    mu.b.chain, omega.b.chain, sigma.b.chain, est.b.Var,
    est.a.M, est.a.Var, omega.a.chain, sigma.a.chain,
    sigma.res.chain, link    ){
    #****
    a <- a.chain
    b <- b.chain
    colnames(a) <- paste0("a[", 1:I, "]")
    colnames(b) <- paste0("b[", 1:I, "]")
    mcmcobj <- cbind( deviance.chain, b )
    # include a parameters
    if( max( apply( a, 2, stats::sd) ) > 0 ){
        mcmcobj <- cbind( mcmcobj, a )
                }
    colnames(mcmcobj)[1] <- "deviance"
    mcmcobj <- cbind(  mcmcobj, "sigma1"=sigma1.chain,
            "sigma2"=sigma2.chain )
    ICC <- sigma2.chain^2 / ( sigma2.chain^2 + sigma1.chain^2 )
    mcmcobj <- cbind( mcmcobj, "ICC"=ICC )
    if ( est.b.M=="h"){ mcmcobj <- cbind( mcmcobj,
                "mu.b"=mu.b.chain, "omega.b"=omega.b.chain )
                        }
    #*** est.b.Var
    if ( est.b.Var %in% c("i") ){
        colnames(sigma.b.chain) <- paste0("sigma.b[", 1:I,"]")
        mcmcobj <- cbind( mcmcobj, sigma.b.chain )
                            }
    if ( est.b.Var=="j" ){
        mcmcobj <- cbind( mcmcobj, "sigma.b"=sigma.b.chain[,1] )
                            }
    #**** est.a.Var
    if ( est.a.M !="f"){
        if ( est.a.Var=="i" ){
            colnames(sigma.a.chain) <- paste0("sigma.a[", 1:I,"]")
            mcmcobj <- cbind( mcmcobj, sigma.a.chain )
                                }
        if ( est.a.Var=="j" ){
            mcmcobj <- cbind( mcmcobj, "sigma.a"=sigma.a.chain[,1] )
                                }
                        }
    if ( est.a.M=="h"){ mcmcobj <- cbind( mcmcobj,
                 "omega.a"=omega.a.chain )
                        }
    if ( link=="normal" ){
        colnames(sigma.res.chain) <- paste0("sigma.res[", 1:I,"]")
        mcmcobj <- cbind( mcmcobj, sigma.res.chain )
                            }
    class(mcmcobj) <- "mcmc"
    attr(mcmcobj, "mcpar") <- c( burnin+1, burnin+SV, 1 )
    mcmcobj <- sirt_import_coda_as.mcmc.list( mcmcobj )
    return(mcmcobj)
        }
######################################################
# calculating the deviance under the normal link function
.mcmc.deviance.normallink.2pno.ml <- function( aM, bM,
        theta, N, I, dat, dat.resp, sigma.res ){
    mexp <- aM*theta - bM
    mres <- dat - mexp
    llres <- - sum( dat.resp *
        log( sirt_dnorm( mres, sd=matrix( sigma.res, N, I, byrow=TRUE ) )    ) )
    return(llres)
        }


####################################################################
# calculate EAP reliability, person parameter EAPs
# and corresponding posterior SDs
.mcmc.person.2pno.ml <- function( theta.chain, weights ){
    ###################
    # EAP reliability
        v1 <- stats::var( colMeans( theta.chain ) )
        if ( is.null(weights) ){
            weights <- rep(1,ncol(theta.chain)) }
        w1 <- weights / sum(weights )
        m1 <- colMeans( theta.chain )
        v1 <- sum( m1^2 * w1 ) - ( sum( m1*w1 ) )^2
        wM <- matrix( w1, nrow=nrow(theta.chain), ncol=ncol(theta.chain), byrow=TRUE )
        h1 <- rowSums( wM * theta.chain^2 ) - ( rowSums( wM * theta.chain ) )^2
        h1 <- mean(h1)
        EAP.rel <- v1 / h1
    ###################
    # person parameter estimates
    person <- data.frame( "EAP"=colMeans( theta.chain ),
                        "SD"=colSds(theta.chain) )
    # output
    res <- list( "EAP.rel"=EAP.rel, "person"=person )
    return(res)
            }
###############################################################

## File Name: lsem_fitsem_joint_estimation_prepare_partable.R
## File Version: 0.321

lsem_fitsem_joint_estimation_prepare_partable <- function(partable, G,
    par_invariant=NULL, par_linear=NULL, par_quadratic=NULL, pw_linear=1,
    pw_quadratic=1)
{
    partable0 <- partable
    partable$id0 <- 1L:nrow(partable)
    partable$con <- 0
    label_list <- partable$plabel
    partable1 <- lsem_fitsem_joint_estimation_prepare_partable_include_group_label(
                        partable=partable, gg=1, label_list=label_list)
    partable_mg <- partable1
    for (gg in 2L:G){
        partable_gg <- partable
        partable_gg <- lsem_fitsem_joint_estimation_prepare_partable_include_group_label(
                            partable=partable_gg, gg=gg, label_list=label_list)
        partable_gg$group <- partable_gg$block <- gg
        for (vv in c('free','id')){
            partable_gg <- lsem_fitsem_joint_estimation_partable_id(
                                partable_gg=partable_gg, partable_mg=partable_mg, vv=vv)
            partable_gg[,vv][ partable[,vv]==0 ] <- 0
        }
        partable_gg$plabel <- paste0(label_list,'g',gg)
        partable_gg$plabel[ label_list==''] <- ''
        partable_mg <- rbind(partable_mg, partable_gg)
    }

    #- parameter names
    pars <- sirt_lavaan_partable_parnames(partable=partable_mg)
    partable_mg$par <- pars

    # handle constraints
    fixed_invariant <- intersect( paste(partable_mg$par[ partable$free==0]),
                                    par_invariant )
    par_invariant <- setdiff( par_invariant, fixed_invariant)
    par1 <- sirt_define_vector( value='inv', names=par_invariant)
    par2 <- sirt_define_vector( value='lin', names=par_linear)
    par3 <- sirt_define_vector( value='quad', names=par_quadratic)
    par_vec <- c(par1, par2, par3)
    NI <- length(par_vec)
    par_vec_names <- names(par_vec)

    if ( NI > 0 ){
        partable1 <- partable_mg[1,]
        NV <- ncol(partable_mg)
        for (vv in 1L:NV){
            if (is.numeric(partable1[1,vv])){
                partable1[1,vv] <- 0
            } else {
                partable1[1,vv] <- ''
            }
        }
        partable1$user <- 2
        partable1$ustart <- NA
        partable1$con <- 0
        partable1$op <- '=='
        partable1c <- partable1

        for (vv in 1L:NI){
            par_vec_vv <- par_vec[vv]
            par_vec_names_vv <- par_vec_names[vv]
            ind_vv <- which( paste(partable_mg$par)==par_vec_names[vv] )
            LV2 <- LV <- length(ind_vv)
            N_segments <- 1
            type1 <- 'inv'
            if (par_vec_vv=='lin'){
                LV2 <- LV - 1
                N_segments <- pw_linear
                type1 <- 'lin'
            }
            if (par_vec_vv=='quad'){
                LV2 <- LV - 2
                N_segments <- pw_quadratic
                type1 <- 'quad'
            }
            plabels <- paste(partable_mg$plabel[ind_vv])
            delta <- (LV-1)/N_segments
            par_segments <- data.frame(group=1L:LV)
            par_segments$N_segments <- N_segments
            par_segments$type <- type1

            par_segments$segment <- ceiling( ( par_segments$group - 1 ) / delta )
            par_segments$segment <- ifelse( par_segments$segment==0, 1,
                                                par_segments$segment)

            # exclude segments
            par_segments$diff_segment <- c(1,diff(par_segments$segment))
            indices <- par_segments$group[ par_segments$diff_segment==1 ]
            par_segments$con_include <- 1 - par_segments$diff_segment

            if (type1=='lin'){
                indices2 <- setdiff( c(LV,indices-1), seq(-4,1,1) )
                par_segments[indices2, 'con_include'] <- 0
            }
            if (type1=='quad'){
                indices2 <- setdiff( c(LV,indices-1,indices-2), seq(-4,1,1) )
                par_segments[indices2, 'con_include'] <- 0
            }

            for (ll in 2L:LV2){
                partable1c$con <- vv
                if (par_vec_vv=='inv'){
                    partable1c$lhs <- plabels[1]
                    partable1c$rhs <- plabels[ll]
                }
                if (par_vec_vv=='lin'){
                    diff1 <- paste0( plabels[ll+1], '-2*', plabels[ll], '+',
                                            plabels[ll-1] )
                    partable1c$lhs <- diff1
                    partable1c$rhs <- 0
                    partable1c$user <- 1
                }
                if (par_vec_vv=='quad'){
                    diff1 <- paste0( plabels[ll+2], '-3*', plabels[ll+1],
                                        '+3*', plabels[ll], '-', plabels[ll-1] )
                    partable1c$lhs <- diff1
                    partable1c$rhs <- 0
                    partable1c$user <- 1
                }
                partable1c$par <- paste0(par_vec_names_vv, '_con', ll-1)
                partable1c$id <- max(partable_mg$id) + 1
                if (par_segments$con_include[ll]==1){
                    partable_mg <- rbind(partable_mg, partable1c)
                }
            }
        }
    }


    #- handle parameter labels
    # same_labels <- setdiff( unique(paste(partable_mg$label)), '')

    #-- output
    return(partable_mg)
}

## File Name: lsem_define_lavaan_est_fun.R
## File Version: 0.101

lsem_define_lavaan_est_fun <- function(lavaan_fct)
{
    TAM::require_namespace_msg('lavaan')
    if (lavaan_fct=='sem'){
        lavaan_est_fun <- lavaan::sem
    }
    if (lavaan_fct=='lavaan'){
        lavaan_est_fun <- lavaan::lavaan
    }
    if (lavaan_fct=='cfa'){
        lavaan_est_fun <- lavaan::cfa
    }
    if (lavaan_fct=='growth'){
        lavaan_est_fun <- lavaan::growth
    }
    return(lavaan_est_fun)
}

## File Name: xxirt_irf_lca_init_par.R
## File Version: 0.05


xxirt_irf_lca_init_par <- function(K, ncat, random_sd=0, rg_val=1.5)
{
    v1 <- NULL
    for (cc in 2L:ncat){
        par <- rg_val*seq( -1, 1, length=K )
        names(par) <- paste0('b',1L:K, '_Cat',cc-1)
        par <- par + stats::rnorm(K, sd=random_sd)
        v1 <- c(v1, par)
    }
    if (ncat==2){
        names(v1) <- paste0('b',1L:K)
    }
    return(v1)
}

## File Name: rasch.mml.raschtype.R
## File Version: 2.641



#*******************************************************
# utility function squeeze
squeeze.mml2 <- function( v1, rgvec ){
    v1 <- ifelse( v1 < rgvec[1], rgvec[1], v1 )
    v1 <- ifelse( v1 > rgvec[2], rgvec[2], v1 )
    return(v1)
        }
#*******************************************************


##################################################
# estimation of group means in the 1dim IRT model
.est.mean <- function( dat1.gg, f.yi.qk.gg, X1, pi.k, pi.k0, gg,
                            mean.trait, sd.trait, theta.k, h)
{
    ll0 <- sum( dat1.gg * log( rowSums( f.yi.qk.gg * outer( X1, pi.k[,gg] ) ) ) )
    pi.k2 <- pi.k0
    pi.k2[,gg] <- sirt_dnorm_discrete( theta.k, mean=mean.trait[gg]+h, sd=sd.trait[gg]  )
    ll1 <- sum( dat1.gg * log( rowSums( f.yi.qk.gg * outer( X1, pi.k2[,gg] ) ) ))
    pi.k2 <- pi.k0
    pi.k2[,gg] <- sirt_dnorm_discrete( theta.k, mean=mean.trait[gg] -h, sd=sd.trait[gg]  )
    ll2 <- sum( dat1.gg * log( rowSums( f.yi.qk.gg * outer( X1, pi.k2[,gg] ) ) ) )
    d1 <- ( ll1 - ll2  ) / ( 2 * h )    # negative sign?
    d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
    d2[ abs(d2) < 10^(-15) ]  <- 10^(-15)
    d.change <- - d1 / d2
    d.change <- ifelse( abs(d.change) > .1, .1*sign(d.change), d.change )
    return(d.change)
}
##################################################



##################################################
# estimation of group SD's in the 1dim IRT model
.est.sd <- function( dat1.gg, f.yi.qk.gg, X1, pi.k, pi.k0, gg,
                            mean.trait, sd.trait, theta.k, h){
        ll0 <- sum( dat1.gg * log( rowSums( f.yi.qk.gg * outer( X1, pi.k[,gg] ) ) ) )
        pi.k2 <- pi.k
        pi.k2[,gg] <- sirt_dnorm_discrete( theta.k, mean=mean.trait[gg], sd=sd.trait[gg] + h )
        ll1 <- sum( dat1.gg * log( rowSums( f.yi.qk.gg * outer( X1, pi.k2[,gg] ) ) ) )
        pi.k2 <- pi.k
        pi.k2[,gg] <- sirt_dnorm_discrete( theta.k, mean=mean.trait[gg], sd=sd.trait[gg] - h )
        pi.k2[,gg] <- pi.k2[,gg] / sum( pi.k2[,gg] )
        ll2 <- sum( dat1.gg * log( rowSums( f.yi.qk.gg * outer( X1, pi.k2[,gg] ) ) ) )
        d1 <- ( ll1 - ll2  ) / ( 2 * h )    # negative sign?
        d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
        d2[ abs(d2) < 10^(-15) ]  <- 10^(-15)
        d.change <- - d1 / d2
        d.change <- ifelse( abs(d.change) > .05, .05*sign(d.change), d.change )
        return(d.change )
                            }
##################################################

## File Name: rm_sdt_calc_probs_gpcm.R
## File Version: 0.11

rm_sdt_calc_probs_gpcm <- function(a.item, tau.item, Qmatrix, theta.k, VV, K, TP,
    eps=0, use_log=FALSE)
{
    a <- a.item
    b <- tau.item
    res <- rm_pcm_calcprobs( a=a, b=b, Qmatrix=Qmatrix, theta.k=theta.k, I=VV, K=K, TP=TP )
    if (eps > 0){
        res[ res < eps ] <- eps
    }
    if (use_log){
        res <- log(res)
    }
    return(res)
}

## File Name: xxirt_classprobs_lca_init_par_create.R
## File Version: 0.04


xxirt_classprobs_lca_init_par_create <- function(K, random_sd=0)
{
    par_Theta <- rep(0, K-1)
    par_Theta <- par_Theta + stats::rnorm(K-1, sd=random_sd)
    return(par_Theta)
}

## File Name: rm_facets_pp_mle_calc_ll.R
## File Version: 0.151


# calculate individual likelihood for item ii
rm_facets_pp_mle_calc_ll <- function( probs, data, ii, eps=1e-20 )
{
    N <- nrow(data)
    probs <- log(probs)
    m1 <- matrix(1L:N, nrow=N, ncol=2)
    m1[,2] <- data[,ii] + 1
    h1 <- probs[ m1 ]
    h1[ is.na(h1) ] <- 0
    return(h1)
}

## File Name: sirt_optimizer_hessian.R
## File Version: 0.04

sirt_optimizer_hessian <- function(res, fn, grad, h, ...)
{
    a1 <- list(...)
    arglist <- list(...)
    if (!is.null(grad)){
        fun <- grad
        hess_fun <- CDM::numerical_gradient
    } else {
        fun <- fn
        hess_fun <- CDM::numerical_Hessian
    }
    arglist <- sirt_remove_arguments_function(fun=fun, args=arglist)
    arglist$par <- res$par
    arglist$FUN <- fun
    arglist$h <- h
    res$hessian <- do.call(what=hess_fun, args=arglist)

    #--- output
    return(res)
}

## File Name: locpolycor_est_thresh_item.R
## File Version: 0.189


locpolycor_est_thresh_item <- function(y, data.mod, x0, w, model, par_init=NULL,
        eps=1e-10, optimizer="optim", use_deriv=TRUE)
{
    K <- max(y)
    N <- length(y)
    x1 <- data.mod
    # starting values
    par0 <- stats::qnorm( seq(1/(K+1), K/(K+1), length=K) )
    if (model=='lin'){
        par1 <- 0*par0
        names(par1) <- paste0('lin', 1L:K)
        par0 <- c(par0, par1)
    }
    if (! is.null(par_init) ){
        par0 <- par_init
    }
    #- optimization
    if (use_deriv){
        grad <- locpolycor_est_thresh_grad_fun
    } else {
        grad <- NULL
    }

    res_optim <- sirt_optimizer(optimizer=optimizer, par=par0,
                        fn=locpolycor_est_thresh_opt_fun,
                        grad=locpolycor_est_thresh_grad_fun,
                        y=y, x1=x1, x0=x0, w=w, model=model, method='L-BFGS-B',
                        K=K, eps=eps, hessian=FALSE )

    #- local thresholds across all persons at x0
    thresh <- res_optim$par
    if (model=='lin'){
        thresh <- thresh[1L:K]
    }
    #- individual threshold predictions
    thresh_ind <- matrix(NA, nrow=N, ncol=K)
    colnames(thresh_ind) <- paste0('Cat',1L:K)
    for (kk in 1L:K){
        if (model=='const'){
            thresh_ind[,kk] <- thresh[kk]
        }
        if (model=='lin'){
            thresh_ind[,kk] <- thresh[kk] + res_optim$par[K+kk] * (x1-x0)
        }
    }

    #*** output
    res <- list(thresh=thresh, thresh_ind=thresh_ind, x0=x0, res_optim=res_optim, N=N,
                        W=sum(w), K=K, model=model)
    return(res)
}

## File Name: rasch_mml2_prob_genlogis_4pl_evaluate.R
## File Version: 0.08


rasch_mml2_prob_genlogis_4pl_evaluate <- function(theta.k, b, fixed.a, fixed.c,
    fixed.d, alpha1, alpha2, Qmatrix, h, h1, h2, incr=NULL)
{
    if (!is.null(incr) & (h!=0) ){
        for (vv in incr){
            if (vv=="b"){ b <- b + h }
            if (vv=="a"){ fixed.a <- fixed.a + h }
            if (vv=="c"){ fixed.c <- fixed.c + h }
            if (vv=="d"){ fixed.d <- fixed.d + h }
            if (vv=="alpha1"){ alpha1 <- alpha1 + h }
            if (vv=="alpha2"){ alpha2 <- alpha2 + h }
        }
    }
    pjk <- prob_genlogis_4pl(theta=theta.k, b=b, a=fixed.a, c=fixed.c,
                d=fixed.d, alpha1=alpha1, alpha2=alpha2, Qmatrix=Qmatrix)
    pjk <- ( pjk + h1 ) / h2
    pjk.M <- t(pjk)
    qjk.M <- 1 - pjk.M
    #-- output
    res <- list(pjk.M=pjk.M, qjk.M=qjk.M)
    return(res)
}

## File Name: sirt_remove_list_entries.R
## File Version: 0.02

sirt_remove_list_entries <- function(list, rem)
{
    NR <- length(rem)
    if (NR>0){
        for (rr in rem){
            list[[rr]] <- NULL
        }
    }
    return(list)
}

## File Name: rm_determine_fixed_tau_parameters.R
## File Version: 0.082

rm_determine_fixed_tau_parameters <- function( K, maxK, VV, tau.item.fixed=NULL, val=99)
{
    if ( min(maxK) < K ){
        for (vv in 1L:VV){
            K.vv <- maxK[vv]
            if ( K.vv < K ){
                for (zz in (K.vv+1):K ){
                    d1 <- data.frame( item=vv, categ=zz, val=val)
                    tau.item.fixed <- rbind( tau.item.fixed, d1 )
                }
            }
        }
        tau.item.fixed <- as.matrix(tau.item.fixed )
    }
    return(tau.item.fixed)
}

## File Name: RcppExports.R
## File Version: 4.002133
# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

gooijer_csn_table <- function(dat, dat_perm, RR, NS, progress, progress_vec, score_index) {
    .Call('_sirt_gooijer_csn_table', PACKAGE='sirt', dat, dat_perm, RR, NS, progress, progress_vec, score_index)
}

isop_tests_C <- function(dat, dat_resp, weights, jackunits, JJ) {
    .Call('_sirt_isop_tests_C', PACKAGE='sirt', dat, dat_resp, weights, jackunits, JJ)
}

interval_index_C <- function(MATR, RN) {
    .Call('_sirt_interval_index_C', PACKAGE='sirt', MATR, RN)
}

rowCumsums2_source <- function(input) {
    .Call('_sirt_rowCumsums2_source', PACKAGE='sirt', input)
}

rowKSmallest_C <- function(MATR, KK1, INDEXMATR, RNMATR) {
    .Call('_sirt_rowKSmallest_C', PACKAGE='sirt', MATR, KK1, INDEXMATR, RNMATR)
}

rowMaxsCPP_source <- function(MATR) {
    .Call('_sirt_rowMaxsCPP_source', PACKAGE='sirt', MATR)
}

rowmins2_bundle_C <- function(m1, v1) {
    .Call('_sirt_rowmins2_bundle_C', PACKAGE='sirt', m1, v1)
}

calc_copula_itemcluster_C <- function(DD, res) {
    .Call('_sirt_calc_copula_itemcluster_C', PACKAGE='sirt', DD, res)
}

md_pattern_csource <- function(dat) {
    .Call('_sirt_md_pattern_csource', PACKAGE='sirt', dat)
}

mle_pcm_group_C <- function(dat, dat_resp, groupM, b, a, maxK, theta0, conv, maxiter) {
    .Call('_sirt_mle_pcm_group_C', PACKAGE='sirt', dat, dat_resp, groupM, b, a, maxK, theta0, conv, maxiter)
}

mle_pcm_C <- function(dat, dat_resp, b, a, maxK, theta0, conv, maxiter) {
    .Call('_sirt_mle_pcm_C', PACKAGE='sirt', dat, dat_resp, b, a, maxK, theta0, conv, maxiter)
}

probs_pcm_groups_C <- function(dat, dat_resp, group, probs, CC, TP) {
    .Call('_sirt_probs_pcm_groups_C', PACKAGE='sirt', dat, dat_resp, group, probs, CC, TP)
}

sirt_rcpp_probs_pcm_groups_C <- function(dat, dat_resp, group, probs, CC, TP) {
    .Call('_sirt_sirt_rcpp_probs_pcm_groups_C', PACKAGE='sirt', dat, dat_resp, group, probs, CC, TP)
}

probs_pcm_nogroups_C <- function(dat, dat_resp, probs, CC, TP) {
    .Call('_sirt_probs_pcm_nogroups_C', PACKAGE='sirt', dat, dat_resp, probs, CC, TP)
}

calccounts_pcm_groups_C <- function(dat, dat_resp, group, fyiqk, pik, CC, weights) {
    .Call('_sirt_calccounts_pcm_groups_C', PACKAGE='sirt', dat, dat_resp, group, fyiqk, pik, CC, weights)
}

sirt_rcpp_calccounts_pcm_groups_C <- function(dat, dat_resp, group, fyiqk, pik, CC, weights) {
    .Call('_sirt_sirt_rcpp_calccounts_pcm_groups_C', PACKAGE='sirt', dat, dat_resp, group, fyiqk, pik, CC, weights)
}

sirt_rcpp_array_mult <- function(AA, dimAA, BB, dimBB) {
    .Call('_sirt_sirt_rcpp_array_mult', PACKAGE='sirt', AA, dimAA, BB, dimBB)
}

RM_CALCPOST <- function(DAT2, DAT2RESP, PROBS, KK) {
    .Call('_sirt_RM_CALCPOST', PACKAGE='sirt', DAT2, DAT2RESP, PROBS, KK)
}

rm_probraterfct1 <- function(CRA, DRA, dimAA, BB, dimBB) {
    .Call('_sirt_rm_probraterfct1', PACKAGE='sirt', CRA, DRA, dimAA, BB, dimBB)
}

rm_facets_calcprobs_cpp <- function(b_item, b_rater, Qmatrix, tau_item, K, I, TP, a_item, a_rater, item_index, rater_index, theta_k) {
    .Call('_sirt_rm_facets_calcprobs_cpp', PACKAGE='sirt', b_item, b_rater, Qmatrix, tau_item, K, I, TP, a_item, a_rater, item_index, rater_index, theta_k)
}

SMIRT_CALCPOST <- function(DAT2, DAT2RESP, PROBS, DAT2IND, PIK, KK1) {
    .Call('_sirt_SMIRT_CALCPOST', PACKAGE='sirt', DAT2, DAT2RESP, PROBS, DAT2IND, PIK, KK1)
}

SMIRT_CALCPROB_COMP <- function(A, B, QQ, THETA, CC, DD) {
    .Call('_sirt_SMIRT_CALCPROB_COMP', PACKAGE='sirt', A, B, QQ, THETA, CC, DD)
}

SMIRT_CALCPROB_NONCOMP <- function(A, B, QQ, THETA, CC, DD) {
    .Call('_sirt_SMIRT_CALCPROB_NONCOMP', PACKAGE='sirt', A, B, QQ, THETA, CC, DD)
}

SMIRT_CALCPROB_PARTCOMP <- function(A, B, QQ, THETA, CC, DD, MUI) {
    .Call('_sirt_SMIRT_CALCPROB_PARTCOMP', PACKAGE='sirt', A, B, QQ, THETA, CC, DD, MUI)
}

MML2_RASCHTYPE_COUNTS <- function(DAT2, DAT2RESP, DAT1, FQKYI, PIK, FYIQK) {
    .Call('_sirt_MML2_RASCHTYPE_COUNTS', PACKAGE='sirt', DAT2, DAT2RESP, DAT1, FQKYI, PIK, FYIQK)
}

MML2_CALCPOST_V1 <- function(DAT2, DAT2RESP, PROBS) {
    .Call('_sirt_MML2_CALCPOST_V1', PACKAGE='sirt', DAT2, DAT2RESP, PROBS)
}

sirt_rcpp_rasch_mml2_calcpost_pseudoll <- function(DAT2, DAT2RESP, PROBS) {
    .Call('_sirt_sirt_rcpp_rasch_mml2_calcpost_pseudoll', PACKAGE='sirt', DAT2, DAT2RESP, PROBS)
}

MML2_CALCPOST_V3 <- function(DAT2, DAT2RESP, PROBS) {
    .Call('_sirt_MML2_CALCPOST_V3', PACKAGE='sirt', DAT2, DAT2RESP, PROBS)
}

sirt_rcpp_ccov_np_compute_ccov_sum_score <- function(index, NS, data) {
    .Call('_sirt_sirt_rcpp_ccov_np_compute_ccov_sum_score', PACKAGE='sirt', index, NS, data)
}

sirt_rcpp_discrete_inverse <- function(x0, y0, y) {
    .Call('_sirt_sirt_rcpp_discrete_inverse', PACKAGE='sirt', x0, y0, y)
}

sirt_rcpp_first_eigenvalue <- function(X, maxit, conv, K) {
    .Call('_sirt_sirt_rcpp_first_eigenvalue', PACKAGE='sirt', X, maxit, conv, K)
}

sirt_rcpp_D_eigenvalues <- function(Xr, D, maxit, conv) {
    .Call('_sirt_sirt_rcpp_D_eigenvalues', PACKAGE='sirt', Xr, D, maxit, conv)
}

sirt_rcpp_choppin_row_averaging <- function(B, I, priorweight) {
    .Call('_sirt_sirt_rcpp_choppin_row_averaging', PACKAGE='sirt', B, I, priorweight)
}

sirt_rcpp_evm_compute <- function(B, I, powD, maxit, conv, K) {
    .Call('_sirt_sirt_rcpp_evm_compute', PACKAGE='sirt', B, I, powD, maxit, conv, K)
}

sirt_rcpp_evm_comp_poly <- function(dat, dat_resp, weights, JJ, jackunits, powD, progress_, row_index, col_index) {
    .Call('_sirt_sirt_rcpp_evm_comp_poly', PACKAGE='sirt', dat, dat_resp, weights, JJ, jackunits, powD, progress_, row_index, col_index)
}

sirt_rcpp_gom_em_calcpost <- function(DAT2, DAT2RESP, PROBS, DAT2IND, PIK, KK1, weights) {
    .Call('_sirt_sirt_rcpp_gom_em_calcpost', PACKAGE='sirt', DAT2, DAT2RESP, PROBS, DAT2IND, PIK, KK1, weights)
}

sirt_rcpp_gom_em_likelihood <- function(probs, ncat, TP, dat2, dat2resp) {
    .Call('_sirt_sirt_rcpp_gom_em_likelihood', PACKAGE='sirt', probs, ncat, TP, dat2, dat2resp)
}

sirt_rcpp_gom_em_log_likelihood <- function(fyiqk, pik, weights) {
    .Call('_sirt_sirt_rcpp_gom_em_log_likelihood', PACKAGE='sirt', fyiqk, pik, weights)
}

sirt_rcpp_gom_em_loglike_gradient <- function(probs, probs_h, ncat, TP, dat2, dat2resp, pik, items, fyiqk, weights) {
    .Call('_sirt_sirt_rcpp_gom_em_loglike_gradient', PACKAGE='sirt', probs, probs_h, ncat, TP, dat2, dat2resp, pik, items, fyiqk, weights)
}

sirt_rcpp_inference_jackknife <- function(PARS) {
    .Call('_sirt_sirt_rcpp_inference_jackknife', PACKAGE='sirt', PARS)
}

sirt_rcpp_invariance_alignment_lambda_transformed <- function(lambda, psi0, meth) {
    .Call('_sirt_sirt_rcpp_invariance_alignment_lambda_transformed', PACKAGE='sirt', lambda, psi0, meth)
}

sirt_rcpp_invariance_alignment_nu_transformed <- function(nu, lambda, alpha0, psi0, reparam, meth) {
    .Call('_sirt_sirt_rcpp_invariance_alignment_nu_transformed', PACKAGE='sirt', nu, lambda, alpha0, psi0, reparam, meth)
}

sirt_rcpp_invariance_alignment_simplicity_function_value <- function(type, parm1, parm2, scale, power, eps) {
    .Call('_sirt_sirt_rcpp_invariance_alignment_simplicity_function_value', PACKAGE='sirt', type, parm1, parm2, scale, power, eps)
}

sirt_rcpp_invariance_alignment_opt_fct <- function(nu, lambda, alpha0, psi0, group_combis, wgt, align_scale, align_pow, eps, wgt_combi, type, reparam, meth) {
    .Call('_sirt_sirt_rcpp_invariance_alignment_opt_fct', PACKAGE='sirt', nu, lambda, alpha0, psi0, group_combis, wgt, align_scale, align_pow, eps, wgt_combi, type, reparam, meth)
}

sirt_rcpp_invariance_alignment_simplicity_function_gradient <- function(type, parm1, parm2, scale, power, eps) {
    .Call('_sirt_sirt_rcpp_invariance_alignment_simplicity_function_gradient', PACKAGE='sirt', type, parm1, parm2, scale, power, eps)
}

sirt_rcpp_invariance_alignment_opt_grad <- function(nu, lambda, alpha0, psi0, group_combis, wgt, align_scale, align_pow, eps, wgt_combi, type, reparam, meth) {
    .Call('_sirt_sirt_rcpp_invariance_alignment_opt_grad', PACKAGE='sirt', nu, lambda, alpha0, psi0, group_combis, wgt, align_scale, align_pow, eps, wgt_combi, type, reparam, meth)
}

sirt_rcpp_linking_haebara_fct_optim_one_item <- function(theta, prob_theta, aM, bM, a, b, mu, sigma, ii, ss, dist, eps, pow) {
    .Call('_sirt_sirt_rcpp_linking_haebara_fct_optim_one_item', PACKAGE='sirt', theta, prob_theta, aM, bM, a, b, mu, sigma, ii, ss, dist, eps, pow)
}

sirt_rcpp_linking_haebara_fct_optim <- function(NI, NS, dist, aM, bM, theta, prob_theta, est_pars, wgtM, a, b, mu, sigma, eps, pow) {
    .Call('_sirt_sirt_rcpp_linking_haebara_fct_optim', PACKAGE='sirt', NI, NS, dist, aM, bM, theta, prob_theta, est_pars, wgtM, a, b, mu, sigma, eps, pow)
}

sirt_rcpp_linking_haebara_grad_optim_one_item <- function(theta, prob_theta, aM, bM, a, b, mu, sigma, ii, ss, dist, eps, NI, NS, index_a, index_b, index_mu, index_sigma, wgtM, grad0, pow) {
    .Call('_sirt_sirt_rcpp_linking_haebara_grad_optim_one_item', PACKAGE='sirt', theta, prob_theta, aM, bM, a, b, mu, sigma, ii, ss, dist, eps, NI, NS, index_a, index_b, index_mu, index_sigma, wgtM, grad0, pow)
}

sirt_rcpp_linking_haebara_grad_optim <- function(NI, NS, dist, aM, bM, theta, prob_theta, est_pars, wgtM, a, b, mu, sigma, eps, index_a, index_b, index_mu, index_sigma, parnames, NP, pow) {
    .Call('_sirt_sirt_rcpp_linking_haebara_grad_optim', PACKAGE='sirt', NI, NS, dist, aM, bM, theta, prob_theta, est_pars, wgtM, a, b, mu, sigma, eps, index_a, index_b, index_mu, index_sigma, parnames, NP, pow)
}

sirt_rcpp_lq_fit_analyze_matrix <- function(X) {
    .Call('_sirt_sirt_rcpp_lq_fit_analyze_matrix', PACKAGE='sirt', X)
}

sirt_rcpp_lq_fit_sparse_matrix_derivative <- function(Z, h1, px) {
    .Call('_sirt_sirt_rcpp_lq_fit_sparse_matrix_derivative', PACKAGE='sirt', Z, h1, px)
}

sirt_rcpp_lq_fit_matrix_mult <- function(Z, y, beta) {
    .Call('_sirt_sirt_rcpp_lq_fit_matrix_mult', PACKAGE='sirt', Z, y, beta)
}

sirt_rcpp_lq_fit_fct_optim <- function(Z, y, beta, pow, w, eps) {
    .Call('_sirt_sirt_rcpp_lq_fit_fct_optim', PACKAGE='sirt', Z, y, beta, pow, w, eps)
}

sirt_rcpp_mgsem_quadform <- function(y, A) {
    .Call('_sirt_sirt_rcpp_mgsem_quadform', PACKAGE='sirt', y, A)
}

sirt_rcpp_mgsem_quadform_logical <- function(y, A, A_logical) {
    .Call('_sirt_sirt_rcpp_mgsem_quadform_logical', PACKAGE='sirt', y, A, A_logical)
}

sirt_rcpp_mgsem_trace_product <- function(A, B) {
    .Call('_sirt_sirt_rcpp_mgsem_trace_product', PACKAGE='sirt', A, B)
}

sirt_rcpp_mgsem_trace_product_logical <- function(A, B, B_logical) {
    .Call('_sirt_sirt_rcpp_mgsem_trace_product_logical', PACKAGE='sirt', A, B, B_logical)
}

sirt_rcpp_mgsem_loglike_derivative_parameters <- function(S1, S3, y, Sigma_der, Sigma_der_logical) {
    .Call('_sirt_sirt_rcpp_mgsem_loglike_derivative_parameters', PACKAGE='sirt', S1, S3, y, Sigma_der, Sigma_der_logical)
}

sirt_rcpp_mgsem_compute_cov <- function(LAM, PHI, PSI) {
    .Call('_sirt_sirt_rcpp_mgsem_compute_cov', PACKAGE='sirt', LAM, PHI, PSI)
}

sirt_rcpp_mgsem_sumproduct_logical <- function(x, y, y_logical) {
    .Call('_sirt_sirt_rcpp_mgsem_sumproduct_logical', PACKAGE='sirt', x, y, y_logical)
}

sirt_rcpp_mgsem_vech_numeric <- function(A) {
    .Call('_sirt_sirt_rcpp_mgsem_vech_numeric', PACKAGE='sirt', A)
}

sirt_rcpp_mgsem_vech_logical <- function(A) {
    .Call('_sirt_sirt_rcpp_mgsem_vech_logical', PACKAGE='sirt', A)
}

sirt_rcpp_mgsem_eval_pen_lp_lasso <- function(y, eps_approx, p, regul_fac) {
    .Call('_sirt_sirt_rcpp_mgsem_eval_pen_lp_lasso', PACKAGE='sirt', y, eps_approx, p, regul_fac)
}

sirt_rcpp_mgsem_eval_pen_lp_scad <- function(y, eps_approx, p, regul_fac, a_scad) {
    .Call('_sirt_sirt_rcpp_mgsem_eval_pen_lp_scad', PACKAGE='sirt', y, eps_approx, p, regul_fac, a_scad)
}

sirt_rcpp_mgsem_eval_pen_lp_lasso_deriv <- function(y, eps_approx, p, regul_fac) {
    .Call('_sirt_sirt_rcpp_mgsem_eval_pen_lp_lasso_deriv', PACKAGE='sirt', y, eps_approx, p, regul_fac)
}

sirt_rcpp_mgsem_eval_lp_penalty <- function(x, fac, n, p, eps_approx, deriv, pen_type, a_scad, h) {
    .Call('_sirt_sirt_rcpp_mgsem_eval_lp_penalty', PACKAGE='sirt', x, fac, n, p, eps_approx, deriv, pen_type, a_scad, h)
}

sirt_rcpp_mgsem_eval_lpdiff_penalty <- function(x, fac, fac_logical, p, eps_approx, a_scad, pen_type, n) {
    .Call('_sirt_sirt_rcpp_mgsem_eval_lpdiff_penalty', PACKAGE='sirt', x, fac, fac_logical, p, eps_approx, a_scad, pen_type, n)
}

sirt_rcpp_mgsem_eval_lpdiff_penalty_deriv <- function(x, fac, n, fac_logical, p, eps_approx, h, a_scad, pen_type) {
    .Call('_sirt_sirt_rcpp_mgsem_eval_lpdiff_penalty_deriv', PACKAGE='sirt', x, fac, n, fac_logical, p, eps_approx, h, a_scad, pen_type)
}

sirt_rcpp_monoreg_rowwise <- function(YM, WM) {
    .Call('_sirt_sirt_rcpp_monoreg_rowwise', PACKAGE='sirt', YM, WM)
}

sirt_rcpp_noharm_sirt_optim_fn_rcpp <- function(gamma_val, delta, I, wgtm, pm, b0_jk, b1_jk, b2_jk, b3_jk) {
    .Call('_sirt_sirt_rcpp_noharm_sirt_optim_fn_rcpp', PACKAGE='sirt', gamma_val, delta, I, wgtm, pm, b0_jk, b1_jk, b2_jk, b3_jk)
}

sirt_rcpp_noharm_sirt_optim_gr_rcpp <- function(gamma_val, NH, I, wgtm, pm, b0_jk, b1_jk, b2_jk, b3_jk, npar, pt_matid, pt_index, pt_row, pt_col, FP, Fmat, Pmat, Psimat) {
    .Call('_sirt_sirt_rcpp_noharm_sirt_optim_gr_rcpp', PACKAGE='sirt', gamma_val, NH, I, wgtm, pm, b0_jk, b1_jk, b2_jk, b3_jk, npar, pt_matid, pt_index, pt_row, pt_col, FP, Fmat, Pmat, Psimat)
}

sirt_rcpp_pnorm0 <- function(z) {
    .Call('_sirt_sirt_rcpp_pnorm0', PACKAGE='sirt', z)
}

sirt_rcpp_pnorm <- function(x) {
    .Call('_sirt_sirt_rcpp_pnorm', PACKAGE='sirt', x)
}

sirt_rcpp_pbvnorm0 <- function(h1, hk, r) {
    .Call('_sirt_sirt_rcpp_pbvnorm0', PACKAGE='sirt', h1, hk, r)
}

sirt_rcpp_pbvnorm <- function(x, y, rho) {
    .Call('_sirt_sirt_rcpp_pbvnorm', PACKAGE='sirt', x, y, rho)
}

sirt_rcpp_dbvnorm0 <- function(x, y, rho, use_log) {
    .Call('_sirt_sirt_rcpp_dbvnorm0', PACKAGE='sirt', x, y, rho, use_log)
}

sirt_rcpp_dbvnorm <- function(x, y, rho, use_log) {
    .Call('_sirt_sirt_rcpp_dbvnorm', PACKAGE='sirt', x, y, rho, use_log)
}

sirt_rcpp_pbivnorm2 <- function(x, y, rho1) {
    .Call('_sirt_sirt_rcpp_pbivnorm2', PACKAGE='sirt', x, y, rho1)
}

sirt_rcpp_dmvnorm_2dim <- function(x, y, rho) {
    .Call('_sirt_sirt_rcpp_dmvnorm_2dim', PACKAGE='sirt', x, y, rho)
}

sirt_rcpp_polychoric2_pbivnorm <- function(x, y, rho, use_pbv) {
    .Call('_sirt_sirt_rcpp_polychoric2_pbivnorm', PACKAGE='sirt', x, y, rho, use_pbv)
}

sirt_rcpp_polychoric2_estimating_equation <- function(frtab, maxK, rho, thresh1n, thresh2n, maxK1, maxK2, use_pbv) {
    .Call('_sirt_sirt_rcpp_polychoric2_estimating_equation', PACKAGE='sirt', frtab, maxK, rho, thresh1n, thresh2n, maxK1, maxK2, use_pbv)
}

sirt_rcpp_polychoric2_est_itempair <- function(v1, v2, maxK_, maxiter, use_pbv, conv, rho_init, weights) {
    .Call('_sirt_sirt_rcpp_polychoric2_est_itempair', PACKAGE='sirt', v1, v2, maxK_, maxiter, use_pbv, conv, rho_init, weights)
}

sirt_rcpp_polychoric2 <- function(dat, maxK, maxiter, use_pbv, conv, rho_init, weights) {
    .Call('_sirt_sirt_rcpp_polychoric2', PACKAGE='sirt', dat, maxK, maxiter, use_pbv, conv, rho_init, weights)
}

sirt_rcpp_tetrachoric2_rcpp <- function(dfr, h, maxiter) {
    .Call('_sirt_sirt_rcpp_tetrachoric2_rcpp', PACKAGE='sirt', dfr, h, maxiter)
}

sirt_rcpp_rm_proc_datasets_na_indicators <- function(dat, K) {
    .Call('_sirt_sirt_rcpp_rm_proc_datasets_na_indicators', PACKAGE='sirt', dat, K)
}

sirt_rcpp_rm_proc_expand_dataset <- function(dat, rater0, pid0, N, R) {
    .Call('_sirt_sirt_rcpp_rm_proc_expand_dataset', PACKAGE='sirt', dat, rater0, pid0, N, R)
}

sirt_rcpp_rm_sdt_posterior <- function(prob_item, prob_rater, I0, K1, TP, item_index, dat2, dat2_resp, I, pi_k0) {
    .Call('_sirt_sirt_rcpp_rm_sdt_posterior', PACKAGE='sirt', prob_item, prob_rater, I0, K1, TP, item_index, dat2, dat2_resp, I, pi_k0)
}

sirt_rcpp_rm_sdt_calc_gradient_item_deriv_a <- function(prob0, prob_D1_dim, theta_k) {
    .Call('_sirt_sirt_rcpp_rm_sdt_calc_gradient_item_deriv_a', PACKAGE='sirt', prob0, prob_D1_dim, theta_k)
}

sirt_rcpp_rm_sdt_calc_gradient_item_deriv_tau <- function(prob0, prob_D1_dim, cat_pp) {
    .Call('_sirt_sirt_rcpp_rm_sdt_calc_gradient_item_deriv_tau', PACKAGE='sirt', prob0, prob_D1_dim, cat_pp)
}

sirt_rcpp_rm_sdt_calc_gradient_likelihood_item_llgrad <- function(logprob_D1, prob_D1_dim, nik_item) {
    .Call('_sirt_sirt_rcpp_rm_sdt_calc_gradient_likelihood_item_llgrad', PACKAGE='sirt', logprob_D1, prob_D1_dim, nik_item)
}

sirt_rcpp_squeeze_eps <- function(x, eps) {
    .Call('_sirt_sirt_rcpp_squeeze_eps', PACKAGE='sirt', x, eps)
}

sirt_rcpp_log <- function(x) {
    .Call('_sirt_sirt_rcpp_log', PACKAGE='sirt', x)
}

sirt_rcpp_plogis <- function(x) {
    .Call('_sirt_sirt_rcpp_plogis', PACKAGE='sirt', x)
}

sirt_rcpp_rm_sdt_calc_probs_gpcm <- function(a, tau, theta_k, VV, K1, TP, eps, use_log) {
    .Call('_sirt_sirt_rcpp_rm_sdt_calc_probs_gpcm', PACKAGE='sirt', a, tau, theta_k, VV, K1, TP, eps, use_log)
}

sirt_rcpp_rm_sdt_calc_probs_grm_rater <- function(c_rater, d_rater, I, K, eps, use_log) {
    .Call('_sirt_sirt_rcpp_rm_sdt_calc_probs_grm_rater', PACKAGE='sirt', c_rater, d_rater, I, K, eps, use_log)
}

sirt_rcpp_rm_sdt_calc_probs_grm_item <- function(tau_item, a_item, theta_k, VV, K, TP, eps, use_log) {
    .Call('_sirt_sirt_rcpp_rm_sdt_calc_probs_grm_item', PACKAGE='sirt', tau_item, a_item, theta_k, VV, K, TP, eps, use_log)
}

sirt_rcpp_xxirt_compute_posterior_expected_counts <- function(dat1_resp_gg, p_aj_xi_gg, weights_gg) {
    .Call('_sirt_sirt_rcpp_xxirt_compute_posterior_expected_counts', PACKAGE='sirt', dat1_resp_gg, p_aj_xi_gg, weights_gg)
}

sirt_rcpp_xxirt_compute_likelihood <- function(dat, dat_resp_bool, probs, TP, maxK) {
    .Call('_sirt_sirt_rcpp_xxirt_compute_likelihood', PACKAGE='sirt', dat, dat_resp_bool, probs, TP, maxK)
}

sirt_rcpp_xxirt_compute_likelihood_person_covariates <- function(dat, dat_resp_bool, probs, TP, maxK) {
    .Call('_sirt_sirt_rcpp_xxirt_compute_likelihood_person_covariates', PACKAGE='sirt', dat, dat_resp_bool, probs, TP, maxK)
}

sirt_rcpp_xxirt_compute_loglike_case_theta_covariates <- function(dat, dat_resp_bool, logprobs, TP, maxK) {
    .Call('_sirt_sirt_rcpp_xxirt_compute_loglike_case_theta_covariates', PACKAGE='sirt', dat, dat_resp_bool, logprobs, TP, maxK)
}

sirt_rcpp_xxirt_hessian_reduced_probs <- function(dat, dat_resp_bool, probs_ratio, TP, maxK, itemnr, itemnr2, use_itemnr2, p_xi_aj) {
    .Call('_sirt_sirt_rcpp_xxirt_hessian_reduced_probs', PACKAGE='sirt', dat, dat_resp_bool, probs_ratio, TP, maxK, itemnr, itemnr2, use_itemnr2, p_xi_aj)
}

sirt_rcpp_xxirt_newton_raphson_derivative_par <- function(dat, dat_resp_bool, ratio, p_xi_aj, item, prior_Theta, group0, weights, ll_case0, eps, person_covariates, person_covariates_items) {
    .Call('_sirt_sirt_rcpp_xxirt_newton_raphson_derivative_par', PACKAGE='sirt', dat, dat_resp_bool, ratio, p_xi_aj, item, prior_Theta, group0, weights, ll_case0, eps, person_covariates, person_covariates_items)
}

sirt_rcpp_xxirt_nr_pml_opt_fun <- function(prior_Theta, probs_items, freq1, freq2, W1, W2_long, G, K, I, TP, NI2, eps) {
    .Call('_sirt_sirt_rcpp_xxirt_nr_pml_opt_fun', PACKAGE='sirt', prior_Theta, probs_items, freq1, freq2, W1, W2_long, G, K, I, TP, NI2, eps)
}

sirt_rcpp_xxirt_nr_pml_grad_fun_eval <- function(prior_Theta, probs_items, freq1, freq2, W1, W2_long, G, K, I, TP, NI2, eps, NP, der_prior_Theta, val1, val2, pp_Theta, der_probs_items, index_freq1, index_freq2) {
    .Call('_sirt_sirt_rcpp_xxirt_nr_pml_grad_fun_eval', PACKAGE='sirt', prior_Theta, probs_items, freq1, freq2, W1, W2_long, G, K, I, TP, NI2, eps, NP, der_prior_Theta, val1, val2, pp_Theta, der_probs_items, index_freq1, index_freq2)
}

sirt_rcpp_xxirt_nr_pml_casewise_opt_fun <- function(prior_Theta, probs_items, W1, W2_long, G, K, I, TP, NI2, eps, group0, weights, dat1, dat_resp) {
    .Call('_sirt_sirt_rcpp_xxirt_nr_pml_casewise_opt_fun', PACKAGE='sirt', prior_Theta, probs_items, W1, W2_long, G, K, I, TP, NI2, eps, group0, weights, dat1, dat_resp)
}

## File Name: rasch_mml2_raschtype_mstep_parameter_group.R
## File Version: 0.160


#--- estimation of c parameter
rasch_mml2_raschtype_mstep_parameter_group <- function( theta.k, b, fixed.a,
            fixed.c, fixed.d, pjk, alpha1, alpha2, h, G, I, r.jk, n.jk, est_val,
            min_val, max_val, iter, old_increment, Qmatrix, parameter, prior=NULL)
{
    numdiff.parm <- h
    old_increment.c <- old_increment
    min.c <- min_val
    max.c <- max_val
    est.c <- est_val

    h1 <- h / 2
    h2 <- 1 + 2*h
    probs_args <- list( theta.k=theta.k, b=b, fixed.a=fixed.a, fixed.c=fixed.c,
                    fixed.d=fixed.d, alpha1=alpha1, alpha2=alpha2, Qmatrix=Qmatrix,
                    h=0, h1=h1, h2=h2, incr=parameter )
    res <- do.call(what=rasch_mml2_prob_genlogis_4pl_evaluate, args=probs_args )
    pjk.M <- res$pjk.M
    qjk.M <- res$qjk.M

    probs_args$h <- h
    res <- do.call(what=rasch_mml2_prob_genlogis_4pl_evaluate, args=probs_args )
    pjk1.M <- res$pjk.M
    qjk1.M <- res$qjk.M

    probs_args$h <- -h
    res <- do.call(what=rasch_mml2_prob_genlogis_4pl_evaluate, args=probs_args )
    pjk2.M <- res$pjk.M
    qjk2.M <- res$qjk.M

    # first order derivative
    # f(x+h) - f(x-h)=2* f'(x) * h
    ll0 <- ll1 <- ll2 <- matrix( 0, nrow=nrow(n.jk), ncol=G)
    for ( gg in 1:G){
        exp_r <- r.jk[,,gg]
        exp_n <- n.jk[,,gg]
        ll0[,gg] <- rasch_mml2_mstep_calc_loglike( exp_r=exp_r, prob1=pjk.M,
                                exp_n=exp_n, prob0=qjk.M)
        ll1[,gg] <- rasch_mml2_mstep_calc_loglike( exp_r=exp_r, prob1=pjk1.M,
                                exp_n=exp_n, prob0=qjk1.M)
        ll2[,gg] <- rasch_mml2_mstep_calc_loglike( exp_r=exp_r, prob1=pjk2.M,
                                exp_n=exp_n, prob0=qjk2.M)
    }
    ll0 <- rowSums(ll0)
    ll1 <- rowSums(ll1)
    ll2 <- rowSums(ll2)
    # aggregate with respect to estimation of a
    a1 <- stats::aggregate( cbind( ll0, ll1, ll2 ), list(est.c), sum, na.rm=TRUE)
    a1 <- a1[ a1[,1] > 0,]
    ll0 <- a1[,2]
    ll1 <- a1[,3]
    ll2 <- a1[,4]

    #** evaluate prior distribution
    prior_fct <- NULL
    parm <- NULL
    if (parameter=="a"){
        parm <- fixed.a
        prior_fct <- stats::dnorm
    }
    if (parameter=="c"){
        parm <- fixed.c
        parm[ parm < h] <- 2*h
        prior_fct <- stats::dbeta
    }
    if (parameter=="d"){
        parm <- fixed.d
        parm[ parm > 1-h] <- 1-2*h
        prior_fct <- stats::dbeta
    }
    res <- rasch_mml2_raschtype_mstep_parameter_group_evaluate_prior(parm=parm,
                h=h, prior=prior, ll0=ll0, ll1=ll1, ll2=ll2, prior_fct=prior_fct )
    ll0 <- res$ll0
    ll1 <- res$ll1
    ll2 <- res$ll2

    #- derivatives
    res <- rasch_mml2_difference_quotient(ll0=ll0, ll1=ll1, ll2=ll2, h=h)
    d1 <- res$d1
    d2 <- res$d2
    # change in item difficulty
    parm_change <- - d1 / d2
    parm_change <- sirt_trim_increment(increment=parm_change,
                            max_increment=max(abs(old_increment.c)))
    parm_change <- parm_change[ match( est.c, a1[,1] ) ]

    if ( any(est.c==0) ){
        parm_change[est.c==0] <- 0
    }
    if (parameter=="a"){ parm <- fixed.a }
    if (parameter=="b"){ parm <- b }
    if (parameter=="c"){ parm <- fixed.c }
    if (parameter=="d"){ parm <- fixed.d }

    parm <- parm + parm_change
    if (parameter=="c"){
        parm[ parm < 2*numdiff.parm ] <- 2*numdiff.parm
    }
    if (parameter=="d"){
        parm[ parm > 1 - 2 * numdiff.parm ] <- 1 - 2*numdiff.parm
    }
    parm[ parm > max.c ] <- max.c
    parm[ parm < min.c ] <- min.c
    #--- output
    res <- list("parm"=parm, "se"=sqrt( 1 /abs(d2) ) )
    return(res)
}

## File Name: sirt_attach_list_elements.R
## File Version: 0.06

sirt_attach_list_elements <- function(x, envir)
{
    vars <- names(x)
    for (vv in vars){
        assign( vv, x[[ vv ]], envir=envir )
    }
}

## File Name: IRT.anova.sirt.R
## File Version: 0.02



IRT.anova.sirt <- function (object, ...)
{
    cl <- match.call()
    cl2 <- paste(cl)[-1]
    if (length(list(object, ...)) !=2) {
        stop('anova method can only be applied for comparison of two models.\n')
    }
    objects <- list(object, ...)
    model1a <- objects[[1]]
    model2a <- objects[[2]]
    model1 <- IRT.IC(model1a)
    model2 <- IRT.IC(model2a)
    dfr1 <- data.frame(Model=cl2[1], loglike=model1['loglike'],
                Deviance=-2 * model1['loglike'])
    dfr1$Npars <- model1['Npars']
    dfr1$AIC <- model1['AIC']
    dfr1$BIC <- model1['BIC']
    dfr2 <- data.frame(Model=cl2[2], loglike=model2['loglike'],
                Deviance=-2 * model2['loglike'])
    dfr2$Npars <- model2['Npars']
    dfr2$AIC <- model2['AIC']
    dfr2$BIC <- model2['BIC']
    dfr <- rbind(dfr1, dfr2)
    dfr <- dfr[ order(dfr$Npars), ]
    dfr$Chisq <- NA
    dfr$df <- NA
    dfr$p <- NA
    dfr[1, 'Chisq'] <- dfr[1, 'Deviance'] - dfr[2, 'Deviance']
    dfr[1, 'df'] <- abs(dfr[1, 'Npars'] - dfr[2, 'Npars'])
    dfr[1, 'p'] <- round(1 - stats::pchisq(dfr[1, 'Chisq'], df=dfr[1,'df']), 5)
    for (vv in 2L:(ncol(dfr))){
        dfr[, vv] <- round(dfr[, vv], 5)
    }
    rownames(dfr) <- NULL
    invisible(dfr)
}

## File Name: mgsem_loglike_suffstat.R
## File Version: 0.136

mgsem_loglike_suffstat <- function(suffstat, Mu, Sigma, output_all=FALSE )
{
    N <- suffstat$N
    M <- suffstat$M
    S <- suffstat$S
    if (missing(Sigma)){
        res <- Mu
        Mu <- res$Mu
        Sigma <- res$Sigma
    }
    S1 <- mgsem_ginv(X=Sigma)
    p <- length(Mu)
    m1 <- M-Mu

    #*** mean structure
    # t1 <- ( t(M-Mu) %*% S1 %*% (M-Mu) )[1,1]
    # t1 <- ( crossprod(m1, S1) %*% m1 )[1,1]
    t1 <- sirt_rcpp_mgsem_quadform(y=M-Mu, A=S1)

    #*** covariance structure
    # t2b <- sum( diag(S %*% S1 ) )
    t2b <- sirt_rcpp_mgsem_trace_product(A=S, B=S1)
    # t2 <- log( det(Sigma) ) + t2b
    t2 <- sirt_logdet(x=Sigma) + t2b

    # normalization
    t3 <- p*log(2*pi)
    # total log likelihood
    res <- -N/2*(t1+t2+t3)

    #-- output_all
    if (output_all){
        res <- list(loglike=res, Sigma=Sigma, S1=S1, mean_residual=m1)
    }
    return(res)
}

## File Name: sirt_rename_list_names.R
## File Version: 0.03


sirt_rename_list_names <- function(x, old, new)
{
    ind1 <- which( names(x)==old )
    if (length(ind1)>0){
        names(x)[ind1] <- new
    }
    return(x)
}

## File Name: summary.invariance.alignment.R
## File Version: 0.282



summary.invariance.alignment <- function( object, digits=3, file=NULL, ...)
{
    # open sink for a file
    sirt_osink(file=file)

    display_string <- sirt_summary_print_display(symbol="-", len=65)
    cat(display_string)

    #- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object)

    #-- informations about optimization
    cat(display_string)
    invariance_alignment_summary_optimization(object=object, digits=digits)

    cat(display_string)
    cat("Number of items per group\n")
    print(object$numb_items)

    cat(display_string)
    cat("Effect Sizes of Approximate Invariance \n")
    obji <- object$es.invariance
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    cat(display_string)
    cat("Group Means and Standard Deviations ")
    cat(paste0("(center=", object$center,")\n"))
    obji <- object$pars
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    cat(display_string)
    cat("Summary Aligned Item Parameters Lambda \n")
    obji <- object$itempars.aligned
    sirt_summary_print_objects(obji=obji, digits=digits, from=1,
                    rownames_null=FALSE, grep_string="lambda")

    cat(display_string)
    cat("Summary Aligned Item Parameters Nu \n")
    obji <- object$itempars.aligned
    sirt_summary_print_objects(obji=obji, digits=digits, from=1,
                    rownames_null=FALSE, grep_string="nu")

    cat(display_string)
    cat("Aligned Lambda Parameters \n")
    obji <- object$lambda.aligned
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    cat(display_string)
    cat("Aligned Nu Parameters \n")
    obji <- object$nu.aligned
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    cat(display_string)
    cat("Summary Absolute Residuals Loadings Lambda \n")
    obji <- as.vector(abs(object$lambda.resid))
    sirt_summary_print_vector_summary(obji=obji, digits=digits)

    cat(display_string)
    cat("Summary Absolute Residuals Intercepts Nu \n")
    obji <- as.vector(abs(object$nu.resid))
    sirt_summary_print_vector_summary(obji=obji, digits=digits)

    sirt_csink(file=file)
}

## File Name: dmlavaan_est_model_bread_matrix_score_derivatives.R
## File Version: 0.115


dmlavaan_est_model_bread_matrix_score_derivatives <- function(fun, args,
        partable, scores, symmetric=TRUE, h=1e-4, remove_duplicated=TRUE,
        parnames=NULL)
{
    N <- nrow(scores)
    NPU <- max(partable$pnid)
    args$do.fit <- FALSE
    hess <- array(NA, dim=c(N, NPU, NPU) )
    partable1 <- partable
    partable1$start <- partable$est
    for (pp in 1L:NPU){
        partable1a <- partable1
        ind_pp <- which( partable$pnid==pp )
        val <- partable1$est[ind_pp[1]]
        h1 <- ifelse( abs(val) > 1, abs(val)*h, h )
        #* add increment
        partable1a$start[ ind_pp ] <- partable1$start[ ind_pp ] + h1
        partable1a$est <- partable1a$start
        args$model <- partable1a
        mod2 <- do.call(what=fun, args=args)
        scores2 <- lavaan::lavScores(object=mod2, remove.duplicated=remove_duplicated)
        #* substract increment
        if (symmetric){
            partable1a$start[ ind_pp ] <- partable1$start[ ind_pp ] - h1
            partable1a$est <- partable1a$start
            args$model <- partable1a
            mod2 <- do.call(what=fun, args=args)
            scores3 <- lavaan::lavScores(object=mod2, remove.duplicated=remove_duplicated)
            fac <- 2
        } else {
            scores3 <- scores
            fac <- 1
        }
        hess[,pp,] <- ( scores2-scores3 ) / (fac*h1)
    }
    B <- matrix(0, nrow=NPU, ncol=NPU)
    for (pp in 1L:NPU){
        for (hh in 1L:NPU){
            B[pp,hh] <- sum( hess[,pp,hh] )
        }
    }
    B <- -B
    # B <- ( B + t(B) ) / 2
    if (!is.null(parnames)){
        colnames(B) <- rownames(B) <- parnames
    }
    #--- output
    return(B)
}

## File Name: rm_sdt_create_partable_include_priors.R
## File Version: 0.05

rm_sdt_create_partable_include_priors <- function(partable, type, obj)
{
    vars <- c("prior_M", "prior_SD")
    for (vv in 1:2){
        partable[ ( partable$type==type) & ( partable$est ), vars[vv] ] <- obj[vv]
    }
    return(partable)
}

## File Name: sirt_var.R
## File Version: 0.01

sirt_var <- function(x, method="ML", na.rm=TRUE)
{
    v1 <- stats::var(x, na.rm=na.rm)
    N <- sum(1-is.na(x))
    v1 <- v1 * (N-1) / N
    return(v1)

}

## File Name: linking_2groups_numerical_gradient.R
## File Version: 0.03


linking_2groups_numerical_gradient <- function(fun, args, h=1e-4)
{
    NP <- length(args$x)
    grad <- rep(0, NP)
    args2 <- args
    for (pp in 1L:NP){
        args2$x <- mgsem_add_increment(x=args$x, h=h, i1=pp)
        f1 <- do.call( what=fun, args=args2)
        args2$x <- mgsem_add_increment(x=args$x, h=-h, i1=pp)
        f2 <- do.call( what=fun, args=args2)
        grad[pp] <- (f1-f2)/(2*h)
    }
    return(grad)
}

## File Name: linking_2groups_haebara_fun.R
## File Version: 0.167


#- Haebara optimization function
linking_2groups_haebara_fun <- function(x, pars, Theta, wgt, type="asymm",
        pow=2, eps=0.001, simultan=FALSE, deriv=FALSE )
{
    TP <- length(Theta)
    I <- nrow(pars)
    mat <- matrix(NA, nrow=TP, ncol=I)
    mu <- x[1]
    sigma <- x[2]
    val <- 0
    if (deriv){
        NP <- 2
        val <- rep(0,2)
    }
    wgtM <- sirt_matrix2(wgt, nrow=I)

    a1M <- pars[,1]
    b1M <- pars[,2]
    a2M <- pars[,3]
    b2M <- pars[,4]
    if (simultan){
        a <- x[ 2 + 1L:I ]
        b <- x[ 2 + I + 1L:I ]
        ind_a <- 2+1L:I
        ind_b <- 2+I+1L:I
        if (deriv){
            val <- rep(0, 2+2*I)
        }
    }

    if (type=='asymm' | type=='symm'){
        Theta1 <- sirt_matrix2(sigma*Theta+mu, nrow=I)
        Theta2 <- sirt_matrix2(Theta, nrow=I)
        P1 <- stats::plogis( a1M * (Theta1-b1M) )
        P2 <- stats::plogis( a2M * (Theta2-b2M) )
        if (!simultan){ # simultan=FALSE
            z <- P1-P2
            d <- linking_2groups_power_loss(x=z, pow=pow, eps=eps, deriv=deriv)
            if (!deriv){  # no derivative
                val <- val + sum( wgtM*d )
            } else {  # derivative
                fac0 <- P1*(1-P1)*a1M
                val[1] <- val[1] + sum( wgtM*d*fac0)
                val[2] <- val[2] + sum( wgtM*d*fac0*Theta2)
            }
        } else {  # simultan=TRUE
            P0 <- stats::plogis( a * (Theta2-b) )
            z1 <- P1-P0
            z2 <- P2-P0
            d1 <- linking_2groups_power_loss(x=z1, pow=pow, eps=eps, deriv=deriv)
            d2 <- linking_2groups_power_loss(x=z2, pow=pow, eps=eps, deriv=deriv)
            if (!deriv){ # deriv=FALSE
                val <- val + sum( wgtM*(d1+d2) )
            } else {  # deriv=TRUE
                fac0 <- P1*(1-P1)*a1M
                val[1] <- val[1] + sum(wgtM*d1*fac0)
                val[2] <- val[2] + sum(wgtM*d1*fac0*Theta2)
                # derivatives item discriminations
                facP0 <- P0*(1-P0)
                fac <- facP0*(Theta2-b)
                wgtM_d <- wgtM*(d1+d2)
                val[ind_a] <- val[ind_a] - rowSums( wgtM_d*fac )
                # derivatives item difficulties
                fac <- facP0*a
                val[ind_b] <- val[ind_b] + rowSums( wgtM_d*fac )

            }
        }  # end simultan
    }

    if (type=='symm'){
        Theta1 <- sirt_matrix2(Theta, nrow=I)
        Theta2 <- sirt_matrix2(1/sigma*Theta-mu/sigma, nrow=I)
        P1 <- stats::plogis( a1M * (Theta1-b1M) )
        P2 <- stats::plogis( a2M * (Theta2-b2M) )
        if (!simultan){  # simultan=FALSE
            z <- P1-P2
            d <- linking_2groups_power_loss(x=z, pow=pow, eps=eps, deriv=deriv)
            if (!deriv){ # deriv=FALSE
                val <- val + sum( wgtM*d )
            } else {  # deriv=TRUE
                fac0 <- P2*(1-P2)*a2M
                val[1] <- val[1] + sum( wgtM*d*fac0/sigma ) # mu
                val[2] <- val[2] + sum( wgtM*d*fac0*Theta2/sigma ) # sigma
            }  # end deriv
        } else {      # simultan=TRUE
            P0 <- stats::plogis( a * (Theta2-b) )
            z1 <- P1-P0
            d1 <- linking_2groups_power_loss(x=z1, pow=pow, eps=eps, deriv=deriv)
            z2 <- P2-P0
            d2 <- linking_2groups_power_loss(x=z2, pow=pow, eps=eps, deriv=deriv)
            if (!deriv){ # deriv=FALSE
                val <- val + sum( wgtM*(d1+d2) )
            } else {  # deriv=TRUE
                fac1 <- - P0*(1-P0)*a
                fac2 <- P2*(1-P2)*a2M - P0*(1-P0)*a
                val[1] <- val[1] + sum(-wgtM*(d1*fac1+d2*fac2)/sigma)  # mu
                val[2] <- val[2] + sum(-wgtM*(d1*fac1+d2*fac2)*Theta2/sigma)  # sigma
                #- derivatives item discriminations
                facP0 <- P0*(1-P0)
                fac <- facP0*(Theta2-b)
                wgtM_d <- wgtM*(d1+d2)
                val[ind_a] <- val[ind_a] - rowSums( wgtM_d*fac )
                #- derivatives item difficulties
                fac <- facP0*a
                val[ind_b] <- val[ind_b] + rowSums( wgtM_d*fac )

            }
        } # end simultan
    }
    return(val)
}

## File Name: plausible.values.raschtype.R
## File Version: 2.27

#''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
#''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
# Routine for plausible value imputation
plausible.value.imputation.raschtype <- function( data=NULL,
        f.yi.qk=NULL, X,
        Z=NULL, beta0=rep(0,ncol(X)), sig0=1,
        b=rep(1,ncol(X)),  a=rep(1, length(b) ), c=rep(0, length(b) ),
        d=1+0*b, alpha1=0, alpha2=0,
        theta.list=seq(-5,5,len=50), cluster=NULL,
        iter, burnin, nplausible=1, printprogress=TRUE )
{
    #........................................................................
    # INPUT:
    # data  ... matrix of dichotomous item responses
    # X     ... matrix of covariates (background variables)
    # Z     ... matrix of covariates allowing for heteroscedasticity
    # beta0 ... initial beta regression coefficients
    # sig0  ... initial residual standard deviation
    # b ... item difficulties
    # a ... item discrimination
    # c ... guessing parameter
    # theta.list ... grid of theta values for posterior density evaluation
    # cluster ... indicates cluster
    # nplausible    ...  number of plausible values
    # iter  ... number of iterations
    # burnin ... number of burn-in iterations
    #...........................................................................
    # indexes for PV estimation
    if ( ! is.null(data) ){
        data <- as.matrix(data)
        if (is.null(colnames(data)) ){ colnames(data) <- paste( "item", seq(1, ncol(data)), sep="") }
        dfrout <- data.frame( "item"=colnames(data), "b"=b, "a"=a, "c"=c, "d"=d )
        cat("\nIRT Plausible Value Imputation - Rasch type model\n")
        nd <- nrow(data)
                        } else {
            nd <- nrow(f.yi.qk)
                    }
    pv.indexes <- sort( unique( round( sort( seq( iter, burnin +1,
                    len=nplausible ) ) )))
    coefs <- matrix( 0, nrow=iter, ncol=ncol(X) + ( ! is.null(cluster) ) )
    pvdraws <- matrix( 0, nrow=length(pv.indexes), ncol=nd )
    # matrices
    X <- as.matrix(X)
    if ( is.null(Z) ){ Z <- matrix( 1, nrow=nrow(X), ncol=1) }
    coefsZ <- matrix( 0, nrow=iter, ncol=ncol(Z))

    #***************
    # Data preparation
    #***
    # data preparation
    if ( ! is.null(data)){
        y <- data
        y[ is.na(y)] <- 0
        dat2.resp <- 1-is.na(data)
        I <- ncol(y)    # number of items
        dat1 <- data.frame( 1+0*y[,1], 1 )
                    }
    LT <- length( theta.list )
    pi.k <- sirt_dnorm( theta.list )
    theta.kM <- matrix( theta.list, nrow=nrow(X),
                    ncol=length(theta.list), byrow=TRUE )
    theta.kM2 <- theta.kM^2
    n <- nrow(X)
    # calculate likelihood if not provided
    if ( is.null( f.yi.qk) ){
        f.yi.qk <- .e.step.raschtype( dat1=dat1, dat2=y, dat2.resp=dat2.resp,
            theta.k=theta.list, pi.k=pi.k, I=I,
            n=nrow(y), b=b, fixed.a=a, fixed.c=c,  fixed.d=d,
            alpha1=alpha1, alpha2=alpha2, group=NULL, pseudoll=0, f.qk.yi=NULL )$f.yi.qk
                    }
    post <- f.yi.qk / rowSums( f.yi.qk )

    ################################
    # begin iterations
    for (ii in 1:iter){
    #   ii <- 1
        # draw plausible values
        if ( ii==1 ){ X1 <- X }
        ########################
        # draw plausible values
        pv1 <- .pv.draw( f.yi.qk, X, beta0, sig0,
                    theta.list, pvdraw=1 )
        # calculate adjusted group mean in case of clustering
        if ( !  is.null( cluster ) ){
          X1 <- cbind( X, .adj.groupmean( variable=pv1$plausible.value[,1],
                                cluster ) )
                                } else { X1 <- X }
        # sample latent regression model / draw regression parameters
        s2 <- .sampling.latent.regression.raschtype( pv=pv1$plausible.value[,1],
                    X=X1, Z=as.matrix(Z) )
        beta0 <- s2$samp.beta   # sampling of regression coefficients
        sig0 <- s2$fitted.sigma
        coefs[ii,] <- beta0
        coefsZ[ii,] <- s2$samp.gamma

        if( ii %in% pv.indexes  ){
            pvdraws[ which( pv.indexes==ii), ] <- pv1$plausible.value[,1]
                                }
        if ( printprogress ){
                if (ii==1 ){ cat("\n Iteration ") }
                cat(paste(ii, ".",sep=""));
                utils::flush.console()
                if ( ( ii %% 10==0 ) | ( ii==iter  ) ){ cat("\n     ") }
                        }
                }
    cat("\n")
    # results pv draw object
    pv1[[  "plausible.value" ]]  <- NULL
    coefs <- coefs[ seq( burnin +1, iter ), ]
    res <- list( "coefsX"=coefs, "coefsZ"=coefsZ[seq( burnin +1, iter ),,drop=FALSE],
            "pvdraws"=t(pvdraws),
            "posterior"=pv1$posterior.density    ,
            "EAP"=pv1$EAP, "SE.EAP"=pv1$SE.EAP     , "pv.indexes"=pv.indexes )
    }
#*******************************************************************************************************
# data=data ; X=X ; weights=rep(1,nrow(data)) ;
#                                beta.init=rep(0,ncol(X)) ; sigma.init=1 ; max.parchange=.0001 ;
#                                theta.list=seq(-5,5,len=50) ; maxiter=300


#***********************************************************************************
# function for drawing plausible values (Raschtype model)
##NS export(plausible.value.draw.raschtype)
.pv.draw <- function( f.yi.qk, X, beta0, sig0,
                    theta.list, pvdraw=1 ){
    #..................................
    # recode missings
#    y <- data
#    y[ is.na(data) ] <- 1
#    respind <- 1 - is.na(data)
    sig0[ sig0 < 0] <- 0
 #   n <- nrow(y)
    # predicted values from latent regression
    M.Regr <- ( X %*% beta0 )[,1]
    if (length(sig0) > 1){ SD.Regr <- sig0 } else { SD.Regr <- rep( sig0, n ) }
    # matrix of theta values
    n <- nrow(X)
    l1 <- rep(1,n)
    theta.listM <- thetaM <- matrix( theta.list, nrow=n, ncol=length(theta.list),
        byrow=TRUE)
    # compute density resulting from regression
    dens.Regr <- sirt_dnorm( thetaM, mean=M.Regr, sd=SD.Regr )
    dens.total <- f.yi.qk * dens.Regr
    dens.total <- dens.total / rowSums( dens.total)
#   theta.listM <- outer( l1, theta.list )
    # mean of individual posterior distribution
    EAP <- rowSums( theta.listM * dens.total )
    # SD of posterior distribution
    SD.Post <- sqrt( rowSums( theta.listM^2 * dens.total ) -  EAP^2 )
    # one draw of plausible values
    if (  ! pvdraw ){ pvdraw <- NULL } else {
            pvdraw <- matrix( stats::rnorm( n*pvdraw,
                mean=rep(EAP,each=pvdraw), sd=rep(SD.Post,each=pvdraw) ), ncol=pvdraw, byrow=T )
                         }
    # results
    res <- list( "theta.grid"=theta.list, "posterior.density"=dens.total,
        "EAP"=EAP, "SE.EAP"=SD.Post,
        "plausible.value"=pvdraw, "M.Regr"=M.Regr, "SD.Regr"=SD.Regr )
    return(res)
    }
#***********************************************************************************

#.........................................................................
# sample parameters for latent regression model
.sampling.latent.regression.raschtype <- function( pv, X,
            Z=rep(1,length(pv)) ){
        # INPUT:
        # pv        ... draw of plausible values
        # X         ... matrix of covariates for latent regression model
        #                 intercept is not automatically included=> create vector of ones!!
        # Z         ... matrix of covariates for explaining residual variance
        #.............................................................
        # latent regression model
        mod <- stats::lm( pv ~ 0 + X )
        res <- list( "est.beta"=stats::coef(mod), "vcov.beta"=stats::vcov(mod) )
        # sample beta parameter
        res$samp.beta <- sirt_rmvnorm( 1, mean=res$est.beta, sigma=res$vcov.beta )
        # residual standard deviation
        n <- nrow(X)
        p <- ncol(X)
        res$est.sigma <- summary(mod)$sigma
        residuals.mod <- ( stats::resid(mod) )^2   * (n-1) / ( n - p - 1)
        mod1 <- stats::lm( residuals.mod ~ 0 + Z )
#        summary(mod1)
        # sample gamma coefficients for heteroscedasticity
        res$samp.gamma <- sirt_rmvnorm( 1, mean=stats::coef(mod1), sigma=stats::vcov(mod1) )
        res$fitted.sigma <- sqrt( stats::fitted(mod1) )
        res$lm.latent.regression <- mod
        res$lm.residualsd <- mod1
        return(res)
        }
#..............................................................................


        #*************************************************************
        # function to calculate adjusted mean: eliminate score on the individual
        # variable <- pv1$plausible.value[,1]
        .adj.groupmean <- function( variable, cluster ){
            a1 <- stats::aggregate( variable,    list( cluster ), mean  )
            a2 <- stats::aggregate( 1+0*variable,    list( cluster ), sum  )
            ind <- match( cluster, a1[,1] )
            ( a2[ind,2] * a1[ ind, 2] - variable ) / a2[ ind, 2]
                    }
    #*****************************************

## File Name: monoreg.colwise.R
## File Version: 0.13


# monotone regression for all columns in a matrix
monoreg.colwise <- function(yM, wM)
{
    yM <- as.matrix(t(yM))
    wM <- as.matrix(t(wM))
    res <- sirt_rcpp_monoreg_rowwise( YM=yM, WM=wM )
    return(t(res))
}

## File Name: xxirt_nr_pml_preproc_data.R
## File Version: 0.11


xxirt_nr_pml_preproc_data <- function(em_args, pml_args)
{

    I <- em_args$I
    ncat <- em_args$ncat
    items <- em_args$items
    G <- em_args$G
    group <- em_args$group
    Theta <- em_args$Theta
    TP <- nrow(Theta)
    K <- max(ncat)
    if (is.null(pml_args$W1)){
        pml_args$W1 <- rep(1/I, I)
    }
    if (is.null(pml_args$W2)){
        W2 <- matrix( 2/I/(I-1), nrow=I, ncol=I)
        rownames(W2) <- colnames(W2) <- items
        W2[ upper.tri(W2)] <- 0
        diag(W2) <- 0
        pml_args$W2 <- W2
    }
    W2 <- pml_args$W2
    # convert W2 into a long format
    W2_long <- data.frame( item1=rep(1L:I,each=I), item2=rep(1L:I, I),
                                    w2=as.numeric(W2) )
    W2_long <- W2_long[ W2_long$w2 > 0, ]
    NI2 <- nrow(W2_long)
    dat1_resp <- em_args$dat1_resp
    weights <- em_args$weights

    freq1 <- array(0, dim=c(I,K,G))
    for (gg in 1L:G){
        for (ii in 1L:I){
            for (hh in 1L:K){
                freq1[ii,hh,gg] <- sum( dat1_resp[,ii,hh] * weights * (group==gg) )
            }
        }
    }
    freq2 <- array(0, dim=c(NI2, K, K,G) )
    nn <- 1
    for (gg in 1L:G){
        for (nn in 1L:NI2){
            for (hh in 1L:K){
                for (jj in 1L:K){
                    freq2[nn,hh,jj,gg] <- sum( dat1_resp[, W2_long[nn,1], hh ] *
                                    dat1_resp[, W2_long[nn,2], jj ] * weights *
                                            ( group==gg) )
                }
            }
        }
    }
    pml_args$W2_long <- W2_long
    pml_args$I <- I
    pml_args$G <- G
    pml_args$K <- K
    pml_args$TP <- TP
    pml_args$NI2 <- NI2
    pml_args$freq1 <- freq1
    pml_args$freq2 <- freq2

    #-- output
    return(pml_args)
}

## File Name: confint.xxirt.R
## File Version: 0.13


#--- confidence interval for xxirt object
confint.xxirt <- function( object, parm, level=.95,  ... )
{
    c1 <- coef.xxirt(object)
    v1 <- vcov.xxirt(object)
    if ( ! missing(parm) ){
        c1 <- c1[parm]
        v1 <- v1[ parm, parm]
    }

    q1 <- ( 1 - level ) / 2
    q2 <- 1 - ( 1 - level ) / 2
    quant <- stats::qnorm(q2)
    se <- sqrt( diag(v1) )
    res <- data.frame( a=c1-quant*se, b=c1+quant*se )
    colnames(res)[1] <- paste0( round( 100*q1,1 ), ' %')
    colnames(res)[2] <- paste0( round( 100*q2,1 ), ' %')
    rownames(res) <- names(c1)
    return(res)
}

## File Name: sirt_permutations.R
## File Version: 0.02


sirt_permutations <- function(r,v)
{
    NL <- length(v)
    NC <- NL^r
    mat <- matrix(0, nrow=NC, ncol=r)
    hh <- 1
    for (dd in seq(r,1,by=-1)){
        m1 <- rep(v, each=NL^(hh-1) )
        m1 <- rep(m1, NC/length(m1) )
        mat[,dd] <- m1
        hh <- hh + 1
    }
    return(mat)
}

## File Name: linking_haebara_summary_optimization.R
## File Version: 0.091

linking_haebara_summary_optimization <- function(object, digits)
{
    cat('Distance function type', '=', object$dist, '\n')
    if (object$dist=='L1'){
        cat('Epsilon Value', '=', object$eps, '\n')
    }
    cat('Optimization Function Value', '=', round(object$res_optim$value, digits), '\n')
    cat('Optimizer', '=', object$res_optim$optimizer, '\n')
    cat('use_rcpp', '=', object$use_rcpp, '\n')
    cat('Number of iterations', '=', object$res_optim$iter, '\n')
    cat('Converged', '=', object$res_optim$converged, '\n')
}

## File Name: rm_sdt_evaluate_prior_derivative.R
## File Version: 0.08


rm_sdt_evaluate_prior_derivative <- function(partable, h)
{
    partable <- partable[ partable$est, ]
    m1 <- partable$prior_M
    sd1 <- partable$prior_SD
    y1 <- sirt_dnorm(partable$value+h, mean=m1, sd=sd1, log=TRUE)
    y2 <- sirt_dnorm(partable$value-h, mean=m1, sd=sd1, log=TRUE)
    res <- - ( y1 - y2 ) / (2*h)
    res <- ifelse(is.na(res),0,res)
    return(res)
}

## File Name: tam2mirt_freed.R
## File Version: 0.156

#*** return lavaan syntax with freed parameters
tam2mirt_freed <- function( D, factors, B, dat, AXsi,
        mean.trait, cov.trait, tamobj )
{
    # create lavaan syntax with constraints
    lavsyn <- NULL
    if ( tamobj$irtmodel=='2PL' ){
        class(tamobj) <- 'tam.mml.2pl' }
    for (dd in 1L:D){
        fac.dd <- factors[dd]
        # create terms for loadings
        B2.dd <- round( B[,2,dd], 4)
        if (inherits(tamobj,'tam.mml')){
            syn0 <- paste0( paste0( B2.dd[ B2.dd!=0], '*',
                            colnames(dat)[ B2.dd!=0] ), collapse='+' )
            syn0 <- paste0( fac.dd, '=~ ', syn0, '\n')
        }
        if (inherits(tamobj,'tam.mml.2pl')){
            d4 <- paste0( B2.dd[ B2.dd!=0] )
            d4 <- paste0( 'a', dd, '_', seq(1,length(d4) ) )
            syn0 <- paste0( paste0( d4, '*', colnames(dat)[ B2.dd!=0] ),
                            collapse='+' )
            syn0 <- paste0( fac.dd, '=~ ', syn0, '\n')
        }
        lavsyn <- paste0( lavsyn, syn0 )
    }
    # create syntax for intercepts
    maxK <- ncol(AXsi) - 1
    for (kk in 1L:maxK){
        t1 <- round( AXsi[,kk+1], 4 )
        string1 <- paste0('t', kk )
        t1 <- paste0(string1, '_', seq(1,length(t1) ) )
        syn0 <- paste0( colnames(dat), ' | ', t1, '*', string1)
        syn0 <- paste0( syn0, collapse='\n')
        lavsyn <- paste0( lavsyn, syn0, '\n')
    }

    # syntax for means
    syn0 <- paste0( factors, ' ~ ', round(as.vector(mean.trait),4), '*1'  )
    syn0 <- paste0( syn0, collapse='\n')
    lavsyn <- paste0( lavsyn, '\n', syn0 )
    # syntax for variances
    if (inherits(tamobj,'tam.mml')){
        g1 <- paste0( 'Cov_', 1L:D, 1L:D )
        syn0 <- paste0( factors, ' ~~ ', g1, '*',factors  )
        syn0 <- paste0( syn0, collapse='\n')
        lavsyn <- paste0( lavsyn, '\n', syn0 )
    }
    if (inherits(tamobj,'tam.mml.2pl')){
        syn0 <- paste0( factors, ' ~~ ',
                        round( as.vector(diag(cov.trait)),4), '*',factors  )
        syn0 <- paste0( syn0, collapse='\n')
        lavsyn <- paste0( lavsyn, '\n', syn0 )
    }
    # syntax for covariances
    if (D>1){
        for (dd in 1L:(D-1)){
            for (ee in (dd+1):D){
                syn0 <- paste0( factors[dd], ' ~~ ',
                            paste0('Cov_',dd,ee), '*',factors[ee]  )
                syn0 <- paste0( syn0, collapse='\n')
                lavsyn <- paste0( lavsyn, '\n', syn0 )
            }
        }
    }
    # finalize lavaan syntax
    lavsyn <- paste0( lavsyn, ' \n')
    return(lavsyn)
}

## File Name: gom_em_loglike_grad.R
## File Version: 0.278



gom_em_loglike_grad <- function(x, ind_lambda, ind_pi, I, K, theta.k, theta0.k,
        dat2, dat2.resp, TP, model="GOM", h=1e-4, ind_mu, ind_sigma, theta_grid,
        weights, lambda_partable, lambda_index_blocks)
{
    ncat <- 2
    #-- function value
    res <- gom_em_loglike_parameter_conversion( x=x, ind_lambda=ind_lambda,
                ind_pi=ind_pi, I=I, K=K, ind_mu=ind_mu, ind_sigma=ind_sigma, model=model,
                theta_grid=theta_grid, lambda_partable=lambda_partable )
    lambda <- res$lambda
    pi.k <- res$pi.k
    res <- gom_em_calc_probs( lambda=lambda, theta.k=theta.k, b=NULL, theta0.k=theta0.k )
    probs00 <- probs <- res$probs
    probs0 <- sirt_probs_dichotomous_to_array(probs=res$probs)
    fyiqk <- sirt_rcpp_gom_em_likelihood( probs=probs0, ncat=ncat, TP=TP,
                        dat2=dat2, dat2resp=dat2.resp)
    ll <- sirt_rcpp_gom_em_log_likelihood( fyiqk=fyiqk, pik=pi.k, weights=weights)

    #-- gradient with respect to item parameters
    NP <- length(x)
    grad1 <- rep(NA, NP)
    item_vec <- 1:I
    B <- length(lambda_index_blocks)
    for (jj in 1:B){
        ind <- lambda_index_blocks[[jj]]
        lbb <- lambda_partable[ lambda_partable$block==jj,, drop=FALSE]
        items <- lbb[ lbb$free==1, 'item'] - 1
        x1 <- sirt_add_increment(x=x, pos=ind, value=h)
        lambda <- gom_em_loglike_parameter_conversion( x=x1, ind_lambda=ind_lambda,
                        ind_pi=ind_pi, I=I, K=K, ind_mu=ind_mu, ind_sigma=ind_sigma,
                        model=model, theta_grid=theta_grid,
                        lambda_partable=lambda_partable )$lambda
        probs <- gom_em_calc_probs( lambda=lambda, theta.k=theta.k, b=NULL,
                        theta0.k=theta0.k )$probs
        probs_h <- sirt_probs_dichotomous_to_array(probs=probs)
        # update log-likelihood in gradient
        ll2 <- sirt_rcpp_gom_em_loglike_gradient( probs=probs0, probs_h=probs_h,
                    ncat=ncat, TP=TP, dat2=dat2, dat2resp=dat2.resp, pik=pi.k,
                    items=items, fyiqk=fyiqk, weights=weights)
        grad1[ind] <- (ll2 - ll) / h
    }

    #-- gradient with respect to trait distribution parameters
    ind_pi1 <- ind_pi
    if (model=='GOMnormal'){
        ind_pi1 <- c(ind_mu, ind_sigma)
    }
    for (pp in ind_pi1){
        x1 <- sirt_add_increment(x=x, pos=pp, value=h)
        res <- gom_em_loglike_parameter_conversion( x=x1, ind_lambda=ind_lambda,
                    ind_pi=ind_pi, I=I, K=K, ind_mu=ind_mu, ind_sigma=ind_sigma,
                    model=model, theta_grid=theta_grid, lambda_partable=lambda_partable)
        pi.k <- res$pi.k
        ll2 <- sirt_rcpp_gom_em_log_likelihood( fyiqk=fyiqk, pik=pi.k, weights=weights)
        grad1[pp] <- (ll2 - ll) / h
    }
    return(grad1)
}

## File Name: gom_em_est_b.R
## File Version: 0.071



#--- estimation of b parameters in Rasch GOM model
gom_em_est_b <- function( lambda, I, K, n.ik, b, theta0.k, numdiff.parm=.001,
        max.increment, theta.k, msteps, mstepconv, eps=.001, progress=progress )
{
    h <- numdiff.parm
    diffindex <- 1L:I
    if (progress){
        cat('  M steps b parameter |')
    }
    an.ik <- aperm( n.ik, c(2,3,1) )
    it <- 0
    conv1 <- 1000
    while( ( it < msteps ) & ( conv1 > mstepconv ) ){
        b0 <- b
        pjk <- gom_em_calc_probs( lambda=lambda, theta.k=theta.k, b=b,
                                    theta0.k=theta0.k )$probsL
        pjk1 <- gom_em_calc_probs( lambda=lambda, theta.k=theta.k, b=b+h,
                                    theta0.k=theta0.k )$probsL
        pjk2 <- gom_em_calc_probs( lambda=lambda, theta.k=theta.k, b=b-h,
                                    theta0.k=theta0.k)$probsL
        # numerical differentiation
        res <- gom_em_numdiff_index( pjk, pjk1, pjk2, an.ik, diffindex,
                        max.increment=max.increment, numdiff.parm )
        b <- b + res$increment
        #b <- b - mean(b)
        conv1 <- max( abs( b - b0 ) )
        it <- it+1
        if (progress){
            cat('-')
        }
    }
    if (progress){
        cat(' ', it, 'Step(s) \n')
    }
    res <- list(b=b, se.b=sqrt(-1/res$d2), ll=sum(res$ll0) )
    return(res)
}

## File Name: plot.lsem.permutationTest.R
## File Version: 0.21

#############################################################
# plot p values permutation test for LSEM
plot.lsem.permutationTest <- function( x, type="global",
        stattype="SD", parindex=NULL,
        sig_add=TRUE, sig_level=.05, sig_pch=17,
        nonsig_pch=2, sig_cex=1,
        sig_lab="p value", stat_lab="Test statistic",
        moderator_lab=NULL, digits=3, title=NULL, parlabels=NULL,
        ask=TRUE, ... )
{

    if ( is.null(parindex) ){
        NP <- max( x$parameters$parindex )
        parindex <- 1:NP
    }

    #######################################
    # global test statistic
    if ( type=="global"){

        teststat <- x$teststat[ parindex,, drop=FALSE]
        test_p <- teststat[, paste0(stattype,"_p") ]
        NP <- nrow(teststat)
        labs <- paste(teststat$par)
        if ( ! is.null(parlabels) ){
            labs <- parlabels
        }
        if (sig_add){
            nc <- nchar(labs)
            NC <- max(nc)
            labs <- paste0( labs, " (p=", formatC( test_p, digits=digits, format="f"), ")")
        }

        main <- title
        if ( is.null(title)){
            main <- paste0( "p values ", stattype)
        }

        xlab1 <- sig_lab
        h1 <- seq(NP, 1 )
        labs <- labs[ h1  ]
        test_p <- test_p[ h1 ]
        graphics::dotchart( test_p, labels=labs, xlim=c(0,1), pch=sig_pch,
                        xlab=xlab1, main=main, lwd=1.2)
        graphics::abline(v=sig_level, col=2, lty=4, lwd=2)
    }
    #**********************************************************
    if (type=="pointwise"){

        ppt <- x$parameters_pointwise_test
        for (pp in parindex ){
            graphics::par(mfrow=c(2,1))
            ind.pp <- which( parindex==pp)
            x.pp <- ppt[ ppt$parindex==pp, ]
            ylab1 <- sig_lab
            if ( is.null(moderator_lab) ){
                moderator_lab <- x$moderator
            }
            if ( is.null(parlabels)){
                t1 <- paste( x.pp$par[1] )
            } else {
                t1 <- parlabels[ind.pp]
            }

            graphics::plot( x.pp$moderator, x.pp$est, xlab=moderator_lab,
                    ylab=stat_lab, main=t1, type="o", pch=16)
            graphics::abline( h=0, lty=3, col="gray")

            # p value
            pch1 <- ifelse( x.pp$p < sig_level, sig_pch, nonsig_pch )
            graphics::plot( x.pp$moderator, x.pp$p, xlab=moderator_lab,
                    ylab=ylab1, main=t1, type="o", pch=pch1,
                    ylim=c(0,1), cex=sig_cex)

            # lines( spline( modgrid[,1], y=x.pp$p, n=100 )  )
            graphics::abline(h=sig_level, col=2, lty=4, lwd=2)
            graphics::par(mfrow=c(1,1))
            graphics::par(ask=ask)
        }
    }
}
###################################################################

## File Name: normal2.cw.R
## File Version: 0.175


#** approximation of the bivariate normal integral
#** by the method of Cox and Wermuth (1991)
normal2.cw <- function( a, b, rho )
{
    # (X,Y) ~ N_2(0, rho)
    #   P( X > a1, Y > b1, rho )
    #=P( -X < - a1, -Y < -b1, rho )
    if ( any( rho < 0 )){
        stop('only positive correlations are allowed!')
    }
    a11 <- a1 <- - a
    b1 <- - b
    # APPROX. (ii)
    ind2 <- which( a1 < 0 & b1 < 0 )
    if ( length(ind2) > 0 ){
        a1[ ind2 ] <- - a1[ind2]
        b1[ ind2 ] <- - b1[ind2]
    }
    # APPROX. (i):
    # a > 0 and b > 0=> a > b
    ind <- which( a1 < b1 )
    t1 <- b1
    if ( length(ind) > 0 ){
        b1[ ind ] <- a1[ind]
        a1[ind] <- t1[ind]
    }
    t1 <- stats::pnorm(-a1)
    mu <- sirt_dnorm(a1) / stats::pnorm(-a1)
    xi <- ( rho * mu - b1 ) / sqrt( 1 - rho^2 )
    # see Hong (1999)
    # sig2 <- 1 + mu - mu^2 #=> formula in Cox & Wermuth (1991)
    sig2 <- 1 + a1*mu - mu^2
    #  Formula (3)
    # t1 * pnorm( xi )
    # Formula (4): correction in Hong (1999)
    prob1 <- t1*( stats::pnorm(xi) - 1/2 * rho^2/( 1 - rho^2 )*xi*sirt_dnorm(xi)*sig2  )
    # adjust formula in case of APPROX. (ii)
    if ( length(ind2) > 0 ){
        # CW. Formula in (ii), p. 264
        prob1[ind2] <- 1 - stats::pnorm( -a1[ind2] ) -
                                stats::pnorm( -b1[ind2] ) + prob1[ind2]
    }
#    # negative correlations
#    if ( length(ind.neg) > 0 ){
#        prob1[ind.neg] <- pnorm( - a11[ind.neg] ) - prob1[ ind.neg]
#    }
    return(prob1)
}

## File Name: xxirt_print_progress.R
## File Version: 0.072

xxirt_print_progress <- function(dev, dev0, dev00, pen_val, conv0, conv1,
        iter, verbose1, verbose2, verbose_index, verbose3=FALSE,
        opt_fun_value, prior_par )
{
    if (verbose1){
        cat( paste( '   Optimization function value=', round( opt_fun_value, 4 ),
            if (iter > 1 ){ ' | Change=' } else {''},
            if( iter > 1){ round( - dev + dev0, 6 )} else { ''}    ,'\n',sep='') )
        cat( paste( '   -Log-likelihood=',
                paste( round( dev00/2, 4), collapse=' ' ), '\n', sep=''))
        cat( paste( '   Prior function value=',
                paste( round( prior_par, 6), collapse=' ' ), '\n', sep=''))
        cat( paste( '   Penalty function=',
                paste( round( pen_val, 6), collapse=' ' ), '\n', sep=''))
        cat( paste( '   Deviance=',
                paste( round( dev00, 4), collapse=' ' ), '\n', sep=''))
        cat( paste( '    Maximum item parameter change=',
                paste( round( conv0, 6), collapse=' ' ), '\n', sep=''))
        cat( paste( '    Maximum theta distribution parameter change=',
                paste( round( conv1, 6), collapse=' ' ), '\n', sep=''))
        utils::flush.console()
    }
    if (verbose2){
        v1 <- paste0( verbose_index, ' | ',
                'Iteration=', iter, ' | Optimization function value=',
                round(dev,4), '\n')
        cat(v1)
        utils::flush.console()
    }
    if (verbose3){
        cat('-')
        utils::flush.console()
    }
}

## File Name: IRT.posterior.mirt.R
## File Version: 0.21



#--- likelihood singleGroup
IRT.posterior.mirt.singleGroup <- function( object, ... )
{
    ll <- IRT_likelihood_mirt_singleGroup( object=object, type="f.qk.yi", ...)
    return(ll)
}
IRT.posterior.ConfirmatoryClass <- IRT.posterior.mirt.singleGroup
IRT.posterior.ExploratoryClass <- IRT.posterior.mirt.singleGroup
IRT.posterior.SingleGroupClass <- IRT.posterior.mirt.singleGroup

IRT.posterior.mirt.multipleGroup <- function( object, ... )
{
    ll <- IRT_likelihood_mirt_multipleGroup( object=object, type="f.qk.yi", ... )
    return(ll)
}
IRT.posterior.MultipleGroupClass <- IRT.posterior.mirt.multipleGroup

## File Name: lsem_fitsem_raw_data_define_pseudo_weights.R
## File Version: 0.142

lsem_fitsem_raw_data_define_pseudo_weights <- function(dat, pseudo_weights)
{
    sampling_weights <- 'weight'
    W <- sum(dat[,sampling_weights])
    if (pseudo_weights>0){
        weights <- dat$weight
        N <- nrow(dat)
        W <- sum(weights)
        fac <- pseudo_weights / W
        weights1 <- fac*weights
        weights2 <- cumsum(weights1)
        tweights2 <- floor(weights2)
        freq <- c(tweights2[1], diff(tweights2))
        ind <- rep(1L:N, freq)
        dat <- dat[ind,]
        sampling_weights <- NULL
    }
    nobs_pseudo <- nrow(dat)
    sum_weight <- W
    #-- output
    res <- list(dat=dat, sampling_weights=sampling_weights,
                    nobs_pseudo=nobs_pseudo, sum_weight=sum_weight)
    return(res)
}

## File Name: Rhat_sirt.R
## File Version: 1.12


####################################
# auxiliary functions for Rhat statistic
    ############################################################################
    # Code from rube package
    # Source: http://www.stat.cmu.edu/~hseltman/rube/rube0.2-16/R/Rhat.R
    # Inference from Iterative Simulation Using Multiple Sequences
    # Author(s): Andrew Gelman and Donald B. Rubin
    # Source: Statistical Science, Vol. 7, No. 4 (Nov., 1992), pp. 457-472
    # Stable URL: http://www.jstor.org/stable/2246093
    ## Matches gelman.diag() from package "coda", but not WinBUGS() "summary" component.
    ## Better than gelman.diag() because multivariate stat is not
    ## bothered to be calculated
Rhat1 <- function(mat)
{
    m <- ncol(mat)
    n <- nrow(mat)
    b <- apply(mat,2,mean)
    B <- sum((b-mean(mat))^2)*n/(m-1)
    w <- apply(mat,2,stats::var)
    W <- mean(w)
    s2hat <- (n-1)/n*W + B/n
    Vhat <- s2hat + B/m/n
    covWB <- n /m * (stats::cov(w,b^2)-2*mean(b)*stats::cov(w,b))
    varV <- (n-1)^2 / n^2 * stats::var(w)/m +
                (m+1)^2 / m^2 / n^2 * 2*B^2/(m-1) +
                2 * (m-1)*(n-1)/m/n^2 * covWB
    df <- 2 * Vhat^2 / varV
    R <- sqrt((df+3) * Vhat / (df+1) / W)
    return(R)
}


Rhat <- function(arr)
{
    dm <- dim(arr)
    if (length(dm)==2) return(Rhat1(arr))
    if (dm[2]==1) return(NULL)
    if (dm[3]==1) return(Rhat1(arr[,,1]))
    return(apply(arr,3,Rhat1))
}

## File Name: gom_em_compute_total_loglikelihood.R
## File Version: 0.02

gom_em_compute_total_loglikelihood <- function(f.yi.qk, pi.k, weights)
{
    N <- nrow(f.yi.qk)
    ll <- sum( weights*log( rowSums( f.yi.qk * sirt_matrix2( x=pi.k, nrow=N ) ) ) )
    return(ll)
}

## File Name: mgsem_bdiag.R
## File Version: 0.06

mgsem_bdiag <- function(x1, x2)
{
    vars <- c(rownames(x1),rownames(x2))
    n1 <- ncol(x1)
    n2 <- ncol(x2)
    mat <- matrix(0,nrow=n1+n2,ncol=n1+n2)
    rownames(mat) <- colnames(mat)
    mat[1L:n1,1L:n1] <- x1
    mat[n1+1L:n2,n1+1L:n2] <- x2
    return(mat)
}

## File Name: xxirt_mstep_itemParameters_evalPrior.R
## File Version: 0.154


#*** evaluate prior in M-step
xxirt_mstep_itemParameters_evalPrior <- function(partable, h=0)
{
    eps <- 1E-300
    partable1 <- partable[ partable$parfree==1, ]
    NP <- nrow(partable1)
    NP1 <- max( NP, 1)
    #*** evaluate prior distributions in partable
    pen <- rep(0,NP1)
    if (NP>0){
        for (pp in 1L:NP){
            if ( ! is.na( partable1[pp,'prior'] ) ) {
                prior_pp <- partable1[pp,'prior']
                val <- partable1[pp,'value'] + h
                prior_args_pp <- list( val, partable1[pp,'prior_par1'],
                                                partable1[pp,'prior_par2'] )

                prior_val <- do.call( prior_pp, prior_args_pp )
                pen[pp] <- - log( prior_val + eps )
            }  # end if prior
        }    # end pp

    }  # end if NP > 0
    return(pen)
}

## File Name: mgsem_vcov_me.R
## File Version: 0.171

mgsem_vcov_me <- function(coef, opt_fun_args, suffstat_vcov, comp_se,
            se_delta_formula=FALSE)
{
    estimator <- opt_fun_args$technical$estimator
    comp_se_me <- (estimator=='ME') & comp_se
    if ( (estimator=='ML') & comp_se & se_delta_formula ){
        comp_se_me <- TRUE
    }
    A <- NULL
    V <- NULL
    if (comp_se_me){
        NP <- length(coef)
        parnames <- names(coef)
        #-- derivative with respect to coef
        grad_der_coef <- matrix(0, nrow=NP, ncol=NP)
        rownames(grad_der_coef) <- colnames(grad_der_coef) <- parnames
        h <- opt_fun_args$technical$h
        for (pp in 1L:NP){
            coef1 <- mgsem_add_increment(x=coef, h=h, i1=pp)
            res1 <- mgsem_grad_fun(x=coef1, opt_fun_args=opt_fun_args, output_all=FALSE)
            coef2 <- mgsem_add_increment(x=coef, h=-h, i1=pp)
            res2 <- mgsem_grad_fun(x=coef2, opt_fun_args=opt_fun_args, output_all=FALSE)
            grad_der_coef[,pp] <- ( res1 - res2 )/(2*h)
        }
        grad_der_coef <- ( grad_der_coef + t(grad_der_coef) ) / 2

        #-- derivative with respect input parameters of sufficient statistics
        suffstat_pars <- suffstat_vcov$suffstat_pars
        SP <- nrow(suffstat_pars)
        V <- suffstat_vcov$suffstat_vcov
        rownames(V) <- suffstat_pars$label
        colnames(V) <- suffstat_pars$label
        grad_der_suffstat <- matrix(0, nrow=NP, ncol=SP)
        rownames(grad_der_suffstat) <- parnames
        colnames(grad_der_suffstat) <- suffstat_pars$label
        opt_fun_args1 <- opt_fun_args
        suffstat <- opt_fun_args$suffstat
        for (pp in 1L:SP){
            suffstat_pars_pp <- suffstat_pars[pp,]
            group_pp <- suffstat_pars_pp$group
            val_pp <- list(NA,2)
            for (oo in 1L:2){
                suffstat1 <- suffstat
                u <- h
                if (oo==2){
                    u <- -h
                }
                if (suffstat_pars_pp$type=='mu'){
                    entry1 <- mgsem_add_increment(x=suffstat1[[group_pp]]$M, h=u,
                                        i1=suffstat_pars_pp$index1)
                    suffstat1[[group_pp]]$M <- entry1
                } else {
                    entry1 <- mgsem_add_increment(x=suffstat1[[group_pp]]$S, h=u,
                                        i1=suffstat_pars_pp$index1,
                                        i2=suffstat_pars_pp$index2, symm=TRUE)
                    suffstat1[[group_pp]]$S <- entry1
                }
                opt_fun_args1$suffstat <- suffstat1
                val_pp[[oo]] <- mgsem_grad_fun(x=coef, opt_fun_args=opt_fun_args1,
                                    output_all=FALSE)
            }
            der_est <- (val_pp[[1]]-val_pp[[2]])/(2*h)

            grad_der_suffstat[,pp] <- der_est
        }

        #-- compute transformation matrix A
        W1 <- mgsem_ginv(X=grad_der_coef)
        A <- - ( W1 %*% grad_der_suffstat )
        vcov <- A %*% V %*% t(A)
        rownames(vcov) <- colnames(vcov) <- names(coef)
        se <- mgsem_sqrt_diag(x=vcov)
        names(se) <- names(coef)
    } else {
        vcov <- NULL
        se <- NULL
    }

    #--- output
    res <- list(vcov=vcov, se=se, A=A, V=V, comp_se_me=comp_se_me)
    return(res)
}

## File Name: linking_haberman_compute_lts_mean.R
## File Version: 0.091

linking_haberman_compute_lts_mean <- function(x, w, lts_prop, maxiter=10)
{
    res <- linking_haberman_remove_missings_vector(x=x,w=w)
    x <- res$x
    w <- res$w
    n <- length(x)
    k <- max(2, ceiling(n*lts_prop))
    if (n < k){
        k <- n
    }
    index <- 1L:k
    m1 <- stats::weighted.mean(x=x, w=w)
    iter <- 0
    iterate <- TRUE
    dfr <- data.frame(index=1L:n, x=x, e=x-m1, w=w)
    while(iterate){
        m0 <- m1
        dfr <- dfr[ order(abs(dfr$e)), ]
        m1 <- stats::weighted.mean(x=dfr$x[1L:k], w=dfr$w[1L:k])
        dfr$e <- dfr$x - m1
        iter <- iter + 1
        if (iter>maxiter){ iterate <- FALSE }
        if (abs(m0-m1) < 1e-4 ){ iterate <- FALSE }
    }
    res <- m1
    return(res)
}

## File Name: xxirt_IRT.se.R
## File Version: 0.11


IRT.se.xxirt <- function( object, ...)
{
    c1 <- coef(object)
    v1 <- vcov(object)
    par1 <- xxirt_partable_extract_freeParameters( object$partable )
    par2 <- xxirt_parTheta_extract_freeParameters( object$customTheta )
    N1 <- length(par1)
    N2 <- length(par2)
    dfr <- data.frame(partype=c( rep('item',N1), rep('Theta',N2) ))
    dfr$parlabel <- names(c1)
    dfr$value <- c1
    dfr$se <- sqrt( diag(v1) )
    return(dfr)
}

## File Name: prmse.subscores.R
## File Version: 0.14

#--------------------------------------------------------------------------
# calculation of PRMSE for Subscores according to Haberman (2007)
prmse.subscores <- function( data.X, data.Z){
    ind <- which( rowSums( is.na( data.Z ) ) > 0 )
#    require(psy)
    if (length(ind) > 0){
        data.X <- data.X[ - ind, ]
        data.Z <- data.Z[ - ind, ]
                    }
    aX <- .cronbach.alpha( data.X )
    aZ <- .cronbach.alpha( data.Z )
    res <- list( "N"=aX$sample.size,  "nX"=aX$number.of.items )
    score.X <- rowSums( data.X )
    score.Z <- rowSums( data.Z )
    res$M.X <- mean( score.X )
    res$Var.X <- stats::var( score.X )
    res$SD.X <- sqrt( res$Var.X )
    res$alpha.X <- aX$alpha
    res$Var.TX <- res$Var.X * res$alpha.X
    res$Var.EX <- res$Var.X * ( 1 - res$alpha.X )
    res$nZ <- aZ$number.of.items
    res$M.Z <- mean( score.Z )
    res$Var.Z <- stats::var( score.Z )
    res$SD.Z <- sqrt( res$Var.Z )
    res$alpha.Z <- aZ$alpha
    res$Var.TZ <- res$Var.Z * res$alpha.Z
    res$Var.EZ <- res$Var.Z * ( 1 - res$alpha.Z )
    res$cor.X_Z <- stats::cor( score.X, score.Z )
    res$cor.X_Y <- stats::cor( score.X, score.Z - score.X )
    res$cor.TX_TY <- res$cor.X_Y / sqrt( res$alpha.X ) /
                sqrt( .cronbach.alpha( data.Z[, setdiff( colnames(data.Z), colnames(data.X) ) ] )$alpha )
    res$Var.TX <- res$Var.X - res$Var.EX
    res$Var.TZ <- res$Var.Z - res$Var.EZ

    res$cor.TX_TZ <- res$cor.X_Z / sqrt( res$alpha.X * res$alpha.Z ) -
                        res$Var.EX / sqrt( res$Var.TX * res$Var.Z )
    res$cor.TX_Z <- res$cor.TX_TZ * sqrt( res$alpha.Z )
    # RMSE based on subscores (Kelley formula)
    res$rmse.X <- sqrt( res$Var.TX * ( 1 - res$alpha.X ) )
    # RMSE based on total scores
    res$rmse.Z <- sqrt( res$Var.TX * ( 1 - res$cor.TX_Z^2 ) )
    # calculation of regression coefficients
    regr <- matrix( 0, nrow=3, ncol=3 )
    colnames(regr) <- c("Int", "beta.X", "beta.Z" )
    rownames(regr) <- c("Mod.X", "Mod.Z", "Mod.XZ" )
    regr[ "Mod.X", "beta.X" ] <- res$alpha.X
    regr[ "Mod.X", "Int" ] <- res$M.X - res$alpha.X * res$M.X
    regr[ "Mod.Z", "beta.Z" ] <- res$cor.TX_Z * sqrt(res$Var.TX) / sqrt( res$Var.Z )
    regr[ "Mod.Z", "Int" ] <- res$M.X - regr[ "Mod.Z", "beta.Z" ] * res$M.Z
    regr[ "Mod.XZ", "beta.X" ] <- ( sqrt( res$Var.TX ) * ( sqrt( res$alpha.X ) - res$cor.TX_Z * res$cor.X_Z ) ) /
                                        ( res$SD.X * ( 1 - res$cor.X_Z^2 ) )
    regr[ "Mod.XZ", "beta.Z" ] <- ( sqrt( res$Var.TX ) * ( res$cor.TX_Z - sqrt(res$alpha.X) * res$cor.X_Z ) ) /
                                        ( res$SD.Z * ( 1 - res$cor.X_Z^2 ) )
    regr[ "Mod.XZ", "Int" ] <- res$M.X - regr[ "Mod.X", "beta.X" ] * res$M.X - regr[ "Mod.Z", "beta.Z" ] * res$M.Z
    # calculation of RMSE of the regression on both subscores and total score
    pcor <- (( res$cor.TX_Z - sqrt( res$alpha.X) * res$cor.X_Z ) /
                    sqrt( 1 - res$alpha.X ) / sqrt( 1 - res$cor.X_Z^2 ) )
    res$rmse.XZ <- res$rmse.X * sqrt( 1 - pcor^2)
    res$prmse.X <- res$alpha.X
    res$prmse.Z <- res$cor.TX_Z^2
    res$prmse.XZ <- 1 - ( 1 - res$alpha.X )*( 1 -pcor^2 )
    list( "res"=res, "regr"=regr )
        }
#..................................................................................



#--------------------------------------------------------------------------------
# calculation of PRMSE for a number of subscales
prmse.subscores.scales <- function( data, subscale ){
        # data ... original data frame
        # scales ... classification of items in data into scales
        scales <- sort( unique( subscale ) )
        dfr <- NULL
        for ( ss in scales ){
            # ss <- scales[1]
            res1 <- prmse.subscores(  data.X=data[, subscale==ss  ], data.Z=data )
            dfr <- cbind( dfr, unlist( res1$res ) )
                }
        colnames(dfr) <- scales
        return(dfr)
            }
#--------------------------------------------------------------------------------




# aux function for Cronbach's Alpha
    .cronbach.alpha <- function( data ){
        # covariance
        c1 <- stats::cov( data, use="pairwise.complete.obs" )
        # mean covariance
        c1a <- c1 ; diag(c1a) <- 0
        I <- ncol(data)
        mc <- sum(c1a) / ( I^2 - I )
        # mean variance
        mv <- mean( diag(c1) )
        alpha <- I * mc / ( mv + (I-1) *mc )
        mean.tot <- mean( rowSums(data) )
        var.tot <- var( rowSums( data ) )
         res <- list( "I"=I, "alpha"=alpha,
                "mean.tot"=mean.tot,  "var.tot"=var.tot,
                "sample.size"=nrow(data),
                "number.of.items"=I )
        return(res)
            }

## File Name: rm_facets_itempar_expanded.R
## File Version: 0.08



#######################################################
# parameters expanded dataset
rm_facets_itempar_expanded <- function( b.item, b.rater, Qmatrix, tau.item,
        VV, K, I, TP, a.item, a.rater, item.index, rater.index,
        theta.k, RR )
{
    b <- tau.item[ item.index, ]
    b0 <- ( matrix( b.rater, nrow=RR, ncol=K) )[ rater.index, ] * Qmatrix[ item.index,]
    b <- b + b0
    # a parameter
    a <- a.item[ item.index ] * a.rater[ rater.index ]
    res <- list(a=a, b=b )
    return(res)
}
#########################################################

## File Name: rm_posterior.R
## File Version: 0.181


#######################################################
# calculate posterior and counts
rm_posterior <- function( dat2, dat2.resp, TP, pi.k,
    K, I, probs, dat2.ind.resp )
{
    #--- calculate likelihood
    probsM <- matrix( aperm( probs, c(2,1,3) ), nrow=I*(K+1), ncol=TP )
    f.yi.qk <- rm_calclike( dat2=dat2, dat2resp=dat2.resp,    probs=probsM, K=K)$fyiqk

    #--- calculate posterior and expected counts
    prior <- matrix( pi.k, nrow=nrow(dat2), ncol=TP, byrow=TRUE )
    f.qk.yi <- f.yi.qk * prior
    f.qk.yi <- f.qk.yi / rowSums( f.qk.yi )
    #--- expected counts
    n.ik <- array( 0, dim=c(TP, I, K+1 ) )
    N.ik <- array( 0, dim=c(TP, I ) )
    for (kk in 1L:(K+1) ){
        n.ik[,,kk] <- crossprod( f.qk.yi, dat2.ind.resp[,,kk] )
        N.ik <- N.ik + n.ik[,,kk]
    }
    pi2 <- sirt_matrix2( x=pi.k, nrow=nrow(f.yi.qk) )
    ll <- sum( log( rowSums( f.yi.qk * pi2 ) ) )

    #--- compute pi.k
    pi.k <- colMeans( f.qk.yi )

    #--- output
    res <- list( f.yi.qk=f.yi.qk, f.qk.yi=f.qk.yi,
            n.ik=n.ik, N.ik=N.ik, pi.k=pi.k, ll=ll)
    return(res)
}


.rm.posterior <- rm_posterior

## File Name: summary.noharm.sirt.R
## File Version: 1.188


#--- summary function for noharm.sirt
summary.noharm.sirt <- function( object, file=NULL, ...)
{
    # open sink
    sirt_osink( file=file )

    display_string <- sirt_summary_print_display(symbol="-", len=65)
    cat(display_string)

    #--- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object)

    cat( paste0( "Function '", class(object), "'\n\n" ) )

    #-- elapsed time
    sirt_summary_print_elapsed_time(object=object)

    #--- model type
    if (object$modtype==2){
        cat( "Multidimensional Exploratory Factor Analysis\n")
    }
    if (object$modtype==3){
        cat( "Multidimensional Confirmatory Factor Analysis\n")
    }

    if (object$modesttype==1){
        cat( "NOHARM approximation\n")
    }
    # if (object$modesttype==2){
    #     cat( "Estimation based on tetrachoric correlations\n\n")
    # }

    sirt_optimizer_summary_print(res=object$res_opt)

    #*********************
    # descriptives
    cat( paste( "Number of Observations: ", object$Nobs, sep=""), "\n" )
    cat( paste( "Number of Items       : ", object$Nitems, sep=""), "\n" )
    cat( paste( "Number of Dimensions  : ", object$dimensions, sep=""), "\n" )
    # cat( paste( "Number of Iterations  : ", object$iter, sep=""), "\n" )
    if (object$modesttype==1){
        cat( paste( "Tanaka Index          : ", round(object$tanaka,object$display.fit), sep=""), "\n" )
        cat( paste( "RMSR                  : ", round(object$rmsr,object$display.fit), sep=""), "\n\n" )
    } else { cat("\n") }

    cat( paste( "Number of Used Item Pairs      : ", object$sumwgtm, sep=""), "\n" )
    cat( paste( "Number of Estimated Parameters : ", object$Nestpars$total, sep=""), "\n" )
    cat( paste( "       # Thresholds            : ", object$Nestpars$thresh, sep=""), "\n" )
    cat( paste( "       # Loadings              : ", object$Nestpars$F, sep=""), "\n" )
    cat( paste( "       # Variances/Covariances : ", object$Nestpars$P, sep=""), "\n" )
    cat( paste( "       # Residual Correlations : ", object$Nestpars$Psi, sep=""), "\n\n" )

    # chi square statistic
    if (object$modtype %in% 2:4){
        if (object$modesttype==1){
            cat("Chi Square Statistic of Gessaroli & De Champlain (1996)\n\n")
            cat( paste( "Chi2                           : ", round(object$chisquare,3), sep=""), "\n" )
            cat( paste( "Degrees of Freedom (df)        : ", object$df, sep=""), "\n" )
            cat( paste( "p(Chi2,df)                     : ", round(object$p.chisquare,3), sep=""), "\n" )
            cat( paste( "Chi2 / df                      : ", round(object$chisquare / object$df,3), sep=""), "\n" )
            cat( paste( "RMSEA                          : ", round(object$rmsea,3), sep=""), "\n\n" )
        }
        cat( paste( "Green-Yang Reliability Omega Total : ", round(object$omega.rel,3),
                        sep=""), "\n\n" )
    }
    if ( object$modtype %in% 2:4){
        #--- factor correlation
        cat( "Factor Covariance Matrix\n")
        print( round( object$factor.cor,3 ) )
        if ( object$modtype %in% 3){
            cat( "\nFactor Correlation Matrix\n")
            print( round( cov2cor(object$factor.cor), 3 ))
        }
    }
    if ( object$modtype %in% 3:4){
        # item parameter
        l1 <- object$loadings.theta
        cat("\nItem Parameters - Latent Trait Model (THETA) Parametrization\n",
                "Loadings, Constants, Asymptotes and Descriptives\n\n")
        m1 <- l1 %*% as.matrix(object$factor.cor) %*% t( l1 )
        v1 <- 1 + diag(m1)
        l1 <- data.frame( l1, final.constant=object$final.constants,
                    lower=object$lower, upper=object$upper,
                    item.variance=round(v1, 3), N=diag(object$N.itempair),
                    p=diag(object$pm) )
        l1 <- round(l1,3)
        print(l1)
        # residual correlation
        if (object$estpars$estPsi==1){
            cat("\nResidual Correlation Matrix\n")
            l1 <- round( object$residcorr, 3 )
            print(l1)
        }
        # factor analysis parametrization
        l1 <- object$loadings
        cat("\nItem Parameters - Common Factor (DELTA) Parametrization\n",
            "Loadings, Thresholds, Uniquenesses and Asymptotes\n\n")
        l1 <- data.frame( l1, threshold=object$thresholds,
                        lower=object$lower, upper=object$upper,
                        uniqueness=object$uniquenesses )
        l1 <- round(l1,3)
        print( l1 )
    }
    if ( object$modtype %in% 2){
        # item parameters
        # l1 <- object$loadings.theta
        cat("\nItem Parameters - Promax Rotated Parameters (THETA)\n",
                "Loadings, Constants, Asymptotes and Descriptives\n\n")
        l1 <- object$promax.theta
        l1 <- data.frame( l1, final.constant=object$final.constants,
                    lower=object$lower, upper=object$upper,
                    N=diag(object$N.itempair), p=diag(object$pm) )
        l1 <- round(l1,3)
        print(l1)
        res <- list( itempars.promax.theta=l1 )
        #****
        cat("\nItem Parameters - Promax Rotated Parameters (DELTA)\n",
                "Loadings, Constants, Asymptotes and Descriptives\n\n")
        l1 <- object$promax
        l1 <- data.frame( l1, thresh=object$threshold,
                    lower=object$lower, upper=object$upper,
                    N=diag(object$N.itempair), p=diag(object$pm) )
        l1 <- round(l1,3)
        print(l1)
    }

    cat("\n--- Parameter table ---\n\n")
    obji <- object$parm_table
    elim <- c("matid", "starts", "est_par")
    obji <- obji[, ! ( colnames(obji) %in% elim ) ]
    digits <- 3
    sirt_summary_print_objects(obji=obji, digits=digits, from=2, rownames_null=TRUE)

    # close sink
    sirt_csink( file=file )
}

## File Name: meas_inv_cfa_proc_partable.R
## File Version: 0.05


meas_inv_cfa_proc_partable <- function(partable, items)
{
    G <- max(partable$group)
    I <- length(items)
    mu <- partable[ partable$lhs=='F' & partable$op=='~1', 'est' ]
    sigma <- sqrt(partable[ partable$lhs=='F' & partable$op=='~~', 'est' ])
    lambda <- partable[ partable$lhs=='F' & partable$op=='=~', 'est' ]
    lambda <- matrix(lambda, nrow=G, ncol=I, byrow=TRUE)
    nu <- partable[ paste(partable$rhs)=='' & partable$op=='~1' &
                        partable$lhs %in% items, 'est' ]
    nu <- matrix(nu, nrow=G, ncol=I, byrow=TRUE)
    theta <- partable[ partable$op=='~~' & partable$lhs %in% items, 'est' ]
    theta <- matrix(theta, nrow=G, ncol=I, byrow=TRUE)

    #--- output
    res <- list(mu=mu, sigma=sigma, lambda=lambda, nu=nu, theta=theta)
    return(res)
}

## File Name: dmlavaan_est_model_parameterTable.R
## File Version: 0.074

dmlavaan_est_model_parameterTable <- function(mod, parnames, coef1, vcov1)
{
    partable <- lavaan::parameterTable(object=mod)
    partable$parname <- ''
    ind <- which( partable$free > 0)
    partable$parname[ind] <- parnames
    x1 <- setdiff( unique(partable$parname), '' )
    partable$pnid <- match( partable$parname, x1 )
    partable$pnid[ is.na(partable$pnid) ] <- 0
    partable$unique <- 1-duplicated(partable$parname)
    partable$unique[ partable$free==0 ] <- 0
    NPU <- max(partable$pnid)
    G <- max(partable$group)

    #* search for lavaan-defined parameter names
    pn1 <- paste(partable$parname)
    ld <- which( substring(pn1,1,2)=='.p')
    partable[ld, 'parname'] <- paste0( partable$lhs, partable$op, partable$rhs,
                                        '.g', partable$group )[ld]
    for (pp in 1L:NPU){
        ind_pp <- which(partable$pnid==pp)
        lab1 <- partable[ind_pp[1],'parname']
        partable[ind_pp,'parname'] <- lab1
    }

    parnames0 <- partable[partable$unique==1, 'parname']
    parnames <- partable[partable$free>0, 'parname']
    names(coef1) <- parnames
    colnames(vcov1) <- rownames(vcov1) <- parnames

    #--- output
    res <- list(partable=partable, parnames=parnames, G=G, NPU=NPU,
                    parnames0=parnames0, coef1=coef1, vcov=vcov)
    return(res)
}

## File Name: mgsem_loglike_suffstat_derivative.R
## File Version: 0.172


mgsem_loglike_suffstat_derivative <- function(suffstat, Mu, Sigma )
{
    N <- suffstat$N
    M <- suffstat$M
    S <- suffstat$S
    if (missing(Sigma)){
        res <- Mu
        Mu <- res$Mu
        Sigma <- res$Sigma
    }
    S1 <- mgsem_ginv(X=Sigma)
    p <- length(Mu)
    m1 <- M-Mu

    #*** mean
    dermean <- as.vector( N*( crossprod(m1, S1 )))

    #*** covariance matrix
    y <- S1 %*% m1
    S2 <- S %*% S1
    S3 <- S1 %*% S2

    #-- output
    res <- list(dermean=dermean, y=y, S1=S1, S2=S2, S3=S3)
    return(res)
}

## File Name: scale_group_means.R
## File Version: 0.08

scale_group_means <- function(M, SD, probs=NULL, M_target=0, SD_target=1)
{
    if (is.null(probs)){
        probs <- rep(1,length(M))
    }
    probs <- probs/sum(probs)
    M1 <- sum(probs*M)
    SD1 <- sqrt( sum(probs*SD^2)+sum(probs*(M-M1)^2) )
    M_z <- ( M - M1 ) / SD1
    SD_z <- SD / SD1
    M1_trafo <- M_target + SD_target*M_z
    SD1_trafo <- SD_target*SD_z
    model_M <- stats::lm(M1_trafo~M)
    # trafo y=a*x+b
    a <- coef(model_M)[2]
    b <- coef(model_M)[1]
    #--- output
    res <- list(M1=M1, SD1=SD1, M_z=M_z, SD_z=SD_z, M_trafo=M1_trafo,
                    SD_trafo=SD1_trafo, a=a, b=b )
    return(res)
}

## File Name: rm.sdt.R
## File Version: 8.888

#################################################################
# Hierarchical rater model
# MML estimation
rm.sdt <- function( dat, pid, rater,Qmatrix=NULL, theta.k=seq(-9,9,len=30),
    est.a.item=FALSE, est.c.rater="n",
    est.d.rater="n", est.mean=FALSE, est.sigma=TRUE, skillspace="normal",
    tau.item.fixed=NULL, a.item.fixed=NULL,
    d.min=.5, d.max=100,  d.start=3,  c.start=NULL, tau.start=NULL, sd.start=1,
    d.prior=c(3,100), c.prior=c(3,100), tau.prior=c(0,1000), a.prior=c(1,100),
    link_item="GPCM", max.increment=1, numdiff.parm=.00001, maxdevchange=.10,
    globconv=.001, maxiter=1000, msteps=4, mstepconv=.001, optimizer="nlminb" )
{
    #..........................................................
    CALL <- match.call()
    s1 <- Sys.time()
    theta.k0 <- theta.k
    pi.k <- sirt_dnorm_discrete(x=theta.k, mean=0, sd=sd.start)
    max.b.increment <- max.increment
    a_center_type <- 2
    PEM <- FALSE

    #-- process data
    procdata <- res <- rm_proc_data( dat=dat, rater=rater, pid=pid )
    dat2 <- as.matrix(res$dat2)
    dat2.resp <- as.matrix(res$dat2.resp)
    rater.index1 <- res$rater.index
    dataproc.vars <- res$dataproc.vars
    VV <- res$VV
    RR <- res$RR
    item.index <- res$dataproc.vars$item.index
    rater.index <- res$dataproc.vars$rater.index
    dat2.ind.resp <- res$dat2.ind.resp
    I0 <- res$I0

    #* processing for Rcpp code
    item_index <- item.index - 1
    dat2_resp <- dat2.resp==1

    deviance.history <- rep(NA, maxiter )

    # maximum categories
    maxK <- sirt_colMaxs(dat)
    K <- max( maxK )
    K1 <- K + 1
    if ( is.null(Qmatrix) ){
        Qmatrix <- matrix( 1:K, nrow=VV, ncol=K, byrow=TRUE)
    }
    TP <- length(theta.k)
    I <- VV*RR

    # define constraints on tau.item parameters
    # if not all categories are observed
    if ( is.null( tau.item.fixed )){
        tau.item.fixed <-  rm_determine_fixed_tau_parameters( K=K, maxK=maxK, VV=VV )
    }

    # starting values for item difficulties
    tau.item <- matrix( 0, nrow=VV, ncol=K )
    rownames(tau.item) <- colnames(dat)

    tau.item <- matrix( seq( -2, 2, len=K ), nrow=VV, ncol=K, byrow=TRUE )
    if ( ! is.null(tau.start) ){
        tau.item <- tau.start
    }
    if ( ! is.null(tau.item.fixed) ){
        tau.item[ tau.item.fixed[,1:2,drop=FALSE] ] <- tau.item.fixed[,3]
    }

    a.item <- rep( 1, VV )
    if (skillspace=="discrete" ){
        est.mean <- TRUE
    }
    if ( ! is.null( a.item.fixed ) ){
        est.a.item <- TRUE
        a.item[ a.item.fixed[,1] ] <- a.item.fixed[,2]
    }

    # rater parameter
    d.rater <- matrix( d.start, nrow=I, ncol=1 )
    if (is.null(c.start)){
        c.rater <- matrix( d.start*((1:K) - .5 ), nrow=I, ncol=K, byrow=TRUE )
    }
    if ( ! is.null(c.start) ){
        c.rater <- c.start
    }

    # set c.rater for fixed items to 99
    c.rater.fixed <- NULL
    if ( ! is.null( tau.item.fixed ) ){
        tau1 <- tau.item.fixed[ tau.item.fixed[,3]==99,, drop=FALSE]
        ind <- match( item.index, tau1[,1] )
        c.rater.fixed <- tau1[ ind, ]
        c.rater.fixed[,1] <- seq( 1, nrow(c.rater.fixed) )
        c.rater.fixed[,3] <- 999
        c.rater.fixed <- c.rater.fixed[ ! is.na( c.rater.fixed[,2] ), ]
        c.rater[ c.rater.fixed[,1:2] ] <- c.rater.fixed[,3]
    }

    #--- indices for derivatives
    diffindex <- rm_sdt_prepare_diffindex( item.index=item.index,
                        rater.index=rater.index, I=I, est.c.rater=est.c.rater,
                        est.d.rater=est.d.rater )

    # init standard errors
    se.d.rater <- NA*d.rater
    se.c.rater <- NA*c.rater
    se.a.item <- NA*a.item

    d.rater.incr <- 2
    tau.item.incr  <- max.b.increment
    c.rater.incr <- max.b.increment
    a.item.incr <- max.b.increment

    #*** create parameter table
    res <- rm_sdt_create_partable( item.index=item.index,
                rater.index=rater.index, est.c.rater=est.c.rater, est.d.rater=est.d.rater,
                tau.item=tau.item, c.rater=c.rater, diffindex=diffindex, tau.prior=tau.prior,
                a.prior=a.prior, d.prior=d.prior, c.prior=c.prior, est.a.item=est.a.item,
                tau.item.fixed=tau.item.fixed, a.item.fixed=a.item.fixed,
                c.rater.fixed=c.rater.fixed)
    partable_item <- res$partable_item
    partable_rater <- res$partable_rater
    par_index <- res$par_index
    pargroup_item <- res$pargroup_item
    pargroup_rater <- res$pargroup_rater

    #*** fill parameter table with initial values
    res <- rm_sdt_fill_init_partables( partable_item=partable_item, partable_rater=partable_rater,
                par_index=par_index, tau.item=tau.item, a.item=a.item, c.rater=c.rater,
                d.rater=d.rater)
    partable_item <- res$partable_item
    partable_rater <- res$partable_rater

    #*** optimization
    if (optimizer=="optim"){
        ctl <- list(maxit=msteps)
    }
    if (optimizer=="nlminb"){
        ctl <- list(iter.max=msteps)
    }

    # inits
    iter <- 0
    dev0 <- dev <- 1E300
    parm_minimal <- list(dev=dev)
    conv <- devchange <- 1000
    mu <- 0
    sigma <- 1
    disp <- "...........................................................\n"
    prob.item <- NULL
    prob.rater <- NULL

    #****************************************************
    # start EM algorithm
    while( ( ( maxdevchange < devchange ) | (globconv < conv) ) &
            ( iter < maxiter ) ){
        cat(disp)
        cat("Iteration", iter+1, "   ", paste( Sys.time() ), "\n" )

        # previous values
        d.rater0 <- d.rater
        tau.item0 <- tau.item
        dev0 <- dev
        mu0 <- mu
        sigma0 <- sigma
        a.item0 <- a.item
        c.rater0 <- c.rater

        #** probabilities item level
        if (link_item=="GPCM" ){
            prob.item <- rm_sdt_calc_probs_gpcm_rcpp( a.item=a.item, tau.item=tau.item, Qmatrix=Qmatrix,
                            theta.k=theta.k, VV=VV, K=K, TP=TP, eps=0, use_log=FALSE, as_vector=FALSE )
        }
        if (link_item=="GRM" ){
            prob.item <- rm_sdt_calc_probs_grm_item_rcpp( tau.item=tau.item,
                        a.item=a.item, theta.k=theta.k, VV=VV, K=K, TP=TP, eps=0, use_log=FALSE )
        }
        prob_item <- as.vector(prob.item)

        #** probabilities rater level
        prob.rater <- rm_sdt_calc_probs_grm_rcpp( c.rater=c.rater, d.rater=d.rater, I=I, K=K,
                                eps=0, use_log=FALSE )
        prob_rater <- as.vector(prob.rater)

        #-- calculate posterior
        res <- sirt_rcpp_rm_sdt_posterior( prob_item=prob_item, prob_rater=prob_rater, I0=I0, K1=K1,
                    TP=TP, item_index=item_index, dat2=dat2, dat2_resp=dat2_resp, I=I, pi_k0=pi.k )
        nik_item <- res$nik_item
        nik_rater <- res$nik_rater
        f.qk.yi <- res$fqkyi
        f.yi.qk <- res$fyiqk
        pi.k <- res$pi_k
        like <- res$like
        ll <- res$ll

        #-- item level
        # (item, true_category, node)
        nik.item <- array(nik_item, dim=c(I0, K1, TP) )

        #-- rater level
        # (item_rater, obs_category, true_category)
        nik.rater <- array(nik_rater, dim=c(I, K1, K1) )

        #---- item level: optimization and gradient function
        if (link_item=="GPCM" ){
            expected_loglike_item <- function(x, ...){
                post <- rm_sdt_mstep_item_function_value( x=x, par_index=par_index,
                            partable_item=partable_item, nik.item=nik.item, Qmatrix=Qmatrix,
                            theta.k=theta.k, VV=VV, K=K, TP=TP, eps=1E-10 )
                return(post)
            }
            gradient_loglike_item <- function(x, ...){
                grad_post <- rm_sdt_mstep_item_function_gradient( x=x, par_index=par_index,
                                    partable_item=partable_item, Qmatrix=Qmatrix, theta.k=theta.k, VV=VV,
                                    K=K, TP=TP, pargroup_item=pargroup_item, nik.item=nik.item,
                                    numdiff.parm=numdiff.parm, eps=1E-10 )
                return(grad_post)
            }
        }
        if (link_item=="GRM" ){
            #- arguments
            probs_fun <- rm_sdt_calc_probs_grm_item_rcpp
            K1 <- K+1
            probs_dim <- c(VV, K1, TP)
            probs_args <- list( VV=VV, K=K, theta.k=theta.k, TP=TP, eps=1E-10, use_log=TRUE )
            update_probs_args <- c('tau.item', 'a.item')
            #- optimization function
            expected_loglike_item <- function(x, ...){
                post <- rm_sdt_mstep_type_function_value( x=x, par_index=par_index,
                            partable=partable_item, type='item', probs_args=probs_args,
                            probs_fun=probs_fun, probs_dim=probs_dim,
                            update_probs_args=update_probs_args, nik=nik_item )
                return(post)
            }
            #- gradient
            gradient_loglike_item <- function(x, ...){
                grad_post <- rm_sdt_mstep_type_function_gradient( x=x,
                        par_index=par_index, partable=partable_item, type='item',
                        pargroup_type=pargroup_item, probs_args=probs_args, probs_fun=probs_fun,
                        probs_dim=probs_dim, update_probs_args=update_probs_args, nik=nik.item,
                        numdiff.parm=numdiff.parm )
                return(grad_post)
            }
        }

        #*** optimization item level
        parm0 <- rm_sdt_extract_par_from_partable(partable=partable_item)
        res <- sirt_optimizer(optimizer=optimizer, par=parm0, fn=expected_loglike_item,
                        grad=gradient_loglike_item, control=ctl)
        parm0 <- res$par
        #-- fill parameter table
        res <- rm_sdt_fill_par_to_partable( par_index=par_index, partable=partable_item,
                        parm0=parm0, type="item" )
        partable_item <- res$partable
        tau.item <- res$parm_list$tau.item
        a.item <- res$parm_list$a.item

        #---- rater level: optimization function
        expected_loglike_rater <- function(x, ...){
            post <- rm_sdt_mstep_rater_function_value( x=x, par_index=par_index,
                        partable_rater=partable_rater, I=I, K=K, nik_rater=nik_rater, eps=1E-10 )
            return(post)
        }
        #--- rater level: gradient
        gradient_loglike_rater <- function(x, ...){
            grad_post <- rm_sdt_mstep_rater_function_gradient( x=x,
                                par_index=par_index, partable_rater=partable_rater,
                                pargroup_rater=pargroup_rater, I=I, K=K, numdiff.parm=numdiff.parm,
                                nik.rater=nik.rater, eps=1E-10 )
            return(grad_post)
        }

        #*** optimization rater level
        parm0 <- rm_sdt_extract_par_from_partable(partable=partable_rater)
        L0 <- length(parm0)
        if (L0>0){
            res <- sirt_optimizer(optimizer=optimizer, par=parm0, fn=expected_loglike_rater,
                        grad=gradient_loglike_rater, control=ctl)
        } else {
            res <- list()
        }
        parm0 <- res$par

        #-- fill parameter table
        res <- rm_sdt_fill_par_to_partable( par_index=par_index, partable=partable_rater,
                        parm0=parm0, type="rater" )
        partable_rater <- res$partable
        c.rater <- res$parm_list$c.rater
        d.rater <- res$parm_list$d.rater

        #-- update distribution
        res <- rm_smooth_distribution( theta.k=theta.k, pi.k=pi.k, est.mean=est.mean,
                    skillspace=skillspace, est.sigma=est.sigma, sigma=sigma )
        pi.k <- res$pi.k
        mu <- res$mu
        sigma <- res$sigma

        #-- save deviance values
        deviance.history[iter+1] <- dev <- -2*ll

        if (dev < parm_minimal$dev ){
            parm_minimal <- list( iter_opt=iter, dev=dev, d.rater=d.rater,
                            c.rater=c.rater, tau.item=tau.item, a.item=a.item,
                            mu=mu, sigma=sigma, pi.k=pi.k)
        }

        #-- convergence criteria
        conv <- max( abs(c.rater-c.rater0), abs( c.rater-c.rater0),
                    abs( tau.item0-tau.item), abs( a.item - a.item0 ) )
        iter <- iter+1
        devchange <- abs( ( dev - dev0 ) / dev0  )

        #-- print progress
        res <- rm_sdt_print_progress( dev=dev, dev0=dev0, c.rater=c.rater, c.rater0=c.rater0,
                    d.rater=d.rater, d.rater0=d.rater0, tau.item=tau.item, tau.item0=tau.item0,
                    a.item=a.item, a.item0=a.item0, mu=mu, sigma=sigma, iter=iter )
    }
    #---------------------------- end EM algorithm

    # *********
    # arrange OUTPUT

    #-- assign elements of parm_minimal
    iter_opt <- NULL
    envir <- environment()
    res <- sirt_attach_list_elements(x=parm_minimal, envir=envir)

    # c parameters
    if ( ! is.null( c.rater.fixed ) ){
        c.rater[ c.rater.fixed[,1:2] ] <- NA
    }

    #---
    # Information criteria
    ic <- rm_sdt_postproc_ic( dev=dev, dat2=dat2, VV=VV, RR=RR, est.mean=est.mean,
                est.sigma=est.sigma, partable_item=partable_item,
                partable_rater=partable_rater, skillspace=skillspace, theta.k=theta.k )

    #---
    # person parameters
    res <- rm_facets_postproc_person( dat2=dat2, dat2.resp=dat2.resp, procdata=procdata,
                maxK=maxK, RR=RR, theta.k=theta.k, f.qk.yi=f.qk.yi )
    person <- res$person
    EAP.rel <- res$EAP.rel

    #---
    # item
    if (!is.null(tau.item.fixed)){
        K <- max(maxK)
        I <- nrow(tau.item)
        for (ii in 1:I){
            if ( maxK[ii] < K ){
                for (kk in seq(maxK[ii]+1,K) ){
                    tau.item[ii,kk] <- NA
                }
            }
        }
        # se.tau.item[ tau.item.fixed[,1:2,drop=FALSE] ] <- NA
    }

    item <- data.frame( "item"=colnames(dat),
            "N"=colSums( 1-is.na(dat)),
            "M"=colMeans( dat, na.rm=TRUE ) )
    for (kk in 1:K){ item[, paste0("tau.Cat",kk) ] <- tau.item[,kk] }
    item$a <- a.item

    # latent mean and standard deviation
    me1 <- rep(NA,VV)
    sd1 <- rep(NA,VV)
    for (ii in 1:VV){
        pii <- prob.item[ii,,]
        qii <- matrix( c(0,Qmatrix[ii,]), nrow=K+1, ncol=ncol(pii) )
        me1[ii] <- sum( colSums( qii * pii ) * pi.k )
        sd1[ii] <- sqrt( sum( colSums( qii^2 * pii ) * pi.k ) - me1[ii]^2  )
    }
    item$latM <- me1
    item$latSD <- sd1

    cat("*********************************\n")
    cat("Item Parameters\n")
    sirt_summary_print_objects(obji=item, digits=3, from=2)

    #---
    # rater
    M1 <- colSums( dat2 ) / colSums( dat2.resp )
    N <- colSums( dat2.resp )
    rater <- data.frame( "item.rater"=colnames(dat2),
            "N"=N, "M"=M1, "d"=d.rater )
    for (zz in 1:(ncol(c.rater) ) ){
        rater[, paste0("c_",zz)] <- c.rater[,zz]
    }
    # transformed c parameters
    for (zz in 1:(ncol(c.rater) ) ){
        rater[, paste0("c_",zz,".trans")] <- c.rater[,zz] / d.rater
    }

    rater <- rater[ order( paste( rater$item.rater) ), ]
    rownames(rater) <- NULL
    rownames(item) <- NULL

    rt1 <- rater
    l1 <- paste(rt1$item.rater)
    l2 <- strsplit( l1, split="-" )
    rt1$item <- unlist( lapply( l2, FUN=function(uu){ uu[[1]] } ) )
    rt1$rater <- unlist( lapply( l2, FUN=function(uu){ uu[[2]] } ) )

    #*****
    # distribution
    skill.distribution <- data.frame("theta.k"=theta.k, "pi.k"=pi.k )

    #*****
    # labels
    dimnames(prob.item) <- list( colnames(dat), paste0("Cat", 0:K), NULL )
    dimnames(prob.rater) <- list( colnames(dat2), paste0("Cat", 0:K), NULL )

    cat("*********************************\n")
    cat("Rater Parameters\n")
    sirt_summary_print_objects(obji=rater, digits=3, from=2)

    cat("*********************************\n")
    cat("EAP Reliability","=", round(EAP.rel,3), "\n")

    s2 <- Sys.time()
    res <- list( deviance=dev, ic=ic, item=item, rater=rater, person=person,
                EAP.rel=EAP.rel, mu=mu, sigma=sigma, theta.k=theta.k, pi.k=pi.k, G=1,
                tau.item=tau.item, a.item=a.item, c.rater=c.rater, d.rater=d.rater,
                f.yi.qk=f.yi.qk, f.qk.yi=f.qk.yi, prob.item=prob.item, prob.rater=prob.rater,
                nik.item=nik.item, nik.rater=nik.rater, maxK=maxK, pi.k=pi.k,
                procdata=procdata, iter=iter, theta.k=theta.k, Qmatrix=Qmatrix, s1=s1, s2=s2,
                tau.item.fixed=tau.item.fixed, rater2=rt1, maxK=maxK, link_item=link_item,
                skill.distribution=skill.distribution, skillspace=skillspace,
                iter_opt=iter_opt, CALL=CALL, deviance.history=deviance.history )
    class(res) <- "rm.sdt"
    return(res)
}

## File Name: ccov_np_compute_ccov_sum_score.R
## File Version: 0.162

ccov_np_compute_ccov_sum_score <- function(score, data, use_rcpp=TRUE)
{
    scores <- sort(unique(score))
    wgt_score <- sirt_sum_norm(table(score))
    NS <- length(scores)
    ccov_ff <- rep(NA,NS)
    if (!use_rcpp){
        for (ss in 1L:NS){
            i1 <- which(score==scores[ss])
            s1 <- stats::cov.wt(x=data[i1,], method='ML')
            ccov_ff[ss] <- s1$cov[1,2]
        }
        ccov_ff[is.na(ccov_ff)] <- 0
    } else {
        index <- match(score, scores)-1
        ccov_ff <- sirt_rcpp_ccov_np_compute_ccov_sum_score( index=index,
                            NS=NS, data=as.matrix(data) )
    }
    ccov_aggr <- sum(wgt_score*ccov_ff)
    res <- list(ccov_ff=ccov_ff, scores=scores, ccov_aggr=ccov_aggr)
    return(res)
}

## File Name: rasch_evm_pcm_dif.R
## File Version: 0.24


rasch_evm_pcm_dif <- function( b_evm, item, PARS_vcov, I, G, group.unique,
        dat, dat.resp )
{
    difstats <- data.frame( item=colnames(dat), X2=NA, df=NA, p=NA, p.holm=NA )
    for (jj in 1:I){
        ii <- jj
        ind.ii <- which( item$item==ii )
        nj <- length(ind.ii)
        delta.jj <- rep(NA, nj*G )
        varmat.jj <- matrix(0, nj*G, nj*G )
        Rdesign.jj <- matrix(0,nj*(G-1), nj*G )
        for (gg in 1:G){
            delta.jj[  1:nj + nj*(gg-1)  ] <- b_evm[[gg]][ ind.ii ]
            varmat.jj[ 1:nj + nj*(gg-1), 1:nj + nj*(gg-1) ] <- PARS_vcov[[gg]][ ind.ii, ind.ii ]
            if (gg <G){
                for (vv in 1:nj){
                    Rdesign.jj[ vv + nj*(gg-1), vv + nj*(gg-1) ] <- 1
                    Rdesign.jj[ vv + nj*(gg-1), vv + nj*(gg) ] <- -1
                }
            }
        }
        # calculate test value
        d0 <- Rdesign.jj %*% delta.jj
        # calculate variance matrix
        ivm <- solve( Rdesign.jj %*% varmat.jj %*% t(Rdesign.jj) )
        difstats[jj,"X2"] <- ( t(d0) %*% ivm %*% d0 )[1,1]
        difstats[jj,"df"] <- nrow(Rdesign.jj)
        # compute item-wise DIF statistics
        difjj <- stats::aggregate( delta.jj, list( rep(1:G, each=nj) ), mean)[,2]
        for (gg1 in 1:(G-1)){
            for (gg2 in (gg1+1):G){
                difstats[jj, paste0("DIF.",group.unique[gg1],".",group.unique[gg2]) ] <- difjj[gg1] - difjj[gg2]
            }
        }
    }
    difstats$p <- 1 - stats::pchisq( difstats$X2, df=difstats$df )
    difstats$p.holm <- stats::p.adjust( difstats$p )
    #-- output
    return(difstats)
}

## File Name: invariance_alignment_constraints_summary_print_item_summary.R
## File Version: 0.101

invariance_alignment_constraints_summary_print_item_summary <- function(x, digits)
{
    cat('Parameter tolerance value', '=', x$parm_tol, '\n')
    cat('Total number of items', '=', x$N_total, '\n')
    cat('Number of unique item parameters', '=', x$N_parm_all, '\n')
    cat('Percentage of noninvariance item parameters', '=',
                round(x$prop_noninvariance,1),'%\n')

    cat('\nUnique item parameters per item\n')
    print(x$N_unique_parm_items)

    cat('\nUnique item parameters per group\n')
    print(x$N_unique_parm_groups)

    cat('\nJoint item parameters\n')
    obji <- x$parm_joint
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    cat('\nEstimated item parameters\n')
    obji <- x$parm_est
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    cat('\nEstimated DIF effects\n')
    obji <- x$parm_dif
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)
}

## File Name: rasch_pairwise_optimize_opt_fun_terms2.R
## File Version: 0.131

rasch_pairwise_optimize_opt_fun_terms2 <- function(eps_horiz, eps_vert, y.ij, y.ji,
        estimator="MINCHI")
{
    tol <- 1e-10
    eps1 <- sqrt(eps_horiz + tol)
    I <- length(eps1)
    if (estimator=='ULS'){
        eps_v <- eps_vert
    }
    if (estimator=='MINCHI'){
        eps_v <- sqrt(eps_vert+tol)
    }
    epsM1 <- matrix(eps_v, nrow=I, ncol=I, byrow=TRUE)
    if (estimator=='MINCHI'){
        h1 <- eps1/epsM1
        t3 <- ( y.ij*h1 - y.ji/h1 )^2
    }
    if (estimator=='ULS'){
        t3 <- ( y.ij/epsM1 - y.ji/eps_horiz )^2
    }
    return(t3)
}

## File Name: mcmc.2pno.ml_alg.R
## File Version: 3.25

#***********************************************
# parts of algorithm:
# * draw Z
# * draw theta2
# * draw theta1 (centering!)
# * draw sigma2
# * draw sigma1
# * draw b_class
# * draw b
# * draw variance(b_class)
# * draw a_class
# * draw a
# * draw variance(a_class)
#********************************************

#############################################
# compute group mean by the rowsum function
.mcmc.groupmean <- function( matr, group, groupsize=NULL ){
    r1 <- rowsum( matr, group )
    if ( is.null(groupsize) ){
            groupsize <- rowsum( 1+0*matr[,1], group )[,1]
                    }
    r1 / groupsize
        }
############################################

####################################################
# draw latent responses Z
.draw.Z.2pno.ml <- function( aM, bM, theta, N, I, threshlow, threshupp ){
    # calculate means
    mij <- aM * theta - bM
    # simulate uniform data
    rij <- matrix( stats::runif( N*I ), nrow=N, ncol=I )
    # calculate corresponding value
    pl <- stats::pnorm( threshlow, mean=mij)
    pu <- stats::pnorm( threshupp, mean=mij)
    pij <- pl + (pu-pl)*rij
    # simulate Z
    Zij <- stats::qnorm( pij, mean=mij )
    return(Zij)
        }
#############################################

########################################
# draw theta=theta.L2 + theta.L1
# here, the "total" theta is sampled!
.draw.theta.2pno.ml <- function( aM, b, bM, N, I, Z,
    sigma1, sigma2, sigma.res, link, theta2, idgroup ){
    #**************************************
    # link=logit
    # Z=aM * theta - bM + eps
    # Z + bM=aM * theta + eps
#    bM <- matrix(b,N,I,byrow=TRUE )
    if (link=="logit"){
#        vtheta <- 1 / ( rowSums( aM^2 ) + ( sigma1^2+sigma2^2 )  )
#        vtheta <- 1 / ( rowSums( aM^2 ) + 1/( sigma1^2+sigma2^2 )  )
#        mtheta <- rowSums( aM * ( Z + bM ) ) * vtheta
        Zres <- Z + bM
        m1ij <- rowSums( aM * Zres )
        prec1 <- rowSums(aM^2 )
        m1ij <- m1ij / prec1
        m2ij <- theta2[ idgroup ]
#        prec2 <- 1 / ( sigma1^2 + sigma2^2 )
        prec2 <- 1 / ( sigma1^2)
                    } # end logit
    #***************************************
    # link=normal
    if (link=="normal"){
# print(sigma.res)
        sigma.resM <- matrix( sigma.res, N, I, byrow=TRUE)
        Zres <- Z + bM
        # correct precision and estimator
        # something seems to go wrong here
#        m1ij <- rowSums( aM * Zres * sigma.resM)
        m1ij <- rowSums( aM * Zres / sigma.resM^2 )
        m2ij <- theta2[ idgroup ]
#        prec1 <- rowSums( aM^2 * sigma.resM^2 )
        prec1 <- rowSums( aM^2 / sigma.resM^2 )
        m1ij <- m1ij / prec1
        prec2 <- 1 / ( sigma1^2 )
                } # end normal
    prectotal <- prec1 + prec2
    mtheta <- ( prec1*m1ij + prec2 * m2ij ) / prectotal
    vtheta <- 1 / prectotal
    theta <- stats::rnorm( N, mean=mtheta, sd=sqrt( vtheta ) )
    theta <- theta - mean(theta)
    return(theta)
            }
#########################################
# draw level 2 latent class mean
.draw.theta2.2pno.ml <- function( theta, idgroup, groupsize,
        sigma1, sigma2, G ){
        # compute latent mean
        mij1 <- .mcmc.groupmean( theta, idgroup, groupsize)[,1]
        prec.ij1 <- 1 / ( sigma1^2 / groupsize )
        prec.ij2 <- 1 / sigma2^2
        prec.tot <- prec.ij1 + prec.ij2
        vtheta <- 1 / prec.tot
        mtheta <- mij1 * prec.ij1 / prec.tot
        theta2 <- stats::rnorm( G, mean=mtheta, sd=sqrt( vtheta ))
        theta2 <- theta2 - mean(theta2)
        return(theta2)
            }
######################################

############################################################
# draw level 1 and level 2 variances
.draw.sigma12.2pno.ml <- function( theta, theta2, idgroup, N,
        G, prior.sigma2 ){
        # level 1 variance
        sig2 <- sum( (theta - theta2[ idgroup ])^2 ) / N
        sigma1 <- sqrt( .mcmc.draw.variance( 1, w0=1, sig02=.7, n=N, sig2=sig2 ) )
        # level 2 variance
        sig2 <- sum( theta2^2 ) / G
        sigma2 <- sqrt( .mcmc.draw.variance( 1,
                    w0=prior.sigma2[1], sig02=prior.sigma2[2]^2, n=G, sig2=sig2 ) )
        res <- list( "sigma1"=sigma1, "sigma2"=sigma2 )
        return(res)
            }
######################################################
# b item parameter single level case
.draw.est.b.sl <- function(    Z, aM, theta, N, I, omega.b, mu.b,
        sigma.res, link ){
        #     Z=aM * theta - bM + eps
        #=>  bM=Z - aM*theta - eps
    Zres <- Z - aM * theta
    mij1 <- -colMeans(Zres)
    #****
    # link=logit
    if (link=="logit"){
        prec1ij <- N
        prec2ij <- 1 / omega.b^2
        prectotal <- prec1ij + prec2ij
        mij <- ( mij1 * prec1ij + mu.b * prec2ij ) / prectotal
        vij <- 1 / prectotal
                }
    #***
    # link=normal
    if (link=="normal"){
        prec1ij <- N / sigma.res^2
        prec2ij <- 1 / omega.b^2
        prectotal <- prec1ij + prec2ij
        mij <- ( mij1 * prec1ij + mu.b * prec2ij ) / prectotal
        vij <- 1 / prectotal
                } # end link=normal
    #*** sample b parameter
    b <- stats::rnorm( I, mean=mij, sd=sqrt( vij ) )
    return(b)
            }
####################################################
# draw hyperparameters of b
.draw.est.b.hyperpars <- function( b, mu.b, omega.b, I,
    prior.omega.b, est.b.M  ){
        if ( est.b.M=="h"){
            # sample mu.b
            mij <- mean(b)
            vij <- omega.b^2 / I
            mu.b <- stats::rnorm(1, mean=mij, sd=sqrt(vij) )
            # sample omega.b
            sig2 <- sum( ( b - mu.b )^2 ) / I
            omega.b <- sqrt( .mcmc.draw.variance( 1,
                        w0=prior.omega.b[1], sig02=prior.omega.b[2]^2,
                        n=I, sig2=sig2 ) )
                        }
        res <- list( "mu.b"=mu.b, "omega.b"=omega.b )
        return(res)
            }

#####################################################
# sampling of b in case of multilevel DIF
.mcmc.est.b.2pno.ml.v2 <- function( N, Z, aM, theta, idgroup, groupsize,
                            b, bG, G, I, sigma.b, omega.b, mu.b, sigma.res, link ){
            # sampling of b (fixed item difficulties)
            # Z=aM *  theta - b - bG + eps
            # Z - aM * theta + bG=- b + eps
            Zres <- Z - aM * theta + bG[ idgroup, ]
#            Zres <- Z - aM * theta
            mij1 <- -colMeans(Zres)
            #****
            # link=logit
            if (link=="logit"){
                prec1ij <- N
                prec2ij <- 1 / omega.b^2
                prectotal <- prec1ij + prec2ij
                mij <- ( mij1 * prec1ij + mu.b * prec2ij ) / prectotal
                vij <- 1 / prectotal
                            }
            #***
            # link=normal
            if (link=="normal"){
                prec1ij <- N / sigma.res^2
                prec2ij <- 1 / omega.b^2
                prectotal <- prec1ij + prec2ij
                mij <- ( mij1 * prec1ij + mu.b * prec2ij ) / prectotal
                vij <- 1 / prectotal
                        } # end link=normal
            #*** sample b parameter
            b <- stats::rnorm( I, mean=mij, sd=sqrt( vij ) )
            return(b)
                }
####################################################################


#########################################################
# estimation of b for clusters
.mcmc.est.b.group.2pno.ml <- function( Z, aM, theta, idgroup, groupsize,
        b, G, I, sigma.b, sigma.res, link, N  ){
        #*****
        # Z=a * theta - b - bG + eps
        #=> - b=Z - a * theta + bG + eps
        bM1 <- matrix( b, nrow=N, ncol=I, byrow=TRUE )
        Zres <- Z - aM * theta + bM1
        # compute means
        mij1 <- - .mcmc.groupmean( matr=Zres, group=idgroup, groupsize )
        mij2 <- matrix( 0, G, I, byrow=TRUE )
        # compute precisions
        prec1 <- matrix( groupsize, G, I, byrow=FALSE) / 1
        if (link=="normal"){
            prec1 <- prec1 / matrix( sigma.res^2, G, I, byrow=TRUE )
                            }
        prec2 <- 1 / matrix( sigma.b^2, G, I, byrow=TRUE )
        prectot <- prec1 + prec2
        # compute total means
        mtot <- ( mij1*prec1 + mij2*prec2 ) / prectot
        vtot <- 1 / prectot
        # sampling of bG
        bG <- matrix( stats::rnorm( G*I, mean=mtot, sd=sqrt(vtot) ), G, I )
        # adjustment
#        bG1 <- rowMeans( bG )
#        bG1 <- bG1 - mean(bG1)
#        bG <- bG - bG1
#        bG1 <- colMeans( bG )
        bG <- as.matrix( scale( bG, scale=FALSE ) )
        return(bG)
            }
##################################################################
# estimation of hierarchical distribution
.mcmc.sigma.b.2pno.ml <- function( bG, mu.b, omega.b, G, I,
            est.b.Var, prior.sigma.b, sigma.b ){
        #*****
        # draw item group standard deviations
        bresG <- bG
        if ( est.b.Var=="i"){
            sig2b <- colSums( bresG^2 ) / G
            sigma.b <- sqrt( .mcmc.draw.variance( I,
                            w0=prior.sigma.b[1], sig02=prior.sigma.b[2]^2,
                            n=G, sig2=sig2b ) )
                                }
        if ( est.b.Var=="j"){
            sig2b <- sum( bresG^2 ) / (G*I)
            sigma.b <- sqrt( .mcmc.draw.variance( 1,
                            w0=prior.sigma.b[1], sig02=prior.sigma.b[2]^2,
                            n=G*I, sig2=sig2b ) )
            sigma.b <- rep( sigma.b, I )
                                }
        return(sigma.b)
            }
##################################################################
# sampling of a parameters
.draw.est.a.sl <- function( Z, bM, theta, mu.a, omega.a, I,
        sigma.res, link ){
        # Z=a * theta - b + eps
        #=> Z + b=a*theta + eps
        # a is obtained as a regression estimate
        eps <- .01
        Zres <- Z + bM
        h1 <- sum( theta^2 )
        h2 <- colSums( Zres * theta )
        # calculate means
        m1ij <- h2 / h1
        m2ij <- mu.a
        # calculate precisions
        prec1 <- h1 * 1  # (X'X)^(-1) * sigma^2_{res}
        if ( link=="normal"){
            prec1 <- prec1 / sigma.res^2
                        }
        prec2 <- 1 / omega.a^2
        prectotal <- prec1 + prec2
        # define mean and variance of posterior
        m1 <- ( m1ij * prec1 + m2ij * prec2 ) / prectotal
        # sampling of a
        a <- stats::rnorm( I, mean=m1, sd=1/sqrt(prectotal) )
#        a <- a - ( mean(a) - 1 )
        a[ a < 0 ] <- eps
        a <- exp( log(a) - mean( log( a ) ) )
#        a <- a / prod(a)
        return(a)
            }
######################################################################
# draw hyperparameters of a
.draw.est.a.hyperpars <- function( a, mu.a, omega.a, I,
    prior.omega.a, est.a.M  ){
        if ( est.a.M=="h"){
            # set mu.a to one
            mu.a <- 1
            # sample omega.a
            sig2 <- sum( ( a - mu.a )^2 ) / I
            omega.a <- sqrt( .mcmc.draw.variance( 1,
                        w0=prior.omega.a[1], sig02=prior.omega.a[2]^2,
                        n=I, sig2=sig2 ) )
                        }
        res <- list( "mu.a"=mu.a, "omega.a"=omega.a )
        return(res)
            }
###################################################################
# sampling of a parameters
.mcmc.a.est.a.2pno.ml <- function( Z, bM, aG, idgroup, theta,
                mu.a, omega.a, I, link, sigma.res ){
        # Z=a * theta + aG*theta - bM + eps
        #=> Z + bM - aG * theta=a*theta + eps
        # a is obtained as a regression estimate
        Zres <- Z + bM - aG[ idgroup, ] * theta
        h1 <- sum( theta^2 )
        h2 <- colSums( Zres * theta )
        # calculate means
        m1ij <- h2 / h1
        m2ij <- mu.a
        # calculate precisions
        prec1 <- h1 * 1  # (X'X)^(-1) * sigma^2_{res}
        if (link=="normal"){
            prec1 <- prec1 / sigma.res^2
                            }
        prec2 <- 1 / omega.a^2
        prectotal <- prec1 + prec2
        # define mean and variance of posterior
        m1 <- ( m1ij * prec1 + m2ij * prec2 ) / prectotal
        # sampling of a
        a <- stats::rnorm( I, mean=m1, sd=1/sqrt(prectotal) )
        a <- a - ( mean(a) - 1 )
        return(a)
            }
###################################################################



########################################################################
# draw a parameters group wise
.mcmc.est.aG.2pno.ml.v2 <- function( Z, bM, theta, idgroup, G, I,
            a, sigma.a, N, link, sigma.res  ){
        #****
        # Z=a * theta + aG*theta - bM + eps
        # Z + bM - a*theta=aG * theta + eps
        aM1 <- matrix( a, N, I, byrow=TRUE )
        Zres <- Z + bM    - aM1*theta
        # calculate means of a parameters
        theta2l <- rowsum( theta^2, idgroup )[,1]
        Zrestheta <- rowsum( Zres*theta, idgroup )
        m1ij <- Zrestheta / theta2l
        m2ij <- matrix( 0, G, I, byrow=TRUE )
        # calculate precisions
        prec1 <- matrix( theta2l, G, I )
        if (link=="normal"){
            prec1 <- prec1 / sigma.res^2
                            }
        # take sigma.res into account!!
        prec2 <- matrix( 1 / sigma.a^2, G, I, byrow=TRUE )
        prectotal <- prec1 + prec2
        m1 <- ( m1ij*prec1 + m2ij * prec2 ) / prectotal
        aG <- matrix( stats::rnorm(G*I, mean=m1, sd=sqrt( 1 / prectotal )), G, I )
        # center aG parameters within each group
#        aG <- aG - ( rowMeans( aG ) - 0 )
        aG <- scale( aG, scale=FALSE)
        return(aG)
            }
#########################################################
# sampling from hierarchical a distribution
.mcmc.a.grouphier.2pno.ml <- function( aG, mu.a, G, omega.a, I,
        prior.sigma.a, est.a.Var, sigma.a ){
        #***
        # draw item group standard deviations
        aresG <- aG
        if ( est.a.Var=="i"){
            sig2b <- colSums( aresG^2 ) / G
            sigma.a <- sqrt( .mcmc.draw.variance( I,
                            w0=prior.sigma.a[1], sig02=prior.sigma.a[2]^2,
                            n=G, sig2=sig2b ) )
                                }
        if ( est.a.Var=="j"){
            sig2b <- sum( aresG^2 ) / (G*I)
            sigma.a <- sqrt( .mcmc.draw.variance( 1,
                            w0=prior.sigma.a[1], sig02=prior.sigma.a[2]^2,
                            n=G*I, sig2=sig2b ) )
            sigma.a <- rep( sigma.a, I )
                                }
        return(sigma.a)
            }
####################################################################
# draw residual standard deviations
.draw.sigma.res.2pno.ml <- function( Z, aM, bM, theta, N, I ){
        # Z=a * theta - b + eps
        Zres <- Z - aM * theta + bM
        sig2 <- colSums( Zres^2 ) / N
        sigma.res <- sqrt( .mcmc.draw.variance( I,
                            w0=.001, sig02=1,
                            n=N, sig2=sig2 ) )
        return(sigma.res)
            }

## File Name: xxirt_createItemList.R
## File Version: 0.141


#--- create item list
xxirt_createItemList <- function( customItems, itemtype,items, partable )
{
    #*** list with customized items
    I <- length(items)
    item_list <- as.list( 1L:I )
    names(item_list) <- items
    CI <- length(customItems)
    for (ii in 1L:I){
        itemtype_ii <- itemtype[ii]
        item_list[[ii]] <- NA
        partable_ii <- partable[ partable$itemnr==ii, ]
        for (vv in 1L:CI){
            if ( paste(customItems[[vv]]$name)==itemtype_ii ){
                item_list[[ii]] <- customItems[[vv]]
            }  # end if
        } # end vv
        item_list[[ii]]$est <- partable_ii$est
        item_list[[ii]]$par <- partable_ii$value
        item_list[[ii]]$prior <- partable_ii$prior
        item_list[[ii]]$prior_par1 <- partable_ii$prior_par1
        item_list[[ii]]$prior_par2 <- partable_ii$prior_par2
    }  # end ii
    return(item_list)
}

## File Name: mgsem_evaluate_penalties.R
## File Version: 0.347


mgsem_evaluate_penalties <- function(x, partable, prior_list, technical,
        h, p, eps_approx, deriv=FALSE, difflp_info=NULL, loop_parms,
        pen_type="lasso", a_scad=3.7)
{
    ND <- nrow(partable)
    if (is.null(loop_parms)){
        loop_parms <- (1L:ND)[ partable$unique==1]
    }
    NP <- max(partable$index)
    if (!deriv){
        NP <- 1
    }
    pen_prior <- pen_l2 <- pen_lp <- pen_difflp <- rep(0,NP)

    res <- list( x=x, pen_prior=pen_prior, pen_l2=pen_l2,
                pen_lp=pen_lp, pen_difflp=pen_difflp, loop_parms=loop_parms,
                h=h, p=p, eps_approx=eps_approx)

    #- loop over parameters
    for (dd in loop_parms){
        index <- partable[dd,'index']
        res <- mgsem_evaluate_penalties_evaluate_entry(
                    x=x, res=res, dd=dd, index=index,
                    partable=partable, technical=technical,
                    h=h, p=p, eps_approx=eps_approx,
                    prior_list=prior_list, deriv=deriv,
                    difflp_info=difflp_info)
    } # end dd

    partable2 <- partable[ loop_parms, ]
    n <- partable2$N_group
    if (partable$ss[1]==0){
        n <- 1+0*n
    }

    #*** L2 penalty
    if (technical$is_pen_l2){
        lambda <- partable2$pen_l2
        if (deriv){
            pen_l2 <- 2*lambda*n*x
        } else {
            pen_l2 <- sum(lambda*n*x^2)
        }
        res$pen_l2 <- pen_l2
    }

    use_rcpp_penalty <- technical$use_rcpp_penalty

    #*** Lp penalty
    if (technical$is_pen_lp){
        fac <- partable2$pen_lp
        args_pen <- list(x=x, p=p, n=n, fac=fac, eps=eps_approx, deriv=deriv,
                        pen_type=pen_type, a=a_scad, h=h)

        if (!use_rcpp_penalty | (pen_type%in%c('smoothic','L0')) ){
            fun_pen <- 'mgsem_eval_lp_penalty_vector'
        } else {
            fun_pen <- 'sirt_rcpp_mgsem_eval_lp_penalty'
        }
        val <- do.call(what=fun_pen, args=args_pen)
        if (!deriv){
            val <- sum(val)
        }
        res$pen_lp <- val
    }

    #*** diffLp penalty
    if (technical$is_pen_difflp){
        indices <- difflp_info$difflp_indices
        x1 <- x[indices]
        fac <- difflp_info$lpdiff_facmat
        fac_logical <- difflp_info$lpdiff_facmat_logical
        n <- difflp_info$lpdiff_n

        if (!deriv){
            #--- no derivative
            args_pen <- list(x=x1, fac=fac, p=p, n=n, h=h, eps_approx=eps_approx,
                            a_scad=a_scad, pen_type=pen_type)
            if (! use_rcpp_penalty){
                fun_pen <- 'mgsem_eval_lp_penalty_matrix'
            } else {
                args_pen$h <- NULL
                args_pen$fac_logical <- fac_logical
                fun_pen <- 'sirt_rcpp_mgsem_eval_lpdiff_penalty'
            }
            z <- do.call( what=fun_pen, args=args_pen)

        } else {
            #--- derivative
            z <- rep(0, length(x))
            args_pen <- list(fac=fac, p=p, eps_approx=eps_approx, h=h,
                            a_scad=a_scad, pen_type=pen_type, n=n)
            if (! use_rcpp_penalty){
                args_pen$par <- x1
                args_pen$FUN <- mgsem_eval_lp_penalty_matrix
                args_pen$gradient <- TRUE
                args_pen$hessian <- FALSE
                fun_pen <- CDM::numerical_Hessian
            } else {
                args_pen$x <- x1
                args_pen$fac_logical <- fac_logical
                fun_pen <- 'sirt_rcpp_mgsem_eval_lpdiff_penalty_deriv'
            }
            val <- do.call(what=fun_pen, args=args_pen)
            z[indices] <- val
        }
        res$pen_difflp <- z
    }

    #*** penalty for diffpar
    res$pen_diffpar_lp <- 0
    if (technical$is_diffpar_pen){
        diffpar_pen_list_entries <- technical$diffpar_pen$diffpar_pen_list_entries
        NDP <- nrow(diffpar_pen_list_entries)
        p <- technical$diffpar_pen$p
        n <- partable2$N_group
        # vector of differences of parameter
        z <- x[ diffpar_pen_list_entries$index1 ] - x[ diffpar_pen_list_entries$index2 ]
        n2 <- sqrt(n[ diffpar_pen_list_entries$index1 ] *
                            n[ diffpar_pen_list_entries$index2 ] )
        args_pen <- list(x=z, fac=diffpar_pen_list_entries$W, n=n2, p=p,
                        eps_approx=eps_approx, pen_type=pen_type, h=h, deriv=deriv)
        fun_pen <- 'mgsem_eval_lp_penalty_vector'
        val <- do.call(what=fun_pen, args=args_pen)

        #* no derivative
        if (!deriv){
            val <- sum(val)
        }

        #* derivative
        if (deriv){
            der_z <- val
            NP <- length(x)
            val <- rep(0,NP)
            for (hh in 1L:NDP){
                i1 <- diffpar_pen_list_entries$index1[hh]
                val[i1] <- val[i1] + der_z[hh]
                i2 <- diffpar_pen_list_entries$index2[hh]
                val[i2] <- val[i2] - der_z[hh]
            }
        }
        res$pen_diffpar_lp <- val
    }

    #*** sum all penalties
    res$pen_all <- res$pen_prior - res$pen_l2 - res$pen_lp - res$pen_difflp -
                            res$pen_diffpar_lp

    #--- output
    return(res)
}

## File Name: attach.environment.sirt.R
## File Version: 0.081


#--- attach all elements of an object in an environment
.attach.environment.sirt <- function( res, envir )
{
    CC <- length(res)
    for (cc in 1L:CC){
        assign( names(res)[cc], res[[cc]], envir=envir )
    }
}

## File Name: noharm_sirt_optim_function.R
## File Version: 0.199


noharm_sirt_optim_function <- function(x, parm_table, parm_index, I, D,
    b0.jk, b1.jk, b2.jk, b3.jk, pm, wgtm, use_rcpp=FALSE )
{

    parm_table <- noharm_sirt_partable_include_par(par=x, parm_table=parm_table)
    Fmat <- noharm_sirt_create_parameter_matrices(mat_label='F', parm_table=parm_table,
                            parm_index=parm_index)
    Pmat <- noharm_sirt_create_parameter_matrices(mat_label='P', parm_table=parm_table,
                            parm_index=parm_index)
    Psimat <- noharm_sirt_create_parameter_matrices(mat_label='Psi',
                            parm_table=parm_table, parm_index=parm_index)

    #- implied covariance: gamma values
    gamma_val <- noharm_sirt_implied_cov(Fmat=Fmat, Pmat=Pmat, Psimat=Psimat)

    ## gamma_ii
    gamma_diag <- diag(gamma_val)
    delta <- 1 + gamma_diag

    use_rcpp <- FALSE

    #- compute least squares function
    if (!use_rcpp){
        val <- noharm_sirt_optim_function_R( gamma_val=gamma_val, delta=delta, I=I,
                    wgtm=wgtm, pm=pm, b0.jk=b0.jk, b1.jk=b1.jk, b2.jk=b2.jk, b3.jk=b3.jk)
    } else {
        val <- sirt_rcpp_noharm_sirt_optim_fn_rcpp( gamma_val=gamma_val,
                    delta=delta, I=I, wgtm=wgtm, pm=pm, b0_jk=b0.jk, b1_jk=b1.jk,
                    b2_jk=b2.jk, b3_jk=b3.jk )
    }

    #-- output
    return(val)
}

## File Name: linking_haberman_vcov_transformation.R
## File Version: 0.12

linking_haberman_vcov_transformation <- function( H1, aj_vcov )
{
    if (is.null(aj_vcov)){
        aj_se <- NA
    } else {
        aj_vcov <- H1 %*% aj_vcov %*% t(H1)
        aj_se <- c( sqrt( diag( aj_vcov ) ) )
    }
    res <- list( vcov=aj_vcov, se=aj_se )
    return(res)
}

## File Name: summary.conf.detect.R
## File Version: 0.15

#**** Summary for conf.detect object
summary.conf.detect <- function( object, digits=3, file=NULL, ...)
{
    # open sink
    sirt_osink( file=file )

    display_string <- sirt_summary_print_display(symbol="-", len=65)
    cat(display_string)

    #- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    itemcluster <- object$itemcluster
    IC <- length( unique(itemcluster) )

    des1 <- paste0( "Confirmatory DETECT Analysis with ", IC, " Item Clusters" )
    cat(des1,"\n")
    cat(paste("Bandwidth Scale:", object$bwscale, "\n" ) )

    cat(display_string)
    cat("Dimensionality Statistics \n")
    obji <- object$detect.summary
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    # close sink
    sirt_csink( file=file )
}

## File Name: mcmc_derivedPars.R
## File Version: 0.324


#--- derived parameters for objects of class mcmc
mcmc_derivedPars <- function( mcmcobj, derivedPars )
{
    mcmcobj <- mcmc_extract_samples_first_chain(mcmcobj=mcmcobj)
    NP <- length(derivedPars)
    data <- as.data.frame( mcmcobj )
    #-- renaming
    res <- mcmc_rename_define_symbols()
    orig <- res$orig
    trans <- res$trans
    colnames(data) <- mcmc_rename_parameter_names( vec=colnames(data),
                                orig=orig, trans=trans)
    for (pp in 1L:NP){
        der_pp <- mcmc_rename_parameter_names( vec=paste0(derivedPars[[pp]]),
                                orig=orig, trans=trans)
        form_pp <- mcmc_as_formula(der_pp)
        data_pp <- stats::model.matrix( form_pp, data )
        if (ncol(data_pp) > 1){
            data_pp <- data_pp[,-1]
        }
        data <- as.data.frame( cbind( data, data_pp) )
        colnames(data)[ ncol(data)] <- names(derivedPars)[pp]
    }
    colnames(data) <- mcmc_rename_undo_parameter_names( vec=colnames(data),
                                orig=orig, trans=trans)
    a1 <- attr( mcmcobj, 'mcpar')
    res <- sirt_import_coda_mcmc( data=data,  start=a1[1], end=a1[2], thin=a1[3] )
    return(res)
}

## File Name: xxirt_nr_grad_fun_pml_casewise.R
## File Version: 0.05


#-- case-wise optimization function for PML
xxirt_nr_grad_fun_pml_casewise <- function(x, args, opt_fun, h=1e-4)
{
    NP <- length(x)
    f0 <- do.call(what=opt_fun, args=args)
    N <- length(f0)
    grad <- matrix(0, nrow=N, ncol=NP)
    colnames(grad) <- names(x)
    for (pp in 1L:NP){
        args1 <- args
        args1$x <- sirt_add_increment(x=x, pos=pp, value=h)
        f1 <- do.call(what=opt_fun, args=args1)
        grad[,pp] <- (f1-f0)/h
    }
    #- output
    return(grad)
}

## File Name: sirt_rename_list_entry.R
## File Version: 0.101

sirt_rename_list_entry <- function(model, name1, name2)
{
    model_temp <- model
    G <- length(model)
    for (gg in 1L:G){
        model_gg <- model_temp[[gg]]
        TP <- length(model_gg)
        for (tt in 1L:TP){
            model_gg_tt <- model_gg[[tt]]
            ind <- which( names(model_gg_tt)==name1)
            if (length(ind)>0){
                names(model_gg_tt)[ind] <- name2
            }
            model_gg[[tt]] <- model_gg_tt
        }
        model_temp[[gg]] <- model_gg
    }
    return(model_temp)
}

## File Name: rasch.pml.R
## File Version: 2.24


#####################################################
# Pairwise marginal likelihood (PML) estimation
##NS export(rasch.pml)
rasch.pml <- function( dat, est.b=seq( 1, ncol(dat) ),
            est.a=rep( 0, ncol(dat) ),
            est.sigma=TRUE,
            itemcluster=NULL,
#            Q=NULL,
#            zero.corrs=NULL,
            weight=rep(1,nrow(dat)),
            numdiff.parm=.001, b.init=NULL,
            a.init=NULL, sigma.init=NULL,
            error.corr=0*diag( 1, ncol(dat) ),
            glob.conv=.001, conv1=.001, pmliter=100,
            progress=TRUE  ){
    ##################################
    # multidimensional version does not work
    Q <- NULL ; combs <- zero.corrs <- NULL
    ##################################
    # load libraries
    # extract information from data
    link <- "probit"
    s1 <- Sys.time()
    I <- ncol(dat)
    if( is.null( colnames(dat)) ){ colnames(dat) <- paste("I", 1:I, sep="") }
    dat <- dat[ rowMeans( is.na( dat) ) < 1, ]
    dat0 <- dat
    dat[ is.na(dat) ] <- 9
    N <- nrow(dat)
        if ( progress  ){
        cat("---------------------------------------------------------------------------------------------------------- \n")
        cat("Pairwise Marginal Likelihood Estimation \n")
        cat(paste( "Raschtype Model with", link, "link" ), "\n")
        cat("---------------------------------------------------------------------------------------------------------- \n")
        flush.console()
      }
    # extract frequencies from items
    p1 <- t(( dat==1 )) %*% ( dat==1 )
    itemfreq <- data.frame( "item"=1:I, "p1"=diag(p1)  )
    # extract frequencies from item pairs
    itempairs <- as.data.frame( t( combn(1:I, 2 ) ) )
    colnames(itempairs) <- c("item1", "item2")
    itempairs <- itempairs[ order( 1000 * itempairs[,2]  + itempairs[,1]  ), ]
    # create weight matrix
    weight <- nrow(dat) * weight / sum(weight)
#    weight <- rep(1,nrow(dat))
    WM <- sqrt( outer( weight, rep(1,I) ) )
    # p11
    p11 <- t(( dat==1 )*WM) %*% (( dat==1 )*WM )
    itempairs$f11 <- p11[ upper.tri(p11) ]
    # p10
    p10 <- t(( dat==1 )*WM) %*% (( dat==0 )*WM )
    itempairs$f10 <- p10[ upper.tri(p10) ]
    # p01
    p10 <- t(( dat==0 )*WM) %*% (( dat==1 )*WM )
    itempairs$f01 <- p10[ upper.tri(p10) ]
    # p01
    p10 <- t(( dat==0 )*WM) %*% (( dat==0 )*WM )
    itempairs$f00 <- p10[ upper.tri(p10) ]

    # error correlations
    est.eps.corr <- itempairs$est.eps.corr <- error.corr[ upper.tri( error.corr ) ]
    est.corrs <- FALSE
    if ( any( itempairs$est.eps.corr !=0 ) ){ est.corrs <- TRUE }
    if ( link=="logit" ){ est.corrs <- FALSE }
    eps.corr <- 0.2 * ( itempairs$est.eps.corr !=0 )

    #*********
    # exclude some item pairs from calculation because they are
    # located in the same itemclusters (local dependence)
    # if error calculations are estimated, then no itemclusters
    # can be selected
    if ( sum( error.corr) > 0 ){ itemcluster <- NULL }

    itemclusters <- unique( itemcluster[ itemcluster !=0 ] )
    IC <- length( itemclusters)
    for ( cc in itemclusters ){
    #    cc <- 1
        icc <- which( itemcluster==cc )
        elim <- intersect( which( itempairs[, "item1" ] %in% icc ), which( itempairs[, "item2" ] %in% icc ) )
        itempairs <- itempairs[ - elim, ]
                }
    if ( ! is.null( itemcluster ) ){
        eps.corr <- rep(0,nrow(itempairs ))
                            }
    #*******
    # evaluate pairwise likelihood
    if ( is.null(b.init)){
        b <- b0 <- - stats::qnorm( colMeans( dat0, na.rm=T) )
     if ( sum( est.b !=    seq( 1, ncol(dat) ))>0 ){
        b <- 0*b
                }
            } #    if ( link=="logit"){
#        b <- b0 <- - qlogis( colMeans( dat0, na.rm=T) )
#        p.ki <- c( .25220, .58522, .16257 )
#        S.ki <- c( .90793, .57778, .36403 )
#                }
    if ( sum(est.a) >0 ){ a <- rep(.5,I) } else { a <- rep(1,I) }
    a1s <- a1b <- a1a <- 0
    if ( ! is.null( b.init) ){ b <- b0 <- b.init }
    if ( ! is.null( a.init) ){ a <- a0 <- a.init }
    if ( is.null(sigma.init)){ sigma <- 1 } else { sigma <- sigma.init }
    #******
                D <- 1
#                }
    IP <- nrow(itempairs )
    itempairs$p1.item2 <- itempairs$p1.item1 <- rep(0,IP)
    itempairs$p11 <- rep(0,IP)
    iter <- 0
    #**********************************
    # BEGIN MARGINAL MAXIMUM LIKELIHOOD ESTIMATION
    dev <- 1 ; par.change <- dev.change <- 1000
    while ( ( dev.change > glob.conv | par.change > conv1  ) & ( iter < pmliter )    ){
        cat( paste(rep("-", 70), collapse=""), "\n")
        k1 <- floor( log10(iter+1) )
        x1 <- "        |"
        x1 <- substring( x1, k1+1 )
        s1c <- Sys.time()
        cat( paste( paste( "PML EM Iter.", iter + 1 ), x1, paste( rep( "*", 10  ), collapse=""), "|  ",
                        s1c, "  ",
                        if ( iter > 0 ){ paste( round(difftime(s1c,s1b, units='secs' ),4), "secs" ) },
                        "\n",sep="") ) #
        s1b <- Sys.time()
        h <- numdiff.parm
        dev0 <- dev
        #************************************
        # estimation of b parameters
        b0 <- b
        # identify different b parameter groups
        bG <- setdiff( unique( est.b ), 0 )
        prbar <- seq( 1, 10, len=length(bG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        cat(" Estimation of b:     |")
        for (bb in bG){
            # bb <- bG[1]
#            est.bb <- est.b * (est.b==bb )
            est.bb <- 1*(est.b==bb )
                if (bb==1 ){
                    respml0 <- .ll.rasch.pml.probit( b,a,  sigma, Q,
                            eps.corr,itempairs, IP, eps=10^(-14) )
                    ll0 <- respml0$ll
                                }
                 respml <- .update.ll.rasch.pml.probit( respml0, b0 + h*est.bb, a, sigma,
                            Q, eps.corr,itempairs, IP, eps=10^(-14) )
                 ll1 <- respml$ll
                 respml <- .update.ll.rasch.pml.probit( respml0, b0 - h*est.bb, a, sigma,
                                Q, eps.corr,itempairs, IP, eps=10^(-14) )
                 ll2 <- respml$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            if ( abs(d2) < 10^(-20) ){ d2 <- 10^20 }
            b.change <- - d1 / d2
            b.change <- ifelse( abs( b.change ) > .5, .5*sign(b.change), b.change )
            b.change <- b.change * est.bb
            b <- b + b.change
#           cat( bb, " ") ;
            cat( paste( rep( "-", prbar[match(bb,bG)] ), collapse="") )
            flush.console()
                    }
        a1b <- max( abs( b - b0 ) )
        cat("|     max. parm. change", round( a1b, 5),"\n")


        #************************************
        # estimation of a parameters
        a0 <- a
        # identify different a parameter groups
        aG <- setdiff( unique( est.a ), 0 )
        prbar <- seq( 1, 10, len=length(aG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        if ( sum(est.a) > 0 ){
            cat(" Estimation of a:     |")
            for (aa in aG){
                # bb <- bG[1]
                est.aa <- 1*(est.a==aa )
                  respml0 <- .ll.rasch.pml.probit( b, a, sigma,
                    Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll0 <- respml0$ll
                  respml0 <- .ll.rasch.pml.probit( b, a+h*est.aa, sigma,
                        Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll1 <- respml0$ll
                  respml0 <- .ll.rasch.pml.probit( b, a-h*est.aa, sigma,
                        Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll2 <- respml0$ll
                d1 <- ( ll1 - ll2  ) / ( 2 * h )
                # second order derivative
                # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
                d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
                if ( abs(d2) < 10^(-20) ){ d2 <- 10^20 }
                a.change <- - d1 / d2
                a.change <- ifelse( abs( a.change ) > .2, .2*sign(a.change), a.change )
                a.change <- a.change * est.aa
                a <- a + a.change
                a[ a < .01 ] <- .01
                cat( paste( rep( "-", prbar[match(aa,aG)] ), collapse="") )
                flush.console()
                        }
            a1a <- max( abs( a - a0 ) )
            cat("|     max. parm. change", round( a1a, 5),"\n")
                    }
        ######################################
        # estimation sigma
        sigma0 <- sigma
        cat(" Estimation of sigma: |")
        if (est.sigma & is.null(Q) ){
                  respml0 <- .ll.rasch.pml.probit( b, a, sigma,
                                Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll0 <- respml0$ll
                  respml0 <- .ll.rasch.pml.probit( b, a, sigma + h,
                                Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll1 <- respml0$ll
                  respml0 <- .ll.rasch.pml.probit( b, a, sigma - h,
                            Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll2 <- respml0$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            alpha.change <- - d1 / d2
            a1k2 <- alpha.change <- ifelse( abs( alpha.change ) > .2, .2*sign(alpha.change), alpha.change )
            sigma <- sigma + alpha.change
                                }
            prbar <- 10
            flush.console()
        if (est.sigma & (!is.null(Q)) ){
            for ( zz in seq(1,nrow(combs)) ){
                ii <- combs[zz,1]
                jj <- combs[zz,2]
                sigma.hh <- 0*sigma
                sigma.hh[ii,jj] <- sigma.hh[jj,ii] <- 1
                  respml0 <- .ll.rasch.pml.probit( b, a, sigma,
                                Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll0 <- respml0$ll
                  respml0 <- .ll.rasch.pml.probit( b, a, sigma + h*sigma.hh,
                                Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll1 <- respml0$ll
                  respml0 <- .ll.rasch.pml.probit( b, a, sigma - h*sigma.hh,
                            Q, eps.corr, itempairs, IP, eps=10^(-14) )
                  ll2 <- respml0$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            alpha.change <- - d1 / d2
            a1k2 <- alpha.change <- ifelse( abs( alpha.change ) > .12, .12*sign(alpha.change), alpha.change )
            alpha.change <- alpha.change*sigma.hh

            sigma <- sigma + alpha.change
                                    }
        diag(sigma)[ diag(sigma) < .001 ] <- .0001
                                }
        cat( paste( rep( "-", prbar), collapse="") )
        a1s <- max( abs( c( sigma - sigma0 )) )
        cat("|     max. parm. change", round( a1s, 5),"\n")


        #************************************
        # estimation of error correlation parameters
        a1e <- 0
        epsG <- NULL
        if ( est.corrs ){
            eps.corr0 <- eps.corr
            # identify different eps parameter groups
            epsG <- setdiff( unique( est.eps.corr ), 0 )
            prbar <- seq( 1, 10, len=length(epsG) )
            prbar <- floor( prbar )
            prbar <- c( prbar[1], diff(prbar) )
            cat(" Estimation of eps:   |")
            for (ee in epsG){
                # bb <- bG[1]
    #            est.bb <- est.b * (est.b==bb )
                est.ee <- 1*(est.eps.corr==ee )
                    respml0 <- .ll.rasch.pml.probit( b, a, sigma,
                                Q, eps.corr, itempairs, IP, eps=10^(-14) )
                    ll0 <- respml0$ll
                    respml <- .ll.rasch.pml.probit( b, a, sigma,
                                Q,eps.corr + h*est.ee, itempairs, IP, eps=10^(-14) )
                     ll1 <- respml$ll
                     respml <- .ll.rasch.pml.probit( b, a, sigma,
                            Q, eps.corr - h*est.ee, itempairs, IP, eps=10^(-14) )
                     ll2 <- respml$ll
                d1 <- ( ll1 - ll2  ) / ( 2 * h )
                # second order derivative
                # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
                d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
                if ( abs(d2) < 10^(-20) ){ d2 <- 10^20 }
                eps.change <- - d1 / d2
                eps.change <- ifelse( abs( eps.change ) > .35, .35*sign(eps.change), eps.change )
                eps.change <- eps.change * est.ee
                eps.corr <- eps.corr + eps.change
                eps.corr[ eps.corr < h & est.eps.corr !=0] <- 2*h
                eps.corr[ eps.corr > 1-h & est.eps.corr !=0 ] <- 1 - 2*h
                cat( paste( rep( "-", prbar[match(ee,epsG)]), collapse="") )
                flush.console()
                        }
            a1e <- max( abs( eps.corr - eps.corr0 ) )
            cat("|     max. parm. change", round( a1e, 5),"\n")
                        }
        ######################################
        # convergence display
        a1 <- stats::aggregate( b, list( est.b), mean )
        a1aa <- stats::aggregate( a, list( est.a), mean )
#        cat("   b parameters: ", paste( round( a1[,2], 3 ), collapse=" " ), "\n" )
#        cat("   a parameters: ", paste( round( a1aa[,2], 3 ), collapse=" " ), "\n" )
#        if ( D==1){
#            cat("   sigma parameter:  ", paste( round( sigma, 3 ), collapse=" " ), "\n" )
#                }
        if (D>1){
            cat("   sigma parameters:  ", paste( round( sigma[ ! lower.tri(sigma) ], 3 ), collapse=" " ), "\n" )
                    }
#        if (est.corrs){
#            a1 <- aggregate( eps.corr, list( est.eps.corr), mean )
#            cat("   eps parameters: ", paste( round( a1[,2], 3 ), collapse=" " ), "\n" )
#                    }
        #******************************************************************************
        iter <- iter + 1
        dev <- -2*ll0
        dev.change <- abs( ( dev - dev0)/ dev0 )
#        par.change <- max( a1a, a1b, a1d, a1k, a1m, a1s)
        par.change <- max( a1b, a1s, a1e, a1a )
        cat( "Pseudolikelihood Objective Function=",   round( dev, 5 ), "| max. parm. change=",
                                        round( par.change, 6 ),  " \n"   )
        if ( ( dev > dev0 ) & ( iter > 4 ) ){ cat("   Objective Function has increased! Convergence Problems?\n") }
        flush.console()
                }
        #********************************************************
    # information criteria
        # calculations for information criteria
        ic <- list( "deviance"=dev, "n"=nrow(dat0) )
        # number of parameters to be estimated
        # these formulas hold when assuming normal distributions
        ic[[ "np" ]] <- length(bG) + est.sigma + length( epsG )
        # AIC
   #     ic$AIC <- dev + 2*ic$np
        # BIC
    #    ic$BIC <- dev + ( log(ic$n) )*ic$np
        # CAIC
     #   ic$CAIC <- dev + ( log(ic$n) + 1 )*ic$np
    #**********************************************************************************
    # results item parameters
    item <- data.frame( "item"=colnames(dat0),
                "N"=colSums(!is.na(dat0)),
                "sumWeights"=colSums( ( !is.na(dat0)) * WM ),
                "p"=colMeans( dat0, na.rm=TRUE ),
                "b"=b, "est.b"=est.b,
                "a"=a, "est.a"=est.a )
    if (D==1){ item$sigma <- sigma }
    item$est.sigma=1*est.sigma
#    item$link"=link
    if ( ! is.null( itemcluster) ){ item$itemcluster <- itemcluster }
    item$b.logit <- item$b * 1.701
    item$a.logit <- item$a*1.701
    if (D==1){    item$sigma.logit <- item$sigma * 1.701 }
    # add results dependency parameter for item clusters
    #item$itemcluster <- itemcluster
    #item$delta <- 0
    cat("---------------------------------------------------------------------------------------------------------- \n")
    # print item summary
    cat("Item Parameter Summary\n")
    cat( " Estimated", length(bG), "Item Parameters\n\n")
    .pr( item, digits=3 )        # print item statistics
    cat("---------------------------------------------------------------------------------------------------------- \n")

    #....................................................................
    # print Trait parameter summary
#    cat("Trait Distribution Summary\n")
#    cat("\nCovariance Matrix (Probit Link)\n")
    cat("Trait SD (Probit Link): ")
    cat( round(sigma, 3 ), "\n")
    cat("Trait SD (Logit Link) : ")
    cat( round( item$sigma[1] * 1.701, 3 ), "\n")
    if (D>1){
        cat("\nCorrelation Matrix\n")
        print( cov2cor(sigma), digits=3 )
            }
   cat("---------------------------------------------------------------------------------------------------------- \n")
    # print correlation parameter summary
    error.corr0 <- error.corr
    if ( est.corrs){
        cat("Residual Correlation Parameter Summary\n")
        cat( " Estimated", length(epsG), "Residual Correlation Parameters\n\n")
        diag(error.corr0) <- 0
        colnames(error.corr) <- rownames(error.corr) <- colnames(dat)
        error.corr[ upper.tri( error.corr) ] <- eps.corr
        error.corr[ lower.tri( error.corr) ] <- 0
        error.corr <- error.corr + t(error.corr)
#        error.corr[ lower.tri( error.corr) ] <- eps.corr
        diag(error.corr) <- 1
       .pr( error.corr, digits=3 )        # print item statistics
       cat("---------------------------------------------------------------------------------------------------------- \n")
            }
        # computational time
        s2 <- Sys.time()
        if (progress){
                cat("---------------------------------------------------------------------------------------------------------- \n")
                cat("Start:", paste( s1), "\n")
                cat("End:", paste(s2), "\n")
                cat("Difference:", print(s2 -s1), "\n")
                cat("---------------------------------------------------------------------------------------------------------- \n")
                    }
    res <- list( "item"=item, "iter"=iter, "deviance"=dev,
                    "b"=b, "sigma"=sigma,
                    "dat"=dat    ,  "ic"=ic    ,
                    "link"=link, "itempairs"=itempairs,
                    "error.corr"=error.corr0         ,
                    "bG"=bG, "aG"=aG, "epsG"=epsG, "est.corrs"=est.corrs,
                    "Q"=Q, "D"=D)
    if (est.corrs){
            res$eps.corr <- eps.corr
            res$eps.corrM <- error.corr
                }
    class(res) <- "rasch.pml"
    return(res)
               }
##########################################################################

## File Name: lsem_fit_initial_model.R
## File Version: 0.212

lsem_fit_initial_model <- function(lavmodel__, lavaan_est_fun, dat, variables_model,
    sampling_weights, has_meanstructure, sufficient_statistics, est_joint=FALSE,
    se="standard", use_lavaan_survey=FALSE, is_imputed=FALSE, Nimp=0, ...)
{
    if (est_joint){
        has_meanstructure <- TRUE
    }
    if (sufficient_statistics) {
        #- compute sufficient statistics
        res <- lsem_fit_initial_model_sufficient_statistics(dat=dat,
                    variables_model=variables_model, sampling_weights=sampling_weights,
                    has_meanstructure=has_meanstructure,
                    is_imputed=is_imputed, Nimp=Nimp)
        wmean <- res$wmean
        wcov <- res$wcov
        Nobs <- res$Nobs
        lavfit <- lavaan_est_fun(model=lavmodel__, sample.cov=wcov,
                    sample.nobs=Nobs, sample.mean=wmean, se=se, ...)
    } else {
        if (! use_lavaan_survey){
            lavfit <- lavaan_est_fun(model=lavmodel__, data=dat, se=se, ...)
        } else {
            lavfit <- lavaan_est_fun(model=lavmodel__, data=dat, ...)
        }
    }
    return(lavfit)
}

## File Name: mcmc.2pnoh.R
## File Version: 1.155


#*** MCMC estimation 2PNO model
mcmc.2pnoh <- function(dat, itemgroups, prob.mastery=c(.5, .8 ),
        weights=NULL, burnin=500, iter=1000, N.sampvalues=1000,
        progress.iter=50, prior.variance=c(1,1), save.theta=FALSE )
{
    s1 <- Sys.time()
    # data preparation
    dat0 <- dat
    dat <- as.matrix(dat)
    dat[ is.na(dat0) ] <- 0
    dat.resp <- 1-is.na(dat0)
    N <- nrow(dat)
    I <- ncol(dat)
    eps <- 10^(-10)
    # itemgroups
    itemgroups.unique <- sort( unique( itemgroups ) )
    K <- length(itemgroups.unique)
    itemgroup <- match( itemgroups, itemgroups.unique )
    Ik <- stats::aggregate( 1 + 0*1:I, list(itemgroup), sum )[,2]
    # redefine weights
    if (! is.null(weights) ){
        weights <- N * weights / sum(weights )
                    }
    # set initial values
    a <- rep(1,I)
    b <- - stats::qnorm( (colMeans(dat0, na.rm=TRUE) + .01 )/1.02 )
    xi <- rep(0,K)
    omega <- rep(1,K)
    sig <- 1.5    # SD of item difficulties
    nu <- .30    # SD of item discriminations
    # item parameters in matrix form
    aM <- matrix( a, nrow=N, ncol=I, byrow=TRUE)
    bM <- matrix( b, nrow=N, ncol=I, byrow=TRUE)
    theta <- stats::qnorm( ( rowMeans( dat0,na.rm=TRUE ) + .01 ) / 1.02 )
    # define lower and upper thresholds
    ZZ <- 1000
    threshlow <- -ZZ + ZZ*dat
    threshlow[ is.na(dat0) ] <- -ZZ
    threshupp <- ZZ*dat
    threshupp[ is.na(dat0) ] <- ZZ
    # saved values
    SV <- min( N.sampvalues, iter - burnin )
    svindex <- round( seq( burnin, iter, len=SV )     )
    a.chain <- matrix( NA, SV, I )
    b.chain <- matrix( NA, SV, I )
    theta.chain <- matrix( NA, SV, N )
    nu.chain <- sig.chain <- deviance.chain <- rep(NA, SV)
    transition.chain <- nonmastery.chain <- mastery.chain <-
            xi.chain <- omega.chain <- matrix(NA, SV, K )
    M.beta.chain <- M.alpha.chain <- matrix(NA,SV,K)
    zz <- 0
    #**********************
    # begin iterations
    for (ii in 1:iter){

        #****
        # draw latent data Z
        Z <- .draw.Z.2pnoh( aM, bM, theta, N, I, threshlow, threshupp )

        #***
        # draw latent traits theta
        res <- .draw.theta.2pnoh( aM, bM, N, I, Z )
        theta <- res$theta

        #***
        # draw item parameters alpha, beta, xi and omega
        res <- .draw.itempars.2pnoh( theta, Z, I, N, a, b,
                xi, omega, sig, nu, itemgroup, K, Ik, weights)
        a <- res$a
        b <- res$b
        xi <- res$xi
        omega <- res$omega

        # draw item variance
        res <- .draw.itemvariances.2pnoh(a,b,I, itemgroup, xi, omega, prior.variance)
        sig <- res$sig
        nu <- res$nu

        # item parameters in matrix form
        aM <- matrix( a, nrow=N, ncol=I, byrow=TRUE)
        bM <- matrix( b, nrow=N, ncol=I, byrow=TRUE)

        # save parameters
        if ( ii %in% svindex ){
            zz <- zz+1
            a.chain[ zz, ] <- a
            b.chain[ zz, ] <- b
            xi.chain[zz,] <- xi
            omega.chain[zz,] <- omega
            theta.chain[ zz, ] <- theta
            # mean alpha and beta
#            M.alpha.chain[zz,] <- aggregate( a, list(itemgroup), mean)[,2]
            M.alpha.chain[zz,] <- rowsum( a, itemgroup )[,1] / Ik
            M.beta.chain[zz,] <- rowsum( b, itemgroup )[,1] / Ik
            # calculate deviance
            deviance.chain[zz] <- .mcmc.deviance.2pl( aM, bM, theta, dat,
                     dat.resp,     weights, eps )
            # calculate mastery probabilities
            res <- .mcmc.mastery.2pnoh( xi, omega, N, K, weights, theta, prob.mastery)
            nonmastery <- res$nonmastery
            transition <- res$transition
            mastery <- res$mastery
            nonmastery.chain[zz,] <- res$nonmastery
            transition.chain[zz,] <- res$transition
            mastery.chain[zz,] <- res$mastery
            nu.chain[zz] <- nu
            sig.chain[zz] <- sig
                        }
        # print progress
        if ( ( ii %% progress.iter )==0 ){
            cat( 'Iteration', ii, ' | ', paste(Sys.time()), '\n')
            flush.console() }

                }
    ##############################
    # output

    # Information criteria
    ic <- .mcmc.ic.2pl( a.chain, b.chain, theta.chain, N, I,
        dat, dat.resp, weights, eps,
        deviance.chain )

    # EAP reliability and person parameter estimates
    res <- .mcmc.person.2pno( theta.chain, weights )
    EAP.rel <- res$EAP.rel
    person <- res$person

    #-----
    # MCMC object
    a <- a.chain
    b <- b.chain
    d <- a*b
    theta <- theta.chain
    colnames(a) <- paste0('alpha[', 1:I, ']')
    colnames(b) <- paste0('beta[', 1:I, ']')
    colnames(d) <- paste0('d[', 1:I, ']')
    colnames(theta) <- paste0('theta[', 1:N, ']')
    mcmcobj <- cbind( deviance.chain, a, b, d )
    colnames(mcmcobj)[1] <- 'deviance'
    colnames(xi.chain) <- paste0('xi[', 1:K, ']')
    colnames(omega.chain) <- paste0('omega[', 1:K, ']')
    colnames(M.alpha.chain) <- paste0('M.alpha[', 1:K, ']')
    colnames(M.beta.chain) <- paste0('M.beta[', 1:K, ']')
    tau.chain <- omega.chain * xi.chain
    colnames(tau.chain) <- paste0('tau[', 1:K, ']')
    mcmcobj <- cbind( mcmcobj, xi.chain, omega.chain,
        M.beta.chain, M.alpha.chain, tau.chain )
    dfr <- cbind( 'sig'=sig.chain, 'nu'=nu.chain )
    mcmcobj <- cbind( mcmcobj, dfr )
    colnames(nonmastery.chain) <- paste0('nonmastery[', 1:K, ']')
    colnames(transition.chain) <- paste0('transition[', 1:K, ']')
    colnames(mastery.chain) <- paste0('mastery[', 1:K, ']')
    mcmcobj <- cbind( mcmcobj, nonmastery.chain, transition.chain, mastery.chain )
    if (save.theta){ mcmcobj <- cbind( mcmcobj, theta ) }
    class(mcmcobj) <- 'mcmc'
    attr(mcmcobj, 'mcpar') <- c( burnin+1, burnin+SV, 1 )
    mcmcobj <- sirt_import_coda_as.mcmc.list( mcmcobj )

    #----
    # summary of the MCMC output
    summary.mcmcobj <- mcmc.list.descriptives( mcmcobj )
    # number of estimated parameters
#    np <- 2*I + 2*K + 2

    s2 <- Sys.time()
    time <- list( start=s1, end=s2, timediff=s2-s1 )
    desc <- '2PNO Hierarchical IRT Model for Criterion-Referenced Measurement'
    #----
    # result list
    res <- list( mcmcobj=mcmcobj, summary.mcmcobj=summary.mcmcobj,
            ic=ic, burnin=burnin, iter=iter,
            alpha.chain=a.chain, beta.chain=b.chain,
            xi.chain=xi.chain, omega.chain=omega.chain,
            sig.chain=sig.chain, nu.chain=nu.chain,
            theta.chain=theta.chain, deviance.chain=deviance.chain,
            EAP.rel=EAP.rel, person=person, dat=dat0, weights=weights,
            time=time, model='2pnoh', description=desc)
    class(res) <- "mcmc.sirt"
    return(res)
    }

## File Name: invariance_alignment_constraints.R
## File Version: 0.251

invariance_alignment_constraints <- function(model, lambda_parm_tol, nu_parm_tol )
{
    CALL <- match.call()
    fpc_maxiter <- 20
    fpc_conv <- 1e-5

    nu <- model$nu.aligned
    lambda <- model$lambda.aligned
    wgt <- model$wgt
    miss_items <- model$miss_items

    #*** reparameterization
    lambda_list <- invariance_alignment_find_parameter_constraints(parm=lambda,
                        parm_tol=lambda_parm_tol, wgt=wgt, maxiter=fpc_maxiter,
                        conv=fpc_conv, miss_items=miss_items)
    nu_list <- invariance_alignment_find_parameter_constraints(parm=nu,
                        parm_tol=nu_parm_tol, wgt=wgt, maxiter=fpc_maxiter,
                        conv=fpc_conv, miss_items=miss_items)

    #--- output
    res <- list(model=model, nu_list=nu_list, lambda_list=lambda_list,
                    CALL=CALL)
    class(res) <- 'invariance_alignment_constraints'
    return(res)
}

## File Name: mirt.model.vars.R
## File Version: 0.24


#-- Input is a mirt syntax specified with variable names instead of numbers
mirt.model.vars <- function( mirtsyn2, data=NULL, items=NULL )
{
    TAM::require_namespace_msg("mirt")
    # mirt syntax splitted
    mirtsyn2 <- gsub( ";", "\n", mirtsyn2 )
    msyn0 <- strsplit( mirtsyn2, c(" ") )[[1]]
    syn <- msyn0
    if ( !is.null(data) ){
        items <- colnames(data)
    }
    if ( is.null(items) ){
        stop("Provide dataset or items.\n")
    }
    # admissible strings
    vecstr <- c( "\n", "\\(", "=", "-", ",", "\\)" )
    #*******
    # process syntax
    for (vv in vecstr){
    syn <- split_syn_string( syn, vv  )
    }
    # postprocess syntax
    syn <- syn[ syn !="" ]
    syn[ syn=="\\(" ] <- "("
    syn[ syn=="\\)" ] <- ")"
    # substitute variables by numbers
    VV <- length(items)
    useditems <- NULL
    for (vv in 1:VV){
        ind <- which( syn==items[vv] )
        if (length(ind) > 0 ){
            syn[ ind ] <- vv
            useditems <- c( useditems, items[vv] )
        }
    }
    syn <- paste0( syn, collapse="")
    mirtmodel <- mirt::mirt.model(syn)
    mirtmodel <- list("model"=mirtmodel, "syntax"=syn )
    return(mirtmodel)
}


#-- split_syn_string vectorized input
split_syn_string_vec <- function( syn, vecstr )
{
    for (vv in vecstr){
        syn <- split_syn_string( syn, vv  )
    }
    return(syn)
}



#-- cleans syntax in a vector from strings vv
split_syn_string <- function( syn, vv )
{
    syn <- as.list(syn )
    syn.vv <- grep( vv, syn )
    LL <- length(syn.vv)
    if (LL>0){
        for (ii in 1:LL){
            ll <- syn.vv[ii]
            syn.ll <- syn[[ll]]
            syn[[ll]] <- split_conc( syn.ll, vv )
        }
    }
    syn <- unlist(syn)
    return(syn)
}


#-- splits a string syn.ll and concatanates it with string vv
split_conc <- function( syn.ll, vv )
{
    g1 <- strsplit( syn.ll, vv, perl=FALSE )[[1]]
    Lg1 <- length(g1)
    vec <- NULL
    if (Lg1==1 ){
        vec <- c( g1, vv )
    }
    if (Lg1 > 1 ){
        vec <- rep("", Lg1 + (Lg1-1) )
        vec[ seq( 1, 2*Lg1, 2 ) ] <- g1
        vec[ seq( 2, 2*Lg1 - 1, 2 ) ] <- vv
        Ls1 <- nchar(syn.ll)
        if ( substring( syn.ll, Ls1, Ls1 )==vv ){
            vec <- c( vec, vv )
        }
    }
    return(vec)
}

## File Name: linking_haberman_bisquare_weight.R
## File Version: 0.02

linking_haberman_bisquare_weight <- function(x, cutoff)
{
    wgt_adj <- ( 1 - ( x / cutoff )^2 )^2
    wgt_adj <- (abs(x)<=cutoff)*wgt_adj
    return(wgt_adj)
}

## File Name: rm_sdt_mstep_rater_function_value.R
## File Version: 0.09


rm_sdt_mstep_rater_function_value <- function(x, par_index, partable_rater, I, K,
        nik_rater, eps=1E-10)
{
    K1 <- K + 1
    probs_dim <- c(I, K1, K1)
    probs_fun <- rm_sdt_calc_probs_grm_rcpp
    update_probs_args <- c('c.rater', 'd.rater')
    probs_args <- list( I=I, K=K, eps=eps, use_log=TRUE )
    post <- rm_sdt_mstep_type_function_value( x=x, par_index=par_index,
                partable=partable_rater, type='rater', probs_args=probs_args,
                probs_fun=probs_fun, probs_dim=probs_dim,
                update_probs_args=update_probs_args, nik=nik_rater )
    return(post)
}

## File Name: summary.linking.haebara.R
## File Version: 0.11



summary.linking.haebara <- function( object, digits=3, file=NULL, ...)
{
    # open sink for a file
    sirt_osink(file=file)

    display_string <- sirt_summary_print_display(symbol="-", len=65)
    cat(display_string)

    #- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object)

    #-- informations about optimization
    cat(display_string)
    linking_haebara_summary_optimization(object=object, digits=digits)

    cat(display_string)
    cat("Number of items per study\n")
    print(object$numb_items)

    cat(display_string)
    cat("Group Means and Standard Deviations ")
    cat(paste0("(center=", object$center,")\n"))
    obji <- object$pars
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    cat(display_string)
    cat("Joint item parameters \n")
    obji <- object$item
    sirt_summary_print_objects(obji=obji, digits=digits, from=2)

    cat(display_string)
    cat("Estimated DIF effects of item slopes \n")
    obji <- object$a.resid
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    cat(display_string)
    cat("Estimated DIF effects of item intercepts \n")
    obji <- object$b.resid
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=FALSE)

    sirt_csink(file=file)
}

## File Name: prior_extract_density.R
## File Version: 0.11


###########################################################
# extract density of all variables
prior_extract_density <- function( prior ){
    NP <- length(prior)
    dens <- rep("",NP)
    names(dens) <- names(prior)
    for (pp in 1:NP){
        # pp <- 1
        v1 <- prior[[pp]][[1]]
        v2 <- prior[[pp]][[2]]
        NV <- length(v2)
        v_pp <- paste0( v1, "(" )
        if (NV > 1){
            for (vv in 2:NV){
                # vv <- 2
                h1 <- names(v2)[vv]
                if ( ! is.null(h1) ){
                    if ( ! is.na(h1) ){
                        if ( !(  h1 %in% c("NA","") ) ){
                            v_pp <- paste0( v_pp, h1, "=" )
                        }
                    }
                }
                v_pp <- paste0( v_pp, v2[vv])
                if (vv < NV){
                    v_pp <- paste0( v_pp, "," )
                }
            }
        }
        dens[pp] <- paste0( v_pp, ")" )
    }
    return(dens)
}

## File Name: xxirt_nr_optim_fun.R
## File Version: 0.145

xxirt_nr_optim_fun <- function(x, em_args, output_all=FALSE)
{
    pen_val <- 0

    customTheta <- em_args$customTheta

    #*** compute prior distribution
    prior_Theta <- xxirt_compute_prior_Theta_from_x(x=x, em_args=em_args)

    #* compute item response probabilities
    probs_items <- xxirt_compute_prob_item_from_x(x=x, em_args=em_args)

    #*** individual likelihood
    p.xi.aj <- xxirt_compute_likelihood( probs_items=probs_items,
                                dat=em_args$dat, dat_resp_bool=em_args$dat_resp_bool )

    #*** compute likelihood function
    ll_case <- xxirt_compute_casewise_likelihood(prior_Theta=prior_Theta,
                                group=em_args$group, p.xi.aj=p.xi.aj,
                                customTheta=customTheta)
    ll0 <- ll <- -sum( em_args$weights*log(ll_case) )

    #*** add prior distributions
    x1 <- x[ em_args$parindex_items ]
    partable1 <- xxirt_partable_include_freeParameters( partable=em_args$partable, x=x1)
    par_prior <- sum( xxirt_mstep_itemParameters_evalPrior(partable1, h=0)    )
    ll <- ll+par_prior

    #*** add penalty function
    if (!is.null(em_args$penalty_fun_item)){
        pen_val <- em_args$penalty_fun_item(x=x1)
        ll <- ll + pen_val
    }

    x2 <- x[ em_args$parindex_Theta ]
    if (!is.null(em_args$penalty_fun_theta)){
        pen2 <- em_args$penalty_fun_theta(x=x2)
        ll <- ll + pen2
        pen_val <- pen_val + pen2
    }

    res <- ll
    if (output_all){
        res <- list(opt_fun=ll, pen_val=pen_val, par_prior=par_prior, ll=ll0)
    }
    return(res)
}

## File Name: lsem_fitsem_compute_sufficient_statistics.R
## File Version: 0.125

lsem_fitsem_compute_sufficient_statistics <- function(G, dat, variables_model,
    weights, moderator_variable=NULL, loc_linear_smooth=NULL, moderator.grid=NULL,
    pd=FALSE, residualized_intercepts=NULL,    has_meanstructure=FALSE,
    residualize=TRUE, is_imputed=FALSE, Nimp=0, moderator=NULL)
{
    wmean <- wcov <- Nobs <- as.list(1L:G)

    dat0 <- dat
    if (!is_imputed){
        dat0 <- list(dat0)
    }
    Nimp <- max(1, Nimp)
    Nobs_tt <- wcov_tt <- wmean_tt <- list()

    for (gg in 1L:G){

        for (ii in 1L:Nimp){
            if (!is_imputed){
                weights_gg <- weights[,gg]
            } else {
                weights_gg <- weights[[ii]][,gg, drop=TRUE]
            }
            dat <- dat0[[ii]]
            data_suff <- dat[, variables_model]
            dat_resp <- 1 - is.na(data_suff)
            if (is_imputed){
                moderator_variable <- dat[, moderator]
            }

            # res <- lsem_weighted_mean( x=data_suff, weights=weights_gg, x_resp=dat_resp)
            res <- lsem_weighted_cov( x=data_suff, weights=weights_gg, x_resp=dat_resp,
                        moderator_variable=moderator_variable,
                        loc_linear_smooth=loc_linear_smooth,
                        moderator_value=moderator.grid[gg], pd=pd,
                        residualized_intercepts=residualized_intercepts,
                        has_meanstructure=has_meanstructure, residualize=residualize)
            wmean_tt[[ii]] <- res$mean
            wcov_tt[[ii]] <- res$cov
            Nobs_tt[[ii]] <- res$Nobs
        }  # end ii

        wmean[[gg]] <- lsem_aggregate_statistics(x=wmean_tt)
        wcov[[gg]] <- lsem_aggregate_statistics(x=wcov_tt)
        Nobs[[gg]] <- round( lsem_aggregate_statistics(x=Nobs_tt) )

    }  # end gg

    #** adapt if mean structure is requested
    if ( has_meanstructure & residualize ){
        for (gg in 1L:G){
            wmean[[gg]] <- residualized_intercepts[gg,]
        }
    }

    #- output
    res <- list(wmean=wmean, wcov=wcov, Nobs=Nobs)
    return(res)
}

## File Name: weighted_colMeans.R
## File Version: 0.09

weighted_colMeans <- function( mat, wgt=NULL)
{
    wgt <- weighted_stats_extend_wgt( wgt=wgt, mat=mat )
    mat1 <- colSums( mat * wgt, na.rm=TRUE)
    mat2 <- colSums( wgt, na.rm=TRUE)
    mat1 <- mat1 / mat2
    return(mat1)
}

## File Name: gom.em.R
## File Version: 5.420


#-- gom EM algorithm
gom.em <- function( dat, K=NULL, problevels=NULL, weights=NULL, model="GOM",
    theta0.k=seq(-5,5,len=15), xsi0.k=exp(seq(-6,3,len=15)),
    max.increment=.3, numdiff.parm=1e-4, maxdevchange=1e-6,
    globconv=1e-4, maxiter=1000, msteps=4, mstepconv=.001,
    theta_adjust=FALSE, lambda.inits=NULL, lambda.index=NULL, pi.k.inits=NULL,
    newton_raphson=TRUE, optimizer="nlminb", progress=TRUE )
{
    CALL <- match.call()
    s1 <- Sys.time()
    e1 <- environment()
    s1 <- Sys.time()
    optimization <- NULL
    if ( model=="GOMRasch"){
        K <- length(theta0.k)
        max.increment <- 1
    }
    theta_grid <- NULL
    mu <- Sigma <- b <- NULL

    #-- data preparation
    res <- gom_em_prepare_data(dat=dat, weights=weights, model=model)
    dat0 <- res$dat0
    dat2 <- res$dat2
    dat2.resp <- res$dat2.resp
    dat2.ind <- res$dat2.ind
    N <- res$N
    I <- res$I
    weights <- res$weights

    #-- index table lambda matrix
    res <- gom_em_prepare_lambda_index(lambda.index=lambda.index, I=I, K=K,
                items=colnames(dat2))
    lambda.index <- res$lambda.index
    lambda_partable <- res$lambda_partable
    lambda_index_blocks <- res$lambda_index_blocks

    # design matrices
    if (model=="GOMRasch"){
        b <- - stats::qlogis( colMeans(dat, na.rm=TRUE)  )
        theta.kM <- as.matrix( expand.grid( theta0.k, xsi0.k ))
        TP <- nrow(theta.kM)
        m1 <- exp( - ( theta.kM[,1] - matrix( theta0.k, TP, K, byrow=TRUE ) )^2 / ( 2*theta.kM[,2]^2 ) )
        theta.k <- m1 / rowSums( m1 )
        #***
        mu <- c(0, .7)
        Sigma <- as.matrix(diag(c(1, 1 )))
        pi.k <- sirt_dmvnorm_discrete( x=theta.kM, mean=mu, sigma=Sigma )
        lambda <- stats::plogis( - outer( b, theta0.k, "-" ) )
        # design matrix skill space
        #        Z <- cbind( 1, theta.kM[,1], theta.kM[,1]^2, theta.kM[,1]^3,
        #                 theta.kM[,2], theta.kM[,2]^2, theta.kM[,2]^3,
        #                 theta.kM[,1]*theta.kM[,2], theta.kM[,1]^2*theta.kM[,2])
        newton_raphson <- FALSE
    }

    if (model=="GOM"){
        theta.k <- gom_em_calc_theta(K=K, problevels=problevels)
        TP <- nrow(theta.k)
        if (is.null(pi.k.inits)){
            pi.k <- as.vector(rep(1/TP, TP ))
        } else {
            pi.k <- pi.k.inits
        }
        lambda <- gom_em_inits_lambda(I=I, K=K, lambda.inits=lambda.inits,
                        lambda_partable=lambda_partable)
        theta0.k <- NULL
    }

    if (model=="GOMnormal"){
        v1 <- list(theta0.k)
        if (K>2){
            for (kk in 2:(K-1)){
                v1[[kk]] <- theta0.k
            }
        }
        theta0 <- theta0.k
        theta_grid0 <- theta_grid <- as.matrix(as.data.frame( expand.grid(v1) ))
        TP <- nrow(theta_grid)
        mu <- rep(0,K-1)
        sigma <- rep(1,K-1)
        if (is.null(pi.k.inits)){
            pi.k <- as.vector(rep(1/TP, TP ))
        } else {
            pi.k <- pi.k.inits
        }
        theta.k <- gom_em_normal_to_membership_scores(theta_grid=theta_grid, K=K, TP=TP)
        lambda <- gom_em_inits_lambda(I=I, K=K, lambda.inits=lambda.inits,
                        lambda_partable=lambda_partable)
        theta0.k <- NULL
        fac1 <- 1
    }

    # inits
    se.b <- se.lambda <- NULL
    n.ik <- array( 0, dim=c(TP,I, 2) )
    iter <- 0
    dev0 <- dev <- 0
    conv <- devchange <- 1000
    disp <- "...........................................................\n"

    #---------- start EM algorithm --------------------
    while( ( ( maxdevchange < devchange ) & (globconv < conv) ) & ( iter < maxiter ) ){
        if (progress){
            cat(disp)
            cat("Iteration", iter+1, "   ", paste( Sys.time() ), "\n" )
        }
        # previous values
        dev0 <- dev
        pi.k0 <- pi.k
        lambda0 <- lambda
        b0 <- b
# a0 <- Sys.time()
        # calculate probabilities
        res <- gom_em_calc_probs( lambda=lambda, theta.k=theta.k, b=b,
                        theta0.k=theta0.k )
        probs <- res$probs
        probs <- problong2probarray( probres=probs, I=I, TP=TP )

        # calculate counts
        probsM <- matrix( aperm( probs, c(2,1,3) ), nrow=I*2, ncol=TP )
        res1 <- sirt_rcpp_gom_em_calcpost( DAT2=dat2, DAT2RESP=dat2.resp, PROBS=probsM,
                                DAT2IND=dat2.ind, PIK=pi.k, KK1=1, weights=weights )
        f.yi.qk <- res1$fyiqk
        f.qk.yi <- res1$f.qk.yi
        pi.k <- res1$pi.k
        n.ik <- array( res1$n.ik, dim=dim(n.ik) )
        N.ik <- res1$N.ik

        ## distribution smoothing for GOMnormal
        if (model=="GOMnormal"){
            res <- stats::cov.wt(x=theta_grid, wt=pi.k, method="ML")
            mu <- res$center
            Sigma <- res$cov
            #- readjust theta_grid
            if (theta_adjust){
                fac1_old <- fac1
                mth <- max(theta0)
                sig_max <- max( sqrt(diag(Sigma)))
                fac1 <- sig_max * ( 6 / mth    )
                theta_grid <- fac1*theta_grid0
                theta.k <- gom_em_normal_to_membership_scores(theta_grid=theta_grid, K=K, TP=TP)
            }
            pi.k <- sirt_dmvnorm_discrete(x=theta_grid, mean=mu, sigma=Sigma)
        }

        # maximize lambda
        if (model %in% c("GOM","GOMnormal") ){
            res <- gom_em_est_lambda( lambda=lambda, I=I, K=K, n.ik=n.ik,
                        numdiff.parm=numdiff.parm, max.increment=max.increment,
                        theta.k=theta.k, msteps=msteps, mstepconv=mstepconv,
                        lambda_partable=lambda_partable, eps=.001, progress=progress )
            lambda <- res$lambda
            se.lambda <- res$se.lambda
            max.increment <- max( abs(lambda-lambda0))/1.2
        }
        if (model=="GOMRasch"){
            res <- gom_em_est_b( lambda=lambda, I=I, K=K, n.ik=n.ik, b=b, theta0.k=theta0.k,
                        numdiff.parm=numdiff.parm, max.increment=max.increment, theta.k=theta.k,
                        msteps=msteps, mstepconv=mstepconv, eps=.001, progress=progress )
            b <- res$b
            se.b <- res$se.b
            lambda <- t( stats::plogis( outer( theta0.k, b, "-" ) ) )
            max.increment <- max( abs(b-b0))/1.2
        }
        utils::flush.console()

        #-- calculate deviance
        ll <- gom_em_compute_total_loglikelihood(f.yi.qk=f.yi.qk, pi.k=pi.k,
                            weights=weights)
        dev <- -2*ll
        # convergence criteria
        conv <- max( abs(lambda-lambda0))
        iter <- iter+1
        devchange <- abs( ( dev - dev0 )/dev0 )

        #**** print progress
        if (progress){
            cat( paste( "   Deviance", "=", round( dev, 4 ),
                if (iter > 1 ){ " | Deviance change=" } else {""},
                if( iter>1){round( - dev + dev0, 6 )} else { ""},"\n",sep="") )
            cat( paste( "    Maximum lambda parameter change=",
                    paste( round(max(abs(lambda-lambda0)),6), collapse=" " ), "\n", sep=""))
            cat( paste( "    Maximum distribution parameter change=",
                    paste( round(max(abs(pi.k-pi.k0)),6), collapse=" " ), "\n", sep=""))
            if (model=="GOMRasch"){
                cat( paste( "    Maximum b parameter change=",
                    paste( round(max(abs(b-b0)),6), collapse=" " ), "\n", sep=""))
            }
        }
    } #--- end EM algorithm

    #--- Newton-Raphson steps
    if (newton_raphson){
        lambda_logit <- as.vector( stats::qlogis(lambda) )
        lambda_logit <- stats::aggregate(lambda_logit,
                                list(lambda_partable$par_index), mean)[,2]
        ind_lambda <- seq(1, max(lambda_partable$par_index))
        names(lambda_logit) <- lambda_partable$par_name[ lambda_partable$free ]
        pi_k_logit <- sirt_probs_to_logit(y=pi.k)
        names(pi_k_logit) <- paste0("pi_Cl", 1:(K-1) )
        ind_pi <- NULL
        ind_mu <- NULL
        ind_sigma <- NULL
        if (model=="GOM"){
            ind_pi <- max(ind_lambda) + seq(1,TP-1)
            x0 <- c(lambda_logit, pi_k_logit)
        }
        if (model=="GOMnormal"){
            K1 <- K-1
            ind_mu <- max(ind_lambda) + seq(1,K1)
            names(mu) <- paste0("mu_",1:K1)
            NC <- K1 * (K1+1) / 2
            ind_sigma <- max(ind_mu) + seq(1,NC)
            Sigma_chol <- t(chol(Sigma))
            x1 <- Sigma_chol[!upper.tri(Sigma_chol)]
            names(x1) <- paste0("sigmachol_",1:NC)
            Sigma1 <- Sigma_chol %*% t(Sigma_chol)
            x0 <- c(lambda_logit, mu, x1)
            names(x0) <- NULL
        }

        #-- optimization
        NP <- length(x0)
        bound <- 8
        lower <- rep(-bound,NP)
        upper <- rep(bound, NP)
        if (model=="GOMnormal"){
            bound2 <- 10
            lower[ c(ind_mu, ind_sigma) ] <- - bound2
            upper[ c(ind_mu, ind_sigma) ] <- bound2
        }

        if (lambda_partable$item_con[1]==0){
            grad <- gom_em_loglike_grad
        } else {
            grad <- NULL
        }
        x0 <- sirt_squeeze(x=x0, lower=lower, upper=upper)

        if (progress){
            cat("\n\n*** Switching to Newton-Raphson Optimization ***\n\n")
        }

        #- optimization
        res <- sirt_optimizer(optimizer=optimizer, par=x0, fn=gom_em_loglike_opt_fun,
                    grad=grad, control=list(trace=2), lower=lower, upper=upper,
                    ind_lambda=ind_lambda, ind_pi=ind_pi, I=I, K=K, theta.k=theta.k,
                    theta0.k=theta0.k, dat2=dat2, dat2.resp=dat2.resp, TP=TP,
                    ind_mu=ind_mu, ind_sigma=ind_sigma, theta_grid=theta_grid,
                    model=model, weights=weights, lambda_partable=lambda_partable,
                    lambda_index_blocks=lambda_index_blocks)
        optimization <- res
        par <- res$par

        #- convert parameters
        res <- gom_em_loglike_parameter_conversion(x=par, ind_lambda=ind_lambda,
                        ind_pi=ind_pi, I=I, K=K, ind_mu=ind_mu, ind_sigma=ind_sigma,
                        theta_grid=theta_grid, model=model, lambda_partable=lambda_partable)
        lambda <- res$lambda
        pi.k <- res$pi.k
        mu <- res$mu
        Sigma <- res$Sigma

        #- recalculate posterior
        res <- gom_em_calc_probs( lambda=lambda, theta.k=theta.k, b=b,
                        theta0.k=theta0.k )
        probs <- res$probs
        probs <- problong2probarray( probres=probs, I=I, TP=TP )
        probsM <- matrix( aperm( probs, c(2,1,3) ), nrow=I*2, ncol=TP )
        res1 <- sirt_rcpp_gom_em_calcpost( DAT2=dat2, DAT2RESP=dat2.resp, PROBS=probsM,
                            DAT2IND=dat2.ind, PIK=pi.k, KK1=1, weights=weights)
        f.yi.qk <- res1$fyiqk
        f.qk.yi <- res1$f.qk.yi
        n.ik <- array( res1$n.ik, dim=dim(n.ik) )
        N.ik <- res1$N.ik
        ll <- gom_em_compute_total_loglikelihood(f.yi.qk=f.yi.qk, pi.k=pi.k,
                            weights=weights)
        dev <- -2*ll
    }

    #--------------- arrange OUTPUT

    #--- Information criteria
    ic <- gom_em_ic( dev=dev, dat2=dat2, I=I, K=K, TP=TP, model=model, weights=weights )

    #--- item parameters
    item <- gom_em_item_parameters( dat2=dat2, dat2.resp=dat2.resp, model=model,
                    b=b, lambda=lambda, K=K, weights=weights, progress=progress )

    EAP.rel <- NULL
    person <- NULL
    if ( model=="GOMRasch"){
        res <- gom_em_est_covariance( f.qk.yi=f.qk.yi, Sigma=Sigma,
                    theta.kM=theta.kM, N=N )
        mu <- res$mu
        Sigma <- res$Sigma
        pi.k <- res$pi.k
        #--- distribution parameters
        c1 <- stats::cov2cor(Sigma)
        if (progress){
            cat("*********************************\n")
            cat("Trait Distribution (Location, Variability)\n")
            cat( " Means: ", round( mu, 3 ), "\n")
            cat( " Standard deviations: ", round( sqrt(diag(Sigma)), 3 ), "\n")
            cat( " Correlation ", round( c1[lower.tri(c1)], 3 ), "\n")
            flush.console()
        }
        # person parameters
        pers <- .smirt.person.parameters( data=dat2, D=2, theta.k=theta.kM,
            p.xi.aj=f.yi.qk, p.aj.xi=f.qk.yi, weights=rep(1,N) )
        person <- pers$person
        EAP.rel <- pers$EAP.rel
        if (progress){
            cat("*********************************\n")
            cat("EAP Reliability=", round(EAP.rel,3), "\n")
        }
    }

    #--- MAP
    MAP <- sirt_MAP(post=f.qk.yi, theta=theta.k)

    #--- EAP
    EAP <- sirt_EAP(post=f.qk.yi, theta=theta.k)
    colnames(EAP) <- paste0("Class",1:K)

    #--- descriptives of classes
    score <- rowSums( weights*dat2 *dat2.resp ) / rowSums( weights*dat2.resp )
    plmat <- FALSE
    if ( is.vector(problevels) ){
        PL <- length(problevels)
    }
    if ( is.matrix(problevels) ){
        PL <- nrow(problevels)
        plmat <- TRUE
    }
    theta.kk0 <- theta.k
    if ( model=="GOMRasch"){
        PL <- 5
        problevels <- seq(0,1,len=PL)
        problevels2 <- c( problevels - diff(problevels)[1]/2, 1.2 )
        for (kk in 1:K){
            for (ll in 1:PL){
                ind.kk <- which( ( theta.k[, kk ] > problevels2[ll] ) &
                    ( theta.k[, kk ] <=problevels2[ll+1] ) )
                theta.kk0[ ind.kk, kk ] <- problevels[ll]
            }
        }
        pi.k <- pi.k[,1]
    }  # end GOMRasch

    classdesc <- NULL
    if ( model=="GOMnormal"){
        plmat <- TRUE
    }
    if ( ! plmat ){
        classdesc <- data.frame( matrix( 0, 2*PL+1, K ) )
        classdesc[1,] <- colSums( theta.kk0 * pi.k )
        for (kk in 1:K){
            for (ll in 1:PL){
                ll1 <- problevels[ll]
                classdesc[ ll+1, kk ] <- sum( pi.k * ( theta.kk0[,kk]==ll1 ) )
            }
        }
        rownames(classdesc)[1] <- "p.Class"
        colnames(classdesc) <- paste0("Class", 1:K )
        rownames(classdesc)[2:(PL+1)] <- paste0( "p.problevel", round( problevels, 3 ), ".class" )
        rownames(classdesc)[PL+2:(PL+1)] <- paste0( "M.problevel", round( problevels, 3 ), ".class" )
        for (kk in 1:K){
            ll.kk <- problevels
            for (ll in 1:PL){
                ind.ll <- which( theta.kk0[,kk]==problevels[ll] )
                rll.wt <- rowSums( f.qk.yi[, ind.ll, drop=FALSE] )
                ll.kk[ll] <- weighted.mean( score, rll.wt )
            }
            classdesc[ PL + 2:(PL+1), kk ]  <- ll.kk
        }
    }

    #--- output
    s2 <- Sys.time()
    res <- list(deviance=dev, ic=ic, item=item, person=person, EAP.rel=EAP.rel,
                MAP=MAP, EAP=EAP, classdesc=classdesc, lambda=lambda, se.lambda=se.lambda,
                mu=mu, Sigma=Sigma, b=b, se.b=se.b, f.yi.qk=f.yi.qk, f.qk.yi=f.qk.yi,
                probs=probs, n.ik=n.ik, iter=iter, dat=dat0, dat2=dat2, dat2.resp=dat2.resp,
                I=I, K=K, TP=TP, G=1, theta.k=theta.k, pi.k=pi.k, problevels=problevels,
                model=model, plmat=plmat, mu=mu, Sigma=Sigma, weights=weights,
                lambda.index=lambda.index, lambda_partable=lambda_partable,
                optimization=optimization, newton_raphson=newton_raphson,
                s1=s1, s2=s2, time_diff=s2-s1, CALL=CALL)
    class(res) <- "gom"
    return(res)
}


# a1 <- Sys.time(); cat("probs \n"); print(a1-a0); a0 <- a1

## File Name: mgsem_add_list_entries.R
## File Version: 0.06


mgsem_add_list_entries <- function(list1, add_list, output_list, elements=NULL)
{
    if (is.null(elements)){
        elements <- names(add_list)
    }
    for (nn in elements){
        output_list[[nn]] <- list1[[nn]] + add_list[[nn]]
    }
    return(output_list)
}

## File Name: rasch.copula3_aux.R
## File Version: 6.19

#----------------------------------------------------------------------------------------------
.ll.rasch.copula320 <- function( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
        bdat2.li, bdat2.li.resp, pattern, GG, copula.type, Ncat.ld,
        Qmatrix=Qmatrix, ... ){
        #-------------------------------------------------------
        # use this function for log likelihood calculation
        # calculation of terms of independent itemclusters?
        calc.ind <- length(itemcluster0) > 0
        G <- GG            # number of groups
        eps1 <- 10^(-14)
        ndat2 <- nrow(dat2.ld)
        M1 <- rep(1,ndat2)
        ntheta <- nrow(theta.k)
        M2 <- rep( 1, ntheta)
         pjk.theta.k <- .prob.raschtype.genlogis( theta.k, b, alpha1, alpha2, a,
                Qmatrix=Qmatrix )
        pjk.theta.k01 <- cbind( pjk.theta.k, 1 - pjk.theta.k, 1  )

        ################################################
        # E step
        ################################################
        #.............................................
        # probabilities for indepedent items
        if ( calc.ind ){
            pjk.theta.k0 <- pjk.theta.k01[, c( itemcluster0, itemcluster0 + I ) ]
                                } else  {
            pjk.theta.k0 <- NULL
                                }
        # probabilities for dependent items
        pjk.theta.kCC <- as.list( 1:CC )

        for (cc in 1:CC){
            # cc <- 2    # itemcluster cc
            dp.ld.cc <- dp.ld[[cc]]
            m1.cc <- pjk.theta.k01[, dp.ld.cc$independent$items ]
            v1.cc <- dp.ld.cc$independent$N.Index1
            #--------------------------------------------
            # Boundary Mixture Copula (Braeken, 2011)
            if (copula.type[cc]=="bound.mixt" ){
                # likelihood under independence
                F0pjk.cc <- .rowProds2.bundle( m1=m1.cc, v1=v1.cc)
                # likelihood under dependence
                m1.cc <- pjk.theta.k01[, dp.ld.cc$dependent$items ]
                v1.cc <- dp.ld.cc$dependent$N.Index1
                pjk.cc <- .rowMins2cpp.bundle( m1=m1.cc, v1=v1.cc)
#                F1pjk.cc <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                F1pjk.cc <- tcrossprod(  pjk.cc,  dp.ld.cc$calc  )
                pjk.theta.kCC[[cc]] <- ( 1 - delta[cc] ) * F0pjk.cc + delta[cc] * F1pjk.cc
                            }
            #-----------------------------------------------
            # include other Copula models here
            # Cook-Johnson copula (Braeken et al., 2007)
            if (copula.type[cc]=="cook.johnson" ){
                F.Xr <-  1- pjk.theta.k[, dp.ld.cc$items ]
                R <- ncol(F.Xr)
                delta.cc <- delta[cc]
                patt.cc <- dp.ld.cc$patt
                pjk.cc  <- matrix( 0, nrow=ntheta, ncol=nrow(patt.cc) )
                for (pp in 1:( nrow(patt.cc) ) ){
                    ppcc <- 1 - patt.cc[pp,]
                    pjk.cc[, pp ] <- ( rowSums( ( F.Xr^(-delta.cc))^( outer( rep(1,ntheta), ppcc )) )
                                                    - R + 1 )^(-1/delta.cc)
                                }
#                    pjk.theta.kCC[[cc]] <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
#                    temp1 <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                    temp1 <- tcrossprod( pjk.cc, dp.ld.cc$calc )
                    temp1[ temp1 < 0 ] <- eps1
                    pjk.theta.kCC[[cc]]    <- temp1
                            }
            #******************************************
            # Frank copula
            if (copula.type[cc]=="frank" ){
                F.Xr <-  1- pjk.theta.k[, dp.ld.cc$items ]
                R <- ncol(F.Xr)
                delta.cc <- delta[cc]
                prod.delta <- ( 1 - exp( - delta.cc ) )^(R-1)
                patt.cc <- dp.ld.cc$patt
                pjk.cc  <- matrix( 0, nrow=ntheta, ncol=nrow(patt.cc) )
                for (pp in 1:( nrow(patt.cc) ) ){
                    ppcc <- 1 - patt.cc[pp,]
                    g1 <- rowProds2( ( 1 - exp( - delta.cc * F.Xr^( outer( rep(1,ntheta), ppcc ))  ) ) )
                    pjk.cc[,pp] <- - log( 1 - g1 / prod.delta ) / delta.cc
                                }
                    temp1 <- tcrossprod( pjk.cc, dp.ld.cc$calc )
                    temp1[ temp1 < 0 ] <- eps1
                    pjk.theta.kCC[[cc]]    <- temp1
                            }  # end Frank copula
                        }

#print( str(pjk.theta.kCC))
# cat( "\n probabilities \n" ) ; aa1 <- Sys.time() ; print(aa1-aa0) ; aa0 <- aa1

        #****
        post0 <- matrix( 1, nrow=ndat2, ncol=ntheta )
        I0 <- length(itemcluster0)
        if ( calc.ind ){
            pjkL <- array( NA, dim=c(2, nrow(theta.k), I0 ) )
            # pjkL: [ #categories, #thetagrid, #items ]
            pjkL[1,,] <- 1 - pjk.theta.k0[,1:I0]
            pjkL[2,,] <- pjk.theta.k0[,1:I0]
            for (ii in 1:I0 ){
                ind.ii <- which( bdat2.li.resp[,ii]==1 )
                post0[ind.ii,] <- post0[ind.ii,] * pjkL[ bdat2.li[ind.ii,ii]+1,,ii]
                        }
                        }

        #****
        post2 <- matrix( 1, nrow=ndat2, ncol=ntheta )
        pjkL <- array( NA, dim=c(Ncat.ld, nrow(theta.k), CC ) )
        for (cc in 1:CC){
            #    cc <- 1
            p1.cc <- t( pjk.theta.kCC[[cc]] )
            pjkL[ seq( 1, nrow(p1.cc) ),, cc ] <- p1.cc
                        }
        for (cc in 1:CC ){
            ind.ii <- which( dat2.ld.resp[,cc]==1 )
            post2[ind.ii,] <- post2[ind.ii,] * pjkL[ dat2.ld[ind.ii,cc],,cc]
                    }


        post <- post0 * post2        # product of independent and dependent parts
        post.unnorm <-  post
        post <- post * outer( M1, wgt.theta )
        post <- post / rowSums( post)    # standardization of posterior distribution

# cat( "\n posterior\n" ) ; aa1 <- Sys.time() ; print(aa1-aa0) ; aa0 <- aa1

        #....................................................
        # Calculate expected counts
        # expected counts independent item responses
        njk0 <- rjk0.0 <- rjk0.1 <- NULL
        # pattern[,gg+1] is the frequency weight of a response pattern in group gg
        gg <- 1
        if ( calc.ind ){
            njk0 <- crossprod( post, pattern[,gg+1] * dat2.li.resp )
            rjk0.1 <- crossprod( post,  pattern[,gg+1] * dat2.li.resp * dat2.li )
            rjk0.0 <- njk0 - rjk0.1
                                    }

        #*****************
        # expected counts dependent items
        rjkCC <- as.list( 1:CC )
        gg <- 1
        for ( cc in 1:CC){
            rjkCC[[cc]] <- crossprod( post,  pattern[,gg+1] * dat3.ld[[cc]] * dat2.ld.resp[,cc] )
                    }

        # total counts
        gg <- 1
        tc <- outer( pattern[,gg+1], rep( 1, ntheta ) )
        nk <- colSums( tc * post)
        nk <- nk + 1e-30
        pik <- nk / sum(nk)

# cat( "\n expected counts \n" ) ; aa1 <- Sys.time() ; print(aa1-aa0) ; aa0 <- aa1

        #---------------------------------------------------------------
        # calculate log likelihood
        #.......................
        # one group

        if (G==1){
             ll0 <- sum( nk * log(pik) )
#             ll0 <- sum( nk * log(wgt.theta) )
             # likelihood part from independent items
             if ( calc.ind ){
                if ( nrow( rjk0.1)==1 ){
                    rjk.temp <- cbind( t( rjk0.1), t(rjk0.0) )
                            } else {
                    rjk.temp <- cbind( rjk0.1, rjk0.0 )
                                }
                    ll0 <- ll0 + sum( log(pjk.theta.k0 ) * rjk.temp )
                            }
             # likelihood part from dependent items
            for (cc in 1:CC){
                ll0 <- ll0 + sum( rjkCC[[cc]] * log( pjk.theta.kCC[[cc]] + 10^(-15) ) )
                                }

                    }
# cat( "\n Likelihood \n" ) ; aa1 <- Sys.time() ; print(aa1-aa0) ; aa0 <- aa1

        #.......................
        # multiple groups
        #    ... to do    ...



            lli <- ll0
        ################################
        # arrange output
        res <- list( "ll"=ll0,  "b"=b, "a"=a, "delta"=delta,
                        "alpha1"=alpha1, "alpha2"=alpha2, "lli"=lli,
                        "post"=post, "post.unnorm"=post.unnorm,
                        "rjk0.1"=rjk0.1,
                        "rjk0.0"=rjk0.0, "rjkCC"=rjkCC,
                        "pjk.theta.kCC"=pjk.theta.kCC,
                        "pjk.theta.k0"=pjk.theta.k0,
                        "nk"=nk, "pik"=pik,"calc.ind"=calc.ind
                                                    )
        return(res)
                }
#----------------------------------------------------------------------------------------------




#----------------------------------------------------------------------------------------------
.update.ll.rasch.copula321 <- function( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
        bdat2.li, bdat2.li.resp, rescopula, itemcluster, pattern, GG, copula.type,
        Qmatrix=Qmatrix, ... ){
        #-------------------------------------------------------
        # use this function for log likelihood calculation
        # look for items which change parameters for necessary update
        G <- GG
        # calculation of terms of independent itemclusters?
        calc.ind <- length(itemcluster0) > 0
        eps1 <- 10^(-14)
        # calculate necessary updates
        ind.b <- which( b !=rescopula$b )
        ind.a <- which( a !=rescopula$a )
        ind.delta <- which( delta !=rescopula$delta )
        ind.alpha1 <- ( alpha1 !=rescopula$alpha1 )    + ( alpha2 !=rescopula$alpha2 )
        if (ind.alpha1 > 0){ ind.alpha <- seq(1, ncol(dat2.ld) ) } else { ind.alpha <- NULL }
        itemset <- union( ind.b, ind.a )
        itemset <- union( itemset, ind.alpha )
        # update term local independence
        li.update <- 1 * ( sum( itemcluster0 %in% itemset ) > 0 )
        # update terms item dependence parameters
        ld.update <- sapply( 1:CC, FUN=function(cc){
                g1 <- intersect( which( itemcluster==cc ), itemset )
                if ( length(g1)){ v1 <- cc } else { v1 <- NULL }
                v1
                    } )
        ld.update <- unique( union( ind.delta, unlist( ld.update) ) )
        ###########################################################################
        ndat2 <- nrow(dat2.ld)
        M1 <- rep(1,ndat2)
        ntheta <- nrow(theta.k)
        M2 <- rep( 1, ntheta)
        pjk.theta.k <- .prob.raschtype.genlogis( theta.k, b, alpha1, alpha2, a,
            Qmatrix=Qmatrix)
        pjk.theta.k01 <- cbind( pjk.theta.k, 1 - pjk.theta.k, 1  )
        #.............................................
        # probabilities for indepedent items
        if ( calc.ind ){
            pjk.theta.k0 <- pjk.theta.k01[, c( itemcluster0, itemcluster0 + I ) ]
                                } else  {
            pjk.theta.k0 <- NULL
                                }
        # probabilities for dependent items
        pjk.theta.kCC <- rescopula$pjk.theta.kCC
        for (cc in ld.update){
            # cc <- 2    # itemcluster cc
            dp.ld.cc <- dp.ld[[cc]]
            m1.cc <- pjk.theta.k01[, dp.ld.cc$independent$items ]
            v1.cc <- dp.ld.cc$independent$N.Index1
            #--------------------------------------------
            # Boundary Mixture Copula (Braeken, 2011)
            if (copula.type[cc]=="bound.mixt" ){
                # likelihood under independence
                F0pjk.cc <- .rowProds2.bundle( m1=m1.cc, v1=v1.cc)
                # likelihood under dependence
                m1.cc <- pjk.theta.k01[, dp.ld.cc$dependent$items ]
                v1.cc <- dp.ld.cc$dependent$N.Index1
                pjk.cc <- .rowMins2cpp.bundle( m1=m1.cc, v1=v1.cc)
#                F1pjk.cc <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                # t( A * t(B) )=B * t(A)
                F1pjk.cc <- tcrossprod(  pjk.cc,  dp.ld.cc$calc  )
                pjk.theta.kCC[[cc]] <- ( 1 - delta[cc] ) * F0pjk.cc + delta[cc] * F1pjk.cc
                                        }
            #-----------------------------------------------
            # Cook-Johnson Copula
            if (copula.type[cc]=="cook.johnson" ){
                F.Xr <-  1- pjk.theta.k[, dp.ld.cc$items ]
                R <- ncol(F.Xr)
                delta.cc <- delta[cc]
                patt.cc <- dp.ld.cc$patt
                pjk.cc  <- matrix( 0, nrow=ntheta, ncol=nrow(patt.cc) )
                for (pp in 1:( nrow(patt.cc) ) ){
                    ppcc <- 1 - patt.cc[pp,]
                    pjk.cc[, pp ] <- ( rowSums( ( F.Xr^(-delta.cc))^( outer( rep(1,ntheta), ppcc )) )
                                                    - R + 1 )^(-1/delta.cc)
        #            temp1 <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                    temp1 <- tcrossprod( pjk.cc, dp.ld.cc$calc )
                    temp1[ temp1 < 0 ] <- eps1
                    pjk.theta.kCC[[cc]]    <- temp1

                                }
                            }
            #******************************************
            # Frank copula
            if (copula.type[cc]=="frank" ){
                F.Xr <-  1- pjk.theta.k[, dp.ld.cc$items ]
                R <- ncol(F.Xr)
                delta.cc <- delta[cc]
                prod.delta <- ( 1 - exp( - delta.cc ) )^(R-1)
                patt.cc <- dp.ld.cc$patt
                pjk.cc  <- matrix( 0, nrow=ntheta, ncol=nrow(patt.cc) )
                for (pp in 1:( nrow(patt.cc) ) ){
                    ppcc <- 1 - patt.cc[pp,]
                    g1 <- rowProds2( ( 1 - exp( - delta.cc * F.Xr^( outer( rep(1,ntheta), ppcc ))  ) ) )
                    pjk.cc[,pp] <- - log( 1 - g1 / prod.delta ) / delta.cc
                                }
#                    temp1 <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                    temp1 <- tcrossprod( pjk.cc, dp.ld.cc$calc )
                    temp1[ temp1 < 0 ] <- eps1
                    pjk.theta.kCC[[cc]]    <- temp1
                            } # end Frank copula
                        }
        #---------------------------------------------------------------


        #####################################
        # rearrange output
                res <- list( "pjk.theta.kCC"=pjk.theta.kCC, "pjk.theta.k0"=pjk.theta.k0  )
                return(res)
                }
#----------------------------------------------------------------------------------------------




#----------------------------------------------------------------------------------------------
.update.ll.rasch.copula320 <- function( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
        bdat2.li, bdat2.li.resp, rescopula, itemcluster, pattern, GG, copula.type,
        Qmatrix=Qmatrix, ... ){
        #-------------------------------------------------------
        # use this function for log likelihood calculation
        # look for items which change parameters for necessary update
        G <- GG
        # calculation of terms of independent itemclusters?
        calc.ind <- length(itemcluster0) > 0
        eps1 <- 10^(-14)
        # calculate necessary updates
        ind.b <- which( b !=rescopula$b )
        ind.a <- which( a !=rescopula$a )
        ind.delta <- which( delta !=rescopula$delta )
        ind.alpha1 <- ( alpha1 !=rescopula$alpha1 )    + ( alpha2 !=rescopula$alpha2 )
        if (ind.alpha1 > 0){ ind.alpha <- seq(1, ncol(dat2.ld) ) } else { ind.alpha <- NULL }
        itemset <- union( ind.b, ind.a )
        itemset <- union( itemset, ind.alpha )
        # update term local independence
        li.update <- 1 * ( sum( itemcluster0 %in% itemset ) > 0 )
        # update terms item dependence parameters
        ld.update <- sapply( 1:CC, FUN=function(cc){
                g1 <- intersect( which( itemcluster==cc ), itemset )
                if ( length(g1)){ v1 <- cc } else { v1 <- NULL }
                v1
                    } )
        ld.update <- unique( union( ind.delta, unlist( ld.update) ) )
        ###########################################################################
        ndat2 <- nrow(dat2.ld)
        M1 <- rep(1,ndat2)
        ntheta <- nrow(theta.k)
        M2 <- rep( 1, ntheta)
        pjk.theta.k <- .prob.raschtype.genlogis( theta.k, b, alpha1, alpha2, a, Qmatrix=Qmatrix)
        pjk.theta.k01 <- cbind( pjk.theta.k, 1 - pjk.theta.k, 1  )
        #.............................................
        # probabilities for indepedent items
        if ( calc.ind ){
            pjk.theta.k0 <- pjk.theta.k01[, c( itemcluster0, itemcluster0 + I ) ]
                                } else  {
            pjk.theta.k0 <- NULL
                                }
        # probabilities for dependent items
        pjk.theta.kCC <- rescopula$pjk.theta.kCC
        for (cc in ld.update){
            # cc <- 2    # itemcluster cc
            dp.ld.cc <- dp.ld[[cc]]
            m1.cc <- pjk.theta.k01[, dp.ld.cc$independent$items ]
            v1.cc <- dp.ld.cc$independent$N.Index1
            #--------------------------------------------
            # Boundary Mixture Copula (Braeken, 2011)
            if (copula.type[cc]=="bound.mixt" ){
                # likelihood under independence
                F0pjk.cc <- .rowProds2.bundle( m1=m1.cc, v1=v1.cc)
                # likelihood under dependence
                m1.cc <- pjk.theta.k01[, dp.ld.cc$dependent$items ]
                v1.cc <- dp.ld.cc$dependent$N.Index1
                pjk.cc <- .rowMins2cpp.bundle( m1=m1.cc, v1=v1.cc)
#                F1pjk.cc <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                F1pjk.cc <- tcrossprod(  pjk.cc,  dp.ld.cc$calc  )
                pjk.theta.kCC[[cc]] <- ( 1 - delta[cc] ) * F0pjk.cc + delta[cc] * F1pjk.cc
                                        }
            #-----------------------------------------------
            # Cook-Johnson Copula
            if (copula.type[cc]=="cook.johnson" ){
                F.Xr <-  1- pjk.theta.k[, dp.ld.cc$items ]
                R <- ncol(F.Xr)
                delta.cc <- delta[cc]
                patt.cc <- dp.ld.cc$patt
                pjk.cc  <- matrix( 0, nrow=ntheta, ncol=nrow(patt.cc) )
                for (pp in 1:( nrow(patt.cc) ) ){
                    ppcc <- 1 - patt.cc[pp,]
                    pjk.cc[, pp ] <- ( rowSums( ( F.Xr^(-delta.cc))^( outer( rep(1,ntheta), ppcc )) )
                                                    - R + 1 )^(-1/delta.cc)
#                    temp1 <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                    temp1 <- tcrossprod( pjk.cc, dp.ld.cc$calc )
                    temp1[ temp1 < 0 ] <- eps1
                    pjk.theta.kCC[[cc]]    <- temp1

                                }
                            }
            #******************************************
            # Frank copula
            if (copula.type[cc]=="frank" ){
                F.Xr <-  1- pjk.theta.k[, dp.ld.cc$items ]
                R <- ncol(F.Xr)
                delta.cc <- delta[cc]
                prod.delta <- ( 1 - exp( - delta.cc ) )^(R-1)
                patt.cc <- dp.ld.cc$patt
                pjk.cc  <- matrix( 0, nrow=ntheta, ncol=nrow(patt.cc) )
                for (pp in 1:( nrow(patt.cc) ) ){
                    ppcc <- 1 - patt.cc[pp,]
                    g1 <- rowProds2( ( 1 - exp( - delta.cc * F.Xr^( outer( rep(1,ntheta), ppcc ))  ) ) )
                    pjk.cc[,pp] <- - log( 1 - g1 / prod.delta ) / delta.cc
                                }
#                    temp1 <- t( dp.ld.cc$calc %*% t( pjk.cc ) )
                    temp1 <- tcrossprod( pjk.cc, dp.ld.cc$calc )
                    temp1[ temp1 < 0 ] <- eps1
                    pjk.theta.kCC[[cc]]    <- temp1
                            } # end Frank copula
                        }
        #---------------------------------------------------------------
        # calculate log likelihood
        rjk0.1 <- rescopula$rjk0.1
        rjk0.0 <- rescopula$rjk0.0
        rjkCC <- rescopula$rjkCC
        #.......................
        # one group
        if (G==1){
            ll0 <-     sum( rescopula$nk * log(rescopula$pik) )
             # likelihood part from independent items
             if ( calc.ind ){
                if ( nrow( rjk0.1)==1 ){
                    rjk.temp <- cbind( t( rjk0.1), t(rjk0.0) )
                            } else {
                    rjk.temp <- cbind( rjk0.1, rjk0.0 )
                                }
                    ll0 <- ll0 + sum( log(pjk.theta.k0) * rjk.temp )
                            }
             # likelihood part from dependent items
            for (cc in 1:CC){
                ll0 <- ll0 + sum( rjkCC[[cc]] * log( pjk.theta.kCC[[cc]] + 10^(-15) ) )
                            }

                                }
#                    }
        #.......................
        # multiple groups
        #    ... to do    ...
            lli <- ll0

        #####################################
        # rearrange output
                res <- list( "ll"=ll0, "lli"=lli )
                return(res)
                }
#----------------------------------------------------------------------------------------------

## File Name: rm_facets_print_progress_trait_distribution.R
## File Version: 0.08


rm_facets_print_progress_trait_distribution <- function( parm, parmlabel, digits_trait )
{
    cat( paste( " ", parmlabel, "=", round( parm, digits_trait ), sep=""), "\n")
}

## File Name: rm_sdt_create_partable_include_fixed_item_category_parameters.R
## File Version: 0.13

rm_sdt_create_partable_include_fixed_item_category_parameters <- function(
    partable, tif, index_no_est_value )
{
    if ( ! is.null(tif) ){
        NT <- nrow(tif)
        for (tt in seq_len(NT) ){
            ind_tt <- which( ( partable$row==tif[tt,"item"] ) &
                                ( partable$col==tif[tt,"categ"] ) )
            partable[ ind_tt, "est"] <- FALSE
            partable[ ind_tt, "fixed"] <- TRUE
            partable[ ind_tt, "value"] <- tif[tt,"val"]
        }
        par_est <- unique( partable$parindex[ partable$est ] )
        partable$parindex <- match( partable$parindex, par_est )
        partable[ partable$fixed, "parindex" ] <- index_no_est_value
    }
    return(partable)
}

## File Name: likelihood_moments.R
## File Version: 0.121

##****
likelihood_moments <- function( likelihood, theta=NULL )
{
    if ( is.null(theta) ){
        theta <- attr( likelihood, 'theta' )
    }
    nstud <- nrow(likelihood)
    TP <- ncol(likelihood)
    thetaM <- matrix( theta, nstud, TP, byrow=TRUE )
    likelihood <- likelihood / rowSums( likelihood )
    M1 <- rowSums( thetaM * likelihood )
    SD1 <- rowSums( thetaM^2 * likelihood )
    SD1 <- sqrt( SD1 - M1^2 )
    res <- list( M=M1, SD=SD1 )
    return(res)
}

## File Name: personfit.R
## File Version: 1.29



#----------------------------------------------------------------------------------
# person fit with logistic regression
pf.logist.regression <- function( data, itemdiff, perc=seq(5,100,5) ){
    P <- nrow( data )
    fits <- rep(0,P)
    convdis <- .display.conv( maxiter=P, perc=perc, P=P )
    for (pp in 1:P){
        # pp <- 40
        y1 <-  t(data[pp,])[,1]
        mod.pp <- stats::glm( y1 ~ itemdiff, family="binomial")
        fits[pp] <- stats::coef(mod.pp)[2]
        ind.pp <- which( convdis[,1] %in% pp )
        if (length(ind.pp) > 0){ cat("\n",paste(convdis[ind.pp,2], "%", sep="") ) ;
        utils::flush.console()}
            }
        cat("\n")
    return( fits)
    }
#...........................................................................
# display convergence progress
.display.conv <- function( maxiter, perc=seq( 5, 100, 5 ), P=P ){
    h1 <- 100 * seq( 1, P ) / P
    h2 <- sapply( perc, FUN=function(perc.pp){
            # perc.pp <- perc[1]
            which.min( abs( h1 - perc.pp ) )[1]
            } )
    cbind( "iter"=h2, "perc"=perc )
        }
#-----------------------------------------------------------------------------------------
# Reise fit
##NS export(pf.reisefit)
# pf.reisefit <- function( data, itemdiff ){
#    dat1 <- data
#    a1 <- data.frame( "y"=matrix(  t(dat1), ncol=1, byrow=T),
#                    "person"=rep(  seq( 1, nrow(dat1) ), each=I ),
#                    "itemdiff"=rep( itemdiff, nrow(dat1) )
#                        )
#    mod <- lmer( y ~ itemdiff + ( 1 + itemdiff | person ), data=a1, family="binomial" )
#    l1 <- list( "reise.fit"=fixef(mod)[2] + ranef(mod)$person[,2], "summary"=summary(mod), "model"=mod )
#    return(l1)
#    }
#-----------------------------------------------------------------------------------------
# calculate l0 and lz likelihood based statistic
##NS export(pf.l0)
pf.l0 <- function( data, wle, itemdiff ){
    dat1 <- data
    dat1.resp <- 1 - is.na(dat1)
    dat1[ is.na(dat1) ] <- 0
    # I <- ncol(dat1)
    Iresp <- rowSums( dat1.resp )
    pmatrix <- stats::plogis( outer( wle, itemdiff, "-" ) )
    # l0
    l0 <- rowSums( ( dat1 * log( pmatrix ) + ( 1 - dat1) * log( 1 - pmatrix ) )*dat1.resp /
                Iresp )
    # standardized l0
    E.l0 <- rowSums( dat1.resp*( pmatrix * log( pmatrix ) + ( 1 - pmatrix) * log( 1 - pmatrix ) )  )
    Var.l0 <- rowSums( dat1.resp*( pmatrix * ( 1 - pmatrix ) * ( stats::qlogis( pmatrix ) )^2 )  )
    # lz <- ( I*l0 - E.l0 ) / sqrt( Var.l0 )
    lz <- ( Iresp*l0 - E.l0 ) / sqrt( Var.l0 )
    list( "l0"=l0, "lz"=lz )
    }
#-----------------------------------------------------------------------------------
# Infit and Outfit (Rasch model)
##NS export(pf.outfit.infit)
pf.outfit.infit <- function( data, wle, itemdiff ){
    dat1 <- data
    # I <- ncol(dat1)
    pmatrix <- stats::plogis( outer( wle, itemdiff, "-" ) )

    dat1.resp <- 1 - is.na(dat1)
    dat1[ is.na(dat1) ] <- 0

    Iresp <- rowSums(dat1.resp)

    # Outfit (unweighted fit)
    U <- rowSums(  ( ( dat1 - pmatrix )^2 / Iresp / pmatrix / ( 1 - pmatrix ) ) * dat1.resp )
    # Infit (weighted fit)
    W <- rowSums(  dat1.resp * ( dat1 - pmatrix )^2  ) /
                    rowSums(  dat1.resp * pmatrix * ( 1 - pmatrix ) )
    list( "Outfit"=U, "Infit"=W )
        }
########################################################################


#.......................................
# U3 statistic
##NS export(pf.U3)
pf.U3 <- function( data, pval ){
    dat1 <- data
    # data processing
    data.proc <- .guttman.data.frame( data=dat1, pval=pval )
    data.proc2 <- .inverse.guttman.data.frame( data=dat1, pval=pval )
    P <- nrow(dat1)
    I <- ncol(dat1)
    # calculate A_i
    pmatrix <- outer( rep(1,P), data.proc$pval )
    pmatrix <- log( pmatrix / ( 1 - pmatrix ) )
    T1 <- rowSums( data.proc$data.guttman * pmatrix)
    T2 <- rowSums( data.proc$data * pmatrix)
    T3 <- rowSums( data.proc2$data.inverse.guttman  * pmatrix )
    GI <- ( T1 - T2 ) / ( T1 - T3 )
    GI
        }
#.......................................
# Caution Index
##NS export(pf.caution)
pf.caution <- function( data, pval ){
    dat1 <- data
    # data processing
    data.proc <- .guttman.data.frame( data=dat1, pval=pval )
    data.proc2 <- .inverse.guttman.data.frame( data=dat1, pval=pval )
    P <- nrow(dat1)
    I <- ncol(dat1)
    # calculate caution index
    pmatrix <- outer( rep(1,P), data.proc$pval )
    T1 <- rowSums( data.proc$data.guttman * pmatrix)
    T2 <- rowSums( data.proc$data * pmatrix)
    T3 <- rowSums( data.proc2$data.inverse.guttman  * pmatrix )
    GI <- ( T1 - T2 ) / ( T1 - T3 )
    GI
        }
#----------------------------------------------------------------
# Correlation Index
##NS export(pf.rbis)
pf.rpbis <- function( data, pval ){
    I <- ncol(data)
    P <- nrow(data)
    cor.rowwise( data, outer( rep(1,P), pval ) )
    }
# Correlation Index
##NS export(pf.rpbis.itemdiff)
pf.rpbis.itemdiff <- function( data, itemdiff ){
    I <- ncol(data)
    P <- nrow(data)
    cor.rowwise( data, outer( rep(1,P), itemdiff ) )
    }
#----------------------------------------------------------------
# Disagreement index
##NS export(pf.dependability)
pf.dependability <- function( data, pval ){
    dat1 <- data
    # data processing
    data.proc <- .guttman.data.frame( data=dat1, pval=pval )
    P <- nrow(dat1)
    I <- ncol(dat1)
    # calculate A_i
    pmatrix <- outer( rep(1,P), data.proc$pval )
    A.max <- rowSums( data.proc$data.guttman * pmatrix) / I
    A.i <- rowSums( data.proc$data * pmatrix) / I
    # disagreement index
    DI <- A.max - A.i
    DI
    }
#...................................................................
# ECI indexes
##NS export(pf.ECI)
pf.eci <- function( data, pval, wle, itemdiff){
    dat1 <- data
    # data processing
    data.proc <- .guttman.data.frame( data=dat1, pval=pval )
    itemdiff2 <- itemdiff[ data.proc$index ]
    P <- nrow(dat1)
    I <- ncol(dat1)
    # calculate matrices
    P.matr <- stats::plogis( outer( wle, itemdiff2, "-")  )
    X.matr <- data.proc$data
    n.matr <- outer( rep( 1, P), data.proc$pval )
    G.matr <- outer( rep(1,P), colMeans( P.matr )  )
    # indices
    ECI1 <- 1 - cov.rowwise( X.matr, n.matr ) / cov.rowwise( P.matr, n.matr )
    ECI2 <- 1 - cov.rowwise( X.matr, G.matr ) / cov.rowwise( P.matr, G.matr )
    ECI3 <- 1 - cor.rowwise( X.matr, G.matr ) / cor.rowwise( P.matr, G.matr )
    ECI4 <- 1 - cov.rowwise( X.matr, P.matr ) / cov.rowwise( G.matr, P.matr )
    ECI5 <- 1 - cor.rowwise( X.matr, P.matr ) / cor.rowwise( G.matr, P.matr )
    ECI6 <- 1 - cov.rowwise( X.matr, P.matr ) / cov.rowwise( P.matr, P.matr )
    ECI <- data.frame( ECI1, ECI2, ECI3, ECI4, ECI5, ECI6 )
    colnames(ECI) <- paste("ECI", 1:6, sep="")
    list( "ECI"=ECI, "ECI1"=ECI1,"ECI2"=ECI2, "ECI3"=ECI3, "ECI4"=ECI4, "ECI5"=ECI5, "ECI6"=ECI6 )
    }
#..................................................................
# calculate covariance rowwise for matrices
cov.rowwise <- function( m1, m2 ){
    I <- ncol(m1)
    ( rowSums( m1 * m2 ) - I * rowMeans(m1) * rowMeans(m2) ) / ( I - 1 )
    }
#...............................................................
# rowwise SD's
sd.rowwise <- function(m1){
    sqrt( cov.rowwise( m1, m1 ) )
    }
#...............................................................
# rowwise correlations
cor.rowwise <- function(m1,m2){
    cov.rowwise(m1,m2) / sd.rowwise( m1 ) / sd.rowwise( m2 )
    }
#................................................................
# create Guttman data frame
.guttman.data.frame <- function( data, pval ){
    data.sort <- .sort.items( data=data, pval=pval )
    data1 <- data.sort$data
    pval1 <- data.sort$pval
    I <- ncol(data1)
    # create Guttman pattern data matrix data.guttman
    score <- rowSums( data1 )
    data.guttman <- 0*data1
    for (ii in 1:I){
            data.guttman[,ii] <- 1*( score >=ii )
                    }
    res <- list( data=data1, pval=pval1, data.guttman=data.guttman, index=data.sort$index )
    return( res )
        }
#.........................................................................
# create inverse Guttman data frame
.inverse.guttman.data.frame <- function( data, pval ){
    data.sort <- .sort.items( data=data, pval=pval )
    data1 <- data.sort$data
    pval1 <- data.sort$pval
    I <- ncol(data1)
    # create Guttman pattern data matrix data.guttman
    score <- rowSums( data1 )
    data.guttman <- 0*data1
    for (ii in 1:I){ data.guttman[,I + 1 - ii] <- 1*( score >=ii ) }
    return( list( data=data1, pval=pval1, data.inverse.guttman=data.guttman ) )
        }
#.........................................................................
#........................................................................
# auxiliary function for sorting items according to p values
.sort.items <- function( data, pval ){
    l1 <- sort( pval, decreasing=TRUE, index.return=TRUE    )
    dat1 <- data[, l1$ix ]
    pval1 <- pval[ l1$ix ]
    return( list( data=dat1, pval=pval1, index=l1$ix) )
        }
#........................................................................

## File Name: rasch_pairwise_optimize_opt_fun_terms.R
## File Version: 0.03

rasch_pairwise_optimize_opt_fun_terms <- function(eps_horiz, eps_vert, n.ij, n.ji)
{
    eps <- eps_horiz
    I <- length(eps)
    epsM <- matrix(eps_vert, nrow=I, ncol=I, byrow=TRUE)
    t1 <- ( n.ij*eps - n.ji*epsM )^2
    t2 <- (n.ij+n.ji)*eps*epsM + 1e-7
    t3 <- t1/t2
    return(t3)
}

## File Name: anova_sirt.R
## File Version: 0.244

##############################################################
# anova rasch.mml
anova.rasch.mml <- function( object, ... )
{
    cl <- match.call()
    model1 <- object
    model2 <- list(object, ... )[[2]]
    res <- IRT.anova.sirt( model1, model2 )
    cl <- paste(cl)[-1]
    ind <- match( res$Model, c('model1', 'model2') )
    res$Model <- cl[ind]
    print(res)
    invisible(res)
}
##############################################################
# anova smirt
anova.smirt <- anova.rasch.mml
##############################################################
# anova rasch.mirtlc
anova.rasch.mirtlc <- anova.rasch.mml
##############################################################
# anova gom
anova.gom <- anova.rasch.mml
##############################################################
# anova rm.facets
anova.rm.facets <- anova.rasch.mml
##############################################################
# anova rm.sdt
anova.rm.sdt <- anova.rasch.mml
##############################################################
# anova prob.guttman
anova.prob.guttman <- anova.rasch.mml
##############################################################
# anova xxirt
anova.xxirt <- anova.rasch.mml
##############################################################


##############################################################
# Likelihood ratio test for rasch.copula2 objects
anova.rasch.copula2 <- function( object, ... )
{
    if (length(list(object, ...)) !=2){
        stop('anova method can only be applied for comparison of two models.\n')
    }
    objects <- list(object, ...)
    model1 <- objects[[1]]
    model2 <- objects[[2]]

    # define some necessary parameters
    model1$AIC <- model1$ic$AIC
    model1$BIC <- model1$ic$BIC
    model1$loglike <- model1$deviance / (-2)
    model1$Npars <- model1$ic$np
    model2$AIC <- model2$ic$AIC
    model2$BIC <- model2$ic$BIC
    model2$loglike <- model2$deviance / (-2)
    model2$Npars <- model2$ic$np
    # test
    dfr1 <- data.frame( Model='Model 1', loglike=model1$loglike,
                Deviance=-2*model1$loglike )
    dfr1$Npars <- sum(model1$Npars)
    dfr1$AIC <- model1$AIC
    dfr1$BIC <- model1$BIC
    dfr2 <- data.frame( Model='Model 2', loglike=model2$loglike,
                Deviance=-2*model2$loglike )
    dfr2$Npars <- sum(model2$Npars)
    dfr2$AIC <- model2$AIC
    dfr2$BIC <- model2$BIC
    dfr <- rbind( dfr1, dfr2 )
    dfr <- dfr[ order( dfr$Npars ), ]
    dfr$Chisq <- NA
    dfr$df <- NA
    dfr$p <- NA
    dfr[1,'Chisq'] <- dfr[1,'Deviance'] - dfr[2,'Deviance']
    dfr[1,'df'] <- abs( dfr[1,'Npars'] - dfr[2,'Npars'] )
    dfr[1, 'p' ] <- round( 1 - stats::pchisq( dfr[1,'Chisq'], df=dfr[1,'df'] ), 5 )
    for ( vv in 2L:( ncol(dfr))){
        dfr[,vv] <- round( dfr[,vv], 5 )
    }
    print(dfr)
    invisible(dfr)
}
anova.rasch.copula3 <- anova.rasch.copula2
##############################################################

## File Name: rasch.copula.R
## File Version: 0.99907



#------------------------------------------------------------------
# Copula estimation in a raschtype model
rasch.copula <- function( dat, itemcluster, group=NULL,
                        progress=TRUE, mmliter=1000, delta=NULL,
                        theta.k=seq(-4,4,len=21), alpha1=0, alpha2=0, numdiff.parm=.000001,
                        est.b=seq(1,ncol(dat)), est.a=rep(1,ncol(dat)), est.delta=NULL,
                        est.alpha=FALSE,
                        glob.conv=.001, alpha.conv=.0001, conv1=.001 )
{
    ###############################################################
    # INPUT:
    # dat    ... data frame
    # itemcluster ... vector of integers
    # progress    ... display progress?
    # mmliter     ... see rasch.mml
    # delta ... initial delta estimate (is relevant for fixing delta parameters)
    # theta.k... number of grid theta points
    # alpha1, alpha2 ... rasch type parameter
    # numdiff.parm ... numerical differentiation parameter
    # est.b    ... which b parameters shall be estimated
    ###############################################################
    s1 <- Sys.time()
    # item clusters
    t1 <- table(itemcluster)
    t1b <- as.numeric( names(t1))
    t1c <- t1[ names(t1) !=0 ]
    if ( any( t1c==1) ){
           stop( "There should be at least two items in an item cluster\n" )
                                    }
            x1 <- seq( 1, max(t1b) )
            x2 <- sort(setdiff( as.numeric(sort(names(t1))), 0 ))
            if ( sum( abs( x1-x2) ) > 10^(-10) ){
               stop( "Item cluster identifiers must be recoded to 1, ..., C\n" )
                                 }
    CC <- length(x1) # number of clusters
    # calculation of number of itemclusters
    if ( progress  ){
        cat("---------------------------------------------------------------------------------------------------------- \n")
        cat("Marginal Maximum Likelihood Estimation \n")
        cat(paste( "Raschtype Copula Model with generalized logistic link function: Estimation of alpha1 and alpha2 \n") )
        cat("Function rasch.copula\n")
        cat("---------------------------------------------------------------------------------------------------------- \n")
        flush.console()
      }
    I <- ncol(dat)
    if ( is.null( colnames(dat))){
        colnames(dat) <- paste("Item", 1:I, sep="")
                }
    # groups
    if ( ! is.null( group) ){
        groups <- unique( group )
        G <- length( groups )    } else { G <- 1 }
    GG <- G
    # data preparation (frequencies)
    dat10 <- dat00 <- dat
    dat10[ is.na( dat10) ] <- 9
    patt <- paste("P",dat10[,1],sep="")
    for (ii in 2:I){ patt <- paste( patt, dat10[,ii],sep="") }
    pattern <- data.frame( table(patt) )
    colnames(pattern) <- c("pattern", "freqwgt")
    # patttern in data
#    pattern.in.data <- data.frame(match( patt, pattern$pattern )
    pattern.in.data <- patt
    # calculate frequencies in multiple group case
    if (G > 1 ){
        for (gg in 1:G){
        # gg <- 1
        t1 <- table( patt[ group==gg  ] )
        pattern <- merge( pattern, t1, by.x=1, by.y=1, all=T )
                    }
        pattern[ is.na(pattern) ] <- 0
        colnames(pattern)[-c(1:2)]     <- paste("freqwgt", 1:G, sep="")
                }
    dat0 <- matrix( 0, nrow=nrow(pattern), I )
    for (ii in 1:I){
            dat0[,ii] <- as.numeric( substring( paste( pattern[,1] ), ii+1, ii+1 ) )
                }
    dat0[ dat0==9 ] <- NA
    colnames(dat0) <- colnames(dat)
    # define mu and sigma
    mu <- rep(0,G)
    sigma <- rep(1,G)
    if ( G > 1){
#            mu[2] <- 1 ; sigma[2] <- 1.2
                }
    #------------------------
    dat <- dat0
    dat2 <- dat
    dat2.resp <- 1 * ( !is.na( dat2) )
    dat2[ is.na(dat2) ] <- 0
        # data preparation in case of item clusters
        dat2.ld <- NULL        # dat2.ld is NULL if there are no item clusters
            if ( is.null(delta)){ delta <- stats::runif( CC, .2, .8 ) }    # initial estimate of delta
            if ( is.null(est.delta)){ est.delta <- 1:CC }
            dp.ld <- as.list( 1:CC )
            # item pattern
            for (cc in 1:CC){
    #            cc <- 1
                icl.cc <- which( itemcluster==cc )
                dp.ld.cc <- .calc.copula.itemcluster( D=length(icl.cc) )
                dp.ld.cc$items <- icl.cc
                dp.ld.cc$N.items <- NCC <- length(icl.cc)
                dp.ld.cc$itemlabels <- colnames(dat)[icl.cc]
                # item selection for independent items
                m1 <- outer( rep(1,2^NCC), icl.cc)
                m2 <- dp.ld.cc$patt * m1   + ( 1 - dp.ld.cc$patt ) * ( m1 + I )
                m2 <- matrix( t(m2),  nrow=1, byrow=T )[1,]
                res <- list( "items"=m2 )
                res1 <- rep(NCC, 2^NCC )
                names(res1) <- rownames( dp.ld.cc$patt )
                res$N.Index1 <- res1
                dp.ld.cc$independent <- res
                # item selection for dependent items
                m2 <- ( 1 - dp.ld.cc$patt ) * ( m1 + I )
                m2 <- matrix( t(m2),  nrow=1, byrow=T )[1,]
                m2 <- c( m2[ m2> 0 ], 2*I + 1 )
                res <- list( "items"=m2 )
                res$N.Index1 <- rowSums( dp.ld.cc$patt==0 )
                res$N.Index1[ length(res$N.Index1) ] <- 1
                dp.ld.cc$dependent <- res
                dp.ld[[cc]] <- dp.ld.cc
                            }
            # create data frame with item response pattern
            dat2.ld <- matrix(0, nrow(dat2), CC )
            dat3.ld <- as.list( 1:CC )
            for (cc in 1:CC){
                # cc <- 1
                dp.cc <- dp.ld[[cc]]
                dat2.cc <- dat2[, dp.cc$items ]
                l1 <- "P"
                for ( vv in seq(1,ncol(dat2.cc))){
                    l1 <- paste( l1, dat2.cc[,vv], sep="")
                        }
                dat2.ld[, cc ] <- match( l1, rownames( dp.cc$patt ) )
                dat2.ld[ rowSums( dat2.resp[, dp.cc$items ]    ) < length(dp.cc$items), cc ] <- NA
                NRR <- nrow( dp.cc$patt )
                dat3.ld.cc <- sapply( seq( 1, NRR), FUN=function(aa){ 1*(dat2.ld[,cc]==aa ) } )
                dat3.ld.cc[ is.na(dat3.ld.cc) ] <- 0
                dat3.ld[[cc]] <- dat3.ld.cc
                            }
                # response indicator
                dat2.ld.resp <- 1 - is.na( dat2.ld )
                # set missings in dat2 to some arbitrary category
                dat2.ld[ is.na( dat2.ld ) ] <- 1
                # create dat2 data sets for local independence items
                itemcluster0 <- ind2 <- which( itemcluster==0 )
                bdat2.li.resp <- dat2.li.resp <- bdat2.li <- dat2.li <- NULL

                if ( length(ind2) > 0 ){
                        dat2.li <- dat2[, ind2 ]
                        dat2.li.resp <- dat2.resp[, ind2 ]
                        bdat2.li <- cbind( dat2.li, 1 - dat2.li )
                        bdat2.li.resp <- cbind( dat2.li.resp, dat2.li.resp )
                                        }
        #########################################################################
    #--------------------------------------------------
    # initial estimate of item difficulty
#    b <- rasch.pairwise( dat, progress=FALSE)$item$itemdiff
    b <- - stats::qlogis( ( colMeans( dat00, na.rm=T ) + .005 ) / 1.01 )
    I <- ncol(dat2)
    # initial estimate of (mean) item discrimination
    a <- rep( 1, I )
    # density weights
    wgt.theta <- stats::dnorm(theta.k)
    wgt.theta <- wgt.theta / sum( wgt.theta )
    if ( G > 1){
        wgt.theta <- matrix(0, length(theta.k), G )
        for ( gg in 1:G){
            wgt.theta[,gg] <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg] )
            wgt.theta[,gg] <- wgt.theta[,gg] / sum( wgt.theta[,gg] )
                        }
                }
    iter <- 0
    #**********************************
    # BEGIN MARGINAL MAXIMUM LIKELIHOOD ESTIMATION
    dev <- 1 ; par.change <- dev.change <- 1000
    maxalphachange <- 0
    while ( ( dev.change > glob.conv | par.change > conv1 | maxalphachange > alpha.conv ) & iter < mmliter ){

        cat( paste(rep("-", 70), collapse=""), "\n")
        k1 <- floor( log10(iter+1) )
        x1 <- "        |"
        x1 <- substring( x1, k1+1 )
        s1c <- Sys.time()
        cat( paste( paste( "MML EM Iter.", iter + 1 ), x1, paste( rep( "*", 10  ), collapse=""), "|  ",
                        s1c, "  ",
                        if ( iter > 0 ){ paste( round(difftime(s1c,s1b, units='secs' ),4), "secs" ) },
                        "\n",sep="") ) #
        s1b <- Sys.time()
        h <- numdiff.parm
        dev0 <- dev
        #************************************
        # estimation of b parameters
        b0 <- b
        # identify different b parameter groups
        bG <- setdiff( unique( est.b ), 0 )
        prbar <- seq( 1, 10, len=length(bG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        cat(" Estimation of b:     |")
        for (bb in bG){
            est.bb <- 1 * (est.b==bb )
            if (bb==1 ){
                rescop <- .ll.rasch.copula2( theta.k, b0, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG  )
                ll0 <- rescop$ll
                            }
            ll1 <- .update.ll.rasch.copula( theta.k, b0 + h*est.bb, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG )$ll
            ll2 <- .update.ll.rasch.copula( theta.k, b0 - h*est.bb, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG )$ll

            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            if ( abs(d2) < 10^(-20) ){ d2 <- 10^20 }
            b.change <- - d1 / d2
            b.change <- ifelse( abs( b.change ) > .5, .5*sign(b.change), b.change )
            b.change <- b.change * est.bb
            b <- b + b.change
#            cat( bb, " ") ;
            cat( paste( rep( "-", prbar[bb]), collapse="") )
            flush.console()
                    }
        a1b <- max( abs( b - b0 ) )
        cat("|     max. parm. change", round( a1b, 5),"\n")
        #******************************************************************************
        # estimation of a parameters
        a0 <- a
        # identify different a parameter groups
        aG <- setdiff( unique( est.a ), 0 )
        prbar <- seq( 1, 10, len=length(aG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        cat(" Estimation of a:     |")
        for (aa in aG){
            est.aa <- 1 * (est.a==aa )
            rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG )
            ll0 <- rescop$ll
            ll1 <- .update.ll.rasch.copula( theta.k, b, alpha1, alpha2, a + h*est.aa, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG )$ll
            ll2 <- .update.ll.rasch.copula( theta.k, b, alpha1, alpha2, a - h*est.aa, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG)$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            a.change <- - d1 / d2
            a.change <- ifelse( abs( a.change ) > .3, .3*sign(a.change), a.change )
            a.change <- a.change * est.aa
            a <- a + a.change
            a[ a < 0 ] <- .01
#            cat( aa, " ") ;
            cat( paste( rep( "-", prbar[aa]), collapse="") )
            flush.console()
                            }
        if ( length(aG) < 2 ){ cat( paste( rep( "-", 10 - length(aG) ), collapse="") ) }
        a1a <- max( abs( a - a0 ) )
        cat("|     max. parm. change", round( a1a, 5),"\n")
        #******************************************************************************
        # estimation of delta parameters
        delta0 <- delta
        dG <- setdiff( unique( est.delta ), 0 )
        prbar <- seq( 1, 10, len=length(dG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        # identify different a parameter groups
        cat(" Estimation of delta: |")
        for (cc in dG){
            est.cc <- 1 * ( est.delta==cc )
            rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp,
                            delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG)
            ll0 <- rescop$ll
            ll1 <- .update.ll.rasch.copula( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta + h*est.cc, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG)$ll
            ll2 <- .update.ll.rasch.copula( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta - h*est.cc, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG)$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            delta.change <- - d1 / d2
            delta.change <- ifelse( abs( delta.change ) > .2, .2*sign(delta.change), delta.change )
            delta.change <- delta.change * est.cc
            delta <- delta + delta.change
            delta[ delta <=0 ] <- 2*numdiff.parm
            delta[ delta >=1 ] <- 1 - 2*numdiff.parm
# adapt delta boundary estimate
#if ( adapt.delta.boundary & ( iter %% 5==0 ) ){
# print("TRUE")
#    delta <- ifelse ( abs( delta - 2*numdiff.parm ) < 10^(-15), .20, delta )
#                }

            cat( paste( rep( "-", prbar[cc]), collapse="") )
            flush.console()
                            }
        if ( length(dG) < 2 ){ cat( paste( rep( "-", 10 - length(dG) ), collapse="") ) }
        a1d <- max( abs( delta - delta0 ) )
        cat("|     max. parm. change", round( a1d, 5),"\n")
        #******************************************************************************
        # estimation of alpha parameters
        alpha10 <- alpha1
        alpha20 <- alpha2
        prbar <- 5
        cat(" Estimation of alpha: |")
        # alpha1
        if (est.alpha){
            rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG)
            ll0 <- rescop$ll
            ll1 <- .update.ll.rasch.copula( theta.k, b, alpha1 + h, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG)$ll
            ll2 <- .update.ll.rasch.copula( theta.k, b, alpha1 - h, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster, pattern, GG)$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            alpha.change <- - d1 / d2
            a1k1 <- alpha.change <- ifelse( abs( alpha.change ) > .1, .1*sign(alpha.change), alpha.change )
            alpha1 <- alpha1 + alpha.change
            }
            cat( paste( rep( "-", prbar), collapse="") )
            flush.console()
        # alpha2
        if (est.alpha){
            rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG )
            ll0 <- rescop$ll
            ll1 <- .update.ll.rasch.copula( theta.k, b, alpha1, alpha2+h, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster,pattern, GG)$ll
            ll2 <- .update.ll.rasch.copula( theta.k, b, alpha1, alpha2 -h, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, rescop, itemcluster,pattern, GG)$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            alpha.change <- - d1 / d2
            a1k2 <- alpha.change <- ifelse( abs( alpha.change ) > .1, .1*sign(alpha.change), alpha.change )
            alpha2 <- alpha2 + alpha.change
            }
            cat( paste( rep( "-", prbar), collapse="") )
            flush.console()
        a1k <- max( abs( c( alpha1 - alpha10, alpha2 - alpha20 )) )
        cat("|     max. parm. change", round( a1k, 5),"\n")
        #******************************************************************************
        # estimation of mu parameters
        a1m <- 0
        if (G>1){
        mu0 <- mu
        # identify different a parameter groups
#        h <- 10 * numdiff.parm
#        dG <- setdiff( unique( est.delta ), 0 )
#        prbar <- seq( 1, 10, len=length(dG) )
#        prbar <- floor( prbar )
#        prbar <- c( prbar[1], diff(prbar) )
#        muG <- GG-1
        muG <- 1:(GG-1)
        prbar <- seq( 1, 10, len=length(muG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        cat(" Estimation of mu:    |")
        for (gg in 2:G){
#            est.aa <- est.a * (est.a==aa )
            rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG )
            ll0 <- rescop$ll
            # mu + h
            w1 <- wgt.theta
            w2 <- stats::dnorm( theta.k, mean=mu[gg] + h, sd=sigma[gg] )
            w1[,gg] <- w2 / sum(w2)
            rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
                            wgt.theta=w1, I,
                            bdat2.li, bdat2.li.resp, pattern, GG )
            ll1 <- rescop$ll
            # mu - h
            w1 <- wgt.theta
            w2 <- dnorm( theta.k, mean=mu[gg] - h, sd=sigma[gg] )
            w1[,gg] <- w2 / sum(w2)
            rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
                            wgt.theta=w1, I,
                            bdat2.li, bdat2.li.resp, pattern, GG )
            ll2 <- rescop$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            mu.change <- - d1 / d2
            mu.change <- ifelse( abs( mu.change ) > .3, .3*sign(mu.change), mu.change )
            mu.change <- mu.change * ( ( 1:G )==gg )
            mu <- mu + mu.change
            w2 <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg] )
            wgt.theta[,gg] <- w2 / sum(w2)
#            cat( aa, " ") ;
            cat( paste( rep( "-", prbar[gg-1]), collapse="") )
            flush.console()
                            }
        if ( length(muG) < 2 ){ cat( paste( rep( "-", 10 - length(muG) ), collapse="") ) }
        a1m <- max( abs( mu - mu0 ) )
        cat("|     max. parm. change", round( a1m, 5),"\n")
                }            # end mu
        ######################################################################
        #******************************************************************************
        # estimation of sigma parameters
        a1s <- 0
        if (G>1){
        sigma0 <- sigma
        # identify different a parameter groups
#        h <- 10 * numdiff.parm
        sigmaG <- seq(1,GG-1)
        prbar <- seq( 1, 10, len=length(sigmaG) )
        prbar <- floor( prbar )
        prbar <- c( prbar[1], diff(prbar) )
        cat(" Estimation of sigma: |")
        for (gg in 2:G){
#            est.aa <- est.a * (est.a==aa )
            rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
                            bdat2.li, bdat2.li.resp, pattern, GG )
            ll0 <- rescop$ll
            # sigma + h
            w1 <- wgt.theta
            w2 <- dnorm( theta.k, mean=mu[gg], sd=sigma[gg] +h)
            w1[,gg] <- w2 / sum(w2)
            rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
                            wgt.theta=w1, I,
                            bdat2.li, bdat2.li.resp, pattern, GG )
            ll1 <- rescop$ll
            # sigma - h
            w1 <- wgt.theta
            w2 <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg]-h )
            w1[,gg] <- w2 / sum(w2)
            rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta,
                            wgt.theta=w1, I,
                            bdat2.li, bdat2.li.resp, pattern, GG )
            ll2 <- rescop$ll
            d1 <- ( ll1 - ll2  ) / ( 2 * h )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
            sigma.change <- - d1 / d2
            sigma.change <- ifelse( abs( sigma.change ) > .3, .3*sign(sigma.change), sigma.change )
            sigma.change <- sigma.change * ( ( 1:G )==gg )
            sigma <- sigma + sigma.change
            w2 <- stats::dnorm( theta.k, mean=mu[gg], sd=sigma[gg] )
            wgt.theta[,gg] <- w2 / sum(w2)
#            cat( aa, " ") ;
            cat( paste( rep( "-", prbar[gg-1]), collapse="") )
            flush.console()
                            }
        if ( length(sigmaG) < 2 ){ cat( paste( rep( "-", 10 - length(sigmaG) ), collapse="") ) }
        a1s <- max( abs( sigma - sigma0 ) )
        cat("|     max. parm. change", round( a1s, 5),"\n")
                }            # end sigma
        ######################################################################



        #**********************************************************************************
        # convergence display
        a1 <- stats::aggregate( b, list( est.b), mean )
        cat("   b parameters: ", paste( round( a1[,2], 3 ), collapse=" " ), "\n" )
        a1 <- stats::aggregate( a, list( est.a), mean )
        cat("   a parameters: ", paste( round( a1[,2], 3 ), collapse=" " ), "\n" )
        cat("   delta parameters: ", paste( round( delta, 3 ), collapse=" " ), "\n" )
        cat("   alpha parameters: ", paste( round( c(alpha1, alpha2), 3 ), collapse=" " ), "\n" )
        cat("   mu parameters: ", paste( round( mu, 3 ), collapse=" " ), "\n" )
        cat("   sigma parameters: ", paste( round( sigma, 3 ), collapse=" " ), "\n" )
        #******************************************************************************
        iter <- iter + 1
        dev <- -2*ll0
        dev.change <- abs( ( dev - dev0)/ dev0 )
        par.change <- max( a1a, a1b, a1d, a1k, a1m, a1s)
        cat( "Deviance=",   round( dev, 5 ), "| max. parm. change=",
                                        round( par.change, 6 ),  " \n"   )
        if ( ( dev > dev0 ) & ( iter > 4 ) ){ cat("   Deviance has increased! Convergence Problems?\n") }
        flush.console()
#stop("herer")
            }
    # end MML iterations
    #**********************************************************************************
    # Standard error estimation (This is a TO DO!)
    iterend <- iter
#    iter <- 1
#    cat( paste(rep("-", 70), collapse=""), "\n")
#    k1 <- floor( log10(iter+1) )
#    x1 <- " |"
#    x1 <- substring( x1, k1+1 )
#    cat( paste( paste( "Standard errors (SE's)"  ), x1, paste( rep( "*", 10  ), collapse=""), "|  ",
#                    Sys.time(), "\n",sep="") ) #
#    h <- numdiff.parm
#    dfr <- NULL
        #************************************
        # standard errors b
        # identify different b parameter groups
#        bG <- setdiff( unique( est.b ), 0 )
#        prbar <- seq( 1, 10, len=length(bG) )
#        prbar <- floor( prbar )
#        prbar <- c( prbar[1], diff(prbar) )
#        cat(" SE's of b:            |")
#        for (bb in bG){
#            est.bb <- est.b * (est.b==bb )
#            if (bb==1 ){
#                rescop <- .ll.rasch.copula2( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
#                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
#                            bdat2.li, bdat2.li.resp )
#                ll0 <- rescop$ll
#                            }
#
#            ll1 <- .update.ll.rasch.copula( theta.k, b + h*est.bb, alpha1, alpha2, a, dat2.li, itemcluster0,
#                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
#                            bdat2.li, bdat2.li.resp, rescop, itemcluster )


#            ll2 <- .update.ll.rasch.copula( theta.k, b - h*est.bb, alpha1, alpha2, a, dat2.li, itemcluster0,
#                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
#                            bdat2.li, bdat2.li.resp, rescop, itemcluster )
            # second order derivative
            # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
#            d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
#            d2 <- sqrt( 1 / sum( - d2 ) )
#            dfr.pp <- data.frame( "parameter"="b", "pargropup"=bb,
#                            "est"=b[ bb ], "se"=d2 )
#            dfr <- rbind( dfr, dfr.pp )
#            cat( paste( rep( "-", prbar[bb]), collapse="") )
#            flush.console()
#                            }
#        cat("|\n")
#    print(dfr)

    #********************************************************
    # information criteria
        # calculations for information criteria
        ic <- list( "deviance"=dev, "n"=nrow(dat00) )
        # number of parameters to be estimated
        # these formulas hold when assuming normal distributions
        ic[[ "np" ]] <- length(bG) + length(aG) + length(dG) + 2*est.alpha
        # AIC
        ic$AIC <- dev + 2*ic$np
        # BIC
        ic$BIC <- dev + ( log(ic$n) )*ic$np
        # CAIC
        ic$CAIC <- dev + ( log(ic$n) + 1 )*ic$np
    #**********************************************************************************
    # results item parameters
    item <- data.frame( "item"=colnames(dat),
                "N"=colSums(!is.na(dat00)),
                "p"=colMeans( dat00, na.rm=T ),
                "b"=b, "est.b"=est.b, "a"=a, "est.a"=est.a )
    item$thresh <- item$a * item$b
    # add results dependency parameter for item clusters
    item$itemcluster <- itemcluster
    item$delta <- 0
    cat("---------------------------------------------------------------------------------------------------------- \n")
    for (cc in 1:CC){
        # cc <- 1
        dcc <- dp.ld[[cc]]
        item[ dcc$items, "delta"] <- delta[cc]
                }
    item$est.delta <- 0
    i1 <- which( item$itemcluster > 0 )
    icl <- item$itemcluster[ i1 ]
    item$est.delta[ i1 ] <- est.delta[ icl ]
    # print item summary
    cat("Parameter summary\n")
    .pr( item, digits=3 )        # print item statistics
#    print(round(item,3))
        # computation time
        s2 <- Sys.time()
        if (progress){
                cat("---------------------------------------------------------------------------------------------------------- \n")
                cat("Start:", paste( s1), "\n")
                cat("End:", paste(s2), "\n")
                cat("Difference:", print(s2 -s1), "\n")
                cat("---------------------------------------------------------------------------------------------------------- \n")
                    }
    # collect results

#     dat2.li, itemcluster0,
#                            CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp
    datalist <- list( pattern.in.data=pattern.in.data, dat0=dat0,
                    dat2=dat2, dat2.resp=dat2.resp, dat2.li=dat2.li,
                    dat2.ld=dat2.ld, dat2.li.resp=dat2.li.resp,
                    dat2.ld.resp=dat2.ld.resp, dp.ld=dp.ld, CC=CC,
                    bdat2.li=bdat2.li, bdat2.li.resp=bdat2.li.resp,
                    itemcluster0=itemcluster0, dat3.ld=dat3.ld
                            )
    res <- list( "N.itemclusters"=CC, "item"=item, "iter"=iterend, "dev"=dev,
                    "delta"=delta, "b"=b, "a"=a, "mu"=mu, "sigma"=sigma,
                    "alpha1"=alpha1, "alpha2"=alpha2, "ic"=ic, "theta.k"=theta.k, "deviance"=dev,
                    "pattern"=pattern    , "datalist"=datalist                )
    class(res) <- "rasch.copula"
    return(res)
        }
#----------------------------------------------------------------------------------






#----------------------------------------------------------------------------------------------
.update.ll.rasch.copula <- function( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
        bdat2.li, bdat2.li.resp, rescopula, itemcluster, pattern, GG, ... ){
        #-------------------------------------------------------
        # use this function for log likelihood calculation
        # look for items which change parameters for necessary update
        G <- GG
        ind.b <- which( b !=rescopula$b )
        ind.a <- which( a !=rescopula$a )
        ind.delta <- which( delta !=rescopula$delta )
        ind.alpha1 <- ( alpha1 !=rescopula$alpha1 )    + ( alpha2 !=rescopula$alpha2 )
        if (ind.alpha1 > 0){ ind.alpha <- seq(1, ncol(dat2.ld) ) } else { ind.alpha <- NULL }
        itemset <- union( ind.b, ind.a )
        itemset <- union( itemset, ind.alpha )
        # update term local independence
        li.update <- 1 * ( sum( itemcluster0 %in% itemset ) > 0 )
        # update terms item dependence parameters
        ld.update <- sapply( 1:CC, FUN=function(cc){
                g1 <- intersect( which( itemcluster==cc ), itemset )
                if ( length(g1)){ v1 <- cc } else { v1 <- NULL }
                v1
                    } )
        ld.update <- unique( union( ind.delta, unlist( ld.update) ) )
        #------------------------------------------
        ndat2 <- nrow(dat2.ld)
        ntheta <- length(theta.k)
        pjk.theta.k <- .prob.raschtype.genlogis( theta.k, b, alpha1, alpha2, a)
        pjk.theta.k.tt <- pjk.theta.k[ rep( seq(1,ntheta), each=ndat2 ), ]
        # define indices for persons
        ind.dat2 <- rep( seq(1,ndat2), ntheta )
        g1 <- rescopula$g1
        M1 <- rep(1,ndat2)
        pqjk.theta.k.tt <- cbind( pjk.theta.k.tt, 1 - pjk.theta.k.tt, 1 )
            #*************************************************************
            if ( ( length(itemcluster0) > 0 ) & ( li.update > 0 ) ){
                pqjk.theta.k.tt0 <- pqjk.theta.k.tt[, c( itemcluster0, itemcluster0+I) ]
                # likelihood for independent items at theta tt
                ll.tt <- ( pqjk.theta.k.tt0 ^ bdat2.li[ind.dat2,] )^bdat2.li.resp[ind.dat2,]
#                g1 <- rowProds( ll.tt )         } else { g1 <- 1 }
                g1[,1] <- rowProds2( ll.tt )         } # else { g1[,1] <- 1 }
                #*************************************************************
                # likelihood for dependent items
            for (cc in ld.update){
                # cc <- 1
                dat3.ld.cc <- dat3.ld[[cc]]
                dat3.ld.cc <- dat3.ld.cc[ ind.dat2, ]
                dp.ld.cc <- dp.ld[[cc]]
                m1.cc <- pqjk.theta.k.tt[, dp.ld.cc$independent$items ]
                v1.cc <- dp.ld.cc$independent$N.Index1
                # product under independence
#                Fpjk.cc <- .rowProds.bundle( m1=m1.cc, v1=v1.cc)
                Fpjk.cc <- .rowProds2.bundle( m1=m1.cc, v1=v1.cc)
                #print( dim(m1.cc))
                # evaluate likelihood
                g1.tt <- ( rowSums(Fpjk.cc * dat3.ld.cc ) )^dat2.ld.resp[ind.dat2,cc]
                # product under dependence
                m1.cc <-  pqjk.theta.k.tt[, dp.ld.cc$dependent$items ]
                v1.cc <- dp.ld.cc$dependent$N.Index1
#                F0pjk.cc <- .rowMins.bundle( m1=m1.cc, v1=v1.cc)
                F0pjk.cc <- .rowMins2.bundle( m1=m1.cc, v1=v1.cc)
                F1pjk.cc <- F0pjk.cc %*% t(dp.ld.cc$calc)
                g2.tt <- ( rowSums(F1pjk.cc * dat3.ld.cc) )^dat2.ld.resp[ind.dat2,cc]
                g3.tt <- ( 1 - delta[cc] ) * g1.tt + delta[cc] * g2.tt
                g1[,cc+1] <- g3.tt
#                g1 <- g1*g3.tt        # end product for item cluster cc
                            }
                res <- g1
                g1 <- rowProds2( g1 )
                ll.theta.k <- matrix( g1, ncol=ntheta )
                # casewise: integrate over the product
                # print(ll.theta.k)
                # print( outer( M1, wgt.theta ) )
#                lli <- log( rowSums(  ll.theta.k * outer( M1, wgt.theta )  ) )
#                ll1 <- sum( pattern$freqwgt * lli )
                # one group
                if (G==1){
                    # casewise: integrate over the product evaluate at grid of trait density
                    lli <- log( rowSums(  ll.theta.k * outer( M1, wgt.theta )  ) )
                    ll1 <- sum( lli * pattern$freqwgt)
                            }
                # mutiple groups
                if (G>1){
                    ll1 <- 0
                    lli <- as.list( rep(1,G) )
                    for (gg in 1:G){
                        lli[[gg]] <- log( rowSums(  ll.theta.k * outer( M1, wgt.theta[,gg] )  ) )
                        ll1 <- ll1 + sum( lli[[gg]] * pattern[,gg+2] )
                                    }
                            }

#                ll1 <- sum( log( rowSums(  ll.theta.k * outer( M1, wgt.theta )  ) ) )
                res <- list( "ll"=ll1, "lli"=lli )
                return(res)
                }
#----------------------------------------------------------------------------------------------




#----------------------------------------------------------------------------------------------
.ll.rasch.copula2 <- function( theta.k, b, alpha1, alpha2, a, dat2.li, itemcluster0,
        CC, dp.ld, dat2.ld, dat3.ld, dat2.ld.resp, dat2.li.resp, delta, wgt.theta, I,
        bdat2.li, bdat2.li.resp, pattern, GG, ... ){
        #-------------------------------------------------------
        # use this function for log likelihood calculation
        G <- GG
        ndat2 <- nrow(dat2.ld)
        ntheta <- length(theta.k)
        pjk.theta.k <- .prob.raschtype.genlogis( theta.k, b, alpha1, alpha2, a)
        pjk.theta.k.tt <- pjk.theta.k[ rep( seq(1,ntheta), each=ndat2 ), ]
        # define indices for persons
        ind.dat2 <- rep( seq(1,ndat2), ntheta )
        g1 <- matrix( 0, nrow(pjk.theta.k.tt), CC + 1 )
        M1 <- rep(1,ndat2)
        pqjk.theta.k.tt <- cbind( pjk.theta.k.tt, 1 - pjk.theta.k.tt, 1 )
            if ( length(itemcluster0) > 0 ){
                pqjk.theta.k.tt0 <- pqjk.theta.k.tt[, c( itemcluster0, itemcluster0+I) ]
                # likelihood for independent items at theta tt
                ll.tt <- ( pqjk.theta.k.tt0 ^ bdat2.li[ind.dat2,] )^bdat2.li.resp[ind.dat2,]
#                g1 <- rowProds( ll.tt )         } else { g1 <- 1 }
                g1[,1] <- rowProds2( ll.tt )         } else { g1[,1] <- 1 }
                # likelihood for dependent items
            for (cc in 1:CC){
                # cc <- 1
                dat3.ld.cc <- dat3.ld[[cc]]
                dat3.ld.cc <- dat3.ld.cc[ ind.dat2, ]
                dp.ld.cc <- dp.ld[[cc]]
                m1.cc <- pqjk.theta.k.tt[, dp.ld.cc$independent$items ]
                v1.cc <- dp.ld.cc$independent$N.Index1
                # product under independence
#                Fpjk.cc <- .rowProds.bundle( m1=m1.cc, v1=v1.cc)
                Fpjk.cc <- .rowProds2.bundle( m1=m1.cc, v1=v1.cc)
                # evaluate likelihood
                g1.tt <- ( rowSums(Fpjk.cc * dat3.ld.cc ) )^dat2.ld.resp[ind.dat2,cc]
                # product under dependence
                m1.cc <-  pqjk.theta.k.tt[, dp.ld.cc$dependent$items ]
                v1.cc <- dp.ld.cc$dependent$N.Index1
#                F0pjk.cc <- .rowMins.bundle( m1=m1.cc, v1=v1.cc)
                F0pjk.cc <- .rowMins2.bundle( m1=m1.cc, v1=v1.cc)
                F1pjk.cc <- F0pjk.cc %*% t(dp.ld.cc$calc)
                g2.tt <- ( rowSums(F1pjk.cc * dat3.ld.cc) )^dat2.ld.resp[ind.dat2,cc]
                g3.tt <- ( 1 - delta[cc] ) * g1.tt + delta[cc] * g2.tt
                g1[,cc+1] <- g3.tt
#                g1 <- g1*g3.tt        # end product for item cluster cc
                            }
                res <- g1
                g1 <- rowProds2( g1 )
                ll.theta.k <- matrix( g1, ncol=ntheta )
                # one group
                if (G==1){
                    # casewise: integrate over the product evaluate at grid of trait density
                    lli <- log( rowSums(  ll.theta.k * outer( M1, wgt.theta )  ) )
                    ll1 <- sum( lli * pattern$freqwgt)
                            }
                # mutiple groups
                if (G>1){
                    ll1 <- 0
                    lli <- as.list( rep(1,G) )
                    for (gg in 1:G){
                        lli[[gg]] <- log( rowSums(  ll.theta.k * outer( M1, wgt.theta[,gg] )  ) )
                        ll1 <- ll1 + sum( lli[[gg]] * pattern[,gg+2] )
                                    }
                            }
                res <- list( "ll"=ll1, "g1"=res, "b"=b, "a"=a, "delta"=delta,
                                    "alpha1"=alpha1, "alpha2"=alpha2, "lli"=lli )
                }
#----------------------------------------------------------------------------------------------

## File Name: sirt_import_psych_fa.R
## File Version: 0.03

sirt_import_psych_fa <- function(r, nfactors, ...)
{
    TAM::require_namespace_msg('psych')
    y <- psych::fa( r=r, nfactors=nfactors, ...)
    return(y)
}

## File Name: rasch_jml_person_parameters_summary.R
## File Version: 0.04

rasch_jml_person_parameters_summary <- function(x)
{
    res <- data.frame(N=length(x), mean=mean(x),
                        median=stats::quantile(x, probs=.5),
                        sd=stats::sd(x), min=min(x), max=max(x)
                        )
    return(res)
}

## File Name: mgsem_duplication_matrix.R
## File Version: 0.121

mgsem_duplication_matrix <- function(x)
{
    NV <- ncol(x)
    dfr1 <- data.frame( index1=rep(1L:NV, NV), index2=rep(1L:NV, each=NV))
    N1 <- nrow(dfr1)
    dfr2 <- dfr1[ dfr1[,1] >=dfr1[,2], ]
    ND <- nrow(dfr2)
    dupl <- matrix(0, nrow=ND, ncol=N1)
    dd <- 1
    for (dd in 1L:ND){
        h1 <- dfr2$index1[dd]
        h2 <- dfr2$index2[dd]
        i1 <- which( ( dfr1$index1==h1 ) & ( dfr1$index2==h2 ) )
        i2 <- which( ( dfr1$index2==h1 ) & ( dfr1$index1==h2 ) )
        if (i1==i2){
            dupl[dd,i1] <- 1
        } else {
            dupl[dd,i1] <- 0.5
            dupl[dd,i2] <- 0.5
        }
    }
    return(dupl)
}

## File Name: sirt_eigenvalues.R
## File Version: 0.25


# calculation of first D eigenvalues
sirt_eigenvalues <- function(X, D, maxit=200, conv=10^(-6) )
{
    # Rcpp::List sirt_rcpp_D_eigenvalues( Rcpp::NumericMatrix Xr, int D, int maxit,
    #                    double conv )
    res <- sirt_rcpp_D_eigenvalues( Xr=X, D=D, maxit=maxit, conv=conv)
    return(res)
}

## File Name: summary.rm.facets.R
## File Version: 0.26


# Summary for rm.facets object
summary.rm.facets <- function( object, file=NULL, ... ){

    # open sink for a file
    sirt_osink( file=file  )

    cat("-----------------------------------------------------------------\n")
    #- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object)

    #-- number of parameters
    cat("Rater Facet Model with Item/Rater Intercepts and Slopes \n")
    cat("-----------------------------------------------------------------\n")
    cat( "Number of iterations","=", object$iter, "\n" )
    cat( "Deviance","=", round( object$deviance, 2 ), " | " )
    cat( "Log Likelihood","=", round( -object$deviance/2, 2 ), "\n" )
    cat( "Number of persons","=", object$ic$n, "\n" )
    cat( "Number of items","=", object$ic$VV, "\n" )
    cat( "Number of raters","=", object$ic$RR, "\n" )
    cat( "Number of estimated parameters","=", object$ic$np, "\n" )
    cat( "   Number of item parameters","=", object$ic$np.item, "\n" )
    cat( "   Number of rater parameters","=", object$ic$np.rater, "\n" )
    cat( "   Number of distribution parameters","=", object$ic$np.trait, "\n\n" )

    #--- information criteria
    rm_summary_information_criteria(object=object)

    #--- trait distribution
    rm_summary_trait_distribution(object=object)

    cat("-----------------------------------------------------------------\n")
    cat("Item Parameters \n")
    obji <- object$item
    sirt_summary_print_objects(obji=obji, digits=3, from=2)

    cat("-----------------------------------------------------------------\n")
    cat("Rater Parameters \n")
    obji <- object$rater
    sirt_summary_print_objects(obji=obji, digits=3, from=2)

    sirt_csink(file=file)
}

## File Name: summary.linking.robust.R
## File Version: 0.04


summary.linking.robust <- function( object, ... )
{
    kmax <- length(object$k.robust)
    cat("Robust linking with trimmed mean\n\n")
    cat(paste0("Number of items", "=", object$I, "\n" ) )
    cat(paste0( "Optimal trimming parameter k", "=", round( object$kopt, 4 ), " | "))
    cat(paste0( " non-robust parameter k", 0, " \n"))
    cat(paste0( "Linking constant=", round( object$meanpars.kopt, 4 ), " | "))
    cat(paste0( " non-robust estimate=", round( object$meanpars[ 1 ], 4 ), " \n"))
    cat(paste0( "Standard error=", round( object$se.kopt, 4 ), " | "))
    cat(paste0( " non-robust estimate=", round( object$se[1], 4 ), " \n"))
    cat(paste0( "DIF SD: MAD=", round( object$mad, 4 ), " (robust) | "))
    cat(paste0( "SD=", round( object$sd, 4 ), " (non-robust) \n"))
}

## File Name: summary.isop.R
## File Version: 0.13


#*** Summary for isop object
summary.isop <- function( object, ... )
{
    cat("-----------------------------------------------------------------\n")
    d1 <- utils::packageDescription("sirt")
    cat( paste( d1$Package, " ", d1$Version, " (", d1$Date, ")", sep=""), "\n\n" )
    cat( "Date of Analysis:", paste( object$s2 ), "\n" )

    cat("ISOP and ADISOP Model \n")
    cat("-----------------------------------------------------------------\n")
    cat( "Number of persons=", dim(object$dat)[1], "\n" )
    cat( "Number of items=", dim(object$dat)[2], "\n\n" )

    cat( "Number of person score groups=", dim(object$freq.correct)[1], "\n" )
    cat( "Number of item score groups=", dim(object$freq.correct)[2], "\n" )

    cat( "\nLog-Likelihood Comparison\n")
    obji <- object$ll
    obji[,2] <- round( obji[,2], 3 )
    obji[,3] <- round( obji[,3], 5 )
    if ( object$model=="np.dich" ){
        obji$fit <- round( c(NA,object$fit ), 4 )
    }
    print(obji)

    cat("*****************************************************************\n")
    cat("Item Statistics and Scoring\n")
    obji <- object$item
    for (vv in 2:( ncol(obji))){
        obji[,vv] <- round( obji[,vv], 4 )
    }
    print(obji)
}

## File Name: rm_sdt_create_partable_include_index.R
## File Version: 0.05


rm_sdt_create_partable_include_index <- function(partable)
{
    partable <- data.frame( index=1:nrow(partable), partable)
    return(partable)
}

## File Name: xxirt_hessian.R
## File Version: 0.585

#--- computation of hessian matrix
xxirt_hessian <- function( object, h=1e-4, use_shortcut=TRUE )
{
    item_list <- object$item_list
    items <- object$items
    Theta <- object$Theta
    ncat <- object$ncat
    partable <- object$partable
    partable_index <- object$partable_index
    dat <- as.matrix(object$dat)
    dat_resp <- object$dat_resp
    dat_resp_bool <- object$dat_resp_bool
    dat1 <- object$dat1
    dat1_resp <- object$dat1_resp
    resp_index <- object$resp_index
    G <- object$G
    group <- object$group
    maxK <- object$maxK
    group_index <- object$group_index
    weights <- object$weights
    customTheta <- object$customTheta
    par_items <- object$par_items
    par_Theta <- object$par_Theta
    NPI <- length(par_items)
    NPT <- length(par_Theta)

    #** preliminary computations
    probs_items <- xxirt_compute_itemprobs( item_list=item_list,
                                items=items, Theta=Theta, ncat=ncat,
                                partable=partable, partable_index=partable_index )

    par1 <- xxirt_partable_extract_freeParameters( partable=partable )

    p.xi.aj <- xxirt_compute_likelihood( probs_items=probs_items, dat=dat,
                             resp_index=resp_index, dat_resp_bool=dat_resp_bool )
    prior_Theta <- xxirt_compute_priorDistribution( Theta=Theta,
                                  customTheta=customTheta, G=G )
    prior1 <- t( prior_Theta[, group ] )
    probs_items0 <- probs_items
    p_xi_aj <- p.xi.aj

    #--- compute Hessian matrix
    par1 <- xxirt_partable_extract_freeParameters( partable=partable )
    par2 <- xxirt_parTheta_extract_freeParameters( customTheta=customTheta )
    par0 <- par <- c(par1, par2)

    #** detect whether there are item-wise parameters
    a1 <- stats::aggregate( partable$itemnr, list(partable$parindex), min )
    a2 <- stats::aggregate( partable$itemnr, list(partable$parindex), max )
    item_wise <- sum(a2[,2]> a1[,2])==0

    #***********************
    fct_irt <- function(x, prior1, par0, item_wise=FALSE, probs_items0,
                    p_xi_aj)
    {
        eps <- 1e-10
        I <- dim(probs_items0)[1]
        #*** include free paramaters in partable
        partable <- xxirt_partable_include_freeParameters( partable, x=x[ 1L:NPI ] )
        #**** include parameter in customTheta
        customTheta$par[ customTheta$est ] <- x[ (NPI+1):(NPI+NPT) ]
        #*** different parameters
        ind <- which(abs(x-par0)>eps)
        max_ind <- 0
        min_ind <- 0
        IA <- 100
        if (length(ind)>0){
            max_ind <- max(ind)
            min_ind <- min(ind)
            items_active <- unique(partable[ partable$parindex %in% ind, 'itemnr' ])
            if (length(items_active)>0){
                IA <- length(items_active)
            }
        }

        #*** compute individual likelihood
        # if ( (! item_wise) | (min_ind==0) | (max_ind>=NPI+1 ) ){

        use_shortcut <- use_shortcut & (IA <=2)

        if (! use_shortcut){
            if (min_ind <=NPI){
                probs_items <- xxirt_compute_itemprobs( item_list=item_list,
                                    items=items, Theta=Theta, ncat=ncat,
                                    partable=partable, partable_index=partable_index )
            }
            p.xi.aj <- xxirt_compute_likelihood( probs_items=probs_items, dat=dat,
                             resp_index=NULL, dat_resp_bool=dat_resp_bool )
        } else {
            itemnr <- partable[ partable$parindex==min_ind, 'itemnr' ]
            itemnr2 <- partable[ partable$parindex==max_ind, 'itemnr' ]
            maxK <- dim(probs_items)[2]
            TP <- dim(probs_items)[3]
            eps <- 1e-16
            use_itemnr2 <- FALSE
            if (length(itemnr2)>0){
                if (itemnr2!=itemnr){
                    use_itemnr2 <- TRUE
                }
            } else {
                if (itemnr>1){
                    itemnr2 <- 1
                } else {
                    itemnr2 <- 2
                }
            }

            item_index <- c(itemnr, itemnr2)
            if (item_index[2]==item_index[1]){
                if (item_index[1]==1){
                    item_index[2] <- 2
                } else {
                    item_index[2] <- 1
                }
            }
            probs_items_temp <- xxirt_compute_itemprobs( item_list=item_list,
                                    items=items, Theta=Theta, ncat=ncat,
                                    partable=partable, partable_index=partable_index,
                                    item_index=item_index)
            probs_ratio1 <- probs_items0

            p0 <- probs_items0[item_index,,]+eps
            probs_ratio1[item_index,,] <- probs_items_temp / p0
            probs_ratio1 <- matrix( probs_ratio1, nrow=I, ncol=maxK*TP )

            p.xi.aj <- sirt_rcpp_xxirt_hessian_reduced_probs(dat=dat,
                            dat_resp_bool=dat_resp_bool, probs_ratio=probs_ratio1,
                            TP=TP, maxK=maxK, itemnr=itemnr-1, itemnr2=itemnr2-1,
                            use_itemnr2=use_itemnr2, p_xi_aj=p_xi_aj)

        }

        #*** compute prior distribution
        if (max_ind >=NPI+1){
            prior_Theta <- xxirt_compute_priorDistribution( Theta=Theta,
                                  customTheta=customTheta, G=G )
            prior1 <- t( prior_Theta[, group ] )
        }
        dev <- xxirt_hessian_compute_loglike(p.xi.aj=p.xi.aj, prior1=prior1,
                    weights=weights)
        return(dev)
    }
    #*******************************

    hess <- CDM::numerical_Hessian( par=par, FUN=fct_irt, h=h, prior1=prior1,
                        par0=par, item_wise=item_wise, probs_items0=probs_items0,
                        p_xi_aj=p_xi_aj)
    rownames(hess) <- colnames(hess) <- names(par)
    return(hess)
}

## File Name: gom_em_normal_to_membership_scores.R
## File Version: 0.03


gom_em_normal_to_membership_scores <- function(theta_grid, K, TP)
{
    theta0 <- matrix(0, nrow=TP, ncol=K)
    for (kk in 1:(K-1)){
        theta0[,kk]    <- theta_grid[,kk]
    }
    theta0_rowmax <- rowMaxs.sirt(matr=theta0)$maxval
    theta0 <- theta0 - theta0_rowmax
    theta.k <- exp(theta0)
    theta.k <- theta.k / rowSums(theta.k)
    return(theta.k)
}

## File Name: sirt_optimizer.R
## File Version: 0.372

sirt_optimizer <- function(optimizer, par, fn, grad=NULL, method="L-BFGS-B",
        hessian=TRUE, control=list(), ...)
{
    s1 <- Sys.time()
    h <- 1e-5

    #-- optim
    if (optimizer=='optim'){
        res <- stats::optim(par=par, fn=fn, gr=grad, method=method,
                        hessian=hessian, control=control, ...)
        res$iter <- res$counts['function']
    }

    #-- Rvmmin
    if (optimizer=='Rvmmin'){
        requireNamespace('optimx')
        args <- list(...)
        NP <- length(par)
        if ( ! ( 'lower' %in% names(args) ) ){
            args$lower <- rep(-Inf,NP)
        }
        if ( ! ( 'upper' %in% names(args) ) ){
            args$upper <- rep(Inf,NP)
        }
        # args$bdmsk <- 1*( ( args$lower > - Inf ) + ( args$upper < Inf )==0 )
        args$par <- par
        args$fn <- fn
        args$gr <- grad
        args$method <- 'Rvmmin'
        args$control <- control
        optimx_fun <- optimx::optimx
        res0 <- do.call(what=optimx_fun, args=args)
        # res <- optimx::Rvmmin(par=par, fn=fn, gr=grad, control=control, ...)
        res <- list()
        m1 <- as.vector(as.numeric(res0[1,1L:NP]))
        res$par <- m1
        res$convergence <- res0$convcode[1]
        res$iter <- res0$feval[1]
        res$value <- res0$value[1]
    }

    #-- nlminb
    if (optimizer=='nlminb'){
        res <- stats::nlminb(start=par, objective=fn, gradient=grad,
                        control=control, ...)
        res$value <- res$objective
        res$iter <- res$iterations
    }

    #-- bobyqa
    if (optimizer=='bobyqa'){
        requireNamespace('minqa')
        # control=list(iprint=2, maxfun=10000, ...)
        res <- minqa::bobyqa(par=par, fn=fn, control=control, ...)
        res$value <- res$fval
        res$iter <- res$feval
        res$convergence <- res$ierr
    }

    #-- nloptr
    if (optimizer=='nloptr'){
        requireNamespace('nloptr')
        # print_level, maxevel
        # control$algorithm <- 'NLOPT_LD_LBFGS'
        control$algorithm <- 'NLOPT_LD_MMA'
        args <- list(...)
        args <- sirt_rename_list_names(x=args, old='lower', new='lb')
        args <- sirt_rename_list_names(x=args, old='upper', new='ub')
        args$x0 <- par
        args$eval_f <- fn
        args$eval_grad_f <- grad
        args$opts <- control
        res <- do.call( what=nloptr::nloptr, args=args )
        res$par <- res$solution
        res$value <- res$objective
        res$iter <- res$iterations
        res$convergence <- 1 - ( res$status %in% c(4) )
    }

    #*****
    #-- compute Hessian
    comp_hess_optimizers <- c('nlminb', 'bobyqa')
    if (hessian & ( optimizer %in% comp_hess_optimizers ) ){
        res <- sirt_optimizer_hessian(res=res, fn=fn, grad=grad, h=h, ...)
    }

    #-- names for Hessian
    if (!is.null(res$hessian) & (!is.null(names(par))) ){
        rownames(res$hessian) <- colnames(res$hessian) <- names(par)
    }

    s2 <- Sys.time()
    res$time_diff <- s2 - s1
    res$optimizer <- optimizer
    res$converged <- res$convergence==0
    #--- output
    return(res)
}

## File Name: rm_facets_postproc_person.R
## File Version: 0.11

rm_facets_postproc_person <- function( dat2, dat2.resp, procdata, maxK, RR, theta.k, f.qk.yi )
{
    person <- procdata$person.index
    NP <- nrow(person)
    person$score <- rowSums( dat2 * dat2.resp )
    mkrr <- rep( maxK, RR )
    person$maxscore <- rowSums( dat2.resp * sirt_matrix2( mkrr, nrow=NP) )
    person$EAP <- rowSums( f.qk.yi * sirt_matrix2( theta.k, nrow=NP) )
    person$SE.EAP <- sqrt( rowSums( f.qk.yi * sirt_matrix2( theta.k^2, nrow=NP) ) - ( person$EAP) ^2 )
    EAP.rel <- rm_eap_reliability( EAP=person$EAP, SE_EAP=person$SE.EAP )
    #--- output
    res <- list( person=person, EAP.rel=EAP.rel )
    return(res)
}

## File Name: summary.smirt.R
## File Version: 0.18
#*******************************************************
# Summary for smirt object
summary.smirt <- function( object, ...){
    cat("-----------------------------------------------------------------\n")
    d1 <- utils::packageDescription("sirt")
    cat( paste( d1$Package, " ", d1$Version, " (", d1$Date, ")", sep=""), "\n\n" )
    cat( "Date of Analysis:", paste( object$s2 ), "\n" )
    cat("Computation Time:", print(object$s2 - object$s1), "\n\n")
    cat("  Function 'smirt' \n")
    if (object$irtmodel=="noncomp"){
        cat("   Noncompensatory item response model\n\n")
                    }
    if (object$irtmodel=="comp"){
        cat("   Compensatory item response model\n\n")
                    }
    if (object$irtmodel=="partcomp"){
        cat("   Partially compensatory item response model\n\n")
                    }

#    modeltype <- object$irtmodel
        cat( "   ", object$ic$n, "Cases, ", ncol(object$dat2), "Items, ", # object$G, "Group(s)", ",",
                object$D, "Dimension(s)\n")

    cat("-----------------------------------------------------------------\n")
    cat( "Number of iterations=", object$iter, "\n" )
    cat( "Deviance=", round( object$deviance, 2 ), " | " )
    cat( "Log Likelihood=", round( -object$deviance/2, 2 ), "\n" )
    cat( "Number of persons=", object$ic$n, "\n" )

    cat( "Number of estimated parameters=", object$ic$np, "\n" )
    cat( "  Number of estimated item parameters=", object$ic$np.item, "\n" )
    cat( "               b parameters=", object$ic$np.item.b, "\n" )
    cat( "               a parameters=", object$ic$np.item.a, "\n" )
    cat( "               c parameters=", object$ic$np.item.c, "\n" )
    cat( "               d parameters=", object$ic$np.item.d, "\n" )
    cat( "            mu.i parameters=", object$ic$np.item.mu.i, "\n" )
    cat( "  Number of estimated distribution parameters=", object$ic$np.cov,
                "\n" )
    cat( "               Means=", object$ic$np.cov.mu, "\n" )
    cat( "               Covariances=", object$ic$np.cov.covM, "\n" )
    cat( "AIC=", round( object$ic$AIC, 2 ), " | penalty=", round( object$ic$AIC - object$ic$deviance,2 ),
            "   | AIC=-2*LL + 2*p  \n" )
    cat( "AICc=", round( object$ic$AICc, 2 )," | penalty=", round( object$ic$AICc - object$ic$deviance,2 ) )
        cat("    | AICc=-2*LL + 2*p + 2*p*(p+1)/(n-p-1)  (bias corrected AIC)\n" )
    cat( "BIC=", round( object$ic$BIC, 2 ), " | penalty=", round( object$ic$BIC - object$ic$deviance,2 ),
            "   | BIC=-2*LL + log(n)*p  \n" )
    cat( "CAIC=", round( object$ic$CAIC, 2 )," | penalty=", round( object$ic$CAIC - object$ic$deviance,2 ) )
        cat("   | CAIC=-2*LL + [log(n)+1]*p  (consistent AIC)\n\n" )

    cat("-----------------------------------------------------------------\n")
#    cat("Trait Distribution\n")
#    obji <- object$pi.k
    cat( "\nM Trait:\n" )
    print( round( object$mean.trait, 3 )  )
    cat( "\nSD Trait:\n" )
    print( round( object$sd.trait, 3 ) )
    cat( "\nCorrelations Trait: \n" )
    print( round( object$cor.trait, 3 ) )
    cat( "\nEAP Reliability:\n" )
    print( round( t(object$EAP.rel ), 3 ) )
    cat("-----------------------------------------------------------------\n")
    cat("Item Parameters \n")
    obji <- object$item
    obji[,-1] <- round( obji[,-1], 3)
    print( obji )
#  cat("\nMean of RMSEA item fit:",
#     round( object$mean.rmsea,3 ), "\n")
            }
#*******************************************************

## File Name: rasch.pml_aux.R
## File Version: 1.21



######################################################################
# calculation of pairwise marginal likelihood
.ll.rasch.pml.probit <- function( b, a, sigma, Q,eps.corr, itempairs, IP, eps=10^(-14) ){
    TAM::require_namespace_msg("pbivnorm")
    # multidimensional case
    if ( ! is.null(Q) ){
        t1 <- a^2 * diag( ( Q %*% sigma %*% t(Q) ) )
                }
    # unidimensional case
    if ( is.null(Q) ){
        t1 <- a^2*sigma^2
                    }
    xi <-   - b  / sqrt( 1 + t1 )
    xi1 <- xi[ itempairs[,"item1"] ]
    xi2 <- xi[ itempairs[,"item2"] ]
    a1 <- a[ itempairs[,"item1"] ]
    a2 <- a[ itempairs[,"item2"] ]
    if ( ! is.null(Q) ){
        t1 <- a1  * a2 * diag( ( Q[ itempairs[,"item1"], ] %*% sigma %*% t(Q[  itempairs[,"item2"], ]) ) )
                }
    if (is.null(Q) ){  # 1-dimensional case
        t1 <- a1*a2*sigma^2
                }
    cor.Sigma <- ( t1 + eps.corr ) / ( 1 + t1 )
    cor.Sigma[ cor.Sigma > 1 ] <- .99
    a1 <- stats::pnorm( xi )

    itempairs$p1.item1 <- a1[ itempairs$item1 ]
    itempairs$p1.item2 <- a1[ itempairs$item2 ]
    itempairs$p11 <- pbivnorm::pbivnorm( x=xi1, y=xi2, rho=cor.Sigma )
    itempairs$p10 <- itempairs$p1.item1 - itempairs$p11
    itempairs$p01 <- itempairs$p1.item2 - itempairs$p11
    itempairs$p00 <- 1 - itempairs$p11 - itempairs$p01 - itempairs$p10
    ind1 <- which( colnames(itempairs) %in% c( "p11", "p10", "p01", "p00" ) )
#    itempairs[ itempairs[, ind1 ] < eps, ind1 ] <- eps
    for (ii in ind1){
        itempairs[, ii ] <- ifelse( itempairs[,ii] < eps, eps, itempairs[,ii] )
                }
    ind2 <- which( colnames(itempairs) %in% c( "f11", "f10", "f01", "f00" ) )
    ll <-  sum( log( itempairs[,ind1] ) * itempairs[,ind2]  )
    res <-list( "ll"=ll, "itempairs"=itempairs, "b"=b, "sigma"=sigma,
                "ind1"=ind1, "ind2"=ind2 )
    return(res)
        }
######################################################################
#              respml <- .update.ll.rasch.pml( respml, b, sigma, itempairs, IP, eps=10^(-14) )
######################################################################
# update calculation of pairwise marginal likelihood
.update.ll.rasch.pml.probit <- function( respml, b, a, sigma, Q,eps.corr,
                        itempairs, IP, eps=10^(-14) ){
    TAM::require_namespace_msg("pbivnorm")
    # multidimensional case
    if ( ! is.null(Q) ){
        t1 <- a^2 * diag( ( Q %*% sigma %*% t(Q) ) )
                }
    # unidimensional case
    if ( is.null(Q) ){
        t1 <- a^2*sigma^2
                    }
    xi <-   - b  / sqrt( 1 + t1 )
    itempairs <- respml$itempairs
    xi1 <- xi[ itempairs[,"item1"] ]
    xi2 <- xi[ itempairs[,"item2"] ]
    a1 <- a[ itempairs[,"item1"] ]
    a2 <- a[ itempairs[,"item2"] ]
    if ( ! is.null(Q) ){
        t1 <- a1  * a2 * diag( ( Q[ itempairs[,"item1"], ] %*% sigma %*% t(Q[  itempairs[,"item2"], ]) ) )
                }
    if (is.null(Q) ){  # 1-dimensional case
        t1 <- a1*a2*sigma^2
                }
    cor.Sigma <- ( t1 + eps.corr ) / ( 1 + t1 )
#    cor.Sigma <- ( a1*a2*sigma^2 + eps.corr ) / ( 1 + a1*a2*sigma^2 )
#        Sigma.ii <- matrix( c(1,0,0,1), ncol=2 )
#    Sigma.ii[1,2] <- Sigma.ii[2,1] <- cor.Sigma[1]
    a1 <- stats::pnorm( xi )
    itempairs$p1.item1 <- a1[ itempairs$item1 ]
    itempairs$p1.item2 <- a1[ itempairs$item2 ]
    ind10 <- which( b !=respml$b )
    ind10 <- c( which( itempairs$item1 %in% ind10  ), which( itempairs$item2 %in% ind10  )  )
    itempairs[ind10,"p11"] <- pbivnorm::pbivnorm( x=xi1[ind10], y=xi2[ind10],
            rho=cor.Sigma[ind10] )
    itempairs$p10 <- itempairs$p1.item1 - itempairs$p11
    itempairs$p01 <- itempairs$p1.item2 - itempairs$p11
    itempairs$p00 <- 1 - itempairs$p11 - itempairs$p01 - itempairs$p10
    ind1 <- which( colnames(itempairs) %in% c( "p11", "p10", "p01", "p00" ) )
    itempairs[ itempairs[, ind1 ] < eps, ind1 ] <- eps
    ind2 <- which( colnames(itempairs) %in% c( "f11", "f10", "f01", "f00" ) )
    ll <-  sum( log( itempairs[,ind1] ) * itempairs[,ind2]  )
    res <- list( "ll"=ll, "itempairs"=itempairs, "b"=b, "sigma"=sigma,
                "ind1"=ind1, "ind2"=ind2 )
    return(res)
        }
######################################################################




######################################################################
# calculation of pairwise marginal likelihood
.ll.rasch.pml.logit <- function( b, sigma, itempairs, IP, eps=10^(-14),
            p.ki, S.ki ){
    K <- length(p.ki)
    xi <- matrix( 0, I, K )
    xi1 <- xi2 <- matrix( 0, IP, K )
    cor.Sigma <- rep(0,K)
    Sigma0 <- matrix( c(1,0,0,1), ncol=2 )
    Sigma.ii <- as.list( rep(1,K))
    for (kk in 1:K){
        xi[,kk] <-   - b * S.ki[kk] / sqrt( 1 + sigma^2 * S.ki[kk]^2 )
        xi1[,kk] <- xi[ itempairs[,"item1"], kk ]
        xi2[,kk] <- xi[ itempairs[,"item2"], kk ]
            }
    a1 <- stats::pnorm( xi )
    PKI <- outer( rep(1,I), p.ki )
    a1 <- rowSums( a1 * PKI )
    itempairs$p1.item1 <- a1[ itempairs$item1 ]
    itempairs$p1.item2 <- a1[ itempairs$item2 ]
    #********
    # calculation of matrix of covariances
    phi.matrix <- matrix( 0, IP, 9)
    PKI.matrix <- phi.matrix
    for (ii in 1:IP){
        for (kk1 in 1:3){
            for (kk2 in 1:3){
            #    kk1 <- 2
            #    kk2 <- 3
                index1 <- 3* ( kk1 - 1 ) + kk2
                index2 <- 3*(kk2 - 1) + kk1
                cor.kk <- S.ki[kk1] * S.ki[kk2] * sigma^2 / sqrt( ( 1 + sigma^2 * S.ki[kk1]^2 ) *
                                    ( 1 + sigma^2 * S.ki[kk2]^2 ) )
                phi.matrix[ii,index1] <- sirt_pmvnorm(lower=c(-Inf,-Inf),upper=c(xi1[ii,kk1],xi2[ii,kk2] ),
                        mean=c(0,0),
                        sigma=matrix( c(1,cor.kk,cor.kk,1), 2, 2 )
                                    )
                PKI.matrix[ii,index1] <- p.ki[kk1] * p.ki[kk2]
#                print( paste( kk1, kk2, index1, index2 ))
                            }
                        }
                    }
    itempairs$p11 <- rowSums( phi.matrix * PKI.matrix )
    itempairs$p10 <- itempairs$p1.item1 - itempairs$p11
    itempairs$p01 <- itempairs$p1.item2 - itempairs$p11
    itempairs$p00 <- 1 - itempairs$p11 - itempairs$p01 - itempairs$p10
    ind1 <- which( colnames(itempairs) %in% c( "p11", "p10", "p01", "p00" ) )
    itempairs[ itempairs[, ind1 ] < eps, ind1 ] <- eps
    ind2 <- which( colnames(itempairs) %in% c( "f11", "f10", "f01", "f00" ) )
    ll <-  sum( log( itempairs[,ind1] ) * itempairs[,ind2]  )
    res <- list( "ll"=ll, "itempairs"=itempairs, "b"=b, "sigma"=sigma,
                "ind1"=ind1, "ind2"=ind2, "phi.matrix"=phi.matrix,
                "PKI.matrix"=PKI.matrix,
                "p.ki"=p.ki, "S.ki"=S.ki)
    return(res)
        }
######################################################################
######################################################################
# update calculation of pairwise marginal likelihood
.update.ll.rasch.pml.logit <- function( respml, b, sigma, itempairs, IP, eps=10^(-14) ){
    # grap objects
#    p.ki <- respml[["p.ki"]]
#    S.ki <- respml[["S.ki"]]
        p.ki <- c( .25220, .58522, .16257 )
        S.ki <- c( .90793, .57778, .36403 )
#print(respml)
    phi.matrix <- respml$phi.matrix
    PKI.matrix <- respml$PKI.matrix
    itempairs <- respml$itempairs
    #*******************
    K <- length(p.ki)
    xi <- matrix( 0, I, K )
    xi1 <- xi2 <- matrix( 0, IP, K )
    cor.Sigma <- rep(0,K)
    Sigma0 <- matrix( c(1,0,0,1), ncol=2 )
    Sigma.ii <- as.list( rep(1,K))
    for (kk in 1:K){
        xi[,kk] <-   - b * S.ki[kk] / sqrt( 1 + sigma^2 * S.ki[kk]^2 )
        xi1[,kk] <- xi[ itempairs[,"item1"], kk ]
        xi2[,kk] <- xi[ itempairs[,"item2"], kk ]
            }
    a1 <- stats::pnorm( xi )
    PKI <- outer( rep(1,I), p.ki )
    a1 <- rowSums( a1 * PKI )
    itempairs$p1.item1 <- a1[ itempairs$item1 ]
    itempairs$p1.item2 <- a1[ itempairs$item2 ]
    #********
    # calculation of matrix of covariances
    ind10 <- which( b !=respml$b )
    ind10 <- c( which( itempairs$item1 %in% ind10  ), which( itempairs$item2 %in% ind10  )  )
    for (ii in ind10 ){
        for (kk1 in 1:3){
            for (kk2 in 1:3){
            #    kk1 <- 2
            #    kk2 <- 3
                index1 <- 3* ( kk1 - 1 ) + kk2
                index2 <- 3*(kk2 - 1) + kk1
                cor.kk <- S.ki[kk1] * S.ki[kk2] * sigma^2 / sqrt( ( 1 + sigma^2 * S.ki[kk1]^2 ) *
                                    ( 1 + sigma^2 * S.ki[kk2]^2 ) )
                phi.matrix[ii,index1] <- sirt_pmvnorm(lower=c(-Inf,-Inf),upper=c(xi1[ii,kk1],xi2[ii,kk2] ),
                        mean=c(0,0),
                        sigma=matrix( c(1,cor.kk,cor.kk,1), 2, 2 )
                                    )
                PKI.matrix[ii,index1] <- p.ki[kk1] * p.ki[kk2]
#                print( paste( kk1, kk2, index1, index2 ))
                            }
                        }
                    }
    itempairs$p11 <- rowSums( phi.matrix * PKI.matrix )
    itempairs$p10 <- itempairs$p1.item1 - itempairs$p11
    itempairs$p01 <- itempairs$p1.item2 - itempairs$p11
    itempairs$p00 <- 1 - itempairs$p11 - itempairs$p01 - itempairs$p10
    ind1 <- which( colnames(itempairs) %in% c( "p11", "p10", "p01", "p00" ) )
    itempairs[ itempairs[, ind1 ] < eps, ind1 ] <- eps
    ind2 <- which( colnames(itempairs) %in% c( "f11", "f10", "f01", "f00" ) )
    ll <-  sum( log( itempairs[,ind1] ) * itempairs[,ind2]  )
    res <-list( "ll"=ll, "itempairs"=itempairs, "b"=b, "sigma"=sigma,
                "ind1"=ind1, "ind2"=ind2, "phi.matrix"=phi.matrix,
                "PKI.matrix"=PKI.matrix,
                "p.ki"=p.ki, "S.ki"=S.ki)
    return(res)
        }
######################################################################

## File Name: rm_ic_criteria.R
## File Version: 0.05

rm_ic_criteria <- function(ic)
{
    dev <- ic$deviance
    # AIC
    ic$AIC <- dev + 2*ic$np
    # BIC
    ic$BIC <- dev + ( log(ic$n) )*ic$np
    # CAIC (consistent AIC)
    ic$CAIC <- dev + ( log(ic$n) + 1 )*ic$np
    # corrected AIC
    ic$AICc <- ic$AIC + 2*ic$np * ( ic$np + 1 ) / ( ic$n - ic$np - 1 )
    #---- output
    return(ic)
}

## File Name: likelihood_adjustment.R
## File Version: 0.182


#*** likelihood adjustment
likelihood.adjustment <- function( likelihood, theta=NULL, prob.theta=NULL,
            adjfac=rep(1,nrow(likelihood)), extreme.item=5,
            target.EAP.rel=NULL, min_tuning=.2, max_tuning=3,
            maxiter=100, conv=.0001, trait.normal=TRUE )
{

    like0 <- likelihood
    eps <- 1E-30
    normal_approx <- trait.normal

    if ( is.null(theta) ){
        theta <- attr( likelihood, 'theta' )[,1]
    }

    if ( is.null(prob.theta) ){
        prob.theta <- attr( likelihood, 'prob.theta' )
    }


    attr(likelihood,'prob.theta') <- NULL
    attr(likelihood,'theta') <- NULL
    attr(likelihood,'G') <- NULL

    #-- add extreme item
    N <- nrow(like0)
    TP <- length(theta)
    thetaM <- matrix( theta, nrow=N, ncol=TP, byrow=TRUE)
    S1 <- stats::plogis( thetaM + extreme.item ) *
                ( 1 - stats::plogis( thetaM - extreme.item ) )
    likelihood <- likelihood * S1

    # likelihood adjustment
    like2 <- likelihood_adjustment_compute( likelihood, theta, thetaM, adjfac )

    #*** compute posterior given likelihood and empirical prior
    if ( ! is.null( target.EAP.rel ) ){
        probs <- prob.theta
        probsM <- matrix( prob.theta, nrow=N, ncol=TP, byrow=TRUE)

        tuning1 <- min_tuning
        tuning2 <- max_tuning


        EAP_rel1 <- likelihood_adjustment_tuning( likelihood, theta, thetaM, adjfac,
                           tuningfac=tuning1, probs=probs, normal_approx=normal_approx )
        EAP_rel2 <- likelihood_adjustment_tuning( likelihood, theta, thetaM, adjfac,
                           tuningfac=tuning2, probs=probs, normal_approx=normal_approx)

        iter <- 0
        change <- 1
        while( ( iter < maxiter ) & ( change > conv ) ){
            tuning0 <- ( tuning1 + tuning2 ) / 2
            res1 <- likelihood_adjustment_tuning( likelihood, theta, thetaM, adjfac,
                            tuningfac=tuning0, probs=probs, normal_approx=normal_approx)
            EAP_rel0 <- res1$EAP.rel
            like2 <- res1$likelihood
            if ( EAP_rel0 < target.EAP.rel ){
                tuning2 <- tuning0
            } else {
                tuning1 <- tuning0
            }
            iter <- iter + 1
            change <- abs( EAP_rel0 - target.EAP.rel )
            cat('Iteration ', iter, ' | EAP reliability=', round( EAP_rel0, 4 ), '\n')
            flush.console()
        }
    }
    res <- like2
    attr( res, 'theta' ) <- matrix( theta, ncol=1)
    attr(like0,'prob.theta') -> attr( res, 'prob.theta')
    attr(like0,'G') -> attr(res, 'G')
    return(res)
}

## File Name: gom_em_calc_theta.R
## File Version: 0.181


#--- calculate theta grid
gom_em_calc_theta <- function( K, problevels, eps=1e-5 )
{
    m1 <- problevels
    if ( ! is.matrix(problevels) ){
        PL <- length(problevels)
        m1 <- matrix(problevels, PL, 1 )
        for (kk in 2L:K){
            NM <- nrow(m1)
            m1 <- cbind( m1[ rep( 1L:NM, PL), ], rep( problevels, each=NM)  )
            m1 <- m1[ rowSums(m1) <=1, ]
        }
    }
    m1 <- m1[ abs( rowSums(m1) - 1 ) < eps, ]
    return(m1)
}



.gom.calc.theta <- gom_em_calc_theta

## File Name: mgsem_power_fun_differentiable_approx.R
## File Version: 0.096

mgsem_power_fun_differentiable_approx <- function(x, p, eps, deriv=FALSE,
        approx_method="lp")
{
    # logcomp <- TRUE
    logcomp <- FALSE
    if (deriv){  ## derivative
        if (approx_method=='lp'){
            if (p>0){
                if (!logcomp){
                    res <- p*((x^2+eps)^(p/2-1))*x
                } else {
                    logp <- log(p)
                    p2 <- (p/2-1)
                    res <- x*exp( p2*log(x^2+eps) + logp )
                }
            } else { # p=0
                res <- 2*x*eps/(x^2+eps)^2
            }
        }
        if (approx_method=='l2'){
            res <- 2*x
        }
    } else {  # no derivative
        if (approx_method=='lp'){
            if (p>0){
                res <- (x^2+eps)^(p/2)
            } else { # p=0
                res <- x^2 / (x^2 + eps)
            }
        }
        if (approx_method=='l2'){
            res <- x^2
        }
    }
    return(res)
}

## File Name: rasch.mml.npirt.R
## File Version: 2.13



#########################################################################################
.mstep.mml.npirt <- function( pjk, r.jk, n.jk, theta.k, npformula, npmodel, G, I,
            npirt.monotone, ICC_model_matrix ){
                    rjk0 <- r.jk[,,1]
                    njk0 <- n.jk[,,1]
                    if (G > 1){
                        for (gg in 2:G){
                            rjk0 <- rjk0 + r.jk[,,gg]
                            njk0 <- njk0 + n.jk[,,gg]
                                    }
                                }

        if ( is.null( npformula ) ){
                    pjk <- t(rjk0 / njk0 )
                    if (npirt.monotone){
                        # monotone smoothing
                        pjk <- monoreg.colwise(yM=pjk, wM=t(njk0) )
                                    }
                    }

        # estimation using a formula for ICC estimation
        if ( ! is.null(npformula) ) {
                LK <- length(theta.k)
                        cat("ICC estimation |")
                        prbar <- floor( 10 * ( 1:I )    / I )
                        prbar <- c( 1, diff(prbar))
                        for (ii in 1:I){
                                #ii <- 3
#                            dfr1 <- data.frame( "theta"=theta.k, "y"=1, "wgt"=rjk0[ii,] )
#                            dfr0 <- data.frame( "theta"=theta.k, "y"=0, "wgt"=njk0[ii,] - rjk0[ii,] )
#                            dafr <- data.frame( rbind( dfr0, dfr1 ) )
                            if ( prbar[ii]==1){ cat("~"); utils::flush.console() }
#                            theta <- dafr$theta.k
#                            wgt <- dafr$wgt
#                            y <- dafr$y
#                            ICC_ <- model.matrix( npformula[[ii]], dafr )
                            y <- rep( c(0,1), each=LK)
                            wgt <- c( njk0[ii,] - rjk0[ii,], rjk0[ii,]  )
                            ICC_ <- ICC_model_matrix[[ii]]
                            npmodel[[ii]] <- stats::glm( y ~ 0 + ICC_, weights=wgt, family="binomial",
                                        control=list(maxit=4) )
#                            npmodel[[ii]] <- glm( npformula[[ii]],
#                                        data=dafr, weights=dafr$wgt, family="binomial")
                            pjk[,ii] <- stats::fitted( npmodel[[ii]] )[ seq( 1, LK ) + LK ]
                                }
                            cat("\n")
                        }
                    res <- list( "pjk"=pjk, "npmodel"=npmodel)
                            }
##########################################################################################

## File Name: smirt_alg_partcomp.R
## File Version: 0.15

############################################
# probability in noncompensatory model
## extern "C" {
## SEXP SMIRT_CALCPROB_NONCOMP( SEXP a, SEXP b, SEXP Q, SEXP thetak, SEXP cc, SEXP dd) ;
calcprob.partcomp <- function (a,b,Q,thetak,cc,dd,mu.i){
    #     Rcpp::NumericMatrix A(a);
    #     Rcpp::NumericMatrix B(b);
    #     Rcpp::NumericMatrix QQ(Q);
    #     Rcpp::NumericMatrix THETA(thetak);
    #     Rcpp::NumericVector CC(cc);
    #     Rcpp::NumericVector DD(dd);
    #     Rcpp::NumericVector MUI(mui) ;
    #**********
    # SEXP SMIRT_CALCPROB_PARTCOMP( SEXP a, SEXP b, SEXP Q, SEXP thetak, SEXP cc, SEXP dd,
    #    SEXP mui ){
    SMIRT_CALCPROB_PARTCOMP(a,b,Q,thetak,cc,dd, mu.i )
                    }



###########################################
# estimation of b
.smirt.est.b.partcomp <- function(   b, a, c, d, mu.i, Qmatrix, est.b, theta.k,
        n.ik, I, K, TP, D, numdiff.parm=.001, max.increment=1,
        msteps,  mstepconv, increment.factor){
    h <- numdiff.parm
    diffindex <- est.b        # zeros are allowed!
    cat("  M steps b parameter   |")
    it <- 0 ;    conv1 <- 1000
    Q2 <- Q1 <- 0*Qmatrix
    Q <- Qmatrix
    se.b <- b
    b00 <- b
    while( ( it < msteps ) & ( conv1 > mstepconv ) ){
        b0 <- b
        for (dd in 1:D){
#                dd <- 2
            Q2 <- Q1
            Q2[,dd] <- 1 * ( Qmatrix[,dd] !=0 )

            probres <- calcprob.partcomp( a, b, Q, thetak=theta.k, c, d, mu.i)
            pjk <- problong2probarray( probres, I, TP )

            probres <- calcprob.partcomp( a, b + h*Q2, Q, thetak=theta.k, c, d, mu.i)
            pjk1 <- problong2probarray( probres, I, TP )

            probres <- calcprob.partcomp( a, b - h*Q2, Q, thetak=theta.k, c, d, mu.i)
            pjk2 <- problong2probarray( probres, I, TP )

            # numerical differentiation
            res <- .rm.numdiff.index( pjk, pjk1, pjk2, n.ik, diffindex[,dd],
                    max.increment=max.increment, numdiff.parm )
            ind <- match( diffindex[,dd], sort(unique( diffindex[,dd] )) )
            b[,dd] <- b[,dd] + (res$increment)[ind]
            se.b[,dd] <- (sqrt(  1 / abs(res$d2) ))[ind]
                        }   # end dd
        conv1 <- max( abs( b - b0 ) )
        it <- it+1
        cat("-") # ; flush.console()
            }
    cat(" ", it, "Step(s) \n")    #; flush.console()
    if ( increment.factor > 1){
        b <- .adj.maxincrement.parameter( oldparm=b00, newparm=b,
                    max.increment=max.increment )
                        }
    res <- list("b"=b, "se.b"=se.b,
            "ll"=sum(res$ll0) )
    return(res)
            }

###########################################
# estimation of a
.smirt.est.a.partcomp <- function(   b, a, c, d,  mu.i,Qmatrix, est.a, theta.k,
        n.ik, I, K, TP, D, numdiff.parm=.001, max.a.increment,
        msteps,  mstepconv, increment.factor){
    h <- numdiff.parm
    diffindex <- est.a        # zeros are allowed!
    cat("  M steps a parameter   |")
    it <- 0 ;    conv1 <- 1000
    Q2 <- Q1 <- 0*Qmatrix
    Q <- Qmatrix
    se.a <- a
    a00 <- a
    while( ( it < msteps ) & ( conv1 > mstepconv ) ){
        a0 <- a
        for (dd in 1:D){
#                dd <- 2
            Q2 <- Q1
            Q2[,dd] <- 1 * ( Qmatrix[,dd] !=0 )

            probres <- calcprob.partcomp( a, b, Q, thetak=theta.k, c, d, mu.i )
            pjk <- problong2probarray( probres, I, TP )

            probres <- calcprob.partcomp( a + h*Q2, b, Q, thetak=theta.k, c, d, mu.i)
            pjk1 <- problong2probarray( probres, I, TP )

            probres <- calcprob.partcomp( a - h*Q2, b, Q, thetak=theta.k, c, d, mu.i)
            pjk2 <- problong2probarray( probres, I, TP )

            # numerical differentiation
            res <- .rm.numdiff.index( pjk, pjk1, pjk2, n.ik, diffindex[,dd],
                    max.increment=max.a.increment[,dd], numdiff.parm )
            ind <- match( diffindex[,dd], sort(unique( diffindex[,dd] )) )
            a[,dd] <- a[,dd] + (res$increment)[ind]
#            max.a.increment[,dd] <- abs( (res$increment)[ind] )
            se.a[,dd] <- (sqrt(  1 / abs(res$d2) ))[ind]
                        }   # end dd
        conv1 <- max( abs( a - a0 ) )
        it <- it+1
        cat("-") # ; flush.console()
            }
    cat(" ", it, "Step(s) \n")    #; flush.console()
    #****
    # post-processing of a parameters
    if ( increment.factor > 1){
        a <- .adj.maxincrement.parameter( oldparm=a00, newparm=a,
                    max.increment=max.a.increment )
                        }
    res <- list("a"=a, "se.a"=se.a, "ll"=sum(res$ll0) )
    return(res)
            }



###########################################
# estimation of c
.smirt.est.c.partcomp <- function(   b, a, c, d, mu.i, Qmatrix, est.c, theta.k,
        n.ik, I, K, TP, D, numdiff.parm=.001, max.increment=max.increment,
        msteps,  mstepconv, increment.factor){
    h <- numdiff.parm
    diffindex <- est.c        # zeros are allowed!
    Q1 <- rep(1,I)
    Q1[ est.c==0 ] <- 0
    Q <- Qmatrix
    c00 <- c
    cat("  M steps c parameter   |")
    it <- 0 ;    conv1 <- 1000
    while( ( it < msteps ) & ( conv1 > mstepconv ) ){
        c0 <- c
            probres <- calcprob.partcomp( a, b, Q, thetak=theta.k, c, d, mu.i)
            pjk <- problong2probarray( probres, I, TP )

            probres <- calcprob.partcomp( a, b, Q, thetak=theta.k, c+h*Q1, d, mu.i)
            pjk1 <- problong2probarray( probres, I, TP )

            probres <- calcprob.partcomp( a, b, Q, thetak=theta.k, c-h*Q1, d, mu.i)
            pjk2 <- problong2probarray( probres, I, TP )

            # numerical differentiation
            res <- .rm.numdiff.index( pjk, pjk1, pjk2, n.ik, diffindex,
                    max.increment=max.increment, numdiff.parm )
            ind <- match( diffindex, sort(unique( diffindex ))    )
            incr <- res$increment
            incr[ is.na(incr ) ] <- 0
            c <- c + incr[ind]
            c[ c< 0 ] <- .001
#            max.a.increment[,dd] <- abs( (res$increment)[ind] )
            se.c <- (sqrt(  1 / abs(res$d2) ))[ind]
        conv1 <- max( abs( c - c0 ) )
        it <- it+1
        cat("-") # ; flush.console()
            }
    cat(" ", it, "Step(s) \n")    #; flush.console()
    if ( increment.factor > 1){
        c <- .adj.maxincrement.parameter( oldparm=c00, newparm=c,
                    max.increment=max.increment )
                        }
    res <- list("c"=c, "se.c"=se.c,
            "ll"=sum(res$ll0) )
    return(res)
            }


###########################################
# estimation of c
.smirt.est.d.partcomp <- function(   b, a, c, d, mu.i, Qmatrix, est.d, theta.k,
        n.ik, I, K, TP, D, numdiff.parm=.001, max.increment=max.increment,
        msteps,  mstepconv, increment.factor){
    h <- numdiff.parm
    diffindex <- est.d        # zeros are allowed!
    Q1 <- rep(1,I)
    Q1[ est.d==0 ] <- 0
    Q <- Qmatrix
    d00 <- d
    cat("  M steps d parameter   |")
    it <- 0 ;    conv1 <- 1000
    while( ( it < msteps ) & ( conv1 > mstepconv ) ){
        d0 <- d
            probres <- calcprob.partcomp( a, b, Q, thetak=theta.k, c, d, mu.i)
            pjk <- problong2probarray( probres, I, TP )

            probres <- calcprob.partcomp( a, b, Q, thetak=theta.k, c, d +h*Q1, mu.i)
            pjk1 <- problong2probarray( probres, I, TP )

            probres <- calcprob.partcomp( a, b, Q, thetak=theta.k, c, d-h*Q1, mu.i)
            pjk2 <- problong2probarray( probres, I, TP )

            # numerical differentiation
            res <- .rm.numdiff.index( pjk, pjk1, pjk2, n.ik, diffindex,
                    max.increment=max.increment, numdiff.parm )
            ind <- match( diffindex, sort(unique( diffindex ))        )
            incr <- res$increment
            incr[ is.na(incr ) ] <- 0
            d <- d + incr[ind]
            d[ d>1 ] <- .999
#            max.a.increment[,dd] <- abs( (res$increment)[ind] )
            se.d <- (sqrt(  1 / abs(res$d2) ))[ind]
        conv1 <- max( abs( d - d0 ) )
        it <- it+1
        cat("-") # ; flush.console()
            }
    cat(" ", it, "Step(s) \n")    #; flush.console()
    if ( increment.factor > 1){
        d <- .adj.maxincrement.parameter( oldparm=d00, newparm=d,
                    max.increment=max.increment )
                        }
    res <- list("d"=d, "se.d"=se.d, "ll"=sum(res$ll0) )
    return(res)
            }




###########################################
# estimation of mu.i
.smirt.est.mu.i.partcomp <- function(   b, a, c, d, mu.i, Qmatrix, est.mu.i, theta.k,
        n.ik, I, K, TP, D, numdiff.parm=.001, max.increment=max.increment,
        msteps,  mstepconv, increment.factor){
    h <- numdiff.parm
    diffindex <- est.mu.i        # zeros are allowed!
    Q1 <- rep(1,I)
    Q1[ est.mu.i==0 ] <- 0
    Q <- Qmatrix
    c00 <- mu.i
    cat("  M steps mu.i parameter   |")
    it <- 0 ;    conv1 <- 1000
    while( ( it < msteps ) & ( conv1 > mstepconv ) ){
        c0 <- mu.i
            probres <- calcprob.partcomp( a, b, Q, thetak=theta.k, c, d, mu.i)
            pjk <- problong2probarray( probres, I, TP )

            probres <- calcprob.partcomp( a, b, Q, thetak=theta.k, c, d, mu.i+h*Q1)
            pjk1 <- problong2probarray( probres, I, TP )

            probres <- calcprob.partcomp( a, b, Q, thetak=theta.k, c, d, mu.i-h*Q1)
            pjk2 <- problong2probarray( probres, I, TP )

            # numerical differentiation
            res <- .rm.numdiff.index( pjk, pjk1, pjk2, n.ik, diffindex,
                    max.increment=max.increment, numdiff.parm )
            ind <- match( diffindex, sort(unique( diffindex ))    )
            incr <- res$increment
            incr[ is.na(incr ) ] <- 0
            mu.i <- mu.i + incr[ind]
            mu.i[  mu.i < 0 ] <- .001
            mu.i[  mu.i > 1 ] <- 1 - .001
#            max.a.increment[,dd] <- abs( (res$increment)[ind] )
            se.c <- (sqrt(  1 / abs(res$d2) ))[ind]
        conv1 <- max( abs( mu.i - c0 ) )
        it <- it+1
        cat("-") # ; flush.console()
            }
    cat(" ", it, "Step(s) \n")    #; flush.console()
    if ( increment.factor > 1){
        mu.i <- .adj.maxincrement.parameter( oldparm=c00, newparm=mu.i,
                    max.increment=max.increment )
                        }
    res <- list("mu.i"=mu.i, "se.c"=se.c,
            "ll"=sum(res$ll0) )
    return(res)
            }

## File Name: plot.rm.sdt.R
## File Version: 0.11


##################################################
# plot.rm.sdt
plot.rm.sdt <- function( x, ask=TRUE, ... ){
    rt1 <- x$rater2
    items <- unique( rt1$item )
    maxK <- x$maxK
    for (ii in items){
        # ii <- items[1]
        rtii <- rt1[ rt1$item==ii, ]
        rtii1 <- rtii[, grep( ".trans", colnames(rtii) ) ]
        # xlim <- range(rtii1)
        # xlim <- c(-1, ncol(rtii1) + 1 )
        xlim <- c(-1, maxK[ii] + 1 )
        # K <- ncol(rtii1)
        K <- maxK[ii]
        for (kk in 1:K){
            # kk <- 1
            m1 <- as.vector(rtii1[,kk])
            if (kk==1){
                graphics::dotchart( m1, xlim=xlim, labels=rtii$rater,
                    main=paste0("Rater Parameters | Item ", ii ), ...)
                        }
            RR <- nrow(rtii)
            graphics::lines( rep( kk - .5, RR ), 1:RR, lty=2, col="darkred" )
            graphics::points( m1, 1:RR, pch=16 )
            graphics::lines( m1, 1:RR, col="darkred" )
                        }
        graphics::text( K + 1 - 0.2, 1:RR, format( rtii$d, digits=2, nsmall=2 )    )
        graphics::text( K + 1 - 0.2, RR + .7, expression(d) )
        graphics::par( ask=ask )
            }
        }
###########################################################

## File Name: xxirt_nr_pml_opt_fun.R
## File Version: 0.127

xxirt_nr_pml_opt_fun <- function(x, em_args, output_all=FALSE, use_rcpp=TRUE)
{

    Theta <- em_args$Theta
    pml_args <- em_args$pml_args

    #*** compute prior distribution
    prior_Theta <- xxirt_compute_prior_Theta_from_x(x=x, em_args=em_args)

    #* compute item response probabilities
    probs_items <- xxirt_compute_prob_item_from_x(x=x, em_args=em_args)
    eps <- 1e-14

    I <- pml_args$I
    K <- pml_args$K
    G <- pml_args$G
    TP <- pml_args$TP
    W1 <- pml_args$W1
    freq1 <- pml_args$freq1
    NI2 <- pml_args$NI2
    W2_long <- as.matrix(pml_args$W2_long)
    freq2 <- pml_args$freq2

    eval_fun_args <- list(prior_Theta=prior_Theta, probs_items=probs_items, freq1=freq1,
                            freq2=freq2, W1=W1, W2_long=W2_long, G=G, K=K,
                            I=I, TP=TP, NI2=NI2, eps=eps )

    if (use_rcpp){
        fun_pml_eval <- sirt_rcpp_xxirt_nr_pml_opt_fun
    } else {
        fun_pml_eval <- xxirt_nr_pml_opt_fun_R
    }

    res <- do.call( what=fun_pml_eval, args=eval_fun_args)
    res <- -res$val

    #-- output
    return(res)
}

## File Name: detect.index.R
## File Version: 0.352



detect.index <- function( ccovtable, itemcluster )
{
    ccovtable <- ccovtable$ccov.table
    ccovtable$delta <- ifelse( itemcluster[ ccovtable$item1ID ]==
                            itemcluster[ ccovtable$item2ID ], 1, -1 )

    #--- calculate unweighted and weighted indizes
    ccov <- ccovtable$ccov            # conditional covariance
    delta <- ccovtable$delta        # indicator for partition
    N <- ccovtable$N
    sqrt_N <- sqrt(N)
    sign_ccov <- sign(ccov)
    abs_ccov <- abs(ccov)
    # number of parameters
    np <- 5
    parnames <- weighted.indizes <- indizes <- rep(NA,np)

    #--- DETECT
    ii <- 1
    indizes[ii] <- 100*mean(ccov*delta)
    weighted.indizes[ii] <- 100*stats::weighted.mean( ccov * delta, sqrt_N )
    parnames[ii] <- 'DETECT'
    #--- ASSI
    ii <- 2
    indizes[ii] <- mean( sign_ccov * delta )
    weighted.indizes[ii] <- stats::weighted.mean( sign_ccov * delta, sqrt_N )
    parnames[ii] <- 'ASSI'
    #--- RATIO
    ii <- 3
    indizes[ii] <- sum( ccov * delta ) / sum( abs_ccov )
    weighted.indizes[ii] <- sum( ccov * delta * sqrt_N ) / sum( abs_ccov * sqrt_N )
    parnames[ii] <- 'RATIO'
    #--- MADCOV
    ii <- 4
    indizes[ii] <- 100 * mean( abs_ccov )
    weighted.indizes[ii] <- 100* stats::weighted.mean( abs_ccov, sqrt_N )
    parnames[ii] <- 'MADCOV100'
    #--- MCOV
    ii <- 5
    indizes[ii] <- 100 * mean( ccov )
    weighted.indizes[ii] <- 100* stats::weighted.mean( ccov, sqrt_N )
    parnames[ii] <- 'MCOV100'

    #--- output
    res <- data.frame( unweighted=indizes, weighted=weighted.indizes )
    rownames(res) <- parnames
    return(res)
}

## File Name: gom_em_est_lambda.R
## File Version: 0.207


# estimation of lambda parameters
gom_em_est_lambda <- function( lambda, I, K, n.ik, numdiff.parm=.001,
        max.increment=1, theta.k, msteps, mstepconv, eps=2*numdiff.parm,
        progress=TRUE, lambda_partable)
{
    h <- numdiff.parm
    diffindex <- 1L:I
    Q0 <- 0*lambda
    se.lambda <- Q0
    an.ik <- aperm( n.ik, c(2,3,1) )
    if (progress){
        cat('  M steps lambda parameter |')
    }
    it <- 0
    conv1 <- 1000
    increment_mat <- d2_mat <- d1_mat <- matrix(0, nrow=I, ncol=K)

    while( ( it < msteps ) & ( conv1 > mstepconv ) ){
        lambda0 <- lambda

        for (kk in 1L:K){
            Q1 <- Q0
            Q1[,kk] <- 1
            parm_temp <- lambda0
            lambda00 <- parm_temp
            lambda01 <- parm_temp + h*Q1
            lambda02 <- parm_temp - h*Q1

            pjk <- gom_em_calc_probs( lambda=lambda00, theta.k=theta.k )$probsL
            pjk1 <- gom_em_calc_probs( lambda=lambda01, theta.k=theta.k )$probsL
            pjk2 <- gom_em_calc_probs( lambda=lambda02, theta.k=theta.k )$probsL
            res <- gom_em_numdiff_index( pjk=pjk, pjk1=pjk1, pjk2=pjk2, an.ik=an.ik,
                        diffindex=diffindex, max.increment=max.increment,
                        numdiff.parm=numdiff.parm )
            d1_mat[,kk] <- res$d1
            d2_mat[,kk] <- res$d2
        }
        d1a <- rowsum(x=as.vector(d1_mat), group=lambda_partable$par_index)[,1]
        d2a <- rowsum(x=as.vector(d2_mat), group=lambda_partable$par_index)[,1]
        increment <- - d1a / d2a
        increment <- sirt_trim_increment(increment=increment, max_increment=max.increment)
        increment <- matrix( increment[ lambda_partable$par_index ], nrow=I, ncol=K)
        lambda <- lambda + increment
        d2 <- matrix(d2a[ lambda_partable$par_index ], nrow=I, ncol=K)
        for (kk in 1L:K){
            lambda[,kk] <- sirt_squeeze(lambda[,kk], lower=eps, upper=1-eps)
            se.lambda[,kk] <- sqrt(-1/d2[,kk])
        }
        conv1 <- max( abs( lambda - lambda0 ) )
        it <- it+1
        if (progress){
            cat('-')
        }
    }
    if (progress){
        cat(' ', it, 'Step(s) \n')
    }
    res <- list(lambda=lambda, se.lambda=se.lambda, ll=sum(res$ll0) )
    return(res)
}

## File Name: summary.rasch.evm.pcm.R
## File Version: 0.18

summary.rasch.evm.pcm <- function( object, digits=3, file=NULL, ... )
{
    # open sink for a file
    sirt_osink(file=file)

    display_string <- sirt_summary_print_display(symbol="-", len=65)
    cat(display_string)

    #- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object)

    cat("Partial Credit Model (estimated with eigenvector method) \n\n")
    options(scipen=999)
    cat( "Number of groups:", object$desc$G[1], "\n" )
    cat( "Number of persons:", object$desc$Nobs, "\n" )
    cat( "Weighted number of persons:", round( (object$desc$sum.weights), 2 ), "\n" )
    cat( "Number of items:", object$desc$N.items[1], "\n" )
    cat( paste0("Number of items per parameter: ",
            "M=", round( object$desc$M.Nitems[1],1),
            " | SD=", round( object$desc$SD.Nitems[1],1),
            " | Min=", object$desc$min.Nitems[1],
            " | Max=", object$desc$max.Nitems[1],
            "\n" )
                )
    cat( "Number of parameters:", sum(object$desc$N.pars), "\n\n" )

    cat(display_string)
    cat("Item Parameters \n\n")
    obji <- object$item
    sirt_summary_print_objects(obji=obji, digits=digits, from=4, rownames_null=FALSE)

    if (! is.null( object$difstats) ){
        cat(display_string)
        cat("DIF Tests \n\n")
        obji <- object$difstats
        sirt_summary_print_objects(obji=obji, digits=digits, from=2, rownames_null=FALSE)
    }
    options(scipen=0)

    sirt_csink(file=file)
}

## File Name: dif.logisticregression.R
## File Version: 1.206



# This function performs itemwise DIF analysis by using logistic regression methods     ##
# uniform and nonuniform DIF                                                            ##
dif.logistic.regression <- function( dat, group, score, quant=1.645)
{
    I <- ncol(dat)
    matr <- NULL
    cat('Items ')
    for (ii in 1L:I){
        dat.ii <- na.omit(data.frame( y=dat[,ii], score=score, group=group ))
        mod1 <- stats::glm( y  ~ score, data=dat.ii, family='binomial')
        mod2 <- stats::glm( y  ~ score + group, data=dat.ii, family='binomial')
        mod3 <- stats::glm( y  ~ score + group + score*group, data=dat.ii,
                                    family='binomial')

        h1 <- data.frame( item=colnames(dat)[ii], N=sum(1-is.na(dat[,ii]), na.rm=TRUE))
        h1$R <- min(group)
        h1$F <- max(group)
        h1$nR <- sum(  ( 1- is.na( dat[,ii] ) )* (1-group), na.rm=T)
        h1$nF <- sum(  ( 1- is.na( dat[,ii] ) )* (group), na.rm=T)
        h1$p <- mean(  dat[,ii], na.rm=TRUE)
        a1 <- stats::aggregate( dat[,ii], list(group), mean, na.rm=T )[,2]
        h1$pR <- a1[1]
        h1$pF <- a1[2]
        h1$pdiff <- h1$pR - h1$pF
        h1$pdiff.adj <- NA
        h1$uniformDIF <- mod2$coef[3]
        h1$se.uniformDIF <- sqrt( diag( stats::vcov(mod2)) )[3]
        h1$t.uniformDIF <- mod2$coef[3] / sqrt( diag( stats::vcov(mod2) ) )[3]
        h1$sig.uniformDIF <- ''
        if ( ! is.na( h1$t.uniformDIF ) ){
            if ( h1$t.uniformDIF > quant ){ h1$sig.uniformDIF <- '+' }
            if ( h1$t.uniformDIF < - quant ){ h1$sig.uniformDIF <- '-' }
        }
        h1$DIF.ETS <- ''
        #****
        # nonuniform DIF
        h1$nonuniformDIF <- mod3$coef[4]
        h1$se.nonuniformDIF <- sqrt( diag( stats::vcov(mod3)) )[4]
        h1$t.nonuniformDIF <- mod3$coef[4] / sqrt( diag( stats::vcov(mod3) ) )[4]
        h1$sig.nonuniformDIF <- ''
        if ( ! is.na( h1$t.nonuniformDIF ) ){
            if ( h1$t.nonuniformDIF > quant ){ h1$sig.nonuniformDIF <- '+' }
            if ( h1$t.nonuniformDIF < - quant ){ h1$sig.nonuniformDIF <- '-' }
        }
        matr <- rbind( matr, h1 )
        cat( ii, ' ' )
        utils::flush.console()
        if ( ii %% 15==0 ){ cat('\n') }
    }
    cat('\n')
    # include variable of adjusted p values
    matr[, 'pdiff.adj' ] <- matr$pR - matr$pF - mean( matr$pR - matr$pF  )
    ind1 <- grep( 'ETS', colnames(matr) )
    #***
    # DIF ETS classifiaction
    stat <- abs( matr$uniformDIF )
    stat.low <- stat - quant * matr$se.uniformDIF
    matr[,'DIF.ETS'] <- 'B'
    # DIF classification C
    ind <- which( ( stat > .64 ) & ( stat.low > .43 ) )
    if (length(ind) > 0){ matr[ind, 'DIF.ETS'] <- 'C' }
    # DIF classification A
    ind <- which( ( stat < .43 ) | ( stat.low < 0 ) )
    if (length(ind) > 0){
        matr[ind, 'DIF.ETS'] <- 'A'
    }
    matr$DIF.ETS <- paste0( matr$DIF.ETS, ifelse( matr$uniformDIF > 0, '+', '-' ))

    #**** calculation of DIF variance
    dif1 <- dif.variance( dif=matr$uniformDIF, se.dif=matr$se.uniformDIF )
    matr <- data.frame( matr[, seq(1,ind1)], uniform.EBDIF=dif1$eb.dif,
                    DIF.SD=dif1$unweighted.DIFSD, matr[, seq(ind1+1, ncol(matr)) ] )
    cat( paste0('\nDIF SD=', round(dif1$unweighted.DIFSD, 3 ) ), '\n')
    # sorting of the items
    g1 <- rank( matr$uniformDIF )
    matr <- data.frame( itemnr=1L:nrow(matr), sortDIFindex=g1, matr )
    return(matr)
}

## File Name: summary.rasch.copula2.R
## File Version: 0.205


#** Summary for rasch.copula object
summary.rasch.copula2 <- function( object, file=NULL, digits=3, ... )
{
    is_copula2 <- inherits(object,"rasch.copula2")
    is_copula3 <- inherits(object,"rasch.copula3")

    # open sink for a file
    sirt_osink(file=file)

    display_string <- sirt_summary_print_display(symbol="-", len=65)
    cat(display_string)

    #- package and R session
    sirt_summary_print_package_rsession(pack="sirt")

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object)

    cat("Marginal Maximum Likelihood Estimation \n")
    cat(paste( "Raschtype Copula Model with generalized logistic link function: Estimation of alpha1 and alpha2 \n") )
    cat( paste0("Function '", class(object), "'\n"))
    cat("alpha1","=", round(object$alpha1, digits)," alpha2", "=",
                        round(object$alpha2, digits), "\n")

    cat(display_string)
    cat( "Deviance", "=", round( object$deviance, 2 ), "\n" )
    cat( "Number of persons", "=", object$ic$n, " (", nrow(object$pattern), " Response Patterns)\n" )
    cat( "Number of estimated parameters", "=", object$ic$np, "\n" )
    cat( "Number of iterations", "=", object$iter, "\n" )
    cat( "AIC", "=", round( object$ic$AIC, 2 ), " | penalty", "=",
                    round( object$ic$AIC - object$ic$deviance,2 ), "\n" )
    cat( "AICc", "=", round( object$ic$AICc, 2 ), " | penalty", "=", round( object$ic$AICc - object$ic$deviance,2 ) )
    cat(" (bias corrected AIC)\n" )
    cat( "BIC", "=", round( object$ic$BIC, 2 ), " | penalty", "=", round( object$ic$BIC - object$ic$deviance,2 ), "\n" )
    cat( "CAIC", "=", round( object$ic$CAIC, 2 )," | penalty", "=", round( object$ic$CAIC - object$ic$deviance,2 ) )
    cat( " (consistent AIC) \n\n" )

    #--- trait distribution
    if (is_copula2){
        cat( "Trait Distribution (", length(object$theta.k), " Knots )\n",
                "Mean", "=", 0, " SD", "=", 1, "\n")
        cat(paste("\nEAP Reliability:", round( object$EAP.Rel, digits)),"\n\n")
    }
    if (is_copula3){
        cat( "Trait Distribution (", length(object$theta.k), " Knots )\n" )
        cat("\nMean Vector\n")
        print( round( object$mu, digits))
        cat("\nCorrelation Matrix\n")
        print( round( object$sigma, digits))
    }

    # item paramters
    cat(display_string)
    cat("Item Parameter \n")
    obji <- object$item
    sirt_summary_print_objects(obji=obji, digits=digits, from=2, rownames_null=TRUE)

    # dependency parameters
    cat("\nDependency parameters\n")
    obji <- object$summary.delta
    sirt_summary_print_objects(obji=obji, digits=digits, from=1, rownames_null=TRUE)

    sirt_csink(file=file)
}

## File Name: xxirt.R
## File Version: 1.188


#--- user specified item response model
xxirt <- function( dat, Theta=NULL, itemtype=NULL, customItems=NULL,
                partable=NULL, customTheta=NULL, group=NULL, weights=NULL,
                globconv=1E-6, conv=1E-4, maxit=1000, mstep_iter=4,
                mstep_reltol=1E-6, maxit_nr=0, optimizer_nr="nlminb",
                estimator="ML", control_nr=list(trace=1),
                h=1E-4, use_grad=TRUE, verbose=TRUE, penalty_fun_item=NULL,
                np_fun_item=NULL, penalty_fun_theta=NULL, np_fun_theta=NULL,
                pml_args=NULL, verbose_index=NULL, cv_kfold=0, cv_maxit=10)
{
    #*** preliminaries
    CALL <- match.call()
    s1 <- Sys.time()

    #*** some data processing of dat
    res <- xxirt_data_proc(dat=dat, group=group, weights=weights )
    N <- res$N
    W <- res$W
    G <- res$G
    group <- res$group
    items <- res$items
    group0 <- res$group0
    groups_unique <- res$groups_unique
    I <- res$I
    maxK <- res$maxK
    ncat <- res$ncat
    weights <- res$weights
    group_index <- res$group_index
    dat_resp <- res$dat_resp
    resp_index <- res$resp_index
    dat1 <- res$dat1
    dat_resp_bool <- res$dat_resp_bool

    #*** default Theta
    if ( is.null(Theta) ){
        Theta <- matrix( seq(-6,6,length=21), ncol=1 )
    }
    Theta <- as.matrix(Theta)
    TP <- nrow(Theta)

    #*** eps - handle numerical instabilities
    eps <- 1E-8

    # create partable if not provided
    if ( is.null(partable) ){
        partable <- xxirt_createParTable( dat=dat, itemtype=itemtype,
                            customItems=customItems )
    }

    # process partable and itemtype
    res <- xxirt_proc_ParTable( itemtype=itemtype, partable=partable, items=items )
    itemtype <- res$itemtype
    partable <- res$partable
    partable_index <- res$partable_index
    ncat <- res$ncat
    maxK <- res$maxK
    mstep_method <- res$mstep_method
    item_index <- res$item_index
    dat <- as.matrix(dat)

    # create item list
    item_list <- xxirt_createItemList( customItems=customItems, itemtype=itemtype,
                        items=items, partable=partable )

    # shortcut for calculating expected counts
    dat1_resp <- xxirt_prepare_response_data(G=G, group_index=group_index,
                        weights=weights, dat1=dat1, dat_resp=dat_resp, maxK=maxK )

    #*** starting values item parameters
    par0 <- xxirt_partable_extract_freeParameters( partable=partable )
    par1 <- xxirt_ThetaDistribution_extract_freeParameters( customTheta=customTheta )

    #***
    if (is.null(customTheta$some_bound)){
        customTheta$some_bound    <- FALSE
    }

    #*** verbose
    verbose1 <- verbose==1
    verbose2 <- verbose==2

    disp <- '...........................................................\n'
    iter <- 1
    dev <- 1E100
    converged <- FALSE

    em_iterate <- TRUE
    do_nr <- maxit_nr>0
    if (!do_nr){
        iter_nr <- 0
    }
    do_cv <- cv_kfold>0
    em_count <- 1

    while(em_iterate){

        #--- create list with arguments for EM algorithm
        em_args <- list( maxit=maxit, verbose1=verbose1, disp=disp,
                    item_list=item_list, items=items, Theta=Theta, ncat=ncat,
                    partable=partable, partable_index=partable_index, dat=dat,
                    I=ncol(dat), resp_index=resp_index, dat_resp=dat_resp,
                    dat_resp_bool=dat_resp_bool, dat1=dat1, dat1_resp=dat1_resp,
                    customTheta=customTheta, G=G,
                    par0=par0, maxK=maxK, group_index=group_index, weights=weights,
                    mstep_iter=mstep_iter, eps=eps, mstep_reltol=mstep_reltol,
                    mstep_method=mstep_method, item_index=item_index, h=h,
                    use_grad=use_grad, penalty_fun_item=penalty_fun_item,
                    penalty_fun_theta=penalty_fun_theta, group=group,
                    par1=par1, globconv=globconv, conv=conv, verbose2=verbose2,
                    verbose3=FALSE, verbose_index=verbose_index)

        if (em_count>1){
            em_args$maxit <- 1
            em_args$verbose1 <- FALSE
            em_args$verbose2 <- FALSE
            em_args$verbose3 <- FALSE
        }

        #-- settings for PML estimation
        if (estimator=='PML'){
            do_nr <- TRUE
            maxit_nr <- maxit
            pml_args <- xxirt_nr_pml_preproc_data(em_args=em_args, pml_args=pml_args)
            em_args$pml_args <- pml_args
        }

        #--- run EM algorithm
        if (estimator=='ML'){
            em_out <- res <- do.call(what=xxirt_em_algorithm, args=em_args)
        } else {
            em_out <- res <- NULL
        }

        #--- collect EM output
        if (em_count==1){
            iter_em <- iter <- res$iter
            converged <- res$converged
        }
        probs_items <- res$probs_items
        p.xi.aj <- res$p.xi.aj
        prior_Theta <- res$prior_Theta
        n.ik <- res$n.ik
        p.aj.xi <- res$p.aj.xi
        N.ik <- res$N.ik
        N.k <- res$N.k
        post_unnorm <- res$post_unnorm
        ll1 <- res$ll1
        partable <- res$partable
        par0 <- res$par0
        pen_val <- res$pen_val
        ll2 <- res$ll2
        customTheta <- res$customTheta
        par1 <- res$par1
        ll_case <- res$ll_case
        dev <- res$dev
        dev00 <- res$dev00

        #*** optional cross-validation
        cv_loglike <- NA
        if (do_cv){
            N <- nrow(dat)
            cv_zone <- ( ( 0L:(N-1) ) %% cv_kfold ) + 1
            cv_loglike <- 0
            em_args$partable <- partable
            em_args$customTheta <- customTheta
            for (jj in 1L:cv_kfold){
                weights1 <- weights*(cv_zone!=jj )
                weights2 <- weights*(cv_zone==jj )
                em_args$weights <- weights1
                em_args$maxit <- cv_maxit
                em_args$verbose1 <- FALSE
                em_args$verbose2 <- FALSE
                em_args$verbose3 <- TRUE
                cat(paste0('\nCross-validation sample ', jj ), ' |' )
                utils::flush.console()
                res <- do.call(what=xxirt_em_algorithm, args=em_args)
                # compute cv log likelihood
                ll_jj <- sum( weights2 * res$ll_case )
                cv_loglike <- cv_loglike + ll_jj
            }
            cat('\n')
        }

        if (!do_nr){
            em_iterate <- FALSE
        }

        #*** Newton-Raphson scoring if requested
        res_opt_nr <- opt_values_nr <- NULL
        if (do_nr){
            res <- xxirt_newton_raphson(em_out=em_out, em_args=em_args, maxit_nr=maxit_nr,
                            optimizer_nr=optimizer_nr, control_nr=control_nr,
                            verbose=verbose, estimator=estimator)
            res_opt_nr <- res$res_opt_nr
            iter <- res_opt_nr$iter + iter
            iter_nr <- res_opt_nr$iter
            opt_values_nr <- res$opt_values
            em_args <- res$em_args
            pml_args <- res$em_args$pml_args
            em_args$partable <- partable <- res$partable
            probs_items <- res$probs_items
            em_args$customTheta <- customTheta <- res$customTheta

        } # end NR optimization
        em_count <- 2
        do_nr <- FALSE
        if (estimator=='PML' | maxit==0){
            em_iterate <- FALSE
        }

    }  # loop for final EM iteration

    #**** post processing
    opt_val <- dev
    dev <- dev00

    #-- collect parameters
    res <- xxirt_postproc_parameters( partable=partable, customTheta=customTheta,
                    items=items, probs_items=probs_items, np_fun_item=np_fun_item,
                    np_fun_theta=np_fun_theta)
    par_items <- res$par_items
    par_Theta <- res$par_Theta
    probs_items <- res$probs_items
    par_items_summary <- res$par_items_summary
    par_items_bounds <- res$par_items_bounds
    np_item <- res$np_item
    np_theta <- res$np_theta

    #-- information criteria
    ic <- xxirt_ic( dev=dev, N=W, par_items=par_items,
                    par_Theta=par_Theta, I=I, par_items_bounds=par_items_bounds,
                    np_item=np_item, np_theta=np_theta, estimator=estimator)

    #-- compute EAP
    if (estimator=='ML'){
        EAP <- xxirt_EAP(p.aj.xi=p.aj.xi, Theta=Theta )
        loglike <- -dev/2
    }
    if (estimator=='PML'){
        EAP <- NULL
        dev <- NULL
        loglike <- NULL
    }

    #--- output
    s2 <- Sys.time()
    time_diff <- s2-s1
    res <- list( partable=partable, par_items=par_items,
                par_items_summary=par_items_summary, par_items_bounds=par_items_bounds,
                par_Theta=par_Theta, Theta=Theta, probs_items=probs_items,
                probs_Theta=prior_Theta, deviance=dev, loglike=loglike,
                opt_val=opt_val, pen_val=pen_val, ic=ic,
                item_list=item_list, customItems=customItems,
                customTheta=customTheta, cv_loglike=cv_loglike,
                p.xi.aj=p.xi.aj, p.aj.xi=p.aj.xi, ll_case=ll_case,
                n.ik=n.ik, EAP=EAP, dat=dat, dat_resp=dat_resp,
                weights=weights, item_index=item_index, G=G, group=group,
                group_orig=group0, ncat=ncat, mstepItem_method=mstep_method,
                partable_index=partable_index, items=items, dat1=dat1,
                dat1_resp=dat1_resp, dat_resp_bool=dat_resp_bool,
                group_index=group_index, maxK=maxK,
                resp_index=resp_index, converged=converged,
                res_opt_nr=res_opt_nr, opt_values_nr=opt_values_nr, maxit_nr=maxit_nr,
                iter=iter-1, iter_em=iter_em-1, iter_nr=iter_nr,
                estimator=estimator, pml_args=pml_args, em_args=em_args,
                CALL=CALL, s1=s1, s2=s2, time_diff=time_diff )
    class(res) <- 'xxirt'
    return(res)
}

## File Name: polychoric2.R
## File Version: 0.332


#---- estimating polychoric correlation using the Olsson method
# of maximum likelihood estimation
polychoric2 <- function( dat, maxiter=100, cor.smooth=TRUE, use_pbv=1,
    conv=1e-10, rho_init=NULL, weights=NULL )
{
    dat1 <- as.matrix(dat)
    NV <- ncol(dat1)
    N <- nrow(dat1)
    # initial values for rho
    if (is.null(rho_init)){
        rho_init <- matrix(0, nrow=NV, ncol=NV)
    }
    if (is.null(weights)){
        weights <- rep(1,N)
    }
    min_val <- apply(dat1, 2, min, na.rm=TRUE)
    if (any(min_val>0)){
        stop("Minimum value must always zero.\n")
    }

    # maximum number of categories
    maxK <- max(dat1, na.rm=TRUE )
    # compute polychoric correlation
    res0 <- sirt_rcpp_polychoric2( dat=dat1, maxK=maxK, maxiter=maxiter,
                    use_pbv=use_pbv, conv=conv, rho_init=rho_init,
                    weights=weights)
    iter <- res0$iter
    rho <- res0$rho
    Nobs <- res0$Nobs
    maxcat <- res0$maxcat
    thresh <- res0$thresh

    #--- output cleaning
    # thresholds
    tau <- thresh[,c(2:(maxK+1))]
    tau[ tau==99 ] <- Inf
    rownames(rho) <- colnames(rho) <- colnames(dat1)
    if ( maxK > 1 ){
        rownames(tau) <- rownames(rho)
        colnames(tau) <- paste0("Cat", 1:maxK)
    }
    if ( maxK==1 ){
        names(tau) <- rownames(rho)
    }
    # handling missing entries in rho
    rho[ Nobs==0 ] <- NA
    diag(rho) <- 1
    if ( sum( is.na(rho) > 0 ) ){
        cor.smooth <- FALSE
    }
    if (cor.smooth){
        rho <- sirt_import_psych_cor.smooth(x=rho)
    }
    # output list
    res <- list(tau=tau, rho=rho, Nobs=Nobs, maxcat=maxcat,
                cor.smooth=cor.smooth, iter=iter)
    return(res)
}

## File Name: mcmclist2coda.R
## File Version: 0.121


# write elements from mcmcmlist into code file
mcmclist2coda <- function( mcmclist, name, coda.digits=5 )
{
    m1 <- mcmclist[[1]]
    vars <- colnames(m1)
    #--- create codaIndex file
    BB <- nrow(m1)
    VV <- length(vars)
    c1 <- paste( vars, seq( 1, BB*VV, BB ), seq( BB, BB*VV, BB ) )
    writeLines( c1, paste0( name, '_codaIndex.txt' ) )
    #--- create coda file
    m2 <- matrix( m1, ncol=1 )
    m2 <- paste( rep(1L:BB, VV ), round( m2[,1], coda.digits ) )
    writeLines( m2, paste0( name, '_coda1.txt' ) )
}

## File Name: print_digits.R
## File Version: 0.03

print_digits <- function(x, digits=NULL)
{
    NC <- ncol(x)
    if (length(digits)==1){
        digits <- rep(digits, NC)
    }
    for (cc in 1:NC){
        y <- x[,cc]
        if (is.numeric(y)){
            x[,cc] <- round(y, digits=digits[cc])
        }
    }
    print(x)
    invisible(x)
}

## File Name: mcmc_Rhat.R
## File Version: 0.121

mcmc_Rhat <- function( mcmc_object, n_splits=3 )
{
    n_samples <- nrow(mcmc_object)
    n_pars <- ncol(mcmc_object)
    n_within <- floor( n_samples / n_splits )
    rhat_vec <- rep(NA, n_pars)
    names(rhat_vec) <- colnames(mcmc_object)
    for (pp in 1L:n_pars){
        matr <- matrix( NA, nrow=n_within, ncol=n_splits)
        for (ss in 1L:n_splits){
            matr[,ss] <- mcmc_object[ (ss-1)* n_within + 1L:n_within, pp ]
        }
        rhat_vec[pp] <- Rhat1(matr)
    }
    return(rhat_vec)
}

## File Name: rasch.copula3.covariance.R
## File Version: 0.16


######################################
# estimation of covariance
rasch.copula3.covariance <- function( f.qk.yi, Sigma, theta.k, N,
        mu.fixed, variance.fixed, D, est.corr, irtmodel,
        freqwgt ){
        Sigma.cov <- Sigma
        delta.theta <- 1
        indD <- rep( 1:nrow(f.qk.yi), freqwgt )
        hwt <- f.qk.yi[ indD, ]
        N <- sum( freqwgt)
#        if (qmcnodes){
#            hwt <- hwt / nrow(theta.k)
#            hwt <- hwt / rowSums( hwt )
#                }
        thetabar <- hwt%*%theta.k
        mu <- rep(0,D)
        # calculation of mu
#        mu <- colSums( thetabar ) / N
#        if ( ! is.null(mu.fixed ) ){
#              if (is.matrix(mu.fixed) ){
#                mu0 <- mu
#                mu[ mu.fixed[,1] ] <- mu.fixed[,2]
#                                    }
#                            }
        # calculation of the covariance matrix
#        theta.k.adj <- theta.k - matrix( mu, nrow=nrow(theta.k),
#                                    ncol=ncol(theta.k), byrow=TRUE)
        theta.k.adj <- theta.k
        for (dd1 in 1:D){
            for (dd2 in dd1:D){
                tk <- theta.k.adj[,dd1]*theta.k.adj[,dd2]
                h1 <- ( hwt %*% tk ) * delta.theta
                Sigma.cov[dd1,dd2] <- sum( h1  ) / N
                if (dd1 < dd2 ){ Sigma.cov[dd2,dd1] <- Sigma.cov[dd1,dd2] }

                                    }
                                }
        Sigma.cov0 <- Sigma.cov

        if ( est.corr ){ Sigma.cov <- stats::cov2cor(Sigma.cov ) }
        if ( ! is.null(variance.fixed ) ){
                Sigma.cov[ variance.fixed[,1:2,drop=FALSE] ] <- variance.fixed[,3]
                Sigma.cov[ variance.fixed[,c(2,1),drop=FALSE] ] <- variance.fixed[,3]
#
                                    }
        diag(Sigma.cov) <- diag(Sigma.cov) + 10^(-10)


        pi.k <- sirt_dmvnorm_discrete( theta.k, mean=mu, sigma=Sigma.cov, as_matrix=TRUE )
        res <- list( "mu"=mu, "Sigma"=Sigma.cov, "pi.k"=pi.k,
            "Sigma0"=Sigma.cov0)
        return(res)
                    }

## File Name: noharm_sirt_partable_extract_par.R
## File Version: 0.03

noharm_sirt_partable_extract_par <- function(parm_table, col="est")
{
    extract_index <- attr(parm_table, 'extract_index')
    x <- parm_table[ extract_index, col]
    return(x)
}

## File Name: sirt_squeeze.R
## File Version: 0.03

sirt_squeeze <- function(x, lower=NULL, upper=NULL)
{
    if (!is.null(lower)){
        x <- ifelse( x<lower, lower, x)
    }
    if (!is.null(upper)){
        x <- ifelse( x>upper, upper, x)
    }
    return(x)
}

## File Name: rm_facets_print_progress_deviance.R
## File Version: 0.08


rm_facets_print_progress_deviance <- function( dev, dev0, digits_deviance, iter )
{
    cat( paste( "   Deviance=", round( dev, digits_deviance ),
        if (iter > 1 ){ " | Deviance change=" } else {""},
        if( iter>1){ round( - dev + dev0, digits_deviance + 2)} else { ""}    ,"\n",sep="") )
}

## File Name: mirt.specify.partable.R
## File Version: 0.111


#######################################################################
##   function parameter table specification
mirt.specify.partable <- function(mirt.partable, parlist, verbose=TRUE)
{
    mirt.partable0 <- mirt.partable
    mirt.partable0$prior.type <- paste(mirt.partable0$prior.type)
    if(length(unique(mirt.partable$group))==1L) ####
        parlist <- list(all=parlist) ####
    parlistfull <- parlist ####
    gnames <- names(parlistfull) ####
    if(!all(gnames %in% unique(mirt.partable$group))) ####
        stop('Defined group variable names in parlist do not match mirt.partable') ####
    #@@@
    items <- paste0( mirt.partable$item )
    for(gg in gnames){ ####
        parlist <- parlistfull[[gg]] ####
        LP <- length(parlist) ####  moved this down from first line
        for (pp in 1:LP) {
            # pp <- 1 extract parameter
            par.pp <- names(parlist)[[pp]]
            # extract data frame with parameter specifications
            dfr.pp <- parlist[[pp]]
            #@@ check item names
            g1 <- sort(setdiff( rownames(dfr.pp), items ) )
            dfr.pp <- dfr.pp[ intersect( rownames(dfr.pp), items ),, drop=FALSE]
            #@@@
            if (verbose){
                cat("*** Process group ", gg, " - parameter ", par.pp, "\n")
                if ( length(g1) > 0 ){
                    cat("  - Following items do not exist in parameter table: ", paste0( g1, collapse=" " ), "\n")
                                }
                           }

            # loop through items and parameters
            NI.pp <- nrow(dfr.pp)
            NP.pp <- ncol(dfr.pp)
            if (NI.pp>=1){
            for (ii in 1:NI.pp) {
                for (cc in 1:NP.pp) {
                    # ii <- 1 ; cc <- 2
                    ind <- which((paste(mirt.partable0$item)==rownames(dfr.pp)[ii]) &
                                     (paste(mirt.partable0$name)==colnames(dfr.pp)[cc]) &
                                     mirt.partable0$group==gg) ####
                    if (!is.na(dfr.pp[ii, cc])) {
                        if(!length(ind))
                                warning(sprintf("Parameter \'%s\' is not relevant for item %s",
                                                         colnames(dfr.pp)[cc], rownames(dfr.pp)[ii]))
                      if (par.pp !="prior.type") {
                        mirt.partable0[ind, par.pp] <- dfr.pp[ii, cc]
                      } else {
                        mirt.partable0[ind, par.pp] <- paste(dfr.pp[ii, cc])
                      }
                    }
                }
            }
            }
        }
    }
    mirt.partable0$prior.type <- as.factor(mirt.partable0$prior.type)
    if(!all(mirt.partable0$prior.type %in% c('none', 'norm', 'beta', 'lnorm'))) ####
        stop('Improper prior.type declared. Please only use the following:
             \'none\', \'norm\', \'beta\', \'lnorm\' ') ####
    return(mirt.partable0)
}
############################################################################

## File Name: lsem_fit_initial_model_sufficient_statistics.R
## File Version: 0.075

lsem_fit_initial_model_sufficient_statistics <- function(dat, variables_model,
    sampling_weights, has_meanstructure, is_imputed=FALSE, Nimp=0)
{

    dat0 <- dat
    if (!is_imputed){
        dat0 <- list(dat0)
    }
    Nimp <- max(1, Nimp)
    wmean <- list()
    wcov <- list()
    Nobs <- list()
    for (ii in 1L:Nimp){
        dat <- dat0[[ii]]
        data_suff <- dat[, variables_model]
        dat_resp <- 1 - is.na(data_suff)
        wmean[[ii]] <- lsem_weighted_mean( x=data_suff, weights=sampling_weights,
                                        x_resp=dat_resp)$mean
        res <- lsem_weighted_cov( x=data_suff, weights=sampling_weights, x_resp=dat_resp)
        wcov[[ii]] <- res$cov
        Nobs[[ii]] <- round(res$Nobs)
    }

    Nobs <- lsem_aggregate_statistics(x=Nobs)
    wcov <- lsem_aggregate_statistics(x=wcov)
    wmean <- lsem_aggregate_statistics(x=wmean)

    if (! has_meanstructure){
        wmean <- NULL
    }

    #--- output
    res <- list(wmean=wmean, wcov=wcov, Nobs=Nobs )
    return(res)
}

## File Name: isop.poly.R
## File Version: 2.15
######################################################################
# Fitting the ISOP and ADISOP model
isop.poly <- function( dat, score.breaks=seq(0,1,len=10 ),
            conv=.0001, maxit=1000, epsilon=.025,
            progress=TRUE ){
    #*****************************************
    # convert matrix into matrix format
    dat <- as.matrix(dat)
    I <- ncol(dat)
    # define response matrix
    dat.resp <- 1-is.na( dat )
    # scoring of persons
    res <- isop.scoring( dat )
    person <- res$person
    item <- res$item
    p.itemcat <- res$p.itemcat
    score.itemcat <- res$score.itemcat
    distr.fct <- res$distr.fct
    K <- max( dat, na.rm=TRUE)
    # person score is the modified percentile score
    stud.p <- person$mpsc
    # different scores of students
    if ( ! is.null( score.breaks) ){
        qsc <- stats::quantile( stud.p, probs=score.breaks)
        qsc[1] <- -1.001
        qsc[ length(qsc) ] <- 1.001
        qsc <- unique( qsc)
        stud.p <- cut( stud.p, breaks=qsc, labels=FALSE)
                        }
    person$scoregroup <- stud.p
    # score groups students
    scores <- sort( unique( stud.p ) )
    SC <- length(scores)
    # scoring of students
    # create item groups
    # create table
    freq.correct <- array( NA, dim=c(SC, I, K) )
    freq.categories <- array( NA, dim=c(SC, I, K+1) )
    dimnames(freq.categories)[[1]] <-
            dimnames(freq.correct)[[1]] <- paste0( "score_", scores )
    dimnames(freq.categories)[[2]] <-
            dimnames(freq.correct)[[2]] <- colnames(dat)
    dimnames(freq.categories)[[3]] <- paste0( "Cat", 0:K )
    dimnames(freq.correct)[[3]] <- paste0( "Cat", 0:(K-1) )
    # compute frequency table
    for (kk in 0:K){
        # kk <- 0
        if (kk < K ){
            a1 <- stats::aggregate( 1*(dat <=kk ), list( stud.p ), mean, na.rm=TRUE )
            freq.correct[,,kk+1] <- as.matrix(a1[,-1])
                    }
        a2 <- stats::aggregate( 1*(dat==kk ), list( stud.p ), sum, na.rm=TRUE )
        freq.categories[,,kk+1] <- as.matrix(a2[,-1])
            }

    # weights
    wgt <- freq.correct[,,1]
    a1 <- stats::aggregate( dat.resp, list( stud.p ), sum, na.rm=TRUE )
    wgt <- a1[,-1]
    freq.isop.fitted <- freq.correct.ordered <- list(1:K)
    for (kk in 0:(K-1) ){
        freq.correct.ordered[[kk+1]] <-
            freq.correct[, order( item$pscore ),kk+1]
                            }
    ######################################################
    #***************
    # calculate ISOP Model (for all categories)
    for (kk in 0:(K-1) ){
        if (progress){ cat("\nFit ISOP Category", kk  ) }
        # kk <- 1
        fc <- - freq.correct.ordered[[kk+1]]
        wc <- wgt[, colnames(fc) ]
        colnames(wc) -> wcc
        res.isop <- fit.isop( freq.correct=fc, wgt=wc, conv=conv,
                        maxit=maxit, progress=progress, calc.ll=FALSE)
        freq.isop.fitted[[kk+1]] <- - res.isop$fX
                    }
    ##############################################################
    #*********
    # ADISOP Model (start with the fitted isop solution)
    # compute survivor function
    surv.isop <- as.list( 1:K )
    for (kk in 0:(K-1)){
        surv.isop[[kk+1]] <- 1 - freq.isop.fitted[[kk+1]]
                    }
    # compute sum of survivor functions
    sum.surv.isop <- surv.isop[[1]]
    for (kk in 2:K){
        sum.surv.isop <- sum.surv.isop + surv.isop[[kk]]
                    }
    # ADISOP fit based on sum of survivor functions
    res.adisop <- fit.adisop( freq.correct=sum.surv.isop / K, wgt,
            conv=conv, maxit=maxit,
            epsilon=epsilon, progress=progress, calc.ll=FALSE )
    item.sc <- res.adisop$item.sc
    person.sc <- res.adisop$person.sc
    # second smoothing step in ADISOP
    # see Scheiblechner (2009)
    adfitted <- res.adisop$freq.fitted
    adfitted1 <- adfitted[,1:6]
    adfitted1 <- adfitted1[ order( 100000*adfitted1$item.index + adfitted1$stud.index ), ]
    # create a long table for survivor function
    surv.smooth <- NULL

    for (kk in 1:K){
        surv.smooth.kk <- adfitted1
        surv.smooth.kk$categ <- kk
        surv.smooth.kk$survisop <- as.vector( surv.isop[[kk]] )
        surv.smooth <- rbind( surv.smooth, surv.smooth.kk )
                    }
    surv.smooth$X1PX2 <- surv.smooth$item.sc + surv.smooth$person.sc
    surv.smooth <- surv.smooth[ order( 1000*rank( surv.smooth$X1PX2 )+
                    100 - surv.smooth$categ ), ]
    SV <- nrow(surv.smooth) / K
    fc1 <- matrix( surv.smooth$survisop, nrow=K, ncol=SV )
    wc1 <- matrix( surv.smooth$wgt, nrow=K, ncol=SV )
    # fit isop
    res.isop2 <- fit.isop( freq.correct=fc1, wgt=wc1, calc.ll=FALSE,
            progress=FALSE)
    r1 <- res.isop2$freq.fitted$freq.fitted
    surv.smooth$survadisop <- matrix( matrix( r1, nrow=K, ncol=SV, byrow=TRUE ),
            ncol=1 )
    # compute distribution function
    surv.smooth$sm.distr.fct <- 1 - surv.smooth$survadisop
    surv.smooth$item <- wcc[ surv.smooth$item.index ]
    F.orig <- array( NA, dim=c(I,K+1,SC ) )
    dimnames(F.orig)[[3]] <- paste0( "score_", scores )
    dimnames(F.orig)[[1]] <- colnames(dat)
    dimnames(F.orig)[[2]] <- paste0( "Cat", 0:K )
    F.orig[,K+1,] <- 1
    F.adisop <- F.isop <- F.orig
    F1 <- matrix( 1, K+1, SC )
    for (ii in 1:I){
        #    ii <- 1
        vv <- colnames(dat)[ii]
        surv.smooth.vv <- surv.smooth[ surv.smooth$item==vv, ]
        ind <- (surv.smooth.vv$item.index)[1]
        ind2 <-  as.matrix(surv.smooth.vv[, c("categ","stud.index") ] )
        F1[ ind2 ] <- surv.smooth.vv$sm.distr.fct
        F.adisop[vv,,] <- F1
        F1[ ind2 ] <- 1-surv.smooth.vv$survisop
        F.isop[vv,,] <- F1
                }
    F.sat1 <- aperm( freq.correct, c(2,3,1) )
    F.saturated <- F.orig
    F.saturated[, 1:K, ] <- F.sat1
    ##########################################
    #*******
    # fit graded response model
    res.grm <- fit.gradedresponse( freq.categories, SC, I, K,
            conv=conv, maxit=maxit, progress=progress )

    #########################################################
    #*****
    # calculate probabilities
    prob.saturated <- prob.isop <- prob.adisop <- F.orig
    #***********
    # calculate probabilities
    prob.saturated <-  calc.prob.distr( Farray=F.saturated)
    prob.adisop <- calc.prob.distr( Farray=F.adisop)
    prob.isop <- calc.prob.distr( Farray=F.isop)
    prob.grm <- aperm( res.grm$prob, c(2,3,1) )
    # [ items, categories, score groups ]
    # calculate survivor functions P( X>=k)
    surv.saturated <- calc.survfct.isop(prob.saturated)
    surv.isop <- calc.survfct.isop(prob.isop)
    surv.adisop <- calc.survfct.isop(prob.adisop)
    surv.grm <- calc.survfct.isop(prob.grm)

    #***
    # calculate log-likelihood
    # calculate frequencies
    freq.saturated <- prob.saturated
    for (kk in 1:(K+1) ){
        freq.saturated[, kk, ] <- t(wgt) * prob.saturated[,kk,]
                    }
    ll <- data.frame( "model"=c("saturated", "isop", "adisop", "grm"),
       "ll"=rep(NA,4) )
    ll$ll[1] <- calc.ll.isop.poly( freq1=freq.saturated, prob1=prob.saturated )
    ll$ll[2] <- calc.ll.isop.poly( freq1=freq.saturated, prob1=prob.isop )
    ll$ll[3] <- calc.ll.isop.poly( freq1=freq.saturated, prob1=prob.adisop )
    ll$ll[4] <- res.grm$ll
    NW <- mean( colSums(wgt) )
    ll$llcase <- ll$ll / NW
    #*******
    #* collect results
    res <- list( "freq.correct"=freq.correct,
              "wgt"=wgt,
              "prob.saturated"=prob.saturated,
              "prob.isop"=prob.isop, "prob.adisop"=prob.adisop,
              "prob.grm"=prob.grm,
              "surv.saturated"=surv.saturated,
              "surv.isop"=surv.isop, "surv.adisop"=surv.adisop,
              "surv.grm"=surv.grm,
              "ll"=ll,
              "person"=person, "item"=item,
              "p.itemcat"=p.itemcat, "score.itemcat"=score.itemcat,
              "fit.grm"=res.grm,
              "s2"=Sys.time(), "dat"=dat,
              "model"="isop.poly")
    class(res) <- "isop"
    return(res)
        }
##########################################################
# auxiliary function for conversion of a distribution
# function into a probability table
calc.prob.distr <- function(Farray){
        prob1 <- Farray
        KK <- dim(Farray)[2]
        prob1[,1,] <- Farray[,1,]
        for (kk in 2:KK){ prob1[,kk,] <- Farray[,kk,] - Farray[,kk-1,] }
        prob1[ prob1 < 0 ] <- 0
        prob1[ prob1 > 1 ] <- 1
        return(prob1)
            }
############################################################
# calculate log-likelihood
calc.ll.isop.poly <- function( freq1, prob1, eps=10^(-14) ){
        prob1[ prob1 < 0 ] <- 0
        prob1[ prob1 > 1 ] <- 1
          ll <- sum( freq1 * log( prob1 + eps ), na.rm=TRUE)
        return(ll)
                }
#############################################################
# calculate survivor functions
calc.survfct.isop <- function(prob1){
        prob2 <- prob1
        K <- dim(prob1)[2]
        for ( kk in seq(K-1,1,-1) ){
            prob2[,kk,] <- prob2[, kk+1, ] + prob1[,kk,]
                            }
        return(prob2)
            }

## File Name: IRT.expectedCounts_sirt.R
## File Version: 0.171



#*** object of class xxirt
IRT.expectedCounts.xxirt <- function( object, ... )
{
    ll <- object$n.ik
    attr(ll,'theta') <- object$Theta
    attr(ll,'prob.theta') <- object$probs_Theta
    attr(ll,'G') <- object$G
    return(ll)
}

#*** object of class rasch.mml
IRT.expectedCounts.rasch.mml <- function( object, ... )
{
    njk <- object$n.jk
    rjk <- object$r.jk
    dims <- dim(njk)
    ll <- array(0, dim=c(dims[1],2,dims[2],dims[3]))
    ll[,2,,] <- rjk
    ll[,1,,] <- njk-rjk
    attr(ll,'theta') <- object$theta.k
    attr(ll,'prob.theta') <- object$pi.k
    attr(ll,'G') <- object$G
    attr(ll,'dimnames') <- list()
    attr(ll,'dimnames')[[1]] <- colnames(object$dat)
    return(ll)
}

## File Name: wle.rasch.R
## File Version: 1.297


#-- WLE ability estimation
wle.rasch <- function( dat, dat.resp=NULL, b, itemweights=1+0*b,
            theta=rep(0, nrow(dat)), conv=.001, maxit=200,
            wle.adj=0, progress=FALSE)
{
    theta.change <- 1
    iter <- 0
    N <- nrow(dat)
    if ( is.null(dat.resp) ){
        dat.resp <- 1 - is.na( dat)
        # multiply response matrix with itemweights
        dat.resp <- dat.resp * sirt_matrix2(itemweights, nrow=N)
    }
    dat[ is.na( dat) ] <- 9
    sufftheta <- rowSums( dat.resp *  dat  )
    if (wle.adj>0){
        nr <- rowSums( dat.resp )
        i1 <- which( nr==sufftheta )
        if ( length(i1) > 0 ){
            sufftheta[ i1 ] <- nr[ i1 ] - wle.adj
        }
        i1 <- which( sufftheta==0 )
        if ( length(i1) > 0 ){
            sufftheta[ i1 ] <- wle.adj
        }
    }
    if ( progress){
        cat('\n  WLE estimation  |' )
    }
    old_increment <- rep( 5, length(theta))
    eps <- 1E-10
    while( ( max(abs( theta.change )) > conv ) & ( iter < maxit ) ){
        # calculate P and Q
        p.ia <- stats::plogis( theta - sirt_matrix2(b, nrow=N) )
        q.ia <- 1 - p.ia
        # Log Likelihood (for every subject)
        M0 <- dat.resp * p.ia
        l1 <- sufftheta - rowSums(M0)
        # I and J terms
        M1 <- M0*q.ia
        I.wle <- rowSums(M1)
        J.wle <- rowSums(M1*(q.ia - p.ia))
        I1.wle <- J.wle
        J1.wle <- rowSums( M1*( 6 * p.ia^2 - 6 * p.ia + 1 ) )
        # calculate objective function
        f.obj <- l1 + J.wle / ( 2 * I.wle + eps)
        # derivative of the objective function
        f1.obj <- - I.wle + ( J1.wle * I.wle - J.wle  * I1.wle )/ ( 2 * I.wle^2 + eps)
        # theta change
        theta.change <- - f.obj / ( f1.obj + eps )
        # define damped increment!!
        max_increment <- abs(old_increment)
        theta.change <- sirt_trim_increment(increment=theta.change,
                                max_increment=max_increment)
        old_increment <- abs(theta.change)
        theta <- theta + theta.change
        iter <- iter + 1
        if ( any( is.nan( theta.change ) ) ){
            stop( 'Numerical problems occur during WLE estimation procedure.')
        }
        if ( progress){
            cat('-')
        }
    }
    res <- list( theta=theta, se.theta=1/sqrt(abs(f1.obj)),
                        dat.resp=dat.resp, p.ia=p.ia )
    #*** compute WLE reliability
    v1 <- stats::var(res$theta)
    v2 <- mean(res$se.theta^2 )
    wle.rel <- ( v1 - v2 ) / v1
    cat('\nWLE Reliability=', round(wle.rel,3), '\n')
    res$wle.rel <- wle.rel
    return(res)
}

## File Name: rasch_mml2_mstep_missing1.R
## File Version: 0.159


#*** M-step for missing data model
rasch_mml2_mstep_missing1 <- function( theta.k, n.ik, mitermax, conv1,
        b, beta, delta.miss, pjk, numdiff.parm,
        constraints, est.delta, min.beta, est_delta, fixed.a=NULL, est.a=NULL,
        min.delta=-200, max.delta=200, irtmodel="missing1")
{
    I <- length(b)
    if (is.null(fixed.a)){
        fixed.a <- rep(1,fixed.a)
    }
    if (is.null(est.a)){
        est.a <- rep(0,I)
    }
    est_a <- sum(est.a) > 0
    se.a <- rep(0,I)

    h <- numdiff.parm
    se.delta <- 0
    miter <- 0
    diffindex <- est.b <- 1:I
    diffindex1 <- est.delta

    prob_fun <- rasch_mml2_calcprob_missing1

    max_incr_b <- 1
    max_incr_beta <- 1
    max_incr_delta <- 1
    max_incr_a <- 1
    miterchange <- 1000
    a0 <- fixed.a
    #---
    while( ( miterchange > conv1 ) & ( miter < mitermax ) ){

        #--- update b
        b0 <- b

        max.increment <- max_incr_b
        entry <- "b"
        diffindex <- est.b

        args0 <- list(theta.k=theta.k, b=b, beta=beta, delta.miss=delta.miss,
                    pjk=pjk, fixed.a=fixed.a, irtmodel=irtmodel)
        res <- rasch_mml2_mstep_one_step(args0=args0, prob_fun=prob_fun, entry=entry,
                        n.ik=n.ik, diffindex=diffindex, max.increment=max.increment,
                        numdiff.parm=numdiff.parm)
        args0 <- res$args0
        b <- args0[[entry]]
        max_incr_b <- res$max_increment
        se.b <- res$se
        if ( ! is.null(constraints) ){
            b[ constraints[,1] ] <- constraints[,2]
        }

        #--- update beta
        beta0 <- beta

        max.increment <- max_incr_beta
        entry <- "beta"
        diffindex <- est.b
        res <- rasch_mml2_mstep_one_step(args0=args0, prob_fun=prob_fun, entry=entry,
                        n.ik=n.ik, diffindex=diffindex, max.increment=max.increment,
                        numdiff.parm=numdiff.parm)
        args0 <- res$args0
        beta <- sirt_squeeze(args0[[entry]], lower=min.beta)
        max_incr_beta <- res$max_increment
        se.beta <- res$se

        #--- update delta
        if (est_delta){
            delta0 <- delta.miss
            max.increment <- max_incr_delta
            entry <- "delta.miss"
            diffindex <- est.delta
            res <- rasch_mml2_mstep_one_step(args0=args0, prob_fun=prob_fun, entry=entry,
                            n.ik=n.ik, diffindex=diffindex, max.increment=max.increment,
                            numdiff.parm=numdiff.parm)
            args0 <- res$args0
            delta.miss <- sirt_squeeze(args0[[entry]], lower=min.delta, upper=max.delta)
            args0[[entry]] <- delta.miss
            max_incr_delta <- res$max_increment
            se.delta <- res$se
        }
        #--- update a
        if (est_a){
            a0 <- fixed.a
            max.increment <- max_incr_a
            entry <- "fixed.a"
            diffindex <- est.a
            res <- rasch_mml2_mstep_one_step(args0=args0, prob_fun=prob_fun, entry=entry,
                            n.ik=n.ik, diffindex=diffindex, max.increment=max.increment,
                            numdiff.parm=numdiff.parm)
            args0 <- res$args0
            fixed.a <- args0[[entry]]
            max_incr_a <- res$max_increment
            se.a <- res$se

        }

        miter <- miter + 1
    }
    a_change <- max(abs(fixed.a-a0))

    #-- output
    res <- list(b=b, se.b=b, beta=beta, se.beta=se.beta,
            delta.miss=delta.miss, se.delta=se.delta, fixed.a=fixed.a,
            max_incr_a=max_incr_a, se.a=se.a, a_change=a_change)
    return(res)
}


.mstep.mml.missing1 <- rasch_mml2_mstep_missing1

## File Name: ccov_np_regression.R
## File Version: 0.141

ccov_np_regression <- function(x, y, xgrid, bwscale=1.1, smooth=TRUE, score=NULL)
{
    N <- length(x)
    if (smooth){
        y <- stats::ksmooth( x=x, y=y, bandwidth=bwscale*N^(-1/5),
                            x.points=xgrid, kernel='normal')$y
    } else {
        a1 <- stats::aggregate(y, list(score), mean, na.rm=TRUE)
        y <- a1[,2]
    }
    return(y)
}

## File Name: categorize.R
## File Version: 0.193


#--- categorize variables into classes
categorize <- function( dat, categorical=NULL, quant=NULL, lowest=0)
{

    dat2 <- dat

    #*** categorize variables, numbered from lowest to max
    dfr <- NULL
    if (! is.null( categorical) ){
        VV <- length(categorical)
        for (vv in 1L:VV){
            var.vv <- categorical[vv]
            dat.vv <- dat[,var.vv]
            vals.vv <- sort( unique( dat.vv ) )
            dfr.vv <- data.frame( index=vv, variable=var.vv,
                            column=which( colnames(dat)==var.vv ), orig=vals.vv,
                            recode=seq( lowest, length(vals.vv) -1 + lowest) )
            dfr <- rbind( dfr, dfr.vv )
            dat2[, var.vv] <- match( dat[,var.vv], vals.vv ) - 1 + lowest
        }
    }

    #*** categorize continuous variables into quantiles
    dfr2 <- NULL

    if ( ! is.null(quant) ){
        vars <- names(quant)
        VV <- length(vars)

        for (vv in 1L:VV){
            vars.vv <- vars[vv]
            q1 <- quant[ vars.vv ]
            quant.vv <- stats::quantile( dat[,vars.vv], na.rm=TRUE,
                            prob=seq( 0, 1, len=q1+1 )  )
            quant.vv[1] <- quant.vv[1] - 1
            quant.vv[q1+1] <- quant.vv[q1+1] + 1
            quant.vv <- unique( quant.vv )
            m1 <- cut( dat[,vars.vv], breaks=quant.vv )
            m2 <- sort( unique(m1) )
            dfr2.vv <- data.frame( index=vv, variable=vars.vv,
                            column=which( colnames(dat)==vars.vv ), orig=m2,
                            min=quant.vv[ - length(quant.vv) ], max=quant.vv[ -1 ],
                            recode=seq( 0, length(m2) -1 + lowest) )
            # The minimum is not included in the interval
            # while the maximum value is included in the interval.
            dfr2 <- rbind( dfr2, dfr2.vv )
            dat2[, vars.vv ] <- match( m1, m2 ) - 1 + lowest
        }
    }

    #--- OUTPUT
    res <- list( data=dat2, categ_design=dfr, quant_design=dfr2 )
    return(res)
}

## File Name: mcmc.3pno.testlet.R
## File Version: 4.29
######################################################
# MCMC estimation 2PNO model
mcmc.3pno.testlet <- function(dat, testlets=rep(NA, ncol(dat)),
        weights=NULL, est.slope=TRUE, est.guess=TRUE, guess.prior=NULL,
        testlet.variance.prior=c( 1, .2 ),
        burnin=500, iter=1000, N.sampvalues=1000,
        progress.iter=50, save.theta=FALSE, save.gamma.testlet=FALSE )
{
    s1 <- Sys.time()
    param <- 1
    # data preparation
    dat0 <- dat
    dat <- as.matrix(dat)
    dat[ is.na(dat0) ] <- 0
    dat.resp <- 1-is.na(dat0)
    N <- nrow(dat)
    I <- ncol(dat)
    eps <- 1E-10
    # testlet groups
    testlets.u <- setdiff( unique( testlets ), c(NA) )
    TT <- length(testlets.u)
    testletgroups <- match( testlets, testlets.u )
    testletgroups[ is.na(testletgroups ) ] <- TT+1
    N.items <- table( testletgroups )

    # redefine weights
    if (! is.null(weights) ){
        weights <- N * weights / sum(weights )
    }
    # set initial values
    a.testlet <- a <- rep(1,I)
    b <- stats::qnorm( (colMeans(dat0, na.rm=TRUE) + .01 )/1.02 )
    # guessing parameters
    if ( is.null(guess.prior) ){
        guess.prior <- matrix( c(1,1), nrow=I, ncol=2, byrow=TRUE )
        guess <- rep( .4, I )
    } else {
        if ( is.vector( guess.prior) ){
            guess.prior <- matrix( guess.prior, nrow=I, ncol=2, byrow=TRUE )
        }
        guess <- guess.prior[,1] / rowSums( guess.prior )
    }
    if ( ! est.guess ){ guess <- rep(0,I) }
    # item parameters in matrix form
    a.testletM <- aM <- matrix( a, nrow=N, ncol=I, byrow=TRUE)
    bM <- matrix( b, nrow=N, ncol=I, byrow=TRUE)
    # theta effects
    theta <- stats::qnorm( ( rowMeans( dat0,na.rm=TRUE ) + .01 ) / 1.02 )
    theta <- theta - mean(theta)
    mu <- 0
    sigma <- 1
    # testlet effects
    gamma.testlet <- matrix( rnorm( N*(TT+1), sd=.4 ), N, TT+1 )
    gamma.testlet[,TT+1] <- 0
    # testlet variances
    sigma.testlet <- rep( .4, TT )
    if (param==3){ sigma.testlet <- rep(1,TT) }
    # define lower and upper thresholds
    ZZ <- 1000
    threshlow <- -ZZ + ZZ*dat
    threshlow[ is.na(dat0) ] <- -ZZ
    threshupp <- ZZ*dat
    threshupp[ is.na(dat0) ] <- ZZ
    W <- dat
    # saved values
    SV <- min( N.sampvalues, iter - burnin )
    svindex <- round( seq( burnin, iter, len=SV )     )
    a.chain <- matrix( NA, SV, I )
    a.testlet.chain <- matrix(NA,SV,I)
    sigma.testlet.chain <- matrix(NA, SV, TT )
    c.chain <- b.chain <- matrix( NA, SV, I )
    theta.chain <- matrix( NA, SV, N )
    sigma.chain <- deviance.chain <- rep(NA, SV)
    gamma.testlet.chain <- matrix(NA, SV, N*(TT+1) )
    zz <- 0

    #************************************************
    #************************************************
    # begin iterations
    for (ii in 1:iter){

        #************
        # param=1
        # tau_{ni}=a_i \theta_n + \gamma_{nt} + b_i
        # param=2
        # tau_{ni}=a_i \theta_n + a_i \gamma_{nt} + b_i
        # define some intermediate parameters
        gamma.testletM <- gamma.testlet[, testletgroups ]
        if (param==1){ tau.ni <- aM * theta + gamma.testletM + bM }
        if (param==2){ tau.ni <- aM * theta + aM * gamma.testletM + bM }
        if (param==3){ tau.ni <- aM * theta + a.testletM * gamma.testletM + bM }
        #***
        # draw latent data W
        if (est.guess){
            W <- .draw.W.3pno.testlet( aM, bM, theta, gamma.testlet,
                    N, I, threshlow, threshupp, testletgroups, param, dat,
                    guess, tau.ni)
        }
        #****
        # draw latent data Z
        Z <- .draw.Z.3pno.testlet( aM, bM, theta, gamma.testlet,
                N, I, threshlow, threshupp, testletgroups, param, W,
                dat.resp, tau.ni)
        #***
        # draw latent traits theta
        theta <- .draw.theta.3pno.testlet( aM, bM, N, I, Z, param, gamma.testletM,
                 sigma,  a.testletM)

        # draw theta SD
        if ( ! est.slope ){
            sigma <- .draw.theta.variance.3pno.testlet( theta, weights, N )
        }
        #***
        # draw testlet effects
        if (TT>0){
            gamma.testlet <- .draw.gamma.3pno.testlet( aM, bM, N, I, Z, param,
                theta, testletgroups, TT, N.items, gamma.testlet,
                sigma.testlet, a.testletM)
        }

        #***
        # draw item parameters a and b
        if ( est.slope ){
            res <- mcmc_3pno_testlet_draw_itempars( theta=theta, Z=Z, I=I, N=N,
                        weights=weights, gamma.testlet=gamma.testlet,
                        testletgroups=testletgroups, param=param, TT=TT, a.testletM=a.testletM )
            a <- res$a
            b <- res$b
        }

        # draw only b item parameters
        if ( ! est.slope ){
            b <- .draw.est.b.3pno.testlet( aM, bM, N, I, Z, param,
                    gamma.testletM, sigma, weights, theta )
        }

        #***
        # draw guessing parameter
        if (est.guess){
            guess <- .draw.guess.3pno.testlet( guess.prior, W,
                    dat, dat.resp, weights, I )
        }
        # draw testlet standard deviations
        if ( (TT>0) & (param!=3) ){
            sigma.testlet <- .draw.testlet.variance.3pno.testlet( gamma.testlet, N,
                       sigma.testlet, testlet.variance.prior, weights, TT)
        }
        # draw testlet slope
        if (param==3){
            a.testlet <- .draw.est.a.testlet.3pno.testlet( aM, bM, N, I, Z, param,
                    gamma.testletM, sigma, weights, theta, testletgroups,
                    gamma.testlet, TT    )
        }

        # define item parameters in matrix form
        aM <- matrix( a, nrow=N, ncol=I, byrow=TRUE)
        bM <- matrix( b, nrow=N, ncol=I, byrow=TRUE)
        if (param==3){ a.testletM <- matrix( a.testlet, nrow=N, ncol=I, byrow=TRUE) }

        # save parameters
        if ( ii %in% svindex ){
            zz <- zz+1
            a.chain[ zz, ] <- a
            b.chain[ zz, ] <- b
            c.chain[ zz, ] <- guess
            if (param==3){
                a.testlet.chain[zz,] <- a.testlet
            }
            sigma.testlet.chain[zz,] <- sigma.testlet
            theta.chain[ zz, ] <- theta
            if (save.gamma.testlet){
                gamma.testlet.chain[zz,] <- matrix( gamma.testlet, nrow=1, ncol=N*(TT-1) )
            }
            sigma.chain[ zz ] <- sigma
            gamma.testlet.chain[zz, ] <- matrix(gamma.testlet, ncol=N*(TT+1), nrow=1 )
            deviance.chain[zz] <- .mcmc.deviance.3pno.testlet( aM, bM,
                    a.testletM, theta, guess,    gamma.testlet, testletgroups,
                    dat, dat.resp, weights, eps, param )
        }
        # print progress
        if ( ( ii %% progress.iter )==0 ){
            cat( "Iteration", ii, " | ", paste(Sys.time()), "\n")
            utils::flush.console()
        }
    }   # end MCMC iterations
    ##############################
    # output

    # EAP reliability and person parameter estimates
    res <- .mcmc.person.3pno.testlet( theta.chain, weights,
                gamma.testlet.chain )
    EAP.rel <- res$EAP.rel
    person <- res$person

    #----
    # calculate information criteria
    ic <- .mcmc.ic.3pno.testlet( a.chain, b.chain, a.testlet.chain, N, I,
        theta.chain, c.chain, gamma.testlet.chain, TT,
        testletgroups, dat, dat.resp, weights, eps, param,
        deviance.chain )

    #-----
    # create MCMC object
    res <- .mcmc.list.3pno.testlet( a.chain, b.chain, a.testlet.chain,
                I, deviance.chain, est.slope, c.chain, sigma.chain,
                est.guess, theta.chain, sigma.testlet.chain, TT,
                burnin, SV, save.theta, testletgroups, param )
    mcmcobj <- res$mcmcobj
    theta <- res$theta

    #----
    # summary of the MCMC output
    summary.mcmcobj <- mcmc.list.descriptives( mcmcobj )

    #****
    # description
    description <- .mcmc.description.3pno.testlet( TT, param, est.guess, est.slope )

    #***
    # time
    s2 <- Sys.time()
    time <- list( "start"=s1, "end"=s2, "timediff"=s2-s1 )
    #----
    # result list
    res <- list( mcmcobj=mcmcobj, summary.mcmcobj=summary.mcmcobj, ic=ic, burnin=burnin,
                iter=iter, theta.chain=theta.chain, gamma.testlet.chain=gamma.testlet.chain,
                deviance.chain=deviance.chain,
                EAP.rel=EAP.rel, person=person, dat=dat0, weights=weights, time=time,
                model='3pno.testlet', description=description, TT=TT )
    class(res) <- "mcmc.sirt"
    return(res)
}

## File Name: dirichlet.simul.R
## File Version: 0.131


#-- simulate from a Dirichlet distribution
dirichlet.simul <- function( alpha )
{
    N <- nrow(alpha)
    K <- ncol(alpha)
    ygamma <- 0*alpha
    for (ii in 1L:K){
        ygamma[,ii] <- stats::rgamma( n=N, shape=alpha[,ii] )
    }
    x <- ygamma / rowSums(ygamma)
    return(x)
}

## File Name: xxirt_prepare_response_data.R
## File Version: 0.095

xxirt_prepare_response_data <- function(G, group_index, weights, dat1,
            dat_resp, maxK )
{
    N <- nrow(dat_resp)
    I <- ncol(dat_resp)
    dat1_resp <- array(0, dim=c(N,I,maxK) )
    for (gg in 1L:G){
        ind_gg <- group_index[[gg]]
        for (kk in 1L:maxK){
            dat1_resp[ind_gg,,kk] <-  weights[ind_gg] * ( dat1[ind_gg, ]==(kk-1) ) *
                    ( dat_resp[ind_gg, ] )
        }  # end kk
    }  # end gg
    return(dat1_resp)
}

## File Name: noharm_sirt_compute_final_constants.R
## File Version: 0.05

noharm_sirt_compute_final_constants <- function(Fval, Pval, betaj,
    modesttype)
{
    #---- calculate final constants
    if (modesttype==2){
        dj2 <- diag( Fval %*% Pval %*% t(Fval) )
        Fval <- Fval / sqrt( 1 - dj2 )
    }
    # recalculation of f0 coefficient (final constant)
    dj <- sqrt( diag( Fval %*% Pval %*% t(Fval) ) )
    ej <- sqrt( 1 + dj^2 )
    f0 <- - betaj * ej
    # uniquenesses
    uqn <- 1 - ( dj^2 / ( 1 + dj^2 ) )
    # standardized loadings
    loadingsF <- Fval / ej

    #--- output
    res <- list(f0=f0, uqn=uqn, loadingsF=loadingsF)
    return(res)
}

## File Name: mirt.wrapper.fscores.R
## File Version: 0.08

##################################################################
# calculation of factor scores (EAP, MAP and MLE) in mirt
# Note that MAP and MLE are evaluated at the discrete grid
mirt.wrapper.fscores <- function( mirt.obj, weights=NULL ){
    # posterior distribution of mirt object
    mirt.post <- mirt.wrapper.posterior(mirt.obj, weights=weights)
    data <- mirt.post$data
    D <- ncol(mirt.post$theta.k)
    theta.k <- mirt.post$theta.k
    if ( is.null(weights) ){
        N <- nrow(data)
        weights <- rep(1,N)
                }
    p.xi.aj <- mirt.post$f.yi.qk
    p.aj.xi <- mirt.post$f.qk.yi
    res <- .smirt.person.parameters( data=data, D=D, theta.k=theta.k,
           p.xi.aj, p.aj.xi, weights=weights )
    return(res)
    }
##################################################################

## File Name: nr.numdiff.R
## File Version: 0.09


nr.numdiff <- function( ll0, ll1, ll2, h, eps=10^(-10) )
{
    d1 <- ( ll1 - ll2  ) / ( 2 * h )    # negative sign?
    # second order derivative
    # f(x+h)+f(x-h)=2*f(x) + f''(x)*h^2
    d2 <- ( ll1 + ll2 - 2*ll0 ) / h^2
    d2 <- ifelse( abs(d2) < eps, eps, d2 )
    delta.change <- - d1 / d2
    return(delta.change)
}

## File Name: mgsem_modify_suffstat.R
## File Version: 0.01


mgsem_modify_suffstat <- function(model, group, entry, value)
{
    for (gg in group){
        for (ee in entry){
            model[[gg]][[ee]] <- value+0*model[[gg]][[ee]]
        }  # end ee
    }  # end gg
    #--- output
    return(model)
}

## File Name: lavaan2mirt.R
## File Version: 0.641


#--- Converting lavaan syntax into mirt syntax
lavaan2mirt <- function( dat, lavmodel, est.mirt=TRUE, poly.itemtype="gpcm", ... )
{
    TAM::require_namespace_msg("mirt")
    # lavaanify model
    lavmodel2 <- TAM::lavaanify.IRT( lavmodel, data=dat )$lavpartable

    # select used items
    items <- intersect( unique( paste(lavmodel2$rhs )), colnames(dat) )
    dat <- dat[, items ]
    # maximum category
    maxK <- max( dat, na.rm=TRUE )
    ind1 <- which( lavmodel2$op=="~~")
    ind2 <- which( ( lavmodel2$lhs==lavmodel2$rhs ) & ( lavmodel2$lhs %in% items ) )
    ind <- intersect(ind1, ind2 )
    if ( length(ind) > 0 ){
        lavmodel2 <- lavmodel2[ - ind, ]
    }

    # variable names
    items <- colnames(dat)
    # extract factors
    factors <- unique( paste(lavmodel2$lhs[ lavmodel2$op=="=~" ] ))
    # create mirt syntax
    mirtmodel <- NULL
    sel.items <- NULL
    #------------
    #**** loop over factors
    for (ff in factors){
        # ff <- factors[1]
        ind.ff <- which( ( lavmodel2$op=="=~"   ) & ( lavmodel2$lhs==ff ) )
        mff <- match( lavmodel2[ ind.ff, "rhs" ], items  )
        sel.items <- c( sel.items, mff )
        mirtmodel <- paste0( mirtmodel,
                paste0( ff, "=", paste0( mff, collapse="," ), "\n")  )
    }

    #------------
    #***** look for constraints    among item parameters (loadings and intercepts)
    lavmodel21 <- lavmodel2
    lavmodel21 <- lavmodel21[ paste(lavmodel21$op) %in% c("=~","|","?="), ]
    lavlabels <- unique(paste(lavmodel21$label))
    lavlabels <- lavlabels[ paste(lavlabels ) !="" ]
    if ( length(lavlabels) > 0 ){
        vv0 <- "CONSTRAIN="
        for (ll in lavlabels ){
            # ll <- lavlabels[1]
            lav2.ll <- lavmodel21[ paste(lavmodel21$label)==ll, ]
            if (lav2.ll$op[1]=="=~"){
                pars.ll <- paste0( "a", match( lav2.ll$lhs, factors )[1] )
                isel <- lav2.ll$rhs
            }
            # Example: CONSTRAIN=(1-12,a1),(1-12,a2),(1-12,a3)
            if (lav2.ll$op[1]=="|"){
                pars.ll <- gsub( "t", "d", lav2.ll$rhs[1] )
                if (maxK==1){ pars.ll <- "d" }
                isel <- lav2.ll$lhs
            }
            if (lav2.ll$op[1]=="?="){
                pars.ll <- gsub( "g1", "g", lav2.ll$rhs[1] )
                pars.ll <- gsub( "s1", "u", pars.ll )
                isel <- lav2.ll$lhs
            }
            vv2 <- paste0( paste0( match( isel, items ), collapse=","), ",", pars.ll )
            vv2 <- paste0("(", vv2, ")")
            if (ll !=lavlabels[1] ){ vv0 <- paste0( vv0, "," ) }
            vv0 <- paste0( vv0, vv2 )
        }
        mirtmodel <- paste0( mirtmodel, vv0, "\n")
    }

    #------------
    #**** estimate variances and covariances
    lavmodel21 <- lavmodel2
    lavmodel21 <- lavmodel21[ paste(lavmodel21$op) %in% c("~~"), ]
    LL <- nrow(lavmodel21)
    vv1 <- "COV="
    if (LL>0){
        for (ll in 1:LL){
            vv2 <- paste0(lavmodel21[ll,"lhs"], "*", lavmodel21[ll,"rhs"] )
            if (ll>1){ vv1 <- paste0( vv1, "," ) }
            vv1 <- paste0( vv1, vv2 )
        }
        mirtmodel <- paste0( mirtmodel, vv1, "\n")
    }
    #------------
    #**** estimate means
    lavmodel21 <- lavmodel2
    lavmodel21 <- lavmodel21[ paste(lavmodel21$op) %in% c("~1"), ]
    lavmodel21 <- lavmodel21[ paste(lavmodel21$lhs) %in% factors, ]
    LL <- nrow(lavmodel21)
    vv1 <- "MEAN="
    if (LL>0){
        for (ll in 1:LL){
            vv2 <- paste0(lavmodel21[ll,"lhs"])
            if (ll>1){ vv1 <- paste0( vv1, "," ) }
            vv1 <- paste0( vv1, vv2 )
        }
        mirtmodel <- paste0( mirtmodel, vv1, "\n")
    }
    #------------
    #**** create object of class mirt.model
    mirtmodel1 <- mirt::mirt.model(mirtmodel)
    #------------
    #**** create parameter values
    # define item types: 4PL or gpcm
    I <- ncol(dat)
    maxK1 <- apply( dat, 2, max, na.rm=TRUE )

    typeK1 <- "4PL"
    ind1 <- which( lavmodel2$op=="=~" )
    ind2 <- which( ( ! lavmodel2$free  ) & ( lavmodel2$ustart %in% c(0,1) ) )
    ind2 <- intersect( ind1, ind2 )
    if ( length(ind1)==length(ind2) ){ typeK1 <- "Rasch" }

    itemtype <- ifelse( maxK1==1, typeK1,  poly.itemtype )

    mirtpars <- mirt::mirt( dat, mirtmodel1, itemtype=itemtype, pars="values")

    #***** handle guessing parameters
    lavmodel21 <- lavmodel2[ lavmodel2$op=="?=", ]
    lavmodel21 <- lavmodel21[ lavmodel21$rhs=="g1", ]
    lavmodel21 <- lavmodel21[ lavmodel21$free > 0, ]


    if ( nrow(lavmodel21) > 0 ){
        ind <- which( ( mirtpars$item %in% lavmodel21$lhs ) &
                            ( mirtpars$name=="g") )
        ind <- match( paste0(lavmodel21$lhs,"-","g"),
                        paste0(mirtpars$item,"-",mirtpars$name) )
        mirtpars[ind,"value"] <- .25
        mirtpars[ind,"est"] <- TRUE
    } else {    # no guessing parameters
        ind <- which( ( mirtpars$item %in% items ) &
                        ( mirtpars$name=="g") )
        if ( length( ind ) > 0 ){
            mirtpars[ind,"value"] <- 0
            mirtpars[ind,"est"] <- FALSE
        }
    }
    #***** handle slipping parameters
    lavmodel21 <- lavmodel2[ lavmodel2$op=="?=", ]
    lavmodel21 <- lavmodel21[ lavmodel21$rhs=="s1", ]
    lavmodel21 <- lavmodel21[ lavmodel21$free > 0, ]
    if ( nrow(lavmodel21) > 0 ){
        ind <- match( paste0(lavmodel21$lhs,"-","u"),
                        paste0(mirtpars$item,"-",mirtpars$name) )
        mirtpars[ind,"value"] <- .95
        mirtpars[ind,"est"] <- TRUE
    } else {    # no guessing parameters
        ind <- which( ( mirtpars$item %in% items ) & ( mirtpars$name=="u") )
        if ( length( ind ) > 0 ){
            mirtpars[ind,"value"] <- 1
            mirtpars[ind,"est"] <- FALSE
        }
    }

    #------------
    #**** constraints for parameter values
    lavmodel21 <- lavmodel2[ lavmodel2$free==0, ]
    LL <- nrow(lavmodel21)
    if (LL>0){
        for (ll in 1:LL){
            item.ll <- NULL
            lavmodel21.ll <- lavmodel21[ll,]
            # factor loadings
            if ( lavmodel21.ll$op=="=~" ){
                par.ll <- paste0("a",match( paste0(lavmodel21.ll$lhs), factors ) )
                item.ll <- paste0(lavmodel21.ll$rhs)
            }
            # thresholds
            if ( lavmodel21.ll$op=="|" ){
                par.ll <- gsub( "t", "d", lavmodel21.ll$rhs[1] )
                if (maxK==1){ par.ll <- "d" }
                item.ll <- paste0(lavmodel21.ll$lhs)
            }
            # guessing/slipping parameters
            if ( lavmodel21.ll$op=="?=" ){
                par.ll <- gsub( "g1", "g", lavmodel21.ll$rhs[1] )
                par.ll <- gsub( "s1", "u", par.ll )
                if (par.ll=="u"){
                    lavmodel21.ll$ustart <- 1 - lavmodel21.ll$ustart
                }
                item.ll <- paste0(lavmodel21.ll$lhs)
            }
            # covariances
            if ( lavmodel21.ll$lhs %in% colnames(dat) ){
                    lavmodel21.ll$op <- ""
            }
            if ( lavmodel21.ll$op=="~~" ){
                item.ll <- "GROUP"
                par.ll <- c( match( lavmodel21.ll$lhs, factors ),
                    match( lavmodel21.ll$rhs, factors ) )
                par.ll <- paste0("COV_",
                        paste0( sort(par.ll, decreasing=TRUE ), collapse="" ) )
            }
            # covariances
            if ( lavmodel21.ll$op=="~1" ){
                item.ll <- "GROUP"
                par.ll <- match( lavmodel21.ll$lhs, factors )
                if ( is.na( par.ll ) ){ item.ll <- NULL }
                par.ll <- paste0("MEAN_", par.ll )
            }
            #++ parameter fixing
            if ( ! is.null(item.ll) ){
                ind.ll <- which( ( mirtpars$item==item.ll ) & ( mirtpars$name==par.ll ) )
                mirtpars[ind.ll,"est"] <- FALSE
                mirtpars[ind.ll,"value"] <- lavmodel21.ll$ustart
            }
        }
    }

    #------------
    #*** estimate mirt model
    res <- NULL
    if ( est.mirt ){
        res$mirt <- mirt::mirt( dat, model=mirtmodel1, pars=mirtpars,
                        itemtype=itemtype,  ... )
    }   # else {
    res$mirt.model <- mirtmodel1
    res$mirt.syntax <- mirtmodel
    res$mirt.pars <- mirtpars
    res$lavaan.model <- lavmodel2
    res$dat <- dat
    return(res)
}

## File Name: L1_polish.R
## File Version: 0.26



L1_polish <- function(x, eps=0.01, maxiter=30, trace.iter=FALSE, type=1)
{
    #- original median polish procedure
    if (type==1){
        res <- stats::medpolish(x=x, eps=eps, maxiter=maxiter, trace.iter=trace.iter,
                    na.rm=TRUE)
        res <- list(overall=res$overall, row=res$row, col=res$col,
                        residuals=res$residuals)
    }
    #- alternating  median computation
    if (type==2){
        nr <- nrow(x)
        nc <- ncol(x)
        xr <- rep(0,nr)
        xc <- rep(0,nc)
        iterate <- TRUE
        iter <- 1
        while(iterate){
            xc0 <- xc
            xr0 <- xr
            #* update study effects
            delta <- x - xr
            xc <- apply(delta, 2L, stats::median, na.rm=TRUE)
            #* update item parameters
            delta <- x - sirt_matrix2(xc, nrow=nr)
            xr <- apply(delta, 1L, stats::median, na.rm=TRUE)
            change <- max(abs(xc0-xc), abs(xr0-xr), na.rm=TRUE)
            iter <- iter + 1
            if (iter > maxiter){ iterate <- FALSE }
            if (change < eps ){ iterate <- FALSE }
        }
        resid <- x - sirt_matrix2(xc, nrow=nr) - xr
        res <- list(iter=iter, overall=0, row=xr, col=xc, residuals=resid)
    }
    return(res)
}

## File Name: testlet.marginalized.R
## File Version: 0.165


testlet.marginalized <- function( tam.fa.obj=NULL, a1=NULL, d1=NULL, testlet=NULL,
    a.testlet=NULL, var.testlet=NULL)
{
    if ( ! is.null(a1) ){
        itemlabel <- rep(1L:length(a1))
    }
    if (! is.null(tam.fa.obj) ){
        mod1 <- tam.fa.obj
        a1 <- mod1$B[,2,1]
        d1 <- mod1$AXsi_[,2]
        testlets <- apply( mod1$B[,2,-1], 1, FUN=function(zz){
            l1 <- which(zz > 0 )
            if (length(l1)==0 ){ l1 <- NA }
            l1
                }
            )
        a.testlet <- mod1$B[, 2,  - 1]
        a.testlet <- a.testlet[ cbind( seq(1,length(testlets) ), testlets ) ]
        var.testlet <- diag( mod1$variance )[-1]
        testlet <- testlets
        itemlabel <- colnames(mod1$resp)
    }
    # compute marginalized item intercepts and item slopes
    k <- 16*sqrt(3)/( 15*pi )     # multiplication constant
            # 1 / k=1.700
    # compute lambda_logit
    multfac <- 1 - is.na(testlet)
    testlet1 <- testlet
    testlet1[ is.na(testlet1) ] <- 1
    a.testlet[ is.na(testlet) ] <- 0
    lambda.logit <- 1 / sqrt( 1 + k^2 * multfac * a.testlet[ testlet1 ]^2 *
                                    var.testlet[ testlet1 ] )
    # compute item parameters
    dfr <- data.frame(item=itemlabel )
    dfr$testlet <- testlet
    dfr$a1 <- a1
    dfr$d1 <- d1
    dfr$a.testlet <- a.testlet
    dfr$var.testlet <- var.testlet[ testlet ]
    # marginal parameters
    dfr$a1_marg <- dfr$a1 * lambda.logit
    dfr$d1_marg <- dfr$d1 * lambda.logit
    #--- output
    return(dfr)
}

## File Name: invariance_alignment_proc_labels.R
## File Version: 0.06

invariance_alignment_proc_labels <- function(x)
{
    G <- nrow(x)
    I <- ncol(x)
    if (is.null(colnames(x))){
        colnames(x) <- paste0('I', 1L:I)
    }
    if (is.null(rownames(x))){
        rownames(x) <- paste0('G', 1L:G)
    }
    return(x)
}

## File Name: rm_sdt_mstep_item_function_value.R
## File Version: 0.06


rm_sdt_mstep_item_function_value <- function(x, par_index, partable_item, nik.item,
    Qmatrix, theta.k, VV, K, TP, eps=1E-10)
{
    res <- rm_sdt_fill_par_to_partable( par_index=par_index, partable=partable_item,
                            parm0=x, type="item" )
    partable1 <- res$partable
    tau.item <- res$parm_list$tau.item
    a.item <- res$parm_list$a.item
    #-- compute log-likelihood
    ll <- rm_sdt_calc_expected_likelihood_item( nik.item=nik.item,
                    a.item=a.item, tau.item=tau.item, Qmatrix=Qmatrix, theta.k=theta.k,
                    VV=VV, K=K, TP=TP, eps=eps )
    ll <- -sum(ll)
    # add prior
    prior <- rm_sdt_evaluate_prior(partable=partable1)
    post <- ll + sum(prior)
    return(post)
}

## File Name: dmlavaan.R
## File Version: 0.164


dmlavaan <- function( fun1, args1, fun2, args2, method="sandwich", R=50)
{
    requireNamespace('lavaan')
    #-- estimate models
    mod1 <- dmlavaan_est_model(fun=fun1, args=args1, method=method)
    mod2 <- dmlavaan_est_model(fun=fun2, args=args2, method=method)

    #-- joint parameter table
    res <- dmlavaan_joint_parameterTable(mod1=mod1, mod2=mod2)
    partable <- res$partable
    NP <- res$NP
    parnames <- res$parnames

    #* NULL objects
    est_boot <- NULL
    boot_sample <- NULL
    V <- NULL

    #-- sandwich estimate
    if (method=='sandwich'){
        res <- dmlavaan_se_sandwich(mod1=mod1, mod2=mod2, partable=partable)
        partable <- res$partable
        V <- res$V
    }

    #-- bootstrap estimate
    if (method=='bootstrap'){
        res <- dmlavaan_se_bootstrap(mod1=mod1, mod2=mod2,
                            partable=partable, R=R)
        partable <- res$partable
        V <- res$V
        est_boot <- res$est_boot
        boot_sample <- res$boot_sample
    }

    #- create vector of coefficients
    coef <- dmlavaan_create_coef(partable=partable)

    #** define output
    partable1 <- mod1$partable
    partable2 <- mod2$partable

    #-- output
    res <- list(coef=coef, V=V, partable=partable, mod1=mod1, mod2=mod2, method=method,
                    parnames=parnames, NP=NP, partable1=partable1,
                    partable2=partable2, est_boot=est_boot,
                    boot_sample=boot_sample, R=R)
    return(res)
}

## File Name: mgsem_grad_fun.R
## File Version: 0.173


mgsem_grad_fun <- function(x, opt_fun_args, output_all=FALSE)
{

    estimator <- opt_fun_args$estimator
    NP <- opt_fun_args$NP
    ND <- opt_fun_args$ND
    G <- opt_fun_args$G
    grad_contrib <- matrix(0, nrow=ND, ncol=G)
    grad <- rep(0,NP)

    partable <- mgsem_coef2partable(coef=x, partable=opt_fun_args$partable)
    model <- mgsem_partable2model(partable=partable,
                            model=opt_fun_args$model, index=FALSE)

    #- group-wise calculations
    res <- mgsem_list_elements_est_total_implied(model=model, is_B=opt_fun_args$is_B)
    implied0 <- res$implied0
    est_total0 <- res$est_total0

    dermoments0 <- list()
    for (gg in 1L:G){
        grad_suffstat_fun_args <- list(suffstat=opt_fun_args$suffstat[[gg]],
                                        Mu=implied0[[gg]])
        if (estimator=='ME'){
            grad_suffstat_fun_args$p <- opt_fun_args$p
            grad_suffstat_fun_args$eps <- opt_fun_args$eps_approx
            grad_suffstat_fun_args$deriv <- TRUE
            grad_suffstat_fun_args$only_deriv <- TRUE
            grad_suffstat_fun_args$approx_method <- opt_fun_args$technical$approx_method
        }
        dermoments0[[gg]] <- do.call(what=opt_fun_args$grad_suffstat_fun,
                                        args=grad_suffstat_fun_args)
    }

    for (dd in 1L:ND){

        group_dd <- partable$group[dd]
        i1_dd <- partable$i1[dd]
        i2_dd <- partable$i2[dd]
        type_dd <- paste(partable$type[dd])
        index_dd <- partable$index[dd]
        recycle_dd <- partable$recycle[dd]
        recycling <- recycle_dd > 0

        gg <- 1
        sel_groups <- seq_len(G)
        if (group_dd>0){
            sel_groups <- group_dd
        }

        for (gg in sel_groups){
            if (! recycling){
                implied <- implied0[[gg]]
                dermoments <- dermoments0[[gg]]
                grad_param_fun_args <- list(est=est_total0[[gg]],
                            dermoments=dermoments, suffstat=opt_fun_args$suffstat[[gg]],
                            type=type_dd, i1=i1_dd, i2=i2_dd, h=opt_fun_args$technical$h,
                            is_B=opt_fun_args$is_B, eps=opt_fun_args$technical$eps_zero,
                            num_approx=opt_fun_args$technical$num_approx )
                gr1 <- do.call(what=opt_fun_args$grad_param_fun,
                                args=grad_param_fun_args)
            } else {
                gr1 <- grad_contrib[ recycle_dd, gg]
            }
            grad_contrib[dd,gg] <- gr1
            grad[index_dd] <- grad[index_dd] + gr1
        }
    }
    grad0 <- grad

    #-- penalty function
    if (opt_fun_args$use_penalty){
        res <- mgsem_evaluate_penalties(x=x, partable=partable,
                            prior_list=opt_fun_args$prior_list,
                            technical=opt_fun_args$technical,
                            h=opt_fun_args$technical$h, p=opt_fun_args$p_pen,
                            eps_approx=opt_fun_args$eps_approx,
                            deriv=TRUE, difflp_info=opt_fun_args$difflp_info,
                            loop_parms=opt_fun_args$loop_parms,
                            pen_type=opt_fun_args$pen_type, a_scad=opt_fun_args$a_scad)
        pen_grad_output <- res
        grad <- grad + res$pen_all
    }


    #-- negative function (minimization problem)
    grad <- -grad

    #-- output_list
    res <- grad
    if (output_all){
        res <- list(grad_fun=grad, grad=grad0, pen=pen_grad_output$pen_all,
                    dermoments=dermoments0, implied=implied0,
                    pen_grad_output=pen_grad_output, estimator=estimator,
                    N=opt_fun_args$N, N_group=opt_fun_args$N_group)
    }

    #-- output
    return(res)

}

## File Name: mgsem_scad_penalty.R
## File Version: 0.02

mgsem_scad_penalty <- function(x, lambda, a=3.7)
{
    a <- max(a,1)
    x <- abs(x)
    res <- ifelse( x < lambda, lambda * x, 0)
    res <- res + ifelse( ( x >=lambda ) & ( x < a*lambda),
                        - ( x^2 - 2*a*lambda*x+lambda^2) / ( 2*(a-1)),0 )
    res <- res + ifelse( x>=a*lambda, (a+1)*lambda^2 / 2, 0 )
    return(res)
}

## File Name: xxirt_em_algorithm.R
## File Version: 0.159

xxirt_em_algorithm <- function(maxit, verbose1, verbose2, verbose3, disp, item_list,
            items, Theta, ncat, partable, partable_index, dat, resp_index,
            dat_resp, dat_resp_bool, dat1, dat1_resp, group, customTheta, G, par0, maxK,
            group_index, weights, mstep_iter, eps, mstep_reltol, mstep_method,
            item_index, h, use_grad, penalty_fun_item, penalty_fun_theta, par1,
            globconv, conv,    verbose_index, I)
{
    iter <- 1
    dev <- 1E100
    converged <- FALSE
    while (
            ( iter < ( maxit + 1 ) ) & ( ! converged )
                ){

        if (verbose1){
            cat(disp)
            cat('Iteration', iter, '   ', paste( Sys.time() ), '\n' )
        }
        dev0 <- dev

        zz0 <- Sys.time()

        person_covariates_items <- attr(partable, 'person_covariates')

        #*** item probabilities
        probs_items <- xxirt_compute_itemprobs( item_list=item_list,
                            items=items, Theta=Theta, ncat=ncat,
                            partable=partable, partable_index=partable_index)

        #*** compute individual likelihood
        p.xi.aj <- xxirt_compute_likelihood( probs_items=probs_items, dat=dat,
                                resp_index=resp_index, dat_resp_bool=dat_resp_bool,
                                person_covariates_items=person_covariates_items )

        #*** compute prior distribution
        prior_Theta <- xxirt_compute_priorDistribution( Theta=Theta,
                              customTheta=customTheta, G=G )

        #*** compute posterior distribution and expected counts
        res <- xxirt_compute_posterior( prior_Theta=prior_Theta, p.xi.aj=p.xi.aj,
                        group=group, G=G, weights=weights, dat1=dat1,
                        dat_resp=dat_resp, maxK=maxK, group_index=group_index,
                        dat1_resp=dat1_resp, customTheta=customTheta )
        n.ik <- res$n.ik
        p.aj.xi <- res$p.aj.xi
        N.ik <- res$N.ik
        N.k <- res$N.k
        post_unnorm <- res$post_unnorm

        #*** M-step item parameters
        par00 <- par0
        res <- xxirt_mstep_itemParameters( partable=partable, item_list=item_list,
                        items=items, Theta=Theta, ncat=ncat,
                        partable_index=partable_index, N.ik=N.ik,
                        mstep_iter=mstep_iter, par0=par0, eps=eps,
                        mstep_reltol=mstep_reltol, mstep_method=mstep_method,
                        item_index=item_index, h=h, use_grad=use_grad,
                        penalty_fun_item=penalty_fun_item, p.aj.xi=p.aj.xi,
                        dat=dat, dat_resp_bool=dat_resp_bool, weights=weights)
        ll1 <- res$ll1
        partable <- res$partable
        par0 <- res$par0
        pen_val <- res$pen_val
        prior_par <- res$prior_par

        #*** M-step theta distribution
        par10 <- par1
        res <- xxirt_mstep_ThetaParameters( customTheta=customTheta, G=G, eps=eps,
                        mstep_iter=mstep_iter, N.k=N.k, par1=par1,
                        mstep_reltol=mstep_reltol, Theta=Theta,
                        penalty_fun_theta=penalty_fun_theta, weights=weights,
                        p.aj.xi=p.aj.xi)
        ll2 <- res$ll2
        customTheta <- res$customTheta
        par1 <- res$par1
        pen_theta <- res$pen_theta

        #- sum of penalty values
        pen_val <- pen_val + pen_theta

        #*** compute deviance
        ll_case <- log( rowSums( post_unnorm ) )
        dev <- - 2 * sum( weights * ll_case )
        dev00 <- dev
        dev <- dev + pen_val
        opt_fun_value <- dev00/2 + pen_val + prior_par
        globconv_temp <- abs( ( - dev + dev0 ) / dev0 )

        conv0 <- 0
        if ( length(par0) > 0){
            conv0 <- max( abs(par0-par00))
        }
        conv1 <- 0
        if ( !is.null(par1) ){
            conv1 <- max( abs(par10-par1))
        }
        conv_temp <- max( conv0, conv1)
        converged <- ( globconv_temp < globconv ) & ( conv_temp < conv )

        #-- print progress
        res <- xxirt_print_progress( opt_fun_value=opt_fun_value, dev=dev, dev0=dev0,
                    dev00=dev00, pen_val=pen_val, conv0=conv0, conv1=conv1, iter=iter,
                    verbose1=verbose1, verbose2=verbose2, verbose_index=verbose_index,
                    verbose3=verbose3, prior_par=prior_par)
        iter <- iter + 1

    }  # end iterations

    #---- output
    res <- list(iter=iter, converged=converged, probs_items=probs_items,
                    p.xi.aj=p.xi.aj, prior_Theta=prior_Theta, n.ik=n.ik,
                    p.aj.xi=p.aj.xi, N.ik=N.ik, N.k=N.k,
                    post_unnorm=post_unnorm, ll1=ll1,
                    partable=partable, par0=par0, pen_val=pen_val,
                    prior_par=prior_par, ll2=ll2, customTheta=customTheta, par1=par1,
                    ll_case=ll_case, dev=dev, dev00=dev00)
    return(res)
}


# cat('xxirt_compute_itemprobs') ; zz1 <- Sys.time(); print(zz1-zz0) ; zz0 <- zz1

## File Name: rm_trim_increments_mstep.R
## File Version: 0.062

rm_trim_increments_mstep <- function( parm, parm0, max.increment )
{
    increment <- parm - parm0
    increment <- rm_numdiff_trim_increment( increment=increment,
                        max.increment=max.increment, eps2=1E-10)
    parm <- parm0 + increment
    return(parm)
}

## File Name: linking_2groups_power_loss.R
## File Version: 0.05

linking_2groups_power_loss <- function(x, pow, eps, deriv=FALSE)
{
    #--- no derivative
    if (!deriv){
        if (pow>0 & pow < 2){
            res <- ( x^2 + eps )^(pow/2)
        }
        if (pow==0){
            res <- x^2 / ( x^2 + eps )
        }
        if (pow==2){
            res <- x^2
        }
    } else {
    #--- derivative
        if (pow>0 & pow < 2){
            res <- x*pow*( x^2 + eps )^(pow/2-1)
        }
        if (pow==0){
            # res <- x^2 / ( x^2 + eps )
            res <- 2*x*eps  / (x^2 + eps)^2
        }
        if (pow==2){
            res <- 2*x
        }
    }
    return(res)
}

## File Name: noharm_sirt_number_estimated_parameters.R
## File Version: 0.03

noharm_sirt_number_estimated_parameters <- function(I, Fpatt, Ppatt, Psipatt)
{
    Nestpars <- list(total=0)
    Nestpars$total <- Nestpars$thresh <- I
    Nestpars$F <- sum( Fpatt > 0 )
    Nestpars$total <- Nestpars$total + Nestpars$F
    Nestpars$P <- sum( diag(Ppatt)==1 ) / 2 +  sum( Ppatt==1 ) / 2
    Nestpars$total <- Nestpars$total + Nestpars$P
    Nestpars$Psi <- 1/2 * sum( Psipatt==1 )
    Nestpars$total <- Nestpars$total + Nestpars$Psi
    #--- output
    return(Nestpars)
}

## File Name: noharm_sirt_optim_gradient_R.R
## File Version: 0.042

noharm_sirt_optim_gradient_R <- function(parm_table_free, Fmat, Pmat, Psimat, FP,
    npar, NH, I, gamma_val, pm, wgtm, b0.jk, b1.jk, b2.jk, b3.jk)
{

    #-- derivatives with respect to diagonale of gamma
    res <- noharm_sirt_optim_gradient_R_der_gamma_item( parm_table_free=parm_table_free,
                Fmat=Fmat, Pmat=Pmat, FP=FP, npar=npar, NH=NH, I=I )
    grad_gamma_diag <- res$grad_gamma_diag
    grad_gamma_diag_bool <- res$grad_gamma_diag_bool

    delta <- 1+diag(gamma_val)
    grad_gamma_diag1 <- grad_gamma_diag / delta
    grad0 <- rep(0, npar)
    for (ii in 1L:(I-1)){
        for (jj in (ii+1):I){
            if (wgtm[ii,jj] >0 ){
                #- compute gradient for item pair
                grad1 <- noharm_sirt_optim_gradient_R_der_gamma_item_pair(
                            parm_table_free=parm_table_free, Fmat=Fmat, Pmat=Pmat,
                            Psimat=Psimat, FP=FP, npar=npar, NH=NH, I=I,
                            gamma_val=gamma_val, grad_gamma_diag1=grad_gamma_diag1,
                            pm=pm, b0.jk=b0.jk, b1.jk=b1.jk, b2.jk=b2.jk, b3.jk=b3.jk,
                            wgtm=wgtm, ii=ii, jj=jj )
                grad0 <- grad0 + grad1
            }
        }
    }
    #--- output
    return(grad0)
}

## File Name: summary.rasch.jml.R
## File Version: 1.12


summary.rasch.jml <- function( object, digits=3, ... )
{
    cat("-----------------------------------------------------------------\n")
    cat("Joint Maximum Likelihood Estimation \n")
    cat("Rasch Model \n\n")

    #-- print packages
    packages <- c("sirt")
    sirt_summary_print_package(pack="sirt")
    #-- print R session
    # sirt_summary_print_rsession()
    cat( TAM::tam_rsessinfo() )
    #- print call
    sirt_summary_print_call(CALL=object$CALL)
    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object)

    #-- some informations about model specification
    sirt_summary_cat_label_equal_value("Number of persons", object$N, "\n")
    sirt_summary_cat_label_equal_value("Number of items", object$I, "\n\n")

    sirt_summary_cat_label_equal_value("method", object$method, "\n")
    sirt_summary_cat_label_equal_value("center", object$center, "\n")

    cat("\n-----------------------------------------------------------------\n")
    sirt_summary_cat_label_equal_value("Deviance", object$deviance, "\n", digits=2)
    sirt_summary_cat_label_equal_value("Number of JML iterations", object$iter, "\n", digits=2)
    cat("\n")

    #** person parameters
    cat("Person Parameters \n\n")
    obji <- object$theta_summary
    sirt_summary_print_objects(obji=obji, digits=digits, from=1)

    #** item parameters
    cat("\nItem Parameters \n\n")
    obji <- object$item
    sirt_summary_print_objects(obji=obji, digits=digits, from=1)
}

## File Name: summary.latent.regression.R
## File Version: 0.12


summary.latent.regression <- function( object, ... )
{
    cat("\nRegression Parameters\n\n")
    .prnum(object$summary.coef,4)        # print results
    cat( paste( "\nResidual Variance=", round( object$sigma^2, 4 ) ), "\n" )
    cat( paste( "Explained Variance=", round( object$explvar, 4 ) ), "\n" )
    cat( paste( "Total Variance=", round( object$totalvar, 4 ) ), "\n" )
    cat( paste( "           R2=", round( object$rsquared, 4 ) ), "\n" )
}

## File Name: regpolca_postproc_count_regularized_parameters.R
## File Version: 0.292

regpolca_postproc_count_regularized_parameters <- function(item, set_equal,
        lca_dich, probs_items, nclasses, ncats)
{
    item1_index <- NULL
    n_reg <- 0
    if (lca_dich){
        ind <- c(1)
        item1 <- item[,-ind]
        TP <- ncol(item1)
        I <- nrow(item)
        item1_index <- 0*item1

        for (ii in 1:I){
            index_ii <- item1_index[ii,]
            item_ii <- item1[ii,]
            NC <- length(item_ii)
            index_ii <- 1:NC
            for (aa in 1:(NC-1)){
                val_aa <- item_ii[aa]
                for (bb in (aa+1):NC){
                    val_bb <- item_ii[bb]
                    if (abs(val_aa-val_bb)<set_equal){
                        index_ii[bb] <- index_ii[aa]
                    }
                }
            }
            ind_ii <- match( index_ii, unique(index_ii) )
            item1_index[ii,] <- ind_ii
            w1 <- t(item_ii)
            a1 <- stats::aggregate(w1[,1], list(ind_ii), mean)
            probs_mod <- a1[ind_ii,2]
            item[ii, -ind] <- probs_mod
            probs_items[ii,2,] <- probs_mod
            probs_items[ii,1,] <- 1 - probs_mod
        }  # end ii
        H <- I*TP
        n_reg <- H - sum( apply(item1_index, 1, max))
    }
    if ((! lca_dich) & (set_equal>0)){
        items <- unique(paste(item$item))
        I <- length(items)
        n_reg <- 0
        lem1 <- c(1,2)
        for (ii in 1:I){
            item_ii <- items[ii]
            ind_ii <- which( item[,'item']==item_ii )
            item_tab_ii <- item[ind_ii,-lem1]
            item_tab_ii[-c(ind_ii[1]),] <-
                            set_equal*round( item_tab_ii[-c(ind_ii[1]),] / set_equal, 0)
            item_tab_ii[1,] <- 1 - colSums(item_tab_ii[-1,])
            item[ind_ii, -c(lem1) ] <- item_tab_ii
            LU <- length( unique( unlist(item_tab_ii[-1,] )))
            H <- (nrow(item_tab_ii)-1)*ncol(item_tab_ii)
            n_reg <- n_reg + H - LU
            for (hh in 1:ncats[ii]){
                v1 <- as.numeric(item_tab_ii[hh,])
                probs_items[ii,hh,] <- v1
            }
        }
    }

    res <- list(item1_index=item1_index, n_reg=n_reg, item=item,
                probs_items=probs_items)
    return(res)
}

## File Name: summary.lsem.R
## File Version: 0.415


#-- summary lsem
summary.lsem <- function( object, file=NULL, digits=3, ... )
{
    # open sink for a file
    sirt_osink( file=file  )

    cat('-----------------------------------------------------------------\n')
    cat('Local Structural Equation Model \n\n')

    #-- print packages
    packages <- c('sirt', 'lavaan')
    if (object$use_lavaan_survey){
        packages <- c(packages, 'lavaan.survey')
    }
    sirt_summary_print_packages(packages=packages)

    #-- print R session
    cat('\n')
    sirt_summary_print_rsession()

    cat(paste0('Function \'sirt::lsem.estimate\', type=\'', object$type,'\''), '\n\n')

    #- print call
    sirt_summary_print_call(CALL=object$CALL)

    #-- print computation time
    sirt_summary_print_computation_time_s1(object=object)

    # space between equality sign
    sp_eq <- paste0( c(' ', '=', ' '), collapse='')

    cat( paste0( 'Number of observations in datasets', sp_eq,
                    round(object$N, digits) ), '\n')
    cat( paste0( 'Used observations in analysis', sp_eq,
                    round(object$nobs, digits) ), '\n')
    cat('Used sampling weights:', ! object$no_sampling_weights, '\n')
    if (object$is_imputed){
        cat( paste0( 'Number of imputed datasets: ', object$Nimp, '\n')    )
    }
    if ( object$type=='LSEM'){
        cat( paste0( 'Bandwidth factor', sp_eq, round(object$h,digits) ), '\n')
        cat( paste0( 'Bandwidth', sp_eq, round(object$bw,digits) ), '\n')
        cat( paste0( 'Number of focal points for moderator', sp_eq,
                            length(object$moderator.grid ) ), '\n')
        cat('\n')
        cat('Used joint estimation:', object$est_joint, '\n')
        cat('Used sufficient statistics:', object$sufficient_statistics, '\n')
        cat('Used local linear smoothing:', object$loc_linear_smooth, '\n')
        cat('Used pseudo weights:', object$use_pseudo_weights, '\n')
        cat('Used lavaan package:', TRUE, '\n')
        cat('Used lavaan.survey package:', object$use_lavaan_survey, '\n\n')
        cat('Mean structure modelled:', object$is_meanstructure, '\n')

        if (object$class_boot){
            v1 <- paste0('\nStatistical inference based on ', object$R,
                                ' bootstrap samples.')
            cat(v1,'\n')
        }
    }

    if ( object$type=='MGM'){
        cat( paste0( 'Number of groups for moderator=',
                            length(object$moderator.grid ) ), '\n')
    }

    cat('\nlavaan Model\n')
    cat(object$lavmodel)

    if (object$est_joint){
        cat('\n\n')
        cat('Global Fit Statistics for Joint Estimation\n\n')
        obji <- object$fitstats_joint
        sirt_summary_print_objects(obji=obji, digits=digits)
    }

    cat('\n\n')
    cat('Parameter Estimate Summary\n\n')
    obji <- object$parameters_summary
    sirt_summary_print_objects(obji=obji, digits=digits, from=2)

    cat('\n')
    cat('Distribution of Moderator: Density and Effective Sample Size\n\n')
    cat( paste0('M=', round(object$m.moderator, digits), ' | SD=',
                round(object$sd.moderator, digits), '\n\n') )
    obji <- object$moderator.density
    sirt_summary_print_objects(obji=obji, digits=digits, from=1)

    cat('\n')
    obji <- object$moderator.stat
    sirt_summary_print_objects(obji=obji, digits=digits, from=2)

    # close file
    sirt_csink(file)
}

## File Name: linking_haberman_lq_pw_le_grad.R
## File Version: 0.101


# define analytical gradient
linking_haberman_lq_pw_le_grad <- function(par_delta, par_gamma, des, h=1e-4)
{
    I <- des$I
    G <- des$G
    G1 <- G-1
    val <- matrix(0,nrow=I,ncol=2)
    design <- des$design
    w <- des$w
    ND <- nrow(design)
    coef_A <- c(1, par_delta[1L:(G-1)] )
    coef_B <- c(0, par_delta[(G-1)+1L:(G-1)] )
    NPD <- length(par_delta)
    grad <- matrix(0, nrow=I, ncol=NPD)

    for (dd in 1L:ND){
        gg_dd <- design$study1[dd]
        hh_dd <- design$study2[dd]
        ii_dd <- design$item[dd]

        #-- item discriminations
        w1 <- log( par_gamma[ 2*I*(gg_dd-1) + 2*(ii_dd-1)+1] )
        w2 <- log( par_gamma[ 2*I*(hh_dd-1) + 2*(ii_dd-1)+1] )
        e1 <- w[dd]*(w1-w2-log(coef_A[gg_dd])+log(coef_A[hh_dd]) )

        if (gg_dd>1){
            grad[ii_dd,gg_dd-1] <- grad[ii_dd,gg_dd-1] - e1
        }
        if (hh_dd>1){
            grad[ii_dd,hh_dd-1] <- grad[ii_dd,hh_dd-1] + e1
        }

        #-- item intercepts
        y1 <- par_gamma[ 2*I*(gg_dd-1) + 2*(ii_dd-1)+2] * coef_A[ gg_dd ]
        y2 <- par_gamma[ 2*I*(hh_dd-1) + 2*(ii_dd-1)+2] * coef_A[ hh_dd ]
        e2 <- w[dd]*(y1-y2-coef_B[gg_dd]+coef_B[hh_dd])
        if (gg_dd>1){
            grad[ii_dd,G1+gg_dd-1] <- grad[ii_dd,G1+gg_dd-1] - e2
        }
        if (hh_dd>1){
            grad[ii_dd,G1+hh_dd-1] <- grad[ii_dd,G1+hh_dd-1] + e2
        }
    }
    # grad <- grad/I

    return(grad)
}

## File Name: linking_haberman_lq_pw_le.R
## File Version: 0.135

linking_haberman_lq_pw_le <- function(des, res_optim, itempars,
        vcov_list=NULL, symm_hess=FALSE)
{
    requireNamespace('MASS')
    ind_studies <- des$ind_studies
    ind_items <- des$ind_items
    design <- des$design
    I <- des$I
    G <- des$G
    par_delta <- c( exp(res_optim$slopes$coefficients),
                        res_optim$intercepts$coefficients )
    parnames <- c( paste0('sig',2L:G), paste0('mu',2L:G) )
    names(par_delta) <- parnames
    par_gamma <- rep(NA, 2*I*G)
    for (gg in 1L:G){
        v1 <- paste0( rep( paste0('I', 1L:I), each=2), '_', rep(c('a','b'), I), '_G', gg )
        names(par_gamma)[ 2*I*(gg-1)+seq(1,2*I) ] <- v1
    }
    NIP <- nrow(itempars)
    for (hh in 1L:NIP){
        ind <- 2*I*(ind_studies[hh]-1)+ 2*(ind_items[hh]-1)+1
        par_gamma[ ind ] <- itempars[hh,3]
        par_gamma[ ind+1 ] <- itempars[hh,4]
    }

    #** arrange variance matrix of item parameters
    Vgamma <- linking_haberman_lq_pw_le_arrange_Vgamma( vcov_list=vcov_list,
                    par_gamma=par_gamma, I=I, G=G, ind_items=ind_items,
                    ind_studies=ind_studies )

    #*** compute derivatives
    args1 <- list(par_delta=par_delta, par_gamma=par_gamma, des=des)

    # H_delta
    grad_delta <- do.call(what=linking_haberman_lq_pw_le_grad, args=args1)
    # H_delta_delta
    hess_delta_delta <- do.call(what=linking_haberman_lq_pw_le_hess_delta, args=args1)
    if (symm_hess){
        hess_delta_delta <- ( hess_delta_delta + t(hess_delta_delta) ) / 2
    }
    HDD <- MASS::ginv(X=hess_delta_delta)

    # H_delta_gamma
    hess_delta_gamma <- do.call(what=linking_haberman_lq_pw_le_hess_gamma, args=args1)

    #-- compute standard errors
    pn2 <- list(parnames, parnames)
    A <- HDD %*% hess_delta_gamma
    V_SE <- A %*% Vgamma %*% t(A)
    dimnames(V_SE) <- pn2
    SE <- sqrt_diag_positive(x=V_SE)

    #-- compute linking errors
    B <- crossprod( grad_delta )
    V_LE <- I/(I-1) * HDD %*% B %*% t(HDD)
    dimnames(V_LE) <- pn2
    LE <- sqrt_diag_positive(x=V_LE)

    #-- compute bias-corrected linking errors
    V_LEbc <- 0*V_LE
    for (ii in 1L:I){
        ind_ii <- rep(2*I*((1L:G)-1),each=2) + rep( 2*(ii-1)+1L:2, G)
        V_gammai <- Vgamma[ind_ii, ind_ii]
        H_delta_gammai <- hess_delta_gamma[,ind_ii]
        V_LEbc <- V_LEbc + H_delta_gammai %*% V_gammai %*% t(H_delta_gammai)
    }
    V_LEbc <- I/(I-1) * HDD %*% V_LEbc %*% t(HDD)
    dimnames(V_LEbc) <- pn2
    V_LEbc <- V_LE - V_LEbc
    LEbc <- sqrt_diag_positive(x=V_LEbc)

    #-- compute total errors
    V_TE <- V_SE + V_LE
    dimnames(V_TE) <- pn2
    TE <- sqrt_diag_positive(x=V_TE)
    TEbc <- sqrt( SE^2 + LEbc^2 )

    #-- output
    res <- list(V_SE=V_SE, SE=SE, V_LE=V_LE, LE=LE, V_TE=V_TE, TE=TE,
                    V_LEbc=V_LEbc, LEbc=LEbc, TEbc=TEbc,
                    grad_delta=grad_delta, hess_delta_delta=hess_delta_delta,
                    hess_delta_gamma=hess_delta_gamma, I=I, G=G)
    return(res)
}

## File Name: mcmcirt_create_partable_Theta.R
## File Version: 0.13


mcmcirt_create_partable_Theta <- function(par, est, prior=NULL,
        prior_par1=NULL, prior_par2=NULL, sd_proposal=NULL)
{
    NP <- length(par)
    par_names <- names(par)
    dfr <- NULL
    for (pp in 1L:NP){
        name_pp <- par_names[pp]
        dfr1 <- data.frame(index=pp, par=name_pp, start=par[pp])
        dfr1$est <- est[name_pp]
        #** prior distribution
        dfr1$prior <- prior[name_pp]
        dfr1$prior_par1 <- prior_par1[name_pp]
        dfr1$prior_par2 <- prior_par2[name_pp]
        dfr1$sd_proposal <- sd_proposal[name_pp]
        dfr <- rbind(dfr, dfr1)
    }
    dfr$parindex <- cumsum(est)
    dfr$value <- dfr$start
    dfr$sampled <- 0
    dfr$accepted <- 0
    dfr$EAP <- 0
    return(dfr)
}

## File Name: smirt_preproc.R
## File Version: 1.11

################################################################
    # define estimation functions depending on condensation type
    #-#-#  calculation of probabilities
    .smirt.calcprob <- function( a, b, Q, thetak, c, d, mu.i, irtmodel ){
            if ( irtmodel=="noncomp"){
                res <- calcprob.noncomp( a, b, Q, thetak, c, d )
                                    }
            if ( irtmodel=="comp"){
                res <- calcprob.comp( a, b, Q, thetak, c, d )
                                }
            if ( irtmodel=="partcomp"){
                res <- calcprob.partcomp( a, b, Q, thetak, c, d, mu.i)
                                }
            return(res)
                                }
################################################################
    #-#-#    estimation of b parameters
    .smirt.est.b <- function(   b, a, c, d, mu.i, Qmatrix, est.b, theta.k,
            n.ik, I, K, TP, D,  numdiff.parm,
            max.increment,    msteps,  mstepconv, irtmodel, increment.factor ){
            if ( irtmodel=="noncomp"){
            res <-    .smirt.est.b.noncomp(   b, a, c, d, Qmatrix, est.b, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.increment,    msteps,  mstepconv, increment.factor)
                                }
            if ( irtmodel=="comp"){
            res <-    .smirt.est.b.comp(   b, a, c, d, Qmatrix, est.b, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.increment,    msteps,  mstepconv, increment.factor)
                                }
            if ( irtmodel=="partcomp"){
            res <-    .smirt.est.b.partcomp(   b, a, c, d, mu.i, Qmatrix, est.b, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.increment,    msteps,  mstepconv, increment.factor)
                                }
            return(res)
                                    }
################################################################
    #-#-#   estimation of a parameters
    .smirt.est.a <- function(  b, a, c, d, mu.i, Qmatrix, est.a, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.a.increment, msteps,  mstepconv, irtmodel, increment.factor){
            if ( irtmodel=="noncomp"){
             res <- .smirt.est.a.noncomp(  b, a, c, d, Qmatrix, est.a, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.a.increment, msteps,  mstepconv, increment.factor)
                                    }
            if ( irtmodel=="comp"){
             res <- .smirt.est.a.comp(  b, a, c, d, Qmatrix, est.a, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.a.increment, msteps,  mstepconv, increment.factor)
                                    }
            if ( irtmodel=="partcomp"){
             res <- .smirt.est.a.partcomp(  b, a, c, d, mu.i,  Qmatrix, est.a, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.a.increment, msteps,  mstepconv, increment.factor)
                                    }
            return(res)
                        }

################################################################
    #-#-#   estimation of c parameters
    .smirt.est.c <- function(  b, a, c, d, mu.i,  Qmatrix, est.c, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.increment, msteps,  mstepconv, irtmodel, increment.factor){
            if ( irtmodel=="noncomp"){
             res <- .smirt.est.c.noncomp(  b, a, c, d, Qmatrix, est.c, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.increment, msteps,  mstepconv, increment.factor)
                                }
            if ( irtmodel=="comp"){
             res <- .smirt.est.c.comp(  b, a, c, d, Qmatrix, est.c, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.increment, msteps,  mstepconv, increment.factor)
                                }
            if ( irtmodel=="partcomp"){
             res <- .smirt.est.c.partcomp(  b, a, c, d, mu.i, Qmatrix, est.c, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.increment, msteps,  mstepconv, increment.factor)
                                }
            return(res)
                        }

################################################################
    #-#-#   estimation of d parameters
    .smirt.est.d <- function(  b, a, c, d, mu.i, Qmatrix, est.d, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.increment, msteps,  mstepconv, irtmodel, increment.factor ){
            if ( irtmodel=="noncomp"){
             res <- .smirt.est.d.noncomp(  b, a, c, d, Qmatrix, est.d, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.increment, msteps,  mstepconv, increment.factor)
                                    }
            if ( irtmodel=="comp"){
             res <- .smirt.est.d.comp(  b, a, c, d, Qmatrix, est.d, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.increment, msteps,  mstepconv, increment.factor)
                                    }
            if ( irtmodel=="partcomp"){
             res <- .smirt.est.d.partcomp(  b, a, c, d, mu.i, Qmatrix, est.d, theta.k,
                n.ik, I, K, TP, D,  numdiff.parm,
                max.increment, msteps,  mstepconv, increment.factor)
                                    }
            return(res)
                        }




########################################################################
.smirt.check.inits <- function( a.init, b.init, irtmodel, Qmatrix){
      #****** check b.init
        if ( ! is.null( b.init) ){
            if (irtmodel=="comp"){
                if ( is.matrix(b.init) ){
                    b.init <- b.init[,1]
                    cat("*** I only use the first column of the b.init matrix \n***",
                         "because a compensatory model is requested!\n" )
                                }
                }  # end comp
            if (irtmodel=="noncomp"){
                if ( is.vector(b.init) ){
                    stop("*** A matrix for b.init is required.")
                                }
                }  # end noncomp
            } # end b.init
        #**************
      #****** check b.init
        if ( ! is.null( a.init) ){
                if ( is.vector(a.init) ){
                    stop("*** A matrix for a.init is required.")
                                }
                if ( is.matrix(a.init) ){
                   if ( ncol(a.init) !=ncol(Qmatrix) ){
                    stop("*** Check number of dimensions for specifying a.init.")
                                        }
                                }
                }  # end a.init

    res <- list("b.init"=b.init, "a.init"=a.init)
    return(res)
        }
##########################################################



#############################################
# restrict maximum increment
.adj.maxincrement.parameter <- function( oldparm, newparm,
        max.increment ){
    ISMATR <- is.matrix( oldparm )
    max.increment0 <- max.increment
    if (ISMATR){
        D <- ncol(newparm)
        for (dd in 1:D){
        if ( is.matrix(max.increment) ){
            max.increment <- max.increment0[1,dd]
                        }
        #    dd <- 1
            increment <- newparm[,dd] - oldparm[,dd]
            increment2 <- ifelse( abs(increment) > max.increment,
                                sign(increment)*max.increment, increment )
            newparm[,dd] <- oldparm[,dd] + increment2
                        }
                }
    if (!ISMATR){
            increment <- newparm - oldparm
            increment2 <- ifelse( abs(increment) > max.increment,
                                sign(increment)*max.increment, increment )
            newparm <- oldparm + increment2
                        }
    return(newparm)
        }
###########################################################

## File Name: rmvn.R
## File Version: 0.05

rmvn <- function(N, mu, Sigma, exact=TRUE)
{
    Sigma_svd <- svd(Sigma)
    D <- ncol(Sigma)
    dat0 <- matrix( stats::rnorm(N*D, mean=0, sd=1), ncol=D)
    dat00 <- dat0

    #-- compute data with exact zero means and identity covariance matrix
    if (exact){
        c1 <- stats::cov.wt(dat0, method='ML')
        dat0 <- dat0 - matrix( c1$center, nrow=N, ncol=D, byrow=TRUE)
        COV0 <- c1$cov
        c0 <- svd(COV0)
        c00 <- t(c0$u) %*% diag(1/sqrt(c0$d))
        c00 <- diag(1/sqrt(c0$d)) %*% t(c0$u)
        dat00 <- dat0 %*% t(c00)
    }

    #-- compute data with prescribed distribution
    c11 <- Sigma_svd$u %*% diag( sqrt(Sigma_svd$d) )
    dat1 <- dat00 %*% t(c11)
    dat1 <- dat1 + matrix( mu, nrow=N, ncol=D, byrow=TRUE)

    #--- output
    return(dat1)
}

## File Name: rasch_mml2_impute_data.R
## File Version: 0.127


rasch_mml2_impute_data <- function(e1, b, beta, delta.miss, pjk, fixed.a,
                            dat, dat.resp, irtmodel, nimps=10)
{
    theta.k <- e1$theta.k
    if (is.vector(theta.k)){
        theta.k <- matrix(theta.k, ncol=1)
    }

    TP <- nrow(theta.k)
    N <- nrow(dat.resp)
    I <- ncol(dat.resp)
    #--- missing1
    if (irtmodel=="missing1"){
        # P(YR|theta,xsi) is given
        # P(Y|R=0,theta, xsi)=P(Y=1,R=0,theta,xsi) /
        #                         { P(Y=0,R=0, theta,xsi)+P(Y=1,R=0, theta,xsi) }
        probs <- rasch_mml2_calcprob_missing1( theta.k=theta.k, b=b, beta=beta,
                        delta.miss=delta.miss, pjk=pjk, fixed.a=fixed.a,
                        return_nonresponse_probs=TRUE, irtmodel=irtmodel)
    }

    #--- raschtype
    if (irtmodel=="raschtype"){
        probs <- array(NA, dim=c(I,2,TP))
        probs[,1,] <- 1-t(pjk)
        probs[,2,] <- t(pjk)
    }
    post <- rowCumsums.sirt(matr=e1$f.qk.yi)
    dat_implist <- list()
    for (uu in 1:nimps){
        dat0 <- dat
        rn <- stats::runif(N)
        indices <- rowIntervalIndex.sirt(matr=post,rn=rn)
        for (ii in 1:I){
            probs_ii <- t(probs[ii,,indices])
            rn_ii <- stats::runif(N)
            y_lat <- 1*(rn_ii>probs_ii[,1])
            ind <- which(paste(dat[,ii])==2)
            if (length(ind)>0){
                dat0[ind,ii] <- y_lat[ind]
            }
        }  # end ii (items)
        dat_implist[[uu]] <- dat0
    }  # end uu (number of imputations)
    #--- output
    return(dat_implist)
}

## File Name: sirt_moving_average.R
## File Version: 0.14


#----- moving window average for time series
sirt_moving_average <- function(x, B, fill=TRUE)
{
    x1 <- cumsum(x)
    N <- length(x)
    y <- rep(NA,N)
    i <- seq(B+1, N-B)
    xdiff <- x1[ -seq(1,B) ] - x1[ -seq(N-B+1,N) ]
    xdiff <- xdiff[ - seq(1,B) ]
    y[i]  <- ( x1[i] + xdiff - c(0,x1[ -seq(N-2*B,N) ]) ) / (2*B+1)

    # fill NAs at beginning and end of time series
    if(fill){
        j <- seq(0,B-1)
        ybeg <- sapply(j, function(z) sum( x[ seq(1,(2*z+1)) ]) / (2*z+1) )
        yend <- sapply(rev(j), function(z) sum( x[ seq(N-2*z,N) ] ) / (2*z+1) )
        y[j+1] <- ybeg
        y[ rev(N-j) ] <- yend
    }
    return(y)
}


.movingAverage <- sirt_moving_average

## File Name: mgsem_proc_model.R
## File Version: 0.308

mgsem_proc_model <- function(model, G=G, random_sd=1e-1, technical, N_group, W,
        prior_list=NULL, pen_type="lasso", fixed_parms=FALSE,
        partable_start=NULL, diffpar_pen=NULL, pen_sample_size=TRUE)
{

    dfr <- NULL
    types <- c('ALPHA', 'NU', 'LAM', 'PHI', 'PSI')
    symm_types <- c( 'PHI', 'PSI')
    N <- sum(N_group)

    names_prior_list <- names(prior_list)
    is_B <- mgsem_proc_model_is_B(model=model)
    technical$is_B <- is_B
    if (is_B){
        types <- c(types, 'B')
    }

    I <- mgsem_proc_model_extract_dimension(model=model, entry='est',
                            type='LAM', nrow=TRUE)
    D <- mgsem_proc_model_extract_dimension(model=model, entry='est',
                            type='LAM', nrow=FALSE)

    #** process case of single model
    model <- mgsem_proc_model_single_group(model=model)

    #--- loop over groups
    for (gg in 0L:G){

        group <- gg
        hh <- gg+1

        #-- include missing entries
        model[[hh]] <- mgsem_proc_model_include_missing_entries(model_hh=model[[hh]],
                            types=types, entries=c('est','index'), I=I, D=D)
        model_hh <- model[[hh]]
        est <- model_hh$est
        index <- model_hh$index



        for (type in types){

            if (type %in% symm_types){
                symm <- TRUE
            } else {
                symm <- FALSE
            }
            symm0 <- symm
            M1 <- est[[type]]
            if (!is.null(M1)){
                M2 <- index[[type]]
                n1 <- nrow(M1)
                n2 <- ncol(M2)
                for (ii in 1L:n1){
                    if (symm){
                        hh <- ii
                    } else {
                        hh <- 1
                    }
                    for (jj in hh:n2){
                        if( M2[ii,jj] !=0 ){   # non-missing entry in 'est'
                            dfr1 <- data.frame( type=type, i1=ii, i2=jj,
                                                group=group)
                            dfr1$name <- paste0(dfr1$type, dfr1$i1, dfr1$i2,
                                                '_G', dfr1$group)
                            dfr1$name2 <- paste0(dfr1$type, dfr1$i1, '-', dfr1$i2,
                                                '_G', dfr1$group)
                            symm <- symm0
                            if (ii==jj){
                                symm <- FALSE
                            }
                            dfr1$symm <- symm
                            dfr1$start <- M1[ii,jj]
                            dfr1$index <- M2[ii,jj]
                            dfr1$est <- dfr1$start
                            dfr1$se <- NA
                            if (gg==0){
                                dfr1$N_group <- N
                            } else {
                                dfr1$N_group <- N_group[gg]
                            }

                            #-- check for entries
                            #-- model specifications
                            entries <- c('lower', 'upper','prior', 'pen_l2',
                                            'pen_lp', 'pen_difflp')
                            dfr1 <- mgsem_proc_model_add_specs_all(model=model_hh,
                                            entries=entries, type=type, ii=ii, jj=jj,
                                            dfr1=dfr1, names_prior_list=names_prior_list,
                                            group=group, N_group=N_group,
                                            pen_type=pen_type)
                            dfr1$unique <- 0
                            dfr1$recycle <- 0

                            #- append to previous parameters
                            dfr <- rbind(dfr, dfr1)
                        }
                    }  # end jj (i2)
                }  # end ii  (i1)
            } # !is.null(M1)
        } #end types

    }    # end gg

    dfr$ss <- 1*pen_sample_size

    if (any(duplicated(dfr$name))){
        dfr$name <- dfr$name2
    }
    dfr$name2 <- NULL

    #** define parameter indices
    res <- mgsem_proc_model_partable_define_index(partable=dfr)
    dfr <- res$partable
    NP <- res$NP
    ND <- res$ND

    #** define lp entries
    res <- mgsem_proc_model_difflp_information(partable=dfr)
    dfr <- res$partable
    difflp_info <- res$difflp_info
    technical$is_pen_difflp <- difflp_info$is_pen_difflp

    #*** specifications in technical
    technical$is_prior <- sum(dfr$prior!='none') > 0
    technical$is_pen_l2 <- sum(dfr$pen_l2>0) > 0
    technical$is_pen_lp <- sum(dfr$pen_lp>0) > 0

    technical$is_diffpar_pen <- ! is.null( diffpar_pen )
    if (technical$is_diffpar_pen){
        coef_indices <- which( rowSums( diffpar_pen$W ) > 0 )
        diffpar_pen$coef_indices <- coef_indices
        dp1 <- NULL
        NW <- ncol(W)
        for (ww in 1L:NW){
            for (uu in 1L:NW){
                val <- W[ww,uu]
                if (abs(val) > 1e-14){
                    dp2 <- data.frame(index1=ww, index2=uu, W=val)
                    dp1 <- rbind(dp1, dp2)
                }
            }
        }
        diffpar_pen$diffpar_pen_list_entries <- dp1
        technical$diffpar_pen <- diffpar_pen
    }

    dfr <- as.data.frame(dfr)

    #** coefficient vector of estimated parameters
    if ( ! is.null(partable_start) ){
        dfr$start <- dfr$est <- partable_start$est
    }
    coef <- mgsem_partable2coef(partable=dfr)

    #** induce some randomness in starting values
    if (random_sd>0){
        coef <- coef + stats::rnorm(NP, sd=random_sd)
        dfr <- mgsem_coef2partable(coef=coef, partable=dfr)
    }

    #* adapt initial values for bounded estimation
    eps1 <- 1e-2
    dfr$est <- ifelse(dfr$est<dfr$lower, dfr$lower + eps1, dfr$est )
    dfr$est <- ifelse(dfr$est>dfr$upper, dfr$upper - eps1, dfr$est )


    #** include parameter indices
    model <- mgsem_partable2model(partable=dfr, model=model, index=TRUE)

    #*** unique parameters
    loop_parms <- (1L:ND)[ dfr$unique==1]

    #- rewrite penalty parameters into model matrices
    entries <- c('pen_l2', 'pen_lp', 'pen_difflp')
    model <- mgsem_proc_model_update_penalties_matrix(partable=dfr,
                    entries=entries, model=model)

    #--- output
    res <- list(model=model, partable=dfr, NP=NP, ND=ND, coef=coef, I=I, D=D,
                    is_B=is_B, technical=technical, types=types,
                    difflp_info=difflp_info, loop_parms=loop_parms)
    return(res)
}

## File Name: lavaan_object_adjust_sample_size.R
## File Version: 0.106

#- adjust standard errors in lavaan object for modified sample size
lavaan_object_adjust_sample_size <- function(object, n_used)
{
    object2 <- object

    # adjustment factor for standard errors
    Data <- object@Data
    ngroups <- Data@ngroups
    nobs_all <- sum(unlist(Data@nobs))
    fac2 <- nobs_all / n_used
    fac <- sqrt(fac2)

    #- Data@nobs
    nobs <- nobs0 <- Data@nobs
    for (gg in 1:ngroups){
        nobs[[gg]] <- round(nobs[[gg]] / fac2, 0)
    }
    Data@nobs <- nobs

    #- Data@weights
    weights <- Data@weights
    NW <- unlist(weights)
    if (is.null(NW)){
        weights <- list()
        for (gg in 1:ngroups){
            weights[[gg]] <- rep(1, nobs0)
        }
    }
    for (gg in 1:ngroups){
        weights[[gg]] <- weights[[gg]] / fac2
    }
    Data@weights <- weights

    #- ParTable$se
    ParTable <- object@ParTable
    ParTable$se <- ParTable$se * fac

    #- Fit$se
    Fit <- object@Fit
    Fit@se <- Fit@se * fac

    #- vcov@vcov
    vcov <- object@vcov
    vcov$vcov <- vcov$vcov * fac2

    #** include all redefined objects
    object2@Data <- Data
    object2@ParTable <- ParTable
    object2@Fit <- Fit
    object2@vcov <- vcov

    #-- output
    return(object2)
}

## File Name: rasch_mml2_prior_information_generate_string.R
## File Version: 0.02


rasch_mml2_prior_information_generate_string <- function(prior, distribution)
{
    string <- "None"
    if (!is.null(prior)){
        string <- paste0(distribution,"(", prior[1], ",", prior[2],")")
    }
    return(string)
}

## File Name: invariance_alignment_cfa_config.R
## File Version: 0.261


invariance_alignment_cfa_config <- function(dat, group, weights=NULL,
    model="2PM", verbose=FALSE, ...)
{
    CALL <- match.call()
    is_data <- sirt_is_data(dat=dat)
    if (! is_data){
        mu_list <- dat[[1]]
        Sigma_list <- dat[[2]]
        N_list <- dat[[3]]
        I <- length(mu_list[[1]])
        group <- seq(1,length(mu_list))
        is_data <- FALSE
    }
    if (is_data){
        ind <- order(group)
        dat <- dat[ ind, ]
        group <- group[ind]
        I <- ncol(dat)
        if (!is.null(weights)){
            weights <- weights[ind]
        }
    }
    groups <- unique(group)
    G <- length(groups)
    items <- colnames(dat)
    N <- rep(NA, G)
    names(N) <- groups
    nu <- matrix(NA, nrow=G, ncol=I)
    rownames(nu) <- groups
    colnames(nu) <- items
    lambda <- nu
    err_var <- nu
    weights_gg <- NULL
    vcov <- matrix(0, nrow=2*I*G, ncol=2*I*G)
    names1 <- c()
    for (gg in 1L:G){
        names2 <- c( paste0(items, '_lam_Gr',gg), paste0(items, '_nu_Gr',gg) )
        names1 <- c( names1, names2)
    }
    colnames(vcov) <- rownames(vcov) <- names1

    vcov_ind0 <- 0
    for (gg in 1L:G){
        if (is_data){
            dat_gg <- dat[ group==groups[gg], ]
            dat_gg <- dat_gg[, colMeans(is.na(dat_gg)) < 1 ]
            items_gg <- colnames(dat_gg)
            ind_gg <- match(items_gg, items)
            if (!is.null(weights)){
                weights_gg <- weights[group==groups[gg]]
            }
            args <- list(dat_gg=dat_gg, weights_gg=weights_gg, model=model,
                            N=nrow(dat_gg),...)
        }
        if (!is_data){
            dat_gg <- list(mu=mu_list[[gg]], Sigma=Sigma_list[[gg]], N=N_list[[gg]])
            args <- list(dat_gg=dat_gg, weights_gg=NULL, model=model)
            args$N <- dat_gg$N
            ind_gg <- 1L:I
        }
        cat( paste0('Compute CFA for group ', gg, ' | model ', model, '\n') )
        utils::flush.console()
        res <- do.call(what='invariance_alignment_cfa_config_estimate', args=args)
        nu[gg, ind_gg] <- res$nu
        lambda[gg, ind_gg] <- res$lambda
        err_var[gg, ind_gg] <- res$err_var
        N[gg] <- res$nobs
        ind_gg1 <- vcov_ind0 + 1L:(2*I)
        vcov[ind_gg1, ind_gg1] <- res$vcov
        vcov_ind0 <- vcov_ind0+2*I
    }
    #-- output
    res <- list(nu=nu, lambda=lambda, err_var=err_var, vcov=vcov, N=N, G=G, I=I,
                    items=items, groups=groups, CALL=CALL)
    return(res)
}

## File Name: genlogis.moments.R
## File Version: 0.02


#-- moments of generalized logistic distribution
genlogis.moments <- function( alpha1, alpha2)
{
    x0 <- seq(-30, 30, len=30000 )
    y0 <- pgenlogis( x=x0, alpha1=alpha1, alpha2=alpha2 )
    wgt <- y0[-1] - y0[ - length(y0) ]
    wgt <- wgt / sum(wgt)
    out <- ( x0[ -1 ] + x0[ - length(x0) ] ) / 2
    M <- sum( wgt * out )
    SD <- sqrt( sum( wgt*out^2 ) - M^2 )
    moments <- c(M, SD, SD^2 )
    names(moments) <- c('M','SD','Var')
    return(moments)
}

## File Name: smirt_postproc.R
## File Version: 0.123

#*** person parameter estimates
.smirt.person.parameters <- function( data, D, theta.k,
    p.xi.aj, p.aj.xi, weights )
{

    person <- data.frame(case=1L:(nrow(data)), M=rowMeans( data, na.rm=TRUE))
    EAP.rel <- rep(0,D)
    names(EAP.rel) <- colnames(theta.k)
    nstudl <- rep(1,nrow(data))
    for (dd in 1L:D){ #dd <- 1
        dd1 <- colnames(theta.k)[dd]
        person$EAP <- rowSums( p.aj.xi * outer( nstudl, theta.k[,dd] ) )
        person$SE.EAP <- sqrt(rowSums( p.aj.xi * outer( nstudl, theta.k[,dd]^2 ) ) -
                                            person$EAP^2)
        EAP.variance <- stats::weighted.mean( person$EAP^2, weights ) -
                        ( stats::weighted.mean( person$EAP, weights ) )^2
        EAP.error <- stats::weighted.mean( person$SE.EAP^2, weights )
        EAP.rel[dd] <- EAP.variance / ( EAP.variance + EAP.error )
        colnames(person)[ which( colnames(person)=="EAP" ) ] <-
                                                paste("EAP.", dd1, sep="")
        colnames(person)[ which( colnames(person)=="SE.EAP" ) ] <-
                                                paste("SE.EAP.", dd1, sep="")
        }
    if ( is.null( names(EAP.rel) ) ){
        names(EAP.rel) <- paste0( "Dim", 1L:(length(EAP.rel)) )
    }
    # MLE
    mle.est <- theta.k[ max.col( p.xi.aj ),, drop=FALSE]
    colnames(mle.est) <- paste0( "MLE.", names(EAP.rel))
    person <- cbind( person, mle.est )
    # MAP
    mle.est <- theta.k[ max.col( p.aj.xi ),, drop=FALSE]
    colnames(mle.est) <- paste0( "MAP.", names(EAP.rel))
    person <- cbind( person, mle.est )
    # results
    res <- list( person=person, EAP.rel=EAP.rel )
    return(res)
}

## File Name: isop.scoring.R
## File Version: 1.11
#################################################
# scoring students and items according to the
# Scheiblechner's ISOP model
isop.scoring <- function( dat, score.itemcat=NULL){
    #***
    # compute modified percentile score
    maxK <- apply( dat, 2, max, na.rm=TRUE )
    K <- max(maxK)
    I <- ncol(dat)
    # define response matrix
    dat.resp <- 1-is.na( dat )
    # p values for every category
    p.itemcat <- matrix(NA, nrow=I, ncol=K+1 )
    rownames(p.itemcat) <- colnames(dat)
    colnames(p.itemcat) <- paste0( "p.Cat", 0:K )
    for (kk in 0:K){
        p.itemcat[,kk+1] <- colMeans( dat==kk, na.rm=TRUE )
                }
    #**************************************
    if ( is.null( score.itemcat )){
        score.itemcat <- 0*p.itemcat
        colnames(score.itemcat) <- paste0( "score.Cat", 0:K )
        # score itemcat
        for (kk in 0:K){
            if (kk>0){
                v1 <- apply( p.itemcat[, 1+seq( 0, kk -1, 1 ), drop=FALSE],
                            1, sum, na.rm=TRUE )
                    } else { v1 <- 0 }
            if (kk<K){
                v2 <- apply( p.itemcat[, 1+seq( kk+1, K, 1 ), drop=FALSE],
                            1, sum, na.rm=TRUE )
                    } else { v2 <- 0 }
            score.itemcat[,kk+1] <- v1-v2
                    }
                }  # end score itemcat
      #*************************
      dat <- dat[, rownames(score.itemcat) ]
      # scoring table
      score.dat <- 0*dat
      for (kk in 0:K){
        score.dat <- score.dat + matrix( score.itemcat[,kk+1], nrow=nrow(dat),
                        ncol=I, byrow=TRUE ) * ( dat==kk )
                    }
      # score persons
      person <- data.frame( "pid"=1:nrow(dat),
                    "score"=rowSums( dat, na.rm=TRUE ),
                    "max"=rowSums( dat.resp ),
                    "M"=rowMeans( dat, na.rm=TRUE ),
                    "mpsc"=rowMeans( score.dat, na.rm=TRUE )
                            )
      # compute item p score
      item.pscore <- rep(NA,I)
      for (ii in 1:I){
#          ii <- 1
          v1 <- colMeans( dat[,ii ] > dat[,-ii] )
          v2 <- colMeans( dat[,ii ] < dat[,-ii] )
          item.pscore[ii] <- mean( v1 - v2 )
                        }
      # score items
      item <- data.frame( "item"=colnames(dat),
               "M"=colMeans( dat, na.rm=TRUE ) )
      item$pscore <- item.pscore
      item <- cbind( item, p.itemcat,  score.itemcat )
      # calculate distribution function
      distr.fct <- t(apply( p.itemcat, 1, FUN=function(ll){
                    cumsum( ll ) } ) )
#      colnames(item)[-c(1:2)] <- colnames( score.itemcat )[-1]
      #***
      # collect results
      res <- list( "person"=person, "item"=item,
            "p.itemcat"=p.itemcat, "score.itemcat"=score.itemcat,
            "distr.fct"=distr.fct)
      return(res)
      }
##############################################################
# example scoring for item with 3 categories
# probabilities p0, p1, p2
# score category 0: s0=- ( p1 + p2 )
# score category 1: s1=p0 - p2
# score category 2: s2=p0 + p1
# ***
# s2 + s0=p0 + p1 - p1 - p2=p0 - p2=s1
# ***
# s2 - s1=p0 + p1 - p0 + p2=p1 + p2
# s1 - s0=p0 - p2 + p1 + p2=p0 + p1

## File Name: lsem_permutationTest_define_exported_objects.R
## File Version: 0.105

lsem_permutationTest_define_exported_objects <- function()
{
    varlist <- c('arglist', 'lsem_permutationTest_collect_output')
    return(varlist)
}

%% File Name: data.timss.Rd
%% File Version: 0.09

\name{data.timss}
\alias{data.timss}
\docType{data}
\title{
Dataset TIMSS Mathematics
}
\description{
This datasets contains TIMSS mathematics data from 345 students
on 25 items.
}
\usage{data(data.timss)}
\format{
This dataset is a list. \code{data} is the dataset containing
student ID (\code{idstud}), a dummy variable for female (\code{girl})
and student age (\code{age}). The following variables (starting with
\code{M} in the variable name are items.

The format is:

\code{List of 2} \cr
\code{ $ data:'data.frame':} \cr
\code{  ..$ idstud  : num [1:345] 4e+09 4e+09 4e+09 4e+09 4e+09 ...} \cr
\code{  ..$ girl    : int [1:345] 0 0 0 0 0 0 0 0 1 0 ...} \cr
\code{  ..$ age     : num [1:345] 10.5 10 10.25 10.25 9.92 ...} \cr
\code{  ..$ M031286 : int [1:345] 0 0 0 1 1 0 1 0 1 0 ...} \cr
\code{  ..$ M031106 : int [1:345] 0 0 0 1 1 0 1 1 0 0 ...} \cr
\code{  ..$ M031282 : int [1:345] 0 0 0 1 1 0 1 1 0 0 ...} \cr
\code{  ..$ M031227 : int [1:345] 0 0 0 0 1 0 0 0 0 0 ...} \cr
\code{    [...]} \cr
\code{  ..$ M041203 : int [1:345] 0 0 0 1 1 0 0 0 0 1 ...} \cr
\code{ $ item:'data.frame':} \cr
\code{  ..$ item            : Factor w/ 25 levels "M031045","M031068",..: ...} \cr
\code{  ..$ Block           : Factor w/ 2 levels "M01","M02": 1 1 1 1 1 1 ..} \cr
\code{  ..$ Format          : Factor w/ 2 levels "CR","MC": 1 1 1 1 2  ...} \cr
\code{  ..$ Content.Domain  : Factor w/ 3 levels "Data Display",..: 3 3 3 3  ...} \cr
\code{  ..$ Cognitive.Domain: Factor w/ 3 levels "Applying","Knowing",..: 2 3 3 ..} \cr
}




%% \keyword{datasets}

%% File Name: IRT.mle.Rd
%% File Version: 0.24

\name{IRT.mle}
\alias{IRT.mle}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Person Parameter Estimation
}
\description{
Computes the maximum likelihood estimate (MLE),
weighted likelihood estimate (WLE) and maximum aposterior
estimate (MAP) of ability in unidimensional item response models
(Penfield & Bergeron, 2005; Warm, 1989). Item response functions can be
defined by the user.
}

\usage{
IRT.mle(data, irffct, arg.list, theta=rep(0,nrow(data)), type="MLE",
     mu=0, sigma=1, maxiter=20, maxincr=3, h=0.001, convP=1e-04,
     maxval=9, progress=TRUE)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
Data frame with item responses
}
\item{irffct}{
User defined item response (see Examples). Arguments must be
specified in \code{arg.list}. The function must contain \code{theta}
and \code{ii} (item index) as arguments.
}
  \item{theta}{
Initial ability estimate
}
  \item{arg.list}{
List of arguments for \code{irffct}.
}
  \item{type}{Type of ability estimate. It can be \code{"MLE"} (the default),
        \code{"WLE"} or \code{"MAP"}.
}
\item{mu}{Mean of normal prior distribution (for \code{type="MAP"})}
\item{sigma}{Standard deviation of normal prior distribution (for \code{type="MAP"})}
  \item{maxiter}{
Maximum number of iterations
}
  \item{maxincr}{
Maximum increment
}
  \item{h}{
Numerical differentiation parameter
}
  \item{convP}{
Convergence criterion
}
  \item{maxval}{
Maximum ability value to be estimated
}
  \item{progress}{
Logical indicating whether iteration progress should be displayed
}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}


\value{
Data frame with estimated abilities (\code{est}) and its standard error
(\code{se}).
}

\references{
Penfield, R. D., & Bergeron, J. M. (2005). Applying a weighted
maximum likelihood latent trait estimator to the generalized
partial credit model. \emph{Applied Psychological Measurement,
29}, 218-233.

Warm, T. A. (1989). Weighted likelihood estimation of ability in item response theory.
\emph{Psychometrika, 54}, 427-450.
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also the \pkg{PP} package for further person parameter
estimation methods.
}

\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Generalized partial credit model
#############################################################################

data(data.ratings1)
dat <- data.ratings1

# estimate model
mod1 <- sirt::rm.facets( dat[, paste0( "k",1:5) ], rater=dat$rater,
             pid=dat$idstud, maxiter=15)
# extract dataset and item parameters
data <- mod1$procdata$dat2.NA
a <- mod1$ipars.dat2$a
b <- mod1$ipars.dat2$b
theta0 <- mod1$person$EAP
# define item response function for item ii
calc.pcm <- function( theta, a, b, ii ){
    K <- ncol(b)
    N <- length(theta)
    matrK <- matrix( 0:K, nrow=N, ncol=K+1, byrow=TRUE)
    eta <- a[ii] * theta * matrK - matrix( c(0,b[ii,]), nrow=N, ncol=K+1, byrow=TRUE)
    eta <- exp(eta)
    probs <- eta / rowSums(eta, na.rm=TRUE)
    return(probs)
}
arg.list <- list("a"=a, "b"=b )

# MLE
abil1 <- sirt::IRT.mle( data, irffct=calc.pcm, theta=theta0, arg.list=arg.list )
str(abil1)
# WLE
abil2 <- sirt::IRT.mle( data, irffct=calc.pcm, theta=theta0, arg.list=arg.list, type="WLE")
str(abil2)
# MAP with prior distribution N(.2, 1.3)
abil3 <- sirt::IRT.mle( data, irffct=calc.pcm, theta=theta0, arg.list=arg.list,
              type="MAP", mu=.2, sigma=1.3 )
str(abil3)

#############################################################################
# EXAMPLE 2: Rasch model
#############################################################################

data(data.read)
dat <- data.read
I <- ncol(dat)

# estimate Rasch model
mod1 <- sirt::rasch.mml2( dat )
summary(mod1)

# define item response function
irffct <- function( theta, b, ii){
    eta <- exp( theta - b[ii] )
    probs <- eta / ( 1 + eta )
    probs <- cbind( 1 - probs, probs )
    return(probs)
}
# initial person parameters and item parameters
theta0 <- mod1$person$EAP
arg.list <- list( "b"=mod1$item$b  )

# estimate WLE
abil <- sirt::IRT.mle( data=dat, irffct=irffct, arg.list=arg.list,
            theta=theta0, type="WLE")
# compare with wle.rasch function
theta <- sirt::wle.rasch( dat, b=mod1$item$b )
cbind( abil[,1], theta$theta, abil[,2], theta$se.theta )

#############################################################################
# EXAMPLE 3: Ramsay quotient model
#############################################################################

data(data.read)
dat <- data.read
I <- ncol(dat)

# estimate Ramsay model
mod1 <- sirt::rasch.mml2( dat, irtmodel="ramsay.qm" )
summary(mod1)
# define item response function
irffct <- function( theta, b, K, ii){
    eta <- exp( theta / b[ii] )
    probs <- eta / ( K[ii] + eta )
    probs <- cbind( 1 - probs, probs )
    return(probs)
}
# initial person parameters and item parameters
theta0 <- exp( mod1$person$EAP )
arg.list <- list( "b"=mod1$item2$b, "K"=mod1$item2$K )
# estimate MLE
res <- sirt::IRT.mle( data=dat, irffct=irffct, arg.list=arg.list, theta=theta0,
            maxval=20, maxiter=50)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.

%% \keyword{Person parameters}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: mcmc_Rhat.Rd
%% File Version: 0.09

\name{mcmc_Rhat}
\alias{mcmc_Rhat}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Computation of the Rhat Statistic from a Single MCMC Chain
}
\description{
Computes the Rhat statistic from a single MCMC chain.
}
\usage{
mcmc_Rhat(mcmc_object, n_splits=3)
}
\arguments{
  \item{mcmc_object}{
Object of class \code{mcmc}
}
  \item{n_splits}{
Number of splits for MCMC chain
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
Numeric vector
}

%\references{
%}

%\author{
%Alexander Robitzsch
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%}


\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Computation Rhat statistic for 2PNO model fitting by MCMC
#############################################################################

data(data.read)

# estimate 2PNO with MCMC with 3000 iterations and 500 burn-in iterations
mod <- sirt::mcmc.2pno( dat=data.read, iter=1000, burnin=100 )
# plot MCMC chains
plot( mod$mcmcobj, ask=TRUE )
# compute Rhat statistics
round( sirt::mcmc_Rhat( mod$mcmcobj[[1]] ), 3 )
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{Testlets}

%% File Name: data.liking.science.Rd
%% File Version: 0.08

\name{data.liking.science}
\alias{data.liking.science}
\docType{data}

\title{
Dataset 'Liking For Science'
}

\description{
Dataset 'Liking for science' published by Wright and Masters (1982).
}

\usage{
data(data.liking.science)
}

\format{
The format is:

\code{ num [1:75, 1:24] 1 2 2 1 1 1 2 2 0 2 ...} \cr
\code{ - attr(*, "dimnames")=List of 2} \cr
\code{  ..$ : NULL} \cr
\code{  ..$ : chr [1:24] "LS01" "LS02" "LS03" "LS04" ...} \cr
}

%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}

%\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%}



\references{
Wright, B. D., & Masters, G. N. (1982). \emph{Rating scale analysis}.
Chicago: MESA Press.
}
%\examples{
%data(data.liking.science)
%## maybe str(data.liking.science) ; plot(data.liking.science) ...
%}
%% \keyword{datasets}

%% File Name: unidim.test.csn.Rd
%% File Version: 1.24

\name{unidim.test.csn}
\alias{unidim.test.csn}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Test for Unidimensionality of CSN
}

\description{
This function tests whether item covariances given the sum
score are non-positive (CSN; see Junker 1993),
i.e. for items \eqn{i} and \eqn{j} it holds that
\deqn{ Cov( X_i, X_j | X^+ ) \le 0 }
Note that this function only works for dichotomous data.
}


\usage{
unidim.test.csn(dat, RR=400, prop.perm=0.75, progress=TRUE)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame with dichotomous item responses. All persons with (some)
missing responses are removed.
}
  \item{RR}{
Number of permutations used for statistical testing
}
  \item{prop.perm}{
A positive value indicating the amount of permutation in an
existing permuted data set
}
  \item{progress}{
An optional logical indicating whether computation progress
should be displayed
}
}

\details{
For each item pair \eqn{(i,j)} and a each sum score group \eqn{k}
a conditional covariance \eqn{r(i,j|k)} is calculated. Then,
the test statistic for CSN is
\deqn{ h=\sum_{k=1}^{I-1} \frac{n_k}{n} \max_{i,j} r(i,j|k) }
where \eqn{n_k} is the number of persons in score group \eqn{k}.
"'Large values"' of \eqn{h} are not in agreement with the null
hypothesis of non-positivity of conditional covariances.

The distribution of the test statistic \eqn{h} under the null
hypothesis is empirically obtained by column wise permutation
of items within all score groups. In the population, this procedure
corresponds to conditional covariances of zero. See de Gooijer and Yuan (2011)
for more details.
}

\value{
A list with following entries
\item{stat}{Value of the statistic}
\item{stat_perm}{Distribution of statistic under \eqn{H_0} of
permuted dataset}
\item{p}{The corresponding p value of the statistic}
\item{H0_quantiles}{Quantiles of the statistic under permutation
(the null hypothesis \eqn{H_0})}

}


\references{
De Gooijer, J. G., & Yuan, A. (2011). Some exact tests for
manifest properties of latent trait models.
\emph{Computational Statistics and Data Analysis, 55},
34-44.

Junker, B.W. (1993). Conditional association, essential independence, and
monotone unidimensional item response models.
\emph{Annals of Statistics, 21}, 1359-1378.
}


%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%MH statistic conditional association
%}


\examples{
#############################################################################
# EXAMPLE 1: Dataset data.read
#############################################################################

data(data.read)
dat <- data.read
set.seed(778)
res <- sirt::unidim.test.csn( dat )
  ##  CSN Statistic=0.04737, p=0.02

\dontrun{
#############################################################################
# EXAMPLE 2: CSN statistic for two-dimensional simulated data
#############################################################################

set.seed(775)
N <- 2000
I <- 30   # number of items
rho <- .60   # correlation between 2 dimensions
t0 <- stats::rnorm(N)
t1 <- sqrt(rho)*t0 + sqrt(1-rho)*stats::rnorm(N)
t2 <- sqrt(rho)*t0 + sqrt(1-rho)*stats::rnorm(N)
dat1 <- sirt::sim.raschtype(t1, b=seq(-1.5,1.5,length=I/2) )
dat2 <- sirt::sim.raschtype(t2, b=seq(-1.5,1.5,length=I/2) )
dat <- as.matrix(cbind( dat1, dat2) )
res <- sirt::unidim.test.csn( dat )
  ##  CSN Statistic=0.06056, p=0.02
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Test for unidimensionality}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: latent.regression.em.raschtype.Rd
%% File Version: 1.481

\name{latent.regression.em.raschtype}
\alias{latent.regression.em.raschtype}
\alias{latent.regression.em.normal}
\alias{summary.latent.regression}


\title{
Latent Regression Model for the Generalized
Logistic Item Response Model and the Linear Model for Normal Responses
}
\description{
This function estimates a unidimensional latent regression model if
a likelihood is specified, parameters from the generalized
item response model (Stukel, 1988) or a mean and a standard error
estimate for individual scores is provided as input.
Item parameters are treated as fixed in the estimation.
}

\usage{
latent.regression.em.raschtype(data=NULL, f.yi.qk=NULL, X,
    weights=rep(1, nrow(X)), beta.init=rep(0,ncol(X)),
    sigma.init=1, b=rep(0,ncol(X)), a=rep(1,length(b)),
    c=rep(0, length(b)), d=rep(1, length(b)), alpha1=0, alpha2=0,
    max.parchange=1e-04, theta.list=seq(-5, 5, len=20),
    maxiter=300, progress=TRUE )

latent.regression.em.normal(y, X, sig.e, weights=rep(1, nrow(X)),
    beta.init=rep(0, ncol(X)), sigma.init=1, max.parchange=1e-04,
    maxiter=300, progress=TRUE)

\method{summary}{latent.regression}(object,...)
}

\arguments{
  \item{data}{ An \eqn{N \times I} data frame of dichotomous item responses.
  If no data frame is supplied, then a user can input the individual
  likelihood \code{f.yi.qk}.
   }
  \item{f.yi.qk}{An optional matrix which contains the individual likelihood.
  This matrix is produced by \code{\link{rasch.mml2}} or
  \code{\link{rasch.copula2}}. The use of this argument allows the
  estimation of the latent regression model independent of the
  parameters of the used item response model.
    }
  \item{X}{An \eqn{N \times K} matrix of \eqn{K} covariates in the latent
    regression model. Note that the intercept (i.e. a vector of ones)
    must be included in \code{X}.
}
  \item{weights}{
Student weights (optional).
}
  \item{beta.init}{
Initial regression coefficients (optional).
}
\item{sigma.init}{
Initial residual standard deviation (optional).
}
  \item{b}{
Item difficulties (optional). They must only be provided
if the likelihood \code{f.yi.qk} is not given as an input.
}
  \item{a}{
Item discriminations (optional).
}
  \item{c}{
Guessing parameter (lower asymptotes) (optional).
}
  \item{d}{
One minus slipping parameter (upper asymptotes) (optional).
}
  \item{alpha1}{
Upper tail parameter \eqn{\alpha_1} in the generalized logistic item response model.
Default is 0.
}
  \item{alpha2}{
Lower tail parameter \eqn{\alpha_2} parameter in the generalized
logistic item response model. Default is 0.
}
  \item{max.parchange}{
Maximum change in regression parameters
}
  \item{theta.list}{
Grid of person ability where theta is evaluated
}
  \item{maxiter}{
Maximum number of iterations
}
\item{progress}{An optional logical indicating whether computation
progress should be displayed.}
\item{y}{Individual scores}
\item{sig.e}{Standard errors for individual scores}
  \item{object}{
Object of class \code{latent.regression}
}
  \item{\dots}{
Further arguments to be passed
}

}
\details{
In the output \emph{Regression Parameters}
the fraction of missing information (\code{fmi}) is reported
which is the increase of variance in regression
parameter estimates because ability is defined as
a latent variable. The effective sample size \code{pseudoN.latent}
corresponds to a sample size when the ability would be
available with a reliability of one.
}


\value{
A list with following entries
  \item{iterations}{Number of iterations needed}
  \item{maxiter}{Maximal number of iterations}
  \item{max.parchange}{Maximum change in parameter estimates}
  \item{coef}{Coefficients}
  \item{summary.coef}{Summary of regression coefficients}
  \item{sigma}{Estimate of residual standard deviation}
  \item{vcov.simple}{Covariance parameters of estimated parameters
      (simplified version)}
  \item{vcov.latent}{Covariance parameters of estimated parameters
                        which accounts for latent ability}
\item{post}{Individual posterior distribution}
\item{EAP}{Individual EAP estimates}
\item{SE.EAP}{Standard error estimates of EAP}
  \item{explvar}{Explained variance in latent regression}
  \item{totalvar}{Total variance in latent regression}
  \item{rsquared}{Explained variance \eqn{R^2} in latent regression}
}

\references{
Adams, R., & Wu. M. (2007). The mixed-coefficients multinomial
logit model: A generalized form of the Rasch model.
In M. von Davier & C. H. Carstensen (Eds.). \emph{Multivariate and mixture
distribution Rasch models: Extensions and applications} (pp. 57-76).
New York: Springer.
\doi{10.1007/978-0-387-49839-3_4}

Mislevy, R. J. (1991). Randomization-based inference about latent variables
from complex samples. \emph{Psychometrika, 56}(2), 177-196.
\doi{10.1007/BF02294457}

Stukel, T. A. (1988). Generalized logistic models.
\emph{Journal of the American Statistical Association, 83}(402), 426-431.
\doi{10.1080/01621459.1988.10478613}
}

%\author{
%Alexander Robitzsch
%}


\note{
Using the defaults in \code{a}, \code{c}, \code{d},
\code{alpha1} and \code{alpha2} corresponds to the
Rasch model.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also \code{\link{plausible.value.imputation.raschtype}}
for plausible value imputation of generalized logistic
item type models.
}

\examples{
#############################################################################
#  EXAMPLE 1: PISA Reading | Rasch model for dichotomous data
#############################################################################

data(data.pisaRead, package="sirt")
dat <- data.pisaRead$data
items <- grep("R", colnames(dat))
# define matrix of covariates
X <- cbind( 1, dat[, c("female","hisei","migra" ) ] )

#***
# Model 1: Latent regression model in the Rasch model
# estimate Rasch model
mod1 <- sirt::rasch.mml2( dat[,items] )
# latent regression model
lm1 <- sirt::latent.regression.em.raschtype( data=dat[,items ], X=X, b=mod1$item$b )

\dontrun{
#***
# Model 2: Latent regression with generalized link function
# estimate alpha parameters for link function
mod2 <- sirt::rasch.mml2( dat[,items], est.alpha=TRUE)
# use model estimated likelihood for latent regression model
lm2 <- sirt::latent.regression.em.raschtype( f.yi.qk=mod2$f.yi.qk,
            X=X, theta.list=mod2$theta.k)

#***
# Model 3: Latent regression model based on Rasch copula model
testlets <- paste( data.pisaRead$item$testlet)
itemclusters <- match( testlets, unique(testlets) )
# estimate Rasch copula model
mod3 <- sirt::rasch.copula2( dat[,items], itemcluster=itemclusters )
# use model estimated likelihood for latent regression model
lm3 <- sirt::latent.regression.em.raschtype( f.yi.qk=mod3$f.yi.qk,
                X=X, theta.list=mod3$theta.k)

#############################################################################
# EXAMPLE 2: Simulated data according to the Rasch model
#############################################################################

set.seed(899)
I <- 21     # number of items
b <- seq(-2,2, len=I)   # item difficulties
n <- 2000       # number of students

# simulate theta and covariates
theta <- stats::rnorm( n )
x <- .7 * theta + stats::rnorm( n, .5 )
y <- .2 * x+ .3*theta + stats::rnorm( n, .4 )
dfr <- data.frame( theta, 1, x, y )

# simulate Rasch model
dat1 <- sirt::sim.raschtype( theta=theta, b=b )

# estimate latent regression
mod <- sirt::latent.regression.em.raschtype( data=dat1, X=dfr[,-1], b=b )
  ## Regression Parameters
  ##
  ##        est se.simple     se        t p   beta    fmi N.simple pseudoN.latent
  ## X1 -0.2554    0.0208 0.0248 -10.2853 0 0.0000 0.2972     2000       1411.322
  ## x   0.4113    0.0161 0.0193  21.3037 0 0.4956 0.3052     2000       1411.322
  ## y   0.1715    0.0179 0.0213   8.0438 0 0.1860 0.2972     2000       1411.322
  ##
  ## Residual Variance=0.685
  ## Explained Variance=0.3639
  ## Total Variance=1.049
  ##                 R2=0.3469

# compare with linear model (based on true scores)
summary( stats::lm( theta  ~ x + y, data=dfr ) )
  ## Coefficients:
  ##             Estimate Std. Error t value Pr(>|t|)
  ## (Intercept) -0.27821    0.01984  -14.02   <2e-16 ***
  ## x            0.40747    0.01534   26.56   <2e-16 ***
  ## y            0.18189    0.01704   10.67   <2e-16 ***
  ## ---
  ##
  ## Residual standard error: 0.789 on 1997 degrees of freedom
  ## Multiple R-squared: 0.3713,     Adjusted R-squared: 0.3707

#***********
# define guessing parameters (lower asymptotes) and
# upper asymptotes ( 1 minus slipping parameters)
cI <- rep(.2, I)        # all items get a guessing parameter of .2
cI[ c(7,9) ] <- .25     # 7th and 9th get a guessing parameter of .25
dI <- rep( .95, I )    # upper asymptote of .95
dI[ c(7,11) ] <- 1        # 7th and 9th item have an asymptote of 1

# latent regression model
mod1 <- sirt::latent.regression.em.raschtype( data=dat1, X=dfr[,-1],
           b=b, c=cI, d=dI    )
  ## Regression Parameters
  ##
  ##        est se.simple     se        t p   beta    fmi N.simple pseudoN.latent
  ## X1 -0.7929    0.0243 0.0315 -25.1818 0 0.0000 0.4044     2000       1247.306
  ## x   0.5025    0.0188 0.0241  20.8273 0 0.5093 0.3936     2000       1247.306
  ## y   0.2149    0.0209 0.0266   8.0850 0 0.1960 0.3831     2000       1247.306
  ##
  ## Residual Variance=0.9338
  ## Explained Variance=0.5487
  ## Total Variance=1.4825
  ##                 R2=0.3701

#############################################################################
# EXAMPLE 3: Measurement error in dependent variable
#############################################################################

set.seed(8766)
N <- 4000       # number of persons
X <- stats::rnorm(N)           # independent variable
Z <- stats::rnorm(N)           # independent variable
y <- .45 * X + .25 * Z + stats::rnorm(N)   # dependent variable true score
sig.e <- stats::runif( N, .5, .6 )       # measurement error standard deviation
yast <- y + stats::rnorm( N, sd=sig.e ) # dependent variable measured with error

#****
# Model 1: Estimation with latent.regression.em.raschtype using
#          individual likelihood
# define theta grid for evaluation of density
theta.list <- mean(yast) + stats::sd(yast) * seq( - 5, 5, length=21)
# compute individual likelihood
f.yi.qk <- stats::dnorm( outer( yast, theta.list, "-" ) / sig.e )
f.yi.qk <- f.yi.qk / rowSums(f.yi.qk)
# define predictor matrix
X1 <- as.matrix(data.frame( "intercept"=1, "X"=X, "Z"=Z ))

# latent regression model
res <- sirt::latent.regression.em.raschtype( f.yi.qk=f.yi.qk,
                    X=X1, theta.list=theta.list)
  ##   Regression Parameters
  ##
  ##                est se.simple     se       t      p   beta    fmi N.simple pseudoN.latent
  ##   intercept 0.0112    0.0157 0.0180  0.6225 0.5336 0.0000 0.2345     4000       3061.998
  ##   X         0.4275    0.0157 0.0180 23.7926 0.0000 0.3868 0.2350     4000       3061.998
  ##   Z         0.2314    0.0156 0.0178 12.9868 0.0000 0.2111 0.2349     4000       3061.998
  ##
  ##   Residual Variance=0.9877
  ##   Explained Variance=0.2343
  ##   Total Variance=1.222
  ##                   R2=0.1917

#****
# Model 2: Estimation with latent.regression.em.normal
res2 <- sirt::latent.regression.em.normal( y=yast, sig.e=sig.e, X=X1)
  ##   Regression Parameters
  ##
  ##                est se.simple     se       t      p   beta    fmi N.simple pseudoN.latent
  ##   intercept 0.0112    0.0157 0.0180  0.6225 0.5336 0.0000 0.2345     4000       3062.041
  ##   X         0.4275    0.0157 0.0180 23.7927 0.0000 0.3868 0.2350     4000       3062.041
  ##   Z         0.2314    0.0156 0.0178 12.9870 0.0000 0.2111 0.2349     4000       3062.041
  ##
  ##   Residual Variance=0.9877
  ##   Explained Variance=0.2343
  ##   Total Variance=1.222
  ##                   R2=0.1917

  ## -> Results between Model 1 and Model 2 are identical because they use
  ##    the same input.

#***
# Model 3: Regression model based on true scores y
mod3 <- stats::lm( y ~ X + Z )
summary(mod3)
  ##   Coefficients:
  ##               Estimate Std. Error t value Pr(>|t|)
  ##   (Intercept)  0.02364    0.01569   1.506    0.132
  ##   X            0.42401    0.01570  27.016   <2e-16 ***
  ##   Z            0.23804    0.01556  15.294   <2e-16 ***
  ##   Residual standard error: 0.9925 on 3997 degrees of freedom
  ##   Multiple R-squared:  0.1923,    Adjusted R-squared:  0.1919
  ##   F-statistic: 475.9 on 2 and 3997 DF,  p-value: < 2.2e-16

#***
# Model 4: Regression model based on observed scores yast
mod4 <- stats::lm( yast ~ X + Z )
summary(mod4)
  ##   Coefficients:
  ##               Estimate Std. Error t value Pr(>|t|)
  ##   (Intercept)  0.01101    0.01797   0.613     0.54
  ##   X            0.42716    0.01797  23.764   <2e-16 ***
  ##   Z            0.23174    0.01783  13.001   <2e-16 ***
  ##   Residual standard error: 1.137 on 3997 degrees of freedom
  ##   Multiple R-squared:  0.1535,    Adjusted R-squared:  0.1531
  ##   F-statistic: 362.4 on 2 and 3997 DF,  p-value: < 2.2e-16
}
}
%% \keyword{Latent regression model}
%% \keyword{Generalized logistic item response model}
%% \keyword{Plausible values}
%% \keyword{summary}

%% File Name: personfit.stat.Rd
%% File Version: 0.26

\name{personfit.stat}
\alias{personfit.stat}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Person Fit Statistics for the Rasch Model
}
\description{
This function collects some person fit statistics
for the Rasch model (Karabatsos, 2003; Meijer & Sijtsma, 2001).
}
\usage{
personfit.stat(dat, abil, b)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
An \eqn{N \times I} data frame of dichotomous item
responses
}
  \item{abil}{
An ability estimate, e.g. the WLE
}
  \item{b}{
Estimated item difficulty
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A data frame with following columns (see Meijer & Sijtsma 2001
for a review of different person fit statistics):
\item{case}{Case index}
\item{abil}{Ability estimate \code{abil}}
\item{mean}{Person mean of correctly solved items}
\item{caution}{Caution index}
\item{depend}{Dependability index}
\item{ECI1}{\eqn{ECI1}}
\item{ECI2}{\eqn{ECI2}}
\item{ECI3}{\eqn{ECI3}}
\item{ECI4}{\eqn{ECI4}}
\item{ECI5}{\eqn{ECI5}}
\item{ECI6}{\eqn{ECI6}}
\item{l0}{Fit statistic \eqn{l_0}}
\item{lz}{Fit statistic \eqn{l_z}}
\item{outfit}{Person outfit statistic}
\item{infit}{Person infit statistic}
\item{rpbis}{Point biserial correlation of item responses
and item \eqn{p} values}
\item{rpbis.itemdiff}{Point biserial correlation of item responses
and item difficulties \code{b}}
\item{U3}{Fit statistic \eqn{U_3}}
}

\references{
Karabatsos, G. (2003). Comparing the aberrant response detection performance
of thirty-six person-fit statistics. \emph{Applied Measurement in Education,
16}, 277-298.

Meijer, R. R., & Sijtsma, K. (2001). Methodology
review: Evaluating person fit. \emph{Applied Psychological
Measurement, 25}, 107-135.
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See \code{\link{pcm.fit}} for person fit in the partial credit model.

See the \pkg{irtProb} and \pkg{PerFit} packages for person fit statistics
and person response curves and functions included in other packages:
\code{\link[mirt:personfit]{mirt::personfit}},
\code{eRm::personfit} and
\code{ltm::person.fit}.
}

% \code{\link[TAM:tam.fa]{tam.fa}} (\pkg{TAM})

\examples{
#############################################################################
# EXAMPLE 1: Person fit Reading Data
#############################################################################

data(data.read)
dat <- data.read

# estimate Rasch model
mod <- sirt::rasch.mml2( dat )
# WLE
wle1 <- sirt::wle.rasch( dat,b=mod$item$b )$theta
b <- mod$item$b # item difficulty

# evaluate person fit
pf1 <- sirt::personfit.stat( dat=dat, abil=wle1, b=b)

\dontrun{
# dimensional analysis of person fit statistics
x0 <- stats::na.omit(pf1[, -c(1:3) ] )
stats::factanal( x=x0, factors=2, rotation="promax" )
  ## Loadings:
  ##                Factor1 Factor2
  ## caution         0.914
  ## depend          0.293   0.750
  ## ECI1            0.869   0.160
  ## ECI2            0.869   0.162
  ## ECI3            1.011
  ## ECI4            1.159  -0.269
  ## ECI5            1.012
  ## ECI6            0.879   0.130
  ## l0              0.409  -1.255
  ## lz             -0.504  -0.529
  ## outfit          0.297   0.702
  ## infit           0.362   0.695
  ## rpbis          -1.014
  ## rpbis.itemdiff  1.032
  ## U3              0.735   0.309
  ##
  ## Factor Correlations:
  ##         Factor1 Factor2
  ## Factor1   1.000  -0.727
  ## Factor2  -0.727   1.000
  ##
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Person fit}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: lsem.estimate.Rd
%% File Version: 0.778

\name{lsem.estimate}
\alias{lsem.estimate}
\alias{summary.lsem}
\alias{plot.lsem}
\alias{lsem.MGM.stepfunctions}
\alias{lsem_local_weights}
\alias{lsem.bootstrap}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Local Structural Equation Models (LSEM)
}
\description{
Local structural equation models (LSEM) are structural equation models (SEM)
which are evaluated for each value of a pre-defined moderator variable
(Hildebrandt et al., 2009, 2016).
As in nonparametric regression models, observations near a focal point - at
which the model is evaluated - obtain higher weights, far distant observations
obtain lower weights. The LSEM can be specified by making use of \pkg{lavaan} syntax.
It is also possible to specify a discretized version of LSEM in which
values of the moderator are grouped and a multiple group SEM is specified.
The LSEM can be tested by employing a permutation test, see
\code{\link{lsem.permutationTest}}.

The function \code{lsem.MGM.stepfunctions} outputs stepwise functions
for a multiple group model evaluated at a grid of focal points of the
moderator, specified in \code{moderator.grid}.

The argument \code{pseudo_weights} provides an ad hoc solution to estimate
an LSEM for any model which can be fitted in \pkg{lavaan}.

It is also possible to constrain some of the parameters along the values
of the moderator in a joint estimation approach (\code{est_joint=TRUE}). Parameter
names can be specified which are assumed to be invariant (in \code{par_invariant}).
In addition, linear or quadratic constraints can be imposed on
parameters (\code{par_linear} or \code{par_quadratic}).

Statistical inference in case of joint estimation (but also for separate estimation)
can be conducted via bootstrap using the function \code{lsem.bootstrap}.
Bootstrap at the level of a cluster identifier is allowed (argument \code{cluster}).
}
\usage{
lsem.estimate(data, moderator, moderator.grid, lavmodel, type="LSEM", h=1.1, bw=NULL,
    residualize=TRUE, fit_measures=c("rmsea", "cfi", "tli", "gfi", "srmr"),
    standardized=FALSE, standardized_type="std.all", lavaan_fct="sem",
    sufficient_statistics=TRUE, pseudo_weights=0,
    sampling_weights=NULL, loc_linear_smooth=TRUE, est_joint=FALSE, par_invariant=NULL,
    par_linear=NULL, par_quadratic=NULL, partable_joint=NULL, pw_linear=1,
    pw_quadratic=1, pd=TRUE, est_DIF=FALSE, se=NULL, kernel="gaussian",
    eps=1e-08, verbose=TRUE, ...)

\method{summary}{lsem}(object, file=NULL, digits=3, ...)

\method{plot}{lsem}(x, parindex=NULL, ask=TRUE, ci=TRUE, lintrend=TRUE,
       parsummary=TRUE, ylim=NULL, xlab=NULL,  ylab=NULL, main=NULL,
       digits=3, ...)

lsem.MGM.stepfunctions( object, moderator.grid )

# compute local weights
lsem_local_weights(data.mod, moderator.grid, h, sampling_weights=NULL, bw=NULL,
     kernel="gaussian", is_imputed=FALSE, Nimp=0, data=NULL, moderator=NULL)

lsem.bootstrap(object, R=100, verbose=TRUE, cluster=NULL,
     repl_design=NULL, repl_factor=NULL, use_starting_values=TRUE,
     n.core=1, cl.type="PSOCK")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
Data frame or a list of imputed datasets
}
  \item{moderator}{
Variable name of the moderator
}
  \item{moderator.grid}{
Focal points at which the LSEM should be evaluated. If \code{type="MGM"},
breaks are defined in this vector.
}
  \item{lavmodel}{
Specified SEM in \pkg{lavaan}.
}
\item{type}{Type of estimated model. The default is \code{type="LSEM"} which means
    that a local structural equation model is estimated.
    A multiple group model with a discretized moderator as the
    grouping variable can be estimated with \code{type="MGM"}. In this
    case, the breaks must be defined in \code{moderator.grid}.
            }
  \item{h}{
Bandwidth factor
}
\item{bw}{Optional bandwidth parameter if \code{h} should not be used}
\item{residualize}{Logical indicating whether a residualization
    should be applied. }
  \item{fit_measures}{
Vector with names of fit measures following the labels in \pkg{lavaan}
}
\item{standardized}{Optional logical indicating whether
 standardized solution should be included as parameters in
 the output using the
\code{\link[lavaan:standardizedSolution]{lavaan::standardizedSolution}}
function. Standardized parameters are labeled as \code{std__}.
}
\item{standardized_type}{Type of standardization if \code{standardized=TRUE}.
The types are described in
\code{\link[lavaan:standardizedSolution]{lavaan::standardizedSolution}}.
}
\item{lavaan_fct}{String whether
\code{\link[lavaan:lavaan]{lavaan::lavaan}} (\code{lavaan_fct="lavaan"}),
\code{\link[lavaan:sem]{lavaan::sem}} (\code{lavaan_fct="sem"}),
\code{\link[lavaan:cfa]{lavaan::cfa}} (\code{lavaan_fct="cfa"}) or
\code{\link[lavaan:growth]{lavaan::growth}} (\code{lavaan_fct="growth"})
should be used.
}
\item{sufficient_statistics}{Logical whether sufficient statistics of weighted
means and covariances should be used for model fitting. This option
can be set to \code{sufficient_statistics=FALSE}
if the data contain missing values. Note that the option
\code{sufficient_statistics=TRUE} is
only valid for (approximate) missing completely at random (MCAR) data.
The option can only be used for continuous data.}
\item{pseudo_weights}{Integer defining a target sample size. Local weights
are multiplied by a factor which is rounded to integers.
This approach is referred as a pseudo weighting approach.
For example, using \code{pseudo_weights=30000} implies
that the sum of local weights at each focal point is \code{30000}.
}
\item{sampling_weights}{Optional vector of sampling weights}
\item{loc_linear_smooth}{Logical indicating whether local linear
smoothing should be used for computing sufficient statistics for
means and covariances. The default is \code{FALSE}.}
\item{est_joint}{Logical indicating whether LSEM should be estimated in a
joint estimation approach. This options only works wih continuous data and
sufficient statistics.}
\item{par_invariant}{Vector of invariant parameters}
\item{par_linear}{Vector of parameters with linear function}
\item{par_quadratic}{Vector of parameters with quadratic function}
\item{partable_joint}{User-defined parameter table if joint estimation is
used (\code{est_joint=TRUE}).}
\item{pw_linear}{Number of segments if piecewise linear estimation of parameters is used}
\item{pw_quadratic}{Number of segments if piecewise quadratic estimation of parameters
is used}
\item{pd}{Logical indicating whether nearest positive definite covariance matrix
should be computed if sufficient statistics are used}
\item{est_DIF}{Logical indicating whether parameters under differential
item functioning (DIF) should be additionally
computed for invariant item parameters}
\item{se}{Type of standard error used in \code{lavaan::lavaan}. If \code{NULL},
the \pkg{lavaan} default is used.}
\item{kernel}{Type of kernel function. Can be \code{"gaussian"},
\code{"uniform"} or \code{"epanechnikov"}.}
\item{is_imputed}{Logical indicating whether datasets are imputed}
\item{Nimp}{Number of imputed datasets}
\item{eps}{Minimum number for weights}
\item{verbose}{Optional logical printing information about computation progress.}
\item{object}{Object of class \code{lsem}}
\item{file}{A file name in which the summary output will be written.}
\item{digits}{Number of digits.}
\item{x}{Object of class \code{lsem}.}
\item{parindex}{Vector of indices for parameters in plot function.}
\item{ask}{A logical which asks for changing the graphic for each parameter.}
\item{ci}{Logical indicating whether confidence intervals should be plotted.}
\item{lintrend}{Logical indicating whether a linear trend should be plotted.}
\item{parsummary}{Logical indicating whether a parameter summary
  should be displayed.}
\item{ylim}{Plot parameter \code{ylim}. Can be a list, see Examples.}
\item{xlab}{Plot parameter \code{xlab}. Can be a vector.}
\item{ylab}{Plot parameter \code{ylab}. Can be a vector.}
\item{main}{Plot parameter \code{main}. Can be a vector.}
  \item{\dots}{
Further arguments to be passed to \code{\link[lavaan:sem]{lavaan::sem}} or
\code{\link[lavaan:lavaan]{lavaan::lavaan}}.
}
\item{data.mod}{Observed values of the moderator}
\item{R}{Number of bootstrap samples}
\item{cluster}{Optional variable name for bootstrap at the level of a cluster identifier}
\item{repl_design}{Optional matrix containing replication weights for computation of
standard errors. Note that sampling weights have to be already included in
\code{repl_design}.}
\item{repl_factor}{Replication factor in variance formula for statistical
inference, e.g., 0.05 in PISA.}
\item{use_starting_values}{Logical indicating whether starting values should be
used from the original sample}
\item{n.core}{A scalar indicating the number of cores that should be used.}
\item{cl.type}{The cluster type.
Default value is \code{"PSOCK"}. Posix machines (Linux, Mac) generally benefit
from much faster cluster computation if type is set to \code{type="FORK"}.
}
}

%\details{
%EXPLAIN
%}

\value{
List with following entries
\item{parameters}{Data frame with all parameters estimated at focal points of
    moderator. Bias-corrected estimates under boostrap can be found in
    the column \code{est_bc}.}
\item{weights}{Data frame with weights at each focal point}
\item{parameters_summary}{Summary table for estimated parameters}
\item{parametersM}{Estimated parameters in matrix form. Parameters are in
columns and values of the grid of the moderator are in rows.}
\item{bw}{Used bandwidth}
\item{h}{Used bandwidth factor}
\item{N}{Sample size}
\item{moderator.density}{Estimated frequencies and effective sample size for
    moderator at focal points}
\item{moderator.stat}{Descriptive statistics for moderator}
\item{moderator}{Variable name of moderator}
\item{moderator.grid}{Used grid of focal points for moderator}
\item{moderator.grouped}{Data frame with informations about grouping of
moderator if \code{type="MGM"}.}
\item{residualized.intercepts}{Estimated intercept functions used for
residualization.}
\item{lavmodel}{Used lavaan model}
\item{data}{Used data frame, possibly residualized if \code{residualize=TRUE}}
\item{model_parameters}{Model parameters in LSEM}
\item{parameters_boot}{Parameter values in each bootstrap sample
(for \code{lsem.bootstrap})}
\item{fitstats_joint_boot}{Fit statistics in each bootstrap sample
(for \code{lsem.bootstrap})}
\item{dif_effects}{Estimated item parameters under DIF}
}


\references{
Hildebrandt, A., Luedtke, O., Robitzsch, A., Sommer, C., &
Wilhelm, O. (2016). Exploring factor model parameters across continuous variables
with local structural equation models.
\emph{Multivariate Behavioral Research, 51}(2-3), 257-278.
\doi{10.1080/00273171.2016.1142856}

Hildebrandt, A., Wilhelm, O., & Robitzsch, A. (2009). Complementary and
competing factor analytic approaches for the investigation of measurement invariance.
\emph{Review of Psychology, 16}, 87-102.
}


\author{
Alexander Robitzsch, Oliver Luedtke, Andrea Hildebrandt
}

% \note{
% }


%% Note that the specified model in \code{lavaan} syntax should be explicitly saved as
%% the object \code{lavmodel} in the global envirnoment as it is done in the examples.


%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See \code{\link{lsem.permutationTest}} for conducting a permutation test
and \code{\link{lsem.test}} for applying a Wald test to a bootstrapped LSEM model.
}

\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: data.lsem01 | Age differentiation
#############################################################################

data(data.lsem01, package="sirt")
dat <- data.lsem01

# specify lavaan model
lavmodel <- "
        F=~ v1+v2+v3+v4+v5
        F ~~ 1*F"

# define grid of moderator variable age
moderator.grid <- seq(4,23,1)

#********************************
#*** Model 1: estimate LSEM with bandwidth 2
mod1 <- sirt::lsem.estimate( dat, moderator="age", moderator.grid=moderator.grid,
               lavmodel=lavmodel, h=2, std.lv=TRUE)
summary(mod1)
plot(mod1, parindex=1:5)

# perform permutation test for Model 1
pmod1 <- sirt::lsem.permutationTest( mod1, B=10 )
          # only for illustrative purposes the number of permutations B is set
          # to a low number of 10
summary(pmod1)
plot(pmod1, type="global")

#* perform permutation test with parallel computation
pmod1a <- sirt::lsem.permutationTest( mod1, B=10, n.core=3 )
summary(pmod1a)

#** estimate Model 1 based on pseudo weights
mod1b <- sirt::lsem.estimate( dat, moderator="age", moderator.grid=moderator.grid,
               lavmodel=lavmodel, h=2, std.lv=TRUE, pseudo_weights=50 )
summary(mod1b)

#** estimation with sampling weights

# generate random sampling weights
set.seed(987)
weights <- stats::runif(nrow(dat), min=.4, max=3 )
mod1c <- sirt::lsem.estimate( dat, moderator="age", moderator.grid=moderator.grid,
               lavmodel=lavmodel, h=2, sampling_weights=weights)
summary(mod1c)

#********************************
#*** Model 2: estimate multiple group model with 4 age groups

# define breaks for age groups
moderator.grid <- seq( 3.5, 23.5, len=5) # 4 groups
# estimate model
mod2 <- sirt::lsem.estimate( dat, moderator="age", moderator.grid=moderator.grid,
           lavmodel=lavmodel, type="MGM", std.lv=TRUE)
summary(mod2)

# output step functions
smod2 <- sirt::lsem.MGM.stepfunctions( object=mod2, moderator.grid=seq(4,23,1) )
str(smod2)

#********************************
#*** Model 3: define standardized loadings as derived variables

# specify lavaan model
lavmodel <- "
        F=~ a1*v1+a2*v2+a3*v3+a4*v4
        v1 ~~ s1*v1
        v2 ~~ s2*v2
        v3 ~~ s3*v3
        v4 ~~ s4*v4
        F ~~ 1*F
        # standardized loadings
        l1 :=a1 / sqrt(a1^2 + s1 )
        l2 :=a2 / sqrt(a2^2 + s2 )
        l3 :=a3 / sqrt(a3^2 + s3 )
        l4 :=a4 / sqrt(a4^2 + s4 )
        "
# estimate model
mod3 <- sirt::lsem.estimate( dat, moderator="age", moderator.grid=moderator.grid,
               lavmodel=lavmodel, h=2, std.lv=TRUE)
summary(mod3)
plot(mod3)

#********************************
#*** Model 4: estimate LSEM and automatically include standardized solutions

lavmodel <- "
        F=~ 1*v1+v2+v3+v4
        F ~~ F"
mod4 <- sirt::lsem.estimate( dat, moderator="age", moderator.grid=moderator.grid,
               lavmodel=lavmodel, h=2, standardized=TRUE)
summary(mod4)
# permutation test (use only few permutations for testing purposes)
pmod1 <- sirt::lsem.permutationTest( mod4, B=3 )

#**** compute LSEM local weights
wgt <- sirt::lsem_local_weights(data.mod=dat$age, moderator.grid=moderator.grid,
             h=2)$weights
print(str(weights))

#********************************
#*** Model 5: invariance parameter constraints and other constraints

lavmodel <- "
        F=~ 1*v1+v2+v3+v4
        F ~~ F"
moderator.grid <- seq(4,23,4)

#- estimate model without constraints
mod5a <- sirt::lsem.estimate( dat, moderator="age", moderator.grid=moderator.grid,
               lavmodel=lavmodel, h=2, standardized=TRUE)
summary(mod5a)
# extract parameter names
mod5a$model_parameters

#- invariance constraints on residual variances
par_invariant <- c("F=~v2","v2~~v2")
mod5b <- sirt::lsem.estimate( dat, moderator="age", moderator.grid=moderator.grid,
               lavmodel=lavmodel, h=2, standardized=TRUE, par_invariant=par_invariant)
summary(mod5b)

#- bootstrap for statistical inference
bmod5b <- sirt::lsem.bootstrap(mod5b, R=100)
# inspect parameter values and standard errors
bmod5b$parameters

#- bootstrap using parallel computing (i.e., multiple cores)
bmod5ba <- sirt::lsem.bootstrap(mod5b, R=100, n.core=3)

#- user-defined replication design
R <- 100    # bootstrap samples
N <- nrow(dat)
repl_design <- matrix(0, nrow=N, ncol=R)
for (rr in 1:R){
    indices <- sort( sample(1:N, replace=TRUE) )
    repl_design[,rr] <- sapply(1:N, FUN=function(ii){ sum(indices==ii) } )
}
head(repl_design)
bmod5b1 <- sirt::lsem.bootstrap(mod5a, repl_design=repl_design, repl_factor=1/R)

#- compare model mod5b with joint estimation without constraints
mod5c <- sirt::lsem.estimate( dat, moderator="age", moderator.grid=moderator.grid,
               lavmodel=lavmodel, h=2, standardized=TRUE, est_joint=TRUE)
summary(mod5c)

#- linear and quadratic functions
par_invariant <- c("F=~v1","v2~~v2")
par_linear <- c("v1~~v1")
par_quadratic <- c("v4~~v4")

mod5d <- sirt::lsem.estimate( dat1, moderator="age", moderator.grid=moderator.grid,
            lavmodel=lavmodel, h=2, par_invariant=par_invariant, par_linear=par_linear,
            par_quadratic=par_quadratic)
summary(mod5d)

#- user-defined constraints: step functions for parameters

# inspect parameter table (from lavaan) of fitted model
pj <- mod5d$partable_joint
#* modify parameter table for user-defined constraints
# define step function for F=~v1 which is constant on intervals 1:4 and 5:7
pj2 <- pj[ pj$con==1, ]
pj2[ c(5,6), "lhs" ] <- "p1g5"
pj2 <- pj2[ -4, ]
partable_joint <- rbind(pj1, pj2)
# estimate model with constraints
mod5e <- lsem::lsem.estimate( dat1, moderator="age", moderator.grid=moderator.grid,
             lavmodel=lavmodel, h=2, std.lv=TRUE, estimator="ML",
             partable_joint=partable_joint)
summary(mod5e)

#############################################################################
# EXAMPLE 2: data.lsem01 | FIML with missing data
#############################################################################

data(data.lsem01)
dat <- data.lsem01
# induce artifical missing values
set.seed(98)
dat[ stats::runif(nrow(dat)) < .5, c("v1")] <- NA
dat[ stats::runif(nrow(dat)) < .25, c("v2")] <- NA

# specify lavaan model
lavmodel1 <- "
        F=~ v1+v2+v3+v4+v5
        F ~~ 1*F"

# define grid of moderator variable age
moderator.grid <- seq(4,23,2)

#*** estimate LSEM with FIML
mod1 <- sirt::lsem.estimate( dat, moderator="age", moderator.grid=moderator.grid,
                lavmodel=lavmodel1, h=2, std.lv=TRUE, estimator="ML", missing="fiml")
summary(mod1)

#############################################################################
# EXAMPLE 3: data.lsem01 | WLSMV estimation
#############################################################################

data(data.lsem01)
dat <- data.lsem01

# create artificial dichotomous data
for (vv in 2:6){
dat[,vv] <- 1*(dat[,vv] > mean(dat[,vv]))
}

# specify lavaan model
lavmodel1 <- "
        F=~ v1+v2+v3+v4+v5
        F ~~ 1*F
        v1 | t1
        v2 | t1
        v3 | t1
        v4 | t1
        v5 | t1
        "

# define grid of moderator variable age
moderator.grid <- seq(4,23,2)

#*** local WLSMV estimation
mod1 <- sirt::lsem.estimate( dat, moderator="age", moderator.grid=moderator.grid,
          lavmodel=lavmodel1, h=2, std.lv=TRUE, estimator="DWLS", ordered=paste0("v",1:5),
          residualize=FALSE, pseudo_weights=10000, parameterization="THETA" )
summary(mod1)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{LSEM}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: linking.haberman.Rd
%% File Version: 0.769

\name{linking.haberman}
\alias{linking.haberman}
\alias{summary.linking.haberman}
\alias{linking.haberman.lq}
\alias{summary.linking.haberman.lq}
\alias{linking_haberman_itempars_prepare}
\alias{linking_haberman_itempars_convert}
\alias{L0_polish}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Linking in the 2PL/Generalized Partial Credit Model
}
\description{
This function does the linking of several studies which are calibrated
using the 2PL or the generalized item response model according to
Haberman (2009). This method is a generalization of log-mean-mean
linking from one study to several studies. The default \code{a_log=TRUE}
logarithmizes item slopes for linking while otherwise an additive regression
model is assumed for the original item loadings (see Details; Battauz, 2017)
}
\usage{
linking.haberman(itempars, personpars, estimation="OLS", a_trim=Inf, b_trim=Inf,
    lts_prop=.5, a_log=TRUE, conv=1e-05, maxiter=1000, progress=TRUE,
    adjust_main_effects=TRUE, vcov=TRUE)

\method{summary}{linking.haberman}(object, digits=3, file=NULL, ...)

linking.haberman.lq(itempars, pow=2, eps=1e-3, a_log=TRUE, use_nu=FALSE,
      est_pow=FALSE, lower_pow=.1, upper_pow=3, method="joint",
      le=FALSE, vcov_list=NULL)

\method{summary}{linking.haberman.lq}(object, digits=3, file=NULL, ...)

## prepare 'itempars' argument for linking.haberman()
linking_haberman_itempars_prepare(b, a=NULL, wgt=NULL)

## conversion of different parameterizations of item parameters
linking_haberman_itempars_convert(itempars=NULL, lambda=NULL, nu=NULL, a=NULL, b=NULL)

## L0 polish precedure minimizing number of interactions in two-way table
L0_polish(x, tol, conv=0.01, maxiter=30, type=1, verbose=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{itempars}{
A data frame with four or five columns. The first four columns contain
in the order: study name, item name, \eqn{a} parameter, \eqn{b} parameter.
The fifth column is an optional weight for every item and every study.
}
\item{personpars}{A list with vectors (e.g. EAPs or WLEs) or data frames
    (e.g. plausible values) containing person parameters which
    should be transformed.
If a data frame in each list entry has \code{se} or \code{SE}
(standard error) in a column name, then the corresponding
column is only multiplied by \eqn{A_t}.
If a column is labeled as \code{pid} (person ID),
then it is left untransformed.
}
\item{estimation}{Estimation method. Can be \code{"OLS"} (ordinary least squares),
\code{"BSQ"} (bisquare weighted regression), \code{"HUB"} (regression using Huber
weights), \code{"MED"} (median regression), \code{"LTS"} (trimmed least squares),
\code{"L1"} (median polish), \code{"L0"} (minimizing number of interactions)
}
\item{a_trim}{Trimming parameter for item slopes \eqn{a_{it}} in
bisquare regression (see Details).}
\item{b_trim}{Trimming parameter for item slopes \eqn{b_{it}} in
bisquare regression (see Details).}
\item{lts_prop}{Proportion of retained observations in \code{"LTS"}
regression estimation}
\item{a_log}{Logical indicating whether item slopes should be logarithmized
for linking.}
\item{conv}{
Convergence criterion.
}
  \item{maxiter}{
Maximum number of iterations.
}
  \item{progress}{
An optional logical indicating whether computational progress
should be displayed.
}
\item{adjust_main_effects}{Logical indicating whether all elements in the vector
of main effects should be simultaneously adjusted}
\item{vcov}{Optional indicating whether covariance matrix for linking errors
should be computed}
\item{pow}{Power \eqn{q}}
\item{eps}{Epsilon value used in differentiable approximating function}
\item{use_nu}{Logical indicating whether item intercepts instead of
item difficulties are used in linking}
\item{est_pow}{Logical indicating whether power values should be estimated}
\item{lower_pow}{Lower bound for estimated power}
\item{upper_pow}{Upper bound for estimated power}
\item{method}{Estimation method type: \code{"joint"} for joint estimation of
distribution and item parameters, while \code{"pw1"} and \code{"pw2"} indicate
pairwise estimation using no weights or weights for item frequencies}
\item{le}{Logical indicating whether linking and standard errors should be
computed}
\item{vcov_list}{List of variance matrices of item parameters}
\item{lambda}{Matrix containing item loadings}
\item{nu}{Matrix containing item intercepts}
\item{object}{Object of class \code{linking.haberman}.}
\item{digits}{Number of digits after decimals for rounding in \code{summary}.}
\item{file}{Optional file name if \code{summary} should be sunk into a file.}
\item{\dots}{Further arguments to be passed}
\item{b}{Matrix of item intercepts (items \eqn{times} studies)}
\item{a}{Matrix of item slopes}
\item{wgt}{Matrix of weights}
\item{x}{Matrix}
\item{tol}{Tolerance value}
\item{type}{Can be \code{1} (using Tukey's median polish) or
\code{2} (alternating median regression).}
\item{verbose}{Logical indicating whether iteration progress should be displayed}
}

% L0_polish(x, tol, eps=0.01, maxiter=30, type=1, verbose=TRUE)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DETAILS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



\details{
For \eqn{t=1,\ldots,T} studies, item difficulties \eqn{b_{it}} and
item slopes \eqn{a_{it}} are available. For dichotomous responses, these
parameters are defined by the 2PL response equation
\deqn{ logit P(X_{pi}=1| \theta_p )=a_i ( \theta_p - b_i ) }
while for polytomous responses the generalized partial credit model holds
\deqn{ log \frac{P(X_{pi}=k| \theta_p )}{P(X_{pi}=k-1| \theta_p )}
=a_i ( \theta_p - b_i + d_{ik} ) }

The parameters \eqn{ \{ a_{it}, b_{it} \}} of all items and studies are
linearly transformed using equations \eqn{a_{it} \approx a_i / A_t}
(if \code{a_log=TRUE}) or \eqn{a_{it} \approx a_i + A_t}
(if \code{a_log=FALSE}) and
\eqn{b_{it} \cdot A_t \approx B_t + b_i}. For identification reasons,
we define \eqn{A_1=1} and \eqn{B_1}=0.

The optimization function (which is a least squares criterion;
see Haberman, 2009) seeks the transformation parameters \eqn{A_t} and
\eqn{B_t} with an alternating least squares
method (\code{estimation="OLS"}). Note that every item \eqn{i} and every study \eqn{t} can
be weighted (specified in the fifth column of \code{itempars}).
Alternatively, a robust regression method based on bisquare weighting (Fox, 2015)
can be employed for linking using the argument \code{estimation="BSQ"}.
For example, in the case of item loadings, bisquare weighting is applied to
residuals \eqn{e_{it}=a_{it} - a_i - A_t } (where logarithmized or non-logarithmized
item loadings are employed) forming weights
\eqn{w_{it}=[ 1 - ( e_{it} / k )^2 ]^2} for \eqn{e_{it} <k} and 0 for \eqn{e_{it} \ge k}
where \eqn{k} is the trimming constant which can be estimated or fixed
during estimation using arguments \code{a_trim} or \code{b_trim}. Items in studies with
large residuals
(i.e., presence differential item functioning) are effectively set to zero in the
linking procedure. Alternatively, Huber weights (\code{estimation="HUB"}) downweight
large residuals by applying \eqn{w_{it}=k / | e_{it} |} for residuals
\eqn{|e_{it}|>k}.  The method \code{estimation="LTS"} employs trimmed least squares
where the proportion
of data retained is specified in \code{lts_prop} with default set to .50.

The method \code{estimation="MED"} estimates item parameters and linking constants
based on alternating median regression. A similar approach is the median polish
procedure of Tukey (Tukey, 1977, p. 362ff.; Maronna, Martin & Yohai, 2006, p. 104;
see also \code{\link[stats:medpolish]{stats::medpolish}}) implemented in
\code{estimation="L1"} which aims to minimize \eqn{\sum_{i,t} | e_{it} |}.
For a pre-specified tolerance value \eqn{t} (in \code{a_trim} or \code{b_trim}),
the approach \code{estimation="L0"} minimizes the number of interactions
(i.e., DIF effects) in the \eqn{e_{it}} effects. In more detail, it minimizes
 \eqn{\sum_{i,t} \# \{ | e_{it} | > t \} } which is computationally conducted
 by repeatedly applying the median polish procedure in which one cell is
omitted (Davies, 2012; Terbeck & Davies, 1998).

Effect sizes of invariance are calculated as R-squared measures
of explained item slopes and intercepts after linking
in comparison to item parameters across groups
(Asparouhov & Muthen, 2014).

The function \eqn{linking.haberman.lq} uses the loss function \eqn{\rho(x)=|x|^q}.
The originally proposed Haberman linking can be obtained with \code{pow=2} (\eqn{q=2}).
The powers can also be estimated (argument \code{est_pow=TRUE}).
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VALUES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\value{
A list with following entries
  \item{transf.pars}{Data frame with transformation parameters
\eqn{A_t} and \eqn{B_t}
  }
  \item{transf.personpars}{Data frame with linear transformation functions
  for person parameters}
  \item{joint.itempars}{Estimated joint item parameters \eqn{a_i} and \eqn{b_i}
      }
  \item{a.trans}{Transformed \eqn{a_{it}} parameters}
  \item{b.trans}{Transformed \eqn{b_{it}} parameters}
  \item{a.orig}{Original \eqn{a_{it}} parameters}
  \item{b.orig}{Original \eqn{b_{it}} parameters}
  \item{a.resid}{Residual \eqn{a_{it}} parameters (DIF parameters)}
  \item{b.resid}{Residual \eqn{b_{it}} parameters (DIF parameters)}
  \item{personpars}{Transformed person parameters}
  \item{es.invariance}{Effect size measures of invariance,
        separately for item slopes and intercepts.
        In the rows, \eqn{R^2} and \eqn{\sqrt{1-R^2}} are reported.}
  \item{es.robust}{Effect size measures of invariance based on
  robust estimation (if used).}
  \item{selitems}{Indices of items which are present in more than one
  study.}
  \item{vcov}{List containing standard, linking and total errors}
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% REFERENCES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\references{
Asparouhov, T., & Muthen, B. (2014). Multiple-group factor analysis alignment.
\emph{Structural Equation Modeling, 21}(4), 1-14.
\doi{10.1080/10705511.2014.919210}

Battauz, M. (2017). Multiple equating of separate IRT calibrations.
\emph{Psychometrika, 82}(3), 610-636.
\doi{10.1007/s11336-016-9517-x}

Davies, P. L. (2012). Interactions in the analysis of variance.
\emph{Journal of the American Statistical Association, 107}(500), 1502-1509.
\doi{10.1080/01621459.2012.726895}

Fox, J. (2015). \emph{Applied regression analysis and generalized linear models}.
Thousand Oaks: Sage.

Haberman, S. J. (2009). \emph{Linking parameter estimates derived
from an item response model through separate calibrations}.
ETS Research Report ETS RR-09-40. Princeton, ETS.
\doi{10.1002/j.2333-8504.2009.tb02197.x}

Kolen, M. J., & Brennan, R. L. (2014). \emph{Test equating, scaling, and linking:
Methods and practices}. New York: Springer.
\doi{10.1007/978-1-4939-0317-7}

Magis, D., & De Boeck, P. (2012). A robust outlier approach to prevent type I error
inflation in differential item functioning.
\emph{Educational and Psychological Measurement, 72}(2), 291-311.
\doi{10.1177/0013164411416975}

Maronna, R. A., Martin, R. D., & Yohai, V. J. (2006). \emph{Robust statistics}.
West Sussex: Wiley. \doi{10.1002/0470010940}

Terbeck, W., & Davies, P. L. (1998). Interactions and outliers in the two-way
analysis of variance. \emph{Annals of Statistics, 26}(4), 1279-1305.
doi: 10.1214/aos/1024691243

Tukey, J. W. (1977). \emph{Exploratory data analysis}. Addison-Wesley.

Weeks, J. P. (2010). \pkg{plink}: An \R package for linking mixed-format tests
using IRT-based methods. \code{Journal of Statistical Software, 35}(12), 1-33.
\doi{10.18637/jss.v035.i12}
}


%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See the \pkg{plink} package (Weeks, 2010) for a diversity of linking methods.

Mean-mean linking, Stocking-Lord and Haebara linking (see Kolen & Brennan, 2014,
for an overview) in the generalized logistic item response model can be conducted with
\code{\link{equating.rasch}}. See also \code{\link[TAM:tam.linking]{TAM::tam.linking}}
in the \pkg{TAM} package. Haebara linking and a robustified version of it can be
found in \code{\link{linking.haebara}}.

The invariance alignment method employs an optimization function based on
pairwise loss functions of item parameters (Asparouhov & Muthen, 2014),
see \code{\link{invariance.alignment}}.
}




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EXAMPLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\examples{
#############################################################################
# EXAMPLE 1: Item parameters data.pars1.rasch and data.pars1.2pl
#############################################################################

# Model 1: Linking three studies calibrated by the Rasch model
data(data.pars1.rasch)
mod1 <- sirt::linking.haberman( itempars=data.pars1.rasch )
summary(mod1)

# Model 1b: Linking these studies but weigh these studies by
#     proportion weights 3 : 0.5 : 1 (see below).
#     All weights are the same for each item but they could also
#     be item specific.
itempars <- data.pars1.rasch
itempars$wgt <- 1
itempars[ itempars$study=="study1","wgt"] <- 3
itempars[ itempars$study=="study2","wgt"] <- .5
mod1b <- sirt::linking.haberman( itempars=itempars )
summary(mod1b)

# Model 2: Linking three studies calibrated by the 2PL model
data(data.pars1.2pl)
mod2 <- sirt::linking.haberman( itempars=data.pars1.2pl )
summary(mod2)

# additive model instead of logarithmic model for item slopes
mod2b <- sirt::linking.haberman( itempars=data.pars1.2pl, a_log=FALSE )
summary(mod2b)

\dontrun{
#############################################################################
# EXAMPLE 2: Linking longitudinal data
#############################################################################
data(data.long)

#******
# Model 1: Scaling with the 1PL model

# scaling at T1
dat1 <- data.long[, grep("T1", colnames(data.long) ) ]
resT1 <- sirt::rasch.mml2( dat1 )
itempartable1 <- data.frame( "study"="T1", resT1$item[, c("item", "a", "b" ) ] )
# scaling at T2
dat2 <- data.long[, grep("T2", colnames(data.long) ) ]
resT2 <- sirt::rasch.mml2( dat2 )
summary(resT2)
itempartable2 <- data.frame( "study"="T2", resT2$item[, c("item", "a", "b" ) ] )
itempartable <- rbind( itempartable1, itempartable2 )
itempartable[,2] <- substring( itempartable[,2], 1, 2 )
# estimate linking parameters
mod1 <- sirt::linking.haberman( itempars=itempartable )

#******
# Model 2: Scaling with the 2PL model

# scaling at T1
dat1 <- data.long[, grep("T1", colnames(data.long) ) ]
resT1 <- sirt::rasch.mml2( dat1, est.a=1:6)
itempartable1 <- data.frame( "study"="T1", resT1$item[, c("item", "a", "b" ) ] )

# scaling at T2
dat2 <- data.long[, grep("T2", colnames(data.long) ) ]
resT2 <- sirt::rasch.mml2( dat2, est.a=1:6)
summary(resT2)
itempartable2 <- data.frame( "study"="T2", resT2$item[, c("item", "a", "b" ) ] )
itempartable <- rbind( itempartable1, itempartable2 )
itempartable[,2] <- substring( itempartable[,2], 1, 2 )
# estimate linking parameters
mod2 <- sirt::linking.haberman( itempars=itempartable )

#############################################################################
# EXAMPLE 3: 2 Studies - 1PL and 2PL linking
#############################################################################
set.seed(789)
I <- 20        # number of items
N <- 2000       # number of persons
# define item parameters
b <- seq( -1.5, 1.5, length=I )
# simulate data
dat1 <- sirt::sim.raschtype( stats::rnorm( N, mean=0,sd=1 ), b=b )
dat2 <- sirt::sim.raschtype( stats::rnorm( N, mean=0.5,sd=1.50 ), b=b )

#*** Model 1: 1PL
# 1PL Study 1
mod1 <- sirt::rasch.mml2( dat1, est.a=rep(1,I) )
summary(mod1)
# 1PL Study 2
mod2 <- sirt::rasch.mml2( dat2, est.a=rep(1,I) )
summary(mod2)

# collect item parameters
dfr1 <- data.frame( "study1", mod1$item$item, mod1$item$a, mod1$item$b )
dfr2 <- data.frame( "study2", mod2$item$item, mod2$item$a, mod2$item$b )
colnames(dfr2) <- colnames(dfr1) <- c("study", "item", "a", "b" )
itempars <- rbind( dfr1, dfr2 )

# Haberman linking
linkhab1 <- sirt::linking.haberman(itempars=itempars)
  ## Transformation parameters (Haberman linking)
  ##    study    At     Bt
  ## 1 study1 1.000  0.000
  ## 2 study2 1.465 -0.512
  ##
  ## Linear transformation for item parameters a and b
  ##    study   A_a   A_b    B_b
  ## 1 study1 1.000 1.000  0.000
  ## 2 study2 0.682 1.465 -0.512
  ##
  ## Linear transformation for person parameters theta
  ##    study A_theta B_theta
  ## 1 study1   1.000   0.000
  ## 2 study2   1.465   0.512
  ##
  ## R-Squared Measures of Invariance
  ##        slopes intercepts
  ## R2          1     0.9979
  ## sqrtU2      0     0.0456

#*** Model 2: 2PL
# 2PL Study 1
mod1 <- sirt::rasch.mml2( dat1, est.a=1:I )
summary(mod1)
# 2PL Study 2
mod2 <- sirt::rasch.mml2( dat2, est.a=1:I )
summary(mod2)

# collect item parameters
dfr1 <- data.frame( "study1", mod1$item$item, mod1$item$a, mod1$item$b )
dfr2 <- data.frame( "study2", mod2$item$item, mod2$item$a, mod2$item$b )
colnames(dfr2) <- colnames(dfr1) <- c("study", "item", "a", "b" )
itempars <- rbind( dfr1, dfr2 )

# Haberman linking
linkhab2 <- sirt::linking.haberman(itempars=itempars)
  ## Transformation parameters (Haberman linking)
  ##    study    At     Bt
  ## 1 study1 1.000  0.000
  ## 2 study2 1.468 -0.515
  ##
  ## Linear transformation for item parameters a and b
  ##    study   A_a   A_b    B_b
  ## 1 study1 1.000 1.000  0.000
  ## 2 study2 0.681 1.468 -0.515
  ##
  ## Linear transformation for person parameters theta
  ##    study A_theta B_theta
  ## 1 study1   1.000   0.000
  ## 2 study2   1.468   0.515
  ##
  ## R-Squared Measures of Invariance
  ##        slopes intercepts
  ## R2     0.9984     0.9980
  ## sqrtU2 0.0397     0.0443

#############################################################################
# EXAMPLE 4: 3 Studies - 1PL and 2PL linking
#############################################################################
set.seed(789)
I <- 20         # number of items
N <- 1500       # number of persons
# define item parameters
b <- seq( -1.5, 1.5, length=I )
# simulate data
dat1 <- sirt::sim.raschtype( stats::rnorm( N, mean=0, sd=1), b=b )
dat2 <- sirt::sim.raschtype( stats::rnorm( N, mean=0.5, sd=1.50), b=b )
dat3 <- sirt::sim.raschtype( stats::rnorm( N, mean=-0.2, sd=0.8), b=b )
# set some items to non-administered
dat3 <- dat3[, -c(1,4) ]
dat2 <- dat2[, -c(1,2,3) ]

#*** Model 1: 1PL in sirt
# 1PL Study 1
mod1 <- sirt::rasch.mml2( dat1, est.a=rep(1,ncol(dat1)) )
summary(mod1)
# 1PL Study 2
mod2 <- sirt::rasch.mml2( dat2, est.a=rep(1,ncol(dat2)) )
summary(mod2)
# 1PL Study 3
mod3 <- sirt::rasch.mml2( dat3, est.a=rep(1,ncol(dat3)) )
summary(mod3)

# collect item parameters
dfr1 <- data.frame( "study1", mod1$item$item, mod1$item$a, mod1$item$b )
dfr2 <- data.frame( "study2", mod2$item$item, mod2$item$a, mod2$item$b )
dfr3 <- data.frame( "study3", mod3$item$item, mod3$item$a, mod3$item$b )
colnames(dfr3) <- colnames(dfr2) <- colnames(dfr1) <- c("study", "item", "a", "b" )
itempars <- rbind( dfr1, dfr2, dfr3 )

# use person parameters
personpars <- list( mod1$person[, c("EAP","SE.EAP") ], mod2$person[, c("EAP","SE.EAP") ],
    mod3$person[, c("EAP","SE.EAP") ] )

# Haberman linking
linkhab1 <- sirt::linking.haberman(itempars=itempars, personpars=personpars)
# compare item parameters
round( cbind( linkhab1$joint.itempars[,-1], linkhab1$b.trans )[1:5,], 3 )
  ##            aj     bj study1 study2 study3
  ##   I0001 0.998 -1.427 -1.427     NA     NA
  ##   I0002 0.998 -1.290 -1.324     NA -1.256
  ##   I0003 0.998 -1.140 -1.068     NA -1.212
  ##   I0004 0.998 -0.986 -1.003 -0.969     NA
  ##   I0005 0.998 -0.869 -0.809 -0.872 -0.926

# summary of person parameters of second study
round( psych::describe( linkhab1$personpars[[2]] ), 2 )
  ##   var    n mean   sd median trimmed  mad   min  max range  skew kurtosis
  ## EAP      1 1500 0.45 1.36   0.41    0.47 1.52 -2.61 3.25  5.86 -0.08    -0.62
  ## SE.EAP   2 1500 0.57 0.09   0.53    0.56 0.04  0.49 0.84  0.35  1.47     1.56
  ##          se
  ## EAP    0.04
  ## SE.EAP 0.00

#*** Model 2: 2PL in TAM
library(TAM)
# 2PL Study 1
mod1 <- TAM::tam.mml.2pl( resp=dat1, irtmodel="2PL" )
pvmod1 <- TAM::tam.pv(mod1, ntheta=300, normal.approx=TRUE) # draw plausible values
summary(mod1)
# 2PL Study 2
mod2 <- TAM::tam.mml.2pl( resp=dat2, irtmodel="2PL" )
pvmod2 <- TAM::tam.pv(mod2, ntheta=300, normal.approx=TRUE)
summary(mod2)
# 2PL Study 3
mod3 <- TAM::tam.mml.2pl( resp=dat3, irtmodel="2PL" )
pvmod3 <- TAM::tam.pv(mod3, ntheta=300, normal.approx=TRUE)
summary(mod3)

# collect item parameters
#!!  Note that in TAM the parametrization is a*theta - b while linking.haberman
#!!  needs the parametrization a*(theta-b)
dfr1 <- data.frame( "study1", mod1$item$item, mod1$B[,2,1], mod1$xsi$xsi / mod1$B[,2,1] )
dfr2 <- data.frame( "study2", mod2$item$item, mod2$B[,2,1], mod2$xsi$xsi / mod2$B[,2,1] )
dfr3 <- data.frame( "study3", mod3$item$item, mod3$B[,2,1], mod3$xsi$xsi / mod3$B[,2,1] )
colnames(dfr3) <- colnames(dfr2) <- colnames(dfr1) <- c("study", "item", "a", "b" )
itempars <- rbind( dfr1, dfr2, dfr3 )

# define list containing person parameters
personpars <- list(  pvmod1$pv[,-1], pvmod2$pv[,-1], pvmod3$pv[,-1] )

# Haberman linking
linkhab2 <- sirt::linking.haberman(itempars=itempars,personpars=personpars)
  ##   Linear transformation for person parameters theta
  ##      study A_theta B_theta
  ##   1 study1   1.000   0.000
  ##   2 study2   1.485   0.465
  ##   3 study3   0.786  -0.192

# extract transformed person parameters
personpars.trans <- linkhab2$personpars

#############################################################################
# EXAMPLE 5: Linking with simulated item parameters containing outliers
#############################################################################

# simulate some parameters
I <- 38
set.seed(18785)
b <- stats::rnorm( I, mean=.3, sd=1.4 )
# simulate DIF effects plus some outliers
bdif <- stats::rnorm(I,mean=.4,sd=.09)+( stats::runif(I)>.9 )* rep( 1*c(-1,1)+.4, each=I/2 )
# create item parameter table
itempars <- data.frame( "study"=paste0("study",rep(1:2, each=I)),
                "item"=paste0( "I", 100 + rep(1:I,2) ), "a"=1,
                 "b"=c( b, b + bdif  )  )

#*** Model 1: Haberman linking with least squares regression
mod1 <- sirt::linking.haberman( itempars=itempars )
summary(mod1)

#*** Model 2: Haberman linking with robust bisquare regression with fixed trimming value
mod2 <- sirt::linking.haberman( itempars=itempars, estimation="BSQ", b_trim=.4)
summary(mod2)

#*** Model 2: Haberman linking with robust bisquare regression with estimated trimming value
mod3 <- sirt::linking.haberman( itempars=itempars, estimation="BSQ")
summary(mod3)

## see also Example 3 of ?sirt::robust.linking

#############################################################################
# EXAMPLE 6: Toy example of Magis and De Boeck (2012)
#############################################################################

# define item parameters from Magis & De Boeck (20212, p. 293)
b1 <- c(1,1,1,1)
b2 <- c(1,1,1,2)
itempars <- data.frame(study=rep(1:2, each=4), item=rep(1:4,2), a=1, b=c(b1,b2) )

#- Least squares regression
mod1 <- sirt::linking.haberman( itempars=itempars, estimation="OLS")
summary(mod1)

#- Bisquare regression with estimated and fixed trimming factors
mod2 <- sirt::linking.haberman( itempars=itempars, estimation="BSQ")
mod2a <- sirt::linking.haberman( itempars=itempars, estimation="BSQ", b_trim=.4)
mod2b <- sirt::linking.haberman( itempars=itempars, estimation="BSQ", b_trim=1.2)
summary(mod2)
summary(mod2a)
summary(mod2b)

#- Least squares trimmed regression
mod3 <- sirt::linking.haberman( itempars=itempars, estimation="LTS")
summary(mod3)

#- median regression
mod4 <- sirt::linking.haberman( itempars=itempars, estimation="MED")
summary(mod4)

#############################################################################
# EXAMPLE 7: Simulated example with directional DIF
#############################################################################

set.seed(98)
I <- 8
mu <- c(-.5, 0, .5)
b <- sample(seq(-1.5,1.5, len=I))
sd_dif <- 0.001
pars <- outer(b, mu, "+") + stats::rnorm(I*3, sd=sd_dif)
ind <- c(1,2); pars[ind,1] <- pars[ind,1] + c(.5,.5)
ind <- c(3,4); pars[ind,2] <- pars[ind,2] + (-1)*c(.6,.6)
ind <- c(5,6); pars[ind,3] <- pars[ind,3] + (-1)*c(1,1)

# median polish (=stats::medpolish())
tmod1 <- sirt:::L1_polish(x=pars)
# L0 polish with tolerance criterion of .3
tmod2 <- sirt::L0_polish(x=pars, tol=.3)

#- prepare itempars input
itempars <- sirt::linking_haberman_itempars_prepare(b=pars)

#- compare different estimation functions for Haberman linking
mod01 <- sirt::linking.haberman(itempars, estimation="L1")
mod02 <- sirt::linking.haberman(itempars, estimation="L0", b_trim=.3)
mod1 <- sirt::linking.haberman(itempars, estimation="OLS")
mod2 <- sirt::linking.haberman(itempars, estimation="BSQ")
mod2a <- sirt::linking.haberman(itempars, estimation="BSQ", b_trim=.4)
mod3 <- sirt::linking.haberman(itempars, estimation="MED")
mod4 <- sirt::linking.haberman(itempars, estimation="LTS")
mod5 <- sirt::linking.haberman(itempars, estimation="HUB")
mod01$transf.pars
mod02$transf.pars
mod1$transf.pars
mod2$transf.pars
mod2a$transf.pars
mod3$transf.pars
mod4$transf.pars
mod5$transf.pars

#############################################################################
# EXAMPLE 8: Many studies and directional DIF
#############################################################################

set.seed(98)
I <- 10 # number of items
S <- 7  # number of studies
mu <- round( seq(0, 1, len=S))
b <- sample(seq(-1.5,1.5, len=I))
sd_dif <- 0.001
pars0 <- pars <- outer(b, mu, "+") + stats::rnorm(I*S, sd=sd_dif)

# select n_dif items at random per group and set it to dif or -dif
n_dif <- 2
dif <- .6
for (ss in 1:S){
    ind <- sample( 1:I, n_dif )
    pars[ind,ss] <- pars[ind,ss] + dif*sign( runif(1) - .5 )
}

# check DIF
pars - pars0

#* estimate models
itempars <- sirt::linking_haberman_itempars_prepare(b=pars)
mod0 <- sirt::linking.haberman(itempars, estimation="L0", b_trim=.2)
mod1 <- sirt::linking.haberman(itempars, estimation="OLS")
mod2 <- sirt::linking.haberman(itempars, estimation="BSQ")
mod2a <- sirt::linking.haberman(itempars, estimation="BSQ", b_trim=.4)
mod3 <- sirt::linking.haberman(itempars, estimation="MED")
mod3a <- sirt::linking.haberman(itempars, estimation="L1")
mod4 <- sirt::linking.haberman(itempars, estimation="LTS")
mod5 <- sirt::linking.haberman(itempars, estimation="HUB")
mod0$transf.pars
mod1$transf.pars
mod2$transf.pars
mod2a$transf.pars
mod3$transf.pars
mod3a$transf.pars
mod4$transf.pars
mod5$transf.pars

#* compare results with Haebara linking
mod11 <- sirt::linking.haebara(itempars, dist="L2")
mod12 <- sirt::linking.haebara(itempars, dist="L1")
summary(mod11)
summary(mod12)

#############################################################################
# EXAMPLE 9: Haberman linking for polytomous data
#############################################################################

#* attach dataset involving two countries
data(data.timssAusTwn.scored, package="TAM")
dat <- data.timssAusTwn.scored
items <- grep("M0", colnames(dat), value=TRUE)

#* separate scaling with the generalized partial credit model (GPCM)
mod2a <- TAM::tam.mml.2pl( dat[dat$IDCNTRY==36,items], irtmodel="GPCM")
mod2b <- TAM::tam.mml.2pl( dat[dat$IDCNTRY==158,items], irtmodel="GPCM")

#* function for extracting item parameters of the GPCM
extract_gpcm_pars <- function(mod, study)
{
    # extract slope parameter
    a <- mod$B[,2,1]
    # maximum score
    K <- rowSums( 1-is.na( mod$AXsi[,-1] ) )
    # extract xsi
    xsi <- mod$xsi
    a1 <- rep( a, K )
    res <- data.frame( study=study, item=rownames(xsi), a=a1, b=xsi[,1] / a1 )
    return(res)
}

itempars1 <- extract_gpcm_pars(mod=mod2a, study="CNT036")
itempars2 <- extract_gpcm_pars(mod=mod2b, study="CNT158")
itempars <- rbind(itempars1, itempars2)

#* apply Haberman linking
lmod1 <- sirt::linking.haberman(itempars=itempars)
lmod1$transf.personpars
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Linking}
%% \keyword{Equating}

%% File Name: sirt_eigenvalues.Rd
%% File Version: 0.15

\name{sirt_eigenvalues}
\alias{sirt_eigenvalues}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
First Eigenvalues of a Symmetric Matrix
}
\description{
This function computes the first \eqn{D} eigenvalues and eigenvectors of a
symmetric positive definite matrices. The eigenvalues are computed
by the Rayleigh quotient method (Lange, 2010, p. 120).
}
\usage{
sirt_eigenvalues( X, D, maxit=200, conv=10^(-6) )
}

%% eigenvalues.sirt <- function (X,D,maxit,conv){
%%    .Call("eigenvalues_sirt", X,D,maxit,conv, PACKAGE="sirt")
%%                    }

%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{X}{
Symmetric matrix
}
  \item{D}{
Number of eigenvalues to be estimated
  }
  \item{maxit}{
Maximum number of iterations
}
  \item{conv}{
Convergence criterion
}
}

%% \details{
%% xxx
%% }

\value{
A list with following entries:
\item{d}{Vector of eigenvalues
}
\item{u}{Matrix with eigenvectors in columns}
}


\references{
Lange, K. (2010). \emph{Numerical Analysis for Statisticians}.
New York: Springer.
}


%\author{
%Alexander Robitzsch
%}

%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~




\examples{
Sigma <- diag(1,3)
Sigma[ lower.tri(Sigma) ] <- Sigma[ upper.tri(Sigma) ] <- c(.4,.6,.8 )
sirt::sirt_eigenvalues(X=Sigma, D=2 )
# compare with svd function
svd(Sigma)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Eigenvalues}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.mcdonald.Rd
%% File Version: 0.22

\name{data.mcdonald}

\alias{data.mcdonald.act15}
\alias{data.mcdonald.LSAT6}
\alias{data.mcdonald.rape}

\docType{data}

\title{
Some Datasets from McDonald's \emph{Test Theory} Book
}

\description{
Some datasets from McDonald (1999), especially related to using
NOHARM for item response modeling. See Examples below.
}


\usage{
data(data.mcdonald.act15)
data(data.mcdonald.LSAT6)
data(data.mcdonald.rape)
}



\format{
\itemize{
\item The format of the ACT15 data \code{data.mcdonald.act15} is:

\code{ num [1:15, 1:15] 0.49 0.44 0.38 0.3 0.29 0.13 0.23 0.16 0.16 0.23 ...} \cr
\code{ - attr(*, "dimnames")=List of 2} \cr
\code{  ..$ : chr [1:15] "A01" "A02" "A03" "A04" ...} \cr
\code{  ..$ : chr [1:15] "A01" "A02" "A03" "A04" ...}

The dataset (which is the product-moment covariance matrix)
is obtained from Ch. 12 in McDonald (1999). \cr

\item The format of the LSAT6 data \code{data.mcdonald.LSAT6} is:

\code{'data.frame':   1004 obs. of  5 variables:} \cr
\code{ $ L1: int  0 0 0 0 0 0 0 0 0 0 ...} \cr
\code{ $ L2: int  0 0 0 0 0 0 0 0 0 0 ...} \cr
\code{ $ L3: int  0 0 0 0 0 0 0 0 0 0 ...} \cr
\code{ $ L4: int  0 0 0 0 0 0 0 0 0 1 ...} \cr
\code{ $ L5: int  0 0 0 1 1 1 1 1 1 0 ...}

The dataset is obtained from Ch. 6 in McDonald (1999). \cr

\item The format of the rape myth scale data  \code{data.mcdonald.rape} is

\code{List of 2} \cr
\code{ $ lambda: num [1:2, 1:19] 1.13 0.88 0.85 0.77 0.79 0.55 1.12 1.01 0.99 0.79 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : chr [1:2] "male" "female"} \cr
\code{  .. ..$ : chr [1:19] "I1" "I2" "I3" "I4" ...} \cr
\code{ $ nu    : num [1:2, 1:19] 2.88 1.87 3.12 2.32 2.13 1.43 3.79 2.6 3.01 2.11 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : chr [1:2] "male" "female"} \cr
\code{  .. ..$ : chr [1:19] "I1" "I2" "I3" "I4" ...} \cr

The dataset is obtained from Ch. 15 in McDonald (1999).

}
}

%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}

\source{
Tables in McDonald (1999)
}


\references{
McDonald, R. P. (1999). \emph{Test theory: A unified treatment}.
Psychology Press.
}


\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: LSAT6 data    | Chapter 12 McDonald (1999)
#############################################################################
data(data.mcdonald.act15)

#************
# Model 1: 2-parameter normal ogive model

#++ NOHARM estimation
I <- ncol(dat)
# covariance structure
P.pattern <- matrix( 0, ncol=1, nrow=1 )
P.init <- 1+0*P.pattern
# fix all entries in the loading matrix to 1
F.pattern <- matrix( 1, nrow=I, ncol=1 )
F.init <- F.pattern
# estimate model
mod1a <- sirt::R2noharm( dat=dat, model.type="CFA", F.pattern=F.pattern,
             F.init=F.init, P.pattern=P.pattern, P.init=P.init,
             writename="LSAT6__1dim_2pno", noharm.path=noharm.path, dec="," )
summary(mod1a, logfile="LSAT6__1dim_2pno__SUMMARY")

#++ pairwise marginal maximum likelihood estimation using the probit link
mod1b <- sirt::rasch.pml3( dat, est.a=1:I, est.sigma=FALSE)

#************
# Model 2: 1-parameter normal ogive model

#++ NOHARM estimation
# covariance structure
P.pattern <- matrix( 0, ncol=1, nrow=1 )
P.init <- 1+0*P.pattern
# fix all entries in the loading matrix to 1
F.pattern <- matrix( 2, nrow=I, ncol=1 )
F.init <- 1+0*F.pattern
# estimate model
mod2a <- sirt::R2noharm( dat=dat, model.type="CFA", F.pattern=F.pattern,
                F.init=F.init, P.pattern=P.pattern, P.init=P.init,
                writename="LSAT6__1dim_1pno", noharm.path=noharm.path, dec="," )
summary(mod2a, logfile="LSAT6__1dim_1pno__SUMMARY")

# PMML estimation
mod2b <- sirt::rasch.pml3( dat, est.a=rep(1,I), est.sigma=FALSE )
summary(mod2b)

#************
# Model 3: 3-parameter normal ogive model with fixed guessing parameters

#++ NOHARM estimation
# covariance structure
P.pattern <- matrix( 0, ncol=1, nrow=1 )
P.init <- 1+0*P.pattern
# fix all entries in the loading matrix to 1
F.pattern <- matrix( 1, nrow=I, ncol=1 )
F.init <- 1+0*F.pattern
# estimate model
mod <- sirt::R2noharm( dat=dat, model.type="CFA",  guesses=rep(.2,I),
            F.pattern=F.pattern, F.init=F.init, P.pattern=P.pattern,
            P.init=P.init, writename="LSAT6__1dim_3pno",
            noharm.path=noharm.path, dec="," )
summary(mod, logfile="LSAT6__1dim_3pno__SUMMARY")

#++ logistic link function employed in smirt function
mod1d <- sirt::smirt(dat, Qmatrix=F.pattern, est.a=matrix(1:I,I,1), c.init=rep(.2,I))
summary(mod1d)

#############################################################################
# EXAMPLE 2: ACT15 data    | Chapter 6 McDonald (1999)
#############################################################################
data(data.mcdonald.act15)
pm <- data.mcdonald.act15

#************
# Model 1: 2-dimensional exploratory factor analysis
mod1 <- sirt::R2noharm( pm=pm, n=1000, model.type="EFA", dimensions=2,
             writename="ACT15__efa_2dim", noharm.path=noharm.path, dec="," )
summary(mod1)

#************
# Model 2: 2-dimensional independent clusters basis solution
P.pattern <- matrix(1,2,2)
diag(P.pattern) <- 0
P.init <- 1+0*P.pattern
F.pattern <- matrix(0,15,2)
F.pattern[ c(1:5,11:15),1] <- 1
F.pattern[ c(6:10,11:15),2] <- 1
F.init <- F.pattern

# estimate model
mod2 <- sirt::R2noharm( pm=pm, n=1000,  model.type="CFA", F.pattern=F.pattern,
            F.init=F.init, P.pattern=P.pattern,P.init=P.init,
            writename="ACT15_indep_clusters", noharm.path=noharm.path, dec="," )
summary(mod2)

#************
# Model 3: Hierarchical model

P.pattern <- matrix(0,3,3)
P.init <- P.pattern
diag(P.init) <- 1
F.pattern <- matrix(0,15,3)
F.pattern[,1] <- 1    # all items load on g factor
F.pattern[ c(1:5,11:15),2] <- 1   # Items 1-5 and 11-15 load on first nested factor
F.pattern[ c(6:10,11:15),3] <- 1  # Items 6-10 and 11-15 load on second nested factor
F.init <- F.pattern

# estimate model
mod3 <- sirt::R2noharm( pm=pm, n=1000,  model.type="CFA", F.pattern=F.pattern,
           F.init=F.init, P.pattern=P.pattern, P.init=P.init,
           writename="ACT15_hierarch_model", noharm.path=noharm.path, dec="," )
summary(mod3)

#############################################################################
# EXAMPLE 3: Rape myth scale | Chapter 15 McDonald (1999)
#############################################################################
data(data.mcdonald.rape)
lambda <- data.mcdonald.rape$lambda
nu <- data.mcdonald.rape$nu

# obtain multiplier for factor loadings (Formula 15.5)
k <- sum( lambda[1,] * lambda[2,] ) / sum( lambda[2,]^2 )
  ##   [1] 1.263243

# additive parameter (Formula 15.7)
c <- sum( lambda[2,]*(nu[1,]-nu[2,]) ) / sum( lambda[2,]^2 )
  ##   [1] 1.247697

# SD in the female group
1/k
  ##   [1] 0.7916132

# M in the female group
- c/k
  ##   [1] -0.9876932

# Burt's coefficient of factorial congruence (Formula 15.10a)
sum( lambda[1,] * lambda[2,] ) / sqrt( sum( lambda[1,]^2 ) * sum( lambda[2,]^2 ) )
  ##   [1] 0.9727831

# congruence for mean parameters
sum(  (nu[1,]-nu[2,]) * lambda[2,] ) / sqrt( sum( (nu[1,]-nu[2,])^2 ) * sum( lambda[2,]^2 ) )
  ##   [1] 0.968176
}
}
%% \keyword{datasets}

%% File Name: np.dich.Rd
%% File Version: 0.21

\name{np.dich}
\alias{np.dich}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Nonparametric Estimation of Item Response Functions
}
\description{
This function does nonparametric item response function
estimation (Ramsay, 1991).
}
\usage{
np.dich(dat, theta, thetagrid, progress=FALSE, bwscale=1.1,
       method="normal")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
An \eqn{N \times I} data frame of dichotomous
item responses
}
  \item{theta}{
Estimated theta values, for example weighted likelihood
estimates from \code{\link{wle.rasch}}
}
  \item{thetagrid}{
A vector of theta values where the nonparametric item
response functions shall be evaluated.
}
  \item{progress}{
Display progress?
}
  \item{bwscale}{
The bandwidth parameter \eqn{h} is calculated by
the formula \eqn{h=}\code{bwscale}\eqn{\cdot N^{-1/5}}
}
  \item{method}{
The default \code{normal} performs kernel regression
with untransformed item responses. The method \code{binomial}
uses nonparametric logistic regression implemented
in the \pkg{sm} library.
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list with following entries
\item{dat}{Original data frame}
\item{thetagrid}{Vector of theta values at which the item response
    functions are evaluated}
\item{theta}{Used theta values as person parameter estimates}
\item{estimate}{Estimated item response functions}
\item{\dots}{}
}
\references{
Ramsay, J. O. (1991). Kernel smoothing approaches to
nonparametric item characteristic curve estimation.
\emph{Psychometrika, 56}, 611-630.
}

%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{
#############################################################################
# EXAMPLE 1: Reading dataset
#############################################################################
data( data.read )
dat <- data.read

# estimate Rasch model
mod <- sirt::rasch.mml2( dat )
# WLE estimation
wle1 <- sirt::wle.rasch( dat=dat, b=mod$item$b )$theta
# nonparametric function estimation
np1 <- sirt::np.dich( dat=dat, theta=wle1, thetagrid=seq(-2.5, 2.5, len=100 ) )
print( str(np1))
# plot nonparametric item response curves
plot( np1, b=mod$item$b )
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Nonparametric item response theory}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.pisaPars.Rd
%% File Version: 0.10

\name{data.pisaPars}
\alias{data.pisaPars}
\docType{data}
\title{
Item Parameters from Two PISA Studies
}
\description{
This data frame contains item parameters from two PISA studies.
Because the Rasch model is used, only item difficulties are considered.
}
\usage{
data(data.pisaPars)
}

\format{
  A data frame with 25 observations on the following 4 variables.
  \describe{
    \item{\code{item}}{Item names}
    \item{\code{testlet}}{Items are arranged in corresponding testlets. These
        names are located in this column.}
    \item{\code{study1}}{Item difficulties of study 1}
    \item{\code{study2}}{Item difficulties of study 2}
        }
}


%% \keyword{datasets}

%% File Name: rmvn.Rd
%% File Version: 0.08

\name{rmvn}
\alias{rmvn}
\alias{ruvn}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Simulation of a Multivariate Normal Distribution with Exact Moments
}
\description{
Simulates a dataset from a multivariate or univariate normal distribution that
exactly fulfils the specified mean vector and the covariance matrix.
}
\usage{
# multivariate normal distribution
rmvn(N, mu, Sigma, exact=TRUE)

# univariate normal distribution
ruvn(N, mean=0, sd=1, exact=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{N}{
Sample size
}
  \item{mu}{
Mean vector
}
  \item{Sigma}{
Covariance matrix
}
  \item{exact}{
Logical indicating whether \code{mu} and \code{Sigma} should be exactly reproduced.
}
\item{mean}{Numeric value for mean}
\item{sd}{Numeric value for standard deviation}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

\value{
A dataframe or a vector
}

%\references{
%% ~put references to the literature/web site here ~
%}
%\author{
%%  ~~who you are~~
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{mvtnorm::rmvnorm}, \code{mgcv::rmvn}
}


\examples{
#############################################################################
# EXAMPLE 1: Simulate multivariate normal data
#############################################################################

# define covariance matrix and mean vector
rho <- .8
Sigma <- matrix(rho,3,3)
diag(Sigma) <- 1
mu <- c(0,.5,1)

#* simulate data
set.seed(87)
dat <- sirt::rmvn(N=200, mu=mu, Sigma=Sigma)
#* check means and covariances
stats::cov.wt(dat, method="ML")

\dontrun{
#############################################################################
# EXAMPLE 2: Simulate univariate normal data
#############################################################################

#* simulate data
x <- sirt::ruvn(N=20, mean=.5, sd=1.2, exact=TRUE)
# check results
stats::var(x)
sirt:::sirt_var(x)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }% use one of  RShowDoc("KEYWORDS")
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: pcm.conversion.Rd
%% File Version: 0.18

\name{pcm.conversion}
\alias{pcm.conversion}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Conversion of the Parameterization of the Partial Credit Model
}
\description{
Converts a parameterization of the partial credit model (see Details).
}

\usage{
pcm.conversion(b)
}
%- maybe also 'usage' for other objects documented here.

\arguments{
  \item{b}{
Matrix of item-category-wise intercepts \eqn{b_{ik}} (see Details).
}

}
\details{
Assume that the input matrix \code{b} containing parameters \eqn{b_{ik}}
is defined according to the following parametrization of the partial credit
model
\deqn{ P( X_{pi}=k | \theta_p ) \propto exp ( k \theta_p - b_{ik} ) }
if item \eqn{i} possesses \eqn{K_i} categories.
The transformed parameterization is defined as
\deqn{b_{ik}=k \delta_i + \sum_{v=1}^{k} \tau_{iv} \quad
\mbox{with} \quad \sum_{k=1}^{K_i} \tau_{ik}=0 }
The function \code{pcm.conversion} has the \eqn{\delta} and \eqn{\tau}
parameters as values. The \eqn{\delta} parameter is simply
\eqn{\delta_i=b_{iK_i} / K_i}.
}



\value{
List with the following entries
  \item{delta}{Vector of \eqn{\delta} parameters}
  \item{tau}{Matrix of \eqn{\tau} parameters}
}

%\references{
%% ~put references to the literature/web site here ~
%}

%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}


\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Transformation PCM for data.mg
#############################################################################

library(CDM)
data(data.mg,package="CDM")
dat <- data.mg[ 1:1000, paste0("I",1:11) ]

#*** Model 1: estimate partial credit model in parameterization "PCM"
mod1a <- TAM::tam.mml( dat, irtmodel="PCM")
# use parameterization "PCM2"
mod1b <- TAM::tam.mml( dat, irtmodel="PCM2")
summary(mod1a)
summary(mod1b)

# convert parameterization of Model 1a into parameterization of Model 1b
b <- mod1a$item[, c("AXsi_.Cat1","AXsi_.Cat2","AXsi_.Cat3") ]
# compare results
pcm.conversion(b)
mod1b$xsi
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Partial credit model}
% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: mgsem.Rd
%% File Version: 0.083

\name{mgsem}
\alias{mgsem}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimation of Multiple-Group Structural Equation Models
}
\description{
Estimates a multiple-group structural equation model. The function allows arbitrary
prior distributions on model parameters and regularized estimation with the SCAD and
the LASSO penalty. Moreover, it can also conduct robust moment estimation using
the \eqn{L_p} loss function \eqn{\rho(x)=|x|^p} for \eqn{p \ge 0}.
See Robitzsch (2023) for more details.
}
\usage{
mgsem(suffstat, model, data=NULL, group=NULL, weights=NULL, estimator="ML",
     p_me=2, p_pen=1, pen_type="scad", diffpar_pen=NULL, pen_sample_size=TRUE,
     a_scad=3.7, eps_approx=0.001, comp_se=TRUE, se_delta_formula=FALSE,
     prior_list=NULL, hessian=TRUE, fixed_parms=FALSE, cd=FALSE,
     cd_control=list(maxiter=20, tol=5*1e-04, interval_length=0.05, method="exact"),
     partable_start=NULL, num_approx=FALSE, technical=NULL, control=list())
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{suffstat}{
List containing sufficient statistics
}
  \item{model}{
Model specification, see examples. Can have entries \code{est}, \code{index},
\code{lower}, \code{upper}, \code{prior}, \code{pen_l2}, \code{pen_lp},
\code{pen_difflp}. Each entry can be defined for model matrices \code{ALPHA},
\code{NU}, \code{LAM}, \code{PHI}, and \code{PSI}.
}
  \item{data}{
Optional data frame
}
  \item{group}{
Optional vector of group identifiers
}
  \item{weights}{
Optional vector of sampling weights
}
  \item{estimator}{
Character. Can be either \code{"ML"} for maximum likelihood fitting function or
\code{"ME"} for robust moment estimation.
}
  \item{p_me}{
Power in $L_p$ loss function for robust moment estimation
}
  \item{p_pen}{
Power for penalty in regularized estimation. For regular LASSO and SCAD penalty
functions, it is $p=1$.
}
  \item{pen_type}{
Penalty type. Can be either \code{"scad"} or \code{"lasso"}.
}
  \item{diffpar_pen}{
List containing values of regularization parameters in fused lasso estimation
}
  \item{pen_sample_size}{
List containing values for sample sizes for regularized estimation
}
  \item{a_scad}{
Parameter $a$ used in SCAD penalty
}
  \item{eps_approx}{
Approximation value for nondifferentiable robust moment fitting function or
penalty function
}
  \item{comp_se}{
Logical indicating whether standard errors should be computed
}
  \item{se_delta_formula}{
Logical indicating whether standard errors should be computed according to the
delta formula
}
  \item{prior_list}{
List containing specifications of the prior distributions
}
  \item{hessian}{
Logical indicating whether the Hessian matrix should be computed
}
  \item{fixed_parms}{
Logical indicating whether all model parameters should be fixed
}
  \item{cd}{
Logical indicating whether coordinate descent should be used for estimation
}
  \item{cd_control}{
Control parameters for coordinate descent estimation
}
  \item{partable_start}{
Starting values for parameter estimation
}
  \item{num_approx}{
Logical indicating whether derivatives should be computed based on numerical
differentiation
}
  \item{technical}{
Parameters used for optimization in \code{sirt_optimizer}
}
  \item{control}{
Control paramaters for optimization
}
}

\details{
[MORE INFORMATION TO BE ADDED]
}


\value{
A list with following values
\item{coef}{Coeffients}
\item{vcov}{Variance matrix}
\item{se}{Vector of standard errors}
\item{partable}{Parameter table}
\item{model}{Specified model}
\item{opt_res}{Result from optimization}
\item{opt_value}{Value of fitting function}
\item{residuals}{Residuals of sufficient statistics}
\item{ic}{Information criteria}
\item{technical}{Specifications of optimizer}
\item{suffstat_vcov}{Variance matrix of sufficient statistics}
\item{me_delta_method}{Input and output matrices for delta method if
\code{estimator="ME"}}
\item{data_proc}{Processed data}
\item{case_ll}{Casewise log-likelihood function}
\item{\dots}{Further values}
}

\references{
Robitzsch, A. (2023). Model-robust estimation of multiple-group structural
equation models. \emph{Algorithms, 16}(4), 210. \doi{10.3390/a16040210}
}


%\author{
%%  ~~who you are~~
%}

%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}


\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Noninvariant item intercepts in a multiple-group SEM
#############################################################################

#---- simulate data
set.seed(65)
G <- 3  # number of groups
I <- 5  # number of items
# define lambda and nu parameters
lambda <- matrix(1, nrow=G, ncol=I)
nu <- matrix(0, nrow=G, ncol=I)
err_var <- matrix(1, nrow=G, ncol=I)

# define extent of noninvariance
dif_int <- .5

#- 1st group: N(0,1)
nu[1,4] <- dif_int
#- 2nd group: N(0.3,1.5)
gg <- 2 ;
nu[gg,1] <- -dif_int
#- 3nd group: N(.8,1.2)
gg <- 3
nu[gg,2] <- -dif_int
#- define distributions of groups
mu <- c(0,.3,.8)
sigma <- sqrt(c(1,1.5,1.2))
N <- rep(1000,3) # sample sizes per group

exact <- FALSE
suffstat <- sirt::invariance_alignment_simulate(nu, lambda, err_var, mu, sigma, N,
                output="suffstat", groupwise=TRUE, exact=exact)

#---- model specification

# model specifications joint group
est <- list(
        ALPHA=matrix( c(0), ncol=1),
        NU=matrix( 0, nrow=I, ncol=1),
        LAM=matrix(1, nrow=I, ncol=1),
        PHI=matrix(0,nrow=1,ncol=1),
        PSI=diag(rep(1,I))
        )

# parameter index
index <- list(
        ALPHA=0*est$ALPHA,
        NU=1+0*est$NU,
        LAM=1+0*est$LAM,
        PHI=0*est$PHI,
        PSI=diag(1,I)
        )

# lower bounds
lower <- list(
        PSI=diag(rep(0.01,I)), PHI=matrix(0.01,1,1)
        )

#*** joint parameters
group0 <- list(est=est, index=index, lower=lower)

#*** group1
est <- list(
        ALPHA=matrix( c(0), ncol=1),
        NU=matrix( 0, nrow=I, ncol=1),
        LAM=matrix(0, nrow=I, ncol=1),
        PHI=matrix(1,nrow=1,ncol=1)
            )

# parameter index
index <- list(
        ALPHA=0*est$ALPHA,
        NU=0*est$NU,
        LAM=1*est$LAM,
        PHI=0*est$PHI
        )

group1 <- list(est=est, index=index, lower=lower)

#*** group 2 and group 3

# modify parameter index
index$ALPHA <- 1+0*est$ALPHA
index$PHI <- 1+0*est$PHI
group3 <- group2 <- list(est=est, index=index, lower=lower)

#*** define model
model <- list(group0=group0, group1=group1, group2=group2, group3=group3)

#-- estimate model with ML
res1 <- sirt::mgsem( suffstat=suffstat, model=model2, eps_approx=1e-4, estimator="ML",
                    technical=list(maxiter=500, optimizer="optim"),
                    hessian=FALSE, comp_se=FALSE, control=list(trace=1) )
str(res1)

#-- robust moment estimation with p=0.5

optimizer <- "optim"
technical <- list(maxiter=500, optimizer=optimizer)
eps_approx <- 1e-3

res2 <- sirt::mgsem( suffstat=suffstat, model=res1$model, p_me=0.5,
                    eps_approx=eps_approx, estimator="ME", technical=technical,
                    hessian=FALSE, comp_se=FALSE, control=list(trace=1) )

#---- regularized estimation

nu_lam <- 0.1    # regularization parameter

# redefine model
define_model <- function(model, nu_lam)
{
    pen_lp <- list( NU=nu_lam+0*model$group1$est$NU)
    ee <- "group1"
    for (ee in c("group1","group2","group3"))
    {
        model[[ee]]$index$NU <- 1+0*index$NU
        model[[ee]]$pen_lp <- pen_lp
    }
    return(model)
}

model3 <- define_model(model=model, nu_lam=nu_lam)
pen_type <- "scad"

res3 <- sirt::mgsem( suffstat=suffstat, model=model3, p_pen=1, pen_type=pen_type,
                    eps_approx=eps_approx, estimator="ML",
                    technical=list(maxiter=500, optimizer="optim"),
                    hessian=FALSE, comp_se=FALSE, control=list(trace=1) )
str(res3)
}
}


% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

%% File Name: cfa_meas_inv.Rd
%% File Version: 0.074

\name{cfa_meas_inv}
\alias{cfa_meas_inv}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimation of a Unidimensional Factor Model under Full and Partial
Measurement Invariance
}
\description{
Estimates a unidimensional factor model based on the normal distribution fitting
function under full and partial measurement invariance.
Item loadings and item intercepts are successively freed based on the largest
modification index and a chosen significance level \code{alpha}.
}
\usage{
cfa_meas_inv(dat, group, weights=NULL, alpha=0.01, verbose=FALSE, op=c("~1","=~"))
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame containing items
}
  \item{group}{
Vector of group identifiers
}
  \item{weights}{
Optional vector of sampling weights
}
  \item{alpha}{
Significance level
}
  \item{verbose}{
Logical indicating whether progress should be shown
}
\item{op}{Operators (intercepts or loadings) for which estimation should be freed}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
List with several entries
  \item{pars_mi}{Model parameters under full invariance}
  \item{pars_pi}{Model parameters under partial invariance}
  \item{mod_mi}{Fitted model under full invariance}
  \item{mod_pi}{Fitted model under partial invariance}

  \item{\ldots}{More output}
}
%\references{
%% ~put references to the literature/web site here ~
%}
%\author{
%%  ~~who you are~~
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also \code{\link[sirt:invariance.alignment]{sirt::invariance.alignment}}
}
\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Factor model under full and partial invariance
#############################################################################

#--- data simulation

set.seed(65)
G <- 3  # number of groups
I <- 5  # number of items
# define lambda and nu parameters
lambda <- matrix(1, nrow=G, ncol=I)
nu <- matrix(0, nrow=G, ncol=I)
err_var <- matrix(1, nrow=G, ncol=I)

# define size of noninvariance
dif <- 1
#- 1st group: N(0,1)
lambda[1,3] <- 1+dif*.4; nu[1,5] <- dif*.5
#- 2nd group: N(0.3,1.5)
gg <- 2 ;
lambda[gg,5] <- 1-.5*dif; nu[gg,1] <- -.5*dif
#- 3nd group: N(.8,1.2)
gg <- 3
lambda[gg,4] <- 1-.7*dif; nu[gg,2] <- -.5*dif
#- define distributions of groups
mu <- c(0,.3,.8)
sigma <- sqrt(c(1,1.5,1.2))
N <- rep(1000,3) # sample sizes per group

#* use simulation function
dat <- sirt::invariance_alignment_simulate(nu, lambda, err_var, mu, sigma, N,
                exact=TRUE)

#--- estimate CFA
mod <- sirt::cfa_meas_inv(dat=dat[,-1], group=dat$group, verbose=TRUE, alpha=0.05)
mod$pars_mi
mod$pars_pi
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

%% File Name: rasch.pairwise.Rd
%% File Version: 0.356

\name{rasch.pairwise}
\alias{rasch.pairwise}
\alias{summary.rasch.pairwise}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Pairwise Estimation Method of the Rasch Model
}
\description{
This function estimates the Rasch model with a minimum chi
square estimation method (cited in Fischer, 2007, p. 544)
which is a pairwise conditional likelihood estimation approach.
}
\usage{
rasch.pairwise(dat, weights=NULL, conv=1e-04, maxiter=3000, progress=TRUE,
        b.init=NULL, zerosum=FALSE, power=1, direct_optim=TRUE)

\method{summary}{rasch.pairwise}(object, digits=3, file=NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
An \eqn{N \times I} data frame of dichotomous item responses
}
\item{weights}{Optional vector of sampling weights}
  \item{conv}{
Convergence criterion
}
  \item{maxiter}{
Maximum number of iterations
}
  \item{progress}{
Display iteration progress?
}
  \item{b.init}{
An optional vector of length \eqn{I} of item difficulties
}
\item{zerosum}{Optional logical indicating whether item difficulties
    should be centered in each iteration. The default is that
        no centering is conducted.}
\item{power}{Power used for computing pairwise response probabilities like
in row averaging approach}
\item{direct_optim}{Logical indicating whether least squares criterion
funcion should be minimized with \code{\link[stats:nlminb]{stats::nlminb}}
}
  \item{object}{
Object of class \code{rasch.pairwise}
}
\item{digits}{Number of digits after decimal for rounding}
\item{file}{Optional file name for summary output}
  \item{\dots}{
Further arguments to be passed
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
An object of class \code{rasch.pairwise} with following entries
\item{b}{Item difficulties}
\item{eps}{Exponentiated item difficulties, i.e. \code{eps=exp(-b)}}
\item{iter}{Number of iterations}
\item{conv}{Convergence criterion}
\item{dat}{Original data frame}
\item{freq.ij}{Frequency table of all item pairs}
\item{item}{Summary table of item parameters}
}
\references{
  Fischer, G. H. (2007). Rasch models.
  In C. R. Rao and S. Sinharay (Eds.), \emph{Handbook of Statistics},
  Vol. 26 (pp. 515-585). Amsterdam: Elsevier.
}

%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See \code{\link{summary.rasch.pairwise}} for a summary.

A slightly different implementation of this conditional pairwise method
is implemented in \cr \code{\link{rasch.pairwise.itemcluster}}.

Pairwise marginal likelihood estimation (also labeled as pseudolikelihood
estimation) can be conducted with \code{\link{rasch.pml3}}.
}
\examples{
#############################################################################
# EXAMPLE 1: Reading data set | pairwise estimation Rasch model
#############################################################################

data(data.read)
dat <- data.read

#*** Model 1: no constraint on item difficulties
mod1 <- sirt::rasch.pairwise(dat)
summary(mod1)

#*** Model 2: sum constraint on item difficulties
mod2 <- sirt::rasch.pairwise(dat, zerosum=TRUE)
summary(mod2)

\dontrun{
#** obtain standard errors by bootstrap
mod2$item$b   # extract item difficulties

# Bootstrap of item difficulties
boot_pw <- function(data, indices ){
    dd <- data[ indices, ] # bootstrap of indices
    mod <- sirt::rasch.pairwise( dat=dd, zerosum=TRUE, progress=FALSE)
    return(mod$item$b)
}
set.seed(986)
library(boot)
bmod2 <- boot::boot(data=dat, statistic=boot_pw, R=999 )
print(bmod2)
summary(bmod2)
# quantiles for bootstrap sample (and confidence interval)
apply(bmod2$t, 2, stats::quantile, probs=c(.025, .5, .975) )
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Pairwise conditional maximum likelihood (PCML)}
%% \keyword{Minchi method}
%% \keyword{summary}

%% File Name: data.read.Rd
%% File Version: 2.001

\name{data.read}
\alias{data.read}
\docType{data}
\title{
Dataset Reading
}
\description{
This dataset contains \eqn{N=328} students and \eqn{I=12} items measuring reading
competence. All 12 items are arranged into 3 testlets (items with common
text stimulus) labeled as
A, B and C. The allocation of items to testlets is indicated by their
variable names.
}
\usage{
data(data.read)
}

\format{
  A data frame with 328 persons on the following 12 variables.
  Rows correspond to persons and columns to items. The following items are
  included in \code{data.read}:

  Testlet A: \code{A1}, \code{A2}, \code{A3}, \code{A4}

  Testlet B: \code{B1}, \code{B2}, \code{B3}, \code{B4}

  Testlet C: \code{C1}, \code{C2}, \code{C3}, \code{C4}

}



\examples{
\dontrun{
data(data.read)
dat <- data.read
I <- ncol(dat)

# list of needed packages for the following examples
packages <- scan(what="character")
     eRm  ltm  TAM mRm  CDM  mirt psychotools  IsingFit  igraph  qgraph  pcalg
     poLCA  randomLCA psychomix MplusAutomation lavaan

# load packages. make an installation if necessary
miceadds::library_install(packages)

#*****************************************************
# Model 1: Rasch model
#*****************************************************

#----  M1a: rasch.mml2 (in sirt)
mod1a <- sirt::rasch.mml2(dat)
summary(mod1a)

#----  M1b: smirt (in sirt)
Qmatrix <- matrix(1,nrow=I, ncol=1)
mod1b <- sirt::smirt(dat,Qmatrix=Qmatrix)
summary(mod1b)

#----  M1c: gdm (in CDM)
theta.k <- seq(-6,6,len=21)
mod1c <- CDM::gdm(dat,theta.k=theta.k,irtmodel="1PL", skillspace="normal")
summary(mod1c)

#----  M1d: tam.mml (in TAM)
mod1d <- TAM::tam.mml( resp=dat )
summary(mod1d)

#----  M1e: RM (in eRm)
mod1e <- eRm::RM( dat )
  # eRm uses Conditional Maximum Likelihood (CML) as the estimation method.
summary(mod1e)
eRm::plotPImap(mod1e)

#----  M1f: mrm (in mRm)
mod1f <- mRm::mrm( dat, cl=1)   # CML estimation
mod1f$beta  # item parameters

#----  M1g: mirt (in mirt)
mod1g <- mirt::mirt( dat, model=1, itemtype="Rasch", verbose=TRUE )
print(mod1g)
summary(mod1g)
coef(mod1g)
    # arrange coefficients in nicer layout
sirt::mirt.wrapper.coef(mod1g)$coef

#----  M1h: rasch (in ltm)
mod1h <- ltm::rasch( dat, control=list(verbose=TRUE ) )
summary(mod1h)
coef(mod1h)

#----  M1i: RaschModel.fit (in psychotools)
mod1i <- psychotools::RaschModel.fit(dat)  # CML estimation
summary(mod1i)
plot(mod1i)

#----  M1j: noharm.sirt (in sirt)
Fpatt <- matrix( 0, I, 1 )
Fval <- 1 + 0*Fpatt
Ppatt <- Pval <- matrix(1,1,1)
mod1j <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt, Fpatt=Fpatt, Fval=Fval, Pval=Pval)
summary(mod1j)
  #   Normal-ogive model, multiply item discriminations with constant D=1.7.
  #   The same holds for other examples with noharm.sirt and R2noharm.
plot(mod1j)

#----  M1k: rasch.pml3 (in sirt)
mod1k <- sirt::rasch.pml3( dat=dat)
  #         pairwise marginal maximum likelihood estimation
summary(mod1k)

#----  M1l: running Mplus (using MplusAutomation package)
mplus_path <- "c:/Mplus7/Mplus.exe"    # locate Mplus executable
#****************
  # specify Mplus object
mplusmod <- MplusAutomation::mplusObject(
    TITLE="1PL in Mplus ;",
    VARIABLE=paste0( "CATEGORICAL ARE ", paste0(colnames(dat),collapse=" ") ),
    MODEL="
       ! fix all item loadings to 1
       F1 BY A1@1 A2@1 A3@1 A4@1 ;
       F1 BY B1@1 B2@1 B3@1 B4@1 ;
       F1 BY C1@1 C2@1 C3@1 C4@1 ;
       ! estimate variance
       F1 ;
            ",
    ANALYSIS="ESTIMATOR=MLR;",
    OUTPUT="stand;",
    usevariables=colnames(dat),  rdata=dat )
#****************

  # write Mplus syntax
filename <- "mod1u"   # specify file name
  # create Mplus syntaxes
res2 <- MplusAutomation::mplusModeler(object=mplusmod, dataout=paste0(filename,".dat"),
               modelout=paste0(filename,".inp"), run=0 )
  # run Mplus model
MplusAutomation::runModels( filefilter=paste0(filename,".inp"), Mplus_command=mplus_path)
  # alternatively, the system() command can also be used
  # get results
mod1l <- MplusAutomation::readModels(target=getwd(), filefilter=filename )
mod1l$summaries    # summaries
mod1l$parameters$unstandardized   # parameter estimates

#*****************************************************
# Model 2: 2PL model
#*****************************************************

#----  M2a: rasch.mml2 (in sirt)
mod2a <- sirt::rasch.mml2(dat, est.a=1:I)
summary(mod2a)

#----  M2b: smirt (in sirt)
mod2b <- sirt::smirt(dat,Qmatrix=Qmatrix,est.a="2PL")
summary(mod2b)

#----  M2c: gdm (in CDM)
mod2c <- CDM::gdm(dat,theta.k=theta.k,irtmodel="2PL", skillspace="normal")
summary(mod2c)

#----  M2d: tam.mml (in TAM)
mod2d <- TAM::tam.mml.2pl( resp=dat )
summary(mod2d)

#----  M2e: mirt (in mirt)
mod2e <- mirt::mirt( dat, model=1, itemtype="2PL" )
print(mod2e)
summary(mod2e)
sirt::mirt.wrapper.coef(mod1g)$coef

#----  M2f: ltm (in ltm)
mod2f <- ltm::ltm( dat ~ z1, control=list(verbose=TRUE ) )
summary(mod2f)
coef(mod2f)
plot(mod2f)

#----  M2g: R2noharm (in NOHARM, running from within R using sirt package)
  # define noharm.path where 'NoharmCL.exe' is located
noharm.path <- "c:/NOHARM"
  # covariance matrix
P.pattern <- matrix( 1, ncol=1, nrow=1 )
P.init <- P.pattern
P.init[1,1] <- 1
  # loading matrix
F.pattern <- matrix(1,I,1)
F.init <- F.pattern
  # estimate model
mod2g <- sirt::R2noharm( dat=dat, model.type="CFA", F.pattern=F.pattern,
             F.init=F.init, P.pattern=P.pattern, P.init=P.init,
             writename="ex2g", noharm.path=noharm.path, dec="," )
summary(mod2g)

#----  M2h: noharm.sirt (in sirt)
mod2h <- sirt::noharm.sirt( dat=dat, Ppatt=P.pattern,Fpatt=F.pattern,
              Fval=F.init, Pval=P.init )
summary(mod2h)
plot(mod2h)

#----  M2i: rasch.pml2 (in sirt)
mod2i <- sirt::rasch.pml2(dat, est.a=1:I)
summary(mod2i)

#----  M2j: WLSMV estimation with cfa (in lavaan)
lavmodel <- "F=~ A1+A2+A3+A4+B1+B2+B3+B4+
                        C1+C2+C3+C4"
mod2j <- lavaan::cfa( data=dat, model=lavmodel, std.lv=TRUE, ordered=colnames(dat))
summary(mod2j, standardized=TRUE, fit.measures=TRUE, rsquare=TRUE)

#*****************************************************
# Model 3: 3PL model (note that results can be quite unstable!)
#*****************************************************

#----  M3a: rasch.mml2 (in sirt)
mod3a <- sirt::rasch.mml2(dat, est.a=1:I, est.c=1:I)
summary(mod3a)

#----  M3b: smirt (in sirt)
mod3b <- sirt::smirt(dat,Qmatrix=Qmatrix,est.a="2PL", est.c=1:I)
summary(mod3b)

#----  M3c: mirt (in mirt)
mod3c <- mirt::mirt( dat, model=1, itemtype="3PL", verbose=TRUE)
summary(mod3c)
coef(mod3c)
  # stabilize parameter estimating using informative priors for guessing parameters
mirtmodel <- mirt::mirt.model("
            F=1-12
            PRIOR=(1-12, g, norm, -1.38, 0.25)
            ")
  # a prior N(-1.38,.25) is specified for transformed guessing parameters: qlogis(g)
  # simulate values from this prior for illustration
N <- 100000
logit.g <- stats::rnorm(N, mean=-1.38, sd=sqrt(.5) )
graphics::plot( stats::density(logit.g) )  # transformed qlogis(g)
graphics::plot( stats::density( stats::plogis(logit.g)) )  # g parameters
  # estimate 3PL with priors
mod3c1 <- mirt::mirt(dat, mirtmodel, itemtype="3PL",verbose=TRUE)
coef(mod3c1)
  # In addition, set upper bounds for g parameters of .35
mirt.pars <- mirt::mirt( dat, mirtmodel, itemtype="3PL",  pars="values")
ind <- which( mirt.pars$name=="g" )
mirt.pars[ ind, "value" ] <- stats::plogis(-1.38)
mirt.pars[ ind, "ubound" ] <- .35
  # prior distribution for slopes
ind <- which( mirt.pars$name=="a1" )
mirt.pars[ ind, "prior_1" ] <- 1.3
mirt.pars[ ind, "prior_2" ] <- 2
mod3c2 <- mirt::mirt(dat, mirtmodel, itemtype="3PL",
                pars=mirt.pars,verbose=TRUE, technical=list(NCYCLES=100) )
coef(mod3c2)
sirt::mirt.wrapper.coef(mod3c2)

#----  M3d: ltm (in ltm)
mod3d <- ltm::tpm( dat, control=list(verbose=TRUE), max.guessing=.3)
summary(mod3d)
coef(mod3d) #=> numerical instabilities

#*****************************************************
# Model 4: 3-dimensional Rasch model
#*****************************************************

# define Q-matrix
Q <- matrix( 0, nrow=12, ncol=3 )
Q[ cbind(1:12, rep(1:3,each=4) ) ] <- 1
rownames(Q) <- colnames(dat)
colnames(Q) <- c("A","B","C")

# define nodes
theta.k <- seq(-6,6,len=13)

#----  M4a: smirt (in sirt)
mod4a <- sirt::smirt(dat,Qmatrix=Q,irtmodel="comp", theta.k=theta.k, maxiter=30)
summary(mod4a)

#----  M4b: rasch.mml2 (in sirt)
mod4b <- sirt::rasch.mml2(dat,Q=Q,theta.k=theta.k, mmliter=30)
summary(mod4b)

#----  M4c: gdm (in CDM)
mod4c <- CDM::gdm( dat, irtmodel="1PL", theta.k=theta.k, skillspace="normal",
            Qmatrix=Q, maxiter=30, centered.latent=TRUE )
summary(mod4c)

#----  M4d: tam.mml (in TAM)
mod4d <- TAM::tam.mml( resp=dat, Q=Q, control=list(nodes=theta.k, maxiter=30) )
summary(mod4d)

#----  M4e: R2noharm (in NOHARM, running from within R using sirt package)
noharm.path <- "c:/NOHARM"
  # covariance matrix
P.pattern <- matrix( 1, ncol=3, nrow=3 )
P.init <- 0.8+0*P.pattern
diag(P.init) <- 1
  # loading matrix
F.pattern <- 0*Q
F.init <- Q
  # estimate model
mod4e <- sirt::R2noharm( dat=dat, model.type="CFA", F.pattern=F.pattern,
    F.init=F.init, P.pattern=P.pattern, P.init=P.init,
    writename="ex4e", noharm.path=noharm.path, dec="," )
summary(mod4e)

#----  M4f: mirt (in mirt)
cmodel <- mirt::mirt.model("
     F1=1-4
     F2=5-8
     F3=9-12
     # equal item slopes correspond to the Rasch model
     CONSTRAIN=(1-4, a1), (5-8, a2), (9-12,a3)
     COV=F1*F2, F1*F3, F2*F3
     " )
mod4f <- mirt::mirt(dat, cmodel, verbose=TRUE)
summary(mod4f)

#*****************************************************
# Model 5: 3-dimensional 2PL model
#*****************************************************

#----  M5a: smirt (in sirt)
mod5a <- sirt::smirt(dat,Qmatrix=Q,irtmodel="comp", est.a="2PL", theta.k=theta.k,
                 maxiter=30)
summary(mod5a)

#----  M5b: rasch.mml2 (in sirt)
mod5b <- sirt::rasch.mml2(dat,Q=Q,theta.k=theta.k,est.a=1:12, mmliter=30)
summary(mod5b)

#----  M5c: gdm (in CDM)
mod5c <- CDM::gdm( dat, irtmodel="2PL", theta.k=theta.k, skillspace="loglinear",
            Qmatrix=Q, maxiter=30, centered.latent=TRUE,
            standardized.latent=TRUE)
summary(mod5c)

#----  M5d: tam.mml (in TAM)
mod5d <- TAM::tam.mml.2pl( resp=dat, Q=Q, control=list(nodes=theta.k, maxiter=30) )
summary(mod5d)

#----  M5e: R2noharm (in NOHARM, running from within R using sirt package)
noharm.path <- "c:/NOHARM"
  # covariance matrix
P.pattern <- matrix( 1, ncol=3, nrow=3 )
diag(P.pattern) <- 0
P.init <- 0.8+0*P.pattern
diag(P.init) <- 1
  # loading matrix
F.pattern <- Q
F.init <- Q
  # estimate model
mod5e <- sirt::R2noharm( dat=dat, model.type="CFA", F.pattern=F.pattern,
    F.init=F.init, P.pattern=P.pattern, P.init=P.init,
    writename="ex5e", noharm.path=noharm.path, dec="," )
summary(mod5e)

#----  M5f: mirt (in mirt)
cmodel <- mirt::mirt.model("
   F1=1-4
   F2=5-8
   F3=9-12
   COV=F1*F2, F1*F3, F2*F3
   "  )
mod5f <- mirt::mirt(dat, cmodel, verbose=TRUE)
summary(mod5f)

#*****************************************************
# Model 6: Network models (Graphical models)
#*****************************************************

#----  M6a: Ising model using the IsingFit package (undirected graph)
#        - fit Ising model using the "OR rule" (AND=FALSE)
mod6a <- IsingFit::IsingFit(x=dat, family="binomial", AND=FALSE)
summary(mod6a)
##           Network Density:                 0.29
##    Gamma:                  0.25
##    Rule used:              Or-rule
# plot results
qgraph::qgraph(mod6a$weiadj,fade=FALSE)

#**-- graph estimation using pcalg package

# some packages from Bioconductor must be downloaded at first (if not yet done)
if (FALSE){  # set 'if (TRUE)' if packages should be downloaded
     source("http://bioconductor.org/biocLite.R")
     biocLite("RBGL")
     biocLite("Rgraphviz")
}

#----  M6b: graph estimation based on Pearson correlations
V <- colnames(dat)
n <- nrow(dat)
mod6b <- pcalg::pc(suffStat=list(C=stats::cor(dat), n=n ),
             indepTest=gaussCItest, ## indep.test: partial correlations
             alpha=0.05, labels=V, verbose=TRUE)
plot(mod6b)
# plot in qgraph package
qgraph::qgraph(mod6b, label.color=rep( c( "red", "blue","darkgreen" ), each=4 ),
         edge.color="black")
summary(mod6b)

#----  M6c: graph estimation based on tetrachoric correlations
mod6c <- pcalg::pc(suffStat=list(C=sirt::tetrachoric2(dat)$rho, n=n ),
             indepTest=gaussCItest, alpha=0.05, labels=V, verbose=TRUE)
plot(mod6c)
summary(mod6c)

#----  M6d: Statistical implicative analysis (in sirt)
mod6d <- sirt::sia.sirt(dat, significance=.85 )
  # plot results with igraph and qgraph package
plot( mod6d$igraph.obj, vertex.shape="rectangle", vertex.size=30 )
qgraph::qgraph( mod6d$adj.matrix )

#*****************************************************
# Model 7: Latent class analysis with 3 classes
#*****************************************************

#----  M7a: randomLCA (in randomLCA)
  #        - use two trials of starting values
mod7a <- randomLCA::randomLCA(dat,nclass=3, notrials=2, verbose=TRUE)
summary(mod7a)
plot(mod7a,type="l", xlab="Item")

#----  M7b: rasch.mirtlc (in sirt)
mod7b <- sirt::rasch.mirtlc( dat, Nclasses=3,seed=-30,  nstarts=2 )
summary(mod7b)
matplot( t(mod7b$pjk), type="l", xlab="Item" )

#----  M7c: poLCA (in poLCA)
  #   define formula for outcomes
f7c <- paste0( "cbind(", paste0(colnames(dat),collapse=","), ") ~ 1 " )
dat1 <- as.data.frame( dat + 1 ) # poLCA needs integer values from 1,2,..
mod7c <- poLCA::poLCA( stats::as.formula(f7c),dat1,nclass=3, verbose=TRUE)
plot(mod7c)

#----  M7d: gom.em (in sirt)
  #    - the latent class model is a special grade of membership model
mod7d <- sirt::gom.em( dat, K=3, problevels=c(0,1),  model="GOM"  )
summary(mod7d)

#---- - M7e: mirt (in mirt)
  # define three latent classes
Theta <- diag(3)
  # define mirt model
I <- ncol(dat)  # I=12
mirtmodel <- mirt::mirt.model("
        C1=1-12
        C2=1-12
        C3=1-12
        ")
  # get initial parameter values
mod.pars <- mirt::mirt(dat, model=mirtmodel,  pars="values")
  # modify parameters: only slopes refer to item-class probabilities
set.seed(9976)
  # set starting values for class specific item probabilities
mod.pars[ mod.pars$name=="d","value" ]  <- 0
mod.pars[ mod.pars$name=="d","est" ]  <- FALSE
b1 <- stats::qnorm( colMeans( dat ) )
mod.pars[ mod.pars$name=="a1","value" ]  <- b1
  # random starting values for other classes
mod.pars[ mod.pars$name \%in\% c("a2","a3"),"value" ]  <- b1 + stats::runif(12*2,-1,1)
mod.pars
  #** define prior for latent class analysis
lca_prior <- function(Theta,Etable){
  # number of latent Theta classes
  TP <- nrow(Theta)
  # prior in initial iteration
  if ( is.null(Etable) ){
    prior <- rep( 1/TP, TP )
  }
  # process Etable (this is correct for datasets without missing data)
  if ( ! is.null(Etable) ){
    # sum over correct and incorrect expected responses
    prior <- ( rowSums(Etable[, seq(1,2*I,2)]) + rowSums(Etable[,seq(2,2*I,2)]) )/I
  }
  prior <- prior / sum(prior)
  return(prior)
}
  #** estimate model
mod7e <- mirt::mirt(dat, mirtmodel, pars=mod.pars, verbose=TRUE,
            technical=list( customTheta=Theta, customPriorFun=lca_prior) )
  # compare estimated results
print(mod7e)
summary(mod7b)
  # The number of estimated parameters is incorrect because mirt does not correctly count
  # estimated parameters from the user customized  prior distribution.
mod7e@nest <- as.integer(sum(mod.pars$est) + 2)  # two additional class probabilities
  # extract log-likelihood
mod7e@logLik
  # compute AIC and BIC
( AIC <- -2*mod7e@logLik+2*mod7e@nest )
( BIC <- -2*mod7e@logLik+log(mod7e@Data$N)*mod7e@nest )
  # RMSEA and SRMSR fit statistic
mirt::M2(mod7e)     # TLI and CFI does not make sense in this example
  #** extract item parameters
sirt::mirt.wrapper.coef(mod7e)
  #** extract class-specific item-probabilities
probs <- apply( coef1[, c("a1","a2","a3") ], 2, stats::plogis )
matplot( probs, type="l", xlab="Item", main="mirt::mirt")
  #** inspect estimated distribution
mod7e@Theta
mod7e@Prior[[1]]

#*****************************************************
# Model 8: Mixed Rasch model with two classes
#*****************************************************

#----  M8a: raschmix (in psychomix)
mod8a <- psychomix::raschmix(data=as.matrix(dat), k=2, scores="saturated")
summary(mod8a)

#----  M8b: mrm (in mRm)
mod8b <- mRm::mrm(data.matrix=dat, cl=2)
mod8b$conv.to.bound
plot(mod8b)
print(mod8b)

#----  M8c: mirt (in mirt)
  #* define theta grid
theta.k <- seq( -5, 5, len=9 )
TP <- length(theta.k)
Theta <- matrix( 0, nrow=2*TP, ncol=4)
Theta[1:TP,1:2] <- cbind(theta.k, 1 )
Theta[1:TP + TP,3:4] <- cbind(theta.k, 1 )
Theta
  # define model
I <- ncol(dat)  # I=12
mirtmodel <- mirt::mirt.model("
        F1a=1-12  # slope Class 1
        F1b=1-12  # difficulty Class 1
        F2a=1-12  # slope Class 2
        F2b=1-12  # difficulty Class 2
        CONSTRAIN=(1-12,a1),(1-12,a3)
        ")
  # get initial parameter values
mod.pars <- mirt::mirt(dat, model=mirtmodel,  pars="values")
  # set starting values for class specific item probabilities
mod.pars[ mod.pars$name=="d","value" ]  <- 0
mod.pars[ mod.pars$name=="d","est" ]  <- FALSE
mod.pars[ mod.pars$name=="a1","value" ]  <- 1
mod.pars[ mod.pars$name=="a3","value" ]  <- 1
  # initial values difficulties
b1 <-  stats::qlogis( colMeans(dat) )
mod.pars[ mod.pars$name=="a2","value" ]  <- b1
mod.pars[ mod.pars$name=="a4","value" ]  <- b1 + stats::runif(I, -1, 1)
  #* define prior for mixed Rasch analysis
mixed_prior <- function(Theta,Etable){
  NC <- 2   # number of theta classes
  TP <- nrow(Theta) / NC
  prior1 <- stats::dnorm( Theta[1:TP,1] )
  prior1 <- prior1 / sum(prior1)
  if ( is.null(Etable) ){   prior <- c( prior1, prior1 ) }
  if ( ! is.null(Etable) ){
    prior <- ( rowSums( Etable[, seq(1,2*I,2)] ) +
                   rowSums( Etable[,seq(2,2*I,2)]) )/I
    a1 <- stats::aggregate( prior, list( rep(1:NC, each=TP) ), sum )
    a1[,2] <- a1[,2] / sum( a1[,2])
    # print some information during estimation
    cat( paste0( " Class proportions: ",
              paste0( round(a1[,2], 3 ), collapse=" " ) ), "\n")
    a1 <- rep( a1[,2], each=TP )
    # specify mixture of two normal distributions
    prior <- a1*c(prior1,prior1)
  }
  prior <- prior / sum(prior)
  return(prior)
}
  #* estimate model
mod8c <- mirt::mirt(dat, mirtmodel, pars=mod.pars, verbose=TRUE,
        technical=list(  customTheta=Theta, customPriorFun=mixed_prior ) )
  # Like in Model 7e, the number of estimated parameters must be included.
mod8c@nest <- as.integer(sum(mod.pars$est) + 1)
      # two class proportions and therefore one probability is freely estimated.
  #* extract item parameters
sirt::mirt.wrapper.coef(mod8c)
  #* estimated distribution
mod8c@Theta
mod8c@Prior

#----  M8d: tamaan (in TAM)

tammodel <- "
ANALYSIS:
  TYPE=MIXTURE ;
  NCLASSES(2);
  NSTARTS(7,20);
LAVAAN MODEL:
  F=~ A1__C4
  F ~~ F
ITEM TYPE:
  ALL(Rasch);
    "
mod8d <- TAM::tamaan( tammodel, resp=dat )
summary(mod8d)
# plot item parameters
I <- 12
ipars <- mod8d$itempartable_MIXTURE[ 1:I, ]
plot( 1:I, ipars[,3], type="o", ylim=range( ipars[,3:4] ), pch=16,
        xlab="Item", ylab="Item difficulty")
lines( 1:I, ipars[,4], type="l", col=2, lty=2)
points( 1:I, ipars[,4],  col=2, pch=2)

#*****************************************************
# Model 9: Mixed 2PL model with two classes
#*****************************************************

#----  M9a: tamaan (in TAM)

tammodel <- "
ANALYSIS:
  TYPE=MIXTURE ;
  NCLASSES(2);
  NSTARTS(10,30);
LAVAAN MODEL:
  F=~ A1__C4
  F ~~ F
ITEM TYPE:
  ALL(2PL);
    "
mod9a <- TAM::tamaan( tammodel, resp=dat )
summary(mod9a)

#*****************************************************
# Model 10: Rasch testlet model
#*****************************************************

#----  M10a: tam.fa (in TAM)
dims <- substring( colnames(dat),1,1 )  # define dimensions
mod10a <- TAM::tam.fa( resp=dat, irtmodel="bifactor1", dims=dims,
                control=list(maxiter=60) )
summary(mod10a)

#----  M10b: mirt (in mirt)
cmodel <- mirt::mirt.model("
        G=1-12
        A=1-4
        B=5-8
        C=9-12
        CONSTRAIN=(1-12,a1), (1-4, a2), (5-8, a3), (9-12,a4)
      ")
mod10b <- mirt::mirt(dat, model=cmodel, verbose=TRUE)
summary(mod10b)
coef(mod10b)
mod10b@logLik   # equivalent is slot( mod10b, "logLik")

#alternatively, using a dimensional reduction approach (faster and better accuracy)
cmodel <- mirt::mirt.model("
      G=1-12
      CONSTRAIN=(1-12,a1), (1-4, a2), (5-8, a3), (9-12,a4)
     ")
item_bundles <- rep(c(1,2,3), each=4)
mod10b1 <- mirt::bfactor(dat, model=item_bundles, model2=cmodel, verbose=TRUE)
coef(mod10b1)

#----  M10c: smirt (in sirt)
  # define Q-matrix
Qmatrix <- matrix(0,12,4)
Qmatrix[,1] <- 1
Qmatrix[ cbind( 1:12, match( dims, unique(dims)) +1 ) ]  <- 1
  # uncorrelated factors
variance.fixed <- cbind( c(1,1,1,2,2,3), c(2,3,4,3,4,4), 0 )
  # estimate model
mod10c <- sirt::smirt( dat, Qmatrix=Qmatrix, irtmodel="comp",
              variance.fixed=variance.fixed, qmcnodes=1000, maxiter=60)
summary(mod10c)

#*****************************************************
# Model 11: Bifactor model
#*****************************************************

#----  M11a: tam.fa (in TAM)
dims <- substring( colnames(dat),1,1 )  # define dimensions
mod11a <- TAM::tam.fa( resp=dat, irtmodel="bifactor2", dims=dims,
                 control=list(maxiter=60) )
summary(mod11a)

#----  M11b: bfactor (in mirt)
dims1 <- match( dims, unique(dims) )
mod11b <- mirt::bfactor(dat, model=dims1, verbose=TRUE)
summary(mod11b)
coef(mod11b)
mod11b@logLik

#----  M11c: smirt (in sirt)
  # define Q-matrix
Qmatrix <- matrix(0,12,4)
Qmatrix[,1] <- 1
Qmatrix[ cbind( 1:12, match( dims, unique(dims)) +1 ) ]  <- 1
  # uncorrelated factors
variance.fixed <- cbind( c(1,1,1,2,2,3), c(2,3,4,3,4,4), 0 )
  # estimate model
mod11c <- sirt::smirt( dat, Qmatrix=Qmatrix, irtmodel="comp", est.a="2PL",
                variance.fixed=variance.fixed, qmcnodes=1000, maxiter=60)
summary(mod11c)

#*****************************************************
# Model 12: Located latent class model: Rasch model with three theta classes
#*****************************************************

# use 10th item as the reference item
ref.item <- 10
# ability grid
theta.k <- seq(-4,4,len=9)

#----  M12a: rasch.mirtlc (in sirt)
mod12a <- sirt::rasch.mirtlc(dat, Nclasses=3, modeltype="MLC1", ref.item=ref.item)
summary(mod12a)

#----  M12b: gdm (in CDM)
theta.k <- seq(-1, 1, len=3)      # initial matrix
b.constraint <- matrix( c(10,1,0), nrow=1,ncol=3)
  # estimate model
mod12b <- CDM::gdm( dat, theta.k=theta.k, skillspace="est", irtmodel="1PL",
              b.constraint=b.constraint, maxiter=200)
summary(mod12b)

#----  M12c: mirt (in mirt)
items <- colnames(dat)
  # define three latent classes
Theta <- diag(3)
  # define mirt model
I <- ncol(dat)  # I=12
mirtmodel <- mirt::mirt.model("
        C1=1-12
        C2=1-12
        C3=1-12
        CONSTRAIN=(1-12,a1),(1-12,a2),(1-12,a3)
        ")
  # get parameters
mod.pars <- mirt(dat, model=mirtmodel,  pars="values")
 # set starting values for class specific item probabilities
mod.pars[ mod.pars$name=="d","value" ]  <- stats::qlogis( colMeans(dat,na.rm=TRUE) )
  # set item difficulty of reference item to zero
ind <- which( ( paste(mod.pars$item)==items[ref.item] ) &
               ( ( paste(mod.pars$name)=="d" ) ) )
mod.pars[ ind,"value" ]  <- 0
mod.pars[ ind,"est" ]  <- FALSE
  # initial values for a1, a2 and a3
mod.pars[ mod.pars$name \%in\% c("a1","a2","a3"),"value" ]  <- c(-1,0,1)
mod.pars
  #* define prior for latent class analysis
lca_prior <- function(Theta,Etable){
  # number of latent Theta classes
  TP <- nrow(Theta)
  # prior in initial iteration
  if ( is.null(Etable) ){
    prior <- rep( 1/TP, TP )
              }
  # process Etable (this is correct for datasets without missing data)
  if ( ! is.null(Etable) ){
    # sum over correct and incorrect expected responses
    prior <- ( rowSums( Etable[, seq(1,2*I,2)] ) + rowSums( Etable[, seq(2,2*I,2)] ) )/I
            }
  prior <- prior / sum(prior)
  return(prior)
   }
 #* estimate model
mod12c <- mirt(dat, mirtmodel, technical=list(
            customTheta=Theta, customPriorFun=lca_prior),
            pars=mod.pars, verbose=TRUE )
  # estimated parameters from the user customized  prior distribution.
mod12c@nest <- as.integer(sum(mod.pars$est) + 2)
  #* extract item parameters
coef1 <- sirt::mirt.wrapper.coef(mod12c)
  #* inspect estimated distribution
mod12c@Theta
coef1$coef[1,c("a1","a2","a3")]
mod12c@Prior[[1]]

#*****************************************************
# Model 13: Multidimensional model with discrete traits
#*****************************************************
# define Q-Matrix
Q <- matrix( 0, nrow=12,ncol=3)
Q[1:4,1] <- 1
Q[5:8,2] <- 1
Q[9:12,3] <- 1
# define discrete theta distribution with 3 dimensions
Theta <- scan(what="character",nlines=1)
  000 100 010 001 110 101 011 111
Theta <- as.numeric( unlist( lapply( Theta, strsplit, split="")   ) )
Theta <- matrix(Theta, 8, 3, byrow=TRUE )
Theta

#----  Model 13a: din (in CDM)
mod13a <- CDM::din( dat, q.matrix=Q, rule="DINA")
summary(mod13a)
# compare used Theta distributions
cbind( Theta, mod13a$attribute.patt.splitted)

#----  Model 13b: gdm (in CDM)
mod13b <- CDM::gdm( dat, Qmatrix=Q, theta.k=Theta, skillspace="full")
summary(mod13b)

#----  Model 13c: mirt (in mirt)
  # define mirt model
I <- ncol(dat)  # I=12
mirtmodel <- mirt::mirt.model("
        F1=1-4
        F2=5-8
        F3=9-12
        ")
  # get parameters
mod.pars <- mirt(dat, model=mirtmodel,  pars="values")
# starting values d parameters (transformed guessing parameters)
ind <- which(  mod.pars$name=="d"  )
mod.pars[ind,"value"] <- stats::qlogis(.2)
# starting values transformed slipping parameters
ind <- which( ( mod.pars$name \%in\% paste0("a",1:3)  ) &  ( mod.pars$est ) )
mod.pars[ind,"value"] <- stats::qlogis(.8) - stats::qlogis(.2)
mod.pars

  #* define prior for latent class analysis
lca_prior <- function(Theta,Etable){
  TP <- nrow(Theta)
  if ( is.null(Etable) ){
    prior <- rep( 1/TP, TP )
              }
  if ( ! is.null(Etable) ){
    prior <- ( rowSums( Etable[, seq(1,2*I,2)] ) + rowSums( Etable[, seq(2,2*I,2)] ) )/I
            }
  prior <- prior / sum(prior)
  return(prior)
}
 #* estimate model
mod13c <- mirt(dat, mirtmodel, technical=list(
            customTheta=Theta, customPriorFun=lca_prior),
            pars=mod.pars, verbose=TRUE )
  # estimated parameters from the user customized  prior distribution.
mod13c@nest <- as.integer(sum(mod.pars$est) + 2)
  #* extract item parameters
coef13c <- sirt::mirt.wrapper.coef(mod13c)$coef
  #* inspect estimated distribution
mod13c@Theta
mod13c@Prior[[1]]

 #-* comparisons of estimated  parameters
# extract guessing and slipping parameters from din
dfr <- coef(mod13a)[, c("guess","slip") ]
colnames(dfr) <- paste0("din.",c("guess","slip") )
# estimated parameters from gdm
dfr$gdm.guess <- stats::plogis(mod13b$item$b)
dfr$gdm.slip <- 1 - stats::plogis( rowSums(mod13b$item[,c("b.Cat1","a.F1","a.F2","a.F3")] ) )
# estimated parameters from mirt
dfr$mirt.guess <- stats::plogis( coef13c$d )
dfr$mirt.slip <- 1 - stats::plogis( rowSums(coef13c[,c("d","a1","a2","a3")]) )
# comparison
round(dfr[, c(1,3,5,2,4,6)],3)
  ##      din.guess gdm.guess mirt.guess din.slip gdm.slip mirt.slip
  ##   A1     0.691     0.684      0.686    0.000    0.000     0.000
  ##   A2     0.491     0.489      0.489    0.031    0.038     0.036
  ##   A3     0.302     0.300      0.300    0.184    0.193     0.190
  ##   A4     0.244     0.239      0.240    0.337    0.340     0.339
  ##   B1     0.568     0.579      0.577    0.163    0.148     0.151
  ##   B2     0.329     0.344      0.340    0.344    0.326     0.329
  ##   B3     0.817     0.827      0.825    0.014    0.007     0.009
  ##   B4     0.431     0.463      0.456    0.104    0.089     0.092
  ##   C1     0.188     0.191      0.189    0.013    0.013     0.013
  ##   C2     0.050     0.050      0.050    0.239    0.238     0.239
  ##   C3     0.000     0.002      0.001    0.065    0.065     0.065
  ##   C4     0.000     0.004      0.000    0.212    0.212     0.212

# estimated class sizes
dfr <- data.frame( "Theta"=Theta, "din"=mod13a$attribute.patt$class.prob,
                   "gdm"=mod13b$pi.k, "mirt"=mod13c@Prior[[1]])
# comparison
round(dfr,3)
  ##     Theta.1 Theta.2 Theta.3   din   gdm  mirt
  ##   1       0       0       0 0.039 0.041 0.040
  ##   2       1       0       0 0.008 0.009 0.009
  ##   3       0       1       0 0.009 0.007 0.008
  ##   4       0       0       1 0.394 0.417 0.412
  ##   5       1       1       0 0.011 0.011 0.011
  ##   6       1       0       1 0.017 0.042 0.037
  ##   7       0       1       1 0.042 0.008 0.016
  ##   8       1       1       1 0.480 0.465 0.467

#*****************************************************
# Model 14: DINA model with two skills
#*****************************************************

# define some simple Q-Matrix (does not really make in this application)
Q <- matrix( 0, nrow=12,ncol=2)
Q[1:4,1] <- 1
Q[5:8,2] <- 1
Q[9:12,1:2] <- 1
# define discrete theta distribution with 3 dimensions
Theta <- scan(what="character",nlines=1)
  00 10 01 11
Theta <- as.numeric( unlist( lapply( Theta, strsplit, split="")   ) )
Theta <- matrix(Theta, 4, 2, byrow=TRUE )
Theta

#----  Model 14a: din (in CDM)
mod14a <- CDM::din( dat, q.matrix=Q, rule="DINA")
summary(mod14a)
# compare used Theta distributions
cbind( Theta, mod14a$attribute.patt.splitted)

#----  Model 14b: mirt (in mirt)
  # define mirt model
I <- ncol(dat)  # I=12
mirtmodel <- mirt::mirt.model("
        F1=1-4
        F2=5-8
        (F1*F2)=9-12
        ")
#-> constructions like (F1*F2*F3) are also allowed in mirt.model
  # get parameters
mod.pars <- mirt(dat, model=mirtmodel,  pars="values")
# starting values d parameters (transformed guessing parameters)
ind <- which(  mod.pars$name=="d"  )
mod.pars[ind,"value"] <- stats::qlogis(.2)
# starting values transformed slipping parameters
ind <- which( ( mod.pars$name \%in\% paste0("a",1:3)  ) &  ( mod.pars$est ) )
mod.pars[ind,"value"] <- stats::qlogis(.8) - stats::qlogis(.2)
mod.pars
 #* use above defined prior lca_prior
 # lca_prior <- function(prior,Etable) ...
 #* estimate model
mod14b <- mirt(dat, mirtmodel, technical=list(
            customTheta=Theta, customPriorFun=lca_prior),
            pars=mod.pars, verbose=TRUE )
  # estimated parameters from the user customized  prior distribution.
mod14b@nest <- as.integer(sum(mod.pars$est) + 2)
  #* extract item parameters
coef14b <- sirt::mirt.wrapper.coef(mod14b)$coef

 #-* comparisons of estimated  parameters
# extract guessing and slipping parameters from din
dfr <- coef(mod14a)[, c("guess","slip") ]
colnames(dfr) <- paste0("din.",c("guess","slip") )
# estimated parameters from mirt
dfr$mirt.guess <- stats::plogis( coef14b$d )
dfr$mirt.slip <- 1 - stats::plogis( rowSums(coef14b[,c("d","a1","a2","a3")]) )
# comparison
round(dfr[, c(1,3,2,4)],3)
  ##      din.guess mirt.guess din.slip mirt.slip
  ##   A1     0.674      0.671    0.030     0.030
  ##   A2     0.423      0.420    0.049     0.050
  ##   A3     0.258      0.255    0.224     0.225
  ##   A4     0.245      0.243    0.394     0.395
  ##   B1     0.534      0.543    0.166     0.164
  ##   B2     0.338      0.347    0.382     0.380
  ##   B3     0.796      0.802    0.016     0.015
  ##   B4     0.421      0.436    0.142     0.140
  ##   C1     0.850      0.851    0.000     0.000
  ##   C2     0.480      0.480    0.097     0.097
  ##   C3     0.746      0.746    0.026     0.026
  ##   C4     0.575      0.577    0.136     0.137

# estimated class sizes
dfr <- data.frame( "Theta"=Theta, "din"=mod13a$attribute.patt$class.prob,
                    "mirt"=mod14b@Prior[[1]])
# comparison
round(dfr,3)
  ##     Theta.1 Theta.2   din  mirt
  ##   1       0       0 0.357 0.369
  ##   2       1       0 0.044 0.049
  ##   3       0       1 0.047 0.031
  ##   4       1       1 0.553 0.551

#*****************************************************
# Model 15: Rasch model with non-normal distribution
#*****************************************************

# A non-normal theta distributed is specified by log-linear smoothing
# the distribution as described in
# Xu, X., & von Davier, M. (2008). Fitting the structured general diagnostic model
# to NAEP data. ETS Research Report ETS RR-08-27. Princeton, ETS.

# define theta grid
theta.k <- matrix( seq(-4,4,len=15), ncol=1 )
# define design matrix for smoothing (up to cubic moments)
delta.designmatrix <- cbind( 1, theta.k, theta.k^2, theta.k^3 )
# constrain item difficulty of fifth item (item B1) to zero
b.constraint <- matrix( c(5,1,0), ncol=3 )

#----  Model 15a: gdm (in CDM)
mod15a <- CDM::gdm( dat, irtmodel="1PL", theta.k=theta.k,
               b.constraint=b.constraint  )
summary(mod15a)
 # plot estimated distribution
graphics::barplot( mod15a$pi.k[,1], space=0, names.arg=round(theta.k[,1],2),
           main="Estimated Skewed Distribution (gdm function)")

#----  Model 15b: mirt (in mirt)
 # define mirt model
mirtmodel <- mirt::mirt.model("
    F=1-12
    ")
 # get parameters
mod.pars <- mirt::mirt(dat, model=mirtmodel, pars="values", itemtype="Rasch")
  # fix variance (just for correct counting of parameters)
mod.pars[ mod.pars$name=="COV_11", "est"] <- FALSE
  # fix item difficulty
ind <- which( ( mod.pars$item=="B1" ) & ( mod.pars$name=="d" ) )
mod.pars[ ind, "value"] <- 0
mod.pars[ ind, "est"] <- FALSE

 # define prior
loglinear_prior <- function(Theta,Etable){
    TP <- nrow(Theta)
    if ( is.null(Etable) ){
    prior <- rep( 1/TP, TP )
           }
    # process Etable (this is correct for datasets without missing data)
    if ( ! is.null(Etable) ){
          # sum over correct and incorrect expected responses
       prior <- ( rowSums( Etable[, seq(1,2*I,2)] ) + rowSums( Etable[, seq(2,2*I,2)] ) )/I
       # smooth prior using the above design matrix and a log-linear model
       # see Xu & von Davier (2008).
       y <- log( prior + 1E-15 )
       lm1 <- lm( y ~ 0 + delta.designmatrix, weights=prior )
       prior <- exp(fitted(lm1))   # smoothed prior
           }
    prior <- prior / sum(prior)
    return(prior)
}

#* estimate model
mod15b <- mirt::mirt(dat, mirtmodel, technical=list(
                customTheta=theta.k, customPriorFun=loglinear_prior ),
                pars=mod.pars, verbose=TRUE )
# estimated parameters from the user customized prior distribution.
mod15b@nest <- as.integer(sum(mod.pars$est) + 3)
#* extract item parameters
coef1 <- sirt::mirt.wrapper.coef(mod15b)$coef

#** compare estimated item parameters
dfr <- data.frame( "gdm"=mod15a$item$b.Cat1, "mirt"=coef1$d )
rownames(dfr) <- colnames(dat)
round(t(dfr),4)
  ##            A1     A2      A3      A4 B1      B2     B3     B4     C1    C2     C3    C4
  ##   gdm  0.9818 0.1538 -0.7837 -1.3197  0 -1.0902 1.6088 -0.170 1.9778 0.006 1.1859 0.135
  ##   mirt 0.9829 0.1548 -0.7826 -1.3186  0 -1.0892 1.6099 -0.169 1.9790 0.007 1.1870 0.136
# compare estimated theta distribution
dfr <- data.frame( "gdm"=mod15a$pi.k, "mirt"=mod15b@Prior[[1]] )
round(t(dfr),4)
  ##        1 2     3     4      5      6      7      8      9     10     11     12     13
  ##   gdm  0 0 1e-04 9e-04 0.0056 0.0231 0.0652 0.1299 0.1881 0.2038 0.1702 0.1129 0.0612
  ##   mirt 0 0 1e-04 9e-04 0.0056 0.0232 0.0653 0.1300 0.1881 0.2038 0.1702 0.1128 0.0611
  ##            14    15
  ##   gdm  0.0279 0.011
  ##   mirt 0.0278 0.011
}
}

%% \keyword{datasets}

%% File Name: invariance.alignment.Rd
%% File Version: 1.712

\name{invariance.alignment}
\alias{invariance.alignment}
\alias{summary.invariance.alignment}
%% \alias{plot.invariance.alignment}
\alias{invariance_alignment_constraints}
\alias{summary.invariance_alignment_constraints}
\alias{invariance_alignment_simulate}
\alias{invariance_alignment_cfa_config}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Alignment Procedure for Linking under Approximate Invariance
}
\description{
The function \code{invariance.alignment} performs alignment under approximate
invariance for \eqn{G} groups and \eqn{I} items
(Asparouhov & Muthen, 2014; Byrne & van de Vijver, 2017; DeMars, 2020; Finch, 2016;
Fischer & Karl, 2019; Flake & McCoach, 2018; Kim et al., 2017; Marsh et al., 2018;
Muthen & Asparouhov, 2014, 2018; Pokropek, Davidov & Schmidt, 2019).
It is assumed that item loadings and intercepts are
previously estimated as a unidimensional factor model under the assumption of a factor
with zero mean and a variance of one.

The function \code{invariance_alignment_constraints} postprocesses the output of the
\code{invariance.alignment} function and estimates item parameters under equality
constraints for prespecified absolute values of parameter tolerance.

The function \code{invariance_alignment_simulate} simulates a one-factor model
for multiple groups for given matrices of \eqn{\nu} and \eqn{\lambda} parameters of
item intercepts and item slopes (see Example 6).

The function \code{invariance_alignment_cfa_config} estimates one-factor
models separately for each group as a preliminary step for invariance
alignment (see Example 6). Sampling weights are accommodated by the
argument \code{weights}.  The computed variance matrix \code{vcov} by this function
can be used to obtain standard errors in the \code{invariance.alignment} function
if it is supplied as the argument \code{vcov}.
}

%% \emph{Note that this function is in an early experimental stage.}


\usage{
invariance.alignment(lambda, nu, wgt=NULL, align.scale=c(1, 1),
    align.pow=c(.5, .5), eps=1e-3, psi0.init=NULL, alpha0.init=NULL, center=FALSE,
    optimizer="optim", fixed=NULL, meth=1, vcov=NULL, eps_grid=seq(0,-10, by=-.5),
    num_deriv=FALSE, ...)

\method{summary}{invariance.alignment}(object, digits=3, file=NULL, ...)

invariance_alignment_constraints(model, lambda_parm_tol, nu_parm_tol )

\method{summary}{invariance_alignment_constraints}(object, digits=3, file=NULL, ...)

invariance_alignment_simulate(nu, lambda, err_var, mu, sigma, N, output="data",
     groupwise=FALSE, exact=FALSE)

invariance_alignment_cfa_config(dat, group, weights=NULL, model="2PM", verbose=FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{lambda}{
A \eqn{G \times I} matrix with item loadings
}
  \item{nu}{
A \eqn{G \times I} matrix with item intercepts
}
  \item{wgt}{
A \eqn{G \times I} matrix for weighing groups
for each item
}
  \item{align.scale}{
A vector of length two containing scale parameter
\eqn{a_\lambda} and \eqn{a_\nu} (see Details)
}
  \item{align.pow}{
A vector of length two containing power
\eqn{p_\lambda} and \eqn{p_\nu} (see Details)
}
  \item{eps}{
A parameter in the optimization function
}
  \item{psi0.init}{
An optional vector of initial \eqn{\psi_0} parameters
}
  \item{alpha0.init}{
An optional vector of initial \eqn{\alpha_0} parameters
}
 \item{center}{
Logical indicating whether estimated means and standard deviations should
be centered.
}
\item{optimizer}{Name of the optimizer chosen for alignment. Options are
   \code{"optim"} (using \code{\link[stats:optim]{stats::optim}})
   or \code{"nlminb"} (using \code{\link[stats:nlminb]{stats::nlminb}}).}
\item{fixed}{Logical indicating whether SD of first group should
be fixed to one. If \code{fixed=FALSE}, the product of all SDs is set to one.
If \code{NULL}, then \code{fixed} is automatically chosen by default. For many groups,
\code{fixed=FALSE} is chosen.}
\item{meth}{Type of method used for optimization function. \code{meth=1} is the default
and the optimization function used in Mplus. \code{meth=2} uses logarithmized
item loadings in alignment. The choice \code{meth=4} uses the constraint
\eqn{\prod_g \psi_g=1} and adds the penalty \eqn{\lambda \sum_g \alpha_g^2} for
a fixed value \eqn{\lambda} that depends on the weights \code{wgt}
(similar to Mplus' free method).
The choice \code{meth=3} only uses the constraint
\eqn{\prod_g \psi_g=1} (similar to Mplus' FIXED method).}
\item{vcov}{Variance matrix produced by \code{invariance_alignment_cfa_config}
for standard error computation. If a matrix is provided, standard errors
are computed.}
\item{eps_grid}{Grid of logarithmized epsilon values in optimization}
\item{num_deriv}{Logical indicating whether numerical derivatives should be used}
  \item{object}{
Object of class \code{invariance.alignment}
  }
\item{digits}{Number of digits used for rounding}
\item{file}{Optional file name in which summary should be sunk}
  \item{\dots}{
Further optional arguments to be passed}
\item{model}{Model of class \code{invariance.alignment}.
For \code{invariance_alignment_cfa_config}: Model type: \code{"2PM"} for two-parameter
model with unequal loadings
and \code{"1PM"} with equal loadings and equal residual variances}
\item{lambda_parm_tol}{Parameter tolerance for \eqn{\lambda} parameters}
\item{nu_parm_tol}{Parameter tolerance for \eqn{\nu} parameters}
\item{err_var}{Error variance}
\item{mu}{Vector of means}
\item{sigma}{Vector of standard deviations}
\item{N}{Vector of sample sizes per group}
\item{output}{Specifies output type: \code{"data"} for dataset and \code{"suffstat"}
for sufficient statistics (i.e., means and covariance matrices)}
\item{groupwise}{Logical indicating whether group-wise output is requested}
\item{exact}{Logical indicating whether distributions should be exactly preserved in
simulated data}
\item{dat}{Dataset with items or a list containing sufficient statistics}
\item{group}{Vector containing group indicators}
\item{weights}{Optional vector of sampling weights}
\item{verbose}{Logical indicating whether progress should be printed}
}

\details{
For \eqn{G} groups and \eqn{I} items, item loadings \eqn{\lambda_{ig0}}
and intercepts \eqn{\nu_{ig0}} are available and have been estimated
in a 1-dimensional factor analysis assuming a standardized factor.

The alignment procedure searches means \eqn{\alpha_{g0}}
and standard deviations \eqn{\psi_{g0}} using an alignment
optimization function \eqn{F}. This function is defined as
\deqn{F=\sum_i \sum_{ g_1 < g_2} w_{i,g1} w_{i,g2}
    f_\lambda( \lambda_{i g_1,1} - \lambda_{i g_2,1} )
    + \sum_i \sum_{ g_1 < g_2} w_{i,g1} w_{i,g2}
f_\nu( \nu_{i g_1,1} - \nu_{i g_2,1} ) }
where the aligned item parameters \eqn{\lambda_{i g,1}}
and \eqn{\nu_{i g,1}} are defined such that

\deqn{ \lambda_{i g,1}=\lambda_{i g 0} / \psi_{g0}
    \qquad \mbox{and} \qquad
    \nu_{i g,1}=\nu_{i g 0} -  \alpha_{g0} \lambda_{ig0} / \psi_{g0}
            }
and the optimization functions are defined as
\deqn{ f_\lambda (x)=| x/ a_\lambda | ^{p_\lambda}
\approx [ ( x/ a_\lambda )^2 + \varepsilon ]^{p_\lambda / 2}
    \qquad \mbox{and} \qquad
    f_\nu (x)=|  x/ a_\nu ]^{p_\nu}
    \approx [ ( x/ a_\nu )^2 + \varepsilon ]^{p_\nu / 2}
            }
using a small \eqn{ \varepsilon > 0} (e.g. .001) to obtain
a differentiable optimization function. For \eqn{p_\nu=0} or \eqn{p_\lambda=0}, the
optimization function essentially counts the number of different parameter
and mimicks a \eqn{L_0} penalty which is zero iff the argument is zero
and one otherwise. It is approximated by
\deqn{f(x)=x^2 (x^2 + \varepsilon )^{-1} }
(O'Neill & Burke, 2023).

For identification reasons, the product \eqn{\Pi_g \psi_{g0}} (\code{meth}=0,0.5)
of all group standard deviations or \eqn{\psi_1} (\code{meth}=1,2)
is set to one. The mean
\eqn{\alpha_{g0}} of the first group is set to zero (\code{meth}=0.5,1,2) or
a penalty function is added to the linking function (\code{meth}=0).

Note that Asparouhov and Muthen (2014) use \eqn{a_\lambda=a_\nu=1}
(which can be modified in \code{align.scale})
and \eqn{p_\lambda=p_\nu=0.5} (which can be modified in \code{align.pow}).
In case of \eqn{p_\lambda=2}, the penalty is approximately
\eqn{f_\lambda(x)=x^2 }, in case of \eqn{p_\lambda=0.5}
it is approximately \eqn{f_\lambda(x)=\sqrt{|x|} }. Note that \pkg{sirt} used a
different parametrization in versions up to 3.5. The \eqn{p} parameters have to be halved
for consistency with previous versions (e.g., the Asparouhov & Muthen parametrization
corresponds to \eqn{p=.25}; see also Fischer & Karl, 2019, for an application of
the previous parametrization).

Effect sizes of approximate invariance based on \eqn{R^2} have
been proposed by Asparouhov and Muthen (2014). These are
calculated separately for item loading and intercepts, resulting
in \eqn{R^2_\lambda} and \eqn{R^2_\nu} measures which are
included in the output \code{es.invariance}. In addition,
the average correlation of aligned item parameters among groups (\code{rbar})
is reported.

\emph{Metric invariance} means that all aligned item loadings \eqn{\lambda_{ig,1}}
are equal across groups and therefore \eqn{R^2_\lambda=1}.
\emph{Scalar invariance} means that all aligned item loadings
\eqn{\lambda_{ig,1}} and aligned item intercepts \eqn{\nu_{ig,1}} are
equal across groups and therefore \eqn{R^2_\lambda=1} and \eqn{R^2_\nu=1}
(see Vandenberg & Lance, 2000).
}

\value{
A list with following entries
\item{pars}{Aligned distribution parameters}
\item{itempars.aligned}{Aligned item parameters for all groups}
\item{es.invariance}{Effect sizes of approximate invariance}
\item{lambda.aligned}{Aligned \eqn{ \lambda_{i g,1}} parameters}
\item{lambda.resid}{Residuals of \eqn{ \lambda_{i g,1}} parameters}
\item{nu.aligned}{Aligned \eqn{ \nu_{i g,1}} parameters}
\item{nu.resid}{Residuals of \eqn{ \nu_{i g,1}} parameters}
\item{Niter}{Number of iterations for \eqn{f_\lambda} and
\eqn{f_\nu} optimization functions}
\item{fopt}{Minimum optimization value}
\item{align.scale}{Used alignment scale parameters}
\item{align.pow}{Used alignment power parameters}
\item{vcov}{Estimated variance matrix of aligned means and
standard deviations}
\item{\dots}{More values}
}


\references{
Asparouhov, T., & Muthen, B. (2014). Multiple-group factor analysis alignment.
\emph{Structural Equation Modeling, 21}(4), 1-14.
\doi{10.1080/10705511.2014.919210}

Byrne, B. M., & van de Vijver, F. J. R. (2017). The maximum likelihood alignment
approach to testing for approximate measurement invariance:
A paradigmatic cross-cultural application. \emph{Psicothema, 29}(4), 539-551.
\doi{10.7334/psicothema2017.178}

DeMars, C. E. (2020). Alignment as an alternative to anchor purification in DIF analyses.
\emph{Structural Equation Modeling, 27}(1), 56-72.
\doi{10.1080/10705511.2019.1617151}

Finch, W. H. (2016). Detection of differential item functioning for more
than two groups: A Monte Carlo comparison of methods.
\emph{Applied Measurement in Education, 29},(1), 30-45,
\doi{10.1080/08957347.2015.1102916}

Fischer, R., & Karl, J. A. (2019). A primer to (cross-cultural) multi-group invariance
testing possibilities in R.
\emph{Frontiers in Psychology | Cultural Psychology, 10}:1507.
\doi{10.3389/fpsyg.2019.01507}

Flake, J. K., & McCoach, D. B. (2018). An investigation of the alignment method with
polytomous indicators under conditions of partial measurement invariance.
\emph{Structural Equation Modeling, 25}(1), 56-70.
\doi{10.1080/10705511.2017.1374187}

Kim, E. S., Cao, C., Wang, Y., & Nguyen, D. T. (2017). Measurement invariance testing
with many groups: A comparison of five approaches.
\emph{Structural Equation Modeling, 24}(4), 524-544.
\doi{10.1080/10705511.2017.1304822}

Marsh, H. W., Guo, J., Parker, P. D., Nagengast, B., Asparouhov, T., Muthen, B.,
& Dicke, T. (2018). What to do when scalar invariance fails: The extended alignment
method for multi-group factor analysis comparison of latent means across many groups.
\emph{Psychological Methods, 23}(3), 524-545.
doi: 10.1037/met0000113

Muthen, B., & Asparouhov, T. (2014). IRT studies of many groups: The alignment method.
\emph{Frontiers in Psychology | Quantitative Psychology and Measurement, 5}:978.
\doi{10.3389/fpsyg.2014.00978}

Muthen, B., & Asparouhov, T. (2018). Recent methods for the study of measurement
invariance with many groups: Alignment and random effects.
\emph{Sociological Methods & Research, 47}(4), 637-664.
\doi{10.1177/0049124117701488}

O'Neill, M., & Burke, K. (2023). Variable selection using a smooth information criterion
for distributional regression models. \emph{Statistics and Computing, 33}(3), 71.
\doi{10.1007/s11222-023-10204-8}

Pokropek, A., Davidov, E., & Schmidt, P. (2019). A Monte Carlo simulation study to
assess the appropriateness of traditional and newer approaches to test for
measurement invariance. \emph{Structural Equation Modeling, 26}(5), 724-744.
\doi{10.1080/10705511.2018.1561293}

Vandenberg, R. J., & Lance, C. E. (2000). A review and synthesis of the
measurement invariance literature: Suggestions, practices, and
recommendations for organizational research. \emph{Organizational Research
Methods, 3}, 4-70.
\doi{10.1177/109442810031002}s
}

%Oelker, M. R., & Tutz, G. (2017). A uniform framework for the combination of penalties in
%generalized structured models. \emph{Advances in Data Analysis and Classification, 11}(1),
%97-120.
%\doi{10.1007/s11634-015-0205-y}

%\author{
%Alexander Robitzsch
%}

%\note{
%vvv
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For IRT linking see also \code{\link{linking.haberman}} or
\code{\link[TAM:tam.linking]{TAM::tam.linking}}.

For modeling random item effects for loadings and intercepts
see \code{\link{mcmc.2pno.ml}}.
}

\examples{
#############################################################################
# EXAMPLE 1: Item parameters cultural activities
#############################################################################

data(data.activity.itempars, package="sirt")
lambda <- data.activity.itempars$lambda
nu <- data.activity.itempars$nu
Ng <-  data.activity.itempars$N
wgt <- matrix( sqrt(Ng), length(Ng), ncol(nu) )

#***
# Model 1: Alignment using a quadratic loss function
mod1 <- sirt::invariance.alignment( lambda, nu, wgt, align.pow=c(2,2) )
summary(mod1)

#****
# Model 2: Different powers for alignment
mod2 <- sirt::invariance.alignment( lambda, nu, wgt,  align.pow=c(.5,1),
              align.scale=c(.95,.95))
summary(mod2)

# compare means from Models 1 and 2
plot( mod1$pars$alpha0, mod2$pars$alpha0, pch=16,
    xlab="M (Model 1)", ylab="M (Model 2)", xlim=c(-.3,.3), ylim=c(-.3,.3) )
lines( c(-1,1), c(-1,1), col="gray")
round( cbind( mod1$pars$alpha0, mod2$pars$alpha0 ), 3 )
round( mod1$nu.resid, 3)
round( mod2$nu.resid,3 )

# L0 penalty
mod2b <- sirt::invariance.alignment( lambda, nu, wgt,  align.pow=c(0,0),
              align.scale=c(.3,.3))
summary(mod2b)

#****
# Model 3: Low powers for alignment of scale and power
# Note that setting increment.factor larger than 1 seems necessary
mod3 <- sirt::invariance.alignment( lambda, nu, wgt, align.pow=c(.5,.75),
            align.scale=c(.55,.55), psi0.init=mod1$psi0, alpha0.init=mod1$alpha0 )
summary(mod3)

# compare mean and SD estimates of Models 1 and 3
plot( mod1$pars$alpha0, mod3$pars$alpha0, pch=16)
plot( mod1$pars$psi0, mod3$pars$psi0, pch=16)

# compare residuals for Models 1 and 3
# plot lambda
plot( abs(as.vector(mod1$lambda.resid)), abs(as.vector(mod3$lambda.resid)),
      pch=16, xlab="Residuals lambda (Model 1)",
      ylab="Residuals lambda (Model 3)", xlim=c(0,.1), ylim=c(0,.1))
lines( c(-3,3),c(-3,3), col="gray")
# plot nu
plot( abs(as.vector(mod1$nu.resid)), abs(as.vector(mod3$nu.resid)),
      pch=16, xlab="Residuals nu (Model 1)", ylab="Residuals nu (Model 3)",
      xlim=c(0,.4),ylim=c(0,.4))
lines( c(-3,3),c(-3,3), col="gray")

\dontrun{
#############################################################################
# EXAMPLE 2: Comparison 4 groups | data.inv4gr
#############################################################################

data(data.inv4gr)
dat <- data.inv4gr
miceadds::library_install("semTools")

model1 <- "
    F=~ I01 + I02 + I03 + I04 + I05 + I06 + I07 + I08 + I09 + I10 + I11
    F ~~ 1*F
    "

res <- semTools::measurementInvariance(model1, std.lv=TRUE, data=dat, group="group")
  ##   Measurement invariance tests:
  ##
  ##   Model 1: configural invariance:
  ##       chisq        df    pvalue       cfi     rmsea       bic
  ##     162.084   176.000     0.766     1.000     0.000 95428.025
  ##
  ##   Model 2: weak invariance (equal loadings):
  ##       chisq        df    pvalue       cfi     rmsea       bic
  ##     519.598   209.000     0.000     0.973     0.039 95511.835
  ##
  ##   [Model 1 versus model 2]
  ##     delta.chisq      delta.df delta.p.value     delta.cfi
  ##         357.514        33.000         0.000         0.027
  ##
  ##   Model 3: strong invariance (equal loadings + intercepts):
  ##       chisq        df    pvalue       cfi     rmsea       bic
  ##    2197.260   239.000     0.000     0.828     0.091 96940.676
  ##
  ##   [Model 1 versus model 3]
  ##     delta.chisq      delta.df delta.p.value     delta.cfi
  ##        2035.176        63.000         0.000         0.172
  ##
  ##   [Model 2 versus model 3]
  ##     delta.chisq      delta.df delta.p.value     delta.cfi
  ##        1677.662        30.000         0.000         0.144
  ##

# extract item parameters separate group analyses
ipars <- lavaan::parameterEstimates(res$fit.configural)
# extract lambda's: groups are in rows, items in columns
lambda <- matrix( ipars[ ipars$op=="=~", "est"], nrow=4,  byrow=TRUE)
colnames(lambda) <- colnames(dat)[-1]
# extract nu's
nu <- matrix( ipars[ ipars$op=="~1"  & ipars$se !=0, "est" ], nrow=4,  byrow=TRUE)
colnames(nu) <- colnames(dat)[-1]

# Model 1: least squares optimization
mod1 <- sirt::invariance.alignment( lambda=lambda, nu=nu )
summary(mod1)
  ##   Effect Sizes of Approximate Invariance
  ##          loadings intercepts
  ##   R2       0.9826     0.9972
  ##   sqrtU2   0.1319     0.0526
  ##   rbar     0.6237     0.7821
  ##   -----------------------------------------------------------------
  ##   Group Means and Standard Deviations
  ##     alpha0  psi0
  ##   1  0.000 0.965
  ##   2 -0.105 1.098
  ##   3 -0.081 1.011
  ##   4  0.171 0.935

# Model 2: sparse target function
mod2 <- sirt::invariance.alignment( lambda=lambda, nu=nu, align.pow=c(.5,.5) )
summary(mod2)
  ##   Effect Sizes of Approximate Invariance
  ##          loadings intercepts
  ##   R2       0.9824     0.9972
  ##   sqrtU2   0.1327     0.0529
  ##   rbar     0.6237     0.7856
  ##   -----------------------------------------------------------------
  ##   Group Means and Standard Deviations
  ##     alpha0  psi0
  ##   1 -0.002 0.965
  ##   2 -0.107 1.098
  ##   3 -0.083 1.011
  ##   4  0.170 0.935

#############################################################################
# EXAMPLE 3: European Social Survey data.ess2005
#############################################################################

data(data.ess2005)
lambda <- data.ess2005$lambda
nu <- data.ess2005$nu

# Model 1: least squares optimization
mod1 <- sirt::invariance.alignment( lambda=lambda, nu=nu, align.pow=c(2,2) )
summary(mod1)

# Model 2: sparse target function and definition of scales
mod2 <- sirt::invariance.alignment( lambda=lambda, nu=nu, control=list(trace=2) )
summary(mod2)

#############################################################################
# EXAMPLE 4: Linking with item parameters containing outliers
#############################################################################

# see Help file in linking.robust

# simulate some item difficulties in the Rasch model
I <- 38
set.seed(18785)
itempars <- data.frame("item"=paste0("I",1:I) )
itempars$study1 <- stats::rnorm( I, mean=.3, sd=1.4 )
# simulate DIF effects plus some outliers
bdif <- stats::rnorm(I, mean=.4, sd=.09) +
             (stats::runif(I)>.9 )*rep( 1*c(-1,1)+.4, each=I/2 )
itempars$study2 <- itempars$study1 + bdif
# create input for function invariance.alignment
nu <- t( itempars[,2:3] )
colnames(nu) <- itempars$item
lambda <- 1+0*nu

# linking using least squares optimization
mod1 <- sirt::invariance.alignment( lambda=lambda, nu=nu )
summary(mod1)
  ##   Group Means and Standard Deviations
  ##          alpha0 psi0
  ##   study1 -0.286    1
  ##   study2  0.286    1

# linking using powers of .5
mod2 <- sirt::invariance.alignment( lambda=lambda, nu=nu, align.pow=c(1,1) )
summary(mod2)
  ##   Group Means and Standard Deviations
  ##          alpha0 psi0
  ##   study1 -0.213    1
  ##   study2  0.213    1

# linking using powers of .25
mod3 <- sirt::invariance.alignment( lambda=lambda, nu=nu, align.pow=c(.5,.5) )
summary(mod3)
  ##   Group Means and Standard Deviations
  ##          alpha0 psi0
  ##   study1 -0.207    1
  ##   study2  0.207    1

#############################################################################
# EXAMPLE 5: Linking gender groups with data.math
#############################################################################

data(data.math)
dat <- data.math$data
dat.male <- dat[ dat$female==0, substring( colnames(dat),1,1)=="M"  ]
dat.female <- dat[ dat$female==1, substring( colnames(dat),1,1)=="M"  ]

#*************************
# Model 1: Linking using the Rasch model
mod1m <- sirt::rasch.mml2( dat.male )
mod1f <- sirt::rasch.mml2( dat.female )

# create objects for invariance.alignment
nu <- rbind( mod1m$item$thresh, mod1f$item$thresh )
colnames(nu) <- mod1m$item$item
rownames(nu) <- c("male", "female")
lambda <- 1+0*nu

# mean of item difficulties
round( rowMeans(nu), 3 )

# Linking using least squares optimization
res1a <- sirt::invariance.alignment( lambda, nu, align.scale=c( .3, .5 ) )
summary(res1a)

# Linking using optimization with absolute value function (pow=.5)
res1b <- sirt::invariance.alignment( lambda, nu, align.scale=c( .3, .5 ),
                align.pow=c(1,1) )
summary(res1b)

#-- compare results with Haberman linking
I <- ncol(dat.male)
itempartable <- data.frame( "study"=rep( c("male", "female"), each=I ) )
itempartable$item <- c( paste0(mod1m$item$item),  paste0(mod1f$item$item) )
itempartable$a <- 1
itempartable$b <- c( mod1m$item$b, mod1f$item$b )
# estimate linking parameters
res1c <- sirt::linking.haberman( itempars=itempartable )

#-- results of sirt::equating.rasch
x <- itempartable[ 1:I, c("item", "b") ]
y <- itempartable[ I + 1:I, c("item", "b") ]
res1d <- sirt::equating.rasch( x, y )
round( res1d$B.est, 3 )
  ##     Mean.Mean Haebara Stocking.Lord
  ##   1     0.032   0.032         0.029

#*************************
# Model 2: Linking using the 2PL model
I <- ncol(dat.male)
mod2m <- sirt::rasch.mml2( dat.male, est.a=1:I)
mod2f <- sirt::rasch.mml2( dat.female, est.a=1:I)

# create objects for invariance.alignment
nu <- rbind( mod2m$item$thresh, mod2f$item$thresh )
colnames(nu) <- mod2m$item$item
rownames(nu) <- c("male", "female")
lambda <- rbind( mod2m$item$a, mod2f$item$a )
colnames(lambda) <- mod2m$item$item
rownames(lambda) <- c("male", "female")

res2a <- sirt::invariance.alignment( lambda, nu, align.scale=c( .3, .5 ) )
summary(res2a)

res2b <- sirt::invariance.alignment( lambda, nu, align.scale=c( .3, .5 ),
                align.pow=c(1,1) )
summary(res2b)

# compare results with Haberman linking
I <- ncol(dat.male)
itempartable <- data.frame( "study"=rep( c("male", "female"), each=I ) )
itempartable$item <- c( paste0(mod2m$item$item),  paste0(mod2f$item$item ) )
itempartable$a <- c( mod2m$item$a, mod2f$item$a )
itempartable$b <- c( mod2m$item$b, mod2f$item$b )
# estimate linking parameters
res2c <- sirt::linking.haberman( itempars=itempartable )

#############################################################################
# EXAMPLE 6: Data from Asparouhov & Muthen (2014) simulation study
#############################################################################

G <- 3  # number of groups
I <- 5  # number of items
# define lambda and nu parameters
lambda <- matrix(1, nrow=G, ncol=I)
nu <- matrix(0, nrow=G, ncol=I)

# define size of noninvariance
dif <- 1

#- 1st group: N(0,1)
lambda[1,3] <- 1+dif*.4; nu[1,5] <- dif*.5

#- 2nd group: N(0.3,1.5)
gg <- 2 ; mu <- .3; sigma <- sqrt(1.5)
lambda[gg,5] <- 1-.5*dif; nu[gg,1] <- -.5*dif
nu[gg,] <- nu[gg,] + mu*lambda[gg,]
lambda[gg,] <- lambda[gg,] * sigma

#- 3nd group: N(.8,1.2)
gg <- 3 ; mu <- .8; sigma <- sqrt(1.2)
lambda[gg,4] <- 1-.7*dif; nu[gg,2] <- -.5*dif
nu[gg,] <- nu[gg,] + mu*lambda[gg,]
lambda[gg,] <- lambda[gg,] * sigma

# define alignment scale
align.scale <- c(.2,.4)   # Asparouhov and Muthen use c(1,1)
# define alignment powers
align.pow <- c(.5,.5)   # as in Asparouhov and Muthen

#*** estimate alignment parameters
mod1 <- sirt::invariance.alignment( lambda, nu, eps=.01, optimizer="optim",
            align.scale=align.scale, align.pow=align.pow, center=FALSE )
summary(mod1)

#--- find parameter constraints for prespecified tolerance
cmod1 <- sirt::invariance_alignment_constraints(model=mod1, nu_parm_tol=.4,
            lambda_parm_tol=.2 )
summary(cmod1)

#############################################################################
# EXAMPLE 7: Similar to Example 6, but with data simulation and CFA estimation
#############################################################################

#--- data simulation

set.seed(65)
G <- 3  # number of groups
I <- 5  # number of items
# define lambda and nu parameters
lambda <- matrix(1, nrow=G, ncol=I)
nu <- matrix(0, nrow=G, ncol=I)
err_var <- matrix(1, nrow=G, ncol=I)

# define size of noninvariance
dif <- 1
#- 1st group: N(0,1)
lambda[1,3] <- 1+dif*.4; nu[1,5] <- dif*.5
#- 2nd group: N(0.3,1.5)
gg <- 2 ;
lambda[gg,5] <- 1-.5*dif; nu[gg,1] <- -.5*dif
#- 3nd group: N(.8,1.2)
gg <- 3
lambda[gg,4] <- 1-.7*dif; nu[gg,2] <- -.5*dif
#- define distributions of groups
mu <- c(0,.3,.8)
sigma <- sqrt(c(1,1.5,1.2))
N <- rep(1000,3) # sample sizes per group

#* simulate data
dat <- sirt::invariance_alignment_simulate(nu, lambda, err_var, mu, sigma, N)
head(dat)

#--- estimate CFA models
pars <- sirt::invariance_alignment_cfa_config(dat[,-1], group=dat$group)
print(pars)

#--- invariance alignment
# define alignment scale
align.scale <- c(.2,.4)
# define alignment powers
align.pow <- c(.5,.5)
mod1 <- sirt::invariance.alignment( lambda=pars$lambda, nu=pars$nu, eps=.01,
            optimizer="optim", align.scale=align.scale, align.pow=align.pow, center=FALSE)
#* find parameter constraints for prespecified tolerance
cmod1 <- sirt::invariance_alignment_constraints(model=mod1, nu_parm_tol=.4,
            lambda_parm_tol=.2 )
summary(cmod1)

#--- estimate CFA models with sampling weights

#* simulate weights
weights <- stats::runif(sum(N), 0, 2)
#* estimate models
pars2 <- sirt::invariance_alignment_cfa_config(dat[,-1], group=dat$group, weights=weights)
print(pars2$nu)
print(pars$nu)

#--- estimate one-parameter model
pars <- sirt::invariance_alignment_cfa_config(dat[,-1], group=dat$group, model="1PM")
print(pars)

#############################################################################
# EXAMPLE 8: Computation of standard errors
#############################################################################

G <- 3  # number of groups
I <- 5  # number of items
# define lambda and nu parameters
lambda <- matrix(1, nrow=G, ncol=I)
nu <- matrix(0, nrow=G, ncol=I)

# define size of noninvariance
dif <- 1

mu1 <- c(0,.3,.8)
sigma1 <- c(1,1.25,1.1)

#- 1st group
lambda[1,3] <- 1+dif*.4; nu[1,5] <- dif*.5

#- 2nd group
gg <- 2
lambda[gg,5] <- 1-.5*dif; nu[gg,1] <- -.5*dif

#- 3nd group
gg <- 3
lambda[gg,4] <- 1-.7*dif; nu[gg,2] <- -.5*dif

dat <- sirt::invariance_alignment_simulate(nu=nu, lambda=lambda, err_var=1+0*lambda,
                mu=mu1, sigma=sigma1, N=500, output="data", exact=TRUE)

#* estimate CFA
res <- sirt::invariance_alignment_cfa_config(dat=dat[,-1], group=dat$group )

#- perform invariance alignment
eps <- .001
align.pow <- 0.5*rep(1,2)
lambda <- res$lambda
nu <- res$nu
mod1 <- sirt::invariance.alignment( lambda=lambda, nu=nu, eps=eps, optimizer="optim",
             align.pow=align.pow, meth=meth, vcov=res$vcov)
# variance matrix and standard errors
mod1$vcov
sqrt(diag(mod1$vcov))

#############################################################################
# EXAMPLE 9: Comparison 2 groups for dichotomous data | data.pisaMath
#############################################################################

data(data.pisaMath)
dat <- data.pisaMath$data
library("lavaan")

model1 <- "
    F=~ M192Q01 + M406Q01 + M406Q02 + M423Q01 + M496Q01 + M496Q02 + M564Q01 +
         M564Q02 + M571Q01 + M603Q01 + M603Q02
    "

fit.configural <- lavaan::cfa(model1, data=dat, group="female",
                  ordered=TRUE, std.lv=TRUE, parameterization="theta")
lavaan::summary(fit.configural, standardized=TRUE)

# extract item parameters separate group analyses
ipars <- lavaan::parameterEstimates(fit.configural)
# extract lambda's: groups are in rows, items in columns
lambda <- matrix( ipars[ ipars$op=="=~", "est"], nrow=2,  byrow=TRUE)
colnames(lambda) <- colnames(dat)[6:16]
# extract nu's
nu <- matrix( ipars[ ipars$op=="|" & ipars$se !=0, "est" ], nrow=2,  byrow=TRUE)
colnames(nu) <- colnames(dat)[6:16]

# Model 1: apply invariance alignment
mod1 <- sirt::invariance.alignment( lambda=lambda, nu=nu )
summary(mod1)
}
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Linking}
%% \keyword{Approximate Invariance}
%% \keyword{Alignment}
%% \keyword{summary}

%% File Name: monoreg.rowwise.Rd
%% File Version: 0.14

\name{monoreg.rowwise}
\alias{monoreg.rowwise}
\alias{monoreg.colwise}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Monotone Regression for Rows or Columns in a Matrix
}

\description{
Monotone (isotone) regression for rows (\code{monoreg.rowwise}) or
columns (\code{monoreg.colwise}) in a matrix.
}

\usage{
monoreg.rowwise(yM, wM)

monoreg.colwise(yM, wM)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
\item{yM}{
Matrix with dependent variable for the regression.
Values are assumed to be sorted.
}
\item{wM}{
Matrix with weights for every entry in the \code{yM}
matrix.
}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

\value{
Matrix with fitted values
}

%\references{
%% ~put references to the literature/web site here ~
%}

\author{
Alexander Robitzsch

The \code{monoreg} function from the \pkg{fdrtool}
package is simply extended to handle matrix input.
}

\note{
This function is used for fitting the ISOP model
(see \code{\link{isop.dich}}).
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also the \code{monoreg} function from the \pkg{fdrtool}
package.
}


\examples{
y <- c(22.5, 23.33, 20.83, 24.25 )
w <- c( 3,3,3,2)
# define matrix input
yM <- matrix( 0, nrow=2, ncol=4 )
wM <- yM
yM[1,] <- yM[2,] <- y
wM[1,] <- w
wM[2,] <- c(1,3,4, 3 )

# fit rowwise monotone regression
monoreg.rowwise( yM, wM )
# compare results with monoreg function from fdrtool package
\dontrun{
miceadds::library_install("fdrtool")
fdrtool::monoreg(x=yM[1,], w=wM[1,])$yf
fdrtool::monoreg(x=yM[2,], w=wM[2,])$yf
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Monotone regression}
%% \keyword{Isotone regression}
% __ONLY ONE__ keyword per line

%% File Name: R2noharm.jackknife.Rd
%% File Version: 1.19

\name{R2noharm.jackknife}
\alias{R2noharm.jackknife}
\alias{summary.R2noharm.jackknife}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Jackknife Estimation of NOHARM Analysis
}
\description{
This function performs a jackknife estimation of NOHARM analysis
to get standard errors based on a replication method (see Christoffersson, 1977).
}
\usage{
R2noharm.jackknife(object, jackunits=NULL)

\method{summary}{R2noharm.jackknife}(object, logfile=NULL, \dots)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{
Object of class \code{R2noharm}
}
  \item{jackunits}{
A vector of integers or a number. If it is a number, then it refers
to the number of jackknife units. If it is a vector of integers, then this vector
defines the allocation of persons jackknife units. Integers corresponds to
row indexes in the data set.
}
\item{logfile}{File name if the summary should be sunk into a file}
\item{\dots}{Further arguments to be passed}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}


\value{
A list of lists with following entries:

\item{partable}{
    Data frame with parameters
        }
\item{se.pars}{List of estimated standard errors for all parameter estimates:
    \code{tanaka.stat}, \code{rmsr.stat}, \code{rmsea.stat},
    \code{chisquare_df.stat}, \code{thresholds.stat}, \code{final.constants.stat},
    \code{uniquenesses.stat}, \code{factor.cor.stat}, \code{loadings.stat},
    \code{loadings.theta.stat}
    }
\item{jackknife.pars}{List with obtained results by jackknifing for all parameters:
    \code{j.tanaka}, \code{j.rmsr}, \code{rmsea}, \code{chisquare_df},
    \code{j.pm}, \code{j.thresholds}, \code{j.factor.cor},
    \code{j.loadings}, \code{j.loadings.theta}
        }
\item{u.jacknunits}{Unique jackknife elements}
}


\references{
Christoffersson, A. (1977). Two-step weighted least squares factor analysis of
dichotomized variables. \emph{Psychometrika, 42}, 433-438.
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{R2noharm}}
}
%\examples{
%# See examples for function R2noharm
%}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{NOHARM}
%% \keyword{summary}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: R2noharm.EAP.Rd
%% File Version: 0.17

\name{R2noharm.EAP}
\alias{R2noharm.EAP}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
EAP Factor Score Estimation
}
\description{
This function performs EAP factor score estimation of
an item response model estimated with NOHARM.
}
\usage{
R2noharm.EAP(noharmobj, theta.k=seq(-6, 6, len=21), print.output=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{noharmobj}{
Object of class \code{R2noharm} or \code{noharm.sirt}
}
  \item{theta.k}{
Vector of discretized theta values on which the posterior
is evaluated. This vector applies to all dimensions.
}
\item{print.output}{An optional logical indicating whether output should
be displayed at the console}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list with following entries
  \item{person}{Data frame of person parameter EAP estimates and
  their corresponding standard errors
    }
  \item{theta}{Grid of multidimensional theta values where the posterior is
      evaluated.}
  \item{posterior}{Individual posterior distribution evaluated at \code{theta}}
  \item{like}{Individual likelihood}
  \item{EAP.rel}{EAP reliabilities of all dimensions}
  \item{probs}{Item response probabilities evaluated at \code{theta}}
}
%\references{
%% ~put references to the literature/web site here ~
%}


%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For examples see \code{\link{R2noharm}} and \code{\link{noharm.sirt}}.
}
%\examples{
%}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Factor scores}
%% \keyword{NOHARM}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: plausible.value.imputation.raschtype.Rd
%% File Version: 1.34

\name{plausible.value.imputation.raschtype}
\alias{plausible.value.imputation.raschtype}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Plausible Value Imputation in Generalized Logistic Item
Response Model
}
\description{
This function performs unidimensional plausible value imputation
(Adams & Wu, 2007; Mislevy, 1991).
}
\usage{
plausible.value.imputation.raschtype(data=NULL, f.yi.qk=NULL, X,
   Z=NULL, beta0=rep(0, ncol(X)), sig0=1, b=rep(1, ncol(X)),
   a=rep(1, length(b)), c=rep(0, length(b)), d=1+0*b,
   alpha1=0, alpha2=0, theta.list=seq(-5, 5, len=50),
   cluster=NULL, iter, burnin, nplausible=1, printprogress=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
An \eqn{N \times I} data frame of dichotomous responses
}
\item{f.yi.qk}{
An optional matrix which contains the individual likelihood.
  This matrix is produced by \code{\link{rasch.mml2}} or
  \code{\link{rasch.copula2}}. The use of this argument allows the
  estimation of the latent regression model independent of the
  parameters of the used item response model.
    }
\item{X}{
A matrix of individual covariates for the latent
regression of \eqn{\theta} on \eqn{X}
}
\item{Z}{
A matrix of individual covariates for the regression
of individual residual variances on \eqn{Z}
}
  \item{beta0}{
Initial vector of regression coefficients
}
  \item{sig0}{
Initial vector of coefficients for the
variance heterogeneity model
}
  \item{b}{
Vector of item difficulties. It must not be provided
if the individual likelihood \code{f.yi.qk} is
specified.
}
  \item{a}{
Optional vector of item slopes
}
  \item{c}{
Optional vector of lower item asymptotes
}
  \item{d}{
Optional vector of upper item asymptotes
}
  \item{alpha1}{
Parameter \eqn{\alpha_1} in generalized
item response model
}
  \item{alpha2}{
Parameter \eqn{\alpha_2} in generalized
item response model
}
  \item{theta.list}{
Vector of theta values at which the ability
distribution should be evaluated
}
  \item{cluster}{
Cluster identifier (e.g. schools or classes)
for including theta means in the plausible imputation.
}
  \item{iter}{
Number of iterations
}
\item{burnin}{
Number of burn-in iterations for plausible value imputation
}
  \item{nplausible}{
Number of plausible values
}
  \item{printprogress}{
A logical indicated whether iteration progress should be displayed
at the console.
}
}

\details{
Plausible values are drawn from the latent regression model with
heterogeneous variances:
\deqn{\theta_p=X_p \beta + \epsilon_p  \quad, \quad
\epsilon_p \sim N( 0, \sigma_p^2 ) \quad, \quad
\log( \sigma_p )=Z_p \gamma + \nu_p }
}


\value{
A list with following entries:
\item{coefs.X}{Sampled regression coefficients for covariates \eqn{X}}
\item{coefs.Z}{Sampled coefficients for modeling variance heterogeneity
for covariates \eqn{Z}}
\item{pvdraws}{Matrix with drawn plausible values}
\item{posterior}{Posterior distribution from last iteration}
\item{EAP}{Individual EAP estimate}
\item{SE.EAP}{Standard error of the EAP estimate}
\item{pv.indexes}{Index of iterations for which plausible
values were drawn}
}

\references{
Adams, R., & Wu. M. (2007). The mixed-coefficients multinomial
logit model: A generalized form of the Rasch model.
In M. von Davier & C. H. Carstensen: \emph{Multivariate and Mixture
Distribution Rasch Models: Extensions and Applications} (pp. 57-76).
New York: Springer.

Mislevy, R. J. (1991). Randomization-based inference about latent
variables from complex samples. \emph{Psychometrika, 56}, 177-196.
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For estimating the latent regression model see
\code{\link{latent.regression.em.raschtype}}.
}



\examples{
#############################################################################
# EXAMPLE 1: Rasch model with covariates
#############################################################################

set.seed(899)
I <- 21     # number of items
b <- seq(-2,2, len=I)   # item difficulties
n <- 2000       # number of students

# simulate theta and covariates
theta <- stats::rnorm( n )
x <- .7 * theta + stats::rnorm( n, .5 )
y <- .2 * x+ .3*theta + stats::rnorm( n, .4 )
dfr <- data.frame( theta, 1, x, y )

# simulate Rasch model
dat1 <- sirt::sim.raschtype( theta=theta, b=b )

# Plausible value draws
pv1 <- sirt::plausible.value.imputation.raschtype(data=dat1, X=dfr[,-1], b=b,
            nplausible=3, iter=10, burnin=5)
# estimate linear regression based on first plausible value
mod1 <- stats::lm( pv1$pvdraws[,1] ~ x+y )
summary(mod1)
  ##               Estimate Std. Error t value Pr(>|t|)
  ##   (Intercept) -0.27755    0.02121  -13.09   <2e-16 ***
  ##   x            0.40483    0.01640   24.69   <2e-16 ***
  ##   y            0.20307    0.01822   11.15   <2e-16 ***

# true regression estimate
summary( stats::lm( theta ~ x + y ) )
  ## Coefficients:
  ##             Estimate Std. Error t value Pr(>|t|)
  ## (Intercept) -0.27821    0.01984  -14.02   <2e-16 ***
  ## x            0.40747    0.01534   26.56   <2e-16 ***
  ## y            0.18189    0.01704   10.67   <2e-16 ***

\dontrun{
#############################################################################
# EXAMPLE 2: Classical test theory, homogeneous regression variance
#############################################################################

set.seed(899)
n <- 3000       # number of students
x <- round( stats::runif( n, 0,1 ) )
y <- stats::rnorm(n)
# simulate true score theta
theta <- .4*x + .5 * y + stats::rnorm(n)
# simulate observed score by adding measurement error
sig.e <- rep( sqrt(.40), n )
theta_obs <- theta + stats::rnorm( n, sd=sig.e)

# define theta grid for evaluation of density
theta.list <- mean(theta_obs) + stats::sd(theta_obs) * seq( - 5, 5, length=21)
# compute individual likelihood
f.yi.qk <- stats::dnorm( outer( theta_obs, theta.list, "-" ) / sig.e )
f.yi.qk <- f.yi.qk / rowSums(f.yi.qk)
# define covariates
X <- cbind( 1, x, y )
# draw plausible values
mod2 <- sirt::plausible.value.imputation.raschtype( f.yi.qk=f.yi.qk,
                  theta.list=theta.list, X=X, iter=10, burnin=5)

# linear regression
mod1 <- stats::lm( mod2$pvdraws[,1] ~ x+y )
summary(mod1)
  ##             Estimate Std. Error t value Pr(>|t|)
  ## (Intercept) -0.01393    0.02655  -0.525      0.6
  ## x            0.35686    0.03739   9.544   <2e-16 ***
  ## y            0.53759    0.01872  28.718   <2e-16 ***

# true regression model
summary( stats::lm( theta ~ x + y ) )
  ##             Estimate Std. Error t value Pr(>|t|)
  ## (Intercept) 0.002931   0.026171   0.112    0.911
  ## x           0.359954   0.036864   9.764   <2e-16 ***
  ## y           0.509073   0.018456  27.584   <2e-16 ***

#############################################################################
# EXAMPLE 3: Classical test theory, heterogeneous regression variance
#############################################################################

set.seed(899)
n <- 5000       # number of students
x <- round( stats::runif( n, 0,1 ) )
y <- stats::rnorm(n)
# simulate true score theta
theta <- .4*x + .5 * y + stats::rnorm(n) * ( 1 - .4 * x )
# simulate observed score by adding measurement error
sig.e <- rep( sqrt(.40), n )
theta_obs <- theta + stats::rnorm( n, sd=sig.e)

# define theta grid for evaluation of density
theta.list <- mean(theta_obs) + stats::sd(theta_obs) * seq( - 5, 5, length=21)
# compute individual likelihood
f.yi.qk <- stats::dnorm( outer( theta_obs, theta.list, "-" ) / sig.e )
f.yi.qk <- f.yi.qk / rowSums(f.yi.qk)
# define covariates
X <- cbind( 1, x, y )
# draw plausible values (assuming variance homogeneity)
mod3a <- sirt::plausible.value.imputation.raschtype( f.yi.qk=f.yi.qk,
                  theta.list=theta.list, X=X, iter=10, burnin=5)
# draw plausible values (assuming variance heterogeneity)
#  -> include predictor Z
mod3b <- sirt::plausible.value.imputation.raschtype( f.yi.qk=f.yi.qk,
                  theta.list=theta.list, X=X, Z=X, iter=10, burnin=5)

# investigate variance of theta conditional on x
res3 <- sapply( 0:1, FUN=function(vv){
        c( stats::var(theta[x==vv]), stats::var(mod3b$pvdraw[x==vv,1]),
              stats::var(mod3a$pvdraw[x==vv,1]))})
rownames(res3) <- c("true", "pv(hetero)", "pv(homog)" )
colnames(res3) <- c("x=0","x=1")
  ## > round( res3, 2 )
  ##             x=0  x=1
  ## true       1.30 0.58
  ## pv(hetero) 1.29 0.55
  ## pv(homog)  1.06 0.77
## -> assuming heteroscedastic variances recovers true conditional variance
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Plausible values}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.timss07.G8.RUS.Rd
%% File Version: 0.07

\name{data.timss07.G8.RUS}
\alias{data.timss07.G8.RUS}
\docType{data}

\title{
TIMSS 2007 Grade 8 Mathematics and Science Russia
}

\description{
This TIMSS 2007 dataset contains item responses of 4472 eigth grade
Russian students in Mathematics and Science.
}


\usage{
data(data.timss07.G8.RUS)
}


\format{
The datasets contains raw responses (\code{raw}), scored responses
(\code{scored}) and item informations (\code{iteminfo}).

The format of the dataset is:

\code{List of 3} \cr
\code{ $ raw     :'data.frame':} \cr
\code{  ..$ idstud  : num [1:4472] 3010101 3010102 3010104 3010105 3010106 ...} \cr
\code{  ..$ M022043 : atomic [1:4472] NA 1 4 NA NA NA NA NA NA NA ...} \cr
\code{  .. ..- attr(*, "value.labels")=Named num [1:7] 9 6 5 4 3 2 1} \cr
\code{  .. .. ..- attr(*, "names")=chr [1:7] "OMITTED" "NOT REACHED" "E" "D*" ...} \cr
\code{  [...]} \cr
\code{  ..$ M032698 : atomic [1:4472] NA NA NA NA NA NA NA 2 1 NA ...} \cr
\code{  .. ..- attr(*, "value.labels")=Named num [1:6] 9 6 4 3 2 1} \cr
\code{  .. .. ..- attr(*, "names")=chr [1:6] "OMITTED" "NOT REACHED" "D" "C" ...} \cr
\code{  ..$ M032097 : atomic [1:4472] NA NA NA NA NA NA NA 2 3 NA ...} \cr
\code{  .. ..- attr(*, "value.labels")=Named num [1:6] 9 6 4 3 2 1} \cr
\code{  .. .. ..- attr(*, "names")=chr [1:6] "OMITTED" "NOT REACHED" "D" "C*" ...} \cr
\code{  .. [list output truncated]} \cr
\code{ $ scored  : num [1:4472, 1:443] 3010101 3010102 3010104 3010105 3010106 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : NULL} \cr
\code{  .. ..$ : chr [1:443] "idstud" "M022043" "M022046" "M022049" ...} \cr
\code{ $ iteminfo:'data.frame':} \cr
\code{  ..$ item      : Factor w/ 442 levels "M022043","M022046",..: 1 2 3 4 5 6 21 7 8 17 ...} \cr
\code{  ..$ content   : Factor w/ 8 levels "Algebra","Biology",..: 7 7 6 1 6 7 4 6 7 7 ...} \cr
\code{  ..$ topic     : Factor w/ 49 levels "Algebraic Expression",..: 32 32 41 29  ...} \cr
\code{  ..$ cognitive : Factor w/ 3 levels "Applying","Knowing",..: 2 1 3 2 1 1 1 1 2 1 ...} \cr
\code{  ..$ item.type : Factor w/ 2 levels "CR","MC": 2 1 2 2 1 2 2 2 2 1 ...} \cr
\code{  ..$ N.options : Factor w/ 4 levels "-"," -","4","5": 4 1 3 4 1 4 4 4 3 1 ...} \cr
\code{  ..$ key       : Factor w/ 7 levels "-"," -","A","B",..: 6 1 6 7 1 5 5 4 6 1 ...} \cr
\code{  ..$ max.points: int [1:442] 1 1 1 1 1 1 1 1 1 2 ...} \cr
\code{  ..$ item.label: Factor w/ 432 levels "1 teacher for every 12 students ",..: 58 351 ...} \cr

}

%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}

\source{
TIMSS 2007 8th Grade, Russian Sample
}

%\references{
%%  ~~ possibly secondary sources and usages ~~
%}



%\section{Example Index}{
%\code{\link{automatic.recode}} (Example 1),
%}


%\examples{
%data(data.timss07.G8.RUS)
%## maybe str(data.timss07.G8.RUS) ; plot(data.timss07.G8.RUS) ...
%}


%% \keyword{datasets}

%% File Name: R2conquest.Rd
%% File Version: 3.263

\name{R2conquest}
\alias{R2conquest}
\alias{summary.R2conquest}
\alias{read.show}
\alias{read.show.term}
\alias{read.show.regression}
\alias{read.pv}
\alias{read.multidimpv}
\alias{read.pimap}



%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Running ConQuest From Within \R
}

\description{
The function \code{R2conquest} runs the IRT software ConQuest
(Wu, Adams, Wilson & Haldane, 2007) from within \R.

Other functions are utility functions for reading item parameters, plausible
values or person-item maps.
}
\usage{
R2conquest(dat, path.conquest, conquest.name="console", converge=0.001,
    deviancechange=1e-04, iter=800, nodes=20, minnode=-6, maxnode=6,
    show.conquestoutput=FALSE, name="rasch", pid=1:(nrow(dat)), wgt=NULL, X=NULL,
    set.constraints=NULL, model="item", regression=NULL,
    itemcodes=seq(0,max(dat,na.rm=TRUE)), constraints=NULL, digits=5, onlysyntax=FALSE,
    qmatrix=NULL, import.regression=NULL, anchor.regression=NULL,
    anchor.covariance=NULL, pv=TRUE, designmatrix=NULL, only.calibration=FALSE,
    init_parameters=NULL, n_plausible=10,  persons.elim=TRUE, est.wle=TRUE,
    save.bat=TRUE, use.bat=FALSE, read.output=TRUE, ignore.pid=FALSE)

\method{summary}{R2conquest}(object, \dots)

# read all terms in a show file or only some terms
read.show(showfile)
read.show.term(showfile, term)

# read regression parameters in a show file
read.show.regression(showfile)

# read unidimensional plausible values form a pv file
read.pv(pvfile, npv=5)
# read multidimensional plausible values
read.multidimpv(pvfile, ndim, npv=5)

# read person-item map
read.pimap(showfile)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame of item responses
}
  \item{path.conquest}{
Directory where the ConQuest executable file is located
}
  \item{conquest.name}{
Name of the ConQuest executable.
}
  \item{converge}{
Maximal change in parameters
}
  \item{deviancechange}{
Maximal change in deviance
}
  \item{iter}{
Maximum number of iterations
}
  \item{nodes}{
Number of nodes for integration
}
\item{minnode}{Minimum value of discrete grid of \eqn{\theta} nodes}
\item{maxnode}{Maximum value of discrete grid of \eqn{\theta} nodes}
  \item{show.conquestoutput}{
Show ConQuest run log file on console?
}
  \item{name}{
Name of the output files. The default is \code{'rasch'}.
}
  \item{pid}{
Person identifier
}
  \item{wgt}{
Vector of person weights
}
  \item{X}{
Matrix of covariates for the latent regression model
(e.g. gender, socioeconomic status, ..) or
for the item design (e.g. raters, booklets, ...)
}
  \item{set.constraints}{
This is the set.constraints in ConQuest. It can be
\code{"cases"} (constraint for persons), \code{"items"}
or \code{"none"}
}
  \item{model}{
Definition model statement. It can be for example
\code{"item+item*step"} or \code{"item+booklet+rater"}
}
  \item{regression}{
The ConQuest regression statement (for example \code{"gender+status"})
}
  \item{itemcodes}{
Vector of valid codes for item responses. E.g. for partial credit
data with at most 3 points it must be \code{c(0,1,2,3)}.
}
  \item{constraints}{
Matrix of item parameter constraints. 1st column: Item names,
2nd column: Item parameters. It only works correctly
for dichotomous data.
}
  \item{digits}{
Number of digits for covariates in the latent regression model
}
  \item{onlysyntax}{
Should only be ConQuest syntax generated?
}
  \item{qmatrix}{
Matrix of item loadings on dimensions in a multidimensional
IRT model
}
  \item{import.regression}{
Name of an file with initial covariance parameters
(follow the ConQuest specification rules!)
}
  \item{anchor.regression}{
Name of an file with anchored regression parameters
}
  \item{anchor.covariance}{
Name of an file with anchored covariance parameters
(follow the ConQuest specification rules!)
}
  \item{pv}{
Draw plausible values?
}
  \item{designmatrix}{
Design matrix for item parameters (see the ConQuest manual)
}
  \item{only.calibration}{
Estimate only item parameters and not person parameters
(no WLEs or plausible values are estimated)?
}
  \item{init_parameters}{
Name of an file with initial item parameters (follow the
ConQuest specification rules!)
}
  \item{n_plausible}{
Number of plausible values
}
  \item{persons.elim}{
Eliminate persons with only missing item responses?
}
  \item{est.wle}{
Estimate weighted likelihood estimate?
}
  \item{save.bat}{
Save bat file?
}
  \item{use.bat}{
Run ConQuest from within \R due a direct call via the \code{system}
command (\code{use.bat=FALSE}) or via a \code{system} call
of a bat file in the working directory (\code{use.bat=TRUE})
}
  \item{read.output}{
Should ConQuest output files be processed? Default is \code{TRUE}.
}
\item{ignore.pid}{Logical indicating whether person identifiers (\code{pid})
should be processed in ConQuest input syntax.}
\item{object}{
Object of class \code{R2conquest}
}
\item{showfile}{
A ConQuest show file (\code{shw} file)
}
\item{term}{
Name of the term to be extracted in the show file
}
\item{pvfile}{File with plausible values}
\item{ndim}{Number of dimensions}
\item{npv}{Number of plausible values}
\item{\dots}{Further arguments to be passed}
}


\details{
Consult the ConQuest manual (Wu et al., 2007) for specification
details.
}



\value{
A list with several entries

\item{item}{Data frame with item parameters and item statistics}
\item{person}{Data frame with person parameters}
\item{shw.itemparameter}{ConQuest output table for item parameters}
\item{shw.regrparameter}{ConQuest output table for regression parameters}
\item{\dots}{More values}
}


\references{
Wu, M. L., Adams, R. J., Wilson, M. R. & Haldane, S. (2007).
\emph{ACER ConQuest Version 2.0}. Mulgrave.
{https://shop.acer.edu.au/acer-shop/group/CON3}.
}


%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also the \pkg{eat} package 
for elaborate functionality of using ConQuest from within \R. See also
the \pkg{conquestr} package for another \R wrapper to the ConQuest software
(at least version 4 of ConQuest has to be installed).

See also the \pkg{TAM} package for similar (and even extended)
functionality for specifying item response models.
}
\examples{
\dontrun{
# define ConQuest path
path.conquest <- "C:/Conquest/"

#############################################################################
# EXAMPLE 1: Dichotomous data (data.pisaMath)
#############################################################################
library(sirt)
data(data.pisaMath)
dat <- data.pisaMath$data

# select items
items <- colnames(dat)[ which( substring( colnames(dat), 1, 1)=="M" ) ]

#***
# Model 11: Rasch model
mod11 <- sirt::R2conquest(dat=dat[,items], path.conquest=path.conquest,
             pid=dat$idstud, name="mod11")
summary(mod11)
# read show file
shw11 <- sirt::read.show( "mod11.shw" )
# read person-item map
pi11 <- sirt::read.pimap(showfile="mod11.shw")

#***
# Model 12: Rasch model with fixed item difficulties (from Model 1)
mod12 <- sirt::R2conquest(dat=dat[,items], path.conquest=path.conquest,
             pid=dat$idstud, constraints=mod11$item[, c("item","itemdiff")],
             name="mod12")
summary(mod12)

#***
# Model 13: Latent regression model with predictors female, hisei and migra
mod13a <- sirt::R2conquest(dat=dat[,items], path.conquest=path.conquest,
             pid=dat$idstud, X=dat[, c("female", "hisei", "migra") ],
             name="mod13a")
summary(mod13a)

# latent regression with a subset of predictors
mod13b <- sirt::R2conquest(dat=dat[,items], path.conquest=path.conquest,
             pid=dat$idstud, X=dat[, c("female", "hisei", "migra") ],
             regression="hisei migra", name="mod13b")

#***
# Model 14: Differential item functioning (female)
mod14 <- sirt::R2conquest(dat=dat[,items], path.conquest=path.conquest,
             pid=dat$idstud, X=dat[, c("female"), drop=FALSE],
             model="item+female+item*female",  regression="",  name="mod14")

#############################################################################
# EXAMPLE 2: Polytomous data (data.Students)
#############################################################################
library(CDM)
data(data.Students)
dat <- data.Students

# select items
items <- grep.vec( "act", colnames(dat) )$x

#***
# Model 21: Partial credit model
mod21 <- sirt::R2conquest(dat=dat[,items], path.conquest=path.conquest,
              model="item+item*step",  name="mod21")

#***
# Model 22: Rating scale model
mod22 <- sirt::R2conquest(dat=dat[,items], path.conquest=path.conquest,
              model="item+step", name="mod22")

#***
# Model 23: Multidimensional model
items <- grep.vec( c("act", "sc" ), colnames(dat),  "OR" )$x
qmatrix <- matrix( 0, nrow=length(items), 2 )
qmatrix[1:5,1] <- 1
qmatrix[6:9,2] <- 1
mod23 <- sirt::R2conquest(dat=dat[,items], path.conquest=path.conquest,
            model="item+item*step", qmatrix=qmatrix, name="mod23")

#############################################################################
# EXAMPLE 3: Multi facet models (data.ratings1)
#############################################################################
library(sirt)
data(data.ratings1)
dat <- data.ratings1

items <- paste0("k",1:5)

# use numeric rater ID's
raters <- as.numeric( substring( paste( dat$rater ), 3 ) )

#***
# Model 31: Rater model 'item+item*step+rater'
mod31 <- sirt::R2conquest(dat=dat[,items], path.conquest=path.conquest,
              itemcodes=0:3, model="item+item*step+rater",
              pid=dat$idstud, X=data.frame("rater"=raters),
              regression="", name="mod31")

#***
# Model 32: Rater model 'item+item*step+rater+item*rater'
mod32 <- sirt::R2conquest(dat=dat[,items], path.conquest=path.conquest,
              model="item+item*step+rater+item*rater",
              pid=dat$idstud, X=data.frame("rater"=raters),
              regression="", name="mod32")
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{ConQuest}
%% \keyword{summary}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: sim.raschtype.Rd
%% File Version: 0.24

\name{sim.raschtype}
\alias{sim.raschtype}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Simulate from Generalized Logistic Item Response Model
}
\description{
This function simulates dichotomous item responses from a
generalized logistic item response model (Stukel, 1988).
The four-parameter logistic item response model
(Loken & Rulison, 2010) is a special case. See \code{\link{rasch.mml2}}
for more details.
}
\usage{
sim.raschtype(theta, b, alpha1=0, alpha2=0, fixed.a=NULL,
    fixed.c=NULL, fixed.d=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{theta}{
Unidimensional ability vector \eqn{\theta}
}
  \item{b}{
Vector of item difficulties \eqn{b}
}
  \item{alpha1}{
Parameter \eqn{\alpha_1} in generalized logistic link function
}
  \item{alpha2}{
Parameter \eqn{\alpha_2} in generalized logistic link function
}
  \item{fixed.a}{
Vector of item slopes \eqn{a}}
  \item{fixed.c}{
Vector of lower item asymptotes \eqn{c}
}
  \item{fixed.d}{
Vector of lower item asymptotes \eqn{d}
}
}
\details{
The class of generalized logistic link functions contain
the most important link functions using the specifications (Stukel, 1988):

logistic link function: \eqn{\alpha_1=0} and \eqn{\alpha_2=0} \cr
probit link function: \eqn{\alpha_1=0.165} and \eqn{\alpha_2=0.165} \cr
loglog link function: \eqn{\alpha_1=-0.037} and \eqn{\alpha_2=0.62} \cr
cloglog link function: \eqn{\alpha_1=0.62} and \eqn{\alpha_2=-0.037} \cr

See \code{\link{pgenlogis}} for exact transformation formulas of
the mentioned link functions.
}

\value{
Data frame with simulated item responses
}


\references{
Loken, E., & Rulison, K. L. (2010). Estimation of a four-parameter
item response theory model. \emph{British Journal of Mathematical
and Statistical Psychology, 63}, 509-525.

Stukel, T. A. (1988). Generalized logistic models.
\emph{Journal of the American Statistical Association,
83}, 426-431.
}

%\author{
%Alexander Robitzsch
%}

%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{rasch.mml2}}, \code{\link{pgenlogis}}
}

\examples{
#############################################################################
## EXAMPLE 1: Simulation of data from a Rasch model (alpha_1=alpha_2=0)
#############################################################################

set.seed(9765)
N <- 500    # number of persons
I <- 11     # number of items
b <- seq( -2, 2, length=I )
dat <- sirt::sim.raschtype( stats::rnorm( N ), b )
colnames(dat) <- paste0( "I", 1:I )
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Simulating IRT models}
%\keyword{ ~kwd1 }

%% File Name: mcmc.3pno.testlet.Rd
%% File Version: 1.28

\name{mcmc.3pno.testlet}
\alias{mcmc.3pno.testlet}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
3PNO Testlet Model
}
\description{
This function estimates the 3PNO testlet model (Wang, Bradlow & Wainer, 2002, 2007)
by Markov Chain Monte Carlo methods (Glas, 2012).
}
\usage{
mcmc.3pno.testlet(dat, testlets=rep(NA, ncol(dat)),
   weights=NULL, est.slope=TRUE, est.guess=TRUE, guess.prior=NULL,
   testlet.variance.prior=c(1, 0.2), burnin=500, iter=1000,
   N.sampvalues=1000, progress.iter=50, save.theta=FALSE, save.gamma.testlet=FALSE )
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame with dichotomous item responses for \eqn{N} persons and \eqn{I} items
}
  \item{testlets}{
An integer or character vector which indicates the allocation of items to
testlets. Same entries corresponds to same testlets.
If an entry is \code{NA}, then this item does not belong to any testlet.
}
  \item{weights}{
An optional vector with student sample weights
}
  \item{est.slope}{
Should item slopes be estimated? The default is \code{TRUE}.
}
  \item{est.guess}{
Should guessing parameters be estimated? The default is \code{TRUE}.
}
  \item{guess.prior}{
A vector of length two or a matrix with \eqn{I} items and two columns
which defines the beta prior distribution of guessing
parameters. The default is a non-informative prior, i.e. the Beta(1,1)
distribution.
}
  \item{testlet.variance.prior}{
A vector of length two which defines the (joint) prior for testlet variances
assuming an inverse chi-squared distribution.
The first entry is the effective sample size of the prior while the second
entry defines the prior variance of the testlet. The default of \code{c(1,.2)}
means that the prior sample size is 1 and the prior testlet variance is .2.
}
  \item{burnin}{
Number of burnin iterations
}
  \item{iter}{
Number of iterations
}
  \item{N.sampvalues}{
Maximum number of sampled values to save
}
  \item{progress.iter}{
Display progress every \code{progress.iter}-th iteration. If no progress
display is wanted, then choose \code{progress.iter} larger than \code{iter}.
}
  \item{save.theta}{
Logical indicating whether theta values should be saved
}
\item{save.gamma.testlet}{
Logical indicating whether gamma values should be saved
}
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DETAILS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\details{
The testlet response model for person \eqn{p} at item \eqn{i}
is defined as
\deqn{ P(X_{pi}=1 )=c_i + ( 1 - c_i )
        \Phi ( a_i \theta_p + \gamma_{p,t(i)} + b_i    ) \quad, \quad
    \theta_p \sim N ( 0,1 ), \gamma_{p,t(i)} \sim N( 0, \sigma^2_t ) }

In case of \code{est.slope=FALSE}, all item slopes \eqn{a_i} are set to 1. Then
a variance \eqn{\sigma^2} of the \eqn{\theta_p} distribution is estimated
which is called the Rasch testlet model in the literature (Wang & Wilson, 2005).

In case of \code{est.guess=FALSE}, all guessing parameters \eqn{c_i} are
set to 0.

After fitting the testlet model, marginal item parameters are calculated (integrating
out testlet effects \eqn{\gamma_{p,t(i)}}) according the defining response equation
\deqn{ P(X_{pi}=1 )=c_i + ( 1 - c_i )
        \Phi ( a_i^\ast \theta_p + b_i^\ast    ) }

}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VALUES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\value{
A list of class \code{mcmc.sirt} with following entries:
  \item{mcmcobj}{Object of class \code{mcmc.list} containing item parameters
  (\code{b_marg} and \code{a_marg} denote marginal item parameters)
  and person parameters (if requested)}
  \item{summary.mcmcobj}{Summary of the \code{mcmcobj} object. In this
  summary the Rhat statistic and the mode estimate MAP is included.
  The variable \code{PercSEratio} indicates the proportion of the Monte Carlo
  standard error in relation to the total standard deviation of the
  posterior distribution.}
  \item{ic}{Information criteria (DIC)}
  \item{burnin}{Number of burnin iterations}
  \item{iter}{Total number of iterations}
  \item{theta.chain}{Sampled values of \eqn{\theta_p} parameters}
  \item{deviance.chain}{Sampled values of deviance values}
  \item{EAP.rel}{EAP reliability}
  \item{person}{Data frame with EAP person parameter estimates for
        \eqn{\theta_p} and their corresponding posterior standard
      deviations and for all testlet effects}
  \item{dat}{Used data frame}
  \item{weights}{Used student weights}
  \item{\dots}{Further values}
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% REFERENCES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\references{
Glas, C. A. W. (2012). \emph{Estimating and testing the extended testlet model.}
LSAC Research Report Series, RR 12-03.

Wainer, H., Bradlow, E. T., & Wang, X. (2007).
\emph{Testlet response theory and its applications}.
Cambridge: Cambridge University Press.

Wang, W.-C., & Wilson, M. (2005). The Rasch testlet model.
\emph{Applied Psychological Measurement, 29}, 126-149.

Wang, X., Bradlow, E. T., & Wainer, H. (2002). A general Bayesian model
for testlets: Theory and applications.
\emph{Applied Psychological Measurement, 26}, 109-128.
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
S3 methods: \code{\link{summary.mcmc.sirt}}, \code{\link{plot.mcmc.sirt}}
}

%For estimating testlet models using the \pkg{lme4} package see
%\code{\link{rasch.testlet.glmer}}.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EXAMPLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Dataset Reading
#############################################################################
data(data.read)
dat <- data.read
I <- ncol(dat)

# set burnin and total number of iterations here (CHANGE THIS!)
burnin <- 200
iter <- 500

#***
# Model 1: 1PNO model
mod1 <- sirt::mcmc.3pno.testlet( dat,  est.slope=FALSE, est.guess=FALSE,
            burnin=burnin, iter=iter )
summary(mod1)
plot(mod1,ask=TRUE) # plot MCMC chains in coda style
plot(mod1,ask=TRUE, layout=2) # plot MCMC output in different layout

#***
# Model 2: 3PNO model with Beta(5,17) prior for guessing parameters
mod2 <- sirt::mcmc.3pno.testlet( dat,  guess.prior=c(5,17),
               burnin=burnin, iter=iter )
summary(mod2)

#***
# Model 3: Rasch (1PNO) testlet model
testlets <- substring( colnames(dat), 1, 1 )
mod3 <- sirt::mcmc.3pno.testlet( dat,  testlets=testlets,  est.slope=FALSE,
           est.guess=FALSE, burnin=burnin, iter=iter )
summary(mod3)

#***
# Model 4: 3PNO testlet model with (almost) fixed guessing parameters .25
mod4 <- sirt::mcmc.3pno.testlet( dat,  guess.prior=1000*c(25,75), testlets=testlets,
              burnin=burnin, iter=iter )
summary(mod4)
plot(mod4, ask=TRUE, layout=2)

#############################################################################
# EXAMPLE 2: Simulated data according to the Rasch testlet model
#############################################################################
set.seed(678)

N <- 3000   # number of persons
I <- 4      # number of items per testlet
TT <- 3     # number of testlets

ITT <- I*TT
b <- round( stats::rnorm( ITT, mean=0, sd=1 ), 2 )
sd0 <- 1 # sd trait
sdt <- seq( 0, 2, len=TT ) # sd testlets

# simulate theta
theta <- stats::rnorm( N, sd=sd0 )
# simulate testlets
ut <- matrix(0,nrow=N, ncol=TT )
for (tt in 1:TT){
    ut[,tt] <- stats::rnorm( N, sd=sdt[tt] )
}
ut <- ut[, rep(1:TT,each=I) ]
# calculate response probability
prob <- matrix( stats::pnorm( theta + ut + matrix( b, nrow=N, ncol=ITT,
            byrow=TRUE ) ), N, ITT)
Y <- (matrix( stats::runif(N*ITT), N, ITT) < prob )*1
colMeans(Y)

# define testlets
testlets <- rep(1:TT, each=I )

burnin <- 300
iter <- 1000

#***
# Model 1: 1PNO model (without testlet structure)
mod1 <- sirt::mcmc.3pno.testlet( dat=Y,  est.slope=FALSE, est.guess=FALSE,
            burnin=burnin, iter=iter, testlets=testlets )
summary(mod1)

summ1 <- mod1$summary.mcmcobj
# compare item parameters
cbind( b, summ1[ grep("b", summ1$parameter ), "Mean" ] )
# Testlet standard deviations
cbind( sdt, summ1[ grep("sigma\\.testlet", summ1$parameter ), "Mean" ] )

#***
# Model 2: 1PNO model (without testlet structure)
mod2 <- sirt::mcmc.3pno.testlet( dat=Y,  est.slope=TRUE, est.guess=FALSE,
           burnin=burnin, iter=iter, testlets=testlets )
summary(mod2)

summ2 <- mod2$summary.mcmcobj
# compare item parameters
cbind( b, summ2[ grep("b\\[", summ2$parameter ), "Mean" ] )
# item discriminations
cbind( sd0, summ2[ grep("a\\[", summ2$parameter ), "Mean" ] )
# Testlet standard deviations
cbind( sdt, summ2[ grep("sigma\\.testlet", summ2$parameter ), "Mean" ] )

#############################################################################
# EXAMPLE 3: Simulated data according to the 2PNO testlet model
#############################################################################
set.seed(678)

N <- 3000    # number of persons
I <- 3      # number of items per testlet
TT <- 5    # number of testlets

ITT <- I*TT
b <- round( stats::rnorm( ITT, mean=0, sd=1 ), 2 )
a <- round( stats::runif( ITT, 0.5, 2 ),2)
sdt <- seq( 0, 2, len=TT ) # sd testlets
sd0 <- 1

# simulate theta
theta <- stats::rnorm( N, sd=sd0 )
# simulate testlets
ut <- matrix(0,nrow=N, ncol=TT )
for (tt in 1:TT){
   ut[,tt] <- stats::rnorm( N, sd=sdt[tt] )
}
ut <- ut[, rep(1:TT,each=I) ]
# calculate response probability
bM <- matrix( b, nrow=N, ncol=ITT, byrow=TRUE )
aM <- matrix( a, nrow=N, ncol=ITT, byrow=TRUE )
prob <- matrix( stats::pnorm( aM*theta + ut + bM ), N, ITT)
Y <- (matrix( stats::runif(N*ITT), N, ITT) < prob )*1
colMeans(Y)

# define testlets
testlets <- rep(1:TT, each=I )

burnin <- 500
iter <- 1500

#***
# Model 1: 2PNO model
mod1 <- sirt::mcmc.3pno.testlet( dat=Y,  est.slope=TRUE, est.guess=FALSE,
             burnin=burnin, iter=iter, testlets=testlets )
summary(mod1)

summ1 <- mod1$summary.mcmcobj
# compare item parameters
cbind( b, summ1[ grep("b\\[", summ1$parameter ), "Mean" ] )
# item discriminations
cbind( a, summ1[ grep("a\\[", summ1$parameter ), "Mean" ] )
# Testlet standard deviations
cbind( sdt, summ1[ grep("sigma\\.testlet", summ1$parameter ), "Mean" ] )
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Testlet model}
%% \keyword{Testlets}
%% \keyword{Markov Chain Monte Carlo (MCMC)}
% __ONLY ONE__ keyword per line

%% File Name: linking_2groups.Rd
%% File Version: 0.054

\name{linking_2groups}
\alias{linking_2groups}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Linking Two Groups
}
\description{
This function performs (robust) Haebara linking or Stocking-Lord for two groups
in the 2PL model. There is also an option for simultaneous estimation in which
joint item parameters should also be estimated.
}
\usage{
linking_2groups(pars, method, type="asymm", pow=2, eps=0.001, simultan=FALSE,
     Theta=seq(-6, 6, len=101), wgt=NULL, par_init=NULL, optimizer="nlminb",
     control_optimizer=list())
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{pars}{
Matrix containing item parameters that must have columns with
names \code{a1}, \code{b1}, \code{a2}, and \code{b2}.
}
  \item{method}{
Linking method. Can be \code{SL} (Stocking-Lord) or \code{Hae} (Haebara).
}
  \item{type}{
Type of linking function. Can be \code{"asymm"} or \code{"symm"} for
asymmetric or symmetric linking, respectively.
}
  \item{pow}{
Power used in the loss function. The values should be larger than or
equal than zero. The default values is 2.
}
  \item{eps}{
Tuning parameter in numerical differentiation
}
  \item{simultan}{
Logical indicating whether joint item parameters should be simultaneously
estimated
}
  \item{Theta}{
Theta grid
}
  \item{wgt}{
Weights for theta grid
}
  \item{par_init}{
Initial vector for parameters
}
  \item{optimizer}{
Chosen optimizer. Can be \code{"nlminb"} or \code{"optim"}.
}
  \item{control_optimizer}{
Control arguments for optimizer
}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

\value{
List with following entries
\item{par}{Estimated linking parameters}
\item{$\ldots$}{More values}
}

%\references{
%% ~put references to the literature/web site here ~
%}
%\author{
%%  ~~who you are~~
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: data.ex16 from TAM package
#############################################################################

#- extract dataset
data(data.ex16, package="TAM")
dat <- data.ex16
items <- colnames(dat)[-c(1,2)]

# fit grade 1
rdat1 <- TAM::tam_remove_missings( dat[ dat$grade==1, ], items=items )
mod1 <- sirt::rasch.mml2( dat=rdat1$resp[, rdat1$items], est.a=1:length(rdat1$items) )

# fit grade 2
rdat2 <- TAM::tam_remove_missings( dat[ dat$grade==2, ], items=items )
mod2 <- sirt::rasch.mml2( dat=rdat2$resp[, rdat2$items], est.a=1:length(rdat2$items) )

# merge item parameters of separate scalings
item1 <- mod1$item[, c("item", "a","b") ]
colnames(item1)[2:3] <- c("a1","b1")
item2 <- mod2$item[, c("item", "a","b") ]
colnames(item2)[2:3] <- c("a2","b2")
pars <- merge(x=item1, y=item2, by="item")

# choose theta grid
Theta <- seq(-6,6, len=101)
wgt <- sirt::sirt_dnorm_discrete(x=Theta, mean=0, sd=2)

#- robust Haebara linking with powers 0.5 or 0 and different types
res1 <- sirt::linking_2groups(pars=pars, method="Hae", type="asymm",
              Theta=Theta, wgt=wgt, pow=0.5, eps=0.001 )
res2 <- sirt::linking_2groups(pars=pars, method="Hae", type="symm",
              Theta=Theta, wgt=wgt, pow=0, eps=0.01 )
res1$par  # linking parameter estimate
res2$par
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

%% File Name: R2noharm.Rd
%% File Version: 2.358

\name{R2noharm}
\alias{R2noharm}
\alias{summary.R2noharm}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimation of a NOHARM Analysis from within \R
}
\description{
This function enables the estimation of a NOHARM analysis
(Fraser & McDonald, 1988; McDonald, 1982a, 1982b, 1997) from within \R.
NOHARM estimates a compensatory multidimensional factor analysis for dichotomous
response data. Arguments of this function strictly follow
the rules of the NOHARM manual (see Fraser & McDonald, 2012; Lee & Lee, 2016).
}


\usage{
R2noharm(dat=NULL,pm=NULL, n=NULL, model.type, weights=NULL, dimensions=NULL,
      guesses=NULL, noharm.path, F.pattern=NULL, F.init=NULL,
      P.pattern=NULL, P.init=NULL, digits.pm=4, writename=NULL,
      display.fit=5,  dec=".", display=TRUE)

\method{summary}{R2noharm}(object, logfile=NULL, \dots)
}
%- maybe also 'usage' for other objects documented here.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ARGUMENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\arguments{
  \item{dat}{
An \eqn{N \times I} data frame of item responses for \eqn{N} subjects
and \eqn{I} items
}
\item{pm}{A matrix or a vector containing product-moment correlations}
\item{n}{Sample size. This value must only be included if \code{pm} is
provided.}
  \item{model.type}{
Can be \code{"EFA"} (exploratory factor analysis) or \code{"CFA"}
(confirmatory factor analysis).
}
\item{weights}{Optional vector of student weights}
  \item{dimensions}{
Number of dimensions in exploratory factor analysis
}
  \item{guesses}{
An optional vector of fixed guessing parameters of length \eqn{I}.
In case of the default \code{NULL}, all guessing parameters are set
to zero.
}
  \item{noharm.path}{
Local path where the NOHARM 4 command line 64-bit version is located.
}
  \item{F.pattern}{
Pattern matrix for \eqn{F} (\eqn{I \times D})
}
  \item{F.init}{
Initial matrix for \eqn{F} (\eqn{I \times D})
}
  \item{P.pattern}{
Pattern matrix for \eqn{P} (\eqn{D \times D})
}
  \item{P.init}{
Initial matrix for \eqn{P} (\eqn{D \times D})
}
  \item{digits.pm}{
Number of digits after decimal separator which are used for
estimation
}
  \item{writename}{
Name for NOHARM input and output files
  }
  \item{display.fit}{
How many digits (after decimal separator) should be used
for printing results on the \R console?
}
  \item{dec}{
Decimal separator (\code{"."} or \code{","})
}
  \item{display}{
Display output?
}
  \item{object}{
Object of class \code{R2noharm}
}
\item{logfile}{File name if the summary should be sunk into a file}
\item{\dots}{Further arguments to be passed}
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DETAILS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\details{
NOHARM estimates a multidimensional compensatory
item response model with the probit link function \eqn{\Phi}.
For item responses \eqn{X_{pi}} of person \eqn{p} on
item \eqn{i} the model equation is defined as
\deqn{P( X_{pi}=1 | \bold{\theta}_p )=c_i + ( 1 - c_i )
\Phi( f_{i0} + f_{i1} \theta_{p1} + ... +
f_{iD} \theta_{pD} ) }
where \eqn{F=(f_{id})} is a loading matrix and \eqn{P}
the covariance matrix of \eqn{\bold{\theta}_p}. The guessing
parameters \eqn{c_i} must be provided as fixed values.

For the definition of \eqn{F} and \eqn{P} matrices, please
consult the NOHARM manual.

This function needs the 64-bit command line version which can be downloaded
from (some links may be broken in the meantime) \cr \cr
http://noharm.niagararesearch.ca/nh4cldl.html \cr
https://noharm.software.informer.com/4.0/   \cr
https://cehs.unl.edu/edpsych/software-urls-and-other-interesting-sites/ \cr
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VALUES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\value{
A list with following entries
\item{tanaka}{Tanaka index}
\item{rmsr}{RMSR statistic}
\item{N.itempair}{Sample sizes of pairwise item observations}
\item{pm}{Product moment matrix}
\item{weights}{Used student weights}
\item{guesses}{Fixed guessing parameters}
\item{residuals}{Residual covariance matrix}
\item{final.constants}{Vector of final constants}
\item{thresholds}{Threshold parameters}
\item{uniquenesses}{Item uniquenesses}
\item{loadings.theta}{Matrix of loadings in theta parametrization
    (common factor parametrization)}
\item{factor.cor}{Covariance matrix of factors}
\item{difficulties}{Item difficulties (for unidimensional models)}
\item{discriminations}{Item discriminations (for unidimensional models)}
\item{loadings}{Loading matrix (latent trait parametrization)}
\item{model.type}{Used model type}
\item{Nobs}{Number of observations}
\item{Nitems}{Number of items}
\item{modtype}{Model type according to the NOHARM specification (see NOHARM manual)}
\item{F.init}{Initial loading matrix for \eqn{F}}
\item{F.pattern}{Pattern loading matrix for \eqn{F}}
\item{P.init}{Initial covariance matrix for \eqn{P}}
\item{P.pattern}{Pattern covariance matrix for \eqn{P}}
\item{dat}{Original data frame}
\item{systime}{System time}
\item{noharm.path}{Used NOHARM directory}
\item{digits.pm}{Number of digits in product moment matrix}
\item{dec}{Used decimal symbol}
\item{display.fit}{Number of digits for fit display}
\item{dimensions}{Number of dimensions}
\item{chisquare}{Statistic \eqn{\chi^2}}
\item{Nestpars}{Number of estimated parameters}
\item{df}{Degrees of freedom}
\item{chisquare_df}{Ratio \eqn{\chi^2 / df}}
\item{rmsea}{RMSEA statistic}
\item{p.chisquare}{Significance for \eqn{\chi^2} statistic}
}
\references{
Fraser, C., & McDonald, R. P. (1988). NOHARM: Least squares item factor analysis.
\emph{Multivariate Behavioral Research, 23}, 267-269.
https://doi.org/10.1207/s15327906mbr2302_9

Fraser, C., & McDonald, R. P. (2012). \emph{NOHARM 4 Manual}. \cr
{http://noharm.niagararesearch.ca/nh4man/nhman.html}.

Lee, J. J., & Lee, M. K. (2016). An overview of the normal ogive harmonic analysis
robust method (NOHARM) approach to item response theory.
\emph{Tutorials in Quantitative Methods for Psychology, 12}(1), 1-8.
https://doi.org/10.20982/tqmp.12.1.p001

McDonald, R. P. (1982a). Linear versus nonlinear models in item response theory.
\emph{Applied Psychological Measurement, 6}(4), 379-396.
\doi{10.1177/014662168200600402}

McDonald, R. P. (1982b). \emph{Unidimensional and multidimensional models for
item response theory}. I.R.T., C.A.T. conference, Minneapolis, 1982, Proceedings.

McDonald, R. P. (1997). Normal-ogive multidimensional model.
In W. van der Linden & R. K. Hambleton (1997):
\emph{Handbook of modern item response theory} (pp. 257-269).
New York: Springer. http://dx.doi.org/10.1007/978-1-4757-2691-6
}



%\author{
%Alexander Robitzsch
%}


\note{
Possible errors often occur due to wrong \code{dec}
specification.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For estimating standard errors see \code{\link{R2noharm.jackknife}}.

For EAP person parameter estimates see \code{\link{R2noharm.EAP}}.

For an \R implementation of the NOHARM model see \code{\link{noharm.sirt}}.
}
%% ~~objects to See Also as \code{\link{help}}, ~~~


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EXAMPLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Data data.noharm18 with 18 items
#############################################################################

# load data
data(data.noharm18)
dat <- data.noharm18
I <- ncol(dat) # number of items

# locate noharm.path
noharm.path <- "c:/NOHARM"

#****************************************
# Model 1: 1-dimensional Rasch model (1-PL model)

# estimate one factor variance
P.pattern <- matrix( 1, ncol=1, nrow=1 )
P.init <- P.pattern
# fix all entries in the loading matrix to 1
F.pattern <- matrix( 0, nrow=I, ncol=1 )
F.init <- 1 + 0*F.pattern       #
# estimate model
mod <- sirt::R2noharm( dat=dat, model.type="CFA",
           F.pattern=F.pattern, F.init=F.init, P.pattern=P.pattern,
           P.init=P.init, writename="ex1__1dim_1pl",
       noharm.path=noharm.path, dec="," )
# summary
summary(mod, logfile="ex1__1dim_1pl__SUMMARY")
# jackknife
jmod <- sirt::R2noharm.jackknife( mod, jackunits=20 )
summary(jmod, logfile="ex1__1dim_1pl__JACKKNIFE")
# compute factor scores (EAPs)
emod <- sirt::R2noharm.EAP(mod)

#*****-----
# Model 1b: Include student weights in estimation
N <- nrow(dat)
weights <- stats::runif( N, 1, 5 )
mod1b <- sirt::R2noharm( dat=dat, model.type="CFA",  weights=weights,
            F.pattern=F.pattern, F.init=F.init, P.pattern=P.pattern,
            P.init=P.init, writename="ex1__1dim_1pl_w",
            noharm.path=noharm.path, dec="," )
summary(mod1b)

#****************************************
# Model 2: 1-dimensional 2PL Model

# set trait variance equal to 1
P.pattern <- matrix( 0, ncol=1, nrow=1 )
P.init <- 1+0*P.pattern
# loading matrix
F.pattern <- matrix( 1, nrow=I, ncol=1 )
F.init <- 1 + 0*F.pattern

mod <- sirt::R2noharm( dat=dat, model.type="CFA",
            F.pattern=F.pattern, F.init=F.init, P.pattern=P.pattern,
            P.init=P.init, writename="ex2__1dim_2pl",
            noharm.path=noharm.path, dec="," )

summary(mod)
jmod <- sirt::R2noharm.jackknife( mod, jackunits=20 )
summary(jmod)

#****************************************
# Model 3: 1-dimensional 3PL Model with fixed guessing parameters

# set trait variance equal to 1
P.pattern <- matrix( 0, ncol=1, nrow=1 )
P.init <- 1+0*P.pattern
# loading matrix
F.pattern <- matrix( 1, nrow=I, ncol=1 )
F.init <- 1 + 0*F.pattern       #
# fix guessing parameters equal to .2 (for all items)
guesses <- rep( .1, I )

mod <- sirt::R2noharm( dat=dat, model.type="CFA",
          F.pattern=F.pattern, F.init=F.init, P.pattern=P.pattern,
          P.init=P.init, guesses=guesses,
          writename="ex3__1dim_3pl", noharm.path=noharm.path, dec=","  )
summary(mod)
jmod <- sirt::R2noharm.jackknife( mod, jackunits=20 )
summary(jmod)

#****************************************
# Model 4: 3-dimensional Rasch model

# estimate one factor variance
P.pattern <- matrix( 1, ncol=3, nrow=3 )
P.init <- .8*P.pattern
diag(P.init) <- 1
# fix all entries in the loading matrix to 1
F.init <- F.pattern <- matrix( 0, nrow=I, ncol=3 )
F.init[1:6,1] <- 1
F.init[7:12,2] <- 1
F.init[13:18,3] <- 1

mod <- sirt::R2noharm( dat=dat, model.type="CFA",
          F.pattern=F.pattern, F.init=F.init, P.pattern=P.pattern,
          P.init=P.init, writename="ex4__3dim_1pl",
          noharm.path=noharm.path, dec="," )
# write output from R console in a file
summary(mod, logfile="ex4__3dim_1pl__SUMMARY.Rout")

jmod <- sirt::R2noharm.jackknife( mod, jackunits=20 )
summary(jmod)

# extract factor scores
emod <- sirt::R2noharm.EAP(mod)

#****************************************
# Model 5: 3-dimensional 2PL model

# estimate one factor variance
P.pattern <- matrix( 1, ncol=3, nrow=3 )
P.init <- .8*P.pattern
diag(P.init) <- 0
# fix all entries in the loading matrix to 1
F.pattern <- matrix( 0, nrow=I, ncol=3 )
F.pattern[1:6,1] <- 1
F.pattern[7:12,2] <- 1
F.pattern[13:18,3] <- 1
F.init <- F.pattern

mod <- sirt::R2noharm( dat=dat, model.type="CFA",
          F.pattern=F.pattern, F.init=F.init, P.pattern=P.pattern,
          P.init=P.init, writename="ex5__3dim_2pl",
          noharm.path=noharm.path, dec="," )
summary(mod)
# use 50 jackknife units with 4 persons within a unit
jmod <- sirt::R2noharm.jackknife( mod, jackunits=seq( 1:50, each=4 ) )
summary(jmod)

#****************************************
# Model 6: Exploratory Factor Analysis with 3 factors

mod <- sirt::R2noharm( dat=dat, model.type="EFA",  dimensions=3,
           writename="ex6__3dim_efa", noharm.path=noharm.path,dec=",")
summary(mod)

jmod <- sirt::R2noharm.jackknife( mod, jackunits=20 )

#############################################################################
# EXAMPLE 2: NOHARM manual Example A
#############################################################################

# See NOHARM manual: http://noharm.niagararesearch.ca/nh4man/nhman.html
# The following text and data is copied from this manual.
#
# In the first example, we demonstrate how to prepare the input for a 2-dimensional
# model using exploratory analysis. Data from a 9 item test were collected from
# 200 students and the 9x9 product-moment matrix of the responses was computed.
#
# Our hypothesis is for a 2-dimensional model with no guessing,
# i.e., all guesses are equal to zero. However, because we are unsure of any
# particular pattern for matrix F, we wish to prescribe an exploratory analysis, i.e.,
# set EX=1. Also, we will content ourselves with letting the program supply all
# initial values.
#
# We would like both the sample product-moment matrix and the residual matrix to
# be included in the output.

# scan product-moment matrix copied from the NOHARM manual
pm <- scan()
     0.8967
     0.2278 0.2356
     0.6857 0.2061 0.7459
     0.8146 0.2310 0.6873 0.8905
     0.4505 0.1147 0.3729 0.4443 0.5000
     0.7860 0.2080 0.6542 0.7791 0.4624 0.8723
     0.2614 0.0612 0.2140 0.2554 0.1914 0.2800 0.2907
     0.7549 0.1878 0.6236 0.7465 0.4505 0.7590 0.2756 0.8442
     0.6191 0.1588 0.5131 0.6116 0.3845 0.6302 0.2454 0.6129 0.6879

ex2 <- sirt::R2noharm( pm=pm, n=200, model.type="EFA", dimensions=2,
         noharm.path=noharm.path, writename="ex2_noharmExA", dec=",")
summary(ex2)

#############################################################################
# EXAMPLE 3: NOHARM manual Example B
#############################################################################

# See NOHARM manual: http://noharm.niagararesearch.ca/nh4man/nhman.html
# The following text and data is copied from this manual.

# Suppose we have the product-moment matrix of data from 125 students on 9 items.
# Our hypothesis is for 2 dimensions with simple structure. In this case,
# items 1 to 5 have coefficients of theta which are to be estimated for one
# latent trait but are to be fixed at zero for the other one.
# For the latent trait for which items 1 to 5 have zero coefficients,
# items 6 to 9 have coefficients which are to be estimated. For the other
# latent trait, items 6 to 9 will have zero coefficients.
# We also wish to estimate the correlation between the latent traits,
# so we prescribe P as a 2x2 correlation matrix.
#
# Our hypothesis prescribes that there was no guessing involved, i.e.,
# all guesses are equal to zero. For demonstration purposes,
# let us not have the program print out the sample product-moment matrix.
# Also let us not supply any starting values but, rather, use the defaults
# supplied by the program.

pm <- scan()
    0.930
    0.762 0.797
    0.541 0.496 0.560
    0.352 0.321 0.261 0.366
    0.205 0.181 0.149 0.110 0.214
    0.858 0.747 0.521 0.336 0.203 0.918
    0.773 0.667 0.465 0.308 0.184 0.775 0.820
    0.547 0.474 0.347 0.233 0.132 0.563 0.524 0.579
    0.329 0.290 0.190 0.140 0.087 0.333 0.308 0.252 0.348

I <- 9    # number of items
# define loading matrix
F.pattern <- matrix(0,I,2)
F.pattern[1:5,1] <- 1
F.pattern[6:9,2] <- 1
F.init <- F.pattern
# define covariance matrix
P.pattern <- matrix(1,2,2)
diag(P.pattern) <- 0
P.init <- 1+P.pattern

ex3 <- sirt::R2noharm( pm=pm, n=125,, model.type="CFA",
           F.pattern=F.pattern, F.init=F.init, P.pattern=P.pattern,
           P.init=P.init, writename="ex3_noharmExB",
           noharm.path=noharm.path, dec="," )
summary(ex3)

#############################################################################
# EXAMPLE 4: NOHARM manual Example C
#############################################################################

data(data.noharmExC)
# See NOHARM manual: http://noharm.niagararesearch.ca/nh4man/nhman.html
# The following text and data is copied from this manual.

# In this example, suppose that from 300 respondents we have item
# responses scored dichotomously, 1 or 0, for 8 items.
#
# Our hypothesis is for a unidimensional model where all eight items
# have coefficients of theta which are to be estimated.
# Suppose that since the items were multiple choice with 5 options each,
# we set the fixed guesses all to 0.2 (not necessarily good reasoning!)
#
# Let's supply initial values for the coefficients of theta (F matrix)
# as .75 for items 1 to 4 and .6 for items 5 to 8.

I <- 8
guesses <- rep(.2,I)
F.pattern <- matrix(1,I,1)
F.init <- F.pattern
F.init[1:4,1] <- .75
F.init[5:8,1] <- .6
P.pattern <- matrix(0,1,1)
P.init <- 1 + 0 * P.pattern

ex4 <- sirt::R2noharm( dat=data.noharmExC,, model.type="CFA",
           guesses=guesses, F.pattern=F.pattern, F.init=F.init,
           P.pattern=P.pattern, P.init=P.init, writename="ex3_noharmExC",
           noharm.path=noharm.path, dec="," )
summary(ex4)

# modify F pattern matrix
# f11=f51 (since both have equal pattern values of 2),
# f21=f61 (since both have equal pattern values of 3),
# f31=f71 (since both have equal pattern values of 4),
# f41=f81 (since both have equal pattern values of 5).
F.pattern[ c(1,5) ] <- 2
F.pattern[ c(2,6) ] <- 3
F.pattern[ c(3,7) ] <- 4
F.pattern[ c(4,8) ] <- 5
F.init <- .5+0*F.init

ex4a <- sirt::R2noharm( dat=data.noharmExC,, model.type="CFA",
           guesses=guesses, F.pattern=F.pattern, F.init=F.init,
           P.pattern=P.pattern, P.init=P.init, writename="ex3_noharmExC1",
           noharm.path=noharm.path, dec="," )
summary(ex4a)
}
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{NOHARM}
%% \keyword{summary}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: greenyang.reliability.Rd
%% File Version: 1.33

\name{greenyang.reliability}
\alias{greenyang.reliability}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Reliability for Dichotomous Item Response Data
Using the Method of Green and Yang (2009)
}
\description{
This function estimates the model-based reliability
of dichotomous data using the Green & Yang (2009) method.
The underlying factor model is \eqn{D}-dimensional where
the dimension \eqn{D} is specified by the argument
\code{nfactors}. The factor solution is subject to the
application of the Schmid-Leiman transformation (see Reise, 2012;
Reise, Bonifay, & Haviland, 2013; Reise, Moore, & Haviland, 2010).
}

\usage{
greenyang.reliability(object.tetra, nfactors)
}


%- maybe also 'usage' for other objects documented here.
\arguments{
\item{object.tetra}{
Object as the output of the function \code{tetrachoric}, the
\code{fa.parallel.poly} from the \pkg{psych} package or the
\code{tetrachoric2} function (from \pkg{sirt}).
This object can also be created
as a list by the user where the tetrachoric correlation
must must be in the list entry \code{rho} and the thresholds must
be in the list entry \code{thresh}.
}
  \item{nfactors}{
Number of factors (dimensions)
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A data frame with columns:
\item{coefficient}{Name of the reliability measure. \code{omega_1} (Omega)
is the reliability estimate for the total score for dichotomous data
based on a one-factor model, \code{omega_t} (Omega Total) is the
estimate for a \eqn{D}-dimensional model. For the nested factor model,
\code{omega_h} (Omega Asymptotic) is the reliability of the general factor model,
\code{omega_ha} (Omega Hierarchical Asymptotic) eliminates item-specific
variance. The explained common variance (\code{ECV}) explained by the
common factor is based on the \eqn{D}-dimensional but does not take
item thresholds into account. The amount of explained
variance \code{ExplVar} is defined as the quotient of the first
eigenvalue of the tetrachoric correlation matrix to the
sum of all eigenvalues. The statistic \code{EigenvalRatio}
is the ratio of the first and second eigenvalue.
}
\item{dimensions}{Number of dimensions}
\item{estimate}{Reliability estimate}
}


\references{
Green, S. B., & Yang, Y. (2009). Reliability of summed item
scores using structural equation modeling: An alternative to
coefficient alpha. \emph{Psychometrika, 74}, 155-167.

Reise, S. P. (2012). The rediscovery of bifactor measurement models.
\emph{Multivariate Behavioral Research, 47}, 667-696.

Reise, S. P., Bonifay, W. E., & Haviland, M. G. (2013).
Scoring and modeling psychological measures in the presence of
multidimensionality. \emph{Journal of Personality Assessment,
95}, 129-140.

Reise, S. P., Moore, T. M., & Haviland, M. G.  (2010).
Bifactor models and rotations: Exploring the extent to which
multidimensional data yield univocal scale scores,
\emph{Journal of Personality Assessment, 92}, 544-559.
}


%\author{
%Alexander Robitzsch
%}


\note{
This function needs the \pkg{psych} package.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See \code{\link{f1d.irt}} for estimating the functional unidimensional
item response model.

This function uses \code{\link{reliability.nonlinearSEM}}.

See also the \code{MBESS::ci.reliability} function for estimating
reliability for polytomous item responses.
}

\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Reliability estimation of Reading dataset data.read
#############################################################################
miceadds::library_install("psych")
set.seed(789)
data( data.read )
dat <- data.read

# calculate matrix of tetrachoric correlations
dat.tetra <- psych::tetrachoric(dat)      # using tetrachoric from psych package
dat.tetra2 <- sirt::tetrachoric2(dat)       # using tetrachoric2 from sirt package

# perform parallel factor analysis
fap <- psych::fa.parallel.poly(dat, n.iter=1 )
  ##   Parallel analysis suggests that the number of factors=3
  ##   and the number of components=2

# parallel factor analysis based on tetrachoric correlation matrix
##       (tetrachoric2)
fap2 <- psych::fa.parallel(dat.tetra2$rho, n.obs=nrow(dat),  n.iter=1 )
  ## Parallel analysis suggests that the number of factors=6
  ## and the number of components=2
  ## Note that in this analysis, uncertainty with respect to thresholds is ignored.

# calculate reliability using a model with 4 factors
greenyang.reliability( object.tetra=dat.tetra, nfactors=4 )
  ##                                            coefficient dimensions estimate
  ## Omega Total (1D)                               omega_1          1    0.771
  ## Omega Total (4D)                               omega_t          4    0.844
  ## Omega Hierarchical (4D)                        omega_h          4    0.360
  ## Omega Hierarchical Asymptotic (4D)            omega_ha          4    0.427
  ## Explained Common Variance (4D)                     ECV          4    0.489
  ## Explained Variance (First Eigenvalue)          ExplVar         NA   35.145
  ## Eigenvalue Ratio (1st to 2nd Eigenvalue) EigenvalRatio         NA    2.121

# calculation of Green-Yang-Reliability based on tetrachoric correlations
#   obtained by tetrachoric2
greenyang.reliability( object.tetra=dat.tetra2, nfactors=4 )

# The same result will be obtained by using fap as the input
greenyang.reliability( object.tetra=fap, nfactors=4 ) }
}
%% \keyword{Reliability}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: detect.index.Rd
%% File Version: 0.17

\name{detect.index}
\alias{detect.index}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Calculation of the DETECT and polyDETECT Index
}
\description{
This function calculated the DETECT and polyDETECT index (Stout, Habing, Douglas
& Kim, 1996; Zhang & Stout, 1999a; Zhang, 2007). At first,
conditional covariances have to be estimated
using the \code{\link{ccov.np}} function.
}
\usage{
detect.index(ccovtable, itemcluster)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{ccovtable}{
A value of \code{\link{ccov.np}}.
}
  \item{itemcluster}{
Item cluster for each item. The order of entries must correspond
to the columns in \code{data} (submitted to \code{\link{ccov.np}}).
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%}
\references{
Stout, W., Habing, B., Douglas, J., & Kim, H. R. (1996).
Conditional covariance-based nonparametric multidimensionality assessment.
\emph{Applied Psychological Measurement, 20}, 331-354.

Zhang, J., & Stout, W. (1999a). Conditional covariance structure
of generalized compensatory multidimensional items.
\emph{Psychometrika, 64}, 129-152.

Zhang, J., & Stout, W. (1999b). The theoretical DETECT index of
dimensionality and its application to approximate simple structure.
\emph{Psychometrika, 64}, 213-249.

Zhang, J. (2007). Conditional covariance theory and DETECT for
polytomous items. \emph{Psychometrika, 72}, 69-91.
}

%\author{
%Alexander Robitzsch
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For examples see \code{\link{conf.detect}}.
}
%\examples{
%% ##
%}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{DETECT}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: rasch.evm.pcm.Rd
%% File Version: 0.46

\name{rasch.evm.pcm}
\alias{rasch.evm.pcm}
\alias{summary.rasch.evm.pcm}
\alias{coef.rasch.evm.pcm}
\alias{vcov.rasch.evm.pcm}


%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimation of the Partial Credit Model using the Eigenvector Method
}

\description{
This function performs the eigenvector approach to estimate item
parameters which is based on a pairwise estimation approach
(Garner & Engelhard, 2002). No assumption about person parameters
is required for item parameter estimation. Statistical inference is performed by
Jackknifing. If a group identifier is provided, tests for differential item
functioning are performed.
}

\usage{
rasch.evm.pcm(dat, jackunits=20, weights=NULL, pid=NULL,
    group=NULL, powB=2, adj_eps=0.3, progress=TRUE )

\method{summary}{rasch.evm.pcm}(object, digits=3, file=NULL, ...)

\method{coef}{rasch.evm.pcm}(object,...)

\method{vcov}{rasch.evm.pcm}(object,...)
}


%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame with dichotomous or polytomous item responses
}
\item{jackunits}{
A number of Jackknife units (if an integer is provided as the argument
value) or a vector in which the Jackknife units are already
defined.
}
  \item{weights}{
Optional vector of sample weights
}
  \item{pid}{
Optional vector of person identifiers
}
\item{group}{
Optional vector of group identifiers. In this case, item parameters are group wise
estimated and tests for differential item functioning are performed.
}
  \item{powB}{
Power created in \eqn{B} matrix which is the basis of parameter estimation
}
  \item{adj_eps}{
Adjustment parameter for person parameter estimation
(see \code{\link{mle.pcm.group}})
}
  \item{progress}{
An optional logical indicating whether progress should be displayed
}
\item{object}{Object of class \code{rasch.evm.pcm}}
\item{digits}{Number of digits after decimals for rounding in \code{summary}.}
\item{file}{Optional file name if \code{summary} should be sunk into a file.}
\item{\dots}{Further arguments to be passed}
}

%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}


\value{
A list with following entries

\item{item}{Data frame with item parameters. The item parameter estimate
    is denoted by \code{est} while a Jackknife bias-corrected estimate
    is \code{est_jack}. The Jackknife standard error is \code{se}.
}
\item{b}{Item threshold parameters}
\item{person}{Data frame with person parameters obtained (MLE)}
\item{B}{Paired comparison matrix}
\item{D}{Transformed paired comparison matrix}
\item{coef}{Vector of estimated coefficients}
\item{vcov}{Covariance matrix of estimated item parameters}
\item{JJ}{Number of jackknife units}
\item{JJadj}{Reduced number of jackknife units}
\item{powB}{Used power of comparison matrix \eqn{B}}
\item{maxK}{Maximum number of categories per item}
\item{G}{Number of groups}
\item{desc}{Some descriptives}
\item{difstats}{Statistics for differential item functioning if \code{group}
is provided as an argument}

}

\references{
Choppin, B. (1985). A fully conditional estimation procedure for Rasch Model
parameters. \emph{Evaluation in Education, 9}, 29-42.

Garner, M., & Engelhard, G. J. (2002). An eigenvector method for
estimating item parameters of the dichotomous and polytomous Rasch models.
\emph{Journal of Applied Measurement, 3}, 107-128.

Wang, J., & Engelhard, G. (2014). A pairwise algorithm in \R for rater-mediated
assessments. \emph{Rasch Measurement Transactions, 28}(1), 1457-1459.
}


%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See the \pkg{pairwise} package for the alternative row averaging
approach of Choppin (1985) and Wang and Engelhard (2014) for an
alternative \R implementation.
}


\examples{
#############################################################################
# EXAMPLE 1: Dataset Liking for Science
#############################################################################

data(data.liking.science)
dat <- data.liking.science

# estimate partial credit model using 10 Jackknife units
mod1 <- sirt::rasch.evm.pcm( dat, jackunits=10 )
summary(mod1)

\dontrun{
# compare results with TAM
library(TAM)
mod2 <- TAM::tam.mml( dat )
r1 <- mod2$xsi$xsi
r1 <- r1 - mean(r1)
# item parameters are similar
dfr <- data.frame( "b_TAM"=r1, mod1$item[,c( "est","est_jack") ] )
round( dfr, 3 )
  ##      b_TAM    est est_jack
  ##  1  -2.496 -2.599   -2.511
  ##  2   0.687  0.824    1.030
  ##  3  -0.871 -0.975   -0.943
  ##  4  -0.360 -0.320   -0.131
  ##  5  -0.833 -0.970   -0.856
  ##  6   1.298  1.617    1.444
  ##  7   0.476  0.465    0.646
  ##  8   2.808  3.194    3.439
  ##  9   1.611  1.460    1.433
  ##  10  2.396  1.230    1.095
  ##  [...]

# partial credit model in eRm package
miceadds::library_install("eRm")
mod3 <- eRm::PCM(X=dat)
summary(mod3)
eRm::plotINFO(mod3)      # plot item and test information
eRm::plotICC(mod3)       # plot ICCs
eRm::plotPImap(mod3)     # plot person-item maps

#############################################################################
# EXAMPLE 2: Garner and Engelhard (2002) toy example dichotomous data
#############################################################################

dat <- scan()
   1 0 1 1   1 1 0 0   1 0 0 0   0 1 1 1   1 1 1 0
   1 1 0 1   1 1 1 1   1 0 1 0   1 1 1 1   1 1 0 0

dat <- matrix( dat, 10, 4, byrow=TRUE)
colnames(dat) <- paste0("I", 1:4 )

# estimate Rasch model with no jackknifing
mod1 <- sirt::rasch.evm.pcm( dat, jackunits=0 )

# paired comparison matrix
mod1$B
  ##          I1_Cat1 I2_Cat1 I3_Cat1 I4_Cat1
  ##  I1_Cat1       0       3       4       5
  ##  I2_Cat1       1       0       3       3
  ##  I3_Cat1       1       2       0       2
  ##  I4_Cat1       1       1       1       0

#############################################################################
# EXAMPLE 3: Garner and Engelhard (2002) toy example polytomous data
#############################################################################

dat <- scan()
   2 2 1 1 1   2 1 2 0 0   1 0 0 0 0   0 1 1 2 0   1 2 2 1 1
   2 2 0 2 1   2 2 1 1 0   1 0 1 0 0   2 1 2 2 2   2 1 0 0 1

dat <- matrix( dat, 10, 5, byrow=TRUE)
colnames(dat) <- paste0("I", 1:5 )

# estimate partial credit model with no jackknifing
mod1 <- sirt::rasch.evm.pcm( dat, jackunits=0, powB=3 )

# paired comparison matrix
mod1$B
  ##          I1_Cat1 I1_Cat2 I2_Cat1 I2_Cat2 I3_Cat1 I3_Cat2 I4_Cat1 I4_Cat2 I5_Cat1 I5_Cat2
  ##  I1_Cat1       0       0       2       0       1       1       2       1       2       1
  ##  I1_Cat2       0       0       0       3       2       2       2       2       2       3
  ##  I2_Cat1       1       0       0       0       1       1       2       0       2       1
  ##  I2_Cat2       0       1       0       0       1       2       0       3       1       3
  ##  I3_Cat1       1       1       1       1       0       0       1       2       3       1
  ##  I3_Cat2       0       1       0       2       0       0       1       1       1       1
  ##  I4_Cat1       0       1       0       0       0       2       0       0       1       2
  ##  I4_Cat2       1       0       0       2       1       1       0       0       1       1
  ##  I5_Cat1       0       1       0       1       2       1       1       2       0       0
  ##  I5_Cat2       0       0       0       1       0       0       0       0       0       0

#############################################################################
# EXAMPLE 4: Partial credit model for dataset data.mg from CDM package
#############################################################################

library(CDM)
data(data.mg,package="CDM")
dat <- data.mg[, paste0("I",1:11) ]

#*** Model 1: estimate partial credit model
mod1 <- sirt::rasch.evm.pcm( dat )
# item parameters
round( mod1$b, 3 )
  ##        Cat1   Cat2  Cat3
  ##  I1  -1.537     NA    NA
  ##  I2  -2.360     NA    NA
  ##  I3  -0.574     NA    NA
  ##  I4  -0.971 -2.086    NA
  ##  I5  -0.104  0.201    NA
  ##  I6   0.470  0.806    NA
  ##  I7  -1.027  0.756 1.969
  ##  I8   0.897     NA    NA
  ##  I9   0.766     NA    NA
  ##  I10  0.069     NA    NA
  ##  I11 -1.122  1.159 2.689

#*** Model 2: estimate PCM with pairwise package
miceadds::library_install("pairwise")
mod2 <- pairwise::pair(daten=dat)
summary(mod2)
plot(mod2)
# compute standard errors
semod2 <- pairwise::pairSE(daten=dat,  nsample=20)
semod2

#############################################################################
# EXAMPLE 5: Differential item functioning for dataset data.mg
#############################################################################

library(CDM)
data(data.mg,package="CDM")
dat <- data.mg[ data.mg$group \%in\% c(2,3,11), ]
# define items
items <- paste0("I",1:11)
# estimate model
mod1 <- sirt::rasch.evm.pcm( dat[,items], weights=dat$weight, group=dat$group )
summary(mod1)

#############################################################################
# EXAMPLE 6: Differential item functioning for Rasch model
#############################################################################

# simulate some data
set.seed(9776)
N <- 1000    # number of persons
I <- 10        # number of items
# simulate data for first group
b <- seq(-1.5,1.5,len=I)
dat1 <- sirt::sim.raschtype( stats::rnorm(N), b )
# simulate data for second group
b1 <- b
b1[4] <- b1[4] + .5 # introduce DIF for fourth item
dat2 <- sirt::sim.raschtype( stats::rnorm(N,mean=.3), b1 )
dat <- rbind(dat1, dat2 )
group <- rep( 1:2, each=N )
# estimate model
mod1 <- sirt::rasch.evm.pcm( dat, group=group )
summary(mod1)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Eigenvector method}
%% \keyword{Pairwise estimation}
%% \keyword{summary}
%% \keyword{coef}
%% \keyword{vcov}

%% File Name: rasch.pairwise.itemcluster.Rd
%% File Version: 0.453

\name{rasch.pairwise.itemcluster}
\alias{rasch.pairwise.itemcluster}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Pairwise Estimation of the Rasch Model for Locally Dependent Items
}
\description{
This function uses pairwise conditional likelihood estimation
for estimating item parameters in the Rasch model.
}
\usage{
rasch.pairwise.itemcluster(dat, itemcluster=NULL, b.fixed=NULL, weights=NULL,
    conv=1e-05, maxiter=3000, progress=TRUE, b.init=NULL, zerosum=FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{An \eqn{N \times I} data frame.
  Missing responses are allowed and must be recoded as \code{NA}.
}
  \item{itemcluster}{
Optional integer vector of itemcluster (see Examples). Different
integers correspond to different item clusters. No item cluster
is set as default.
}
  \item{b.fixed}{
Matrix for fixing item parameters. The first columns contains the item
(number or name), the second column the parameter to be fixed.
}
\item{weights}{Optional Vector of sampling weights}
  \item{conv}{
Convergence criterion in maximal absolute parameter change
}
  \item{maxiter}{
Maximal number of iterations
}
  \item{progress}{
A logical which displays progress. Default is \code{TRUE}.
}
  \item{b.init}{
Vector of initial item difficulty estimates. Default is \code{NULL}.
}
\item{zerosum}{Optional logical indicating whether item difficulties
    should be centered in each iteration. The default is that
        no centering is conducted.}

}
\details{
This is an adaptation of the algorithm of van der Linden and Eggen (1986). Only item pairs
of different item clusters are taken into account for item difficulty estimation.
Therefore, the problem of locally dependent items within each itemcluster is (almost)
eliminated (see Examples below) because contributions of local dependencies
do not appear in the pairwise likelihood terms. In detail, the estimation rests
on observed frequency tables of items \eqn{i} and \eqn{j} and therefore on conditional
probabilities
\deqn{ \frac{P(X_i=x, X_j=y)}{P(X_i + X_j=1 )} \quad \mbox{with}
\quad x,y=0,1 \quad \mbox{and} \quad x+y=1 }
If for some item pair \eqn{(i,j)} a higher positive (or negative) correlation
is expected (i.e. deviation from local dependence), then this pair is
removed from estimation. Clearly, there is a loss in precision but item
parameters can be less biased.
}
\value{
Object of class \code{rasch.pairwise} with elements
  \item{b}{
    Vector of item difficulties
         }
 \item{item}{
        Data frame of item parameters (\eqn{N}, \eqn{p} and item difficulty)
            }
}
\references{
van der Linden, W. J., & Eggen, T. J. H. M. (1986). \emph{An empirical
Bayes approach to item banking}. Research Report 86-6, University of Twente.

Zwinderman, A. H. (1995). Pairwise parameter estimation in Rasch models.
\emph{Applied Psychological Measurement, 19}, 369-375.
}



%\author{
%Alexander Robitzsch
%}


\note{
No standard errors are provided by this function. Use resampling
methods for conducting statistical inference.

Formulas for asymptotic standard errors of this pairwise estimation method are
described in Zwinderman (1995).
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{rasch.pairwise}}, \code{\link{summary.rasch.pairwise}},

Pairwise marginal likelihood estimation (also labeled as pseudolikelihood
estimation) can be conducted with \code{\link{rasch.pml3}}.

Other estimation methods are implemented in \code{\link{rasch.copula2}} or
\code{\link{rasch.mml2}}.

For simulation of locally dependent data see \code{\link{sim.rasch.dep}}.
}


\examples{
#############################################################################
# EXAMPLE 1: Example with locally dependent items
#      12 Items: Cluster 1 -> Items 1,...,4
#                Cluster 2 -> Items 6,...,9
#                Cluster 3 -> Items 10,11,12
#############################################################################

set.seed(7896)
I <- 12                             # number of items
n <- 5000                           # number of persons
b <- seq(-2,2, len=I)               # item difficulties
bsamp <- b <- sample(b)             # sample item difficulties
theta <- stats::rnorm( n, sd=1 ) # person abilities
# itemcluster
itemcluster <- rep(0,I)
itemcluster[ 1:4 ] <- 1
itemcluster[ 6:9 ] <- 2
itemcluster[ 10:12 ] <- 3
# residual correlations
rho <- c( .55, .25, .45 )

# simulate data
dat <- sirt::sim.rasch.dep( theta, b, itemcluster, rho )
colnames(dat) <- paste("I", seq(1,ncol(dat)), sep="")

# estimation with pairwise Rasch model
mod3 <- sirt::rasch.pairwise( dat )
summary(mod3)

# use item cluster in rasch pairwise estimation
mod <- sirt::rasch.pairwise.itemcluster( dat=dat, itemcluster=itemcluster )
summary(mod)

\dontrun{
# Rasch MML estimation
mod4 <- sirt::rasch.mml2( dat )
summary(mod4)

# Rasch Copula estimation
mod5 <- sirt::rasch.copula2( dat, itemcluster=itemcluster )
summary(mod5)

# compare different item parameter estimates
M1 <- cbind( "true.b"=bsamp, "b.rasch"=mod4$item$b, "b.rasch.copula"=mod5$item$thresh,
         "b.rasch.pairwise"=mod3$b, "b.rasch.pairwise.cluster"=mod$b )
# center item difficulties
M1 <- scale( M1, scale=FALSE )
round( M1, 3 )
round( apply( M1, 2, stats::sd ), 3 )

#  Below the output of the example is presented.
#  The rasch.pairwise.itemcluster is pretty close to the estimate in the Rasch copula model.

  ##   > round( M1, 3 )
  ##       true.b b.rasch b.rasch.copula b.rasch.pairwise b.rasch.pairwise.cluster
  ##   I1   0.545   0.561          0.526            0.628                    0.524
  ##   I2  -0.182  -0.168         -0.174           -0.121                   -0.156
  ##   I3  -0.909  -0.957         -0.867           -0.971                   -0.899
  ##   I4  -1.636  -1.726         -1.625           -1.765                   -1.611
  ##   I5   1.636   1.751          1.648            1.694                    1.649
  ##   I6   0.909   0.892          0.836            0.898                    0.827
  ##   I7  -2.000  -2.134         -2.020           -2.051                   -2.000
  ##   I8  -1.273  -1.355         -1.252           -1.303                   -1.271
  ##   I9  -0.545  -0.637         -0.589           -0.581                   -0.598
  ##   I10  1.273   1.378          1.252            1.308                    1.276
  ##   I11  0.182   0.241          0.226            0.109                    0.232
  ##   I12  2.000   2.155          2.039            2.154                    2.026
  ##   > round( apply( M1, 2, sd ), 3 )
  ##                     true.b                  b.rasch           b.rasch.copula
  ##                      1.311                    1.398                    1.310
  ##      b.rasch.pairwise    b.rasch.pairwise.cluster
  ##                 1.373                       1.310

# set item parameters of first item to 0 and of second item to -0.7
b.fixed <- cbind( c(1,2), c(0,-.7) )
mod5 <- sirt::rasch.pairwise.itemcluster( dat=dat, b.fixed=b.fixed,
             itemcluster=itemcluster )
# difference between estimations 'mod' and 'mod5'
dfr <- cbind( mod5$item$b, mod$item$b )
plot( mod5$item$b, mod$item$b, pch=16)
apply( dfr, 1, diff)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Pairwise conditional maximum likelihood (PCML)}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
%% \keyword{Local dependence}

%% File Name: xxirt_createParTable.Rd
%% File Version: 1.172

\name{xxirt_createParTable}
\alias{xxirt_createParTable}
\alias{xxirt_createDiscItem}
\alias{xxirt_modifyParTable}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Create Item Response Functions and Item Parameter Table
}
\description{
Create item response functions and item parameter table
}

\usage{
xxirt_createDiscItem( name, par, est, P, lower=-Inf, upper=Inf,
     prior=NULL, prior_par1=NULL, prior_par2=NULL, X=NULL)

xxirt_createParTable(dat, itemtype, customItems=NULL)

xxirt_modifyParTable( partable, parname, item=NULL, value=NULL,
     est=NULL, parlabel=NULL, parindex=NULL, lower=NULL,
     upper=NULL, prior=NULL, prior_par1=NULL, prior_par2=NULL )
}



%- maybe also 'usage' for other objects documented here.
\arguments{
\item{name}{Type of item response function}
\item{par}{Named vector of starting values of item parameters}
\item{est}{Logical vector indicating which parameters should be estimated}
\item{P}{Item response function}
\item{lower}{Lower bounds}
\item{upper}{Upper bounds}
\item{prior}{Prior distribution}
\item{prior_par1}{First parameter prior distribution}
\item{prior_par2}{Second parameter prior distribution}
\item{X}{optional matrix of person covariates}
  \item{dat}{
Data frame with item responses
}
  \item{itemtype}{
Vector of item types
}
 \item{customItems}{
List with item objects created by \code{xxirt_createDiscItem}
}
\item{partable}{Item parameter table}
\item{parname}{Parameter name}
\item{item}{Item}
\item{value}{Value of item parameter}
\item{parindex}{Parameter index}
\item{parlabel}{Item parameter label}
}

% xxirt_modifyParTable( partable, parname, item=NULL, value=NULL,
%     est=NULL, parlabel=NULL, parindex=NULL, lower=NULL,
%     upper=NULL, prior=NULL, prior_par1=NULL, prior_par2=NULL )


%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%}
%\references{
%% ~put references to the literature/web site here ~
%}

%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{xxirt}}

See \code{\link[mirt:createItem]{mirt::createItem}} for similar
functionality.
}

\examples{
#############################################################################
## EXAMPLE 1: Definition of item response functions
#############################################################################

data(data.read)
dat <- data.read

#------ Definition of item response functions
#*** IRF 2PL
P_2PL <- function( par, Theta, ncat){
    a <- par[1]
    b <- par[2]
    TP <- nrow(Theta)
    P <- matrix( NA, nrow=TP, ncol=ncat)
    P[,1] <- 1
    for (cc in 2:ncat){
        P[,cc] <- exp( (cc-1) * a * Theta[,1] - b )
    }
    P <- P / rowSums(P)
    return(P)
}

#*** IRF 1PL
P_1PL <- function( par, Theta, ncat){
    b <- par[1]
    TP <- nrow(Theta)
    par0 <- c(1,b)
    P <- P_2PL( par=par0, Theta=Theta, ncat=ncat)
    return(P)
}

#** created item classes of 1PL and 2PL models
par <- c( "a"=1, "b"=0 )
# define some slightly informative prior of 2PL
item_2PL <- sirt::xxirt_createDiscItem( name="2PL", par=par, est=c(TRUE,TRUE),
                P=P_2PL, prior=c( a="dlnorm"), prior_par1=c(a=0),
                prior_par2=c(a=5) )
item_1PL <- sirt::xxirt_createDiscItem( name="1PL", par=par[2], est=c(TRUE),
                P=P_1PL )
# list of item classes in customItems
customItems <- list( item_1PL,  item_2PL )

#-- create parameter table
itemtype <- rep( "1PL", 12 )
partable <- sirt::xxirt_createParTable(dat, itemtype=itemtype, customItems=customItems)
# privide starting values
partable1 <- sirt::xxirt_modifyParTable( partable, parname="b",
                   value=- stats::qlogis( colMeans(dat) ) )
# equality constraint of parameters and definition of lower bounds
partable1 <- sirt::xxirt_modifyParTable( partable1, item=c("A1","A2"),
                parname="b", parindex=110, lower=-1, value=0)
print(partable1)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: matrixfunctions.sirt.Rd
%% File Version: 1.162

\name{matrixfunctions.sirt}

\alias{rowMaxs.sirt}
\alias{rowMins.sirt}
\alias{rowCumsums.sirt}
\alias{colCumsums.sirt}
\alias{rowIntervalIndex.sirt}
\alias{rowKSmallest.sirt}
\alias{rowKSmallest2.sirt}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Some Matrix Functions
}
\description{
Some matrix functions which are written in \pkg{Rcpp} for speed
reasons.
}
\usage{
rowMaxs.sirt(matr)      # rowwise maximum

rowMins.sirt(matr)      # rowwise minimum

rowCumsums.sirt(matr)   # rowwise cumulative sum

colCumsums.sirt(matr)   # columnwise cumulative sum

rowIntervalIndex.sirt(matr,rn) # first index in row nn when matr(nn,zz) > rn(nn)

rowKSmallest.sirt(matr, K, break.ties=TRUE) # k smallest elements in a row
rowKSmallest2.sirt(matr, K )
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{matr}{
A numeric matrix
}
\item{rn}{A vector, usually a random number in applications}
\item{K}{An integer indicating the number of smallest elements to be
extracted}
\item{break.ties}{A logical which indicates if ties are randomly
    broken. The default is \code{TRUE}.
}
}

\details{
The function \code{rowIntervalIndex.sirt} searches for all rows \code{n}
the first index \code{i} for which \code{matr(n,i) > rn(n)} holds.

The functions \code{rowKSmallest.sirt} and \code{rowKSmallest2.sirt}
extract the \eqn{K} smallest entries in a matrix row. For small
numbers of \eqn{K} the function \code{rowKSmallest2.sirt} is
the faster one.
}

\value{
The output of \code{rowMaxs.sirt} is a list with the elements
\code{maxval} (rowwise maximum values) and \code{maxind} (rowwise
maximum indices). The output of \code{rowMins.sirt} contains
corresponding minimum values with entries \code{minval} and
\code{minind}.

The output of \code{rowKSmallest.sirt} are two matrices:
\code{smallval} contains the \eqn{K} smallest values whereas
\code{smallind} contains the \eqn{K} smallest indices.
}


%\references{
%% ~put references to the literature/web site here ~
%}


\author{
Alexander Robitzsch

The \pkg{Rcpp} code for \code{rowCumsums.sirt} is copied from code of
Romain Francois.
}

%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For other matrix functions see the \pkg{matrixStats} package.
}


\examples{
#############################################################################
# EXAMPLE 1: a small toy example (I)
#############################################################################
set.seed(789)
N1 <- 10 ; N2 <- 4
M1 <- round( matrix( runif(N1*N2), nrow=N1, ncol=N2), 1 )

rowMaxs.sirt(M1)      # rowwise maximum
rowMins.sirt(M1)      # rowwise minimum
rowCumsums.sirt(M1)   # rowwise cumulative sum

# row index for exceeding a certain threshold value
matr <- M1
matr <- matr / rowSums( matr )
matr <- sirt::rowCumsums.sirt( matr )
rn <- runif(N1)    # generate random numbers
rowIntervalIndex.sirt(matr,rn)

# select the two smallest values
rowKSmallest.sirt(matr=M1, K=2)
rowKSmallest2.sirt(matr=M1, K=2)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Matrix utilities}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.inv4gr.Rd
%% File Version: 0.08

\name{data.inv4gr}
\alias{data.inv4gr}
\docType{data}
\title{
Dataset for Invariance Testing with 4 Groups
}
\description{
Dataset for invariance testing with 4 groups.
}
\usage{
data(data.inv4gr)
}

\format{
  A data frame with 4000 observations on the following 12 variables.
  The first variable is a group identifier, the other variables are items.
  \describe{
    \item{\code{group}}{A group identifier}
    \item{\code{I01}}{a numeric vector}
    \item{\code{I02}}{a numeric vector}
    \item{\code{I03}}{a numeric vector}
    \item{\code{I04}}{a numeric vector}
    \item{\code{I05}}{a numeric vector}
    \item{\code{I06}}{a numeric vector}
    \item{\code{I07}}{a numeric vector}
    \item{\code{I08}}{a numeric vector}
    \item{\code{I09}}{a numeric vector}
    \item{\code{I10}}{a numeric vector}
    \item{\code{I11}}{a numeric vector}
  }
}

%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}

\source{
Simulated dataset
}

%\references{
%%  ~~ possibly secondary sources and usages ~~
%}



%\examples{
%data(data.inv4gr)
%## maybe str(data.inv4gr) ; plot(data.inv4gr) ...
%}
%% \keyword{datasets}

%% File Name: person.parameter.rasch.copula.Rd
%% File Version: 1.202

\name{person.parameter.rasch.copula}
\alias{person.parameter.rasch.copula}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Person Parameter Estimation of the Rasch Copula Model (Braeken, 2011)
}
\description{
Ability estimates as maximum likelihood estimates (MLE)
are provided by the Rasch copula model.
}
\usage{
person.parameter.rasch.copula(raschcopula.object, numdiff.parm=0.001,
    conv.parm=0.001, maxiter=20, stepwidth=1,
    print.summary=TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{raschcopula.object}{
Object which is generated by the \code{rasch.copula2} function.
}
  \item{numdiff.parm}{
Parameter \eqn{h} for numerical differentiation
}
  \item{conv.parm}{
Convergence criterion
}
  \item{maxiter}{
Maximum number of iterations
}
  \item{stepwidth}{
Maximal increment in iterations
}
  \item{print.summary}{
Print summary?
}
  \item{\dots}{
Further arguments to be passed
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list with following entries
\item{person}{Estimated person parameters}
\item{se.inflat}{Inflation of individual standard errors due
to local dependence}
\item{theta.table}{Ability estimates for each unique response pattern}
\item{pattern.in.data}{Item response pattern}
\item{summary.theta.table}{Summary statistics of person parameter estimates}
}

%\references{
%% ~put references to the literature/web site here ~
%}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See \code{\link{rasch.copula2}} for estimating Rasch copula models.
}
\examples{
#############################################################################
# EXAMPLE 1: Reading Data
#############################################################################

data(data.read)
dat <- data.read

# define item cluster
itemcluster <- rep( 1:3, each=4 )
mod1 <- sirt::rasch.copula2( dat, itemcluster=itemcluster )
summary(mod1)

# person parameter estimation under the Rasch copula model
pmod1 <- sirt::person.parameter.rasch.copula(raschcopula.object=mod1 )
## Mean percentage standard error inflation
##   missing.pattern Mperc.seinflat
## 1               1           6.35

\dontrun{
#############################################################################
# EXAMPLE 2: 12 items nested within 3 item clusters (testlets)
#   Cluster 1 -> Items 1-4; Cluster 2 -> Items 6-9;  Cluster 3 -> Items 10-12
#############################################################################

set.seed(967)
I <- 12                             # number of items
n <- 450                            # number of persons
b <- seq(-2,2, len=I)               # item difficulties
b <- sample(b)                      # sample item difficulties
theta <- stats::rnorm( n, sd=1 ) # person abilities
# itemcluster
itemcluster <- rep(0,I)
itemcluster[ 1:4 ] <- 1
itemcluster[ 6:9 ] <- 2
itemcluster[ 10:12 ] <- 3
# residual correlations
rho <- c( .35, .25, .30 )

# simulate data
dat <- sirt::sim.rasch.dep( theta, b, itemcluster, rho )
colnames(dat) <- paste("I", seq(1,ncol(dat)), sep="")

# estimate Rasch copula model
mod1 <- sirt::rasch.copula2( dat, itemcluster=itemcluster )
summary(mod1)

# person parameter estimation under the Rasch copula model
pmod1 <- sirt::person.parameter.rasch.copula(raschcopula.object=mod1 )
  ## Mean percentage standard error inflation
  ##   missing.pattern Mperc.seinflat
  ## 1               1          10.48
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{IRT copula models}
%% \keyword{Person parameter estimation}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: isop.scoring.Rd
%% File Version: 1.26

\name{isop.scoring}
\alias{isop.scoring}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Scoring Persons and Items in the ISOP Model
}
\description{
This function does the scoring in the isotonic probabilistic
model (Scheiblechner, 1995, 2003, 2007).
Person parameters are ordinally scaled but the ISOP
model also allows \emph{specific objective} (ordinal) comparisons
for persons (Scheiblechner, 1995).
}
\usage{
isop.scoring(dat,score.itemcat=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame with dichotomous or polytomous item responses
}
\item{score.itemcat}{
Optional data frame with scoring points for every item and
every category (see Example 2).
}
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DETAILS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\details{
This function extracts the scoring rule of the ISOP model
(if \code{score.itemcat !=NULL}) and calculates the
modified percentile score for every person. The score
\eqn{s_{ik}} for item \eqn{i} and category \eqn{k}
is calculated as
\deqn{ s_{ik}=\sum_{j=0}^{k-1} f_{ij} -
\sum_{j=k+1}^K f_{ij}=P( X_i < k  )
- P( X_i > k ) }
where \eqn{f_{ik}} is the relative frequency of item \eqn{i}
in category \eqn{k} and \eqn{K} is the maximum category.
The modified percentile score \eqn{\rho_p}
for subject \eqn{p} (\code{mpsc} in \code{person}) is
defined by
\deqn{ \rho_p=\frac{1}{I} \sum_{i=1}^I
        \sum_{j=0}^K s_{ik} \mathbf{1}( X_{pi}=k ) }
Note that for dichotomous items, the sum score is a
sufficient statistic for \eqn{\rho_p} but this is
not the case for polytomous items.
The modified percentile score \eqn{\rho_p}
ranges between -1 and 1.

The modified item P-score \eqn{\rho_i} (Scheiblechner, 2007, p. 52) is
defined by
\deqn{ \rho_i=\frac{1}{I-1} \cdot \sum_j \left[ P( X_j < X_i )
        - P( X_j > X_i ) \right ] }
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VALUES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\value{
A list with following entries:
  \item{person}{
  A data frame with person parameters. The modified
  percentile score \eqn{\rho_p} is denoted by \code{mpsc}.
      }
  \item{item}{Item statistics and scoring parameters.
    The item P-scores \eqn{\rho_i}
   are labeled as \code{pscore}.}
  \item{p.itemcat}{Frequencies for every item category}
  \item{score.itemcat}{
        Scoring points for every item category
                }
  \item{distr.fct}{Empirical distribution function}
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% REFERENCES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\references{
Scheiblechner, H. (1995). Isotonic ordinal
probabilistic models (ISOP). \emph{Psychometrika,
60}, 281-304.

Scheiblechner, H. (2003). \emph{Nonparametric IRT:
Scoring functions and ordinal parameter estimation
of isotonic probabilistic models (ISOP)}.
Technical Report, Philipps-Universitaet Marburg.

Scheiblechner, H. (2007). A unified nonparametric IRT model for
d-dimensional psychological test data (d-ISOP).
\emph{Psychometrika, 72}, 43-67.
}



%\author{
%Alexander Robitzsch
%}

%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For fitting the ISOP and ADISOP model see
\code{\link{isop.dich}} or \code{\link{fit.isop}}.
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EXAMPLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\examples{
#############################################################################
# EXAMPLE 1: Dataset Reading
#############################################################################

data( data.read )
dat <- data.read

# Scoring according to the ISOP model
msc <- sirt::isop.scoring( dat )
# plot student scores
boxplot( msc$person$mpsc ~ msc$person$score )

#############################################################################
# EXAMPLE 2: Dataset students from CDM package | polytomous items
#############################################################################

library("CDM")
data( data.Students, package="CDM")
dat <- stats::na.omit(data.Students[, -c(1:2) ])

# Scoring according to the ISOP model
msc <- sirt::isop.scoring( dat )
# plot student scores
boxplot( msc$person$mpsc ~ msc$person$score )

# scoring with known scoring rule for activity items
items <- paste0( "act", 1:5 )
score.itemcat <- msc$score.itemcat
score.itemcat <- score.itemcat[ items, ]
msc2 <- sirt::isop.scoring( dat[,items], score.itemcat=score.itemcat )
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{ISOP model}
%% \keyword{Nonparametric IRT}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: nedelsky.sim.Rd
%% File Version: 0.25

\name{nedelsky-methods}

\alias{nedelsky.sim}
\alias{nedelsky.latresp}
\alias{nedelsky.irf}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Functions for the Nedelsky Model
}

\description{
Functions for simulating and estimating the Nedelsky model
(Bechger et al., 2003, 2005). \code{nedelsky.sim} can be used for
simulating the model, \code{nedelsky.irf} computes the item response
function and can be used for example when estimating the
Nedelsky model in the \pkg{mirt} package or using the
\code{\link{xxirt}} function in the \pkg{sirt} package.
}

\usage{
# simulating the Nedelsky model
nedelsky.sim(theta, b, a=NULL, tau=NULL)

# creating latent responses of the Nedelsky model
nedelsky.latresp(K)

# computing the item response function of the Nedelsky model
nedelsky.irf(Theta, K, b, a, tau, combis, thdim=1)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{theta}{
Unidimensional ability (theta)
}
  \item{b}{
Matrix of category difficulties
}
  \item{a}{
Vector of item discriminations
}
  \item{tau}{
Category attractivity parameters \eqn{\tau} (see Bechger et al., 2005)
}
\item{K}{(Maximum) Number of distractors of the used multiple choice items}
  \item{Theta}{
Theta vector. Note that the Nedelsky model can be only specified
  as models with between item dimensionality (defined in \code{thdim}).
}
  \item{combis}{
Latent response classes as produced by \code{nedelsky.latresp}.
}
  \item{thdim}{
Theta dimension at which the item loads
}
}

\details{
Assume that for item \eqn{i} there exists \eqn{K+1} categories \eqn{0,1,...,K}.
The category 0 denotes the correct alternative. The Nedelsky model assumes
that a respondent eliminates all distractors which are thought to be
incorrect and guesses the solution from the remaining alternatives.
This means, that for item \eqn{i}, \eqn{K} latent variables \eqn{S_{ik}}
are defined which indicate whether alternative \eqn{k} has been correctly
identified as a distractor. By definition, the correct alternative is never
been judged as wrong by the respondent.

Formally, the Nedelsky model assumes a 2PL model for eliminating each of the
distractors
\deqn{P(S_{ik}=1 | \theta )=invlogit[ a_i ( \theta - b_{ik} ) ] }
where \eqn{\theta} is the person ability and \eqn{b_{ik}} are
distractor difficulties.

The guessing process of the Nedelsky model is defined as
\deqn{P(X_i=j | \theta, S_{i1}, ..., S_{iK} )=
\frac{ ( 1- S_{ij} ) \tau_{ij} }{ \sum_{k=0}^K [ ( 1- S_{ik} ) \tau_{ik} ] }}
where \eqn{\tau_{ij}} are attractivity parameters of alternative \eqn{j}.
By definition \eqn{\tau_{i0}} is set to 1. By default, all attractivity parameters
are set to 1.
}



%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%}


\references{
Bechger, T. M., Maris, G., Verstralen, H. H. F. M., & Verhelst, N. D. (2003).
\emph{The Nedelsky model for multiple-choice items}.
CITO Research Report, 2003-5.

Bechger, T. M., Maris, G., Verstralen, H. H. F. M., & Verhelst, N. D. (2005).
The Nedelsky model for multiple-choice items.
In L. van der Ark, M. Croon, & Sijtsma, K. (Eds.).
\emph{New developments in categorical data analysis for the social and behavioral
sciences}, pp. 187-206. Mahwah, Lawrence Erlbaum.
}



%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}


\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Simulated data according to the Nedelsky model
#############################################################################

#*** simulate data
set.seed(123)
I <- 20          # number of items
b <- matrix(NA,I,ncol=3)
b[,1] <- -0.5 + stats::runif( I, -.75, .75 )
b[,2] <- -1.5 + stats::runif( I, -.75, .75 )
b[,3] <- -2.5 + stats::runif( I, -.75, .75 )
K <- 3           # number of distractors
N <- 2000        # number of persons
# apply simulation function
dat <- sirt::nedelsky.sim( theta=stats::rnorm(N,sd=1.2), b=b )

#*** latent response patterns
K <- 3
combis <- sirt::nedelsky.latresp(K=3)

#*** defining the Nedelsky item response function for estimation in mirt
par <- c( 3, rep(-1,K), 1, rep(1,K+1),1)
names(par) <- c("K", paste0("b",1:K), "a", paste0("tau", 0:K),"thdim")
est <- c( FALSE, rep(TRUE,K), rep(FALSE, K+1 + 2 ) )
names(est) <- names(par)
nedelsky.icc <- function( par, Theta, ncat ){
     K <- par[1]
     b <- par[ 1:K + 1]
     a <- par[ K+2]
     tau <- par[1:(K+1) + (K+2) ]
     thdim <- par[ K+2+K+1 +1 ]
     probs <- sirt::nedelsky.irf( Theta, K=K, b=b, a=a, tau=tau, combis,
                    thdim=thdim  )$probs
     return(probs)
}
name <- "nedelsky"
# create item response function
nedelsky.itemfct <- mirt::createItem(name, par=par, est=est, P=nedelsky.icc)

#*** define model in mirt
mirtmodel <- mirt::mirt.model("
           F1=1-20
           COV=F1*F1
           # define some prior distributions
           PRIOR=(1-20,b1,norm,-1,2),(1-20,b2,norm,-1,2),
                   (1-20,b3,norm,-1,2)
        " )

itemtype <- rep("nedelsky", I )
customItems <- list("nedelsky"=nedelsky.itemfct)
# define parameters to be estimated
mod1.pars <- mirt::mirt(dat, mirtmodel, itemtype=itemtype,
                   customItems=customItems, pars="values")
# estimate model
mod1 <- mirt::mirt(dat,mirtmodel, itemtype=itemtype, customItems=customItems,
               pars=mod1.pars, verbose=TRUE  )
# model summaries
print(mod1)
summary(mod1)
mirt.wrapper.coef( mod1 )$coef
mirt.wrapper.itemplot(mod1,ask=TRUE)

#******************************************************
# fit Nedelsky model with xxirt function in sirt

# define item class for xxirt
item_nedelsky <- sirt::xxirt_createDiscItem( name="nedelsky", par=par,
                est=est, P=nedelsky.icc,
                prior=c( b1="dnorm", b2="dnorm", b3="dnorm" ),
                prior_par1=c( b1=-1, b2=-1, b3=-1),
                prior_par2=c(b1=2, b2=2, b3=2) )
customItems <- list( item_nedelsky )

#---- definition theta distribution
#** theta grid
Theta <- matrix( seq(-6,6,length=21), ncol=1 )
#** theta distribution
P_Theta1 <- function( par, Theta, G){
    mu <- par[1]
    sigma <- max( par[2], .01 )
    TP <- nrow(Theta)
    pi_Theta <- matrix( 0, nrow=TP, ncol=G)
    pi1 <- dnorm( Theta[,1], mean=mu, sd=sigma )
    pi1 <- pi1 / sum(pi1)
    pi_Theta[,1] <- pi1
    return(pi_Theta)
}
#** create distribution class
par_Theta <- c( "mu"=0, "sigma"=1 )
customTheta <- sirt::xxirt_createThetaDistribution( par=par_Theta, est=c(FALSE,TRUE),
                   P=P_Theta1 )

#-- create parameter table
itemtype <- rep( "nedelsky", I )
partable <- sirt::xxirt_createParTable( dat, itemtype=itemtype, customItems=customItems)

# estimate model
mod2 <- sirt::xxirt( dat=dat, Theta=Theta, partable=partable, customItems=customItems,
                    customTheta=customTheta)
summary(mod2)
# compare sirt::xxirt and mirt::mirt
logLik(mod2)
mod1@Fit$logLik

#############################################################################
# EXAMPLE 2: Multiple choice dataset data.si06
#############################################################################

data(data.si06)
dat <- data.si06

#*** create latent responses
combis <- sirt::nedelsky.latresp(K)
I <- ncol(dat)
#*** define item response function
K <- 3
par <- c( 3, rep(-1,K), 1, rep(1,K+1),1)
names(par) <- c("K", paste0("b",1:K), "a", paste0("tau", 0:K),"thdim")
est <- c( FALSE, rep(TRUE,K), rep(FALSE, K+1 + 2 ) )
names(est) <- names(par)
nedelsky.icc <- function( par, Theta, ncat ){
     K <- par[1]
     b <- par[ 1:K + 1]
     a <- par[ K+2]
     tau <- par[1:(K+1) + (K+2) ]
     thdim <- par[ K+2+K+1 +1 ]
     probs <- sirt::nedelsky.irf( Theta, K=K, b=b, a=a, tau=tau, combis,
                    thdim=thdim  )$probs
     return(probs)
}
name <- "nedelsky"
# create item response function
nedelsky.itemfct <- mirt::createItem(name, par=par, est=est, P=nedelsky.icc)

#*** define model in mirt
mirtmodel <- mirt::mirt.model("
           F1=1-14
           COV=F1*F1
           PRIOR=(1-14,b1,norm,-1,2),(1-14,b2,norm,-1,2),
                   (1-14,b3,norm,-1,2)
        " )

itemtype <- rep("nedelsky", I )
customItems <- list("nedelsky"=nedelsky.itemfct)
# define parameters to be estimated
mod1.pars <- mirt::mirt(dat, mirtmodel, itemtype=itemtype,
                   customItems=customItems, pars="values")

#*** estimate model
mod1 <- mirt::mirt(dat,mirtmodel, itemtype=itemtype, customItems=customItems,
               pars=mod1.pars, verbose=TRUE )
#*** summaries
print(mod1)
summary(mod1)
mirt.wrapper.coef( mod1 )$coef
mirt.wrapper.itemplot(mod1,ask=TRUE)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.

%% \keyword{Nedelsky model}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.math.Rd
%% File Version: 0.15

\name{data.math}
\alias{data.math}
\docType{data}
\title{
Dataset Mathematics
}
\description{
This is an example dataset involving Mathematics items for
German fourth graders. Items are classified into several domains and
subdomains (see Section Format).
The dataset contains 664 students on 30 items.
}
\usage{data(data.math)}
\format{
The dataset is a list. The list element \code{data}
contains the dataset with the demographic variables
student ID (\code{idstud}) and a dummy variable
for female students (\code{female}). The remaining
variables (starting with \code{M} in the name) are
the mathematics items. \cr
The item metadata are included in the list element
\code{item} which contains item name (\code{item}) and the
testlet label (\code{testlet}). An item not included
in a testlet is indicated by \code{NA}.
Each item is allocated to one and only competence domain (\code{domain}).
\cr

The format is:

\code{List of 2} \cr
\code{ $ data:'data.frame':} \cr
\code{  ..$ idstud: int [1:664] 1001 1002 1003 ...} \cr
\code{  ..$ female: int [1:664] 1 1 0 0 1 1 1 0 0 1 ...} \cr
\code{  ..$ MA1   : int [1:664] 1 1 1 0 0 1 1 1 1 1 ...} \cr
\code{  ..$ MA2   : int [1:664] 1 1 1 1 1 0 0 0 0 1 ...} \cr
\code{  ..$ MA3   : int [1:664] 1 1 0 0 0 0 0 1 0 0 ...} \cr
\code{  ..$ MA4   : int [1:664] 0 1 1 1 0 0 1 0 0 0 ...} \cr
\code{  ..$ MB1   : int [1:664] 0 1 0 1 0 0 0 0 0 1 ...} \cr
\code{  ..$ MB2   : int [1:664] 1 1 1 1 0 1 0 1 0 0 ...} \cr
\code{  ..$ MB3   : int [1:664] 1 1 1 1 0 0 0 1 0 1 ...} \cr
\code{  [...]} \cr
\code{  ..$ MH3   : int [1:664] 1 1 0 1 0 0 1 0 1 0 ...} \cr
\code{  ..$ MH4   : int [1:664] 0 1 1 1 0 0 0 0 1 0 ...} \cr
\code{  ..$ MI1   : int [1:664] 1 1 0 1 0 1 0 0 1 0 ...} \cr
\code{  ..$ MI2   : int [1:664] 1 1 0 0 0 1 1 0 1 1 ...} \cr
\code{  ..$ MI3   : int [1:664] 0 1 0 1 0 0 0 0 0 0 ...} \cr
\code{ $ item:'data.frame':} \cr
\code{  ..$ item     : Factor w/ 30 levels "MA1","MA2","MA3",..: 1 2 3 4 5 ...} \cr
\code{  ..$ testlet  : Factor w/ 9 levels "","MA","MB","MC",..: 2 2 2 2 3 3  ...} \cr
\code{  ..$ domain   : Factor w/ 3 levels "arithmetic","geometry",..: 1 1 1  ...} \cr
\code{  ..$ subdomain: Factor w/ 9 levels "","addition",..: 2 2 2 2 7 7  ...} \cr
}





%% \keyword{datasets}

%% File Name: data.pisa2006Read.Rd
%% File Version: 0.01

\name{data.pisa2006Read}
\alias{data.pisa2006Read}
\docType{data}
\title{
Dataset PISA 2006 Reading
}
\description{
This is the PISA 2006 dataset of item responses from 26 OECD countries.
}
\usage{
data(data.pisa2006Read)
}


\format{
The dataset includes item responses, student weights and replicate weights.
}



%% \keyword{datasets}

%% File Name: categorize.Rd
%% File Version: 0.17

\name{categorize}
\alias{categorize}
\alias{decategorize}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Categorize and Decategorize Variables in a Data Frame
}

\description{
The function \code{categorize} defines categories for variables in
a data frame, starting with a user-defined index (e.g. 0 or 1).
Continuous variables can be categorized by defining categories by
discretizing the variables in different quantile groups.

The function \code{decategorize} does the reverse operation.
}


\usage{
categorize(dat, categorical=NULL, quant=NULL, lowest=0)

decategorize(dat, categ_design=NULL)
}
%- maybe also 'usage' for other objects documented here.

\arguments{
  \item{dat}{
Data frame
}
  \item{categorical}{
Vector with variable names which should be converted into categories,
beginning with integer \code{lowest}
}
\item{quant}{Vector with number of classes for each variables.
    Variables are categorized among quantiles. The vector must
have names containing variable names.}
  \item{lowest}{
Lowest category index. Default is 0.
}
\item{categ_design}{Data frame containing informations about
categorization which is the output of \code{categorize}.}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

\value{
For \code{categorize}, it is a list with entries
  \item{data}{Converted data frame}
  \item{categ_design}{Data frame containing some informations
    about categorization}

For \code{decategorize} it is a data frame.
}

%\references{
%% ~put references to the literature/web site here ~
%}
%\author{
%Alexander Robitzsch
%}

%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~


%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}

\examples{
\dontrun{
library(mice)
library(miceadds)

#############################################################################
# EXAMPLE 1: Categorize questionnaire data
#############################################################################

data(data.smallscale, package="miceadds")
dat <- data.smallscale

# (0) select dataset
dat <- dat[, 9:20 ]
summary(dat)
categorical <- colnames(dat)[2:6]

# (1) categorize data
res <- sirt::categorize( dat, categorical=categorical )

# (2) multiple imputation using the mice package
dat2 <- res$data
VV <- ncol(dat2)
impMethod <- rep( "sample", VV )    # define random sampling imputation method
names(impMethod) <- colnames(dat2)
imp <- mice::mice( as.matrix(dat2), impMethod=impMethod, maxit=1, m=1 )
dat3 <- mice::complete(imp,action=1)

# (3) decategorize dataset
dat3a <- sirt::decategorize( dat3, categ_design=res$categ_design )

#############################################################################
# EXAMPLE 2: Categorize ordinal and continuous data
#############################################################################

data(data.ma01,package="miceadds")
dat <- data.ma01
summary(dat[,-c(1:2)] )

# define variables to be categorized
categorical <- c("books", "paredu" )
# define quantiles
quant <-  c(6,5,11)
names(quant) <- c("math", "read", "hisei")

# categorize data
res <- sirt::categorize( dat, categorical=categorical, quant=quant)
str(res)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Utilities}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: eigenvalues.manymatrices.Rd
%% File Version: 0.18

\name{eigenvalues.manymatrices}
\alias{eigenvalues.manymatrices}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Computation of Eigenvalues of Many Symmetric Matrices
}
\description{
This function computes the eigenvalue decomposition of \eqn{N}
symmetric positive definite matrices. The eigenvalues are computed
by the Rayleigh quotient method (Lange, 2010, p. 120). In addition,
the inverse matrix can be calculated.
}
\usage{
eigenvalues.manymatrices(Sigma.all, itermax=10, maxconv=0.001,
    inverse=FALSE )
}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{Sigma.all}{
An \eqn{N \times D^2} matrix containing the \eqn{D^2} entries
of \eqn{N} symmetric matrices of dimension \eqn{D \times D}
}
  \item{itermax}{
Maximum number of iterations
}
  \item{maxconv}{
Convergence criterion for convergence of eigenvectors
}
\item{inverse}{A logical which indicates if the inverse matrix
    shall be calculated}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list with following entries
  \item{lambda}{Matrix with eigenvalues}
  \item{U}{An \eqn{N \times D^2} Matrix of orthonormal eigenvectors}
  \item{logdet}{Vector of logarithm of determinants}
  \item{det}{Vector of determinants}
  \item{Sigma.inv}{Inverse matrix if \code{inverse=TRUE}.}
%% ...
}
\references{
Lange, K. (2010). \emph{Numerical Analysis for Statisticians}.
New York: Springer.
}

%\author{
%Alexander Robitzsch
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{
# define matrices
Sigma <- diag(1,3)
Sigma[ lower.tri(Sigma) ] <- Sigma[ upper.tri(Sigma) ] <- c(.4,.6,.8 )
Sigma1 <- Sigma

Sigma <- diag(1,3)
Sigma[ lower.tri(Sigma) ] <- Sigma[ upper.tri(Sigma) ] <- c(.2,.1,.99 )
Sigma2 <- Sigma

# collect matrices in a "super-matrix"
Sigma.all <- rbind( matrix( Sigma1, nrow=1, byrow=TRUE),
                matrix( Sigma2, nrow=1, byrow=TRUE) )
Sigma.all <- Sigma.all[ c(1,1,2,2,1 ), ]

# eigenvalue decomposition
m1 <- sirt::eigenvalues.manymatrices( Sigma.all )
m1

# eigenvalue decomposition for Sigma1
s1 <- svd(Sigma1)
s1
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Eigenvalues}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: md.pattern.sirt.Rd
%% File Version: 0.20

\name{md.pattern.sirt}
\alias{md.pattern.sirt}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Response Pattern in a Binary Matrix
}
\description{
Computes different statistics of the response pattern in a binary
matrix.
}
\usage{
md.pattern.sirt(dat)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
A binary data matrix
}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

\value{
A list with following entries:
\item{dat}{Original dataset}
\item{dat.resp1}{Indices for responses of 1's}
\item{dat.resp0}{Indices for responses of 0's}
\item{resp_patt}{Vector of response patterns}
\item{unique_resp_patt}{Unique response patterns}
\item{unique_resp_patt_freq}{Frequencies of unique response patterns}
\item{unique_resp_patt_firstobs}{First observation in original dataset
    \code{dat} of a unique response pattern}
\item{freq1}{Frequencies of 1's}
\item{freq0}{Frequencies of 0's}
\item{dat.ordered}{Dataset according to response patterns}
}

%\references{
%% ~put references to the literature/web site here ~
%}



%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also the \code{md.pattern} function in the \pkg{mice} package.
}

\examples{
#############################################################################
# EXAMPLE 1: Response patterns
#############################################################################
set.seed(7654)
N <- 21         # number of rows
I <- 4          # number of columns
dat <- matrix( 1*( stats::runif(N*I) > .3 ), N, I )
res <- sirt::md.pattern.sirt(dat)
# plot of response patterns
res$dat.ordered
image( z=t(res$dat.ordered), y=1:N, x=1:I, xlab="Items", ylab="Persons")
# 0's are yellow and 1's are red

#############################################################################
# EXAMPLE 2: Item response patterns for dataset data.read
#############################################################################

data(data.read)
dat <- data.read  ; N <- nrow(dat) ; I <- ncol(dat)
# order items according to p values
dat <- dat[, order(colMeans(dat, na.rm=TRUE )) ]
# analyzing response pattern
res <- sirt::md.pattern.sirt(dat)
res$dat.ordered
image( z=t(res$dat.ordered), y=1:N, x=1:I, xlab="Items", ylab="Persons")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Utilities}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: dirichlet.mle.Rd
%% File Version: 0.23

\name{dirichlet.mle}
\alias{dirichlet.mle}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Maximum Likelihood Estimation of the Dirichlet Distribution
}
\description{
Maximum likelihood estimation of the parameters of the Dirichlet
distribution
}
\usage{
dirichlet.mle(x, weights=NULL, eps=10^(-5), convcrit=1e-05, maxit=1000,
     oldfac=.3, progress=FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
Data frame with \eqn{N} observations and \eqn{K} variables
of a Dirichlet distribution
}
\item{weights}{Optional vector of frequency weights}
  \item{eps}{
Tolerance number which is added to prevent from logarithms of zero
}
  \item{convcrit}{
Convergence criterion
}
  \item{maxit}{
Maximum number of iterations
}
\item{oldfac}{Convergence acceleration factor. It must be a parameter
between 0 and 1.}
  \item{progress}{
Display iteration progress?
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list with following entries
  \item{alpha}{Vector of \eqn{\alpha} parameters}
  \item{alpha0}{The concentration parameter \eqn{\alpha_0=\sum_k \alpha_k} }
  \item{xsi}{Vector of proportions \eqn{\xi_k=\alpha_k / \alpha_0} }
}
\references{
Minka, T. P. (2012). \emph{Estimating a Dirichlet distribution}.
Technical Report.
}

%\author{
%Alexander Robitzsch
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For simulating Dirichlet vectors with matrix-wise
\eqn{\bold{\alpha}} parameters see \code{\link{dirichlet.simul}}.

For a variety of functions concerning the Dirichlet distribution
see the \pkg{DirichletReg} package.
}

%% ~~objects to See Also as \code{\link{help}}, ~~~

\examples{
#############################################################################
# EXAMPLE 1: Simulate and estimate Dirichlet distribution
#############################################################################

# (1) simulate data
set.seed(789)
N <- 200
probs <- c(.5, .3, .2 )
alpha0 <- .5
alpha <- alpha0*probs
alpha <- matrix( alpha, nrow=N, ncol=length(alpha), byrow=TRUE  )
x <- sirt::dirichlet.simul( alpha )

# (2) estimate Dirichlet parameters
dirichlet.mle(x)
  ##   $alpha
  ##   [1] 0.24507708 0.14470944 0.09590745
  ##   $alpha0
  ##   [1] 0.485694
  ##   $xsi
  ##   [1] 0.5045916 0.2979437 0.1974648

\dontrun{
#############################################################################
# EXAMPLE 2: Fitting Dirichlet distribution with frequency weights
#############################################################################

# define observed data
x <- scan( nlines=1)
    1 0   0 1   .5 .5
x <- matrix( x, nrow=3, ncol=2, byrow=TRUE)

# transform observations x into (0,1)
eps <- .01
x <- ( x + eps ) / ( 1 + 2 * eps )

# compare results with likelihood fitting package maxLik
miceadds::library_install("maxLik")
# define likelihood function
dirichlet.ll <- function(param) {
    ll <- sum( weights * log( ddirichlet( x, param ) ) )
    ll
}

#*** weights 10-10-1
weights <- c(10, 10, 1 )
mod1a <- sirt::dirichlet.mle( x, weights=weights )
mod1a
# estimation in maxLik
mod1b <- maxLik::maxLik(loglik, start=c(.5,.5))
print( mod1b )
coef( mod1b )

#*** weights 10-10-10
weights <- c(10, 10, 10 )
mod2a <- sirt::dirichlet.mle( x, weights=weights )
mod2a
# estimation in maxLik
mod2b <- maxLik::maxLik(loglik, start=c(.5,.5))
print( mod2b )
coef( mod2b )

#*** weights 30-10-2
weights <- c(30, 10, 2 )
mod3a <- sirt::dirichlet.mle( x, weights=weights )
mod3a
# estimation in maxLik
mod3b <- maxLik::maxLik(loglik, start=c(.25,.25))
print( mod3b )
coef( mod3b )
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Dirichlet distribution}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: mle.pcm.group.Rd
%% File Version: 0.25

\name{mle.pcm.group}
\alias{mle.pcm.group}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Maximum Likelihood Estimation of Person or Group Parameters
in the Generalized Partial Credit Model
}
\description{
This function estimates person or group parameters in the
partial credit model (see Details).
}
\usage{
mle.pcm.group(dat, b, a=rep(1, ncol(dat)), group=NULL,
    pid=NULL, adj_eps=0.3, conv=1e-04, maxiter=30)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
A numeric \eqn{N \times I} matrix
}
  \item{b}{
Matrix with item thresholds
}
  \item{a}{
Vector of item slopes
}
  \item{group}{
Vector of group identifiers
}
  \item{pid}{
Vector of person identifiers
}
  \item{adj_eps}{
Numeric value which is used in \eqn{\varepsilon} adjustment
of the likelihood. A value of zero (or a very small
\eqn{\varepsilon>0}) corresponds to the usual maximum
likelihood estimate.
}
  \item{conv}{
Convergence criterion
}
  \item{maxiter}{
Maximum number of iterations
}
}
\details{
It is assumed that the generalized partial credit model holds.
In case one estimates a person parameter \eqn{\theta_p},
the log-likelihood is maximized and the following
estimating equation results: (see Penfield & Bergeron, 2005):
\deqn{ 0=( \log L )'=\sum_i a_i \cdot [ \tilde{x}_{pi} -
    E(X_{pi} | \theta_p ) ] }
where \eqn{E(X_{pi} | \theta_p )} denotes the expected item response
conditionally on \eqn{\theta_p}.

With the method of \eqn{\varepsilon}-adjustment (Bertoli-Barsotti & Punzo, 2012;
Bertoli-Barsotti, Lando & Punzo, 2014),
the observed item responses \eqn{x_{pi}} are transformed such that
no perfect scores arise and bias is reduced. If \eqn{S_p}
is the sum score of person \eqn{p} and \eqn{M_p} the maximum
score of this person, then the transformed sum scores \eqn{\tilde{S}_p}
are
\deqn{ \tilde{S}_p=\varepsilon + \frac{M_p - 2 \varepsilon}{M_p} S_p}
However, the adjustment is directly conducted on item responses
to also handle the case of the generalized partial credit
model with item slope parameters different from 1.

In case one estimates a group parameter \eqn{\theta_g},
the following estimating equation is used:
\deqn{ 0=(\log L )'=\sum_p \sum_i a_i \cdot [ \tilde{x}_{pgi} -
    E(X_{pgi} | \theta_g ) ] }
where persons \eqn{p} are nested within a group \eqn{g}.
The \eqn{\varepsilon}-adjustment is then performed at the
group level, not at the individual level.
}
\value{
A list with following entries:
  \item{person}{Data frame with person or group parameters}
  \item{data_adjeps}{Modified dataset according to the
  \eqn{\varepsilon} adjustment.
  }
}
\references{
Bertoli-Barsotti, L., & Punzo, A. (2012).
Comparison of two bias reduction techniques for the Rasch model.
\emph{Electronic Journal of Applied Statistical Analysis, 5}, 360-366.

Bertoli-Barsotti, L., Lando, T., & Punzo, A. (2014).
Estimating a Rasch Model via fuzzy empirical probability functions.
In D. Vicari, A. Okada, G. Ragozini &  C. Weihs (Eds.).
\emph{Analysis and Modeling of Complex Data in Behavioral and Social Sciences},
Springer.

Penfield, R. D., & Bergeron, J. M. (2005). Applying a weighted
maximum likelihood latent trait estimator to the generalized
partial credit model. \emph{Applied Psychological Measurement,
29}, 218-233.
}

%\author{
%Alexander Robitzsch
%}


%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Estimation of a group parameter for only one item per group
#############################################################################

data(data.si01)
dat <- data.si01
# item parameter estimation (partial credit model) in TAM
library(TAM)
mod <- TAM::tam.mml( dat[,2:3], irtmodel="PCM")
# extract item difficulties
b <- matrix( mod$xsi$xsi, nrow=2, byrow=TRUE )
# groupwise estimation
res1 <- sirt::mle.pcm.group( dat[,2:3], b=b, group=dat$idgroup )
# individual estimation
res2 <- sirt::mle.pcm.group( dat[,2:3], b=b  )

#############################################################################
# EXAMPLE 2: Data Reading data.read
#############################################################################

data(data.read)
# estimate Rasch model
mod <- sirt::rasch.mml2( data.read )
score <- rowSums( data.read )
data.read <- data.read[ order(score), ]
score <- score[ order(score) ]
# compare different epsilon-adjustments
res30 <- sirt::mle.pcm.group( data.read, b=matrix( mod$item$b, 12, 1 ),
               adj_eps=.3 )$person
res10 <- sirt::mle.pcm.group( data.read, b=matrix( mod$item$b, 12, 1 ),
             adj_eps=.1 )$person
res05 <- sirt::mle.pcm.group( data.read, b=matrix( mod$item$b, 12, 1 ),
              adj_eps=.05 )$person
# plot different scorings
plot( score, res05$theta, type="l", xlab="Raw score", ylab=expression(theta[epsilon]),
         main="Scoring with different epsilon-adjustments")
lines( score, res10$theta, col=2, lty=2 )
lines( score, res30$theta, col=4, lty=3 )
}
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Person parameters}
%% \keyword{Group parameters}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: conf.detect.Rd
%% File Version: 1.484

\name{conf.detect}
\alias{conf.detect}
\alias{summary.conf.detect}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Confirmatory DETECT and polyDETECT Analysis
}
\description{
This function computes the DETECT statistics for dichotomous item responses
and the polyDETECT statistic for polytomous item responses
under a confirmatory specification of item clusters
(Stout, Habing, Douglas & Kim, 1996; Zhang & Stout,
1999a, 1999b; Zhang, 2007; Bonifay, Reise, Scheines, & Meijer, 2015).

Item responses in a multi-matrix design are allowed (Zhang, 2013).

An exploratory DETECT analysis can be conducted using the
\code{\link{expl.detect}} function.
}
\usage{
conf.detect(data, score, itemcluster, bwscale=1.1, progress=TRUE,
        thetagrid=seq(-3, 3, len=200), smooth=TRUE, use_sum_score=FALSE, bias_corr=TRUE)

\method{summary}{conf.detect}(object, digits=3, file=NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
An \eqn{N \times I} data frame of dichotomous or polytomous responses.
Missing responses are allowed.
}
  \item{score}{
An ability estimate, e.g. the WLE, sum score or mean score
}
  \item{itemcluster}{
Item cluster for each item. The order of entries must correspond
to the columns in \code{data}.
}
  \item{bwscale}{
Bandwidth factor for calculation of conditional covariance
(see \code{\link{ccov.np}})
}
  \item{progress}{
Display progress?
}
\item{smooth}{Logical indicating whether smoothing should be
applied for conditional covariance estimation}
  \item{thetagrid}{
A vector which contains theta values where conditional
covariances are evaluated.
}
\item{use_sum_score}{Logical indicating whether sum score should be used.
With this option, the bias corrected conditional covariance of Zhang and
Stout (1999) is used.}
\item{bias_corr}{Logical indicating whether bias correction (Zhang & Stout, 1999)
should be utilized if \code{use_sum_score=TRUE}.}
\item{object}{Object of class \code{conf.detect}}
\item{digits}{Number of digits for rounding in \code{summary}}
\item{file}{Optional file name to be sunk for \code{summary}}
\item{\dots}{Further arguments to be passed}
}
\details{
The result of DETECT are the indices \code{DETECT}, \code{ASSI}
and \code{RATIO} (see Zhang 2007 for details) calculated
for the options \code{unweighted} and \code{weighted}.
The option \code{unweighted} means that all conditional covariances of
item pairs are equally weighted, \code{weighted} means that
these covariances are weighted by the sample size of item pairs.
In case of multi matrix item designs, both types of indices can
differ.

The classification scheme of these indices are as follows
(Jang & Roussos, 2007; Zhang, 2007):

\tabular{ll}{
 Strong multidimensionality \tab DETECT > 1.00 \cr
Moderate multidimensionality \tab .40 < DETECT < 1.00 \cr
Weak multidimensionality \tab .20 < DETECT < .40 \cr
Essential unidimensionality \tab DETECT < .20 \cr
    }
\tabular{lll}{
 Maximum value under simple structure \tab ASSI=1 \tab RATIO=1 \cr
Essential deviation from unidimensionality \tab ASSI > .25 \tab
RATIO > .36 \cr
Essential unidimensionality \tab  ASSI < .25 \tab
RATIO < .36 \cr
    }


Note that the expected value of a conditional covariance for an item pair
is negative when a unidimensional model holds. In consequence,
the DETECT index can become negative for unidimensional data
(see Example 3). This can be also seen in the statistic
\code{MCOV100} in the value \code{detect}.
}
\value{
A list with following entries:
\item{detect}{Data frame with statistics DETECT, ASSI, RATIO, MADCOV100
and MCOV100}
\item{ccovtable}{Individual contributions to conditional covariance}
\item{ccov.matrix}{Evaluated conditional covariance}
}


\references{
Bonifay, W. E., Reise, S. P., Scheines, R., & Meijer, R. R. (2015).
When are multidimensional data unidimensional enough for structural
equation modeling? An evaluation of the DETECT multidimensionality index.
\emph{Structural Equation Modeling, 22}(4), 504-516.
\doi{10.1080/10705511.2014.938596}

Jang, E. E., & Roussos, L. (2007). An investigation into the dimensionality
of TOEFL using conditional covariance-based nonparametric approach.
\emph{Journal of Educational Measurement, 44}(1), 1-21.
\doi{10.1111/j.1745-3984.2007.00024.x}

Stout, W., Habing, B., Douglas, J., & Kim, H. R. (1996).
Conditional covariance-based nonparametric multidimensionality assessment.
\emph{Applied Psychological Measurement, 20}(4), 331-354.
\doi{10.1177/014662169602000403}

Zhang, J. (2007). Conditional covariance theory and DETECT for
polytomous items. \emph{Psychometrika, 72}(1), 69-91.
\doi{10.1007/s11336-004-1257-7}

Zhang, J. (2013). A procedure for dimensionality analyses of
response data from various test designs. \emph{Psychometrika, 78}(1), 37-58.
\doi{10.1007/s11336-012-9287-z}

Zhang, J., & Stout, W. (1999a). Conditional covariance structure
of generalized compensatory multidimensional items.
\emph{Psychometrika, 64}(2), 129-152.
\doi{10.1007/BF02294532}

Zhang, J., & Stout, W. (1999b). The theoretical DETECT index of
dimensionality and its application to approximate simple structure.
\emph{Psychometrika, 64}(2), 213-249.
\doi{10.1007/BF02294536}
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For a download of the free \emph{DIM-Pack} software (DIMTEST, DETECT) see
\emph{https://psychometrics.onlinehelp.measuredprogress.org/tools/dim/}.

See \code{\link{expl.detect}} for exploratory DETECT analysis.
}


\examples{
#############################################################################
# EXAMPLE 1: TIMSS mathematics data set (dichotomous data)
#############################################################################
data(data.timss)

# extract data
dat <- data.timss$data
dat <- dat[, substring( colnames(dat),1,1)=="M" ]
# extract item informations
iteminfo <- data.timss$item
# estimate Rasch model
mod1 <- sirt::rasch.mml2( dat )
# estimate WLEs
wle1 <- sirt::wle.rasch( dat, b=mod1$item$b )$theta

# DETECT for content domains
detect1 <- sirt::conf.detect( data=dat, score=wle1,
                    itemcluster=iteminfo$Content.Domain )
  ##          unweighted weighted
  ##   DETECT      0.316    0.316
  ##   ASSI        0.273    0.273
  ##   RATIO       0.355    0.355

\dontrun{
# DETECT cognitive domains
detect2 <- sirt::conf.detect( data=dat, score=wle1,
                    itemcluster=iteminfo$Cognitive.Domain )
  ##          unweighted weighted
  ##   DETECT      0.251    0.251
  ##   ASSI        0.227    0.227
  ##   RATIO       0.282    0.282

# DETECT for item format
detect3 <- sirt::conf.detect( data=dat, score=wle1,
                    itemcluster=iteminfo$Format )
  ##          unweighted weighted
  ##   DETECT      0.056    0.056
  ##   ASSI        0.060    0.060
  ##   RATIO       0.062    0.062

# DETECT for item blocks
detect4 <- sirt::conf.detect( data=dat, score=wle1,
                    itemcluster=iteminfo$Block )
  ##          unweighted weighted
  ##   DETECT      0.301    0.301
  ##   ASSI        0.193    0.193
  ##   RATIO       0.339    0.339 }

# Exploratory DETECT: Application of a cluster analysis employing the Ward method
detect5 <- sirt::expl.detect( data=dat, score=wle1,
                nclusters=10, N.est=nrow(dat)  )
# Plot cluster solution
pl <- graphics::plot( detect5$clusterfit, main="Cluster solution" )
stats::rect.hclust(detect5$clusterfit, k=4, border="red")

\dontrun{
#############################################################################
# EXAMPLE 2: Big 5 data set (polytomous data)
#############################################################################

# attach Big5 Dataset
data(data.big5)

# select 6 items of each dimension
dat <- data.big5
dat <- dat[, 1:30]

# estimate person score by simply using a transformed sum score
score <- stats::qnorm( ( rowMeans( dat )+.5 )  / ( 30 + 1 ) )

# extract item cluster (Big 5 dimensions)
itemcluster <- substring( colnames(dat), 1, 1 )

# DETECT Item cluster
detect1 <- sirt::conf.detect( data=dat, score=score, itemcluster=itemcluster )
  ##        unweighted weighted
  ## DETECT      1.256    1.256
  ## ASSI        0.384    0.384
  ## RATIO       0.597    0.597

# Exploratory DETECT
detect5 <- sirt::expl.detect( data=dat, score=score,
                     nclusters=9, N.est=nrow(dat)  )
  ## DETECT (unweighted)
  ## Optimal Cluster Size is  6  (Maximum of DETECT Index)
  ##   N.Cluster N.items N.est N.val      size.cluster DETECT.est ASSI.est RATIO.est
  ## 1         2      30   500     0              6-24      1.073    0.246     0.510
  ## 2         3      30   500     0           6-10-14      1.578    0.457     0.750
  ## 3         4      30   500     0         6-10-11-3      1.532    0.444     0.729
  ## 4         5      30   500     0        6-8-11-2-3      1.591    0.462     0.757
  ## 5         6      30   500     0       6-8-6-2-5-3      1.610    0.499     0.766
  ## 6         7      30   500     0     6-3-6-2-5-5-3      1.557    0.476     0.740
  ## 7         8      30   500     0   6-3-3-2-3-5-5-3      1.540    0.462     0.732
  ## 8         9      30   500     0 6-3-3-2-3-5-3-3-2      1.522    0.444     0.724

# Plot Cluster solution
pl <- graphics::plot( detect5$clusterfit, main="Cluster solution" )
stats::rect.hclust(detect5$clusterfit, k=6, border="red")

#############################################################################
# EXAMPLE 3: DETECT index for unidimensional data
#############################################################################

set.seed(976)
N <- 1000
I <- 20
b <- sample( seq( -2, 2, len=I) )
dat <- sirt::sim.raschtype( stats::rnorm(N), b=b )

# estimate Rasch model and corresponding WLEs
mod1 <- TAM::tam.mml( dat )
wmod1 <- TAM::tam.wle(mod1)$theta

# define item cluster
itemcluster <- c( rep(1,5), rep(2,I-5) )

# compute DETECT statistic
detect1 <- sirt::conf.detect( data=dat, score=wmod1, itemcluster=itemcluster)
  ##            unweighted weighted
  ##  DETECT        -0.184   -0.184
  ##  ASSI          -0.147   -0.147
  ##  RATIO         -0.226   -0.226
  ##  MADCOV100      0.816    0.816
  ##  MCOV100       -0.786   -0.786
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{DETECT}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.pw01.Rd
%% File Version: 0.06

\name{data.pw}
\alias{data.pw01}
\docType{data}
\title{
Datasets for Pairwise Comparisons
}
\description{
Some datasets for pairwise comparisons.
}
\usage{
data(data.pw01)
}
\format{
The dataset \code{data.pw01} contains results of a German football league
from the season 2000/01.
}

%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}
%\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%}
%\references{
%%  ~~ possibly secondary sources and usages ~~
%}
%\examples{
%data(data.pw01)
%}
%% \keyword{datasets}

%% File Name: fit.isop.Rd
%% File Version: 1.20

\name{fit.isop}
\alias{fit.isop}
\alias{fit.adisop}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Fitting the ISOP and ADISOP Model for Frequency Tables
}
\description{
Fit the isotonic probabilistic model (ISOP;
Scheiblechner, 1995) and the additive isotonic probabilistic
model (ADISOP; Scheiblechner, 1999).
}

\usage{
fit.isop(freq.correct, wgt, conv=1e-04, maxit=100,
      progress=TRUE, calc.ll=TRUE)

fit.adisop(freq.correct, wgt, conv=1e-04, maxit=100,
      epsilon=0.01, progress=TRUE, calc.ll=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{freq.correct}{
Frequency table
}
  \item{wgt}{
Weights for frequency table (number of persons in
each cell)
}
  \item{conv}{
Convergence criterion
}
  \item{maxit}{
Maximum number of iterations
}
\item{epsilon}{Additive constant to handle cell frequencies
of 0 or 1 in \code{fit.adisop}
}
  \item{progress}{
Display progress?
}
\item{calc.ll}{Calculate log-likelihood values?
The default is \code{TRUE}.}
}
\details{
See \code{\link{isop.dich}} for more details of the
ISOP and ADISOP model.
}


\value{
A list with following entries
\item{fX}{Fitted frequency table}
\item{ResX}{Residual frequency table}
\item{fit}{Fit statistic: weighted least squares
  of deviations between observed and expected frequencies}
\item{item.sc}{Estimated item parameters}
\item{person.sc}{
    Estimated person parameters
            }
\item{ll}{Log-likelihood of the model}
\item{freq.fitted}{Fitted frequencies in a long data frame}
}

\references{
Scheiblechner, H. (1995). Isotonic ordinal
probabilistic models (ISOP). \emph{Psychometrika,
60}, 281-304.

Scheiblechner, H. (1999). Additive conjoint isotonic
probabilistic models (ADISOP). \emph{Psychometrika,
64}, 295-316.
}



%\author{
%Alexander Robitzsch
%}



\note{
For fitting the ADISOP model it is recommended to first
fit the ISOP model and then proceed with the fitted frequency
table from ISOP (see Examples).
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For fitting the ISOP model to dichotomous and
polytomous data see \code{\link{isop.dich}}.
}


\examples{
#############################################################################
# EXAMPLE 1: Dataset Reading
#############################################################################

data(data.read)
dat <- as.matrix( data.read)
dat.resp <- 1 - is.na(dat) # response indicator matrix
I <- ncol(dat)

#***
# (1) Data preparation
#     actually only freq.correct and wgt are needed
#     but these matrices must be computed in advance.

# different scores of students
stud.p <- rowMeans( dat, na.rm=TRUE )
# different item p values
item.p <- colMeans( dat, na.rm=TRUE )
item.ps <- sort( item.p, index.return=TRUE)
dat <- dat[,  item.ps$ix ]
# define score groups students
scores <- sort( unique( stud.p ) )
SC <- length(scores)
# create table
freq.correct <- matrix( NA, SC, I )
wgt <- freq.correct
# percent correct
a1 <- stats::aggregate( dat==1, list( stud.p ), mean, na.rm=TRUE )
freq.correct <- a1[,-1]
# weights
a1 <- stats::aggregate( dat.resp, list( stud.p ), sum, na.rm=TRUE )
wgt <- a1[,-1]

#***
# (2) Fit ISOP model
res.isop <- sirt::fit.isop( freq.correct, wgt )
# fitted frequency table
res.isop$fX

#***
# (3) Fit ADISOP model
# use monotonely smoothed frequency table from ISOP model
res.adisop <- sirt::fit.adisop( freq.correct=res.isop$fX, wgt )
# fitted frequency table
res.adisop$fX
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{ISOP model}
%% \keyword{ADISOP model}

%% File Name: sim.rasch.dep.Rd
%% File Version: 0.33

\name{sim.rasch.dep}
\alias{sim.rasch.dep}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Simulation of the Rasch Model with Locally Dependent Responses
}
\description{
This function simulates dichotomous item responses where for some
itemclusters residual correlations can be defined.
}
\usage{
sim.rasch.dep(theta, b, itemcluster, rho)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{theta}{
Vector of person abilities of length \eqn{N}
}
  \item{b}{
Vector of item difficulties of length \eqn{I}
}
  \item{itemcluster}{
Vector of integers (including 0) of length \eqn{I}. Different integers
correspond to different itemclusters.
}
  \item{rho}{
Vector of residual correlations. The length of vector must be equal
to the number of itemclusters.
}
}


%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
An \eqn{N \times I} data frame of dichotomous item responses.
}

%\author{
%Alexander Robitzsch
%}


\note{
The specification of the simulation models follows a marginal interpretation
of the latent trait. Local dependencies are only interpreted as nuisance
and not of substantive interest. If local dependencies should be substantively
interpreted, a testlet model seems preferable
(see \code{\link{mcmc.3pno.testlet}}).
}




%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
To simulate the generalized logistic item response model see
\code{\link{sim.raschtype}}. Ramsay's quotient model can be simulated
using \code{\link{sim.qm.ramsay}}.

Marginal item reponse models for locally dependent item responses can be
estimated with \code{\link{rasch.copula2}}, \code{\link{rasch.pairwise}} or
\code{\link{rasch.pairwise.itemcluster}}.
}
\examples{
#############################################################################
# EXAMPLE 1: 11 Items: 2 itemclusters with 2 resp. 3 dependent items
#             and 6 independent items
#############################################################################

set.seed(7654)
I <- 11                             # number of items
n <- 1500                           # number of persons
b <- seq(-2,2, len=I)               # item difficulties
theta <- stats::rnorm( n, sd=1 )        # person abilities
# itemcluster
itemcluster <- rep(0,I)
itemcluster[ c(3,5)] <- 1
itemcluster[c(2,4,9)] <- 2
# residual correlations
rho <- c( .7, .5 )

# simulate data
dat <- sirt::sim.rasch.dep( theta, b, itemcluster, rho )
colnames(dat) <- paste("I", seq(1,ncol(dat)), sep="")

# estimate Rasch copula model
mod1 <- sirt::rasch.copula2( dat, itemcluster=itemcluster )
summary(mod1)

# compare result with Rasch model estimation in rasch.copula
# delta must be set to zero
mod2 <- sirt::rasch.copula2( dat, itemcluster=itemcluster, delta=c(0,0),
            est.delta=c(0,0)  )
summary(mod2)

# estimate Rasch model with rasch.mml2 function
mod3 <- sirt::rasch.mml2( dat )
summary(mod3)

\dontrun{
#############################################################################
# EXAMPLE 2: 12 Items: Cluster 1 -> Items 1,...,4;
#       Cluster 2 -> Items 6,...,9; Cluster 3 -> Items 10,11,12
#############################################################################

set.seed(7896)
I <- 12                             # number of items
n <- 450                            # number of persons
b <- seq(-2,2, len=I)               # item difficulties
b <- sample(b)                      # sample item difficulties
theta <- stats::rnorm( n, sd=1 )        # person abilities
# itemcluster
itemcluster <- rep(0,I)
itemcluster[ 1:4 ] <- 1
itemcluster[ 6:9 ] <- 2
itemcluster[ 10:12 ] <- 3
# residual correlations
rho <- c( .55, .25, .45 )

# simulate data
dat <- sirt::sim.rasch.dep( theta, b, itemcluster, rho )
colnames(dat) <- paste("I", seq(1,ncol(dat)), sep="")

# estimate Rasch copula model
mod1 <- sirt::rasch.copula2( dat, itemcluster=itemcluster, numdiff.parm=.001 )
summary(mod1)

# Rasch model estimation
mod2 <- sirt::rasch.copula2( dat, itemcluster=itemcluster,
            delta=rep(0,3), est.delta=rep(0,3) )
summary(mod2)

# estimation with pairwise Rasch model
mod3 <- sirt::rasch.pairwise( dat )
summary(mod3)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Simulating IRT models}
%\keyword{ ~kwd1 }

%% File Name: rasch.mirtlc.Rd
%% File Version: 2.945

\name{rasch.mirtlc}
\alias{rasch.mirtlc}
\alias{summary.rasch.mirtlc}
\alias{logLik.rasch.mirtlc}
\alias{anova.rasch.mirtlc}
\alias{IRT.irfprob.rasch.mirtlc}
\alias{IRT.likelihood.rasch.mirtlc}
\alias{IRT.posterior.rasch.mirtlc}
\alias{IRT.modelfit.rasch.mirtlc}
\alias{summary.IRT.modelfit.rasch.mirtlc}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Multidimensional Latent Class 1PL and 2PL Model
}
\description{
This function estimates the multidimensional latent class
Rasch (1PL) and 2PL model (Bartolucci, 2007; Bartolucci, Montanari & Pandolfi,
2012) for dichotomous data which emerges from the original latent class model
(Goodman, 1974) and a multidimensional IRT model.
}
\usage{
rasch.mirtlc(dat, Nclasses=NULL, modeltype="LC", dimensions=NULL,
    group=NULL, weights=rep(1,nrow(dat)), theta.k=NULL, ref.item=NULL,
    distribution.trait=FALSE,  range.b=c(-8,8), range.a=c(.2, 6 ),
    progress=TRUE, glob.conv=10^(-5), conv1=10^(-5), mmliter=1000,
    mstep.maxit=3, seed=0, nstarts=1, fac.iter=.35)

\method{summary}{rasch.mirtlc}(object,...)

\method{anova}{rasch.mirtlc}(object,...)

\method{logLik}{rasch.mirtlc}(object,...)

\method{IRT.irfprob}{rasch.mirtlc}(object,...)

\method{IRT.likelihood}{rasch.mirtlc}(object,...)

\method{IRT.posterior}{rasch.mirtlc}(object,...)

\method{IRT.modelfit}{rasch.mirtlc}(object,...)

\method{summary}{IRT.modelfit.rasch.mirtlc}(object,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
An \eqn{N \times I} data frame
}
  \item{Nclasses}{
Number of latent classes. If the trait vector (or matrix)
\code{theta.k} is specified, then \code{Nclasses} is set
to the dimension of \code{theta.k}.
}
  \item{modeltype}{
Modeltype. \code{LC} is the latent class model of Goodman (1974).
\code{MLC1} is the multidimensional latent class Rasch model with
item discrimination parameter of 1. \code{MLC2} allows for the
estimation of item discriminations.
}
\item{dimensions}{
    Vector of dimension integers which allocate items to dimensions.
        }
  \item{group}{
A group identifier for multiple group estimation
}
  \item{weights}{
 Vector of sample weights
 }
  \item{theta.k}{
A grid of theta values can be specified if theta
should not be estimated. In the one-dimensional case, it
must be a vector, in the \eqn{D}-dimensional case it must
be a matrix of dimension \eqn{D}.
}
\item{ref.item}{An optional vector of integers which indicate the items
    whose intercept and slope are fixed at 0 and 1, respectively.
    }
\item{distribution.trait}{A type of the assumed theta distribution can
be specified. One alternative is \code{normal}
for the normal distribution assumption.
The options \code{smooth2}, \code{smooth3} and
\code{smooth4} use the log-linear smoothing of
 Xu and von Davier (2008) to smooth the distribution
 up to two, three or four moments, respectively.
This function only works in unidimensional models. \cr
If a different string is provided as an input (e.g. \code{no}),
then no smoothing is conducted.
    }
  \item{range.b}{
 Range of item difficulties which are allowed for estimation
 }
  \item{range.a}{
 Range of item slopes which are allowed for estimation
 }
  \item{progress}{
Display progress? Default is \code{TRUE}.
}
  \item{glob.conv}{
Global relative deviance convergence criterion
}
  \item{conv1}{
Item parameter convergence criterion
}
  \item{mmliter}{
Maximum number of iterations
}
  \item{mstep.maxit}{
Maximum number of iterations within an M step
}
  \item{seed}{
Set random seed for latent class estimation. A seed
can be specified. If the seed is negative, then
the function will generate a random seed.
}
\item{nstarts}{
    If a positive integer is provided, then a \code{nstarts}
    starts with different starting values are conducted.
}
\item{fac.iter}{A parameter between 0 and 1 to control the maximum
increment in each iteration. The larger the parameter the more increments
will become smaller from iteration to iteration.
}
  \item{object}{
Object of class \code{rasch.mirtlc}
}
  \item{\dots}{
Further arguments to be passed
}
}
\details{
The multidimensional latent class Rasch model (Bartolucci, 2007)
is an item response model which combines ideas from
latent class analysis and item response models with continuous variables.
With \code{modeltype="MLC2"} the following \eqn{D}-dimensional
item response model is estimated
\deqn{logit P(X_{pi}=1 | \theta_p )=a_i \theta_{pcd}- b_i}{%
logit P(X_{pi}=1 | \theta_p )=a_i \theta_{pcd}- b_i }
Besides the item thresholds \eqn{b_i} and item slopes \eqn{a_i},
for a prespecified number of latent classes \eqn{c=1,\ldots,C}{c=1,...,C}
a set of \eqn{C} \eqn{D}-dimensional \eqn{\{\theta_{cd} \}_{cd}}{{\theta_{cd} }_{cd}}
vectors are estimated.
These vectors represent the locations of latent classes. If the user
provides a grid of theta distribution \code{theta.k} as an argument in
\code{rasch.mirtlc}, then the ability distribution is fixed.

In the unidimensional Rasch model with \eqn{I} items, \eqn{(I+1)/2}
(if \eqn{I} odd) or \eqn{I/2 + 1} (if \eqn{I} even) trait location
parameters are identified (see De Leeuw & Verhelst, 1986;
Lindsay et al., 1991; for a review see Formann, 2007).
}


\value{
A list with following entries
\item{pjk}{Item probabilities evaluated at discretized ability distribution}
\item{rprobs}{Item response probabilities like in \code{pjk},
    but for each item category}
\item{pi.k}{Estimated trait distribution}
\item{theta.k}{Discretized ability distribution}
\item{item}{Estimated item parameters}
\item{trait}{Estimated ability distribution (\code{theta.k} and \code{pi.k})}
\item{mean.trait}{Estimated mean of ability distribution}
\item{sd.trait}{Estimated standard deviation of ability distribution}
\item{skewness.trait}{Estimated skewness of ability distribution}
\item{cor.trait}{Estimated correlation between abilities (only applies for
    multidimensional models)}
\item{ic}{Information criteria}
\item{D}{Number of dimensions}
\item{G}{Number of groups}
\item{deviance}{Deviance}
\item{ll}{Log-likelihood}
\item{Nclasses}{Number of classes}
\item{modeltype}{Used model type}
\item{estep.res}{Result from E step: \code{f.qk.yi} is the individual posterior,
    \code{f.yi.qk} is the individual likelihood}
\item{dat}{Original data frame}
\item{devL}{Vector of deviances if multiple random starts were conducted}
\item{seedL}{Vector of seed if multiple random starts were conducted}
\item{iter}{Number of iterations}
}



\references{
Bartolucci, F. (2007). A class of multidimensional
IRT models for testing unidimensionality and clustering
items. \emph{Psychometrika, 72}(2), 141-157.
\doi{10.1007/s11336-005-1376-9}

Bartolucci, F., Montanari, G. E., & Pandolfi, S. (2012).
Dimensionality of the latent structure and item selection via latent
class multidimensional IRT models.
\emph{Psychometrika, 77}(4), 782-802.
\doi{10.1007/s11336-012-9278-0}

De Leeuw, J., & Verhelst, N. (1986). Maximum likelihood estimation in
generalized Rasch models. \emph{Journal of Educational and Behavioral Statistics, 11}(3),
183-196. \doi{10.3102/10769986011003183}

Formann, A. K. (2007). (Almost) Equivalence between conditional and mixture
maximum likelihood estimates for some models of the Rasch type.
In M. von Davier & C. H. Carstensen:
\emph{Multivariate and Mixture Distribution Rasch Models} (pp. 177-189).
Springer: New York.
\doi{10.1007/978-0-387-49839-3_11}

Goodman, L. A. (1974). Exploratory latent structure
analysis using both identifiable and unidentifiable
models. \emph{Biometrika, 61}(2), 215-231.
\doi{10.1093/biomet/61.2.215}

Lindsay, B., Clogg, C. C., & Grego, J. (1991).
Semiparametric estimation in the Rasch model and related exponential response
models, including a simple latent class model for item analysis.
\emph{Journal of the American Statistical Association, 86}(413), 96-107.
\doi{10.1080/01621459.1991.10475008}

Xu, X., & von Davier, M. (2008). \emph{Fitting the structured general
diagnostic model to NAEP data}. ETS Research Report ETS RR-08-27.
Princeton, ETS.
\doi{10.1002/j.2333-8504.2008.tb02113.x}
}


%\author{
%Alexander Robitzsch
%}



\note{
For the estimation of latent class models,
rerunning the model with different starting values
(different random seeds) is recommended.
}

% For fixed theta estimation in the multidimensional
% case, large vectors are generated during estimation
% leading to memory overflow in \R.

%% ~Make other sections like Warning with \section{Warning }{....} ~

%% \link[CDM]{\code{gdm}}
\seealso{
See also the \code{\link[CDM:gdm]{CDM::gdm}}
function in the \pkg{CDM} package.

For an assessment of global model fit see \code{\link{modelfit.sirt}}.

The estimation of the multidimensional latent class item response
model for polytomous data can be conducted in the \pkg{MultiLCIRT}
package. Latent class analysis can be carried out with
\pkg{poLCA} and \pkg{randomLCA} packages.
}
\examples{
#############################################################################
# EXAMPLE 1: Reading data
#############################################################################
data( data.read )
dat <- data.read

#***************
# latent class models

# latent class model with 1 class
mod1 <- sirt::rasch.mirtlc( dat, Nclasses=1 )
summary(mod1)

# latent class model with 2 classes
mod2 <- sirt::rasch.mirtlc( dat, Nclasses=2 )
summary(mod2)

\dontrun{
# latent class model with 3 classes
mod3 <- sirt::rasch.mirtlc( dat, Nclasses=3, seed=- 30)
summary(mod3)

# extract individual likelihood
lmod3 <- IRT.likelihood(mod3)
str(lmod3)
# extract likelihood value
logLik(mod3)
# extract item response functions
IRT.irfprob(mod3)

# compare models 1, 2 and 3
anova(mod2,mod3)
IRT.compareModels(mod1,mod2,mod3)
# avsolute and relative model fit
smod2 <- IRT.modelfit(mod2)
smod3 <- IRT.modelfit(mod3)
summary(smod2)
IRT.compareModels(smod2,smod3)

# latent class model with 4 classes and 3 starts with different seeds
mod4 <- sirt::rasch.mirtlc( dat, Nclasses=4,seed=-30,  nstarts=3 )
# display different solutions
sort(mod4$devL)
summary(mod4)

# latent class multiple group model
# define group identifier
group <- rep( 1, nrow(dat))
group[ 1:150 ] <- 2
mod5 <- sirt::rasch.mirtlc( dat, Nclasses=3, group=group )
summary(mod5)

#*************
# Unidimensional IRT models with ordered trait

# 1PL model with 3 classes
mod11 <- sirt::rasch.mirtlc( dat, Nclasses=3, modeltype="MLC1", mmliter=30)
summary(mod11)

# 1PL model with 11 classes
mod12 <- sirt::rasch.mirtlc( dat, Nclasses=11,modeltype="MLC1", mmliter=30)
summary(mod12)

# 1PL model with 11 classes and fixed specified theta values
mod13 <- sirt::rasch.mirtlc( dat,  modeltype="MLC1",
             theta.k=seq( -4, 4, len=11 ), mmliter=100)
summary(mod13)

# 1PL model with fixed theta values and normal distribution
mod14 <- sirt::rasch.mirtlc( dat,  modeltype="MLC1", mmliter=30,
             theta.k=seq( -4, 4, len=11 ), distribution.trait="normal")
summary(mod14)

# 1PL model with a smoothed trait distribution (up to 3 moments)
mod15 <- sirt::rasch.mirtlc( dat,  modeltype="MLC1", mmliter=30,
             theta.k=seq( -4, 4, len=11 ),  distribution.trait="smooth3")
summary(mod15)

# 2PL with 3 classes
mod16 <- sirt::rasch.mirtlc( dat, Nclasses=3, modeltype="MLC2", mmliter=30 )
summary(mod16)

# 2PL with fixed theta and smoothed distribution
mod17 <- sirt::rasch.mirtlc( dat, theta.k=seq(-4,4,len=12), mmliter=30,
             modeltype="MLC2", distribution.trait="smooth4"  )
summary(mod17)

# 1PL multiple group model with 8 classes
# define group identifier
group <- rep( 1, nrow(dat))
group[ 1:150 ] <- 2
mod21 <- sirt::rasch.mirtlc( dat, Nclasses=8, modeltype="MLC1", group=group )
summary(mod21)

#***************
# multidimensional latent class IRT models

# define vector of dimensions
dimensions <- rep( 1:3, each=4 )

# 3-dimensional model with 8 classes and seed 145
mod31 <- sirt::rasch.mirtlc( dat, Nclasses=8, mmliter=30,
             modeltype="MLC1", seed=145, dimensions=dimensions )
summary(mod31)

# try the model above with different starting values
mod31s <- sirt::rasch.mirtlc( dat, Nclasses=8,
             modeltype="MLC1", seed=-30, nstarts=30, dimensions=dimensions )
summary(mod31s)

# estimation with fixed theta vectors
#=> 4^3=216 classes
theta.k <- seq(-4, 4, len=6 )
theta.k <- as.matrix( expand.grid( theta.k, theta.k, theta.k ) )
mod32 <- sirt::rasch.mirtlc( dat,  dimensions=dimensions,
              theta.k=theta.k, modeltype="MLC1"  )
summary(mod32)

# 3-dimensional 2PL model
mod33 <- sirt::rasch.mirtlc( dat, dimensions=dimensions, theta.k=theta.k, modeltype="MLC2")
summary(mod33)

#############################################################################
# EXAMPLE 2: Skew trait distribution
#############################################################################
set.seed(789)
N <- 1000   # number of persons
I <- 20     # number of items
theta <- sqrt( exp( stats::rnorm( N ) ) )
theta <- theta - mean(theta )
# calculate skewness of theta distribution
mean( theta^3 ) / stats::sd(theta)^3
# simulate item responses
dat <- sirt::sim.raschtype( theta, b=seq(-2,2,len=I ) )

# normal distribution
mod1 <- sirt::rasch.mirtlc( dat, theta.k=seq(-4,4,len=15), modeltype="MLC1",
               distribution.trait="normal", mmliter=30)

# allow for skew distribution with smoothed distribution
mod2 <- sirt::rasch.mirtlc( dat, theta.k=seq(-4,4,len=15), modeltype="MLC1",
               distribution.trait="smooth3", mmliter=30)

# nonparametric distribution
mod3 <- sirt::rasch.mirtlc( dat, theta.k=seq(-4,4,len=15), modeltype="MLC1", mmliter=30)

summary(mod1)
summary(mod2)
summary(mod3)

#############################################################################
# EXAMPLE 3: Stouffer-Toby dataset data.si02 with 5 items
#############################################################################

data(dat.si02)
dat <- data.si02$data
weights <- data.si02$weights   # extract weights

# Model 1: 2 classes Rasch model
mod1 <- sirt::rasch.mirtlc( dat, Nclasses=2, modeltype="MLC1", weights=weights,
                 ref.item=4, nstarts=5)
summary(mod1)

# Model 2: 3 classes Rasch model: not all parameters are identified
mod2 <- sirt::rasch.mirtlc( dat, Nclasses=3, modeltype="MLC1", weights=weights,
                ref.item=4, nstarts=5)
summary(mod2)

# Model 3: Latent class model with 2 classes
mod3 <- sirt::rasch.mirtlc( dat, Nclasses=2, modeltype="LC", weights=weights, nstarts=5)
summary(mod3)

# Model 4: Rasch model with normal distribution
mod4 <- sirt::rasch.mirtlc( dat,  modeltype="MLC1", weights=weights,
            theta.k=seq( -6, 6, len=21 ), distribution.trait="normal", ref.item=4)
summary(mod4)
}

#############################################################################
# EXAMPLE 4: 5 classes, 3 dimensions and 27 items
#############################################################################

set.seed(979)
I <- 9
N <- 5000
b <- seq( - 1.5, 1.5, len=I)
b <- rep(b,3)
# define class locations
theta.k <- c(-3.0, -4.1, -2.8, 1.7, 2.3, 1.8,
   0.2, 0.4, -0.1,   2.6, 0.1, -0.9, -1.1,-0.7, 0.9 )

Nclasses <- 5
theta.k0 <- theta.k <- matrix( theta.k, Nclasses, 3, byrow=TRUE )
pi.k <- c(.20,.25,.25,.10,.15)
theta <- theta.k[ rep( 1:Nclasses, round(N*pi.k) ), ]
dimensions <- rep( 1:3, each=I)
# simulate item responses
dat <- matrix( NA, nrow=N, ncol=I*3)
for (ii in 1:(3*I) ){
    dat[,ii] <- 1 * ( stats::runif(N) < stats::plogis( theta[,dimensions[ii]] - b[ii]))
}
colnames(dat) <- paste0( rep( LETTERS[1:3], each=I ), 1:(3*I) )

# estimate model
mod1 <- sirt::rasch.mirtlc( dat, Nclasses=Nclasses, dimensions=dimensions,
             modeltype="MLC1", ref.item=c(5,14,23), glob.conv=.0005, conv1=.0005)

round( cbind( mod1$theta.k, mod1$pi.k ), 3 )
  ##          [,1]   [,2]   [,3]  [,4]
  ##   [1,] -2.776 -3.791 -2.667 0.250
  ##   [2,] -0.989 -0.605  0.957 0.151
  ##   [3,]  0.332  0.418 -0.046 0.246
  ##   [4,]  2.601  0.171 -0.854 0.101
  ##   [5,]  1.791  2.330  1.844 0.252
cbind( theta.k, pi.k )
  ##                       pi.k
  ##   [1,] -3.0 -4.1 -2.8 0.20
  ##   [2,]  1.7  2.3  1.8 0.25
  ##   [3,]  0.2  0.4 -0.1 0.25
  ##   [4,]  2.6  0.1 -0.9 0.10
  ##   [5,] -1.1 -0.7  0.9 0.15

# plot class locations
plot( 1:3, mod1$theta.k[1,], xlim=c(1,3), ylim=c(-5,3), col=1, pch=1, type="n",
    axes=FALSE, xlab="Dimension", ylab="Location")
axis(1, 1:3 ) ;  axis(2) ; axis(4)
for (cc in 1:Nclasses){ # cc <- 1
    lines(1:3, mod1$theta.k[cc,], col=cc, lty=cc )
    points(1:3, mod1$theta.k[cc,], col=cc,  pch=cc )
}

\dontrun{
#------
# estimate model with gdm function in CDM package
library(CDM)
# define Q-matrix
Qmatrix <- matrix(0,3*I,3)
Qmatrix[ cbind( 1:(3*I), rep(1:3, each=I) ) ] <- 1

set.seed(9176)
# random starting values for theta locations
theta.k <- matrix( 2*stats::rnorm(5*3), 5, 3 )
colnames(theta.k) <- c("Dim1","Dim2","Dim3")
# try possibly different starting values

# estimate model in CDM
b.constraint  <- cbind( c(5,14,23), 1, 0 )
mod2 <- CDM::gdm( dat, theta.k=theta.k, b.constraint=b.constraint, skillspace="est",
               irtmodel="1PL",  Qmatrix=Qmatrix)
summary(mod2)

#------
# estimate model with MultiLCIRT package
miceadds::library_install("MultiLCIRT")

# define matrix to allocate each item to one dimension
multi1 <- matrix( 1:(3*I), nrow=3, byrow=TRUE )
# define reference items in item-dimension allocation matrix
multi1[ 1, c(1,5)  ] <- c(5,1)
multi1[ 2, c(10,14) - 9  ] <- c(14,9)
multi1[ 3, c(19,23) - 18 ] <- c(23,19)

# Rasch model with 5 latent classes (random start: start=1)
mod3 <- MultiLCIRT::est_multi_poly(S=dat,k=5,       # k=5 ability levels
                start=1,link=1,multi=multi1,tol=10^-5,
                output=TRUE, disp=TRUE, fort=TRUE)
# estimated location points and class probabilities in MultiLCIRT
cbind( t( mod3$Th ), mod3$piv )
# compare results with rasch.mirtlc
cbind( mod1$theta.k, mod1$pi.k )
# simulated data parameters
cbind( theta.k, pi.k )

#----
# estimate model with cutomized input in mirt
library(mirt)
#-- define Theta design matrix for 5 classes
Theta <- diag(5)
Theta <- cbind( Theta, Theta, Theta )
r1 <- rownames(Theta) <- paste0("C",1:5)
colnames(Theta) <- c( paste0(r1, "D1"), paste0(r1, "D2"), paste0(r1, "D3") )
  ##      C1D1 C2D1 C3D1 C4D1 C5D1 C1D2 C2D2 C3D2 C4D2 C5D2 C1D3 C2D3 C3D3 C4D3 C5D3
  ##   C1    1    0    0    0    0    1    0    0    0    0    1    0    0    0    0
  ##   C2    0    1    0    0    0    0    1    0    0    0    0    1    0    0    0
  ##   C3    0    0    1    0    0    0    0    1    0    0    0    0    1    0    0
  ##   C4    0    0    0    1    0    0    0    0    1    0    0    0    0    1    0
  ##   C5    0    0    0    0    1    0    0    0    0    1    0    0    0    0    1
#-- define mirt model
I <- ncol(dat)  # I=27
mirtmodel <- mirt::mirt.model("
        C1D1=1-9 \n C2D1=1-9 \n  C3D1=1-9 \n  C4D1=1-9  \n  C5D1=1-9
        C1D2=10-18 \n C2D2=10-18 \n  C3D2=10-18 \n  C4D2=10-18  \n  C5D2=10-18
        C1D3=19-27 \n C2D3=19-27 \n  C3D3=19-27 \n  C4D3=19-27  \n  C5D3=19-27
        CONSTRAIN=(1-9,a1),(1-9,a2),(1-9,a3),(1-9,a4),(1-9,a5),
                    (10-18,a6),(10-18,a7),(10-18,a8),(10-18,a9),(10-18,a10),
                    (19-27,a11),(19-27,a12),(19-27,a13),(19-27,a14),(19-27,a15)
                ")
#-- get initial parameter values
mod.pars <- mirt::mirt(dat, model=mirtmodel,  pars="values")
#-- redefine initial parameter values
# set all d parameters initially to zero
ind <- which( ( mod.pars$name=="d" ) )
mod.pars[ ind,"value" ]  <- 0
# fix item difficulties of reference items to zero
mod.pars[ ind[ c(5,14,23) ], "est"] <- FALSE
mod.pars[ind,]
# initial item parameters of cluster locations (a1,...,a15)
ind <- which( ( mod.pars$name \%in\% paste0("a", c(1,6,11) ) ) & ( mod.pars$est ) )
mod.pars[ind,"value"] <- -2
ind <- which( ( mod.pars$name \%in\% paste0("a", c(1,6,11)+1 ) ) & ( mod.pars$est ) )
mod.pars[ind,"value"] <- -1
ind <- which( ( mod.pars$name \%in\% paste0("a", c(1,6,11)+2 ) ) & ( mod.pars$est ) )
mod.pars[ind,"value"] <- 0
ind <- which( ( mod.pars$name \%in\% paste0("a", c(1,6,11)+3 ) ) & ( mod.pars$est ) )
mod.pars[ind,"value"] <- 1
ind <- which( ( mod.pars$name \%in\% paste0("a", c(1,6,11)+4 ) ) & ( mod.pars$est ) )
mod.pars[ind,"value"] <- 0
#-- define prior for latent class analysis
lca_prior <- function(Theta,Etable){
  TP <- nrow(Theta)
  if ( is.null(Etable) ){ prior <- rep( 1/TP, TP ) }
  if ( ! is.null(Etable) ){
    prior <- ( rowSums(Etable[, seq(1,2*I,2)]) + rowSums(Etable[,seq(2,2*I,2)]) )/I
  }
  prior <- prior / sum(prior)
  return(prior)
}

#-- estimate model in mirt
mod4 <- mirt::mirt(dat, mirtmodel, pars=mod.pars, verbose=TRUE,
              technical=list( customTheta=Theta, customPriorFun=lca_prior,
                    MAXQUAD=1E20) )
# correct number of estimated parameters
mod4@nest <- as.integer(sum(mod.pars$est) + nrow(Theta)-1 )
# extract coefficients
# source.all(pfsirt)
cmod4 <- sirt::mirt.wrapper.coef(mod4)

# estimated item difficulties
dfr <- data.frame( "sim"=b, "mirt"=-cmod4$coef$d, "sirt"=mod1$item$thresh )
round( dfr, 4 )
  ##         sim    mirt    sirt
  ##   1  -1.500 -1.3782 -1.3382
  ##   2  -1.125 -1.0059 -0.9774
  ##   3  -0.750 -0.6157 -0.6016
  ##   4  -0.375 -0.2099 -0.2060
  ##   5   0.000  0.0000  0.0000
  ##   6   0.375  0.5085  0.4984
  ##   7   0.750  0.8661  0.8504
  ##   8   1.125  1.3079  1.2847
  ##   9   1.500  1.5891  1.5620
  ##   [...]

#-- reordering estimated latent clusters to make solutions comparable
#* extract estimated cluster locations from sirt
order.sirt <- c(1,5,3,4,2)  # sort(order.sirt)
round(mod1$trait[,1:3],3)
dfr <- data.frame( "sim"=theta.k, mod1$trait[order.sirt,1:3] )
colnames(dfr)[4:6] <- paste0("sirt",1:3)
#* extract estimated cluster locations from mirt
c4 <- cmod4$coef[, paste0("a",1:15) ]
c4 <- apply( c4,2, FUN=function(ll){ ll[ ll!=0 ][1] } )
trait.loc <- matrix(c4,5,3)
order.mirt <- c(1,4,3,5,2)  # sort(order.mirt)
dfr <- cbind( dfr, trait.loc[ order.mirt, ] )
colnames(dfr)[7:9] <- paste0("mirt",1:3)
# compare estimated cluster locations
round(dfr,3)
  ##     sim.1 sim.2 sim.3  sirt1  sirt2  sirt3  mirt1  mirt2  mirt3
  ##   1  -3.0  -4.1  -2.8 -2.776 -3.791 -2.667 -2.856 -4.023 -2.741
  ##   5   1.7   2.3   1.8  1.791  2.330  1.844  1.817  2.373  1.869
  ##   3   0.2   0.4  -0.1  0.332  0.418 -0.046  0.349  0.421 -0.051
  ##   4   2.6   0.1  -0.9  2.601  0.171 -0.854  2.695  0.166 -0.876
  ##   2  -1.1  -0.7   0.9 -0.989 -0.605  0.957 -1.009 -0.618  0.962
#* compare estimated cluster sizes
dfr <- data.frame( "sim"=pi.k, "sirt"=mod1$pi.k[order.sirt,1],
            "mirt"=mod4@Prior[[1]][ order.mirt] )
round(dfr,4)
  ##      sim   sirt   mirt
  ##   1 0.20 0.2502 0.2500
  ##   2 0.25 0.2522 0.2511
  ##   3 0.25 0.2458 0.2494
  ##   4 0.10 0.1011 0.0986
  ##   5 0.15 0.1507 0.1509

#############################################################################
# EXAMPLE 5: Dataset data.si04 from Bartolucci et al. (2012)
#############################################################################

data(data.si04)

# define reference items
ref.item <- c(7,17,25,44,64)
dimensions <- data.si04$itempars$dim

# estimate a Rasch latent class with 9 classes
mod1 <- sirt::rasch.mirtlc( data.si04$data, Nclasses=9, dimensions=dimensions,
             modeltype="MLC1", ref.item=ref.item, glob.conv=.005, conv1=.005,
             nstarts=1, mmliter=200 )

# compare estimated distribution with simulated distribution
round( cbind( mod1$theta.k, mod1$pi.k ), 4 ) # estimated
  ##            [,1]    [,2]    [,3]    [,4]    [,5]   [,6]
  ##    [1,] -3.6043 -5.1323 -5.3022 -6.8255 -4.3611 0.1341
  ##    [2,]  0.2083 -2.7422 -2.8754 -5.3416 -2.5085 0.1573
  ##    [3,] -2.8641 -4.0272 -5.0580 -0.0340 -0.9113 0.1163
  ##    [4,] -0.3575 -2.0081 -1.7431  1.2992 -0.1616 0.0751
  ##    [5,]  2.9329  0.3662 -1.6516 -3.0284  0.1844 0.1285
  ##    [6,]  1.5092 -2.0461 -4.3093  1.0481  1.0806 0.1094
  ##    [7,]  3.9899  3.1955 -4.0010  1.8879  2.2988 0.1460
  ##    [8,]  4.3062  0.7080 -1.2324  1.4351  2.0893 0.1332
  ##    [9,]  5.0855  4.1214 -0.9141  2.2744  1.5314 0.0000

round(d2,4) # simulated
  ##         class      A      B      C      D      E     pi
  ##    [1,]     1 -3.832 -5.399 -5.793 -7.042 -4.511 0.1323
  ##    [2,]     2 -2.899 -4.217 -5.310 -0.055 -0.915 0.1162
  ##    [3,]     3 -0.376 -2.137 -1.847  1.273 -0.078 0.0752
  ##    [4,]     4  0.208 -2.934 -3.011 -5.526 -2.511 0.1583
  ##    [5,]     5  1.536 -2.137 -4.606  1.045  1.143 0.1092
  ##    [6,]     6  2.042 -0.573 -0.404 -4.331 -1.044 0.0471
  ##    [7,]     7  3.853  0.841 -2.993 -2.746  0.803 0.0822
  ##    [8,]     8  4.204  3.296 -4.328  1.892  2.419 0.1453
  ##    [9,]     9  4.466  0.700 -1.334  1.439  2.161 0.1343
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Multidimensional latent class Rasch model}
%% \keyword{summary}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: equating.rasch.Rd
%% File Version: 1.35

\name{equating.rasch}
\alias{equating.rasch}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Equating in the Generalized Logistic Rasch Model
}
\description{
This function does the linking in the generalized
logistic item response model. Only item difficulties (\eqn{b}
item parameters) are allowed. Mean-mean linking and the methods
of Haebara and Stocking-Lord are implemented (Kolen & Brennan, 2004).
}
\usage{
equating.rasch(x, y, theta=seq(-4, 4, len=100),
       alpha1=0, alpha2=0)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
Matrix with two columns: First column items, second column item
difficulties
}
  \item{y}{
Matrix with two columns: First columns item, second column item
difficulties
}
  \item{theta}{
Vector of theta values at which the linking functions
should be evaluated. If a weighting according to a prespecified normal
distribution \eqn{N( \mu,\sigma^2)} is aimed, then choose
\code{theta=stats::qnorm( seq(.001, .999, len=100), mean=mu, sd=sigma)}
}
  \item{alpha1}{
Fixed \eqn{\alpha_1} parameter in the generalized item
response model
}
  \item{alpha2}{
Fixed \eqn{\alpha_2} parameter in the generalized item
response model
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
\item{B.est}{Estimated linking constants according to the methods
\code{Mean.Mean} (Mean-mean linking), \code{Haebara} (Haebara method)
and \code{Stocking.Lord} (Stocking-Lord method).}
\item{descriptives}{Descriptives of the linking. The linking error
(\code{linkerror}) is calculated under the assumption of simple
random sampling of items}
\item{anchor}{Original and transformed item parameters of anchor items}
\item{transf.par}{Original and transformed item parameters of all items}
}
\references{
Kolen, M. J., & Brennan, R. L. (2004). \emph{Test Equating, Scaling, and Linking:
Methods and Practices}. New York: Springer.
}

%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For estimating standard errors (due to inference with respect to
the item domain) of this procedure see \code{\link{equating.rasch.jackknife}}.

For linking several studies see \code{\link{linking.haberman}} or
\code{\link{invariance.alignment}}.

A robust alternative to mean-mean linking is implemented in
\code{\link{linking.robust}}.

For linking under more general item response models
see the \pkg{plink} package.
}


\examples{
#############################################################################
# EXAMPLE 1: Linking item parameters of the PISA study
#############################################################################

data(data.pisaPars)
pars <- data.pisaPars

# linking the two studies with the Rasch model
mod <- sirt::equating.rasch(x=pars[,c("item","study1")], y=pars[,c("item","study2")])
  ##   Mean.Mean    Haebara Stocking.Lord
  ## 1   0.08828 0.08896269    0.09292838

\dontrun{
#*** linking using the plink package
# The plink package is not available on CRAN anymore.
# You can download the package with
# utils::install.packages("plink", repos="http://www2.uaem.mx/r-mirror")
library(plink)
I <- nrow(pars)
pm <- plink::as.poly.mod(I)
# linking parameters
plink.pars1 <- list( "study1"=data.frame( 1, pars$study1, 0 ),
                     "study2"=data.frame( 1, pars$study2, 0 ) )
      # the parameters are arranged in the columns:
      # Discrimination, Difficulty, Guessing Parameter
# common items
common.items <- cbind("study1"=1:I,"study2"=1:I)
# number of categories per item
cats.item <- list( "study1"=rep(2,I), "study2"=rep(2,I))
# convert into plink object
x <- plink::as.irt.pars( plink.pars1, common.items, cat=cats.item,
          poly.mod=list(pm,pm))
# linking using plink: first group is reference group
out <- plink::plink(x, rescale="MS", base.grp=1, D=1.7)
# summary for linking
summary(out)
  ##   -------  group2/group1*  -------
  ##   Linking Constants
  ##
  ##                        A         B
  ##   Mean/Mean     1.000000 -0.088280
  ##   Mean/Sigma    1.000000 -0.088280
  ##   Haebara       1.000000 -0.088515
  ##   Stocking-Lord 1.000000 -0.096610
# extract linked parameters
pars.out <- plink::link.pars(out)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Equating}
%% \keyword{Linking}

%% File Name: data.g308.Rd
%% File Version: 0.193

\name{data.g308}
\alias{data.g308}

\docType{data}
\title{
C-Test Datasets
}

\description{
Some datasets of C-tests are provided. The dataset \code{data.g308}
was used in Schroeders, Robitzsch and Schipolowski (2014).
}

\usage{
data(data.g308)
}

\format{
\itemize{
\item The dataset \code{data.g308} is a C-test containing 20 items and is
used in Schroeders, Robitzsch and Schipolowski (2014) and is of the
following format \cr

\code{'data.frame':   747 obs. of  21 variables:} \cr
\code{ $ id    : int  1 2 3 4 5 6 7 8 9 10 ...} \cr
\code{ $ G30801: int  1 1 1 1 1 0 0 1 1 1 ...} \cr
\code{ $ G30802: int  1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{ $ G30803: int  1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{ $ G30804: int  1 1 1 1 1 0 1 1 1 1 ...} \cr
\code{[...]} \cr
\code{ $ G30817: int  0 0 0 0 1 0 1 0 1 0 ...} \cr
\code{ $ G30818: int  0 0 1 0 0 0 0 1 1 0 ...} \cr
\code{ $ G30819: int  1 1 1 1 0 0 1 1 1 0 ...} \cr
\code{ $ G30820: int  1 1 1 1 0 0 0 1 1 0 ...} \cr
}
}

%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}

%\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%}

\references{
Schroeders, U., Robitzsch, A., & Schipolowski, S. (2014).
A comparison of different psychometric approaches to modeling testlet
structures: An example with C-tests.
\emph{Journal of Educational Measurement, 51}(4), 400-418.
}

\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Dataset G308 from Schroeders et al. (2014)
#############################################################################

data(data.g308)
dat <- data.g308

library(TAM)
library(sirt)

# define testlets
testlet <- c(1, 1, 2, 2, 2, 2, 2, 2, 3, 3, 4, 4, 4, 4, 4, 5, 5, 6, 6, 6)

#****************************************
#*** Model 1: Rasch model
mod1 <- TAM::tam.mml(resp=dat, control=list(maxiter=300, snodes=1500))
summary(mod1)

#****************************************
#*** Model 2: Rasch testlet model

# testlets are dimensions, assign items to Q-matrix
TT <- length(unique(testlet))
Q <- matrix(0, nrow=ncol(dat), ncol=TT + 1)
Q[,1] <- 1 # First dimension constitutes g-factor
for (tt in 1:TT){Q[testlet==tt, tt+1] <- 1}

# In a testlet model, all dimensions are uncorrelated among
# each other, that is, all pairwise correlations are set to 0,
# which can be accomplished with the "variance.fixed" command
variance.fixed <- cbind(t( utils::combn(TT+1,2)), 0)
mod2 <- TAM::tam.mml(resp=dat, Q=Q, variance.fixed=variance.fixed,
            control=list(snodes=1500, maxiter=300))
summary(mod2)

#****************************************
#*** Model 3: Partial credit model

scores <- list()
testlet.names <- NULL
dat.pcm <- NULL
for (tt in 1:max(testlet) ){
   scores[[tt]] <- rowSums (dat[, testlet==tt, drop=FALSE])
   dat.pcm <- c(dat.pcm, list(c(scores[[tt]])))
   testlet.names <- append(testlet.names, paste0("testlet",tt) )
   }
dat.pcm <- as.data.frame(dat.pcm)
colnames(dat.pcm) <- testlet.names
mod3 <- TAM::tam.mml(resp=dat.pcm, control=list(snodes=1500, maxiter=300) )
summary(mod3)

#****************************************
#*** Model 4: Copula model

mod4 <- sirt::rasch.copula2 (dat=dat, itemcluster=testlet)
summary(mod4)
}
}



%% \keyword{datasets}

%% File Name: linking.haebara.Rd
%% File Version: 0.203

\name{linking.haebara}
\alias{linking.haebara}
\alias{summary.linking.haebara}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Haebara Linking of the 2PL Model for Multiple Studies
}


\description{
The function \code{linking.haebara} is a generalization of Haebara linking
of the 2PL model to multiple groups (or multiple studies; see Battauz, 2017,
for a similar approach). The optimization estimates transformation parameters for
means and standard deviations of the groups and joint item parameters.
The function allows two different distance functions \code{dist="L2"} and
\code{dist="L1"} where the latter is a robustified version of
Haebara linking (see Details; He, Cui, & Osterlind, 2015; He & Cui, 2020;
Hu, Rogers, & Vukmirovic, 2008).
}


\usage{
linking.haebara(itempars, dist="L2", theta=seq(-4,4, length=61),
        optimizer="optim", center=FALSE, eps=1e-3, par_init=NULL, use_rcpp=TRUE,
        pow=2, use_der=TRUE, ...)

\method{summary}{linking.haebara}(object, digits=3, file=NULL, ...)
}


%- maybe also 'usage' for other objects documented here.
\arguments{
\item{itempars}{
A data frame with four or five columns. The first four columns contain
in the order: study name, item name, \eqn{a} parameter, \eqn{b} parameter.
The fifth column is an optional weight for every item and every study. See
\code{\link{linking.haberman}} for a function which uses the same argument.
}
\item{dist}{Distance function. Options are \code{"L2"} for squared loss and
\code{"L1"} for absolute value loss.}
\item{theta}{Grid of theta points for 2PL item response functions}
\item{optimizer}{Name of the optimizer chosen for alignment. Options are
   \code{"optim"} (using \code{\link[stats:optim]{stats::optim}})
   or \code{"nlminb"} (using \code{\link[stats:nlminb]{stats::nlminb}}).
}
\item{center}{Logical indicating whether means and standard deviations should
be centered after estimation}
\item{eps}{Small value for smooth approximation of the absolute value function}
\item{par_init}{Optional vector of initial parameter estimates}
\item{use_rcpp}{Logical indicating whether \pkg{Rcpp} is used for computation}
\item{pow}{Power for method \code{dist="Lq"}}
\item{use_der}{Logical indicating whether analytical derivative should be used}
\item{object}{Object of class \code{linking.haabara}.}
\item{digits}{Number of digits after decimals for rounding in \code{summary}.}
\item{file}{Optional file name if \code{summary} should be sunk into a file.}
\item{\dots}{Further arguments to be passed}
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DETAILS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



\details{
For \eqn{t=1,\ldots,T} studies, item difficulties \eqn{b_{it}} and
item slopes \eqn{a_{it}} are available. The 2PL item response functions are given by
\deqn{ logit P(X_{pi}=1| \theta_p )=a_i ( \theta_p - b_i ) }

Haebara linking compares the observed item response functions \eqn{P_{it}}
based on the equation for the logits \eqn{a_{it}(\theta - b_{it})} and the expected
item response functions \eqn{P_{it}^\ast} based on the equation for the logits
\eqn{a_i^\ast \sigma_t ( \theta - ( b_i - \mu_t)/\sigma_t )} where the joint
item parameters \eqn{a_i} and \eqn{b_i} and means \eqn{\mu_t} and standard
deviations \eqn{\sigma_t} are estimated.

Two loss functions are implemented. The quadratic loss of Haebara linking
(\code{dist="L2"}) minimizes
\deqn{f_{opt, L2}=\sum_t \sum_i \int ( P_{it} (\theta ) - P_{it}^\ast (\theta ) )^2 w(\theta)}
was originally proposed by Haebara. A robustified version (\code{dist="L1"})
uses the optimization function (He et al., 2015)
\deqn{f_{opt, L1}=\sum_t \sum_i \int | P_{it} (\theta ) - P_{it}^\ast (\theta ) |  w(\theta)}
As a further generalization, the follwing distance function (\code{dist="Lp"})
can be minimized:
\deqn{f_{opt, Lp}=\sum_t \sum_i \int | P_{it} (\theta ) - P_{it}^\ast (\theta ) |^p  w(\theta)}
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VALUES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\value{
A list with following entries
  \item{pars}{Estimated means and standard deviations (transformation parameters)}
  \item{item}{Estimated joint item parameters}
  \item{a.orig}{Original \eqn{a_{it}} parameters}
  \item{b.orig}{Original \eqn{b_{it}} parameters}
  \item{a.resid}{Residual \eqn{a_{it}} parameters (DIF parameters)}
  \item{b.resid}{Residual \eqn{b_{it}} parameters (DIF parameters)}
  \item{res_optim}{Value of optimization routine}
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% REFERENCES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\references{
Battauz, M. (2017). Multiple equating of separate IRT calibrations.
\emph{Psychometrika, 82}, 610-636.
\doi{10.1007/s11336-016-9517-x}

He, Y., Cui, Z., & Osterlind, S. J. (2015). New robust scale transformation methods
in the presence of outlying common items. \emph{Applied Psychological Measurement, 39}(8),
613-626.
\doi{10.1177/0146621615587003}

He, Y., & Cui, Z. (2020). Evaluating robust scale transformation methods with multiple
outlying common items under IRT true score equating.
\emph{Applied Psychological Measurement, 44}(4), 296-310.
\doi{10.1177/0146621619886050}

Hu, H., Rogers, W. T., & Vukmirovic, Z. (2008). Investigation of IRT-based equating
methods in the presence of outlier common items. \emph{Applied Psychological Measurement,
32}(4), 311-333.
\doi{10.1177/0146621606292215}
}


%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See \code{\link{invariance.alignment}} and \code{\link{linking.haberman}}
for alternative linking methods in the \pkg{sirt} package. See also
\code{TAM::tam.linking} in the \pkg{TAM} package for more linking functionality
and the \R packages \pkg{plink}, \pkg{equateIRT}, \pkg{equateMultiple} and
\pkg{SNSequate}.
}




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EXAMPLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Robust linking methods in the presence of outliers
#############################################################################

#** simulate data
I <- 10
a <- seq(.9, 1.1, len=I)
b <- seq(-2, 2, len=I)

#- define item parameters
item_names <- paste0("I",100+1:I)
# th=SIG*TH+MU=> logit(p)=a*(SIG*TH+MU-b)=a*SIG*(TH-(-MU)/SIG-b/SIG)
d1 <- data.frame( study="S1", item=item_names, a=a, b=b )
mu <- .5; sigma <- 1.3
d2 <- data.frame( study="S2", item=item_names, a=a*sigma, b=(b-mu)/sigma )
mu <- -.3; sigma <- .7
d3 <- data.frame( study="S3", item=item_names, a=a*sigma, b=(b-mu)/sigma )

#- define DIF effect
# dif <- 0  # no DIF effects
dif <- 1
d2[4,"a"] <- d2[4,"a"] * (1-.8*dif)
d3[5,"b"] <- d3[5,"b"] - 2*dif
itempars <- rbind(d1, d2, d3)

#* Haebara linking non-robust
mod1 <- sirt::linking.haebara( itempars, dist="L2", control=list(trace=2) )
summary(mod1)

#* Haebara linking robust
mod2 <- sirt::linking.haebara( itempars, dist="L1", control=list(trace=2) )
summary(mod2)

#* using initial parameter estimates
par_init <- mod1$res_optim$par
mod2b <- sirt::linking.haebara( itempars, dist="L1", par_init=par_init)
summary(mod2b)

#* power p=.25
mod2c <- sirt::linking.haebara( itempars, dist="Lp", pow=.25, par_init=par_init)
summary(mod2c)

#* Haberman linking non-robust
mod3 <- sirt::linking.haberman(itempars)
summary(mod3)

#* Haberman linking robust
mod4 <- sirt::linking.haberman(itempars, estimation="BSQ", a_trim=.25, b_trim=.5)
summary(mod4)

#* compare transformation parameters (means and standard deviations)
mod1$pars
mod2$pars
mod3$transf.personpars
mod4$transf.personpars
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{Linking}
%\keyword{Equating}

%% File Name: mcmc.2pno.ml.Rd
%% File Version: 0.45

\name{mcmc.2pno.ml}
\alias{mcmc.2pno.ml}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Random Item Response Model / Multilevel IRT Model
}
\description{
This function enables the estimation of random item models and multilevel
(or hierarchical) IRT models (Chaimongkol, Huffer & Kamata, 2007;
Fox & Verhagen, 2010; van den Noortgate, de Boeck & Meulders, 2003;
Asparouhov & Muthen, 2012; Muthen & Asparouhov, 2013, 2014).
Dichotomous response data is supported using a probit link. Normally
distributed responses can also be analyzed.
See Details for a description of the implemented item
response models.
}
\usage{
mcmc.2pno.ml(dat, group, link="logit", est.b.M="h", est.b.Var="n",
    est.a.M="f", est.a.Var="n", burnin=500, iter=1000,
    N.sampvalues=1000, progress.iter=50, prior.sigma2=c(1, 0.4),
    prior.sigma.b=c(1, 1), prior.sigma.a=c(1, 1), prior.omega.b=c(1, 1),
    prior.omega.a=c(1, 0.4), sigma.b.init=.3 )
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ARGUMENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\arguments{
  \item{dat}{
  Data frame with item responses.
}
  \item{group}{
Vector of group identifiers (e.g. classes, schools or countries)
}
\item{link}{Link function. Choices are \code{"logit"} for dichotomous data
    and \code{"normal"} for data under normal distribution assumptions
        }
  \item{est.b.M}{
Estimation type of \eqn{b_i} parameters: \cr \code{n} - non-hierarchical prior
distribution, i.e. \eqn{\omega_b} is set to a very high value and is not
estimated \cr \code{h} - hierarchical prior distribution with estimated
distribution parameters \eqn{\mu_b} and \eqn{\omega_b}
}
 \item{est.b.Var}{
Estimation type of standard deviations of item difficulties \eqn{b_i}. \cr
\code{n} -- no estimation of the item variance, i.e. \eqn{\sigma_{b,i}} is
assumed to be zero \cr
\code{i} -- item-specific standard deviation of item difficulties \cr
\code{j} -- a joint standard deviation of all item difficulties is estimated,
i.e. \eqn{\sigma_{b,1}=\ldots=\sigma_{b,I}=\sigma_b}
}
\item{est.a.M}{
Estimation type of \eqn{a_i} parameters: \cr
\code{f} - no estimation of item slopes, i.e all item slopes \eqn{a_i} are
fixed at one \cr
\code{n} - non-hierarchical prior distribution, i.e. \eqn{\omega_a=0} \cr
\code{h} - hierarchical prior distribution with estimated
distribution parameter \eqn{\omega_a}
}
  \item{est.a.Var}{
Estimation type of standard deviations of item slopes \eqn{a_i}. \cr
\code{n} -- no estimation of the item variance \cr
\code{i} -- item-specific standard deviation of item slopes \cr
\code{j} -- a joint standard deviation of all item slopes is estimated,
i.e. \eqn{\sigma_{a,1}=\ldots=\sigma_{a,I}=\sigma_a}
}
  \item{burnin}{
Number of burnin iterations
}
\item{iter}{
    Total number of iterations
}
  \item{N.sampvalues}{
   Maximum number of sampled values to save
}
\item{progress.iter}{
   Display progress every \code{progress.iter}-th iteration. If no progress
   display is wanted, then choose \code{progress.iter} larger than \code{iter}.
}
  \item{prior.sigma2}{
Prior for Level 2 standard deviation \eqn{\sigma_{L2}}
}
  \item{prior.sigma.b}{
Priors for item difficulty standard deviations \eqn{\sigma_{b,i}}
}
  \item{prior.sigma.a}{
Priors for item difficulty standard deviations \eqn{\sigma_{a,i}}
}
  \item{prior.omega.b}{
Prior for \eqn{\omega_b}
  }
  \item{prior.omega.a}{
Prior for \eqn{\omega_a}
}
\item{sigma.b.init}{
Initial standard deviation for \eqn{\sigma_{b,i}} parameters
}
}




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DETAILS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\details{
For dichotomous item responses (\code{link="logit"}) of persons \eqn{p} in
group \eqn{j} on
item \eqn{i}, the probability of a correct response is defined as
\deqn{P( X_{pji}=1 | \theta_{pj} )=\Phi ( a_{ij} \theta_{pj} - b_{ij} )}
The ability \eqn{\theta_{pj}} is decomposed into a Level 1 and a Level 2
effect \deqn{\theta_{pj}=u_j + e_{pj} \quad, \quad
    u_j \sim N ( 0, \sigma_{L2}^2 ) \quad, \quad
    e_{pj} \sim N ( 0, \sigma_{L1}^2 ) }
In a multilevel IRT model (or a random item model), item parameters are
allowed to vary across groups:
\deqn{ b_{ij} \sim N( b_i, \sigma^2_{b,i} ) \quad, \quad
    a_{ij} \sim N( a_i, \sigma^2_{a,i} ) }
In a hierarchical IRT model, a hierarchical distribution of the (main)
item parameters is assumed
\deqn{ b_{i} \sim N( \mu_b, \omega^2_{b} ) \quad, \quad
    a_{i} \sim N( 1, \omega^2_{a} ) }
Note that for identification purposes, the mean of all item slopes \eqn{a_i}
is set to one. Using the arguments \code{est.b.M}, \code{est.b.Var},
\code{est.a.M} and \code{est.a.Var} defines which variance components
should be estimated.

For normally distributed item responses (\code{link="normal"}), the model
equations remain the same except the item response model which is now written as
\deqn{ X_{pji}=a_{ij} \theta_{pj} - b_{ij} + \varepsilon_{pji} \quad,
\quad \varepsilon_{pji} \sim N( 0, \sigma^2_{res,i} ) }
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VALUES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\value{
A list of class \code{mcmc.sirt} with following entries:
  \item{mcmcobj}{Object of class \code{mcmc.list}}
  \item{summary.mcmcobj}{Summary of the \code{mcmcobj} object. In this
  summary the Rhat statistic and the mode estimate MAP is included.
  The variable \code{PercSEratio} indicates the proportion of the Monte Carlo
  standard error in relation to the total standard deviation of the
  posterior distribution.}
  \item{ic}{Information criteria (DIC)}
  \item{burnin}{Number of burnin iterations}
  \item{iter}{Total number of iterations}
  \item{theta.chain}{Sampled values of \eqn{\theta_{pj}} parameters}
  \item{theta.chain}{Sampled values of \eqn{u_{j}} parameters}
  \item{deviance.chain}{Sampled values of Deviance values}
  \item{EAP.rel}{EAP reliability}
  \item{person}{Data frame with EAP person parameter estimates for
        \eqn{\theta_pj} and their corresponding posterior standard
      deviations}
  \item{dat}{Used data frame}
  \item{\dots}{Further values}
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% REFERENCES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\references{
Asparouhov, T. & Muthen, B. (2012). General random effect latent variable
modeling: Random subjects, items, contexts, and parameters.
\emph{http://www.statmodel.com/papers_date.shtml}.

Chaimongkol, S., Huffer, F. W., & Kamata, A. (2007).
An explanatory differential item functioning (DIF)
model by the WinBUGS 1.4.
\emph{Songklanakarin Journal of Science and Technology, 29}, 449-458.

Fox, J.-P., & Verhagen, A.-J. (2010). Random item effects modeling for
cross-national survey data.
In E. Davidov, P. Schmidt, & J. Billiet (Eds.),
\emph{Cross-cultural Analysis: Methods and Applications}
(pp. 467-488), London: Routledge Academic.

Muthen, B. & Asparouhov, T. (2013). New methods for the study of measurement
invariance with many groups. \emph{http://www.statmodel.com/papers_date.shtml}

Muthen, B. & Asparouhov, T. (2014).
Item response modeling in Mplus: A multi-dimensional, multi-level,
and multi-timepoint example. In W. Linden & R. Hambleton (2014).
\emph{Handbook of item response theory: Models, statistical tools, and
applications}. \emph{http://www.statmodel.com/papers_date.shtml}

van den Noortgate, W., De Boeck, P., & Meulders, M. (2003).
Cross-classification multilevel logistic models in psychometrics.
\emph{Journal of Educational and Behavioral Statistics, 28}, 369-386.
}



%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
S3 methods: \code{\link{summary.mcmc.sirt}}, \code{\link{plot.mcmc.sirt}}

For MCMC estimation of three-parameter (testlet) models see
\code{\link{mcmc.3pno.testlet}}.

See also the \pkg{MLIRT} package (http://www.jean-paulfox.com).

For more flexible estimation of multilevel IRT models see the
\pkg{MCMCglmm} and \pkg{lme4} packages.
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EXAMPLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Dataset Multilevel data.ml1 - dichotomous items
#############################################################################
data(data.ml1)
dat <- data.ml1[,-1]
group <- data.ml1$group
# just for a try use a very small number of iterations
burnin <- 50 ; iter <- 100

#***
# Model 1: 1PNO with no cluster item effects
mod1 <- sirt::mcmc.2pno.ml( dat, group, est.b.Var="n", burnin=burnin, iter=iter )
summary(mod1)    # summary
plot(mod1,layout=2,ask=TRUE) # plot results
# write results to coda file
mcmclist2coda( mod1$mcmcobj, name="data.ml1_mod1" )

#***
# Model 2: 1PNO with cluster item effects of item difficulties
mod2 <- sirt::mcmc.2pno.ml( dat, group, est.b.Var="i", burnin=burnin, iter=iter )
summary(mod2)
plot(mod2, ask=TRUE, layout=2 )

#***
# Model 3: 2PNO with cluster item effects of item difficulties but
#          joint item slopes
mod3 <- sirt::mcmc.2pno.ml( dat, group, est.b.Var="i", est.a.M="h",
              burnin=burnin, iter=iter )
summary(mod3)

#***
# Model 4: 2PNO with cluster item effects of item difficulties and
#          cluster item effects with a jointly estimated SD
mod4 <- sirt::mcmc.2pno.ml( dat, group, est.b.Var="i", est.a.M="h",
                est.a.Var="j", burnin=burnin, iter=iter )
summary(mod4)

#############################################################################
# EXAMPLE 2: Dataset Multilevel data.ml2 - polytomous items
#            assuming a normal distribution for polytomous items
#############################################################################
data(data.ml2)
dat <- data.ml2[,-1]
group <- data.ml2$group
# set iterations for all examples (too few!!)
burnin <- 100 ; iter <- 500

#***
# Model 1: no intercept variance, no slopes
mod1 <- sirt::mcmc.2pno.ml( dat=dat, group=group, est.b.Var="n",
             burnin=burnin, iter=iter, link="normal",  progress.iter=20  )
summary(mod1)

#***
# Model 2a: itemwise intercept variance, no slopes
mod2a <- sirt::mcmc.2pno.ml( dat=dat, group=group, est.b.Var="i",
            burnin=burnin, iter=iter,link="normal",  progress.iter=20  )
summary(mod2a)

#***
# Model 2b: homogeneous intercept variance, no slopes
mod2b <- sirt::mcmc.2pno.ml( dat=dat, group=group, est.b.Var="j",
              burnin=burnin, iter=iter,link="normal",  progress.iter=20  )
summary(mod2b)

#***
# Model 3: intercept variance and slope variances
#          hierarchical item and slope parameters
mod3 <- sirt::mcmc.2pno.ml( dat=dat, group=group,
               est.b.M="h", est.b.Var="i", est.a.M="h", est.a.Var="i",
               burnin=burnin, iter=iter,link="normal",  progress.iter=20  )
summary(mod3)

#############################################################################
# EXAMPLE 3: Simulated random effects model | dichotomous items
#############################################################################
set.seed(7698)

#*** model parameters
sig2.lev2 <- .3^2   # theta level 2 variance
sig2.lev1 <- .8^2   # theta level 1 variance
G <- 100            # number of groups
n <- 20             # number of persons within a group
I <- 12             # number of items
#*** simuate theta
theta2 <- stats::rnorm( G, sd=sqrt(sig2.lev2) )
theta1 <- stats::rnorm( n*G, sd=sqrt(sig2.lev1) )
theta  <- theta1 + rep( theta2, each=n )
#*** item difficulties
b <- seq( -2, 2, len=I )
#*** define group identifier
group <- 1000 + rep(1:G, each=n )
#*** SD of group specific difficulties for items 3 and 5
sigma.item <- rep(0,I)
sigma.item[c(3,5)] <- 1
#*** simulate group specific item difficulties
b.class <- sapply( sigma.item, FUN=function(sii){ stats::rnorm( G, sd=sii ) } )
b.class <- b.class[ rep( 1:G,each=n ), ]
b <- matrix( b, n*G, I, byrow=TRUE ) + b.class
#*** simulate item responses
m1 <- stats::pnorm( theta - b )
dat <- 1 * ( m1 > matrix( stats::runif( n*G*I ), n*G, I ) )

#*** estimate model
mod <- sirt::mcmc.2pno.ml( dat, group=group, burnin=burnin, iter=iter,
            est.b.M="n", est.b.Var="i", progress.iter=20)
summary(mod)
plot(mod, layout=2, ask=TRUE )
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Markov Chain Monte Carlo (MCMC)}
%% \keyword{Multilevel item response model}
%% \keyword{Multilevel DIF}

%% File Name: data.pisaMath.Rd
%% File Version: 0.15

\name{data.pisaMath}
\alias{data.pisaMath}
\docType{data}
\title{
Dataset PISA Mathematics
}
\description{
This is an example PISA dataset of reading items from the PISA
2009 study of students from Austria. The
dataset contains 565 students who worked on the 11 mathematics items
from item cluster M3.
}
\usage{
data(data.pisaMath)
}


\format{
The dataset is a list. The list element \code{data}
contains the dataset with the demographical variables
student ID (\code{idstud}), school ID (\code{idschool}), a dummy variable for female
students (\code{female}), socioeconomic status (\code{hisei})
and migration background (\code{migra}). The remaining
variables (starting with \code{M} in the name) are
the mathematics items. \cr
The item metadata are included in the list element
\code{item} which contains item name (\code{item}) and the
testlet label (\code{testlet}). An item not included
in a testlet is indicated by \code{NA}.

The format is:

\code{List of 2} \cr
\code{ $ data:'data.frame':} \cr
\code{  ..$ idstud  : num [1:565] 9e+10 9e+10 9e+10 9e+10 9e+10 ...} \cr
\code{  ..$ idschool: int [1:565] 900015 900015 900015 900015  ...} \cr
\code{  ..$ female  : int [1:565] 0 0 0 0 0 0 0 0 0 0 ...} \cr
\code{  ..$ hisei   : num [1:565] -1.16 -1.099 -1.588 -0.365 -1.588 ...} \cr
\code{  ..$ migra   : int [1:565] 0 0 0 0 0 0 0 0 0 1 ...} \cr
\code{  ..$ M192Q01 : int [1:565] 1 0 1 1 1 1 1 0 0 0 ...} \cr
\code{  ..$ M406Q01 : int [1:565] 1 1 1 0 1 0 0 0 1 0 ...} \cr
\code{  ..$ M406Q02 : int [1:565] 1 0 0 0 1 0 0 0 1 0 ...} \cr
\code{  ..$ M423Q01 : int [1:565] 0 1 0 1 1 1 1 1 1 0 ...} \cr
\code{  ..$ M496Q01 : int [1:565] 1 0 0 0 0 0 0 0 1 0 ...} \cr
\code{  ..$ M496Q02 : int [1:565] 1 0 0 1 0 1 0 1 1 0 ...} \cr
\code{  ..$ M564Q01 : int [1:565] 1 1 1 1 1 1 0 0 1 0 ...} \cr
\code{  ..$ M564Q02 : int [1:565] 1 0 1 1 1 0 0 0 0 0 ...} \cr
\code{  ..$ M571Q01 : int [1:565] 1 0 0 0 1 0 0 0 0 0 ...} \cr
\code{  ..$ M603Q01 : int [1:565] 1 0 0 0 1 0 0 0 0 0 ...} \cr
\code{  ..$ M603Q02 : int [1:565] 1 0 0 0 1 0 0 0 1 0 ...} \cr
\code{ $ item:'data.frame':} \cr
\code{  ..$ item   : Factor w/ 11 levels "M192Q01","M406Q01",..: 1 2 3 4  ...} \cr
\code{  ..$ testlet: chr [1:11] NA "M406" "M406" NA ...} \cr
}



%% \keyword{datasets}

%% File Name: plot.np.dich.Rd
%% File Version: 0.18

\name{plot.np.dich}
\alias{plot.np.dich}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Plot Method for Object of Class \code{np.dich}
}
\description{
This function plots nonparametric item response
functions estimated with \code{dich.np}.
}
\usage{
\method{plot}{np.dich}(x, b, infit=NULL, outfit=NULL,
    nsize=100, askplot=TRUE, progress=TRUE, bands=FALSE,
    plot.b=FALSE, shade=FALSE, shadecol="burlywood1", ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
Object of class \code{\link{np.dich}}
}
  \item{b}{
Estimated item difficulty (threshold)
}
  \item{infit}{
Infit (optional)
}
  \item{outfit}{
Outfit (optional)
}
  \item{nsize}{
XXX
}
  \item{askplot}{
Ask for new plot?
}
  \item{progress}{
Display progress?
}
  \item{bands}{
Draw confidence bands?
}
  \item{plot.b}{
Plot difficulty parameter?
}
  \item{shade}{
Shade curves?
}
  \item{shadecol}{
Shade color
}
  \item{\dots}{
Further arguments to be passed
}
}


%\author{
%Alexander Robitzsch
%}


%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For examples see \code{\link{np.dich}}.
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Nonparametric item response theory}
%% \keyword{plot}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: modelfit.sirt.Rd
%% File Version: 0.624

\name{modelfit.sirt}
\alias{modelfit.sirt}
\alias{modelfit.cor.poly}
\alias{IRT.modelfit.sirt}

\title{
Assessing Model Fit and Local Dependence by Comparing Observed and Expected
Item Pair Correlations
}

\description{
This function computes several measures of absolute model fit and local
dependence indices for dichotomous item responses which are
based on comparing observed and expected frequencies of item pairs
(Chen, de la Torre & Zhang, 2013; see \code{\link[CDM:modelfit.cor]{modelfit.cor}}
for more details).
}
\usage{
modelfit.sirt(object)

modelfit.cor.poly( data, probs, theta.k, f.qk.yi)

\method{IRT.modelfit}{sirt}(object, mod, ...)
}
\arguments{
  \item{object}{
An object generated by \code{\link{rasch.mml2}},
\code{\link{rasch.mirtlc}}, \code{\link{rasch.pml3}} (\code{\link{rasch.pml2}}),
\code{\link{smirt}}, \code{\link{R2noharm}}, \code{\link{noharm.sirt}},
\code{\link{gom.em}}, \code{\link[TAM:tam.mml]{TAM::tam.mml}},
\code{\link[TAM:tam.mml.2pl]{TAM::tam.mml.2pl}},
\code{\link[TAM:tam.fa]{TAM::tam.fa}},
  \code{\link[mirt:mirt]{mirt::mirt}}
}
\item{data}{Dataset with polytomous item responses}
\item{probs}{Item response probabilities at grid \code{theta.k}}
\item{theta.k}{Grid of theta vector}
\item{f.qk.yi}{Individual posterior}
\item{mod}{Model name}
\item{\ldots}{Further arguments to be passed}
}

\value{
A list with following entries:
  \item{modelfit}{Model fit statistics:

   \code{MADcor}: mean of absolute deviations in observed and expected correlations
        (DiBello et al., 2007)

   \code{SRMSR}: standardized mean square root of squared residuals
        (Maydeu-Olivares, 2013; Maydeu-Olivares & Joe, 2014)

    \code{MX2}: Mean of \eqn{\chi^2} statistics of all item pairs
            (Chen & Thissen, 1997)

    \code{MADRESIDCOV}: Mean of absolute deviations of residual
            covariances (McDonald & Mok, 1995)

    \code{MADQ3}: Mean of absolute values of \eqn{Q_3} statistic (Yen, 1984)

    \code{MADaQ3}: Mean of absolute values of centered \eqn{Q_3} statistic
            }

  \item{itempairs}{Fit of every item pair}
}
\references{
Chen, W., & Thissen, D. (1997). Local dependence indexes for item pairs
 using item response theory. \emph{Journal of Educational and Behavioral Statistics,
 22}, 265-289.

  DiBello, L. V., Roussos, L. A., & Stout, W. F. (2007) Review of
  cognitively diagnostic assessment and a summary of psychometric models.
  In C. R. Rao and S. Sinharay (Eds.), \emph{Handbook of Statistics},
  Vol. 26 (pp. 979--1030). Amsterdam: Elsevier.

Maydeu-Olivares, A. (2013). Goodness-of-fit assessment of item response
theory models (with discussion).
\emph{Measurement: Interdisciplinary Research and Perspectives,
11}, 71-137.

Maydeu-Olivares, A., & Joe, H. (2014). Assessing approximate fit in categorical
data analysis. \emph{Multivariate Behavioral Research, 49}, 305-328.

  McDonald, R. P., & Mok, M. M.-C. (1995). Goodness of fit in item response models.
  \emph{Multivariate Behavioral Research, 30}, 23-40.

Yen, W. M. (1984). Effects of local item dependence on the fit and equating
performance of the three-parameter logistic model.
\emph{Applied Psychological Measurement, 8}, 125-145.
}


%\author{
%Alexander Robitzsch
%}




\note{
The function \code{modelfit.cor.poly} is just a wrapper to
\code{\link[TAM:tam.modelfit]{TAM::tam.modelfit}} in the \pkg{TAM} package.
}

\seealso{
Supported classes: \code{\link{rasch.mml2}},
\code{\link{rasch.mirtlc}}, \code{\link{rasch.pml3}} (\code{\link{rasch.pml2}}),
\code{\link{smirt}}, \code{\link{R2noharm}}, \code{\link{noharm.sirt}},
\code{\link{gom.em}},
\code{\link[TAM:tam.mml]{TAM::tam.mml}},
\code{\link[TAM:tam.mml.2pl]{TAM::tam.mml.2pl}},
\code{\link[TAM:tam.fa]{TAM::tam.fa}},
  \code{\link[mirt:mirt]{mirt::mirt}}

For more details on fit statistics of this function
see \code{\link[CDM:modelfit.cor]{CDM::modelfit.cor}}.
 }

\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Reading data
#############################################################################
data(data.read)
dat <- data.read
I <- ncol(dat)

#*** Model 1: Rasch model
mod1 <- sirt::rasch.mml2(dat)
fmod1 <- sirt::modelfit.sirt( mod1 )
summary(fmod1)

#*** Model 1b: Rasch model in TAM package
library(TAM)
mod1b <- TAM::tam.mml(dat)
fmod1b <- sirt::modelfit.sirt( mod1b )
summary(fmod1b)

#*** Model 2: Rasch model with smoothed distribution
mod2 <- sirt::rasch.mml2( dat, distribution.trait="smooth3" )
fmod2 <- sirt::modelfit.sirt( mod2 )
summary(fmod2 )

#*** Model 3: 2PL model
mod3 <- sirt::rasch.mml2( dat, distribution.trait="normal", est.a=1:I )
fmod3 <- sirt::modelfit.sirt( mod3 )
summary(fmod3 )

#*** Model 3: 2PL model in TAM package
mod3b <- TAM::tam.mml.2pl( dat )
fmod3b <- sirt::modelfit.sirt(mod3b)
summary(fmod3b)
# model fit in TAM package
tmod3b <- TAM::tam.modelfit(mod3b)
summary(tmod3b)
# model fit in mirt package
library(mirt)
mmod3b <- sirt::tam2mirt(mod3b)   # convert to mirt object
mirt::M2(mmod3b$mirt)         # global fit statistic
mirt::residuals( mmod3b$mirt, type="LD")  # local dependence statistics

#*** Model 4: 3PL model with equal guessing parameter
mod4 <- TAM::rasch.mml2( dat, distribution.trait="smooth3", est.a=1:I, est.c=rep(1,I) )
fmod4 <- sirt::modelfit.sirt( mod4 )
summary(fmod4 )

#*** Model 5: Latent class model with 2 classes
mod5 <- sirt::rasch.mirtlc( dat, Nclasses=2 )
fmod5 <- sirt::modelfit.sirt( mod5 )
summary(fmod5 )

#*** Model 6: Rasch latent class model with 3 classes
mod6 <- sirt::rasch.mirtlc( dat, Nclasses=3, modeltype="MLC1", mmliter=100)
fmod6 <- sirt::modelfit.sirt( mod6 )
summary(fmod6 )

#*** Model 7: PML estimation
mod7 <- sirt::rasch.pml3( dat )
fmod7 <- sirt::modelfit.sirt( mod7 )
summary(fmod7 )

#*** Model 8: PML estimation
#      Modelling error correlations:
#          joint residual correlations for each item cluster
error.corr <- diag(1,ncol(dat))
itemcluster <- rep( 1:4,each=3 )
for ( ii in 1:3){
    ind.ii <- which( itemcluster==ii )
    error.corr[ ind.ii, ind.ii ] <- ii
        }
mod8 <- sirt::rasch.pml3( dat, error.corr=error.corr )
fmod8 <- sirt::modelfit.sirt( mod8 )
summary(fmod8 )

#*** Model 9: 1PL in smirt
Qmatrix <- matrix( 1, nrow=I, ncol=1 )
mod9 <- sirt::smirt( dat, Qmatrix=Qmatrix )
fmod9 <- sirt::modelfit.sirt( mod9 )
summary(fmod9 )

#*** Model 10: 3-dimensional Rasch model in NOHARM
noharm.path <- "c:/NOHARM"
Q <- matrix( 0, nrow=12, ncol=3 )
Q[ cbind(1:12, rep(1:3,each=4) ) ] <- 1
rownames(Q) <- colnames(dat)
colnames(Q) <- c("A","B","C")
# covariance matrix
P.pattern <- matrix( 1, ncol=3, nrow=3 )
P.init <- 0.8+0*P.pattern
diag(P.init) <- 1
# loading matrix
F.pattern <- 0*Q
F.init <- Q
# estimate model
mod10 <- sirt::R2noharm( dat=dat, model.type="CFA", F.pattern=F.pattern,
            F.init=F.init, P.pattern=P.pattern, P.init=P.init,
            writename="ex4e", noharm.path=noharm.path, dec="," )
fmod10 <- sirt::modelfit.sirt( mod10 )
summary(fmod10)

#*** Model 11: Rasch model in mirt package
library(mirt)
mod11 <- mirt::mirt(dat, 1, itemtype="Rasch",verbose=TRUE)
fmod11 <- sirt::modelfit.sirt( mod11 )
summary(fmod11)
# model fit in mirt package
mirt::M2(mod11)
mirt::residuals(mod11)
}
}
%% \keyword{Model fit}

%% File Name: xxirt.Rd
%% File Version: 0.540

\name{xxirt}
\alias{xxirt}
\alias{summary.xxirt}
\alias{print.xxirt}
\alias{logLik.xxirt}
\alias{anova.xxirt}
\alias{coef.xxirt}
\alias{vcov.xxirt}
\alias{confint.xxirt}
\alias{IRT.se.xxirt}
\alias{IRT.expectedCounts.xxirt}
\alias{IRT.irfprob.xxirt}
\alias{IRT.likelihood.xxirt}
\alias{IRT.posterior.xxirt}
\alias{IRT.modelfit.xxirt}
\alias{summary.IRT.modelfit.xxirt}
\alias{IRT.factor.scores.xxirt}
\alias{xxirt_hessian}
\alias{xxirt_sandwich_pml}


%- Also NEED an '\alias' for EACH other topic documented here.
\title{
User Defined Item Response Model
}

\description{
Estimates a user defined item response model. Both, item response functions
and latent trait distributions can be specified by the user (see Details).
By default, the EM algorithm is used for estimation. The number of maximum
EM iterations can be defined with the argument \code{maxit}. The \code{xxirt}
function also allows Newton-Raphson optimization by specifying values of maximum
number of iterations in \code{maxit_nr} larger than zero. Typically, a small initial
number of EM iterations should be chosen to obtain reasonable starting values.
}

\usage{
xxirt(dat, Theta=NULL, itemtype=NULL, customItems=NULL, partable=NULL,
       customTheta=NULL, group=NULL, weights=NULL, globconv=1e-06, conv=1e-04,
       maxit=1000, mstep_iter=4, mstep_reltol=1e-06, maxit_nr=0, optimizer_nr="nlminb",
       estimator="ML", control_nr=list(trace=1), h=1E-4, use_grad=TRUE, verbose=TRUE,
       penalty_fun_item=NULL, np_fun_item=NULL, penalty_fun_theta=NULL,
       np_fun_theta=NULL, pml_args=NULL, verbose_index=NULL,
       cv_kfold=0, cv_maxit=10)

\method{summary}{xxirt}(object, digits=3, file=NULL, ...)

\method{print}{xxirt}(x, ...)

\method{anova}{xxirt}(object,...)

\method{coef}{xxirt}(object,...)

\method{logLik}{xxirt}(object,...)

\method{vcov}{xxirt}(object,...)

\method{confint}{xxirt}(object, parm, level=.95, ... )

\method{IRT.expectedCounts}{xxirt}(object,...)

\method{IRT.factor.scores}{xxirt}(object, type="EAP", ...)

\method{IRT.irfprob}{xxirt}(object,...)

\method{IRT.likelihood}{xxirt}(object,...)

\method{IRT.posterior}{xxirt}(object,...)

\method{IRT.modelfit}{xxirt}(object,...)

\method{summary}{IRT.modelfit.xxirt}(object,...)

\method{IRT.se}{xxirt}(object,...)

# computes Hessian matrix
xxirt_hessian(object, h=1e-4, use_shortcut=TRUE)

#- sandwich estimate for pairwise maximum likelihood estimation
xxirt_sandwich_pml(object, h=1e-4)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame with item responses
}
  \item{Theta}{
Matrix with \eqn{\bold{\theta}} grid vector of latent trait
}
  \item{itemtype}{
Vector of item types
}
  \item{customItems}{
List containing types of item response functions created by
\code{\link{xxirt_createDiscItem}}.
}
 \item{partable}{
Item parameter table which is initially created by
\code{\link{xxirt_createParTable}} and which can be modified by
\code{\link{xxirt_modifyParTable}}.
 }
  \item{customTheta}{
User defined \eqn{\bold{\theta}} distribution created by
\code{\link{xxirt_createThetaDistribution}}.
}
  \item{group}{
Optional vector of group indicators
}
  \item{weights}{
Optional vector of person weights
}
  \item{globconv}{
Convergence criterion for relative change in deviance
}
  \item{conv}{
Convergence criterion for absolute change in parameters
}
  \item{maxit}{
Maximum number of iterations in the EM algorithm
}
  \item{mstep_iter}{
Maximum number of iterations in M-step
}
  \item{mstep_reltol}{
Convergence criterion in M-step
}
\item{maxit_nr}{Number of Newton-Raphson iterations after EM algorithm}
\item{optimizer_nr}{Type of optimizer for Newton-Raphson optimization.
Alternatives are \code{"optim"} or \code{"nlminb"} or other options of
\code{\link{sirt_optimizer}}.}
\item{estimator}{Marginal maximum likelihood (\code{"ML"}) or pairwise
maximum likelihood (\code{"PML"})
}
\item{control_nr}{Argument \code{control} for optimizer.}
\item{h}{Numerical differentiation parameter}
\item{use_grad}{Logical indicating whether the gradient should be supplied
to \code{\link[stats:optim]{stats::optim}}}
  \item{verbose}{
Logical indicating whether iteration progress should be displayed
}
\item{penalty_fun_item}{Optional penalty function used in regularized
estimation. Used as a function of \code{x} (vector of item parameters)}
\item{np_fun_item}{Function that counts the number of item parameters in regularized
estimation. Used as a function of \code{x} (vector of item parameters)}
\item{penalty_fun_theta}{Optional penalty function used in regularized
estimation. Used as a function of \code{x} (vector of theta parameters)}
\item{np_fun_theta}{Function that counts the number of theta parameters in regularized
estimation. Used as a function of \code{x} (vector of theta parameters)}
\item{object}{Object of class \code{xxirt}}
\item{digits}{Number of digits to be rounded}
\item{file}{Optional file name to which \code{summary} output is written}
\item{parm}{Optional vector of parameters}
\item{level}{Confidence level}
\item{pml_args}{Weight matrices for \code{estimator="PML"}}
\item{verbose_index}{Logical indicating whether item index should be
printed in estimation output}
\item{cv_kfold}{Number of k folds in cross validation. The default is 0 (no
cross-validation)}
\item{cv_maxit}{Maximum number of iterations for each cross-validation sample}
\item{x}{Object of class \code{xxirt}}
\item{type}{Type of person parameter estimate. Currently, only
\code{EAP} is implemented.}
\item{use_shortcut}{Logical indicating whether a shortcut in the computation
should be utilized}
\item{\dots}{
Further arguments to be passed
}
}

\details{
Item response functions can be specified as functions of unknown parameters
\eqn{\bold{\delta}_i} such that
\eqn{P(X_{i}=x | \bold{\theta})=f_i( x | \bold{\theta} ; \bold{\delta}_i  )}
The item response model is estimated under the assumption of
local stochastic independence of items. Equality constraints of
item parameters \eqn{\bold{\delta}_i} among items are allowed.

The probability distribution \eqn{P(\bold{\theta})} are specified as functions
of an unknown parameter vector \eqn{\bold{\gamma}}.

A penalty function for item parameters can be specified in
\code{penalty_fun_item}. The penalty function should be differentiable and
a non-differentiable function (e.g., the absolute value function) should
be approximated by a differentiable function.
}

\value{
List with following entries

\item{partable}{Item parameter table}
\item{par_items}{Vector with estimated item parameters}
\item{par_items_summary}{Data frame with item parameters}
\item{par_items_bounds}{Data frame with summary on bounds of estimated
 item parameters}
\item{par_Theta}{Vector with estimated parameters of theta distribution}
\item{Theta}{Matrix with \eqn{\bold{\theta}} grid}
\item{probs_items}{Item response functions}
\item{probs_Theta}{Theta distribution}
\item{deviance}{Deviance}
\item{loglik}{Log likelihood value}
\item{ic}{Information criteria}
\item{item_list}{List with item functions}
\item{customItems}{Used customized item response functions}
\item{customTheta}{Used customized theta distribution}
\item{cv_loglike}{Cross-validated log-likelihood value (if \code{cv_kfold>0})}
\item{p.xi.aj}{Individual likelihood}
\item{p.aj.xi}{Individual posterior}
\item{ll_case}{Case-wise log-likelihood values}
\item{n.ik}{Array of expected counts}
\item{EAP}{EAP person parameter estimates}
\item{dat}{Used dataset with item responses}
\item{dat_resp}{Dataset with response indicators}
\item{weights}{Vector of person weights}
\item{G}{Number of groups}
\item{group}{Integer vector of group indicators}
\item{group_orig}{Vector of original group_identifiers}
\item{ncat}{Number of categories per item}
\item{converged}{Logical whether model has converged}
\item{iter}{Number of iterations needed}
}

%\references{
%% ~put references to the literature/web site here ~
%}



%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See the \code{\link[mirt:createItem]{mirt::createItem}} and
\code{\link[mirt:mirt]{mirt::mirt}} functions in the \pkg{mirt}
package for similar functionality.
}

\examples{
\dontrun{
#############################################################################
## EXAMPLE 1: Unidimensional item response functions
#############################################################################

data(data.read)
dat <- data.read

#------ Definition of item response functions

#*** IRF 2PL
P_2PL <- function( par, Theta, ncat){
    a <- par[1]
    b <- par[2]
    TP <- nrow(Theta)
    P <- matrix( NA, nrow=TP, ncol=ncat)
    P[,1] <- 1
    for (cc in 2:ncat){
        P[,cc] <- exp( (cc-1) * a * Theta[,1] - b )
    }
    P <- P / rowSums(P)
    return(P)
}

#*** IRF 1PL
P_1PL <- function( par, Theta, ncat){
    b <- par[1]
    TP <- nrow(Theta)
    P <- matrix( NA, nrow=TP, ncol=ncat)
    P[,1] <- 1
    for (cc in 2:ncat){
        P[,cc] <- exp( (cc-1) * Theta[,1] - b )
    }
    P <- P / rowSums(P)
    return(P)
}

#** created item classes of 1PL and 2PL models
par <- c( "a"=1, "b"=0 )
# define some slightly informative prior of 2PL
item_2PL <- sirt::xxirt_createDiscItem( name="2PL", par=par, est=c(TRUE,TRUE),
               P=P_2PL, prior=c(a="dlnorm"), prior_par1=c( a=0 ),
               prior_par2=c(a=5) )
item_1PL <- sirt::xxirt_createDiscItem( name="1PL", par=par[2], est=c(TRUE),
               P=P_1PL )
customItems <- list( item_1PL,  item_2PL )

#---- definition theta distribution

#** theta grid
Theta <- matrix( seq(-6,6,length=21), ncol=1 )

#** theta distribution
P_Theta1 <- function( par, Theta, G){
    mu <- par[1]
    sigma <- max( par[2], .01 )
    TP <- nrow(Theta)
    pi_Theta <- matrix( 0, nrow=TP, ncol=G)
    pi1 <- dnorm( Theta[,1], mean=mu, sd=sigma )
    pi1 <- pi1 / sum(pi1)
    pi_Theta[,1] <- pi1
    return(pi_Theta)
}
#** create distribution class
par_Theta <- c( "mu"=0, "sigma"=1 )
customTheta  <- sirt::xxirt_createThetaDistribution( par=par_Theta, est=c(FALSE,TRUE),
                       P=P_Theta1 )

#****************************************************************************
#******* Model 1: Rasch model

#-- create parameter table
itemtype <- rep( "1PL", 12 )
partable <- sirt::xxirt_createParTable( dat, itemtype=itemtype,
                        customItems=customItems )

# estimate model
mod1 <- sirt::xxirt( dat=dat, Theta=Theta, partable=partable,
                   customItems=customItems, customTheta=customTheta)
summary(mod1)

# estimate Rasch model by providing starting values
partable1 <- sirt::xxirt_modifyParTable( partable, parname="b",
                   value=- stats::qlogis( colMeans(dat) ) )
# estimate model again
mod1b <- sirt::xxirt( dat=dat, Theta=Theta, partable=partable1,
                   customItems=customItems, customTheta=customTheta )
summary(mod1b)

# extract coefficients, covariance matrix and standard errors
coef(mod1b)
vcov(mod1b)
IRT.se(mod1b)

#** start with EM and finalize with Newton-Raphson algorithm
mod1c <- sirt::xxirt( dat=dat, Theta=Theta, partable=partable,
                   customItems=customItems, customTheta=customTheta,
                   maxit=20, maxit_nr=300)
summary(mod1c)

#****************************************************************************
#******* Model 2: 2PL Model with three groups of item discriminations

#-- create parameter table
itemtype <- rep( "2PL", 12 )
partable <- sirt::xxirt_createParTable( dat, itemtype=itemtype, customItems=customItems)
# modify parameter table: set constraints for item groups A, B and C
partable1 <- sirt::xxirt_modifyParTable(partable, item=paste0("A",1:4),
                         parname="a", parindex=111)
partable1 <- sirt::xxirt_modifyParTable(partable1, item=paste0("B",1:4),
                         parname="a", parindex=112)
partable1 <- sirt::xxirt_modifyParTable(partable1, item=paste0("C",1:4),
                         parname="a", parindex=113)
# delete prior distributions
partable1 <- sirt::xxirt_modifyParTable(partable1, parname="a", prior=NA)

#-- fix sigma to 1
customTheta1 <- customTheta
customTheta1$est <- c("mu"=FALSE,"sigma"=FALSE )

# estimate model
mod2 <- sirt::xxirt( dat=dat, Theta=Theta, partable=partable1,
                  customItems=customItems, customTheta=customTheta1 )
summary(mod2)

#****************************************************************************
#******* Model 3: Cloglog link function

#*** IRF cloglog
P_1N <- function( par, Theta, ncat){
    b <- par
    TP <- nrow(Theta)
    P <- matrix( NA, nrow=TP, ncol=ncat)
    P[,2] <- 1 - exp( - exp( Theta - b ) )
    P[,1] <- 1 - P[,2]
    return(P)
}
par <- c("b"=0)
item_1N <- sirt::xxirt_createDiscItem( name="1N", par=par, est=c(TRUE),
                    P=P_1N )
customItems <- list( item_1N )
itemtype <- rep( "1N", I )
partable <- sirt::xxirt_createParTable( dat[,items], itemtype=itemtype,
                      customItems=customItems )
partable <- sirt::xxirt_modifyParTable( partable=partable, parname="b",
                 value=- stats::qnorm( colMeans(dat[,items] )) )

#*** estimate model
mod3 <- sirt::xxirt( dat=dat, Theta=Theta, partable=partable, customItems=customItems,
                customTheta=customTheta )
summary(mod3)
IRT.compareModels(mod1,mod3)

#****************************************************************************
#******* Model 4: Latent class model

K <- 3 # number of classes
Theta <- diag(K)

#*** Theta distribution
P_Theta1 <- function( par, Theta, G  ){
    logitprobs <- par[1:(K-1)]
    l1 <- exp( c( logitprobs, 0 ) )
    probs <- matrix( l1/sum(l1), ncol=1)
    return(probs)
}

par_Theta <- stats::qlogis( rep( 1/K, K-1 ) )
names(par_Theta) <- paste0("pi",1:(K-1) )
customTheta  <- sirt::xxirt_createThetaDistribution( par=par_Theta,
                     est=rep(TRUE,K-1), P=P_Theta1)

#*** IRF latent class
P_lc <- function( par, Theta, ncat){
    b <- par
    TP <- nrow(Theta)
    P <- matrix( NA, nrow=TP, ncol=ncat)
    P[,1] <- 1
    for (cc in 2:ncat){
        P[,cc] <- exp( Theta \%*\% b )
    }
    P <- P / rowSums(P)
    return(P)
}
par <- seq( -1.5, 1.5, length=K )
names(par) <- paste0("b",1:K)
item_lc <- sirt::xxirt_createDiscItem( name="LC", par=par,
                 est=rep(TRUE,K), P=P_lc )
customItems <- list( item_lc )

# create parameter table
itemtype <- rep( "LC", 12 )
partable <- sirt::xxirt_createParTable( dat, itemtype=itemtype, customItems=customItems)
partable

#*** estimate model
mod4 <- sirt::xxirt( dat=dat, Theta=Theta, partable=partable, customItems=customItems,
                customTheta=customTheta)
summary(mod4)
# class probabilities
mod4$probs_Theta
# item response functions
imod4 <- IRT.irfprob( mod5 )
round( imod4[,2,], 3 )

#****************************************************************************
#******* Model 5: Ordered latent class model

K <- 3 # number of classes
Theta <- diag(K)
Theta <- apply( Theta, 1, cumsum )

#*** Theta distribution
P_Theta1 <- function( par, Theta, G  ){
    logitprobs <- par[1:(K-1)]
    l1 <- exp( c( logitprobs, 0 ) )
    probs <- matrix( l1/sum(l1), ncol=1)
    return(probs)
}
par_Theta <- stats::qlogis( rep( 1/K, K-1 ) )
names(par_Theta) <- paste0("pi",1:(K-1) )
customTheta  <- sirt::xxirt_createThetaDistribution( par=par_Theta,
                est=rep(TRUE,K-1), P=P_Theta1  )

#*** IRF ordered latent class
P_olc <- function( par, Theta, ncat){
    b <- par
    TP <- nrow(Theta)
    P <- matrix( NA, nrow=TP, ncol=ncat)
    P[,1] <- 1
    for (cc in 2:ncat){
        P[,cc] <- exp( Theta \%*\% b )
    }
    P <- P / rowSums(P)
    return(P)
}

par <- c( -1, rep( .5,, length=K-1 ) )
names(par) <- paste0("b",1:K)
item_olc <- sirt::xxirt_createDiscItem( name="OLC", par=par, est=rep(TRUE,K),
                    P=P_olc, lower=c( -Inf, 0, 0 ) )
customItems <- list( item_olc )
itemtype <- rep( "OLC", 12 )
partable <- sirt::xxirt_createParTable( dat, itemtype=itemtype, customItems=customItems)
partable

#*** estimate model
mod5 <- sirt::xxirt( dat=dat, Theta=Theta, partable=partable, customItems=customItems,
                customTheta=customTheta )
summary(mod5)
# estimated item response functions
imod5 <- IRT.irfprob( mod5 )
round( imod5[,2,], 3 )

#############################################################################
## EXAMPLE 2: Multiple group models with xxirt
#############################################################################

data(data.math)
dat <- data.math$data
items <- grep( "M[A-Z]", colnames(dat), value=TRUE )
I <- length(items)

Theta <- matrix( seq(-8,8,len=31), ncol=1 )

#****************************************************************************
#******* Model 1: Rasch model, single group

#*** Theta distribution
P_Theta1 <- function( par, Theta, G  ){
    mu <- par[1]
    sigma <- max( par[2], .01 )
    p1 <- stats::dnorm( Theta[,1], mean=mu, sd=sigma)
    p1 <- p1 / sum(p1)
    probs <- matrix( p1, ncol=1)
    return(probs)
}

par_Theta <- c(0,1)
names(par_Theta) <- c("mu","sigma")
customTheta  <- sirt::xxirt_createThetaDistribution( par=par_Theta,
                   est=c(FALSE,TRUE), P=P_Theta1  )
customTheta

#*** IRF 1PL logit
P_1PL <- function( par, Theta, ncat){
    b <- par
    TP <- nrow(Theta)
    P <- matrix( NA, nrow=TP, ncol=ncat)
    P[,2] <- plogis( Theta - b )
    P[,1] <- 1 - P[,2]
    return(P)
}
par <- c("b"=0)
item_1PL <- sirt::xxirt_createDiscItem( name="1PL", par=par, est=c(TRUE), P=P_1PL)
customItems <- list( item_1PL )

itemtype <- rep( "1PL", I )
partable <- sirt::xxirt_createParTable( dat[,items], itemtype=itemtype,
                       customItems=customItems )
partable <- sirt::xxirt_modifyParTable( partable=partable, parname="b",
                  value=- stats::qlogis( colMeans(dat[,items] )) )

#*** estimate model
mod1 <- sirt::xxirt( dat=dat[,items], Theta=Theta, partable=partable,
                customItems=customItems, customTheta=customTheta )
summary(mod1)

#****************************************************************************
#******* Model 2: Rasch model, multiple groups

#*** Theta distribution
P_Theta2 <- function( par, Theta, G  ){
    mu1 <- par[1]
    mu2 <- par[2]
    sigma1 <- max( par[3], .01 )
    sigma2 <- max( par[4], .01 )
    TP <- nrow(Theta)
    probs <- matrix( NA, nrow=TP, ncol=G)
    p1 <- stats::dnorm( Theta[,1], mean=mu1, sd=sigma1)
    probs[,1] <- p1 / sum(p1)
    p1 <- stats::dnorm( Theta[,1], mean=mu2, sd=sigma2)
    probs[,2] <- p1 / sum(p1)
    return(probs)
}
par_Theta <- c(0,0,1,1)
names(par_Theta) <- c("mu1","mu2","sigma1","sigma2")
customTheta2  <- sirt::xxirt_createThetaDistribution( par=par_Theta,
                    est=c(FALSE,TRUE,TRUE,TRUE), P=P_Theta2  )
print(customTheta2)

#*** estimate model
mod2 <- sirt::xxirt( dat=dat[,items], group=dat$female, Theta=Theta, partable=partable,
           customItems=customItems, customTheta=customTheta2, maxit=40)
summary(mod2)
IRT.compareModels(mod1, mod2)

#*** compare results with TAM package
library(TAM)
mod2b <- TAM::tam.mml( resp=dat[,items], group=dat$female )
summary(mod2b)
IRT.compareModels(mod1, mod2, mod2b)

#############################################################################
## EXAMPLE 3: Regularized 2PL model
#############################################################################

data(data.read, package="sirt")
dat <- data.read

#------ Definition of item response functions

#*** IRF 2PL
P_2PL <- function( par, Theta, ncat){
    a <- par[1]
    b <- par[2]
    TP <- nrow(Theta)
    P <- matrix( NA, nrow=TP, ncol=ncat)
    P[,1] <- 1
    for (cc in 2:ncat){
        P[,cc] <- exp( (cc-1) * a * Theta[,1] - b )
    }
    P <- P / rowSums(P)
    return(P)
}

#** created item classes of 1PL and 2PL models
par <- c( "a"=1, "b"=0 )
# define some slightly informative prior of 2PL
item_2PL <- sirt::xxirt_createDiscItem( name="2PL", par=par, est=c(TRUE,TRUE),
               P=P_2PL, prior=c(a="dlnorm"), prior_par1=c( a=0 ),
               prior_par2=c(a=5) )
customItems <- list( item_2PL )

#---- definition theta distribution

#** theta grid
Theta <- matrix( seq(-6,6,length=21), ncol=1 )

#** theta distribution
P_Theta1 <- function( par, Theta, G){
    mu <- par[1]
    sigma <- max( par[2], .01 )
    TP <- nrow(Theta)
    pi_Theta <- matrix( 0, nrow=TP, ncol=G)
    pi1 <- dnorm( Theta[,1], mean=mu, sd=sigma )
    pi1 <- pi1 / sum(pi1)
    pi_Theta[,1] <- pi1
    return(pi_Theta)
}
#** create distribution class
par_Theta <- c( "mu"=0, "sigma"=1 )
customTheta  <- sirt::xxirt_createThetaDistribution( par=par_Theta, est=c(FALSE,FALSE),
                       P=P_Theta1 )

#****************************************************************************
#******* Model 1: 2PL model

itemtype <- rep( "2PL", 12 )
partable <- sirt::xxirt_createParTable( dat, itemtype=itemtype,
                        customItems=customItems )

mod1 <- sirt::xxirt( dat=dat, Theta=Theta, partable=partable,
                   customItems=customItems, customTheta=customTheta)
summary(mod1)

#****************************************************************************
#******* Model 2: Regularized 2PL model with regularization on item loadings

# define regularized estimation of item loadings
parindex <- partable[ partable$parname=="a","parindex"]

#** penalty is defined by -N*lambda*sum_i (a_i-1)^2
N <- nrow(dat)
lambda <- .02
penalty_fun_item <- function(x)
{
    val <- N*lambda*sum( ( x[parindex]-1)^2)
    return(val)
}
# estimate standard deviation
customTheta1  <- sirt::xxirt_createThetaDistribution( par=par_Theta, est=c(FALSE,TRUE),
                       P=P_Theta1 )
mod2 <- sirt::xxirt( dat=dat, Theta=Theta, partable=partable,
                   customItems=customItems, customTheta=customTheta1,
                   penalty_fun_item=penalty_fun_item)
summary(mod2)

#############################################################################
## EXAMPLE 4: 2PL mixture model
#############################################################################

#*** simulate data
set.seed(123)
N <- 4000   # number of persons
I <- 15     # number of items
prop <- .25 # mixture proportion for second class

# discriminations and difficulties in first class
a1 <- rep(1,I)
b1 <- seq(-2,2,len=I)
# distribution in second class
mu2 <- 1
sigma2 <- 1.2
# compute parameters with constraint N(0,1) in second class
# a*(sigma*theta+mu-b)=a*sigma*(theta-(b-mu)/sigma)
#=> a2=a*sigma and b2=(b-mu)/sigma
a2 <- a1
a2[c(2,4,6,8)] <- 0.2  # some items with different discriminations
a2 <- a2*sigma2
b2 <- b1
b2[1:5] <- 1   # first 5 item with different difficulties
b2 <- (b2-mu2)/sigma2
dat1 <- sirt::sim.raschtype(theta=stats::rnorm(N*(1-prop)), b=b1, fixed.a=a1)
dat2 <- sirt::sim.raschtype(theta=stats::rnorm(N*prop), b=b2, fixed.a=a2)
dat <- rbind(dat1, dat2)

#**** model specification

#*** define theta distribution
TP <- 21
theta <- seq(-6,6,length=TP)
# stack theta vectors below each others=> 2 latent classes
Theta <- matrix( c(theta, theta ), ncol=1 )
# distribution of theta (i.e., N(0,1))
w_theta <- dnorm(theta)
w_theta <- w_theta / sum(w_theta)

P_Theta1 <- function( par, Theta, G){
    p2_logis <- par[1]
    p2 <- stats::plogis( p2_logis )
    p1 <- 1-p2
    pi_Theta <- c( p1*w_theta, p2*w_theta)
    pi_Theta <- matrix(pi_Theta, ncol=1)
    return(pi_Theta)
}

par_Theta <- c( p2_logis=qlogis(.25))
customTheta  <- sirt::xxirt_createThetaDistribution( par=par_Theta, est=c(TRUE),
                       P=P_Theta1)

# IRF for 2-class mixture 2PL model
par <- c(a1=1, a2=1, b1=0, b2=.5)

P_2PLmix <- function( par, Theta, ncat)
{
    a1 <- par[1]
    a2 <- par[2]
    b1 <- par[3]
    b2 <- par[4]
    P <- matrix( NA, nrow=2*TP, ncol=ncat)
    TP <- nrow(Theta)/2
    P1 <- stats::plogis( a1*(Theta[1:TP,1]-b1) )
    P2 <- stats::plogis( a2*(Theta[TP+1:(2*TP),1]-b2) )
    P[,2] <- c(P1, P2)
    P[,1] <- 1-P[,2]
    return(P)
}

# define some slightly informative prior of 2PL
item_2PLmix <- sirt::xxirt_createDiscItem( name="2PLmix", par=par,
               est=c(TRUE,TRUE,TRUE,TRUE), P=P_2PLmix )
customItems <- list( item_2PLmix )

#****************************************************************************
#******* Model 1: 2PL mixture model

itemtype <- rep( "2PLmix", I )
partable <- sirt::xxirt_createParTable( dat, itemtype=itemtype,
                        customItems=customItems )
mod1 <- sirt::xxirt( dat=dat, Theta=Theta, partable=partable,
                   customItems=customItems, customTheta=customTheta)
summary(mod1)

#############################################################################
## EXAMPLE 5: Partial credit model with MML and PML
#############################################################################

data(data.gpcm, package="TAM")
dat <- data.gpcm

# recode data and include some missings
dat[ dat[,1]==3, 1] <- 2
dat[ 1, c(1,2) ] <- NA
dat[2,3] <- NA

#------ Definition of item response functions
#*** IRF 2PL
P_2PL <- function( par, Theta, ncat)
{
    b <- par
    TP <- nrow(Theta)
    P <- matrix( NA, nrow=TP, ncol=ncat)
    P[,1] <- 1
    for (cc in 2:ncat){
        P[,cc] <- exp( Theta[,1] - b[cc-1] )
    }
    P <- P / rowSums(P)
    return(P)
}

P_PCM2 <- function( par, Theta, ncat=3)
{
    P <- P_2PL(par=par, Theta=Theta, ncat=ncat)
    return(P)
}

P_PCM3 <- function( par, Theta, ncat=4)
{
    P <- P_2PL(par=par, Theta=Theta, ncat=ncat)
    return(P)
}

# define some slightly informative prior of 2PL
par <- c( b1=-1, b2=1 )
NP <- length(par)
item_PCM2 <- sirt::xxirt_createDiscItem( name="PCM2", par=par, est=rep(TRUE,NP),
               P=P_PCM2 )
par <- c( b1=-1, b2=0, b3=1 ) ; NP <- length(par)
item_PCM3 <- sirt::xxirt_createDiscItem( name="PCM3", par=par, est=rep(TRUE,NP),
               P=P_PCM3 )

customItems <- list( item_PCM2,  item_PCM3 )

#---- definition theta distribution
#** theta grid
Theta <- matrix( seq(-6,6,length=21), ncol=1 )

#** theta distribution
P_Theta1 <- function( par, Theta, G){
    mu <- par[1]
    sigma <- max( par[2], .01 )
    TP <- nrow(Theta)
    pi_Theta <- matrix( 0, nrow=TP, ncol=G)
    pi1 <- dnorm( Theta[,1], mean=mu, sd=sigma )
    pi1 <- pi1 / sum(pi1)
    pi_Theta[,1] <- pi1
    return(pi_Theta)
}
#** create distribution class
par_Theta <- c( "mu"=0, "sigma"=1 )
customTheta  <- sirt::xxirt_createThetaDistribution( par=par_Theta, est=c(FALSE,TRUE),
                       P=P_Theta1 )

#-- create parameter table
itemtype <- c("PCM2", "PCM3", "PCM3")
partable <- sirt::xxirt_createParTable( dat, itemtype=itemtype,
                        customItems=customItems )

#***** Model 1: MML
mod1 <- sirt::xxirt( dat=dat, Theta=Theta, partable=partable,
                   customItems=customItems, customTheta=customTheta)
summary(mod1)

#***** Model 2: PML
I <- ncol(dat)
W1 <- rep(1,I)
W2 <- 1-diag(I)
W2[3,2] <- 0
W2[ upper.tri(W2)] <- 0
pml_args <- list(W1=W1/sum(W1), W2=W2/sum(W2) )

mod2 <- sirt::xxirt( dat=dat, Theta=Theta, partable=partable,
                   customItems=customItems, customTheta=mod1$customTheta,
                   estimator="PML", pml_args=pml_args)

# variance matrix
smod2 <- sirt::xxirt_sandwich_pml(object=mod2)
smod2


#############################################################################
## EXAMPLE 6: Person covariates for item parameters and trait distribution
#############################################################################

#-- simulate data
set.seed(2025)
N <- 2000
I <- 10
b <- c(0, seq(-2,2,len=I-1))
b1 <- b
b1[1] <- 1
dat1 <- sirt::sim.raschtype( stats::rnorm(N, mean=0, sd=1), b=b )
dat2 <- sirt::sim.raschtype( stats::rnorm(N, mean=1, sd=1.5), b=b1 )
dat <- data.frame( group=rep( c(1,2), each=N), rbind(dat1, dat2))

#***************************************************************************
#***** Model 1: two-group Rasch model

#*** IRF 1PL
P_1PL <- function( par, Theta, ncat){
    b <- par[1]
    TP <- nrow(Theta)
    P <- matrix( NA, nrow=TP, ncol=ncat)
    P[,1] <- 1
    for (cc in 2:ncat){
        P[,cc] <- exp( (cc-1) * Theta[,1] - b )
    }
    P <- P / rowSums(P)
    return(P)
}

#** create item classes of 1PL and 2PL models
par <- c( b=0 )
item_1PL <- sirt::xxirt_createDiscItem( name="1PL", par=par, est=c(TRUE), P=P_1PL )
customItems <- list( item_1PL )

#---- definition theta distribution

#** theta grid
Theta <- matrix( seq(-6,6,length=21), ncol=1 )

#** theta distribution
P_Theta1 <- function( par, Theta, G)
{
    mu1 <- par[1]
    sigma1 <- max( par[2], .01 )
    mu2 <- par[3]
    sigma2 <- max( par[4], .01 )
    TP <- nrow(Theta)
    pi_Theta <- matrix( 0, nrow=TP, ncol=G)
    pi1 <- dnorm( Theta[,1], mean=mu1, sd=sigma1 )
    pi1 <- pi1 / sum(pi1)
    pi_Theta[,1] <- pi1
    pi1 <- dnorm( Theta[,1], mean=mu2, sd=sigma2 )
    pi1 <- pi1 / sum(pi1)
    pi_Theta[,2] <- pi1
    return(pi_Theta)
}
#** create distribution class
par_Theta <- c( "mu1"=0, "sigma1"=1, "mu2"=0, "sigma2"=1 )
customTheta  <- sirt::xxirt_createThetaDistribution( par=par_Theta,
                       est=c(FALSE,TRUE,TRUE,TRUE), P=P_Theta1 )

#-- create parameter table
itemtype <- rep( "1PL", I )
partable <- sirt::xxirt_createParTable( dat[,-1], itemtype=itemtype,
                        customItems=customItems )

# estimate model
mod1 <- sirt::xxirt( dat=dat[,-1], Theta=Theta, partable=partable, group=dat$group,
                   customItems=customItems, customTheta=customTheta)
summary(mod1)

#***************************************************************************
#***** Model 2: Rasch model with person covariates in trait distribution

#- matrix with person covariates
X <- dat[,"group", drop=FALSE]

#** theta distribution
P_Theta2 <- function( par, Theta, G, X)
{
    mu1 <- par[1]
    sigma1 <- max( par[2], .01 )
    mu2 <- par[3]
    sigma2 <- max( par[4], .01 )
    TP <- nrow(Theta)
    N <- nrow(X)
    pi_Theta <- matrix( 0, nrow=TP, ncol=N)
    mu <- par[ 2*(X[,1]-1)+1 ]
    sigma <- par[ 2*(X[,1]-1)+2 ]
    for (tt in 1:TP){
        pi_Theta[tt,] <- stats::dnorm(Theta[tt,1], mean=mu, sd=sigma)
    }
    cp <- sirt::sirt_matrix2( colSums(pi_Theta), nrow=TP)
    pi_Theta <- pi_Theta / ( cp + 1e-80 )
    return(pi_Theta)
}

par_Theta2 <- mod1$customTheta$par

customTheta2  <- sirt::xxirt_createThetaDistribution( par=par_Theta2,
                       est=c(FALSE,TRUE,TRUE,TRUE),
                       P=P_Theta2, X=X, lower=c(-10,0.01, -10, 0.01) )

partable2 <- mod1$partable

mod2 <- sirt::xxirt( dat=dat[,-1], Theta=Theta, partable=partable2,
                   customItems=customItems, customTheta=customTheta2)
summary(mod2)

#***************************************************************************
#***** Model 3: Rasch model with person covariates in trait distribution and
#*****          item parameters

#*** IRF 1PL
P_1PL <- function( par, Theta, ncat, X)
{
    TP <- nrow(Theta)
    N <- nrow(X)
    P <- array( 0, dim=c(TP,ncat,N) )
    efficient <- TRUE
    if (! efficient ){
        bM <- sirt::sirt_matrix2( par[ X[,1] ], nrow=TP)
        P1 <- stats::plogis(Theta[,1] - bM )
    } else {
        h1 <- exp(-Theta[,1])
        h2 <- sirt::sirt_matrix2( exp(par[ X[,1] ]), nrow=TP)
        P1 <- 1/ ( 1+h1*h2 )
    }
    P[,2,] <- P1
    P[,1,] <- 1 - P[,2,]
    return(P)
}

#** create item classes of 1PL and 2PL models
par <- c( b1=0, b2=-1 )
item_1PL <- sirt::xxirt_createDiscItem( name="1PL", par=par, est=c(TRUE,TRUE),
                            P=P_1PL, X=X )

customItems <- list( item_1PL )

#-- create parameter table
itemtype <- rep( "1PL", I )
partable2 <- sirt::xxirt_createParTable( dat[,-1], itemtype=itemtype,
                        customItems=customItems )

#-- distribution parameters
customTheta2b <- mod2$customTheta
customTheta2b$par[3] <- 0      # fix mu2 to 0
customTheta2b$est <- c(FALSE,TRUE,FALSE,TRUE)

# estimate model
mod3 <- sirt::xxirt( dat=resp, Theta=Theta, partable=partable2,
                   customItems=customItems, customTheta=customTheta2b, mstep_iter=2)
summary(mod3)

}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: automatic.recode.Rd
%% File Version: 0.19

\name{automatic.recode}
\alias{automatic.recode}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Automatic Method of Finding Keys in a Dataset with Raw Item Responses
}
\description{
This function calculates keys of a dataset with raw item responses.
It starts with setting the most frequent category of an item to 1.
Then, in each iteration keys are changed such that the highest
item discrimination is found.
}
\usage{
automatic.recode(data, exclude=NULL, pstart.min=0.6, allocate=200,
    maxiter=20, progress=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
Dataset with raw item responses
}
  \item{exclude}{
Vector with categories to be excluded for searching the key
}
  \item{pstart.min}{
Minimum probability for an initial solution of keys.
}
  \item{allocate}{
Maximum number of categories per item. This argument is used in
the function \code{tam.ctt3} of the \pkg{TAM} package.
}
  \item{maxiter}{
Maximum number of iterations
}
  \item{progress}{
A logical which indicates if iteration progress should be displayed
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list with following entries
\item{item.stat}{Data frame with item name, p value, item discrimination
and the calculated key}
\item{data.scored}{Scored data frame using calculated keys
in \code{item.stat}
}
\item{categ.stats}{Data frame with statistics for all categories
of all items}
}
%\references{
%% ~put references to the literature/web site here ~
%}
%\author{
%Alexander Robitzsch
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: data.raw1
#############################################################################
data(data.raw1)

# recode data.raw1 and exclude keys 8 and 9 (missing codes) and
# start with initially setting all categories larger than 50 % frequency to 1
res1 <- sirt::automatic.recode( data.raw1, exclude=c(8,9), pstart.min=.50 )
# inspect calculated keys
res1$item.stat

#############################################################################
# EXAMPLE 2: data.timssAusTwn from TAM package
#############################################################################

miceadds::library_install("TAM")
data(data.timssAusTwn,package="TAM")
raw.resp <- data.timssAusTwn[,1:11]
res2 <- sirt::automatic.recode( data=raw.resp )
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{R utilities}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: resp_groupwise.Rd
%% File Version: 0.06

\name{resp_groupwise}
\alias{resp_groupwise}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Creates Group-Wise Item Response Dataset
}
\description{
Creates group-wise item response dataset.
}
\usage{
resp_groupwise(resp, group, items_group)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{resp}{
Dataset with item responses
}
  \item{group}{
Vector of group identifiers
}
  \item{items_group}{
List containing vectors of groups for each item which should be made group-specific
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
Dataset
}
%\references{
%% ~put references to the literature/web site here ~
%}
%\author{
%Alexander Robitzsch
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Toy dataset
#############################################################################

library(CDM)
library(TAM)

data(data.ex11, package="TAM")
dat <- data.ex11
dat[ dat==9 ] <- 0
resp <- dat[,-1]

# group labels
booklets <- sort( unique(paste(dat$booklet)))

#- fit initial model
mod0 <- TAM::tam.mml( resp, group=dat$booklet)
summary(mod0)

# fit statistics
fmod <- IRT.RMSD(mod)
stat <- abs(fmod$MD[,-1])
stat[ is.na( fmod$RMSD[,2:4] ) ] <- NA
thresh <- .01
round(stat,3)
# define list define groups for group-specific items
items_group <- apply( stat, 1, FUN=function(ll){
                v1 <- booklets[ which( ll > thresh ) ]
                v1[ ! is.na(v1) ]  } )

#- create extended response dataset
dat2 <- sirt::resp_groupwise(resp=resp, group=paste(dat$booklet), items_group=items_group)
colSums( ! is.na(dat2) )

#- fit model for extended response dataset
mod2 <- TAM::tam.mml( dat2, group=dat$booklet)
summary(mod2)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{R utilities}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: summary.mcmc.sirt.Rd
%% File Version: 0.11

\name{summary.mcmc.sirt}
\alias{summary.mcmc.sirt}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Summary Method for Objects of Class \code{mcmc.sirt}
}
\description{
S3 method to summarize objects of class \code{mcmc.sirt}.
This object is generated by following functions: \code{\link{mcmc.2pno}},
\code{\link{mcmc.2pnoh}}, \code{\link{mcmc.3pno.testlet}},
\code{\link{mcmc.2pno.ml}}
}
\usage{
\method{summary}{mcmc.sirt}(object,digits=3, file=NULL, ...)
}


\arguments{
  \item{object}{
Object of class \code{mcmc.sirt}
}
\item{digits}{Number of digits after decimal}
\item{file}{Optional file name to which \code{summary} output is written}
  \item{\dots}{
Further arguments to be passed
}
}

%\author{
%Alexander Robitzsch
%}



%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{mcmc.2pno}}, \code{\link{mcmc.2pnoh}},
\code{\link{mcmc.3pno.testlet}}, \code{\link{mcmc.2pno.ml}}
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
%% \keyword{summary}

%% File Name: data.si.Rd
%% File Version: 0.416

\name{data.sirt}

\alias{data.sirt}
\alias{data.si01}
\alias{data.si02}
\alias{data.si03}
\alias{data.si04}
\alias{data.si05}
\alias{data.si06}
\alias{data.si07}
\alias{data.si08}
\alias{data.si09}
\alias{data.si10}

\docType{data}


\title{
Some Example Datasets for the \code{sirt} Package
}


\description{
Some example datasets for the \code{sirt} package.
}


\usage{
data(data.si01)
data(data.si02)
data(data.si03)
data(data.si04)
data(data.si05)
data(data.si06)
data(data.si07)
data(data.si08)
data(data.si09)
data(data.si10)
}


\format{

\itemize{
\item The format of the dataset \code{data.si01} is:

\code{'data.frame':   1857 obs. of  3 variables:} \cr
\code{ $ idgroup: int  1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{ $ item1  : int  NA NA NA NA NA NA NA NA NA NA ...} \cr
\code{ $ item2  : int  4 4 4 4 4 4 4 2 4 4 ...} \cr

\item The dataset \code{data.si02} is the Stouffer-Toby-dataset published
in Lindsay, Clogg and Grego (1991; Table 1, p.97, Cross-classification A):

\code{List of 2} \cr
\code{ $ data   : num [1:16, 1:4] 1 0 1 0 1 0 1 0 1 0 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : NULL} \cr
\code{  .. ..$ : chr [1:4] "I1" "I2" "I3" "I4"} \cr
\code{ $ weights: num [1:16] 42 1 6 2 6 1 7 2 23 4 ...} \cr

\item The format of the dataset \code{data.si03} (containing item
parameters of two studies) is:

\code{'data.frame':   27 obs. of  3 variables:} \cr
\code{ $ item    : Factor w/ 27 levels "M1","M10","M11",..: 1 12 21 22 ...} \cr
\code{ $ b_study1: num  0.297 1.163 0.151 -0.855 -1.653 ...} \cr
\code{ $ b_study2: num  0.72 1.118 0.351 -0.861 -1.593 ...} \cr

\item The dataset \code{data.si04} is adapted from Bartolucci, Montanari
and Pandolfi (2012; Table 4, Table 7). The data contains 4999 persons,
79 items on 5 dimensions. See \code{\link{rasch.mirtlc}} for using the
data in an analysis.

\code{List of 3} \cr
\code{ $ data        : num [1:4999, 1:79] 0 1 1 0 1 1 0 0 1 1 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : NULL} \cr
\code{  .. ..$ : chr [1:79] "A01" "A02" "A03" "A04" ...} \cr
\code{ $ itempars    :'data.frame':   79 obs. of  4 variables:} \cr
\code{  ..$ item      : Factor w/ 79 levels "A01","A02","A03",..: 1 2 3 4 5 6 7 8 9 10 ...} \cr
\code{  ..$ dim       : num [1:79] 1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{  ..$ gamma     : num [1:79] 1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{  ..$ gamma.beta: num [1:79] -0.189 0.25 0.758 1.695 1.022 ...} \cr
\code{ $ distribution: num [1:9, 1:7] 1 2 3 4 5 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : NULL} \cr
\code{  .. ..$ : chr [1:7] "class" "A" "B" "C" ...} \cr

\item The dataset \code{data.si05} contains double ratings of two
exchangeable raters for three items which are in \code{Ex1}, \code{Ex2}
and \code{Ex3}, respectively.

\code{List of 3} \cr
\code{ $ Ex1:'data.frame':    199 obs. of  2 variables:} \cr
\code{  ..$ C7040: num [1:199] NA 1 0 1 1 0 0 0 1 0 ...} \cr
\code{  ..$ C7041: num [1:199] 1 1 0 0 0 0 0 0 1 0 ...} \cr
\code{ $ Ex2:'data.frame':    2000 obs. of  2 variables:} \cr
\code{  ..$ rater1: num [1:2000] 2 0 3 1 2 2 0 0 0 0 ...} \cr
\code{  ..$ rater2: num [1:2000] 4 1 3 2 1 0 0 0 0 2 ...} \cr
\code{ $ Ex3:'data.frame':    2000 obs. of  2 variables:} \cr
\code{  ..$ rater1: num [1:2000] 5 1 6 2 3 3 0 0 0 0 ...} \cr
\code{  ..$ rater2: num [1:2000] 7 2 6 3 2 1 0 1 0 3 ...} \cr

\item The dataset \code{data.si06} contains multiple choice item
responses. The correct alternative is denoted as 0, distractors
are indicated by the codes 1, 2 or 3.

\code{'data.frame':   4441 obs. of  14 variables:} \cr
\code{ $ WV01: num  0 0 0 0 0 0 0 0 0 3 ...} \cr
\code{ $ WV02: num  0 0 0 3 0 0 0 0 0 1 ...} \cr
\code{ $ WV03: num  0 1 0 0 0 0 0 0 0 0 ...} \cr
\code{ $ WV04: num  0 0 0 0 0 0 0 0 0 1 ...} \cr
\code{ $ WV05: num  3 1 1 1 0 0 1 1 0 2 ...} \cr
\code{ $ WV06: num  0 1 3 0 0 0 2 0 0 1 ...} \cr
\code{ $ WV07: num  0 0 0 0 0 0 0 0 0 0 ...} \cr
\code{ $ WV08: num  0 1 1 0 0 0 0 0 0 0 ...} \cr
\code{ $ WV09: num  0 0 0 0 0 0 0 0 0 2 ...} \cr
\code{ $ WV10: num  1 1 3 0 0 2 0 0 0 0 ...} \cr
\code{ $ WV11: num  0 0 0 0 0 0 0 0 0 0 ...} \cr
\code{ $ WV12: num  0 0 0 2 0 0 2 0 0 0 ...} \cr
\code{ $ WV13: num  3 1 1 3 0 0 3 0 0 0 ...} \cr
\code{ $ WV14: num  3 1 2 3 0 3 1 3 3 0 ...} \cr

\item The dataset \code{data.si07} contains parameters of the empirical illustration
of DeCarlo (2020). The simulation function \code{sim_fun} can be used for
simulating data from the IRSDT model (see DeCarlo, 2020)

\code{List of 3} \cr
\code{ $ pars   :'data.frame':        16 obs. of  3 variables:} \cr
\code{  ..$ item: Factor w/ 16 levels "I01","I02","I03",..: 1 2 3 4 5 6 7 8 9 10 ...} \cr
\code{  ..$ b   : num [1:16] -1.1 -0.18 1.44 1.78 -1.19 0.45 -1.12 0.33 0.82 -0.43 ...} \cr
\code{  ..$ d   : num [1:16] 2.69 4.6 6.1 3.11 3.2 ...} \cr
\code{ $ trait  :'data.frame':        20 obs. of  2 variables:} \cr
\code{  ..$ x   : num [1:20] 0.025 0.075 0.125 0.175 0.225 0.275 0.325 0.375 0.425 0.475 ...} \cr
\code{  ..$ prob: num [1:20] 0.0238 0.1267 0.105 0.0594 0.0548 ...} \cr
\code{ $ sim_fun:function (lambda, b, d, items)  } \cr

\item The dataset \code{data.si08} contains 5 items with respect to knowledge
about lung cancer and the kind of information acquisition (Goodman, 1970;
see also Rasch, Kubinger & Yanagida, 2011).
\code{L1}: reading newspapers, \code{L2}: listening radio,
\code{L3}: reading books and magazines,
\code{L4}: attending talks, \code{L5}: knowledge about lung cancer

\code{'data.frame':   32 obs. of  6 variables:} \cr
\code{ $ L1 : num  1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{ $ L2 : num  1 1 1 1 1 1 1 1 0 0 ...} \cr
\code{ $ L3 : num  1 1 1 1 0 0 0 0 1 1 ...} \cr
\code{ $ L4 : num  1 1 0 0 1 1 0 0 1 1 ...} \cr
\code{ $ L5 : num  1 0 1 0 1 0 1 0 1 0 ...} \cr
\code{ $ wgt: num  23 8 102 67 8 4 35 59 27 18 ...} \cr


\item The dataset \code{data.si09} was used in Fischer and Karl (2019) and they
asked employees in a eight countries, to report whether they typically help
other employees (helping behavior, seven items, \code{help}) and whether
they make suggestions to improve work conditions and products
(voice behavior, five items, \code{voice}). Individuals responded to these items
on a 1-7 Likert-type scale. The dataset was downloaded from \url{https://osf.io/wkx8c/}.

\code{'data.frame':   5201 obs. of  13 variables:} \cr
\code{ $ country: Factor w/ 8 levels "BRA","CAN","KEN",..: 5 5 5 5 5 5 5 5 5 5 ...} \cr
\code{ $ help1  : int  6 6 5 5 5 6 6 6 4 6 ...} \cr
\code{ $ help2  : int  3 6 5 6 6 6 6 6 6 7 ...} \cr
\code{ $ help3  : int  5 6 6 7 7 6 5 6 6 7 ...} \cr
\code{ $ help4  : int  7 6 5 6 6 7 7 6 6 7 ...} \cr
\code{ $ help5  : int  5 5 5 6 6 6 6 6 6 7 ...} \cr
\code{ $ help6  : int  3 4 5 6 6 7 7 6 6 5 ...} \cr
\code{ $ help7  : int  5 4 4 5 5 7 7 6 6 6 ...} \cr
\code{ $ voice1 : int  3 6 5 6 4 7 6 6 5 7 ...} \cr
\code{ $ voice2 : int  3 6 4 7 6 5 6 6 4 7 ...} \cr
\code{ $ voice3 : int  6 6 5 7 6 5 6 6 6 5 ...} \cr
\code{ $ voice4 : int  6 6 6 5 5 7 5 6 6 6 ...} \cr
\code{ $ voice5 : int  6 7 4 7 6 6 6 6 5 7 ...} \cr


\item The dataset \code{data.si10} contains votes of 435 members of the U.S. House of
Representatives, 267 Democrates and 168 Republicans. The dataset was
used by Fop and Murphy (2017).

\code{'data.frame':   435 obs. of  17 variables:} \cr
\code{ $ party : Factor w/ 2 levels "democrat","republican": 2 2 1 1 1 1 1 2 2 1 ...} \cr
\code{ $ vote01: num  0 0 NA 0 1 0 0 0 0 1 ...} \cr
\code{ $ vote02: num  1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{ $ vote03: num  0 0 1 1 1 1 0 0 0 1 ...} \cr
\code{ $ vote04: num  1 1 NA 0 0 0 1 1 1 0 ...} \cr
\code{ $ vote05: num  1 1 1 NA 1 1 1 1 1 0 ...} \cr
\code{ $ vote06: num  1 1 1 1 1 1 1 1 1 0 ...} \cr
\code{ $ vote07: num  0 0 0 0 0 0 0 0 0 1 ...} \cr
\code{ $ vote08: num  0 0 0 0 0 0 0 0 0 1 ...} \cr
\code{ $ vote09: num  0 0 0 0 0 0 0 0 0 1 ...} \cr
\code{ $ vote10: num  1 0 0 0 0 0 0 0 0 0 ...} \cr
\code{ $ vote11: num  NA 0 1 1 1 0 0 0 0 0 ...} \cr
\code{ $ vote12: num  1 1 0 0 NA 0 0 0 1 0 ...} \cr
\code{ $ vote13: num  1 1 1 1 1 1 NA 1 1 0 ...} \cr
\code{ $ vote14: num  1 1 1 0 1 1 1 1 1 0 ...} \cr
\code{ $ vote15: num  0 0 0 0 1 1 1 NA 0 NA ...} \cr
\code{ $ vote16: num  1 NA 0 1 1 1 1 1 1 NA ...} \cr


}
}





%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}


\seealso{
Some free datasets can be obtained from \cr
Psychological questionnaires: http://personality-testing.info/_rawdata/ \cr
PISA 2012: http://pisa2012.acer.edu.au/downloads.php \cr
PIAAC: http://www.oecd.org/site/piaac/publicdataandanalysis.htm \cr
TIMSS 2011: http://timssandpirls.bc.edu/timss2011/international-database.html \cr
ALLBUS: http://www.gesis.org/allbus/allbus-home/
}

%\source{
%  ~~ reference to a publication or URL from which the data were obtained ~~
%}



\references{
Bartolucci, F., Montanari, G. E., & Pandolfi, S. (2012).
Dimensionality of the latent structure and item selection via latent
class multidimensional IRT models. \emph{Psychometrika, 77}(4), 782-802.
\doi{10.1007/s11336-012-9278-0}

DeCarlo, L. T. (2020). An item response model for true-false exams based on signal
detection theory. \emph{Applied Psychological Measurement, 34}(3). 234-248.
\doi{10.1177/0146621619843823}

Fischer, R., & Karl, J. A. (2019). A primer to (cross-cultural) multi-group invariance
testing possibilities in R.
\emph{Frontiers in Psychology | Cultural Psychology, 10}:1507.
\doi{10.3389/fpsyg.2019.01507}

Fop, M., & Murphy, T. B. (2018). Variable selection methods for model-based clustering.
\emph{Statistics Surveys, 12}, 18-65. https://doi.org/10.1214/18-SS119

Goodman, L. A. (1970). The multivariate analysis of qualitative data: Interactions
among multiple classifications.
\emph{Journal of the American Statistical Association, 65}(329), 226-256.
\doi{10.1080/01621459.1970.10481076}

Lindsay, B., Clogg, C. C., & Grego, J. (1991).
Semiparametric estimation in the Rasch model and related exponential response
models, including a simple latent class model for item analysis.
\emph{Journal of the American Statistical Association, 86}(413), 96-107.
\doi{10.1080/01621459.1991.10475008}

Rasch, D., Kubinger, K. D., & Yanagida, T. (2011).
\emph{Statistics in psychology using R and SPSS}. New York: Wiley.
\doi{10.1002/9781119979630}
}


\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Nested logit model multiple choice dataset data.si06
#############################################################################

data(data.si06, package="sirt")
dat <- data.si06

#** estimate 2PL nested logit model
library(mirt)
mod1 <- mirt::mirt( dat, model=1, itemtype="2PLNRM", key=rep(0,ncol(dat) ),
            verbose=TRUE  )
summary(mod1)
cmod1 <- sirt::mirt.wrapper.coef(mod1)$coef
cmod1[,-1] <- round( cmod1[,-1], 3)

#** normalize item parameters according Suh and Bolt (2010)
cmod2 <- cmod1

# slope parameters
ind <-  grep("ak",colnames(cmod2))
h1 <- cmod2[,ind ]
cmod2[,ind] <- t( apply( h1, 1, FUN=function(ll){ ll - mean(ll) } ) )
# item intercepts
ind <-  paste0( "d", 0:9 )
ind <- which( colnames(cmod2) \%in\% ind )
h1 <- cmod2[,ind ]
cmod2[,ind] <- t( apply( h1, 1, FUN=function(ll){ ll - mean(ll) } ) )
cmod2[,-1] <- round( cmod2[,-1], 3)

#############################################################################
# EXAMPLE 2: Item response modle based on signal detection theory (IRSDT model)
#############################################################################

data(data.si07, package="sirt")
data <- data.si07

#-- simulate data
set.seed(98)
N <- 2000 # define sample size
# generate membership scores
lambda <- sample(size=N, x=data$trait$x, prob=data$trait$prob, replace=TRUE)
b <- data$pars$b
d <- data$pars$d
items <- data$pars$item
dat <- data$sim_fun(lambda=lambda, b=b, d=d, items=items)

#- estimate IRSDT model as a grade of membership model with two classes
problevels <- seq( 0.025, 0.975, length=20 )
mod1 <- sirt::gom.em( dat, K=2, problevels=problevels )
summary(mod1)
}
}
%% \keyword{datasets}

%% File Name: isop.Rd
%% File Version: 3.31

\name{isop}
\alias{isop.dich}
\alias{isop.poly}
\alias{summary.isop}
\alias{plot.isop}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Fit Unidimensional ISOP and ADISOP Model to Dichotomous
and Polytomous Item Responses
}
\description{
Fit the unidimensional isotonic probabilistic model (ISOP;
Scheiblechner, 1995, 2007) and the additive istotonic
probabilistic model (ADISOP; Scheiblechner, 1999).
The \code{isop.dich} function can be used for dichotomous
data while the \code{isop.poly} function can be applied
to polytomous data. Note that for applying the ISOP model for
polytomous data it is necessary that all items do have the
same number of categories.
}
\usage{
isop.dich(dat, score.breaks=NULL, merge.extreme=TRUE,
     conv=.0001, maxit=1000, epsilon=.025, progress=TRUE)

isop.poly( dat, score.breaks=seq(0,1,len=10 ),
     conv=.0001, maxit=1000, epsilon=.025, progress=TRUE )

\method{summary}{isop}(object,...)

\method{plot}{isop}(x,ask=TRUE,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame with dichotomous or polytomous item responses
}
\item{score.breaks}{
Vector with breaks to define score groups. For dichotomous
data, the person score grouping is applied for the mean
person score, for polytomous data it is applied to
the modified percentile score.
}
  \item{merge.extreme}{
Merge extreme groups with zero and maximum score
with succeeding score categories? The default is
\code{TRUE}.
}
  \item{conv}{
Convergence criterion
}
  \item{maxit}{
Maximum number of iterations
}
\item{epsilon}{Additive constant to handle cell frequencies
of 0 or 1 in \code{\link{fit.adisop}}
}
  \item{progress}{
Display progress?
  }
\item{object}{Object of class \code{isop} (generated by
    \code{isop.dich} or \code{isop.poly})}
\item{x}{Object of class \code{isop} (generated by
    \code{isop.dich} or \code{isop.poly})}
\item{ask}{Ask for a new plot?}
\item{\dots}{Further arguments to be passed}
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DETAILS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\details{
The ISOP model for dichotomous data was firstly proposed
by Irtel and Schmalhofer (1982). Consider person groups
\eqn{p} (ordered from low to high scores)
and items \eqn{i} (ordered from difficult to easy items).
Here, \eqn{F(p,i)} denotes
the proportion correct for item \eqn{i}  in score group
\eqn{p}, while \eqn{n_{pi}} denotes the number of persons
in group \eqn{p} and on item \eqn{i}. The isotonic
probabilistic model (Scheiblechner, 1995) monotonically
smooths this distribution function \eqn{F} such that
\deqn{ P( X_{pi}=1 | p, i )=F^\ast( p, i ) }
where the two-dimensional distribution function
\eqn{F^\ast} is isotonic in \eqn{p} and \eqn{i}. Model fit is
assessed by the square root of weighted squares of deviations
\deqn{Fit=\sqrt{ \frac{1}{I} \sum_{p,i} w_{pi} \left(  F(p, i) -
    F^\ast(p,i ) \right )^2 }}
with frequency weights \eqn{w_{pi}} and
\eqn{\sum_p w_{pi}=1} for every item \eqn{i}.
The additive isotonic model (ADISOP; Scheiblechner, 1999)
assumes the existence of person parameters \eqn{\theta_p}
and item parameters \eqn{\delta_i} such that
\deqn{ P( X_{pi}=1 | p )=g( \theta_p + \delta_i )}
and \eqn{g} is a nonparametrically estimated isotonic
function. The functions \code{isop.dich} and \code{isop.poly} uses \eqn{F^\ast}
from the ISOP models and estimates person and item parameters of the
ADISOP model. For comparison, \code{isop.dich} also fits a model with
the logistic function \eqn{g} which results in the Rasch
model.

For polytomous data, the starting point is the empirical
distribution function
\deqn{ P( X_i \le k | p  )=F( k ; p, i )  } which is increasing
in the argument \eqn{k} (the item categories).
The ISOP model is defined to be antitonic in \eqn{p} and \eqn{i}
while items are ordered with respect to item P-scores and persons are ordered
according to modified percentile scores (Scheiblechner, 2007).
The estimated ISOP model results in a distribution
function \eqn{F^\ast}. Using this function, the additive
isotonic probabilistic model (ADISOP) aims at estimating
a distribution function
\deqn{P( X_i \le k ; p  )=F^{\ast \ast} ( k ; p, i )=F^{ \ast \ast }
    ( k, \theta_p + \delta_i ) }
which is antitonic in \eqn{k} and in \eqn{\theta_p + \delta_i}.
Due to this additive relation, the ADISOP scale values
are claimed to be measured at interval scale level (Scheiblechner, 1999).

The ADISOP model is compared to the graded response model which is
defined by the response equation
\deqn{P( X_i \le k ; p  )=g( \theta_p + \delta_i + \gamma_k ) }
where \eqn{g} denotes the logistic function.
Estimated parameters are in the value \code{fit.grm}:
person parameters \eqn{\theta_p} (\code{person.sc}),
item parameters \eqn{\delta_i} (\code{item.sc}) and
category parameters \eqn{\gamma_k} (\code{cat.sc}).

The calculation of person and item scores is explained
in \code{\link{isop.scoring}}.

For an application of the ISOP and ADISOP model
see Scheiblechner and Lutz (2009).
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VALUES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\value{
A list with following entries:
\item{freq.correct}{
   Used frequency table (distribution function) for
   dichotomous and polytomous data
}
\item{wgt}{
    Used weights (frequencies)
}
\item{prob.saturated}{
    Frequencies of the saturated model
        }
\item{prob.isop}{
    Fitted frequencies of the ISOP model
        }
\item{prob.adisop}{
    Fitted frequencies of the ADISOP model
        }
\item{prob.logistic}{
    Fitted frequencies of the logistic model
    (only for \code{isop.dich})
        }
\item{prob.grm}{
    Fitted frequencies of the graded response model
    (only for \code{isop.poly})
        }
\item{ll}{List with log-likelihood values}
\item{fit}{Vector of fit statistics}
\item{person}{Data frame of person parameters}
\item{item}{Data frame of item parameters}
\item{p.itemcat}{Frequencies for every item category}
\item{score.itemcat}{
        Scoring points for every item category
                }
\item{fit.isop}{Values of fitting the ISOP model
(see \code{\link{fit.isop}}) }
\item{fit.isop}{Values of fitting the ADISOP model
(see \code{\link{fit.adisop}}) }
\item{fit.logistic}{Values of fitting the logistic model
 (only for \code{isop.dich}) }
\item{fit.grm}{Values of fitting the graded response model
 (only for \code{isop.poly}) }
\item{\dots}{Further values}
}



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% REFERENCES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\references{
Irtel, H., & Schmalhofer, F. (1982).
Psychodiagnostik auf Ordinalskalenniveau:
Messtheoretische Grundlagen, Modelltest und
Parameterschaetzung. \emph{Archiv fuer Psychologie,
134}, 197-218.

Scheiblechner, H. (1995). Isotonic ordinal
probabilistic models (ISOP). \emph{Psychometrika,
60}, 281-304.

Scheiblechner, H. (1999). Additive conjoint isotonic
probabilistic models (ADISOP). \emph{Psychometrika,
64}, 295-316.

Scheiblechner, H. (2007). A unified nonparametric IRT model for
d-dimensional psychological test data (d-ISOP).
\emph{Psychometrika, 72}, 43-67.

Scheiblechner, H., & Lutz, R. (2009).
Die Konstruktion eines optimalen eindimensionalen
Tests mittels nichtparametrischer Testtheorie (NIRT)
am Beispiel des MR SOC. \emph{Diagnostica, 55},
41-54.
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
This function uses \code{\link{isop.scoring}},
\code{\link{fit.isop}} and \code{\link{fit.adisop}}.

Tests of the W1 axiom of the ISOP model (Scheiblechner, 1995) can be performed with
\code{\link{isop.test}}.

See also the \pkg{ISOP} package at \emph{Rforge}: \emph{http://www.rforge.net/ISOP/}.

Install this package using

\code{install.packages("ISOP",repos="http://www.rforge.net/")}
}



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EXAMPLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\examples{
#############################################################################
# EXAMPLE 1: Dataset Reading (dichotomous items)
#############################################################################

data(data.read)
dat <- as.matrix( data.read)
I <- ncol(dat)

# Model 1: ISOP Model (11 score groups)
mod1 <- sirt::isop.dich( dat )
summary(mod1)
plot(mod1)

\dontrun{
# Model 2: ISOP Model (5 score groups)
score.breaks <- seq( -.005, 1.005, len=5+1 )
mod2 <- sirt::isop.dich( dat, score.breaks=score.breaks)
summary(mod2)

#############################################################################
# EXAMPLE 2: Dataset PISA mathematics (dichotomous items)
#############################################################################

data(data.pisaMath)
dat <- data.pisaMath$data
dat <- dat[, grep("M", colnames(dat) ) ]

# fit ISOP model
# Note that for this model many iterations are needed
#   to reach convergence for ADISOP
mod1 <- sirt::isop.dich( dat, maxit=4000)
summary(mod1)
}

#############################################################################
# EXAMPLE 3: Dataset Students (polytomous items)
#############################################################################

# Dataset students: scale cultural activities
library(CDM)
data(data.Students, package="CDM")
dat <- stats::na.omit( data.Students[, paste0("act",1:4) ] )

# fit models
mod1 <- sirt::isop.poly( dat )
summary(mod1)
plot(mod1)
}

%% \keyword{ISOP model}
%% \keyword{ADISOP model}
%% \keyword{Nonparametric IRT}
%% \keyword{summary}
%% \keyword{plot}

%% File Name: data.trees.Rd
%% File Version: 0.15

\name{data.trees}
\alias{data.trees}
\docType{data}

\title{
Dataset Used in Stoyan, Pommerening and Wuensche (2018)
}
\description{
Dataset used in Stoyan, Pommerening and Wuensche (2018; see also
Pommerening et al., 2018). In the dataset, 15 forest managers classify 387 trees
either as trees to be maintained or as trees to be removed. They assign tree marks,
either 0 or 1, where mark 1 means remove.
}

\usage{
data(data.trees)
}

\format{
The dataset has the following structure. \cr

\code{'data.frame':   387 obs. of  16 variables:} \cr
\code{ $ Number: int  142 184 9 300 374 42 382 108 125 201 ...} \cr
\code{ $ FM1   : int  1 1 1 1 1 1 1 1 1 0 ...} \cr
\code{ $ FM2   : int  1 1 1 0 1 1 1 1 1 1 ...} \cr
\code{ $ FM3   : int  1 0 1 1 1 1 1 1 1 1 ...} \cr
\code{ $ FM4   : int  1 1 1 1 1 1 0 1 1 1 ...} \cr
\code{ $ FM5   : int  1 1 1 1 1 1 0 0 0 1 ...} \cr
\code{ $ FM6   : int  1 1 1 1 0 1 1 1 1 0 ...} \cr
\code{ $ FM7   : int  1 0 1 1 0 0 1 0 1 1 ...} \cr
\code{ $ FM8   : int  1 1 1 1 1 0 0 1 0 1 ...} \cr
\code{ $ FM9   : int  1 1 0 1 1 1 1 0 1 1 ...} \cr
\code{ $ FM10  : int  0 1 1 0 1 1 1 1 0 0 ...} \cr
\code{ $ FM11  : int  1 1 1 1 0 1 1 0 1 0 ...} \cr
\code{ $ FM12  : int  1 1 1 1 1 1 0 1 0 0 ...} \cr
\code{ $ FM13  : int  0 1 0 0 1 1 1 1 1 1 ...} \cr
\code{ $ FM14  : int  1 1 1 1 1 0 1 1 1 1 ...} \cr
\code{ $ FM15  : int  1 1 0 1 1 0 1 0 0 1 ...} \cr

}

%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}
\source{
\url{https://www.pommerening.org/wiki/images/d/dc/CoedyBreninSortedforPublication.txt}
}

\references{
Pommerening, A., Ramos, C. P., Kedziora, W., Haufe, J., & Stoyan, D. (2018).
Rating experiments in forestry: How much agreement is there in tree marking?
\emph{PloS ONE, 13}(3), e0194747. \doi{10.1371/journal.pone.0194747}

Stoyan, D., Pommerening, A., & Wuensche, A. (2018).
Rater classification by means of set-theoretic methods applied to forestry data.
\emph{Journal of Environmental Statistics, 8}(2), 1-17.
}


\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Latent class models, latent trait models, mixed membership models
#############################################################################

data(data.trees, package="sirt")
dat <- data.trees[,-1]
I <- ncol(dat)

#** latent class models with 2, 3, and 4 classes
problevels <- seq( 0, 1, len=2 )
mod02 <- sirt::gom.em(dat, K=2, problevels, model="GOM")
mod03 <- sirt::gom.em(dat, K=3, problevels, model="GOM")
mod04 <- sirt::gom.em(dat, K=4, problevels, model="GOM")

#** grade of membership models
mod11 <- sirt::gom.em(dat, K=2, theta0.k=10*seq(-1,1,len=11), model="GOMnormal")
problevels <- seq( 0, 1, len=3 )
mod12 <- sirt::gom.em(dat, K=2, problevels, model="GOM")
mod13 <- sirt::gom.em(dat, K=3, problevels, model="GOM")
mod14 <- sirt::gom.em(dat, K=4, problevels, model="GOM")
problevels <- seq( 0, 1, len=4 )
mod22 <- sirt::gom.em(dat, K=2, problevels, model="GOM")
mod23 <- sirt::gom.em(dat, K=3, problevels, model="GOM")
mod24 <- sirt::gom.em(dat, K=4, problevels, model="GOM")

#** latent trait models
#- 1PL
mod31 <- sirt::rasch.mml2(dat)
#- 2PL
mod32 <- sirt::rasch.mml2(dat, est.a=1:I)

#- model comparison
IRT.compareModels(mod02, mod03, mod04, mod11, mod12, mod13, mod14,
                     mod22, mod23, mod24, mod31, mod32)

#-- inspect model results
summary(mod12)
round( cbind( mod12$theta.k, mod12$pi.k ),3)

summary(mod13)
round(cbind( mod13$theta.k, mod13$pi.k ),3)
}
}



%% \keyword{datasets}

%% File Name: rm.facets.Rd
%% File Version: 0.824

\name{rm.facets}
\alias{rm.facets}
\alias{summary.rm.facets}
\alias{logLik.rm.facets}
\alias{anova.rm.facets}
\alias{IRT.irfprob.rm.facets}
\alias{IRT.likelihood.rm.facets}
\alias{IRT.posterior.rm.facets}
\alias{IRT.modelfit.rm.facets}
\alias{IRT.factor.scores.rm.facets}
\alias{summary.IRT.modelfit.rm.facets}
\alias{rm_proc_data}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Rater Facets Models with Item/Rater Intercepts and Slopes
}
\description{
This function estimates the unidimensional rater facets model (Lincare, 1994)
and an extension to slopes (see Details; Robitzsch & Steinfeld, 2018). The estimation
is conducted by an EM algorithm employing marginal
maximum likelihood.
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ARGUMENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\usage{
rm.facets(dat, pid=NULL, rater=NULL, Qmatrix=NULL, theta.k=seq(-9, 9, len=30),
    est.b.rater=TRUE, est.a.item=FALSE, est.a.rater=FALSE, rater_item_int=FALSE,
    est.mean=FALSE, tau.item.fixed=NULL, a.item.fixed=NULL, b.rater.fixed=NULL,
    a.rater.fixed=NULL, b.rater.center=2, a.rater.center=2, a.item.center=2, a_lower=.05,
    a_upper=10, reference_rater=NULL, max.b.increment=1, numdiff.parm=0.00001,
    maxdevchange=0.1, globconv=0.001, maxiter=1000, msteps=4, mstepconv=0.001,
    PEM=FALSE, PEM_itermax=maxiter)

\method{summary}{rm.facets}(object, file=NULL, ...)

\method{anova}{rm.facets}(object,...)

\method{logLik}{rm.facets}(object,...)

\method{IRT.irfprob}{rm.facets}(object,...)

\method{IRT.factor.scores}{rm.facets}(object, type="EAP", ...)

\method{IRT.likelihood}{rm.facets}(object,...)

\method{IRT.posterior}{rm.facets}(object,...)

\method{IRT.modelfit}{rm.facets}(object,...)

\method{summary}{IRT.modelfit.rm.facets}(object, ...)

## function for processing data
rm_proc_data( dat, pid, rater, rater_item_int=FALSE, reference_rater=NULL )
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Original data frame. Ratings on variables must be in rows,
i.e. every row corresponds to a person-rater combination.
}
  \item{pid}{
Person identifier.
}
  \item{rater}{
Rater identifier
}
  \item{Qmatrix}{
An optional Q-matrix. If this matrix is not provided,
then by default the ordinary scoring of categories
(from 0 to the maximum score of \eqn{K}) is used.
}
  \item{theta.k}{
A grid of theta values for the ability distribution.
}
  \item{est.b.rater}{
Should the rater severities \eqn{b_r} be estimated?
}
  \item{est.a.item}{
Should the item slopes \eqn{a_i} be estimated?
}
  \item{est.a.rater}{
Should the rater slopes \eqn{a_r} be estimated?
}
\item{rater_item_int}{Logical indicating whether rater-item-interactions
should be modeled.}
\item{est.mean}{Optional logical indicating whether the mean of the
trait distribution should be estimated.}
\item{tau.item.fixed}{Matrix with fixed \eqn{\tau} parameters. Non-fixed
parameters must be declared by \code{NA} values.}
\item{a.item.fixed}{Vector with fixed item discriminations}
\item{b.rater.fixed}{Vector with fixed rater intercept parameters}
\item{a.rater.fixed}{Vector with fixed rater discrimination parameters}
\item{b.rater.center}{Centering method for rater intercept parameters. The
    value \code{0} corresponds to no centering, the values \code{1} and
\code{2} to different methods to ensure that they sum to zero.}
\item{a.rater.center}{Centering method for rater discrimination parameters. The
    value \code{0} corresponds to no centering, the values \code{1} and
\code{2} to different methods to ensure that their product equals one.}
\item{a.item.center}{Centering method for item discrimination parameters. The
    value \code{0} corresponds to no centering, the values \code{1} and
\code{2} to different methods to ensure that their product equals one.}
\item{a_lower}{Lower bound for \eqn{a} parameters}
\item{a_upper}{Upper bound for \eqn{a} parameters}
\item{reference_rater}{Identifier for rater as a reference rater for which
a fixed rater mean of 0 and a fixed rater slope of 1 is assumed.}
\item{max.b.increment}{
Maximum increment of item parameters during estimation
}
  \item{numdiff.parm}{
Numerical differentiation step width
}
  \item{maxdevchange}{
Maximum relative deviance change as a convergence criterion
}
  \item{globconv}{
Maximum parameter change
}
  \item{maxiter}{
Maximum number of iterations
}
  \item{msteps}{
Maximum number of iterations during an M step
}
  \item{mstepconv}{
Convergence criterion in an M step
}
\item{PEM}{Logical indicating whether the P-EM acceleration should be
applied (Berlinet & Roland, 2012).}
\item{PEM_itermax}{Number of iterations in which the P-EM method should be
applied.}
  \item{object}{
Object of class \code{rm.facets}
}
\item{file}{Optional file name in which summary should be written.}
\item{type}{Factor score estimation method. Factor score types
    \code{"EAP"}, \code{"MLE"} and \code{"WLE"} are supported.}
  \item{\dots}{
Further arguments to be passed
}
}


% \item{skillspace}{Type of skill space which is used to smooth the distribution
%    on nodes \code{theta.k}. \code{skillspace="normal"} corresponds to
%    a normal distribution, \code{skillspace="loglinear"} to log-linear
%    modelling of the skill space distribution for which a design matrix \eqn{Z}
%    has to be specified, and \code{skillspace="no"} corresponds to an
% unsmoothed distribution.}

%\item{Z}{Design matrix for trait distribution if \code{skillspace="loglinear"}.
%By using appropriate design matrices, non-normal trait distributions
%can be estimated.}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DETAILS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\details{
This function models ratings \eqn{X_{pri}}
for person \eqn{p}, rater \eqn{r} and item \eqn{i}
and category \eqn{k} (see also Robitzsch & Steinfeld, 2018; Uto & Ueno, 2010; Wu, 2017)
\deqn{P( X_{pri}=k | \theta_p ) \propto
    \exp( a_i a_r q_{ik} \theta_p - q_{ik} b_r -   \tau_{ik} ) \quad,
    \quad \theta_p \sim N( 0, \sigma^2 )}{%
    P( X_{pri}=k | \theta_p ) \propto
    \exp( a_i a_r q_{ik} \theta_p - q_{ik} b_r - \tau_{ik} ),
            \theta_p ~ N( 0, \sigma^2 )}
By default, the scores in the \eqn{Q} matrix are
\eqn{q_{ik}=k}. Item slopes \eqn{a_i} and rater slopes
\eqn{a_r} are standardized such that their product equals
one, i.e. \eqn{ \prod_i a_i=\prod_r a_r=1}.
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VALUES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\value{
A list with following entries:
\item{deviance}{Deviance}
\item{ic}{Information criteria and number of parameters}
\item{item}{Data frame with item parameters}
\item{rater}{Data frame with rater parameters}
\item{person}{Data frame with person parameters: EAP and corresponding
    standard errors}
\item{EAP.rel}{EAP reliability}
\item{mu}{Mean of the trait distribution}
\item{sigma}{Standard deviation of the trait distribution}
\item{theta.k}{Grid of theta values}
\item{pi.k}{Fitted distribution at \code{theta.k} values}
\item{tau.item}{Item parameters \eqn{\tau_{ik}}}
\item{se.tau.item}{Standard error of item parameters \eqn{\tau_{ik}}}
\item{a.item}{Item slopes \eqn{a_i}}
\item{se.a.item}{Standard error of item slopes \eqn{a_i}}
\item{delta.item}{Delta item parameter. See
    \code{\link{pcm.conversion}}.
       }
\item{b.rater}{Rater severity parameter \eqn{b_r}}
\item{se.b.rater}{Standard error of rater severity parameter \eqn{b_r}}
\item{a.rater}{Rater slope parameter \eqn{a_r}}
\item{se.a.rater}{Standard error of rater slope parameter \eqn{a_r}}
\item{f.yi.qk}{Individual likelihood}
\item{f.qk.yi}{Individual posterior distribution}
\item{probs}{Item probabilities at grid \code{theta.k}}
\item{n.ik}{Expected counts}
\item{maxK}{Maximum number of categories}
\item{procdata}{Processed data}
\item{iter}{Number of iterations}
\item{ipars.dat2}{Item parameters for expanded dataset \code{dat2}}
\item{\dots}{Further values}
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% REFERENCES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\references{
Berlinet, A. F., & Roland, C. (2012).
Acceleration of the EM algorithm: P-EM versus epsilon algorithm.
\emph{Computational Statistics & Data Analysis, 56}(12), 4122-4137.

Linacre, J. M. (1994). \emph{Many-Facet Rasch Measurement}.
Chicago: MESA Press.

Robitzsch, A., & Steinfeld, J. (2018). Item response models for human ratings: Overview,
estimation methods, and implementation in R.
\emph{Psychological Test and Assessment Modeling, 60}(1), 101-139.

Uto, M., & Ueno, M. (2016). Item response theory for peer assessment.
\emph{IEEE Transactions on Learning Technologies, 9}(2), 157-170.

Wu, M. (2017). Some IRT-based analyses for interpreting rater effects.
\emph{Psychological Test and Assessment Modeling, 59}(4), 453-470.
}



%\author{
%Alexander Robitzsch
%}



\note{
If the trait standard deviation \code{sigma} strongly
differs from 1, then a user should investigate the sensitivity
of results using different theta integration points \code{theta.k}.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also the \pkg{TAM} package for the estimation
of more complicated facet models.

See \code{\link{rm.sdt}} for estimating a hierarchical rater model.
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EXAMPLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\examples{
#############################################################################
# EXAMPLE 1: Partial Credit Model and Generalized partial credit model
#                   5 items and 1 rater
#############################################################################
data(data.ratings1)
dat <- data.ratings1

# select rater db01
dat <- dat[ paste(dat$rater)=="db01", ]

#****  Model 1: Partial Credit Model
mod1 <- sirt::rm.facets( dat[, paste0( "k",1:5) ], pid=dat$idstud )

#****  Model 2: Generalized Partial Credit Model
mod2 <- sirt::rm.facets( dat[, paste0( "k",1:5) ],  pid=dat$idstud, est.a.item=TRUE)

summary(mod1)
summary(mod2)

\dontrun{
#############################################################################
# EXAMPLE 2: Facets Model: 5 items, 7 raters
#############################################################################

data(data.ratings1)
dat <- data.ratings1

#****  Model 1: Partial Credit Model: no rater effects
mod1 <- sirt::rm.facets( dat[, paste0( "k",1:5) ], rater=dat$rater,
             pid=dat$idstud, est.b.rater=FALSE )

#****  Model 2: Partial Credit Model: intercept rater effects
mod2 <- sirt::rm.facets( dat[, paste0( "k",1:5) ], rater=dat$rater, pid=dat$idstud)

# extract individual likelihood
lmod1 <- IRT.likelihood(mod1)
str(lmod1)
# likelihood value
logLik(mod1)
# extract item response functions
pmod1 <- IRT.irfprob(mod1)
str(pmod1)
# model comparison
anova(mod1,mod2)
# absolute and relative model fit
smod1 <- IRT.modelfit(mod1)
summary(smod1)
smod2 <- IRT.modelfit(mod2)
summary(smod2)
IRT.compareModels( smod1, smod2 )
# extract factor scores (EAP is the default)
IRT.factor.scores(mod2)
# extract WLEs
IRT.factor.scores(mod2, type="WLE")

#****  Model 2a: compare results with TAM package
#   Results should be similar to Model 2
library(TAM)
mod2a <- TAM::tam.mml.mfr( resp=dat[, paste0( "k",1:5) ],
             facets=dat[, "rater", drop=FALSE],
             pid=dat$pid, formulaA=~ item*step + rater )

#****  Model 2b: Partial Credit Model: some fixed parameters
# fix rater parameters for raters 1, 4 and 5
b.rater.fixed <- rep(NA,7)
b.rater.fixed[ c(1,4,5) ] <- c(1,-.8,0)  # fixed parameters
# fix item parameters of first and second item
tau.item.fixed <- round( mod2$tau.item, 1 )    # use parameters from mod2
tau.item.fixed[ 3:5, ] <- NA    # free item parameters of items 3, 4 and 5
mod2b <- sirt::rm.facets( dat[, paste0( "k",1:5) ], rater=dat$rater,
             b.rater.fixed=b.rater.fixed, tau.item.fixed=tau.item.fixed,
             est.mean=TRUE, pid=dat$idstud)
summary(mod2b)

#****  Model 3: estimated rater slopes
mod3 <- sirt::rm.facets( dat[, paste0( "k",1:5) ], rater=dat$rater,
            est.a.rater=TRUE)

#****  Model 4: estimated item slopes
mod4 <- sirt::rm.facets( dat[, paste0( "k",1:5) ], rater=dat$rater,
             pid=dat$idstud, est.a.item=TRUE)

#****  Model 5: estimated rater and item slopes
mod5 <- sirt::rm.facets( dat[, paste0( "k",1:5) ], rater=dat$rater,
             pid=dat$idstud, est.a.rater=TRUE, est.a.item=TRUE)
summary(mod1)
summary(mod2)
summary(mod2a)
summary(mod3)
summary(mod4)
summary(mod5)

#****  Model 5a: Some fixed parameters in Model 5
# fix rater b parameters for raters 1, 4 and 5
b.rater.fixed <- rep(NA,7)
b.rater.fixed[ c(1,4,5) ] <- c(1,-.8,0)
# fix rater a parameters for first four raters
a.rater.fixed <- rep(NA,7)
a.rater.fixed[ c(1,2,3,4) ] <- c(1.1,0.9,.85,1)
# fix item b parameters of first item
tau.item.fixed <- matrix( NA, nrow=5, ncol=3 )
tau.item.fixed[ 1, ] <- c(-2,-1.5, 1 )
# fix item a parameters
a.item.fixed <- rep(NA,5)
a.item.fixed[ 1:4 ] <- 1
# estimate model
mod5a <- sirt::rm.facets( dat[, paste0( "k",1:5) ], rater=dat$rater,
             pid=dat$idstud, est.a.rater=TRUE, est.a.item=TRUE,
             tau.item.fixed=tau.item.fixed, b.rater.fixed=b.rater.fixed,
             a.rater.fixed=a.rater.fixed, a.item.fixed=a.item.fixed,
             est.mean=TRUE)
summary(mod5a)

#****  Model 6: Estimate rater model with reference rater 'db03'
mod6 <- sirt::rm.facets( dat[, paste0( "k",1:5) ], rater=dat$rater, est.a.item=TRUE,
             est.a.rater=TRUE, pid=dat$idstud, reference_rater="db03" )
summary(mod6)

#**** Model 7: Modelling rater-item-interactions
mod7 <- sirt::rm.facets( dat[, paste0( "k",1:5) ], rater=dat$rater, est.a.item=FALSE,
             est.a.rater=TRUE, pid=dat$idstud, reference_rater="db03",
             rater_item_int=TRUE)
summary(mod7)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Rater model}
%% \keyword{Facets model}
%% \keyword{summary}

%% File Name: data.bs.Rd
%% File Version: 0.13


\name{data.bs}
\alias{data.bs}
\alias{data.bs07a}

\docType{data}


\title{
Datasets from Borg and Staufenbiel (2007)
}

\description{
Datasets of the book of Borg and Staufenbiel (2007)
\emph{Lehrbuch Theorien and Methoden der Skalierung}.
}


\usage{
data(data.bs07a)
}


\format{
\itemize{
\item The dataset \code{data.bs07a} contains the data
\emph{Gefechtsangst} (p. 130) and contains 8 of the original 9 items.
The items are symptoms of anxiety in engagement. \cr
\code{GF1}: starkes Herzklopfen, \code{GF2}: flaues Gefuehl in der
Magengegend, \code{GF3}: Schwaechegefuehl, \code{GF4}: Uebelkeitsgefuehl,
\code{GF5}: Erbrechen, \code{GF6}: Schuettelfrost,
\code{GF7}: in die Hose urinieren/einkoten, \code{GF9}: Gefuehl der
Gelaehmtheit

The format is

\code{'data.frame':   100 obs. of  9 variables:} \cr
\code{ $ idpatt: int  44 29 1 3 28 50 50 36 37 25 ...} \cr
\code{ $ GF1   : int  1 1 1 1 1 0 0 1 1 1 ...} \cr
\code{ $ GF2   : int  0 1 1 1 1 0 0 1 1 1 ...} \cr
\code{ $ GF3   : int  0 0 1 1 0 0 0 0 0 1 ...} \cr
\code{ $ GF4   : int  0 0 1 1 0 0 0 1 0 1 ...} \cr
\code{ $ GF5   : int  0 0 1 1 0 0 0 0 0 0 ...} \cr
\code{ $ GF6   : int  1 1 1 1 1 0 0 0 0 0 ...} \cr
\code{ $ GF7   : num  0 0 1 1 0 0 0 0 0 0 ...} \cr
\code{ $ GF9   : int  0 0 1 1 1 0 0 0 0 0 ...} \cr

\item \emph{MORE DATASETS}

  }
}


%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}

%\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%}


\references{
Borg, I., & Staufenbiel, T. (2007).
\emph{Lehrbuch Theorie und Methoden der Skalierung}.
Bern: Hogrefe.
}


\examples{
\dontrun{
#############################################################################
# EXAMPLE 07a: Dataset Gefechtsangst
#############################################################################

data(data.bs07a)
dat <- data.bs07a
items <- grep( "GF", colnames(dat), value=TRUE )

#************************
# Model 1: Rasch model
mod1 <- TAM::tam.mml(dat[,items] )
summary(mod1)
IRT.WrightMap(mod1)

#************************
# Model 2: 2PL model
mod2 <- TAM::tam.mml.2pl(dat[,items] )
summary(mod2)

#************************
# Model 3: Latent class analysis (LCA) with two classes
tammodel <- "
ANALYSIS:
  TYPE=LCA;
  NCLASSES(2)
  NSTARTS(5,10)
LAVAAN MODEL:
  F=~ GF1__GF9
  "
mod3 <- TAM::tamaan( tammodel, dat )
summary(mod3)

#************************
# Model 4: LCA with three classes
tammodel <- "
ANALYSIS:
  TYPE=LCA;
  NCLASSES(3)
  NSTARTS(5,10)
LAVAAN MODEL:
  F=~ GF1__GF9
  "
mod4 <- TAM::tamaan( tammodel, dat )
summary(mod4)

#************************
# Model 5: Located latent class model (LOCLCA) with two classes
tammodel <- "
ANALYSIS:
  TYPE=LOCLCA;
  NCLASSES(2)
  NSTARTS(5,10)
LAVAAN MODEL:
  F=~ GF1__GF9
  "
mod5 <- TAM::tamaan( tammodel, dat )
summary(mod5)

#************************
# Model 6: Located latent class model with three classes
tammodel <- "
ANALYSIS:
  TYPE=LOCLCA;
  NCLASSES(3)
  NSTARTS(5,10)
LAVAAN MODEL:
  F=~ GF1__GF9
  "
mod6 <- TAM::tamaan( tammodel, dat )
summary(mod6)

#************************
# Model 7: Probabilistic Guttman model
mod7 <- sirt::prob.guttman( dat[,items] )
summary(mod7)

#-- model comparison
IRT.compareModels( mod1, mod2, mod3, mod4, mod5, mod6, mod7 )
}
}
%% \keyword{datasets}

%% File Name: dmlavaan.Rd
%% File Version: 0.12

\name{dmlavaan}
\alias{dmlavaan}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Comparing Regression Parameters of Different lavaan Models Fitted to the
Same Dataset
}
\description{
The function \code{dmlavaan} compares model parameters from different \pkg{lavaan}
models fitted to the same dataset. This leads to dependent coefficients.
Statistical inference is either conducted by M-estimation (i.e., robust
sandwich method; \code{method="bootstrap"}) or bootstrap (\code{method="bootstrap"}).
See Mize et al. (2019) or Weesie (1999) for more details.
}
\usage{
dmlavaan(fun1, args1, fun2, args2, method="sandwich", R=50)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{fun1}{
lavaan function of the first model (e.g., \code{"lavaan"}, \code{"cfa"}, or
\code{"sem"})
}
  \item{args1}{
arguments for lavaan function in the first model
}
  \item{fun2}{
lavaan function of the second model (e.g., \code{"lavaan"}, \code{"cfa"}, or
\code{"sem"})
}
  \item{args2}{
arguments for lavaan function in the second model
}
  \item{method}{
estimation method for standard errors
}
  \item{R}{
Number of bootstrap samples
}
}

\details{
In bootstrap estimation, a normal approximation is applied in the
computation of confidence intervals. Hence, \code{R} could be chosen
relatively small. \cr \cr
TO DO (not yet implemented): \cr
\tabular{ll}{
1) \tab inclusion of sampling weights \cr
2) \tab cluster robust standard errors in hierarchical sampling \cr
3) \tab stratification \cr
}
}

\value{
A list with following entries
\item{coef}{Model parameters of both models}
\item{vcov}{Covariance matrix of model parameters of both models}
\item{partable}{Parameter table containing all univariate model parameters}
\item{\ldots}{More entries}
}

\references{
Mize, T.D., Doan, L., & Long, J.S. (2019).
A general framework for comparing predictions and marginal effects across models.
\emph{Sociological Methodology, 49}(1), 152-189.
\doi{10.1177/0081175019852763}

Weesie, J. (1999) Seemingly unrelated estimation and the cluster-adjusted sandwich
estimator. \emph{Stata Technical Bulletin, 9}, 231-248.
}


%\author{
%%  ~~who you are~~
%}

%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}


\examples{
\dontrun{
############################################################################
# EXAMPLE 1: Confirmatory factor analysis with and without fourth item
#############################################################################

#**** simulate data
N <- 200  # number of persons
I <- 4    # number of items

# loadings and error correlations
lam <- seq(.7,.4, len=I)
PSI <- diag( 1-lam^2 )

# define some model misspecification
sd_error <- .1
S1 <- matrix( c( -1.84, 0.39,-0.68, 0.13,
  0.39,-1.31,-0.07,-0.27,
 -0.68,-0.07, 0.90, 1.91,
  0.13,-0.27, 1.91,-0.56 ), nrow=4, ncol=4, byrow=TRUE)
S1 <- ( S1 - mean(S1) ) / sd(S1) * sd_error

Sigma <- lam \%*\% t(lam) + PSI + S1
dat <- MASS::mvrnorm(n=N, mu=rep(0,I), Sigma=Sigma)
colnames(dat) <- paste0("X",1:4)
dat <- as.data.frame(dat)
rownames(Sigma) <- colnames(Sigma) <- colnames(dat)


#*** define two lavaan models
lavmodel1 <- "F=~ X1 + X2 + X3 + X4"
lavmodel2 <- "F=~ X1 + X2 + X3"

#*** define lavaan estimation arguments and functions
fun2 <- fun1 <- "cfa"
args1 <- list( model=lavmodel1, data=dat, std.lv=TRUE, estimator="MLR")
args2 <- args1
args2$model <- lavmodel2

#* run model comparison
res1 <- sirt::dmlavaan( fun1=fun1, args1=args1, fun2=fun2, args2=args2)

# inspect results
sirt:::print_digits(res1$partable, digits=3)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

%% File Name: expl.detect.Rd
%% File Version: 1.274

\name{expl.detect}
\alias{expl.detect}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Exploratory DETECT Analysis
}
\description{
This function estimates the DETECT index (Stout, Habing, Douglas & Kim, 1996;
Zhang & Stout, 1999a, 1999b) in an exploratory way.
Conditional covariances of itempairs are transformed into a distance
matrix such that items are clustered by the hierarchical Ward
algorithm (Roussos, Stout & Marden, 1998). Note that the function will not
provide the same output as the original DETECT software.
}
\usage{
expl.detect(data, score, nclusters, N.est=NULL, seed=NULL, bwscale=1.1,
    smooth=TRUE, use_sum_score=FALSE, hclust_method="ward.D", estsample=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
An \eqn{N \times I} data frame of dichotomous or polytomous responses.
Missing responses are allowed.
}
  \item{score}{
An ability estimate, e.g. the WLE, sum score or mean score
}
  \item{nclusters}{
Maximum number of clusters used in the exploratory analysis
}
  \item{N.est}{
Number of students in a (possible) validation of the DETECT index.
\code{N.est} students are drawn at random from \code{data}.
}
  \item{seed}{
Random seed
}
  \item{bwscale}{
Bandwidth scale factor
}
\item{smooth}{Logical indicating whether smoothing should be
applied for conditional covariance estimation}
\item{use_sum_score}{Logical indicating whether sum score should be used.
With this option, the bias corrected conditional covariance of Zhang and
Stout (1999) is used.}
\item{hclust_method}{Clustering method used as the argument
\code{method} in \code{\link[stats:hclust]{stats::hclust}}.
}
\item{estsample}{Optional vector of subject indices that defines the
estimation sample}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list with following entries
\item{detect.unweighted}{Unweighted DETECT statistics}
\item{detect.weighted}{Weighted DETECT statistics. Weighting is done
proportionally to sample sizes of item pairs.}
\item{clusterfit}{Fit of the cluster method}
\item{itemcluster}{Cluster allocations}
use_sum_score
}
\references{
Roussos, L. A., Stout, W. F., & Marden, J. I. (1998). Using new proximity
measures with hierarchical cluster analysis to detect multidimensionality.
\emph{Journal of Educational Measurement,
35}, 1-30.

Stout, W., Habing, B., Douglas, J., & Kim, H. R. (1996).
Conditional covariance-based nonparametric multidimensionality assessment.
\emph{Applied Psychological Measurement, 20}, 331-354.

Zhang, J., & Stout, W. (1999a). Conditional covariance structure
of generalized compensatory multidimensional items,
\emph{Psychometrika, 64}, 129-152.

Zhang, J., & Stout, W. (1999b). The theoretical DETECT index of
dimensionality and its application to approximate simple structure,
\emph{Psychometrika, 64}, 213-249.
}


%\author{
%Alexander Robitzsch
%}


\seealso{
For examples see \code{\link{conf.detect}}.
}
%% ~~objects to See Also as \code{\link{help}}, ~~~
%% \keyword{DETECT}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.mixed1.Rd
%% File Version: 0.12

\name{data.mixed1}
\alias{data.mixed1}
\docType{data}

\title{
Dataset with Mixed Dichotomous and Polytomous Item Responses
}

\description{
Dataset with mixed dichotomous and polytomous item responses.
}

\usage{data(data.mixed1)}
\format{
  A data frame with 1000 observations on the following 37 variables.

\code{'data.frame':   1000 obs. of  37 variables:} \cr
\code{ $ I01: num  1 1 1 1 1 1 1 0 1 1 ...} \cr
\code{ $ I02: num  1 1 1 1 1 1 1 1 0 1 ...} \cr
\code{ [...]} \cr
\code{ $ I36: num  1 1 1 1 0 0 0 0 1 1 ...} \cr
\code{ $ I37: num  0 1 1 1 0 1 0 0 1 1 ...} \cr
}

%%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%%}

%%\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%%}
%%\references{
%%  ~~ possibly secondary sources and usages ~~
%%}



\examples{
data(data.mixed1)
apply( data.mixed1, 2, max )
  ##   I01 I02 I03 I04 I05 I06 I07 I08 I09 I10 I11 I12 I13 I14 I15 I16
  ##     1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
  ##   I17 I18 I19 I20 I21 I22 I23 I24 I25 I26 I27 I28 I29 I30 I31 I32
  ##     1   1   1   1   4   4   1   1   1   1   1   1   1   1   1   1
  ##   I33 I34 I35 I36 I37
  ##     1   1   1   1   1
}
%% \keyword{datasets}

%% File Name: rasch.jml.Rd
%% File Version: 1.467

\name{rasch.jml}
\alias{rasch.jml}
\alias{summary.rasch.jml}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Joint Maximum Likelihood (JML) Estimation of the Rasch Model
}
\description{
This function estimates the Rasch model using joint maximum likelihood
estimation (Lincare, 1994). The PROX algorithm (Lincare, 1994) is used
for the generation of starting values of item parameters.
}
\usage{
rasch.jml(dat, method="MLE", b.init=NULL, constraints=NULL, weights=NULL,
    center="persons", glob.conv=10^(-6), conv1=1e-05, conv2=0.001, progress=TRUE,
    bsteps=4, thetasteps=2, wle.adj=0, jmliter=100, prox=TRUE,
    proxiter=30, proxconv=0.01, dp=NULL, theta.init=NULL, calc.fit=TRUE,
    prior_sd=NULL)

\method{summary}{rasch.jml}(object, digits=3, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{An \eqn{N \times I} data frame of dichotomous item responses
      where \eqn{N} indicates the number of persons
and \eqn{I} the number of items
}
  \item{method}{
Method for estimating person parameters during JML iterations.
\code{MLE} is maximum likelihood estimation (where person with
perfect scores are deleted from analysis). \code{WLE} uses weighted
likelihood estimation (Warm, 1989) for person parameter estimation.
Default is \code{MLE}.
}
  \item{b.init}{
Initial values of item difficulties
}
  \item{constraints}{
Optional matrix or data.frame with two columns. First column is an integer of
item indexes or item names (\code{colnames(dat)}) which shall be fixed
during estimation. The second column is the corresponding
item difficulty.
}
  \item{weights}{
Person sample weights. Default is \code{NULL}, i.e. all persons in the sample
are equally weighted.
}
\item{center}{Character indicator whether persons (\code{"persons"}),
items (\code{"items"}) should be centered or (\code{"none"})
should be conducted.
}
  \item{glob.conv}{
Global convergence criterion with respect to the log-likelihood function
}
  \item{conv1}{
Convergence criterion for estimation of item parameters
}
  \item{conv2}{
Convergence criterion for estimation of person parameters
}
  \item{progress}{
Display progress? Default is \code{TRUE}
}
%% bsteps=4,thetasteps=2, wle.adj=0,
  \item{bsteps}{
Number of steps for b parameter estimation
}
  \item{thetasteps}{
Number of steps for theta parameter estimation
}
  \item{wle.adj}{
Score adjustment for WLE estimation
}
  \item{jmliter}{
Number of maximal iterations during JML estimation
}
  \item{prox}{
Should the PROX algorithm (see \code{\link{rasch.prox}}) be used as initial estimations?
Default is \code{TRUE}.
}
  \item{proxiter}{
Number of maximal PROX iterations
}
  \item{proxconv}{
Convergence criterion for PROX iterations
}
  \item{dp}{
 Object created from data preparation function (\code{.data.prep})
 which could be created in earlier JML runs. Default is \code{NULL}.
 }
\item{theta.init}{
 Initial person parameter estimate
 }
\item{calc.fit}{
 Should itemfit being calculated?
 }
 \item{prior_sd}{
 Optional value for standard deviation of prior distribution for
 ability values if penalized JML should be utilized
 }
  \item{object}{
Object of class \code{rasch.jml}
}
\item{digits}{Number of digits used for rounding}
  \item{\dots}{
Further arguments to be passed
}
}

\details{
The estimation is known to have a bias in item parameters for
a fixed (finite) number of items. In literature (Lincare, 1994), a simple
bias correction formula is proposed and included in the value
\code{item$itemdiff.correction} in this function. If \eqn{I} denotes the number
of items, then the correction factor is \eqn{\frac{I-1}{I}}.
}
%%If there are \eqn{J}
%%item parameter constraints, then the correction factor amounts to
%%\eqn{\frac{I-J-1}{I-J}}.



\value{
A list with following entries
\item{item}{Estimated item parameters}
\item{person}{Estimated person parameters}
\item{method}{Person parameter estimation method}
\item{dat}{Original data frame}
\item{deviance}{Deviance}
\item{data.proc}{Processed data frames excluding persons with extreme scores}
\item{dp}{Value of data preparation (it is used in the function
\code{\link{rasch.jml.jackknife1}})}
}

\references{
 Linacre, J. M. (1994). \emph{Many-Facet Rasch Measurement}. Chicago: MESA Press.

 Warm, T. A. (1989). Weighted likelihood estimation of ability in the item
 response theory. \emph{Psychometrika, 54}, 427-450.
    }



%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
Get a summary with \code{\link{summary.rasch.jml}}.

See \code{\link{rasch.prox}} for the PROX algorithm as initial iterations.

For a bias correction of the JML method try \code{\link{rasch.jml.jackknife1}}.

JML estimation can also be conducted with the \pkg{TAM}
(\code{\link[TAM:tam.jml]{TAM::tam.jml}})
and \pkg{immer} (\code{immer::immer_jml})
packages.

See also marginal maximum likelihood estimation with \code{\link{rasch.mml2}}
or the \R package \pkg{ltm}.
}

% (\code{\link[mixRasch:mixRasch]{mixRasch::mixRasch}})


\examples{
#############################################################################
# EXAMPLE 1: Simulated data from the Rasch model
#############################################################################

set.seed(789)
N <- 500    # number of persons
I <- 11     # number of items
b <- seq( -2, 2, length=I )
dat <- sirt::sim.raschtype( stats::rnorm( N, mean=.5 ), b )
colnames(dat) <- paste( "I", 1:I, sep="")

# JML estimation of the Rasch model (centering persons)
mod1 <- sirt::rasch.jml( dat )
summary(mod1)

# JML estimation of the Rasch model (centering items)
mod1b <- sirt::rasch.jml( dat, center="items" )
summary(mod1b)

# MML estimation with rasch.mml2 function
mod2 <- sirt::rasch.mml2( dat )
summary(mod2)

# Pairwise method of Fischer
mod3 <- sirt::rasch.pairwise( dat )
summary(mod3)

# JML estimation in TAM
\dontrun{
library(TAM)
mod4 <- TAM::tam.jml( resp=dat )

#******
# item parameter constraints in JML estimation
# fix item difficulties: b[4]=-.76 and b[6]=.10
constraints <- matrix( cbind( 4, -.76,
                              6, .10 ),
                  ncol=2, byrow=TRUE )
mod6 <- sirt::rasch.jml( dat, constraints=constraints )
summary(mod6)
  # For constrained item parameters, it this not obvious
  # how to calculate a 'right correction' of item parameter bias
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{Rasch model}
%% \keyword{Joint maximum likelihood (JML)}
%% \keyword{summary}
% __ONLY ONE__ keyword per line

%% File Name: rasch.jml.jackknife1.Rd
%% File Version: 2.192

\name{rasch.jml.jackknife1}
\alias{rasch.jml.jackknife1}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Jackknifing the IRT Model Estimated by Joint Maximum Likelihood (JML)
}
\description{
Jackknife estimation is an alternative to other ad hoc proposed
methods for bias correction (Hahn & Newey, 2004).
}
\usage{
rasch.jml.jackknife1(jmlobj)
}
\arguments{
  \item{jmlobj}{
Output of \code{rasch.jml}
}
}
\details{
Note that items are used for jackknifing (Hahn & Newey, 2004).
By default, all \eqn{I} items in the data frame are used as
jackknife units.
}
\value{
A list with following entries
  \item{item}{
        A data frame with item parameters
        \itemize{
            \item \code{b.JML}: Item difficulty from JML estimation
            \item \code{b.JMLcorr}: Item difficulty from JML estimation by
                    applying the correction factor \eqn{(I-1)/I}
            \item \code{b.jack}: Item difficulty from Jackknife estimation
            \item \code{b.jackse}: Standard error of Jackknife estimation
                        for item difficulties.
                        Note that this parameter refer to the standard error
                    with respect to item sampling
            \item \code{b.JMLse}: Standard error for item difficulties
                    obtained from JML estimation
                }
            }
  \item{jack.itemdiff}{A matrix containing all item difficulties obtained by Jackknife}
}
\references{
Hahn, J., & Newey, W. (2004). Jackknife and analytical bias reduction for
nonlinear panel models. \emph{Econometrica, 72}, 1295-1319.
}

%\author{
%Alexander Robitzsch
%}



\seealso{
For JML estimation \code{\link{rasch.jml}}.

For analytical bias correction methods see \code{\link{rasch.jml.biascorr}}.
}
\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Simulated data from the Rasch model
#############################################################################
set.seed(7655)
N <- 5000    # number of persons
I <- 11      # number of items
b <- seq( -2, 2, length=I )
dat <- sirt::sim.raschtype( rnorm( N ), b )
colnames(dat) <- paste( "I", 1:I, sep="")

# estimate the Rasch model with JML
mod <- sirt::rasch.jml(dat)
summary(mod)

# re-estimate the Rasch model using Jackknife
mod2 <- sirt::rasch.jml.jackknife1( mod )
  ##
  ##   Joint Maximum Likelihood Estimation
  ##   Jackknife Estimation
  ##   11 Jackknife Units are used
  ##   |--------------------PROGRESS--------------------|
  ##   |------------------------------------------------|
  ##
  ##          N     p  b.JML b.JMLcorr b.jack b.jackse b.JMLse
  ##   I1  4929 0.853 -2.345    -2.131 -2.078    0.079   0.045
  ##   I2  4929 0.786 -1.749    -1.590 -1.541    0.075   0.039
  ##   I3  4929 0.723 -1.298    -1.180 -1.144    0.065   0.036
  ##   I4  4929 0.657 -0.887    -0.806 -0.782    0.059   0.035
  ##   I5  4929 0.576 -0.420    -0.382 -0.367    0.055   0.033
  ##   I6  4929 0.492  0.041     0.038  0.043    0.054   0.033
  ##   I7  4929 0.409  0.502     0.457  0.447    0.056   0.034
  ##   I8  4929 0.333  0.939     0.854  0.842    0.058   0.035
  ##   I9  4929 0.264  1.383     1.257  1.229    0.065   0.037
  ##   I10 4929 0.210  1.778     1.617  1.578    0.071   0.040
  ##   I11 4929 0.154  2.266     2.060  2.011    0.077   0.044
#-> Item parameters obtained by jackknife seem to be acceptable.
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Joint maximum likelihood (JML)}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: rasch.va.Rd
%% File Version: 0.20

\name{rasch.va}
\alias{rasch.va}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimation of the Rasch Model with Variational Approximation
}
\description{
This function estimates the Rasch model by the estimation method
of variational approximation (Rijmen & Vomlel, 2008).
}
\usage{
rasch.va(dat, globconv=0.001, maxiter=1000)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame with dichotomous item responses
}
  \item{globconv}{
Convergence criterion for item parameters
}
  \item{maxiter}{
Maximal number of iterations
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}


\value{
A list with following entries:
  \item{sig}{Standard deviation of the trait}
  \item{item}{Data frame with item parameters}
  \item{xsi.ij}{Data frame with variational parameters \eqn{\xi_{ij}}}
  \item{mu.i}{Vector with individual means \eqn{\mu_i}}
\item{sigma2.i}{Vector with individual variances \eqn{\sigma_i^2}}
}

\references{
Rijmen, F., & Vomlel, J. (2008). Assessing the performance of
variational methods for mixed logistic regression models.
\emph{Journal of Statistical Computation and Simulation,
78}, 765-779.
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{
#############################################################################
# EXAMPLE 1: Rasch model
#############################################################################
set.seed(8706)
N <- 5000
I <- 20
dat <- sirt::sim.raschtype( stats::rnorm(N,sd=1.3), b=seq(-2,2,len=I) )

# estimation via variational approximation
mod1 <- sirt::rasch.va(dat)

# estimation via marginal maximum likelihood
mod2 <- sirt::rasch.mml2(dat)

# estmation via joint maximum likelihood
mod3 <- sirt::rasch.jml(dat)

# compare sigma
round( c( mod1$sig, mod2$sd.trait ), 3 )
## [1] 1.222 1.314

# compare b
round( cbind( mod1$item$b, mod2$item$b, mod3$item$itemdiff), 3 )
##         [,1]   [,2]   [,3]
##  [1,] -1.898 -1.967 -2.090
##  [2,] -1.776 -1.841 -1.954
##  [3,] -1.561 -1.618 -1.715
##  [4,] -1.326 -1.375 -1.455
##  [5,] -1.121 -1.163 -1.228
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Variational approximation}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: rm.sdt.Rd
%% File Version: 1.494

\name{rm.sdt}
\alias{rm.sdt}
\alias{summary.rm.sdt}
\alias{logLik.rm.sdt}
\alias{anova.rm.sdt}
\alias{IRT.irfprob.rm.sdt}
\alias{IRT.likelihood.rm.sdt}
\alias{IRT.posterior.rm.sdt}
\alias{IRT.modelfit.rm.sdt}
\alias{summary.IRT.modelfit.rm.sdt}
\alias{IRT.factor.scores.rm.sdt}
\alias{plot.rm.sdt}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Hierarchical Rater Model Based on Signal Detection Theory (HRM-SDT)
}

\description{
This function estimates a version of the hierarchical rater
model (HRM) based on signal detection theory (HRM-SDT; DeCarlo, 2005;
DeCarlo, Kim & Johnson, 2011; Robitzsch & Steinfeld, 2018).
The model is estimated by means of an EM algorithm adapted from
multilevel latent class analysis (Vermunt, 2008).
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ARGUMENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\usage{
rm.sdt(dat, pid, rater, Qmatrix=NULL, theta.k=seq(-9, 9, len=30),
    est.a.item=FALSE, est.c.rater="n", est.d.rater="n", est.mean=FALSE, est.sigma=TRUE,
    skillspace="normal", tau.item.fixed=NULL, a.item.fixed=NULL,
    d.min=0.5, d.max=100, d.start=3, c.start=NULL, tau.start=NULL, sd.start=1,
    d.prior=c(3,100), c.prior=c(3,100), tau.prior=c(0,1000), a.prior=c(1,100),
    link_item="GPCM", max.increment=1, numdiff.parm=0.00001, maxdevchange=0.1,
    globconv=.001, maxiter=1000, msteps=4, mstepconv=0.001, optimizer="nlminb" )

\method{summary}{rm.sdt}(object, file=NULL, ...)

\method{plot}{rm.sdt}(x, ask=TRUE, ...)

\method{anova}{rm.sdt}(object,...)

\method{logLik}{rm.sdt}(object,...)

\method{IRT.factor.scores}{rm.sdt}(object, type="EAP", ...)

\method{IRT.irfprob}{rm.sdt}(object,...)

\method{IRT.likelihood}{rm.sdt}(object,...)

\method{IRT.posterior}{rm.sdt}(object,...)

\method{IRT.modelfit}{rm.sdt}(object,...)

\method{summary}{IRT.modelfit.rm.sdt}(object,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Original data frame. Ratings on variables must be in rows,
i.e. every row corresponds to a person-rater combination.
}
  \item{pid}{
Person identifier.
}
  \item{rater}{
Rater identifier.
}
  \item{Qmatrix}{
An optional Q-matrix. If this matrix is not provided,
then by default the ordinary scoring of categories
(from 0 to the maximum score of \eqn{K}) is used.
}
  \item{theta.k}{
A grid of theta values for the ability distribution.
}
  \item{est.a.item}{
Should item parameters \eqn{a_i} be estimated?
}
  \item{est.c.rater}{
Type of estimation for item-rater parameters \eqn{c_{ir}}
in the signal detection model. Options are \code{'n'} (no estimation),
\code{'e'} (set all parameters equal to each other),
\code{'i'} (itemwise estimation), \code{'r'} (rater wise estimation)
and \code{'a'} (all parameters are estimated independently
from each other).
}
  \item{est.d.rater}{
Type of estimation of \eqn{d} parameters. Options are
the same as in \code{est.c.rater}.
}
\item{est.mean}{Optional logical indicating whether the mean of the
trait distribution should be estimated.}
\item{est.sigma}{Optional logical indicating whether the standard deviation of the
trait distribution should be estimated.}
\item{skillspace}{Specified \eqn{\theta} distribution type. It can be
    \code{"normal"} or \code{"discrete"}. In the latter case,
    all probabilities of the distribution are separately
       estimated.}
\item{tau.item.fixed}{Optional matrix with three columns specifying
fixed \eqn{\tau} parameters. The first two columns denote item and
category indices, the third the fixed value. See Example 3.}
\item{a.item.fixed}{Optional matrix with two columns specifying fixed
    \eqn{a} parameters. First column: Item index. Second column:
Fixed \eqn{a} parameter.}
  \item{d.min}{
Minimal \eqn{d} parameter to be estimated
}
  \item{d.max}{
Maximal \eqn{d} parameter to be estimated
}
\item{d.start}{
Starting value(s) of \eqn{d} parameters
}
\item{c.start}{
Starting values of \eqn{c} parameters
}
\item{tau.start}{
Starting values of \eqn{\tau} parameters
}
\item{sd.start}{
Starting value for trait standard deviation
}
\item{d.prior}{Normal prior \eqn{N(M,S^2)} for \eqn{d} parameters}
\item{c.prior}{Normal prior for \eqn{c} parameters. The prior for
    parameter \eqn{c_{irk}} is defined as \eqn{M \cdot ( k - 0.5) }
    where \eqn{M} is \code{c.prior[1]}. }
\item{tau.prior}{Normal prior for \eqn{\tau} parameters}
\item{a.prior}{Normal prior for \eqn{a} parameters}
\item{link_item}{Type of item response function for latent responses.
Can be \code{"GPCM"} for the generalized partial credit model or
\code{"GRM"} for the graded response model.
}
  \item{max.increment}{
Maximum increment of item parameters during estimation
}
  \item{numdiff.parm}{
Numerical differentiation step width
}
  \item{maxdevchange}{
Maximum relative deviance change as a convergence criterion
}
  \item{globconv}{
Maximum parameter change
}
  \item{maxiter}{
Maximum number of iterations
}
  \item{msteps}{
Maximum number of iterations during an M step
}
  \item{mstepconv}{
Convergence criterion in an M step
}
\item{optimizer}{Choice of optimization function in M-step for
item parameters. Options are \code{"nlminb"} for \code{\link[stats:nlminb]{stats::nlminb}}
and \code{"optim"} for \code{\link[stats:optim]{stats::optim}}.
}
  \item{object}{
Object of class \code{rm.sdt}
}
\item{file}{Optional file name in which summary should be written.}
\item{x}{Object of class \code{rm.sdt}}
\item{ask}{Optional logical indicating whether a new plot should be asked for.}
\item{type}{Factor score estimation method. Up to now,
    only \code{type="EAP"} is supported.}
\item{\dots}{
Further arguments to be passed
}
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DETAILS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\details{
The specification of the model follows DeCarlo et al. (2011).
The second level models the ideal rating (latent response) \eqn{\eta=0, ...,K}
of person \eqn{p} on item \eqn{i}. The option \code{link_item='GPCM'} follows the
generalized partial credit model
\deqn{ P( \eta_{pi}=\eta | \theta_p ) \propto
exp( a_{i} q_{i \eta } \theta_p - \tau_{i \eta } ) }. The option \code{link_item='GRM'} employs the
graded response model   \deqn{ P( \eta_{pi}=\eta | \theta_p )=
\Psi( \tau_{i,\eta + 1} - a_i \theta_p ) - \Psi( \tau_{i,\eta} - a_i \theta_p ) }

At the first level, the ratings \eqn{X_{pir}} for
person \eqn{p} on item \eqn{i} and rater \eqn{r}
are modeled as a signal detection model
\deqn{ P( X_{pir} \le k | \eta_{pi} )=
G( c_{irk} - d_{ir} \eta_{pi} )}
where \eqn{G} is the logistic distribution function
and the categories are \eqn{k=1,\ldots, K+1}.
Note that the item response model can be equivalently written
as \deqn{ P( X_{pir} \ge k | \eta_{pi} )=
G(   d_{ir} \eta_{pi} - c_{irk})}

The thresholds \eqn{c_{irk}} can be further restricted to
\eqn{c_{irk}=c_{k}} (\code{est.c.rater='e'}),
\eqn{c_{irk}=c_{ik}} (\code{est.c.rater='i'}) or
\eqn{c_{irk}=c_{ir}} (\code{est.c.rater='r'}). The same
holds for rater precision parameters \eqn{d_{ir}}.
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VALUES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\value{
A list with following entries:
\item{deviance}{Deviance}
\item{ic}{Information criteria and number of parameters}
\item{item}{Data frame with item parameters. The columns
\code{N} and \code{M} denote the number of observed ratings and the
observed mean of all ratings, respectively. \cr
In addition to item parameters \eqn{\tau_{ik}} and \eqn{a_i}, the mean
for the latent response (\code{latM}) is computed as
\eqn{E( \eta_i )=\sum_p P( \theta_p ) q_{ik} P( \eta_i=k | \theta_p ) }
which provides an item parameter at the original metric of ratings. The latent standard
deviation (\code{latSD}) is computed in the same manner.
}
\item{rater}{Data frame with rater parameters.
Transformed \eqn{c} parameters
(\code{c_x.trans}) are computed as \eqn{c_{irk} / ( d_{ir}  )}.
}
\item{person}{Data frame with person parameters: EAP and corresponding standard
errors}
\item{EAP.rel}{EAP reliability}
\item{EAP.rel}{EAP reliability}
\item{mu}{Mean of the trait distribution}
\item{sigma}{Standard deviation of the trait distribution}
\item{tau.item}{Item parameters \eqn{\tau_{ik}}}
\item{se.tau.item}{Standard error of item parameters \eqn{\tau_{ik}}}
\item{a.item}{Item slopes \eqn{a_i}}
\item{se.a.item}{Standard error of item slopes \eqn{a_i}}
\item{c.rater}{Rater parameters \eqn{c_{irk}}}
\item{se.c.rater}{Standard error of rater severity parameter \eqn{c_{irk}}}
\item{d.rater}{Rater slope parameter \eqn{d_{ir}}}
\item{se.d.rater}{Standard error of rater slope parameter \eqn{d_{ir}}}
\item{f.yi.qk}{Individual likelihood}
\item{f.qk.yi}{Individual posterior distribution}
\item{probs}{Item probabilities at grid \code{theta.k}. Note that these
  probabilities are calculated on the pseudo items \eqn{i \times r},
  i.e. the interaction of item and rater.}
\item{prob.item}{Probabilities \eqn{P( \eta_i=\eta | \theta )} of
    latent item responses evaluated at theta grid \eqn{\theta_p}.
}
\item{n.ik}{Expected counts}
\item{pi.k}{Estimated trait distribution \eqn{P(\theta_p)}.
}
\item{maxK}{Maximum number of categories}
\item{procdata}{Processed data}
\item{iter}{Number of iterations}
\item{\dots}{Further values}
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% REFERENCES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\references{
DeCarlo, L. T. (2005). A model of rater behavior in essay grading
based on signal detection theory.
\emph{Journal of Educational Measurement, 42}, 53-76.

DeCarlo, L. T. (2010). \emph{Studies of a latent-class signal-detection model
for constructed response scoring II: Incomplete and hierarchical designs}.
ETS Research Report ETS RR-10-08. Princeton NJ: ETS.

DeCarlo, T., Kim, Y., & Johnson, M. S. (2011).
A hierarchical rater model for constructed responses,
with a signal detection rater model.
\emph{Journal of Educational Measurement, 48}, 333-356.

Robitzsch, A., & Steinfeld, J. (2018). Item response models for human ratings: Overview,
estimation methods, and implementation in R.
\emph{Psychological Test and Assessment Modeling, 60}(1), 101-139.

Vermunt, J. K. (2008). Latent class and finite mixture models for
multilevel data sets. \emph{Statistical Methods in Medical Research, 17}, 33-51.
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
The facets rater model can be estimated with \code{\link{rm.facets}}.
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EXAMPLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\examples{
#############################################################################
# EXAMPLE 1: Hierarchical rater model (HRM-SDT) data.ratings1
#############################################################################
data(data.ratings1)
dat <- data.ratings1

\dontrun{
# Model 1: Partial Credit Model: no rater effects
mod1 <- sirt::rm.sdt( dat[, paste0( "k",1:5) ], rater=dat$rater,
            pid=dat$idstud, est.c.rater="n", d.start=100,  est.d.rater="n" )
summary(mod1)

# Model 2: Generalized Partial Credit Model: no rater effects
mod2 <- sirt::rm.sdt( dat[, paste0( "k",1:5) ], rater=dat$rater,
            pid=dat$idstud, est.c.rater="n", est.d.rater="n",
            est.a.item=TRUE, d.start=100)
summary(mod2)

# Model 3: Equal effects in SDT
mod3 <- sirt::rm.sdt( dat[, paste0( "k",1:5) ], rater=dat$rater,
            pid=dat$idstud, est.c.rater="e", est.d.rater="e")
summary(mod3)

# Model 4: Rater effects in SDT
mod4 <- sirt::rm.sdt( dat[, paste0( "k",1:5) ], rater=dat$rater,
            pid=dat$idstud, est.c.rater="r", est.d.rater="r")
summary(mod4)

#############################################################################
# EXAMPLE 2: HRM-SDT data.ratings3
#############################################################################

data(data.ratings3)
dat <- data.ratings3
dat <- dat[ dat$rater < 814, ]
psych::describe(dat)

# Model 1: item- and rater-specific effects
mod1 <- sirt::rm.sdt( dat[, paste0( "crit",c(2:4)) ], rater=dat$rater,
            pid=dat$idstud, est.c.rater="a", est.d.rater="a" )
summary(mod1)
plot(mod1)

# Model 2: Differing number of categories per variable
mod2 <- sirt::rm.sdt( dat[, paste0( "crit",c(2:4,6)) ], rater=dat$rater,
            pid=dat$idstud, est.c.rater="a", est.d.rater="a")
summary(mod2)
plot(mod2)

#############################################################################
# EXAMPLE 3: Hierarchical rater model with discrete skill spaces
#############################################################################

data(data.ratings3)
dat <- data.ratings3
dat <- dat[ dat$rater < 814, ]
psych::describe(dat)

# Model 1: Discrete theta skill space with values of 0,1,2 and 3
mod1 <- sirt::rm.sdt( dat[, paste0( "crit",c(2:4)) ], theta.k=0:3, rater=dat$rater,
            pid=dat$idstud, est.c.rater="a", est.d.rater="a", skillspace="discrete" )
summary(mod1)
plot(mod1)

# Model 2: Modelling of one item by using a discrete skill space and
#          fixed item parameters

# fixed tau and a parameters
tau.item.fixed <- cbind( 1, 1:3,  100*cumsum( c( 0.5, 1.5, 2.5)) )
a.item.fixed <- cbind( 1, 100 )
# fit HRM-SDT
mod2 <- sirt::rm.sdt( dat[, "crit2", drop=FALSE], theta.k=0:3, rater=dat$rater,
            tau.item.fixed=tau.item.fixed,a.item.fixed=a.item.fixed, pid=dat$idstud,
            est.c.rater="a", est.d.rater="a", skillspace="discrete" )
summary(mod2)
plot(mod2)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Rater model}
%% \keyword{Signal detection model}
%% \keyword{summary}
%% \keyword{plot}

%% File Name: wle.rasch.Rd
%% File Version: 1.18

\name{wle.rasch}
\alias{wle.rasch}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Weighted Likelihood Estimation of Person Abilities
}
\description{
This function computes weighted likelihood estimates for dichotomous responses based
on the Rasch model (Warm, 1989).
}
\usage{
wle.rasch(dat, dat.resp=NULL, b, itemweights=1 + 0 * b,
    theta=rep(0, nrow(dat)), conv=0.001, maxit=200,
    wle.adj=0, progress=FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{ An \eqn{N \times I} data frame of
  dichotomous item responses
}
  \item{dat.resp}{
    Optional data frame with dichotomous response indicators
}
  \item{b}{
    Vector of length \eqn{I} with fixed item difficulties
}
  \item{itemweights}{
    Optional vector of fixed item discriminations
}
  \item{theta}{
    Optional vector of initial person parameter estimates
}
  \item{conv}{
    Convergence criterion
}
  \item{maxit}{
    Maximal number of iterations
}
  \item{wle.adj}{
    Constant for WLE adjustment
}
  \item{progress}{
    Display progress?
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
%%  ~Describe the value returned
A list with following entries
  \item{theta}{
        Estimated weighted likelihood estimate
        }
  \item{dat.resp}{
        Data frame with dichotomous response indicators. A one indicates
        an observed response, a zero a missing response. See also \code{dat.resp}
        in the list of arguments of this function.
        }
   \item{p.ia}{Matrix with expected item response, i.e.
    the probabilities \eqn{P(X_{pi}=1|\theta_p )=invlogit( \theta_p - b_i )}.
            }
   \item{wle}{WLE reliability (Adams, 2005)}
}


\references{
Adams, R. J. (2005). Reliability as a measurement design effect.
\emph{Studies in Educational Evaluation, 31}, 162-172.

Warm, T. A. (1989). Weighted likelihood estimation of ability in item response theory.
\emph{Psychometrika, 54}, 427-450.
}



%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For standard errors of weighted likelihood estimates estimated via jackknife
see \code{\link{wle.rasch.jackknife}}.

For a joint estimation of item and person parameters see the joint maximum
likelihood estimation method in \code{\link{rasch.jml}}.
}
\examples{
#############################################################################
# EXAMPLE 1: Dataset Reading
#############################################################################
data(data.read)

# estimate the Rasch model
mod <- sirt::rasch.mml2(data.read)
mod$item

# estmate WLEs
mod.wle <- sirt::wle.rasch( dat=data.read, b=mod$item$b )
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Weighted likelihood estimation (WLE)}
%% \keyword{Person parameter estimation}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: pbivnorm2.Rd
%% File Version: 0.263

\name{pbivnorm2}
\alias{pbivnorm2}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Cumulative Function for the Bivariate Normal Distribution
}
\description{
This function evaluates the bivariate normal distribution
\eqn{\Phi_2 ( x, y ; \rho )}
assuming zero means and unit variances. It uses a simple approximation
by Cox and Wermuth (1991) with corrected formulas in Hong (1999).
}
\usage{
pbivnorm2(x, y, rho)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
Vector of \eqn{x} coordinates
}
  \item{y}{
Vector of \eqn{y} coordinates
}
  \item{rho}{
Vector of correlations between random normal variates
}
}
%% \details{
%%  ~~ If necessary, more details than the description above ~~
%% }

\value{
Vector of probabilities
}

%% Cox: 10.2307/1403446
%% \doi{10.2307/1403446}
\references{
Cox, D. R., & Wermuth, N. (1991). A simple approximation for bivariate and
trivariate normal integrals.
\emph{International Statistical Review, 59}(2), 263-269.

Hong, H. P. (1999). An approximation to bivariate and trivariate normal
integrals. \emph{Engineering and Environmental Systems, 16}(2), 115-127.
\doi{10.1080/02630259908970256}
}



%\author{
%Alexander Robitzsch
%}


\note{
The function is less precise for correlations near 1 or -1.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also the
\code{\link[pbivnorm:pbivnorm]{pbivnorm::pbivnorm}}
function in the \pkg{pbivnorm} package.
}
\examples{
library(pbivnorm)
# define input
x <- c(0, 0,  .5, 1, 1  )
y <- c( 0, -.5,  1, 3, .5 )
rho <- c( .2, .8, -.4, .6, .5 )
# compare pbivnorm2 and pbivnorm functions
pbiv2 <- sirt::pbivnorm2( x=x, y=y, rho=rho )
pbiv <- pbivnorm::pbivnorm(  x,  y, rho=rho )
max( abs(pbiv-pbiv2))
  ## [1] 0.0030626
round( cbind( x, y, rho,pbiv, pbiv2 ), 4 )
  ##          x    y  rho   pbiv  pbiv2
  ##   [1,] 0.0  0.0  0.2 0.2820 0.2821
  ##   [2,] 0.0 -0.5  0.8 0.2778 0.2747
  ##   [3,] 0.5  1.0 -0.4 0.5514 0.5514
  ##   [4,] 1.0  3.0  0.6 0.8412 0.8412
  ##   [5,] 1.0  0.5  0.5 0.6303 0.6304
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Bivariate normal distribution}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: rasch.pml3.Rd
%% File Version: 2.76



\name{rasch.pml3}
\alias{rasch.pml3}
\alias{summary.rasch.pml}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Pairwise Marginal Likelihood Estimation for the Probit Rasch Model
}
\description{
This function estimates unidimensional 1PL and 2PL models with
the probit link using pairwise marginal maximum likelihood
estimation (PMML; Renard, Molenberghs & Geys, 2004).
Item pairs within an itemcluster can be excluded from the
pairwise likelihood (argument \code{itemcluster}).
The other alternative is to model a residual
error structure with itemclusters (argument \code{error.corr}).
}
\usage{
rasch.pml3(dat, est.b=seq(1, ncol(dat)), est.a=rep(0,ncol(dat)),
    est.sigma=TRUE, itemcluster=NULL, weight=rep(1, nrow(dat)), numdiff.parm=0.001,
    b.init=NULL, a.init=NULL,  sigma.init=NULL, error.corr=0*diag( 1, ncol(dat) ),
    err.constraintM=NULL, err.constraintV=NULL, glob.conv=10^(-6), conv1=10^(-4),
    pmliter=300, progress=TRUE, use.maxincrement=TRUE )

\method{summary}{rasch.pml}(object,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
An \eqn{N \times I} data frame of dichotomous item responses
}
  \item{est.b}{
Vector of integers of length \eqn{I}. Same integers mean that the
corresponding items do have the same item difficulty \code{b}.
Entries of \code{0} mean fixing item parameters to values
specified in \code{b.init}.
}
\item{est.a}{
Vector of integers of length \eqn{I}. Same integers mean that the
corresponding items do have the same item slope \code{a}.
Entries of \code{0} mean fixing item parameters to values
specified in \code{a.init}.
}
  \item{est.sigma}{
Should sigma (the trait standard deviation) be estimated?
The default is \code{TRUE}.
}
  \item{itemcluster}{
Optional vector of length \eqn{I} of integers which indicates itemclusters.
Same integers correspond to the same itemcluster. An entry of \code{0}
correspond to an item which is not included in any itemcluster.
}
  \item{weight}{
Optional vector of person weights
}
  \item{numdiff.parm}{
Step parameter for numerical differentiation
}
  \item{b.init}{
Initial or fixed item difficulty
}
  \item{a.init}{
Initial or fixed item slopes
}
  \item{sigma.init}{
Initial or fixed trait standard deviation
}
  \item{error.corr}{
An optional \eqn{I \times I} integer matrix
which defines the estimation of residual correlations.
Entries of zero indicate that the corresponding
residual correlation should not be estimated.
Integers which differ from zero indicate correlations to be estimated.
All entries with an equal integer are estimated by the same residual
correlation. The default of \code{error.corr} is a diagonal matrix
which means that no residual correlation is estimated. If \code{error.corr}
deviates from this default, then the argument \code{itemcluster} is set
to \code{NULL}.\cr
If some error correlations are estimated, then no itempairs in
\code{itemcluster} can be excluded from the pairwise modeling.
}
%     err.constraintM=NULL, err.constraintV=NULL,
\item{err.constraintM}{
An optional \eqn{P \times L} matrix where \eqn{P} denotes
the number of item pairs in pseudolikelihood estimation
and \eqn{L} is the number of linear constraints for residual
correlations (see Details).
        }
\item{err.constraintV}{
An optional \eqn{L \times 1} matrix with specified values
for linear constraints on residual correlations (see Details).
        }
  \item{glob.conv}{
Global convergence criterion
}
  \item{conv1}{
Convergence criterion for model parameters
}
  \item{pmliter}{
Maximum number of iterations
}
  \item{progress}{
Display progress?
}
\item{use.maxincrement}{Optional logical whether increments in
    slope parameters should be controlled in size in iterations.
    The default is \code{TRUE}.
}
  \item{object}{
Object of class \code{rasch.pml}
}
  \item{\dots}{
Further arguments to be passed
}
}
\details{
The probit item response model can be estimated with this function:
\deqn{P(X_{pi}=1|\theta_p )=\Phi( a_i  \theta_p - b_i ) \quad, \quad
    \theta_p \sim N ( 0, \sigma^2 )}{%
P(X_pi=1|\theta_p )=\Phi( a_i  \theta_p - b_i ), \theta_p ~ N ( 0, \sigma^2 )
    }
where \eqn{\Phi} denotes the normal distribution function. This model can
also be expressed as a latent variable model which assumes
a latent response tendency \eqn{X_{pi}^\ast}{X_pi*} which is equal to
1 if \eqn{X_{pi}> - b_i} and otherwise zero. If \eqn{\epsilon_{pi}} is
standard normally distributed, then
\deqn{X_{pi}^{\ast}=a_i \theta_p - b_i + \epsilon_{pi} }{%
 X_pi*=a_i \theta_p - b_i + \epsilon_{pi}
            }
An arbitrary pattern of residual correlations between
\eqn{\epsilon_{pi}} and \eqn{\epsilon_{pj}} for item pairs \eqn{i}
and \eqn{j} can be imposed using the \code{error.corr} argument.

Linear constraints \eqn{Me=v} on residual correlations
\eqn{e=Cov( \epsilon_{pi}, \epsilon_{pj})_{ij}} (in a vectorized form) can be
specified using the arguments \code{err.constraintM} (matrix \eqn{M})
and \code{err.constraintV} (vector \eqn{v}). The estimation
is described in Neuhaus (1996).

For the pseudo likelihood information criterion (PLIC)
see Stanford and Raftery (2002).
}
\value{
A list with following entries:
\item{item}{Data frame with estimated item parameters}
\item{iter}{Number of iterations}
\item{deviance}{Pseudolikelihood multiplied by minus 2}
\item{b}{Estimated item difficulties}
\item{sigma}{Estimated standard deviation}
\item{dat}{Original dataset}
\item{ic}{Data frame with information criteria (sample size,
number of estimated parameters, pseudolikelihood
information criterion \code{PLIC})}
\item{link}{Used link function (only probit is permitted)}
\item{itempairs}{Estimated statistics of item pairs}
\item{error.corr}{Estimated error correlation matrix}
\item{eps.corr}{
    Vectorized error correlation matrix
        }
\item{omega.rel}{
    Reliability of the sum score according to Green and Yang (2009).
    If some item pairs are excluded in the estimation, the residual
    correlation for these item pairs is assumed to be zero.
        }

\item{\dots}{}
}

\references{
Green, S. B., & Yang, Y. (2009). Reliability of summed item
scores using structural equation modeling: An alternative to
coefficient alpha. \emph{Psychometrika, 74}, 155-167.

Neuhaus, W. (1996). Optimal estimation under
linear constraints. \emph{Astin Bulletin, 26}, 233-245.

Renard, D., Molenberghs, G., & Geys, H. (2004). A pairwise likelihood
approach to estimation in multilevel probit models. \emph{Computational Statistics
& Data Analysis, 44}, 649-667.

Stanford, D. C., & Raftery, A. E. (2002).
Approximate Bayes factors for image segmentation:
The pseudolikelihood information criterion (PLIC).
\emph{IEEE Transactions on Pattern Analysis and
Machine Intelligence, 24}, 1517-1520.
}




%\author{
%Alexander Robitzsch
%}


\note{
This function needs the \pkg{combinat} library.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
Get a summary of \code{rasch.pml3} with \code{\link{summary.rasch.pml}}.

For simulation of locally dependent items see \code{\link{sim.rasch.dep}}.

For pairwise conditional likelihood estimation see \code{\link{rasch.pairwise}}
or \code{\link{rasch.pairwise.itemcluster}}.

For an assessment of global model fit see \code{\link{modelfit.sirt}}.
}

%% print % operator in example by using \%
%% see the tam.mml manual
%% a <- E \%*\% lambda


\examples{
#############################################################################
# EXAMPLE 1: Reading data set
#############################################################################

data(data.read)
dat <- data.read

#******
# Model 1: Rasch model with PML estimation
mod1 <- sirt::rasch.pml3( dat )
summary(mod1)

#******
# Model 2: Excluding item pairs with local dependence
#          from bivariate composite likelihood
itemcluster <- rep( 1:3, each=4)
mod2 <- sirt::rasch.pml3( dat, itemcluster=itemcluster )
summary(mod2)

\dontrun{
#*****
# Model 3: Modelling error correlations:
#          joint residual correlations for each itemcluster
error.corr <- diag(1,ncol(dat))
for ( ii in 1:3){
    ind.ii <- which( itemcluster==ii )
    error.corr[ ind.ii, ind.ii ] <- ii
        }
# estimate the model with error correlations
mod3 <- sirt::rasch.pml3( dat, error.corr=error.corr )
summary(mod3)

#****
# Model 4: model separate residual correlations
I <- ncol(error.corr)
error.corr1 <- matrix( 1:(I*I), ncol=I )
error.corr <- error.corr1 * ( error.corr > 0 )
# estimate the model with error correlations
mod4 <- sirt::rasch.pml3( dat, error.corr=error.corr )
summary(mod4)

#****
# Model 5:  assume equal item difficulties:
# b_1=b_7 and b_2=b_12
# fix item difficulty of the 6th item to .1
est.b <- 1:I
est.b[7] <- 1; est.b[12] <- 2 ; est.b[6] <- 0
b.init <- rep( 0, I ) ; b.init[6] <- .1
mod5 <- sirt::rasch.pml3( dat, est.b=est.b, b.init=b.init)
summary(mod5)

#****
# Model 6: estimate three item slope groups
est.a <- rep(1:3, each=4 )
mod6 <- sirt::rasch.pml3( dat, est.a=est.a, est.sigma=0)
summary(mod6)

#############################################################################
# EXAMPLE 2: PISA reading
#############################################################################

data(data.pisaRead)
dat <- data.pisaRead$data

# select items
dat <- dat[, substring(colnames(dat),1,1)=="R" ]

#******
# Model 1: Rasch model with PML estimation
mod1 <- sirt::rasch.pml3( as.matrix(dat) )
  ## Trait SD (Logit Link) : 1.419

#******
# Model 2: Model correlations within testlets
error.corr <- diag(1,ncol(dat))
testlets <- paste( data.pisaRead$item$testlet )
itemcluster <- match( testlets, unique(testlets ) )
for ( ii in 1:(length(unique(testlets))) ){
    ind.ii <- which( itemcluster==ii )
    error.corr[ ind.ii, ind.ii ] <- ii
        }
# estimate the model with error correlations
mod2 <- sirt::rasch.pml3( dat, error.corr=error.corr )
  ## Trait SD (Logit Link) : 1.384

#****
# Model 3: model separate residual correlations
I <- ncol(error.corr)
error.corr1 <- matrix( 1:(I*I), ncol=I )
error.corr <- error.corr1 * ( error.corr > 0 )
# estimate the model with error correlations
mod3 <- sirt::rasch.pml3( dat, error.corr=error.corr )
  ## Trait SD (Logit Link) : 1.384

#############################################################################
# EXAMPLE 3: 10 locally independent items
#############################################################################

#**********
# simulate some data
set.seed(554)
N <- 500    # persons
I <- 10        # items
theta <- stats::rnorm(N,sd=1.3 )    # trait SD of 1.3
b <- seq(-2, 2, length=I) # item difficulties

# simulate data from the Rasch model
dat <- sirt::sim.raschtype( theta=theta, b=b )

# estimation with rasch.pml and probit link
mod1 <- sirt::rasch.pml3( dat )
summary(mod1)

# estimation with rasch.mml2 function
mod2 <- sirt::rasch.mml2( dat )

# estimate item parameters for groups with five item parameters each
est.b <- rep( 1:(I/2), each=2 )
mod3 <- sirt::rasch.pml3( dat, est.b=est.b )
summary(mod3)

# compare parameter estimates
summary(mod1)
summary(mod2)
summary(mod3)

#############################################################################
# EXAMPLE 4: 11 items and 2 item clusters with 2 and 3 items
#############################################################################

set.seed(5698)
I <- 11                             # number of items
n <- 5000                           # number of persons
b <- seq(-2,2, len=I)               # item difficulties
theta <- stats::rnorm( n, sd=1 ) # person abilities
# itemcluster
itemcluster <- rep(0,I)
itemcluster[c(3,5)] <- 1
itemcluster[c(2,4,9)] <- 2
# residual correlations
rho <- c( .7, .5 )

# simulate data (under the logit link)
dat <- sirt::sim.rasch.dep( theta, b, itemcluster, rho )
colnames(dat) <- paste("I", seq(1,ncol(dat)), sep="")

#***
# Model 1: estimation using the Rasch model (with probit link)
mod1 <- sirt::rasch.pml3( dat )
#***
# Model 2: estimation when pairs of locally dependent items are eliminated
mod2 <- sirt::rasch.pml3( dat, itemcluster=itemcluster)

#***
# Model 3: Positive correlations within testlets
est.corrs <- diag( 1, I )
est.corrs[ c(3,5), c(3,5) ] <- 2
est.corrs[ c(2,4,9), c(2,4,9) ] <- 3
mod3 <- sirt::rasch.pml3( dat, error.corr=est.corrs )

#***
# Model 4: Negative correlations between testlets
est.corrs <- diag( 1, I )
est.corrs[ c(3,5), c(2,4,9) ] <- 2
est.corrs[ c(2,4,9), c(3,5) ] <- 2
mod4 <- sirt::rasch.pml3( dat, error.corr=est.corrs )

#***
# Model 5: sum constraint of zero within and between testlets
est.corrs <- matrix( 1:(I*I),  I, I )
cluster2 <- c(2,4,9)
est.corrs[ setdiff( 1:I, c(cluster2)),  ] <- 0
est.corrs[, setdiff( 1:I, c(cluster2))  ] <- 0
# define an error constraint matrix
itempairs0 <- mod4$itempairs
IP <- nrow(itempairs0)
err.constraint <- matrix( 0, IP, 1 )
err.constraint[ ( itempairs0$item1 \%in\% cluster2 )
       & ( itempairs0$item2 \%in\% cluster2 ), 1 ] <- 1
# set sum of error covariances to 1.2
err.constraintV <- matrix(3*.4,1,1)

mod5 <- sirt::rasch.pml3( dat, error.corr=est.corrs,
         err.constraintM=err.constraint, err.constraintV=err.constraintV)

#****
# Model 6: Constraint on sum of all correlations
est.corrs <- matrix( 1:(I*I),  I, I )
# define an error constraint matrix
itempairs0 <- mod4$itempairs
IP <- nrow(itempairs0)
# define two side conditions
err.constraint <- matrix( 0, IP, 2 )
err.constraintV <- matrix( 0, 2, 1)
# sum of all correlations is zero
err.constraint[, 1 ] <- 1
err.constraintV[1,1] <- 0
# sum of items cluster c(1,2,3) is 0
cluster2 <- c(1,2,3)
err.constraint[ ( itempairs0$item1 \%in\%  cluster2 )
       & ( itempairs0$item2 \%in\% cluster2 ), 2 ] <- 1
err.constraintV[2,1] <- 0

mod6 <- sirt::rasch.pml3( dat, error.corr=est.corrs,
    err.constraintM=err.constraint,  err.constraintV=err.constraintV)
summary(mod6)

#############################################################################
# EXAMPLE 5: 10 Items: Cluster 1 -> Items 1,2
#         Cluster 2 -> Items 3,4,5;   Cluster 3 -> Items 7,8,9
#############################################################################

set.seed(7650)
I <- 10                             # number of items
n <- 5000                           # number of persons
b <- seq(-2,2, len=I)               # item difficulties
bsamp <- b <- sample(b)             # sample item difficulties
theta <- stats::rnorm( n, sd=1 ) # person abilities
# define itemcluster
itemcluster <- rep(0,I)
itemcluster[ 1:2 ] <- 1
itemcluster[ 3:5 ] <- 2
itemcluster[ 7:9 ] <- 3
# define residual correlations
rho <- c( .55, .35, .45)

# simulate data
dat <- sirt::sim.rasch.dep( theta, b, itemcluster, rho )
colnames(dat) <- paste("I", seq(1,ncol(dat)), sep="")

#***
# Model 1: residual correlation (equal within item clusters)
# define a matrix of integers for estimating error correlations
error.corr <- diag(1,ncol(dat))
for ( ii in 1:3){
    ind.ii <- which( itemcluster==ii )
    error.corr[ ind.ii, ind.ii ] <- ii
        }
# estimate the model
mod1 <- sirt::rasch.pml3( dat, error.corr=error.corr )

#***
# Model 2: residual correlation (different within item clusters)
# define again a matrix of integers for estimating error correlations
error.corr <- diag(1,ncol(dat))
for ( ii in 1:3){
    ind.ii <- which( itemcluster==ii )
    error.corr[ ind.ii, ind.ii ] <- ii
        }
I <- ncol(error.corr)
error.corr1 <- matrix( 1:(I*I), ncol=I )
error.corr <- error.corr1 * ( error.corr > 0 )
# estimate the model
mod2 <- sirt::rasch.pml3( dat, error.corr=error.corr )

#***
# Model 3: eliminate item pairs within itemclusters for PML estimation
mod3 <- sirt::rasch.pml3( dat, itemcluster=itemcluster )

#***
# Model 4: Rasch model ignoring dependency
mod4 <- sirt::rasch.pml3( dat )

# compare different models
summary(mod1)
summary(mod2)
summary(mod3)
summary(mod4)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Pairwise marginal maximum likelihood (PMML)}
%% \keyword{Local dependence}
%% \keyword{summary}

%% File Name: data.ratings1.Rd
%% File Version: 0.21

\name{data.ratings}
\alias{data.ratings}
\alias{data.ratings1}
\alias{data.ratings2}
\alias{data.ratings3}


\docType{data}
\title{
Rating Datasets
}
\description{
Some rating datasets.
}
\usage{
data(data.ratings1)
data(data.ratings2)
data(data.ratings3)
}



\format{

\itemize{
\item Dataset \code{data.ratings1}: \cr

Data frame with 274 observations containing 5 criteria (\code{k1}, ..., \code{k5}),
135 students and 7 raters.

\code{'data.frame':   274 obs. of  7 variables:} \cr
\code{ $ idstud: int  100020106 100020106 100070101 100070101 100100109  ...} \cr
\code{ $ rater : Factor w/ 16 levels "db01","db02",..: 3 15 5 10 2 1 5 4 1 5 ...} \cr
\code{ $ k1    : int  1 1 0 1 2 0 1 3 0 0 ...} \cr
\code{ $ k2    : int  1 1 1 1 1 0 0 3 0 0 ...} \cr
\code{ $ k3    : int  1 1 1 1 2 0 0 3 1 0 ...} \cr
\code{ $ k4    : int  1 1 1 2 1 0 0 2 0 1 ...} \cr
\code{ $ k5    : int  2 2 1 2 0 1 0 3 1 0 ...} \cr

Data from a 2009 Austrian survey of national educational
standards for 8th graders in German language writing.
Variables \code{k1} to \code{k5} denote several rating
criteria of writing competency. \cr

\item Dataset \code{data.ratings2}: \cr

Data frame with 615 observations containing 5 criteria (\code{k1}, ..., \code{k5}),
178 students and 16 raters.

\code{'data.frame':   615 obs. of  7 variables:} \cr
\code{ $ idstud: num  1001 1001 1002 1002 1003 ...} \cr
\code{ $ rater : chr  "R03" "R15" "R05" "R10" ...} \cr
\code{ $ k1    : int  1 1 0 1 2 0 1 3 3 0 ...} \cr
\code{ $ k2    : int  1 1 1 1 1 0 0 3 3 0 ...} \cr
\code{ $ k3    : int  1 1 1 1 2 0 0 3 3 1 ...} \cr
\code{ $ k4    : int  1 1 1 2 1 0 0 2 2 0 ...} \cr
\code{ $ k5    : int  2 2 1 2 0 1 0 3 2 1 ...} \cr

\item Dataset \code{data.ratings3}: \cr

Data frame with 3169 observations containing 4 criteria (\code{crit2}, ..., \code{crit6}),
561 students and 52 raters.

\code{'data.frame':   3169 obs. of  6 variables:} \cr
\code{ $ idstud: num  10001 10001 10002 10002 10003 ...} \cr
\code{ $ rater : num  840 838 842 808 830 845 813 849 809 802 ...} \cr
\code{ $ crit2 : int  1 3 3 1 2 2 2 2 3 3 ...} \cr
\code{ $ crit3 : int  2 2 2 2 2 2 2 2 3 3 ...} \cr
\code{ $ crit4 : int  1 2 2 2 1 1 1 2 2 2 ...} \cr
\code{ $ crit6 : num  4 4 4 3 4 4 4 4 4 4 ...} \cr


}
}

%\source{
%xxx
%}






%\examples{
%# include an example of rating data here
%}

%% \keyword{datasets}

%% File Name: linking.robust.Rd
%% File Version: 0.273

\name{linking.robust}
\alias{linking.robust}
\alias{summary.linking.robust}
\alias{plot.linking.robust}


%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Robust Linking of Item Intercepts
}

\description{
This function implements a robust alternative of mean-mean linking which
employs trimmed means instead of means.
The linking constant is calculated for varying trimming parameters \eqn{k}.
The treatment of differential item functioning as outliers and application of
robust statistics is discussed in Magis and De Boeck (2011, 2012).
}

\usage{
linking.robust(itempars)

\method{summary}{linking.robust}(object,...)

\method{plot}{linking.robust}(x, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{itempars}{
Data frame of item parameters (item intercepts). The first column
contains the item label, the 2nd and 3rd columns item parameters of
two studies.
}
\item{object}{Object of class \code{linking.robust}}
\item{x}{Object of class \code{linking.robust}}
\item{\dots}{Further arguments to be passed}
}



\value{
A list with following entries

\item{ind.kopt}{Index for optimal scale parameter}
\item{kopt}{Optimal scale parameter}
\item{meanpars.kopt}{Linking constant for optimal scale parameter}
\item{se.kopt}{Standard error for linking constant obtained with optimal scale parameter}
\item{meanpars}{Linking constant dependent on the scale parameter}
\item{se}{Standard error of the linking constant dependent on the scale parameter}
\item{sd}{DIF standard deviation (non-robust estimate)}
\item{mad}{DIF standard deviation (robust estimate using the MAD measure)}
\item{pars}{Original item parameters}
\item{k.robust}{Used vector of scale parameters}
\item{I}{Number of items}
\item{itempars}{Used data frame of item parameters}
}

\references{
Magis, D., & De Boeck, P. (2011). Identification of differential item functioning in
multiple-group settings: A multivariate outlier detection approach.
\emph{Multivariate Behavioral Research, 46}(5), 733-755.
\doi{10.1080/00273171.2011.606757}

Magis, D., & De Boeck, P. (2012). A robust outlier approach to prevent type I error
inflation in differential item functioning.
\emph{Educational and Psychological Measurement, 72}(2), 291-311.
\doi{10.1177/0013164411416975}
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~


\seealso{
Other functions for linking: \code{\link{linking.haberman}},
\code{\link{equating.rasch}}

See also the \pkg{plink} package.
}
%% ~~objects to See Also as \code{\link{help}}, ~~~

\examples{
#############################################################################
# EXAMPLE 1: Linking data.si03
#############################################################################

data(data.si03)
res1 <- sirt::linking.robust( itempars=data.si03 )
summary(res1)
  ##   Number of items=27
  ##   Optimal trimming parameter k=8 |  non-robust parameter k=0
  ##   Linking constant=-0.0345 |  non-robust estimate=-0.056
  ##   Standard error=0.0186 |  non-robust estimate=0.027
  ##   DIF SD: MAD=0.0771 (robust) | SD=0.1405 (non-robust)
plot(res1)

\dontrun{
#############################################################################
# EXAMPLE 2: Linking PISA item parameters data.pisaPars
#############################################################################

data(data.pisaPars)

# Linking with items
res2 <- sirt::linking.robust( data.pisaPars[, c(1,3,4)] )
summary(res2)
  ##   Optimal trimming parameter k=0 |  non-robust parameter k=0
  ##   Linking constant=-0.0883 |  non-robust estimate=-0.0883
  ##   Standard error=0.0297 |  non-robust estimate=0.0297
  ##   DIF SD: MAD=0.1824 (robust) | SD=0.1487 (non-robust)
##  -> no trimming is necessary for reducing the standard error
plot(res2)

#############################################################################
# EXAMPLE 3: Linking with simulated item parameters containing outliers
#############################################################################

# simulate some parameters
I <- 38
set.seed(18785)
itempars <- data.frame("item"=paste0("I",1:I) )
itempars$study1 <- stats::rnorm( I, mean=.3, sd=1.4 )
# simulate DIF effects plus some outliers
bdif <- stats::rnorm(I,mean=.4,sd=.09)+( stats::runif(I)>.9 )* rep( 1*c(-1,1)+.4, each=I/2 )
itempars$study2 <- itempars$study1 + bdif

# robust linking
res <- sirt::linking.robust( itempars )
summary(res)
  ##   Number of items=38
  ##   Optimal trimming parameter k=12 |  non-robust parameter k=0
  ##   Linking constant=-0.4285 |  non-robust estimate=-0.5727
  ##   Standard error=0.0218 |  non-robust estimate=0.0913
  ##   DIF SD: MAD=0.1186 (robust) | SD=0.5628 (non-robust)
## -> substantial differences of estimated linking constants in this case of
##    deviations from normality of item parameters
plot(res)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Linking}
%% \keyword{Robust linking}% __ONLY ONE__ keyword per line
%% \keyword{summary}
%% \keyword{plot}

%% File Name: data.pisaRead.Rd
%% File Version: 0.13

\name{data.pisaRead}
\alias{data.pisaRead}
\docType{data}
\title{
Dataset PISA Reading
}
\description{
This is an example PISA dataset of reading items from the PISA
2009 study of students from Austria. The
dataset contains 623 students who worked on the 12 reading items
from item cluster R7.
}
\usage{
data(data.pisaRead)
}
\format{
The dataset is a list. The list element \code{data}
contains the dataset with the demographical variables
student ID (\code{idstud}), school ID (\code{idschool}), a dummy variable for female
students (\code{female}), socioeconomic status (\code{hisei})
and migration background (\code{migra}). The remaining
variables (starting with \code{R} in the name) are
the reading items. \cr
The item metadata are included in the list element
\code{item} which contains item name (\code{item}),
testlet label (\code{testlet}), item format (\code{ItemFormat}),
text type (\code{TextType}) and text aspect (\code{Aspect}).

The format is:

\code{List of 2} \cr
\code{ $ data:'data.frame':} \cr
\code{  ..$ idstud  : num [1:623] 9e+10 9e+10 9e+10 9e+10 9e+10 ...} \cr
\code{  ..$ idschool: int [1:623] 900003 900003 900003 900003 ...} \cr
\code{  ..$ female  : int [1:623] 1 0 1 0 0 0 1 0 1 0 ...} \cr
\code{  ..$ hisei   : num [1:623] -1.16 -0.671 1.286 0.185 1.225 ...} \cr
\code{  ..$ migra   : int [1:623] 0 0 0 0 0 0 0 0 0 0 ...} \cr
\code{  ..$ R432Q01 : int [1:623] 1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{  ..$ R432Q05 : int [1:623] 1 1 1 1 1 0 1 1 1 0 ...} \cr
\code{  ..$ R432Q06 : int [1:623] 0 0 0 0 0 0 0 0 0 0 ...} \cr
\code{  ..$ R456Q01 : int [1:623] 1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{  ..$ R456Q02 : int [1:623] 1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{  ..$ R456Q06 : int [1:623] 1 1 1 1 1 1 0 0 1 1 ...} \cr
\code{  ..$ R460Q01 : int [1:623] 1 1 0 0 0 0 0 1 1 1 ...} \cr
\code{  ..$ R460Q05 : int [1:623] 1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{  ..$ R460Q06 : int [1:623] 0 1 1 1 1 1 0 0 1 1 ...} \cr
\code{  ..$ R466Q02 : int [1:623] 0 1 0 1 1 0 1 0 0 1 ...} \cr
\code{  ..$ R466Q03 : int [1:623] 0 0 0 1 0 0 0 1 0 1 ...} \cr
\code{  ..$ R466Q06 : int [1:623] 0 1 1 1 1 1 0 1 1 1 ...} \cr
\code{ $ item:'data.frame':} \cr
\code{  ..$ item      : Factor w/ 12 levels "R432Q01","R432Q05",..: 1 2 3 4  ...} \cr
\code{  ..$ testlet   : Factor w/ 4 levels "R432","R456",..: 1 1 1 2  ...} \cr
\code{  ..$ ItemFormat: Factor w/ 2 levels "CR","MC": 1 1 2 2 1 1 1 2 2 1 ...} \cr
\code{  ..$ TextType  : Factor w/ 3 levels "Argumentation",..: 1 1 1 3  ...} \cr
\code{  ..$ Aspect    : Factor w/ 3 levels "Access_and_retrieve",..: 2 3 2 1 ...} \cr
}




%% \keyword{datasets}

%% File Name: data.befki.Rd
%% File Version: 0.14

\name{data.befki}
\alias{data.befki}
\alias{data.befki_resp}

\docType{data}

\title{
BEFKI Dataset (Schroeders, Schipolowski, & Wilhelm, 2015)
}

\description{
The synthetic dataset is based on the standardization sample of the Berlin
Test of Fluid and Crystallized Intelligence (BEFKI, Wilhelm, Schroeders, &
Schipolowski, 2014). The underlying sample consists of N=11,756 students
from all German federal states (except for the smallest one) and all school
types of the general educational system attending Grades 5 to 12. A
detailed description of the study, the sample, and the measure is given in
Schroeders, Schipolowski, and Wilhelm (2015).
}


\usage{
data(data.befki)
data(data.befki_resp)
}


\format{
\itemize{
\item The dataset \code{data.befki} contains 11756 students, nested within
581 classes.

\code{'data.frame':   11756 obs. of  12 variables:} \cr
\code{ $ idclass: int  1276 1276 1276 1276 1276 1276 1276 1276 1276 1276 ...} \cr
\code{ $ idstud : int  127601 127602 127603 127604 127605 127606 127607 127608 127609 127610 ...} \cr
\code{ $ grade  : int  5 5 5 5 5 5 5 5 5 5 ...} \cr
\code{ $ gym    : int  0 0 0 0 0 0 0 0 0 0 ...} \cr
\code{ $ female : int  0 1 0 0 0 0 1 0 0 0 ...} \cr
\code{ $ age    : num  12.2 11.8 11.5 10.8 10.9 ...} \cr
\code{ $ sci    : num  -3.14 -3.44 -2.62 -2.16 -1.01 -1.91 -1.01 -4.13 -2.16 -3.44 ...} \cr
\code{ $ hum    : num  -1.71 -1.29 -2.29 -2.48 -0.65 -0.92 -1.71 -2.31 -1.99 -2.48 ...} \cr
\code{ $ soc    : num  -2.87 -3.35 -3.81 -2.35 -1.32 -1.11 -1.68 -2.96 -2.69 -3.35 ...} \cr
\code{ $ gfv    : num  -2.25 -2.19 -2.25 -1.17 -2.19 -3.05 -1.7 -2.19 -3.05 -1.7 ...} \cr
\code{ $ gfn    : num  -2.2 -1.85 -1.85 -1.85 -1.85 -0.27 -1.37 -2.58 -1.85 -3.13 ...} \cr
\code{ $ gff    : num  -0.91 -0.43 -1.17 -1.45 -0.61 -1.78 -1.17 -1.78 -1.78 -3.87 ...} \cr

\item The dataset \code{data.befki_resp} contains response indicators for observed data points
in the dataset \code{data.befki}.

\code{ num [1:11756, 1:12] 1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{ - attr(*, "dimnames")=List of 2} \cr
\code{  ..$ : NULL} \cr
\code{  ..$ : chr [1:12] "idclass" "idstud" "grade" "gym" ...} \cr

}


}

\details{
The procedure for generating this dataset is based on a factorization of the
joint distribution. All variables are simulated from unidimensional conditional
parametric regression models including several interaction and quadratic terms.
The multilevel structure is approximated by including
cluster means as predictors in the regression models.
}

\source{
Synthetic dataset
}


\references{
Schroeders, U., Schipolowski, S., & Wilhelm, O. (2015). Age-related
changes in the mean and covariance structure of fluid and crystallized
intelligence in childhood and adolescence. \emph{Intelligence, 48}, 15-29.
\doi{10.1016/j.intell.2014.10.006}

Wilhelm, O., Schroeders, U., & Schipolowski, S. (2014). \emph{Berliner Test
zur Erfassung fluider und kristalliner Intelligenz fuer die 8. bis 10.
Jahrgangsstufe} [Berlin test of fluid and crystallized intelligence for
grades 8-10]. Goettingen: Hogrefe.
}


%\examples{
%data(data.befki)
%}
%\keyword{datasets}

%% File Name: pgenlogis.Rd
%% File Version: 0.281

\name{pgenlogis}
\alias{pgenlogis}
\alias{genlogis.moments}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Calculation of Probabilities and Moments for the
Generalized Logistic Item Response Model
}
\description{
Calculation of probabilities and moments for the generalized logistic
item response model (Stukel, 1988).
}
\usage{
pgenlogis(x, alpha1=0, alpha2=0)

genlogis.moments(alpha1, alpha2)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
Vector
}
  \item{alpha1}{
Upper tail parameter \eqn{\alpha_1} in the generalized
logistic item response model. The default is 0.
}
  \item{alpha2}{
Lower tail parameter \eqn{\alpha_2} parameter in the generalized
logistic item response model. The default is 0.
}
}
\details{
The class of generalized logistic link functions contain
the most important link functions using the specifications (Stukel, 1988):

\itemize{
\item logistic link function \eqn{L}:
\deqn{ L(x) \approx G_{ ( \alpha_1=0, \alpha_2=0)}[ x ] }

\item probit link function \eqn{\Phi}:
\deqn{ \Phi(x) \approx G_{ ( \alpha_1=0.165, \alpha_2=0.165)}[ 1.47 x ] }


\item loglog link function \eqn{H}:
\deqn{ H(x) \approx G_{ (\alpha_1=-0.037, \alpha_2=0.62)}[
-0.39+1.20x-0.007x^2] }

\item cloglog link function \eqn{H}:
\deqn{ H(x) \approx G_{ ( \alpha_1=0.62, \alpha_2=-0.037)}[
0.54+1.64x+0.28x^2+0.046x^3] }

}
}

% # loglog link
% y <- pgenlogis( -.39 + 1.20*x -.007*x^2, alpha1=-.037, alpha2=.62 )

% # cloglog link
% y <- pgenlogis( .54+1.64*x +.28*x^2 + .046*x^3, alpha1=.062, alpha2=-.037 )

\value{
Vector of probabilities or moments
}

\references{
Stukel, T. A. (1988). Generalized logistic models.
\emph{Journal of the American Statistical Association, 83}(402), 426-431.
\doi{10.1080/01621459.1988.10478613}
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{
sirt::pgenlogis( x=c(-.3, 0, .25, 1 ), alpha1=0, alpha2=.6 )
  ##   [1] 0.4185580 0.5000000 0.5621765 0.7310586

####################################################################
# compare link functions
x <- seq( -3,3, .1 )

#***
# logistic link
y <- sirt::pgenlogis( x, alpha1=0, alpha2=0 )
plot( x, stats::plogis(x), type="l", main="Logistic Link", lwd=2)
points( x, y, pch=1, col=2 )

#***
# probit link
round( sirt::genlogis.moments( alpha1=.165, alpha2=.165 ), 3 )
  ##       M    SD   Var
  ##   0.000 1.472 2.167
# SD of generalized logistic link function is 1.472
y <- sirt::pgenlogis( x * 1.47, alpha1=.165, alpha2=.165 )
plot( x, stats::pnorm(x), type="l", main="Probit Link", lwd=2)
points( x, y, pch=1, col=2 )

#***
# loglog link
y <- sirt::pgenlogis( -.39 + 1.20*x -.007*x^2, alpha1=-.037, alpha2=.62 )
plot( x, exp( - exp( -x ) ), type="l", main="Loglog Link", lwd=2,
    ylab="loglog(x)=exp(-exp(-x))" )
points( x, y, pch=17, col=2 )

#***
# cloglog link
y <- sirt::pgenlogis( .54+1.64*x +.28*x^2 + .046*x^3, alpha1=.062, alpha2=-.037 )
plot( x, 1-exp( - exp(x) ), type="l", main="Cloglog Link", lwd=2,
    ylab="loglog(x)=1-exp(-exp(x))" )
points( x, y, pch=17, col=2 )
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Generalized logistic item response model}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: parmsummary_extend.Rd
%% File Version: 0.13

\name{parmsummary_extend}
\alias{parmsummary_extend}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Includes Confidence Interval in Parameter Summary Table
}
\description{
Includes confidence interval in parameter summary table.
}
\usage{
parmsummary_extend(dfr, level=.95, est_label="est", se_label="se",
      df_label="df")
}
%- maybe also 'usage' for other objects documented here.


\arguments{
  \item{dfr}{
Data frame containing parameter summary
}
  \item{level}{
Significance level
}
  \item{est_label}{
Label for parameter estimate
}
  \item{se_label}{
Label for standard error
}
  \item{df_label}{
Label for degrees of freedom
}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}


\value{
Extended parameter summary table
}

%\references{
%% ~put references to the literature/web site here ~
%}
%\author{
%%  ~~who you are~~
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link[stats:confint]{stats::confint}}
}


\examples{
#############################################################################
## EXAMPLE 1: Toy example parameter summary table
#############################################################################

dfr <- data.frame( "parm"=c("b0", "b1" ), "est"=c(0.1, 1.3 ),
                "se"=c(.21, .32) )
print( sirt::parmsummary_extend(dfr), digits=4 )
  ##    parm est   se      t         p lower95 upper95
  ##  1   b0 0.1 0.21 0.4762 6.339e-01 -0.3116  0.5116
  ##  2   b1 1.3 0.32 4.0625 4.855e-05  0.6728  1.9272
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.lsem.Rd
%% File Version: 0.094

\name{data.lsem}
\alias{data.lsem01}
\alias{data.lsem02}
\alias{data.lsem03}

\docType{data}

\title{
Datasets for Local Structural Equation Models / Moderated Factor Analysis
}


\description{
Datasets for local structural equation models or moderated factor analysis.
}


\usage{
data(data.lsem01)
data(data.lsem02)
data(data.lsem03)
}



\format{

\itemize{
\item The dataset \code{data.lsem01} has the following structure

\code{'data.frame':   989 obs. of  6 variables:} \cr
\code{ $ age: num  4 4 4 4 4 4 4 4 4 4 ...} \cr
\code{ $ v1 : num  1.83 2.38 1.85 4.53 -0.04 4.35 2.38 1.83 4.81 2.82 ...} \cr
\code{ $ v2 : num  6.06 9.08 7.41 8.24 6.18 7.4 6.54 4.28 6.43 7.6 ...} \cr
\code{ $ v3 : num  1.42 3.05 6.42 -1.05 -1.79 4.06 -0.17 -2.64 0.84 6.42 ...} \cr
\code{ $ v4 : num  3.84 4.24 3.24 3.36 2.31 6.07 4 5.93 4.4 3.49 ...} \cr
\code{ $ v5 : num  7.84 7.51 6.62 8.02 7.12 7.99 7.25 7.62 7.66 7.03 ...} \cr

\item The dataset \code{data.lsem02} is a slightly perturbed dataset of the
Woodcock-Johnson  III (WJ-III) Tests of Cognitive Abilities used in Hildebrandt et al.
(2016) and has the following structure

\code{'data.frame':   1129 obs. of  8 variables:} \cr
\code{ $ age : int  4 4 4 4 4 4 4 4 4 4 ...} \cr
\code{ $ gcw : num  -3.53 -3.73 -3.77 -3.84 -4.26 -4.6 -3.66 -4.31 -4.46 -3.64 ...} \cr
\code{ $ gvw : num  -1.98 -1.35 -1.66 -3.24 -1.17 -2.78 -2.97 -3.88 -3.22 -0.68 ...} \cr
\code{ $ gfw : num  -2.49 -2.41 -4.48 -4.17 -4.43 -5.06 -3.94 -3.66 -3.7 -2.74 ...} \cr
\code{ $ gsw : num  -4.85 -5.05 -5.66 -4.3 -5.23 -5.63 -4.91 -5.75 -6.29 -5.47 ...} \cr
\code{ $ gsmw: num  -2.99 -1.13 -4.21 -3.59 -3.79 -4.77 -2.98 -4.48 -2.99 -3.83 ...} \cr
\code{ $ glrw: num  -2.49 -2.91 -3.45 -2.91 -3.31 -3.78 -3.5 -3.96 -2.97 -3.14 ...} \cr
\code{ $ gaw : num  -3.22 -3.77 -3.54 -3.6 -3.22 -3.5 -1.27 -2.08 -2.23 -3.25 ...} \cr

\item The dataset \code{data.lsem03} is a synthetic dataset of the SON-R application
used in Hueluer et al. (2011) has the following structure

\code{'data.frame':   1027 obs. of  10 variables:} \cr
\code{ $ id       : num  10001 10002 10003 10004 10005 ...} \cr
\code{ $ female   : int  0 0 0 0 0 0 0 0 0 0 ...} \cr
\code{ $ age      : num  2.62 2.65 2.66 2.67 2.68 2.68 2.68 2.69 2.71 2.71 ...} \cr
\code{ $ age_group: int  1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{ $ p1       : num  -1.98 -1.98 -1.67 -2.29 -1.67 -1.98 -2.29 -1.98 -2.6 -1.67 ...} \cr
\code{ $ p2       : num  -1.51 -1.51 -0.55 -1.84 -1.51 -1.84 -2.16 -1.84 -2.48 -1.84 ...} \cr
\code{ $ p3       : num  -1.4 -2.31 -1.1 -2 -1.4 -1.7 -2.31 -1.4 -2.31 -0.79 ...} \cr
\code{ $ r1       : num  -1.46 -1.14 -0.49 -2.11 -1.46 -1.46 -2.11 -1.46 -2.75 -1.78 ...} \cr
\code{ $ r2       : num  -2.67 -1.74 0.74 -1.74 -0.81 -1.43 -2.05 -1.43 -1.74 -1.12 ...} \cr
\code{ $ r3       : num  -1.64 -1.64 -1.64 -0.9 -1.27 -3.11 -2.74 -1.64 -2.37 -1.27 ...} \cr

The subtests  Mosaics (\code{p1}), Puzzles (\code{p1}), and Patterns (\code{p3})
constitute the performance subscale;
the subtests Categories (\code{r1}), Analogies (\code{r2}), and
Situations (\code{r3}) constitute the reasoning subscale.
}
}


%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}
%\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%}
\references{
Hildebrandt, A., Luedtke, O., Robitzsch, A., Sommer, C., &
Wilhelm, O. (2016). Exploring factor model parameters across continuous variables
with local structural equation models.
\emph{Multivariate Behavioral Research, 51}(2-3), 257-278.
\doi{10.1080/00273171.2016.1142856}

Hueluer, G., Wilhelm, O., & Robitzsch, A. (2011). Intelligence differentiation in
early childhood. \emph{Journal of Individual Differences, 32}(3), 170-179.
\doi{10.1027/1614-0001/a000049}
}

%\examples{
%data(data.lsem01)
%}

%% \keyword{datasets}

%% File Name: rasch.copula.Rd
%% File Version: 1.739

\name{rasch.copula2}
\alias{rasch.copula2}
\alias{rasch.copula3}
\alias{summary.rasch.copula2}
\alias{summary.rasch.copula3}
\alias{anova.rasch.copula2}
\alias{anova.rasch.copula3}
\alias{logLik.rasch.copula2}
\alias{logLik.rasch.copula3}
\alias{IRT.likelihood.rasch.copula2}
\alias{IRT.likelihood.rasch.copula3}
\alias{IRT.posterior.rasch.copula2}
\alias{IRT.posterior.rasch.copula3}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Multidimensional IRT Copula Model
}
\description{
This function handles local dependence by specifying
copulas for residuals in multidimensional item response
models for dichotomous item responses
(Braeken, 2011; Braeken, Tuerlinckx & de Boeck, 2007;
Schroeders, Robitzsch & Schipolowski, 2014).
Estimation is allowed for item difficulties, item slopes and
a generalized logistic link function (Stukel, 1988).

The function \code{rasch.copula3} allows the estimation of multidimensional
models while \code{rasch.copula2} only handles unidimensional models.
}
\usage{
rasch.copula2(dat, itemcluster, weights=NULL, copula.type="bound.mixt",
    progress=TRUE, mmliter=1000, delta=NULL,
    theta.k=seq(-4, 4, len=21), alpha1=0, alpha2=0,
    numdiff.parm=1e-06,  est.b=seq(1, ncol(dat)),
    est.a=rep(1, ncol(dat)), est.delta=NULL, b.init=NULL, a.init=NULL,
    est.alpha=FALSE, glob.conv=0.0001, alpha.conv=1e-04, conv1=0.001,
    dev.crit=.2, increment.factor=1.01)

rasch.copula3(dat, itemcluster, dims=NULL, copula.type="bound.mixt",
    progress=TRUE, mmliter=1000, delta=NULL,
    theta.k=seq(-4, 4, len=21), alpha1=0, alpha2=0,
    numdiff.parm=1e-06,  est.b=seq(1, ncol(dat)),
    est.a=rep(1, ncol(dat)), est.delta=NULL, b.init=NULL, a.init=NULL,
    est.alpha=FALSE, glob.conv=0.0001, alpha.conv=1e-04, conv1=0.001,
    dev.crit=.2, rho.init=.5, increment.factor=1.01)

\method{summary}{rasch.copula2}(object, file=NULL, digits=3, ...)
\method{summary}{rasch.copula3}(object, file=NULL, digits=3, ...)

\method{anova}{rasch.copula2}(object,...)
\method{anova}{rasch.copula3}(object,...)

\method{logLik}{rasch.copula2}(object,...)
\method{logLik}{rasch.copula3}(object,...)

\method{IRT.likelihood}{rasch.copula2}(object,...)
\method{IRT.likelihood}{rasch.copula3}(object,...)

\method{IRT.posterior}{rasch.copula2}(object,...)
\method{IRT.posterior}{rasch.copula3}(object,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
An \eqn{N \times I} data frame. Cases with only missing responses
are removed from the analysis.
}
  \item{itemcluster}{
An integer vector of length \eqn{I} (number of items). Items with
the same integers define a joint item cluster of (positively) locally
dependent items. Values of zero indicate that the corresponding item
is not included in any item cluster of dependent responses.
}
\item{weights}{Optional vector of sampling weights}
\item{dims}{
A vector indicating to which dimension an item is allocated.
The default is that all items load on the first dimension.
}
\item{copula.type}{
A character or a vector containing one of the following copula
types: \code{bound.mixt} (boundary mixture copula),
\code{cook.johnson} (Cook-Johnson copula) or \code{frank} (Frank copula)
(see Braeken, 2011).
The vector \code{copula.type} must match the number of different
itemclusters. For every itemcluster, a different copula type
may be specified (see Examples).
}  %% elaborate this further in the Details section
  \item{progress}{
Print progress? Default is \code{TRUE}.
}
  \item{mmliter}{
Maximum number of iterations.
}
  \item{delta}{
An optional vector of starting values for the dependency parameter \code{delta}.
}
  \item{theta.k}{
Discretized trait distribution
}
  \item{alpha1}{
\code{alpha1} parameter in the generalized logistic item response model
(Stukel, 1988). The default is 0 which leads together with \code{alpha2=0}
to the logistic link function.
}
  \item{alpha2}{
\code{alpha2} parameter in the generalized logistic item response model
}
  \item{numdiff.parm}{
Parameter for numerical differentiation
}
  \item{est.b}{
Integer vector of item difficulties to be estimated
}
  \item{est.a}{
Integer vector of item discriminations to be estimated
}
  \item{est.delta}{
Integer vector of length \code{length(itemcluster)}. Nonzero integers
correspond to \code{delta} parameters which are estimated.
Equal integers indicate parameter equality constraints.
}
  \item{b.init}{
Initial \eqn{b} parameters
}
  \item{a.init}{
Initial \eqn{a} parameters
}
  \item{est.alpha}{
Should both alpha parameters be estimated? Default is \code{FALSE}.
}
 \item{glob.conv}{
Convergence criterion for all parameters
}
  \item{alpha.conv}{
Maximal change in alpha parameters for convergence
}
  \item{conv1}{
Maximal change in item parameters for convergence
}
  \item{dev.crit}{
Maximal change in the deviance. Default is \code{.2}.
}
\item{rho.init}{Initial value for off-diagonal elements in correlation matrix}
\item{increment.factor}{A numeric value larger than one which controls the
    size of increments in iterations. To stabilize convergence,
    choose values 1.05 or 1.1 in some situations.}
  \item{object}{
Object of class \code{rasch.copula2} or \code{rasch.copula3}
}
\item{file}{Optional file name for \code{summary} output}
\item{digits}{Number of digits after decimal in \code{summary} output}
  \item{\dots}{
Further arguments to be passed
}
}
%\details{
%% Details about alpha1 and alpha2 for generalized logistic function?
%}
\value{
A list with following entries
\item{N.itemclusters}{Number of item clusters}
\item{item}{Estimated item parameters}
\item{iter}{Number of iterations}
\item{dev}{Deviance}
\item{delta}{Estimated dependency parameters \eqn{\delta}}
\item{b}{Estimated item difficulties}
\item{a}{Estimated item slopes}
\item{mu}{Mean}
\item{sigma}{Standard deviation}
\item{alpha1}{Parameter \eqn{\alpha_1} in the generalized item response model}
\item{alpha2}{Parameter \eqn{\alpha_2} in the generalized item response model}
\item{ic}{Information criteria}
\item{theta.k}{Discretized ability distribution}
\item{pi.k}{Fixed \eqn{\theta} distribution}
\item{deviance}{Deviance}
\item{pattern}{Item response patterns with frequencies and posterior
distribution}
\item{person}{Data frame with person parameters}
\item{datalist}{List of generated data frames during estimation}
\item{EAP.rel}{Reliability of the EAP}
\item{copula.type}{Type of copula}
\item{summary.delta}{Summary for estimated \eqn{\delta} parameters}
\item{f.qk.yi}{Individual posterior}
\item{f.yi.qk}{Individual likelihood}
\item{\dots}{Further values}
}

\references{
Braeken, J. (2011). A boundary mixture approach to violations of conditional
independence. \emph{Psychometrika, 76}(1), 57-76.
\doi{10.1007/s11336-010-9190-4}

Braeken, J., Kuppens, P., De Boeck, P., & Tuerlinckx, F. (2013).
Contextualized personality questionnaires: A case for copulas in structural
equation models for categorical data. \emph{Multivariate Behavioral Research, 48}(6),
845-870.
\doi{10.1080/00273171.2013.827965}

Braeken, J., & Tuerlinckx, F. (2009). Investigating latent constructs with item
response models: A MATLAB IRTm toolbox.
\emph{Behavior Research Methods, 41}(4), 1127-1137.

Braeken, J., Tuerlinckx, F., & De Boeck, P. (2007).
Copula functions for residual dependency. \emph{Psychometrika, 72}(3), 393-411.
\doi{10.1007/s11336-007-9005-4}

Schroeders, U., Robitzsch, A., & Schipolowski, S. (2014). A comparison of different
psychometric approaches to modeling testlet structures: An example with C-tests.
\emph{Journal of Educational Measurement, 51}(4), 400-418.
\doi{10.1111/jedm.12054}

Stukel, T. A. (1988). Generalized logistic models.
\emph{Journal of the American Statistical Association, 83}(402), 426-431.
\doi{10.1080/01621459.1988.10478613}
}

%% Braeken & Tuerlinckx (2009). 10.3758/BRM.41.4.1127   \doi{10.3758/BRM.41.4.1127}

%\author{
%Alexander Robitzsch
%}



%\note{
%The function \code{rasch.copula2} is substantially faster than
%\code{rasch.copula}, but it is in a more experimental status.
%}

%\section{Warning}{
%    The multiple group option only works for \code{rasch.copula},
%    but not for \code{rasch.copula2}. This option will be implemented
%    in the near future.
%                }

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For a summary see \code{\link{summary.rasch.copula2}}.

For simulating locally dependent item responses see \code{\link{sim.rasch.dep}}.

Person parameters estimates are obtained by \code{\link{person.parameter.rasch.copula}}.

See \code{\link{rasch.mml2}} for the generalized logistic link function.

See also Braeken and Tuerlinckx (2009) for alternative (and more expanded)
copula models implemented in the MATLAB software. See
\url{https://ppw.kuleuven.be/okp/software/irtm/}.

See Braeken, Kuppens, De Boeck and Tuerlinckx (2013) for an extension of the
copula modeling approach to polytomous data.
}


\examples{
#############################################################################
# EXAMPLE 1: Reading Data
#############################################################################

data(data.read)
dat <- data.read

# define item clusters
itemcluster <- rep( 1:3, each=4 )

# estimate Copula model
mod1 <- sirt::rasch.copula2( dat=dat, itemcluster=itemcluster)

\dontrun{
# estimate Rasch model
mod2 <- sirt::rasch.copula2( dat=dat, itemcluster=itemcluster,
        delta=rep(0,3), est.delta=rep(0,3) )
summary(mod1)
summary(mod2)

# estimate copula 2PL model
I <- ncol(dat)
mod3 <- sirt::rasch.copula2( dat=dat, itemcluster=itemcluster, est.a=1:I,
                increment.factor=1.05)
summary(mod3)

#############################################################################
# EXAMPLE 2: 11 items nested within 2 item clusters (testlets)
#    with 2 resp. 3 dependent and 6 independent items
#############################################################################

set.seed(5698)
I <- 11                             # number of items
n <- 3000                           # number of persons
b <- seq(-2,2, len=I)               # item difficulties
theta <- stats::rnorm( n, sd=1 ) # person abilities
# define item clusters
itemcluster <- rep(0,I)
itemcluster[ c(3,5 )] <- 1
itemcluster[c(2,4,9)] <- 2
# residual correlations
rho <- c( .7, .5 )

# simulate data
dat <- sirt::sim.rasch.dep( theta, b, itemcluster, rho )
colnames(dat) <- paste("I", seq(1,ncol(dat)), sep="")

# estimate Rasch copula model
mod1 <- sirt::rasch.copula2( dat, itemcluster=itemcluster )
summary(mod1)

# both item clusters have Cook-Johnson copula as dependency
mod1c <- sirt::rasch.copula2( dat, itemcluster=itemcluster,
            copula.type="cook.johnson")
summary(mod1c)

# first item boundary mixture and second item Cook-Johnson copula
mod1d <- sirt::rasch.copula2( dat, itemcluster=itemcluster,
            copula.type=c( "bound.mixt", "cook.johnson" ) )
summary(mod1d)

# compare result with Rasch model estimation in rasch.copula2
# delta must be set to zero
mod2 <- sirt::rasch.copula2( dat, itemcluster=itemcluster, delta=c(0,0),
            est.delta=c(0,0) )
summary(mod2)

#############################################################################
# EXAMPLE 3: 12 items nested within 3 item clusters (testlets)
#   Cluster 1 -> Items 1-4; Cluster 2 -> Items 6-9;  Cluster 3 -> Items 10-12
#############################################################################

set.seed(967)
I <- 12                             # number of items
n <- 450                            # number of persons
b <- seq(-2,2, len=I)               # item difficulties
b <- sample(b)                      # sample item difficulties
theta <- stats::rnorm( n, sd=1 ) # person abilities
# itemcluster
itemcluster <- rep(0,I)
itemcluster[ 1:4 ] <- 1
itemcluster[ 6:9 ] <- 2
itemcluster[ 10:12 ] <- 3
# residual correlations
rho <- c( .35, .25, .30 )

# simulate data
dat <- sirt::sim.rasch.dep( theta, b, itemcluster, rho )
colnames(dat) <- paste("I", seq(1,ncol(dat)), sep="")

# estimate Rasch copula model
mod1 <- sirt::rasch.copula2( dat, itemcluster=itemcluster )
summary(mod1)

# person parameter estimation assuming the Rasch copula model
pmod1 <- sirt::person.parameter.rasch.copula(raschcopula.object=mod1 )

# Rasch model estimation
mod2 <- sirt::rasch.copula2( dat, itemcluster=itemcluster,
             delta=rep(0,3), est.delta=rep(0,3) )
summary(mod1)
summary(mod2)

#############################################################################
# EXAMPLE 4: Two-dimensional copula model
#############################################################################

set.seed(5698)
I <- 9
n <- 1500                           # number of persons
b <- seq(-2,2, len=I)               # item difficulties
theta0 <- stats::rnorm( n, sd=sqrt( .6 ) )

#*** Dimension 1
theta <- theta0 + stats::rnorm( n, sd=sqrt( .4 ) )   # person abilities
# itemcluster
itemcluster <- rep(0,I)
itemcluster[ c(3,5 )] <- 1
itemcluster[c(2,4,9)] <- 2
itemcluster1 <- itemcluster
# residual correlations
rho <- c( .7, .5 )
# simulate data
dat <- sirt::sim.rasch.dep( theta, b, itemcluster, rho )
colnames(dat) <- paste("A", seq(1,ncol(dat)), sep="")
dat1 <- dat
# estimate model of dimension 1
mod0a <- sirt::rasch.copula2( dat1, itemcluster=itemcluster1)
summary(mod0a)

#*** Dimension 2
theta <- theta0 + stats::rnorm( n, sd=sqrt( .8 ) )        # person abilities
# itemcluster
itemcluster <- rep(0,I)
itemcluster[ c(3,7,8 )] <- 1
itemcluster[c(4,6)] <- 2
itemcluster2 <- itemcluster
# residual correlations
rho <- c( .2, .4 )
# simulate data
dat <- sirt::sim.rasch.dep( theta, b, itemcluster, rho )
colnames(dat) <- paste("B", seq(1,ncol(dat)), sep="")
dat2 <- dat
# estimate model of dimension 2
mod0b <- sirt::rasch.copula2( dat2, itemcluster=itemcluster2)
summary(mod0b)

# both dimensions
dat <- cbind( dat1, dat2 )
itemcluster2 <- ifelse( itemcluster2 > 0, itemcluster2 + 2, 0 )
itemcluster <- c( itemcluster1, itemcluster2 )
dims <- rep( 1:2, each=I)

# estimate two-dimensional copula model
mod1 <- sirt::rasch.copula3( dat, itemcluster=itemcluster, dims=dims, est.a=dims,
            theta.k=seq(-5,5,len=15) )
summary(mod1)

#############################################################################
# EXAMPLE 5: Subset of data Example 2
#############################################################################

set.seed(5698)
I <- 11                             # number of items
n <- 3000                           # number of persons
b <- seq(-2,2, len=I)               # item difficulties
theta <- stats::rnorm( n, sd=1.3 )  # person abilities
# define item clusters
itemcluster <- rep(0,I)
itemcluster[ c(3,5)] <- 1
itemcluster[c(2,4,9)] <- 2
# residual correlations
rho <- c( .7, .5 )
# simulate data
dat <- sirt::sim.rasch.dep( theta, b, itemcluster, rho )
colnames(dat) <- paste("I", seq(1,ncol(dat)), sep="")

# select subdataset with only one dependent item cluster
item.sel <- scan( what="character", nlines=1 )
    I1 I6 I7 I8 I10 I11 I3 I5
dat1 <- dat[,item.sel]

#******************
#*** Model 1a: estimate Copula model in sirt
itemcluster <- rep(0,8)
itemcluster[c(7,8)] <- 1
mod1a <- sirt::rasch.copula2( dat3, itemcluster=itemcluster )
summary(mod1a)

#******************
#*** Model 1b: estimate Copula model in mirt
library(mirt)
#*** redefine dataset for estimation in mirt
dat2 <- dat1[, itemcluster==0 ]
dat2 <- as.data.frame(dat2)
# combine items 3 and 5
dat2$C35 <- dat1[,"I3"] + 2*dat1[,"I5"]
table( dat2$C35, paste0( dat1[,"I3"],dat1[,"I5"]) )
#* define mirt model
mirtmodel <- mirt::mirt.model("
      F=1-7
      CONSTRAIN=(1-7,a1)
      " )
#-- Copula function with two dependent items
# define item category function for pseudo-items like C35
P.copula2 <- function(par,Theta, ncat){
     b1 <- par[1]
     b2 <- par[2]
     a1 <- par[3]
     ldelta <- par[4]
     P1 <- stats::plogis( a1*(Theta - b1 ) )
     P2 <- stats::plogis( a1*(Theta - b2 ) )
     Q1 <- 1-P1
     Q2 <- 1-P2
     # define vector-wise minimum function
     minf2 <- function( x1, x2 ){
         ifelse( x1 < x2, x1, x2 )
                                }
     # distribution under independence
     F00 <- Q1*Q2
     F10 <- Q1*Q2 + P1*Q2
     F01 <- Q1*Q2 + Q1*P2
     F11 <- 1+0*Q1
     F_ind <- c(F00,F10,F01,F11)
     # distribution under maximal dependence
     F00 <- minf2(Q1,Q2)
     F10 <- Q2              #=minf2(1,Q2)
     F01 <- Q1              #=minf2(Q1,1)
     F11 <- 1+0*Q1          #=minf2(1,1)
     F_dep <- c(F00,F10,F01,F11)
     # compute mixture distribution
     delta <- stats::plogis(ldelta)
     F_tot <- (1-delta)*F_ind + delta * F_dep
     # recalculate probabilities of mixture distribution
     L1 <- length(Q1)
     v1 <- 1:L1
     F00 <- F_tot[v1]
     F10 <- F_tot[v1+L1]
     F01 <- F_tot[v1+2*L1]
     F11 <- F_tot[v1+3*L1]
     P00 <- F00
     P10 <- F10 - F00
     P01 <- F01 - F00
     P11 <- 1 - F10 - F01 + F00
     prob_tot <- c( P00, P10, P01, P11 )
     return(prob_tot)
        }
# create item
copula2 <- mirt::createItem(name="copula2", par=c(b1=0, b2=0.2, a1=1, ldelta=0),
                est=c(TRUE,TRUE,TRUE,TRUE), P=P.copula2,
                lbound=c(-Inf,-Inf,0,-Inf), ubound=c(Inf,Inf,Inf,Inf) )
# define item types
itemtype <- c( rep("2PL",6), "copula2" )
customItems <- list("copula2"=copula2)
# parameter table
mod.pars <- mirt::mirt(dat2, 1, itemtype=itemtype,
                customItems=customItems, pars='values')
# estimate model
mod1b <- mirt::mirt(dat2, mirtmodel, itemtype=itemtype, customItems=customItems,
                verbose=TRUE, pars=mod.pars,
                technical=list(customTheta=as.matrix(seq(-4,4,len=21)) ) )
# estimated coefficients
cmod <- sirt::mirt.wrapper.coef(mod)$coef

# compare common item discrimination
round( c("sirt"=mod1a$item$a[1], "mirt"=cmod$a1[1] ), 4 )
  ##     sirt   mirt
  ##   1.2845 1.2862
# compare delta parameter
round( c("sirt"=mod1a$item$delta[7], "mirt"=stats::plogis( cmod$ldelta[7] ) ), 4 )
  ##     sirt   mirt
  ##   0.6298 0.6297
# compare thresholds a*b
dfr <- cbind( "sirt"=mod1a$item$thresh,
               "mirt"=c(- cmod$d[-7],cmod$b1[7]*cmod$a1[1], cmod$b2[7]*cmod$a1[1]))
round(dfr,4)
  ##           sirt    mirt
  ##   [1,] -1.9236 -1.9231
  ##   [2,] -0.0565 -0.0562
  ##   [3,]  0.3993  0.3996
  ##   [4,]  0.8058  0.8061
  ##   [5,]  1.5293  1.5295
  ##   [6,]  1.9569  1.9572
  ##   [7,] -1.1414 -1.1404
  ##   [8,] -0.4005 -0.3996
}
}


% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{IRT copula models}
%% \keyword{Local dependence}
%% \keyword{summary}
%% \keyword{logLik}
%% \keyword{anova}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: xxirt_createThetaDistribution.Rd
%% File Version: 0.154

\name{xxirt_createThetaDistribution}
\alias{xxirt_createThetaDistribution}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Creates a User Defined Theta Distribution
}

\description{
Creates a user defined theta distribution.
}

\usage{
xxirt_createThetaDistribution(par, est, P, prior=NULL, prior_par1=NULL,
       prior_par2=NULL, lower=NULL, upper=NULL, X=NULL)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{par}{
Parameter vector with starting values
}
  \item{est}{
Vector of logicals indicating which parameters should be estimated
}
  \item{P}{
Distribution function for \eqn{\bold{\theta}}
}
  \item{prior}{
Prior distribution
}
  \item{prior_par1}{
First parameter of prior distribution
}
  \item{prior_par2}{
Second parameter of prior distribution
}
\item{lower}{Lower bounds for parameters}
\item{upper}{Upper bounds for parameters}
\item{X}{optional matrix of person covariates}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%}
%\references{
%% ~put references to the literature/web site here ~
%}


%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{xxirt}}
}


\examples{
#############################################################################
## EXAMPLE 1: Definition of theta distribution
#############################################################################

#** theta grid
Theta <- matrix( seq(-10,10,length=31), ncol=1 )

#** theta distribution
P_Theta1 <- function( par, Theta, G){
    mu <- par[1]
    sigma <- max( par[2], .01 )
    TP <- nrow(Theta)
    pi_Theta <- matrix( 0, nrow=TP, ncol=G)
    pi1 <- stats::dnorm( Theta[,1], mean=mu, sd=sigma )
    pi1 <- pi1 / sum(pi1)
    pi_Theta[,1] <- pi1
    return(pi_Theta)
                }
#** create distribution class
par_Theta <- c( "mu"=0, "sigma"=1 )
customTheta  <- sirt::xxirt_createThetaDistribution( par=par_Theta,
                       est=c(FALSE,TRUE), P=P_Theta1 )
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: btm.Rd
%% File Version: 0.426

\name{btm}
\alias{btm}
\alias{summary.btm}
\alias{predict.btm}
\alias{btm_sim}


%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Extended Bradley-Terry Model
}
\description{
The function \code{btm} estimates an extended Bradley-Terry model (Hunter, 2004; see Details).
Parameter estimation uses a bias corrected joint maximum likelihood
estimation method based on \eqn{\varepsilon}-adjustment (see Bertoli-Barsotti, Lando & Punzo, 2014).
See Details for the algorithm.

The function \code{btm_sim} simulated data from the extended Bradley-Terry model.
}

\usage{
btm(data, judge=NULL, ignore.ties=FALSE, fix.eta=NULL, fix.delta=NULL, fix.theta=NULL,
       maxiter=100, conv=1e-04, eps=0.3, wgt.ties=.5)

\method{summary}{btm}(object, file=NULL, digits=4,...)

\method{predict}{btm}(object, data=NULL, ...)

btm_sim(theta, eta=0, delta=-99, repeated=FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
Data frame with three columns. The first two columns contain labels
from the units in the pair comparison. The third column contains the
result of the comparison. "1" means that the first units wins, "0" means
that the second unit wins and "0.5" means a draw (a tie).
}
\item{judge}{Optional vector of judge identifiers (if multiple
judges are available)}
  \item{ignore.ties}{
Logical indicating whether ties should be ignored.
}
  \item{fix.eta}{
Numeric value for a fixed \eqn{\eta} value
}
  \item{fix.delta}{
Numeric value for a fixed \eqn{\delta} value
}
  \item{fix.theta}{
A vector with entries for fixed theta values.
}
  \item{maxiter}{
Maximum number of iterations
}
  \item{conv}{
Convergence criterion
}
  \item{eps}{
The \eqn{\varepsilon} parameter for the \eqn{\varepsilon}-adjustment
method (see Bertoli-Barsotti, Lando & Punzo, 2014) which reduces bias
in ability estimates. In case of \eqn{\varepsilon=0}, persons with
extreme scores are removed from the pairwise comparison.
}
\item{wgt.ties}{Weighting parameter for ties, see formula in Details.
The default is .5}
\item{object}{Object of class \code{btm}}
\item{file}{Optional file name for sinking the summary into}
\item{digits}{Number of digits after decimal to print}
\item{\dots}{Further arguments to be passed.}
\item{theta}{Vector of abilities}
\item{eta}{Value of \eqn{\eta} parameter}
\item{delta}{Value of \eqn{\delta} parameter}
\item{repeated}{Logical indicating whether repeated ratings of dyads (for home advantage
effect) should be simulated}
}

\details{
The extended Bradley-Terry model for the comparison of individuals
\eqn{i} and \eqn{j} is defined as
\deqn{P(X_{ij}=1 ) \propto \exp( \eta + \theta_i ) }
\deqn{P(X_{ij}=0 ) \propto \exp(  \theta_j ) }
\deqn{P(X_{ij}=0.5) \propto \exp( \delta + w_T ( \eta  + \theta_i +\theta_j ) ) }

The parameters \eqn{\theta_i} denote the abilities, \eqn{\delta} is the
tendency of the occurrence of ties and \eqn{\eta} is the home-advantage
effect. The weighting parameter \eqn{w_T} governs the importance of ties and can be
chosen in the argument \code{wgt.ties}.

A joint maximum likelihood (JML) estimation is applied for simulataneous
estimation of \eqn{\eta}, \eqn{\delta} and all \eqn{\theta_i} parameters.
In the Rasch model, it was shown that JML can result in biased parameter
estimates. The \eqn{\varepsilon}-adjustment approach has been proposed
to reduce the bias in parameter estimates (Bertoli-Bersotti, Lando & Punzo, 2014).
This estimation approach is adapted to the Bradley-Terry model in
the \code{btm} function. To this end, the likelihood function is
modified for the purpose of bias reduction. It can be easily shown that there
exist sufficient statistics for \eqn{\eta}, \eqn{\delta} and all \eqn{\theta_i}
parameters. In the \eqn{\varepsilon}-adjustment approach, the sufficient
statistic for the \eqn{\theta_i} parameter is modified. In JML estimation
of the Bradley-Terry model, \eqn{S_i=\sum_{j \ne i} ( x_{ij} + x_{ji} )} is
a sufficient statistic for \eqn{\theta_i}. Let \eqn{M_i} the maximum score
for person \eqn{i} which is the number of \eqn{x_{ij}} terms appearing in
\eqn{S_i}. In the \eqn{\varepsilon}-adjustment approach, the sufficient statistic
\eqn{S_i} is modified to
\deqn{S_{i, \varepsilon}=\varepsilon +
\frac{M_i - 2 \varepsilon}{M_i} S_i } and \eqn{S_{i, \varepsilon}} instead of
\eqn{S_{i}} is used in JML estimation. Hence, original scores \eqn{S_i} are
linearly transformed for all persons \eqn{i}.
}

\value{
List with following entries
  \item{pars}{Parameter summary for \eqn{\eta} and \eqn{\delta}}
  \item{effects}{Parameter estimates for \eqn{\theta} and
      outfit and infit statistics}
  \item{summary.effects}{Summary of \eqn{\theta} parameter estimates}
  \item{mle.rel}{MLE reliability, also known as separation reliability}
  \item{sepG}{Separation index \eqn{G}}
  \item{probs}{Estimated probabilities}
  \item{data}{Used dataset with integer identifiers}
  \item{fit_judges}{Fit statistics (outfit and infit) for judges if \code{judge}
   is provided. In addition, average agreement of the rating with the mode of
   the ratings is calculated for each judge (at least three ratings per dyad has
   to be available for computing the agreement).}
\item{residuals}{Unstandardized and standardized residuals for each observation}
}

\references{
Bertoli-Barsotti, L., Lando, T., & Punzo, A. (2014). Estimating a Rasch Model
via fuzzy empirical probability functions. In D. Vicari, A. Okada, G. Ragozini
& C. Weihs (Eds.). \emph{Analysis and Modeling of Complex Data in Behavioral
and Social Sciences}. Springer.
\doi{10.1007/978-3-319-06692-9_4}

Hunter, D. R. (2004). MM algorithms for generalized Bradley-Terry models.
\emph{Annals of Statistics, 32}, 384-406. doi: 10.1214/aos/1079120141
}

%\author{
%Alexander Robitzsch
%}

%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also the \R packages \pkg{BradleyTerry2}, \pkg{psychotools},
\pkg{psychomix} and \pkg{prefmod}.
}


\examples{
#############################################################################
# EXAMPLE 1: Bradley-Terry model | data.pw01
#############################################################################

data(data.pw01)

dat <- data.pw01
dat <- dat[, c("home_team", "away_team", "result") ]

# recode results according to needed input
dat$result[ dat$result==0 ] <- 1/2   # code for ties
dat$result[ dat$result==2 ] <- 0     # code for victory of away team

#********************
# Model 1: Estimation with ties and home advantage
mod1 <- sirt::btm( dat)
summary(mod1)

\dontrun{
#*** Model 2: Estimation with ties, no epsilon adjustment
mod2 <- sirt::btm( dat, eps=0)
summary(mod2)

#*** Model 3: Estimation with ties, no epsilon adjustment, weight for ties of .333 which
#    corresponds to the rule of 3 points for a victory and 1 point of a draw in football
mod3 <- sirt::btm( dat, eps=0, wgt.ties=1/3)
summary(mod3)

#*** Model 4: Some fixed abilities
fix.theta <- c("Anhalt Dessau"=-1 )
mod4 <- sirt::btm( dat, eps=0, fix.theta=fix.theta)
summary(mod4)

#*** Model 5: Ignoring ties, no home advantage effect
mod5 <- sirt::btm( dat, ignore.ties=TRUE, fix.eta=0)
summary(mod5)

#*** Model 6: Ignoring ties, no home advantage effect (JML approach and eps=0)
mod6 <- sirt::btm( dat, ignore.ties=TRUE, fix.eta=0, eps=0)
summary(mod5)

#############################################################################
# EXAMPLE 2: Venice chess data
#############################################################################

# See http://www.rasch.org/rmt/rmt113o.htm
# Linacre, J. M. (1997). Paired Comparisons with Standard Rasch Software.
# Rasch Measurement Transactions, 11:3, 584-585.

# dataset with chess games -> "D" denotes a draw (tie)
chessdata <- scan( what="character")
    1D.0..1...1....1.....1......D.......D........1.........1.......... Browne
    0.1.D..0...1....1.....1......D.......1........D.........1......... Mariotti
    .D0..0..1...D....D.....1......1.......1........1.........D........ Tatai
    ...1D1...D...D....1.....D......D.......D........1.........0....... Hort
    ......010D....D....D.....1......D.......1........1.........D...... Kavalek
    ..........00DDD.....D.....D......D.......1........D.........1..... Damjanovic
    ...............00D0DD......D......1.......1........1.........0.... Gligoric
    .....................000D0DD.......D.......1........D.........1... Radulov
    ............................DD0DDD0D........0........0.........1.. Bobotsov
    ....................................D00D00001.........1.........1. Cosulich
    .............................................0D000D0D10..........1 Westerinen
    .......................................................00D1D010000 Zichichi

L <- length(chessdata) / 2
games <- matrix( chessdata, nrow=L, ncol=2, byrow=TRUE )
G <- nchar(games[1,1])
# create matrix with results
results <- matrix( NA, nrow=G, ncol=3 )
for (gg in 1:G){
    games.gg <- substring( games[,1], gg, gg )
    ind.gg <- which( games.gg !="." )
    results[gg, 1:2 ] <- games[ ind.gg, 2]
    results[gg, 3 ] <- games.gg[ ind.gg[1] ]
}
results <- as.data.frame(results)
results[,3] <- paste(results[,3] )
results[ results[,3]=="D", 3] <- 1/2
results[,3] <- as.numeric( results[,3] )

# fit model ignoring draws
mod1 <- sirt::btm( results, ignore.ties=TRUE, fix.eta=0, eps=0 )
summary(mod1)

# fit model with draws
mod2 <- sirt::btm( results, fix.eta=0, eps=0 )
summary(mod2)

#############################################################################
# EXAMPLE 3: Simulated data from the Bradley-Terry model
#############################################################################

set.seed(9098)
N <- 22
theta <- seq(2,-2, len=N)

#** simulate and estimate data without repeated dyads
dat1 <- sirt::btm_sim(theta=theta)
mod1 <- sirt::btm( dat1, ignore.ties=TRUE, fix.delta=-99, fix.eta=0)
summary(mod1)

#*** simulate data with home advantage effect and ties
dat2 <- sirt::btm_sim(theta=theta, eta=.8, delta=-.6, repeated=TRUE)
mod2 <- sirt::btm(dat2)
summary(mod2)

#############################################################################
# EXAMPLE 4: Estimating the Bradley-Terry model with multiple judges
#############################################################################

#*** simulating data with multiple judges
set.seed(987)
N <- 26  # number of objects to be rated
theta <- seq(2,-2, len=N)
s1 <- stats::sd(theta)
dat <- NULL
# judge discriminations which define tendency to provide reliable ratings
discrim <- c( rep(.9,10), rep(.5,2), rep(0,2) )
#=> last four raters provide less reliable ratings

RR <- length(discrim)
for (rr in 1:RR){
    theta1 <- discrim[rr]*theta + stats::rnorm(N, mean=0, sd=s1*sqrt(1-discrim[rr]))
    dat1 <- sirt::btm_sim(theta1)
    dat1$judge <- rr
    dat <- rbind(dat, dat1)
}

#** estimate the Bradley-Terry model and compute judge-specific fit statistics
mod <- sirt::btm( dat[,1:3], judge=paste0("J",100+dat[,4]), fix.eta=0, ignore.ties=TRUE)
summary(mod)
}
}


% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Bradley-Terry model}
%% \keyword{summary}% __ONLY ONE__ keyword per line

%% File Name: reliability.nonlinearSEM.Rd
%% File Version: 0.23

\name{reliability.nonlinearSEM}
\alias{reliability.nonlinearSEM}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimation of Reliability for Confirmatory Factor Analyses
Based on Dichotomous Data
}
\description{
This function estimates a model based reliability using confirmatory factor
analysis (Green & Yang, 2009).
}
\usage{
reliability.nonlinearSEM(facloadings, thresh, resid.cov=NULL, cor.factors=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{facloadings}{
Matrix of factor loadings
}
  \item{thresh}{
Vector of thresholds
}
\item{resid.cov}{Matrix of residual covariances}
  \item{cor.factors}{
Optional matrix of covariances (correlations) between factors. The
default is a diagonal matrix with variances of 1.
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list. The reliability is the list element \code{omega.rel}
}

\references{
Green, S. B., & Yang, Y. (2009). Reliability of summed item
scores using structural equation modeling: An alternative to
coefficient alpha. \emph{Psychometrika, 74}, 155-167.
}

%\author{
%Alexander Robitzsch
%}


\note{
This function needs the \pkg{mvtnorm} package.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
This function is used in \code{\link{greenyang.reliability}}.
}
\examples{
#############################################################################
# EXAMPLE 1: Reading data set
#############################################################################
data(data.read)
dat <- data.read
I <- ncol(dat)

# define item clusters
itemcluster <- rep( 1:3, each=4)
error.corr <- diag(1,ncol(dat))
for ( ii in 1:3){
    ind.ii <- which( itemcluster==ii )
    error.corr[ ind.ii, ind.ii ] <- ii
        }
# estimate the model with error correlations
mod1 <- sirt::rasch.pml3( dat, error.corr=error.corr)
summary(mod1)

# extract item parameters
thresh <- - matrix( mod1$item$a * mod1$item$b, I, 1 )
A <- matrix( mod1$item$a * mod1$item$sigma, I, 1 )
# extract estimated correlation matrix
corM <- mod1$eps.corrM
# compute standardized factor loadings
facA <- 1 / sqrt( A^2 + 1 )
resvar <- 1 - facA^2
covM <- outer( sqrt(resvar[,1]), sqrt(resvar[,1] ) ) * corM
facloadings <- A *facA

# estimate reliability
rel1 <- sirt::reliability.nonlinearSEM( facloadings=facloadings, thresh=thresh,
           resid.cov=covM)
rel1$omega.rel
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Reliability}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: sirt-package.Rd
%% File Version: 2.703

\name{sirt-package}
\alias{sirt-package}
\alias{sirt}
\docType{package}

\title{
\packageTitle{sirt}
}

\description{
\packageDescription{sirt}
}

\author{
\packageAuthor{sirt}

Maintainer: \packageMaintainer{sirt}
}


\details{

The \pkg{sirt} package enables the estimation of following models:

\itemize{

%% M-dim generalized item response model
\item Multidimensional marginal maximum likelihood estimation (MML)
      of generalized logistic Rasch type models using the
      generalized logistic link function (Stukel, 1988) can be conducted
      with \code{\link{rasch.mml2}} and the argument \code{itemtype="raschtype"}.
      This model also allows the estimation of the 4PL item
      response model (Loken & Rulison, 2010).
      Multiple group estimation, latent regression models and
      plausible value imputation are supported. In addition, pseudo-likelihood
      estimation for fractional item response data can be conducted.

%% M-dim noncompensatory and compensatory IRT model
\item Multidimensional noncompensatory, compensatory and partially compensatory
item response models for dichotomous item responses (Reckase, 2009) can be estimated
with the \code{\link{smirt}} function and the options \code{irtmodel="noncomp"}
, \code{irtmodel="comp"} and \code{irtmodel="partcomp"}.

%% 1-dim Ramsay type model
\item The unidimensional quotient model (Ramsay, 1989) can be estimated
using \code{\link{rasch.mml2}} with \code{itemtype="ramsay.qm"}.

%% 1-dim nonparametric IRT models
\item Unidimensional nonparametric item response models can be estimated
 employing MML estimation (Rossi, Wang & Ramsay, 2002) by making use of
 \code{\link{rasch.mml2}} with \code{itemtype="npirt"}.
 Kernel smoothing for item response function estimation (Ramsay, 1991)
 is implemented in \code{\link{np.dich}}.

%% 1-dim Copula model
\item The multidimensional IRT copula model (Braeken, 2011) can be applied
    for handling local dependencies, see \code{\link{rasch.copula3}}.

%% 1-dim JML
\item Unidimensional joint maximum likelihood estimation (JML) of the Rasch model
is possible with the \code{\link{rasch.jml}} function. Bias correction methods
    for item parameters are included in \code{\link{rasch.jml.jackknife1}}
    and \code{\link{rasch.jml.biascorr}}.

%% M-dim LC Rasch model
\item The multidimensional latent class Rasch and 2PL model (Bartolucci, 2007)
which employs a discrete trait distribution can be estimated
with \code{\link{rasch.mirtlc}}.

%% Rater Models
\item The unidimensional 2PL rater facets model (Lincare, 1994) can be estimated
with \code{\link{rm.facets}}. A hierarchical rater model based on
signal detection theory (DeCarlo, Kim & Johnson, 2011) can be conducted
with \code{\link{rm.sdt}}. A simple latent class model for two exchangeable
raters is implemented in \code{\link{lc.2raters}}. See Robitzsch and Steinfeld (2018)
for more details.

%% Grade of membership model
\item The discrete grade of membership model (Erosheva, Fienberg & Joutard, 2007)
and the Rasch grade of membership model can be estimated by \code{\link{gom.em}}.

%% MCMC estimation multilevel IRT models
\item Some hierarchical IRT models and random item models for dichotomous
and normally distributed data (van den Noortgate, de Boeck & Meulders, 2003;
Fox & Verhagen, 2010) can be estimated with \code{\link{mcmc.2pno.ml}}.

%% 1-dim PCML
\item Unidimensional pairwise conditional likelihood estimation
    (PCML; Zwinderman, 1995) is implemented in \code{\link{rasch.pairwise}} or
    \code{\link{rasch.pairwise.itemcluster}}.

%% 1-dim PMML
\item Unidimensional pairwise marginal likelihood estimation
   (PMML; Renard, Molenberghs & Geys, 2004)
    can be conducted using \code{\link{rasch.pml3}}. In this function
    local dependence can be handled by imposing residual error structure
    or omitting item pairs within a dependent item cluster from the
    estimation. \cr
    The function \code{\link{rasch.evm.pcm}} estimates the multiple group
    partial credit model based on the pairwise eigenvector approach
    which avoids iterative estimation.

%% MCMC estimation of some models
\item Some item response models in \pkg{sirt} can be estimated via
Markov Chain Monte Carlo (MCMC) methods. In \code{\link{mcmc.2pno}}
the two-parameter normal ogive model can be estimated. A hierarchical
version of this model (Janssen, Tuerlinckx, Meulders & de Boeck, 2000)
is implemented in \code{\link{mcmc.2pnoh}}. The 3PNO testlet model
(Wainer, Bradlow & Wang, 2007; Glas, 2012) can be estimated with
\code{\link{mcmc.3pno.testlet}}.
Some hierarchical IRT models and random item models
(van den Noortgate, de Boeck & Meulders, 2003) can be estimated
with \code{\link{mcmc.2pno.ml}}.

%% NOHARM
\item For dichotomous response data, the free NOHARM software
(McDonald, 1982, 1997) estimates the multidimensional compensatory 3PL model and the function
\code{\link{R2noharm}} runs NOHARM from within \R. Note that NOHARM must be
downloaded from \emph{http://noharm.niagararesearch.ca/nh4cldl.html}
at first. A pure \R implementation of the NOHARM model with some extensions
can be found in \code{\link{noharm.sirt}}.

%% Nonparametric item response theory / ISOP model
\item The measurement theoretic founded nonparametric item
response models of Scheiblechner (1995, 1999) -- the ISOP
and the ADISOP model -- can be estimated with
\code{\link{isop.dich}} or \code{\link{isop.poly}}.
Item scoring within this theory can be conducted with
\code{\link{isop.scoring}}.

%% Functional unidimensional item response model
\item The functional unidimensional item response model
(Ip et al., 2013) can be estimated with \code{\link{f1d.irt}}.

%% 1-dim Rasch model variational approximation
\item The Rasch model can be estimated by variational approximation
(Rijmen & Vomlel, 2008) using \code{\link{rasch.va}}.

%% 1-dim Guttman model
\item The unidimensional probabilistic Guttman model (Proctor, 1970) can be
    specified with \code{\link{prob.guttman}}.

%% jackknife WLE
\item A jackknife method for the estimation of standard errors of the
    weighted likelihood trait estimate (Warm, 1989) is available in
    \code{\link{wle.rasch.jackknife}}.

%% reliability
\item Model based reliability for dichotomous data can be calculated by the method
of Green and Yang (2009) with \code{\link{greenyang.reliability}} and the
marginal true score method of Dimitrov (2003) using the function
\code{\link{marginal.truescore.reliability}}.

%% DETECT
\item Essential unidimensionality can be assessed by the DETECT
    index (Stout, Habing, Douglas & Kim, 1996), see the function
    \code{\link{conf.detect}}.

%% linking / alignment
\item Item parameters from several studies can be linked using the Haberman
method (Haberman, 2009) in \code{\link{linking.haberman}}. See also
\code{\link{equating.rasch}} and \code{\link{linking.robust}}.
The alignment procedure (Asparouhov & Muthen, 2013)
\code{\link{invariance.alignment}} is originally for comfirmatory factor
analysis and aims at obtaining approximate invariance.


%% Person Fit
\item Some person fit statistics in the Rasch model (Meijer & Sijtsma, 2001)
    are included in \code{\link{personfit.stat}}.

%% LSDM
\item An alternative to the linear logistic test model (LLTM), the
    so called least squares distance model for cognitive diagnosis
    (LSDM; Dimitrov, 2007), can be estimated with the function
    \code{\link{lsdm}}.

%% LSEM
\item Local structural equation models (LSEM) can be estimated with the
\code{\link{lsem.estimate}} function (Hildebrandt et al., 2016).

}

}




\references{
Asparouhov, T., & Muthen, B. (2014). Multiple-group factor analysis alignment.
\emph{Structural Equation Modeling, 21}(4), 1-14.
\doi{10.1080/10705511.2014.919210}

Bartolucci, F. (2007). A class of multidimensional
IRT models for testing unidimensionality and clustering
items. \emph{Psychometrika, 72}, 141-157.

Braeken, J. (2011). A boundary mixture approach to violations of conditional
independence. \emph{Psychometrika, 76}(1), 57-76.
\doi{10.1007/s11336-010-9190-4}

DeCarlo, T., Kim, Y., & Johnson, M. S. (2011).
A hierarchical rater model for constructed responses, with a signal detection
rater model. \emph{Journal of Educational Measurement, 48}(3), 333-356.
\doi{10.1111/j.1745-3984.2011.00143.x}

Dimitrov, D. (2003). Marginal true-score measures and reliability
for binary items as a function of their IRT parameters.
\emph{Applied Psychological Measurement, 27}, 440-458.

Dimitrov, D. M. (2007). Least squares distance method of cognitive validation
and analysis for binary items using their item response theory parameters.
\emph{Applied Psychological Measurement, 31}, 367-387.

Erosheva, E. A., Fienberg, S. E., & Joutard, C. (2007).
Describing disability through individual-level mixture models
for multivariate binary data. \emph{Annals of Applied Statistics,
1}, 502-537.

Fox, J.-P. (2010). \emph{Bayesian item response modeling}. New York: Springer.
\doi{10.1007/978-1-4419-0742-4}

Fox, J.-P., & Verhagen, A.-J. (2010). Random item effects modeling for
cross-national survey data.
In E. Davidov, P. Schmidt, & J. Billiet (Eds.),
\emph{Cross-cultural Analysis: Methods and Applications}
(pp. 467-488), London: Routledge Academic.

Fraser, C., & McDonald, R. P. (1988). NOHARM: Least squares item factor analysis.
\emph{Multivariate Behavioral Research, 23}, 267-269.

Glas, C. A. W. (2012). \emph{Estimating and testing the extended testlet model.}
LSAC Research Report Series, RR 12-03.

Green, S.B., & Yang, Y. (2009). Reliability of summed item
scores using structural equation modeling: An alternative to
coefficient alpha. \emph{Psychometrika, 74}, 155-167.

Haberman, S. J. (2009). \emph{Linking parameter estimates derived
from an item response model through separate calibrations}.
ETS Research Report ETS RR-09-40. Princeton, ETS.
\doi{10.1002/j.2333-8504.2009.tb02197.x}

Hildebrandt, A., Luedtke, O., Robitzsch, A., Sommer, C., &
Wilhelm, O. (2016). Exploring factor model parameters across continuous variables
with local structural equation models.
\emph{Multivariate Behavioral Research, 51}(2-3), 257-278.
\doi{10.1080/00273171.2016.1142856}

Ip, E. H., Molenberghs, G., Chen, S. H., Goegebeur, Y., &
De Boeck, P. (2013). Functionally unidimensional item
response models for multivariate binary data.
\emph{Multivariate Behavioral Research, 48}, 534-562.

Janssen, R., Tuerlinckx, F., Meulders, M., & de Boeck, P. (2000).
A hierarchical IRT model for criterion-referenced measurement.
\emph{Journal of Educational and Behavioral Statistics, 25}, 285-306.

Jeon, M., & Rijmen, F. (2016). A modular approach for item response theory modeling with 
the \R package \pkg{flirt}. \emph{Behavior Research Methods, 48}(2), 742-755.
\doi{10.3758/s13428-015-0606-z}

Linacre, J. M. (1994). \emph{Many-Facet Rasch Measurement}.
Chicago: MESA Press.

Loken, E. & Rulison, K. L. (2010). Estimation of a four-parameter
item response theory model. \emph{British Journal of Mathematical
and Statistical Psychology, 63}, 509-525.

McDonald, R. P. (1982). Linear versus nonlinear models in item response theory.
\emph{Applied Psychological Measurement, 6}(4), 379-396.
\doi{10.1177/014662168200600402}

McDonald, R. P. (1997). Normal-ogive multidimensional model.
In W. van der Linden & R. K. Hambleton (1997):
\emph{Handbook of modern item response theory} (pp. 257-269).
New York: Springer.
\doi{10.1007/978-1-4757-2691-6_15}

Meijer, R. R., & Sijtsma, K. (2001). Methodology
review: Evaluating person fit. \emph{Applied Psychological
Measurement, 25}, 107-135.

Proctor, C. H. (1970). A probabilistic formulation and statistical analysis
for Guttman scaling. \emph{Psychometrika, 35}, 73-78.

Ramsay, J. O. (1989). A comparison of three simple test theory models.
\emph{Psychometrika, 54}, 487-499.

Ramsay, J. O. (1991). Kernel smoothing approaches to
nonparametric item characteristic curve estimation.
\emph{Psychometrika, 56}, 611-630.

Reckase, M. (2009). \emph{Multidimensional item response theory}.
New York: Springer. \doi{10.1007/978-0-387-89976-3}

Renard, D., Molenberghs, G., & Geys, H. (2004). A pairwise likelihood
approach to estimation in multilevel probit models. \emph{Computational Statistics
& Data Analysis, 44}, 649-667.

Rijmen, F., & Vomlel, J. (2008). Assessing the performance of
variational methods for mixed logistic regression models.
\emph{Journal of Statistical Computation and Simulation, 78}, 765-779.

Robitzsch, A., & Steinfeld, J. (2018). Item response models for human ratings: Overview,
estimation methods, and implementation in R.
\emph{Psychological Test and Assessment Modeling, 60}(1), 101-139.

Rossi, N., Wang, X. & Ramsay, J. O. (2002). Nonparametric item response function
estimates with the EM algorithm.
\emph{Journal of Educational and Behavioral Statistics, 27}, 291-317.

Rusch, T., Mair, P., & Hatzinger, R. (2013).
\emph{Psychometrics with \R: A Review of CRAN Packages for Item Response Theory.}
http://epub.wu.ac.at/4010/1/resrepIRThandbook.pdf

Scheiblechner, H. (1995). Isotonic ordinal
probabilistic models (ISOP). \emph{Psychometrika, 60}(2), 281-304.
\doi{10.1007/BF02301417}

Scheiblechner, H. (1999). Additive conjoint isotonic
probabilistic models (ADISOP). \emph{Psychometrika, 64}, 295-316.

Schroeders, U., Robitzsch, A., & Schipolowski, S. (2014). A comparison of different
psychometric approaches to modeling testlet structures: An example with C-tests.
\emph{Journal of Educational Measurement, 51}(4), 400-418.
\doi{10.1111/jedm.12054}

Stout, W., Habing, B., Douglas, J., & Kim, H. R. (1996).
Conditional covariance-based nonparametric multidimensionality assessment.
\emph{Applied Psychological Measurement, 20}(4), 331-354.
\doi{10.1177/014662169602000403}

Stukel, T. A. (1988). Generalized logistic models.
\emph{Journal of the American Statistical Association, 83}(402), 426-431.
\doi{10.1080/01621459.1988.10478613}

Uenlue, A., & Yanagida, T. (2011). \R you ready for \R?:
The CRAN psychometrics task view.
\emph{British Journal of Mathematical and Statistical Psychology, 64}(1), 182-186.
\doi{10.1348/000711010X519320}

van den Noortgate, W., De Boeck, P., & Meulders, M. (2003).
Cross-classification multilevel logistic models in psychometrics.
\emph{Journal of Educational and Behavioral Statistics, 28}, 369-386.

Warm, T. A. (1989). Weighted likelihood estimation of ability in item response theory.
\emph{Psychometrika, 54}, 427-450.

Wainer, H., Bradlow, E. T., & Wang, X. (2007).
\emph{Testlet response theory and its applications}.
Cambridge: Cambridge University Press.

Zwinderman, A. H. (1995). Pairwise parameter estimation in Rasch models.
\emph{Applied Psychological Measurement, 19}, 369-375.
}



\seealso{
For estimating multidimensional models for polytomous item responses
see the \pkg{mirt}, \pkg{flirt} (Jeon & Rijmen, 2016) and \pkg{TAM} packages.

For conditional maximum likelihood estimation see the \pkg{eRm} package.

For pairwise estimation likelihood methods (also known as composite
likelihood methods) see \pkg{pln} or \pkg{lavaan}.

The estimation of cognitive diagnostic models is possible
using the \pkg{CDM} package.

For the multidimensional latent class IRT model see the \pkg{MultiLCIRT}
package which also allows the estimation IRT models with polytomous item responses.

Latent class analysis can be carried out with \pkg{covLCA}, \pkg{poLCA},
\pkg{BayesLCA}, \pkg{randomLCA} or \pkg{lcmm} packages.

Markov Chain Monte Carlo estimation for item response models can also
be found in the \pkg{MCMCpack} package (see the \code{MCMCirt} functions
therein).

See Rusch, Mair and Hatzinger (2013) and Uenlue and Yanagida (2011)
for reviews of psychometrics packages in \R.
}


\examples{
##
##   |-----------------------------------------------------------------|
##   | sirt 0.40-4 (2013-11-26)                                        |
##   | Supplementary Item Response Theory                              |
##   | Maintainer: Alexander Robitzsch <a.robitzsch at bifie.at >      |
##   | https://sites.google.com/site/alexanderrobitzsch/software       |
##   |-----------------------------------------------------------------|
##
##                       _/              _/
##              _/_/_/      _/  _/_/  _/_/_/_/
##           _/_/      _/  _/_/        _/
##              _/_/  _/  _/          _/
##         _/_/_/    _/  _/            _/_/
##
}

\keyword{package}

%% File Name: fuzdiscr.Rd
%% File Version: 0.21

\name{fuzdiscr}
\alias{fuzdiscr}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimation of a Discrete Distribution for Fuzzy Data (Data in Belief Function
Framework)
}
\description{
This function estimates a discrete distribution for uncertain data
based on the belief function framework (Denoeux, 2013; see Details).
}
\usage{
fuzdiscr(X, theta0=NULL, maxiter=200, conv=1e-04)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{X}{
Matrix with fuzzy data. Rows corresponds to subjects and columns to
values of the membership function
}
  \item{theta0}{
Initial vector of parameter estimates
}
  \item{maxiter}{
Maximum number of iterations
}
  \item{conv}{
Convergence criterion
}
}


\details{
For \eqn{n} subjects, membership functions \eqn{m_n(k)} are observed which indicate
the belief in data \eqn{X_n=k}. The membership function is interpreted as
\emph{epistemic uncertainty} (Denoeux, 2011). However, associated parameters
in statistical models are crisp which means that models are formulated at the
basis of precise (crisp) data if they would be observed.

In the present estimation problem of a discrete distribution,
the parameters of interest are category probabilities
\eqn{\theta_k=P( X=k)}.

The parameter estimation follows the evidential EM algorithm (Denoeux, 2013).
}

\value{
Vector of probabilities of the discrete distribution
}


\references{
Denoeux, T. (2011). Maximum likelihood estimation from fuzzy data using the
EM algorithm. \emph{Fuzzy Sets and Systems, 183}, 72-91.

Denoeux, T. (2013). Maximum likelihood estimation from uncertain data
in the belief function framework.
\emph{IEEE Transactions on Knowledge and Data Engineering, 25}, 119-130.
}


%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}


\examples{
#############################################################################
# EXAMPLE 1: Binomial distribution Denoeux Example 4.3 (2013)
#############################################################################

#*** define uncertain data
X_alpha <- function( alpha ){
    Q <- matrix( 0, 6, 2 )
    Q[5:6,2] <- Q[1:3,1] <- 1
    Q[4,] <- c( alpha, 1 - alpha )
    return(Q)
        }

# define data for alpha=0.5
X <- X_alpha( alpha=.5 )
  ##   > X
  ##        [,1] [,2]
  ##   [1,]  1.0  0.0
  ##   [2,]  1.0  0.0
  ##   [3,]  1.0  0.0
  ##   [4,]  0.5  0.5
  ##   [5,]  0.0  1.0
  ##   [6,]  0.0  1.0

  ## The fourth observation has equal plausibility for the first and the
  ## second category.

# parameter estimate uncertain data
fuzdiscr( X )
  ##   > sirt::fuzdiscr( X )
  ##   [1] 0.5999871 0.4000129

# parameter estimate pseudo likelihood
colMeans( X )
  ##   > colMeans( X )
  ##   [1] 0.5833333 0.4166667
##-> Observations are weighted according to belief function values.

#*****
# plot parameter estimates as function of alpha
alpha <- seq( 0, 1, len=100 )
res <- sapply( alpha, FUN=function(aa){
             X <- X_alpha( alpha=aa )
             c( sirt::fuzdiscr( X )[1], colMeans( X )[1] )
                    } )
# plot
plot( alpha, res[1,], xlab=expression(alpha), ylab=expression( theta[alpha] ), type="l",
        main="Comparison Belief Function and Pseudo-Likelihood (Example 1)")
lines( alpha, res[2,], lty=2, col=2)
legend( 0, .67, c("Belief Function", "Pseudo-Likelihood" ), col=c(1,2), lty=c(1,2) )

#############################################################################
# EXAMPLE 2: Binomial distribution (extends Example 1)
#############################################################################

X_alpha <- function( alpha ){
    Q <- matrix( 0, 6, 2 )
    Q[6,2] <- Q[1:2,1] <- 1
    Q[3:5,] <- matrix( c( alpha, 1 - alpha ), 3, 2, byrow=TRUE)
    return(Q)
        }

X <- X_alpha( alpha=.5 )
alpha <- seq( 0, 1, len=100 )
res <- sapply( alpha, FUN=function(aa){
           X <- X_alpha( alpha=aa )
           c( sirt::fuzdiscr( X )[1], colMeans( X )[1] )
                    } )
# plot
plot( alpha, res[1,], xlab=expression(alpha), ylab=expression( theta[alpha] ), type="l",
        main="Comparison Belief Function and Pseudo-Likelihood (Example 2)")
lines( alpha, res[2,], lty=2, col=2)
legend( 0, .67, c("Belief Function", "Pseudo-Likelihood" ), col=c(1,2), lty=c(1,2) )

#############################################################################
# EXAMPLE 3: Multinomial distribution with three categories
#############################################################################

# define uncertain data
X <- matrix( c( 1,0,0, 1,0,0,   0,1,0, 0,0,1, .7, .2, .1,
         .4, .6, 0 ), 6, 3, byrow=TRUE )
  ##   > X
  ##        [,1] [,2] [,3]
  ##   [1,]  1.0  0.0  0.0
  ##   [2,]  1.0  0.0  0.0
  ##   [3,]  0.0  1.0  0.0
  ##   [4,]  0.0  0.0  1.0
  ##   [5,]  0.7  0.2  0.1
  ##   [6,]  0.4  0.6  0.0

##->  Only the first four observations are crisp.

#*** estimation for uncertain data
fuzdiscr( X )
  ##   > sirt::fuzdiscr( X )
  ##   [1] 0.5772305 0.2499931 0.1727764

#*** estimation pseudo-likelihood
colMeans(X)
  ##   > colMeans(X)
  ##   [1] 0.5166667 0.3000000 0.1833333

##-> Obviously, the treatment uncertainty is different in belief function
##   and in pseudo-likelihood framework.
}


% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Fuzzy data}
%% \keyword{Belief function}

%% File Name: gom.em.Rd
%% File Version: 1.803

\name{gom.em}
\alias{gom.em}
\alias{summary.gom}
\alias{logLik.gom}
\alias{anova.gom}
\alias{IRT.irfprob.gom}
\alias{IRT.likelihood.gom}
\alias{IRT.posterior.gom}
\alias{IRT.modelfit.gom}
\alias{summary.IRT.modelfit.gom}


%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Discrete (Rasch) Grade of Membership Model
}
\description{
This function estimates the grade of membership model (Erosheva, Fienberg
& Joutard, 2007; also called mixed membership model) by the EM algorithm
assuming a discrete membership score distribution. The function is restricted
to dichotomous item responses.
}
\usage{
gom.em(dat, K=NULL, problevels=NULL, weights=NULL, model="GOM", theta0.k=seq(-5,5,len=15),
    xsi0.k=exp(seq(-6, 3, len=15)), max.increment=0.3, numdiff.parm=1e-4,
    maxdevchange=1e-6, globconv=1e-4, maxiter=1000, msteps=4, mstepconv=0.001,
    theta_adjust=FALSE, lambda.inits=NULL, lambda.index=NULL, pi.k.inits=NULL,
    newton_raphson=TRUE, optimizer="nlminb", progress=TRUE)

\method{summary}{gom}(object, file=NULL, ...)

\method{anova}{gom}(object,...)

\method{logLik}{gom}(object,...)

\method{IRT.irfprob}{gom}(object,...)

\method{IRT.likelihood}{gom}(object,...)

\method{IRT.posterior}{gom}(object,...)

\method{IRT.modelfit}{gom}(object,...)

\method{summary}{IRT.modelfit.gom}(object,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame with dichotomous responses
}
  \item{K}{
Number of classes (only applies for \code{model="GOM"})
}
  \item{problevels}{
Vector containing probability levels for membership functions
(only applies for \code{model="GOM"}). If a specific space of probability
levels should be estimated, then a matrix can be supplied (see Example 1,
Model 2a).
}
\item{weights}{Optional vector of sampling weights}
  \item{model}{
The type of grade of membership model. The default \code{"GOM"}
is the nonparametric grade of membership model. A parametric multivariate normal
representation can be requested by \code{"GOMnormal"}.
The probabilities and membership
functions specifications described in Details are called via \code{"GOMRasch"}.
}
  \item{theta0.k}{
Vector of \eqn{\tilde{\theta}_k} grid (applies only for \code{model="GOMRasch"})
}
  \item{xsi0.k}{
Vector of \eqn{\xi_p} grid (applies only for \code{model="GOMRasch"})
}
  \item{max.increment}{
Maximum increment
}
  \item{numdiff.parm}{
Numerical differentiation parameter
}
  \item{maxdevchange}{
Convergence criterion for change in relative deviance
}
  \item{globconv}{
Global convergence criterion for parameter change
}
  \item{maxiter}{
Maximum number of iterations
}
  \item{msteps}{
Number of iterations within a M step
}
  \item{mstepconv}{
Convergence criterion within a M step
}
\item{theta_adjust}{Logical indicating whether multivariate normal distribution
should be adaptively chosen during the EM algorithm.}

\item{lambda.inits}{Initial values for item parameters}
\item{lambda.index}{Optional integer matrix with integers indicating
equality constraints among \eqn{\lambda} item parameters}
\item{pi.k.inits}{Initial values for distribution parameters}
\item{newton_raphson}{Logical indicating whether Newton-Raphson should be
used for final iterations}
\item{optimizer}{Type of optimizer. Can be \code{"optim"} or \code{"nlminb"}.}
  \item{progress}{
Display iteration progress? Default is \code{TRUE}.
}
  \item{object}{
Object of class \code{gom}
}
\item{file}{Optional file name for summary output}
  \item{\dots}{
Further arguments to be passed
}
}


%--- details
\details{
The item response model of the grade of membership model
(Erosheva, Fienberg & Junker, 2002;
Erosheva, Fienberg & Joutard, 2007) with \eqn{K} classes
for dichotomous correct responses \eqn{X_{pi}}
of person \eqn{p} on item \eqn{i} is as follows (\code{model="GOM"})
\deqn{
    P(X_{pi}=1 | g_{p1}, \ldots, g_{pK} )=\sum_k \lambda_{ik} g_{pk}
\quad, \quad \sum_{k=1}^K g_{pk}=1
\quad, \quad 0 \leq g_{pk} \leq 1
                }

In most applications (e.g. Erosheva et al., 2007), the grade of
membership function \eqn{\{g_{pk}\}} is assumed to follow a Dirichlet
distribution. In our \code{gom.em} implementation
the membership function is assumed to be discretely represented
by a grid \eqn{u=(u_1, \ldots, u_L)} with entries between 0 and 1
(e.g. \code{seq(0,1,length=5)} with \eqn{L=5}).
The values \eqn{g_{pk}} of the membership function can then
only take values in \eqn{\{ u_1, \ldots, u_L \}} with the restriction
\eqn{\sum_k g_{pk} \sum_l \bold{1}(g_{pk}=u_l )=1}.
The grid \eqn{u} is specified by using the argument \code{problevels}.

The Rasch grade of membership model (\code{model="GOMRasch"}) poses constraints
on probabilities \eqn{\lambda_{ik}} and membership functions \eqn{g_{pk}}.
The membership
function of person \eqn{p} is parameterized by a location parameter \eqn{\theta_p}
and a variability parameter \eqn{\xi_p}. Each class \eqn{k} is represented by
a location parameter \eqn{\tilde{\theta}_k}. The membership function is defined as
\deqn{ g_{pk} \propto
\exp \left[ - \frac{ (\theta_p - \tilde{\theta}_k)^2 }{2 \xi_p^2 } \right]
}

The person parameter \eqn{\theta_p} indicates the usual 'ability', while
\eqn{\xi_p} describes the individual tendency to change between classes
\eqn{1,\ldots,K} and their corresponding locations
\eqn{\tilde{\theta}_1, \ldots,\tilde{\theta}_K}.
The extremal class probabilities \eqn{\lambda_{ik}} follow the Rasch model
\deqn{ \lambda_{ik}=invlogit( \tilde{\theta}_k - b_i  )=
\frac{ \exp( \tilde{\theta}_k - b_i ) }{ 1 + \exp( \tilde{\theta}_k - b_i ) }}

Putting these assumptions together leads to the model equation
\deqn{
    P(X_{pi}=1 | g_{p1}, \ldots, g_{pK} )=
    P(X_{pi}=1 | \theta_p, \xi_p  )=
        \sum_k \frac{ \exp( \tilde{\theta}_k - b_i ) }{ 1 + \exp(\tilde{\theta}_k - b_i ) }
        \cdot \exp \left[ - \frac{ (\theta_p - \tilde{\theta}_k)^2 }{2 \xi_p^2 } \right]
                }

In the extreme case of a very small \eqn{\xi_p=\varepsilon > 0} and
\eqn{\theta_p=\theta_0}, the Rasch model is obtained

\deqn{
    P(X_{pi}=1 | \theta_p, \xi_p  )=
    P(X_{pi}=1 | \theta_0, \varepsilon  )=
        \frac{ \exp( \theta_0 - b_i ) }{ 1 + \exp( \theta_0 - b_i ) }
                }

See Erosheva et al. (2002), Erosheva (2005, 2006) or Galyart (2015)
for a comparison of grade of membership models with latent trait models
and latent class models.

The grade of membership model is also published under the name
Bernoulli aspect model, see Bingham, Kaban and Fortelius (2009).
}


\value{
A list with following entries:
\item{deviance}{Deviance}
\item{ic}{Information criteria}
\item{item}{Data frame with item parameters}
\item{person}{Data frame with person parameters}
\item{EAP.rel}{EAP reliability (only applies for \code{model="GOMRasch"})}
\item{MAP}{Maximum aposteriori estimate of the membership function
    }
\item{EAP}{EAP estimate for individual membership scores}
\item{classdesc}{Descriptives for class membership}
\item{lambda}{Estimated response probabilities \eqn{\lambda_{ik}}}
\item{se.lambda}{Standard error for estimated response probabilities
\eqn{\lambda_{ik}}}
\item{mu}{Mean of the distribution of \eqn{(\theta_p, \xi_p)}
    (only applies for \code{model="GOMRasch"})}
\item{Sigma}{Covariance matrix of \eqn{(\theta_p, \xi_p)}
    (only applies for \code{model="GOMRasch"})}
\item{b}{Estimated item difficulties (only applies for \code{model="GOMRasch"})}
\item{se.b}{Standard error of estimated difficulties
(only applies for \code{model="GOMRasch"})}
\item{f.yi.qk}{Individual likelihood}
\item{f.qk.yi}{Individual posterior}
\item{probs}{Array with response probabilities}
\item{n.ik}{Expected counts}
\item{iter}{Number of iterations}
\item{I}{Number of items}
\item{K}{Number of classes}
\item{TP}{Number of discrete integration points for \eqn{(g_{p1},...,g_{pK})}}
\item{theta.k}{Used grid of membership functions}
\item{\dots}{Further values}
}
\references{
Bingham, E., Kaban, A., & Fortelius, M. (2009).
The aspect Bernoulli model: multiple causes of presences and absences.
\emph{Pattern Analysis and Applications, 12}(1), 55-78.

Erosheva, E. A. (2005). Comparing latent structures of the grade of membership,
Rasch, and latent class models. \emph{Psychometrika, 70}, 619-628.

Erosheva, E. A. (2006). \emph{Latent class representation of the grade of membership
model}. Seattle: University of Washington.

Erosheva, E. A., Fienberg, S. E., & Junker, B. W. (2002).
Alternative statistical models and representations for large sparse
multi-dimensional contingency tables.
\emph{Annales-Faculte Des Sciences Toulouse Mathematiques, 11},
485-505.

Erosheva, E. A., Fienberg, S. E., & Joutard, C. (2007).
Describing disability through individual-level mixture models
for multivariate binary data. \emph{Annals of Applied Statistics,
1}, 502-537.

Galyardt, A. (2015).
Interpreting mixed membership models: Implications of Erosheva's representation
theorem. In E. M. Airoldi, D. Blei, E. A. Erosheva, & S. E. Fienberg (Eds.).
\emph{Handbook of Mixed Membership Models} (pp. 39-65). Chapman & Hall.
}


%\author{
%Alexander Robitzsch
%}




%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For joint maximum likelihood estimation of the grade of membership model
see \code{\link{gom.jml}}.

See also the \pkg{mixedMem} package for estimating mixed membership
models by a variational EM algorithm.

The C code of Erosheva et al. (2007) can be downloaded from
\emph{http://projecteuclid.org/euclid.aoas/1196438029#supplemental}.

Code from Manrique-Vallier can be downloaded from
\emph{http://pages.iu.edu/~dmanriqu/software.html}.

See \emph{http://users.ics.aalto.fi/ella/publications/aspect_bernoulli.m}
for a Matlab implementation of the algorithm in Bingham, Kaban and
Fortelius (2009).
}
\examples{
#############################################################################
# EXAMPLE 1: PISA data mathematics
#############################################################################

data(data.pisaMath)
dat <- data.pisaMath$data
dat <- dat[, grep("M", colnames(dat)) ]

#***
# Model 1: Discrete GOM with 3 classes and 5 probability levels
problevels <- seq( 0, 1, len=5 )
mod1 <- sirt::gom.em( dat, K=3, problevels, model="GOM")
summary(mod1)

\dontrun{
#-- some plots

#* multivariate scatterplot
car::scatterplotMatrix(mod1$EAP, regLine=FALSE, smooth=FALSE, pch=16, cex=.4)
#* ternary plot
vcd::ternaryplot(mod1$EAP, pch=16, col=1, cex=.3)

#***
# Model 1a: Multivariate normal distribution
problevels <- seq( 0, 1, len=5 )
mod1a <- sirt::gom.em( dat, K=3, theta0.k=seq(-15,15,len=21), model="GOMnormal" )
summary(mod1a)

#***
# Model 2: Discrete GOM with 4 classes and 5 probability levels
problevels <- seq( 0, 1, len=5 )
mod2 <- sirt::gom.em( dat, K=4, problevels,  model="GOM"  )
summary(mod2)

# model comparison
smod1 <- IRT.modelfit(mod1)
smod2 <- IRT.modelfit(mod2)
IRT.compareModels(smod1,smod2)

#***
# Model 2a: Estimate discrete GOM with 4 classes and restricted space of probability levels
#  the 2nd, 4th and 6th class correspond to "intermediate stages"
problevels <- scan()
 1  0  0  0
.5 .5  0  0
 0  1  0  0
 0 .5 .5  0
 0  0  1  0
 0  0 .5 .5
 0  0  0  1

problevels <- matrix( problevels, ncol=4, byrow=TRUE)
mod2a <- sirt::gom.em( dat, K=4, problevels,  model="GOM" )
# probability distribution for latent classes
cbind( mod2a$theta.k, mod2a$pi.k )
  ##        [,1] [,2] [,3] [,4]       [,5]
  ##   [1,]  1.0  0.0  0.0  0.0 0.17214630
  ##   [2,]  0.5  0.5  0.0  0.0 0.04965676
  ##   [3,]  0.0  1.0  0.0  0.0 0.09336660
  ##   [4,]  0.0  0.5  0.5  0.0 0.06555719
  ##   [5,]  0.0  0.0  1.0  0.0 0.27523678
  ##   [6,]  0.0  0.0  0.5  0.5 0.08458620
  ##   [7,]  0.0  0.0  0.0  1.0 0.25945016
}

#***
# Model 3: Rasch GOM
mod3 <- sirt::gom.em( dat, model="GOMRasch", maxiter=20 )
summary(mod3)

#***
# Model 4: 'Ordinary' Rasch model
mod4 <- sirt::rasch.mml2( dat )
summary(mod4)

\dontrun{
#############################################################################
# EXAMPLE 2: Grade of membership model with 2 classes
#############################################################################

#********* DATASET 1 *************
# define an ordinary 2 latent class model
set.seed(8765)
I <- 10
prob.class1 <- stats::runif( I, 0, .35 )
prob.class2 <- stats::runif( I, .70, .95 )
probs <- cbind( prob.class1, prob.class2 )

# define classes
N <- 1000
latent.class <- c( rep( 1, 1/4*N ), rep( 2,3/4*N ) )

# simulate item responses
dat <- matrix( NA, nrow=N, ncol=I )
for (ii in 1:I){
    dat[,ii] <- probs[ ii, latent.class ]
    dat[,ii] <- 1 * ( stats::runif(N) < dat[,ii] )
}
colnames(dat) <- paste0( "I", 1:I)

# Model 1: estimate latent class model
mod1 <- sirt::gom.em(dat, K=2, problevels=c(0,1), model="GOM" )
summary(mod1)
# Model 2: estimate GOM
mod2 <- sirt::gom.em(dat, K=2, problevels=seq(0,1,0.5), model="GOM" )
summary(mod2)
# estimated distribution
cbind( mod2$theta.k, mod2$pi.k )
  ##       [,1] [,2]        [,3]
  ##  [1,]  1.0  0.0 0.243925644
  ##  [2,]  0.5  0.5 0.006534278
  ##  [3,]  0.0  1.0 0.749540078

#********* DATASET 2 *************
# define a 2-class model with graded membership
set.seed(8765)
I <- 10
prob.class1 <- stats::runif( I, 0, .35 )
prob.class2 <- stats::runif( I, .70, .95 )
prob.class3 <- .5*prob.class1+.5*prob.class2  # probabilities for 'fuzzy class'
probs <- cbind( prob.class1, prob.class2, prob.class3)
# define classes
N <- 1000
latent.class <- c( rep(1,round(1/3*N)),rep(2,round(1/2*N)),rep(3,round(1/6*N)))
# simulate item responses
dat <- matrix( NA, nrow=N, ncol=I )
for (ii in 1:I){
    dat[,ii] <- probs[ ii, latent.class ]
    dat[,ii] <- 1 * ( stats::runif(N) < dat[,ii] )
        }
colnames(dat) <- paste0( "I", 1:I)

#** Model 1: estimate latent class model
mod1 <- sirt::gom.em(dat, K=2, problevels=c(0,1), model="GOM" )
summary(mod1)

#** Model 2: estimate GOM
mod2 <- sirt::gom.em(dat, K=2, problevels=seq(0,1,0.5), model="GOM" )
summary(mod2)
# inspect distribution
cbind( mod2$theta.k, mod2$pi.k )
  ##       [,1] [,2]      [,3]
  ##  [1,]  1.0  0.0 0.3335666
  ##  [2,]  0.5  0.5 0.1810114
  ##  [3,]  0.0  1.0 0.4854220

#***
# Model2m: estimate discrete GOM in mirt
# define latent classes
Theta <- scan( nlines=1)
   1 0   .5 .5    0 1
Theta <- matrix( Theta, nrow=3, ncol=2,byrow=TRUE)
# define mirt model
I <- ncol(dat)
#*** create customized item response function for mirt model
name <- 'gom'
par <- c("a1"=-1, "a2"=1 )
est <- c(TRUE, TRUE)
P.gom <- function(par,Theta,ncat){
    # GOM for two extremal classes
    pext1 <- stats::plogis(par[1])
    pext2 <- stats::plogis(par[2])
    P1 <- Theta[,1]*pext1 + Theta[,2]*pext2
    cbind(1-P1, P1)
}
# create item response function
icc_gom <- mirt::createItem(name, par=par, est=est, P=P.gom)
#** define prior for latent class analysis
lca_prior <- function(Theta,Etable){
  # number of latent Theta classes
  TP <- nrow(Theta)
  # prior in initial iteration
  if ( is.null(Etable) ){ prior <- rep( 1/TP, TP ) }
  # process Etable (this is correct for datasets without missing data)
  if ( ! is.null(Etable) ){
    # sum over correct and incorrect expected responses
    prior <- ( rowSums(Etable[, seq(1,2*I,2)]) + rowSums(Etable[,seq(2,2*I,2)]) )/I
                 }
  prior <- prior / sum(prior)
  return(prior)
}
#*** estimate discrete GOM in mirt package
mod2m <- mirt::mirt(dat, 1, rep( "icc_gom",I), customItems=list("icc_gom"=icc_gom),
           technical=list( customTheta=Theta, customPriorFun=lca_prior)  )
# correct number of estimated parameters
mod2m@nest <- as.integer(sum(mod.pars$est) + nrow(Theta)-1 )
# extract log-likelihood and compute AIC and BIC
mod2m@logLik
( AIC <- -2*mod2m@logLik+2*mod2m@nest )
( BIC <- -2*mod2m@logLik+log(mod2m@Data$N)*mod2m@nest )
# extract coefficients
( cmod2m <- sirt::mirt.wrapper.coef(mod2m) )
# compare estimated distributions
round( cbind( "sirt"=mod2$pi.k, "mirt"=mod2m@Prior[[1]] ), 5 )
  ##           sirt    mirt
  ##   [1,] 0.33357 0.33627
  ##   [2,] 0.18101 0.17789
  ##   [3,] 0.48542 0.48584
# compare estimated item parameters
dfr <- data.frame( "sirt"=mod2$item[,4:5] )
dfr$mirt <- apply(cmod2m$coef[, c("a1", "a2") ], 2, stats::plogis )
round(dfr,4)
  ##      sirt.lam.Cl1 sirt.lam.Cl2 mirt.a1 mirt.a2
  ##   1        0.1157       0.8935  0.1177  0.8934
  ##   2        0.0790       0.8360  0.0804  0.8360
  ##   3        0.0743       0.8165  0.0760  0.8164
  ##   4        0.0398       0.8093  0.0414  0.8094
  ##   5        0.1273       0.7244  0.1289  0.7243
  ##   [...]

#############################################################################
# EXAMPLE 3: Lung cancer dataset; using sampling weights
#############################################################################

data(data.si08, package="sirt")
dat <- data.si08

#- Latent class model with 3 classes
problevels <- c(0,1)
mod1 <- sirt::gom.em( dat[,1:5], weights=dat$wgt, K=3, problevels=problevels )
summary(mod1)

#- Grade of membership model with discrete distribution
problevels <- seq(0,1,length=5)
mod2 <- sirt::gom.em( dat[,1:5], weights=dat$wgt, K=3, problevels=problevels )
summary(mod2)

#- Grade of membership model with multivariate normal distribution
mod3 <- sirt::gom.em( dat[,1:5], weights=dat$wgt, K=3, theta0.k=10*seq(-1,1,len=11),
            model="GOMnormal", optimizer="nlminb" )
summary(mod3)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Grade of membership model}
%% \keyword{Rasch grade of membership model}
%% \keyword{summary}

%% File Name: data.long.Rd
%% File Version: 0.29

\name{data.long}
\alias{data.long}
\docType{data}
\title{
Longitudinal Dataset
}
\description{
This dataset contains 200 observations on
12 items. 6 items (\code{I1T1}, ...,\code{I6T1})
were administered at measurement occasion T1
and 6 items at T2 (\code{I3T2}, ..., \code{I8T2}). There were 4 anchor items
which were presented at both time points.
The first column in the dataset contains the student identifier.
}
\usage{
data(data.long)
}

\format{
The format of the dataset is

\code{'data.frame':   200 obs. of  13 variables:} \cr
\code{ $ idstud: int  1001 1002 1003 1004 1005 1006 1007 1008 1009 1010 ...} \cr
\code{ $ I1T1  : int  1 1 1 1 1 1 1 0 1 1 ...} \cr
\code{ $ I2T1  : int  0 0 1 1 1 1 0 1 1 1 ...} \cr
\code{ $ I3T1  : int  1 0 1 1 0 1 0 0 0 0 ...} \cr
\code{ $ I4T1  : int  1 0 0 1 0 0 0 0 1 1 ...} \cr
\code{ $ I5T1  : int  1 0 0 1 0 0 0 0 1 0 ...} \cr
\code{ $ I6T1  : int  1 0 0 0 0 0 0 0 0 0 ...} \cr
\code{ $ I3T2  : int  1 1 0 0 1 1 1 1 0 1 ...} \cr
\code{ $ I4T2  : int  1 1 0 0 1 1 0 0 0 1 ...} \cr
\code{ $ I5T2  : int  1 0 1 1 1 1 1 0 1 1 ...} \cr
\code{ $ I6T2  : int  1 1 0 0 0 0 0 0 0 1 ...} \cr
\code{ $ I7T2  : int  1 0 0 0 0 0 0 0 0 1 ...} \cr
\code{ $ I8T2  : int  0 0 0 0 1 0 0 0 0 0 ...} \cr
}





\examples{
\dontrun{
data(data.long)
dat <- data.long
dat <- dat[,-1]
I <- ncol(dat)

#*************************************************
# Model 1: 2-dimensional Rasch model
#*************************************************
# define Q-matrix
Q <- matrix(0,I,2)
Q[1:6,1] <- 1
Q[7:12,2] <- 1
rownames(Q) <- colnames(dat)
colnames(Q) <- c("T1","T2")

# vector with same items
itemnr <- as.numeric( substring( colnames(dat),2,2) )
# fix mean at T2 to zero
mu.fixed <- cbind( 2,0 )

#--- M1a: rasch.mml2 (in sirt)
mod1a <- sirt::rasch.mml2(dat, Q=Q, est.b=itemnr, mu.fixed=mu.fixed)
summary(mod1a)

#--- M1b: smirt (in sirt)
mod1b <- sirt::smirt(dat, Qmatrix=Q, irtmodel="comp", est.b=itemnr,
                  mu.fixed=mu.fixed )

#--- M1c: tam.mml (in TAM)

# assume equal item difficulty of I3T1 and I3T2, I4T1 and I4T2, ...
# create draft design matrix and modify it
A <- TAM::designMatrices(resp=dat)$A
dimnames(A)[[1]] <- colnames(dat)
  ##   > str(A)
  ##    num [1:12, 1:2, 1:12] 0 0 0 0 0 0 0 0 0 0 ...
  ##    - attr(*, "dimnames")=List of 3
  ##     ..$ : chr [1:12] "Item01" "Item02" "Item03" "Item04" ...
  ##     ..$ : chr [1:2] "Category0" "Category1"
  ##     ..$ : chr [1:12] "I1T1" "I2T1" "I3T1" "I4T1" ...
A1 <- A[,, c(1:6, 11:12 ) ]
A1[7,2,3] <- -1     # difficulty(I3T1)=difficulty(I3T2)
A1[8,2,4] <- -1     # I4T1=I4T2
A1[9,2,5] <- A1[10,2,6] <- -1
dimnames(A1)[[3]] <- substring( dimnames(A1)[[3]],1,2)
  ##   > A1[,2,]
  ##        I1 I2 I3 I4 I5 I6 I7 I8
  ##   I1T1 -1  0  0  0  0  0  0  0
  ##   I2T1  0 -1  0  0  0  0  0  0
  ##   I3T1  0  0 -1  0  0  0  0  0
  ##   I4T1  0  0  0 -1  0  0  0  0
  ##   I5T1  0  0  0  0 -1  0  0  0
  ##   I6T1  0  0  0  0  0 -1  0  0
  ##   I3T2  0  0 -1  0  0  0  0  0
  ##   I4T2  0  0  0 -1  0  0  0  0
  ##   I5T2  0  0  0  0 -1  0  0  0
  ##   I6T2  0  0  0  0  0 -1  0  0
  ##   I7T2  0  0  0  0  0  0 -1  0
  ##   I8T2  0  0  0  0  0  0  0 -1

# estimate model
# set intercept of second dimension (T2) to zero
beta.fixed <- cbind( 1, 2, 0 )
mod1c <- TAM::tam.mml( resp=dat, Q=Q, A=A1, beta.fixed=beta.fixed)
summary(mod1c)

#*************************************************
# Model 2: 2-dimensional 2PL model
#*************************************************

# set variance at T2 to 1
variance.fixed <- cbind(2,2,1)

# M2a: rasch.mml2 (in sirt)
mod2a <- sirt::rasch.mml2(dat, Q=Q, est.b=itemnr, est.a=itemnr, mu.fixed=mu.fixed,
             variance.fixed=variance.fixed, mmliter=100)
summary(mod2a)

#*************************************************
# Model 3: Concurrent calibration by assuming invariant item parameters
#*************************************************

library(mirt)   # use mirt for concurrent calibration
data(data.long)
dat <- data.long[,-1]
I <- ncol(dat)

# create user defined function for between item dimensionality 4PL model
name <- "4PLbw"
par <- c("low"=0,"upp"=1,"a"=1,"d"=0,"dimItem"=1)
est <- c(TRUE, TRUE,TRUE,TRUE,FALSE)
# item response function
irf <- function(par,Theta,ncat){
     low <- par[1]
     upp <- par[2]
     a <- par[3]
     d <- par[4]
     dimItem <- par[5]
     P1 <- low + ( upp - low ) * plogis( a*Theta[,dimItem] + d )
     cbind(1-P1, P1)
}

# create item response function
fourPLbetw <- mirt::createItem(name, par=par, est=est, P=irf)
head(dat)

# create mirt model (use variable names in mirt.model)
mirtsyn <- "
     T1=I1T1,I2T1,I3T1,I4T1,I5T1,I6T1
     T2=I3T2,I4T2,I5T2,I6T2,I7T2,I8T2
     COV=T1*T2,,T2*T2
     MEAN=T1
     CONSTRAIN=(I3T1,I3T2,d),(I4T1,I4T2,d),(I5T1,I5T2,d),(I6T1,I6T2,d),
                 (I3T1,I3T2,a),(I4T1,I4T2,a),(I5T1,I5T2,a),(I6T1,I6T2,a)
        "
# create mirt model
mirtmodel <- mirt::mirt.model( mirtsyn, itemnames=colnames(dat) )
# define parameters to be estimated
mod3.pars <- mirt::mirt(dat, mirtmodel$model, rep( "4PLbw",I),
                   customItems=list("4PLbw"=fourPLbetw), pars="values")
# select dimensions
ind <- intersect( grep("T2",mod3.pars$item), which( mod3.pars$name=="dimItem" ) )
mod3.pars[ind,"value"] <- 2
# set item parameters low and upp to non-estimated
ind <- which( mod3.pars$name \%in\% c("low","upp") )
mod3.pars[ind,"est"] <- FALSE

# estimate 2PL model
mod3 <- mirt::mirt(dat, mirtmodel$model, itemtype=rep( "4PLbw",I),
                customItems=list("4PLbw"=fourPLbetw), pars=mod3.pars, verbose=TRUE,
                technical=list(NCYCLES=50)  )
mirt.wrapper.coef(mod3)

#****** estimate model in lavaan
library(lavaan)

# specify syntax
lavmodel <- "
             #**** T1
             F1=~ a1*I1T1+a2*I2T1+a3*I3T1+a4*I4T1+a5*I5T1+a6*I6T1
             I1T1 | b1*t1 ; I2T1 | b2*t1 ; I3T1 | b3*t1 ; I4T1 | b4*t1
             I5T1 | b5*t1 ; I6T1 | b6*t1
             F1 ~~ 1*F1
             #**** T2
             F2=~ a3*I3T2+a4*I4T2+a5*I5T2+a6*I6T2+a7*I7T2+a8*I8T2
             I3T2 | b3*t1 ; I4T2 | b4*t1 ; I5T2 | b5*t1 ; I6T2 | b6*t1
             I7T2 | b7*t1 ; I8T2 | b8*t1
             F2 ~~ NA*F2
             F2 ~ 1
             #*** covariance
             F1 ~~ F2
                "
# estimate model using theta parameterization
mod3lav <- lavaan::cfa( data=dat, model=lavmodel,
            std.lv=TRUE, ordered=colnames(dat), parameterization="theta")
summary(mod3lav, standardized=TRUE, fit.measures=TRUE, rsquare=TRUE)

#*************************************************
# Model 4: Linking with items of different item slope groups
#*************************************************

data(data.long)
dat <- data.long
# dataset for T1
dat1 <- dat[, grep( "T1", colnames(dat) ) ]
colnames(dat1) <- gsub("T1","", colnames(dat1) )
# dataset for T2
dat2 <- dat[, grep( "T2", colnames(dat) ) ]
colnames(dat2) <- gsub("T2","", colnames(dat2) )

# 2PL model with slope groups T1
mod1 <- sirt::rasch.mml2( dat1, est.a=c( rep(1,2), rep(2,4) ) )
summary(mod1)

# 2PL model with slope groups T2
mod2 <- sirt::rasch.mml2( dat2, est.a=c( rep(1,4), rep(2,2) ) )
summary(mod2)

#------- Link 1: Haberman Linking
# collect item parameters
dfr1 <- data.frame( "study1", mod1$item$item, mod1$item$a, mod1$item$b )
dfr2 <- data.frame( "study2", mod2$item$item, mod2$item$a, mod2$item$b )
colnames(dfr2) <- colnames(dfr1) <- c("study", "item", "a", "b" )
itempars <- rbind( dfr1, dfr2 )
# Linking
link1 <- sirt::linking.haberman(itempars=itempars)

#------- Link 2: Invariance alignment method
# create objects for invariance.alignment
nu <- rbind( c(mod1$item$thresh,NA,NA), c(NA,NA,mod2$item$thresh) )
lambda <- rbind( c(mod1$item$a,NA,NA), c(NA,NA,mod2$item$a ) )
colnames(lambda) <- colnames(nu) <- paste0("I",1:8)
rownames(lambda) <- rownames(nu) <- c("T1", "T2")
# Linking
link2a <- sirt::invariance.alignment( lambda, nu )
summary(link2a)
}
}
%% \keyword{datasets}

%% File Name: rasch.jml.biascorr.Rd
%% File Version: 0.23

\name{rasch.jml.biascorr}
\alias{rasch.jml.biascorr}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Bias Correction of Item Parameters for Joint Maximum Likelihood Estimation
in the Rasch model
}
\description{
This function computes an analytical bias correction for the Rasch
model according to the method of Arellano and Hahn (2007).
}
\usage{
rasch.jml.biascorr(jmlobj,itemfac=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{jmlobj}{
An object which is the output of the \code{\link{rasch.jml}} function
}
\item{itemfac}{Number of items which are used for bias correction.
By default it is the average number of item responses per person.
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list with following entries
\item{b.biascorr}{Matrix of item difficulty estimates. The column
\code{b.analytcorr1} contains item difficulties by analytical bias
correction of Method 1 in Arellano and Hahn (2007) whereas \code{b.analytcorr2}
corresponds to Method 2.}
\item{b.bias1}{Estimated bias by Method 1}
\item{b.bias2}{Estimated bias by Method 2}
\item{itemfac}{Number of items which are used as the factor for bias
    correction}
}
\references{
Arellano, M., & Hahn, J. (2007). Understanding bias in nonlinear panel
models: Some recent developments. In R. Blundell, W. Newey &
T. Persson (Eds.): \emph{Advances in Economics and
Econometrics, Ninth World Congress}, Cambridge University Press.
}

%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See \code{\link{rasch.jml.jackknife1}} for bias correction based on
Jackknife.

See also the \pkg{bife} \R package for analytical bias corrections.
}


\examples{
#############################################################################
# EXAMPLE 1: Dataset Reading
#############################################################################
data(data.read)
dat <- data( data.read )

# estimate Rasch model
mod <- sirt::rasch.jml( data.read  )

# JML with analytical bias correction
res1 <- sirt::rasch.jml.biascorr( jmlobj=mod  )
print( res1$b.biascorr, digits=3 )
  ##        b.JML b.JMLcorr b.analytcorr1 b.analytcorr2
  ##   1  -2.0086   -1.8412        -1.908        -1.922
  ##   2  -1.1121   -1.0194        -1.078        -1.088
  ##   3  -0.0718   -0.0658        -0.150        -0.127
  ##   4   0.5457    0.5002         0.393         0.431
  ##   5  -0.9504   -0.8712        -0.937        -0.936
  ##  [...]
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Joint maximum likelihood (JML)}
%\keyword{xxx}

%% File Name: mcmc.list.descriptives.Rd
%% File Version: 0.27

\name{mcmc.list.descriptives}
\alias{mcmc.list.descriptives}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Computation of Descriptive Statistics for a \code{mcmc.list} Object
}
\description{
Computation of descriptive statistics, Rhat convergence statistic
and MAP for a \code{mcmc.list} object. The Rhat statistic
is computed by splitting one Monte Carlo chain into three segments of equal
length. The MAP is the mode estimate of the posterior distribution which is
approximated by the mode of the kernel density estimate.
}
\usage{
mcmc.list.descriptives( mcmcobj, quantiles=c(.025,.05,.1,.5,.9,.95,.975) )
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{mcmcobj}{
Object of class \code{mcmc.list}
}
\item{quantiles}{Quantiles to be calculated for all parameters}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A data frame with descriptive statistics for all parameters in
the \code{mcmc.list} object.
}
%\references{
%% ~put references to the literature/web site here ~
%}

%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See \code{\link{mcmclist2coda}} for writing an object of class \code{mcmc.list}
into a coda file (see also the \pkg{coda} package).
}

\examples{
\dontrun{
miceadds::library_install("coda")
miceadds::library_install("R2WinBUGS")

#############################################################################
# EXAMPLE 1: Logistic regression
#############################################################################

#***************************************
# (1) simulate data
set.seed(8765)
N <- 500
x1 <- stats::rnorm(N)
x2 <- stats::rnorm(N)
y <- 1*( stats::plogis( -.6 + .7*x1 + 1.1 *x2 ) > stats::runif(N) )

#***************************************
# (2) estimate logistic regression with glm
mod <- stats::glm( y ~ x1 + x2, family="binomial" )
summary(mod)

#***************************************
# (3) estimate model with rcppbugs package
b <- rcppbugs::mcmc.normal( stats::rnorm(3),mu=0,tau=0.0001)
y.hat <- rcppbugs::deterministic(function(x1,x2,b) {
             stats::plogis( b[1] + b[2]*x1 + b[3]*x2 ) }, x1, x2, b)
y.lik <- rcppbugs::mcmc.bernoulli( y, p=y.hat, observed=TRUE)
m <- rcppbugs::create.model(b, y.hat, y.lik)

#*** estimate model in rcppbugs; 5000 iterations, 1000 burnin iterations
ans <- rcppbugs::run.model(m, iterations=5000, burn=1000, adapt=1000, thin=5)
print(rcppbugs::get.ar(ans))     # get acceptance rate
print(apply(ans[["b"]],2,mean))  # get means of posterior

#*** convert rcppbugs into mcmclist object
mcmcobj <- data.frame( ans$b  )
colnames(mcmcobj) <- paste0("b",1:3)
mcmcobj <- as.matrix(mcmcobj)
class(mcmcobj) <- "mcmc"
attr(mcmcobj, "mcpar") <- c( 1, nrow(mcmcobj), 1 )
mcmcobj <- coda::as.mcmc.list( mcmcobj )

# plot results
plot(mcmcobj)

# summary
summ1 <-  sirt::mcmc.list.descriptives( mcmcobj )
summ1
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.noharm.Rd
%% File Version: 2.11

\name{data.noharm}


\alias{data.noharm18}
\alias{data.noharmExC}


\docType{data}
\title{
Datasets for NOHARM Analysis
}
\description{
Datasets for analyses in NOHARM (see \code{\link{R2noharm}}).
}
\usage{
data(data.noharmExC)
data(data.noharm18)
}


\format{

\itemize{

\item  \code{data.noharmExC} \cr

The format of this dataset is

\code{'data.frame':   300 obs. of  8 variables:} \cr
\code{ $ C1: int  1 1 1 1 1 0 1 1 1 1 ...} \cr
\code{ $ C2: int  1 1 1 1 0 1 1 1 1 1 ...} \cr
\code{ $ C3: int  1 1 1 1 1 0 0 0 1 1 ...} \cr
\code{ $ C4: int  0 0 1 1 1 1 1 0 1 0 ...} \cr
\code{ $ C5: int  1 1 1 1 1 0 0 1 1 0 ...} \cr
\code{ $ C6: int  1 0 0 0 1 0 1 1 0 1 ...} \cr
\code{ $ C7: int  1 1 0 0 1 1 0 0 0 1 ...} \cr
\code{ $ C8: int  1 0 1 0 1 0 1 0 1 1 ...} \cr


\item \code{data.noharm18} \cr

A data frame with 200 observations on the following 18 variables \code{I01},
  ..., \code{I18}. The format is

\code{'data.frame':   200 obs. of  18 variables:} \cr
\code{ $ I01: int  1 1 1 1 1 0 1 1 0 1 ...} \cr
\code{ $ I02: int  1 1 0 1 1 0 1 1 1 1 ...} \cr
\code{ $ I03: int  1 0 0 1 0 0 1 1 0 1 ...} \cr
\code{ $ I04: int  0 1 0 1 0 0 0 1 1 1 ...} \cr
\code{ $ I05: int  1 0 0 0 1 0 1 1 0 1 ...} \cr
\code{ $ I06: int  1 1 0 1 0 0 1 1 0 1 ...} \cr
\code{ $ I07: int  1 1 1 1 0 1 1 1 1 1 ...} \cr
\code{ $ I08: int  1 1 1 1 1 1 1 1 0 1 ...} \cr
\code{ $ I09: int  1 1 1 1 0 0 1 1 0 1 ...} \cr
\code{ $ I10: int  1 0 0 1 1 0 1 1 0 1 ...} \cr
\code{ $ I11: int  1 1 1 1 0 0 1 1 0 1 ...} \cr
\code{ $ I12: int  0 0 0 0 0 1 0 0 0 0 ...} \cr
\code{ $ I13: int  1 1 1 1 0 1 1 0 1 1 ...} \cr
\code{ $ I14: int  1 1 1 0 1 0 1 1 0 1 ...} \cr
\code{ $ I15: int  1 1 1 0 0 1 1 1 0 1 ...} \cr
\code{ $ I16: int  1 1 0 1 1 0 1 0 1 1 ...} \cr
\code{ $ I17: int  0 1 0 0 0 0 1 1 0 1 ...} \cr
\code{ $ I18: int  0 0 0 0 0 0 0 0 1 0 ...} \cr
}
}
%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}
%\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%}
%\references{
%%  ~~ possibly secondary sources and usages ~~
%}



%\examples{
%data(data.noharm)
%## maybe str(data.noharm) ; plot(data.noharm) ...
%}
%% \keyword{datasets}

%% File Name: data.raw1.Rd
%% File Version: 0.07

\name{data.raw1}
\alias{data.raw1}
\docType{data}
\title{
Dataset with Raw Item Responses
}
\description{
Dataset with raw item responses
}
\usage{data(data.raw1)}
\format{
A data frame with raw item responses of 1200 persons on the following 77 items:

\code{'data.frame':   1200 obs. of  77 variables:} \cr
\code{ $ I101: num  0 0 0 2 0 0 0 0 0 0 ...} \cr
\code{ $ I102: int  NA NA 2 1 2 1 3 2 NA NA ...} \cr
\code{ $ I103: int  1 1 NA NA NA NA NA NA 1 1 ...} \cr
\code{ ...} \cr
\code{ $ I179: chr  "E" "C" "D" "E" ...} \cr

}
%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}
%\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%}
%\references{
%%  ~~ possibly secondary sources and usages ~~
%}




%\examples{
%data(data.raw1)
%## maybe str(data.raw1) ; plot(data.raw1) ...
%}
%% \keyword{datasets}

%% File Name: data.ml.Rd
%% File Version: 1.10

\name{data.ml}
\alias{data.ml}
\alias{data.ml1}
\alias{data.ml2}

\docType{data}
\title{
Multilevel Datasets
}
\description{
Datasets for conducting multilevel IRT analysis. This dataset is used
in the examples of the function \code{\link{mcmc.2pno.ml}}.
}
\usage{
data(data.ml1)
data(data.ml2)
}
\format{
\itemize{
\item  \code{data.ml1}

  A data frame with 2000 student observations in 100 classes on 17 variables.
  The first variable \code{group} contains the class identifier.
  The remaining 16 variables are dichotomous test items.

\code{'data.frame':   2000 obs. of  17 variables:} \cr
\code{ $ group: num  1001 1001 1001 1001 1001 ...} \cr
\code{ $ X1   : num  1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{ $ X2   : num  1 1 1 0 1 1 1 1 1 1 ...} \cr
\code{ $ X3   : num  0 1 1 0 1 0 1 0 1 0 ...} \cr
\code{ $ X4   : num  1 1 1 0 0 1 1 1 1 1 ...} \cr
\code{ $ X5   : num  0 0 0 1 1 1 0 0 1 1 ...} \cr
\code{[...]} \cr
\code{ $ X16  : num  0 0 1 0 0 0 1 0 0 0 ...} \cr


\item  \code{data.ml2}

  A data frame with 2000 student observations in 100 classes on 6 variables.
  The first variable \code{group} contains the class identifier.
  The remaining 5 variables are polytomous test items.

  \code{'data.frame':   2000 obs. of  6 variables:} \cr
\code{ $ group: num  1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{ $ X1   : num  2 3 4 3 3 3 1 4 4 3 ...} \cr
\code{ $ X2   : num  2 2 4 3 3 2 2 3 4 3 ...} \cr
\code{ $ X3   : num  3 4 5 4 2 3 3 4 4 2 ...} \cr
\code{ $ X4   : num  2 3 3 2 1 3 1 4 4 3 ...} \cr
\code{ $ X5   : num  2 3 3 2 3 3 1 3 2 2 ...} \cr

 }
}




%% \keyword{datasets}

%% File Name: data.ess2005.Rd
%% File Version: 0.11

\name{data.ess2005}
\alias{data.ess2005}
\docType{data}
\title{
Dataset European Social Survey 2005
}
\description{
This dataset contains item loadings \eqn{\lambda} and intercepts \eqn{\nu}
for 26 countries for the European Social Survey (ESS 2005;
see Asparouhov & Muthen, 2014).
}

\usage{
data(data.ess2005)
}

\format{
The format of the dataset is:

\code{List of 2} \cr
\code{ $ lambda: num [1:26, 1:4] 0.688 0.721 0.72 0.687 0.625 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : NULL} \cr
\code{  .. ..$ : chr [1:4] "ipfrule" "ipmodst" "ipbhprp" "imptrad"} \cr
\code{ $ nu    : num [1:26, 1:4] 3.26 2.52 3.41 2.84 2.79 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : NULL} \cr
\code{  .. ..$ : chr [1:4] "ipfrule" "ipmodst" "ipbhprp" "imptrad"} \cr

}

%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}

%\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%}

% \section{Example Index}{
% \code{\link{invariance.alignment}} (Example 3)
% }

\references{
Asparouhov, T., & Muthen, B. (2014). Multiple-group factor analysis alignment.
\emph{Structural Equation Modeling, 21}(4), 1-14.
\doi{10.1080/10705511.2014.919210}
}
%\examples{
%data(data.ess2005)
%## maybe str(data.ess2005) ; plot(data.ess2005) ...
%}


%% \keyword{datasets}

%% File Name: equating.rasch.jackknife.Rd
%% File Version: 0.24

\name{equating.rasch.jackknife}
\alias{equating.rasch.jackknife}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Jackknife Equating Error in Generalized Logistic Rasch Model
}
\description{
This function estimates the linking error in linking
based on Jackknife (Monseur & Berezner, 2007).
}
\usage{
equating.rasch.jackknife(pars.data, display=TRUE,
   se.linkerror=FALSE, alpha1=0, alpha2=0)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{pars.data}{
Data frame with four columns: jackknife unit (1st column),
item parameter study 1 (2nd column), item parameter study 2 (3rd column),
item (4th column)
}
  \item{display}{
Display progress?
}
  \item{se.linkerror}{
Compute standard error of the linking error
}
  \item{alpha1}{
Fixed \eqn{\alpha_1} parameter in the generalized item
response model
}
  \item{alpha2}{
Fixed \eqn{\alpha_2} parameter in the generalized item
response model
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list with following entries:
\item{pars.data}{Used item parameters}
\item{itemunits}{Used units for jackknife}
\item{descriptives}{Descriptives for Jackknife.
\code{linkingerror.jackknife} is the estimated linking error.}
}
\references{
Monseur, C., & Berezner, A. (2007). The computation of equating errors
in international surveys in education.
\emph{Journal of Applied Measurement, 8}, 323-335.
}

%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For more details on linking methods see \code{\link{equating.rasch}}.
}
\examples{
#############################################################################
# EXAMPLE 1: Linking errors PISA study
#############################################################################

data(data.pisaPars)
pars <- data.pisaPars

# Linking error: Jackknife unit is the testlet
vars <- c("testlet","study1","study2","item")
res1 <- sirt::equating.rasch.jackknife(pars[, vars])
res1$descriptives
  ##   N.items N.units      shift        SD linkerror.jackknife SE.SD.jackknife
  ## 1      25       8 0.09292838 0.1487387          0.04491197      0.03466309

# Linking error: Jackknife unit is the item
res2 <- sirt::equating.rasch.jackknife(pars[, vars ] )
res2$descriptives
  ##   N.items N.units      shift        SD linkerror.jackknife SE.SD.jackknife
  ## 1      25      25 0.09292838 0.1487387          0.02682839      0.02533327
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Equating}
%% \keyword{Linking}% __ONLY ONE__ keyword per line

%% File Name: plot.mcmc.sirt.Rd
%% File Version: 0.15

\name{plot.mcmc.sirt}
\alias{plot.mcmc.sirt}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Plot Function for Objects of Class \code{mcmc.sirt}
}
\description{
Plot function for objects of class \code{mcmc.sirt}. These objects are generated
by: \code{\link{mcmc.2pno}}, \code{\link{mcmc.2pnoh}},
\code{\link{mcmc.3pno.testlet}}, \code{\link{mcmc.2pno.ml}}
}
\usage{
\method{plot}{mcmc.sirt}( x, layout=1, conflevel=0.9, round.summ=3,
   lag.max=.1, col.smooth="red", lwd.smooth=2, col.ci="orange",
   cex.summ=1, ask=FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
Object of class \code{mcmc.sirt}
}
  \item{layout}{
Layout type. \code{layout=1} is the standard coda plot output,
\code{layout=2} gives a slightly different display.
}
  \item{conflevel}{
Confidence level (only applies to \code{layout=2})
}
  \item{round.summ}{
Number of digits to be rounded in summary (only applies to \code{layout=2})
}
  \item{lag.max}{
Maximum lag for autocorrelation plot (only applies to \code{layout=2}).
The default of .1 means that it is set to 1/10 of the number of iterations.
}
  \item{col.smooth}{
Color of smooth trend in traceplot (only applies to \code{layout=2})
}
  \item{lwd.smooth}{
Line type of smooth trend in traceplot (only applies to \code{layout=2})
}
  \item{col.ci}{
Color for displaying confidence interval (only applies to \code{layout=2})
}
  \item{cex.summ}{
Cex size for descriptive summary (only applies to \code{layout=2})
}
  \item{ask}{
Ask for a new plot (only applies to \code{layout=2})
}
  \item{\dots}{
Further arguments to be passed
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%}
%\references{
%% ~put references to the literature/web site here ~
%}

%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{mcmc.2pno}}, \code{\link{mcmc.2pnoh}},
\code{\link{mcmc.3pno.testlet}}, \code{\link{mcmc.2pno.ml}}
}
%\examples{
%##---- Should be DIRECTLY executable !! ----
%}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{plot}
% \keyword{methods}% __ONLY ONE__ keyword per line

%% File Name: prior_model_parse.Rd
%% File Version: 0.10

\name{prior_model_parse}
\alias{prior_model_parse}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Parsing a Prior Model
}
\description{
Parses a string specifying a prior model which is needed
for the \code{prior} argument in \code{LAM::amh}
}
\usage{
prior_model_parse(prior_model)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{prior_model}{
String specifying the prior conforming to \R syntax.
}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

\value{
List with specified prior distributions for parameters
as needed for the \code{prior} argument in \code{LAM::amh}
}
%

%\references{
%% ~put references to the literature/web site here ~
%}

%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{LAM::amh}
}

\examples{
#############################################################################
# EXAMPLE 1: Toy example prior distributions
#############################################################################

#*** define prior model as a string
prior_model <- "
  # prior distributions means
  mu1 ~ dnorm( NA, mean=0, sd=1 )
  mu2 ~ dnorm(NA)       # mean T2 and T3
  # prior distribution standard deviation
  sig1 ~ dunif(NA,0, max=10)
      "

#*** convert priors into a list
res <- sirt::prior_model_parse( prior_model )
str(res)
  ##  List of 3
  ##   $ mu1 :List of 2
  ##    ..$ : chr "dnorm"
  ##    ..$ :List of 3
  ##    .. ..$ NA  : num NA
  ##    .. ..$ mean: num 0
  ##    .. ..$ sd  : num 1
  ##   $ mu2 :List of 2
  ##    ..$ : chr "dnorm"
  ##    ..$ :List of 1
  ##    .. ..$ : num NA
  ##   $ sig1:List of 2
  ##    ..$ : chr "dunif"
  ##    ..$ :List of 3
  ##    .. ..$ NA : num NA
  ##    .. ..$ NA : num 0
  ##    .. ..$ max: num 10
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: stratified.cronbach.alpha.Rd
%% File Version: 0.296

\name{stratified.cronbach.alpha}
\alias{stratified.cronbach.alpha}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Stratified Cronbach's Alpha
}
\description{
This function computes the stratified Cronbach's Alpha
for composite scales (Cronbach, Schoenemann & McKie, 1965;
He, 2010; Meyer, 2010).
}
\usage{
stratified.cronbach.alpha(data, itemstrata=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
An \eqn{N \times I} data frame
}
  \item{itemstrata}{
A matrix with two columns defining the item stratification.
The first column contains the item names, the second column
the item stratification label (these can be integers).
The default \code{NULL} does only compute Cronbach's Alpha
for the whole scale.
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%}
\references{
Cronbach, L. J., Schoenemann, P., & McKie, D. (1965).
Alpha coefficient for stratified-parallel tests.
\emph{Educational and Psychological Measurement, 25}, 291-312.
\doi{10.1177/001316446502500201}

He, Q. (2010). \emph{Estimating the reliability of composite scores}.
Ofqual/10/4703. Coventry: The Office of Qualifications and Examinations Regulation.

Meyer, P. (2010). \emph{Reliability}. Cambridge: Oxford University Press.
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{
#############################################################################
# EXAMPLE 1: data.read
#############################################################################

data(data.read, package="sirt")
dat <- data.read
I <- ncol(dat)

# apply function without defining item strata
sirt::stratified.cronbach.alpha( data.read  )

# define item strata
itemstrata <- cbind( colnames(dat), substring( colnames(dat), 1,1 ) )
sirt::stratified.cronbach.alpha( dat, itemstrata=itemstrata )
  ##   scale  I alpha mean.tot var.tot alpha.stratified
  ## 1 total 12 0.677    8.680   5.668            0.703
  ## 2     A  4 0.545    2.616   1.381               NA
  ## 3     B  4 0.381    2.811   1.059               NA
  ## 4     C  4 0.640    3.253   1.107               NA

\dontrun{
#**************************
# reliability analysis in psych package
library(psych)
# Cronbach's alpha and item discriminations
psych::alpha(dat)
# McDonald's omega
psych::omega(dat, nfactors=1)     # 1 factor
  ##   Alpha:                 0.69
  ##   Omega Total            0.69
##=> Note that alpha in this function is the standardized Cronbach's
##     alpha, i.e. alpha computed for standardized variables.
psych::omega(dat, nfactors=2)     # 2 factors
  ##   Omega Total            0.72
psych::omega(dat, nfactors=3)     # 3 factors
  ##   Omega Total            0.74
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Reliability}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: dif.variance.Rd
%% File Version: 0.13

\name{dif.variance}
\alias{dif.variance}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
DIF Variance
}
\description{
This function calculates the variance of DIF effects, the so called
DIF variance (Longford, Holland & Thayer, 1993).
}
\usage{
dif.variance(dif, se.dif, items=paste("item", 1:length(dif), sep="") )
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dif}{
Vector of uniform DIF effects
}
  \item{se.dif}{
Standard error of uniform DIF effects
}
  \item{items}{
Optional vector of item names
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list with following entries
\item{weighted.DIFSD}{Weighted DIF standard deviation}
\item{unweigted.DIFSD}{DIF standard deviation}
\item{mean.se.dif}{Mean of standard errors of DIF effects}
\item{eb.dif}{Empirical Bayes estimates of DIF effects}
}
\references{
Longford, N. T., Holland, P. W., & Thayer, D. T. (1993).
Stability of the MH D-DIF statistics across populations.
In P. W. Holland & H. Wainer (Eds.). \emph{Differential
Item Functioning} (pp. 171-196). Hillsdale, NJ: Erlbaum.
}

%\author{
%Alexander Robitzsch
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See \code{\link{dif.logistic.regression}} for examples.
}
%\examples{
%% ### example
%}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Differential item functioning (DIF)}
%% \keyword{DIF variance}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: scale_group_means.Rd
%% File Version: 0.08

\name{scale_group_means}
\alias{scale_group_means}
\alias{predict_scale_group_means}

%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Scaling of Group Means and Standard Deviations
}

\description{
Scales a vector of means and standard deviations containing group values.
}


\usage{
scale_group_means(M, SD, probs=NULL, M_target=0, SD_target=1)

## predict method
predict_scale_group_means(object, M, SD)
}
%- maybe also 'usage' for other objects documented here.

\arguments{
\item{M}{Vector of means}
\item{SD}{Vector of standard deviations}
\item{probs}{Optional vector containing probabilities}
 \item{M_target}{Target value for mean}
 \item{SD_target}{Target value for standard deviation}
 \item{object}{Fitted object from \code{scale_group_means}}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

\value{
List with entries
  \item{M1}{total mean}
  \item{SD1}{total standard deviation}
\item{M_z}{standardized means}
\item{SD_z}{standardized standard deviations}
\item{M_trafo}{transformed means}
\item{SD_trafo}{transformed standard deviations}
}

%% list(M1=M1, SD1=SD1, M_z=M_z, SD_z=SD_z)

%\references{
%% ~put references to the literature/web site here ~
%}
%\author{
%Alexander Robitzsch
%}

%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~


%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}

\examples{
#############################################################################
# EXAMPLE 1: Toy example
#############################################################################

M <- c(-.03, .18, -.23, -.15, .29)
SD <- c(.97, 1.13, .77, 1.05, 1.17)
sirt::scale_group_means(M=M, SD=SD)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Utilities}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: testlet.marginalized.Rd
%% File Version: 0.24

\name{testlet.marginalized}
\alias{testlet.marginalized}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Marginal Item Parameters from a Testlet (Bifactor) Model
}
\description{
This function computes marginal item parameters of a general
factor if item parameters from a testlet (bifactor) model are
provided as an input (see Details).
}


\usage{
testlet.marginalized(tam.fa.obj=NULL,a1=NULL, d1=NULL, testlet=NULL,
      a.testlet=NULL, var.testlet=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{tam.fa.obj}{Optional object of class \code{tam.fa}
    generated by \code{\link[TAM:tam.fa]{TAM::tam.fa}} from
    the \pkg{TAM} package. }
  \item{a1}{
Vector of item discriminations of general factor
}
  \item{d1}{
Vector of item intercepts of general factor
}
  \item{testlet}{
Integer vector of testlet (bifactor) identifiers (must be integers
between 1 to \eqn{T}).
}
  \item{a.testlet}{
Vector of testlet (bifactor) item discriminations
}
  \item{var.testlet}{
Vector of testlet (bifactor) variances
}
}

\details{
A testlet (bifactor) model is assumed to be estimated:
\deqn{P(X_{pit}=1 | \theta_{p}, u_{pt} )=
invlogit( a_{i1} \theta_p + a_t u_{pt} - d_{i} ) }
with \eqn{Var( u_{pt} )=\sigma_t^2 }. This multidimensional
item response model with locally independent items is
equivalent to a unidimensional IRT model with locally
dependent items (Ip, 2010). Marginal item parameters \eqn{a_i^\ast}
and \eqn{d_i^\ast} are obtained according to the response
equation \deqn{P(X_{pit}=1 | \theta_{p}^\ast  )=
invlogit( a_{i}^\ast \theta_p^\ast - d_{i}^\ast ) }
Calculation details can be found in Ip (2010).
}

\value{
A data frame containing all input item parameters and
marginal item intercept \eqn{d_i^\ast} (\code{d1_marg}) and
marginal item slope \eqn{a_i^\ast} (\code{a1_marg}).
}

\references{
Ip, E. H. (2010). Empirically indistinguishable multidimensional
IRT and locally dependent unidimensional item response models.
\emph{British Journal of Mathematical and Statistical Psychology,
63}, 395-416.
}


%\author{
%Alexander Robitzsch
%}



%%\note{
%%  ~~further notes~~
%% }

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For estimating a testlet (bifactor) model see
\code{\link[TAM:tam.fa]{TAM::tam.fa}}.
}

\examples{
#############################################################################
# EXAMPLE 1: Small numeric example for Rasch testlet model
#############################################################################

# Rasch testlet model with 9 items contained into 3 testlets
# the third testlet has essentially no dependence and therefore
# no testlet variance
testlet <- rep( 1:3, each=3 )
a1 <- rep(1, 9 )   # item slopes first dimension
d1 <- rep( c(-1.25,0,1.5), 3 ) # item intercepts
a.testlet <- rep( 1, 9 )  # item slopes testlets
var.testlet <- c( .8, .2, 0 )  # testlet variances

# apply function
res <- sirt::testlet.marginalized( a1=a1, d1=d1, testlet=testlet,
            a.testlet=a.testlet, var.testlet=var.testlet )
round( res, 2 )
  ##    item testlet a1    d1 a.testlet var.testlet a1_marg d1_marg
  ##  1    1       1  1 -1.25         1         0.8    0.89   -1.11
  ##  2    2       1  1  0.00         1         0.8    0.89    0.00
  ##  3    3       1  1  1.50         1         0.8    0.89    1.33
  ##  4    4       2  1 -1.25         1         0.2    0.97   -1.21
  ##  5    5       2  1  0.00         1         0.2    0.97    0.00
  ##  6    6       2  1  1.50         1         0.2    0.97    1.45
  ##  7    7       3  1 -1.25         1         0.0    1.00   -1.25
  ##  8    8       3  1  0.00         1         0.0    1.00    0.00
  ##  9    9       3  1  1.50         1         0.0    1.00    1.50

\dontrun{
#############################################################################
# EXAMPLE 2: Dataset reading
#############################################################################

library(TAM)
data(data.read)
resp <- data.read
maxiter <-  100

# Model 1: Rasch testlet model with 3 testlets
dims <- substring( colnames(resp),1,1 )  # define dimensions
mod1 <- TAM::tam.fa( resp=resp, irtmodel="bifactor1", dims=dims,
               control=list(maxiter=maxiter) )
# marginal item parameters
res1 <- sirt::testlet.marginalized( mod1 )

#***
# Model 2: estimate bifactor model but assume that items 3 and 5 do not load on
#           specific factors
dims1 <- dims
dims1[c(3,5)] <- NA
mod2 <- TAM::tam.fa( resp=resp, irtmodel="bifactor2", dims=dims1,
              control=list(maxiter=maxiter) )
res2 <- sirt::testlet.marginalized( mod2 )
res2
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Testlet model}
%% \keyword{Bifactor model}% __ONLY ONE__ keyword per line

%% File Name: mirt.specify.partable.Rd
%% File Version: 0.09

\name{mirt.specify.partable}
\alias{mirt.specify.partable}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Specify or modify a Parameter Table in \pkg{mirt}
}

\description{
Specify or modify a parameter table in \pkg{mirt}.
}

\usage{
mirt.specify.partable(mirt.partable, parlist, verbose=TRUE)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{mirt.partable}{
Parameter table in \pkg{mirt} package
}
  \item{parlist}{
List of parameters which are used for specification in the parameter
table. See Examples.
  }
  \item{verbose}{
An optional logical indicating whether the some warnings
should be printed.
}

}

%\details{
%}

\value{
A modified parameter table
}

%\references{
%% ~put references to the literature/web site here ~
%}

\author{
Alexander Robitzsch, Phil Chalmers
}

%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%See \url{http://lavaan.ugent.be/} for \pkg{lavaan} resources.
%}

\examples{
#############################################################################
# EXAMPLE 1: Modifying a parameter table for single group
#############################################################################

library(mirt)
data(LSAT7,package="mirt")
data <- mirt::expand.table(LSAT7)

mirt.partable <- mirt::mirt(data, 1, pars="values")
colnames(mirt.partable)
## > colnames(mirt.partable) [1] 'group' 'item' 'class' 'name' 'parnum' 'value'
##   'lbound' 'ubound' 'est' 'prior.type' 'prior_1' 'prior_2'

# specify some values of item parameters
value <- data.frame(d=c(0.7, -1, NA), a1=c(1, 1.2, 1.3), g=c(NA, 0.25, 0.25))
rownames(value) <- c("Item.1", "Item.4", "Item.3")

# fix some item paramters
est1 <- data.frame(d=c(TRUE, NA), a1=c(FALSE, TRUE))
rownames(est1) <- c("Item.4", "Item.3")

# estimate all guessing parameters
est2 <- data.frame(g=rep(TRUE, 5))
rownames(est2) <- colnames(data)

# prior distributions
prior.type <- data.frame(g=rep("norm", 4))
rownames(prior.type) <- c("Item.1", "Item.2", "Item.4", "Item.5")
prior_1 <- data.frame(g=rep(-1.38, 4))
rownames(prior_1) <- c("Item.1", "Item.2", "Item.4", "Item.5")
prior_2 <- data.frame(g=rep(0.5, 4))
rownames(prior_2) <- c("Item.1", "Item.2", "Item.4", "Item.5")

# misspecify some entries
rownames(prior_2)[c(3,2)] <- c("A", "B")
rownames(est1)[2] <- c("B")

# define complete list with parameter specification
parlist <- list(value=value, est=est1, est=est2, prior.type=prior.type,
      prior_1=prior_1, prior_2=prior_2)

# modify parameter table
mirt.specify.partable(mirt.partable, parlist)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{mirt}
% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: mcmc_coef.Rd
%% File Version: 0.13

\name{mcmc_coef}
\alias{mcmc_coef}
\alias{mcmc_vcov}
\alias{mcmc_confint}
\alias{mcmc_plot}
\alias{mcmc_summary}
\alias{mcmc_derivedPars}
\alias{mcmc_WaldTest}
\alias{summary.mcmc_WaldTest}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Some Methods for Objects of Class \code{mcmc.list}
}

\description{
Some methods for objects of class \code{mcmc.list} created
from the \pkg{coda} package.
}

\usage{
## coefficients
mcmc_coef(mcmcobj, exclude="deviance")

## covariance matrix
mcmc_vcov(mcmcobj, exclude="deviance")

## confidence interval
mcmc_confint( mcmcobj, parm, level=.95, exclude="deviance" )

## summary function
mcmc_summary( mcmcobj, quantiles=c(.025,.05,.50,.95,.975) )

## plot function
mcmc_plot(mcmcobj, ...)

## inclusion of derived parameters in mcmc object
mcmc_derivedPars( mcmcobj, derivedPars )

## Wald test for parameters
mcmc_WaldTest( mcmcobj, hypotheses )

\method{summary}{mcmc_WaldTest}(object, digits=3, ...)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{mcmcobj}{
Objects of class \code{mcmc.list} as created by
\code{\link[coda:mcmc]{coda::mcmc}}
}
  \item{exclude}{
Vector of parameters which should be excluded in calculations
}
\item{parm}{Optional vector of parameters}
\item{level}{Confidence level}
\item{quantiles}{Vector of quantiles to be computed.}
\item{\dots}{Parameters to be passed to \code{mcmc_plot}.
See \code{LAM::plot.amh} for arguments.
}
\item{derivedPars}{List with derived parameters (see examples).}
\item{hypotheses}{List with hypotheses of the form
\eqn{g_i( \bold{\theta})=0}.}
\item{object}{Object of class \code{mcmc_WaldTest}.}
\item{digits}{Number of digits used for rounding.}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

%\value{
%}
%\references{
%% ~put references to the literature/web site here ~
%}


%\author{
%Alexander Robitzsch
%}




%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link[coda:mcmc]{coda::mcmc}}
}

\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Logistic regression in rcppbugs package
#############################################################################


#***************************************
# (1) simulate data
set.seed(8765)
N <- 500
x1 <- stats::rnorm(N)
x2 <- stats::rnorm(N)
y <- 1*( stats::plogis( -.6 + .7*x1 + 1.1 *x2 ) > stats::runif(N) )

#***************************************
# (2) estimate logistic regression with glm
mod <- stats::glm( y ~ x1 + x2, family="binomial" )
summary(mod)

#***************************************
# (3) estimate model with rcppbugs package
library(rcppbugs)
b <- rcppbugs::mcmc.normal( stats::rnorm(3),mu=0,tau=0.0001)
y.hat <- rcppbugs::deterministic( function(x1,x2,b){
                stats::plogis( b[1] + b[2]*x1 + b[3]*x2 ) },
                  x1, x2, b)
y.lik <- rcppbugs::mcmc.bernoulli( y, p=y.hat, observed=TRUE)
model <- rcppbugs::create.model(b, y.hat, y.lik)

#*** estimate model in rcppbugs; 5000 iterations, 1000 burnin iterations
n.burnin <- 500 ; n.iter <- 2000 ; thin <- 2
ans <- rcppbugs::run.model(model, iterations=n.iter, burn=n.burnin, adapt=200, thin=thin)
print(rcppbugs::get.ar(ans)) # get acceptance rate
print(apply(ans[["b"]],2,mean)) # get means of posterior

#*** convert rcppbugs into mcmclist object
mcmcobj <- data.frame( ans$b )
colnames(mcmcobj) <- paste0("b",1:3)
mcmcobj <- as.matrix(mcmcobj)
class(mcmcobj) <- "mcmc"
attr(mcmcobj, "mcpar") <- c( n.burnin+1, n.iter, thin )
mcmcobj <- coda::mcmc( mcmcobj )

# coefficients, variance covariance matrix and confidence interval
mcmc_coef(mcmcobj)
mcmc_vcov(mcmcobj)
mcmc_confint( mcmcobj, level=.90 )

# summary and plot
mcmc_summary(mcmcobj)
mcmc_plot(mcmcobj, ask=TRUE)

# include derived parameters in mcmc object
derivedPars <- list( "diff12"=~ I(b2-b1), "diff13"=~ I(b3-b1) )
mcmcobj2 <- sirt::mcmc_derivedPars(mcmcobj, derivedPars=derivedPars )
mcmc_summary(mcmcobj2)

#*** Wald test for parameters
 # hyp1: b2 - 0.5=0
 # hyp2: b2 * b3=0
hypotheses <- list( "hyp1"=~ I( b2 - .5 ), "hyp2"=~ I( b2*b3 ) )
test1 <- sirt::mcmc_WaldTest( mcmcobj, hypotheses=hypotheses )
summary(test1)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: fuzcluster.Rd
%% File Version: 0.24

\name{fuzcluster}
\alias{fuzcluster}
\alias{summary.fuzcluster}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Clustering for Continuous Fuzzy Data
}
\description{
This function performs clustering for continuous fuzzy data
for which membership functions are assumed to be Gaussian
(Denoeux, 2013). The mixture is also assumed to be Gaussian and
(conditionally cluster membership) independent.
}
\usage{
fuzcluster(dat_m, dat_s, K=2, nstarts=7, seed=NULL, maxiter=100,
     parmconv=0.001, fac.oldxsi=0.75, progress=TRUE)

\method{summary}{fuzcluster}(object,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat_m}{
Centers for individual item specific membership functions
}
  \item{dat_s}{
Standard deviations for individual item specific membership functions
}
  \item{K}{
Number of latent classes
}
  \item{nstarts}{
Number of random starts. The default is 7 random starts.
}
  \item{seed}{
Simulation seed. If one value is provided, then only one
start is performed.
}
  \item{maxiter}{
Maximum number of iterations
}
  \item{parmconv}{
Maximum absolute change in parameters
}
\item{fac.oldxsi}{Convergence acceleration factor which
should take values between 0 and 1. The default is 0.75.}
  \item{progress}{
An optional logical indicating whether iteration progress
should be displayed.
}
\item{object}{Object of class \code{fuzcluster}}
\item{\dots}{Further arguments to be passed}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}


\value{
A list with following entries

\item{deviance}{Deviance}
\item{iter}{Number of iterations}
\item{pi_est}{Estimated class probabilities}
\item{mu_est}{Cluster means}
\item{sd_est}{Cluster standard deviations}
\item{posterior}{Individual posterior distributions of cluster membership}
\item{seed}{Simulation seed for cluster solution}
\item{ic}{Information criteria}
}

\references{
Denoeux, T. (2013). Maximum likelihood estimation from uncertain data
in the belief function framework.
\emph{IEEE Transactions on Knowledge and Data Engineering, 25}, 119-130.
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See \code{\link{fuzdiscr}} for estimating discrete distributions for
fuzzy data.

See the \pkg{fclust} package for fuzzy clustering.
}


\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: 2 classes and 3 items
#############################################################################

#*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
# simulate data (2 classes and 3 items)
set.seed(876)
library(mvtnorm)
Ntot <- 1000  # number of subjects
# define SDs for simulating uncertainty
sd_uncertain <- c( .2, 1, 2 )

dat_m <- NULL   # data frame containing mean of membership function
dat_s <- NULL   # data frame containing SD of membership function

# *** Class 1
pi_class <- .6
Nclass <- Ntot * pi_class
mu <- c(3,1,0)
Sigma <- diag(3)
# simulate data
dat_m1 <- mvtnorm::rmvnorm( Nclass, mean=mu, sigma=Sigma )
dat_s1 <- matrix( stats::runif( Nclass * 3 ), nrow=Nclass )
for ( ii in 1:3){ dat_s1[,ii] <- dat_s1[,ii] * sd_uncertain[ii] }
dat_m <- rbind( dat_m, dat_m1 )
dat_s <- rbind( dat_s, dat_s1 )

# *** Class 2
pi_class <- .4
Nclass <- Ntot * pi_class
mu <- c(0,-2,0.4)
Sigma <- diag(c(0.5, 2, 2 ) )
# simulate data
dat_m1 <- mvtnorm::rmvnorm( Nclass, mean=mu, sigma=Sigma )
dat_s1 <- matrix( stats::runif( Nclass * 3 ), nrow=Nclass )
for ( ii in 1:3){ dat_s1[,ii] <- dat_s1[,ii] * sd_uncertain[ii] }
dat_m <- rbind( dat_m, dat_m1 )
dat_s <- rbind( dat_s, dat_s1 )
colnames(dat_s) <- colnames(dat_m) <- paste0("I", 1:3 )

#*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
# estimation

#*** Model 1: Clustering with 8 random starts
res1 <- sirt::fuzcluster(K=2,dat_m, dat_s, nstarts=8, maxiter=25)
summary(res1)
  ##  Number of iterations=22 (Seed=5090 )
  ##  ---------------------------------------------------
  ##  Class probabilities (2 Classes)
  ##  [1] 0.4083 0.5917
  ##
  ##  Means
  ##           I1      I2     I3
  ##  [1,] 0.0595 -1.9070 0.4011
  ##  [2,] 3.0682  1.0233 0.0359
  ##
  ##  Standard deviations
  ##         [,1]   [,2]   [,3]
  ##  [1,] 0.7238 1.3712 1.2647
  ##  [2,] 0.9740 0.8500 0.7523

#*** Model 2: Clustering with one start with seed 4550
res2 <- sirt::fuzcluster(K=2,dat_m, dat_s, nstarts=1, seed=5090 )
summary(res2)

#*** Model 3: Clustering for crisp data
#             (assuming no uncertainty, i.e. dat_s=0)
res3 <- sirt::fuzcluster(K=2,dat_m, dat_s=0*dat_s, nstarts=30, maxiter=25)
summary(res3)
  ##  Class probabilities (2 Classes)
  ##  [1] 0.3645 0.6355
  ##
  ##  Means
  ##           I1      I2      I3
  ##  [1,] 0.0463 -1.9221  0.4481
  ##  [2,] 3.0527  1.0241 -0.0008
  ##
  ##  Standard deviations
  ##         [,1]   [,2]   [,3]
  ##  [1,] 0.7261 1.4541 1.4586
  ##  [2,] 0.9933 0.9592 0.9535

#*** Model 4: kmeans cluster analysis
res4 <- stats::kmeans( dat_m, centers=2 )
  ##   K-means clustering with 2 clusters of sizes 607, 393
  ##   Cluster means:
  ##             I1        I2          I3
  ##   1 3.01550780  1.035848 -0.01662275
  ##   2 0.03448309 -2.008209  0.48295067
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Fuzzy data}
%% \keyword{Clustering}
%% \keyword{summary}% __ONLY ONE__ keyword per line

%% File Name: lsem.permutationTest.Rd
%% File Version: 0.334

\name{lsem.permutationTest}
\alias{lsem.permutationTest}
\alias{summary.lsem.permutationTest}
\alias{plot.lsem.permutationTest}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Permutation Test for a Local Structural Equation Model
}

\description{
Performs a permutation test for testing the hypothesis that model
parameter are independent of a moderator variable (see Hildebrandt,
Wilhelm, & Robitzsch, 2009; Hildebrandt, Luedtke, Robitzsch, Sommer, & Wilhelm, 2016).
}


\usage{
lsem.permutationTest(lsem.object, B=1000, residualize=TRUE, verbose=TRUE,
     n.core=1, cl.type="PSOCK")

\method{summary}{lsem.permutationTest}(object, file=NULL, digits=3, ...)

\method{plot}{lsem.permutationTest}(x, type="global", stattype="SD",
    parindex=NULL, sig_add=TRUE, sig_level=0.05, sig_pch=17, nonsig_pch=2,
    sig_cex=1, sig_lab="p value",  stat_lab="Test statistic",
    moderator_lab=NULL, digits=3, title=NULL, parlabels=NULL,
    ask=TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{lsem.object}{
Fitted object of class \code{lsem} with \code{\link{lsem.estimate}}
}
  \item{B}{
Number of permutation samples
}
  \item{residualize}{
Optional logical indicating whether residualization of the moderator
should be performed for each permutation sample.
}
  \item{verbose}{
Optional logical printing information about computation progress.
}
\item{n.core}{A scalar indicating the number of cores that should be used.}
\item{cl.type}{The cluster type.
Default value is \code{"PSOCK"}. Posix machines (Linux, Mac) generally benefit
from much faster cluster computation if type is set to \code{type="FORK"}.
}
\item{object}{Object of class \code{lsem}}
\item{file}{A file name in which the summary output will be written.}
\item{digits}{Number of digits.}
\item{\dots}{Further arguments to be passed.}
  \item{x}{
Object of class \code{lsem}
}
  \item{type}{
Type of the statistic to be plotted. If \code{type="global"}, a global
test will be displayed. If \code{type="pointwise"} for each value at the
focal point (defined in \code{moderator.grid}) are calculated.
}
  \item{stattype}{
Type of test statistics. Can be \code{MAD} (mean absolute deviation),
\code{SD} (standard deviation) or \code{lin_slo} (linear slope).
}
  \item{parindex}{
Vector of indices of selected parameters.
}
  \item{sig_add}{
Logical indicating whether significance values (p values) should
be displayed.
}
  \item{sig_level}{
Significance level.
}
  \item{sig_pch}{
Point symbol for significant values.
}
  \item{nonsig_pch}{
Point symbol for non-significant values.
}
\item{sig_cex}{Point size for graphic displaying p values}
  \item{sig_lab}{
Label for significance value (p value).
}
\item{stat_lab}{Label of y axis for graphic with pointwise test statistic}
  \item{moderator_lab}{
Label of the moderator.
}
  \item{title}{
Title of the plot. Can be a vector.
}
  \item{parlabels}{
Labels of the parameters. Can be a vector.
}
\item{ask}{A logical which asks for changing the graphic for each parameter.}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

\value{
List with following entries
\item{teststat}{Data frame with global test statistics. The statistics
are \code{SD}, \code{MAD} and \code{lin_slo} with their corresponding
p values.}
\item{parameters_pointwise_test}{Data frame with pointwise test statistics.}
\item{parameters}{Original parameters.}
\item{parameters}{Parameters in permutation samples.}
\item{parameters_summary}{Original parameter summary.}
\item{parameters_summary_M}{Mean of each parameter in permutation sample.}
\item{parameters_summary_SD}{Standard deviation (SD) statistic in permutation slope.}
\item{parameters_summary_MAD}{Mean absolute deviation (MAD)
    statistic in permutation sample.}
\item{parameters_summary_MAD}{Linear slope parameter in permutation sample.}
\item{nonconverged_rate}{Percentage of permuted dataset in which a LSEM model
did not converge}
}

%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...


\references{
Hildebrandt, A., Luedtke, O., Robitzsch, A., Sommer, C., &
Wilhelm, O. (2016). Exploring factor model parameters across continuous variables
with local structural equation models.
\emph{Multivariate Behavioral Research, 51}(2-3), 257-278.
\doi{10.1080/00273171.2016.1142856}

Hildebrandt, A., Wilhelm, O., & Robitzsch, A. (2009). Complementary and
competing factor analytic approaches for the investigation of measurement invariance.
\emph{Review of Psychology, 16}, 87-102.
}

\author{
Alexander Robitzsch, Oliver Luedtke, Andrea Hildebrandt
}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For Examples see \code{\link{lsem.estimate}}.
}

%\examples{
%}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{LSEM}
% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: truescore.irt.Rd
%% File Version: 0.23

\name{truescore.irt}
\alias{truescore.irt}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Conversion of Trait Scores \eqn{\theta} into
True Scores \eqn{\tau ( \theta )}
}
\description{
This function computes the true score
\eqn{\tau=\tau(\theta)=\sum_{i=1}^I P_i(\theta)}
in a unidimensional item response model with \eqn{I} items. In addition, it also
transforms conditional standard errors if they are provided.
}
\usage{
truescore.irt(A, B, c=NULL, d=NULL, theta=seq(-3, 3, len=21),
    error=NULL, pid=NULL, h=0.001)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{A}{
Matrix or vector of item slopes. See Examples for polytomous responses.
}
  \item{B}{
Matrix or vector of item intercepts. Note that the entries in
\code{B} refer to item intercepts and not to item difficulties.
}
  \item{c}{
Optional vector of guessing parameters
}
  \item{d}{
Optional vector of slipping parameters
}
  \item{theta}{
Vector of trait values
}
  \item{error}{
Optional vector of standard errors of trait
}
  \item{pid}{
Optional vector of person identifiers
}
  \item{h}{
Numerical differentiation parameter
}
}

\details{
In addition, the function \eqn{\pi(\theta)=\frac{1}{I} \cdot \tau( \theta)}
of the expected percent score is approximated by a logistic function
\deqn{ \pi ( \theta ) \approx l + ( u - l ) \cdot invlogit ( a \theta + b )
}
}

\value{
A data frame with following columns:
\item{truescore}{True scores \eqn{\tau=\tau ( \theta )}}
\item{truescore.error}{Standard errors of true scores}
\item{percscore}{Expected correct scores which is \eqn{\tau}
divided by the maximum true score}
\item{percscore.error}{Standard errors of expected correct scores}
\item{lower}{The \eqn{l} parameter}
\item{upper}{The \eqn{u} parameter}
\item{a}{The \eqn{a} parameter}
\item{b}{The \eqn{b} parameter}
}

%\references{
%% ~put references to the literature/web site here ~
%}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%}

\examples{
#############################################################################
# EXAMPLE 1: Dataset with mixed dichotomous and polytomous responses
#############################################################################

data(data.mixed1)
dat <- data.mixed1

#****
# Model 1: Partial credit model
# estimate model with TAM package
library(TAM)
mod1 <- TAM::tam.mml( dat )
# estimate person parameter estimates
wmod1 <- TAM::tam.wle( mod1 )
wmod1 <- wmod1[ order(wmod1$theta), ]
# extract item parameters
A <- mod1$B[,-1,1]
B <- mod1$AXsi[,-1]
# person parameters and standard errors
theta <- wmod1$theta
error <- wmod1$error

# estimate true score transformation
dfr <- sirt::truescore.irt( A=A, B=B, theta=theta, error=error )

# plot different person parameter estimates and standard errors
par(mfrow=c(2,2))
plot( theta, dfr$truescore, pch=16, cex=.6, xlab=expression(theta), type="l",
    ylab=expression(paste( tau, "(",theta, ")" )), main="True Score Transformation" )
plot( theta, dfr$percscore, pch=16, cex=.6, xlab=expression(theta), type="l",
    ylab=expression(paste( pi, "(",theta, ")" )), main="Percent Score Transformation" )
points( theta, dfr$lower + (dfr$upper-dfr$lower)*
                stats::plogis(dfr$a*theta+dfr$b), col=2, lty=2)
plot( theta, error, pch=16, cex=.6, xlab=expression(theta), type="l",
    ylab=expression(paste("SE(",theta, ")" )), main="Standard Error Theta" )
plot( dfr$truescore, dfr$truescore.error, pch=16, cex=.6, xlab=expression(tau),
    ylab=expression(paste("SE(",tau, ")" ) ), main="Standard Error True Score Tau",
    type="l")
par(mfrow=c(1,1))

\dontrun{
#****
# Model 2: Generalized partial credit model
mod2 <- TAM::tam.mml.2pl( dat, irtmodel="GPCM")
# estimate person parameter estimates
wmod2 <- TAM::tam.wle( mod2 )
# extract item parameters
A <- mod2$B[,-1,1]
B <- mod2$AXsi[,-1]
# person parameters and standard errors
theta <- wmod2$theta
error <- wmod2$error
# estimate true score transformation
dfr <- sirt::truescore.irt( A=A, B=B, theta=theta, error=error )

#############################################################################
# EXAMPLE 2: Dataset Reading data.read
#############################################################################
data(data.read)

#****
# Model 1: estimate difficulty + guessing model
mod1 <- sirt::rasch.mml2( data.read, fixed.c=rep(.25,12) )
mod1$person <- mod1$person[ order( mod1$person$EAP), ]
# person parameters and standard errors
theta <- mod1$person$EAP
error <- mod1$person$SE.EAP
A <- rep(1,12)
B <- - mod1$item$b
c <- rep(.25,12)
# estimate true score transformation
dfr <- sirt::truescore.irt( A=A, B=B, theta=theta, error=error,c=c)

plot( theta, dfr$percscore, pch=16, cex=.6, xlab=expression(theta), type="l",
    ylab=expression(paste( pi, "(",theta, ")" )), main="Percent Score Transformation" )
points( theta, dfr$lower + (dfr$upper-dfr$lower)*
             stats::plogis(dfr$a*theta+dfr$b), col=2, lty=2)

#****
# Model 2: Rasch model
mod2 <- sirt::rasch.mml2( data.read  )
# person parameters and standard errors
theta <- mod2$person$EAP
error <- mod2$person$SE.EAP
A <- rep(1,12)
B <- - mod2$item$b
# estimate true score transformation
dfr <- sirt::truescore.irt( A=A, B=B, theta=theta, error=error )
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{True scores}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: smirt.Rd
%% File Version: 2.411

\name{smirt}
\alias{smirt}
\alias{summary.smirt}
\alias{logLik.smirt}
\alias{anova.smirt}
\alias{IRT.irfprob.smirt}
\alias{IRT.likelihood.smirt}
\alias{IRT.posterior.smirt}
\alias{IRT.modelfit.smirt}
\alias{summary.IRT.modelfit.smirt}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Multidimensional Noncompensatory, Compensatory and Partially
Compensatory Item Response Model
}
\description{
This function estimates the noncompensatory and compensatory multidimensional
item response model (Bolt & Lall, 2003; Reckase, 2009) as well as
the partially compensatory item response model (Spray et al., 1990)
for dichotomous data.
}
\usage{
smirt(dat, Qmatrix, irtmodel="noncomp", est.b=NULL, est.a=NULL,
     est.c=NULL, est.d=NULL, est.mu.i=NULL, b.init=NULL, a.init=NULL,
     c.init=NULL, d.init=NULL, mu.i.init=NULL, Sigma.init=NULL,
     b.lower=-Inf, b.upper=Inf, a.lower=-Inf, a.upper=Inf,
     c.lower=-Inf, c.upper=Inf, d.lower=-Inf, d.upper=Inf,
     theta.k=seq(-6,6,len=20), theta.kDES=NULL,
     qmcnodes=0, mu.fixed=NULL, variance.fixed=NULL,  est.corr=FALSE,
     max.increment=1, increment.factor=1, numdiff.parm=0.0001,
     maxdevchange=0.1, globconv=0.001, maxiter=1000, msteps=4,
     mstepconv=0.001)

\method{summary}{smirt}(object,...)

\method{anova}{smirt}(object,...)

\method{logLik}{smirt}(object,...)

\method{IRT.irfprob}{smirt}(object,...)

\method{IRT.likelihood}{smirt}(object,...)

\method{IRT.posterior}{smirt}(object,...)

\method{IRT.modelfit}{smirt}(object,...)

\method{summary}{IRT.modelfit.smirt}(object,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame with dichotomous item responses
}
  \item{Qmatrix}{
The Q-matrix which specifies the loadings to be estimated
}
  \item{irtmodel}{
The item response model. Options are the noncompensatory model (\code{"noncomp"}),
the compensatory model (\code{"comp"}) and
the partially compensatory model (\code{"partcomp"}).
See Details for more explanations.
}
  \item{est.b}{
An integer matrix (if \code{irtmodel="noncomp"}) or integer vector
(if \code{irtmodel="comp"}) for \eqn{b} parameters to be estimated
}
  \item{est.a}{
An integer matrix for \eqn{a} parameters to be estimated.
If \code{est.a="2PL"}, then all item loadings will be estimated and the
variances are set to one (and therefore \code{est.corr=TRUE}).
}
  \item{est.c}{
An integer vector for \eqn{c} parameters to be estimated
}
  \item{est.d}{
An integer vector for \eqn{d} parameters to be estimated
}
  \item{est.mu.i}{
An integer vector for \eqn{\mu_i} parameters to be estimated
}
  \item{b.init}{
Initial \eqn{b} coefficients. For \code{irtmodel="noncomp"} it must be a
matrix, for \code{irtmodel="comp"} it is a vector.
}
  \item{a.init}{
Initial \eqn{a} coefficients arranged in a matrix
}
  \item{c.init}{
Initial \eqn{c} coefficients
}
  \item{d.init}{
Initial \eqn{d} coefficients
}
\item{mu.i.init}{
Initial \eqn{d} coefficients
}
\item{Sigma.init}{
Initial covariance matrix \eqn{\Sigma}
}
\item{b.lower}{Lower bound for \eqn{b} parameter}
\item{b.upper}{Upper bound for \eqn{b} parameter}
\item{a.lower}{Lower bound for \eqn{a} parameter}
\item{a.upper}{Upper bound for \eqn{a} parameter}
\item{c.lower}{Lower bound for \eqn{c} parameter}
\item{c.upper}{Upper bound for \eqn{c} parameter}
\item{d.lower}{Lower bound for \eqn{d} parameter}
\item{d.upper}{Upper bound for \eqn{d} parameter}
  \item{theta.k}{
Vector of discretized trait distribution. This vector is expanded in all
dimensions by using the \code{\link[base:expand.grid]{base::expand.grid}}
function. If a user specifies a design matrix \code{theta.kDES} of transformed
\eqn{\bold{\theta}_p}
values (see Details and Examples), then \code{theta.k} must be a matrix, too.
}
  \item{theta.kDES}{
An optional design matrix. This matrix will differ from the ordinary
theta grid in case of nonlinear item response models.
}
  \item{qmcnodes}{
Number of integration nodes for quasi Monte Carlo integration (see Pan &
Thompson, 2007; Gonzales et al., 2006). Integration points are obtained by using
the function \code{\link{qmc.nodes}}. Note that when using quasi Monte Carlo nodes,
no theta design matrix \code{theta.kDES} can be specified.
See Example 1, Model 11.
}
  \item{mu.fixed}{
Matrix with fixed entries in the mean vector. By default, all means
are set to zero.
}
  \item{variance.fixed}{
Matrix (with rows and three columns) with fixed entries in the covariance matrix
(see Examples). The entry \eqn{c_{kd}} of the covariance between dimensions
  \eqn{k} and \eqn{d} is set to \eqn{c_0} iff \code{variance.fixed} has a row with
  a \eqn{k} in the first column, a \eqn{d} in the second column and the value
  \eqn{c_0} in the third column.
}
  \item{est.corr}{
Should only a correlation matrix instead of a covariance matrix
be estimated?
}
  \item{max.increment}{
Maximum increment
}
  \item{increment.factor}{
A value (larger than one) which defines the extent of the decrease of the maximum
increment of item parameters in every iteration. The maximum increment in iteration
\code{iter} is defined as \code{max.increment*increment.factor^(-iter)}
where \code{max.increment=1}. Using a value larger than 1 helps
to reach convergence in some non-converging analyses (use values of 1.01, 1.02
or even 1.05). See also Example 1 Model 2a.
}
  \item{numdiff.parm}{
Numerical differentiation parameter
}
  \item{maxdevchange}{
Convergence criterion for change in relative deviance
}
  \item{globconv}{
Global convergence criterion for parameter change
}
  \item{maxiter}{
Maximum number of iterations
}
  \item{msteps}{
Number of iterations within a M step
}
  \item{mstepconv}{
Convergence criterion within a M step
}
  \item{object}{
Object of class \code{smirt}
}
  \item{\dots}{
Further arguments to be passed
}
}
\details{
The noncompensatory item response model (\code{irtmodel="noncomp"};
e.g. Bolt & Lall, 2003) is defined as
\deqn{P(X_{pi}=1 | \bold{\theta}_p )=c_i + (d_i - c_i )
\prod_l invlogit( a_{il} q_{il}  \theta_{pl} - b_{il} ) }
where \eqn{i}, \eqn{p}, \eqn{l} denote items, persons and dimensions
respectively.

The compensatory item response model (\code{irtmodel="comp"}) is defined by
\deqn{P(X_{pi}=1 | \bold{\theta}_p )=c_i + (d_i - c_i )
invlogit( \sum_l  a_{il}  q_{il} \theta_{pl} - b_{i} ) }
Using a design matrix \code{theta.kDES} the model can be made even more general
in a model which is linear in item parameters
\deqn{P(X_{pi}=1 | \bold{\theta}_p )=c_i + (d_i - c_i )
invlogit( \sum_l  a_{il}  q_{il} t_l ( \bold{ \theta_{p} } )   - b_{i} ) }
with known functions \eqn{t_l} of the trait vector \eqn{\bold{\theta}_p}.
Fixed values of the functions \eqn{t_l} are specified in the
\eqn{\bold{\theta}_p} design matrix \code{theta.kDES}.

The partially compensatory item response model (\code{irtmodel="partcomp"})
is defined by
\deqn{P(X_{pi}=1 | \bold{\theta}_p )=c_i + (d_i - c_i )
\frac{ \exp \left( \sum_l ( a_{il}  q_{il} \theta_{pl} - b_{il} ) \right) }
{  \mu_i \prod_l ( 1 + \exp ( a_{il}  q_{il} \theta_{pl} - b_{il} ) ) +
    ( 1- \mu_i) ( 1 + \exp \left( \sum_l  ( a_{il}  q_{il} \theta_{pl} - b_{il} ) \right) )
        } }
with item parameters \eqn{\mu_i} indicating the degree of compensatory.
\eqn{\mu_i=1} indicates a noncompensatory model while \eqn{\mu_i=0}
indicates a (fully) compensatory model.

The models are estimated by an EM algorithm employing marginal maximum
likelihood.
}
\value{
A list with following entries:
\item{deviance}{Deviance}
\item{ic}{Information criteria}
\item{item}{Data frame with item parameters}
\item{person}{Data frame with person parameters. It includes
    the person mean of all item responses (\code{M}; percentage correct
    of all non-missing items), the
EAP estimate and its corresponding standard error for all dimensions
(\code{EAP} and \code{SE.EAP}) and the maximum likelihood estimate
as well as the mode of the posterior distribution (\code{MLE} and \code{MAP}).}
\item{EAP.rel}{EAP reliability}
\item{mean.trait}{Means of trait}
\item{sd.trait}{Standard deviations of trait}
\item{Sigma}{Trait covariance matrix}
\item{cor.trait}{Trait correlation matrix}
\item{b}{Matrix (vector) of \eqn{b} parameters}
\item{se.b}{Matrix (vector) of standard errors \eqn{b} parameters}
\item{a}{Matrix of \eqn{a} parameters}
\item{se.a}{Matrix of standard errors of \eqn{a} parameters}
\item{c}{Vector of \eqn{c} parameters}
\item{se.c}{Vector of standard errors of \eqn{c} parameters}
\item{d}{Vector of \eqn{d} parameters}
\item{se.d}{Vector of standard errors of \eqn{d} parameters}
\item{mu.i}{Vector of \eqn{\mu_i} parameters}
\item{se.mu.i}{Vector of standard errors of \eqn{\mu_i} parameters}
\item{f.yi.qk}{Individual likelihood}
\item{f.qk.yi}{Individual posterior}
\item{probs}{Probabilities of item response functions evaluated at \code{theta.k}}
\item{n.ik}{Expected counts}
\item{iter}{Number of iterations}
\item{dat2}{Processed data set}
\item{dat2.resp}{Data set of response indicators}
\item{I}{Number of items}
\item{D}{Number of dimensions}
\item{K}{Maximum item response score}
\item{theta.k}{Used theta integration grid}
\item{pi.k}{Distribution function evaluated at \code{theta.k}}
\item{irtmodel}{Used IRT model}
\item{Qmatrix}{Used Q-matrix}
}
\references{
Bolt, D. M., & Lall, V. F. (2003). Estimation of compensatory and
noncompensatory multidimensional item response models using Markov chain
Monte Carlo. \emph{Applied Psychological Measurement, 27}, 395-414.

Gonzalez, J., Tuerlinckx, F., De Boeck, P., & Cools, R. (2006).
Numerical integration in logistic-normal models.
\emph{Computational Statistics & Data Analysis, 51}, 1535-1548.

Pan, J., & Thompson, R. (2007). Quasi-Monte Carlo estimation in
generalized linear mixed models. \emph{Computational Statistics &
Data Analysis, 51}, 5765-5775.

Reckase, M. (2009). \emph{Multidimensional item response theory}.
New York: Springer. \doi{10.1007/978-0-387-89976-3}

Spray, J. A., Davey, T. C., Reckase, M. D., Ackerman, T. A., & Carlson, J. E. (1990).
\emph{Comparison of two logistic multidimensional item response theory models}.
ACT Research Report No. ACT-RR-ONR-90-8.
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See the \code{\link[mirt:mirt]{mirt::mirt}} and
\code{itemtype="partcomp"} for estimating noncompensatory item response models
using the \pkg{mirt} package. See also \code{\link[mirt:mixedmirt]{mirt::mixedmirt}}.

Other multidimensional IRT models can also be estimated
with \code{\link{rasch.mml2}} and \code{\link{rasch.mirtlc}}.

See \code{\link[CDM:itemfit.sx2]{itemfit.sx2}} (\pkg{CDM}) for item fit
statistics.

See also the \pkg{mirt} and \pkg{TAM} packages for estimation of
compensatory multidimensional item response models.
}
\examples{
#############################################################################
## EXAMPLE 1: Noncompensatory and compensatory IRT models
#############################################################################
set.seed(997)

# (1) simulate data from a two-dimensional noncompensatory
#     item response model
#   -> increase number of iterations in all models!

N <- 1000    # number of persons
I <- 10        # number of items
theta0 <- rnorm( N, sd=1 )
theta1 <- theta0 + rnorm(N, sd=.7 )
theta2 <- theta0 + rnorm(N, sd=.7 )
Q <- matrix( 1, nrow=I,ncol=2 )
Q[ 1:(I/2), 2 ] <- 0
Q[ I,1] <- 0
b <- matrix( rnorm( I*2 ), I, 2 )
a <- matrix( 1, I, 2 )

# simulate data
prob <- dat <- matrix(0, nrow=N, ncol=I )
for (ii in 1:I){
prob[,ii] <- ( stats::plogis( theta1 - b[ii,1]  ) )^Q[ii,1]
prob[,ii] <- prob[,ii] * ( stats::plogis( theta2 - b[ii,2]  ) )^Q[ii,2]
            }
dat[ prob > matrix( stats::runif( N*I),N,I) ] <- 1
colnames(dat) <- paste0("I",1:I)

#***
# Model 1: Noncompensatory 1PL model
mod1 <- sirt::smirt(dat, Qmatrix=Q, maxiter=10 ) # change number of iterations
summary(mod1)

\dontrun{
#***
# Model 2: Noncompensatory 2PL model
mod2 <- sirt::smirt(dat,Qmatrix=Q, est.a="2PL", maxiter=15 )
summary(mod2)

# Model 2a: avoid convergence problems with increment.factor
mod2a <- sirt::smirt(dat,Qmatrix=Q, est.a="2PL", maxiter=30, increment.factor=1.03)
summary(mod2a)

#***
# Model 3: some fixed c and d parameters different from zero or one
c.init <- rep(0,I)
c.init[ c(3,7)] <- .2
d.init <- rep(1,I)
d.init[c(4,8)] <- .95
mod3 <- sirt::smirt( dat, Qmatrix=Q, c.init=c.init, d.init=d.init )
summary(mod3)

#***
# Model 4: some estimated c and d parameters (in parameter groups)
est.c <- c.init <- rep(0,I)
c.estpars <- c(3,6,7)
c.init[ c.estpars ] <- .2
est.c[c.estpars] <- 1
est.d <- rep(0,I)
d.init <- rep(1,I)
d.estpars <- c(6,9)
d.init[ d.estpars ] <- .95
est.d[ d.estpars ] <- d.estpars   # different d parameters
mod4 <- sirt::smirt(dat,Qmatrix=Q, est.c=est.c, c.init=c.init,
            est.d=est.d, d.init=d.init  )
summary(mod4)

#***
# Model 5: Unidimensional 1PL model
Qmatrix <- matrix( 1, nrow=I, ncol=1 )
mod5 <- sirt::smirt( dat, Qmatrix=Qmatrix )
summary(mod5)

#***
# Model 6: Unidimensional 2PL model
mod6 <- sirt::smirt( dat, Qmatrix=Qmatrix, est.a="2PL" )
summary(mod6)

#***
# Model 7: Compensatory model with between item dimensionality
# Note that the data is simulated under the noncompensatory condition
# Therefore Model 7 should have a worse model fit than Model 1
Q1 <- Q
Q1[ 6:10, 1] <- 0
mod7 <- sirt::smirt(dat,Qmatrix=Q1, irtmodel="comp", maxiter=30)
summary(mod7)

#***
# Model 8: Compensatory model with within item dimensionality
#         assuming zero correlation between dimensions
variance.fixed <- as.matrix( cbind( 1,2,0) )
# set the covariance between the first and second dimension to zero
mod8 <- sirt::smirt(dat,Qmatrix=Q, irtmodel="comp", variance.fixed=variance.fixed,
            maxiter=30)
summary(mod8)

#***
# Model 8b: 2PL model with starting values for a and b parameters
b.init <- rep(0,10)  # set all item difficulties initially to zero
# b.init <- NULL
a.init <- Q       # initialize a.init with Q-matrix
# provide starting values for slopes of first three items on Dimension 1
a.init[1:3,1] <- c( .55, .32, 1.3)

mod8b <- sirt::smirt(dat,Qmatrix=Q, irtmodel="comp", variance.fixed=variance.fixed,
              b.init=b.init, a.init=a.init, maxiter=20, est.a="2PL" )
summary(mod8b)

#***
# Model 9: Unidimensional model with quadratic item response functions
# define theta
theta.k <- seq( - 6, 6, len=15 )
theta.k <- as.matrix( theta.k, ncol=1 )
# define design matrix
theta.kDES <- cbind( theta.k[,1], theta.k[,1]^2 )
# define Q-matrix
Qmatrix <- matrix( 0, I, 2 )
Qmatrix[,1] <- 1
Qmatrix[ c(3,6,7), 2 ] <- 1
colnames(Qmatrix) <- c("F1", "F1sq" )
# estimate model
mod9 <- sirt::smirt(dat,Qmatrix=Qmatrix, maxiter=50, irtmodel="comp",
           theta.k=theta.k, theta.kDES=theta.kDES, est.a="2PL" )
summary(mod9)

#***
# Model 10: Two-dimensional item response model with latent interaction
#           between dimensions
theta.k <- seq( - 6, 6, len=15 )
theta.k <- expand.grid( theta.k, theta.k )    # expand theta to 2 dimensions
# define design matrix
theta.kDES <- cbind( theta.k, theta.k[,1]*theta.k[,2] )
# define Q-matrix
Qmatrix <- matrix( 0, I, 3 )
Qmatrix[,1] <- 1
Qmatrix[ 6:10, c(2,3) ] <- 1
colnames(Qmatrix) <- c("F1", "F2", "F1iF2" )
# estimate model
mod10 <- sirt::smirt(dat,Qmatrix=Qmatrix,irtmodel="comp", theta.k=theta.k,
            theta.kDES=theta.kDES, est.a="2PL" )
summary(mod10)

#****
# Model 11: Example Quasi Monte Carlo integration
Qmatrix <- matrix( 1, I, 1 )
mod11 <- sirt::smirt( dat, irtmodel="comp", Qmatrix=Qmatrix, qmcnodes=1000 )
summary(mod11)

#############################################################################
## EXAMPLE 2: Dataset Reading data.read
##            Multidimensional models for dichotomous data
#############################################################################

data(data.read)
dat <- data.read
I <- ncol(dat)    # number of items

#***
# Model 1: 3-dimensional 2PL model

# define Q-matrix
Qmatrix <- matrix(0,nrow=I,ncol=3)
Qmatrix[1:4,1] <- 1
Qmatrix[5:8,2] <- 1
Qmatrix[9:12,3] <- 1

# estimate model
mod1 <- sirt::smirt( dat, Qmatrix=Qmatrix, irtmodel="comp", est.a="2PL",
            qmcnodes=1000, maxiter=20)
summary(mod1)

#***
# Model 2: 3-dimensional Rasch model
mod2 <- sirt::smirt( dat, Qmatrix=Qmatrix, irtmodel="comp",
              qmcnodes=1000, maxiter=20)
summary(mod2)

#***
# Model 3: 3-dimensional 2PL model with uncorrelated dimensions
# fix entries in variance matrix
variance.fixed <- cbind( c(1,1,2), c(2,3,3), 0 )
# set the following covariances to zero: cov[1,2]=cov[1,3]=cov[2,3]=0

# estimate model
mod3 <- sirt::smirt( dat, Qmatrix=Qmatrix, irtmodel="comp", est.a="2PL",
             variance.fixed=variance.fixed, qmcnodes=1000, maxiter=20)
summary(mod3)

#***
# Model 4: Bifactor model with one general factor (g) and
#          uncorrelated specific factors

# define a new Q-matrix
Qmatrix1 <- cbind( 1, Qmatrix )
# uncorrelated factors
variance.fixed <- cbind( c(1,1,1,2,2,3), c(2,3,4,3,4,4), 0 )
# The first dimension refers to the general factors while the other
# dimensions refer to the specific factors.
# The specification means that:
# Cov[1,2]=Cov[1,3]=Cov[1,4]=Cov[2,3]=Cov[2,4]=Cov[3,4]=0

# estimate model
mod4 <- sirt::smirt( dat, Qmatrix=Qmatrix1, irtmodel="comp", est.a="2PL",
             variance.fixed=variance.fixed, qmcnodes=1000, maxiter=20)
summary(mod4)

#############################################################################
## EXAMPLE 3: Partially compensatory model
#############################################################################

#**** simulate data
set.seed(7656)
I <- 10         # number of items
N <- 2000        # number of subjects
Q <- matrix( 0, 3*I,2)  # Q-matrix
Q[1:I,1] <- 1
Q[1:I + I,2] <- 1
Q[1:I + 2*I,1:2] <- 1
b <- matrix( stats::runif( 3*I *2, -2, 2 ), nrow=3*I, 2 )
b <- b*Q
b <- round( b, 2 )
mui <- rep(0,3*I)
mui[ seq(2*I+1, 3*I) ] <- 0.65
# generate data
dat <- matrix( NA, N, 3*I )
colnames(dat) <- paste0("It", 1:(3*I) )
# simulate item responses
library(mvtnorm)
theta <- mvtnorm::rmvnorm(N, mean=c(0,0), sigma=matrix( c( 1.2, .6,.6,1.6),2, 2 ) )
for (ii in 1:(3*I)){
    # define probability
    tmp1 <- exp( theta[,1] * Q[ii,1] - b[ii,1] +  theta[,2] * Q[ii,2] - b[ii,2] )
    # non-compensatory model
    nco1 <- ( 1 + exp( theta[,1] * Q[ii,1] - b[ii,1] ) ) *
                  ( 1 + exp( theta[,2] * Q[ii,2] - b[ii,2] ) )
    co1 <- ( 1 + tmp1 )
    p1 <- tmp1 / ( mui[ii] * nco1 + ( 1 - mui[ii] )*co1 )
    dat[,ii] <- 1 * ( stats::runif(N) < p1 )
}

#*** Model 1: Joint mu.i parameter for all items
est.mu.i <- rep(0,3*I)
est.mu.i[ seq(2*I+1,3*I)] <- 1
mod1 <- sirt::smirt( dat, Qmatrix=Q, irtmodel="partcomp", est.mu.i=est.mu.i)
summary(mod1)

#*** Model 2: Separate mu.i parameter for all items
est.mu.i[ seq(2*I+1,3*I)] <- 1:I
mod2 <- sirt::smirt( dat, Qmatrix=Q, irtmodel="partcomp", est.mu.i=est.mu.i)
summary(mod2)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Multidimensional item response model}
%% \keyword{summary}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: rasch.prox.Rd
%% File Version: 1.17

\name{rasch.prox}
\alias{rasch.prox}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
PROX Estimation Method for the Rasch Model
}
\description{
This function estimates the Rasch model using the PROX algorithm
(cited in Wright & Stone, 1999).
}
\usage{
rasch.prox(dat, dat.resp=1 - is.na(dat), freq=rep(1,nrow(dat)),
    conv=0.001, maxiter=30, progress=FALSE)
}
\arguments{
  \item{dat}{
An \eqn{N \times I} data frame of dichotomous response data. \code{NA}s
are not allowed and must be indicated by zero entries in the
response indicator matrix \code{dat.resp}.
}
  \item{dat.resp}{
An \eqn{N \times I} indicator data frame of nonmissing item responses.
}
  \item{freq}{
A vector of frequencies (or weights) of all rows in data frame \code{dat}.
}
  \item{conv}{
Convergence criterion for item parameters
}
  \item{maxiter}{
Maximum number of iterations
}
  \item{progress}{
Display progress?
}
}
%\details{
%%For details on the algorithm see Wright and Stone (1999, p. 135ff.).
%}
\value{
A list with following entries
\item{b}{Estimated item difficulties}
\item{theta}{Estimated person abilities}
\item{iter}{Number of iterations}
\item{sigma.i}{Item standard deviations}
\item{sigma.n}{Person standard deviations}
}
\references{
Wright, B., & Stone, W. (1999). \emph{Measurement Essentials}.
Wilmington: Wide Range.
}

%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{
#############################################################################
# EXAMPLE 1: PROX data.read
#############################################################################

data(data.read)
mod <- sirt::rasch.prox( data.read )
mod$b       # item difficulties
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{PROX algorithm}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: rasch.mml.Rd
%% File Version: 4.019

\name{rasch.mml2}
\alias{rasch.mml2}
\alias{summary.rasch.mml}
\alias{plot.rasch.mml}
\alias{logLik.rasch.mml}
\alias{anova.rasch.mml}
\alias{IRT.irfprob.rasch.mml}
\alias{IRT.likelihood.rasch.mml}
\alias{IRT.posterior.rasch.mml}
\alias{IRT.modelfit.rasch.mml}
\alias{IRT.expectedCounts.rasch.mml}
\alias{summary.IRT.modelfit.rasch.mml}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimation of the Generalized Logistic Item Response Model,
Ramsay's Quotient Model, Nonparametric Item Response Model,
Pseudo-Likelihood Estimation and a Missing Data Item Response Model
}
\description{
This function employs marginal maximum likelihood estimation
of item response models for dichotomous data.
First, the Rasch type model (generalized
item response model) can be estimated. The generalized logistic
link function (Stukel, 1988) can be estimated or fixed for conducting
IRT with different link functions than the logistic one. The Four-Parameter
logistic item response model is a special case of this model
(Loken & Rulison, 2010). Second, Ramsay's quotient model (Ramsay, 1989)
can be estimated by specifying \code{irtmodel="ramsay.qm"}.
Third, quite general item response functions can be estimated
in a nonparametric framework (Rossi, Wang & Ramsay, 2002).
Fourth, pseudo-likelihood estimation for fractional item responses can be
conducted for Rasch type models. Fifth, a simple two-dimensional
missing data item response model (\code{irtmodel='missing1'};
Mislevy & Wu, 1996) can be estimated.

See Details for more explanations.
}
\usage{
rasch.mml2( dat, theta.k=seq(-6,6,len=21), group=NULL, weights=NULL,
   constraints=NULL, glob.conv=10^(-5), parm.conv=10^(-4), mitermax=4,
   mmliter=1000, progress=TRUE,  fixed.a=rep(1,ncol(dat)),
   fixed.c=rep(0,ncol(dat)), fixed.d=rep(1,ncol(dat)),
   fixed.K=rep(3,ncol(dat)), b.init=NULL, est.a=NULL, est.b=NULL,
   est.c=NULL, est.d=NULL, min.b=-99, max.b=99, min.a=-99, max.a=99,
   min.c=0, max.c=1, min.d=0, max.d=1, prior.b=NULL, prior.a=NULL, prior.c=NULL,
   prior.d=NULL, est.K=NULL, min.K=1, max.K=20, min.delta=-20, max.delta=20,
   beta.init=NULL, min.beta=-8, pid=1:(nrow(dat)), trait.weights=NULL,  center.trait=TRUE,
   center.b=FALSE, alpha1=0, alpha2=0,est.alpha=FALSE, equal.alpha=FALSE,
   designmatrix=NULL, alpha.conv=parm.conv, numdiff.parm=0.00001,
   numdiff.alpha.parm=numdiff.parm, distribution.trait="normal", Qmatrix=NULL,
   variance.fixed=NULL, variance.init=NULL,
   mu.fixed=cbind(seq(1,ncol(Qmatrix)),rep(0,ncol(Qmatrix))),
   irtmodel="raschtype", npformula=NULL, npirt.monotone=TRUE,
   use.freqpatt=is.null(group), delta.miss=0, est.delta=rep(NA,ncol(dat)),
   nimps=0, ... )

\method{summary}{rasch.mml}(object, file=NULL, ...)

\method{plot}{rasch.mml}(x, items=NULL, xlim=NULL, main=NULL, ...)

\method{anova}{rasch.mml}(object,...)

\method{logLik}{rasch.mml}(object,...)

\method{IRT.irfprob}{rasch.mml}(object,...)

\method{IRT.likelihood}{rasch.mml}(object,...)

\method{IRT.posterior}{rasch.mml}(object,...)

\method{IRT.modelfit}{rasch.mml}(object,...)

\method{IRT.expectedCounts}{rasch.mml}(object,...)

\method{summary}{IRT.modelfit.rasch.mml}(object,...)
}
%- maybe also 'usage' for other objects documented here.

%% Undocumented arguments in documentation object 'rasch.mml2'
%%  'Qmatrix' 'variance.fixed' 'mu.fixed'


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  ARGUMENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\arguments{
  \item{dat}{
An \eqn{N \times I} data frame of dichotomous item responses. \cr
For the missing data item response model (\code{irtmodel='missing1'}),
code item responses by \code{9} which should be treated by the missing
  data model. Other missing responses can be coded by \code{NA}.
}
\item{theta.k}{
Optional vector of discretized theta values. For multidimensional
IRT models with \eqn{D} dimensions, it is a matrix with \eqn{D} columns.
}
  \item{group}{
Vector of integers with group identifiers in multiple group estimation.
The multiple group does not work for \code{irtmodel="missing1"}.
}
  \item{weights}{
Optional vector of person weights (sample weights).
}
  \item{constraints}{
Constraints on \code{b} parameters (item difficulties). It must be
a matrix with two columns: the first column contains item names, the
second column fixed parameter values.
}
  \item{glob.conv}{
Convergence criterion for deviance
}
  \item{parm.conv}{
Convergence criterion for item parameters
}
  \item{mitermax}{
Maximum number of iterations in M step. This argument does only
apply for the estimation of the \eqn{b} parameters.
}
  \item{mmliter}{
Maximum number of iterations
}
  \item{progress}{
Should progress be displayed at the console?
}
  \item{fixed.a}{
Fixed or initial \eqn{a} parameters
}
  \item{fixed.c}{
Fixed or initial \eqn{c} parameters
}
  \item{fixed.d}{
Fixed or initial \eqn{d} parameters
}
  \item{fixed.K}{
Fixed or initial \eqn{K} parameters in Ramsay's
quotient model.
}
  \item{b.init}{
Initial \eqn{b} parameters
}
  \item{est.a}{
Vector of integers which indicate which \eqn{a}
parameters should be estimated. Equal integers correspond
to the same estimated parameters.
}
  \item{est.b}{
Vector of integers which indicate which \eqn{b}
parameters should be estimated. Equal integers correspond
to the same estimated parameters.
}
  \item{est.c}{
Vector of integers which indicate which \eqn{c}
parameters should be estimated. Equal integers correspond
to the same estimated parameters.
}
  \item{est.d}{
Vector of integers which indicate which \eqn{d}
parameters should be estimated. Equal integers correspond
to the same estimated parameters.
}
  \item{min.b}{
Minimal \eqn{b} parameter to be estimated
}
  \item{max.b}{
Maximal \eqn{b} parameter to be estimated
}
  \item{min.a}{
Minimal \eqn{a} parameter to be estimated
}
  \item{max.a}{
Maximal \eqn{a} parameter to be estimated
}
  \item{min.c}{
Minimal \eqn{c} parameter to be estimated
}
  \item{max.c}{
Maximal \eqn{c} parameter to be estimated
}
  \item{min.d}{
Minimal \eqn{d} parameter to be estimated
}
  \item{max.d}{
Maximal \eqn{d} parameter to be estimated
}
\item{prior.b}{Optional prior distribution for \eqn{b} parameters:
\eqn{N(\mu, \sigma)}. Input is a vector of length two with parameters
\eqn{\mu} and \eqn{\sigma}.}
\item{prior.a}{Optional prior distribution for \eqn{a} parameters:
\eqn{N(\mu, \sigma)}. Input is a vector of length two with parameters
\eqn{\mu} and \eqn{\sigma}.}
\item{prior.c}{Optional prior distribution for \eqn{c} parameters:
\eqn{Beta(a, b)}. Input is a vector of length two with parameters
\eqn{a} and \eqn{b}.}
\item{prior.d}{Optional prior distribution for \eqn{d} parameters:
\eqn{Beta(a, b)}. Input is a vector of length two with parameters
\eqn{a} and \eqn{b}.}
  \item{est.K}{
Vector of integers which indicate which \eqn{K}
parameters should be estimated. Equal integers correspond
to the same estimated parameters.
}
\item{min.K}{ Minimal \eqn{K} parameter to be estimated}
\item{max.K}{ Maximal \eqn{K} parameter to be estimated}
\item{min.delta}{ Minimal \eqn{delta.miss} parameter to be estimated}
\item{max.delta}{ Maximal \eqn{delta.miss} parameter to be estimated}
\item{beta.init}{Optional vector of initial \eqn{\beta} parameters}
\item{min.beta}{Minimum \eqn{\beta} parameter to be estimated.}
  \item{pid}{
Optional vector of person identifiers
}
  \item{trait.weights}{
Optional vector of trait weights for a fixing
the trait distribution.
}
  \item{center.trait}{
Should the trait distribution be centered
}
\item{center.b}{An optional logical indicating whether \eqn{b} parameters
should be centered at each dimension}
  \item{alpha1}{
Fixed or initial \eqn{\alpha_1} parameter
}
  \item{alpha2}{
Fixed or initial \eqn{\alpha_2} parameter
}
  \item{est.alpha}{
Should \eqn{\alpha} parameters be estimated?
}
  \item{equal.alpha}{
Estimate \eqn{\alpha} parameters under the
assumption \eqn{\alpha_1=\alpha_2}?
}
  \item{designmatrix}{
Design matrix for item difficulties \eqn{b} to estimate
linear logistic test models
}
  \item{alpha.conv}{
Convergence criterion for \eqn{\alpha} parameter
}
  \item{numdiff.parm}{
Parameter for numerical differentiation
}
  \item{numdiff.alpha.parm}{
Parameter for numerical differentiation for \eqn{\alpha}
parameter
}
  \item{distribution.trait}{
Assumed trait distribution. The default is the normal
distribution (\code{"normal"}). Log-linear smoothing of the
trait distribution is also possible (\code{"smooth2"},
\code{"smooth3"} or \code{"smooth4"} for smoothing up to
2, 3 or 4 moments, respectively).
}
  \item{Qmatrix}{
The Q-matrix
}
  \item{variance.fixed}{
Matrix for fixing covariance matrix (See Examples)
}
\item{variance.init}{Optional initial covariance matrix}
  \item{mu.fixed}{
Matrix for fixing mean vector (See Examples)
}

  \item{irtmodel}{
Specify estimable IRT models: \code{raschtype} (Rasch type model),
\code{ramsay.qm} (Ramsay's quotient model), \code{npirt} (Nonparametric
item response model). If \code{npirt} is used as the argument
for \code{irtmodel}, the argument \code{npformula}
specifies different item response functions in the
\R formula framework (like \code{"y~I(theta^2)"}; see Examples).
For estimating the missing data item response model, use
\code{irtmodel='missing1'}.
}
  \item{npformula}{
A string or a vector which contains \R formula objects for specifying
the item response function. For example, \code{"y~theta"} is the specification
of the 2PL model (see Details). If \code{irtmodel="npirt"} and \code{npformula}
is not specified, then an unrestricted item response functions on the
grid of \eqn{\theta} values is estimated.
}

\item{npirt.monotone}{Should nonparametrically estimated item response functions
    be monotone? The default is \code{TRUE}. This function applies only
    to \code{irtmodel='npirt'} and \code{npformula=NULL}.
}
\item{use.freqpatt}{A logical if frequencies of pattern should be used or not.
    The default is \code{is.null(group)}. This means that for single
    group analyses, frequency patterns are used but not for multiple
    groups. If data processing times are large, then \code{use.freqpatt=FALSE}
    is recommended.}
\item{delta.miss}{Missingness parameter \eqn{\delta} quantifying the meaning
    of responding to an item between the two extremes of ignoring
     missing responses and setting all missing responses to incorrect}
\item{est.delta}{Vector with indices indicating the \eqn{\delta} parameters
    to be estimated if \code{irtmodel="missing1"}.}
\item{nimps}{Number of imputed datasets of item responses}
  \item{object}{
Object of class \code{rasch.mml}
}
\item{x}{Object of class \code{rasch.mml}}
\item{items}{Vector of integer or item names which should be plotted}
\item{xlim}{Specification for \code{xlim} in plot}
\item{main}{Title of the plot}
\item{file}{Optional file name for summary output}
  \item{\dots}{
Further arguments to be passed
}
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  DETAILS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\details{
The item response function of the generalized item response model
(\code{irtmodel="raschtype"}; Stukel, 1988) can be written as
\deqn{P( X_{pi}=1 | \theta_{pd} )=c_i + (d_i - c_i ) g_{\alpha_1, \alpha_2}
[ a_i ( \theta_{pd} - b_i ) ] }
where \eqn{g} is the generalized logistic link function depending
on parameters \eqn{\alpha_1} and \eqn{\alpha_2}.

For the most important link functions the specifications are (Stukel, 1988):

logistic link function: \eqn{\alpha_1=0} and \eqn{\alpha_2=0} \cr
probit link function: \eqn{\alpha_1=0.165} and \eqn{\alpha_2=0.165} \cr
loglog link function: \eqn{\alpha_1=-0.037} and \eqn{\alpha_2=0.62} \cr
cloglog link function: \eqn{\alpha_1=0.62} and \eqn{\alpha_2=-0.037}

See \code{\link{pgenlogis}} for exact transformation formulas of
the mentioned link functions. \cr

A \eqn{D}-dimensional model can also be specified
but only allows for between item dimensionality
(one item loads on one and only dimension).
Setting \eqn{c_i=0}, \eqn{d_i=1} and  \eqn{a_i=1} for all items \eqn{i},
an additive item response model
\deqn{P( X_{pi}=1 | \theta_p )=g_{\alpha_1, \alpha_2} ( \theta_p - b_i  ) }
is estimated.

Ramsay's quotient model (\code{irtmodel="qm.ramsay"}) uses
the item response function
\deqn{P( X_{pi}=1 | \theta_p )=\frac{ \exp(\theta_p / b_i)}
    { K_i + \exp (\theta_p / b_i )} }

Quite general unidimensional item response models can be estimated
in a nonparametric framework (\code{irtmodel="npirt"}). The response
functions are a linear combination of transformed \eqn{\theta}
values
\deqn{logit[ P( X_{pi}=1 | \theta_p ) ]=Y_\theta \beta }
Where \eqn{Y_\theta} is a design matrix of \eqn{\theta} and
\eqn{\beta} are item parameters to be estimated.
The formula \eqn{Y_\theta \beta} can be specified in the \R formula
framework (see Example 3, Model 3c).

Pseudo-likelihood estimation can be conducted for fractional item response data
as input (i.e. some item response \eqn{x_{pi}} do have values
between 0 and 1). Then the pseudo-likelihood \eqn{L_p} for person \eqn{p} is defined as
\deqn{ L_p=\prod_i P_i ( \theta_p )^{x_{pi}} [1-P_i ( \theta_p )]^{(1-x_{pi})}}
Note that for dichotomous responses this term corresponds to the ordinary
likelihood. See Example 7.

A special two-dimensional missing data item response model (\code{irtmodel="missing1"})
is implemented according to Mislevy and Wu (1996).
Besides an unidimensional ability \eqn{\theta_p},
an individual response propensity \eqn{\xi_p} is proposed. We define
item responses \eqn{X_{pi}} and response indicators \eqn{R_{pi}} indicating whether
item responses \eqn{X_{pi}} are observed or not. Denoting the logistic function
by \eqn{\Psi}, the item response model for ability is defined as
\deqn{ P( X_{pi}=1  | \theta_p, \xi_p )=P( X_{pi}=1 | \theta_p )
=\Psi( a_i (\theta_p - b_i ))}
We also define a measurement model for response indicators \eqn{R_{pi}} which depends
on the item response \eqn{X_{pi}} itself:
\deqn{P( R_{pi}=1 | X_{pi}=k, \theta_p, \xi_p )=
    P( R_{pi}=1 | X_{pi}=k, \xi_p )=
\Psi \left[  \xi_p -  \beta_i - k \delta _i  \right] \quad \mbox{ for }
\quad k=0,1}
If \eqn{\delta _i=0}, then the probability of responding to an item is independent
of the incompletely observed item \eqn{X_{pi}} which is an
item response model with nonignorable missings (Holman & Glas, 2005;
see also Pohl, Graefe & Rose, 2014).
If \eqn{\delta _i} is a large negative number (e.g. \eqn{\delta=-100}), then
it follows \eqn{P( R_{pi}=1 | X_{pi}=1, \theta_p, \xi_p )=1}
and as a consequence it holds that \eqn{P(X_{pi}=1 | R_{pi}=0, \theta_p, \xi_p)=0},
which is equivalent to treating
all missing item responses as incorrect. The missingness parameter
\eqn{\delta} can be specified
by the user and studied as a sensitivity analysis under different
missing not at random assumptions or can be estimated by choosing
\code{est.delta=TRUE}.
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  VALUES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\value{
A list with following entries
\item{dat}{Original data frame}
\item{item}{Estimated item parameters in the generalized
    item response model}
\item{item2}{Estimated item parameters for Ramsay's quotient model}
\item{trait.distr}{Discretized ability distribution points and probabilities}
\item{mean.trait}{Estimated mean vector}
\item{sd.trait}{Estimated standard deviations}
\item{skewness.trait}{Estimated skewnesses}
\item{deviance}{Deviance}
\item{pjk}{Estimated probabilities of item correct evaluated at \code{theta.k}}
\item{rprobs}{Item response probabilities like in \code{pjk}, but slightly
extended to accommodate all categories}
\item{person}{Person parameter estimates: mode (\code{MAP}) and
    mean (\code{EAP}) of the posterior distribution}
\item{pid}{Person identifier}
\item{ability.est.pattern}{Response pattern estimates}
\item{f.qk.yi}{Individual posterior distribution}
\item{f.yi.qk}{Individual likelihood}
\item{fixed.a}{Estimated \eqn{a} parameters}
\item{fixed.c}{Estimated \eqn{c} parameters}
\item{G}{Number of groups}
\item{alpha1}{Estimated \eqn{\alpha_1} parameter in generalized logistic
    item response model}
\item{alpha2}{Estimated \eqn{\alpha_2} parameter in generalized logistic
    item response model}
\item{se.b}{Standard error of \eqn{b} parameter in generalized logistic model
    or Ramsay's quotient model}
\item{se.a}{Standard error of \eqn{a} parameter in generalized logistic model}
\item{se.c}{Standard error of \eqn{c} parameter in generalized logistic model}
\item{se.d}{Standard error of \eqn{d} parameter in generalized logistic model}
\item{se.alpha}{Standard error of \eqn{\alpha} parameter in generalized
    logistic model}
\item{se.K}{Standard error of \eqn{K} parameter in Ramsay's quotient model}
\item{iter}{Number of iterations}
\item{reliability}{EAP reliability}
\item{irtmodel}{Type of estimated item response model}
\item{D}{Number of dimensions}
\item{mu}{Mean vector (for multidimensional models)}
\item{Sigma.cov}{Covariance matrix (for multdimensional models)}
\item{theta.k}{Grid of discretized ability distributions}
\item{trait.weights}{Fixed vector of probabilities for the ability distribution}
\item{pi.k}{Trait distribution}
\item{ic}{Information criteria}
\item{esttype}{Estimation type: \code{ll} (Log-Likelihood),
    \code{pseudoll} (Pseudo-Log-Likelihood)
}
\item{\dots}{}
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  REFERENCES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\references{
Holman, R., & Glas, C. A. (2005). Modelling non-ignorable missing-data mechanisms
with item response theory models.
\emph{British Journal of Mathematical and Statistical Psychology, 58}(1), 1-17.
\doi{10.1348/000711005X47168}

Loken, E., & Rulison, K. L. (2010). Estimation of a four-parameter
item response theory model. \emph{British Journal of Mathematical
and Statistical Psychology, 63}(3), 509-525.
\doi{10.1348/000711009X474502}

Mislevy, R. J., & Wu, P. K. (1996). \emph{Missing responses and IRT ability
estimation: Omits, choice, time Limits, and adaptive testing}.
ETS Research Report ETS RR-96-30. Princeton, ETS.
\doi{10.1002/j.2333-8504.1996.tb01708.x}

Pohl, S., Graefe, L., & Rose, N. (2014). Dealing with omitted and
not-reached items in competence tests evaluating approaches accounting for
missing responses in item response theory models.
\emph{Educational and Psychological Measurement, 74}(3), 423-452.
\doi{10.1177/0013164413504926}

Ramsay, J. O. (1989). A comparison of three simple test theory models.
\emph{Psychometrika, 54}, 487-499.
\doi{10.1007/BF02294631}

Rossi, N., Wang, X., & Ramsay, J. O. (2002). Nonparametric item response
function estimates with the EM algorithm.
\emph{Journal of Educational and Behavioral Statistics, 27}(3), 291-317.
\doi{10.3102/10769986027003291}

Stukel, T. A. (1988). Generalized logistic models.
\emph{Journal of the American Statistical Association, 83}(402), 426-431.
\doi{10.1080/01621459.1988.10478613}

van der Maas, H. J. L., Molenaar, D., Maris, G., Kievit, R. A., &
Borsboom, D. (2011).
Cognitive psychology meets psychometric theory: On the relation between
process models for decision making and latent variable models for
individual differences.
\emph{Psychological Review, 118}(2), 339-356.
doi: 10.1037/a0022749
}


\note{
Multiple group estimation is not possible for
Ramsay's quotient model and multdimensional models.
}

%Be careful with this function, especially with multiple groups.
%It is just in an experimental stage.

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
Simulate the generalized logistic Rasch model with \code{\link{sim.raschtype}}.

Simulate Ramsay's quotient model with \code{\link{sim.qm.ramsay}}.

Simulate locally dependent item response data using \code{\link{sim.rasch.dep}}.

For an assessment of global model fit see \code{\link{modelfit.sirt}}.

See \code{\link[CDM:itemfit.sx2]{CDM::itemfit.sx2}} for item fit
statistics.
}

% \code{\link[mice:mice]{mice}} (\pkg{mice})

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  EXAMPLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



\examples{
#############################################################################
# EXAMPLE 1: Reading dataset
#############################################################################

library(CDM)
data(data.read)
dat <- data.read
I <- ncol(dat) # number of items

# Rasch model
mod1 <- sirt::rasch.mml2( dat )
summary(mod1)
plot( mod1 )    # plot all items
# title 'Rasch model', display curves from -3 to 3 only for items 1, 5 and 8
plot(mod1, main="Rasch model Items 1, 5 and 8", xlim=c(-3,3), items=c(1,5,8) )

# Rasch model with constraints on item difficulties
# set item parameters of A1 and C3 equal to -2
constraints <- data.frame( c("A1","C3"), c(-2,-2) )
mod1a <- sirt::rasch.mml2( dat, constraints=constraints)
summary(mod1a)

# estimate equal item parameters for 1st and 11th item
est.b <- 1:I
est.b[11] <- 1
mod1b <- sirt::rasch.mml2( dat, est.b=est.b )
summary(mod1b)

# estimate Rasch model with skew trait distribution
mod1c <- sirt::rasch.mml2( dat, distribution.trait="smooth3")
summary(mod1c)

# 2PL model
mod2 <- sirt::rasch.mml2( dat, est.a=1:I )
summary(mod2)
plot(mod2)    # plot 2PL item response curves

# extract individual likelihood
llmod2 <- IRT.likelihood(mod2)
str(llmod2)

\dontrun{
library(CDM)
# model comparisons
CDM::IRT.compareModels(mod1, mod1c, mod2 )
anova(mod1,mod2)

# assess model fit
smod1 <- IRT.modelfit(mod1)
smod2 <- IRT.modelfit(mod2)
IRT.compareModels(smod1, smod2)

# set some bounds for a and b parameters
mod2a <- sirt::rasch.mml2( dat, est.a=1:I, min.a=.7, max.a=2, min.b=-2 )
summary(mod2a)

# 3PL model
mod3 <- sirt::rasch.mml2( dat, est.a=1:I, est.c=1:I,
              mmliter=400 # maximal 400 iterations
                 )
summary(mod3)

# 3PL model with fixed guessing paramters of .25 and equal slopes
mod4 <- sirt::rasch.mml2( dat, fixed.c=rep( .25, I )   )
summary(mod4)

# 3PL model with equal guessing paramters for all items
mod5 <- sirt::rasch.mml2( dat, est.c=rep(1, I )   )
summary(mod5)

# difficulty + guessing model
mod6 <- sirt::rasch.mml2( dat, est.c=1:I   )
summary(mod6)

# 4PL model
mod7 <- sirt::rasch.mml2( dat, est.a=1:I, est.c=1:I, est.d=1:I,
            min.d=.95, max.c=.25)
        # set minimal d and maximal c parameter to .95 and .25
summary(mod7)

# 4PL model with prior distributions
mod7b <- sirt::rasch.mml2( dat, est.a=1:I, est.c=1:I, est.d=1:I, prior.a=c(1,2),
            prior.c=c(5,17), prior.d=c(20,2) )
summary(mod7b)

# constrained 4PL model
# equal slope, guessing and slipping parameters
mod8 <- sirt::rasch.mml2( dat,est.c=rep(1,I), est.d=rep(1,I) )
summary(mod8)

# estimation of an item response model with an
# uniform theta distribution
theta.k <- seq( 0.01, .99, len=20 )
trait.weights <- rep( 1/length(theta.k), length(theta.k) )
mod9 <- sirt::rasch.mml2( dat, theta.k=theta.k, trait.weights=trait.weights,
              normal.trait=FALSE, est.a=1:12  )
summary(mod9)

#############################################################################
# EXAMPLE 2: Longitudinal data
#############################################################################

data(data.long)
dat <- data.long[,-1]

# define Q loading matrix
Qmatrix <- matrix( 0, 12, 2 )
Qmatrix[1:6,1] <- 1 # T1 items
Qmatrix[7:12,2] <- 1    # T2 items

# define restrictions on item difficulties
est.b <- c(1,2,3,4,5,6,   3,4,5,6,7,8)
mu.fixed <- cbind(1,0)
    # set first mean to 0 for identification reasons

# Model 1: 2-dimensional Rasch model
mod1 <- sirt::rasch.mml2( dat, Qmatrix=Qmatrix, miterstep=4,
            est.b=est.b,  mu.fixed=mu.fixed, mmliter=30 )
summary(mod1)
plot(mod1)
##     Plot function is only applicable for unidimensional models
}

#############################################################################
# EXAMPLE 3: One group, estimation of alpha parameter in the generalized logistic model
#############################################################################

# simulate theta values
set.seed(786)
N <- 1000                  # number of persons
theta <- stats::rnorm( N, sd=1.5 ) # N persons with SD 1.5
b <- seq( -2, 2, len=15)

# simulate data
dat <- sirt::sim.raschtype( theta=theta, b=b, alpha1=0, alpha2=-0.3 )

#  estimating alpha parameters
mod1 <- sirt::rasch.mml2( dat, est.alpha=TRUE, mmliter=30 )
summary(mod1)
plot(mod1)

\dontrun{
# fixed alpha parameters
mod1b <- sirt::rasch.mml2( dat, est.alpha=FALSE, alpha1=0, alpha2=-.3 )
summary(mod1b)

# estimation with equal alpha parameters
mod1c <- sirt::rasch.mml2( dat, est.alpha=TRUE, equal.alpha=TRUE )
summary(mod1c)

# Ramsay QM
mod2a <- sirt::rasch.mml2( dat, irtmodel="ramsay.qm" )
summary(mod2a)
}

# Ramsay QM with estimated K parameters
mod2b <- sirt::rasch.mml2( dat, irtmodel="ramsay.qm", est.K=1:15, mmliter=30)
summary(mod2b)
plot(mod2b)

\dontrun{
# nonparametric estimation of monotone item response curves
mod3a <- sirt::rasch.mml2( dat, irtmodel="npirt", mmliter=100,
            theta.k=seq( -3, 3, len=10) ) # evaluations at 10 theta grid points
# nonparametric ICC of first 4 items
round( t(mod3a$pjk)[1:4,], 3 )
summary(mod3a)
plot(mod3a)

# nonparametric IRT estimation without monotonicity assumption
mod3b <- sirt::rasch.mml2( dat, irtmodel="npirt", mmliter=10,
            theta.k=seq( -3, 3, len=10), npirt.monotone=FALSE)
plot(mod3b)

# B-Spline estimation of ICCs
library(splines)
mod3c <- sirt::rasch.mml2( dat, irtmodel="npirt",
             npformula="y~bs(theta,df=3)", theta.k=seq(-3,3,len=15) )
summary(mod3c)
round( t(mod3c$pjk)[1:6,], 3 )
plot(mod3c)

# estimation of quadratic item response functions: ~ theta + I( theta^2)
mod3d <- sirt::rasch.mml2( dat, irtmodel="npirt",
             npformula="y~theta + I(theta^2)" )
summary(mod3d)
plot(mod3d)

# estimation of a stepwise ICC function
# ICCs are constant on the theta domains: [-Inf,-1], [-1,1], [1,Inf]
mod3e <- sirt::rasch.mml2( dat, irtmodel="npirt",
             npformula="y~I(theta>-1 )+I(theta>1)" )
summary(mod3e)
plot(mod3e, xlim=c(-2.5,2.5) )

# 2PL model
mod4 <- sirt::rasch.mml2( dat,  est.a=1:15)
summary(mod4)

#############################################################################
# EXAMPLE 4: Two groups, estimation of generalized logistic model
#############################################################################

# simulate generalized logistic Rasch model in two groups
set.seed(8765)
N1 <- 1000     # N1=1000 persons in group 1
N2 <- 500      # N2=500 persons in group 2
dat1 <- sirt::sim.raschtype( theta=stats::rnorm( N1, sd=1.5 ), b=b,
            alpha1=-0.3, alpha2=0)
dat2 <- sirt::sim.raschtype( theta=stats::rnorm( N2, mean=-.5, sd=.75),
            b=b, alpha1=-0.3, alpha2=0)
dat1 <- rbind( dat1, dat2 )
group <- c( rep(1,N1), rep(2,N2))

mod1 <-  sirt::rasch.mml2( dat1, parm.conv=.0001, group=group, est.alpha=TRUE )
summary(mod1)

#############################################################################
# EXAMPLE 5: Multidimensional model
#############################################################################

#***
# (1) simulate data
set.seed(785)
library(mvtnorm)
N <- 500
theta <- mvtnorm::rmvnorm( N,mean=c(0,0), sigma=matrix( c(1.45,.5,.5,1.7), 2, 2 ))
I <- 10
# 10 items load on the first dimension
p1 <- stats::plogis( outer( theta[,1], seq( -2, 2, len=I ), "-" ) )
resp1 <- 1 * ( p1 > matrix( stats::runif( N*I ), nrow=N, ncol=I ) )
# 10 items load on the second dimension
p1 <- stats::plogis( outer( theta[,2], seq( -2, 2, len=I ), "-" ) )
resp2 <- 1 * ( p1 > matrix( stats::runif( N*I ), nrow=N, ncol=I ) )
#Combine the two sets of items into one response matrix
resp <- cbind(resp1,resp2)
colnames(resp) <- paste("I", 1:(2*I), sep="")
dat <- resp

# define Q-matrix
Qmatrix <- matrix( 0, 2*I, 2 )
Qmatrix[1:I,1] <- 1
Qmatrix[1:I+I,2] <- 1

#***
# (2) estimation of models
# 2-dimensional Rasch model
mod1 <- sirt::rasch.mml2( dat, Qmatrix=Qmatrix )
summary(mod1)

# 2-dimensional 2PL model
mod2 <- sirt::rasch.mml2( dat, Qmatrix=Qmatrix, est.a=1:(2*I) )
summary(mod2)

# estimation with some fixed variances and covariances
# set variance of 1st dimension to 1 and
#  covariance to zero
variance.fixed <- matrix( cbind(c(1,1), c(1,2), c(1,0)),
             byrow=FALSE, ncol=3 )
mod3 <- sirt::rasch.mml2( dat, Qmatrix=Qmatrix, variance.fixed=variance.fixed )
summary(mod3)

# constraints on item difficulties
#  useful for example in longitudinal linking
est.b <- c( 1:I, 1:I )
    # equal indices correspond to equally estimated item parameters
mu.fixed <- cbind( 1, 0 )
mod4 <- sirt::rasch.mml2( dat, Qmatrix=Qmatrix, est.b=est.b, mu.fixed=mu.fixed )
summary(mod4)

#############################################################################
# EXAMPLE 6: Two booklets with same items but with item context effects.
# Therefore, item slopes and item difficulties are assumed to be shifted in the
# second design group.
#############################################################################

#***
# simulate data
set.seed(987)
I <- 10     # number of items
# define person design groups 1 and 2
n1 <- 700
n2 <- 1500
# item difficulties group 1
b1 <- seq(-1.5,1.5,length=I)
# item slopes group 1
a1 <- rep(1, I)
# simulate data group 1
dat1 <- sirt::sim.raschtype( stats::rnorm(n1), b=b1, fixed.a=a1 )
colnames(dat1) <- paste0("I", 1:I, "des1" )
# group 2
b2 <- b1 - .15
a2 <- 1.1*a1
# Item parameters are slightly transformed in the second group
# compared to the first group. This indicates possible item context effects.

# simulate data group 2
dat2 <- sirt::sim.raschtype( stats::rnorm(n2), b=b2, fixed.a=a2 )
colnames(dat2) <- paste0("I", 1:I, "des2" )
# define joint dataset
dat <- matrix( NA, nrow=n1+n2, ncol=2*I)
colnames(dat) <- c( colnames(dat1), colnames(dat2) )
dat[ 1:n1, 1:I ] <- dat1
dat[ n1 + 1:n2, I + 1:I ] <- dat2
# define group identifier
group <- c( rep(1,n1), rep(2,n2) )

#***
# Model 1: Rasch model two groups
itemindex <- rep( 1:I, 2 )
mod1 <- sirt::rasch.mml2( dat, group=group, est.b=itemindex )
summary(mod1)

#***
# Model 2: two item slope groups and designmatrix for intercepts
designmatrix <- matrix( 0, 2*I, I+1)
designmatrix[ ( 1:I )+ I,1:I] <- designmatrix[1:I,1:I] <- diag(I)
designmatrix[ ( 1:I )+ I,I+1] <- 1
mod2 <- sirt::rasch.mml2( dat, est.a=rep(1:2,each=I), designmatrix=designmatrix )
summary(mod2)

#############################################################################
# EXAMPLE 7: PIRLS dataset with missing responses
#############################################################################

data(data.pirlsmissing)
items <- grep( "R31", colnames(data.pirlsmissing), value=TRUE )
I <- length(items)
dat <- data.pirlsmissing

#****
# Model 1: recode missing responses as missing (missing are ignorable)

# data recoding
dat1 <- dat
dat1[ dat1==9 ] <- NA
# estimate Rasch model
mod1 <- sirt::rasch.mml2( dat1[,items], weights=dat$studwgt, group=dat$country )
summary(mod1)
##   Mean=0 0.341 -0.134 0.219
##   SD=1.142 1.166 1.197 0.959

#****
# Model 2: recode missing responses as wrong

# data recoding
dat2 <- dat
dat2[ dat2==9 ] <- 0
# estimate Rasch model
mod2 <- sirt::rasch.mml2( dat2[,items], weights=dat$studwgt, group=dat$country )
summary(mod2)
  ##   Mean=0 0.413 -0.172 0.446
  ##   SD=1.199 1.263 1.32 0.996

#****
# Model 3: recode missing responses as rho * P_i( theta ) and
#          apply pseudo-log-likelihood estimation
# Missing item responses are predicted by the model implied probability
# P_i( theta ) where theta is the ability estimate when ignoring missings (Model 1)
# and rho is an adjustment parameter. rho=0 is equivalent to Model 2 (treating
# missing as wrong) and rho=1 is equivalent to Model 1 (treating missing as ignorable).

# data recoding
dat3 <- dat
# simulate theta estimate from posterior distribution
theta <- stats::rnorm( nrow(dat3), mean=mod1$person$EAP, sd=mod1$person$SE.EAP )
rho <- .3   # define a rho parameter value of .3
for (ii in items){
    ind <- which( dat[,ii]==9 )
    dat3[ind,ii] <- rho*stats::plogis( theta[ind] - mod1$item$b[ which( items==ii ) ] )
                }

# estimate Rasch model
mod3 <- sirt::rasch.mml2( dat3[,items], weights=dat$studwgt, group=dat$country )
summary(mod3)
  ##   Mean=0 0.392 -0.153 0.38
  ##   SD=1.154 1.209 1.246 0.973

#****
# Model 4: simulate missing responses as rho * P_i( theta )
# The definition is the same as in Model 3. But it is now assumed
# that the missing responses are 'latent responses'.
set.seed(789)

# data recoding
dat4 <- dat
# simulate theta estimate from posterior distribution
theta <- stats::rnorm( nrow(dat4), mean=mod1$person$EAP, sd=mod1$person$SE.EAP )
rho <- .3   # define a rho parameter value of .3
for (ii in items){
    ind <- which( dat[,ii]==9 )
    p3 <- rho*stats::plogis( theta[ind] - mod1$item$b[ which( items==ii ) ] )
    dat4[ ind, ii ] <- 1*( stats::runif( length(ind), 0, 1 ) < p3)
                }

# estimate Rasch model
mod4 <- sirt::rasch.mml2( dat4[,items], weights=dat$studwgt, group=dat$country )
summary(mod4)
  ##   Mean=0 0.396 -0.156 0.382
  ##   SD=1.16 1.216 1.253 0.979

#****
# Model 5: recode missing responses for multiple choice items with four alternatives
#          to 1/4 and apply pseudo-log-likelihood estimation.
#          Missings for constructed response items are treated as incorrect.

# data recoding
dat5 <- dat
items_mc <- items[ substring( items, 7,7)=="M" ]
items_cr <- items[ substring( items, 7,7)=="C" ]
for (ii in items_mc){
    ind <- which( dat[,ii]==9 )
    dat5[ind,ii] <- 1/4
                }
for (ii in items_cr){
    ind <- which( dat[,ii]==9 )
    dat5[ind,ii] <- 0
                }

# estimate Rasch model
mod5 <- sirt::rasch.mml2( dat5[,items], weights=dat$studwgt, group=dat$country )
summary(mod5)
  ##   Mean=0 0.411 -0.165 0.435
  ##   SD=1.19 1.245 1.293 0.995

#*** For the following analyses, we ignore sample weights and the
#    country grouping.
data(data.pirlsmissing)
items <- grep( "R31", colnames(data.pirlsmissing), value=TRUE )
dat <- data.pirlsmissing
dat1 <- dat
dat1[ dat1==9 ] <- 0

#*** Model 6: estimate item difficulties assuming incorrect missing data treatment
mod6 <- sirt::rasch.mml2( dat1[,items], mmliter=50 )
summary(mod6)

#*** Model 7: reestimate model with constrained item difficulties
I <- length(items)
constraints <- cbind( 1:I, mod6$item$b )
mod7 <- sirt::rasch.mml2( dat1[,items], constraints=constraints)
summary(mod7)

#*** Model 8: score all missings responses as missing items
dat2 <- dat[,items]
dat2[ dat2==9 ] <- NA
mod8 <- sirt::rasch.mml2( dat2, constraints=constraints, mu.fixed=NULL )
summary(mod8)

#*** Model 9: estimate missing data model 'missing1' assuming a missingness
#       parameter delta.miss of zero
dat2 <-  dat[,items]    # note that missing item responses must be defined by 9
mod9 <- sirt::rasch.mml2( dat2, constraints=constraints, irtmodel="missing1",
            theta.k=seq(-5,5,len=10), delta.miss=0, mitermax=4, mu.fixed=NULL )
summary(mod9)

#*** Model 10: estimate missing data model with a large negative missing delta parameter
#=> This model is equivalent to treating missing responses as wrong
mod10 <- sirt::rasch.mml2( dat2, constraints=constraints, irtmodel="missing1",
             theta.k=seq(-5, 5, len=10), delta.miss=-10, mitermax=4, mmliter=200,
             mu.fixed=NULL )
summary(mod10)

#*** Model 11: choose a missingness delta parameter of -1
mod11 <- sirt::rasch.mml2( dat2, constraints=constraints, irtmodel="missing1",
             theta.k=seq(-5, 5, len=10), delta.miss=-1, mitermax=4,
             mmliter=200, mu.fixed=NULL )
summary(mod11)

#*** Model 12: estimate joint delta parameter
mod12 <- sirt::rasch.mml2( dat2, irtmodel="missing1", mu.fixed=cbind( c(1,2), 0 ),
             theta.k=seq(-8, 8, len=10), delta.miss=0, mitermax=4,
             mmliter=30, est.delta=rep(1,I)  )
summary(mod12)

#*** Model 13: estimate delta parameter in item groups defined by item format
est.delta <- 1 + 1 * ( substring( colnames(dat2),7,7 )=="M" )
mod13 <- sirt::rasch.mml2( dat2, irtmodel="missing1", mu.fixed=cbind( c(1,2), 0 ),
             theta.k=seq(-8, 8, len=10), delta.miss=0, mitermax=4,
             mmliter=30, est.delta=est.delta  )
summary(mod13)

#*** Model 14: estimate item specific delta parameter
mod14 <- sirt::rasch.mml2( dat2, irtmodel="missing1", mu.fixed=cbind( c(1,2), 0 ),
             theta.k=seq(-8, 8, len=10), delta.miss=0, mitermax=4,
             mmliter=30, est.delta=1:I  )
summary(mod14)

#############################################################################
# EXAMPLE 8: Comparison of different models for polytomous data
#############################################################################

data(data.Students, package="CDM")
head(data.Students)
dat <- data.Students[, paste0("act",1:5) ]
I <- ncol(dat)

#**************************************************
#*** Model 1: Partial Credit Model (PCM)

#*** Model 1a: PCM in TAM
mod1a <- TAM::tam.mml( dat )
summary(mod1a)

#*** Model 1b: PCM in sirt
mod1b <- sirt::rm.facets( dat )
summary(mod1b)

#*** Model 1c: PCM in mirt
mod1c <- mirt::mirt( dat, 1, itemtype=rep("Rasch",I), verbose=TRUE )
print(mod1c)

#**************************************************
#*** Model 2: Sequential Model (SM): Equal Loadings

#*** Model 2a: SM in sirt
dat1 <- CDM::sequential.items(dat)
resp <- dat1$dat.expand
iteminfo <- dat1$iteminfo
# fit model
mod2a <- sirt::rasch.mml2( resp )
summary(mod2a)

#**************************************************
#*** Model 3: Sequential Model (SM): Different Loadings

#*** Model 3a: SM in sirt
mod3a <- sirt::rasch.mml2( resp, est.a=iteminfo$itemindex )
summary(mod3a)

#**************************************************
#*** Model 4: Generalized partial credit model (GPCM)

#*** Model 4a: GPCM in TAM
mod4a <- TAM::tam.mml.2pl( dat, irtmodel="GPCM")
summary(mod4a)

#**************************************************
#*** Model 5: Graded response model (GRM)

#*** Model 5a: GRM in mirt
mod5a <- mirt::mirt( dat, 1, itemtype=rep("graded",I), verbose=TRUE)
print(mod5a)

# model comparison
logLik(mod1a);logLik(mod1b);mod1c@logLik  # PCM
logLik(mod2a)   # SM (Rasch)
logLik(mod3a)   # SM (GPCM)
logLik(mod4a)   # GPCM
mod5a@logLik    # GRM
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Marginal maximum likelihood (MML)}
%% \keyword{Ramsay's quotient model}
%% \keyword{Nonparametric item response theory}
%% \keyword{Pseudo-likelihood estimation}
%% \keyword{summary}
%% \keyword{plot}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: prmse.subscores.scales.Rd
%% File Version: 0.30

\name{prmse.subscores.scales}
\alias{prmse.subscores.scales}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Proportional Reduction of Mean Squared
Error (PRMSE) for Subscale Scores
}
\description{
This function estimates the proportional reduction of mean squared
error (PRMSE) according to
Haberman (Haberman 2008; Haberman, Sinharay & Puhan, 2008; see
Meijer et al. 2017 for an overview).
}
\usage{
prmse.subscores.scales(data, subscale)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
An \eqn{N \times I} data frame of item responses
}
  \item{subscale}{
Vector of labels corresponding to subscales
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
Matrix with columns corresponding to subscales \cr
The symbol \code{X} denotes the subscale and \code{Z}
the whole scale (see also in the Examples section for the structure of
this matrix).
}

\references{
Haberman, S. J. (2008). When can subscores have value?
\emph{Journal of Educational and Behavioral Statistics,
33}, 204-229.

Haberman, S., Sinharay, S., & Puhan, G. (2008).
Reporting subscores for institutions.
\emph{British Journal of Mathematical and Statistical Psychology,
62}, 79-95.

Meijer, R. R., Boeve, A. J., Tendeiro, J. N., Bosker, R. J., & Albers, C. J. (2017).
The use of subscores in higher education: When is this useful?.
\emph{Frontiers in Psychology | Educational Psychology, 8}.
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See the \pkg{subscore} package for computing subscores and the PRMSE measures,
especially \code{subscore::CTTsub}.
}
%% \code{\link[subscore:CTTsub]{subscore::CTTsub}}

\examples{
#############################################################################
# EXAMPLE 1: PRMSE Reading data data.read
#############################################################################

data( data.read )
p1 <- sirt::prmse.subscores.scales(data=data.read,
         subscale=substring( colnames(data.read), 1,1 ) )
print( p1, digits=3 )
  ##                 A       B       C
  ## N         328.000 328.000 328.000
  ## nX          4.000   4.000   4.000
  ## M.X         2.616   2.811   3.253
  ## Var.X       1.381   1.059   1.107
  ## SD.X        1.175   1.029   1.052
  ## alpha.X     0.545   0.381   0.640
  ## [...]
  ## nZ         12.000  12.000  12.000
  ## M.Z         8.680   8.680   8.680
  ## Var.Z       5.668   5.668   5.668
  ## SD.Z        2.381   2.381   2.381
  ## alpha.Z     0.677   0.677   0.677
  ## [...]
  ## cor.TX_Z    0.799   0.835   0.684
  ## rmse.X      0.585   0.500   0.505
  ## rmse.Z      0.522   0.350   0.614
  ## rmse.XZ     0.495   0.350   0.478
  ## prmse.X     0.545   0.381   0.640
  ## prmse.Z     0.638   0.697   0.468
  ## prmse.XZ    0.674   0.697   0.677
#-> Scales A and B do not have lower RMSE,
#   but for scale C the RMSE is smaller than the RMSE of a
#   prediction based on a whole scale.
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Proportional reduction of mean squared error (PRMSE)}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: isop.test.Rd
%% File Version: 0.25

\name{isop.test}
\alias{isop.test}
\alias{summary.isop.test}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Testing the ISOP Model
}
\description{
This function performs tests of the W1 axiom of the ISOP model
(Scheiblechner, 2003). Standard errors of the corresponding \eqn{W1_i} statistics
are obtained by Jackknife.
}

\usage{
isop.test(data, jackunits=20, weights=rep(1, nrow(data)))

\method{summary}{isop.test}(object,...)
}


%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
Data frame with item responses
}
  \item{jackunits}{
A number of Jackknife units (if an integer is provided as the argument
value) or a vector in the Jackknife units are already
defined.
}
  \item{weights}{
Optional vector of sampling weights
}
\item{object}{Object of class \code{isop.test}}
\item{\dots}{Further arguments to be passed}
}



%\details{
%%  ~~ If necessary, more details than the description above ~~
%}



\value{
A list with following entries

  \item{itemstat}{Data frame with test and item statistics for the W1 axiom.
  The \eqn{W1_i} statistic is denoted as \code{est} while \code{se}
  is the corresponding standard error of the statistic.
  The sample size per item is \code{N} and \code{M} denotes the item mean.}
  \item{Es}{Number of concordances per item}
  \item{Ed}{Number of disconcordances per item}

The \eqn{W1_i} statistics are printed by the \code{summary} method.
}

\references{
Scheiblechner, H. (2003). Nonparametric IRT: Testing the bi-isotonicity of
isotonic probabilistic models (ISOP). \emph{Psychometrika, 68},
79-96.
}


%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
Fit the ISOP model with \code{\link{isop.dich}} or \code{\link{isop.poly}}.

See also the \pkg{ISOP} package at \emph{Rforge}: \emph{http://www.rforge.net/ISOP/}.
}


\examples{
#############################################################################
# EXAMPLE 1: ISOP model data.Students
#############################################################################

data(data.Students, package="CDM")
dat <- data.Students[, paste0("act",1:5) ]
dat <- dat[1:300, ]    # select first 300 students

# perform the ISOP test
mod <- sirt::isop.test(dat)
summary(mod)
  ## -> W1i statistics
  ##     parm   N     M   est    se      t
  ##   1 test 300    NA 0.430 0.036 11.869
  ##   2 act1 278 0.601 0.451 0.048  9.384
  ##   3 act2 275 0.473 0.473 0.035 13.571
  ##   4 act3 274 0.277 0.352 0.098  3.596
  ##   5 act4 291 1.320 0.381 0.054  7.103
  ##   6 act5 276 0.460 0.475 0.042 11.184
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{ISOP model}
%% \keyword{summary}
%% \keyword{Nonparametric IRT}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.pirlsmissing.Rd
%% File Version: 0.11

\name{data.pirlsmissing}
\alias{data.pirlsmissing}

\docType{data}

\title{
Dataset from PIRLS Study with Missing Responses
}

\description{
This is a dataset of the PIRLS 2011 study for 4th graders for the reading
booklet 13 (the 'PIRLS reader') and 4 countries (Austria, Germany, France, Netherlands).
Missing responses (missing by intention and not reached) are coded by 9.
}

\usage{
data(data.pirlsmissing)
}

\format{
A data frame with 3480 observations on the following 38 variables.

The format is:

\code{'data.frame':   3480 obs. of  38 variables:} \cr
\code{ $ idstud  : int  1000001 1000002 1000003 1000004 1000005 ...} \cr
\code{ $ country : Factor w/ 4 levels "AUT","DEU","FRA",..: 1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{ $ studwgt : num  1.06 1.06 1.06 1.06 1.06 ...} \cr
\code{ $ R31G01M : int  1 1 1 1 1 1 0 1 1 0 ...} \cr
\code{ $ R31G02C : int  0 9 0 1 0 0 0 0 1 0 ...} \cr
\code{ $ R31G03M : int  1 1 1 1 0 1 0 0 1 1 ...} \cr
\code{ [...]} \cr
\code{ $ R31P15C : int  1 9 0 1 0 0 0 0 1 0 ...} \cr
\code{ $ R31P16C : int  0 0 0 0 0 0 0 9 0 1 ...} \cr

}

%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}
%\source{
%}
%\references{
%%  ~~ possibly secondary sources and usages ~~
%}



\examples{
data(data.pirlsmissing)
# inspect missing rates
round( colMeans( data.pirlsmissing==9 ), 3 )
  ##    idstud  country  studwgt  R31G01M  R31G02C  R31G03M  R31G04C  R31G05M
  ##     0.000    0.000    0.000    0.009    0.076    0.012    0.203    0.018
  ##   R31G06M  R31G07M R31G08CZ R31G08CA R31G08CB  R31G09M  R31G10C  R31G11M
  ##     0.010    0.020    0.189    0.225    0.252    0.019    0.126    0.023
  ##   R31G12C R31G13CZ R31G13CA R31G13CB R31G13CC  R31G14M  R31P01M  R31P02C
  ##     0.202    0.170    0.198    0.220    0.223    0.074    0.013    0.039
  ##   R31P03C  R31P04M  R31P05C  R31P06C  R31P07C  R31P08M  R31P09C  R31P10M
  ##     0.056    0.012    0.075    0.043    0.074    0.024    0.062    0.025
  ##   R31P11M  R31P12M  R31P13M  R31P14C  R31P15C  R31P16C
  ##     0.027    0.030    0.030    0.126    0.130    0.127
}


%% \keyword{datasets}

%% File Name: Q3.testlet.Rd
%% File Version: 1.15

\name{Q3.testlet}
\alias{Q3.testlet}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
\eqn{Q_3} Statistic of Yen (1984) for Testlets
}
\description{
This function calculates the average \eqn{Q_3} statistic (Yen, 1984) within and
between testlets.
}
\usage{
Q3.testlet(q3.res, testlet.matrix, progress=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{q3.res}{
An object generated by \code{\link{Q3}}
}
  \item{testlet.matrix}{
A matrix with two columns. The first column contains names of the testlets and
the second names of the items. See the examples for the definition of such
matrices.
}
\item{progress}{Logical indicating whether computation
progress should be displayed.}

}
\value{
A list with following entries
  \item{testlet.q3}{Data frame with average \eqn{Q_3} statistics within testlets}
  \item{testlet.q3.korr}{Matrix of average \eqn{Q_3} statistics within and
      between testlets}
}

\references{
Yen, W. M. (1984). Effects of local item dependence on the fit and equating
performance of the three-parameter logistic model.
\emph{Applied Psychological Measurement, 8}, 125-145.
}


%\author{
%Alexander Robitzsch
%}




%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For estimating all \eqn{Q_3} statistics between item pairs use
\code{\link{Q3}}.
}


\examples{
#############################################################################
# EXAMPLE 1: data.read. The 12 items are arranged in 4 testlets
#############################################################################
data(data.read)

# estimate the Rasch model
mod <- sirt::rasch.mml2( data.read)
mod$item

# estmate WLEs
mod.wle <- sirt::wle.rasch( dat=data.read, b=mod$item$b )

# Yen's Q3 statistic
mod.q3 <- sirt::Q3( dat=data.read, theta=mod.wle$theta, b=mod$item$b )

# Yen's Q3 statistic with testlets
items <- colnames(data.read)
testlet.matrix <- cbind( substring(  items,1,1), items )
mod.testletq3 <- sirt::Q3.testlet( q3.res=mod.q3,testlet.matrix=testlet.matrix)
mod.testletq3
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Testlets}
%% \keyword{Local dependence}
%% \keyword{Q3}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: sia.sirt.Rd
%% File Version: 0.242

\name{sia.sirt}
\alias{sia.sirt}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Statistical Implicative Analysis (SIA)
}


\description{
This function is a simplified implementation of statistical implicative
analysis (Gras & Kuntz, 2008) which aims at deriving implications
\eqn{X_i \rightarrow X_j}. This means that solving item \eqn{i} implies
solving item \eqn{j}.
}

\usage{
sia.sirt(dat, significance=0.85)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame with dichotomous item responses
}
  \item{significance}{
Minimum implicative probability for inclusion of an arrow in the graph.
The probability can be interpreted as a kind of significance level, i.e.
higher probabilities indicate more probable implications.
}
}

\details{
The test statistic for selection an implicative relation follows
Gras and Kuntz (2008).
Transitive arrows (implications) are removed from the graph.
If some implications are symmetric, then only the more probable
implication will be retained.
}


\value{
A list with following entries

  \item{adj.matrix}{Adjacency matrix of the graph. Transitive and
  symmetric implications (arrows) have been removed.}
  \item{adj.pot}{Adjacency matrix including all powers, i.e. all direct and
  indirect paths from item \eqn{i} to item \eqn{j}.}
  \item{adj.matrix.trans}{Adjacency matrix including transitive arrows.}
  \item{desc}{List with descriptive statistics of the graph.}
  \item{desc.item}{Descriptive statistics for each item.}
  \item{impl.int}{Implication intensity (probability) as the basis
  for deciding the significance of an arrow}
  \item{impl.t}{Corresponding \eqn{t} values of \code{impl.int}}
   \item{impl.significance}{Corresponding \eqn{p} values (significancies)
   of \code{impl.int}}
  \item{conf.loev}{Confidence according to Loevinger (see Gras & Kuntz, 2008).
  This values are just conditional probabilities \eqn{P( X_j=1|X_i=1)}.}
  \item{graph.matr}{Matrix containing all arrows. Can be used
  for example for the \pkg{Rgraphviz} package.}
  \item{graph.edges}{Vector containing all edges of the graph, e.g. for
  the \pkg{Rgraphviz} package.}
  \item{igraph.matr}{Matrix containing all arrows for the \pkg{igraph}
      package.}
  \item{igraph.obj}{An object of the graph for the \pkg{igraph} package.}

}

%  \item{comp1 }{xx}
%  \item{comp2 }{xx}


\references{
Gras, R., & Kuntz, P. (2008). An overview of the statistical implicative analysis
(SIA) development. In R. Gras, E. Suzuki, F. Guillet, & F. Spagnolo (Eds.).
\emph{Statistical Implicative Analysis} (pp. 11-40).
Springer, Berlin Heidelberg.
}


%\author{
%Alexander Robitzsch
%}




\note{
For an implementation of statistical implicative analysis in the
C.H.I.C. (Classification Hierarchique, Implicative et Cohesitive)
software.

See \url{https://ardm.eu/partenaires/logiciel-danalyse-de-donnees-c-h-i-c/}.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also the \pkg{IsingFit} package for calculating a graph for
dichotomous item responses using the Ising model.
}

%% ~~objects to See Also as \code{\link{help}}, ~~~

\examples{
#############################################################################
# EXAMPLE 1: SIA for data.read
#############################################################################

data(data.read)
dat <- data.read

res <- sirt::sia.sirt(dat, significance=.85 )

#*** plot results with igraph package
library(igraph)
plot( res$igraph.obj ) #, vertex.shape="rectangle", vertex.size=30 )

\dontrun{
#*** plot results with qgraph package
miceadds::library_install(qgraph)
qgraph::qgraph( res$adj.matrix )

#*** plot results with Rgraphviz package
# Rgraphviz can only be obtained from Bioconductor
# If it should be downloaded, select TRUE for the following lines
if (FALSE){
     source("http://bioconductor.org/biocLite.R")
     biocLite("Rgraphviz")
            }
# define graph
grmatrix <- res$graph.matr
res.graph <- new("graphNEL", nodes=res$graph.edges, edgemode="directed")
# add edges
RR <- nrow(grmatrix)
for (rr in 1:RR){
    res.graph <- Rgraphviz::addEdge(grmatrix[rr,1], grmatrix[rr,2], res.graph, 1)
                    }
# define cex sizes and shapes
V <- length(res$graph.edges)
size2 <- rep(16,V)
shape2 <- rep("rectangle", V )
names(shape2) <- names(size2) <- res$graph.edges
# plot graph
Rgraphviz::plot( res.graph, nodeAttrs=list("fontsize"=size2, "shape"=shape2) )
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Statistical implicative analysis}
%% \keyword{Graphical modeling}
% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: lc.2raters.Rd
%% File Version: 0.23

\name{lc.2raters}
\alias{lc.2raters}
\alias{summary.lc.2raters}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Latent Class Model for Two Exchangeable Raters and One Item
}


\description{
This function computes a latent class model for ratings on an item
based on exchangeable raters (Uebersax & Grove, 1990). Additionally,
several measures of rater agreement are computed (see e.g. Gwet, 2010).
}

\usage{
lc.2raters(data, conv=0.001, maxiter=1000, progress=TRUE)

\method{summary}{lc.2raters}(object,...)
}



%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
Data frame with item responses (must be ordered from 0 to \eqn{K}) and two
columns which correspond to ratings of two (exchangeable) raters.
}
  \item{conv}{
Convergence criterion
}
  \item{maxiter}{
Maximum number of iterations
}
  \item{progress}{
An optional logical indicating whether iteration progress should be displayed.
}
\item{object}{Object of class \code{lc.2raters}}
\item{\dots}{Further arguments to be passed}
}


\details{
For two exchangeable raters which provide ratings on an item, a latent
class model with \eqn{K+1} classes (if there are \eqn{K+1} item categories
\eqn{0,...,K}) is defined. Where \eqn{P(X=x, Y=y | c)} denotes
the probability that the first rating is \eqn{x} and the second rating is
\eqn{y} given the true but unknown item category (class) \eqn{c}. Ratings are
assumed to be locally independent, i.e.
\deqn{ P(X=x, Y=y | c )=P( X=x | c) \cdot P(Y=y | c )=p_{x|c} \cdot p_{y|c}}
Note that \eqn{P(X=x|c)=P(Y=x|c)=p_{x|c}} holds due to the exchangeability of raters.
The latent class model estimates true class proportions \eqn{\pi_c} and
conditional item probabilities \eqn{p_{x|c}}.
}

\value{
A list with following entries

\item{classprob.1rater.like}{Classification probability \eqn{P(c|x)} of latent
category \eqn{c} given a manifest rating \eqn{x} (estimated by maximum likelihood)}
\item{classprob.1rater.post}{Classification probability \eqn{P(c|x)} of latent
category \eqn{c} given a manifest rating \eqn{x} (estimated by the posterior
distribution)}
\item{classprob.2rater.like}{Classification probability \eqn{P(c|(x,y))}
of latent category \eqn{c} given two manifest ratings \eqn{x} and \eqn{y}
(estimated by maximum likelihood)}
\item{classprob.2rater.post}{Classification probability \eqn{P(c|(x,y))}
of latent category \eqn{c} given two manifest ratings \eqn{x} and \eqn{y}
(estimated by posterior distribution)}
\item{f.yi.qk}{Likelihood of each pair of ratings}
\item{f.qk.yi}{Posterior of each pair of ratings}
\item{probs}{Item response probabilities \eqn{p_{x|c}}}
\item{pi.k}{Estimated class proportions \eqn{\pi_c}}
\item{pi.k.obs}{Observed manifest class proportions}
\item{freq.long}{Frequency table of ratings in long format}
\item{freq.table}{Symmetrized frequency table of ratings}
\item{agree.stats}{Measures of rater agreement. These measures include
percentage agreement (\code{agree0}, \code{agree1}), Cohen's kappa and weighted
Cohen's kappa (\code{kappa}, \code{wtd.kappa.linear}),
Gwet's AC1 agreement measures (\code{AC1}; Gwet, 2008, 2010) and
Aickin's alpha (\code{alpha.aickin}; Aickin, 1990).
}
\item{data}{Used dataset}
\item{N.categ}{Number of categories}
}

\references{
Aickin, M. (1990). Maximum likelihood estimation of agreement in the constant
predictive probability model, and its relation to Cohen's kappa.
\emph{Biometrics, 46}, 293-302.

Gwet, K. L. (2008). Computing inter-rater reliability and its variance
in the presence of high agreement.
\emph{British Journal of Mathematical and Statistical Psychology,
61}, 29-48.

Gwet, K. L. (2010). \emph{Handbook of Inter-Rater Reliability}.
Advanced Analytics, Gaithersburg. \emph{http://www.agreestat.com/}

Uebersax, J. S., & Grove, W. M. (1990). Latent class analysis of diagnostic
agreement. \emph{Statistics in Medicine, 9}, 559-572.
}



%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also \code{\link{rm.facets}} and \code{\link{rm.sdt}} for
specifying rater models.

See also the \pkg{irr} package for measures of rater agreement.
}
% rm.hrm, rm.facets


\examples{
#############################################################################
# EXAMPLE 1: Latent class models for rating datasets data.si05
#############################################################################

data(data.si05)

#*** Model 1: one item with two categories
mod1 <- sirt::lc.2raters( data.si05$Ex1)
summary(mod1)

#*** Model 2: one item with five categories
mod2 <- sirt::lc.2raters( data.si05$Ex2)
summary(mod2)

#*** Model 3: one item with eight categories
mod3 <- sirt::lc.2raters( data.si05$Ex3)
summary(mod3)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{summary}
%% \keyword{Rater model}
%% \keyword{Latent class model}

%% File Name: data.reck.Rd
%% File Version: 0.29

\name{data.reck}

\alias{data.reck}
\alias{data.reck21}
\alias{data.reck61DAT1}
\alias{data.reck61DAT2}
\alias{data.reck73C1a}
\alias{data.reck73C1b}
\alias{data.reck75C2}
\alias{data.reck78ExA}
\alias{data.reck79ExB}

\docType{data}


\title{
Datasets from Reckase' Book \emph{Multidimensional Item Response Theory}
}
\description{
Some simulated datasets from Reckase (2009).
}
\usage{
data(data.reck21)
data(data.reck61DAT1)
data(data.reck61DAT2)
data(data.reck73C1a)
data(data.reck73C1b)
data(data.reck75C2)
data(data.reck78ExA)
data(data.reck79ExB)
}

\format{
\itemize{
\item The format of the \code{data.reck21} (Table 2.1, p. 45) is:

\code{List of 2} \cr
\code{ $ data: num [1:2500, 1:50] 0 0 0 1 1 0 0 0 1 0 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : NULL} \cr
\code{  .. ..$ : chr [1:50] "I0001" "I0002" "I0003" "I0004" ...} \cr
\code{ $ pars:'data.frame':} \cr
\code{  ..$ a: num [1:50] 1.83 1.38 1.47 1.53 0.88 0.82 1.02 1.19 1.15 0.18 ...} \cr
\code{  ..$ b: num [1:50] 0.91 0.81 0.06 -0.8 0.24 0.99 1.23 -0.47 2.78 -3.85 ...} \cr
\code{  ..$ c: num [1:50] 0 0 0 0.25 0.21 0.29 0.26 0.19 0 0.21 ...} \cr

\item The format of the datasets \code{data.reck61DAT1} and
\code{data.reck61DAT2} (Table 6.1, p. 153) is

\code{List of 4} \cr
\code{ $ data : num [1:2500, 1:30] 1 0 0 1 1 0 0 1 1 0 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : NULL} \cr
\code{  .. ..$ : chr [1:30] "A01" "A02" "A03" "A04" ...} \cr
\code{ $ pars :'data.frame':} \cr
\code{  ..$ a1: num [1:30] 0.747 0.46 0.861 1.014 0.552 ...} \cr
\code{  ..$ a2: num [1:30] 0.025 0.0097 0.0067 0.008 0.0204 0.0064 0.0861 ...} \cr
\code{  ..$ a3: num [1:30] 0.1428 0.0692 0.404 0.047 0.1482 ...} \cr
\code{  ..$ d : num [1:30] 0.183 -0.192 -0.466 -0.434 -0.443 ...} \cr
\code{ $ mu   : num [1:3] -0.4 -0.7 0.1} \cr
\code{ $ sigma: num [1:3, 1:3] 1.21 0.297 1.232 0.297 0.81 ...} \cr



The dataset \code{data.reck61DAT2} has correlated dimensions while
\code{data.reck61DAT1} has uncorrelated dimensions.

\item Datasets \code{data.reck73C1a} and \code{data.reck73C1b} use item parameters
from Table 7.3 (p. 188). The dataset \code{C1a} has uncorrelated dimensions,
while \code{C1b} has perfectly correlated dimensions. The items are sensitive to
3 dimensions. The format of the datasets is

\code{List of 4} \cr
\code{ $ data : num [1:2500, 1:30] 1 0 1 1 1 0 1 1 1 1 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : NULL} \cr
\code{  .. ..$ : chr [1:30] "A01" "A02" "A03" "A04" ...} \cr
\code{ $ pars :'data.frame':  30 obs. of  4 variables:} \cr
\code{  ..$ a1: num [1:30] 0.747 0.46 0.861 1.014 0.552 ...} \cr
\code{  ..$ a2: num [1:30] 0.025 0.0097 0.0067 0.008 0.0204 0.0064 ...} \cr
\code{  ..$ a3: num [1:30] 0.1428 0.0692 0.404 0.047 0.1482 ...} \cr
\code{  ..$ d : num [1:30] 0.183 -0.192 -0.466 -0.434 -0.443 ...} \cr
\code{ $ mu   : num [1:3] 0 0 0} \cr
\code{ $ sigma: num [1:3, 1:3] 0.167 0.236 0.289 0.236 0.334 ...} \cr

\item The dataset \code{data.reck75C2} is simulated using item parameters
from Table 7.5 (p. 191). It contains items which are sensitive to only
one dimension but individuals which have abilities in three
uncorrelated dimensions. The format is

\code{List of 4} \cr
\code{ $ data : num [1:2500, 1:30] 0 0 1 1 1 0 0 1 1 1 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : NULL} \cr
\code{  .. ..$ : chr [1:30] "A01" "A02" "A03" "A04" ...} \cr
\code{ $ pars :'data.frame':  30 obs. of  4 variables:} \cr
\code{  ..$ a1: num [1:30] 0.56 0.48 0.67 0.57 0.54 0.74 0.7 0.59 0.63 0.64 ...} \cr
\code{  ..$ a2: num [1:30] 0.62 0.53 0.63 0.69 0.58 0.69 0.75 0.63 0.64 0.64 ...} \cr
\code{  ..$ a3: num [1:30] 0.46 0.42 0.43 0.51 0.41 0.48 0.46 0.5 0.51 0.46 ...} \cr
\code{  ..$ d : num [1:30] 0.1 0.06 -0.38 0.46 0.14 0.31 0.06 -1.23 0.47 1.06 ...} \cr
\code{ $ mu   : num [1:3] 0 0 0} \cr
\code{ $ sigma: num [1:3, 1:3] 1 0 0 0 1 0 0 0 1} \cr

\item The dataset \code{data.reck78ExA} contains simulated item responses
from Table 7.8 (p. 204 ff.). There are three item clusters and
two ability dimensions. The format is

\code{List of 4} \cr
\code{ $ data : num [1:2500, 1:50] 0 1 1 0 1 0 0 0 0 0 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : NULL} \cr
\code{  .. ..$ : chr [1:50] "A01" "A02" "A03" "A04" ...} \cr
\code{ $ pars :'data.frame':  50 obs. of  3 variables:} \cr
\code{  ..$ a1: num [1:50] 0.889 1.057 1.047 1.178 1.029 ...} \cr
\code{  ..$ a2: num [1:50] 0.1399 0.0432 0.016 0.0231 0.2347 ...} \cr
\code{  ..$ d : num [1:50] 0.2724 1.2335 -0.0918 -0.2372 0.8471 ...} \cr
\code{ $ mu   : num [1:2] 0 0} \cr
\code{ $ sigma: num [1:2, 1:2] 1 0 0 1} \cr

\item The dataset \code{data.reck79ExB} contains simulated item responses
from Table 7.9 (p. 207 ff.). There are three item clusters and
three ability dimensions. The format is

\code{List of 4} \cr
\code{ $ data : num [1:2500, 1:50] 1 1 0 1 0 0 0 1 1 0 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : NULL} \cr
\code{  .. ..$ : chr [1:50] "A01" "A02" "A03" "A04" ...} \cr
\code{ $ pars :'data.frame':  50 obs. of  4 variables:} \cr
\code{  ..$ a1: num [1:50] 0.895 1.032 1.036 1.163 1.022 ...} \cr
\code{  ..$ a2: num [1:50] 0.052 0.132 0.144 0.13 0.165 ...} \cr
\code{  ..$ a3: num [1:50] 0.0722 0.1923 0.0482 0.1321 0.204 ...} \cr
\code{  ..$ d : num [1:50] 0.2724 1.2335 -0.0918 -0.2372 0.8471 ...} \cr
\code{ $ mu   : num [1:3] 0 0 0} \cr
\code{ $ sigma: num [1:3, 1:3] 1 0 0 0 1 0 0 0 1} \cr

}
  }
%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}

\source{
Simulated datasets
}



\references{
Reckase, M. (2009). \emph{Multidimensional item response theory}.
New York: Springer.
\doi{10.1007/978-0-387-89976-3}
}

\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: data.reck21 dataset, Table 2.1, p. 45
#############################################################################
data(data.reck21)

dat <- data.reck21$dat      # extract dataset

# items with zero guessing parameters
guess0 <- c( 1, 2, 3, 9,11,27,30,35,45,49,50 )
I <- ncol(dat)

#***
# Model 1: 3PL estimation using rasch.mml2
est.c <- est.a <- 1:I
est.c[ guess0 ] <- 0
mod1 <- sirt::rasch.mml2( dat, est.a=est.a, est.c=est.c, mmliter=300 )
summary(mod1)

#***
# Model 2: 3PL estimation using smirt
Q <- matrix(1,I,1)
mod2 <- sirt::smirt( dat, Qmatrix=Q, est.a="2PL", est.c=est.c, increment.factor=1.01)
summary(mod2)

#***
# Model 3: estimation in mirt package
library(mirt)
itemtype <- rep("3PL", I )
itemtype[ guess0 ] <- "2PL"
mod3 <- mirt::mirt(dat, 1, itemtype=itemtype, verbose=TRUE)
summary(mod3)

c3 <- unlist( coef(mod3) )[ 1:(4*I) ]
c3 <- matrix( c3, I, 4, byrow=TRUE )
# compare estimates of rasch.mml2, smirt and true parameters
round( cbind( mod1$item$c, mod2$item$c,c3[,3],data.reck21$pars$c ), 2 )
round( cbind( mod1$item$a, mod2$item$a.Dim1,c3[,1], data.reck21$pars$a ), 2 )
round( cbind( mod1$item$b, mod2$item$b.Dim1 / mod2$item$a.Dim1, - c3[,2] / c3[,1],
            data.reck21$pars$b ), 2 )

#############################################################################
# EXAMPLE 2: data.reck61 dataset, Table 6.1, p. 153
#############################################################################

data(data.reck61DAT1)
dat <- data.reck61DAT1$data

#***
# Model 1: Exploratory factor analysis

#-- Model 1a: tam.fa in TAM
library(TAM)
mod1a <- TAM::tam.fa( dat, irtmodel="efa", nfactors=3 )
# varimax rotation
varimax(mod1a$B.stand)

# Model 1b: EFA in NOHARM (Promax rotation)
mod1b <- sirt::R2noharm( dat=dat, model.type="EFA",  dimensions=3,
              writename="reck61__3dim_efa", noharm.path="c:/NOHARM",dec=",")
summary(mod1b)

# Model 1c: EFA with noharm.sirt
mod1c <- sirt::noharm.sirt( dat=dat, dimensions=3  )
summary(mod1c)
plot(mod1c)

# Model 1d: EFA with 2 dimensions in noharm.sirt
mod1d <- sirt::noharm.sirt( dat=dat, dimensions=2  )
summary(mod1d)
plot(mod1d, efa.load.min=.2)   # plot loadings of at least .20

#***
# Model 2: Confirmatory factor analysis

#-- Model 2a: tam.fa in TAM
dims <- c( rep(1,10), rep(3,10), rep(2,10)  )
Qmatrix <- matrix( 0, nrow=30, ncol=3 )
Qmatrix[ cbind( 1:30, dims) ] <- 1
mod2a <- TAM::tam.mml.2pl( dat,Q=Qmatrix,
            control=list( snodes=1000, QMC=TRUE, maxiter=200) )
summary(mod2a)

#-- Model 2b: smirt in sirt
mod2b <- sirt::smirt( dat,Qmatrix=Qmatrix, est.a="2PL", maxiter=20, qmcnodes=1000 )
summary(mod2b)

#-- Model 2c: rasch.mml2 in sirt
mod2c <- sirt::rasch.mml2( dat,Qmatrix=Qmatrix, est.a=1:30,
                mmliter=200, theta.k=seq(-5,5,len=11) )
summary(mod2c)

#-- Model 2d: mirt in mirt
cmodel <- mirt::mirt.model("
     F1=1-10
     F2=21-30
     F3=11-20
     COV=F1*F2, F1*F3, F2*F3 " )
mod2d <- mirt::mirt(dat, cmodel, verbose=TRUE)
summary(mod2d)
coef(mod2d)

#-- Model 2e: CFA in NOHARM
# specify covariance pattern
P.pattern <- matrix( 1, ncol=3, nrow=3 )
P.init <- .4*P.pattern
diag(P.pattern) <- 0
diag(P.init) <- 1
# fix all entries in the loading matrix to 1
F.pattern <- matrix( 0, nrow=30, ncol=3 )
F.pattern[1:10,1] <- 1
F.pattern[21:30,2] <- 1
F.pattern[11:20,3] <- 1
F.init <- F.pattern
# estimate model
mod2e <- sirt::R2noharm( dat=dat, model.type="CFA", P.pattern=P.pattern,
            P.init=P.init, F.pattern=F.pattern, F.init=F.init,
            writename="reck61__3dim_cfa", noharm.path="c:/NOHARM",dec=",")
summary(mod2e)

#-- Model 2f: CFA with noharm.sirt
mod2f <- sirt::noharm.sirt( dat=dat, Fval=F.init, Fpatt=F.pattern,
                 Pval=P.init, Ppatt=P.pattern )
summary(mod2f)

#############################################################################
# EXAMPLE 3: DETECT analysis data.reck78ExA and data.reck79ExB
#############################################################################

data(data.reck78ExA)
data(data.reck79ExB)

#************************
# Example A
dat <- data.reck78ExA$data
#- estimate person score
score <- stats::qnorm( ( rowMeans( dat )+.5 )  / ( ncol(dat) + 1 ) )
#- extract item cluster
itemcluster <- substring( colnames(dat), 1, 1 )
#- confirmatory DETECT Item cluster
detectA <- sirt::conf.detect( data=dat, score=score, itemcluster=itemcluster )
  ##          unweighted weighted
  ##   DETECT      0.571    0.571
  ##   ASSI        0.523    0.523
  ##   RATIO       0.757    0.757

#- exploratory DETECT analysis
detect_explA <- sirt::expl.detect(data=dat, score, nclusters=10, N.est=nrow(dat)/2  )
  ##  Optimal Cluster Size is  5  (Maximum of DETECT Index)
  ##     N.Cluster N.items N.est N.val         size.cluster DETECT.est ASSI.est
  ##   1         2      50  1250  1250                31-19      0.531    0.404
  ##   2         3      50  1250  1250             10-19-21      0.554    0.407
  ##   3         4      50  1250  1250           10-19-14-7      0.630    0.509
  ##   4         5      50  1250  1250         10-19-3-7-11      0.653    0.546
  ##   5         6      50  1250  1250       10-12-7-3-7-11      0.593    0.458
  ##   6         7      50  1250  1250      10-12-7-3-7-9-2      0.604    0.474
  ##   7         8      50  1250  1250    10-12-7-3-3-9-4-2      0.608    0.481
  ##   8         9      50  1250  1250  10-12-7-3-3-5-4-2-4      0.617    0.494
  ##   9        10      50  1250  1250 10-5-7-7-3-3-5-4-2-4      0.592    0.460

# cluster membership
cluster_membership <- detect_explA$itemcluster$cluster3
# Cluster 1:
colnames(dat)[ cluster_membership==1 ]
  ##   [1] "A01" "A02" "A03" "A04" "A05" "A06" "A07" "A08" "A09" "A10"
# Cluster 2:
colnames(dat)[ cluster_membership==2 ]
  ##    [1] "B11" "B12" "B13" "B14" "B15" "B16" "B17" "B18" "B19" "B20" "B21" "B22"
  ##   [13] "B23" "B25" "B26" "B27" "B28" "B29" "B30"
# Cluster 3:
colnames(dat)[ cluster_membership==3 ]
  ##    [1] "B24" "C31" "C32" "C33" "C34" "C35" "C36" "C37" "C38" "C39" "C40" "C41"
  ##   [13] "C42" "C43" "C44" "C45" "C46" "C47" "C48" "C49" "C50"

#************************
# Example B
dat <- data.reck79ExB$data
#- estimate person score
score <- stats::qnorm( ( rowMeans( dat )+.5 )  / ( ncol(dat) + 1 ) )
#- extract item cluster
itemcluster <- substring( colnames(dat), 1, 1 )
#- confirmatory DETECT Item cluster
detectB <- sirt::conf.detect( data=dat, score=score, itemcluster=itemcluster )
  ##          unweighted weighted
  ##   DETECT      0.715    0.715
  ##   ASSI        0.624    0.624
  ##   RATIO       0.855    0.855

#- exploratory DETECT analysis
detect_explB <- sirt::expl.detect(data=dat, score, nclusters=10, N.est=nrow(dat)/2  )
  ##   Optimal Cluster Size is  4  (Maximum of DETECT Index)
  ##
  ##     N.Cluster N.items N.est N.val         size.cluster DETECT.est ASSI.est
  ##   1         2      50  1250  1250                30-20      0.665    0.546
  ##   2         3      50  1250  1250             10-20-20      0.686    0.585
  ##   3         4      50  1250  1250           10-20-8-12      0.728    0.644
  ##   4         5      50  1250  1250         10-6-14-8-12      0.654    0.553
  ##   5         6      50  1250  1250       10-6-14-3-12-5      0.659    0.561
  ##   6         7      50  1250  1250      10-6-14-3-7-5-5      0.664    0.576
  ##   7         8      50  1250  1250     10-6-7-7-3-7-5-5      0.616    0.518
  ##   8         9      50  1250  1250   10-6-7-7-3-5-5-5-2      0.612    0.512
  ##   9        10      50  1250  1250 10-6-7-7-3-5-3-5-2-2      0.613    0.512
}
}
%% \keyword{datasets}

%% File Name: gom.jml.Rd
%% File Version: 0.23

\name{gom.jml}
\alias{gom.jml}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Grade of Membership Model (Joint Maximum Likelihood Estimation)
}
\description{
This function estimates the grade of membership model
employing a joint maximum likelihood estimation
method (Erosheva, 2002; p. 23ff.).
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ARGUMENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\usage{
gom.jml(dat, K=2, seed=NULL, globconv=0.001, maxdevchange=0.001,
        maxiter=600, min.lambda=0.001, min.g=0.001)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
    Data frame of dichotomous item responses
}
  \item{K}{
    Number of classes
}
\item{seed}{
    Seed value of random number generator. Deterministic starting values
    are used for the default value \code{NULL}.
}
  \item{globconv}{
    Global parameter convergence criterion
}
  \item{maxdevchange}{
    Maximum change in relative deviance
}
  \item{maxiter}{
    Maximum number of iterations
}
  \item{min.lambda}{
    Minimum \eqn{\lambda_{ik}} parameter to be estimated
}
  \item{min.g}{
    Minimum \eqn{g_{pk}} parameter to be estimated
}
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DETAILS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\details{
The item response model of the grade of membership model with \eqn{K} classes
for dichotomous correct responses \eqn{X_{pi}}
of person \eqn{p} on item \eqn{i} is
\deqn{
    P(X_{pi}=1 | g_{p1}, \ldots, g_{pK} )=\sum_k \lambda_{ik} g_{pk}
\quad, \quad \sum_k g_{pk}=1
                }{%
    P(X_{pi}=1 | g_{p1}, \ldots, g_{pK} )=\sum_k \lambda_{ik} g_{pk}
,    \sum_k g_{pk}=1
            }
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VALUES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\value{
A list with following entries:
\item{lambda}{Data frame of item parameters \eqn{\lambda_{ik}}}
\item{g}{Data frame of individual membership scores \eqn{g_{pk}}}
\item{g.mean}{Mean membership scores}
\item{gcut}{Discretized membership scores}
\item{gcut.distr}{Distribution of discretized membership scores}
\item{K}{Number of classes}
\item{deviance}{Deviance}
\item{ic}{Information criteria}
\item{N}{Number of students}
\item{score}{Person score}
\item{iter}{Number of iterations}
\item{datproc}{List with processed data (recoded data, starting values, ...)}
\item{\dots}{Further values}
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% REFERENCES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



\references{
Erosheva, E. A. (2002). \emph{Grade of membership
and latent structure models with application to
disability survey data}.
PhD thesis, Carnegie Mellon University, Department
of Statistics.
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
S3 method \code{\link{summary.gom}}
}



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EXAMPLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\examples{
#############################################################################
# EXAMPLE 1: TIMSS data
#############################################################################

data( data.timss)
dat <- data.timss$data[, grep("M", colnames(data.timss$data) ) ]

# 2 Classes (deterministic starting values)
m2 <- sirt::gom.jml(dat,K=2, maxiter=10 )
summary(m2)

\dontrun{
# 3 Classes with fixed seed and maximum number of iterations
m3 <- sirt::gom.jml(dat,K=3, maxiter=50,seed=89)
summary(m3)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Grade of membership model}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: sirt-utilities.Rd
%% File Version: 0.408

\name{sirt-utilities}
\alias{sirt-utilities}
\alias{tracemat}
\alias{pow}
\alias{soft_thresholding}
\alias{hard_thresholding}
\alias{bounds_parameters}
\alias{dimproper}
\alias{ginverse_sym}
\alias{sirt_rbind_fill}
\alias{sirt_fisherz}
\alias{sirt_antifisherz}
\alias{sirt_attach_list_elements}
\alias{sirt_summary_print_objects}
\alias{sirt_summary_print_package_rsession}
\alias{sirt_summary_print_package}
\alias{sirt_summary_print_rsession}
\alias{sirt_summary_print_call}
\alias{sirt_optimizer}
\alias{sirt_matrix2}
\alias{sirt_colMeans}
\alias{sirt_colSDs}
\alias{sirt_colMins}
\alias{sirt_colMaxs}
\alias{sirt_colMedians}
\alias{sirt_sum_norm}
\alias{sirt_dnorm_discrete}
\alias{sirt_abs_smooth}
\alias{sirt_permutations}
\alias{sirt_rcpp_discrete_inverse}
\alias{move_variables_df}
\alias{print_digits}


\title{Utility Functions in \pkg{sirt}}

\description{
Utility functions in \pkg{sirt}.
}

\usage{
# bounds entries in a vector
bounds_parameters( pars, lower=NULL, upper=NULL)

# improper density function which always returns a value of 1
dimproper(x)

# generalized inverse of a symmetric function
ginverse_sym(A, eps=1E-8)
# hard thresholding function
hard_thresholding(x, lambda)
# soft thresholding function
soft_thresholding(x, lambda)

# power function x^a, like in Cpp
pow(x, a)
# trace of a matrix
tracemat(A)

#** matrix functions
sirt_matrix2(x, nrow)   # matrix() function with byrow=TRUE
sirt_colMeans(x, na.rm=TRUE)
sirt_colSDs(x, na.rm=TRUE)
sirt_colMins(x, na.rm=TRUE)
sirt_colMaxs(x, na.rm=TRUE)
sirt_colMedians(x, na.rm=TRUE)

#* normalize vector to have sum of one
sirt_sum_norm(x, na.rm=TRUE)
#* discrete normal distribution
sirt_dnorm_discrete(x, mean=0, sd=1, ...)

# plyr::rbind.fill implementation in sirt
sirt_rbind_fill(x, y)

# Fisher-z transformation, see psych::fisherz
sirt_fisherz(rho)
# inverse Fisher-z transformation, see psych::fisherz2r
sirt_antifisherz(z)

# smooth approximation of the absolute value function
sirt_abs_smooth(x, deriv=0, eps=1e-4)

# permutations with replacement
sirt_permutations(r,v)
  #-> is equivalent to gtools::permutations(n=length(v), r=D, v=v, repeats.allowed=TRUE)

# attach all elements in a list in a specified environment
sirt_attach_list_elements(x, envir)

# switch between stats::optim and stats::nlminb
sirt_optimizer(optimizer, par, fn, grad=NULL, method="L-BFGS-B", hessian=TRUE,
                   control=list(), ...)

# print objects in a summary
sirt_summary_print_objects(obji, from=NULL, to=NULL, digits=3, rownames_null=TRUE,
      grep_string=NULL)
# print package version and R session
sirt_summary_print_package_rsession(pack)
# print package version
sirt_summary_print_package(pack)
# print R session
sirt_summary_print_rsession()
# print call
sirt_summary_print_call(CALL)

# print a data frame x with fixed numbers of digits after the decimal
print_digits(x, digits=NULL)

# discrete inverse function
sirt_rcpp_discrete_inverse(x0, y0, y)

# move variables in a data frame
move_variables_df(x, after_var, move_vars)}

\arguments{
\item{pars}{Numeric vector}
\item{lower}{Numeric vector}
\item{upper}{Numeric vector}
\item{x}{Numeric vector or a matrix or a list}
\item{eps}{Numerical. Shrinkage parameter of eigenvalue in \code{ginverse_sym}}
\item{a}{Numeric vector}
\item{lambda}{Numeric value}
\item{A}{Matrix}
\item{nrow}{Integer}
\item{na.rm}{Logical}
\item{mean}{Numeric}
\item{sd}{Numeric}
\item{y}{Matrix}
\item{rho}{Numeric}
\item{deriv}{Integer indicating the order of derivative}
\item{z}{Numeric}
\item{r}{Integer}
\item{v}{Vector}
\item{envir}{Environment}
\item{optimizer}{Can be one of the following optimizers: \code{optim}, \code{nlminb},
\code{bobyqa} (from the \pkg{minqa} packlage), \code{Rvmmin} (from the
\pkg{optimx} package) or \code{nloptr} (from the \pkg{nloptr} package using
the argument \code{opts$algorithm="NLOPT_LD_MMA"}).
}
\item{par}{Initial parameter}
\item{fn}{Function}
\item{grad}{Gradient function}
\item{method}{Optimization method}
\item{hessian}{Logical}
\item{control}{Control list for R optimizers}
\item{\ldots}{Further arguments to be passed}
\item{obji}{Data frame}
\item{from}{Integer}
\item{to}{Integer}
\item{digits}{Integer}
\item{rownames_null}{Logical}
\item{grep_string}{String}
\item{pack}{Package name}
\item{CALL}{Call statement}
\item{x0}{Vector}
\item{y0}{Vector}
\item{after_var}{String indicating variable name after which variable specified
variables in \code{move_vars} should be moved}
\item{move_vars}{Variables which should be moved after \code{after_var}}
}

%\details{
%xxx
%%}


\examples{
#############################################################################
## EXAMPLE 1: Trace of a matrix
#############################################################################

set.seed(86)
A <- matrix( stats::runif(4), 2,2 )
tracemat(A)
sum(diag(A))    #=sirt::tracemat(A)

#############################################################################
## EXAMPLE 2: Power function
#############################################################################

x <- 2.3
a <- 1.7
pow(x=x,a=a)
x^a            #=sirt::pow(x,a)

#############################################################################
## EXAMPLE 3: Soft and hard thresholding function (e.g. in LASSO estimation)
#############################################################################

x <- seq(-2, 2, length=100)
y <- sirt::soft_thresholding( x, lambda=.5)
graphics::plot( x, y, type="l")

z <- sirt::hard_thresholding( x, lambda=.5)
graphics::lines( x, z, lty=2, col=2)

#############################################################################
## EXAMPLE 4: Bounds on parameters
#############################################################################

pars <- c(.721, .346)
bounds_parameters( pars=pars, lower=c(-Inf, .5), upper=c(Inf,1) )

#############################################################################
## EXAMPLE 5: Smooth approximation of absolute value function
#############################################################################

x <- seq(-1,1,len=100)
graphics::plot(x, abs(x), lwd=2, col=1, lty=1, type="l", ylim=c(-1,1) )
# smooth approximation
tt <- 2
graphics::lines(x, sirt::sirt_abs_smooth(x), lty=tt, col=tt, lwd=2)
# first derivative
tt <- 3
graphics::lines(x, sirt::sirt_abs_smooth(x, deriv=1), lty=tt, col=tt, lwd=2)
# second derivative
tt <- 4
graphics::lines(x, sirt::sirt_abs_smooth(x, deriv=2), lty=tt, col=tt, lwd=2)

# analytic computation of first and second derivative
stats::deriv( ~ sqrt(x^2 + eps), namevec="x", hessian=TRUE )

\dontrun{
#############################################################################
## EXAMPLE 6: Permutations with replacement
#############################################################################

D <- 4
v <- 0:1
sirt::sirt_permutations(r=D, v=v)
gtools::permutations(n=length(v), r=D, v=v, repeats.allowed=TRUE)
}
}

%% File Name: marginal.truescore.reliability.Rd
%% File Version: 0.27

\name{marginal.truescore.reliability}
\alias{marginal.truescore.reliability}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
True-Score Reliability for Dichotomous Data
}
\description{
This function computes the marginal true-score reliability for
dichotomous data (Dimitrov, 2003; May & Nicewander, 1994) for
the four-parameter logistic item response model
(see \code{\link{rasch.mml2}} for details regarding this IRT model).
}
\usage{
marginal.truescore.reliability(b, a=1+0*b,c=0*b,d=1+0*b,
    mean.trait=0, sd.trait=1, theta.k=seq(-6,6,len=200) )
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{b}{
Vector of item difficulties
}
  \item{a}{
Vector of item discriminations
}
  \item{c}{
Vector of guessing parameters
}
  \item{d}{
Vector of upper asymptotes
}
  \item{mean.trait}{
Mean of trait distribution
}
  \item{sd.trait}{
Standard deviation of trait distribution
}
  \item{theta.k}{
Grid at which the trait distribution should be evaluated
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list with following entries:
  \item{rel.test}{Reliability of the test}
  \item{item}{True score variance (\code{sig2.true}, error variance
  (\code{sig2.error}) and item reliability (\code{rel.item}).
  Expected proportions correct are in the column \code{pi}.
    }
  \item{pi}{Average proportion correct for all items and persons}
  \item{sig2.tau}{True score variance \eqn{\sigma^2_{\tau}}
  (calculated by the formula in May & Nicewander, 1994)}
  \item{sig2.error}{Error variance \eqn{\sigma^2_{e}}}
}

\references{
Dimitrov, D. (2003). Marginal true-score measures and reliability
for binary items as a function of their IRT parameters.
\emph{Applied Psychological Measurement, 27}, 440-458.

May, K., & Nicewander, W. A. (1994). Reliability and information
functions for percentile ranks. \emph{Journal of Educational
Measurement, 31}, 313-325.
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See \code{\link{greenyang.reliability}} for calculating the reliability
for multidimensional measures.
}
\examples{
#############################################################################
# EXAMPLE 1: Dimitrov (2003) Table 1 - 2PL model
#############################################################################

# item discriminations
a <- 1.7*c(0.449,0.402,0.232,0.240,0.610,0.551,0.371,0.321,0.403,0.434,0.459,
    0.410,0.302,0.343,0.225,0.215,0.487,0.608,0.341,0.465)
# item difficulties
b <- c( -2.554,-2.161,-1.551,-1.226,-0.127,-0.855,-0.568,-0.277,-0.017,
    0.294,0.532,0.773,1.004,1.250,1.562,1.385,2.312,2.650,2.712,3.000 )

marginal.truescore.reliability( b=b, a=a )
  ##   Reliability=0.606

#############################################################################
# EXAMPLE 2: Dimitrov (2003) Table 2
#  3PL model: Poetry items (4 items)
#############################################################################

# slopes, difficulties and guessing parameters
a <- 1.7*c(1.169,0.724,0.554,0.706 )
b <- c(0.468,-1.541,-0.042,0.698 )
c <- c(0.159,0.211,0.197,0.177 )

res <- sirt::marginal.truescore.reliability( b=b, a=a, c=c)
  ##   Reliability=0.403
  ##   > round( res$item, 3 )
  ##     item    pi sig2.tau sig2.error rel.item
  ##   1    1 0.463    0.063      0.186    0.252
  ##   2    2 0.855    0.017      0.107    0.135
  ##   3    3 0.605    0.026      0.213    0.107
  ##   4    4 0.459    0.032      0.216    0.130

#############################################################################
# EXAMPLE 3: Reading Data
#############################################################################
data( data.read)

#***
# Model 1: 1PL
mod <- sirt::rasch.mml2( data.read )
marginal.truescore.reliability( b=mod$item$b )
  ##   Reliability=0.653

#***
# Model 2: 2PL
mod <- sirt::rasch.mml2( data.read, est.a=1:12 )
marginal.truescore.reliability( b=mod$item$b, a=mod$item$a)
  ##   Reliability=0.696

\dontrun{
# compare results with Cronbach's alpha and McDonald's omega
# posing a 'wrong model' for normally distributed data
library(psych)
psych::omega(dat, nfactors=1)     # 1 factor
  ##  Omega_h for 1 factor is not meaningful, just omega_t
  ##   Omega
  ##   Call: omega(m=dat, nfactors=1)
  ##   Alpha:                 0.69
  ##   G.6:                   0.7
  ##   Omega Hierarchical:    0.66
  ##   Omega H asymptotic:    0.95
  ##   Omega Total            0.69

##! Note that alpha in psych is the standardized one.
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Reliability}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: brm.sim.Rd
%% File Version: 0.533

\name{brm-Methods}
\alias{brm.sim}
\alias{brm.irf}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Functions for the Beta Item Response Model
}


\description{
Functions for simulating and estimating the Beta item response model
(Noel & Dauvier, 2007). \code{brm.sim} can be used for
simulating the model, \code{brm.irf} computes the item response
function. The Beta item response model is estimated as a discrete
version to enable estimation in \emph{standard} IRT software like
\pkg{mirt} or \pkg{TAM} packages.
}

\usage{
# simulating the beta item response model
brm.sim(theta, delta, tau, K=NULL)

# computing the item response function of the beta item response model
brm.irf( Theta, delta, tau, ncat, thdim=1, eps=1E-10 )
}


%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{theta}{
Ability vector of \eqn{\theta} values
}
  \item{delta}{
Vector of item difficulty parameters
}
  \item{tau}{
Vector item dispersion parameters
}
  \item{K}{
Number of discretized categories. The default is \code{NULL} which
means that the simulated item responses are real number values
between 0 and 1. If an integer \code{K} chosen, then values
are discretized such that values of 0, 1, ..., \eqn{K}-1 arise.
}
\item{Theta}{Matrix of the ability vector \eqn{\bold{\theta}}}
\item{ncat}{Number of categories}
\item{thdim}{Theta dimension in the matrix \code{Theta} on
which the item loads.}
\item{eps}{Nuisance parameter which stabilize probabilities.}
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%% DETAILS  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\details{
The discrete version of the beta item response model is defined as follows.
Assume that for item \eqn{i} there are \eqn{K} categories resulting in
values \eqn{k=0,1,\dots,K-1}. Each value \eqn{k} is associated with a
corresponding the transformed value in \eqn{[0,1]}, namely
\eqn{ q (k)=1/(2 \cdot K), 1/(2 \cdot K) + 1/K, \ldots,  1 - 1/(2 \cdot K) }.
The item response model is defined as
\deqn{ P( X_{pi}=x_{pi} | \theta_p)  \propto
      q( x_{pi} )^{ m_{pi} - 1 } [ 1- q( x_{pi} ) ]^{ n_{pi} - 1 } }
This density is a discrete version of a Beta distribution with
shape parameters \eqn{m_{pi}} and \eqn{n_{pi}}. These parameters are
defined as
\deqn{ m_{pi}=\mathrm{exp} \left[ ( \theta_p - \delta_i + \tau_i ) / 2 \right]
    \qquad \mbox{and} \qquad
    n_{pi}=\mathrm{exp} \left[ ( - \theta_p + \delta_i + \tau_i ) / 2 \right]
                }

The item response function can also be formulated as
\deqn{ \mathrm{log} \left[ P( X_{pi}=x_{pi} | \theta_p) \right]  \propto
      ( m_{pi} - 1 ) \cdot \mathrm{log} [ q( x_{pi} ) ] +
      ( n_{pi} - 1 )  \cdot \mathrm{log} [ 1- q( x_{pi} ) ]
                      }

The item parameters can be reparameterized as
\eqn{ a_{i}=\mathrm{exp} \left[ ( - \delta_i + \tau_i ) / 2 \right]} and
\eqn{ b_{i}=\mathrm{exp} \left[ ( \delta_i + \tau_i ) / 2 \right]}.

Then, the original item parameters can be retrieved by
\eqn{\tau_i=\mathrm{log} ( a_i b_i)} and
\eqn{\delta_i=\mathrm{log} ( b_i / a_i)}.
Using \eqn{ \gamma _p=\mathrm{exp} ( \theta_p / 2) }, we obtain

\deqn{ \mathrm{log} \left[ P( X_{pi}=x_{pi} | \theta_p) \right]  \propto
       a_{i} \gamma_p  \cdot \mathrm{log} [ q( x_{pi} ) ] +
       b_i / \gamma_p   \cdot \mathrm{log} [ 1- q( x_{pi} ) ] -
      \left[ \mathrm{log} q( x_{pi} ) + \mathrm{log} [ 1- q( x_{pi} ) ] \right]
                      }

This formulation enables the specification of the Beta item response
model as a structured latent class model
(see \code{\link[TAM:tam.mml.3pl]{TAM::tam.mml.3pl}};
Example 1).

See Smithson and Verkuilen (2006) for motivations for treating
continuous indicators not as normally distributed variables.
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%% VALUE  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\value{
A simulated dataset of item responses if \code{brm.sim} is applied.

A matrix of item response probabilities if \code{brm.irf} is applied.
}


\references{
Gruen, B., Kosmidis, I., & Zeileis, A. (2012). Extended Beta regression
in \R: Shaken, stirred, mixed, and partitioned.
\emph{Journal of Statistical Software, 48}(11), 1-25.
\doi{10.18637/jss.v048.i11}

Noel, Y., & Dauvier, B. (2007). A beta item response model for continuous
bounded responses. \emph{Applied Psychological Measurement,
31}(1), 47-73. \doi{10.1177/0146621605287691}

Smithson, M., & Verkuilen, J. (2006). A better lemon squeezer?
Maximum-likelihood regression with beta-distributed dependent variables.
\emph{Psychological Methods, 11}(1), 54-71.
doi: 10.1037/1082-989X.11.1.54
}

%\author{
%Alexander Robitzsch
%}

%\note{
%
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~


\seealso{
See also the \pkg{betareg} package for fitting Beta regression
regression models in \R (Gruen, Kosmidis & Zeileis, 2012).
}

\examples{
#############################################################################
# EXAMPLE 1: Simulated data beta response model
#############################################################################

#*** (1) Simulation of the beta response model
# Table 3 (p. 65) of Noel and Dauvier (2007)
delta <- c( -.942, -.649, -.603, -.398, -.379, .523, .649, .781, .907 )
tau <- c( .382, .166, 1.799, .615, 2.092, 1.988, 1.899, 1.439, 1.057 )
K <- 5        # number of categories for discretization
N <- 500        # number of persons
I <- length(delta) # number of items

set.seed(865)
theta <- stats::rnorm( N )
dat <- sirt::brm.sim( theta=theta, delta=delta, tau=tau, K=K)
psych::describe(dat)

#*** (2) some preliminaries for estimation of the model in mirt
#*** define a mirt function
library(mirt)
Theta <- matrix( seq( -4, 4, len=21), ncol=1 )

# compute item response function
ii <- 1     # item ii=1
b1 <- sirt::brm.irf( Theta=Theta, delta=delta[ii], tau=tau[ii],  ncat=K )
# plot item response functions
graphics::matplot( Theta[,1], b1, type="l" )

#*** defining the beta item response function for estimation in mirt
par <- c( 0, 1,  1)
names(par) <- c( "delta", "tau","thdim")
est <- c( TRUE, TRUE, FALSE )
names(est) <- names(par)
brm.icc <- function( par, Theta, ncat ){
     delta <- par[1]
     tau <- par[2]
     thdim <- par[3]
     probs <- sirt::brm.irf( Theta=Theta, delta=delta, tau=tau,  ncat=ncat,
            thdim=thdim)
     return(probs)
            }
name <- "brm"
# create item response function
brm.itemfct <- mirt::createItem(name, par=par, est=est, P=brm.icc)
#*** define model in mirt
mirtmodel <- mirt::mirt.model("
           F1=1-9
            " )
itemtype <- rep("brm", I )
customItems <- list("brm"=brm.itemfct)

# define parameters to be estimated
mod1.pars <- mirt::mirt(dat, mirtmodel, itemtype=itemtype,
                   customItems=customItems, pars="values")

\dontrun{
#*** (3) estimate beta item response model in mirt
mod1 <- mirt::mirt(dat,mirtmodel, itemtype=itemtype, customItems=customItems,
               pars=mod1.pars, verbose=TRUE  )
# model summaries
print(mod1)
summary(mod1)
coef(mod1)
# estimated coefficients and comparison with simulated data
cbind( sirt::mirt.wrapper.coef( mod1 )$coef, delta, tau )
mirt.wrapper.itemplot(mod1,ask=TRUE)

#---------------------------
# estimate beta item response model in TAM
library(TAM)

# define the skill space: standard normal distribution
TP <- 21                   # number of theta points
theta.k <- diag(TP)
theta.vec <-  seq( -6,6, len=TP)
d1 <- stats::dnorm(theta.vec)
d1 <- d1 / sum(d1)
delta.designmatrix <- matrix( log(d1), ncol=1 )
delta.fixed <- cbind( 1, 1, 1 )

# define design matrix E
E <- array(0, dim=c(I,K,TP,2*I + 1) )
dimnames(E)[[1]] <- items <- colnames(dat)
dimnames(E)[[4]] <- c( paste0( rep( items, each=2 ),
        rep( c("_a","_b" ), I) ), "one" )
for (ii in 1:I){
    for (kk in 1:K){
      for (tt in 1:TP){
        qk <- (2*(kk-1)+1)/(2*K)
        gammap <- exp( theta.vec[tt] / 2 )
        E[ii, kk, tt, 2*(ii-1) + 1 ] <- gammap * log( qk )
        E[ii, kk, tt, 2*(ii-1) + 2 ] <- 1 / gammap * log( 1 - qk )
        E[ii, kk, tt, 2*I+1 ] <- - log(qk) - log( 1 - qk )
                    }
            }
        }
gammaslope.fixed <- cbind( 2*I+1, 1 )
gammaslope <- exp( rep(0,2*I+1) )

# estimate model in TAM
mod2 <- TAM::tam.mml.3pl(resp=dat, E=E,control=list(maxiter=100),
              skillspace="discrete", delta.designmatrix=delta.designmatrix,
              delta.fixed=delta.fixed, theta.k=theta.k, gammaslope=gammaslope,
              gammaslope.fixed=gammaslope.fixed, notA=TRUE )
summary(mod2)

# extract original tau and delta parameters
m1 <- matrix( mod2$gammaslope[1:(2*I) ], ncol=2, byrow=TRUE )
m1 <- as.data.frame(m1)
colnames(m1) <- c("a","b")
m1$delta.TAM <- log( m1$b / m1$a)
m1$tau.TAM <- log( m1$a * m1$b )

# compare estimated parameter
m2 <- cbind( sirt::mirt.wrapper.coef( mod1 )$coef, delta, tau )[,-1]
colnames(m2) <- c(  "delta.mirt", "tau.mirt", "thdim","delta.true","tau.true"   )
m2 <- cbind(m1,m2)
round( m2, 3 )
}
}


% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Beta item response model}
% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: f1d.irt.Rd
%% File Version: 1.35

\name{f1d.irt}
\alias{f1d.irt}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Functional Unidimensional Item Response Model
}

\description{
Estimates the functional unidimensional item response model for
dichotomous data (Ip, Molenberghs, Chen, Goegebeur & De Boeck, 2013).
Either the IRT model is estimated using a probit link and employing tetrachoric
correlations or item discriminations and intercepts of a pre-estimated multidimensional IRT
model are provided as input.
}


\usage{
f1d.irt(dat=NULL, nnormal=1000, nfactors=3, A=NULL, intercept=NULL,
    mu=NULL, Sigma=NULL, maxiter=100, conv=10^(-5), progress=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame with dichotomous item responses
}
  \item{nnormal}{
Number of \eqn{\theta_p} grid points for approximating the
normal distribution
}
  \item{nfactors}{
Number of dimensions to be estimated
}
  \item{A}{
Matrix of item discriminations (if the IRT model is already estimated)
}
  \item{intercept}{
Vector of item intercepts (if the IRT model is already estimated)
}
\item{mu}{Vector of estimated means. In the default it is assumed
     that all means are zero.}
\item{Sigma}{Estimated covariance matrix. In the default it is
the identity matrix.}
  \item{maxiter}{
Maximum number of iterations
}
  \item{conv}{
Convergence criterion
}
  \item{progress}{
Display progress? The default is \code{TRUE}.
}
}
\details{
The functional unidimensional item response model (F1D model)
for dichotomous item responses is based on a multidimensional model with a
link function \eqn{g} (probit or logit):
\deqn{ P( X_{pi}=1 | \bold{\theta}_p )=
g( \sum_d a_{id} \theta_{pd} - d_i ) }
It is assumed that \eqn{\bold{\theta}_p} is multivariate normally
distribution with a zero mean vector and identity covariance matrix.

The F1D model estimates unidimensional item response functions
such that
\deqn{ P( X_{pi}=1 | \theta_p^\ast ) \approx
g \left( a_{i}^\ast \theta_{p}^\ast - d_i^\ast \right) }
The optimization function \eqn{F} minimizes the deviations of
the approximation equations
\deqn{ a_{i}^\ast \theta_{p}^\ast - d_i^\ast
\approx \sum_d a_{id} \theta_{pd} - d_i }
The optimization function \eqn{F} is defined by
\deqn{ F( \{ a_i^\ast, d_i^\ast \}_i, \{ \theta_p^\ast \}_p )=
    \sum_p \sum_i w_p ( a_{id} \theta_{pd} - d_i-
a_{i}^\ast \theta_{p}^\ast + d_i^\ast )^2 \rightarrow Min! }
All items \eqn{i} are equally weighted whereas the ability
distribution of persons \eqn{p} are weighted according to the
multivariate normal distribution (using weights \eqn{w_p}).
The estimation is conducted using an alternating least squares algorithm
(see Ip et al. 2013 for a different algorithm). The ability distribution
\eqn{\theta_p^\ast} of the functional unidimensional model is assumed
to be standardized, i.e. does have a zero mean and a standard deviation of one.
}

\value{
A list with following entries:
\item{item}{Data frame with estimated item parameters: Item intercepts
    for the functional unidimensional \eqn{a_{i}^\ast} (\code{ai.ast}) and
    the ('ordinary') unidimensional (\code{ai0}) item response
    model. The same holds for item intercepts \eqn{d_{i}^\ast} (\code{di.ast} and
    \code{di0} respectively).
}
\item{person}{Data frame with estimated \eqn{\theta_p^\ast}
  distribution. Locations are \code{theta.ast} with corresponding
  probabilities in \code{wgt}.}
\item{A}{Estimated or provided item discriminations}
\item{intercept}{Estimated or provided intercepts}
\item{dat}{Used dataset}
\item{tetra}{Object generated by \code{\link{tetrachoric2}} if \code{dat}
    is specified as input. This list entry is useful for applying
    \code{\link{greenyang.reliability}}.
}
}


\references{
Ip, E. H., Molenberghs, G., Chen, S. H., Goegebeur, Y., &
De Boeck, P. (2013). Functionally unidimensional item
response models for multivariate binary data.
\emph{Multivariate Behavioral Research, 48}, 534-562.
}


%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For estimation of bifactor models and Green-Yang reliability
based on tetrachoric correlations see \code{\link{greenyang.reliability}}.

For estimation of bifactor models based on marginal maximum likelihood
(i.e. full information maximum likelihood) see the
\code{\link[TAM:tam.fa]{TAM::tam.fa}} function in the \pkg{TAM} package.
}


\examples{
#############################################################################
# EXAMPLE 1: Dataset Mathematics data.math | Exploratory multidimensional model
#############################################################################
data(data.math)
dat <- ( data.math$data )[, -c(1,2) ] # select Mathematics items

#****
# Model 1: Functional unidimensional model based on original data

#++ (1) estimate model with 3 factors
mod1 <- sirt::f1d.irt( dat=dat, nfactors=3)

#++ (2) plot results
     par(mfrow=c(1,2))
# Intercepts
plot( mod1$item$di0, mod1$item$di.ast, pch=16, main="Item Intercepts",
        xlab=expression( paste( d[i], " (Unidimensional Model)" )),
        ylab=expression( paste( d[i], " (Functional Unidimensional Model)" )))
abline( lm(mod1$item$di.ast ~ mod1$item$di0), col=2, lty=2 )
# Discriminations
plot( mod1$item$ai0, mod1$item$ai.ast, pch=16, main="Item Discriminations",
        xlab=expression( paste( a[i], " (Unidimensional Model)" )),
        ylab=expression( paste( a[i], " (Functional Unidimensional Model)" )))
abline( lm(mod1$item$ai.ast ~ mod1$item$ai0), col=2, lty=2 )
     par(mfrow=c(1,1))

#++ (3) estimate bifactor model and Green-Yang reliability
gy1 <- sirt::greenyang.reliability( mod1$tetra, nfactors=3 )

\dontrun{
#****
# Model 2: Functional unidimensional model based on estimated multidimensional
#          item response model

#++ (1) estimate 2-dimensional exploratory factor analysis with 'smirt'
I <- ncol(dat)
Q <- matrix( 1, I,2 )
Q[1,2] <- 0
variance.fixed <- cbind( 1,2,0 )
mod2a <- sirt::smirt( dat, Qmatrix=Q, irtmodel="comp", est.a="2PL",
                variance.fixed=variance.fixed, maxiter=50)
#++ (2) input estimated discriminations and intercepts for
#       functional unidimensional model
mod2b <- sirt::f1d.irt( A=mod2a$a, intercept=mod2a$b )

#############################################################################
# EXAMPLE 2: Dataset Mathematics data.math | Confirmatory multidimensional model
#############################################################################

data(data.math)
library(TAM)

# dataset
dat <- data.math$data
dat <- dat[, grep("M", colnames(dat) ) ]
# extract item informations
iteminfo <- data.math$item
I <- ncol(dat)
# define Q-matrix
Q <- matrix( 0, nrow=I, ncol=3 )
Q[ grep( "arith", iteminfo$domain ), 1 ] <- 1
Q[ grep( "Meas", iteminfo$domain ), 2 ] <- 1
Q[ grep( "geom", iteminfo$domain ), 3 ] <- 1

# fit three-dimensional model in TAM
mod1 <- TAM::tam.mml.2pl(  dat, Q=Q, control=list(maxiter=40, snodes=1000) )
summary(mod1)

# specify functional unidimensional model
intercept <- mod1$xsi[, c("xsi") ]
names(intercept) <- rownames(mod1$xsi)
fumod1 <- sirt::f1d.irt( A=mod1$B[,2,], intercept=intercept, Sigma=mod1$variance)
fumod1$item
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Functional unidimensional item response model}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: lavaan2mirt.Rd
%% File Version: 0.441

\name{lavaan2mirt}
\alias{lavaan2mirt}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Converting a \code{lavaan} Model into a \code{mirt} Model
}

\description{
Converts a \code{lavaan} model into a \code{mirt} model.
Optionally, the model can be estimated with the
\code{\link[mirt:mirt]{mirt::mirt}} function
(\code{est.mirt=TRUE}) or just \code{mirt} syntax
is generated (\code{est.mirt=FALSE}).

Extensions of the \code{lavaan} syntax include guessing and slipping
parameters (operators \code{?=g1} and \code{?=s1})
and a shortage operator for item groups (see \code{__}).
See \code{\link[TAM:lavaanify.IRT]{TAM::lavaanify.IRT}} for more details.
}

\usage{
lavaan2mirt(dat, lavmodel, est.mirt=TRUE, poly.itemtype="gpcm", ...)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Dataset with item responses
}
  \item{lavmodel}{
Model specified in \code{lavaan} syntax
(see \code{\link[lavaan:lavaanify]{lavaan::lavaanify}})
}
  \item{est.mirt}{
An optional logical indicating whether the model
should be estimated with \code{\link[mirt:mirt]{mirt::mirt}}
}
\item{poly.itemtype}{Item type for polytomous data. This can
    be \code{gpcm} for the generalized partial credit model
      or \code{graded} for the graded response model.}
  \item{\dots}{
Further arguments to be passed for estimation in
\code{\link[mirt:mirt]{mirt}}
}
}

\details{
This function uses the \code{\link[lavaan:lavaanify]{lavaan::lavaanify}}
(\pkg{lavaan}) function.

Only single group models are supported (for now).
}

\value{
A list with following entries

\item{mirt}{Object generated by \code{mirt} function if
\code{est.mirt=TRUE}}
\item{mirt.model}{Generated \code{mirt} model}
\item{mirt.syntax}{Generated \code{mirt} syntax}
\item{mirt.pars}{Generated parameter specifications
   in \code{mirt}}
\item{lavaan.model}{Used \code{lavaan} model transformed by
\code{lavaanify} function}
\item{dat}{Used dataset. If necessary, only items used in the
model are included in the dataset.}
}

%\references{
%% ~put references to the literature/web site here ~
%}


%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See \url{https://lavaan.ugent.be/} for \pkg{lavaan} resources.

See \url{https://groups.google.com/forum/#!forum/lavaan}
for discussion about the \pkg{lavaan} package. \cr

See \code{\link{mirt.wrapper}} for convenience wrapper functions
for \code{\link[mirt:mirt]{mirt::mirt}} objects.

See \code{\link[TAM:lavaanify.IRT]{TAM::lavaanify.IRT}}
for extensions of \code{lavaanify}.

See \code{\link{tam2mirt}} for converting fitted objects in the \pkg{TAM}
package into fitted \code{\link[mirt:mirt]{mirt::mirt}} objects.
}

\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Convert some lavaan syntax to mirt syntax for data.read
#############################################################################

library(mirt)
data(data.read)
dat <- data.read

#******************
#*** Model 1: Single factor model
lavmodel <- "
     # omit item C3
     F=~ A1+A2+A3+A4 + C1+C2+C4 + B1+B2+B3+B4
     F ~~ 1*F
            "

# convert syntax and estimate model
res <- sirt::lavaan2mirt( dat,  lavmodel, verbose=TRUE, technical=list(NCYCLES=3) )
# inspect coefficients
coef(res$mirt)
mirt.wrapper.coef(res$mirt)
# converted mirt model and parameter table
cat(res$mirt.syntax)
res$mirt.pars

#******************
#*** Model 2: Rasch Model with first six items
lavmodel <- "
     F=~ a*A1+a*A2+a*A3+a*A4+a*B1+a*B2
     F ~~ 1*F
            "
# convert syntax and estimate model
res <- sirt::lavaan2mirt( dat,  lavmodel, est.mirt=FALSE)
# converted mirt model
cat(res$mirt.syntax)
# mirt parameter table
res$mirt.pars
# estimate model using generated objects
res2 <- mirt::mirt( res$dat, res$mirt.model, pars=res$mirt.pars )
mirt.wrapper.coef(res2)     # parameter estimates

#******************
#*** Model 3: Bifactor model
lavmodel <- "
     G=~ A1+A2+A3+A4 + B1+B2+B3+B4  + C1+C2+C3+C4
     A=~ A1+A2+A3+A4
     B=~ B1+B2+B3+B4
     C=~ C1+C2+C3+C4
     G ~~ 1*G
     A ~~ 1*A
     B ~~ 1*B
     C ~~ 1*C
            "
res <- sirt::lavaan2mirt( dat,  lavmodel, est.mirt=FALSE )
# mirt syntax and mirt model
cat(res$mirt.syntax)
res$mirt.model
res$mirt.pars

#******************
#*** Model 4: 3-dimensional model with some parameter constraints
lavmodel <- "
     # some equality constraints among loadings
     A=~ a*A1+a*A2+a2*A3+a2*A4
     B=~ B1+B2+b3*B3+B4
     C=~ c*C1+c*C2+c*C3+c*C4
     # some equality constraints among thresholds
     A1 | da*t1
     A3 | da*t1
     B3 | da*t1
     C3 | dg*t1
     C4 | dg*t1
     # standardized latent variables
     A ~~ 1*A
     B ~~ 1*B
     C ~~ 1*C
     # estimate Cov(A,B) and Cov(A,C)
     A ~~ B
     A ~~ C
     # estimate mean of B
     B ~ 1
            "
res <- sirt::lavaan2mirt( dat,  lavmodel, verbose=TRUE, technical=list(NCYCLES=3) )
# estimated parameters
mirt.wrapper.coef(res$mirt)
# generated mirt syntax
cat(res$mirt.syntax)
# mirt parameter table
mirt::mod2values(res$mirt)

#******************
#*** Model 5: 3-dimensional model with some parameter constraints and
#             parameter fixings
lavmodel <- "
     A=~ a*A1+a*A2+1.3*A3+A4  # set loading of A3 to 1.3
     B=~ B1+1*B2+b3*B3+B4
     C=~ c*C1+C2+c*C3+C4
     A1 | da*t1
     A3 | da*t1
     C4 | dg*t1
     B1 | 0*t1
     B3 | -1.4*t1   # fix item threshold of B3 to -1.4
     A ~~ 1*A
     B ~~ B         # estimate variance of B freely
     C ~~ 1*C
     A ~~ B         # estimate covariance between A and B
     A ~~ .6 * C    # fix covariance to .6
     A ~ .5*1       # set mean of A to .5
     B ~ 1          # estimate mean of B
            "
res <- sirt::lavaan2mirt( dat,  lavmodel, verbose=TRUE, technical=list(NCYCLES=3) )
mirt.wrapper.coef(res$mirt)

#******************
#*** Model 6: 1-dimensional model with guessing and slipping parameters
#******************

lavmodel <- "
     F=~ c*A1+c*A2+1*A3+1.3*A4 + C1__C4 + a*B1+b*B2+b*B3+B4
     # guessing parameters
     A1+A2 ?=guess1*g1
     A3 ?=.25*g1
     B1+C1 ?=g1
     B2__B4 ?=0.10*g1
     # slipping parameters
     A1+A2+C3 ?=slip1*s1
     A3 ?=.02*s1
     # fix item intercepts
     A1 | 0*t1
     A2 | -.4*t1
     F ~ 1    # estimate mean of F
     F ~~ 1*F   # fix variance of F
            "
# convert syntax and estimate model
res <- sirt::lavaan2mirt( dat,  lavmodel, verbose=TRUE, technical=list(NCYCLES=3) )
# coefficients
mirt.wrapper.coef(res$mirt)
# converted mirt model
cat(res$mirt.syntax)

#############################################################################
# EXAMPLE 2: Convert some lavaan syntax to mirt syntax for
#            longitudinal data data.long
#############################################################################

data(data.long)
dat <- data.long[,-1]

#******************
#*** Model 1: Rasch model for T1
lavmodel <- "
     F=~ 1*I1T1 +1*I2T1+1*I3T1+1*I4T1+1*I5T1+1*I6T1
     F ~~ F
            "
# convert syntax and estimate model
res <- sirt::lavaan2mirt( dat,  lavmodel, verbose=TRUE, technical=list(NCYCLES=20) )
# inspect coefficients
mirt.wrapper.coef(res$mirt)
# converted mirt model
cat(res$mirt.syntax)

#******************
#*** Model 2: Rasch model for two time points
lavmodel <- "
     F1=~ 1*I1T1 +1*I2T1+1*I3T1+1*I4T1+1*I5T1+1*I6T1
     F2=~ 1*I3T2 +1*I4T2+1*I5T2+1*I6T2+1*I7T2+1*I8T2
     F1 ~~ F1
     F1 ~~ F2
     F2 ~~ F2
     # equal item difficulties of same items
     I3T1 | i3*t1
     I3T2 | i3*t1
     I4T1 | i4*t1
     I4T2 | i4*t1
     I5T1 | i5*t1
     I5T2 | i5*t1
     I6T1 | i6*t1
     I6T2 | i6*t1
     # estimate mean of F1, but fix mean of F2
     F1 ~ 1
     F2 ~ 0*1
            "
# convert syntax and estimate model
res <- sirt::lavaan2mirt( dat,  lavmodel, verbose=TRUE, technical=list(NCYCLES=20) )
# inspect coefficients
mirt.wrapper.coef(res$mirt)
# converted mirt model
cat(res$mirt.syntax)

#-- compare estimation with smirt function
# define Q-matrix
I <- ncol(dat)
Q <- matrix(0,I,2)
Q[1:6,1] <- 1
Q[7:12,2] <- 1
rownames(Q) <- colnames(dat)
colnames(Q) <- c("T1","T2")
# vector with same items
itemnr <- as.numeric( substring( colnames(dat),2,2) )
# fix mean at T2 to zero
mu.fixed <- cbind( 2,0 )
# estimate model in smirt
mod1 <- sirt::smirt(dat, Qmatrix=Q, irtmodel="comp", est.b=itemnr, mu.fixed=mu.fixed )
summary(mod1)

#############################################################################
# EXAMPLE 3: Converting lavaan syntax for polytomous data
#############################################################################

data(data.big5)
# select some items
items <- c( grep( "O", colnames(data.big5), value=TRUE )[1:6],
            grep( "N", colnames(data.big5), value=TRUE )[1:4] )
#  O3  O8  O13 O18 O23 O28 N1  N6  N11 N16
dat <- data.big5[, items ]
library(psych)
psych::describe(dat)

#******************
#*** Model 1: Partial credit model
lavmodel <- "
      O=~ 1*O3+1*O8+1*O13+1*O18+1*O23+1*O28
      O ~~ O
         "
# estimate model in mirt
res <- sirt::lavaan2mirt( dat, lavmodel, technical=list(NCYCLES=20), verbose=TRUE)
# estimated mirt model
mres <- res$mirt
# mirt syntax
cat(res$mirt.syntax)
  ##   O=1,2,3,4,5,6
  ##   COV=O*O
# estimated parameters
mirt.wrapper.coef(mres)
# some plots
mirt::itemplot( mres, 3 )   # third item
plot(mres)   # item information
plot(mres,type="trace")  # item category functions

# graded response model with equal slopes
res1 <- sirt::lavaan2mirt( dat, lavmodel, poly.itemtype="graded", technical=list(NCYCLES=20),
              verbose=TRUE )
mirt.wrapper.coef(res1$mirt)

#******************
#*** Model 2: Generalized partial credit model with some constraints
lavmodel <- "
      O=~ O3+O8+O13+a*O18+a*O23+1.2*O28
      O ~ 1   # estimate mean
      O ~~ O  # estimate variance
      # some constraints among thresholds
      O3  | d1*t1
      O13 | d1*t1
      O3  | d2*t2
      O8  | d3*t2
      O28 | (-0.5)*t1
         "
# estimate model in mirt
res <- sirt::lavaan2mirt( dat, lavmodel, technical=list(NCYCLES=5), verbose=TRUE)
# estimated mirt model
mres <- res$mirt
# estimated parameters
mirt.wrapper.coef(mres)

#*** generate syntax for mirt for this model and estimate it in mirt package
# Items: O3  O8  O13 O18 O23 O28
mirtmodel <- mirt::mirt.model( "
             O=1-6
             # a(O18)=a(O23), t1(O3)=t1(O18), t2(O3)=t2(O8)
             CONSTRAIN=(4,5,a1), (1,3,d1), (1,2,d2)
             MEAN=O
             COV=O*O
               ")
# initial table of parameters in mirt
mirt.pars <- mirt::mirt( dat[,1:6], mirtmodel, itemtype="gpcm", pars="values")
# fix slope of item O28 to 1.2
ind <- which( ( mirt.pars$item=="O28" ) & ( mirt.pars$name=="a1") )
mirt.pars[ ind, "est"] <- FALSE
mirt.pars[ ind, "value"] <- 1.2
# fix d1 of item O28 to -0.5
ind <- which( ( mirt.pars$item=="O28" ) & ( mirt.pars$name=="d1") )
mirt.pars[ ind, "est"] <- FALSE
mirt.pars[ ind, "value"] <- -0.5
# estimate model
res2 <- mirt::mirt( dat[,1:6], mirtmodel, pars=mirt.pars,
             verbose=TRUE, technical=list(NCYCLES=4) )
mirt.wrapper.coef(res2)
plot(res2, type="trace")
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{mirt}
%% \keyword{lavaan}
% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: mcmclist2coda.Rd
%% File Version: 0.17

\name{mcmclist2coda}
\alias{mcmclist2coda}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Write Coda File from an Object of Class \code{mcmc.list}
}
\description{
This function  writes a coda file from an object of class \code{mcmc.list}.
Note that only first entry (i.e. one chain) will be processed.
}
\usage{
mcmclist2coda(mcmclist, name, coda.digits=5)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{mcmclist}{
An object of class \code{mcmc.list}.
}
  \item{name}{
Name of the coda file to be written
}
  \item{coda.digits}{
Number of digits after decimal in the coda file
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
The coda file and a corresponding index file are
written into the working directory.
}
%\references{
%% ~put references to the literature/web site here ~
%}

%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: MCMC estimation 2PNO dataset Reading
#############################################################################

data(data.read)
# estimate 2PNO with MCMC with 3000 iterations and 500 burn-in iterations
mod <- sirt::mcmc.2pno( dat=data.read, iter=3000, burnin=500 )
# plot MCMC chains
plot( mod$mcmcobj, ask=TRUE )
# write sampled chains into codafile
mcmclist2coda( mod$mcmcobj, name="dataread_2pl" )
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{coda}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: sim.qm.ramsay.Rd
%% File Version: 0.39

\name{sim.qm.ramsay}
\alias{sim.qm.ramsay}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Simulate from Ramsay's Quotient Model
}
\description{
This function simulates dichotomous item response data
according to Ramsay's quotient model (Ramsay, 1989).
}
\usage{
sim.qm.ramsay(theta, b, K)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{theta}{
Vector of of length \eqn{N} person parameters (must be positive!)
}
  \item{b}{
Vector of length \eqn{I} of item difficulties (must be positive)
}
  \item{K}{
Vector of length \eqn{I} of guessing parameters (must be positive)
}
}
\details{
Ramsay's quotient model (Ramsay, 1989) is defined by the equation
\deqn{P(X_{pi}=1 | \theta_p )=\frac{ \exp { ( \theta_p / b_i ) } }
    { K_i + \exp { ( \theta_p / b_i ) } }}
}
\value{
An \eqn{N \times I}  data frame with dichotomous item responses.
}

\references{
Ramsay, J. O. (1989). A comparison of three simple test theory models.
\emph{Psychometrika, 54}, 487-499.

van der Maas, H. J. L., Molenaar, D., Maris, G., Kievit, R. A., &
Borsboom, D. (2011).
Cognitive psychology meets psychometric theory: On the relation between
process models for decision making and latent variable models for
individual differences.
\emph{Psychological Review, 318}, 339-356.
}


%\author{
%Alexander Robitzsch
%}




%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See \code{\link{rasch.mml2}} for estimating Ramsay's quotient model.

See \code{\link{sim.raschtype}} for simulating response data from
the generalized logistic item response model.
}

%% ~~objects to See Also as \code{\link{help}}, ~~~

\examples{
#############################################################################
# EXAMPLE 1: Estimate Ramsay Quotient Model with rasch.mml2
#############################################################################

set.seed(657)
# simulate data according to the Ramsay model
N <- 1000       # persons
I <- 11         # items
theta <- exp( stats::rnorm( N ) )  # person ability
b <- exp( seq(-2,2,len=I))  # item difficulty
K <- rep( 3, I )           # K parameter (=> guessing)

# apply simulation function
dat <- sirt::sim.qm.ramsay( theta, b, K )

#***
# analysis
mmliter <- 50       # maximum number of iterations
I <- ncol(dat)
fixed.K <- rep( 3, I )

# Ramsay QM with fixed K parameter (K=3 in fixed.K specification)
mod1 <- sirt::rasch.mml2( dat, mmliter=mmliter, irtmodel="ramsay.qm",
              fixed.K=fixed.K )
summary(mod1)

# Ramsay QM with joint estimated K parameters
mod2 <- sirt::rasch.mml2( dat, mmliter=mmliter, irtmodel="ramsay.qm",
             est.K=rep(1,I)  )
summary(mod2)

\dontrun{
# Ramsay QM with itemwise estimated K parameters
mod3 <- sirt::rasch.mml2( dat, mmliter=mmliter, irtmodel="ramsay.qm",
              est.K=1:I  )
summary(mod3)

# Rasch model
mod4 <- sirt::rasch.mml2( dat )
summary(mod4)

# generalized logistic model
mod5 <- sirt::rasch.mml2( dat, est.alpha=TRUE, mmliter=mmliter)
summary(mod5)

# 2PL model
mod6 <- sirt::rasch.mml2( dat, est.a=rep(1,I) )
summary(mod6)

# Difficulty + Guessing (b+c) Model
mod7 <- sirt::rasch.mml2( dat, est.c=rep(1,I) )
summary(mod7)

# estimate separate guessing (c) parameters
mod8 <- sirt::rasch.mml2( dat, est.c=1:I  )
summary(mod8)

#*** estimate Model 1 with user defined function in mirt package

# create user defined function for Ramsay's quotient model
name <- 'ramsayqm'
par <- c("K"=3, "b"=1 )
est <- c(TRUE, TRUE)
P.ramsay <- function(par,Theta){
     eps <- .01
     K <- par[1]
     b <- par[2]
     num <- exp( exp( Theta[,1] ) / b )
     denom <- K + num
     P1 <- num / denom
     P1 <- eps + ( 1 - 2*eps ) * P1
     cbind(1-P1, P1)
}

# create item response function
ramsayqm <- mirt::createItem(name, par=par, est=est, P=P.ramsay)
# define parameters to be estimated
mod1m.pars <- mirt::mirt(dat, 1, rep( "ramsayqm",I),
                   customItems=list("ramsayqm"=ramsayqm), pars="values")
mod1m.pars[ mod1m.pars$name=="K", "est" ] <- FALSE
# define Theta design matrix
Theta <- matrix( seq(-3,3,len=10), ncol=1)
# estimate model
mod1m <- mirt::mirt(dat, 1, rep( "ramsayqm",I), customItems=list("ramsayqm"=ramsayqm),
               pars=mod1m.pars, verbose=TRUE,
               technical=list( customTheta=Theta, NCYCLES=50)
                )
print(mod1m)
summary(mod1m)
cmod1m <- sirt::mirt.wrapper.coef( mod1m )$coef
# compare simulated and estimated values
dfr <- cbind( b, cmod1m$b, exp(mod1$item$b ) )
colnames(dfr) <- c("simulated", "mirt", "sirt_rasch.mml2")
round( dfr, 2 )
  ##      simulated mirt sirt_rasch.mml2
  ## [1,]      0.14 0.11            0.11
  ## [2,]      0.20 0.17            0.18
  ## [3,]      0.30 0.27            0.29
  ## [4,]      0.45 0.42            0.43
  ## [5,]      0.67 0.65            0.67
  ## [6,]      1.00 1.00            1.01
  ## [7,]      1.49 1.53            1.54
  ## [8,]      2.23 2.21            2.21
  ## [9,]      3.32 3.00            2.98
  ##[10,]      4.95 5.22            5.09
  ##[11,]      7.39 5.62            5.51
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Simulating IRT models}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: noharm.sirt.Rd
%% File Version: 0.543

\name{noharm.sirt}
\alias{noharm.sirt}
\alias{summary.noharm.sirt}
% \alias{plot.noharm.sirt}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
NOHARM Model in \R
}

\description{
The function is an \R implementation of the normal ogive harmonic analysis
robust method (the NOHARM model; McDonald, 1997). Exploratory and confirmatory
multidimensional item response models for dichotomous data using the
probit link function can be estimated. Lower asymptotes (guessing parameters)
and upper asymptotes (one minus slipping parameters) can be provided as
fixed values.
}

\usage{
noharm.sirt(dat, pm=NULL, N=NULL, weights=NULL, Fval=NULL, Fpatt=NULL, Pval=NULL,
   Ppatt=NULL, Psival=NULL, Psipatt=NULL, dimensions=NULL, lower=0, upper=1, wgtm=NULL,
   pos.loading=FALSE, pos.variance=FALSE, pos.residcorr=FALSE, maxiter=1000, conv=1e-6,
   optimizer="nlminb", par_lower=NULL, reliability=FALSE, \dots)

\method{summary}{noharm.sirt}(object, file=NULL, \dots)
}

%% \method{plot}{noharm.sirt}(x, what="est", efa.load.min=.3, ...)

%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Matrix of dichotomous item responses. This matrix may contain missing data (indicated
by \code{NA}) but missingness is assumed to be missing completely at random (MCAR).
Alternatively, a product-moment matrix \code{pm} can be used as input.
}
\item{pm}{Optional product-moment matrix}
\item{N}{Sample size if \code{pm} is provided}
  \item{weights}{
Optional vector of student weights.
}
  \item{Fval}{
Initial or fixed values of the loading matrix \eqn{\bold{F}}.
}
  \item{Fpatt}{
Pattern matrix of the loading matrix \eqn{\bold{F}}. If elements should be
estimated, then an entry of 1 must be included in the pattern matrix. Parameters
which should be estimated with equality constraints must be indicated by same
integers but values largers than 1.
}
  \item{Pval}{
Initial or fixed values for the covariance matrix \eqn{\bold{P}}.
}
  \item{Ppatt}{
Pattern matrix for the covariance matrix \eqn{\bold{P}}.
}
  \item{Psival}{
Initial or fixed values for the matrix of residual correlations \eqn{\bold{\Psi}}.
}
  \item{Psipatt}{
Pattern matrix for the matrix of residual correlations \eqn{\bold{\Psi}}.
}
  \item{dimensions}{
Number of dimensions if an exploratory factor analysis should be estimated.
}
  \item{lower}{
Fixed vector (or numeric) of lower asymptotes \eqn{c_i}.
}
  \item{upper}{
Fixed vector (or numeric) of upper asymptotes \eqn{d_i}.
}
  \item{wgtm}{
Matrix with positive entries which indicates by a positive entry which
item pairs should be used for estimation.
}
\item{pos.loading}{An optional logical indicating whether all entries in the
loading matrix \eqn{\bold{F}} should be positive}
\item{pos.variance}{An optional logical indicating whether all variances (i.e.
    diagonal entries in \eqn{\bold{P}}) should be positive}
\item{pos.residcorr}{An optional logical indicating whether all
    entries in the matrix of residual correlations \eqn{\bold{\Psi}} should
    be positive}
\item{par_lower}{Optional vector of lower parameter bounds}
  \item{maxiter}{
Maximum number of iterations
}
  \item{conv}{
Convergence criterion for parameters
}
  \item{optimizer}{
Optimization function to be used. Can be \code{"nlminb"} for
\code{\link[stats:nlminb]{stats::nlminb}}
or \code{"optim"} for \code{\link[stats:optim]{stats::optim}}.
}
\item{reliability}{Logical indicating whether reliability should be computed.}
\item{\dots}{Further arguments to be passed. }
\item{object}{Object of class \code{noharm.sirt}}
\item{file}{String indicating a file name for summary.}
}

%\item{x}{Object of class \code{noharm.sirt}}
%\item{what}{A character indicating whether the original estimates (\code{est})
%or standardized estimates (\code{stand}) should be displayed.}
%\item{efa.load.min}{Minimum absolute factor loading to be displayed in
%    plot for exploratory factor analysis.
%}

% For \code{plot.noharm.sirt},
% additional arguments for \code{\link[semPlot:semPaths]{semPlot::semPaths}} (from
%\pkg{semPlot} package) can be specified.

\details{
The NOHARM item response model follows the response equation
\deqn{P( X_{pi}=1 | \bold{\theta}_p )=c_i + ( d_i - c_i )
\Phi( f_{i0} + f_{i1} \theta_{p1} + ... +
f_{iD} \theta_{pD} ) }
for item responses \eqn{X_{pi}} of person \eqn{p} on
item \eqn{i}, \eqn{\bold{F}=(f_{id})} is a loading matrix and \eqn{\bold{P}}
the covariance matrix of \eqn{\bold{\theta}_p}. The lower
asymptotes \eqn{c_i} and upper asymptotes \eqn{d_i} must be
provided as fixed values.
The response equation can be equivalently written by introducing a latent
continuous item response \eqn{X_{pi}^\ast}
\deqn{ X_{pi}^\ast=f_{i0} + f_{i1} \theta_{p1} + ... +  f_{iD} \theta_{pD} + e_{pi} }
with a standard normally distributed residual \eqn{e_{pi}}. These residuals
have a correlation matrix \eqn{\bold{\Psi}} with ones in the diagonal.
In this \R implementation of the NOHARM model, correlations between residuals
are allowed.

The estimation relies on a Hermite series approximation of the normal ogive
item response functions. In more detail, a series expansion
\deqn{\Phi(x)=b_0 + b_1 H_1(x) + b_2 H_2(x) + b_3 H_3(x)} is used
(McDonald, 1982a).
This enables to express cross products \eqn{p_{ij}=P(X_i=1, X_j=1)} as a function of
unknown model parameters
\deqn{\hat{p}_{ij}=b_{0i} b_{0j} + \sum_{m=1}^3 b_{mi} b_{mj}
\left( \frac{\bold{f}_i \bold{P} \bold{f}_j }{\sqrt{ (1+\bold{f}_i \bold{P} \bold{f}_i)
(1+\bold{f}_j \bold{P} \bold{f}_j)}} \right) ^m }
where \eqn{b_{0i}=p_{i}=P(X_i=1)=c_i + (d_i - c_i) \Phi(\tau_i)},
\eqn{b_{1i}=(d_i-c_i)\phi(\tau_i)}, \eqn{b_{2i}=(d_i-c_i)\tau_i \phi(\tau_i) / \sqrt{2}},
and \eqn{b_{3i}=(d_i-c_i)(\tau_i^2 - 1)\phi(\tau_i) / \sqrt{6}}.

The least squares criterion \eqn{\sum_{i<j} ( p_{ij} - \hat{p}_{ij})^2} is used
for estimating unknown model parameters (McDonald, 1982a, 1982b, 1997).

For derivations of standard errors and fit statistics see
Maydeu-Olivares (2001) and Swaminathan and Rogers (2016).

For the statistical properties of the NOHARM approach see
Knol and Berger (1991), Finch (2011) or Svetina and Levy (2016).
}


\value{
A list. The most important entries are
\item{tanaka}{Tanaka fit statistic}
\item{rmsr}{RMSR fit statistic}
\item{N.itempair}{Sample size per item pair}
\item{pm}{Product moment matrix}
\item{wgtm}{Matrix of weights for each item pair}
\item{sumwgtm}{Sum of lower triangle matrix \code{wgtm}}
\item{lower}{Lower asymptotes}
\item{upper}{Upper asymptotes}
\item{residuals}{Residual matrix from approximation of the \code{pm} matrix}
\item{final.constants}{Final constants}
\item{factor.cor}{Covariance matrix}
\item{thresholds}{Threshold parameters}
\item{uniquenesses}{Uniquenesses}
\item{loadings}{Matrix of standardized factor loadings (delta parametrization)}
\item{loadings.theta}{Matrix of factor loadings \eqn{\bold{F}} (theta parametrization)}
\item{residcorr}{Matrix of residual correlations}
\item{Nobs}{Number of observations}
\item{Nitems}{Number of items}
\item{Fpatt}{Pattern loading matrix for \eqn{\bold{F}}}
\item{Ppatt}{Pattern loading matrix for \eqn{\bold{P}}}
\item{Psipatt}{Pattern loading matrix for \eqn{\bold{\Psi}}}
\item{dat}{Used dataset}
\item{dimensions}{Number of dimensions}
\item{iter}{Number of iterations}
\item{Nestpars}{Number of estimated parameters}
\item{chisquare}{Statistic \eqn{\chi^2}}
\item{df}{Degrees of freedom}
\item{chisquare_df}{Ratio \eqn{\chi^2 / df}}
\item{rmsea}{RMSEA statistic}
\item{p.chisquare}{Significance for \eqn{\chi^2} statistic}
\item{omega.rel}{Reliability of the sum score according to Green and Yang (2009)}
}




\references{
Finch, H. (2011). Multidimensional item response theory parameter estimation with
nonsimple structure items. \emph{Applied Psychological Measurement, 35}(1), 67-82.
\doi{10.1177/0146621610367787}

Fraser, C., & McDonald, R. P. (1988). NOHARM: Least squares item factor analysis.
\emph{Multivariate Behavioral Research, 23}, 267-269.
\doi{10.1207/s15327906mbr2302_9}

Fraser, C., & McDonald, R. P. (2012). \emph{NOHARM 4 Manual}. \cr
http://noharm.niagararesearch.ca/nh4man/nhman.html.

Knol, D. L., & Berger, M. P. (1991). Empirical comparison between factor analysis
and multidimensional item response models. \emph{Multivariate Behavioral Research, 26}(3),
457-477. \doi{10.1207/s15327906mbr2603_5}

Maydeu-Olivares, A. (2001). Multidimensional item response theory modeling of binary data:
Large sample properties of NOHARM estimates.
\emph{Journal of Educational and Behavioral Statistics, 26}(1), 51-71.
\doi{10.3102/10769986026001051}

McDonald, R. P. (1982a). Linear versus nonlinear models in item response theory.
\emph{Applied Psychological Measurement, 6}(4), 379-396.
\doi{10.1177/014662168200600402}

McDonald, R. P. (1982b). \emph{Unidimensional and multidimensional models for
item response theory}. I.R.T., C.A.T. conference, Minneapolis, 1982, Proceedings.

McDonald, R. P. (1997). Normal-ogive multidimensional model.
In W. van der Linden & R. K. Hambleton (1997):
\emph{Handbook of modern item response theory} (pp. 257-269).
New York: Springer. \doi{10.1007/978-1-4757-2691-6}

Svetina, D., & Levy, R. (2016). Dimensionality in compensatory MIRT when complex structure
exists: Evaluation of DETECT and NOHARM. \emph{The Journal of Experimental Education, 84}(2),
398-420. \doi{10.1080/00220973.2015.1048845}

Swaminathan, H., & Rogers, H. J. (2016). Normal-ogive multidimensional models.
In W. J. van der Linden (Ed.). \emph{Handbook of item response theory.
Volume One: Models} (pp. 167-187). Boca Raton: CRC Press.
\doi{10.1201/9781315374512}
}


%\author{
%Alexander Robitzsch
%}




%\note{
%%  ~~further notes~~
%}



%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
EAP person parameter estimates can be obtained by \code{\link{R2noharm.EAP}}.

Model fit can be assessed by \code{\link{modelfit.sirt}}.

See \code{\link{R2noharm}} for running the NOHARM software from within \R.

See Fraser and McDonald (1988, 2012) for an implementation of the NOHARM model which
is available as freeware (http://noharm.niagararesearch.ca/;
the link seems to be broken in the meanwhile).
}


\examples{
#############################################################################
# EXAMPLE 1: Two-dimensional IRT model with 10 items
#############################################################################

#**** data simulation
set.seed(9776)
N <- 3400 # sample size
# define difficulties
f0 <- c( .5, .25, -.25, -.5, 0, -.5, -.25, .25, .5, 0 )
I <- length(f0)
# define loadings
f1 <- matrix( 0, I, 2 )
f1[ 1:5,1] <- c(.8,.7,.6,.5, .5)
f1[ 6:10,2] <- c(.8,.7,.6,.5, .5 )
# covariance matrix
Pval <- matrix( c(1,.5,.5,1), 2, 2 )
# simulate theta
library(mvtnorm)
theta <- mvtnorm::rmvnorm(N, mean=c(0,0), sigma=Pval )
# simulate item responses
dat <- matrix( NA, N, I )
for (ii in 1:I){ # ii <- 1
    dat[,ii] <- 1*( stats::pnorm(f0[ii]+theta[,1]*f1[ii,1]+theta[,2]*f1[ii,2])>
                     stats::runif(N) )
        }
colnames(dat) <- paste0("I", 1:I)

#**** Model 1: Two-dimensional CFA with estimated item loadings
# define pattern matrices
Pval <- .3+0*Pval
Ppatt <- 1*(Pval>0)
diag(Ppatt) <- 0
diag(Pval) <- 1
Fval <- .7 * ( f1>0)
Fpatt <- 1 * ( Fval > 0 )
# estimate model
mod1 <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt, Fpatt=Fpatt, Fval=Fval, Pval=Pval )
summary(mod1)
# EAP ability estimates
pmod1 <- sirt::R2noharm.EAP(mod1, theta.k=seq(-4,4,len=10) )
# model fit
summary( sirt::modelfit.sirt(mod1) )

\dontrun{
#*** compare results with NOHARM software
noharm.path <- "c:/NOHARM"   # specify path for noharm software
mod1a <- sirt::R2noharm( dat=dat, model.type="CFA",  F.pattern=Fpatt, F.init=Fval,
             P.pattern=Ppatt, P.init=Pval, writename="r2noharm_example",
             noharm.path=noharm.path, dec="," )
summary(mod1a)

#**** Model 1c: put some equality constraints
Fpatt[ c(1,4),1] <- 3
Fpatt[ cbind( c(3,7), c(1,2)) ] <- 4
mod1c <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt, Fpatt=Fpatt, Fval=Fval, Pval=Pval)
summary(mod1c)

#**** Model 2: Two-dimensional CFA with correlated residuals
# define pattern matrix for residual correlation
Psipatt <- 0*diag(I)
Psipatt[1,2] <- 1
Psival <- 0*Psipatt
# estimate model
mod2 <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt,Fpatt=Fpatt, Fval=Fval, Pval=Pval,
            Psival=Psival, Psipatt=Psipatt )
summary(mod2)

#**** Model 3: Two-dimensional Rasch model
# pattern matrices
Fval <- matrix(0,10,2)
Fval[1:5,1] <- Fval[6:10,2] <- 1
Fpatt <- 0*Fval
Ppatt <- Pval <- matrix(1,2,2)
Pval[1,2] <- Pval[2,1] <- 0
# estimate model
mod3 <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt,Fpatt=Fpatt, Fval=Fval, Pval=Pval )
summary(mod3)
# model fit
summary( sirt::modelfit.sirt( mod3 ))

#** compare fit with NOHARM
noharm.path <- "c:/NOHARM"
P.pattern <- Ppatt ; P.init <- Pval
F.pattern <- Fpatt ; F.init <- Fval
mod3b <- sirt::R2noharm( dat=dat, model.type="CFA",
             F.pattern=F.pattern, F.init=F.init, P.pattern=P.pattern,
             P.init=P.init, writename="example_sim_2dim_rasch",
             noharm.path=noharm.path, dec="," )
summary(mod3b)

#############################################################################
# EXAMPLE 2: data.read
#############################################################################

data(data.read)
dat <- data.read
I <- ncol(dat)

#**** Model 1: Unidimensional Rasch model
Fpatt <- matrix( 0, I, 1 )
Fval <- 1 + 0*Fpatt
Ppatt <- Pval <- matrix(1,1,1)
# estimate model
mod1 <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt,Fpatt=Fpatt, Fval=Fval, Pval=Pval )
summary(mod1)
plot(mod1)    # semPaths plot

#**** Model 2: Rasch model in which item pairs within a testlet are excluded
wgtm <- matrix( 1, I, I )
wgtm[1:4,1:4] <- wgtm[5:8,5:8] <- wgtm[ 9:12, 9:12] <- 0
# estimation
mod2 <- sirt::noharm.sirt(dat=dat, Ppatt=Ppatt,Fpatt=Fpatt, Fval=Fval, Pval=Pval, wgtm=wgtm)
summary(mod2)

#**** Model 3: Rasch model with correlated residuals
Psipatt <- Psival <- 0*diag(I)
Psipatt[1:4,1:4] <- Psipatt[5:8,5:8] <- Psipatt[ 9:12, 9:12] <- 1
diag(Psipatt) <- 0
Psival <- .6*(Psipatt>0)
# estimation
mod3 <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt,Fpatt=Fpatt, Fval=Fval, Pval=Pval,
            Psival=Psival, Psipatt=Psipatt )
summary(mod3)
# allow only positive residual correlations
mod3b <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt, Fpatt=Fpatt, Fval=Fval, Pval=Pval,
            Psival=Psival, Psipatt=Psipatt, pos.residcorr=TRUE)
summary(mod3b)
#* constrain residual correlations
Psipatt[1:4,1:4] <- 2
Psipatt[5:8,5:8] <- 3
Psipatt[ 9:12, 9:12] <- 4
mod3c <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt, Fpatt=Fpatt, Fval=Fval, Pval=Pval,
            Psival=Psival, Psipatt=Psipatt, pos.residcorr=TRUE)
summary(mod3c)

#**** Model 4: Rasch testlet model
Fval <- Fpatt <- matrix( 0, I, 4 )
Fval[,1] <- Fval[1:4,2] <- Fval[5:8,3] <- Fval[9:12,4 ] <- 1
Ppatt <- Pval <- diag(4)
colnames(Ppatt) <- c("g", "A", "B","C")
Pval <- .5*Pval
# estimation
mod4 <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt,Fpatt=Fpatt, Fval=Fval, Pval=Pval  )
summary(mod4)
# allow only positive variance entries
mod4b <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt,Fpatt=Fpatt, Fval=Fval, Pval=Pval,
               pos.variance=TRUE )
summary(mod4b)

#**** Model 5: Bifactor model
Fval <- matrix( 0, I, 4 )
Fval[,1] <- Fval[1:4,2] <- Fval[5:8,3] <- Fval[9:12,4 ] <- .6
Fpatt <- 1 * ( Fval > 0 )
Pval <- diag(4)
Ppatt <- 0*Pval
colnames(Ppatt) <- c("g", "A", "B","C")
# estimation
mod5 <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt,Fpatt=Fpatt, Fval=Fval, Pval=Pval  )
summary(mod5)
# allow only positive loadings
mod5b <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt,Fpatt=Fpatt, Fval=Fval, Pval=Pval,
              pos.loading=TRUE )
summary(mod5b)
summary( sirt::modelfit.sirt(mod5b))

#**** Model 6: 3-dimensional Rasch model
Fval <- matrix( 0, I, 3 )
Fval[1:4,1] <- Fval[5:8,2] <- Fval[9:12,3 ] <- 1
Fpatt <- 0*Fval
Pval <- .6*diag(3)
diag(Pval) <- 1
Ppatt <- 1+0*Pval
colnames(Ppatt) <- c("A", "B","C")
# estimation
mod6 <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt,Fpatt=Fpatt, Fval=Fval, Pval=Pval  )
summary(mod6)
summary( sirt::modelfit.sirt(mod6) )  # model fit

#**** Model 7: 3-dimensional 2PL model
Fval <- matrix( 0, I, 3 )
Fval[1:4,1] <- Fval[5:8,2] <- Fval[9:12,3 ] <- 1
Fpatt <- Fval
Pval <- .6*diag(3)
diag(Pval) <- 1
Ppatt <- 1+0*Pval
diag(Ppatt) <- 0
colnames(Ppatt) <- c("A", "B","C")
# estimation
mod7 <- sirt::noharm.sirt( dat=dat, Ppatt=Ppatt,Fpatt=Fpatt, Fval=Fval, Pval=Pval  )
summary(mod7)
summary( sirt::modelfit.sirt(mod7) )

#**** Model 8: Exploratory factor analysis with 3 dimensions
# estimation
mod8 <- sirt::noharm.sirt( dat=dat, dimensions=3  )
summary(mod8)

#############################################################################
# EXAMPLE 3: Product-moment matrix input, McDonald (1997)
#############################################################################

# data from Table 1 of McDonald (1997, p. 266)
pm0 <- "
0.828
0.567 0.658
0.664 0.560 0.772
0.532 0.428 0.501 0.606
0.718 0.567 0.672 0.526 0.843
"
pm <- miceadds::string_to_matrix(x=pm0, as_numeric=TRUE, extend=TRUE)
I <- nrow(pm)
rownames(pm) <- colnames(pm) <- paste0("I", 1:I)

#- Model 1: Unidimensional model
Fval <- matrix(.7, nrow=I, ncol=1)
Fpatt <- 1+0*Fval
Pval <- matrix(1, nrow=1,ncol=1)
Ppatt <- 0*Pval

mod1 <- sirt::noharm.sirt(pm=pm, N=1000, Fval=Fval, Fpatt=Fpatt, Pval=Pval, Ppatt=Ppatt)
summary(mod1)

#- Model 2: Twodimensional exploratory model
mod2 <- sirt::noharm.sirt(pm=pm, N=1000, dimensions=2)
summary(mod2)

#- Model 3: Unidimensional model with correlated residuals
Psival <- matrix(0, nrow=I, ncol=I)
Psipatt <- 0*Psival
Psipatt[5,1] <- 1

mod3 <- sirt::noharm.sirt(pm=pm, N=1000, Fval=Fval, Fpatt=Fpatt, Pval=Pval, Ppatt=Ppatt,
            Psival=Psival, Psipatt=Psipatt)
summary(mod3)
}
}

% plot(mod8)
% # plot only standardized loadings larger than .40
% plot(mod8, efa.load.min=.4 )

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{NOHARM}
%% \keyword{summary}
% \keyword{plot}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: dirichlet.simul.Rd
%% File Version: 0.16

\name{dirichlet.simul}
\alias{dirichlet.simul}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Simulation of a Dirichlet Distributed Vectors
}
\description{
This function makes random draws from a Dirichlet distribution.
}
\usage{
dirichlet.simul(alpha)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{alpha}{
A matrix with \eqn{\bold{\alpha}} parameters of the
Dirichlet distribution
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A data frame with Dirichlet distributed responses
}
%\references{
%% ~put references to the literature/web site here ~
%}
%\author{
%Alexander Robitzsch
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%For examples see \code{\link{dirichlet.mle}}
%}

\examples{
#############################################################################
# EXAMPLE 1: Simulation with two components
#############################################################################

set.seed(789)
N <- 2000
probs <- c(.7, .3)    # define (extremal) class probabilities

#*** alpha0=.2  -> nearly crisp latent classes
alpha0 <- .2
alpha <- alpha0*probs
alpha <- matrix( alpha, nrow=N, ncol=length(alpha), byrow=TRUE  )
x <- sirt::dirichlet.simul( alpha )
htitle <- expression(paste( alpha[0], "=.2, ", p[1], "=.7"   ) )
hist( x[,1], breaks=seq(0,1,len=20), main=htitle)

#*** alpha0=3 -> strong deviation from crisp membership
alpha0 <- 3
alpha <- alpha0*probs
alpha <- matrix( alpha, nrow=N, ncol=length(alpha), byrow=TRUE  )
x <- sirt::dirichlet.simul( alpha )
htitle <- expression(paste( alpha[0], "=3, ", p[1], "=.7"   ) )
hist( x[,1], breaks=seq(0,1,len=20), main=htitle)

\dontrun{
#############################################################################
# EXAMPLE 2: Simulation with three components
#############################################################################

set.seed(986)
N <- 2000
probs <- c( .5, .35, .15 )

#*** alpha0=.2
alpha0 <- .2
alpha <- alpha0*probs
alpha <- matrix( alpha, nrow=N, ncol=length(alpha), byrow=TRUE  )
x <- sirt::dirichlet.simul( alpha )
htitle <- expression(paste( alpha[0], "=.2, ", p[1], "=.7"   ) )
miceadds::library_install("ade4")
ade4::triangle.plot(x, label=NULL, clabel=1)

#*** alpha0=3
alpha0 <- 3
alpha <- alpha0*probs
alpha <- matrix( alpha, nrow=N, ncol=length(alpha), byrow=TRUE  )
x <- sirt::dirichlet.simul( alpha )
htitle <- expression(paste( alpha[0], "=3, ", p[1], "=.7"   ) )
ade4::triangle.plot(x, label=NULL, clabel=1)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Dirichlet distribution}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: pcm.fit.Rd
%% File Version: 0.20

\name{pcm.fit}
\alias{pcm.fit}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Item and Person Fit Statistics for the Partial Credit Model
}

\description{
Computes item and person fit statistics in the
partial credit model (Wright & Masters, 1990).
The rating scale model is accommodated as a particular partial
credit model (see Example 3).
}

\usage{
pcm.fit(b, theta, dat)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{b}{
Matrix with item category parameters (see Examples)
}
  \item{theta}{
Vector with estimated person parameters
}
  \item{dat}{
Dataset with item responses
}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

\value{
A list with entries
 \item{itemfit}{Item fit statistics}
 \item{personfit}{Person fit statistics}
}

\references{
Wright, B. D., & Masters, G. N. (1990). Computation of outfit and
infit statistics. \emph{Rasch Measurement Transactions, 3}:4, 84-85.
}

%\author{
%%  ~~who you are~~
%}

%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also \code{personfit.stat} for person fit statistics for dichotomous
item responses. See also the \pkg{PerFit} package for further person
fit statistics.

Item fit in other \R packages:
\code{eRm::itemfit},
\code{\link[TAM:tam.fit]{TAM::tam.fit}},
\code{\link[mirt:itemfit]{mirt::itemfit}},
\code{ltm::item.fit},

Person fit in other \R packages:
\code{eRm::itemfit},
\code{\link[mirt:itemfit]{mirt::itemfit}},
\code{ltm::person.fit},

See \code{\link{pcm.conversion}} for conversions of different
parametrizations of the partial credit model.
}



\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Partial credit model
#############################################################################

data(data.Students,package="CDM")
dat <- data.Students
# select items
items <- c(paste0("sc", 1:4 ), paste0("mj", 1:4 ) )
dat <- dat[,items]
dat <- dat[ rowSums( 1 - is.na(dat) ) > 0, ]

#*** Model 1a: Partial credit model in TAM
# estimate model
mod1a <- TAM::tam.mml( resp=dat )
summary(mod1a)
# estimate person parameters
wle1a <- TAM::tam.wle(mod1a)
# extract item parameters
b1 <- - mod1a$AXsi[, -1 ]
# parametrization in xsi parameters
b2 <- matrix( mod1a$xsi$xsi, ncol=3, byrow=TRUE )
# convert b2 to b1
b1b <- 0*b1
b1b[,1] <- b2[,1]
b1b[,2] <- rowSums( b2[,1:2] )
b1b[,3] <- rowSums( b2[,1:3] )
# assess fit
fit1a <- sirt::pcm.fit(b=b1, theta=wle1a$theta, dat)
fit1a$item

#############################################################################
# EXAMPLE 2: Rasch model
#############################################################################

data(data.read)
dat <- data.read

#*** Rasch model in TAM
# estimate model
mod <- TAM::tam.mml( resp=dat )
summary(mod)
# estimate person parameters
wle <- TAM::tam.wle(mod)
# extract item parameters
b1 <- - mod$AXsi[, -1 ]
# assess fit
fit1a <- sirt::pcm.fit(b=b1, theta=wle$theta, dat)
fit1a$item

#############################################################################
# EXAMPLE 3: Rating scale model
#############################################################################

data(data.Students,package="CDM")
dat <- data.Students
items <- paste0("sc", 1:4 )
dat <- dat[,items]
dat <- dat[ rowSums( 1 - is.na(dat) ) > 0, ]

#*** Model 1: Rating scale model in TAM
# estimate model
mod1 <- tam.mml( resp=dat, irtmodel="RSM")
summary(mod1)
# estimate person parameters
wle1 <- tam.wle(mod1)
# extract item parameters
b1 <- - mod1a$AXsi[, -1 ]
# fit statistic
pcm.fit(b=b1, theta=wle1$theta, dat)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Item fit}
%% \keyword{Person fit}% __ONLY ONE__ keyword per line

%% File Name: lsdm.Rd
%% File Version: 2.232


\name{lsdm}
\alias{lsdm}
\alias{summary.lsdm}
\alias{plot.lsdm}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Least Squares Distance Method of Cognitive Validation
}
\description{
This function estimates the least squares distance method
of cognitive validation (Dimitrov, 2007; Dimitrov & Atanasov, 2012)
which assumes a multiplicative relationship of attribute response
probabilities to explain item response probabilities. The argument \code{distance}
allows the estimation of a squared loss function (\code{distance="L2"})
and an absolute value loss function (\code{distance="L1"}).

The function also estimates the classical linear logistic test
model (LLTM; Fischer, 1973) which assumes a linear relationship
for item difficulties in the Rasch model.
}

\usage{
lsdm(data, Qmatrix, theta=seq(-3,3,by=.5), wgt_theta=rep(1, length(theta)), distance="L2",
   quant.list=c(0.5,0.65,0.8), b=NULL, a=rep(1,nrow(Qmatrix)), c=rep(0,nrow(Qmatrix)) )

\method{summary}{lsdm}(object, file=NULL, digits=3, ...)

\method{plot}{lsdm}(x, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
An \eqn{I \times L} matrix of dichotomous item responses.
The \code{data} consists of \eqn{I} item response functions
(parametrically or nonparametrically estimated) which are
evaluated at a discrete grid of \eqn{L} \code{theta} values
(person parameters)
and are specified in the argument \code{theta}.
 }
  \item{Qmatrix}{
An \eqn{I \times K} matrix where the allocation of items
to attributes is coded. Values of zero and one and all values between
zero and one are permitted. There must not be any items with only zero
Q-matrix entries in a row.
}
  \item{theta}{
The discrete grid points \eqn{\theta} where item response functions
are evaluated for doing the LSDM method.
}
\item{wgt_theta}{Optional vector for weights of discrete \eqn{\theta} points}
  \item{quant.list}{
A vector of quantiles where attribute response functions are
evaluated.
}
\item{distance}{Type of distance function for minimizing the discrepancy between
observed and expected item response functions. Options are \code{"L2"} which is the
squared distance (proposed in the original LSDM formulation in Dimitrov, 2007)
and the absolute value distance \code{"L1"} (see Details).
}
  \item{b}{
An optional vector of item difficulties. If it is specified,
then no \code{data} input is necessary.
}
  \item{a}{
An optional vector of item discriminations.
}
  \item{c}{
An optional vector of guessing parameters.
}
\item{object}{Object of class \code{lsdm}}
\item{file}{Optional file name for \code{summary} output}
\item{digits}{Number of digits aftert decimal in \code{summary}}
\item{\dots}{Further arguments to be passed}
\item{x}{Object of class \code{lsdm}}
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DETAILS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\details{
The least squares distance method (LSDM; Dimitrov 2007) is based on the
assumption that estimated item response functions \eqn{P(X_i=1 | \theta)}
can be decomposed in a multiplicative way (in the implemented
conjunctive model):
\deqn{ P( X_i=1  | \theta ) \approx \prod_{k=1}^K [ P( A_k=1 | \theta ) ]^{q_{ik}} }
where \eqn{P( A_k=1 | \theta )} are attribute response functions and
\eqn{q_{ik}} are entries of the Q-matrix. Note that the multiplicative form
can be rewritten by taking the logarithm
\deqn{ \log P( X_i=1  | \theta ) \approx
    \sum_{k=1}^K q_{ik} \log [ P( A_k=1 | \theta ) ] }
The item and attribute response functions are evaluated on a grid of \eqn{\theta} values.
Using the definitions of matrices \eqn{\bold{L}=\{ \log P( X_i=1 ) | \theta ) \} },
\eqn{\bold{Q}=\{ q_{ik} \} } and
\eqn{\bold{X}=\{ \log P( A_k=1  | \theta ) \} }, the estimation problem can be formulated
as \eqn{ \bold{L} \approx \bold{Q} \bold{X}}. Two different loss functions for minimizing
the discrepancy between \eqn{ \bold{L}} and \eqn{\bold{Q} \bold{X}} are implemented.
First, the squared loss function computes the weighted difference
\eqn{|| \bold{L} - \bold{Q} \bold{X}||_2=\sum_i ( l_i - \sum_t q_{it} x_{it})^2}
 (\code{distance="L2"}) and has
been originally proposed by Dimitrov (2007). Second, the
absolute value loss function
\eqn{|| \bold{L} - \bold{Q} \bold{X}||_1=\sum_i | l_i - \sum_t q_{it} x_{it} |}
(\code{distance="L1"}) is more robust to outliers (i.e., items which
show misfit to the assumed multiplicative LSDM formulation).

After fitting the attribute response functions, empirical item-attribute
discriminations \eqn{w_{ik}} are calculated as the approximation of the following
equation
\deqn{ \log P( X_i=1  | \theta )=
\sum_{k=1}^K w_{ik} q_{ik} \log [ P( A_k=1 | \theta ) ] }
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VALUES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\value{
A list with following entries
\item{mean.mad.lsdm0}{Mean of \eqn{MAD} statistics for LSDM}
\item{mean.mad.lltm}{Mean of \eqn{MAD} statistics for LLTM}
\item{attr.curves}{Estimated attribute response curves evaluated at \code{theta}}
\item{attr.pars}{Estimated attribute parameters for LSDM and LLTM}
\item{data.fitted}{LSDM-fitted item response functions evaluated at \code{theta}}
\item{theta}{Grid of ability distributions at which
functions are evaluated}
\item{item}{Item statistics (p value, \eqn{MAD}, ...)}
\item{data}{Estimated or fixed item response functions evaluated at \code{theta}}
\item{Qmatrix}{Used Q-matrix}
\item{lltm}{Model output of LLTM (\code{lm} values)}
\item{W}{Matrix with empirical item-attribute discriminations}
}


\references{

Al-Shamrani, A., & Dimitrov, D. M. (2016). Cognitive diagnostic analysis of reading
comprehension items: The case of English proficiency assessment in Saudi Arabia.
\emph{International Journal of School and Cognitive Psychology, 4}(3). 1000196.
http://dx.doi.org/10.4172/2469-9837.1000196

DiBello, L. V., Roussos, L. A., & Stout, W. F. (2007). Review of
cognitively diagnostic assessment and a summary of psychometric models.
In C. R. Rao and S. Sinharay (Eds.), \emph{Handbook of Statistics},
Vol. 26 (pp. 979-1030). Amsterdam: Elsevier.

Dimitrov, D. M. (2007). Least squares distance method of cognitive validation
and analysis for binary items using their item response theory parameters.
\emph{Applied Psychological Measurement, 31}, 367-387.
http://dx.doi.org/10.1177/0146621606295199

Dimitrov, D. M., & Atanasov, D. V. (2012). Conjunctive and disjunctive
extensions of the least squares distance model of cognitive diagnosis.
\emph{Educational and Psychological Measurement, 72}, 120-138.
http://dx.doi.org/10.1177/0013164411402324

Dimitrov, D. M., Gerganov, E. N., Greenberg, M., & Atanasov, D. V. (2008).
\emph{Analysis of cognitive attributes for mathematics items in the framework of
Rasch measurement}. AERA 2008, New York.

Fischer, G. H. (1973). The linear logistic test model as an instrument
in educational research. \emph{Acta Psychologica, 37}, 359-374.
http://dx.doi.org/10.1016/0001-6918(73)90003-6

Sonnleitner, P. (2008). Using the LLTM to evaluate an item-generating system
for reading comprehension. \emph{Psychology Science, 50}, 345-362.
}


%  Groemping, U. (2010). Inference with linear equality and inequality
%  constraints using \R: The package \pkg{ic.infer}.
%  \emph{Journal of Statistical Software, 33}(10), 1-31.

%\author{
%Alexander Robitzsch
%}


%\note{
%xx
%}


% This function needs the \pkg{ic.infer} package.

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
Get a summary of the LSDM analysis with \code{\link{summary.lsdm}}.

See the \pkg{CDM} package for the estimation of related
cognitive diagnostic models (DiBello, Roussos & Stout, 2007).
}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EXAMPLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



\examples{
#############################################################################
# EXAMPLE 1: Dataset Fischer (see Dimitrov, 2007)
#############################################################################

# item difficulties
b <- c( 0.171,-1.626,-0.729,0.137,0.037,-0.787,-1.322,-0.216,1.802,
    0.476,1.19,-0.768,0.275,-0.846,0.213,0.306,0.796,0.089,
    0.398,-0.887,0.888,0.953,-1.496,0.905,-0.332,-0.435,0.346,
    -0.182,0.906)
# read Q-matrix
Qmatrix <- c( 1,1,0,1,0,0,0,0,1,0,1,0,0,0,0,0,1,0,0,1,0,0,0,0,
    1,0,1,1,0,0,0,0,1,0,0,1,0,0,0,0,0,1,0,0,1,1,0,0,1,0,1,0,1,0,0,0,
    1,0,1,0,1,1,0,0,1,0,1,1,0,1,0,0,1,0,0,1,0,1,0,0,1,0,1,1,1,0,0,0,
    1,0,0,1,0,0,1,0,1,0,0,1,0,0,1,0,1,0,1,0,0,0,1,0,1,1,0,1,0,1,1,0,
    1,0,1,1,0,0,1,0,1,0,0,1,0,0,0,1,1,0,1,1,0,0,0,1,1,0,0,1,0,0,0,1,
    0,1,0,0,0,1,0,1,1,1,0,1,0,1,0,1,1,0,0,1,0,1,0,0,1,1,0,0,1,0,0,0,
    1,0,0,1,1,0,0,0,1,1,0,1,0,0,0,0,1,0,1,1,0,0,0,0,1,0,1,1,0,1,0,0,
    1,1,0,1,0,0,0,0,1,0,1,1,1,1,0,0 )
Qmatrix <- matrix( Qmatrix, nrow=29, byrow=TRUE )
colnames(Qmatrix) <- paste("A",1:8,sep="")
rownames(Qmatrix) <- paste("Item",1:29,sep="")

#* Model 1: perform a LSDM analysis with defaults
mod1 <- sirt::lsdm( b=b, Qmatrix=Qmatrix )
summary(mod1)
plot(mod1)

#* Model 2: different theta values and weights
theta <- seq(-4,4,len=31)
wgt_theta <- stats::dnorm(theta)
mod2 <- sirt::lsdm( b=b, Qmatrix=Qmatrix, theta=theta, wgt_theta=wgt_theta )
summary(mod2)

#* Model 3: absolute value distance function
mod3 <- sirt::lsdm( b=b, Qmatrix=Qmatrix, distance="L1" )
summary(mod3)

#############################################################################
# EXAMPLE 2: Dataset Henning (see Dimitrov, 2007)
#############################################################################

# item difficulties
b <- c(-2.03,-1.29,-1.03,-1.58,0.59,-1.65,2.22,-1.46,2.58,-0.66)
# item slopes
a <- c(0.6,0.81,0.75,0.81,0.62,0.75,0.54,0.65,0.75,0.54)
# define Q-matrix
Qmatrix <- c(1,0,0,0,0,0,1,0,0,0,0,1,0,1,0,0,1,0,0,0,0,1,1,0,0,
    0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,0,0,1,1,1,0,1,0,0 )
Qmatrix <- matrix( Qmatrix, nrow=10, byrow=TRUE )
colnames(Qmatrix) <- paste("A",1:5,sep="")
rownames(Qmatrix) <- paste("Item",1:10,sep="")

# LSDM analysis
mod <- sirt::lsdm( b=b, a=a, Qmatrix=Qmatrix )
summary(mod)

\dontrun{
#############################################################################
# EXAMPLE 3: PISA reading (data.pisaRead)
#    using nonparametrically estimated item response functions
#############################################################################

data(data.pisaRead)
# response data
dat <- data.pisaRead$data
dat <- dat[, substring( colnames(dat),1,1)=="R" ]
# define Q-matrix
pars <- data.pisaRead$item
Qmatrix <- data.frame(  "A0"=1*(pars$ItemFormat=="MC" ),
                  "A1"=1*(pars$ItemFormat=="CR" ) )

# start with estimating the 1PL in order to get person parameters
mod <- sirt::rasch.mml2( dat )
theta <- sirt::wle.rasch( dat=dat,b=mod$item$b )$theta
# Nonparametric estimation of item response functions
mod2 <- sirt::np.dich( dat=dat, theta=theta, thetagrid=seq(-3,3,len=100) )

# LSDM analysis
lmod <- sirt::lsdm( data=mod2$estimate, Qmatrix=Qmatrix, theta=mod2$thetagrid)
summary(lmod)
plot(lmod)

#############################################################################
# EXAMPLE 4: Fraction subtraction dataset
#############################################################################

data( data.fraction1, package="CDM")
data <- data.fraction1$data
q.matrix <- data.fraction1$q.matrix

#****
# Model 1: 2PL estimation
mod1 <- sirt::rasch.mml2( data, est.a=1:nrow(q.matrix) )

# LSDM analysis
lmod1 <- sirt::lsdm( b=mod1$item$b, a=mod1$item$a, Qmatrix=q.matrix )
summary(lmod1)

#****
# Model 2: 1PL estimation
mod2 <- sirt::rasch.mml2(data)

# LSDM analysis
lmod2 <- sirt::lsdm( b=mod1$item$b, Qmatrix=q.matrix )
summary(lmod2)

#############################################################################
# EXAMPLE 5: Dataset LLTM Sonnleitner Reading Comprehension (Sonnleitner, 2008)
#############################################################################

# item difficulties Table 7, p. 355 (Sonnleitner, 2008)
b <- c(-1.0189,1.6754,-1.0842,-.4457,-1.9419,-1.1513,2.0871,2.4874,-1.659,-1.197,-1.2437,
    2.1537,.3301,-.5181,-1.3024,-.8248,-.0278,1.3279,2.1454,-1.55,1.4277,.3301)
b <- b[-21] # remove Item 21

# Q-matrix Table 9, p. 357 (Sonnleitner, 2008)
Qmatrix <- scan()
   1 0 0 0 0 0 0 7 4 0 0 0   0 1 0 0 0 0 0 5 1 0 0 0   1 1 0 1 0 0 0 9 1 0 1 0
   1 1 1 0 0 0 0 5 2 0 1 0   1 1 0 0 1 0 0 7 5 1 1 0   1 1 0 0 0 0 0 7 3 0 0 0
   0 1 0 0 0 0 2 6 1 0 0 0   0 0 0 0 0 0 2 6 1 0 0 0   1 0 0 0 0 0 1 7 4 1 0 0
   0 1 0 0 0 0 0 6 2 1 1 0   0 1 0 0 0 1 0 7 3 1 0 0   0 1 0 0 0 0 0 5 1 0 0 0
   0 0 0 0 0 1 0 4 1 0 0 1   0 0 0 0 0 0 0 6 1 0 1 1   0 0 1 0 0 0 0 6 3 0 1 1
   0 0 0 1 0 0 1 7 5 0 0 1   0 1 0 0 0 0 1 2 2 0 0 1   0 1 1 0 0 0 1 4 1 0 0 1
   0 1 0 0 1 0 0 5 1 0 0 1   0 1 0 0 0 0 1 7 2 0 0 1   0 0 0 0 0 1 0 5 1 0 0 1

Qmatrix <- matrix( as.numeric(Qmatrix), nrow=21, ncol=12, byrow=TRUE )
colnames(Qmatrix) <- scan( what="character", nlines=1)
   pc ic ier inc iui igc ch nro ncro td a t

# divide Q-matrix entries by maximum in each column
Qmatrix <- round(Qmatrix / matrix(apply(Qmatrix,2,max),21,12,byrow=TRUE),3)
# LSDM analysis
mod <- sirt::lsdm( b=b, Qmatrix=Qmatrix )
summary(mod)

#############################################################################
# EXAMPLE 6: Dataset Dimitrov et al. (2008)
#############################################################################

Qmatrix <- scan()
1 0 0 0 1 1 0 1 1 0 0 0 1 0 0 0 1 1 0 0 1 1 0 1 0 0 1 1 1 0 1 0 0 0 1 0

Qmatrix <- matrix(Qmatrix, ncol=4, byrow=TRUE)
colnames(Qmatrix) <- paste0("A",1:4)
rownames(Qmatrix) <- paste0("I",1:9)

b <- scan()
0.068 1.095 -0.641 -1.129 -0.061 1.218 1.244 -0.648 -1.146

# estimate model
mod <- sirt::lsdm( b=b, Qmatrix=Qmatrix )
summary(mod)
plot(mod)

#############################################################################
# EXAMPLE 7: Dataset Al-Shamrani & Dimitrov et al. (2017)
#############################################################################

I <- 39  # number of items

Qmatrix <- scan()
0 0 0 1 0 0 0 0 1 0 1 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 1 0 0 0 0 0 0 1 0 0 0 0 0 1 0 0
0 0 0 0 0 1 1 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 0 0 0 0 0 0 0 0 0 0 1 0 0
0 0 0 0 1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 1 0 0 0
0 0 0 1 0 0 0 0 0 0 1 0 0 0 0 0 0 1 0 0 0 0 0 0 1 0 1 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0
0 0 1 0 0 0 0 1 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 1 0 0 0 0 0 1 0 0 0 0 0 0 0 1 0 0 0 0 0
0 1 0 0 0 0 0 0 1 1 1 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 1 0 0 0 0 0 0 1 0 0 1 0 0 0
0 0 1 0 0 0 0 0 0 0 0 0 0 1 0

Qmatrix <- matrix(Qmatrix, nrow=I, byrow=TRUE)
colnames(Qmatrix) <- paste0("A",1:7)
rownames(Qmatrix) <- paste0("I",1:I)

pars <- scan()
1.952 0.9833 0.1816 1.1053 0.9631 0.1653 1.3904 1.3208 0.2545 0.7391 1.9367 0.2083 2.0833
1.8627 0.1873 1.4139 1.0107 0.2454 0.8274 0.9913 0.2137 1.0338 -0.0068 0.2368 2.4803
0.7939 0.1997 1.4867 1.1705 0.2541 1.4482 1.4176 0.2889 1.0789 0.8062 0.269 1.6258 1.1739
0.1723 1.5995 1.0936 0.2054 1.1814 1.0909 0.2623 2.0389 1.5023 0.2466 1.3636 1.1485 0.2059
1.8468 1.2755 0.192 1.9461 1.4947 0.2001 1.194 0.0889 0.2275 1.2114 0.8925 0.2367 2.0912
0.5961 0.2036 2.5769 1.3014 0.186 1.4554 1.2529 0.2423 1.4919 0.4763 0.2482 2.6787 1.7069
0.1796 1.5611 1.3991 0.2312 1.4353 0.678 0.1851 0.9127 1.3523 0.2525 0.6886 -0.3652 0.207
0.7039 -0.2494 0.2315 1.3683 0.8953 0.2326 1.4992 0.1025 0.2403 1.0727 0.2591 0.2152
1.3854 1.3802 0.2448 0.7748 0.4304 0.184 1.0218 1.8964 0.1949 1.5773 1.8934 0.2231 0.8631
1.4145 0.2132

pars <- matrix(pars, nrow=I, byrow=TRUE)
colnames(pars) <- c("a","b","c")
rownames(pars) <- paste0("I",1:I)
pars <- as.data.frame(pars)

#* Model 1: fit LSDM to 3PL curves (as in Al-Shamrani)
mod1 <- sirt::lsdm(b=pars$b, a=pars$a, c=pars$c, Qmatrix=Qmatrix)
summary(mod1)
plot(mod1)

#* Model 2: fit LSDM to 2PL curves
mod2 <- sirt::lsdm(b=pars$b, a=pars$a, Qmatrix=Qmatrix)
summary(mod2)
plot(mod2)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Least Squares Distance Method (LSDM)}
%% \keyword{summary}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: locpolycor.Rd
%% File Version: 0.082

\name{locpolycor}
\alias{locpolycor}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Local Modeling of Thresholds and Polychoric Correlations
}
\description{
Estimates thresholds and polychoric correlations as a function of a
continuous moderator variable \eqn{x}.
}
\usage{
locpolycor(y, data.mod, moderator.grid, h=1.1, model_thresh, model_polycor,
     sampling_weights=NULL, kernel="gaussian", eps=1e-10)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{y}{
Matrix with columns referrring to ordinal items
}
  \item{data.mod}{
Values of the moderator variable \eqn{x}
}
  \item{moderator.grid}{
Grid of \eqn{x} values to be used for local estimation of thresholds
and polychoric correlations
}
  \item{h}{
Bandwidth factor
}
  \item{model_thresh}{
Model for thresholds: can be \code{'const'} (constant function) or \code{'lin'}
(linear function)
}
  \item{model_polycor}{
Model for polychoric correlations: can be \code{'const'} (constant function) or
\code{'lin'} (linear function)
}
\item{sampling_weights}{Optional vector of sampling weights}
  \item{kernel}{
Used kernel function, see \code{\link{lsem_local_weights}}
}
  \item{eps}{
Parameter added in likelihood optimization
}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

\value{
A list with entries
\item{thresh_list}{Threshold parameters}
\item{thresh_stat}{Estimated thresholds}
\item{polycor_stat}{Estimated polychoric correlations}
\item{\ldots}{...}
}

%\references{
%% ~put references to the literature/web site here ~
%}

%\author{
%%  ~~who you are~~
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}


\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Two items, moderator on (0,1)
#############################################################################


#*** simulate data

# functions for thresholds
th1_fun <- function(x){ -0.3*(x-2)^2 + .2 }
th2_fun <- function(x){ 0.4*(x+1)^2 - 0.6 }
zh1_fun <- function(x){ 0.3*(x-1) }
# function polychoric correlation
cor_x12 <- function(x){ 0.2+0.1*(x-0.5)+0.09*(x-0.5)^2 }

# simulate moderator
x <- stats::runif(N)

# simulate data
yast <- matrix( NA, nrow=N, ncol=2)
for (nn in 1:N){
    rho12 <- cor_x12(x[nn])
    Sigma <- matrix(0, 2,2)
    Sigma[1,2] <- rho12
    Sigma <- Sigma + t(Sigma)
    diag(Sigma) <- 1
    yast_nn <- MASS::mvrnorm( 1, mu=rep(0,2), Sigma=Sigma )
    yast[nn,] <- yast_nn
}
y <- 0*yast
th1_x <- th1_fun(x)
th2_x <- th2_fun(x)
zh1_x <- zh1_fun(x)
y[,1] <- 1*( yast[,1] > th1_x ) + 1*( yast[,1] > th2_x )
y[,2] <- 1*( yast[,2] > zh1_x )
colnames(y) <- paste0("I",1:ncol(y))

dat <- data.frame(x=x, y)

#-- local modeling
res <- sirt::locpolycor(y, data.mod=x, moderator.grid=c(0, .25, .5, .75, 1 ), h=2,
                 model_thresh="lin", model_polycor="lin")
str(res)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

%% File Name: qmc.nodes.Rd
%% File Version: 0.18

\name{qmc.nodes}
\alias{qmc.nodes}
\title{
Calculation of Quasi Monte Carlo Integration Points
}
\description{
This function calculates integration nodes based on the multivariate
normal distribution with zero mean vector and identity covariance
matrix. See Pan and Thompson (2007) and Gonzales et al. (2006)
for details.
}
\usage{
qmc.nodes(snodes, ndim)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{snodes}{
Number of integration nodes
}
  \item{ndim}{
Number of dimensions
}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

\value{
 \item{theta}{A matrix of integration points}
}
\references{
Gonzalez, J., Tuerlinckx, F., De Boeck, P., & Cools, R. (2006).
Numerical integration in logistic-normal models.
\emph{Computational Statistics & Data Analysis, 51}, 1535-1548.

Pan, J., & Thompson, R. (2007). Quasi-Monte Carlo estimation in
generalized linear mixed models. \emph{Computational Statistics &
Data Analysis, 51}, 5765-5775.
}


%\author{
%Alexander Robitzsch
%}



\note{
This function uses the
\code{\link[sfsmisc:QUnif]{sfsmisc::QUnif}} function from
the \pkg{sfsmisc} package.
}
% sfsmisc::QUnif

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}

\examples{
## some toy examples

# 5 nodes on one dimension
qmc.nodes( snodes=5, ndim=1 )
  ##            [,1]
  ## [1,]  0.0000000
  ## [2,] -0.3863753
  ## [3,]  0.8409238
  ## [4,] -0.8426682
  ## [5,]  0.3850568

# 7 nodes on two dimensions
qmc.nodes( snodes=7, ndim=2 )
  ##             [,1]        [,2]
  ## [1,]  0.00000000 -0.43072730
  ## [2,] -0.38637529  0.79736332
  ## [3,]  0.84092380 -1.73230641
  ## [4,] -0.84266815 -0.03840544
  ## [5,]  0.38505683  1.51466109
  ## [6,] -0.00122394 -0.86704605
  ## [7,]  1.35539115  0.33491073
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Quasi Monte Carlo integration}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.activity.itempars.Rd
%% File Version: 0.09

\name{data.activity.itempars}
\alias{data.activity.itempars}
\docType{data}
\title{
Item Parameters Cultural Activities
}
\description{
List with item parameters for cultural activities
of Austrian students for 9 Austrian countries.
}
\usage{
data(data.activity.itempars)
}

\format{
The format is a list with number of students per group
(\code{N}), item loadings (\code{lambda}) and
item intercepts (\code{nu}):

\code{List of 3} \cr
\code{ $ N     : 'table' int [1:9(1d)] 2580 5279 15131 14692 5525 11005 7080 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 1} \cr
\code{  .. ..$ : chr [1:9] "1" "2" "3" "4" ...} \cr
\code{ $ lambda: num [1:9, 1:5] 0.423 0.485 0.455 0.437 0.502 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : chr [1:9] "country1" "country2" "country3" "country4" ...} \cr
\code{  .. ..$ : chr [1:5] "act1" "act2" "act3" "act4" ...} \cr
\code{ $ nu    : num [1:9, 1:5] 1.65 1.53 1.7 1.59 1.7 ...} \cr
\code{  ..- attr(*, "dimnames")=List of 2} \cr
\code{  .. ..$ : chr [1:9] "country1" "country2" "country3" "country4" ...} \cr
\code{  .. ..$ : chr [1:5] "act1" "act2" "act3" "act4" ...} \cr
}
%%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%% }
%% \source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%%}
%%\references{
%%  ~~ possibly secondary sources and usages ~~
%% }



%%\examples{
%%data(data.activity.itempars)
%%## maybe str(data.activity.itempars) ; plot(data.activity.itempars) ...
%%}
%% \keyword{datasets}

%% File Name: Q3.Rd
%% File Version: 1.21

\name{Q3}
\alias{Q3}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimation of the \eqn{Q_3} Statistic (Yen, 1984)
}
\description{
This function estimates the \eqn{Q_3} statistic according to Yen (1984).
The statistic \eqn{Q_3} is calculated for every item pair \eqn{(i,j)}
which is the correlation between item residuals after fitting the Rasch model.
}
\usage{
Q3(dat, theta, b, progress=TRUE)
}
\arguments{
  \item{dat}{
An \eqn{N \times I} data frame of dichotomous item responses
}
  \item{theta}{
Vector of length \eqn{N} of person parameter estimates (e.g. obtained from
    \code{\link{wle.rasch}})
}
  \item{b}{
Vector of length \eqn{I} (e.g. obtained from \code{\link{rasch.mml2}})
}
  \item{progress}{
Should iteration progress be displayed?
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list with following entries
\item{q3.matrix}{An \eqn{I \times I} matrix of \eqn{Q_3} statistics}
\item{q3.long}{Just the \code{q3.matrix} in long matrix format where every row
                  corresponds to an item pair}
\item{expected}{An \eqn{N \times I} matrix of expected probabilities
    by the Rasch model}
\item{residual}{An \eqn{N \times I} matrix of residuals obtained after
    fitting the Rasch model}
\item{Q3.stat}{Vector with descriptive statistics of \eqn{Q_3}}
}


\references{
Yen, W. M. (1984). Effects of local item dependence on the fit and equating
performance of the three-parameter logistic model.
\emph{Applied Psychological Measurement, 8}, 125-145.
}



%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
For the estimation of the average \eqn{Q_3} statistic within testlets see
\code{\link{Q3.testlet}}.

For modeling testlet effects see \code{\link{mcmc.3pno.testlet}}.

For handling local dependencies in IRT models see
\code{\link{rasch.copula2}}, \code{\link{rasch.pml3}} or \cr
\code{\link{rasch.pairwise.itemcluster}}.
}
\examples{
#############################################################################
# EXAMPLE 1: data.read. The 12 items are arranged in 4 testlets
#############################################################################
data(data.read)

# estimate the Rasch model
mod <- sirt::rasch.mml2( data.read)
# estmate WLEs
mod.wle <- sirt::wle.rasch( dat=data.read, b=mod$item$b )
# calculate Yen's Q3 statistic
mod.q3 <- sirt::Q3( dat=data.read, theta=mod.wle$theta, b=mod$item$b )
  ##   Yen's Q3 Statistic based on an estimated theta score
  ##   *** 12 Items | 66 item pairs
  ##   *** Q3 Descriptives
  ##        M     SD    Min    10%    25%    50%    75%    90%    Max
  ##   -0.085  0.110 -0.261 -0.194 -0.152 -0.107 -0.051  0.041  0.412

# plot Q3 statistics
I <- ncol(data.read)
image( 1:I, 1:I, mod.q3$q3.matrix, col=gray( 1 - (0:32)/32),
        xlab="Item", ylab="Item")
abline(v=c(5,9)) # borders for testlets
abline(h=c(5,9))

\dontrun{
# obtain Q3 statistic from modelfit.sirt function which is based on the
# posterior distribution of theta and not on observed values
fitmod <- sirt::modelfit.sirt( mod )
# extract Q3 statistic
q3stat <- fitmod$itempairs$Q3
  ##  > summary(q3stat)
  ##      Min.  1st Qu.   Median     Mean  3rd Qu.     Max.
  ##  -0.21760 -0.11590 -0.07280 -0.05545 -0.01220  0.44710
  ##  > sd(q3stat)
  ##  [1] 0.1101451
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Testlets}
%% \keyword{Local dependence}
%% \keyword{Q3}

%% File Name: mcmc.2pno.Rd
%% File Version: 1.33

\name{mcmc.2pno}
\alias{mcmc.2pno}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
MCMC Estimation of the Two-Parameter Normal Ogive Item Response Model
}
\description{
This function estimates the Two-Parameter normal ogive item response model
by MCMC sampling (Johnson & Albert, 1999, p. 195ff.).
}
\usage{
mcmc.2pno(dat, weights=NULL, burnin=500, iter=1000, N.sampvalues=1000,
      progress.iter=50, save.theta=FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame with dichotomous item responses
}
\item{weights}{
    An optional vector with student sample weights
        }
  \item{burnin}{
Number of burnin iterations
}
  \item{iter}{
Total number of iterations
}
  \item{N.sampvalues}{
Maximum number of sampled values to save
}
  \item{progress.iter}{
Display progress every \code{progress.iter}-th iteration. If no progress
display is wanted, then choose \code{progress.iter} larger than \code{iter}.
}
  \item{save.theta}{
Should theta values be saved?
}
}
\details{
The two-parameter normal ogive item response model with a probit link
function is defined by
\deqn{ P(X_{pi}=1 | \theta_p )=\Phi ( a_i \theta_p - b_i )
        \quad,  \quad \theta_p \sim N(0,1) }
Note that in this implementation non-informative priors for the item
parameters are chosen (Johnson & Albert, 1999, p. 195ff.).
}
\value{
A list of class \code{mcmc.sirt} with following entries:
  \item{mcmcobj}{Object of class \code{mcmc.list}}
  \item{summary.mcmcobj}{Summary of the \code{mcmcobj} object. In this
  summary the Rhat statistic and the mode estimate MAP is included.
  The variable \code{PercSEratio} indicates the proportion of the Monte Carlo
  standard error in relation to the total standard deviation of the
  posterior distribution.}
  \item{burnin}{Number of burnin iterations}
  \item{iter}{Total number of iterations}
  \item{a.chain}{Sampled values of \eqn{a_i} parameters}
  \item{b.chain}{Sampled values of \eqn{b_i} parameters}
  \item{theta.chain}{Sampled values of \eqn{\theta_p} parameters}
  \item{deviance.chain}{Sampled values of Deviance values}
  \item{EAP.rel}{EAP reliability}
  \item{person}{Data frame with EAP person parameter estimates for
        \eqn{\theta_p} and their corresponding posterior standard
      deviations}
  \item{dat}{Used data frame}
  \item{weights}{Used student weights}
  \item{\dots}{Further values}
}
% "EAP.rel"=EAP.rel, "dat"=dat0, "weights"=weights,

\references{
Johnson, V. E., & Albert, J. H. (1999). \emph{Ordinal Data Modeling}.
New York: Springer.
}

%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
S3 methods: \code{\link{summary.mcmc.sirt}}, \code{\link{plot.mcmc.sirt}}

For estimating the 2PL model with marginal maximum likelihood see
\code{\link{rasch.mml2}} or \code{\link{smirt}}.

A hierarchical version of this model can be estimated with
\code{\link{mcmc.2pnoh}}.
}
\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Dataset Reading
#############################################################################
data(data.read)
# estimate 2PNO with MCMC with 3000 iterations and 500 burn-in iterations
mod <- sirt::mcmc.2pno( dat=data.read, iter=3000, burnin=500 )
# plot MCMC chains
plot( mod$mcmcobj, ask=TRUE )
# write sampled chains into codafile
mcmclist2coda( mod$mcmcobj, name="dataread_2pno" )
# summary
summary(mod)

#############################################################################
# EXAMPLE 2
#############################################################################
# simulate data
N <- 1000
I <- 10
b <- seq( -1.5, 1.5, len=I )
a <- rep( c(1,2), I/2 )
theta1 <- stats::rnorm(N)
dat <- sirt::sim.raschtype( theta=theta1, fixed.a=a, b=b )

#***
# Model 1: estimate model without weights
mod1 <- sirt::mcmc.2pno( dat, iter=1500, burnin=500)
mod1$summary.mcmcobj
plot( mod1$mcmcobj, ask=TRUE )

#***
# Model 2: estimate model with weights
# define weights
weights <- c( rep( 5, N/4 ), rep( .2, 3/4*N ) )
mod2 <- sirt::mcmc.2pno( dat, weights=weights, iter=1500, burnin=500)
mod1$summary.mcmcobj
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Markov Chain Monte Carlo (MCMC)}
% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: ccov.np.Rd
%% File Version: 0.232

\name{ccov.np}
\alias{ccov.np}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Nonparametric Estimation of Conditional Covariances of Item Pairs
}
\description{
This function estimates conditional covariances of itempairs
(Stout, Habing, Douglas & Kim, 1996; Zhang & Stout,
1999a). The function is used for the estimation of the DETECT index.
The \code{ccov.np} function has the (default) option to smooth item response
functions (argument \code{smooth}) in the computation of conditional covariances
(Douglas, Kim, Habing, & Gao, 1998).
}
\usage{
ccov.np(data, score, bwscale=1.1, thetagrid=seq(-3, 3, len=200),
    progress=TRUE, scale_score=TRUE, adjust_thetagrid=TRUE, smooth=TRUE,
    use_sum_score=FALSE, bias_corr=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
An \eqn{N \times I} data frame of dichotomous responses.
Missing responses are allowed.
}
  \item{score}{
An ability estimate, e.g. the WLE
}
  \item{bwscale}{
Bandwidth factor for calculation of conditional covariance. The bandwidth
used in the estimation is \code{bwscale} times \eqn{N^{-1/5}}.
}
  \item{thetagrid}{
A vector which contains theta values where conditional
covariances are evaluated.
}
  \item{progress}{
Display progress?
}
\item{scale_score}{Logical indicating whether \code{score}
should be z standardized in advance of the calculation of
conditional covariances
}
\item{adjust_thetagrid}{Logical indicating whether \code{thetagrid} should be
adjusted if observed values in \code{score} are outside of \code{thetagrid}.
}
\item{smooth}{Logical indicating whether smoothing should be
applied for conditional covariance estimation}
\item{use_sum_score}{Logical indicating whether sum score should be used.
With this option, the bias corrected conditional covariance of Zhang and
Stout (1999) is used.}
\item{bias_corr}{Logical indicating whether bias correction (Zhang & Stout, 1999)
should be utilized if \code{use_sum_score=TRUE}.}
}

\references{
Douglas, J., Kim, H. R., Habing, B., & Gao, F. (1998). Investigating local dependence
with conditional covariance functions.
\emph{Journal of Educational and Behavioral Statistics, 23}(2), 129-151.
\doi{10.3102/10769986023002129}

Stout, W., Habing, B., Douglas, J., & Kim, H. R. (1996).
Conditional covariance-based nonparametric multidimensionality assessment.
\emph{Applied Psychological Measurement, 20}(4), 331-354.
\doi{10.1177/014662169602000403}

Zhang, J., & Stout, W. (1999). Conditional covariance structure
of generalized compensatory multidimensional items.
\emph{Psychometrika, 64}(2), 129-152.
\doi{10.1007/BF02294532}
}

%\author{
%Alexander Robitzsch
%}

\note{
This function is used in \code{\link{conf.detect}} and \code{\link{expl.detect}}.
}


\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: data.read | different settings for computing conditional covariance
#############################################################################

data(data.read, package="sirt")
dat <- data.read

#* fit Rasch model
mod <- sirt::rasch.mml2(dat)
score <- sirt::wle.rasch(dat=dat, b=mod$item$b)$theta

#* ccov with smoothing
cmod1 <- sirt::ccov.np(data=dat, score=score, bwscale=1.1)
#* ccov without smoothing
cmod2 <- sirt::ccov.np(data=dat, score=score, smooth=FALSE)

#- compare results
100*cbind( cmod1$ccov.table[1:6, "ccov"], cmod2$ccov.table[1:6, "ccov"])
}
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{DETECT}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: mcmc.2pnoh.Rd
%% File Version: 0.29

\name{mcmc.2pnoh}
\alias{mcmc.2pnoh}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
MCMC Estimation of the Hierarchical IRT Model for Criterion-Referenced
Measurement
}
\description{
This function estimates the hierarchical IRT model for criterion-referenced
measurement which is based on a two-parameter normal ogive response
function (Janssen, Tuerlinckx, Meulders & de Boeck, 2000).
}
\usage{
mcmc.2pnoh(dat, itemgroups, prob.mastery=c(.5,.8), weights=NULL,
      burnin=500, iter=1000, N.sampvalues=1000,
      progress.iter=50, prior.variance=c(1,1), save.theta=FALSE)
}

\arguments{
  \item{dat}{
Data frame with dichotomous item responses
}
\item{itemgroups}{Vector with characters or integers which define the
    criterion to which an item is associated.}
\item{prob.mastery}{Probability levels which define nonmastery, transition
    and mastery stage (see Details) }
\item{weights}{
    An optional vector with student sample weights
        }
  \item{burnin}{
Number of burnin iterations
}
  \item{iter}{
Total number of iterations
}
  \item{N.sampvalues}{
Maximum number of sampled values to save
}
  \item{progress.iter}{
Display progress every \code{progress.iter}-th iteration. If no progress
display is wanted, then choose \code{progress.iter} larger than \code{iter}.
}
\item{prior.variance}{Scale parameter of the inverse gamma distribution
    for the \eqn{\sigma^2} and \eqn{\nu^2} item variance parameters}
  \item{save.theta}{
Should theta values be saved?
}
}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DETAILS
\details{
The hierarchical IRT model for criterion-referenced measurement
(Janssen et al., 2000) assumes that every item \eqn{i} intends
to measure a criterion \eqn{k}. The item response function is defined as
\deqn{ P(X_{pik}=1 | \theta_p )=
        \Phi [ \alpha_{ik} ( \theta_p - \beta_{ik} ) ]
        \quad,  \quad \theta_p \sim N(0,1) }
Item parameters \eqn{(\alpha_{ik},\beta_{ik})} are hierarchically modeled, i.e.
\deqn{ \beta_{ik} \sim N( \xi_k, \sigma^2 ) \quad \mbox{and} \quad
    \alpha_{ik} \sim N( \omega_k, \nu^2 ) }
In the \code{mcmc.list} output object, also the derived parameters
\eqn{d_{ik}=\alpha_{ik} \beta_{ik}} and \eqn{\tau_k=\xi_k \omega_k} are
calculated.
Mastery and nonmastery probabilities are based on a reference item \eqn{Y_{k}}
of criterion \eqn{k} and a response function
\deqn{ P(Y_{pk}=1 | \theta_p )=
        \Phi [ \omega_{k} ( \theta_p - \xi_{k} ) ]
        \quad,  \quad \theta_p \sim N(0,1) }
With known item parameters and person parameters, response probabilities of
criterion \eqn{k} are calculated. If a response probability of criterion \eqn{k}
is larger than \code{prob.mastery[2]}, then a student is defined as a
master. If this probability is smaller than \code{prob.mastery[1]}, then
a student is a nonmaster. In all other cases, students are in a transition
stage.

In the \code{mcmcobj} output object, the parameters \code{d[i]} are defined by
\eqn{d_{ik}=\alpha_{ik} \cdot \beta_{ik}} while \code{tau[k]} are defined by
\eqn{ \tau_k=\xi_k \cdot \omega_k }.
}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VALUE

\value{
A list of class \code{mcmc.sirt} with following entries:
  \item{mcmcobj}{Object of class \code{mcmc.list}}
  \item{summary.mcmcobj}{Summary of the \code{mcmcobj} object. In this
  summary the Rhat statistic and the mode estimate MAP is included.
  The variable \code{PercSEratio} indicates the proportion of the Monte Carlo
  standard error in relation to the total standard deviation of the
  posterior distribution.}
  \item{burnin}{Number of burnin iterations}
  \item{iter}{Total number of iterations}
  \item{alpha.chain}{Sampled values of \eqn{\alpha_{ik}} parameters}
  \item{beta.chain}{Sampled values of \eqn{\beta_{ik}} parameters}
  \item{xi.chain}{Sampled values of \eqn{\xi_{k}} parameters}
  \item{omega.chain}{Sampled values of \eqn{\omega_{k}} parameters}
  \item{sigma.chain}{Sampled values of \eqn{\sigma} parameter}
  \item{nu.chain}{Sampled values of \eqn{\nu} parameter}
  \item{theta.chain}{Sampled values of \eqn{\theta_p} parameters}
  \item{deviance.chain}{Sampled values of Deviance values}
  \item{EAP.rel}{EAP reliability}
  \item{person}{Data frame with EAP person parameter estimates for
        \eqn{\theta_p} and their corresponding posterior standard
      deviations}
  \item{dat}{Used data frame}
  \item{weights}{Used student weights}
  \item{\dots}{Further values}
}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% REFERENCES
\references{
Janssen, R., Tuerlinckx, F., Meulders, M., & De Boeck, P. (2000).
A hierarchical IRT model for criterion-referenced measurement.
\emph{Journal of Educational and Behavioral Statistics, 25}, 285-306.
}

%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
S3 methods: \code{\link{summary.mcmc.sirt}}, \code{\link{plot.mcmc.sirt}}

The two-parameter normal ogive model can be estimated with
\code{\link{mcmc.2pno}}.
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EXAMPLES

\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Simulated data according to Janssen et al. (2000, Table 2)
#############################################################################

N <- 1000
Ik <- c(4,6,8,5,9,6,8,6,5)
xi.k <- c( -.89, -1.13, -1.23, .06, -1.41, -.66, -1.09, .57, -2.44)
omega.k <- c(.98, .91, .76, .74, .71, .80, .79, .82, .54)

# select 4 attributes
K <- 4
Ik <- Ik[1:K] ; xi.k <- xi.k[1:K] ; omega.k <- omega.k[1:K]
sig2 <- 3.02
nu2 <- .09
I <- sum(Ik)
b <- rep( xi.k, Ik ) + stats::rnorm(I, sd=sqrt(sig2) )
a <- rep( omega.k, Ik ) + stats::rnorm(I, sd=sqrt(nu2) )
theta1 <- stats::rnorm(N)
t1 <- rep(1,N)
p1 <- stats::pnorm( outer(t1,a) * ( theta1 - outer(t1,b) ) )
dat <- 1  * ( p1 > stats::runif(N*I)  )
itemgroups <- rep( paste0("A", 1:K ), Ik )

# estimate model
mod <- sirt::mcmc.2pnoh(dat, itemgroups, burnin=200, iter=1000 )
# summary
summary(mod)
# plot
plot(mod$mcmcobj, ask=TRUE)
# write coda files
mcmclist2coda( mod$mcmcobj, name="simul_2pnoh" )
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Markov Chain Monte Carlo (MCMC)}
% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.wide2long.Rd
%% File Version: 0.24

\name{data.wide2long}
\alias{data.wide2long}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Converting a Data Frame from Wide Format in a Long Format
}
\description{
Converts a data frame in wide format into long format.
}
\usage{
data.wide2long(dat, id=NULL, X=NULL, Q=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
Data frame with item responses and a person identifier if
\code{id !=NULL}.
}
  \item{id}{
An optional string with the variable name of the person identifier.
}
  \item{X}{Data frame with person covariates for inclusion in the
        data frame of long format
}
  \item{Q}{Data frame with item predictors. Item labels must be included
        as a column named by \code{"item"}.
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
Data frame in long format
}
%\references{
%% ~put references to the literature/web site here ~
%}
%\author{
%Alexander Robitzsch
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: data.pisaRead
#############################################################################
miceadds::library_install("lme4")

data(data.pisaRead)
dat <- data.pisaRead$data
Q <- data.pisaRead$item   # item predictors

# define items
items <- colnames(dat)[ substring( colnames(dat), 1, 1 )=="R" ]
dat1 <- dat[, c( "idstud", items ) ]
# matrix with person predictors
X <- dat[, c("idschool", "hisei", "female", "migra") ]

# create dataset in long format
dat.long <- sirt::data.wide2long( dat=dat1, id="idstud", X=X, Q=Q )

#***
# Model 1: Rasch model
mod1 <- lme4::glmer( resp ~ 0 + ( 1 | idstud ) + as.factor(item), data=dat.long,
            family="binomial", verbose=TRUE)
summary(mod1)

#***
# Model 2: Rasch model and inclusion of person predictors
mod2 <- lme4::glmer( resp ~ 0 + ( 1 | idstud ) + as.factor(item) + female + hisei + migra,
           data=dat.long, family="binomial", verbose=TRUE)
summary(mod2)

#***
# Model 3: LLTM
mod3 <- lme4::glmer(resp ~ (1|idstud) + as.factor(ItemFormat) + as.factor(TextType),
            data=dat.long, family="binomial", verbose=TRUE)
summary(mod3)

#############################################################################
# EXAMPLE 2: Rasch model in lme4
#############################################################################

set.seed(765)
N <- 1000  # number of persons
I <- 10    # number of items
b <- seq(-2,2,length=I)
dat <- sirt::sim.raschtype( stats::rnorm(N,sd=1.2), b=b )
dat.long <- sirt::data.wide2long( dat=dat )
#***
# estimate Rasch model with lmer
library(lme4)
mod1 <- lme4::glmer( resp ~ 0 + as.factor( item ) + ( 1 | id_index), data=dat.long,
             verbose=TRUE, family="binomial")
summary(mod1)
  ##   Random effects:
  ##    Groups   Name        Variance Std.Dev.
  ##    id_index (Intercept) 1.454    1.206
  ##   Number of obs: 10000, groups: id_index, 1000
  ##
  ##   Fixed effects:
  ##                        Estimate Std. Error z value Pr(>|z|)
  ##   as.factor(item)I0001  2.16365    0.10541  20.527  < 2e-16 ***
  ##   as.factor(item)I0002  1.66437    0.09400  17.706  < 2e-16 ***
  ##   as.factor(item)I0003  1.21816    0.08700  14.002  < 2e-16 ***
  ##   as.factor(item)I0004  0.68611    0.08184   8.383  < 2e-16 ***
  ##   [...]
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{R utilities}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: tetrachoric2.Rd
%% File Version: 1.364

\name{tetrachoric2}
\alias{tetrachoric2}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Tetrachoric Correlation Matrix
}
\description{
This function estimates a tetrachoric correlation matrix according to
the maximum likelihood estimation of Olsson (Olsson, 1979; \code{method="Ol"}),
the Tucker method (Method 2 of Froemel, 1971; \code{method="Tu"}) and
Divgi (1979, \code{method="Di"}).
In addition, an alternative non-iterative approximation of
Bonett and Price (2005; \code{method="Bo"}) is provided.
}
\usage{
tetrachoric2(dat, method="Ol", delta=0.007, maxit=1000000, cor.smooth=TRUE,
   progress=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
A data frame of dichotomous response
}
\item{method}{Computation method for calculating the tetrachoric correlation.
    The ML method is \code{method="Ol"} (which is the default),
    the Tucker method is \code{method="Tu"},
    the Divgi method is \code{method="Di"}
    the method of Bonett and
    Price (2005) is \code{method="Bo"}.}
  \item{delta}{
The step parameter. It is set by default to \eqn{2^{-7}}
which is approximately .007.
}
  \item{maxit}{
Maximum number of iterations.
}
\item{cor.smooth}{
Should smoothing of the tetrachoric correlation matrix
be performed to ensure positive definiteness? Choosing
\code{cor.smooth=TRUE}, the function \code{cor.smooth}
from the \pkg{psych} package is used for obtaining a positive definite
tetrachoric correlation matrix.
}
\item{progress}{Display progress? Default is \code{TRUE}.}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list with following entries
\item{tau}{Item thresholds}
\item{rho}{Tetrachoric correlation matrix}
}


\references{
Bonett, D. G., & Price, R. M. (2005). Inferential methods for the tetrachoric
correlation coefficient. \emph{Journal of Educational and Behavioral Statistics,
30}(2), 213-225. \doi{10.3102/10769986030002213}

Divgi, D. R. (1979). Calculation of the tetrachoric correlation coefficient.
\emph{Psychometrika, 44}(2), 169-172.
\doi{10.1007/BF02293968}

Froemel, E. C. (1971). A comparison of computer routines for the
calculation of the tetrachoric correlation coefficient.
\emph{Psychometrika, 36}(2), 165-174.
\doi{10.1007/BF02291396}

Olsson, U. (1979). Maximum likelihood estimation of the polychoric correlation
coefficient. \emph{Psychometrika, 44}(4), 443-460.
\doi{10.1007/BF02296207}
}


\author{
Alexander Robitzsch

The code is adapted from an \R script of Cengiz Zopluoglu.
See \emph{http://sites.education.miami.edu/zopluoglu/software-programs/}.
}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also the \code{psych::tetrachoric}
function in the \pkg{psych} package
 and the function \code{irtoys::tet} in the \pkg{irtoys} package.

See \code{\link{polychoric2}} for estimating polychoric correlations.
}
\examples{
#############################################################################
# EXAMPLE 1: data.read
#############################################################################

data(data.read)

# tetrachoric correlation from psych package
library(psych)
t0 <- psych::tetrachoric( data.read )$rho
# Olsson method (maximum likelihood estimation)
t1 <- sirt::tetrachoric2( data.read )$rho
# Divgi method
t2 <- sirt::tetrachoric2( data.read, method="Di"  )$rho
# Tucker method
t3 <- sirt::tetrachoric2( data.read, method="Tu" )$rho
# Bonett method
t4 <- sirt::tetrachoric2( data.read, method="Bo" )$rho

# maximum absolute deviation ML method
max( abs( t0 - t1 ) )
  ##   [1] 0.008224986
# mean absolute deviation Divgi method
max( abs( t0 - t2 ) )
  ##   [1] 0.1766688
# mean absolute deviation Tucker method
max( abs( t0 - t3 ) )
  ##   [1] 0.1766292
# mean absolute deviation Bonett method
max( abs( t0 - t4 ) )
  ##   [1] 0.05695522
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Tetrachoric correlation}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: rinvgamma2.Rd
%% File Version: 0.18

\name{rinvgamma2}
\alias{rinvgamma2}
\alias{dinvgamma2}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Inverse Gamma Distribution in Prior Sample Size Parameterization
}
\description{
Random draws and density of inverse gamma distribution parameterized
in prior sample size \code{n0} and prior variance \code{var0}
(see Gelman et al., 2014).
}
\usage{
rinvgamma2(n, n0, var0)

dinvgamma2(x, n0, var0)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{n}{
Number of draws for inverse gamma distribution
}
  \item{n0}{
Prior sample size
}
  \item{var0}{
Prior variance
}
\item{x}{Vector with numeric values for density evaluation}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A vector containing random draws or density values
}


%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...


\references{
Gelman, A., Carlin, J. B., Stern, H. S., Dunson, D. B., Vehtari, A.,
& Rubin, D. B. (2014).
\emph{Bayesian data analysis} (Vol. 3). Boca Raton, FL, USA: Chapman & Hall/CRC.
}


%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{MCMCpack::rinvgamma},
\code{\link[stats:GammaDist]{stats::rgamma}},
\code{MCMCpack::dinvgamma},
\code{\link[stats:GammaDist]{stats::dgamma}}
}
\examples{
#############################################################################
# EXAMPLE 1: Inverse gamma distribution
#############################################################################

# prior sample size of 100 and prior variance of 1.5
n0 <- 100
var0 <- 1.5

# 100 random draws
y1 <- sirt::rinvgamma2( n=100, n0, var0 )
summary(y1)
graphics::hist(y1)

# density y at grid x
x <- seq( 0, 2, len=100 )
y <- sirt::dinvgamma2( x, n0, var0 )
graphics::plot( x, y, type="l")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: polychoric2.Rd
%% File Version: 0.296

\name{polychoric2}
\alias{polychoric2}
\alias{sirt_rcpp_polychoric2}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Polychoric Correlation
}
\description{
This function estimates the polychoric correlation coefficient
using maximum likelihood estimation (Olsson, 1979).
}
\usage{
polychoric2(dat, maxiter=100, cor.smooth=TRUE, use_pbv=1, conv=1e-10,
      rho_init=NULL, weights=NULL)

## exported Rcpp function
sirt_rcpp_polychoric2( dat, maxK, maxiter, use_pbv, conv, rho_init, weights)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
A dataset with integer values \eqn{0,1,\ldots,K}
}
  \item{maxiter}{
Maximum number of iterations
}
  \item{cor.smooth}{
An optional logical indicating whether the polychoric correlation
matrix should be smooth to ensure positive definiteness.
}
\item{use_pbv}{Integer indicating whether the \pkg{pbv} package is used
for computation of bivariate normal distribution. \code{0} stands for
the simplest approximation in \pkg{sirt} (Cox & Wermuth, 1991, as
implemented in \code{\link{polychoric2}})
while versions \code{1} and
\code{2} uses the algorithm of \pkg{pbv} (the first one copied into
the \pkg{sirt} package, the second one linking Rcpp code to \pkg{pbv}.)}
\item{conv}{Convergence criterion}
\item{rho_init}{Optional matrix of initial values for polychoric correlations}
\item{weights}{Optional vector of sampling weights}
\item{maxK}{Maximum number of categories}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

\value{
A list with following entries
\item{tau}{Matrix of thresholds}
\item{rho}{Polychoric correlation matrix}
\item{Nobs}{Sample size for every item pair}
\item{maxcat}{Maximum number of categories per item}
}


\references{
Cox, D. R., & Wermuth, N. (1991). A simple approximation for bivariate and
trivariate normal integrals.
\emph{International Statistical Review, 59}(2), 263-269.

Olsson, U. (1979). Maximum likelihood estimation of the polychoric correlation
coefficient. \emph{Psychometrika, 44}(4), 443-460.
\doi{10.1007/BF02296207}
}
% Cox: \doi{10.2307/1403446}

%\author{
%Alexander Robitzsch
%}



%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See the \code{\link[psych:polychoric]{psych::polychoric}}
function in the \pkg{psych} package.

For estimating tetrachoric correlations see \code{\link{tetrachoric2}}.
}


\examples{
#############################################################################
# EXAMPLE 1: data.Students | activity scale
#############################################################################

data(data.Students, package="CDM")
dat <- data.Students[, paste0("act", 1:5 ) ]

# tetrachoric correlation from psych package
library(psych)
t0 <- psych::polychoric(dat)$rho
# Olsson method (maximum likelihood estimation)
t1 <- sirt::polychoric2(dat)$rho
# maximum absolute difference
max( abs( t0 - t1 ) )
  ##   [1] 0.004102429
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Polychoric correlation}
%% \keyword{Tetrachoric correlation}

%% File Name: prob.guttman.Rd
%% File Version: 1.33

\name{prob.guttman}
\alias{prob.guttman}
\alias{summary.prob.guttman}
\alias{logLik.prob.guttman}
\alias{anova.prob.guttman}
\alias{IRT.likelihood.prob.guttman}
\alias{IRT.posterior.prob.guttman}
\alias{IRT.irfprob.prob.guttman}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Probabilistic Guttman Model
}
\description{
This function estimates the probabilistic Guttman model which
is a special case of an ordered latent trait model (Hanson, 2000;
Proctor, 1970).
}
\usage{
prob.guttman(dat, pid=NULL, guess.equal=FALSE,  slip.equal=FALSE,
    itemlevel=NULL, conv1=0.001, glob.conv=0.001, mmliter=500)

\method{summary}{prob.guttman}(object,...)

\method{anova}{prob.guttman}(object,...)

\method{logLik}{prob.guttman}(object,...)

\method{IRT.irfprob}{prob.guttman}(object,...)

\method{IRT.likelihood}{prob.guttman}(object,...)

\method{IRT.posterior}{prob.guttman}(object,...)
}



%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dat}{
An \eqn{N \times I} data frame of dichotomous item responses
}
  \item{pid}{
Optional vector of person identifiers
}
  \item{guess.equal}{
Should the same guessing parameters for all the items estimated?
}
  \item{slip.equal}{
Should the same slipping parameters for all the items estimated?
}
  \item{itemlevel}{
A vector of item levels of the Guttman scale for each item. If there
are \eqn{K} different item levels, then the Guttman scale possesses
\eqn{K} ordered trait values.
}
  \item{conv1}{
Convergence criterion for item parameters
}
  \item{glob.conv}{
Global convergence criterion for the deviance
}
  \item{mmliter}{
Maximum number of iterations
}
  \item{object}{
Object of class \code{prob.guttman}
}
  \item{\dots}{
Further arguments to be passed
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
An object of class \code{prob.guttman}
\item{person}{Estimated person parameters}
\item{item}{Estimated item parameters}
\item{theta.k}{Ability levels}
\item{trait}{Estimated trait distribution}
\item{ic}{Information criteria}
\item{deviance}{Deviance}
\item{iter}{Number of iterations}
\item{itemdesign}{Specified allocation of items to trait levels}
}
\references{
Hanson, B. (2000). \emph{IRT parameter estimation using the EM algorithm}.
Technical Report.

Proctor, C. H. (1970). A probabilistic formulation and statistical analysis
for Guttman scaling. \emph{Psychometrika, 35}, 73-78.
}

%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%A summary can be obtained using \code{\link{summary.prob.guttman}}.
%}


\examples{
#############################################################################
# EXAMPLE 1: Dataset Reading
#############################################################################
data(data.read)
dat <- data.read

#***
# Model 1: estimate probabilistic Guttman model
mod1 <- sirt::prob.guttman( dat )
summary(mod1)

#***
# Model 2: probabilistic Guttman model with equal guessing and slipping parameters
mod2 <- sirt::prob.guttman( dat, guess.equal=TRUE, slip.equal=TRUE)
summary(mod2)

#***
# Model 3: Guttman model with three a priori specified item levels
itemlevel <- rep(1,12)
itemlevel[ c(2,5,8,10,12) ] <- 2
itemlevel[ c(3,4,6) ] <- 3
mod3 <- sirt::prob.guttman( dat, itemlevel=itemlevel )
summary(mod3)

\dontrun{
#***
# Model3m: estimate Model 3 in mirt

library(mirt)
# define four ordered latent classes
Theta <- scan(nlines=1)
   0 0 0    1 0 0   1 1 0   1 1 1
Theta <- matrix( Theta, nrow=4, ncol=3,byrow=TRUE)

# define mirt model
I <- ncol(dat)  # I=12
mirtmodel <- mirt::mirt.model("
        # specify factors for each item level
        C1=1,7,9,11
        C2=2,5,8,10,12
        C3=3,4,6
        ")
# get initial parameter values
mod.pars <- mirt::mirt(dat, model=mirtmodel,  pars="values")
# redefine initial parameter values
mod.pars[ mod.pars$name=="d","value" ]  <- -1
mod.pars[ mod.pars$name \%in\% paste0("a",1:3) & mod.pars$est,"value" ]  <- 2
mod.pars
# define prior for latent class analysis
lca_prior <- function(Theta,Etable){
  # number of latent Theta classes
  TP <- nrow(Theta)
  # prior in initial iteration
  if ( is.null(Etable) ){ prior <- rep( 1/TP, TP ) }
  # process Etable (this is correct for datasets without missing data)
  if ( ! is.null(Etable) ){
    # sum over correct and incorrect expected responses
    prior <- ( rowSums(Etable[, seq(1,2*I,2)]) + rowSums(Etable[,seq(2,2*I,2)]) )/I
                 }
  prior <- prior / sum(prior)
  return(prior)
}
# estimate model in mirt
mod3m <- mirt::mirt(dat, mirtmodel, pars=mod.pars, verbose=TRUE,
            technical=list( customTheta=Theta, customPriorFun=lca_prior) )
# correct number of estimated parameters
mod3m@nest <- as.integer(sum(mod.pars$est) + nrow(Theta)-1 )
# extract log-likelihood and compute AIC and BIC
mod3m@logLik
( AIC <- -2*mod3m@logLik+2*mod3m@nest )
( BIC <- -2*mod3m@logLik+log(mod3m@Data$N)*mod3m@nest )
# compare with information criteria from prob.guttman
mod3$ic
# model fit in mirt
mirt::M2(mod3m)
# extract coefficients
( cmod3m <- sirt::mirt.wrapper.coef(mod3m) )
# compare estimated distributions
round( cbind( "sirt"=mod3$trait$prob, "mirt"=mod3m@Prior[[1]] ), 5 )
  ##           sirt    mirt
  ##   [1,] 0.13709 0.13765
  ##   [2,] 0.30266 0.30303
  ##   [3,] 0.15239 0.15085
  ##   [4,] 0.40786 0.40846
# compare estimated item parameters
ipars <- data.frame( "guess.sirt"=mod3$item$guess,
                     "guess.mirt"=plogis( cmod3m$coef$d ) )
ipars$slip.sirt <- mod3$item$slip
ipars$slip.mirt <- 1-plogis( rowSums(cmod3m$coef[, c("a1","a2","a3","d") ] ) )
round( ipars, 4 )
  ##      guess.sirt guess.mirt slip.sirt slip.mirt
  ##   1      0.7810     0.7804    0.1383    0.1382
  ##   2      0.4513     0.4517    0.0373    0.0368
  ##   3      0.3203     0.3200    0.0747    0.0751
  ##   4      0.3009     0.3007    0.3082    0.3087
  ##   5      0.5776     0.5779    0.1800    0.1798
  ##   6      0.3758     0.3759    0.3047    0.3051
  ##   7      0.7262     0.7259    0.0625    0.0623
  ##   [...]

#***
# Model 4: Monotone item response function estimated in mirt

# define four ordered latent classes
Theta <- scan(nlines=1)
   0 0 0    1 0 0   1 1 0   1 1 1
Theta <- matrix( Theta, nrow=4, ncol=3,byrow=TRUE)

# define mirt model
I <- ncol(dat)  # I=12
mirtmodel <- mirt::mirt.model("
        # specify factors for each item level
        C1=1-12
        C2=1-12
        C3=1-12
        ")
# get initial parameter values
mod.pars <- mirt::mirt(dat, model=mirtmodel,  pars="values")
# redefine initial parameter values
mod.pars[ mod.pars$name=="d","value" ]  <- -1
mod.pars[ mod.pars$name \%in\% paste0("a",1:3) & mod.pars$est,"value" ]  <- .6
# set lower bound to zero ton ensure monotonicity
mod.pars[ mod.pars$name \%in\% paste0("a",1:3),"lbound" ]  <- 0
mod.pars
# estimate model in mirt
mod4 <- mirt::mirt(dat, mirtmodel, pars=mod.pars, verbose=TRUE,
            technical=list( customTheta=Theta, customPriorFun=lca_prior) )
# correct number of estimated parameters
mod4@nest <- as.integer(sum(mod.pars$est) + nrow(Theta)-1 )
# extract coefficients
cmod4 <- sirt::mirt.wrapper.coef(mod4)
cmod4
# compute item response functions
cmod4c <- cmod4$coef[, c("d", "a1", "a2", "a3" ) ]
probs4 <- t( apply( cmod4c, 1, FUN=function(ll){
                 plogis(cumsum(as.numeric(ll))) } ) )
matplot( 1:4,  t(probs4), type="b", pch=1:I)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Probabilistic Guttman model}
%% \keyword{summary}
%% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: likelihood.adjustment.Rd
%% File Version: 0.13

\name{likelihood.adjustment}
\alias{likelihood.adjustment}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Adjustment and Approximation of Individual Likelihood Functions
}

\description{
Approximates individual likelihood functions \eqn{L(\bold{X}_p | \theta)}
by normal distributions (see Mislevy, 1990). Extreme response patterns
are handled by adding pseudo-observations of items with extreme item
difficulties (see argument \code{extreme.item}. The individual standard
deviations of the likelihood, used in the normal approximation, can be
modified by individual adjustment factors which are specified in \code{adjfac}.
In addition, a reliability of the adjusted likelihood can be specified
in \code{target.EAP.rel}.
}

\usage{
likelihood.adjustment(likelihood, theta=NULL, prob.theta=NULL,
     adjfac=rep(1, nrow(likelihood)), extreme.item=5, target.EAP.rel=NULL,
     min_tuning=0.2, max_tuning=3, maxiter=100, conv=1e-04,
     trait.normal=TRUE)
}


%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{likelihood}{
A matrix containing the individual likelihood \eqn{L(\bold{X}_p | \theta)} or
an object of class \code{IRT.likelihood}.
}
  \item{theta}{
Optional vector of (unidimensional) \eqn{\theta} values
}
  \item{prob.theta}{
Optional vector of probabilities of \eqn{\theta} trait distribution
}
  \item{adjfac}{
Vector with individual adjustment factors of the standard deviations of the
likelihood
}
  \item{extreme.item}{
Item difficulties of two extreme pseudo items which are added as additional
observed data to the likelihood. A large number (e.g. \code{extreme.item=15})
leaves the likelihood almost unaffected. See also Mislevy (1990).
}
  \item{target.EAP.rel}{
Target EAP reliability. An additional tuning parameter is estimated
which adjusts the likelihood to obtain a pre-specified reliability.
}
  \item{min_tuning}{
Minimum value of tuning parameter (if \code{! is.null(target.EAP.rel) })
}
  \item{max_tuning}{
Maximum value of tuning parameter (if \code{! is.null(target.EAP.rel) })
}
  \item{maxiter}{
Maximum number of iterations (if \code{! is.null(target.EAP.rel) })
}
  \item{conv}{
Convergence criterion (if \code{! is.null(target.EAP.rel) })
}
  \item{trait.normal}{
Optional logical indicating whether the trait distribution should be
normally distributed (if \code{! is.null(target.EAP.rel) }).
}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

\value{
Object of class \code{IRT.likelihood}.
}

\references{
Mislevy, R. (1990). Scaling procedures. In E. Johnson & R. Zwick (Eds.),
\emph{Focusing the new design: The NAEP 1988 technical report} (ETS RR 19-20).
Princeton, NJ: Educational Testing Service.
}



%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link[CDM:IRT.likelihood]{CDM::IRT.likelihood}},
\code{\link[TAM:tam.latreg]{TAM::tam.latreg}}
}

\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: Adjustment of the likelihood | data.read
#############################################################################

library(CDM)
library(TAM)
data(data.read)
dat <- data.read

# define theta grid
theta.k <- seq(-6,6,len=41)

#*** Model 1: fit Rasch model in TAM
mod1 <- TAM::tam.mml( dat, control=list( nodes=theta.k) )
summary(mod1)

#*** Model 2: fit Rasch copula model
testlets <- substring( colnames(dat), 1, 1 )
mod2 <- sirt::rasch.copula2( dat, itemcluster=testlets, theta.k=theta.k)
summary(mod2)

# model comparison
IRT.compareModels( mod1, mod2 )

# extract EAP reliabilities
rel1 <- mod1$EAP.rel
rel2 <- mod2$EAP.Rel
# variance inflation factor
vif <- (1-rel2) / (1-rel1)
  ##  > vif
  ##  [1] 1.211644

# extract individual likelihood
like1 <- IRT.likelihood( mod1 )
# adjust likelihood from Model 1 to obtain a target EAP reliability of .599
like1b <- sirt::likelihood.adjustment( like1, target.EAP.rel=.599 )

# compare estimated latent regressions
lmod1a <- TAM::tam.latreg( like1, Y=NULL )
lmod1b <- TAM::tam.latreg( like1b, Y=NULL )
summary(lmod1a)
summary(lmod1b)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Likelihood adjustment}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.eid.Rd
%% File Version: 0.30

\name{data.eid}
\alias{data.eid}
\alias{data.eid.kap4}
\alias{data.eid.kap5}
\alias{data.eid.kap6}
\alias{data.eid.kap7}

\docType{data}


\title{
Examples with Datasets from Eid and Schmidt (2014)
}

\description{
Examples with datasets from Eid and Schmidt (2014),
illustrations with several \R packages. The examples
follow closely the online material of Hosoya (2014).
The datasets are completely synthetic datasets which were
resimulated from the originally available data.
}

\usage{
data(data.eid.kap4)
data(data.eid.kap5)
data(data.eid.kap6)
data(data.eid.kap7)
}

\format{
\itemize{
\item  \code{data.eid.kap4} is the dataset from Chapter 4.

\code{'data.frame':   193 obs. of  11 variables:} \cr
\code{ $ sex     : int  0 0 0 0 0 0 1 0 0 1 ...} \cr
\code{ $ Freude_1: int  1 1 1 0 1 1 1 1 1 1 ...} \cr
\code{ $ Wut_1   : int  1 1 1 0 1 1 1 1 1 1 ...} \cr
\code{ $ Angst_1 : int  1 0 0 0 1 1 1 0 1 0 ...} \cr
\code{ $ Trauer_1: int  1 1 1 0 1 1 1 1 1 1 ...} \cr
\code{ $ Ueber_1 : int  1 1 1 0 1 1 0 1 1 1 ...} \cr
\code{ $ Trauer_2: int  0 1 1 1 1 1 1 1 1 0 ...} \cr
\code{ $ Angst_2 : int  0 0 1 0 0 1 0 0 0 0 ...} \cr
\code{ $ Wut_2   : int  1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{ $ Ueber_2 : int  1 0 1 0 1 1 1 0 1 1 ...} \cr
\code{ $ Freude_2: int  1 1 1 0 1 1 1 1 1 1 ...} \cr

\item \code{data.eid.kap5} is the dataset from Chapter 5.

\code{'data.frame':   499 obs. of  7 variables:} \cr
\code{ $ sex   : int  0 0 0 0 1 1 1 0 0 0 ...} \cr
\code{ $ item_1: int  2 3 3 2 4 1 0 0 0 2 ...} \cr
\code{ $ item_2: int  1 1 4 1 3 3 2 1 2 3 ...} \cr
\code{ $ item_3: int  1 3 3 2 3 3 0 0 0 1 ...} \cr
\code{ $ item_4: int  2 4 3 4 3 3 3 2 0 2 ...} \cr
\code{ $ item_5: int  1 3 2 2 0 0 0 0 1 2 ...} \cr
\code{ $ item_6: int  4 3 4 3 4 3 2 1 1 3 ...} \cr


\item \code{data.eid.kap6} is the dataset from Chapter 6.

\code{'data.frame':   238 obs. of  7 variables:} \cr
\code{ $ geschl: int  1 1 0 0 0 1 0 1 1 0 ...} \cr
\code{ $ item_1: int  3 3 3 3 2 0 1 4 3 3 ...} \cr
\code{ $ item_2: int  2 2 2 2 2 0 2 3 1 3 ...} \cr
\code{ $ item_3: int  2 2 1 3 2 0 0 3 1 3 ...} \cr
\code{ $ item_4: int  2 3 3 3 3 0 2 4 3 4 ...} \cr
\code{ $ item_5: int  1 2 1 2 2 0 1 2 2 2 ...} \cr
\code{ $ item_6: int  2 2 2 2 2 0 1 2 1 2 ...} \cr


\item \code{data.eid.kap7} is the dataset \emph{Emotionale Klarheit} from Chapter 7.

\code{'data.frame':   238 obs. of  9 variables:} \cr
\code{ $ geschl : int  1 0 1 1 0 1 0 1 0 1 ...} \cr
\code{ $ reakt_1: num  2.13 1.78 1.28 1.82 1.9 1.63 1.73 1.49 1.43 1.27 ...} \cr
\code{ $ reakt_2: num  1.2 1.73 0.95 1.5 1.99 1.75 1.58 1.71 1.41 0.96 ...} \cr
\code{ $ reakt_3: num  1.77 1.42 0.76 1.54 2.36 1.84 2.06 1.21 1.75 0.92 ...} \cr
\code{ $ reakt_4: num  2.18 1.28 1.39 1.82 2.09 2.15 2.1 1.13 1.71 0.78 ...} \cr
\code{ $ reakt_5: num  1.47 1.7 1.08 1.77 1.49 1.73 1.96 1.76 1.88 1.1 ...} \cr
\code{ $ reakt_6: num  1.63 0.9 0.82 1.63 1.79 1.37 1.79 1.11 1.27 1.06 ...} \cr
\code{ $ kla_th1: int  8 11 11 8 10 11 12 5 6 12 ...} \cr
\code{ $ kla_th2: int  7 11 12 8 10 11 12 5 8 11 ...} \cr

}


}

%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}

\source{
The material and original datasets can be downloaded from
\emph{http://www.hogrefe.de/buecher/lehrbuecher/psychlehrbuchplus/lehrbuecher/
testtheorie-und-testkonstruktion/zusatzmaterial/}.
}

\references{
Eid, M., & Schmidt, K. (2014). \emph{Testtheorie und Testkonstruktion}.
Goettingen, Hogrefe.

Hosoya, G. (2014). \emph{Einfuehrung in die Analyse testtheoretischer
Modelle mit \R}. Available at
\emph{http://www.hogrefe.de/buecher/lehrbuecher/psychlehrbuchplus/lehrbuecher/testtheorie-und-testkonstruktion/zusatzmaterial/}.
}


\examples{
\dontrun{
miceadds::library_install("foreign")
#---- load some IRT packages in R
miceadds::library_install("TAM")        # package (a)
miceadds::library_install("mirt")       # package (b)
miceadds::library_install("sirt")       # package (c)
miceadds::library_install("eRm")        # package (d)
miceadds::library_install("ltm")        # package (e)
miceadds::library_install("psychomix")  # package (f)

#############################################################################
# EXAMPLES Ch. 4: Unidimensional IRT models | dichotomous data
#############################################################################

data(data.eid.kap4)
data0 <- data.eid.kap4

# load data
data0 <- foreign::read.spss( linkname, to.data.frame=TRUE, use.value.labels=FALSE)
# extract items
dat <- data0[,2:11]

#*********************************************************
# Model 1: Rasch model
#*********************************************************

#-----------
#-- 1a: estimation with TAM package

# estimation with tam.mml
mod1a <- TAM::tam.mml(dat)
summary(mod1a)

# person parameters in TAM
pp1a <- TAM::tam.wle(mod1a)

# plot item response functions
plot(mod1a,export=FALSE,ask=TRUE)

# Infit and outfit in TAM
itemf1a <- TAM::tam.fit(mod1a)
itemf1a

# model fit
modf1a <- TAM::tam.modelfit(mod1a)
summary(modf1a)

#-----------
#-- 1b: estimation with mirt package

# estimation with mirt
mod1b <- mirt::mirt( dat, 1, itemtype="Rasch")
summary(mod1b)
print(mod1b)

# person parameters
pp1b <- mirt::fscores(mod1b, method="WLE")

# extract coefficients
sirt::mirt.wrapper.coef(mod1b)

# plot item response functions
plot(mod1b, type="trace" )
par(mfrow=c(1,1))

# item fit
itemf1b <- mirt::itemfit(mod1b)
itemf1b

# model fit
modf1b <- mirt::M2(mod1b)
modf1b

#-----------
#-- 1c: estimation with sirt package

# estimation with rasch.mml2
mod1c <- sirt::rasch.mml2(dat)
summary(mod1c)

# person parameters (EAP)
pp1c <- mod1c$person

# plot item response functions
plot(mod1c, ask=TRUE )

# model fit
modf1c <- sirt::modelfit.sirt(mod1c)
summary(modf1c)

#-----------
#-- 1d: estimation with eRm package

# estimation with RM
mod1d <- eRm::RM(dat)
summary(mod1d)

# estimation person parameters
pp1d <- eRm::person.parameter(mod1d)
summary(pp1d)

# plot item response functions
eRm::plotICC(mod1d)

# person-item map
eRm::plotPImap(mod1d)

# item fit
itemf1d <- eRm::itemfit(pp1d)

# person fit
persf1d <- eRm::personfit(pp1d)

#-----------
#-- 1e: estimation with ltm package

# estimation with rasch
mod1e <- ltm::rasch(dat)
summary(mod1e)

# estimation person parameters
pp1e <- ltm::factor.scores(mod1e)

# plot item response functions
plot(mod1e)

# item fit
itemf1e <- ltm::item.fit(mod1e)

# person fit
persf1e <- ltm::person.fit(mod1e)

# goodness of fit with Bootstrap
modf1e <- ltm::GoF.rasch(mod1e,B=20)    # use more bootstrap samples
modf1e

#*********************************************************
# Model 2: 2PL model
#*********************************************************

#-----------
#-- 2a: estimation with TAM package

# estimation
mod2a <- TAM::tam.mml.2pl(dat)
summary(mod2a)

# model fit
modf2a <- TAM::tam.modelfit(mod2a)
summary(modf2a)

# item response functions
plot(mod2a, export=FALSE, ask=TRUE)

# model comparison
anova(mod1a,mod2a)

#-----------
#-- 2b: estimation with mirt package

# estimation
mod2b <- mirt::mirt(dat,1,itemtype="2PL")
summary(mod2b)
print(mod2b)
sirt::mirt.wrapper.coef(mod2b)

# model fit
modf2b <- mirt::M2(mod2b)
modf2b

#-----------
#-- 2c: estimation with sirt package

I <- ncol(dat)
# estimation
mod2c <- sirt::rasch.mml2(dat,est.a=1:I)
summary(mod2c)

# model fit
modf2c <- sirt::modelfit.sirt(mod2c)
summary(modf2c)

#-----------
#-- 2e: estimation with ltm package

# estimation
mod2e <- ltm::ltm(dat ~ z1 )
summary(mod2e)

# item response functions
plot(mod2e)

#*********************************************************
# Model 3: Mixture Rasch model
#*********************************************************

#-----------
#-- 3a: estimation with TAM package

# avoid "_" in column names if the "__" operator is used in
# the tamaan syntax
dat1 <- dat
colnames(dat1) <- gsub("_", "", colnames(dat1) )
# define tamaan model
tammodel <- "
ANALYSIS:
  TYPE=MIXTURE ;
  NCLASSES(2);
  NSTARTS(20,25);   # 20 random starts with 25 initial iterations each
LAVAAN MODEL:
  F=~ Freude1__Freude2
  F ~~ F
ITEM TYPE:
  ALL(Rasch);
    "
mod3a <- TAM::tamaan( tammodel, resp=dat1 )
summary(mod3a)
# extract item parameters
ipars <- mod2$itempartable_MIXTURE[ 1:10, ]
plot( 1:10, ipars[,3], type="o", ylim=range( ipars[,3:4] ), pch=16,
        xlab="Item", ylab="Item difficulty")
lines( 1:10, ipars[,4], type="l", col=2, lty=2)
points( 1:10, ipars[,4],  col=2, pch=2)

#-----------
#-- 3f: estimation with psychomix package

# estimation
mod3f <- psychomix::raschmix( as.matrix(dat), k=2, scores="meanvar")
summary(mod3f)
# plot class-specific item difficulties
plot(mod3f)

#############################################################################
# EXAMPLES Ch. 5: Unidimensional IRT models | polytomous data
#############################################################################

data(data.eid.kap5)
data0 <- data.eid.kap5
# extract items
dat <- data0[,2:7]

#*********************************************************
# Model 1: Partial credit model
#*********************************************************

#-----------
#-- 1a: estimation with TAM package

# estimation with tam.mml
mod1a <- TAM::tam.mml(dat)
summary(mod1a)

# person parameters in TAM
pp1a <- tam.wle(mod1a)

# plot item response functions
plot(mod1a,export=FALSE,ask=TRUE)

# Infit and outfit in TAM
itemf1a <- TAM::tam.fit(mod1a)
itemf1a

# model fit
modf1a <- TAM::tam.modelfit(mod1a)
summary(modf1a)

#-----------
#-- 1b: estimation with mirt package

# estimation with tam.mml
mod1b <- mirt::mirt( dat, 1, itemtype="Rasch")
summary(mod1b)
print(mod1b)
sirt::mirt.wrapper.coef(mod1b)

# plot item response functions
plot(mod1b, type="trace" )
par(mfrow=c(1,1))

# item fit
itemf1b <- mirt::itemfit(mod1b)
itemf1b

#-----------
#-- 1c: estimation with sirt package

# estimation with rm.facets
mod1c <- sirt::rm.facets(dat)
summary(mod1c)
summary(mod1a)

#-----------
#-- 1d: estimation with eRm package

# estimation
mod1d <- eRm::PCM(dat)
summary(mod1d)

# plot item response functions
eRm::plotICC(mod1d)

# person-item map
eRm::plotPImap(mod1d)

# item fit
itemf1d <- eRm::itemfit(pp1d)

#-----------
#-- 1e: estimation with ltm package

# estimation
mod1e <- ltm::gpcm(dat, constraint="1PL")
summary(mod1e)
# plot item response functions
plot(mod1e)

#*********************************************************
# Model 2: Generalized partial credit model
#*********************************************************

#-----------
#-- 2a: estimation with TAM package

# estimation with tam.mml
mod2a <- TAM::tam.mml.2pl(dat, irtmodel="GPCM")
summary(mod2a)

# model fit
modf2a <- TAM::tam.modelfit(mod2a)
summary(modf2a)

#-----------
#-- 2b: estimation with mirt package

# estimation
mod2b <- mirt::mirt( dat, 1, itemtype="gpcm")
summary(mod2b)
print(mod2b)
sirt::mirt.wrapper.coef(mod2b)

#-----------
#-- 2c: estimation with sirt package

# estimation with rm.facets
mod2c <- sirt::rm.facets(dat, est.a.item=TRUE)
summary(mod2c)

#-----------
#-- 2e: estimation with ltm package

# estimation
mod2e <- ltm::gpcm(dat)
summary(mod2e)
plot(mod2e)
}
}


%% \keyword{datasets}

%% File Name: mirt.wrapper.Rd
%% File Version: 1.867

\name{mirt.wrapper}
\alias{mirt.wrapper}
\alias{mirt.wrapper.coef}
\alias{mirt.wrapper.posterior}
\alias{mirt.wrapper.fscores}
\alias{mirt.wrapper.itemplot}
\alias{IRT.irfprob.SingleGroupClass}
\alias{IRT.irfprob.MultipleGroupClass}
\alias{IRT.likelihood.SingleGroupClass}
\alias{IRT.posterior.SingleGroupClass}
\alias{IRT.likelihood.MultipleGroupClass}
\alias{IRT.posterior.MultipleGroupClass}
\alias{IRT.expectedCounts.SingleGroupClass}
\alias{IRT.expectedCounts.MultipleGroupClass}


\alias{mirt_summary}

% SingleGroupClass

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Some Functions for Wrapping with the \pkg{mirt} Package
}
\description{
Some functions for wrapping with the \pkg{mirt} package.
}
\usage{
# extract coefficients
mirt.wrapper.coef(mirt.obj)

# summary output
mirt_summary(object, digits=4, file=NULL, ...)

# extract posterior, likelihood, ...
mirt.wrapper.posterior(mirt.obj, weights=NULL, group=NULL)
\method{IRT.likelihood}{SingleGroupClass}(object, \dots)
\method{IRT.likelihood}{MultipleGroupClass}(object, \dots)
\method{IRT.posterior}{SingleGroupClass}(object, \dots)
\method{IRT.posterior}{MultipleGroupClass}(object, \dots)
\method{IRT.expectedCounts}{SingleGroupClass}(object, \dots)
\method{IRT.expectedCounts}{MultipleGroupClass}(object, \dots)

# S3 method for extracting item response functions
\method{IRT.irfprob}{SingleGroupClass}(object, \dots)
\method{IRT.irfprob}{MultipleGroupClass}(object, group=1, \dots)

# compute factor scores
mirt.wrapper.fscores(mirt.obj, weights=NULL)

# convenience function for itemplot
mirt.wrapper.itemplot( mirt.obj, ask=TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{mirt.obj}{
A fitted model in \pkg{mirt} package
}
\item{object}{A fitted object in \pkg{mirt} package of class
    \code{SingleGroupClass} or \code{MultipleGroupClass}.
}
\item{group}{Group index for \code{IRT.irfprob} (only
applicable for object of class \code{MultipleGroupClass})}
\item{digits}{Number of digits after decimal used for rounding}
\item{file}{File name for sinking summary output}
\item{weights}{Optional vector of student weights}
\item{ask}{Optional logical indicating whether each new plot should be
confirmed.}
\item{\dots}{Further arguments to be passed.}
}

\details{
The function \code{mirt.wrapper.coef} collects all item parameters
in a data frame.

The function \code{mirt.wrapper.posterior} extracts the individual
likelihood, individual likelihood and expected counts. This function does not
yet cover the case of multiple groups.

The function \code{mirt.wrapper.fscores} computes factor scores
EAP, MAP and MLE. The factor scores are computed on the
discrete grid of latent traits (contrary to the computation in \code{mirt}) as
specified in \code{mirt.obj@Theta}. This function does also not work
for multiple groups.

The function \code{mirt.wrapper.itemplot} displays all item plots
after each other.
}

\value{

Function \code{mirt.wrapper.coef} -- List with entries

\item{coef}{Data frame with item parameters}
\item{GroupPars}{Data frame or list with distribution parameters}


Function \code{mirt.wrapper.posterior} -- List with entries

\item{theta.k}{Grid of theta points}
\item{pi.k}{Trait distribution on \code{theta.k}}
\item{f.yi.qk}{Individual likelihood}
\item{f.qk.yi}{Individual posterior}
\item{n.ik}{Expected counts}
\item{data}{Used dataset}

Function \code{mirt.wrapper.fscores} -- List with entries

\item{person}{Data frame with person parameter estimates (factor scores)
    EAP, MAP and MLE for all dimensions.}
\item{EAP.rel}{EAP reliabilities}
}


%\references{
%% ~put references to the literature/web site here ~
%}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~


\seealso{
See the \pkg{mirt} package manual for more information.

See for the main estimation functions in \pkg{mirt}:
\code{\link[mirt:mirt]{mirt::mirt}},
\code{\link[mirt:multipleGroup]{mirt::multipleGroup}}
and \code{\link[mirt:bfactor]{mirt::bfactor}}.

See \code{\link[mirt:coef-method]{mirt::coef-method}} for extracting
coefficients.

See \code{\link[mirt:mod2values]{mirt::mod2values}} for collecting
parameter values in a mirt parameter table.

See \code{\link{lavaan2mirt}} for converting \code{lavaan} syntax
to \code{mirt} syntax.

See \code{\link{tam2mirt}} for converting fitted \code{tam} models
into \code{mirt} objects.

See also \code{\link[CDM:IRT.likelihood]{CDM::IRT.likelihood}},
\code{\link[CDM:IRT.posterior]{CDM::IRT.posterior}} and
\code{\link[CDM:IRT.irfprob]{CDM::IRT.irfprob}} for general
extractor functions.
}


% See \code{\link{mirt.model.vars}} for specifying \pkg{mirt} models
% using variable names.

%% ~~objects to See Also as \code{\link{help}}, ~~~




\section{Examples for the \pkg{mirt} Package}{

%\itemize{
\enumerate{

%1
\item Latent class analysis (\code{\link{data.read}}, Model 7)

%2
\item Mixed Rasch model  (\code{\link{data.read}}, Model 8)

%3
\item Located unidimensional and multidimensional
latent class models / Multidimensional latent class IRT models
(\code{\link{data.read}}, Model 12;
\code{\link{rasch.mirtlc}}, Example 4)

%4
\item Multidimensional IRT model with discrete latent traits
 (\code{\link{data.read}}, Model 13)

%5
\item DINA model (\code{\link{data.read}}, Model 14;
\code{\link[CDM:data.dcm]{data.dcm}}, \pkg{CDM}, Model 1m)

%6
\item Unidimensional IRT model with non-normal distribution
 (\code{\link{data.read}}, Model 15)

%7
\item Grade of membership model  (\code{\link{gom.em}}, Example 2)

%8
\item Rasch copula model (\code{\link{rasch.copula2}}, Example 5)

%9
\item Additive GDINA model
(\code{\link[CDM:data.dcm]{data.dcm}}, \pkg{CDM}, Model 6m)

%10
\item Longitudinal Rasch model (\code{\link{data.long}}, Model 3)

%11
\item Normally distributed residuals (\code{\link{data.big5}}, Example 1, Model 5)

% 12
\item Nedelsky model (\code{\link{nedelsky.irf}},
Examples 1, 2)

% 13
\item Beta item response model (\code{\link{brm.irf}}, Example 1)
}

}


\examples{
\dontrun{
# A development version can be installed from GitHub
if (FALSE){ # default is set to FALSE, use the installed version
   library(devtools)
   devtools::install_github("philchalmers/mirt")
          }
# now, load mirt
library(mirt)

#############################################################################
# EXAMPLE 1: Extracting item parameters and posterior LSAT data
#############################################################################

data(LSAT7, package="mirt")
data <- mirt::expand.table(LSAT7)

#*** Model 1: 3PL model for item 5 only, other items 2PL
mod1 <- mirt::mirt(data, 1, itemtype=c("2PL","2PL","2PL","2PL","3PL"), verbose=TRUE)
print(mod1)
summary(mod1)
# extracting coefficients
coef(mod1)
mirt.wrapper.coef(mod1)$coef
# summary output
mirt_summary(mod1)
# extract parameter values in mirt
mirt::mod2values(mod1)
# extract posterior
post1 <- sirt::mirt.wrapper.posterior(mod1)
# extract item response functions
probs1 <- IRT.irfprob(mod1)
str(probs1)
# extract individual likelihood
likemod1 <- IRT.likelihood(mod1)
str(likemod1)
# extract individual posterior
postmod1 <- IRT.posterior(mod1)
str(postmod1)

#*** Model 2: Confirmatory model with two factors
cmodel <- mirt::mirt.model("
        F1=1,4,5
        F2=2,3
        ")
mod2 <- mirt::mirt(data, cmodel, verbose=TRUE)
print(mod2)
summary(mod2)
# extract coefficients
coef(mod2)
mirt.wrapper.coef(mod2)$coef
# extract posterior
post2 <- sirt::mirt.wrapper.posterior(mod2)

#############################################################################
# EXAMPLE 2: Extracting item parameters and posterior for differering
#            number of response catagories | Dataset Science
#############################################################################

data(Science,package="mirt")
library(psych)
psych::describe(Science)

# modify dataset
dat <- Science
dat[ dat[,1] > 3,1] <- 3
psych::describe(dat)

# estimate generalized partial credit model
mod1 <- mirt::mirt(dat, 1, itemtype="gpcm")
print(mod1)
# extract coefficients
coef(mod1)
mirt.wrapper.coef(mod1)$coef
# extract posterior
post1 <- sirt::mirt.wrapper.posterior(mod1)

#############################################################################
# EXAMPLE 3: Multiple group model; simulated dataset from mirt package
#############################################################################

#*** simulate data (copy from the multipleGroup manual site in mirt package)
set.seed(1234)
a <- matrix(c(abs( stats::rnorm(5,1,.3)), rep(0,15),abs( stats::rnorm(5,1,.3)),
          rep(0,15),abs( stats::rnorm(5,1,.3))), 15, 3)
d <- matrix( stats::rnorm(15,0,.7),ncol=1)
mu <- c(-.4, -.7, .1)
sigma <- matrix(c(1.21,.297,1.232,.297,.81,.252,1.232,.252,1.96),3,3)
itemtype <- rep("dich", nrow(a))
N <- 1000
dataset1 <- mirt::simdata(a, d, N, itemtype)
dataset2 <- mirt::simdata(a, d, N, itemtype, mu=mu, sigma=sigma)
dat <- rbind(dataset1, dataset2)
group <- c(rep("D1", N), rep("D2", N))

#group models
model <- mirt::mirt.model("
   F1=1-5
   F2=6-10
   F3=11-15
      ")

# separate analysis
mod_configural <- mirt::multipleGroup(dat, model, group=group, verbose=TRUE)
mirt.wrapper.coef(mod_configural)

# equal slopes (metric invariance)
mod_metric <- mirt::multipleGroup(dat, model, group=group, invariance=c("slopes"),
                verbose=TRUE)
mirt.wrapper.coef(mod_metric)

# equal slopes and intercepts (scalar invariance)
mod_scalar <- mirt::multipleGroup(dat, model, group=group,
          invariance=c("slopes","intercepts","free_means","free_varcov"), verbose=TRUE)
mirt.wrapper.coef(mod_scalar)

# full constraint
mod_fullconstrain <- mirt::multipleGroup(dat, model, group=group,
             invariance=c("slopes", "intercepts", "free_means", "free_var"), verbose=TRUE )
mirt.wrapper.coef(mod_fullconstrain)

#############################################################################
# EXAMPLE 4: Nonlinear item response model
#############################################################################

data(data.read)
dat <- data.read
# specify mirt model with some interactions
mirtmodel <- mirt.model("
   A=1-4
   B=5-8
   C=9-12
   (A*B)=4,8
   (C*C)=9
   (A*B*C)=12
   " )
# estimate model
res <- mirt::mirt( dat, mirtmodel, verbose=TRUE, technical=list(NCYCLES=3) )
# look at estimated parameters
mirt.wrapper.coef(res)
coef(res)
mirt::mod2values(res)
# model specification
res@model

#############################################################################
# EXAMPLE 5: Extracting factor scores
#############################################################################

data(data.read)
dat <- data.read
# define lavaan model and convert syntax to mirt
lavmodel <- "
    A=~ a*A1+a*A2+1.3*A3+A4       # set loading of A3 to 1.3
    B=~ B1+1*B2+b3*B3+B4
    C=~ c*C1+C2+c*C3+C4
    A1 | da*t1
    A3 | da*t1
    C4 | dg*t1
    B1 | 0*t1
    B3 | -1.4*t1                  # fix item threshold of B3 to -1.4
    A ~~ B                        # estimate covariance between A and B
    A ~~ .6 * C                   # fix covariance to .6
    B ~~ B                        # estimate variance of B
    A ~ .5*1                      # set mean of A to .5
    B ~ 1                         # estimate mean of B
    "
res <- sirt::lavaan2mirt( dat, lavmodel, verbose=TRUE, technical=list(NCYCLES=3) )
# estimated coefficients
mirt.wrapper.coef(res$mirt)
# extract factor scores
fres <- sirt::mirt.wrapper.fscores(res$mirt)
# look at factor scores
head( round(fres$person,2))
  ##     case    M EAP.Var1 SE.EAP.Var1 EAP.Var2 SE.EAP.Var2 EAP.Var3 SE.EAP.Var3 MLE.Var1
  ##   1    1 0.92     1.26        0.67     1.61        0.60     0.05        0.69     2.65
  ##   2    2 0.58     0.06        0.59     1.14        0.55    -0.80        0.56     0.00
  ##   3    3 0.83     0.86        0.66     1.15        0.55     0.48        0.74     0.53
  ##   4    4 1.00     1.52        0.67     1.57        0.60     0.73        0.76     2.65
  ##   5    5 0.50    -0.13        0.58     0.85        0.48    -0.82        0.55    -0.53
  ##   6    6 0.75     0.41        0.63     1.09        0.54     0.27        0.71     0.00
  ##     MLE.Var2 MLE.Var3 MAP.Var1 MAP.Var2 MAP.Var3
  ##   1     2.65    -0.53     1.06     1.59     0.00
  ##   2     1.06    -1.06     0.00     1.06    -1.06
  ##   3     1.06     2.65     1.06     1.06     0.53
  ##   4     2.65     2.65     1.59     1.59     0.53
  ##   5     0.53    -1.06    -0.53     0.53    -1.06
  ##   6     1.06     2.65     0.53     1.06     0.00
# EAP reliabilities
round(fres$EAP.rel,3)
  ##    Var1  Var2  Var3
  ##   0.574 0.452 0.541
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{mirt}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: lq_fit.Rd
%% File Version: 0.182

\name{lq_fit}
\alias{lq_fit}
\alias{lq_fit_estimate_power}
\alias{dexppow}
\alias{rexppow}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Fit of a \eqn{L_q} Regression Model
}
\description{
Fits a regression model in the \eqn{L_q} norm (also labeled as the \eqn{L_p} norm).
In more detail,
the optimization function \eqn{ \sum_i | y_i - x_i \beta | ^p} is optimized.
The nondifferentiable function is approximated by a differentiable approximation,
i.e., we use \eqn{|x| \approx \sqrt{x^2 + \varepsilon } }. The power \eqn{p}
can also be estimated by using \code{est_pow=TRUE}, see
Giacalone, Panarello and Mattera (2018). The algorithm iterates between estimating
regression coefficients and the estimation of power values. The estimation of the
power based on a vector of residuals \code{e} can be conducted using the
function \code{lq_fit_estimate_power}.

Using the \eqn{L_q} norm in the regression is equivalent to assuming an expontial
power function for residuals (Giacalone et al., 2018). The density function and
a simulation function is provided by \code{dexppow} and \code{rexppow}, respectively.
See also the \pkg{normalp} package.
}
\usage{
lq_fit(y, X, w=NULL, pow=2, eps=0.001, beta_init=NULL, est_pow=FALSE, optimizer="optim",
    eps_vec=10^seq(0,-10, by=-.5), conv=1e-4, miter=20, lower_pow=.1, upper_pow=5)

lq_fit_estimate_power(e, pow_init=2, lower_pow=.1, upper_pow=10)

dexppow(x, mu=0, sigmap=1, pow=2, log=FALSE)

rexppow(n, mu=0, sigmap=1, pow=2, xbound=100, xdiff=.01)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{y}{
Dependent variable
}
  \item{X}{
Design matrix
}
  \item{w}{
Optional vector of weights
}
  \item{pow}{
Power \eqn{p} in \eqn{L_q} norm- The power \eqn{p=0} is handled using
the loss function \eqn{f(e)=e^2/(e^2+\varepsilon)}.
}
  \item{est_pow}{
Logical indicating whether power should be estimated
}
  \item{eps}{
Parameter governing the differentiable approximation
}
\item{e}{Vector of resiuals}
\item{pow_init}{Initial value of power}
  \item{beta_init}{
Initial vector
}
  \item{optimizer}{
Can be \code{"optim"} or \code{"nlminb"}.
}
\item{eps_vec}{Vector with decreasing \eqn{\varepsilon} values used in
optimization}
\item{conv}{Convergence criterion}
\item{miter}{Maximum number of iterations}
\item{lower_pow}{Lower bound for estimated power}
\item{upper_pow}{Upper bound for estimated power}
\item{x}{Vector}
\item{mu}{Location parameter}
\item{sigmap}{Scale parameter}
\item{log}{Logical indicating whether the logarithm should be provided}
\item{n}{Sample size}
\item{xbound}{Lower and upper bound for density approximation}
\item{xdiff}{Grid width for density approximation}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}

\value{
List with following several entries
\item{coefficients}{Vector of coefficients}
\item{res_optim}{Results of optimization}
\item{\ldots}{More values}
}

\references{
Giacalone, M., Panarello, D., & Mattera, R. (2018).
Multicollinearity in regression: an efficiency comparison between $L_p$-norm and least
squares estimators. \emph{Quality & Quantity, 52}(4), 1831-1859.
\doi{10.1007/s11135-017-0571-y}
}

%\author{
%%  ~~who you are~~
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}


\examples{
#############################################################################
# EXAMPLE 1: Small simulated example with fixed power
#############################################################################

set.seed(98)
N <- 300
x1 <- stats::rnorm(N)
x2 <- stats::rnorm(N)
par1 <- c(1,.5,-.7)
y <- par1[1]+par1[2]*x1+par1[3]*x2 + stats::rnorm(N)
X <- cbind(1,x1,x2)

#- lm function in stats
mod1 <- stats::lm.fit(y=y, x=X)

#- use lq_fit function
mod2 <- sirt::lq_fit( y=y, X=X, pow=2, eps=1e-4)
mod1$coefficients
mod2$coefficients

\dontrun{
#############################################################################
# EXAMPLE 2: Example with estimated power values
#############################################################################

#*** simulate regression model with residuals from the exponential power distribution
#*** using a power of .30
set.seed(918)
N <- 2000
X <- cbind( 1, c(rep(1,N), rep(0,N)) )
e <- sirt::rexppow(n=2*N, pow=.3, xdiff=.01, xbound=200)
y <- X \%*\% c(1,.5) + e

#*** estimate model
mod <- sirt::lq_fit( y=y, X=X, est_pow=TRUE, lower_pow=.1)
mod1 <- stats::lm( y ~ 0 + X )
mod$coefficients
mod$pow
mod1$coefficients
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }% use one of  RShowDoc("KEYWORDS")
% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: lsem.test.Rd
%% File Version: 0.083

\name{lsem.test}
\alias{lsem.test}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Test a Local Structural Equation Model Based on Bootstrap
}
\description{
Performs global and parameter tests for a fitted local structural equation model.
The LSEM must have been fitted and bootstrap estimates of the LSEM model must be
available for statistical inference. The hypothesis of a constant parameter is tested
by means of a Wald test. Moreover, regression functions can be specified and tested
if these are specified in the argument \code{models}.
}
\usage{
lsem.test(mod, bmod, models=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{mod}{
Fitted LSEM object
}
  \item{bmod}{
Fitted LSEM bootstrap object. The argument \code{bmod} can also be missing.
}
  \item{models}{
List of model formulas for named LSEM model parameters
}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}


\value{
List with following entries

\item{wald_test_global}{Global Wald test for model parameters}
\item{test_models}{Output for fitted regression models. The test for parameter
heterogeneity is included in \code{chisq_het}, while the test of a sufficient
fit of a parameter curve is included in \code{chisq_fit}.}
\item{parameters}{Original model parameters after fitting (i.e., smoothing) a particular
parameter using a regression model specified in \code{models}.}
\item{parameters_boot}{Bootstrapped model parameters after fitting (i.e., smoothing)
a particular parameter using a regression model specified in \code{models}.}
}

%\references{
%% ~put references to the literature/web site here ~
%}
%\author{
%%  ~~who you are~~
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also \code{\link{lsem.estimate}} for estimating LSEM models and
\code{\link{lsem.bootstrap}} for bootstrapping LSEM models.
}



\examples{
\dontrun{
#############################################################################
# EXAMPLE 1: data.lsem01 | Age differentiation and tested models
#############################################################################

data(data.lsem01, package="sirt")
dat <- data.lsem01

# specify lavaan model
lavmodel <- "
        F=~ v1+v2+v3+v4+v5
        F ~~ 1*F
    "

# define grid of moderator variable age
moderator.grid <- seq(4,23,1)

#-- estimate LSEM with bandwidth 2
mod <- sirt::lsem.estimate( dat, moderator="age", moderator.grid=moderator.grid,
               lavmodel=lavmodel, h=2, std.lv=TRUE)
summary(mod)

#-- bootstrap model
bmod <- sirt::lsem.bootstrap(mod, R=200)

#-- test models
models <- list( "F=~v1"=y ~ m + I(m^2),
                "F=~v2"=y ~ I( splines::bs(m, df=4) ) )
tmod <- sirt::lsem.test(mod=mod, bmod=bmod, models=models)
str(tmod)
sirt::print_digits(wald_test_global, 3)
sirt::print_digits(test_models, 3)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

%% File Name: class.accuracy.rasch.Rd
%% File Version: 0.261

\name{class.accuracy.rasch}
\alias{class.accuracy.rasch}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Classification Accuracy in the Rasch Model
}
\description{
This function computes the classification accuracy in the Rasch model
for the maximum likelihood (person parameter) estimate according
to the method of Rudner (2001).
}
\usage{
class.accuracy.rasch(cutscores, b, meantheta, sdtheta, theta.l, n.sims=0)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{cutscores}{
Vector of cut scores
}
  \item{b}{
Vector of item difficulties
}
  \item{meantheta}{
Mean of the trait distribution
}
  \item{sdtheta}{
Standard deviation of the trait distribution
}
  \item{theta.l}{
Discretized theta distribution
}
  \item{n.sims}{
Number of simulated persons in a data set. The default is 0
which means that no simulation is performed.
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list with following entries:
  \item{class.stats}{Data frame containing classification accuracy statistics. The
  column \code{agree0} refers to absolute agreement, \code{agree1} to
  the agreement of at most a difference of one level.
     }
  \item{class.prob}{Probability table of classification}
}


\references{
Rudner, L.M. (2001). Computing the expected proportions of misclassified examinees.
\emph{Practical Assessment, Research & Evaluation, 7}(14).
}

%\author{
%Alexander Robitzsch
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
Classification accuracy of other IRT models
can be obtained with the \R package \pkg{cacIRT}.
}

\examples{
#############################################################################
# EXAMPLE 1: Reading dataset
#############################################################################
data( data.read, package="sirt")
dat <- data.read

# estimate the Rasch model
mod <- sirt::rasch.mml2( dat )

# estimate classification accuracy (3 levels)
cutscores <- c( -1, .3 )    # cut scores at theta=-1 and theta=.3
sirt::class.accuracy.rasch( cutscores=cutscores, b=mod$item$b,
           meantheta=0,  sdtheta=mod$sd.trait,
           theta.l=seq(-4,4,len=200), n.sims=3000)
  ##   Cut Scores
  ##   [1] -1.0  0.3
  ##
  ##   WLE reliability (by simulation)=0.671
  ##   WLE consistency (correlation between two parallel forms)=0.649
  ##
  ##   Classification accuracy and consistency
  ##              agree0 agree1 kappa consistency
  ##   analytical   0.68  0.990 0.492          NA
  ##   simulated    0.70  0.997 0.489       0.599
  ##
  ##   Probability classification table
  ##               Est_Class1 Est_Class2 Est_Class3
  ##   True_Class1      0.136      0.041      0.001
  ##   True_Class2      0.081      0.249      0.093
  ##   True_Class3      0.009      0.095      0.294
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Classification accuracy}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.pars1.rasch.Rd
%% File Version: 0.12

\name{data.pars1.rasch}
\alias{data.pars1.rasch}
\alias{data.pars1.2pl}

\docType{data}
\title{
Item Parameters for Three Studies Obtained by 1PL and 2PL Estimation
}
\description{
The datasets contain item parameters to be prepared for linking
using the function \code{\link{linking.haberman}}.
}
\usage{
data(data.pars1.rasch)
data(data.pars1.2pl)
}
\format{
\itemize{
\item The format of \code{data.pars1.rasch} is:

\code{'data.frame':   22 obs. of  4 variables:} \cr
\code{ $ study: chr  "study1" "study1" "study1" "study1" ...} \cr
\code{ $ item : Factor w/ 12 levels "M133","M176",..: 1 2 3 4 5 1 6 7 3 8 ...} \cr
\code{ $ a    : num  1 1 1 1 1 1 1 1 1 1 ...} \cr
\code{ $ b    : num  -1.5862 0.40762 1.78031 2.00382 0.00862 ...}

Item slopes \code{a} are fixed to 1 in 1PL estimation. Item difficulties
are denoted by \code{b}. \cr

\item The format of \code{data.pars1.2pl} is:

\code{'data.frame':   22 obs. of  4 variables:} \cr
\code{ $ study: chr  "study1" "study1" "study1" "study1" ...} \cr
\code{ $ item : Factor w/ 12 levels "M133","M176",..: 1 2 3 4 5 1 6 7 3 8 ...} \cr
\code{ $ a    : num  1.238 0.957 1.83 1.927 2.298 ...} \cr
\code{ $ b    : num  -1.16607 0.35844 1.06571 1.17159 0.00792 ...} \cr
}
}

%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}

%\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%}
%\references{
%%  ~~ possibly secondary sources and usages ~~
%}


%data.pars1.rasch
%data.pars1.2pl

%\examples{
%data(data.pars1.rasch)
%## maybe str(data.pars1.rasch) ; plot(data.pars1.rasch) ...
%}
%% \keyword{datasets}

%% File Name: wle.rasch.jackknife.Rd
%% File Version: 1.272

\name{wle.rasch.jackknife}
\alias{wle.rasch.jackknife}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Standard Error Estimation of WLE by Jackknifing
}
\description{
This function calculates standard errors of WLEs (Warm, 1989) for
stratified item designs and item designs with testlets for the
Rasch model.
}
\usage{
wle.rasch.jackknife(dat, b, itemweights=1 + 0 * b, pid=NULL,
    testlet=NULL, stratum=NULL, size.itempop=NULL)
}
\arguments{
  \item{dat}{
An \eqn{N \times I} data frame of item responses
}
  \item{b}{
Vector of item difficulties
}
  \item{itemweights}{
Weights for items, i.e. fixed item discriminations
}
  \item{pid}{
Person identifier
}
  \item{testlet}{
A vector of length \eqn{I} which defines which item
belongs to which testlet. If some items
does not belong to any testlet, then define separate
testlet labels for these single items.
}
  \item{stratum}{
Item stratum
}
  \item{size.itempop}{
Number of items in an item stratum of the finite item population.
}
}
\details{
The idea of Jackknife in item response models can be found in
Wainer and Wright (1980).
}
\value{
A list with following entries:
  \item{wle}{Data frame with some estimated statistics. The column
  \code{wle} is the WLE and \code{wle.jackse} its corresponding
  standard error estimated by jackknife.
  }
  \item{wle.rel}{WLE reliability (Adams, 2005)}
}


\references{
Adams, R. J. (2005). Reliability as a measurement design effect.
\emph{Studies in Educational Evaluation, 31}(2-3), 162-172.
\doi{10.1016/j.stueduc.2005.05.008}

Gershunskaya, J., Jiang, J., & Lahiri, P. (2009). Resampling methods in surveys.
In D. Pfeffermann and C.R. Rao (Eds.). \emph{Handbook of Statistics 29B; Sample
Surveys: Inference and Analysis} (pp. 121-151). Amsterdam: North Holland.
\doi{10.1016/S0169-7161(09)00228-4}

Wainer, H., & Wright, B. D. (1980). Robust estimation of ability in the Rasch
model. \emph{Psychometrika, 45}(3), 373-391.
\doi{10.1007/BF02293910}

Warm, T. A. (1989). Weighted likelihood estimation of ability in item response theory.
\emph{Psychometrika, 54}(3), 427-450.
\doi{10.1007/BF02294627}
}


%\author{
%Alexander Robitzsch
%}


%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{wle.rasch}}
}


\examples{
#############################################################################
# EXAMPLE 1: Dataset Reading
#############################################################################
data(data.read)
dat <- data.read

# estimation of the Rasch model
res <- sirt::rasch.mml2( dat, parm.conv=.001)

# WLE estimation
wle1 <- sirt::wle.rasch(dat, b=res$item$thresh )

# simple jackknife WLE estimation
wle2 <- sirt::wle.rasch.jackknife(dat, b=res$item$thresh )
  ## WLE Reliability=0.651

# SE(WLE) for testlets A, B and C
wle3 <- sirt::wle.rasch.jackknife(dat, b=res$item$thresh,
           testlet=substring( colnames(dat),1,1) )
  ## WLE Reliability=0.572

# SE(WLE) for item strata A,B, C
wle4 <- sirt::wle.rasch.jackknife(dat, b=res$item$thresh,
             stratum=substring( colnames(dat),1,1) )
  ## WLE Reliability=0.683

# SE (WLE) for finite item strata
# A (10 items), B (7 items), C (4 items -> no sampling error)
# in every stratum 4 items were sampled
size.itempop <- c(10,7,4)
names(size.itempop) <- c("A","B","C")
wle5 <- sirt::wle.rasch.jackknife(dat, b=res$item$thresh,
             stratum=substring( colnames(dat),1,1),
             size.itempop=size.itempop )
  ## Stratum  A (Mean) Correction Factor 0.6
  ## Stratum  B (Mean) Correction Factor 0.42857
  ## Stratum  C (Mean) Correction Factor 0
  ## WLE Reliability=0.876

# compare different estimated standard errors
a2 <- stats::aggregate( wle2$wle$wle.jackse, list( wle2$wle$wle), mean )
colnames(a2) <- c("wle", "se.simple")
a2$se.testlet <- stats::aggregate( wle3$wle$wle.jackse, list( wle3$wle$wle), mean )[,2]
a2$se.strata <- stats::aggregate( wle4$wle$wle.jackse, list( wle4$wle$wle), mean )[,2]
a2$se.finitepop.strata <- stats::aggregate( wle5$wle$wle.jackse,
    list( wle5$wle$wle), mean )[,2]
round( a2, 3 )
  ## > round( a2, 3 )
  ##       wle se.simple se.testlet se.strata se.finitepop.strata
  ## 1  -5.085     0.440      0.649     0.331               0.138
  ## 2  -3.114     0.865      1.519     0.632               0.379
  ## 3  -2.585     0.790      0.849     0.751               0.495
  ## 4  -2.133     0.715      1.177     0.546               0.319
  ## 5  -1.721     0.597      0.767     0.527               0.317
  ## 6  -1.330     0.633      0.623     0.617               0.377
  ## 7  -0.942     0.631      0.643     0.604               0.365
  ## 8  -0.541     0.655      0.678     0.617               0.384
  ## 9  -0.104     0.671      0.646     0.659               0.434
  ## 10  0.406     0.771      0.706     0.751               0.461
  ## 11  1.080     1.118      0.893     1.076               0.630
  ## 12  2.332     0.400      0.631     0.272               0.195
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Weighted likelihood estimation (WLE)}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: dif.logistic.regression.Rd
%% File Version: 0.375

\name{dif.logistic.regression}
\alias{dif.logistic.regression}
\title{
Differential Item Functioning using Logistic Regression Analysis
}
\description{
This function assesses differential item
functioning using logistic regression analysis (Zumbo, 1999).
}
\usage{
dif.logistic.regression(dat, group, score,quant=1.645)
}
\arguments{
  \item{dat}{
Data frame with dichotomous item responses
}
  \item{group}{
Group identifier
}
  \item{score}{
Ability estimate, e.g. the WLE.
}
\item{quant}{Used quantile of the normal distribution
for assessing statistical significance}
}

\details{
Items are classified into A (negligible DIF), B (moderate DIF) and
C (large DIF) levels according to the
ETS classification system (Longford, Holland & Thayer, 1993, p. 175).
See also Monahan, McHorney, Stump and Perkins (2007) for further DIF effect size
classifications.
}


\value{
A data frame with following variables:

\item{itemnr}{Numeric index of the item}
\item{sortDIFindex}{Rank of item with respect to the uniform DIF
    (from negative to positive values)}
\item{item}{Item name}
\item{N}{Sample size per item}
\item{R}{Value of \code{group} variable for reference group}
\item{F}{Value of \code{group} variable for focal group}
\item{nR}{Sample size per item in reference group}
\item{nF}{Sample size per item in focal group}
\item{p}{Item \eqn{p} value}
\item{pR}{Item \eqn{p} value in reference group}
\item{pF}{Item \eqn{p} value in focal group}
\item{pdiff}{Item \eqn{p} value differences}
\item{pdiff.adj}{Adjusted \eqn{p} value difference}
\item{uniformDIF}{Uniform DIF estimate}
\item{se.uniformDIF}{Standard error of uniform DIF}
\item{t.uniformDIF}{The \eqn{t} value for uniform DIF}
\item{sig.uniformDIF}{Significance label for uniform DIF}
\item{DIF.ETS}{DIF classification according to the ETS classification
    system (see Details)}
\item{uniform.EBDIF}{Empirical Bayes estimate of uniform DIF (Longford,
    Holland & Thayer, 1993) which takes degree of DIF standard error
        into account}
\item{DIF.SD}{Value of the DIF standard deviation}
\item{nonuniformDIF}{Nonuniform DIF estimate}
\item{se.nonuniformDIF}{Standard error of nonuniform DIF}
\item{t.nonuniformDIF}{The \eqn{t} value for nonuniform DIF}
\item{sig.nonuniformDIF}{Significance label for nonuniform DIF}
}


\references{
Longford, N. T., Holland, P. W., & Thayer, D. T. (1993).
Stability of the MH D-DIF statistics across populations.
In P. W. Holland & H. Wainer (Eds.). \emph{Differential
Item Functioning} (pp. 171-196). Hillsdale, NJ: Erlbaum.

Magis, D., Beland, S., Tuerlinckx, F., & De Boeck, P. (2010). A general framework and an
\R package for the detection of dichotomous differential item functioning.
\emph{Behavior Research Methods, 42}(3), 847-862.
\doi{10.3758/BRM.42.3.847}

Monahan, P. O., McHorney, C. A., Stump, T. E., & Perkins, A. J. (2007).
Odds ratio, delta, ETS classification, and standardization measures of
DIF magnitude for binary logistic regression.
\emph{Journal of Educational and Behavioral Statistics, 32}(1), 92-109.
\doi{10.3102/1076998606298035}

Zumbo, B. D. (1999). \emph{A handbook on the theory and methods of differential
item functioning (DIF): Logistic regression modeling as a unitary framework
for binary and Likert-type (ordinal) item scores}.
Ottawa ON: Directorate of Human Resources Research and Evaluation,
Department of National Defense.
}

%\author{
%Alexander Robitzsch
%}

\seealso{
For assessing DIF variance see \code{\link{dif.variance}} and
\code{\link{dif.strata.variance}}

See also \code{\link{rasch.evm.pcm}} for assessing differential item
functioning in the partial credit model.

See the \pkg{difR} package for a large collection of DIF detection
methods (Magis, Beland, Tuerlinckx, & De Boeck, 2010).

For a download of the free \emph{DIF-Pack} software (SIBTEST, ...) see
\emph{http://psychometrictools.measuredprogress.org/home}.

}
\examples{
#############################################################################
# EXAMPLE 1: Mathematics data | Gender DIF
#############################################################################

data( data.math )
dat <- data.math$data
items <- grep( "M", colnames(dat))

# estimate item parameters and WLEs
mod <- sirt::rasch.mml2( dat[,items] )
wle <- sirt::wle.rasch( dat[,items], b=mod$item$b )$theta

# assess DIF by logistic regression
mod1 <- sirt::dif.logistic.regression( dat=dat[,items], score=wle, group=dat$female)

# calculate DIF variance
dif1 <- sirt::dif.variance( dif=mod1$uniformDIF, se.dif=mod1$se.uniformDIF )
dif1$unweighted.DIFSD
  ## > dif1$unweighted.DIFSD
  ## [1] 0.1963958

# calculate stratified DIF variance
# stratification based on domains
dif2 <- sirt::dif.strata.variance( dif=mod1$uniformDIF, se.dif=mod1$se.uniformDIF,
              itemcluster=data.math$item$domain )
  ## $unweighted.DIFSD
  ## [1] 0.1455916

\dontrun{
#****
# Likelihood ratio test and graphical model test in eRm package
miceadds::library_install("eRm")
# estimate Rasch model
res <- eRm::RM( dat[,items] )
summary(res)
# LR-test with respect to female
lrres <- eRm::LRtest(res, splitcr=dat$female)
summary(lrres)
# graphical model test
eRm::plotGOF(lrres)

#############################################################################
# EXAMPLE 2: Comparison with Mantel-Haenszel test
#############################################################################

library(TAM)
library(difR)

#*** (1) simulate data
set.seed(776)
N <- 1500   # number of persons per group
I <- 12     # number of items
mu2 <- .5   # impact (group difference)
sd2 <- 1.3  # standard deviation group 2

# define item difficulties
b <- seq( -1.5, 1.5, length=I)
# simulate DIF effects
bdif <- scale( stats::rnorm(I, sd=.6 ), scale=FALSE )[,1]
# item difficulties per group
b1 <- b + 1/2 * bdif
b2 <- b - 1/2 * bdif
# simulate item responses
dat1 <- sirt::sim.raschtype( theta=stats::rnorm(N, mean=0, sd=1 ), b=b1 )
dat2 <- sirt::sim.raschtype( theta=stats::rnorm(N, mean=mu2, sd=sd2 ), b=b2 )
dat <- rbind( dat1, dat2 )
group <- rep( c(1,2), each=N ) # define group indicator

#*** (2) scale data
mod <- TAM::tam.mml( dat, group=group )
summary(mod)

#*** (3) extract person parameter estimates
mod_eap <- mod$person$EAP
mod_wle <- tam.wle( mod )$theta

#*********************************
# (4) techniques for assessing differential item functioning

# Model 1: assess DIF by logistic regression and WLEs
dif1 <- sirt::dif.logistic.regression( dat=dat, score=mod_wle, group=group)
# Model 2: assess DIF by logistic regression and EAPs
dif2 <- sirt::dif.logistic.regression( dat=dat, score=mod_eap, group=group)
# Model 3: assess DIF by Mantel-Haenszel statistic
dif3 <- difR::difMH(Data=dat, group=group, focal.name="1",  purify=FALSE )
print(dif3)
  ##  Mantel-Haenszel Chi-square statistic:
  ##
  ##        Stat.    P-value
  ##  I0001  14.5655   0.0001 ***
  ##  I0002 300.3225   0.0000 ***
  ##  I0003   2.7160   0.0993 .
  ##  I0004 191.6925   0.0000 ***
  ##  I0005   0.0011   0.9740
  ##  [...]
  ##  Signif. codes: 0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
  ##  Detection threshold: 3.8415 (significance level: 0.05)
  ##
  ##  Effect size (ETS Delta scale):
  ##
  ##  Effect size code:
  ##   'A': negligible effect
  ##   'B': moderate effect
  ##   'C': large effect
  ##
  ##        alphaMH deltaMH
  ##  I0001  1.3908 -0.7752 A
  ##  I0002  0.2339  3.4147 C
  ##  I0003  1.1407 -0.3093 A
  ##  I0004  2.8515 -2.4625 C
  ##  I0005  1.0050 -0.0118 A
  ##  [...]
  ##
  ##  Effect size codes: 0 'A' 1.0 'B' 1.5 'C'
  ##   (for absolute values of 'deltaMH')

# recompute DIF parameter from alphaMH
uniformDIF3 <- log(dif3$alphaMH)

# compare different DIF statistics
dfr <- data.frame( "bdif"=bdif, "LR_wle"=dif1$uniformDIF,
        "LR_eap"=dif2$uniformDIF, "MH"=uniformDIF3 )
round( dfr, 3 )
  ##       bdif LR_wle LR_eap     MH
  ##  1   0.236  0.319  0.278  0.330
  ##  2  -1.149 -1.473 -1.523 -1.453
  ##  3   0.140  0.122  0.038  0.132
  ##  4   0.957  1.048  0.938  1.048
  ##  [...]
colMeans( abs( dfr[,-1] - bdif ))
  ##      LR_wle     LR_eap         MH
  ##  0.07759187 0.19085743 0.07501708
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{Differential item functioning (DIF)}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File Name: data.big5.Rd
%% File Version: 0.51

\name{data.big5}
\alias{data.big5}
\alias{data.big5.qgraph}


\docType{data}
\title{
Dataset Big 5 from \pkg{qgraph} Package
}

\description{
This is a Big 5 dataset from the \pkg{qgraph} package (Dolan, Oorts,
Stoel, Wicherts, 2009). It contains 500 subjects on 240 items.
}

\usage{
data(data.big5)
data(data.big5.qgraph)
}

\format{
\itemize{
\item The format of \code{data.big5} is: \cr
\code{ num [1:500, 1:240] 1 0 0 0 0 1 1 2 0 1 ...} \cr
\code{ - attr(*, "dimnames")=List of 2} \cr
\code{  ..$ : NULL} \cr
\code{  ..$ : chr [1:240] "N1" "E2" "O3" "A4" ...} \cr

\item The format of \code{data.big5.qgraph} is: \cr

\code{ num [1:500, 1:240] 2 3 4 4 5 2 2 1 4 2 ...} \cr
\code{ - attr(*, "dimnames")=List of 2} \cr
\code{  ..$ : NULL} \cr
\code{  ..$ : chr [1:240] "N1" "E2" "O3" "A4" ...} \cr
}
}


\details{
In these datasets, there exist 48 items for each dimension. The Big 5
dimensions are Neuroticism (\code{N}), Extraversion (\code{E}),
Openness (\code{O}), Agreeableness (\code{A}) and
Conscientiousness (\code{C}). Note that the \code{data.big5} differs from
\code{data.big5.qgraph} in a way that original items were recoded into
three categories 0,1 and 2.
}


\source{
See \code{big5} in \pkg{qgraph} package.
}


\references{
Dolan, C. V., Oort, F. J., Stoel, R. D., & Wicherts, J. M. (2009).
Testing measurement invariance in the target rotates multigroup exploratory
factor model. \emph{Structural Equation Modeling, 16}, 295-314.
}






\examples{
\dontrun{
# list of needed packages for the following examples
packages <- scan(what="character")
     sirt   TAM   eRm   CDM   mirt  ltm   mokken  psychotools  psychomix
     psych

# load packages. make an installation if necessary
miceadds::library_install(packages)

#############################################################################
# EXAMPLE 1: Unidimensional models openness scale
#############################################################################

data(data.big5)
# extract first 10 openness items
items <- which( substring( colnames(data.big5), 1, 1 )=="O"  )[1:10]
dat <- data.big5[, items ]
I <- ncol(dat)
summary(dat)
  ##   > colnames(dat)
  ##    [1] "O3"  "O8"  "O13" "O18" "O23" "O28" "O33" "O38" "O43" "O48"
# descriptive statistics
psych::describe(dat)

#****************
# Model 1: Partial credit model
#****************

#-- M1a: rm.facets (in sirt)
m1a <- sirt::rm.facets( dat )
summary(m1a)

#-- M1b: tam.mml (in TAM)
m1b <- TAM::tam.mml( resp=dat )
summary(m1b)

#-- M1c: gdm (in CDM)
theta.k <- seq(-6,6,len=21)
m1c <- CDM::gdm( dat, irtmodel="1PL",theta.k=theta.k, skillspace="normal")
summary(m1c)
# compare results with loglinear skillspace
m1c2 <- CDM::gdm( dat, irtmodel="1PL",theta.k=theta.k, skillspace="loglinear")
summary(m1c2)

#-- M1d: PCM (in eRm)
m1d <- eRm::PCM( dat )
summary(m1d)

#-- M1e: gpcm (in ltm)
m1e <- ltm::gpcm( dat, constraint="1PL", control=list(verbose=TRUE))
summary(m1e)

#-- M1f: mirt (in mirt)
m1f <- mirt::mirt( dat, model=1, itemtype="1PL", verbose=TRUE)
summary(m1f)
coef(m1f)

#-- M1g: PCModel.fit (in psychotools)
mod1g <- psychotools::PCModel.fit(dat)
summary(mod1g)
plot(mod1g)

#****************
# Model 2: Generalized partial credit model
#****************

#-- M2a: rm.facets (in sirt)
m2a <- sirt::rm.facets( dat, est.a.item=TRUE)
summary(m2a)
# Note that in rm.facets the mean of item discriminations is fixed to 1

#-- M2b: tam.mml.2pl (in TAM)
m2b <- TAM::tam.mml.2pl( resp=dat, irtmodel="GPCM")
summary(m2b)

#-- M2c: gdm (in CDM)
m2c <- CDM::gdm( dat, irtmodel="2PL",theta.k=seq(-6,6,len=21),
                   skillspace="normal", standardized.latent=TRUE)
summary(m2c)

#-- M2d: gpcm (in ltm)
m2d <- ltm::gpcm( dat, control=list(verbose=TRUE))
summary(m2d)

#-- M2e: mirt (in mirt)
m2e <- mirt::mirt( dat, model=1,  itemtype="GPCM", verbose=TRUE)
summary(m2e)
coef(m2e)

#****************
# Model 3: Nonparametric item response model
#****************

#-- M3a: ISOP and ADISOP model - isop.poly (in sirt)
m3a <- sirt::isop.poly( dat )
summary(m3a)
plot(m3a)

#-- M3b: Mokken scale analysis (in mokken)
# Scalability coefficients
mokken::coefH(dat)
# Assumption of monotonicity
monotonicity.list <- mokken::check.monotonicity(dat)
summary(monotonicity.list)
plot(monotonicity.list)
# Assumption of non-intersecting ISRFs using method restscore
restscore.list <- mokken::check.restscore(dat)
summary(restscore.list)
plot(restscore.list)

#****************
# Model 4: Graded response model
#****************

#-- M4a: mirt (in mirt)
m4a <- mirt::mirt( dat, model=1,  itemtype="graded", verbose=TRUE)
print(m4a)
mirt.wrapper.coef(m4a)

#----  M4b: WLSMV estimation with cfa (in lavaan)
lavmodel <- "F=~ O3__O48
             F ~~ 1*F
                "
# transform lavaan syntax with lavaanify.IRT
lavmodel <- TAM::lavaanify.IRT( lavmodel, items=colnames(dat) )$lavaan.syntax
mod4b <- lavaan::cfa( data=as.data.frame(dat), model=lavmodel, std.lv=TRUE,
                 ordered=colnames(dat),  parameterization="theta")
summary(mod4b, standardized=TRUE, fit.measures=TRUE, rsquare=TRUE)
coef(mod4b)

#****************
# Model 5: Normally distributed residuals
#****************

#----  M5a: cfa (in lavaan)
lavmodel <- "F=~ O3__O48
             F ~~ 1*F
             F ~ 0*1
             O3__O48 ~ 1
                "
lavmodel <- TAM::lavaanify.IRT( lavmodel, items=colnames(dat) )$lavaan.syntax
mod5a <- lavaan::cfa( data=as.data.frame(dat), model=lavmodel, std.lv=TRUE,
                 estimator="MLR" )
summary(mod5a, standardized=TRUE, fit.measures=TRUE, rsquare=TRUE)

#----  M5b: mirt (in mirt)

# create user defined function
name <- 'normal'
par <- c("d"=1, "a1"=0.8, "vy"=1)
est <- c(TRUE, TRUE,FALSE)
P.normal <- function(par,Theta,ncat){
     d <- par[1]
     a1 <- par[2]
     vy <- par[3]
     psi <- vy - a1^2
     # expected values given Theta
     mui <- a1*Theta[,1] + d
     TP <- nrow(Theta)
     probs <- matrix( NA, nrow=TP, ncol=ncat )
     eps <- .01
     for (cc in 1:ncat){
        probs[,cc] <- stats::dnorm( cc, mean=mui, sd=sqrt( abs( psi + eps) ) )
                    }
     psum <- matrix( rep(rowSums( probs ),each=ncat), nrow=TP, ncol=ncat, byrow=TRUE)
     probs <- probs / psum
     return(probs)
}

# create item response function
normal <- mirt::createItem(name, par=par, est=est, P=P.normal)
customItems <- list("normal"=normal)
itemtype <- rep( "normal",I)
# define parameters to be estimated
mod5b.pars <- mirt::mirt(dat, 1, itemtype=itemtype,
                   customItems=customItems, pars="values")
ind <- which( mod5b.pars$name=="vy")
vy <- apply( dat, 2, var, na.rm=TRUE )
mod5b.pars[ ind, "value" ] <- vy
ind <- which( mod5b.pars$name=="a1")
mod5b.pars[ ind, "value" ] <- .5* sqrt(vy)
ind <- which( mod5b.pars$name=="d")
mod5b.pars[ ind, "value" ] <- colMeans( dat, na.rm=TRUE )

# estimate model
mod5b <- mirt::mirt(dat, 1, itemtype=itemtype, customItems=customItems,
                 pars=mod5b.pars, verbose=TRUE    )
sirt::mirt.wrapper.coef(mod5b)$coef

# some item plots
    par(ask=TRUE)
plot(mod5b, type='trace', layout=c(1,1))
    par(ask=FALSE)
# Alternatively:
sirt::mirt.wrapper.itemplot(mod5b)
}
}


%% \keyword{datasets}

%% File Name: sirt-defunct.Rd
%% File Version: 0.08

\name{sirt-defunct}
\alias{sirt-defunct}
\alias{rasch.conquest}
\alias{rasch.pml2}
\alias{yen.q3}
\alias{testlet.yen.q3}



\title{Defunct \pkg{sirt} Functions}

\description{
These functions have been removed or replaced in the \pkg{sirt}
package.
}

\usage{
rasch.conquest(...)
rasch.pml2(...)
testlet.yen.q3(...)
yen.q3(...)
}

\arguments{
    \item{\dots}{Arguments to be passed.}
}

\details{
The \code{rasch.conquest} function has been replaced by
\code{\link{R2conquest}}.

The \code{rasch.pml2} function has been superseded by
\code{\link{rasch.pml3}}.

The \code{testlet.yen.q3} function has been replaced by
\code{\link{Q3.testlet}}.

The \code{yen.q3} function has been replaced by
\code{\link{Q3}}.
}

% \examples{
% #############################################################################
% }

%% File Name: tam2mirt.Rd
%% File Version: 0.23

\name{tam2mirt}
\alias{tam2mirt}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Converting a fitted \code{TAM} Object into a \code{mirt} Object
}
\description{
Converts a fitted \code{TAM} object into a \code{mirt} object.
As a by-product, \code{lavaan} syntax is generated which can
be used with \code{\link{lavaan2mirt}} for re-estimating
the model in the \pkg{mirt} package.
Up to now, only single group models are supported.
There must not exist background covariates (no latent regression
models!).
}

\usage{
tam2mirt(tamobj)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{tamobj}{
Object of class  \code{\link[TAM:tam.mml]{TAM::tam.mml}}
}
}

%\details{
%%  ~~ If necessary, more details than the description above ~~
%}


\value{
A list with following entries

\item{mirt}{Object generated by \code{mirt} function if
\code{est.mirt=TRUE}}
\item{mirt.model}{Generated \code{mirt} model}
\item{mirt.syntax}{Generated \code{mirt} syntax}
\item{mirt.pars}{Generated parameter specifications
   in \code{mirt}}
\item{lavaan.model}{Used \code{lavaan} model transformed by
\code{lavaanify} function}
\item{dat}{Used dataset. If necessary, only items used in the
model are included in the dataset.}
\item{lavaan.syntax.fixed}{Generated \code{lavaan} syntax
with fixed parameter estimates.}
\item{lavaan.syntax.freed}{Generated \code{lavaan} syntax
with freed parameters for estimation.}
}


%\references{
%% ~put references to the literature/web site here ~
%}


%\author{
%Alexander Robitzsch
%}



%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See \code{\link{mirt.wrapper}} for convenience wrapper functions
for \code{\link[mirt:mirt]{mirt}} objects.

See \code{\link{lavaan2mirt}} for converting \code{lavaan}
syntax to \code{mirt} syntax.
}


\examples{
\dontrun{
library(TAM)
library(mirt)

#############################################################################
# EXAMPLE 1: Estimations in TAM for data.read dataset
#############################################################################

data(data.read)
dat <- data.read

#**************************************
#*** Model 1: Rasch model
#**************************************

# estimation in TAM package
mod <- TAM::tam.mml( dat )
summary(mod)
# conversion to mirt
res <- sirt::tam2mirt(mod)
# generated lavaan syntax
cat(res$lavaan.syntax.fixed)
cat(res$lavaan.syntax.freed)
# extract object of class mirt
mres <- res$mirt
# print and parameter values
print(mres)
mirt::mod2values(mres)
# model fit
mirt::M2(mres)
# residual statistics
mirt::residuals(mres, type="Q3")
mirt::residuals(mres, type="LD")
# item fit
mirt::itemfit(mres)
# person fit
mirt::personfit(mres)
# compute several types of factor scores (quite slow)
f1 <- mirt::fscores(mres, method='WLE',response.pattern=dat[1:10,])
     # method=MAP and EAP also possible
# item plot
mirt::itemplot(mres,"A3")    # item A3
mirt::itemplot(mres,4)       # fourth item
# some more plots
plot(mres,type="info")
plot(mres,type="score")
plot(mres,type="trace")
# compare estimates with estimated Rasch model in mirt
mres1 <- mirt::mirt(dat,1,"Rasch" )
print(mres1)
mirt.wrapper.coef(mres1)

#**************************************
#*** Model 2: 2PL model
#**************************************

# estimation in TAM
mod <- TAM::tam.mml.2pl( dat )
summary(mod)
# conversion to mirt
res <- sirt::tam2mirt(mod)
mres <- res$mirt
# lavaan syntax
cat(res$lavaan.syntax.fixed)
cat(res$lavaan.syntax.freed)
# parameter estimates
print(mres)
mod2values(mres)
mres@nest   # number of estimated parameters
# some plots
plot(mres,type="info")
plot(mres,type="score")
plot(mres,type="trace")
# model fit
mirt::M2(mres)
# residual statistics
mirt::residuals(mres, type="Q3")
mirt::residuals(mres, type="LD")
# item fit
mirt::itemfit(mres)

#**************************************
#*** Model 3: 3-dimensional Rasch model
#**************************************

# define Q-matrix
Q <- matrix( 0, nrow=12, ncol=3 )
Q[ cbind(1:12, rep(1:3,each=4) ) ] <- 1
rownames(Q) <- colnames(dat)
colnames(Q) <- c("A","B","C")
# estimation in TAM
mod <- TAM::tam.mml( resp=dat, Q=Q, control=list(snodes=1000,maxiter=30) )
summary(mod)
# mirt conversion
res <- sirt::tam2mirt(mod)
mres <- res$mirt
# mirt syntax
cat(res$mirt.syntax)
  ##   Dim01=1,2,3,4
  ##   Dim02=5,6,7,8
  ##   Dim03=9,10,11,12
  ##   COV=Dim01*Dim01,Dim02*Dim02,Dim03*Dim03,Dim01*Dim02,Dim01*Dim03,Dim02*Dim03
  ##   MEAN=Dim01,Dim02,Dim03
# lavaan syntax
cat(res$lavaan.syntax.freed)
  ##   Dim01=~ 1*A1+1*A2+1*A3+1*A4
  ##   Dim02=~ 1*B1+1*B2+1*B3+1*B4
  ##   Dim03=~ 1*C1+1*C2+1*C3+1*C4
  ##   A1 | t1_1*t1
  ##   A2 | t1_2*t1
  ##   A3 | t1_3*t1
  ##   A4 | t1_4*t1
  ##   B1 | t1_5*t1
  ##   B2 | t1_6*t1
  ##   B3 | t1_7*t1
  ##   B4 | t1_8*t1
  ##   C1 | t1_9*t1
  ##   C2 | t1_10*t1
  ##   C3 | t1_11*t1
  ##   C4 | t1_12*t1
  ##   Dim01 ~ 0*1
  ##   Dim02 ~ 0*1
  ##   Dim03 ~ 0*1
  ##   Dim01 ~~ Cov_11*Dim01
  ##   Dim02 ~~ Cov_22*Dim02
  ##   Dim03 ~~ Cov_33*Dim03
  ##   Dim01 ~~ Cov_12*Dim02
  ##   Dim01 ~~ Cov_13*Dim03
  ##   Dim02 ~~ Cov_23*Dim03
# model fit
mirt::M2(mres)
# residual statistics
residuals(mres,type="LD")
# item fit
mirt::itemfit(mres)

#**************************************
#*** Model 4: 3-dimensional 2PL model
#**************************************

# estimation in TAM
mod <- TAM::tam.mml.2pl( resp=dat, Q=Q, control=list(snodes=1000,maxiter=30) )
summary(mod)
# mirt conversion
res <- sirt::tam2mirt(mod)
mres <- res$mirt
# generated lavaan syntax
cat(res$lavaan.syntax.fixed)
cat(res$lavaan.syntax.freed)
# write lavaan syntax on disk
  sink( "mod4_lav_freed.txt", split=TRUE )
cat(res$lavaan.syntax.freed)
  sink()
# some statistics from mirt
print(mres)
summary(mres)
mirt::M2(mres)
mirt::residuals(mres)
mirt::itemfit(mres)

# estimate mirt model by using the generated lavaan syntax with freed parameters
res2 <- sirt::lavaan2mirt( dat, res$lavaan.syntax.freed,
            technical=list(NCYCLES=3), verbose=TRUE)
                 # use only few cycles for illustrational purposes
mirt.wrapper.coef(res2$mirt)
summary(res2$mirt)
print(res2$mirt)

#############################################################################
# EXAMPLE 4: mirt conversions for polytomous dataset data.big5
#############################################################################

data(data.big5)
# select some items
items <- c( grep( "O", colnames(data.big5), value=TRUE )[1:6],
     grep( "N", colnames(data.big5), value=TRUE )[1:4] )
# O3 O8 O13 O18 O23 O28 N1 N6 N11 N16
dat <- data.big5[, items ]
library(psych)
psych::describe(dat)

library(TAM)
#******************
#*** Model 1: Partial credit model in TAM
mod1 <- TAM::tam.mml( dat[,1:6] )
summary(mod1)
# convert to mirt object
mmod1 <- sirt::tam2mirt( mod1 )
rmod1 <- mmod1$mirt
# coefficients in mirt
coef(rmod1)
mirt.wrapper.coef(rmod1)
# model fit
mirt::M2(rmod1)
# item fit
mirt::itemfit(rmod1)
# plots
plot(rmod1,type="trace")
plot(rmod1, type="trace", which.items=1:4 )
mirt::itemplot(rmod1,"O3")

#******************
#*** Model 2: Generalized partial credit model in TAM
mod2 <- TAM::tam.mml.2pl( dat[,1:6], irtmodel="GPCM" )
summary(mod2)
# convert to mirt object
mmod2 <- sirt::tam2mirt( mod2 )
rmod2 <- mmod2$mirt
# coefficients in mirt
mirt.wrapper.coef(rmod2)
# model fit
mirt::M2(rmod2)
# item fit
mirt::itemfit(rmod2)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%% \keyword{TAM}
%% \keyword{mirt}
